/* BaseDeFiches - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.storage.BalayagesStorage;
import fr.exemole.bdfserver.storage.BdfExtensionStorage;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.OutputStorage;
import fr.exemole.bdfserver.storage.StorageCollection;
import fr.exemole.bdfserver.storage.StorageProcessLog;
import fr.exemole.bdfserver.storage.TableExportStorage;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.storage.directory.balayage.DirectoryBalayagesStorage;
import fr.exemole.bdfserver.storage.directory.bdf.DirectoryBdfStorage;
import fr.exemole.bdfserver.storage.directory.extension.DirectoryBdfExtensionStorage;
import fr.exemole.bdfserver.storage.directory.output.DirectoryOutputStorage;
import fr.exemole.bdfserver.storage.directory.resource.DirectoryBdfResourceStorage;
import fr.exemole.bdfserver.storage.directory.tableexportstorage.DirectoryTableExportStorage;
import fr.exemole.bdfserver.storage.directory.transformationstorage.DirectoryTransformationStorage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.buildinfo.BuildInfo;
import net.mapeadores.util.buildinfo.BuildInfoParser;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.io.MimeTypeResolver;
import net.mapeadores.util.servlets.ServletUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class BaseDeFichesInit implements BaseDeFichesConstants {

    public final static String BDF_CONF_FILE_PARAMETER = "bdfConfFile";
    public final static String BDF_SMTP_FILE_PARAMETER = "bdfSmtpFile";

    public static SmtpManager initSmtpManager(ServletContext servletContext, HttpServletRequest request) {
        File bdfSmtpFile = getFileFromParameters(BDF_SMTP_FILE_PARAMETER, request, servletContext);
        if (bdfSmtpFile == null) {
            bdfSmtpFile = new File(servletContext.getRealPath("/WEB-INF/bdf-smtp.properties"));
        }
        SmtpManager smtpManager = new SmtpManager(bdfSmtpFile);
        return smtpManager;
    }

    public static BdfServer initBdfServer(String bdfServerName, ServletContext servletContext, HttpServletRequest request) {
        String contextPath = ServletUtils.getContextPath(servletContext);
        BaseDeFichesDirectories baseDeFichesDirectories = null;
        File contextDirectory = new File(servletContext.getRealPath("/"));
        File bdfConfFile = getFileFromParameters(BDF_CONF_FILE_PARAMETER, request, servletContext);
        if (bdfConfFile != null) {
            baseDeFichesDirectories = initBaseDeFichesDirectories(bdfConfFile, contextPath, contextDirectory);
        } else {
            bdfConfFile = new File(servletContext.getRealPath("/WEB-INF/bdf-conf.xml"));
            if (bdfConfFile.exists()) {
                baseDeFichesDirectories = initBaseDeFichesDirectories(bdfConfFile, contextPath, contextDirectory);
            } else {
                baseDeFichesDirectories = getOldBaseDeFichesDirectories(contextDirectory);
            }
        }
        MimeTypeResolver mimeTypeResolver = ServletContextUtils.getMimeTypeResolver(servletContext);
        StorageCollection storageCollection = BaseDeFichesInit.toStorageCollection(baseDeFichesDirectories, mimeTypeResolver);
        BdfServer bdfServer = new BdfServer(storageCollection);
        bdfServer.setBuildInfo(getBuildInfo());
        return bdfServer;
    }

    private static File getFileFromParameters(String paramName, HttpServletRequest request, ServletContext servletContext) {
        String paramValue = request.getParameter(paramName);
        if (paramValue == null) {
            paramValue = servletContext.getInitParameter(paramName);
        }
        if (paramValue != null) {
            return new File(paramValue);
        } else {
            return null;
        }
    }

    private static BaseDeFichesDirectories initBaseDeFichesDirectories(File bdfConfFile, String contextPath, File relativeDirectory) {
        if (!bdfConfFile.exists()) {
            throw new BdfInitException("bdfConfFile does not exist : " + bdfConfFile.getAbsolutePath());
        }
        if (bdfConfFile.isDirectory()) {
            throw new BdfInitException("bdfConfFile is a directory : " + bdfConfFile.getAbsolutePath());
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(bdfConfFile);
            BdfConfReader reader = new BdfConfReader(contextPath, relativeDirectory);
            reader.readConf(doc.getDocumentElement());
            return reader.toBaseDeFichesDirectories();
        } catch (ParserConfigurationException pce) {
            throw new BdfInitException("ParserConfigurationException : " + pce.getMessage());
        } catch (IOException ioe) {
            throw new BdfInitException("IOException when reading bdfConfFile : " + ioe.getMessage() + " / " + bdfConfFile.getAbsolutePath());
        } catch (SAXException sax) {
            throw new BdfInitException("XML Error when reading bdfConfFile : " + sax.getMessage() + " / " + bdfConfFile.getAbsolutePath());
        }
    }

    private static BaseDeFichesDirectories getOldBaseDeFichesDirectories(File contextDirectory) {
        InternalBaseDeFichesDirectories bdfDirectories = new InternalBaseDeFichesDirectories();
        bdfDirectories.setDirectory(ETCDIR_BALAYAGES, new File(contextDirectory, "_balayages"));
        bdfDirectories.setDirectory(ETCDIR_TRANSFORMATION, new File(contextDirectory, "_xslt"));
        bdfDirectories.setDirectory(ETCDIR_TABLEEXPORT, new File(contextDirectory, "_tableexport"));
        bdfDirectories.setDirectory(ETCDIR_RESOURCES, new File(contextDirectory, "_rsc"));
        bdfDirectories.setDirectory(VARDIR_BDFSTORAGE, new File(contextDirectory, "BDF-INF"));
        bdfDirectories.setDirectory(VARDIR_OUTPUT, new File(contextDirectory, "BDF-OUT"));
        bdfDirectories.setDirectory(VARDIR_RESOURCES, new File(contextDirectory, "BDF-RSC"));
        bdfDirectories.setDirectory(VARDIR_CACHE, new File(contextDirectory, "BDF-SRZ"));
        bdfDirectories.setDirectory(VARDIR_BACKUP, new File(contextDirectory, "BDF-BAK"));
        File extensionDirectory = new File(contextDirectory, "BDF-EXT");
        File scriptDirectory = new File(extensionDirectory, "scripts");
        bdfDirectories.setDirectory(ETCDIR_SCRIPTS, scriptDirectory);
        File configDirectory = new File(extensionDirectory, "config");
        if (!configDirectory.exists()) {
            File oldDir = new File(extensionDirectory, "parameters");
            if (oldDir.exists()) {
                configDirectory = oldDir;
            }
        }
        bdfDirectories.setDirectory(ETCDIR_EXTENSIONCONFIG, configDirectory);
        File dataDirectory = new File(extensionDirectory, "data");
        bdfDirectories.setDirectory(VARDIR_EXTENSIONDATA, dataDirectory);
        return bdfDirectories;
    }

    public static StorageCollection toStorageCollection(BaseDeFichesDirectories directories, MimeTypeResolver mimeTypeResolver) {
        StorageCollection storageCollection = new StorageCollection();
        StorageProcessLog storageProcessLog = new StorageProcessLog();
        storageCollection.setStorageProcessLog(storageProcessLog);
        BalayagesStorage balayagesStorage = new DirectoryBalayagesStorage(directories.getDirectory(ETCDIR_BALAYAGES));
        storageCollection.setBalayagesStorage(balayagesStorage);
        TableExportStorage tableExportStorage = new DirectoryTableExportStorage(directories.getDirectory(ETCDIR_TABLEEXPORT));
        storageCollection.setTableExportStorage(tableExportStorage);
        File resourceConfDir = directories.getDirectory(ETCDIR_RESOURCES);
        File resourceVarDir = directories.getDirectory(VARDIR_RESOURCES);
        DirectoryBdfResourceStorage bdfResourceStorage = new DirectoryBdfResourceStorage(mimeTypeResolver, resourceConfDir, resourceVarDir);
        initDefaultResourceReferenceClasses(bdfResourceStorage);
        storageCollection.setInnerResourceStorage(bdfResourceStorage);
        TransformationStorage transformationStorage = new DirectoryTransformationStorage(directories.getDirectory(ETCDIR_TRANSFORMATION));
        storageCollection.setTransformationStorage(transformationStorage);
        BdfStorage bdfStorage = new DirectoryBdfStorage(directories.getDirectory(VARDIR_BDFSTORAGE), directories.getDirectory(VARDIR_CACHE), directories.getDirectory(VARDIR_BACKUP), storageProcessLog);
        storageCollection.setBdfStorage(bdfStorage);
        OutputStorage outputStorage = new DirectoryOutputStorage(directories.getDirectory(VARDIR_OUTPUT));
        storageCollection.setOutputStorage(outputStorage);
        BdfExtensionStorage bdfExtensionStorage = new DirectoryBdfExtensionStorage(directories.getDirectory(ETCDIR_EXTENSIONCONFIG), directories.getDirectory(VARDIR_EXTENSIONDATA));
        storageCollection.setBdfExtensionStorage(bdfExtensionStorage);
        File scriptDirectory = directories.getDirectory(ETCDIR_SCRIPTS);
        if (scriptDirectory != null) {
            String scriptPath = scriptDirectory.getAbsolutePath();
            if (scriptPath.charAt((scriptPath.length() - 1)) != File.separatorChar) {
                scriptPath = scriptPath + File.separator;
            }
            storageCollection.setExternalScriptPath(scriptPath);
        }
        return storageCollection;
    }

    private static void initDefaultResourceReferenceClasses(DirectoryBdfResourceStorage bdfResourceStorage) {
        bdfResourceStorage.setDefaultBdfResourceReferenceClass(BdfResourceConstants.CSS_RESOURCE, BdfServer.class);
        bdfResourceStorage.setDefaultBdfResourceReferenceClass(BdfResourceConstants.JQUERY_RESOURCE, BdfHtmlProducer.class);
        bdfResourceStorage.setDefaultBdfResourceReferenceClass(BdfResourceConstants.JS_RESOURCE, BdfHtmlProducer.class);
        bdfResourceStorage.setDefaultBdfResourceReferenceClass(BdfResourceConstants.THEME_RESOURCE, BdfHtmlProducer.class);
        bdfResourceStorage.setDefaultBdfResourceReferenceClass(BdfResourceConstants.OTHER_RESOURCE, BdfServer.class);
    }

    public static BuildInfo getBuildInfo() {
        InputStream inputStream = BaseDeFichesInit.class.getResourceAsStream("VERSION");
        if (inputStream != null) {
            try {
                BuildInfo buildInfo = BuildInfoParser.parse(inputStream);
                inputStream.close();
                return buildInfo;
            } catch (IOException ioe) {
                throw new InternalResourceException(ioe);
            }
        } else {
            return null;
        }
    }


    private static class InternalBaseDeFichesDirectories implements BaseDeFichesDirectories {

        private Map<String, File> dirMap = new HashMap<String, File>();

        public File getDirectory(String dirName) {
            return dirMap.get(dirName);
        }

        private void setDirectory(String dirName, File dir) {
            dirMap.put(dirName, dir);
        }

    }


    private static class BdfConfReader {

        String contextPath;
        File relativeDirectory;
        File etcDirectory;
        File varDirectory;
        InternalBaseDeFichesDirectories bdfDirectories;
        Map<String, DirectoryInfo> directoryInfoMap;

        BdfConfReader(String contextPath, File relativeDirectory) {
            this.contextPath = contextPath;
            this.relativeDirectory = relativeDirectory;
        }

        private void readConf(Element element_xml) {
            DirectoryInfo etcDirectoryInfo = null;
            DirectoryInfo varDirectoryInfo = null;
            directoryInfoMap = new HashMap<String, DirectoryInfo>();
            NodeList nodeList = element_xml.getChildNodes();
            for (int i = 0; i < nodeList.getLength(); i++) {
                Node nd = nodeList.item(i);
                if (nd instanceof Element) {
                    Element el = (Element) nd;
                    String tagName = el.getTagName();
                    if (tagName.equals("etc-dir")) {
                        etcDirectoryInfo = toDirectoryInfo(el);
                    } else if (tagName.equals("var-dir")) {
                        varDirectoryInfo = toDirectoryInfo(el);
                    } else if (tagName.equals("dir")) {
                        DirectoryInfo directoryInfo = toDirectoryInfo(el);
                        if (directoryInfo != null) {
                            directoryInfoMap.put(directoryInfo.getDirKey(), directoryInfo);
                        }
                    }
                }
            }
            if (etcDirectoryInfo == null) {
                throw new BdfInitException("Missing <etc-dir> element in bdfConfFile");
            }
            if (varDirectoryInfo == null) {
                throw new BdfInitException("Missing <var-dir> element in bdfConfFile");
            }
            etcDirectory = etcDirectoryInfo.toFile();
            varDirectory = varDirectoryInfo.toFile();
        }

        private BaseDeFichesDirectories toBaseDeFichesDirectories() {
            bdfDirectories = new InternalBaseDeFichesDirectories();
            initDir(ETCDIR_BALAYAGES, "balayages");
            initDir(ETCDIR_TRANSFORMATION, "xslt");
            initDir(ETCDIR_TABLEEXPORT, "tableexport");
            initDir(ETCDIR_RESOURCES, "rsc");
            initDir(ETCDIR_SCRIPTS, "scripts");
            initDir(ETCDIR_EXTENSIONCONFIG, "extconfig");
            initDir(VARDIR_BDFSTORAGE, "bdfdata");
            initDir(VARDIR_OUTPUT, "output");
            initDir(VARDIR_RESOURCES, "rscdata");
            initDir(VARDIR_CACHE, "cache");
            initDir(VARDIR_BACKUP, "backup");
            initDir(VARDIR_EXTENSIONDATA, "extdata");
            return bdfDirectories;
        }

        private void initDir(String key, String defaultDirName) {
            DirectoryInfo directoryInfo = directoryInfoMap.get(key);
            if (directoryInfo != null) {
                bdfDirectories.setDirectory(key, directoryInfo.toFile());
            } else {
                File dir = null;
                if (key.startsWith("etc.")) {
                    dir = new File(etcDirectory, defaultDirName);
                } else {
                    dir = new File(varDirectory, defaultDirName);
                }
                if ((dir.exists() && (!dir.isDirectory()))) {
                    throw new BdfInitException("Path " + dir.getAbsolutePath() + " is not a directory");
                }
                bdfDirectories.setDirectory(key, dir);
            }
        }

        private DirectoryInfo toDirectoryInfo(Element el) {
            String dirKey = null;
            String path = el.getAttribute("path");
            if (path.length() == 0) {
                return null;
            }
            String appendContextString = el.getAttribute("append-context").toLowerCase();
            boolean appendContext = false;
            if ((appendContextString.equals("true"))) {
                appendContext = true;
            }
            String elementString = null;
            String tagName = el.getTagName();
            if (tagName.equals("dir")) {
                dirKey = el.getAttribute("key");
                if (dirKey.length() == 0) {
                    return null;
                }
                elementString = "<dir key=\"" + dirKey + "\">";
            } else {
                elementString = "<" + tagName + ">";
            }
            DirectoryInfo directoryInfo = new DirectoryInfo(path, appendContext, elementString, dirKey);
            return directoryInfo;
        }


        private class DirectoryInfo {

            private String elementString;
            private String path;
            private boolean appendContext = false;
            private String dirKey;

            private DirectoryInfo(String path, boolean appendContext, String elementString, String dirKey) {
                this.elementString = elementString;
                this.path = path;
                this.appendContext = appendContext;
                this.dirKey = dirKey;
            }

            public String getDirKey() {
                return dirKey;
            }

            private File toFile() {
                File directory = new File(path);
                if (!directory.isAbsolute()) {
                    directory = new File(getRelativeDirectory(), path);
                }
                if (appendContext) {
                    directory = new File(directory, contextPath);
                }
                if (!directory.exists()) {
                    if (testCreation()) {
                        directory.mkdirs();
                        return directory;
                    } else if (testMandatory()) {
                        throw new BdfInitException("Directory defined by " + elementString + " in bdfConfFile does not exist");
                    }
                } else if (!directory.isDirectory()) {
                    throw new BdfInitException("Directory defined by " + elementString + " in bdfConfFile is not a directory");
                }
                return directory;
            }

            private boolean testCreation() {
                if (elementString.equals("<etc-dir>")) {
                    return false;
                }
                if (elementString.equals("<var-dir>")) {
                    return true;
                }
                if (dirKey.startsWith("var.")) {
                    return true;
                }
                return false;
            }

            private boolean testMandatory() {
                if (elementString.equals("<etc-dir>")) {
                    return true;
                }
                return false;
            }

            private File getRelativeDirectory() {
                if (elementString.equals("<etc-dir>")) {
                    return relativeDirectory;
                }
                if (elementString.equals("<var-dir>")) {
                    return relativeDirectory;
                }
                if (dirKey.startsWith("var.")) {
                    return varDirectory;
                }
                if (dirKey.startsWith("etc.")) {
                    return etcDirectory;
                }
                return relativeDirectory;
            }

        }

    }

}
