/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.scrutariexport.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.association.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.roles.*;
import fr.exemole.bdfserver.servlets.util.PiocheParameters;
import fr.exemole.bdfserver.servlets.util.*;
import fr.exemole.bdfserver.servlets.resources.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class BdfHtmlPage extends UtilHtmlPrinter implements ResponseHandler, JavascriptCatalog, ClassCatalog {

    protected BdfServer bdfServer;
    protected int workingLangInteger;
    protected BdfUser bdfUser;
    protected String messageKey;
    protected RedacteurKey redacteurKey;
    protected Locale formatLocale;
    protected LocalisationInfo localisationInfo;
    protected CommandMessage message;
    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;
    private boolean onFieldSet = false;
    protected AppelantWriter appelantWriter;
    private boolean withJQuery = false;
    private boolean admin = false;
    private String themePath;
    private boolean jqueryIncluded = false;

    public BdfHtmlPage(BdfResponseParameters responseParameters) {
        this.bdfServer = responseParameters.getBdfServer();
        this.bdfUser = responseParameters.getBdfUser();
        this.themePath = responseParameters.getReversePath() + "theme/css/";
        this.localisationInfo = responseParameters.getLocalisationInfo();
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
        this.formatLocale = localisationInfo.getFormatLocale();
        addResourceLocalisation(ServletCommons.getResourceLocalisation(formatLocale));
        ResourceLocalisation commandKeyLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.bdfserver.servlets.resources.l10n.bdfcommand", formatLocale));
        setCommandKeyLocalisation(commandKeyLocalisation);
        this.message = responseParameters.getCommandMessage();
        this.admin = responseParameters.isAdmin();
        if (bdfUser != null) {
            this.redacteurKey = bdfUser.getRedacteurKey();
            this.withJQuery = bdfUser.isWithJQuery();
        }
        getCommandBoxParameters().getFormAttributes().setAction(getFormAction());
        getCommandBoxParameters().getFormAttributes().setMethod("POST");
        appelantWriter = new AppelantWriter(bdfServer.getBasedefiches(), this, workingLangInteger);
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public boolean isAdmin() {
        return admin;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public int getWorkingLangInteger() {
        return workingLangInteger;
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public boolean isWithJQuery() {
        return withJQuery;
    }

    protected void addJavascript(String name) {
        if (isWithJQuery()) {
            if (!jqueryIncluded) {
                jqueryIncluded = true;
                addJsUrl("jquery/jquery.js");
            }
            if (name != null) {
                if (name.indexOf(',') != -1) {
                    String[] tokens = StringUtils.getCleanedTokens(name, ',');
                    int length = tokens.length;
                    for (int i = 0; i < length; i++) {
                        addJsUrl("js/" + tokens[i]);
                    }
                } else {
                    addJsUrl("js/" + name);
                }
            }
        }
    }

    protected void addJqueryExtensions(String name) {
        if (isWithJQuery()) {
            if (!jqueryIncluded) {
                jqueryIncluded = true;
                addJsUrl("jquery/jquery.js");
            }
            String[] tokens = StringUtils.getCleanedTokens(name, ',');
            int length = tokens.length;
            for (int i = 0; i < length; i++) {
                addJsUrl("jquery/jquery." + name + ".js");
            }
        }
    }

    protected void addExternalJavascript(String url) {
        if (isWithJQuery()) {
            if (!jqueryIncluded) {
                jqueryIncluded = true;
                addJsUrl("js/_jquery.js");
            }
            if (url != null) {
                addJsUrl(url);
            }
        }
    }

    protected void initResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());

    }

    protected void allowsTest() {
        getCommandBoxParameters().getFormAttributes().setOnSubmit("return testCommand(this);");
        addJavascript("_testcommand.js");
    }

    public Map<String, String> getHiddenParameterMap(Object obj) {
        return RequestUtils.getHiddenParameterMap(obj);
    }

    protected void printMessage() {
        if (message != null) {
            printMessage(message);
        }
    }

    public void start() {
        setTitle(getTitle());
        setIconPng(getIconPng());
        String cssThemeFile = getCssThemeFile();
        if (cssThemeFile != null) {
            addCssUrl(themePath + cssThemeFile);
            short screenWidth = BdfUserUtils.getScreenWidth(bdfUser);
            switch (screenWidth) {
                case BdfUserConstants.SCREENWIDTH_800:
                    addCssUrl(themePath + "_screen_800.css");
                    break;
                case BdfUserConstants.SCREENWIDTH_1024:
                    addCssUrl(themePath + "_screen_1024.css");
                    break;
                case BdfUserConstants.SCREENWIDTH_1280:
                    addCssUrl(themePath + "_screen_1280.css");
                    break;

            }
        }
        super.start(localisationInfo.getFormatLocaleInteger());
    }

    public String getIconPng() {
        return "rsc/icon.png";
    }

    public void endForm(String submitKey) {
        endFormBlock(submitKey);
    }

    protected void endFormBlock(String submitKey) {
        if (submitKey != null) {
            startP();
            addSubmit(submitKey, false);
            endP();
        }
        if (onFieldSet) {
            onFieldSet = false;
            endFieldset();
        }
        endForm();
    }

    protected void endInLineForm(String submitKey) {
        HTMLUtils.printFormEnd(pw, getLocalization(submitKey), HTMLUtils.ENDFORM_INLINE);
        endP();
    }

    protected void endFormWithConfirm(String submitKey) {
        startP();
        localize("global_lib_confirmpassword");
        pw.print(" <input type=\"password\" value=\"\" name=\"");
        pw.print(RequestUtils.CONFIRMPASSWORD_PARAMNAME);
        pw.print("\">");
        pw.print("<input type=\"submit\" value=\"");
        localize(submitKey);
        pw.print("\">");
        endP();
        pw.print("</form>");
    }

    protected void printConfirmPassword() {
        startP();
        startEm();
        printLibColon("global_lib_confirmpassword");
        endEm();
        appendPasswordInput(RequestUtils.CONFIRMPASSWORD_PARAMNAME, 20);
        endP();
    }

    protected void startPostForm(String command) {
        beginFormBlock(command, null);
    }

    protected void beginFormBlock(String command, String fieldsetKey) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        printFieldSet(fieldsetKey);
    }

    protected void startGetForm(String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "get", command);
    }

    protected void startPageForm(String pageValue) {
        HTMLUtils.printFormStart(pw, getFormAction(), "get", null);
        addPageParameter(pageValue);
    }

    protected void startTargettedForm(String command, String target) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command, target, null);
    }

    protected void startPostForm(Thesaurus thesaurus, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(thesaurus));
    }

    protected void startGetForm(Thesaurus thesaurus, String command, String target) {
        HTMLUtils.printFormStart(pw, getFormAction(), "get", command, target, null);
        pw.print(RequestUtils.toHiddenParameter(thesaurus));
    }

    protected void startForm(Motcle motcle, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(motcle));
    }

    protected void startForm(Motcle motcle, String command, String formName) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command, null, formName);
        pw.print(RequestUtils.toHiddenParameter(motcle));
    }

    protected void startInLineForm(Motcle motcle, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(motcle));
        pw.print("<p>");
    }

    protected void startForm(Sphere sphere, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(sphere));
    }

    protected void startForm(Redacteur redacteur, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(redacteur));
    }

    protected void startPostForm(Corpus corpus, String command) {
        beginFormBlock(corpus, command, null);
    }

    protected void beginFormBlock(Corpus corpus, String command, String fieldsetKey) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(corpus));
        printFieldSet(fieldsetKey);
    }

    private void printFieldSet(String fieldsetKey) {
        if (fieldsetKey != null) {
            pw.println("<fieldset>");
            if (fieldsetKey.length() > 0) {
                pw.print("<legend>");
                localize(fieldsetKey);
                pw.println("</legend>");
            }
            onFieldSet = true;
        }
    }

    protected void startGetForm(Corpus corpus, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "get", command);
        pw.print(RequestUtils.toHiddenParameter(corpus));
    }

    protected void startForm(AbregeFiche abregeFiche, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(abregeFiche));
    }

    protected void startInLineForm(AbregeFiche abregeFiche, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(abregeFiche));
        pw.print("<p>");
    }

    protected void startForm(Document document, String command, AbregeFiche abregeFiche) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(document));
        if (abregeFiche != null) {
            pw.print(RequestUtils.toHiddenParameter(abregeFiche));
        }
    }

    protected void startMultiPartForm(String command) {
        HTMLUtils.printMultiPartFormStart(pw, getFormAction(), "post", command);
    }

    protected void startMultiPartForm(Document document, String command, AbregeFiche abregeFiche) {
        HTMLUtils.printMultiPartFormStart(pw, getFormAction(), "post", command);
        if (document != null) {
            pw.print(RequestUtils.toHiddenParameter(document));
        }
        if (abregeFiche != null) {
            pw.print(RequestUtils.toHiddenParameter(abregeFiche));
        }
    }

    protected void startMultiPartForm(AbregeFiche abregeFiche, String command) {
        HTMLUtils.printMultiPartFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(abregeFiche));
    }

    protected void startMultiPartForm(Corpus corpus, String command) {
        HTMLUtils.printMultiPartFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(corpus));
    }

    protected void startInLineForm(Document document, String command, AbregeFiche abregeFiche) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(document));
        if (abregeFiche != null) {
            pw.print(RequestUtils.toHiddenParameter(abregeFiche));
        }
        pw.print("<p>");
    }

    protected void startForm(ScrutariExport scrutariExport, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(scrutariExport));
    }

    protected void startForm(SqlExportBean sqlExportBean, String command) {
        HTMLUtils.printFormStart(pw, getFormAction(), "post", command);
        pw.print(RequestUtils.toHiddenParameter(sqlExportBean));
    }

    public abstract String getFormAction();

    public abstract String getTitle();

    public abstract String getCssThemeFile();

    public void addParameter(Sphere sphere) {
        pw.print(RequestUtils.toHiddenParameter(sphere));
    }

    public void addParameter(Redacteur redacteur) {
        pw.print(RequestUtils.toHiddenParameter(redacteur));
    }

    public void addParameter(Thesaurus thesaurus) {
        pw.print(RequestUtils.toHiddenParameter(thesaurus));
    }

    public void addParameter(Motcle motcle) {
        pw.print(RequestUtils.toHiddenParameter(motcle));
    }

    public void addParameter(String paramName, String paramValue) {
        appendHiddenInput(paramName, paramValue);
    }

    protected void addStepParameter(String stepValue) {
        addParameter("step", stepValue);
    }

    protected void addPageParameter(String pageValue) {
        addParameter("page", pageValue);
    }

    protected void addResponseFactoryParameter(String pageValue) {
        addParameter("responsefactory", pageValue);
    }

    protected void printTiret() {
        HTMLPrinterUtils.printTiret(this);
    }

    protected void printLib(String libKey) {
        localize(libKey);
        pw.print(" ");
    }

    protected void printFixedSubmit(String key) {
        SubmitParameters submitParameters = new SubmitParameters();
        submitParameters.setEncloseInDiv(true);
        HTMLAttributes divAttr = new HTMLAttributes();
        divAttr.setStyle("position:fixed;right:10px;top:40px;padding:3px;background-color:white;z-index: 10;");
        submitParameters.setDivAttributes(divAttr);
        addSubmit(new ResourceItem(key), submitParameters);
    }

    protected void printLangFieldRow(String name, int langInteger, String value) {
        HTMLUtils.printLangFieldRow(pw, langInteger, name, TextParser.escapeString(value), 50, 1);
    }

    public void printH1WithIcon(String stringKey, String iconKey, String iconPath) {
        startH1();
        print16pxStaticIcon(iconKey, iconPath);
        space();
        localize(stringKey);
        endH1();
    }

    public void print16pxStaticIcon(String iconKey, String iconPath) {
        HTMLAttributes imgAtt = new HTMLAttributes();
        String src = "theme/icons/16x16/" + iconKey + ".png";
        if (iconPath.length() > 0) {
            src = iconPath + "/" + src;
        }
        imgAtt.setSrc(src);
        imgAtt.setWidth("16");
        imgAtt.setHeight("16");
        imgAtt.setAlt("");
        appendImg(imgAtt);
    }

    public HTMLAttributes get32pxIconAttributes(String iconKey) {
        HTMLAttributes imgAtt = new HTMLAttributes();
        imgAtt.setSrc("theme/icons/32x16/" + iconKey + ".png");
        imgAtt.setWidth("32");
        imgAtt.setHeight("16");
        imgAtt.setClasses("Icon");
        return imgAtt;
    }

    public void printMotcleLibelle(Motcle motcle) {
        printMotcleLibelle(motcle, ThesaurusUtils.checkLangIntegerDisponibility(motcle.getThesaurus(), workingLangInteger));
    }

    public void printMotcleLibelle(Motcle motcle, int langInteger) {
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            append(idalpha);
            append(" \u2013 ");
        }
        String lib = motcle.getLibelleString(langInteger, ThesaurusUtils.isBabelienType(motcle.getThesaurus()));
        if (lib == null) {
            append("?");
        } else {
            escape(lib);
        }
        if (idalpha == null) {
            startSmall();
            append(" (idths=");
            append(String.valueOf(motcle.getIdths()));
            append(")");
            endSmall();
        }
    }

    protected void printRoleIntitule(Role role) {
        if (bdfServer.getPermissionManager().isDefaultRole(role)) {
            startEm();
            localize("admin_lib_defaultrole");
            endEm();
        } else {
            startCode();
            append("[");
            append(role.getName());
            append("]");
            endCode();
        }
        printTiret();
        escape(role.getRoleLibelleHolder().getLibelleString(workingLangInteger, true));
    }

    public void checkCommandBoxStyle(String commandKey) {
        int idx = commandKey.indexOf("-");
        if (idx == -1) {
            return;
        }
        String family = commandKey.substring(0, idx);
        getCommandBoxParameters().getFieldsetAttributes().setClasses("fs_" + family);
    }

    protected void printOneLine(String name, String libelle, String value, int size, AppelantParameters appelantParameters) {
        HTMLAttributes txtAtt = new HTMLAttributes();
        txtAtt.setName(name);
        txtAtt.setValue(value);
        txtAtt.setSize(String.valueOf(size));
        txtAtt.setId(getId(appelantParameters, name));
        startRow();
        printLitteralLibCells(libelle, false);
        startCell();
        appendTextInput(txtAtt);
        printAppelant(appelantParameters);
        endCell();
        endRow();
    }

    protected void printPredeterminedLine(String name, String libelle, String value) {
        startRow();
        printLitteralLibCells(libelle, false);
        startCell();
        appendHiddenInput(name, value);
        startCode();
        escape(value);
        endCode();
        endCell();
        endRow();
    }

    protected void printMultiLine(String name, String libelle, String value, int rows, AppelantParameters appelantParameters) {
        startRow();
        startCell(3, 0);
        escape(libelle);
        append("&nbsp;:");
        endCell();
        endRow();
        startRow();
        startCell(3, 0);
        startTextArea(name, 69, rows, getId(appelantParameters, name));
        escape(value, true);
        endTextArea();
        printAppelant(appelantParameters);
        endCell();
        endRow();
    }

    private void printAppelant(AppelantParameters appelantParameters) {
        if (appelantParameters == null) {
            return;
        }
        if (appelantParameters instanceof PiocheParameters) {
            printPiocheur((PiocheParameters) appelantParameters);
        }
    }

    protected void printPiocheur(PiocheParameters piocheParameters) {
        space();
        appelantWriter.printPiocheur(piocheParameters);
    }

    private String getId(AppelantParameters appelantParameters, String name) {
        if (appelantParameters == null) {
            return name;
        }
        String appelantId = appelantParameters.getAppelantId();
        if (appelantId == null) {
            return name;
        }
        return appelantId;
    }

    public void appendCommandLink(String href, String commandKey, String target) {
        startA(href, target);
        appendCommandLocString(commandKey);
        endA();
    }

    public void appendCommandLocString(String commandKey) {
        escape(getCommandKeyLocalisation().toString(commandKey, null));
    }

}
