/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.html.utils.SubsetGroupingPrinter;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.sphere.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.servlets.util.*;
import fr.exemole.bdfserver.servlets.instructions.affichage.*;


/**
 *
 * @author Vincent Calame
 */
public class BdfHtmlUtils implements ClassCatalog {

    private BdfHtmlUtils() {
    }

    public static void printThesaurusLinksDeployBlock(BdfHtmlPage page, String thesaurusSubId, String hrefBase) {
        HTMLAttributes statsAttributes = new HTMLAttributes();
        HTMLAttributes ulAttributes = new HTMLAttributes();
        ulAttributes.setId("DEPLOYBLOCK_" + thesaurusSubId);
        ulAttributes.addClass(UL_GLB_SMALLLINKLIST);
        if (page.isWithJQuery()) {
            ulAttributes.addClass(INVISIBLE);
        }
        page.startUl(ulAttributes);
        appendLi(page, thesaurusSubId, MotsclesInstruction.STATS_OPTION, "thesaurus_link_stats", hrefBase);
        appendLi(page, thesaurusSubId, MotsclesInstruction.STATSSEL_OPTION, "thesaurus_link_statssel", hrefBase);
        appendLi(page, thesaurusSubId, MotsclesInstruction.INVERSE_OPTION, "thesaurus_link_listepermutee", hrefBase);
        page.endUl();
    }

    private static void appendLi(BdfHtmlPage page, String thesaurusSubId, String option, String locKey, String hrefBase) {
        String href = hrefBase + thesaurusSubId + "-" + option;
        HTMLAttributes aAttributes = new HTMLAttributes();
        aAttributes.setHref(href + ".html");
        aAttributes.setTarget("Saisie");
        page.startLi();
        page.startA(aAttributes);
        page.localize(locKey);
        page.endA();
        page.space();
        printOds(page, href);
        page.endLi();
    }

    /**
     * Ne pas indiquer l'extension .ods dans href
     */
    public static void printOds(BdfHtmlPage page, String href) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setHref(href + ".ods");
        page.append("(");
        page.startA(htmlAttributes);
        page.append("ods");
        page.endA();
        page.append(")");
    }

    /**
     * Les blocs à déployer doivent avoir comme identifiant DEPLOYBLOCK_ + idKey
     */
    public static void printDeployTitle(BdfHtmlPage page, String idKey, String locKey) {
        printDeploieTitle(page, "bdf_Deploie('" + idKey + "');", "DEPLOYBOX_" + idKey, locKey);
    }

    public static void printDeploieTitle(BdfHtmlPage page, String javascript, String deployBoxId, String locKey) {
        if (!page.isWithJQuery()) {
            return;
        }
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.addClass(DIV_GLB_DEPLOYTITLE);
        page.startDiv(divAttributes);
        page.startA("javascript:" + javascript);
        HTMLAttributes deployBoxAttributes = new HTMLAttributes();
        deployBoxAttributes.addClass(SPAN_GLB_DEPLOYBOX);
        deployBoxAttributes.setId(deployBoxId);
        page.startSpan(deployBoxAttributes);
        page.append("+");
        page.endSpan();
        page.append(" ");
        page.localize(locKey);
        page.endA();
        page.endDiv();
    }

    public static void startDeployBlock(BdfHtmlPage page, String id) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.addClass(DIV_GLB_DEPLOYBLOCK);
        if (page.isWithJQuery()) {
            htmlAttributes.addClass(INVISIBLE);
        }
        htmlAttributes.setId(id);
        page.startDiv(htmlAttributes);
    }

    public static void printChangeSubsetSelect(BdfHtmlPage page, Subset currentSubset, String currentPage) {
        if (!page.isWithJQuery()) {
            return;
        }
        UIManager uiManager = page.getBdfServer().getUIManager();
        if (currentPage == null) {
            currentPage = "";
        }
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setOnChange("bdf_ChangeSubset(this,'" + currentPage + "');");
        String name = null;
        SubsetGrouping subsetGrouping = null;
        if (currentSubset instanceof Corpus) {
            name = "corpus";
            subsetGrouping = uiManager.getCorpusSubsetKeyTree();
        } else if (currentSubset instanceof Thesaurus) {
            name = "thesaurus";
            subsetGrouping = uiManager.getThesaurusSubsetKeyTree();
        } else if (currentSubset instanceof Sphere) {
            name = "sphere";
            subsetGrouping = uiManager.getSphereSubsetKeyTree();
        }
        selectAtt.setName(name);
        printSubsetSelect(page, subsetGrouping, selectAtt, currentSubset.getSubsetKey());

    }

    public static void printSubsetSelect(BdfHtmlPage page, SubsetGrouping subsetGrouping, HTMLAttributes selectAttributes, SubsetKey selectedKey) {
        SubsetGroupingPrinter stw = new SubsetGroupingPrinter(page.getBdfServer().getBasedefiches(), page, page.getWorkingLangInteger());
        stw.setSubsetGrouping(subsetGrouping);
        stw.setOnlySubIdValues(true);
        stw.setSelectedSubsetKey(selectedKey);
        stw.printTreeSelect(selectAttributes);
    }

    public static void printSubsetSelect(BdfHtmlPage page, SubsetKeyTree subsetKeyTree, String name, SubsetKey selectedKey) {
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setName(name);
        printSubsetSelect(page, subsetKeyTree, selectAtt, selectedKey);
    }

}
