/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.commands.BdfCommand;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.users.*;
import java.util.Locale;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class CommandManager {

    public static final String ACTION_PARAMETER = "ACTION";

    public CommandManager() {
    }

    public static String getDefaultPackage(String servletPath) {
        if (servletPath.equals("/Thesaurus")) {
            return "fr.exemole.bdfserver.servlets.thesaurus";
        }
        if (servletPath.equals("/Selection")) {
            return "fr.exemole.bdfserver.servlets.selection";
        }
        if (servletPath.equals("/Configuration")) {
            return "fr.exemole.bdfserver.servlets.configuration";
        }
        if (servletPath.equals("/Envoi")) {
            return "fr.exemole.bdfserver.servlets.envoi";
        }
        if (servletPath.equals("/Session")) {
            return "fr.exemole.bdfserver.servlets.session";
        }
        if (servletPath.equals("/Administration")) {
            return "fr.exemole.bdfserver.servlets.administration";
        }
        if (servletPath.equals("/Corpus")) {
            return "fr.exemole.bdfserver.servlets.corpus";
        }
        if (servletPath.equals("/Association")) {
            return "fr.exemole.bdfserver.servlets.association";
        }
        if (servletPath.equals("/Exportation")) {
            return "fr.exemole.bdfserver.servlets.exportation";
        }
        if (servletPath.equals("/Importation")) {
            return "fr.exemole.bdfserver.servlets.importation";
        }
        if (servletPath.startsWith("/_exportation")) {
            return "fr.exemole.bdfserver.servlets.exportation";
        }
        throw new SwitchException("serlvetPath = " + servletPath);
    }

    /**
     *
     *
     */
    public static BdfCommand getCommandInstance(BdfRequestMap bdfRequestMap) throws ParameterException {
        String command = bdfRequestMap.getCommandParameter();
        String servletPath = bdfRequestMap.getRequest().getServletPath();
        String defaultpackage = null;
        if (servletPath.equals("/Test")) {
            String actionParameter = bdfRequestMap.getParameter(ACTION_PARAMETER);
            if (actionParameter == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, ACTION_PARAMETER);
            }
            try {
                defaultpackage = getDefaultPackage("/" + actionParameter);
            } catch (SwitchException se) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, ACTION_PARAMETER + "=" + actionParameter);
            }
        } else {
            defaultpackage = getDefaultPackage(servletPath);
        }
        if (command == null) {
            if (defaultpackage == null) {
                throw new ParameterException(ParameterException.CMD_UNDEFINED, null);
            }
            command = defaultpackage + ".DefaultCommand";
        } else if (command.indexOf('.') == -1) {
            if (defaultpackage == null) {
                throw new ParameterException(ParameterException.CMD_CLASS_NOTFOUND, command);
            }
            command = defaultpackage + "." + command + "Command";
        }
        try {
            Class commandClass = Class.forName(command);
            return (BdfCommand) commandClass.newInstance();
        } catch (ClassNotFoundException cnfe) {
            throw new ParameterException(ParameterException.CMD_CLASS_NOTFOUND, command);
        } catch (InstantiationException ie) {
            throw new ImplementationException(ie);
        } catch (IllegalAccessException ia) {
            throw new ImplementationException(ia);
        }
    }

    /**
     * Déclenche une commande de modification. Cette méthode est synchronisée e l'accès à l'instance bdfServer.
     */
    public static ResponseHandler doCommand(BdfCommand command, BdfBundle bdfBundle, BdfRequestMap bdfRequestMap) throws ParameterException {
        if (command.getCommandType() == BdfCommand.ADMIN_ONLY_COMMAND) {
            if (!BdfUserUtils.isAdmin(bdfBundle.getBdfUser())) {
                throw new ParameterException(ParameterException.ADMIN_ONLY, null);
            }
        }
        BdfResponseParameters responseParameters = null;
        command.setBdfBundle(bdfBundle);
        command.setRequestMap(bdfRequestMap);
        if (command.needSynchronisation()) {
            synchronized (bdfBundle.getBdfServer()) {  //à vérifier et à étudier de plus près
                responseParameters = command.doCommand();
            }
        } else {
            responseParameters = command.doCommand();
        }
        String responseFactoryClassName = bdfRequestMap.getResponseFactoryParameter();
        if (responseFactoryClassName != null) {
            try {
                ResponseFactory responseFactory = (ResponseFactory) Class.forName(responseFactoryClassName).newInstance();
                return responseFactory.getResponseHandler(responseParameters);
            } catch (ClassNotFoundException cnfe) {
                throw new ParameterException(ParameterException.RESPONSEFACTORY_CLASS_NOTFOUND, responseFactoryClassName);
            } catch (InstantiationException ie) {
                throw new ImplementationException(ie);
            } catch (IllegalAccessException iae) {
                throw new ImplementationException(iae);
            }
        }
        if (bdfRequestMap.asksForXml() && (responseParameters.getResponseCategory() == BdfResponseParameters.HTMLPAGE_CATEGORY)) {
            return XMLReportResponsePrinter.newReport(responseParameters, ServletCommons.getResourceLocalisation(bdfBundle.getBdfUser().getLocalisationInfo().getFormatLocale()));
        } else {
            ResponsePrinterFactory factory = command.getResponsePrinterFactory();
            return factory.getResponsePrinterInstance(responseParameters);
        }
    }

}
