/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.administration.*;
import fr.exemole.bdfserver.servlets.exportation.*;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import java.io.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class QueryServlet extends HttpServlet {

    public void doGet(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    public void doPost(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response) throws IOException {
        BdfServer bdfServer = ServletContextUtils.getBdfContext(getServletContext(), request).getBdfServer();
        String ficheParam = request.getParameter("fiche");
        if (ficheParam != null) {
            processFicheRequest(bdfServer, ficheParam, response);
        }
    }

    private void processFicheRequest(BdfServer bdfServer, String ficheParam, HttpServletResponse response) throws IOException {
        int idx = ficheParam.indexOf('/');
        if (idx == -1) {
            return;
        }
        Corpus corpus = BasedefichesUtils.getCorpus(bdfServer.getBasedefiches(), ficheParam.substring(0, idx));
        if (corpus == null) {
            return;
        }
        try {
            int idcorpus = Integer.parseInt(ficheParam.substring(idx + 1));
            AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche == null) {
                return;
            }
            response.setContentType("text/plain;charset=UTF-8");
            PrintWriter pw = response.getWriter();
            pw.print(abregeFiche.getTitre());
            pw.close();
        } catch (NumberFormatException nfe) {
            return;
        }
    }

}
