/* BaseDeFiches - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.extension.BdfExtensionManager;
import fr.exemole.bdfserver.session.SessionManager;
import fr.exemole.bdfserver.email.SmtpManager;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import net.mapeadores.util.css.SacParserInit;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.MimeTypeResolver;
import net.mapeadores.util.text.tableparser.TableParser;


/**
 *
 * @author Vincent Calame
 */
public class ServletContextUtils {

    private final static String INIT_KEY = "fr.exemole.Init";
    private final static String MIMETYPERESOLVER_KEY = "fr.exemole.MimeTypeResolver";
    private final static String BDFSERVER_KEY = "fr.exemole.BdfServer";
    private final static String SESSIONMANAGER_KEY = "fr.exemole.SessionManager";
    private final static String BDFEXTENSIONMANAGER_KEY = "fr.exemole.BdfExtensionManager";
    private final static String SMTPMANAGER_KEY = "fr.exemole.SmtpManager";
    private final static String BDFCONTEXT_KEY = "fr.exemole.BdfContext";

    private ServletContextUtils() {
    }

    public static MimeTypeResolver getMimeTypeResolver(ServletContext servletContext) {
        synchronized (servletContext) {
            MimeTypeResolver mimeTypeResolver = (MimeTypeResolver) servletContext.getAttribute(MIMETYPERESOLVER_KEY);
            if (mimeTypeResolver == null) {
                mimeTypeResolver = new ServletMimeTypeResolver(servletContext);
                servletContext.setAttribute(MIMETYPERESOLVER_KEY, mimeTypeResolver);
            }
            return mimeTypeResolver;
        }
    }

    public static BdfContext getBdfContext(ServletContext servletContext, HttpServletRequest request) {
        String bdfServerName = "_default";
        String key = BDFCONTEXT_KEY + "/" + bdfServerName;
        BdfContext bdfContext = (BdfContext) servletContext.getAttribute(key);
        if (bdfContext == null) {
            String intern = key.intern();
            synchronized (intern) {
                bdfContext = (BdfContext) servletContext.getAttribute(key);
                if (bdfContext == null) {
                    BdfServer bdfServer = getBdfServer(servletContext, request);
                    InternalBdfContext internalBdfContext = new InternalBdfContext(bdfServer);
                    internalBdfContext.putContextObject(BdfContextConstants.SESSIONMANAGER_NAME, ServletContextUtils.getSessionManager(servletContext, bdfServer));
                    internalBdfContext.putContextObject(BdfContextConstants.BDFEXTENSIONMANAGER_NAME, ServletContextUtils.getBdfExtensionManager(servletContext));
                    internalBdfContext.putContextObject(BdfContextConstants.SMTPMANAGER_NAME, ServletContextUtils.getSmtpManager(servletContext, request));
                    bdfContext = internalBdfContext;
                }
            }
        }
        return bdfContext;
    }

    private static BdfServer getBdfServer(ServletContext servletContext, HttpServletRequest request) {
        String bdfServerName = "_default";
        String key = BDFSERVER_KEY + "/" + bdfServerName;
        BdfServer bdfServer = (BdfServer) servletContext.getAttribute(key);
        if (bdfServer == null) {
            ServletContextUtils.init(servletContext);
            String intern = bdfServerName.intern();
            synchronized (intern) {
                bdfServer = (BdfServer) servletContext.getAttribute(key);
                if (bdfServer == null) {
                    bdfServer = BaseDeFichesInit.initBdfServer(bdfServerName, servletContext, request);
                    servletContext.setAttribute(key, bdfServer);
                    //initListeners(bdfServer, extensionDirectory, servletContext);
                    File contextDirectory = new File(servletContext.getRealPath("/")); //à supprimer bientôt
                    bdfServer.setContextDirectory(contextDirectory);
                }
            }
        }
        return bdfServer;
    }

    private static SessionManager getSessionManager(ServletContext servletContext, BdfServer bdfServer) {
        String bdfServerName = "_default";
        String key = SESSIONMANAGER_KEY + "/" + bdfServerName;
        SessionManager sessionManager = (SessionManager) servletContext.getAttribute(key);
        if (sessionManager == null) {
            String intern = bdfServerName.intern();
            synchronized (intern) {
                sessionManager = (SessionManager) servletContext.getAttribute(key);
                if (sessionManager == null) {
                    sessionManager = new SessionManager(bdfServer);
                    servletContext.setAttribute(key, sessionManager);
                }
            }
        }
        return sessionManager;
    }

    private static BdfExtensionManager getBdfExtensionManager(ServletContext servletContext) {
        String key = BDFEXTENSIONMANAGER_KEY;
        BdfExtensionManager bdfExtensionManager = bdfExtensionManager = (BdfExtensionManager) servletContext.getAttribute(key);
        if (bdfExtensionManager == null) {
            String intern = key.intern();
            synchronized (intern) {
                bdfExtensionManager = bdfExtensionManager = (BdfExtensionManager) servletContext.getAttribute(key);
                if (bdfExtensionManager == null) {
                    File dir = new File(servletContext.getRealPath("WEB-INF/bdf-ext"));
                    if (!dir.exists()) {
                        File oldDir = new File(servletContext.getRealPath("WEB-INF/bdfext"));
                        if ((oldDir.exists()) && (oldDir.isDirectory())) {
                            dir = oldDir;
                        }
                    }
                    bdfExtensionManager = new BdfExtensionManager(dir);
                    servletContext.setAttribute(BDFEXTENSIONMANAGER_KEY, bdfExtensionManager);
                }
            }
        }
        return bdfExtensionManager;
    }

    private static SmtpManager getSmtpManager(ServletContext servletContext, HttpServletRequest request) {
        String key = SMTPMANAGER_KEY;
        SmtpManager smtpManager = (SmtpManager) servletContext.getAttribute(key);
        if (smtpManager == null) {
            String intern = key.intern();
            synchronized (intern) {
                smtpManager = (SmtpManager) servletContext.getAttribute(key);
                if (smtpManager == null) {
                    smtpManager = BaseDeFichesInit.initSmtpManager(servletContext, request);
                    servletContext.setAttribute(SMTPMANAGER_KEY, smtpManager);
                }
            }
        }
        return smtpManager;
    }

    public static void init(ServletContext servletContext) {
        synchronized (servletContext) {
            Boolean bool = (Boolean) servletContext.getAttribute(INIT_KEY);
            if (bool != null) {
                return;
            }
            servletContext.setAttribute(INIT_KEY, Boolean.TRUE);
            File implementationPropFile = new File(servletContext.getRealPath("WEB-INF/implementation.properties"));
            if (!implementationPropFile.exists()) {
                return;
            }
            if (implementationPropFile.isDirectory()) {
                return;
            }
            Properties implementationProp = new Properties();
            try {
                InputStream is = new FileInputStream(implementationPropFile);
                implementationProp.load(is);
                is.close();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
            String sacparser = implementationProp.getProperty("sacparser");
            if (sacparser != null) {
                SacParserInit.init(sacparser);
            }
            String csvparser = implementationProp.getProperty("csvparser");
            if (csvparser != null) {
                TableParser.initCsvParser(csvparser);
            }
        }
    }


    private static class InternalBdfContext implements BdfContext {

        private BdfServer bdfServer;
        private Map<String, Object> contextObjectMap = new HashMap<String, Object>();

        InternalBdfContext(BdfServer bdfServer) {
            this.bdfServer = bdfServer;
        }

        public BdfServer getBdfServer() {
            return bdfServer;
        }

        public Object getContextObject(String objectName) {
            return contextObjectMap.get(objectName);
        }

        private void putContextObject(String objectName, Object object) {
            contextObjectMap.put(objectName, object);
        }

    }

}
