/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.extension.*;
import fr.exemole.bdfserver.response.*;
import java.io.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.roles.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractAdministrationCommand extends AbstractBdfCommand {

    protected AdministrationResponseParameters responseParameters;

    public AbstractAdministrationCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        checkParameters();
        if (responseParameters.mayBeDone()) {
            doAdministrationCommand();
        }
        return responseParameters;
    }

    protected CommandTest checkParameters() throws ParameterException {
        responseParameters = new AdministrationResponseParameters(bdfBundle);
        checkAdministrationParameters();
        return responseParameters;
    }

    protected abstract void checkAdministrationParameters() throws ParameterException;

    protected abstract void doAdministrationCommand();

    public int getStepNumber() throws ParameterException {
        String param_step = requestMap.getParameter("step");
        if (param_step == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, "step");
        }
        try {
            int step = Integer.parseInt(param_step);
            if ((step < 1) || (step > 2)) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "step" + " = " + param_step);
            }
            return step;
        } catch (NumberFormatException nfe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "step" + " = " + param_step);
        }
    }

    public boolean checkConfirmPassword() throws ParameterException {
        return RequestUtils.checkConfirmPassword(requestMap, bdfServer);
    }

    public File getBackupFile() {
        File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY);
        String backupName = "bdf-" + bdfServer.getBasedefiches().getBasedefichesMetadata().getBasedefichesName() + ".zip";
        File backupFile = new File(outputDirectory, backupName);
        return backupFile;
    }

    public File getSqlExportFile() {
        File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PUBLIC_OUTPUTDIRECTORY);
        String sqlExportName = "bdf-" + bdfServer.getBasedefiches().getBasedefichesMetadata().getBasedefichesName() + ".sql";
        File sqlExportFile = new File(outputDirectory, sqlExportName);
        return sqlExportFile;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new AdministrationResponsePrinterFactory();
    }

    protected Role initRole() throws ParameterException {
        Role role = RequestUtils.getRole(requestMap, bdfServer.getPermissionManager());
        return role;
    }

}

  
     
     
   
