/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.sphere.*;
import java.util.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheDeleteCommand extends AbstractAdministrationCommand {

    public final static String CORPUS_PARAMNAME = "corpus";
    public final static String ABREGEFICHELIST_PARAMNAME = "abregefichelist";
    private int step;
    private Corpus corpus;
    private List<AbregeFiche> abregeFicheList;

    public FicheDeleteCommand() {
    }

    public static String getCommandString() {
        return "FicheDelete";
    }

    protected void doAdministrationCommand() {
        if (step == 1) {
            responseParameters.putParameter(CORPUS_PARAMNAME, corpus);
            responseParameters.putParameter(ABREGEFICHELIST_PARAMNAME, abregeFicheList);
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.DELETECONFIRM_PAGE);
        } else {
            System.out.println(getLog());
            int result = delete(corpus, abregeFicheList);
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.DELETEFORM_PAGE);
            responseParameters.setDoneMessage("admin_msg_fichessuppr", new Integer(result));
        }
    }

    protected void checkAdministrationParameters() throws ParameterException {
        step = getStepNumber();
        corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
        String[] idcorpusArray = requestMap.getParameterValues("idcorpusarray");
        if ((idcorpusArray == null) || (idcorpusArray.length == 0)) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.DELETEFORM_PAGE);
            responseParameters.setErrorMessage("admin_msg_aucunesuppression", null);
        }
        abregeFicheList = getAbregeFicheList(corpus, idcorpusArray);
        if (abregeFicheList.size() == 0) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.DELETEFORM_PAGE);
            responseParameters.setErrorMessage("admin_msg_aucunesuppression", null);
        }
        if (step != 1) {
            if (!checkConfirmPassword()) {
                responseParameters.putParameter(CORPUS_PARAMNAME, corpus);
                responseParameters.putParameter(ABREGEFICHELIST_PARAMNAME, abregeFicheList);
                responseParameters.setHtmlPageCode(AdministrationResponseParameters.DELETECONFIRM_PAGE);
                responseParameters.setErrorMessage("global_err_password", null);
                return;
            }
        }
    }

    private String getLog() {
        StringBuffer buf = new StringBuffer();
        Date date = new Date();
        buf.append(bdfUser.getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE));
        buf.append(" ");
        buf.append(new Date());
        return buf.toString();
    }

    private List<AbregeFiche> getAbregeFicheList(Corpus corpus, String[] idcorpusArray) {
        List<AbregeFiche> list = new ArrayList();
        for (int i = 0; i < idcorpusArray.length; i++) {
            try {
                int idcorpus = Integer.parseInt(idcorpusArray[i]);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    list.add(abregeFiche);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        return list;
    }

    private int delete(Corpus corpus, List abregeFicheList) {
        CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
        int p = 0;
        for (Iterator it = abregeFicheList.iterator(); it.hasNext();) {
            AbregeFiche abregeFiche = (AbregeFiche) it.next();
            boolean done = corpusEditor.removeFiche(abregeFiche);
            if (done) {
                p++;
            }
        }
        return p;
    }

}
