/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.roles.*;


/**
 *
 * @author Vincent Calame
 */
public class RoleCreationCommand extends AbstractAdministrationCommand {

    public final static String NEWROLE_PARAMNAME = "newrole";
    private String newRoleName;

    public RoleCreationCommand() {
    }

    public static String getCommandString() {
        return "RoleCreation";
    }

    public static String getCommandKey() {
        return "ROL-01";
    }

    public void doAdministrationCommand() {
        bdfServer.getPermissionManager().createRole(newRoleName);
        responseParameters.setDoneMessage("admin_done_rolecreation", newRoleName);
        responseParameters.setHtmlPageCode(AdministrationResponseParameters.ROLEINDEX_PAGE);
    }

    protected void checkAdministrationParameters() throws ParameterException {
        newRoleName = getMandatoryParameter(NEWROLE_PARAMNAME);
        newRoleName = newRoleName.trim();
        if (newRoleName.length() == 0) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.ROLECREATION_PAGE);
            responseParameters.setErrorMessage("admin_err_emptyrolename", newRoleName);
            return;
        }
        if (!StringUtils.isTechnicalName(newRoleName, true)) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.ROLECREATION_PAGE);
            responseParameters.setErrorMessage("admin_err_wrongrolename", newRoleName);
            return;
        }
        Role role = bdfServer.getPermissionManager().getRole(newRoleName);
        if (role != null) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.ROLECREATION_PAGE);
            responseParameters.setErrorMessage("admin_err_existingrolename", newRoleName);
            return;
        }
    }

}
