/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.managers.transformation.*;
import java.text.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.xsltfactory.*;
import fr.exemole.bdfserver.xsltfactory.xslpattern.*;
import net.mapeadores.util.exceptions.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class TransformationCreateCommand extends AbstractAdministrationCommand implements XSLTFactoryConstants {

    public final static String TRANSFORMATIONKEY_PARAMNAME = "transformationkey";
    public final static String TRANSFORMERNAME_PARAMNAME = "transformername";
    public final static String WITHIF_PARAMNAME = "withif";
    public final static String FORMAT_PARAMNAME = "format";
    private short format;
    private boolean withIf = false;
    private String transformerName;
    private TransformationKey transformationKey;

    public TransformationCreateCommand() {
    }

    protected void doAdministrationCommand() {
        String patternString = XSLTPatternBuilder.getXSLTPattern(bdfServer, transformationKey, withIf, format);
        try {
            InputStream inputStream = new ByteArrayInputStream(patternString.getBytes("UTF-8"));
            bdfServer.getTransformationManager().saveTransformer(getKey(), inputStream, true);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        String[] res = {transformerName, transformationKey.toString()};
        responseParameters.setHtmlPageCode(AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE);
        responseParameters.setDoneMessage("admin_done_transformationcreate", res);
    }

    protected void checkAdministrationParameters() throws ParameterException {
        String transformationKeyString = getMandatoryParameter(TRANSFORMATIONKEY_PARAMNAME);
        try {
            transformationKey = TransformationKey.parse(transformationKeyString);
        } catch (ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TRANSFORMATIONKEY_PARAMNAME + " = " + transformationKeyString);
        }
        transformerName = getMandatoryParameter(TRANSFORMERNAME_PARAMNAME);
        transformerName = transformerName.trim();
        if (transformerName.length() == 0) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE);
            responseParameters.setErrorMessage("admin_err_emptytransformername", transformerName);
            return;
        }
        if (!StringUtils.isTechnicalName(transformerName, true)) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE);
            responseParameters.setErrorMessage("admin_err_wrongtransformername", transformerName);
            return;
        }
        withIf = getBooleanParameter(WITHIF_PARAMNAME);
        String formatString = getMandatoryParameter(FORMAT_PARAMNAME);
        try {
            format = XSLTFactoryUtils.formatTypeToShort(formatString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FORMAT_PARAMNAME + " = " + formatString);
        }
        if (transformationKey.isCorpusTransformationKey()) {
            Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(transformationKey.toCorpusKey());
            if (corpus == null) {
                throw new ParameterException(ParameterException.CORPUS_NOTEXIST, transformationKey.toCorpusKey().getSubsetName());
            }
        }
    }

    public static String getCommandString() {
        return "TransformationCreate";
    }

    public static String getCommandKey() {
        return "EXP-01";
    }

    private Object getKey() {
        switch (format) {
            case XSL_FORMAT:
                return new XslTransformerKey(transformationKey, transformerName);
            case ODT_FORMAT:
                return new StreamTransformerKey(transformationKey, "odt", transformerName);
            default:
                throw new SwitchException("format = " + format);
        }
    }

}
