/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.servlets.administration.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class DeleteFormPage extends AdministrationHtmlPage {

    public DeleteFormPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printMessage();
        appendH1("admin_title_sansredac");
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        HTMLAttributes att = new HTMLAttributes();
        att.setName("idcorpusarray");
        att.setChecked(false);
        att.setType(CHECKBOX_TYPE);
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            boolean formStarted = false;
            for (int j = 0; j < corpus.getTicketLength(); j++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                if (abregeFiche != null) {
                    if (abregeFiche.isRemoveable()) {
                        if (!formStarted) {
                            formStarted = true;
                            printFormStart(corpus);
                        }
                        startLi();
                        att.setValue(String.valueOf(abregeFiche.getIdcorpus()));
                        appendInput(att);
                        String href = "Saisie?" + RequestUtils.toParameterString(abregeFiche);
                        startA(href);
                        escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
                        endA();
                        endLi();
                    }
                }
            }
            if (formStarted) {
                endUl();
                endForm("admin_bt_supprimerselection");
            }
        }
        end();
    }

    private void printFormStart(Corpus corpus) {
        startH2();
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endH2();
        HTMLUtils.printFormStart(pw, "Administration", "post", FicheDeleteCommand.getCommandString());
        addParameter(RequestUtils.CORPUS_PARAMNAME, corpus.getSubsetKey().getSubsetName());
        addStepParameter("1");
        startUl();
    }

}
