/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.servlets.administration.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.html.HTMLUtils;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class HorsSphereFormPage extends AdministrationHtmlPage {

    Basedefiches basedefiches;
    int bitmaskRedacteurs = BitmaskUtils.getBitValue(0);

    public HorsSphereFormPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
        basedefiches = bdfServer.getBasedefiches();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1("admin_title_redacsanssphere");
        printMessage();
        startUl();
        /*TicketAllocator horsSphereTicketAllocator = (TicketAllocator) basedefiches.getSubset(SubsetKey.ITEM_HORS_SPHERE);
        for (int i = 0; i < horsSphereTicketAllocator.size();i++) {
        if (!horsSphereTicketAllocator.isEnabledIndex(i)) continue;
        HorsSphereBundle horsSphereBundle = new HorsSphereBundle(i, (FicheItem) horsSphereTicketAllocator.get(i));
        if (horsSphereBundle.isNotEmpty()) {
        horsSphereBundle.print();
        }

        }*/
        endUl();
        end();
    }


    class HorsSphereBundle {

        int ticket;
        FicheItem ficheItem;
        List corpusBundleList = new ArrayList();

        HorsSphereBundle(int ticket, FicheItem ficheItem) {
            this.ticket = ticket;
            this.ficheItem = ficheItem;
            analyse();
        }

        void analyse() {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                analyseCorpus(corpus);
            }
        }

        void analyseCorpus(Corpus corpus) {
            /*MaskCroisement croisement = (MaskCroisement) basedefiches.getCroisement(corpus.getCorpusKey(), SubsetKey.ITEM_HORS_SPHERE);
            CorpusBundle corpusBundle = new CorpusBundle(corpus);
            long[] longArray = croisement.getLongArray(SubsetKey.ITEM_HORS_SPHERE,ticket,Croisement.TICKET_CROSSING_LONG);
            for(int i = 0; i < longArray.length;i++) {
            long l = longArray[i];
            int ficheTicket = PrimUtils.getInt1(l);
            int crossing = PrimUtils.getInt2(l);
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getValueByIndex(ficheTicket);
            if ((crossing & bitmaskRedacteurs) != 0) {
            corpusBundle.addRedacteurFiche(abregeFiche);
            } else {
            corpusBundle.addNoRedacteurFiche(abregeFiche);
            }
            }
            if (corpusBundle.isNotEmpty()) {
            corpusBundleList.add(corpusBundle);
            }*/
        }

        boolean isNotEmpty() {
            return !(corpusBundleList.isEmpty());
        }

        void print() {
            startLi();
            startP();
            escape(FicheItemToString.toString(ficheItem, null));
            endP();
            startUl();
            startLi();
            HTMLUtils.printFormStart(pw, "Administration", "post", RedacteurReplaceCommand.getCommandString());
            addParameter("ticket", String.valueOf(ticket));
            escape("admin_lib_remplacerredac");
            pw.print(" <input type=\"text\" size=\"15\" name=\"nvredac\" value=\"\">");
            HTMLUtils.printFormEnd(pw, "OK", HTMLUtils.ENDFORM_INLINE);
            endLi();
            for (Iterator it = corpusBundleList.iterator(); it.hasNext();) {
                CorpusBundle corpusBundle = (CorpusBundle) it.next();
                corpusBundle.print();
            }
            endUl();
            endLi();
        }

    }


    class CorpusBundle {

        Corpus corpus;
        List ficheList = new ArrayList();
        List ficheHorsRedacteur = new ArrayList();

        CorpusBundle(Corpus corpus) {
            this.corpus = corpus;
        }

        boolean isNotEmpty() {
            return !(ficheList.isEmpty());
        }

        void addRedacteurFiche(AbregeFiche abregeFiche) {
            ficheList.add(abregeFiche);
        }

        void addNoRedacteurFiche(AbregeFiche abregeFiche) {
            ficheHorsRedacteur.add(abregeFiche);
        }

        void print() {
            startLi();
            startP();
            escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
            endP();
            if (ficheList.size() > 0) {
                startUl();
                startLi();
                appendP(null, "admin_horssphere_ficheredac");
                startUl();
                for (Iterator it = ficheList.iterator(); it.hasNext();) {
                    AbregeFiche abregeFiche = (AbregeFiche) it.next();
                    startLi();
                    String href = "Saisie?" + RequestUtils.toParameterString(abregeFiche);
                    startA(href);
                    escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
                    endA();
                    endLi();
                }
                endUl();
                endLi();
                if (ficheHorsRedacteur.size() > 0) {
                    startLi();
                    appendP(null, "admin_horssphere_noficheredac");
                    startUl();
                    for (Iterator it = ficheHorsRedacteur.iterator(); it.hasNext();) {
                        AbregeFiche abregeFiche = (AbregeFiche) it.next();
                        startLi();
                        String href = "Saisie?" + RequestUtils.toParameterString(abregeFiche);
                        startA(href);
                        escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
                        endA();
                        endLi();
                    }
                    endUl();
                    endLi();
                }
                endUl();
            }
            endLi();
        }

    }

}
