/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import fr.exemole.bdfserver.servlets.administration.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.balayages.*;
import fr.exemole.bdfserver.servlets.exportation.htmlpages.*;
import fr.exemole.bdfserver.storage.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.htmlproducers.album.*;
import fr.exemole.bdfserver.htmlproducers.configuration.ConfigurationHtmlConstants;
import net.mapeadores.util.buildinfo.BuildInfo;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdministrationHtmlPage {

    public IndexPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        HTMLAttributes ulLinkAttr = UlAttrs.getLinkAttr();
        start();
        printMessage();
        startUl(ulLinkAttr);
        StorageProcessLog storageProcessLog = bdfServer.getStorageProcessLog();
        if (storageProcessLog.hasLog()) {
            printLink("admin/storageprocesslog.txt", "admin_lib_storageprocesslog");
        }
        printLink("Importation", "saisie_title_massindex");
        printPageLink(DefaultCommand.DELETEFORM_PAGE_PARAMVALUE, "admin_title_sansredac");
        printPageLink(DefaultCommand.HORSSPHEREFORM_PAGE_PARAMVALUE, "admin_title_redacsanssphere");
        printLink("admin/globalstats.html", "admin_title_globalstats");
        printPageLink("Association", null, "association_title_index");
        printPageLink(DefaultCommand.SESSION_PAGE_PARAMVALUE, "admin_title_sessions");
        printPageLink("Configuration?page=" + ConfigurationHtmlConstants.COREFORM_PAGE, null, "configuration_title_general");
        printPageLink("Corpus", null, "corpus_title_index");
        printPageLink("Album", AlbumHtmlConstants.ALBUMINDEX_PAGE, "album_title_index");
        printPageLink("Administration", DefaultCommand.ROLEINDEX_PAGE_PARAMVALUE, "admin_title_roleindex");
        printPageLink("Exportation", fr.exemole.bdfserver.servlets.exportation.DefaultCommand.SCRUTARIEXPORTINDEX_PAGE_PARAMVALUE, "exportation_title_scrutariexportindex");
        printPageLink("Exportation", fr.exemole.bdfserver.servlets.exportation.DefaultCommand.SQLEXPORTINDEX_PAGE_PARAMVALUE, "exportation_title_sqlexportindex");
        printPageLink("Administration", DefaultCommand.TRANSFORMATIONINDEX_PAGE_PARAMVALUE, "admin_title_transformationindex");
        printPageLink("Administration", DefaultCommand.TABLEEXPORTINDEX_PAGE_PARAMVALUE, "admin_title_tableexportindex");

        BalayageManager balayageManager = bdfServer.getBalayageManager();
        int count = balayageManager.getBalayageCount();
        if (count > 0) {
            startLi(LiAttrs.getGroupAttr());
            appendP(null, "exportation_title_balayageindex");
            startUl(ulLinkAttr);
            for (int i = 0; i < count; i++) {
                Balayage balayage = balayageManager.getBalayage(i);
                startLi();
                startP();
                ExportationHtmlPage.printBalayageLinks(this, balayage);
                endP();
                endLi();
            }
            startLi();
            startP();
            startSmall();
            append("(");
            appendA("Administration?page=" + DefaultCommand.BALAYAGEINDEX_PAGE_PARAMVALUE + "&reinit=1", "admin_link_balayageupdate");
            append(")");
            endSmall();
            endP();
            endLi();
            endUl();
            endLi();
        } else {
            printLink("Administration?page=" + DefaultCommand.BALAYAGEINDEX_PAGE_PARAMVALUE + "&reinit=1", "admin_link_balayageupdate");
        }

        endUl();
        startP();
        startSmall();
        printLibColon("admin_lib_memory");
        Runtime run = Runtime.getRuntime();
        append(String.valueOf((int) Math.floor(((double) run.freeMemory()) / (1024 * 1024))));
        append("/");
        append(String.valueOf((int) Math.ceil(((double) run.totalMemory()) / (1024 * 1024))));
        append("/");
        append(String.valueOf((int) Math.ceil(((double) run.maxMemory()) / (1024 * 1024))));
        endSmall();
        endP();
        startP();
        startSmall();
        printLibColon("admin_lib_softwareversion");
        BuildInfo buildInfo = bdfServer.getBuildInfo();
        if (buildInfo != null) {
            escape(buildInfo.getSoftwareVersion());
        }
        endSmall();
        endP();

        /*startP();
        startSmall();
        escape(System.getProperty("javax.xml.transform.TransformerFactory"));
        append(" / ");
        try {
        TransformerFactory te = TransformerFactory.newInstance();
        escape(te.getClass().getName());
        }
        catch (Exception e) {
        }



        endSmall();
        endP();*/

        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Administration");
        commandBox.setCommandString(BackupCommand.getCommandString());
        commandBox.setCommandKey(BackupCommand.getCommandKey());
        startCommandBox(commandBox);
        endCommandBox("admin_submit_backup");

        end();
    }

    private void printPageLink(String pageParameter, String key) {
        printPageLink("Administration", pageParameter, key);
    }

    private void printPageLink(String scope, String pageParameter, String key) {
        StringBuffer buf = new StringBuffer();
        buf.append(scope);
        if (pageParameter != null) {
            buf.append("?page=");
            buf.append(pageParameter);
        }
        printLink(buf.toString(), key);
    }

    private void printLink(String href, String key) {
        startLi();
        startP();
        appendA(href, key);
        endP();
        endLi();
    }

}
