/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.administration.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.servlets.util.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.logging.*;


/**
 *
 * @author Vincent Calame
 */
public class TableExportIndexPage extends AdministrationHtmlPage {

    private TableExportManager manager;

    public TableExportIndexPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
        manager = bdfServer.getTableExportManager();
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1("admin_title_tableexportindex");
        printMessage();
        startUl(UlAttrs.getLinkAttr());
        Set<String> set = manager.getTableExportNameSet();
        for (String tableExportName : set) {
            printValidTableExportLi(tableExportName);
        }
        Set<String> set2 = manager.getWrongNameSet();
        for (String name : set2) {
            printUnvalidTableExportLi(name);
        }
        startLi();
        startP();
        startSmall();
        append("(");
        appendA("Administration?page=" + DefaultCommand.TABLEEXPORTINDEX_PAGE_PARAMVALUE + "&reinit=1", "admin_link_tableexportupdate");
        append(")");
        endSmall();
        endP();
        endLi();
        endUl();
        printTableExportCreateForm();
        end();
    }

    private void printValidTableExportLi(String tableExportName) {
        startLi();
        startP();
        startCode();
        append(tableExportName);
        endCode();
        endP();
        TableExport tableExport = manager.getTableExport(tableExportName);
        SubsetKey[] array = tableExport.getSubsetKeyArray();
        startUl();
        for (int i = 0; i < array.length; i++) {
            startLi();
            startP();
            startCode();
            append(array[i].toString());
            append(".txt");
            endCode();
            endP();
            endLi();
        }
        printProcessLog(tableExportName);
        endUl();
        endLi();
    }

    private void printUnvalidTableExportLi(String name) {
        startLi();
        startP();
        startSpan(ERRORMESSAGE_SPANCLASS);
        startCode();
        append(name);
        endCode();
        endSpan();
        endP();
        startUl();
        printProcessLog(name);
        endUl();
        endLi();
    }

    private void printProcessLog(String name) {
        ProcessLog processLog = manager.getProcessLog(name);
        if (processLog.hasLog()) {
            startLi();
            startP(CssClass.P_SUBTITLE);
            printLibColon("admin_lib_error_process");
            endP();
            startP();
            startPre();
            escape(ProcessLogUtils.toString(processLog), true);
            endPre();
            endP();
            endLi();
        }
    }

    private void printTableExportCreateForm() {
        TableExportParameters.Printer printer = new TableExportParameters.Printer(new TableExportParametersBean(), this);
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Administration");
        commandBox.setCommandString(TableExportCreateCommand.getCommandString());
        commandBox.setCommandKey(TableExportCreateCommand.getCommandKey());
        startCommandBox(commandBox);
        startP();
        printLibColon("admin_lib_tableexportname");
        appendMediumInput(TableExportParameters.TABLEEXPORTNAME_PARAMNAME, "");
        endP();
        startP();
        printer.printBooleanParameters();
        appendBr();
        printer.printIndexationRadios();
        endP();
        endCommandBox("admin_submit_tableexportcreate");
    }

}
