/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.*;
import net.mapeadores.util.xml.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.*;
import net.mapeadores.basedefiches.xml.extraction.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.util.io.*;


/**
 *
 *
 * @author Vincent Calame
 */
public abstract class AffichageContentStream extends AbstractContentStream implements AffichageConstants {

    public AffichageContentStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
    }

    public static ResponseHandler getInstance(BdfResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case FICHE_HTML_STREAM:
            case FICHE_OCTET_STREAM:
            case FICHE_XMLXSL_STREAM:
            case FICHE_XML_STREAM:
                return new FicheStream(responseParameters);
            case THESAURUS_ODS_STREAM:
                return new ThesaurusOdsStream(responseParameters);
            case THS_STATS_XML_STREAM:
            case THS_STATS_HTML_STREAM:
            case THS_STATSSEL_XML_STREAM:
            case THS_STATSSEL_HTML_STREAM:
                return new StatsThesaurusStream(responseParameters);
            case THS_STATS_ODS_STREAM:
            case THS_STATSSEL_ODS_STREAM:
                return new StatsThesaurusOdsStream(responseParameters);
            case MOTCLE_HTML_STREAM:
            case MOTCLE_XML_STREAM:
                return new MotcleStream(responseParameters);
            case COMPILATION_HTML_STREAM:
            case COMPILATION_XML_STREAM:
            case COMPILATION_OCTET_STREAM:
                return new CompilationStream(responseParameters);
            case THS_LIBS_XML_STREAM:
                return new LibsStream(responseParameters);
            case THS_INVERSE_XML_STREAM:
            case THS_INVERSE_HTML_STREAM:
            case THS_INVERSE_ODS_STREAM:
                return new InverseThesaurusStream(responseParameters);
            case UI_ODS_STREAM:
                return new UIOdsStream(responseParameters);
            case SPHERE_ODS_STREAM:
                return new SphereOdsStream(responseParameters);
            case STORAGEPROCESSLOG_TXT_STREAM:
                return new StorageProcessLogTxtStream(responseParameters);
            case BALAYAGEUNIT_OCTET_STREAM:
                return new BalayageUnitStream(responseParameters);
            case IMAGE_OCTET_STREAM:
                return new ImageStream(responseParameters);
        }
        return null;
    }

    protected void printHtml(HttpServletResponse reponse, Extraction extraction, Object extractionSource, TransformationKey transformationKey) throws IOException {
        printHtml(reponse, ExtractionXMLWriter.toStreamSource(extraction, extractionSource), transformationKey);
    }

    protected void printHtml(HttpServletResponse reponse, Extraction extraction, Object extractionSource, Transformer transformer) throws IOException {
        printHtml(reponse, ExtractionXMLWriter.toStreamSource(extraction, extractionSource), transformer);
    }

    protected void printHtml(HttpServletResponse reponse, String sourceString, TransformationKey transformationKey) throws IOException {
        StringReader stringReader = new StringReader(sourceString);
        StreamSource streamSource = new StreamSource(stringReader);
        printHtml(reponse, streamSource, transformationKey);
    }

    private void printHtml(HttpServletResponse reponse, StreamSource streamSource, TransformationKey transformationKey) throws IOException {
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        Transformer transformer = bdfServer.getTransformationManager().getXslTransformer(xslTransformerKey);
        TransformerParameters transformerParameters = new TransformerParameters();
        transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
        transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, "../");
        transformerParameters.initParameters(transformer);
        printHtml(reponse, streamSource, transformer);
    }

    private void printHtml(HttpServletResponse reponse, StreamSource streamSource, Transformer transformer) throws IOException {
        String method = transformer.getOutputProperty("method");
        if (method == null) {
            method = "xml";
        }
        String encoding = transformer.getOutputProperty("encoding");
        if (encoding == null) {
            encoding = "UTF-8";
        }
        if (method.equals("xml")) {
            reponse.setContentType("text/xml;charset=" + encoding);
        } else if (method.equals("html")) {
            reponse.setContentType("text/html;charset=" + encoding);
        } else {
            reponse.setContentType("text/plain;charset=" + encoding);
        }
        try {
            transformer.transform(streamSource, new StreamResult(reponse.getOutputStream()));
        } catch (TransformerException te) {
            throw new NestedTransformerException(te);
        }
    }

    protected void printXml(HttpServletResponse reponse, Extraction extraction, Object extractionSource, String stylesheetPath) throws IOException {
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        int nbretabulations = 0;
        XMLUtils.printXMLDeclaration(pw);
        if (stylesheetPath != null) {
            XMLUtils.printXMLStylesheet(pw, "text/xsl", "../" + stylesheetPath);
            nbretabulations = -999;
        }
        ExtractionXMLWriter xmlWriter = new ExtractionXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(pw);
        xmlWriter.setIndentLength(nbretabulations);
        xmlWriter.addExtraction(extraction, new Date(), extractionSource);
        pw.close();
    }

    public void writeStream(HttpServletResponse reponse, TransformationKey transformationKey, Extraction extraction, Object extractionSource) throws IOException {
        StreamTransformer streamTransformer = getStreamTransformer(transformationKey);
        if (streamTransformer == null) {
            throw new HttpAccessException(HttpAccessException.NOT_FOUND);
        }
        try {
            Source xmlSource = ExtractionXMLWriter.toStreamSource(extraction, extractionSource);
            reponse.setContentType(streamTransformer.getMimeType());
            TransformerParameters transformerParameters = new TransformerParameters();
            transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
            transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, bdfUser.getBaseDeFichesURL());
            streamTransformer.transform(xmlSource, reponse.getOutputStream(), transformerParameters.toMap());
        } catch (TransformerException te) {
            throw new NestedTransformerException(te);
        }
    }

    private StreamTransformer getStreamTransformer(TransformationKey transformationKey) {
        String extension = (String) responseParameters.getParameter(EXTENSION_PARAMNAME);
        StreamTransformerKey streamTransformerKey = bdfUser.getStreamTransformerKey(transformationKey, extension);
        if (streamTransformerKey == null) {
            streamTransformerKey = StreamTransformerKey.getDefaultStreamTransformerKey(transformationKey, extension);
        }
        return bdfServer.getTransformationManager().getStreamTransformer(streamTransformerKey);
    }

}
