/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;
import net.mapeadores.util.io.*;
import fr.exemole.bdfserver.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class ImageStream extends AffichageContentStream implements AffichageConstants {

    private AlbumImage albumImage;
    private AlbumDim albumDim;

    public ImageStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        albumImage = (AlbumImage) responseParameters.getParameter(ALBUMIMAGE_OBJ);
        albumDim = (AlbumDim) responseParameters.getParameter(ALBUMDIM_OBJ);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        InputStream inputStream = albumImage.getInputStream(albumDim);
        if (inputStream == null) {
            throw new HttpAccessException(HttpAccessException.INTERNAL_ERROR, "no inputstream for " + AlbumUtils.toFileName(albumImage));
        }
        response.setContentType(AlbumUtils.getMimeType(albumImage.getType()));
        OutputStream output = response.getOutputStream();
        IOUtils.copy(inputStream, output);
        inputStream.close();
        output.close();
    }

}

