/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.misc.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.text.lexie.xml.*;
import fr.exemole.bdfserver.managers.transformation.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.opendocument.io.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.tools.odsexport.*;


/**
 *
 * @author Vincent Calame
 */
public class InverseThesaurusStream extends AffichageContentStream {

    Thesaurus thesaurus;
    short contentStreamType;
    LexieMap lexieMap;

    public InverseThesaurusStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        thesaurus = (Thesaurus) responseParameters.getParameter(THESAURUS_OBJ);
        contentStreamType = responseParameters.getResponseCode();
        int langInteger = responseParameters.getLocalisationInfo().getWorkingLangInteger();
        langInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, langInteger);
        lexieMap = ThesaurusInverseFactory.getLibelleLexieMap(thesaurus, langInteger);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        switch (contentStreamType) {
            case THS_INVERSE_XML_STREAM:
                printXml(response);
                break;
            case THS_INVERSE_HTML_STREAM:
                printHtml(response, LexieMapXMLWriter.getLexieMapXML(lexieMap, false), TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY);
            case THS_INVERSE_ODS_STREAM:
                printOds(response);
                break;
        }
    }

    private void printXml(HttpServletResponse reponse) throws IOException {
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        LexieMapXMLWriter writer = new LexieMapXMLWriter();
        writer.setAppendable(pw);
        writer.setIndentLength(0);
        writer.appendXMLDeclaration();
        writer.appendLexieMap(lexieMap);
        pw.close();
    }

    private void printOds(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        LexieOdsWriter xmlWriter = new LexieOdsWriter(thesaurus, lexieMap, bdfServer);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(xmlWriter);
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
        outputStream.close();
    }


    private static class LexieOdsWriter extends SpreadSheetXMLWriter {

        private LexieMap lexieMap;
        private BdfServer bdfServer;
        private Thesaurus thesaurus;
        private boolean withIdalpha;

        LexieOdsWriter(Thesaurus thesaurus, LexieMap lexieMap, BdfServer bdfServer) {
            this.lexieMap = lexieMap;
            this.bdfServer = bdfServer;
            this.thesaurus = thesaurus;
            withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        }

        public void printTable() throws IOException {
            String thesaurusName = thesaurus.getSubsetKey().getSubsetName();
            openTable(thesaurusName);
            printFirstRow();
            for (Iterator it = lexieMap.getCollatedKeyMap().values().iterator(); it.hasNext();) {
                TextLexieUnit lexieUnit = (TextLexieUnit) it.next();
                printLexieUnit(lexieUnit);
            }
            closeTable();
        }

        private void printLexieUnit(TextLexieUnit lexieUnit) throws IOException {
            String canonical = lexieUnit.getCanonicalLexie();
            List list = lexieUnit.getOccurenceList();
            boolean premier = true;
            LexieSource previousLexieSource = null;
            for (Iterator it = list.iterator(); it.hasNext();) {
                Occurence occurence = (Occurence) it.next();
                LexieSource lexieSource = occurence.getParsedLexieSource().getLexieSource();
                if (previousLexieSource != null) {
                    if (previousLexieSource.equals(lexieSource)) {
                        continue;
                    }
                }
                previousLexieSource = lexieSource;
                openTableRow();
                if (premier) {
                    addStringTableCell(canonical);
                    premier = false;
                } else {
                    addEmptyTableCell();
                }
                addStringTableCell(lexieSource.getSourceText());
                Motcle motcle = (Motcle) lexieSource.getSourceObject();
                addIntegerTableCell(motcle.getIdths());
                if (withIdalpha) {
                    addStringTableCell(motcle.getIdalpha());
                }
                addStringTableCell(canonical);
                closeTableRow();
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("lexie");
            addStringTableCell("lib");
            addStringTableCell("idths");
            if (withIdalpha) {
                addStringTableCell("idalpha");
            }
            addStringTableCell("lexie_rep");
            closeTableRow();
        }

    }

}
