/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.beans.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.opendocument.io.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.sphere.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.tools.odsexport.*;


/**
 *
 * @author Vincent Calame
 */
public class SphereOdsStream extends AffichageContentStream {

    private Sphere sphere;

    public SphereOdsStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        sphere = (Sphere) responseParameters.getParameter(SPHERE_OBJ);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(new SphereXMLWriter(sphere, bdfServer));
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
        outputStream.close();
    }


    private static class SphereXMLWriter extends SpreadSheetXMLWriter {

        private Sphere sphere;
        private BdfServer bdfServer;
        private TreeMap<String, Redacteur> actifMap = new TreeMap<String, Redacteur>();
        private TreeMap<String, Redacteur> inactifMap = new TreeMap<String, Redacteur>();
        private PermissionManager permissionManager;
        private boolean all = false;

        SphereXMLWriter(Sphere sphere, BdfServer bdfServer) {
            this.sphere = sphere;
            this.bdfServer = bdfServer;
            this.permissionManager = bdfServer.getPermissionManager();
            this.all = (sphere == null);
            init();
        }

        private void init() {
            if (all) {
                Basedefiches basedefiches = bdfServer.getBasedefiches();
                int sphereCount = basedefiches.getSphereCount();
                for (int j = 0; j < sphereCount; j++) {
                    Sphere sphere = basedefiches.getSphere(j);
                    for (int i = 0; i < sphere.getTicketLength(); i++) {
                        Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
                        if (redacteur != null) {
                            Map map = (redacteur.isActif()) ? actifMap : inactifMap;
                            map.put(redacteur.getRedacteurKey().toString(RedacteurKey.UNDERSCORE_STYLE), redacteur);
                        }
                    }
                }
            } else {
                for (int i = 0; i < sphere.getTicketLength(); i++) {
                    Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
                    if (redacteur != null) {
                        Map map = (redacteur.isActif()) ? actifMap : inactifMap;
                        map.put(redacteur.getRedacteurKey().getIdsphereString(), redacteur);
                    }
                }
            }
        }

        public void printTable() throws IOException {
            String name = (sphere != null) ? sphere.getSubsetKey().getSubsetName() : "_all";
            openTable(name);
            printFirstRow();
            printMap(actifMap);
            printMap(inactifMap);
            closeTable();
        }

        private void printMap(Map<String, Redacteur> map) throws IOException {
            for (Map.Entry<String, Redacteur> mapEntry : map.entrySet()) {
                String strg_idsphere = mapEntry.getKey();
                Redacteur redacteur = mapEntry.getValue();
                printRedacteur(redacteur);
            }
        }

        private void printRedacteur(Redacteur redacteur) throws IOException {
            RedacteurKey redacteurKey = redacteur.getRedacteurKey();
            openTableRow();
            int actif = (redacteur.isActif()) ? 1 : 0;
            addIntegerTableCell(actif);
            if (all) {
                addStringTableCell(redacteurKey.getSphereKey().getSubsetName());
            }
            String idsphere = redacteurKey.getIdsphereString();
            addStringTableCell(idsphere);
            PersonneBean personneBean = redacteur.getPersonneBean();
            addStringTableCell(personneBean.getNom());
            addStringTableCell(personneBean.getPrenom());
            int nomavant = (personneBean.isNomAvant()) ? 1 : 0;
            addIntegerTableCell(nomavant);
            CourrielBean courrielBean = redacteur.getCourrielBean();
            if (courrielBean != null) {
                addStringTableCell(courrielBean.getAddrSpec());
            } else {
                addEmptyTableCell();
            }
            if (actif == 1) {
                int admin = (permissionManager.isAdmin(redacteurKey)) ? 1 : 0;
                addIntegerTableCell(admin);

                BdfUser redacUser = new BdfUser(bdfServer, redacteurKey);
                String[] roleArray = redacUser.getRoleNameArray();
                if (roleArray == null) {
                    addStringTableCell("_default");
                } else if (roleArray.length == 0) {
                    addEmptyTableCell();
                } else {
                    StringBuffer buf = new StringBuffer();
                    for (int i = 0; i < roleArray.length; i++) {
                        if (i > 0) {
                            buf.append("; ");
                        }
                        buf.append(roleArray[i]);
                    }
                    addStringTableCell(buf.toString());
                }
            }
            closeTableRow();
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("actif");
            if (all) {
                addStringTableCell("sphere");
            }
            addStringTableCell("idsphere");
            addStringTableCell("nom");
            addStringTableCell("prenom");
            addStringTableCell("nomavant");
            addStringTableCell("courriel");
            addStringTableCell("admin");
            addStringTableCell("roles");
            closeTableRow();
        }

    }

}
