/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.opendocument.io.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.tools.odsexport.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class StatsThesaurusOdsStream extends AffichageContentStream {

    private Thesaurus thesaurus;
    private FicheEligibility ficheEligibility;

    public StatsThesaurusOdsStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        thesaurus = (Thesaurus) responseParameters.getParameter(THESAURUS_OBJ);
        short contentStreamType = responseParameters.getResponseCode();
        ficheEligibility = getFicheEligibility(contentStreamType);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        StatsThesaurusXMLWriter xmlWriter = new StatsThesaurusXMLWriter(thesaurus, bdfServer, ficheEligibility, ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, workingLangInteger));
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(xmlWriter);
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
        outputStream.close();
    }

    private FicheEligibility getFicheEligibility(short contentStreamType) {
        if (contentStreamType != THS_STATSSEL_ODS_STREAM) {
            return null;
        }
        return bdfUser.getAbregeFicheSortedList();
    }


    private static class StatsThesaurusXMLWriter extends SpreadSheetXMLWriter {

        private Thesaurus thesaurus;
        private BdfServer bdfServer;
        private FicheEligibility ficheEligibility;
        private SubsetKey thesaurusKey;
        private Croisement[] croisements;
        private Corpus[] corpusArray;
        private boolean withIdalpha;
        private boolean isBabelien;
        private int thesaurusLangInteger;
        private MotcleStats motcleStats;

        StatsThesaurusXMLWriter(Thesaurus thesaurus, BdfServer bdfServer, FicheEligibility ficheEligibility, int thesaurusLangInteger) {
            this.thesaurus = thesaurus;
            this.bdfServer = bdfServer;
            this.ficheEligibility = ficheEligibility;
            thesaurusKey = thesaurus.getSubsetKey();
            ThesaurusMetadata thesaurusMetaData = thesaurus.getThesaurusMetadata();
            short thesaurusType = thesaurusMetaData.getThesaurusType();
            withIdalpha = (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE);
            isBabelien = (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE);
            this.thesaurusLangInteger = thesaurusLangInteger;
            initCroisements();
            motcleStats = new MotcleStats();
        }

        private void initCroisements() {
            Basedefiches basedefiches = thesaurus.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            croisements = new Croisement[corpusCount];
            corpusArray = new Corpus[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
                croisements[i] = croisement;
                corpusArray[i] = corpus;
            }
        }

        public void printTable() throws IOException {
            String thesaurusName = thesaurusKey.getSubsetName();
            openTable(thesaurusName);
            printFirstRow();
            MotcleList motcleList = thesaurus.getFirstLevelList();
            for (int i = 0; i < motcleList.getMotcleCount(); i++) {
                printMotcleRow(motcleList.getMotcle(i));
            }
            closeTable();
        }

        private void printMotcleRow(Motcle motcle) throws IOException {
            motcleStats.updateStats(motcle);
            int total = motcleStats.getTotal();
            openTableRow();
            addIntegerTableCell(motcle.getIdths());
            if (withIdalpha) {
                addStringTableCell(motcle.getIdalpha());
            }
            if (isBabelien) {
                addStringTableCell(motcle.getLibelleString(thesaurusLangInteger, true));
            } else {
                addStringTableCell(motcle.getLibelleString(thesaurusLangInteger, true));
            }
            addIntegerTableCell(total);
            int[] parCroisement = motcleStats.getParCroisement();
            for (int i = 0; i < parCroisement.length; i++) {
                int ct = parCroisement[i];
                if (ct == 0) {
                    addEmptyTableCell();
                } else {
                    addIntegerTableCell(ct);
                }
            }
            closeTableRow();
            MotcleList motcleList = motcle.getChildList();
            for (int i = 0; i < motcleList.getMotcleCount(); i++) {
                printMotcleRow(motcleList.getMotcle(i));
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("idths");
            if (withIdalpha) {
                addStringTableCell("idalpha");
            }
            if (isBabelien) {
                addStringTableCell("lib");
            } else {
                addStringTableCell("lib_" + LangInteger.toString(thesaurusLangInteger));
            }
            addStringTableCell("_total");
            int croisementLength = croisements.length;
            for (int i = 0; i < croisementLength; i++) {
                String s = croisements[i].getSubsetCouple().getSubsetKey1().getSubsetName();
                addStringTableCell(s);
            }
            closeTableRow();
        }


        private class MotcleStats {

            private int total = 0;
            private int[] parCroisement;

            MotcleStats() {
                parCroisement = new int[croisements.length];
            }

            void updateStats(Motcle motcle) {
                total = 0;
                int length = croisements.length;
                for (int i = 0; i < length; i++) {
                    Croisement croisement = croisements[i];
                    CroisementItem ci = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
                    int ct = 0;
                    int size = ci.getCount();
                    if (ficheEligibility == null) {
                        ct = size;
                    } else {
                        Corpus corpus = corpusArray[i];
                        for (int j = 0; j < size; j++) {
                            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ci.getTicket(j));
                            if (abregeFiche != null) {
                                if (ficheEligibility.accept(abregeFiche)) {
                                    ct++;
                                }
                            }
                        }
                    }
                    parCroisement[i] = ct;
                    total = total + ct;
                }
            }

            public int getTotal() {
                return total;
            }

            public int[] getParCroisement() {
                return parCroisement;
            }

        }

    }

}
