/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.managers.transformation.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class StatsThesaurusStream extends AffichageContentStream {

    Thesaurus thesaurus;
    Extraction extraction;
    short contentStreamType;

    public StatsThesaurusStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        thesaurus = (Thesaurus) responseParameters.getParameter(THESAURUS_OBJ);
        contentStreamType = responseParameters.getResponseCode();
        FicheEligibility ficheEligibility = getFicheEligibility(contentStreamType);
        extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        MotcleFilter motcleFilter = MotcleFilter.getThesaurusInstance(ficheEligibility);
        MotcleFilterWrapper wrapper = new MotcleFilterWrapper(motcleFilter);
        extraction.setDynamicMotcleFilterWrapper(wrapper);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        switch (contentStreamType) {
            case THS_STATS_XML_STREAM:
            case THS_STATSSEL_XML_STREAM:
                printXml(response, extraction, thesaurus, null);
                break;
            case THS_STATS_HTML_STREAM:
            case THS_STATSSEL_HTML_STREAM:
                printHtml(response, extraction, thesaurus, TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY);
                break;
        }
    }

    private FicheEligibility getFicheEligibility(short contentStreamType) {
        if ((contentStreamType != THS_STATSSEL_XML_STREAM) && (contentStreamType != THS_STATSSEL_HTML_STREAM)) {
            return null;
        }
        return bdfUser.getAbregeFicheSortedList();
    }

}
