/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.opendocument.io.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.tools.odsexport.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.tableexport.thesaurus.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusOdsStream extends AffichageContentStream {

    private Thesaurus thesaurus;

    public ThesaurusOdsStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        thesaurus = (Thesaurus) responseParameters.getParameter(THESAURUS_OBJ);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(new ThesaurusXMLWriter(thesaurus, bdfServer));
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
        outputStream.close();
    }


    private static class ThesaurusXMLWriter extends SpreadSheetXMLWriter implements ThesaurusTableWriter {

        private Thesaurus thesaurus;
        private BdfServer bdfServer;
        private SubsetKey thesaurusKey;
        private boolean withIdalpha;
        private boolean isBabelien;
        private int[] codeslangues;
        private ThesaurusTableExport thesaurusTableExport;

        ThesaurusXMLWriter(Thesaurus thesaurus, BdfServer bdfServer) {
            thesaurusTableExport = ThesaurusTableExport.getDefault(thesaurus);
            this.thesaurus = thesaurus;
            this.bdfServer = bdfServer;
            thesaurusKey = thesaurus.getSubsetKey();
            ThesaurusMetadata thesaurusMetaData = thesaurus.getThesaurusMetadata();
            short thesaurusType = thesaurusMetaData.getThesaurusType();
            withIdalpha = (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE);
            isBabelien = (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE);
            if (!isBabelien) {
                codeslangues = thesaurusMetaData.getAuthorizedLangIntegerArray();
            }
        }

        public void printTable() throws IOException {
            String name = thesaurusKey.getSubsetName();
            openTable(name);
            printFirstRow();
            try {
                thesaurusTableExport.export(this);
            } catch (NestedIOException ioe) {
                throw ioe.getIOException();
            }
            closeTable();
        }

        private void printFirstRow() throws IOException {
            ThesaurusFieldKey[] thesaurusFieldKeyArray = thesaurusTableExport.getThesaurusFieldKeyArray();
            openTableRow();
            for (int i = 0; i < thesaurusFieldKeyArray.length; i++) {
                addStringTableCell(thesaurusFieldKeyArray[i].toString());
            }
            closeTableRow();
        }

        public void startRow(Motcle motcle) {
            try {
                openTableRow();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addIdthsCell(ThesaurusFieldKey thesaurusFieldKey, Integer itg) {
            try {
                if (itg != null) {
                    addIntegerTableCell(itg.intValue());
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addIdalphaCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
            try {
                if (s != null) {
                    addStringTableCell(s);
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addLibCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
            try {
                if (s != null) {
                    addStringTableCell(s);
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addBabelienLangCell(int langInteger) {
            try {
                String lg = LangInteger.toString(langInteger);
                addStringTableCell(lg);
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void endRow() {
            try {
                closeTableRow();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

    }

}
