/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.opendocument.io.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.sphere.metadata.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.tools.odsexport.*;


/**
 *
 * @author Vincent Calame
 */
public class UIOdsStream extends AffichageContentStream {

    public UIOdsStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        odZip.setContentZipEntrySource(new SpreadSheetZipEntrySource(new UiXMLWriter()));
        odZip.zip(outputStream);
        outputStream.close();
    }


    private class UiXMLWriter extends SpreadSheetXMLWriter {

        UIManager uiManager;
        Basedefiches basedefiches;
        int[] langIntegerArray;

        UiXMLWriter() {
            uiManager = bdfServer.getUIManager();
            basedefiches = bdfServer.getBasedefiches();
            langIntegerArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        }

        public void printTable() throws IOException {
            openTable("libs");
            printFirstRow();
            printMetadata();
            printThesaurus();
            printSphere();
            printCorpus();
            closeTable();
        }

        private void printMetadata() throws IOException {
            BasedefichesMetadata metadata = basedefiches.getBasedefichesMetadata();
            int intituleCount = metadata.getIntituleCount();
            for (int i = 0; i < intituleCount; i++) {
                Intitule intitule = metadata.getIntitule(i);
                String key = "basedefiches/intitule_" + intitule.getName();
                printRow(key, intitule);
            }
        }

        private void printThesaurus() throws IOException {
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int j = 0; j < thesaurusCount; j++) {
                Thesaurus thesaurus = basedefiches.getThesaurus(j);
                String rootKey = thesaurus.getSubsetKey().toString();
                ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
                int intituleCount = metadata.getIntituleCount();
                for (int i = 0; i < intituleCount; i++) {
                    Intitule intitule = metadata.getIntitule(i);
                    String key = rootKey + "/intitule_" + intitule.getName();
                    printRow(key, intitule);
                }
            }
        }

        private void printSphere() throws IOException {
            int sphereCount = basedefiches.getSphereCount();
            for (int j = 0; j < sphereCount; j++) {
                Sphere sphere = basedefiches.getSphere(j);
                String rootKey = sphere.getSubsetKey().toString();
                SphereMetadata metadata = sphere.getSphereMetadata();
                int intituleCount = metadata.getIntituleCount();
                for (int i = 0; i < intituleCount; i++) {
                    Intitule intitule = metadata.getIntitule(i);
                    String key = rootKey + "/intitule_" + intitule.getName();
                    printRow(key, intitule);

                }
            }
        }

        private void printCorpus() throws IOException {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                printCorpus(basedefiches.getCorpus(i));
            }
        }

        private void printCorpus(Corpus corpus) throws IOException {
            String rootKey = corpus.getSubsetKey().toString();
            CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
            int intituleCount = corpusMetadata.getIntituleCount();
            for (int i = 0; i < intituleCount; i++) {
                Intitule intitule = corpusMetadata.getIntitule(i);
                String key = rootKey + "/intitule_" + intitule.getName();
                printRow(key, intitule);

            }
            for (Iterator it = CorpusMetadataUtils.getCorpusFieldList(corpusMetadata).iterator(); it.hasNext();) {
                CorpusField corpusField = (CorpusField) it.next();
                String key = rootKey + "/" + corpusField.getFieldKey().toString();
                printRow(key, corpusField);
            }
            CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
            List list = corpusUI.getOrderedComponentUIList(CorpusUI.INCLUDEUI_LIST);
            for (Iterator it = list.iterator(); it.hasNext();) {
                IncludeUI includeUI = (IncludeUI) it.next();
                if (!includeUI.useDefaultLibelle()) {
                    String key = rootKey + "/" + includeUI.getFiteredIncludeKey().toString();
                    printRow(key, includeUI.getCustomLibelle());
                }
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("key");
            for (int i = 0; i < langIntegerArray.length; i++) {
                int langInteger = langIntegerArray[i];
                addStringTableCell("lib_" + LangInteger.toString(langInteger));
            }
            closeTableRow();
        }

        private void printRow(String key, LibelleHolder libelleHolder) throws IOException {
            openTableRow();
            addStringTableCell(key);
            for (int i = 0; i < langIntegerArray.length; i++) {
                int langInteger = langIntegerArray[i];
                Libelle libelle = libelleHolder.getLibelleByLang(langInteger);
                if (libelle == null) {
                    addEmptyTableCell();
                } else {
                    addStringTableCell(libelle.getLibelleString());
                }
            }
            closeTableRow();
        }

    }

}
