/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.htmlpages;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class CorpusThesaurusPage extends AffichageHtmlPage {

    Thesaurus thesaurus;
    Corpus corpus;
    InfoMotcle[] firstlevelInfos;
    Croisement croisement;

    public CorpusThesaurusPage(BdfResponseParameters responseParameters) {
        super(responseParameters);
        Object[] objectArray = (Object[]) responseParameters.getParameter(OBJECTARRAY_OBJ);
        corpus = (Corpus) objectArray[0];
        thesaurus = (Thesaurus) objectArray[1];
        croisement = bdfServer.getBasedefiches().getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
        MotcleList firstlevel = thesaurus.getFirstLevelList();
        firstlevelInfos = new InfoMotcle[firstlevel.getMotcleCount()];
        for (int i = 0; i < firstlevel.getMotcleCount(); i++) {
            firstlevelInfos[i] = new InfoMotcle(firstlevel.getMotcle(i));
        }
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        HTMLUtils.printBodyStart(pw, false, "Corpus Thesaurus", null);
        pw.println("<table border=\"0\">");
        writeLigneEnteteTexte();
        AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
        SubsetItemList abregeFicheList = abregeFicheSortedList.getAbregeFicheListByCorpus(corpus);
        int ficheCount = abregeFicheList.getSubsetItemCount();
        for (int i = 0; i < ficheCount; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) abregeFicheList.getSubsetItem(i);
            writeLigne(abregeFiche);
        }
        writeLigneEnteteCode();
        pw.println("</table>");
        HTMLUtils.printBodyEnd(pw);
    }

    void writeLigneEnteteTexte() {
        pw.println("<tr>");
        pw.print("<td>");
        pw.print("id");
        pw.println("</td>");
        pw.print("<td>");
        pw.print("");
        pw.println("</td>");
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].writeLigneEnteteTexte();
        }
        pw.println("</tr>");
    }

    void writeLigneEnteteCode() {
        pw.println("<tr>");
        pw.print("<td>");
        pw.print("");
        pw.println("</td>");
        pw.print("<td>");
        pw.print("");
        pw.println("</td>");
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].writeLigneEnteteCode(pw);
        }
        pw.println("</tr>");
    }

    void writeLigne(AbregeFiche abrege) {
        IntHashMap mapitems = new IntHashMap();
        CroisementItem croisementItem = croisement.getCroisementItem(corpus.getSubsetKey(), abrege.getTicket());
        int ciCount = croisementItem.getCount();
        for (int i = 0; i < ciCount; i++) {
            int ticket = croisementItem.getTicket(i);
            int poids = croisementItem.getPoids(i);
            mapitems.put(ticket, new Integer(poids));
        }
        pw.println("<tr>");
        pw.print("<td>");
        pw.print(String.valueOf(abrege.getIdcorpus()));
        pw.println("</td>");
        pw.print("<td>");
        escape(abrege.getTitre());
        pw.println("</td>");
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].writeLigne(mapitems);
        }
        pw.println("</tr>");
    }


    class InfoMotcle {

        Motcle motcle;
        InfoMotcle[] infos;
        int taille;

        InfoMotcle(Motcle motcle) {
            this.motcle = motcle;
            MotcleList childList = motcle.getChildList();
            taille = childList.getMotcleCount();
            infos = new InfoMotcle[taille];
            for (int i = 0; i < taille; i++) {
                infos[i] = new InfoMotcle(childList.getMotcle(i));
            }
        }

        void writeLigneEnteteTexte() {
            pw.print("<td>");
            String idalpha = motcle.getIdalpha();
            pw.print(motcle.getLibelleString(workingLangInteger, true));
            pw.println("</td>");
            for (int i = 0; i < taille; i++) {
                infos[i].writeLigneEnteteTexte();
            }
        }

        void writeLigneEnteteCode(PrintWriter pw) {
            pw.print("<td>");
            String idalpha = motcle.getIdalpha();
            if (idalpha != null) {
                pw.print(idalpha);
            } else {
                pw.print(String.valueOf(motcle.getIdths()));
            }
            pw.println("</td>");
            for (int i = 0; i < taille; i++) {
                infos[i].writeLigneEnteteCode(pw);
            }
        }

        void writeLigne(IntHashMap mapitems) {
            pw.print("<td>");
            Integer poids = (Integer) mapitems.get(motcle.getTicket());
            if (poids != null) {
                pw.print(poids.toString());
            }
            /*if (type == TYPE_X) {
            if (item != null) pw.print("X");
            } else if (type == TYPE_POIDS) {
            if (item == null) pw.print("0");
            else pw.print(String.valueOf(item.getPoids()));
            }*/
            pw.println("</td>");
            for (int i = 0; i < taille; i++) {
                infos[i].writeLigne(mapitems);
            }
        }

    }

}
