/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.htmlpages;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.selection.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.util.HTMLPrinterUtils;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class GeolocalisationPage extends AffichageHtmlPage implements ValueResolver {

    public GeolocalisationPage(BdfResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        HTMLPrinterUtils.printTemplate(getPrintWriter(), ServletCommons.getTemplateInputStream("affichage_geolocalisation"), this);
    }

    public String getValue(AccoladeArgument accoladeArgument) {
        String key = accoladeArgument.getName();
        if (key.equals("TITRE")) {
            return getEscapedLocalization("listefiches_geolocalisation", null);
        } else if (key.equals("PATH")) {
            return "../";
        } else if (key.equals("CREATION_MARQUEURS")) {
            StringBuffer buf = new StringBuffer();
            AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
            int corpusCount = abregeFicheSortedList.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = abregeFicheSortedList.getCorpus(i);
                CorpusField geolocalisationField = corpus.getCorpusMetadata().getGeolocalisationField();
                if (geolocalisationField != null) {
                    SubsetItemList abregeFicheList = abregeFicheSortedList.getCorpusAbregeFicheList(i);
                    int ficheCount = abregeFicheList.getSubsetItemCount();
                    for (int j = 0; j < ficheCount; j++) {
                        AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                        FicheItem ficheItem = (FicheItem) FichePointeur.getValue(abrege,geolocalisationField.getFieldKey());
                        if ((ficheItem != null) && (ficheItem instanceof Geopoint)) {
                            Geopoint geopoint = (Geopoint) ficheItem;
                            buf.append("GEO_CreateMarqueur(");
                            buf.append(geopoint.getLatitude().toString());
                            buf.append(",");
                            buf.append(geopoint.getLongitude().toString());
                            buf.append(",\"");
                            StringUtils.escapeDoubleQuote(CorpusMetadataUtils.getFicheIntituleString(abrege, localisationInfo), buf);
                            buf.append("\");\n");
                        }
                    }
                }
            }
            return buf.toString();
        } else {
            return getEscapedLocalization(key, null);
        }
    }

}
