/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.htmlpages;

import fr.exemole.bdfserver.response.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import net.mapeadores.basedefiches.*;
import java.util.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.corpus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class GlobalStatsPage extends AffichageHtmlPage {

    Basedefiches basedefiches;

    public GlobalStatsPage(BdfResponseParameters responseParameters) {
        super(responseParameters);
        basedefiches = bdfServer.getBasedefiches();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        UIManager uiManager = bdfServer.getUIManager();
        start();
        appendH1("admin_title_globalstats");
        printSubsetStats(uiManager.getCorpusSubsetKeyTree(), "global_list_corpus", 1);
        printSubsetStats(uiManager.getThesaurusSubsetKeyTree(), "global_list_thesaurus", 2);
        printSubsetStats(uiManager.getSphereSubsetKeyTree(), "global_list_sphere", 3);
        end();
    }

    private void printSubsetStats(SubsetKeyTree subsetKeyTree, String titleKey, int type) {
        int corpusCount = subsetKeyTree.getSubsetKeyCount();
        startH2();
        localize(titleKey);
        printColon();
        append(String.valueOf(corpusCount));
        endH2();
        startUl();
        for (Iterator it = subsetKeyTree.getSubsetKeyGroupList().iterator(); it.hasNext();) {
            SubsetKeyGroup subsetKeyGroup = (SubsetKeyGroup) it.next();
            boolean isTemplate = !subsetKeyGroup.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(subsetKeyGroup.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (Iterator it2 = subsetKeyGroup.getSubsetKeyList().iterator(); it2.hasNext();) {
                SubsetKey subsetKey = (SubsetKey) it2.next();
                startLi();
                switch (type) {
                    case 1:
                        printCorpus(subsetKey);
                        break;
                    case 2:
                        printThesaurus(subsetKey);
                        break;
                    case 3:
                        printSphere(subsetKey);
                        break;
                }
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpus(SubsetKey subsetKey) {
        Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        printColon();
        append(String.valueOf(corpus.size()));
    }

    private void printThesaurus(SubsetKey subsetKey) {
        Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(subsetKey);
        escape(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        printColon();
        append(String.valueOf(thesaurus.size()));
    }

    private void printSphere(SubsetKey subsetKey) {
        Sphere sphere = (Sphere) basedefiches.getSubset(subsetKey);
        escape(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
        printColon();
        append(String.valueOf(sphere.size()));
    }

}
