/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractAssociationCommand extends AbstractBdfCommand {

    protected Document document;
    protected DocumentSubset documentSubset;
    protected DocumentSubsetEditor documentSubsetEditor;
    protected AbregeFiche abregeFiche;

    public AbstractAssociationCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        documentSubset = (DocumentSubset) bdfServer.getBasedefiches().getSubset(SubsetKey.DOCUMENT_SUBSET);
        documentSubsetEditor = bdfServer.getBasedefichesEditor().getDocumentSubsetEditor();
        if (needDocumentInit()) {
            initDocument();
        }
        try {
            initAbregeFiche();
        } catch (ParameterException pe) {
            if (needAbregeFicheInit()) {
                throw pe;
            }
        }
        return doAssociationCommand();
    }

    protected abstract AssociationResponseParameters doAssociationCommand() throws ParameterException;

    public abstract boolean needDocumentInit();

    public abstract boolean needAbregeFicheInit();

    protected void initDocument() throws ParameterException {
        document = RequestUtils.getDocument(requestMap, bdfServer.getBasedefiches());
    }

    protected void initAbregeFiche() throws ParameterException {
        abregeFiche = RequestUtils.getAbregeFiche(requestMap, bdfServer.getBasedefiches());
    }

    public AssociationResponseParameters initResponseParameters() {
        AssociationResponseParameters responseParameters = new AssociationResponseParameters(bdfServer, bdfUser);
        if (document != null) {
            responseParameters.setDocument(document);
        }
        if (abregeFiche != null) {
            responseParameters.setAbregeFiche(abregeFiche);
        }
        return responseParameters;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new AssociationResponsePrinterFactory();
    }

}
