/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.response.AssociationResponseParameters;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.croisement.*;
import java.io.*;
import fr.exemole.bdfserver.request.ParameterException;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.request.FileValue;


/**
 *
 * @author Vincent Calame
 */
public class AssociationCreationCommand extends AbstractAssociationCommand {

    public final static String FILE_PARAMNAME = "file";

    public AssociationCreationCommand() {
    }

    protected AssociationResponseParameters doAssociationCommand() throws ParameterException {
        AssociationResponseParameters responseParameters = initResponseParameters();
        responseParameters.setHtmlPageCode(AssociationResponseParameters.ASSOCIATIONFORM_PAGE);
        createDocument(responseParameters);
        return responseParameters;
    }

    public static String getCommandString() {
        return "AssociationCreation";
    }

    public boolean needDocumentInit() {
        return false;
    }

    public boolean needAbregeFicheInit() {
        return true;
    }

    void createDocument(AssociationResponseParameters responseParameters) throws ParameterException {
        FileValue fileValue = requestMap.getFileValue(FILE_PARAMNAME);
        if (fileValue == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FILE_PARAMNAME);
        }
        String fileName = fileValue.getName();
        VersionName.ParseResult parseResult = VersionName.parse(fileName, true);
        if (parseResult == null) {
            responseParameters.setErrorMessage("association_err_nofile", null);
            return;
        }
        if ((!parseResult.hasExtension()) || (!AssociationUtils.validateExtension(parseResult.getExtension()))) {
            responseParameters.setErrorMessage("association_err_wrongextension", fileName);
        } else {
            String basename = AssociationUtils.checkBasename(parseResult.getBasename(), documentSubset);
            VersionName versionName = VersionName.getInstance(basename, parseResult.getExtension());
            try {
                Document document = documentSubsetEditor.createDocument(versionName);
                Version version = document.getVersionByExtension(versionName.getExtension());
                DefaultDocumentCroisementList documentList = new DefaultDocumentCroisementList(document);
                bdfServer.getBasedefichesEditor().getCroisementEditor().updateCroisement(abregeFiche, documentList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
                try {
                    documentSubsetEditor.updateVersion(version, fileValue.getInputStream());
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            } catch (ExistingBasenameException ebe) {
                throw new ImplementationException(ebe);
            }
            Object[] values = {fileName, basename};
            if (basename.equals(parseResult.getBasename())) {
                responseParameters.setDoneMessage("association_done_associationcreation_nochange", values);
            } else {
                responseParameters.setDoneMessage("association_done_associationcreation_change", values);
            }
        }
        fileValue.free();
    }

}
