/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractBdfCommand {

    public final static String ASSOCIATIONREMOVE_PAGE_PARAMVALUE = "associationremove";
    private Document document;
    private AbregeFiche abregeFiche;

    public DefaultCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        short pageType = getPageType();
        AssociationResponseParameters responseParameters = new AssociationResponseParameters(bdfServer, bdfUser);
        if (document != null) {
            responseParameters.setDocument(document);
        }
        if (abregeFiche != null) {
            responseParameters.setAbregeFiche(abregeFiche);
        }
        responseParameters.setHtmlPageCode(pageType);
        return responseParameters;
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            if (requestMap.getParameter(RequestUtils.IDSUBSET_PARAMNAME) != null) {
                initDocument();
                try {
                    initAbregeFiche();
                } catch (ParameterException pe) {
                }
                return AssociationResponseParameters.DOCUMENTFORM_PAGE;
            } else if (requestMap.getParameter(RequestUtils.IDCORPUS_PARAMNAME) != null) {
                initAbregeFiche();
                return AssociationResponseParameters.ASSOCIATIONFORM_PAGE;
            } else {
                return AssociationResponseParameters.ASSOCIATIONINDEX_PAGE;
            }
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(ASSOCIATIONREMOVE_PAGE_PARAMVALUE)) {
            initAbregeFiche();
            initDocument();
            return AssociationResponseParameters.ASSOCIATIONREMOVE_PAGE;
        }
        throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
    }

    private void initDocument() throws ParameterException {
        document = RequestUtils.getDocument(requestMap, bdfServer.getBasedefiches());
    }

    private void initAbregeFiche() throws ParameterException {
        abregeFiche = RequestUtils.getAbregeFiche(requestMap, bdfServer.getBasedefiches());
    }

    public static String getCommandString() {
        return null;
    }

    public static String getPageTypeString(short pageType) {
        switch (pageType) {
            case AssociationResponseParameters.ASSOCIATIONINDEX_PAGE:
                return "";
            case AssociationResponseParameters.ASSOCIATIONREMOVE_PAGE:
                return ASSOCIATIONREMOVE_PAGE_PARAMVALUE;
            default:
                throw new SwitchException("type = " + pageType);
        }
    }

    public boolean needSynchronisation() {
        return true;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new AssociationResponsePrinterFactory();
    }

}
