/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import net.mapeadores.util.request.FileValue;
import fr.exemole.bdfserver.response.AssociationResponseParameters;
import net.mapeadores.basedefiches.association.*;
import java.io.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class VersionUpdateCommand extends AbstractAssociationCommand {

    public final static String VERSIONFILE_PARAMNAME = "versionfile";

    public VersionUpdateCommand() {
    }

    protected AssociationResponseParameters doAssociationCommand() throws ParameterException {
        AssociationResponseParameters responseParameters = initResponseParameters();
        responseParameters.setHtmlPageCode(AssociationResponseParameters.DOCUMENTFORM_PAGE);
        saveVersion(responseParameters);
        return responseParameters;
    }

    public static String getCommandString() {
        return "VersionUpdate";
    }

    public boolean needDocumentInit() {
        return true;
    }

    public boolean needAbregeFicheInit() {
        return false;
    }

    void saveVersion(AssociationResponseParameters responseParameters) throws ParameterException {
        FileValue fileValue = requestMap.getFileValue(VERSIONFILE_PARAMNAME);
        if (fileValue == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, VERSIONFILE_PARAMNAME);
        }
        String fileName = fileValue.getName();
        VersionName.ParseResult parseResult = VersionName.parse(fileName, true);
        if (parseResult == null) {
            responseParameters.setErrorMessage("association_err_nofile", null);
            return;
        }
        if (!parseResult.hasExtension()) {
            responseParameters.setErrorMessage("association_err_wrongextension", fileName);
        } else {
            String extension = parseResult.getExtension();
            Version version = document.getVersionByExtension(extension);
            Object[] values = {fileName, extension};
            if (version != null) {
                responseParameters.setDoneMessage("association_done_versionupdate_existing", values);
            } else {
                try {
                    version = documentSubsetEditor.createVersion(document, extension);
                    responseParameters.setDoneMessage("association_done_versionupdate_new", values);
                } catch (ExistingVersionException eve) {
                    throw new ImplementationException(eve);
                } catch (IllegalArgumentException iae) {
                    responseParameters.setErrorMessage("association_err_wrongextension", fileName);
                }
            }
            if (version != null) {
                try {
                    documentSubsetEditor.updateVersion(version, fileValue.getInputStream());
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }
        }
        fileValue.free();
    }

}
