/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association.htmlpages;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.association.*;
import fr.exemole.bdfserver.users.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.croisement.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class AssociationFormPage extends AssociationHtmlPage {

    public AssociationFormPage(AssociationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1("association_title_associationform");
        startP(CssClass.P_SUBTITLE);
        escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
        endP();
        printMessage();
        printCurrent();
        printCreation();
        printAdd();

        end();
    }

    private void printCurrent() {
        appendHr();
        DocumentSubset documentSubset = (DocumentSubset) bdfServer.getBasedefiches().getSubset(SubsetKey.DOCUMENT_SUBSET);
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = bdfServer.getBasedefiches().getCroisement(corpusKey, SubsetKey.DOCUMENT_SUBSET);
        CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, abregeFiche.getTicket());
        int length = croisementItem.getCount();
        if (length == 0) {
            appendP(null, "association_info_noassociation");
        } else {
            appendH2("association_title_currentassociation");
            startUl();
            for (int i = 0; i < length; i++) {
                int ticket = croisementItem.getTicket(i);
                Document document = (Document) documentSubset.getSubsetItemByTicket(ticket);
                printCurrentDocument(document);
            }
            endUl();
        }
    }

    private void printCreation() {
        appendHr();
        appendH2("association_title_associationcreation");
        startMultiPartForm(document, AssociationCreationCommand.getCommandString(), abregeFiche);
        appendP(null, "association_lib_upload");
        startP();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setType("file");
        htmlAttributes.setName(AssociationCreationCommand.FILE_PARAMNAME);
        htmlAttributes.setSize("75");
        appendInput(htmlAttributes);
        endP();
        endForm("association_submit_upload");
    }

    private void printCurrentDocument(Document document) {
        String parameters = RequestUtils.toParameterString(abregeFiche) + "&amp;" + RequestUtils.toParameterString(document);
        startLi();
        startP();
        append(document.getBasename());
        space();
        RequestUtils.printDocument(this, document, formatLocale, true);
        endP();
        startP();
        startSpan(CssClass.SPAN_SMALLLINKS);
        printLink("association_link_documentchange", parameters, null);
        printTiret();
        printLink("association_link_associationremove", parameters, DefaultCommand.ASSOCIATIONREMOVE_PAGE_PARAMVALUE);
        endSpan();
        endP();
        endLi();
    }

    private void printLink(String key, String parameters, String page) {
        StringBuilder buf = new StringBuilder();
        buf.append("Association?");
        if (page != null) {
            buf.append("page=");
            buf.append(page);
            buf.append("&amp;");
        }
        buf.append(parameters);
        appendA(buf.toString(), key);
    }

    private void printAdd() {
        appendHr();
        appendH2("association_title_associationadd");
        startInLineForm(abregeFiche, AssociationAddCommand.getCommandString());
        printLib("association_lib_associationadd");
        appendMediumInput(AssociationAddCommand.BASENAME_PARAMNAME, "");
        endInLineForm("association_submit_associationadd");
    }

}
