/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association.htmlpages;

import fr.exemole.bdfserver.response.AssociationResponseParameters;
import fr.exemole.bdfserver.servlets.association.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.association.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import net.mapeadores.basedefiches.croisement.*;
import fr.exemole.bdfserver.request.RequestUtils;


/**
 *
 * @author Vincent Calame
 */
public class DocumentFormPage extends AssociationHtmlPage {

    public DocumentFormPage(AssociationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        appendHr();
        startInLineForm(document, BasenameCommand.getCommandString(), abregeFiche);
        printLib("association_lib_basename");
        appendMediumInput(BasenameCommand.BASENAME_PARAMNAME, document.getBasename());
        endInLineForm("association_submit_basename");
        printVersionUpdate();
        if (document.getVersionCount() > 1) {
            printVersionRemove();
        }
        if (!CroisementUtils.hasAssociation(document)) {
            printDocumentDelete();
        }
        end();
    }

    void printVersionUpdate() {
        appendHr();
        appendH2("association_title_versionupdate");
        startMultiPartForm(document, VersionUpdateCommand.getCommandString(), abregeFiche);
        appendP(null, "association_lib_upload");
        startP();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setType("file");
        htmlAttributes.setName(VersionUpdateCommand.VERSIONFILE_PARAMNAME);
        htmlAttributes.setSize("75");
        appendInput(htmlAttributes);
        endP();
        endForm("association_submit_upload");
    }

    void printDocumentDelete() {
        appendHr();
        appendH2("association_title_documentremove");
        startForm(document, DocumentRemoveCommand.getCommandString(), abregeFiche);
        endFormWithConfirm("association_submit_documentremove");
    }

    void printVersionRemove() {
        appendHr();
        appendH2("association_title_versionremove");
        startP();
        startForm(document, VersionRemoveCommand.getCommandString(), abregeFiche);
        printLib("association_lib_versionremove");
        startSelect(VersionRemoveCommand.VERSION_PARAMNAME);
        startOption("", true);
        space();
        endOption();
        for (int i = 0; i < document.getVersionCount(); i++) {
            Version version = document.getVersion(i);
            String extension = version.getVersionName().getExtension();
            startOption(extension, false);
            append(".");
            append(extension);
            endOption();
        }
        endSelect();
        endP();
        endForm("association_submit_versionremove");
    }

    void printTitre() {
        startP(CssClass.P_TOOLBARLINKS);
        if (abregeFiche == null) {
            appendA("Association", "association_title_index");
        } else {
            appendA("Association?" + RequestUtils.toParameterString(abregeFiche), "association_link_backassociation");
        }
        endP();
        startH1();
        append(document.getBasename());
        space();
        RequestUtils.printDocument(this, document, formatLocale, true);
        endH1();
        printMessage();
    }

}
