/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.ui.corpus.*;
import java.util.*;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractCommentUICommand extends AbstractCorpusCommand {

    public final static String NAME_PARAMNAME = "name";
    public final static String LOCATION_PARAMNAME = "location";
    public final static String LOCATION_FORM_PARAMVALUE = "form";
    public final static String LOCATION_AFFICHAGE_PARAMVALUE = "affichage";
    public final static String LOCATION_BOTH_PARAMVALUE = "both";
    protected CorpusUI corpusUI;
    protected ComponentUIEditor componentUIEditor;

    public AbstractCommentUICommand() {
    }

    protected void init() {
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        componentUIEditor = corpusUI.getComponentUIEditor();
    }

    protected void updateLibelle(CommentUI commentUI) {
        Set set = requestMap.getParameterNameSet();
        for (Iterator it = set.iterator(); it.hasNext();) {
            String paramName = (String) it.next();
            if (!paramName.startsWith("comment/")) {
                continue;
            }
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(8));
                String nvlibelle = requestMap.getParameter(paramName);
                componentUIEditor.setLibelleString(commentUI, currentLangInteger, nvlibelle);
            } catch (LangIntegerException ile) {
            }
        }
    }

    protected int getPositionMask() throws ParameterException {
        String positionString = requestMap.getParameter(LOCATION_PARAMNAME);
        if (positionString == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, LOCATION_PARAMNAME);
        }
        if (positionString.equals(LOCATION_FORM_PARAMVALUE)) {
            return CommentUI.FORM_BITVALUE;
        } else if (positionString.equals(LOCATION_AFFICHAGE_PARAMVALUE)) {
            return CommentUI.AFFICHAGE_BITVALUE;
        } else if (positionString.equals(LOCATION_BOTH_PARAMVALUE)) {
            return CommentUI.ALL_MASK;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, LOCATION_PARAMNAME + " = " + positionString);
        }
    }

}
