/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractCorpusCommand extends AbstractBdfCommand {

    public static String KEY_PARAMNAME = "key";
    protected Corpus corpus;
    protected CorpusMetadata corpusMetadata;
    protected CorpusMetadataEditor corpusMetadataEditor;
    protected CorpusEditor corpusEditor;
    protected CorpusResponseParameters responseParameters;

    public AbstractCorpusCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        checkParameters();
        if (responseParameters.mayBeDone()) {
            corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
            corpusMetadataEditor = corpusEditor.getCorpusMetadataEditor();
            doCorpusCommand();
        }
        return responseParameters;
    }

    protected CommandTest checkParameters() throws ParameterException {
        corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
        responseParameters = new CorpusResponseParameters(bdfServer, bdfUser);
        responseParameters.setCorpus(corpus);
        corpusMetadata = corpus.getCorpusMetadata();
        checkCorpusParameters();
        return responseParameters;
    }

    protected abstract void doCorpusCommand();

    protected abstract void checkCorpusParameters() throws ParameterException;

    protected void clearDefaultTemplates() {
        bdfServer.getTransformationManager().clearDefaultTransformer();
    }

    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new CorpusResponsePrinterFactory();
    }

}
