/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;
import java.util.*;
import net.mapeadores.basedefiches.format.*;
import fr.exemole.bdfserver.ui.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class CloneCommand extends AbstractCorpusCommand {

    public static String NEWCORPUS_PARAMNAME = "newcorpus";
    private SubsetKey newCorpusKey;
    private CorpusEditor newCorpusEditor;
    private CorpusMetadataEditor newCorpusMetadataEditor;
    private CorpusMetadata newCorpusMetadata;
    private CorpusUI corpusUI;
    private CorpusUI newCorpusUI;

    public CloneCommand() {
    }

    protected void doCorpusCommand() {
        bdfServer.bufferBasedefichesListener();
        try {
            newCorpusEditor = bdfServer.getBasedefichesEditor().createCorpus(newCorpusKey, null);
            newCorpusMetadataEditor = newCorpusEditor.getCorpusMetadataEditor();
            newCorpusMetadata = (CorpusMetadata) newCorpusMetadataEditor.getMetadata();
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
        int intituleCount = corpusMetadata.getIntituleCount();
        for (int i = 0; i < intituleCount; i++) {
            Intitule intitule = corpusMetadata.getIntitule(i);
            Intitule newIntitule = newCorpusMetadataEditor.createIntitule(intitule.getName());
            int libelleCount = intitule.getLibelleCount();
            for (int j = 0; j < libelleCount; j++) {
                Libelle libelle = intitule.getLibelleByIndex(j);
                String newLib = libelle.getLibelleString() + " (2)";
                newCorpusMetadataEditor.setIntituleLibelle(newIntitule, newLib, libelle.getLangInteger());
            }
        }
        copyMandatoryField(FieldKey.TITRE);
        copyMandatoryField(FieldKey.LANG);
        copyMandatoryField(FieldKey.REDACTEURS);
        copySpecialField(FieldKey.SOUSTITRE);
        copySpecialField(FieldKey.TEXTE);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            copyField(corpusField);
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            copyField(corpusField);
        }
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        for (int i = 0; i < annexeCount; i++) {
            CorpusField corpusField = corpusMetadata.getAnnexeField(i);
            copyField(corpusField);
        }
        copyTitreGenerationOption();
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        newCorpusUI = bdfServer.getUIManager().getCorpusUI(newCorpusEditor.getCorpus());
        copyCorpusUI();
        bdfServer.flushBasedefichesListener();
        responseParameters.setHtmlPageCode(CorpusResponseParameters.COMMANDS_PAGE);
        String[] val = new String[2];
        val[0] = corpus.getSubsetKey().toString();
        val[1] = newCorpusKey.toString();
        responseParameters.setDoneMessage("corpus_done_clone", val);
    }

    protected void checkCorpusParameters() throws ParameterException {
        String newCorpusName = getMandatoryParameter(NEWCORPUS_PARAMNAME);
        if (newCorpusName == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, NEWCORPUS_PARAMNAME);
        }
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getBasedefiches().getSubset(newCorpusKey) != null) {
                responseParameters.setHtmlPageCode(CorpusResponseParameters.MESSAGE_SERVLETPAGE);
                responseParameters.setErrorMessage("corpus_err_existingcorpusname", newCorpusKey.toString());
                return;
            }
        } catch (java.text.ParseException pe) {
            responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
            responseParameters.setErrorMessage("corpus_err_wrongcorpusname", newCorpusName);
            return;
        }
    }

    public static String getCommandString() {
        return "Clone";
    }

    private void copyMandatoryField(FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        CorpusField newCorpusField = newCorpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        copyField(corpusField, newCorpusField);
    }

    private void copySpecialField(FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            return;
        }
        try {
            CorpusField newCorpusField = newCorpusMetadataEditor.createCorpusField(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
            copyField(corpusField, newCorpusField);
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("new corpus");
        }
    }

    private void copyField(CorpusField corpusField) {
        try {
            CorpusField newCorpusField = newCorpusMetadataEditor.createCorpusField(corpusField.getFieldKey(), corpusField.getFicheItemType());
            copyField(corpusField, newCorpusField);
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("new corpus");
        }
    }

    private void copyField(CorpusField corpusField, CorpusField newCorpusField) {
        int libelleCount = corpusField.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = corpusField.getLibelleByIndex(i);
            newCorpusMetadataEditor.setFieldLibelle(newCorpusField, libelle.getLibelleString(), libelle.getLangInteger());
        }
        FieldKey fieldKey = newCorpusField.getFieldKey();
        if ((fieldKey.isProprieteFieldKey()) || (fieldKey.isInformationFieldKey())) {
            copyOption(FieldOptions.DEFAULTSPHEREKEY_OPTION, corpusField, newCorpusField);
        }
        if (fieldKey.isProprieteFieldKey()) {
            copyOption(FieldOptions.SUBFIELDDISPLAY_OPTION, corpusField, newCorpusField);
            copyOption(FieldOptions.CURRENCYARRAY_OPTION, corpusField, newCorpusField);
            copyOption(FieldOptions.BASEURL_OPTION, corpusField, newCorpusField);
            copyOption(FieldOptions.IMAGESOURCE_OPTION, corpusField, newCorpusField);
            copyOption(FieldOptions.ALBUMKEY_OPTION, corpusField, newCorpusField);
        }
        if (fieldKey.isInformationFieldKey()) {
            copyOption(FieldOptions.INFORMATIONDISPLAY_OPTION, corpusField, newCorpusField);
        }
    }

    private void copyOption(String optionName, CorpusField corpusField, CorpusField newCorpusField) {
        Object obj = corpusField.getOption(optionName);
        if (obj != null) {
            try {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, optionName, obj);
            } catch (FieldOptionException foe) {
                throw new ShouldNotOccurException("clone corpus");
            }
        }
    }

    private void copyTitreGenerationOption() {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
        BasedefichesFormatDef ficheFormatDef = (BasedefichesFormatDef) corpusField.getOption(FieldOptions.TITREGENERATIONFORMAT_OPTION);
        if (ficheFormatDef != null) {
            CorpusField newCorpusField = newCorpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
            try {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptions.TITREGENERATIONFORMAT_OPTION, ficheFormatDef);
            } catch (FieldOptionException foe) {
                throw new ShouldNotOccurException("clone corpus");
            }
        }
    }

    private void copyCorpusUI() {
        ComponentUIEditor newEditor = newCorpusUI.getComponentUIEditor();
        List fieldUIList = corpusUI.getOrderedComponentUIList(CorpusUI.FIELDUI_LIST);
        for (Iterator it = fieldUIList.iterator(); it.hasNext();) {
            FieldUI fieldUI = (FieldUI) it.next();
            FieldUI newFieldUI = (FieldUI) newCorpusUI.getFieldUI(fieldUI.getCorpusField().getFieldKey());
            newEditor.setInputOption(newFieldUI, fieldUI.getInputOption());
        }

        List includeUIList = corpusUI.getOrderedComponentUIList(CorpusUI.INCLUDEUI_LIST);
        for (Iterator it = includeUIList.iterator(); it.hasNext();) {
            IncludeUI includeUI = (IncludeUI) it.next();
            IncludeUI newIncludeUI = newEditor.getIncludeUI(includeUI.getFiteredIncludeKey(), true);
            newEditor.setInputOption(newIncludeUI, includeUI.getInputOption());
        }
        if (corpusUI.isAssociationActive()) {
            newCorpusUI.setAssociationActive(true);
        }
        List commentUIList = corpusUI.getOrderedComponentUIList(CorpusUI.COMMENT_LIST);
        for (Iterator it = commentUIList.iterator(); it.hasNext();) {
            CommentUI commentUI = (CommentUI) it.next();
            CommentUI newCommentUI = (CommentUI) newEditor.getCommentUI(commentUI.getName(), true);
            newEditor.setLocation(newCommentUI, commentUI.getLocation());
            int libelleCount = commentUI.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = commentUI.getLibelleByIndex(i);
                newEditor.setLibelleString(newCommentUI, libelle.getLangInteger(), libelle.getLibelleString());
            }
        }
        String[] array = corpusUI.getPositionArray();
        newEditor.setPositionArray(array);
    }

}
