/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import java.util.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.include.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class ComponentUIInputCommand extends AbstractCorpusCommand {

    public final static String WIDTH_PARAMSUFFIX = "|width";
    public final static String ROWS_PARAMSUFFIX = "|rows";
    public final static String INPUT_PARAMSUFFIX = "|input";
    public final static String NONEW_PARAMSUFFIX = "|nonew";
    public final static String DEFAULT_PARAMSUFFIX = "|default";
    public final static String MANDATORY_PARAMSUFFIX = "|mandatory";
    private CorpusUI corpusUI;
    private List uiCoupleList;

    public ComponentUIInputCommand() {
    }

    protected void doCorpusCommand() {
        ComponentUIEditor editor = corpusUI.getComponentUIEditor();
        for (Iterator it = uiCoupleList.iterator(); it.hasNext();) {
            UICouple o = (UICouple) it.next();
            ComponentUI componentUI = o.getComponentUI();
            if (componentUI instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) componentUI;
                editor.setInputOption(fieldUI, o.getInputOption());
                if (o.hasValueOption()) {
                    editor.setValueOption(fieldUI, o.getValueOption());
                }
            } else if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                FilteredIncludeKey filteredIncludeKey = includeUI.getFiteredIncludeKey();
                IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
                if (o.hasInputOption()) {
                    editor.setInputOption(includeUI, o.getInputOption());
                }
                if (o.hasLimitOption()) {
                    editor.setLimitOption(includeUI, o.getLimitOption());
                }
                if (o.hasValueOption()) {
                    editor.setValueOption(includeUI, o.getValueOption());
                }
            }
        }
        responseParameters.setHtmlPageCode(CorpusResponseParameters.UIFORM_PAGE);
        responseParameters.setDoneMessage("corpus_done_componentinput", null);
        bdfServer.saveUIManager();
    }

    public static String getCommandString() {
        return "ComponentUIInput";
    }

    protected void checkCorpusParameters() throws ParameterException {
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);
        uiCoupleList = new ArrayList();
        for (Iterator it = list.iterator(); it.hasNext();) {
            ComponentUI obj = (ComponentUI) it.next();
            InputOption inputOption = null;
            LimitOption limitOption = null;
            ValueOption valueOption = null;
            if (obj instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) obj;
                String racine = fieldUI.getCorpusField().getFieldKey().toString();
                inputOption = update(racine, fieldUI.getInputOption(), false);
                valueOption = update(racine, fieldUI.getValueOption());
            } else if (obj instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) obj;
                FilteredIncludeKey filteredIncludeKey = includeUI.getFiteredIncludeKey();
                IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
                String racine = filteredIncludeKey.toString();
                if (includeKey.isIndexationIncludeKey()) {
                    inputOption = update(racine, includeUI.getInputOption(), true);
                    valueOption = update(racine, includeUI.getValueOption());
                } else if (includeKey.equals(IncludeKey.LIAGE)) {
                    limitOption = update(racine, includeUI.getLimitOption());
                }
            }
            UICouple o = new UICouple();
            o.setComponentUI(obj);
            o.setInputOption(inputOption);
            o.setLimitOption(limitOption);
            o.setValueOption(valueOption);
            uiCoupleList.add(o);
        }
    }

    private void updateList() throws ParameterException {
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);

    }

    private LimitOption update(String key, LimitOption limitOption) throws ParameterException {
        MutableLimitOption mutableLimitOption = new MutableLimitOption(limitOption);
        boolean nonewAllowed = this.getBooleanParameter(key + NONEW_PARAMSUFFIX);
        mutableLimitOption.setNoNewAllowed(nonewAllowed);
        return mutableLimitOption;
    }

    private InputOption update(String key, InputOption inputOption, boolean withInputType) throws ParameterException {
        MutableInputOption newInputOption = new MutableInputOption(inputOption);
        String width = requestMap.getParameter(key + WIDTH_PARAMSUFFIX);
        if (width != null) {
            try {
                short widthType = DefaultInputOption.widthTypeToShort(width);
                newInputOption.setWidthType(widthType);
            } catch (IllegalArgumentException iae) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, key + WIDTH_PARAMSUFFIX + " = " + width);
            }
        }
        String rows = requestMap.getParameter(key + ROWS_PARAMSUFFIX);
        if (rows != null) {
            try {
                int r = Integer.parseInt(rows);
                if (r > 0) {
                    newInputOption.setRows(r);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        if (withInputType) {
            String param = requestMap.getParameter(key + INPUT_PARAMSUFFIX);
            if (param != null) {
                try {
                    short inputType = DefaultInputOption.inputTypeToShort(param);
                    newInputOption.setInputType(inputType);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, key + INPUT_PARAMSUFFIX + " = " + param);
                }
            }
        }
        return newInputOption;
    }

    private ValueOption update(String key, ValueOption valueOption) throws ParameterException {
        if (valueOption == null) {
            return null;
        }
        MutableValueOption mutableValueOption = new MutableValueOption();
        String mandatory = requestMap.getParameter(key + MANDATORY_PARAMSUFFIX);
        if ((mandatory != null) && (mandatory.equals("1"))) {
            mutableValueOption.setMandatory(true);
        } else {
            mutableValueOption.setMandatory(false);
        }
        String defaultValue = requestMap.getParameter(key + DEFAULT_PARAMSUFFIX);
        if (defaultValue != null) {
            defaultValue = StringUtils.cleanString(defaultValue);
            mutableValueOption.setDefaultValue(defaultValue);
        }
        return mutableValueOption;
    }


    private class UICouple {

        private ComponentUI componentUI;
        private InputOption inputOption;
        private LimitOption limitOption;
        private ValueOption valueOption;

        public ComponentUI getComponentUI() {
            return componentUI;
        }

        public void setComponentUI(ComponentUI componentUI) {
            this.componentUI = componentUI;
        }

        public boolean hasInputOption() {
            return (inputOption != null);
        }

        public boolean hasLimitOption() {
            return (limitOption != null);
        }

        public boolean hasValueOption() {
            return (valueOption != null);
        }

        public InputOption getInputOption() {
            return inputOption;
        }

        public void setInputOption(InputOption inputOption) {
            this.inputOption = inputOption;
        }

        public LimitOption getLimitOption() {
            return limitOption;
        }

        public void setLimitOption(LimitOption limitOption) {
            this.limitOption = limitOption;
        }

        public ValueOption getValueOption() {
            return valueOption;
        }

        public void setValueOption(ValueOption valueOption) {
            this.valueOption = valueOption;
        }

    }

}
