
/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.conversion.*;
import net.mapeadores.util.exceptions.*;
import java.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.logging.*;


/**
 *
 * @author Vincent Calame
 */
public class ConversionCommand extends AbstractCorpusCommand {

    public final static String FIELD_PARAMNAME = "field";
    public final static String NEWFIELD_PARAMNAME = "newfield";
    CorpusField sourceField;
    FieldKey newFieldKey;

    public ConversionCommand() {
    }

    protected void doCorpusCommand() {
        CorpusField newCorpusField = null;
        bdfServer.bufferBasedefichesListener();
        try {
            short ficheItemType = getFicheItemType();
            newCorpusField = corpusMetadataEditor.createCorpusField(newFieldKey, ficheItemType);
            if ((newFieldKey.isInformationFieldKey()) && (sourceField.getFieldKey().ownsToCorpsdefiche())) {
                corpusMetadataEditor.setFieldOption(newCorpusField, FieldOptions.INFORMATIONDISPLAY_OPTION, new Short(FieldOptions.BLOCK_DISPLAY));
            }
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        bdfServer.flushBasedefichesListener();
        FicheConverter ficheConverter = FicheConverter.getInstance(sourceField, newCorpusField);
        int size = corpus.getTicketLength();
        int ficheCount = 0;
        for (int i = 0; i < size; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
            if (abregeFiche != null) {
                Fiche fiche = corpus.getFiche(abregeFiche);
                ficheConverter.convert(abregeFiche, fiche);
                corpusEditor.saveFiche(abregeFiche, fiche);
                ficheCount++;
            }
        }
        ResourceItem resourceItem = ResourceItem.newInstance("corpus_done_conversion", sourceField.getFieldKey().toString(), newFieldKey.toString(), new Integer(ficheCount));
        CommandMessage commandMessage = CommandMessage.log(resourceItem, ficheConverter.getProcessLog());
        responseParameters.setCommandMessage(commandMessage);
        responseParameters.setHtmlPageCode(CorpusResponseParameters.COMMANDS_PAGE);
    }

    protected void checkCorpusParameters() throws ParameterException {
        String fieldParamName = requestMap.getParameter(FIELD_PARAMNAME);
        if (fieldParamName == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FIELD_PARAMNAME);
        }
        try {
            FieldKey fieldKey = FieldKey.parse(fieldParamName);
            sourceField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (sourceField == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELD_PARAMNAME + "= " + fieldParamName);
            }
        } catch (ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELD_PARAMNAME + "= " + fieldParamName);
        }
        String newFieldParamName = requestMap.getParameter(NEWFIELD_PARAMNAME);
        if (newFieldParamName == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, NEWFIELD_PARAMNAME);
        }
        try {
            newFieldKey = FieldKey.parse(newFieldParamName);
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, NEWFIELD_PARAMNAME + "= " + newFieldParamName);
            }
            if (newFieldKey.isSpecialFieldKey()) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, NEWFIELD_PARAMNAME + "= " + newFieldParamName);
            }
        } catch (ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, NEWFIELD_PARAMNAME + "= " + newFieldParamName);
        }
    }

    public static String getCommandString() {
        return "Conversion";
    }

    private short getFicheItemType() {
        if (newFieldKey.isAnnexeFieldKey()) {
            return CorpusField.NO_FICHEITEM_FIELD;
        }
        FieldKey sourceFieldKey = sourceField.getFieldKey();
        if ((sourceFieldKey.isAnnexeFieldKey()) || (sourceFieldKey.equals(FieldKey.TEXTE))) {
            return CorpusField.PARA_FIELD;
        }
        short current = sourceField.getFicheItemType();
        if (current == CorpusField.NO_FICHEITEM_FIELD) {
            return CorpusField.ITEM_FIELD;
        }
        return current;
    }

}
