/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCreationCommand extends AbstractBdfCommand {

    public final static String NEWCORPUS_PARAMNAME = "newcorpus";
    public final static String CORPUSTYPE_PARAMNAME = "corpustype";
    public final static String NORMAL_PARAMVALUE = "normal";
    public final static String CORPUSSLAVE_PARAMVALUE = "corpusslave";
    public final static String THESAURUSSLAVE_PARAMVALUE = "thesaurusslave";
    public final static String MASTERTHESAURUS_PARAMNAME = "masterthesaurus";
    public final static String MASTERCORPUS_PARAMNAME = "mastercorpus";
    private SubsetKey newCorpusKey;
    private Subset masterSubset;

    public CorpusCreationCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        CorpusResponseParameters responseParameters = (CorpusResponseParameters) checkParameters();
        if (!responseParameters.mayBeDone()) {
            return responseParameters;
        }
        try {
            CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().createCorpus(newCorpusKey, masterSubset);
            responseParameters.setCorpus(corpusEditor.getCorpus());
            responseParameters.setHtmlPageCode(CorpusResponseParameters.INTITULESFORM_PAGE);
            responseParameters.setDoneMessage("corpus_done_corpuscreation", newCorpusKey.toString());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
        return responseParameters;
    }

    protected CommandTest checkParameters() throws ParameterException {
        String newCorpusName = getMandatoryParameter(NEWCORPUS_PARAMNAME);
        initSubsetMaster();
        CorpusResponseParameters responseParameters = new CorpusResponseParameters(bdfServer, bdfUser);
        newCorpusName = newCorpusName.trim();
        if (newCorpusName.length() == 0) {
            responseParameters.setHtmlPageCode(CorpusResponseParameters.CORPUSCREATIONFORM_PAGE);
            responseParameters.setErrorMessage("corpus_err_missingcorpusname", newCorpusName);
            return responseParameters;
        }
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getBasedefiches().getSubset(newCorpusKey) != null) {
                responseParameters.setHtmlPageCode(CorpusResponseParameters.CORPUSCREATIONFORM_PAGE);
                responseParameters.setErrorMessage("corpus_err_existingcorpusname", newCorpusKey.toString());
                return responseParameters;
            }
        } catch (java.text.ParseException pe) {
            responseParameters.setHtmlPageCode(CorpusResponseParameters.CORPUSCREATIONFORM_PAGE);
            responseParameters.setErrorMessage("corpus_err_wrongcorpusname", newCorpusName);
            return responseParameters;
        }
        return responseParameters;
    }

    private void initSubsetMaster() throws ParameterException {
        String corpusType = getMandatoryParameter(CORPUSTYPE_PARAMNAME);
        if (corpusType.equals(NORMAL_PARAMVALUE)) {
            masterSubset = null;
        } else if (corpusType.equals(THESAURUSSLAVE_PARAMVALUE)) {
            String masterthesaurusId = getMandatoryParameter(MASTERTHESAURUS_PARAMNAME);
            masterSubset = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), masterthesaurusId);
            if (masterSubset == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, MASTERTHESAURUS_PARAMNAME + " = " + masterthesaurusId);
            }
        } else if (corpusType.equals(CORPUSSLAVE_PARAMVALUE)) {
            String mastercorpusId = getMandatoryParameter(MASTERCORPUS_PARAMNAME);
            masterSubset = BasedefichesUtils.getCorpus(bdfServer.getBasedefiches(), mastercorpusId);
            if (masterSubset == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, MASTERCORPUS_PARAMNAME + " = " + mastercorpusId);
            }
            if (masterSubset.getMasterSubset() != null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "slave corpus cannot be a master corpus = " + mastercorpusId);
            }
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CORPUSTYPE_PARAMNAME + " = " + corpusType);
        }
    }

    public static String getCommandString() {
        return "CorpusCreation";
    }

    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new CorpusResponsePrinterFactory();
    }

    public static String getCommandKey() {
        return "CRP-01";
    }

}
