/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.ResponsePrinterFactory;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractBdfCommand {

    public final static String INTITULESFORM_PAGE_PARAMVALUE = "intitulesform";
    public final static String FIELDCREATIONFORM_PAGE_PARAMVALUE = "fieldcreationform";
    public final static String OPTIONSFORM_PAGE_PARAMVALUE = "optionsform";
    public final static String CORPUSCREATIONFORM_PAGE_PARAMVALUE = "corpuscreationform";
    public final static String INSERTFORM_PAGE_PARAMVALUE = "insertform";
    public final static String INSERTFORM_INDEXATION_PAGE_PARAMVALUE = "insertform_indexation";
    public final static String INSERTFORM_LIAGE_PAGE_PARAMVALUE = "insertform_liage";
    public final static String INSERTFORM_COMMENT_PAGE_PARAMVALUE = "insertform_comment";
    public final static String UIFORM_PAGE_PARAMVALUE = "uiform";
    public final static String CORPUSREMOVEFORM_PAGE_PARAMVALUE = "corpusremoveform";
    public final static String FIELDREMOVEFORM_PAGE_PARAMVALUE = "fieldremoveform";
    public final static String COMMANDS_PAGE_PARAMVALUE = "commands";
    private Corpus corpus;

    public DefaultCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        CorpusResponseParameters responseParameters = new CorpusResponseParameters(bdfServer, bdfBundle.getBdfUser());
        short pageType = getPageType();
        if (corpus != null) {
            responseParameters.setCorpus(corpus);
        }
        responseParameters.setHtmlPageCode(pageType);
        return responseParameters;
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            return CorpusResponseParameters.INDEX_PAGE;
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(INTITULESFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.INTITULESFORM_PAGE;
        } else if (page_param.equals(FIELDCREATIONFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.FIELDCREATIONFORM_PAGE;
        } else if (page_param.equals(OPTIONSFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.OPTIONSFORM_PAGE;
        } else if (page_param.equals(INSERTFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.INSERTFORM_PAGE;
        } else if (page_param.equals(INSERTFORM_INDEXATION_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE;
        } else if (page_param.equals(INSERTFORM_LIAGE_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.INSERTFORM_LIAGE_PAGE;
        } else if (page_param.equals(INSERTFORM_COMMENT_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.INSERTFORM_COMMENT_PAGE;
        } else if (page_param.equals(UIFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.UIFORM_PAGE;
        } else if (page_param.equals(CORPUSCREATIONFORM_PAGE_PARAMVALUE)) {
            return CorpusResponseParameters.CORPUSCREATIONFORM_PAGE;
        } else if (page_param.equals(CORPUSREMOVEFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.CORPUSREMOVEFORM_PAGE;
        } else if (page_param.equals(FIELDREMOVEFORM_PAGE_PARAMVALUE)) {
            initCorpus();
            return CorpusResponseParameters.FIELDREMOVEFORM_PAGE;
        } else if (page_param.equals(COMMANDS_PAGE_PARAMVALUE)) {
            return CorpusResponseParameters.COMMANDS_PAGE;
        }
        throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
    }

    void initCorpus() throws ParameterException {
        corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
    }

    public static String getPageTypeString(short pageType) {
        switch (pageType) {
            case CorpusResponseParameters.INDEX_PAGE:
                return "";
            case CorpusResponseParameters.INTITULESFORM_PAGE:
                return INTITULESFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.FIELDCREATIONFORM_PAGE:
                return FIELDCREATIONFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.OPTIONSFORM_PAGE:
                return OPTIONSFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.CORPUSCREATIONFORM_PAGE:
                return CORPUSCREATIONFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.INSERTFORM_PAGE:
                return INSERTFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.UIFORM_PAGE:
                return UIFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.CORPUSREMOVEFORM_PAGE:
                return CORPUSREMOVEFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.FIELDREMOVEFORM_PAGE:
                return FIELDREMOVEFORM_PAGE_PARAMVALUE;
            case CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE:
                return INSERTFORM_INDEXATION_PAGE_PARAMVALUE;
            case CorpusResponseParameters.INSERTFORM_LIAGE_PAGE:
                return INSERTFORM_LIAGE_PAGE_PARAMVALUE;
            case CorpusResponseParameters.INSERTFORM_COMMENT_PAGE:
                return INSERTFORM_COMMENT_PAGE_PARAMVALUE;
            case CorpusResponseParameters.COMMANDS_PAGE:
                return COMMANDS_PAGE_PARAMVALUE;
            default:
                throw new SwitchException("pageType = " + pageType);
        }
    }

    public static String getCommandString() {
        return null;
    }

    public boolean needSynchronisation() {
        return false;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new CorpusResponsePrinterFactory();
    }

}
