/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import java.util.*;
import fr.exemole.bdfserver.servlets.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FieldCreationCommand extends AbstractCorpusCommand {

    public final static String CATEGORY_PARAMNAME = "category";
    public final static String TYPE_PARAMNAME = "type";
    public final static String FICHEITEMTYPE_PARAMNAME = "ficheitemtype";
    private FieldKey newFieldKey;
    private short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;

    public FieldCreationCommand() {
    }

    protected void doCorpusCommand() {
        responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
        if (newFieldKey.isSpecialFieldKey()) {
            try {
                corpusMetadataEditor.createCorpusField(newFieldKey, CorpusField.NO_FICHEITEM_FIELD);
                responseParameters.setDoneMessage("corpus_done_activation_" + newFieldKey.toString(), null);
            } catch (ExistingFieldKeyException efke) {
                throw new ShouldNotOccurException("test done before");
            }
        } else {
            try {
                corpusMetadataEditor.createCorpusField(newFieldKey, ficheItemType);
                responseParameters.setDoneMessage("corpus_done_fieldcreation", newFieldKey.toString());
            } catch (ExistingFieldKeyException efke) {
                throw new ShouldNotOccurException("test done before");
            }
        }
    }

    protected void checkCorpusParameters() throws ParameterException {
        String categoryParameter = requestMap.getParameter(CATEGORY_PARAMNAME);
        if (categoryParameter == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, CATEGORY_PARAMNAME);
        }
        short category = 0;
        try {
            category = FieldKey.categoryToShort(categoryParameter);
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CATEGORY_PARAMNAME + " = " + categoryParameter);
        }
        String typeParameter = requestMap.getParameter(TYPE_PARAMNAME);
        if (typeParameter == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, TYPE_PARAMNAME);
        }
        typeParameter = typeParameter.trim();
        if (typeParameter.length() == 0) {
            responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
            responseParameters.setErrorMessage("corpus_err_missingfieldtype", null);
            return;
        }
        if (category == FieldKey.SPECIAL_CATEGORY) {
            if (typeParameter.equals("soustitre")) {
                newFieldKey = FieldKey.SOUSTITRE;
            } else if (typeParameter.equals("texte")) {
                newFieldKey = FieldKey.TEXTE;
            } else {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TYPE_PARAMNAME + " = " + typeParameter);
            }
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
                responseParameters.setErrorMessage("corpus_err_stillactivated_" + newFieldKey.toString(), null);
                return;
            }
        } else {
            if (!StringUtils.isTechnicalName(typeParameter, false)) {
                responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
                responseParameters.setErrorMessage("corpus_err_badtype", typeParameter);
                return;
            }
            switch (category) {
                case FieldKey.PROPRIETE_CATEGORY:
                    newFieldKey = FieldKey.getProprieteFieldKey(typeParameter);
                    break;
                case FieldKey.ANNEXE_CATEGORY:
                    newFieldKey = FieldKey.getAnnexeFieldKey(typeParameter);
                    break;
                case FieldKey.INFORMATION_CATEGORY:
                    newFieldKey = FieldKey.getInformationFieldKey(typeParameter);
                    break;
                default:
                    throw new SwitchException("category = " + category);
            }
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
                responseParameters.setErrorMessage("corpus_err_existingtype", newFieldKey.toString());
                return;
            }
            if ((category == FieldKey.INFORMATION_CATEGORY) || (category == FieldKey.PROPRIETE_CATEGORY)) {
                String ficheItemTypeParameter = requestMap.getParameter(FICHEITEMTYPE_PARAMNAME);
                if (ficheItemTypeParameter == null) {
                    throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FICHEITEMTYPE_PARAMNAME);
                }
                if (ficheItemTypeParameter.length() == 0) {
                    responseParameters.setErrorMessage("corpus_err_noficheitemtype", typeParameter);
                    responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDCREATIONFORM_PAGE);
                    return;
                }
                try {
                    ficheItemType = CorpusMetadataUtils.ficheItemTypeToShort(ficheItemTypeParameter);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FICHEITEMTYPE_PARAMNAME + " = " + ficheItemTypeParameter);
                }
            }
        }
    }

    public static String getCommandString() {
        return "FieldCreation";
    }

    public static String getCommandKey(short category) {
        switch (category) {
            case FieldKey.PROPRIETE_CATEGORY:
                return "CRP-04";
            case FieldKey.ANNEXE_CATEGORY:
                return "CRP-05";
            case FieldKey.INFORMATION_CATEGORY:
                return "CRP-06";
            default:
                throw new SwitchException("category = " + category);
        }
    }

    public static String getCommandKey(FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            return "CRP-02";
        }
        if (fieldKey.equals(FieldKey.TEXTE)) {
            return "CRP-03";
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

}
