/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FieldRemoveCommand extends AbstractCorpusCommand {

    public final static String FIELDKEY_PARAMNAME = "fieldkey";
    CorpusField corpusField;

    public FieldRemoveCommand() {
    }

    protected void doCorpusCommand() {
        try {
            String key = corpusField.getFieldKey().toString();
            responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDREMOVEFORM_PAGE);
            corpusMetadataEditor.removeCorpusField(corpusField);
            responseParameters.setDoneMessage("corpus_done_fieldremove", key);
        } catch (MandatoryFieldException mfe) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected void checkCorpusParameters() throws ParameterException {
        String fieldKeyValue = requestMap.getParameter(FIELDKEY_PARAMNAME);
        if (fieldKeyValue == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FIELDKEY_PARAMNAME);
        }
        fieldKeyValue = fieldKeyValue.trim();
        if (fieldKeyValue.length() == 0) {
            responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDREMOVEFORM_PAGE);
            responseParameters.setErrorMessage("corpus_err_missingfieldtype", null);
            return;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldKeyValue);
        } catch (java.text.ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELDKEY_PARAMNAME + " = " + fieldKeyValue);
        }
        corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if ((corpusField == null) || (fieldKey.isMandatoryFieldKey())) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELDKEY_PARAMNAME + " = " + fieldKeyValue);
        }
        if (!RequestUtils.checkConfirmPassword(requestMap, bdfServer)) {
            responseParameters.setHtmlPageCode(CorpusResponseParameters.FIELDREMOVEFORM_PAGE);
            responseParameters.setErrorMessage("global_err_password", null);
            return;
        }
    }

    public static String getCommandString() {
        return "FieldRemove";
    }

    public static String getCommandKey() {
        return "CRP-08";
    }

}
