/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.include.*;
import fr.exemole.bdfserver.request.ParameterException;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class IncludeUIAddCommand extends AbstractCorpusCommand {

    public static String SUBSET_PARAMNAME = "subset";
    public static String POIDSFILTER_PARAMNAME = "poidsfilter";
    public static String NO_POIDSFILTER_PARAMVALUE = "0";
    public static String YES_POIDSFILTER_PARAMVALUE = "1";
    public static String POIDSFILTERVALUE_PARAMNAME = "poidsfiltervalue";
    CorpusUI corpusUI;
    Subset subset;
    short pageType;
    FilteredIncludeKey filteredIncludeKey;

    public IncludeUIAddCommand() {
    }

    protected void doCorpusCommand() {
        corpusUI.getComponentUIEditor().getIncludeUI(filteredIncludeKey, true);
        responseParameters.setHtmlPageCode(pageType);
        responseParameters.setDoneMessage("corpus_done_includeuiadd", filteredIncludeKey);
        bdfServer.saveUIManager();
    }

    protected void checkCorpusParameters() throws ParameterException {
        String subsetString = getMandatoryParameter(SUBSET_PARAMNAME);
        SubsetKey subsetKey = null;
        try {
            subsetKey = SubsetKey.parse(subsetString);
        } catch (java.text.ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, SUBSET_PARAMNAME + " = " + subsetString);
        }
        subset = bdfServer.getBasedefiches().getSubset(subsetKey);
        if (subset == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, SUBSET_PARAMNAME + " = " + subsetString);
        }
        IncludeKey includeKey = null;
        if (subset instanceof Thesaurus) {
            pageType = CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE;
            includeKey = IncludeKey.createIndexationInstance(subsetKey);
        } else if (subset instanceof Corpus) {
            pageType = CorpusResponseParameters.INSERTFORM_LIAGE_PAGE;
            includeKey = IncludeKey.createLiageInstance(subsetKey);
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, SUBSET_PARAMNAME + " = " + subsetString);
        }
        String poidsFilterString = getMandatoryParameter(POIDSFILTER_PARAMNAME);
        int poidsFilter = -1;
        if (poidsFilterString.equals(YES_POIDSFILTER_PARAMVALUE)) {
            String poidsFilterValue = requestMap.getParameter(POIDSFILTERVALUE_PARAMNAME);
            if (poidsFilterValue == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, POIDSFILTERVALUE_PARAMNAME);
            }
            boolean error = false;
            try {
                poidsFilter = Integer.parseInt(poidsFilterValue);
                if (poidsFilter < 1) {
                    error = true;
                }
            } catch (NumberFormatException nfe) {
                error = true;
            }
            if (error) {
                responseParameters.setHtmlPageCode(pageType);
                responseParameters.setErrorMessage("corpus_err_poidsfiltervalue", poidsFilterValue);
                return;
            }
        } else if (!poidsFilterString.equals(NO_POIDSFILTER_PARAMVALUE)) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, POIDSFILTER_PARAMNAME + " = " + poidsFilterString);
        }
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        if (corpusUI.ownsToRattachement(subsetKey)) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, SUBSET_PARAMNAME + " = " + subsetString);
        }
        filteredIncludeKey = new FilteredIncludeKey(includeKey, poidsFilter);
        if (corpusUI.getIncludeUI(filteredIncludeKey) != null) {
            responseParameters.setHtmlPageCode(pageType);
            responseParameters.setErrorMessage("corpus_err_existingincludekey", filteredIncludeKey.toString());
            return;
        }
    }

    public static String getCommandString() {
        return "IncludeUIAdd";
    }

}
