/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import java.util.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.parsers.*;
import net.mapeadores.basedefiches.include.*;


/**
 *
 * @author Vincent Calame
 */
public class IncludeUIChangeCommand extends AbstractCorpusCommand {

    public final static String DEFAULTLIBELLEUSAGE_PARAMNAME = "defaultlibelleusage";
    private CorpusUI corpusUI;
    private ComponentUIEditor editor;
    private IncludeUI includeUI;
    private short pageType;

    public IncludeUIChangeCommand() {
    }

    public static String getCommandString() {
        return "IncludeUIChange";
    }

    protected void doCorpusCommand() {
        responseParameters.setHtmlPageCode(pageType);
        boolean done = updateIncludeUI(includeUI);
        if (done) {
            responseParameters.setDoneMessage("corpus_done_includeuichange", includeUI.getFiteredIncludeKey().toString());
        }
        bdfServer.saveUIManager();
    }

    protected void checkCorpusParameters() throws ParameterException {
        String key = getMandatoryParameter(KEY_PARAMNAME);
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        editor = corpusUI.getComponentUIEditor();
        try {
            FilteredIncludeKey filteredIncludeKey = FilteredIncludeKey.parse(key);
            includeUI = (IncludeUI) corpusUI.getIncludeUI(filteredIncludeKey);
            if (includeUI == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
            }
        } catch (java.text.ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
        }
        IncludeKey includeKey = includeUI.getFiteredIncludeKey().getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            pageType = CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE;
        } else if (includeKey.isLiageIncludeKey()) {
            pageType = CorpusResponseParameters.INSERTFORM_LIAGE_PAGE;
        } else {
            if ((includeKey.equals(IncludeKey.LIAGE)) || (includeKey.equals(IncludeKey.ASSOCIATION)) || (includeKey.equals(IncludeKey.RATTACHEMENT))) {
                pageType = CorpusResponseParameters.INSERTFORM_PAGE;
            } else {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
            }
        }
    }

    private boolean updateIncludeUI(IncludeUI includeUI) {
        String param = requestMap.getParameter(DEFAULTLIBELLEUSAGE_PARAMNAME);
        if ((param != null) && (param.equals("1"))) {
            if (includeUI.useDefaultLibelle()) {
                return false;
            } else {
                editor.setDefaultLibelleUsage(includeUI, true);
                return true;
            }
        }
        editor.setDefaultLibelleUsage(includeUI, false);
        TextParser textParser = new TextParser();

        Set set = requestMap.getParameterNameSet();
        boolean done = false;
        for (Iterator it = set.iterator(); it.hasNext();) {
            String paramName = (String) it.next();
            if (!paramName.startsWith("lib/")) {
                continue;
            }
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(4));
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
                textParser.setQuoteOptions(typoOptions.getQuoteOptions());
                String nvlibelle = textParser.parseString(requestMap.getParameter(paramName));
                boolean b = editor.setLibelleString(includeUI, currentLangInteger, nvlibelle);
                if (b) {
                    done = true;
                }
            } catch (LangIntegerException ile) {
            }
        }
        return done;
    }

}
