/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.request.ParameterException;
import net.mapeadores.basedefiches.include.*;


/**
 *
 * @author Vincent Calame
 */
public class IncludeUIRemoveCommand extends AbstractCorpusCommand {

    FilteredIncludeKey filteredIncludeKey;
    CorpusUI corpusUI;
    short pageType;

    public IncludeUIRemoveCommand() {
    }

    protected void doCorpusCommand() {
        responseParameters.setHtmlPageCode(pageType);
        corpusUI.removeFilteredInclude(filteredIncludeKey);
        responseParameters.setDoneMessage("corpus_done_includeuiremove", filteredIncludeKey.toString());
        bdfServer.saveUIManager();
    }

    protected void checkCorpusParameters() throws ParameterException {
        String key = getMandatoryParameter(KEY_PARAMNAME);
        try {
            filteredIncludeKey = FilteredIncludeKey.parse(key);
        } catch (java.text.ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
        }
        IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            pageType = CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE;
        } else if (includeKey.isLiageIncludeKey()) {
            pageType = CorpusResponseParameters.INSERTFORM_LIAGE_PAGE;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
        }
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        if (corpusUI.getIncludeUI(filteredIncludeKey) == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, KEY_PARAMNAME + " = " + key);
        }
    }

    public static String getCommandString() {
        return "IncludeUIRemove";
    }

}
