/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.*;
import java.util.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class IntitulesCommand extends AbstractCorpusCommand {

    public IntitulesCommand() {
    }

    protected void doCorpusCommand() {
        boolean done = false;
        TextParser textParser = new TextParser();
        Set set = requestMap.getParameterNameSet();
        for (Iterator it = set.iterator(); it.hasNext();) {
            String paramName = (String) it.next();
            int idx = paramName.lastIndexOf('/');
            if (idx == -1) {
                continue;
            }
            String fieldName = paramName.substring(0, idx);
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(idx + 1));
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
                textParser.setQuoteOptions(typoOptions.getQuoteOptions());
                String nvlibelle = textParser.parseString(requestMap.getParameter(paramName));
                if (fieldName.startsWith("intitule_")) {
                    String intituleName = fieldName.substring(9);
                    boolean stepDone = changeIntitule(intituleName, nvlibelle, currentLangInteger);
                    if (stepDone) {
                        done = true;
                    }
                } else {
                    try {
                        FieldKey fieldKey = FieldKey.parse(fieldName);
                        boolean stepDone = changeIntitule(fieldKey, nvlibelle, currentLangInteger);
                        if (stepDone) {
                            done = true;
                        }
                    } catch (java.text.ParseException pe) {
                    }
                }
            } catch (LangIntegerException ile) {
            }
        }
        responseParameters.setHtmlPageCode(CorpusResponseParameters.INTITULESFORM_PAGE);
        if (done) {
            responseParameters.setDoneMessage("corpus_done_intitules", null);
            clearDefaultTemplates();
        }
    }

    protected void checkCorpusParameters() throws ParameterException {
    }

    private boolean changeIntitule(String intituleName, String nvlibelle, int currentLangInteger) {
        Intitule intitule = BasedefichesUtils.getOrCreateIntitule(corpusMetadataEditor, intituleName);
        return corpusMetadataEditor.setIntituleLibelle(intitule, nvlibelle, currentLangInteger);
    }

    private boolean changeIntitule(FieldKey fieldKey, String nvlibelle, int currentLangInteger) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField != null) {
            return corpusMetadataEditor.setFieldLibelle(corpusField, nvlibelle, currentLangInteger);
        } else {
            return false;
        }
    }

    public static String getCommandString() {
        return "Intitules";
    }

}
