/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import java.util.*;
import net.mapeadores.basedefiches.format.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class OptionsCommand extends AbstractCorpusCommand {

    public static final String DEFAULTSPHERE_SUFFIX = "|defaultsphere";
    public static final String BLOCKDISPLAY_SUFFIX = "|blockdisplay";
    public static final String SUBFIELDDISPLAY_SUFFIX = "|subfielddisplay";
    public static final String CURRENCYARRAY_SUFFIX = "|currencyarray";
    public static final String BASEURL_SUFFIX = "|baseurl";
    public static final String IMAGESOURCE_SUFFIX = "|imagesource";
    public static final String ALBUMKEY_SUFFIX = "|albumkey";
    public static final String TITREGENERATIONPATTERN_PARAMNAME = "titregenerationpattern";
    public static final String TITREGENERATION_SOURCE_PARAMNAME = "titregeneration_source";
    public static final String TITREGENERATION_PATTERN_PARAMNAME = "titregeneration_pattern";
    public static final String TITREGENERATION_INSTRUCTION_PARAMNAME = "titregeneration_instruction";
    public static final String ADDRESSFIELDARRAY_SUFFIX = "|addressfieldarray";
    public static final String GEOLOCALISATIONFIELD_PARAMNAME = "geolocalisationfield";
    BasedefichesFormatDef ficheFormatDef = null;
    boolean initTitreGeneration = false;
    boolean initGeolocalisationField = false;
    CorpusField geolocalisationField = null;

    public OptionsCommand() {
    }

    protected void doCorpusCommand() {
        boolean done = false;
        if (initTitreGeneration) {
            CorpusField titreField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
            try {
                done = corpusMetadataEditor.setFieldOption(titreField, FieldOptions.TITREGENERATIONFORMAT_OPTION, ficheFormatDef);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
        if (initGeolocalisationField) {
            boolean geoDone = false;
            try {
                geoDone = corpusMetadataEditor.setGeolocalisationField(geolocalisationField);
                if (geoDone) {
                    done = true;
                }
            } catch (IllegalArgumentException iae) {
                throw new ImplementationException(iae);
            }
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            if (setOptions(corpusField)) {
                done = true;
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            if (setOptions(corpusField)) {
                done = true;
            }
        }
        responseParameters.setHtmlPageCode(CorpusResponseParameters.OPTIONSFORM_PAGE);
        if (done) {
            responseParameters.setDoneMessage("corpus_done_options", null);
            clearDefaultTemplates();
        }
    }

    private boolean setOptions(CorpusField corpusField) {
        boolean done = false;
        if (FieldOptions.testFieldOption(FieldOptions.DEFAULTSPHEREKEY_OPTION, corpusField)) {
            if (setDefaultSphere(corpusField)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.INFORMATIONDISPLAY_OPTION, corpusField)) {
            if (setDisplay(corpusField)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.SUBFIELDDISPLAY_OPTION, corpusField)) {
            if (setSubfieldDisplay(corpusField)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.CURRENCYARRAY_OPTION, corpusField)) {
            if (setCurrencyArray(corpusField)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.ADDRESSFIELDARRAY_OPTION, corpusField)) {
            if (setAddressFieldArray(corpusField)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.BASEURL_OPTION, corpusField)) {
            if (setStringOption(corpusField, FieldOptions.BASEURL_OPTION, BASEURL_SUFFIX)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.IMAGESOURCE_OPTION, corpusField)) {
            if (setStringOption(corpusField, FieldOptions.IMAGESOURCE_OPTION, IMAGESOURCE_SUFFIX)) {
                done = true;
            }
        }
        if (FieldOptions.testFieldOption(FieldOptions.ALBUMKEY_OPTION, corpusField)) {
            if (setAlbumKey(corpusField)) {
                done = true;
            }
        }
        return done;
    }

    protected void checkCorpusParameters() throws ParameterException {
        String formatSource = requestMap.getParameter(TITREGENERATION_SOURCE_PARAMNAME);
        if (formatSource == null) {
            initTitreGeneration = false;
        } else {
            initTitreGeneration = true;
            formatSource = formatSource.trim();
            if (formatSource.length() > 0) {
                String formatPattern = requestMap.getParameter(TITREGENERATION_PATTERN_PARAMNAME);
                if (formatPattern == null) {
                    throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, TITREGENERATION_PATTERN_PARAMNAME);
                }
                String formatInstruction = requestMap.getParameter(TITREGENERATION_INSTRUCTION_PARAMNAME);
                if (formatInstruction == null) {
                    throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, TITREGENERATION_INSTRUCTION_PARAMNAME);
                }
                FicheFormatErrorHandler ffeh = new FicheFormatErrorHandler();
                ffeh.setFormatSource(formatSource);
                ffeh.setFormatPattern(formatPattern);
                ffeh.setFormatInstruction(formatInstruction);
                ficheFormatDef = BasedefichesFormatDefParser.parse(formatSource, formatPattern, formatInstruction, ffeh);
                if (ffeh.hasError()) {
                    responseParameters.setHtmlPageCode(CorpusResponseParameters.OPTIONSFORM_PAGE);
                    responseParameters.setMultiErrorMessage("corpus_err_options_list", ffeh.getErrorList());
                    return;
                }
            }
        }
        String geolocalisationFieldString = requestMap.getParameter(GEOLOCALISATIONFIELD_PARAMNAME);
        if (geolocalisationFieldString == null) {
            initGeolocalisationField = false;
        } else {
            initGeolocalisationField = true;
            if (geolocalisationFieldString.length() == 0) {
                geolocalisationField = null;
            } else {
                boolean done = false;
                try {
                    FieldKey fieldKey = FieldKey.parse(geolocalisationFieldString);
                    if (fieldKey.isProprieteFieldKey()) {
                        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                        if ((corpusField != null) && (corpusField.getFicheItemType() == CorpusField.GEOPOINT_FIELD)) {
                            done = true;
                            geolocalisationField = corpusField;
                        }
                    }
                } catch (java.text.ParseException pe) {
                }
                if (!done) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, GEOLOCALISATIONFIELD_PARAMNAME + " = " + geolocalisationFieldString);
                }
            }
        }
    }

    public static String getCommandString() {
        return "Options";
    }

    public static String getCommandKey() {
        return "CRP-07";
    }

    private boolean setDisplay(CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + BLOCKDISPLAY_SUFFIX);
        short value = FieldOptions.INLINE_DISPLAY;
        if ((param != null) && (param.equals("1"))) {
            value = FieldOptions.BLOCK_DISPLAY;
        }
        boolean done = false;
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.INFORMATIONDISPLAY_OPTION, new Short(value));
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setDefaultSphere(CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + DEFAULTSPHERE_SUFFIX);
        if (param == null) {
            return false;
        }
        boolean done = false;
        SubsetKey defaultSphereKey = null;
        if (param.length() > 0) {
            try {
                defaultSphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, param);
            } catch (java.text.ParseException pe) {
            }
        }
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.DEFAULTSPHEREKEY_OPTION, defaultSphereKey);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setAlbumKey(CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + ALBUMKEY_SUFFIX);
        if (param == null) {
            return false;
        }
        boolean done = false;
        SubsetKey albumKey = null;
        if (param.length() > 0) {
            try {
                albumKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, param);
            } catch (java.text.ParseException pe) {
            }
        }
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.ALBUMKEY_OPTION, albumKey);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setSubfieldDisplay(CorpusField corpusField) {
        if (!FieldOptions.haveSubfield(corpusField.getFicheItemType())) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + SUBFIELDDISPLAY_SUFFIX);
        boolean done = false;
        try {
            if ((param == null) || (!param.equals("1"))) {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.SUBFIELDDISPLAY_OPTION, null);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setStringOption(CorpusField corpusField, String optionName, String suffix) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + suffix);
        System.out.println("String option = " + param);
        try {
            boolean done = corpusMetadataEditor.setFieldOption(corpusField, optionName, param);
            return done;
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private boolean setAddressFieldArray(CorpusField corpusField) {
        if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + ADDRESSFIELDARRAY_SUFFIX);
        boolean done = false;
        try {
            CorpusField[] addressFieldArray = new CorpusField[0];
            if (param != null) {
                String[] tokenArray = StringUtils.getTechnicalTokens(param, false);
                int length = tokenArray.length;
                addressFieldArray = new CorpusField[length];
                int p = 0;
                for (int i = 0; i < length; i++) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(tokenArray[i]);
                        CorpusField otherCorpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                        if (otherCorpusField != null) {
                            addressFieldArray[p] = otherCorpusField;
                            p++;
                        }
                    } catch (java.text.ParseException pe) {
                    }
                }
                if (p < length) {
                    CorpusField[] tempArray = new CorpusField[p];
                    System.arraycopy(addressFieldArray, 0, tempArray, 0, p);
                    addressFieldArray = tempArray;
                }
            }
            if (addressFieldArray.length == 0) {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.ADDRESSFIELDARRAY_OPTION, null);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.ADDRESSFIELDARRAY_OPTION, addressFieldArray);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setCurrencyArray(CorpusField corpusField) {
        if (corpusField.getFicheItemType() != CorpusField.MONTANT_FIELD) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + CURRENCYARRAY_SUFFIX);
        boolean done = false;
        try {
            int[] currencyIntegerArray = new int[0];
            if (param != null) {
                String[] tokenArray = StringUtils.getTechnicalTokens(param, false);
                int length = tokenArray.length;
                currencyIntegerArray = new int[length];
                int p = 0;
                for (int i = 0; i < length; i++) {
                    try {
                        int currencyInteger = CurrencyInteger.parse(tokenArray[i]);
                        currencyIntegerArray[p] = currencyInteger;
                        p++;
                    } catch (CurrencyIntegerException cie) {
                    }
                }
                if (p < length) {
                    int[] tempArray = new int[p];
                    System.arraycopy(currencyIntegerArray, 0, tempArray, 0, p);
                    currencyIntegerArray = tempArray;
                }
            }
            if (currencyIntegerArray.length == 0) {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.CURRENCYARRAY_OPTION, null);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptions.CURRENCYARRAY_OPTION, currencyIntegerArray);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }


    class FicheFormatErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        String formatPattern;
        String formatInstruction;
        String formatSource;
        List<ResourceItem> errorList = new ArrayList();

        void setFormatSource(String formatSource) {
            this.formatSource = formatSource;
        }

        void setFormatPattern(String formatPattern) {
            this.formatPattern = formatPattern;
        }

        void setFormatInstruction(String formatInstruction) {
            this.formatInstruction = formatInstruction;
        }

        boolean hasError() {
            return (errorList.size() != 0);
        }

        List getErrorList() {
            return errorList;
        }

        private void addError(String key, Object value) {
            errorList.add(new ResourceItem("corpus_err_titregeneration_" + key, value));
        }

        public void nullFormatSourceError() {
            throw new ImplementationException("nullFormatSourceError");
        }

        public void wrongFormatSourceError() {
            addError("wrongformatsource", formatSource);
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            addError("wrongformatsource", formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            addError("unknownargumentkey", argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            addError("missingargumentvalue", argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            String[] vals = {argumentKey, argumentValue};
            addError("wrongargumentvalue", vals);
        }

    }

}
