/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.commands.*;
import fr.exemole.bdfserver.servlets.BdfHtmlPage;
import org.xml.sax.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class CommandsPage extends CorpusHtmlPage {

    public CommandsPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        startH1();
        localize("corpus_title_commands");
        endH1();
        printMessage();

        CommandDefList commandDefList = getCoreCommandList();
        printCommandDefList(this, commandDefList);
        end();
    }

    private CommandDefList getCoreCommandList() {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("/fr/exemole/bdfserver/servlets/corpus/corpus-command-list.xml");
            CommandDefList result = CommandFactory.build(inputStream);
            return result;
        } catch (SAXException saxe) {
            throw new InternalResourceException(saxe);
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }

    }

    public static void printCommandDefList(BdfHtmlPage page, CommandDefList commandDefList) {
        int count = commandDefList.getCommandDefCount();
        for (int i = 0; i < count; i++) {
            CommandDef commandDef = commandDefList.getCommandDef(i);
            String key = commandDef.getKey();
            String name = commandDef.getName();
            CommandBox commandBox = new CommandBox();
            commandBox.setFormAction("Corpus");
            commandBox.setCommandFamily("CRP");
            commandBox.setCommandString(name);
            commandBox.setCommandKey(key);
            page.startCommandBox(commandBox);
            int paramCount = commandDef.getParamDefCount();
            for (int j = 0; j < paramCount; j++) {
                ParamDef paramDef = commandDef.getParamDef(j);
                String paramName = paramDef.getName();
                page.startP();
                page.append(paramName);
                page.printColon();
                page.appendMediumInput(paramName, "");
                page.endP();
            }
            page.endCommandBox("global_submit_ok");
        }
    }

}
