/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.html.utils.SubsetGroupingPrinter;
import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.servlets.util.*;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.util.html.*;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCreationPage extends CorpusHtmlPage {

    SubsetGroupingPrinter subsetGroupingWriter;

    public CorpusCreationPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        allowsTest();
        subsetGroupingWriter = new SubsetGroupingPrinter(bdfServer.getBasedefiches(), this, workingLangInteger);
        subsetGroupingWriter.setOnlySubIdValues(true);
        subsetGroupingWriter.setSubsetStringPrint(true);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setCommandString(CorpusCreationCommand.getCommandString());
        commandBox.setCommandKey(CorpusCreationCommand.getCommandKey());
        startCommandBox(commandBox);
        startP();
        printLib("corpus_lib_corpuscreation");
        appendMediumInput(CorpusCreationCommand.NEWCORPUS_PARAMNAME, "");
        endP();
        printRadio(CorpusCreationCommand.NORMAL_PARAMVALUE, true);
        printThesaurusRadio();
        printCorpusRadio();
        endCommandBox("corpus_submit_corpuscreation");
        end();
    }

    private void printRadio(String value, boolean checked) {
        startP();
        appendRadioInput(CorpusCreationCommand.CORPUSTYPE_PARAMNAME, value, checked, "corpus_lib_corpustype_" + value);
        endP();
    }

    private void printThesaurusRadio() {
        SubsetKeyTree keyTree = bdfServer.getUIManager().getThesaurusSubsetKeyTree();
        if (keyTree.getSubsetCount() > 0) {
            subsetGroupingWriter.setSubsetGrouping(keyTree);
            HTMLAttributes selectAttributes = new HTMLAttributes();
            selectAttributes.setName(CorpusCreationCommand.MASTERTHESAURUS_PARAMNAME);
            printRadio(CorpusCreationCommand.THESAURUSSLAVE_PARAMVALUE, false);
            startUl();
            startLi();
            startP();
            printLibColon("global_type_thesaurus");
            subsetGroupingWriter.printTreeSelect(selectAttributes);
            endP();
            endLi();
            endUl();
        }
    }

    private void printCorpusRadio() {
        SubsetGrouping subsetGrouping = SubsetGroupingUtils.checkNoSlaveSubset(bdfServer.getBasedefiches(), bdfServer.getUIManager().getCorpusSubsetKeyTree());
        if (subsetGrouping.getSubsetKeyGroupList().size() > 0) {
            subsetGroupingWriter.setSubsetGrouping(subsetGrouping);
            HTMLAttributes selectAttributes = new HTMLAttributes();
            selectAttributes.setName(CorpusCreationCommand.MASTERCORPUS_PARAMNAME);
            printRadio(CorpusCreationCommand.CORPUSSLAVE_PARAMVALUE, false);
            startUl();
            startLi();
            startP();
            printLibColon("global_type_corpus");
            subsetGroupingWriter.printTreeSelect(selectAttributes);
            endP();
            endLi();
            endUl();
        }
    }

}
