/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.html.utils.SubsetGroupingPrinter;
import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.*;
import java.util.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.servlets.util.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class CorpusHtmlPage extends BdfHtmlPage implements CorpusConstants {

    protected CorpusResponseParameters responseParameters;
    protected Corpus corpus;
    protected CorpusMetadata corpusMetadata;
    protected int[] workingArray;
    protected CorpusUI corpusUI;

    public CorpusHtmlPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(BDF_JS);
        this.corpus = responseParameters.getCorpus();
        if (corpus != null) {
            corpusMetadata = corpus.getCorpusMetadata();
        }
        this.responseParameters = responseParameters;
    }

    protected void customInit() {
        if (needWorkingLangArray()) {
            LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
            workingArray = langConfig.getWorkingLangIntegerArray();
        }
        if (needCorpusUI()) {
            corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        }
    }

    public boolean needWorkingLangArray() {
        return false;
    }

    public boolean needCorpusUI() {
        return false;
    }

    public static ResponseHandler getInstance(CorpusResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case BdfResponseParameters.MESSAGE_SERVLETPAGE:
                return new MessagePage(responseParameters);
            case CorpusResponseParameters.INDEX_PAGE:
                return new IndexPage(responseParameters);
            case CorpusResponseParameters.INTITULESFORM_PAGE:
                return new IntitulesFormPage(responseParameters);
            case CorpusResponseParameters.FIELDCREATIONFORM_PAGE:
                return new FieldCreationFormPage(responseParameters);
            case CorpusResponseParameters.OPTIONSFORM_PAGE:
                return new OptionsFormPage(responseParameters);
            case CorpusResponseParameters.CORPUSCREATIONFORM_PAGE:
                return new CorpusCreationPage(responseParameters);
            case CorpusResponseParameters.INSERTFORM_PAGE:
                return new InsertFormPage(responseParameters);
            case CorpusResponseParameters.UIFORM_PAGE:
                return new UIFormPage(responseParameters);
            case CorpusResponseParameters.CORPUSREMOVEFORM_PAGE:
                return new CorpusRemovePage(responseParameters);
            case CorpusResponseParameters.FIELDREMOVEFORM_PAGE:
                return new FieldRemovePage(responseParameters);
            case CorpusResponseParameters.INSERTFORM_INDEXATION_PAGE:
                return new InsertFormIndexationPage(responseParameters);
            case CorpusResponseParameters.INSERTFORM_LIAGE_PAGE:
                return new InsertFormLiagePage(responseParameters);
            case CorpusResponseParameters.INSERTFORM_COMMENT_PAGE:
                return new InsertFormCommentPage(responseParameters);
            case CorpusResponseParameters.COMMANDS_PAGE:
                return new CommandsPage(responseParameters);
        }
        return null;
    }

    protected void printTitre() {
        String currentPage = DefaultCommand.getPageTypeString(responseParameters.getResponseCode());
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Corpus", "corpus_title_index");
        printTiret();
        printLinks(corpus, currentPage);
        endP();
        startH1();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        if (isWithJQuery()) {
            printTiret();
            BdfHtmlUtils.printChangeSubsetSelect(this, corpus, currentPage);
        }
        endH1();
        printMessage();
    }

    protected void printInsertionTitre() {
        String subpage = DefaultCommand.getPageTypeString(responseParameters.getResponseCode());
        String currentPage = DefaultCommand.INSERTFORM_PAGE_PARAMVALUE;
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Corpus", "corpus_title_index");
        printTiret();
        printLinks(corpus, currentPage);
        endP();
        startH1();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        if (isWithJQuery()) {
            printTiret();
            BdfHtmlUtils.printChangeSubsetSelect(this, corpus, currentPage);
        }
        endH1();
        startP(CssClass.P_TOOLBARLINKS);
        printInsertLinks(corpus, subpage);
        endP();
        printMessage();

    }

    private void printInsertLinks(Corpus currentCorpus, String pageActu) {
        printLink(currentCorpus, DefaultCommand.INSERTFORM_PAGE_PARAMVALUE, "corpus_title_specialinclude", pageActu);
        printTiret();
        printLink(currentCorpus, DefaultCommand.INSERTFORM_INDEXATION_PAGE_PARAMVALUE, "corpus_title_insert_indexation", pageActu);
        printTiret();
        printLink(currentCorpus, DefaultCommand.INSERTFORM_LIAGE_PAGE_PARAMVALUE, "corpus_title_insert_liage", pageActu);
        printTiret();
        printLink(currentCorpus, DefaultCommand.INSERTFORM_COMMENT_PAGE_PARAMVALUE, "corpus_title_insert_comment", pageActu);
    }

    protected void printLinks(Corpus currentCorpus, String pageActu) {
        printLink(currentCorpus, DefaultCommand.INTITULESFORM_PAGE_PARAMVALUE, "corpus_title_intitules", pageActu);
        printTiret();
        printLink(currentCorpus, DefaultCommand.FIELDCREATIONFORM_PAGE_PARAMVALUE, "corpus_title_fieldcreation", pageActu);
        printTiret();
        printCommandLink(currentCorpus, DefaultCommand.OPTIONSFORM_PAGE_PARAMVALUE, OptionsCommand.getCommandKey(), pageActu);
        if (CorpusMetadataUtils.containsRemoveableField(currentCorpus.getCorpusMetadata())) {
            printTiret();
            printCommandLink(currentCorpus, DefaultCommand.FIELDREMOVEFORM_PAGE_PARAMVALUE, FieldRemoveCommand.getCommandKey(), pageActu);
        }
        printTiret();
        ;
        printLink(currentCorpus, DefaultCommand.INSERTFORM_PAGE_PARAMVALUE, "corpus_title_insert", pageActu);
        printTiret();
        printLink(currentCorpus, DefaultCommand.UIFORM_PAGE_PARAMVALUE, "corpus_title_ui", pageActu);
        if (BasedefichesUtils.isRemoveable(currentCorpus)) {
            printTiret();
            printCommandLink(currentCorpus, DefaultCommand.CORPUSREMOVEFORM_PAGE_PARAMVALUE, CorpusRemoveCommand.getCommandKey(), pageActu);
        }
    }

    protected void printLink(Corpus currentCorpus, String page, String l10nKey, String pageActu) {
        if (pageActu.equals(page)) {
            startSpan(CssClass.SPAN_CURRENTPAGE);
            localize(l10nKey);
            endSpan();
        } else {
            String href = "Corpus?" + RequestUtils.toParameterString(currentCorpus) + "&amp;page=" + page;
            appendA(href, l10nKey);
        }
    }

    protected void printCommandLink(Corpus currentCorpus, String page, String commandKey, String pageActu) {
        if (pageActu.equals(page)) {
            startSpan(CssClass.SPAN_CURRENTPAGE);
            appendCommandLocString(commandKey);
            endSpan();
        } else {
            String href = "Corpus?" + RequestUtils.toParameterString(currentCorpus) + "&amp;page=" + page;
            appendCommandLink(href, commandKey, null);
        }
    }

    protected void printCorpusState(Corpus currentCorpus) {
        CorpusMetadata currentCorpusMetadata = currentCorpus.getCorpusMetadata();
        boolean containsSoustitre = CorpusMetadataUtils.hasSoustitreField(currentCorpusMetadata);
        boolean containsTexte = CorpusMetadataUtils.hasTexteField(currentCorpusMetadata);
        int proprieteCount = currentCorpusMetadata.getProprieteFieldCount();
        int informationCount = currentCorpusMetadata.getInformationFieldCount();
        int annexeCount = currentCorpusMetadata.getAnnexeFieldCount();
        startP();
        startCode();
        if ((!containsSoustitre) && (!containsTexte) && (proprieteCount == 0) && (annexeCount == 0) && (informationCount == 0)) {
            localize("corpus_info_state_onlymandatory");
        } else {
            localize("corpus_info_state_mandatory");
            if (containsSoustitre) {
                appendBr();
                appendBr();
                append(FieldKey.SOUSTITRE.toString());
            }
            if (proprieteCount > 0) {
                appendBr();
                for (int i = 0; i < proprieteCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getProprieteField(i);
                    appendBr();
                    append(corpusField.getFieldKey().toString());
                }
            }
            if (containsTexte) {
                appendBr();
                appendBr();
                append(FieldKey.TEXTE.toString());
            }
            if (annexeCount > 0) {
                appendBr();
                for (int i = 0; i < annexeCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getAnnexeField(i);
                    appendBr();
                    append(corpusField.getFieldKey().toString());
                }
            }
            if (informationCount > 0) {
                appendBr();
                for (int i = 0; i < informationCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getInformationField(i);
                    appendBr();
                    append(corpusField.getFieldKey().toString());
                }
            }
        }
        endCode();
        endP();
    }

    protected void printIncludeUIAddForm(SubsetGrouping subsetGrouping) {
        subsetGrouping = SubsetGroupingUtils.chekcNoRattachementSubset(corpus, subsetGrouping);
        if (subsetGrouping.getSubsetKeyGroupList().size() == 0) {
            return;
        }
        SubsetGroupingPrinter stw = new SubsetGroupingPrinter(bdfServer.getBasedefiches(), this, workingLangInteger);
        beginFormBlock(corpus, IncludeUIAddCommand.getCommandString(), "corpus_title_includeuiadd");
        startP();
        localize("corpus_lib_includeuiadd");
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setName(IncludeUIAddCommand.SUBSET_PARAMNAME);
        stw.setSubsetGrouping(subsetGrouping);
        stw.setSubsetStringPrint(true);
        stw.printTreeSelect(selectAtt);
        endP();
        startP();
        appendRadioInput(IncludeUIAddCommand.POIDSFILTER_PARAMNAME, IncludeUIAddCommand.NO_POIDSFILTER_PARAMVALUE, true, "corpus_lib_poidsfilter_no");
        endP();
        startP();
        appendRadioInput(IncludeUIAddCommand.POIDSFILTER_PARAMNAME, IncludeUIAddCommand.YES_POIDSFILTER_PARAMVALUE, false, "corpus_lib_poidsfilter_yes");
        space();
        appendSmallestInput(IncludeUIAddCommand.POIDSFILTERVALUE_PARAMNAME, "");
        endP();
        endForm("corpus_submit_includeuiadd");
    }

    protected void printSuppBox(IncludeUI includeUI) {
        String key = includeUI.getFiteredIncludeKey().toString();
        BdfHtmlUtils.printDeploieTitle(this, "displaySupp('" + key + "');", "suppbox_" + key, "global_link_remove");
        BdfHtmlUtils.startDeployBlock(this, "suppblock_" + key);
        beginFormBlock(corpus, IncludeUIRemoveCommand.getCommandString(), "");
        addParameter(IncludeUIRemoveCommand.KEY_PARAMNAME, key);
        endForm("corpus_submit_includeuiremove");
        endDiv();

    }

    protected void printChgtBox(IncludeUI includeUI) {
        String key = includeUI.getFiteredIncludeKey().toString();
        BdfHtmlUtils.printDeploieTitle(this, "displayChgt('" + key + "');", "chgtbox_" + key, "global_link_change");
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setId("current_" + key);
        divAtt.addClass(DIV_GLB_DEPLOYBLOCK);
        startDiv(divAtt);
        LibelleHolder libelleHolder = includeUI.getCustomLibelle();
        boolean defaultLibUsage = (libelleHolder == null);
        if (defaultLibUsage) {
            appendP(CssClass.P_SUBTITLE, "corpus_info_defaultlibelleusage");
        } else {
            startTable();
            for (int i = 0; i < workingArray.length; i++) {
                int langInteger = workingArray[i];
                String lib = libelleHolder.getLibelleString(langInteger, false);
                if (lib == null) {
                    lib = "";
                }
                HTMLUtils.printLangValueRow(pw, langInteger, libelleHolder.getLibelleString(langInteger, false), false);
            }
            endTable();
        }
        endDiv();
        BdfHtmlUtils.startDeployBlock(this, "chgtblock_" + key);
        beginFormBlock(corpus, IncludeUIChangeCommand.getCommandString(), "");
        addParameter(IncludeUIChangeCommand.KEY_PARAMNAME, key);
        startP();
        HTMLAttributes checkAttributes = new HTMLAttributes();
        if (isWithJQuery()) {
            checkAttributes.setOnClick("bdf_DeploieFromInvertCheck(this,'libtable_" + key + "');");
        }
        checkAttributes.setName(IncludeUIChangeCommand.DEFAULTLIBELLEUSAGE_PARAMNAME);
        checkAttributes.setValue("1");
        checkAttributes.setChecked((libelleHolder == null));
        addCheck(checkAttributes, "corpus_lib_defaultlibelleusage", null);
        endP();
        HTMLAttributes tableAttributes = new HTMLAttributes();
        tableAttributes.setId("libtable_" + key);
        if (isWithJQuery()) {
            if (defaultLibUsage) {
                tableAttributes.addClass(INVISIBLE);
            }
        }
        startTable(tableAttributes);
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String lib = null;
            if (libelleHolder != null) {
                lib = libelleHolder.getLibelleString(langInteger, false);
            }
            if (lib == null) {
                lib = "";
            }
            printLangFieldRow("lib", langInteger, lib);
        }
        endTable();
        endForm("corpus_submit_includeuichange");
        endDiv();
    }

    private void printList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        appendBr();
        for (int i = 0; i < size; i++) {
            CorpusField corpusField = (CorpusField) list.get(i);
            appendBr();
            pw.print(corpusField.getFieldKey().toString());
        }
    }

    public String getTitle() {
        return "Corpus";
    }

    public String getCssThemeFile() {
        return "corpus.css";
    }

    public String getFormAction() {
        return "Corpus";
    }

}
