/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.util.html.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class FieldCreationFormPage extends CorpusHtmlPage {

    public FieldCreationFormPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        printState();
        if (!CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            printSpecialCreation(FieldKey.SOUSTITRE);
        }
        if (!CorpusMetadataUtils.hasTexteField(corpusMetadata)) {
            printSpecialCreation(FieldKey.TEXTE);
        }
        printCreation(FieldKey.PROPRIETE_CATEGORY);
        printCreation(FieldKey.ANNEXE_CATEGORY);
        printCreation(FieldKey.INFORMATION_CATEGORY);
        end();
    }

    private void printSpecialCreation(FieldKey fieldKey) {
        HTMLAttributes divAttributes = getCommandBoxParameters().getSubmitParameters().getDivAttributes();
        String currentClassName = divAttributes.getClasses();
        divAttributes.setClasses(null);
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setCommandString(FieldCreationCommand.getCommandString());
        commandBox.setCommandKey(FieldCreationCommand.getCommandKey(fieldKey));
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        hiddenParameterMap.put(FieldCreationCommand.CATEGORY_PARAMNAME, FieldKey.categoryToString(FieldKey.SPECIAL_CATEGORY));
        hiddenParameterMap.put(FieldCreationCommand.TYPE_PARAMNAME, fieldKey.toString());
        startCommandBox(commandBox);
        endCommandBox("corpus_submit_fieldactivation");
        hiddenParameterMap.remove(FieldCreationCommand.TYPE_PARAMNAME);
        divAttributes.setClasses(currentClassName);
    }

    private void printCreation(short fieldcategory) {
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setCommandString(FieldCreationCommand.getCommandString());
        commandBox.setCommandKey(FieldCreationCommand.getCommandKey(fieldcategory));
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.getHiddenAttributesMap().put(FieldCreationCommand.CATEGORY_PARAMNAME, FieldKey.categoryToString(fieldcategory));
        startCommandBox(commandBox);
        startP();
        printLibColon("corpus_lib_fieldcreation_type");
        appendMediumInput(FieldCreationCommand.TYPE_PARAMNAME, "");
        endP();
        if (fieldcategory != FieldKey.ANNEXE_CATEGORY) {
            printSelect();
        }
        endCommandBox("corpus_submit_fieldcreation");
    }

    private void printState() {
        appendH2("corpus_title_state");
        printCorpusState(corpus);
    }

    private void printSelect() {
        startP();
        printLibColon("corpus_lib_fieldcreation_ficheitemtype");
        startSelect(FieldCreationCommand.FICHEITEMTYPE_PARAMNAME);
        HTMLAttributes attr = new HTMLAttributes();
        attr.setValue("");
        attr.setSelected(true);
        startOption(attr);
        append("---");
        endOption();
        printFicheItemTypeOption(CorpusField.ITEM_FIELD);
        printFicheItemTypeOption(CorpusField.PERSONNE_FIELD);
        printFicheItemTypeOption(CorpusField.LANGUE_FIELD);
        printFicheItemTypeOption(CorpusField.PAYS_FIELD);
        printFicheItemTypeOption(CorpusField.DATATION_FIELD);
        printFicheItemTypeOption(CorpusField.LINK_FIELD);
        printFicheItemTypeOption(CorpusField.COURRIEL_FIELD);
        printFicheItemTypeOption(CorpusField.NOMBRE_FIELD);
        printFicheItemTypeOption(CorpusField.MONTANT_FIELD);
        printFicheItemTypeOption(CorpusField.GEOPOINT_FIELD);
        printFicheItemTypeOption(CorpusField.PARA_FIELD);
        printFicheItemTypeOption(CorpusField.IMAGE_FIELD);
        endSelect();
        endP();
    }

    private void printFicheItemTypeOption(short ficheItemType) {
        String ficheItemString = CorpusMetadataUtils.ficheItemTypeToString(ficheItemType);
        HTMLAttributes attr = new HTMLAttributes();
        attr.setSelected(false);
        attr.setValue(ficheItemString);
        startOption(attr);
        append("[");
        append(ficheItemString);
        append("] - ");
        localize("corpus_ficheitem_" + ficheItemString);
        endOption();

    }

}
