/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import net.mapeadores.util.html.*;


/**
 *
 * @author Vincent Calame
 */
public class FieldRemovePage extends CorpusHtmlPage {

    public FieldRemovePage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        if (CorpusMetadataUtils.containsRemoveableField(corpusMetadata)) {
            CommandBox commandBox = new CommandBox();
            commandBox.setFormAction("Corpus");
            commandBox.setCommandFamily("CRP");
            commandBox.setCommandString(FieldRemoveCommand.getCommandString());
            commandBox.setCommandKey(FieldRemoveCommand.getCommandKey());
            CommandBoxUtils.insertCorpus(commandBox, corpus);
            startCommandBox(commandBox);
            startP();
            printLib("corpus_lib_fieldremove");
            startSelect(FieldRemoveCommand.FIELDKEY_PARAMNAME);
            HTMLAttributes attr = new HTMLAttributes();
            attr.setValue("");
            attr.setSelected(true);
            startOption(attr);
            append("---");
            endOption();
            if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
                printFieldKeyOption(FieldKey.SOUSTITRE);
            }
            if (CorpusMetadataUtils.hasTexteField(corpusMetadata)) {
                printFieldKeyOption(FieldKey.TEXTE);
            }
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                printFieldKeyOption(corpusField.getFieldKey());
            }
            int annexeCount = corpusMetadata.getAnnexeFieldCount();
            for (int i = 0; i < annexeCount; i++) {
                CorpusField corpusField = corpusMetadata.getAnnexeField(i);
                printFieldKeyOption(corpusField.getFieldKey());
            }
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                printFieldKeyOption(corpusField.getFieldKey());
            }
            endSelect();
            endP();
            printConfirmPassword();
            endCommandBox("corpus_submit_fieldremove");
        }
        end();
    }

    private void printFieldKeyOption(FieldKey fieldKey) {
        appendOption(fieldKey.toString(), false);
    }

}
