/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import java.util.*;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends CorpusHtmlPage {

    SubsetKeyTree corpusKeyTree;

    public IndexPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        corpusKeyTree = bdfServer.getUIManager().getCorpusSubsetKeyTree();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printMessage();
        appendH1("corpus_title_index");
        if (bdfServer.getBasedefiches().getCorpusCount() > 0) {
            printCorpusList();
            printCorpusState();
        }
        String href = "Corpus?page=" + DefaultCommand.CORPUSCREATIONFORM_PAGE_PARAMVALUE;
        startH2();
        appendCommandLink(href, CorpusCreationCommand.getCommandKey(), null);
        endH2();
        startH2();
        startA("Corpus?page=" + DefaultCommand.COMMANDS_PAGE_PARAMVALUE);
        localize("corpus_title_commands");
        endA();
        endH2();
        end();
    }

    private void printCorpusList() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        startUl();
        for (Iterator it = corpusKeyTree.getSubsetKeyGroupList().iterator(); it.hasNext();) {
            SubsetKeyGroup subsetKeyGroup = (SubsetKeyGroup) it.next();
            boolean isTemplate = !subsetKeyGroup.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(subsetKeyGroup.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (Iterator it2 = subsetKeyGroup.getSubsetKeyList().iterator(); it2.hasNext();) {
                SubsetKey subsetKey = (SubsetKey) it2.next();
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                startLi();
                printCorpusInfo(corpus);
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpusState() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        appendH2("corpus_title_state");
        startUl();
        for (Iterator it = corpusKeyTree.getSubsetKeyGroupList().iterator(); it.hasNext();) {
            SubsetKeyGroup subsetKeyGroup = (SubsetKeyGroup) it.next();
            boolean isTemplate = !subsetKeyGroup.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(subsetKeyGroup.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (Iterator it2 = subsetKeyGroup.getSubsetKeyList().iterator(); it2.hasNext();) {
                SubsetKey subsetKey = (SubsetKey) it2.next();
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                startLi();
                printCurrentState(corpus);
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpusInfo(Corpus corpus) {
        startP();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        printTiret();
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        Subset masterSubset = corpus.getMasterSubset();
        if (masterSubset != null) {
            space();
            startSmall();
            append("(");
            if (masterSubset instanceof Thesaurus) {
                printLibColon("corpus_lib_corpustype_thesaurusslave");
                escape(ThesaurusUtils.getThesaurusIntitule((Thesaurus) masterSubset, workingLangInteger));
            } else if (masterSubset instanceof Corpus) {
                printLibColon("corpus_lib_corpustype_corpusslave");
                escape(CorpusMetadataUtils.getCorpusIntituleString((Corpus) masterSubset, workingLangInteger));
            }
            append(")");
            endSmall();
        }
        endP();
        startP(CssClass.P_SMALLLINKS);
        printLinks(corpus, "");
        endP();
    }

    void printCurrentState(Corpus corpus) {
        startP();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        printTiret();
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endP();
        printCorpusState(corpus);
    }

}
