/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class InsertFormCommentPage extends CorpusHtmlPage {

    CorpusUI corpusUI;
    int[] workingArray;

    public InsertFormCommentPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(CORPUS_INSERTFORM_JS);
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        workingArray = langConfig.getWorkingLangIntegerArray();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printInsertionTitre();
        printComment();
        end();
    }

    private void printComment() {
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.COMMENT_LIST);
        if (list.size() > 0) {
            appendH2("corpus_title_commentlist");
            startUl();
            for (Iterator it = list.iterator(); it.hasNext();) {
                CommentUI commentUI = (CommentUI) it.next();
                startLi();
                startP();
                startCode();
                append("[comment_");
                append(commentUI.getName());
                append("]");
                endCode();
                endP();
                printChgtBox(commentUI);
                printSuppBox(commentUI);
                endLi();
            }
            endUl();
        }
        beginFormBlock(corpus, CommentUICreationCommand.getCommandString(), "corpus_title_commentcreation");
        printRadioList(CommentUI.ALL_MASK);
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            HTMLUtils.printLangFieldRow(pw, langInteger, "comment", "", 77, 9);
        }
        endTable();
        endForm("corpus_submit_commentcreation");
    }

    private void printRadioList(int position) {
        startP();
        printLib("corpus_lib_commentlocation");
        printRadio(CommentUIChangeCommand.LOCATION_FORM_PARAMVALUE, (position == CommentUI.FORM_BITVALUE));
        space();
        printRadio(CommentUIChangeCommand.LOCATION_AFFICHAGE_PARAMVALUE, (position == CommentUI.AFFICHAGE_BITVALUE));
        space();
        printRadio(CommentUIChangeCommand.LOCATION_BOTH_PARAMVALUE, (position == CommentUI.ALL_MASK));
        endP();
    }

    private void printRadio(String value, boolean checked) {
        this.appendRadioInput(CommentUIChangeCommand.LOCATION_PARAMNAME, value, checked, "corpus_commentlocation_" + value);
    }

    private String getCurrentPositionValue(int position) {
        if (position == CommentUI.FORM_BITVALUE) {
            return CommentUIChangeCommand.LOCATION_FORM_PARAMVALUE;
        }
        if (position == CommentUI.AFFICHAGE_BITVALUE) {
            return CommentUIChangeCommand.LOCATION_AFFICHAGE_PARAMVALUE;
        }
        if (position == CommentUI.ALL_MASK) {
            return CommentUIChangeCommand.LOCATION_BOTH_PARAMVALUE;
        }
        return null;
    }

    private void printChgtBox(CommentUI commentUI) {
        String name = commentUI.getName();
        BdfHtmlUtils.printDeploieTitle(this, "displayChgt('" + name + "');", "chgtbox_" + name, "global_link_change");
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setId("current_" + name);
        divAtt.addClass(DIV_GLB_DEPLOYBLOCK);
        startDiv(divAtt);
        startP(CssClass.P_SUBTITLE);
        printLib("corpus_lib_commentlocation");
        printPositionList(commentUI.getLocation());
        endP();
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            HTMLUtils.printLangValueRow(pw, langInteger, getLibelle(commentUI, langInteger), true);
        }
        endTable();
        endDiv();
        BdfHtmlUtils.startDeployBlock(this, "chgtblock_" + name);
        beginFormBlock(corpus, CommentUIChangeCommand.getCommandString(), "");
        addParameter(CommentUIChangeCommand.NAME_PARAMNAME, name);
        printRadioList(commentUI.getLocation());
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            HTMLUtils.printLangFieldRow(pw, langInteger, "comment", getLibelle(commentUI, langInteger), 77, 9);
        }
        endTable();
        endForm("corpus_submit_commentchange");
        endDiv();
    }

    private String getLibelle(CommentUI commentUI, int langInteger) {
        String s = commentUI.getLibelleString(langInteger, false);
        if (s == null) {
            return "";
        }
        return s;
    }

    private void printPositionList(int position) {
        boolean suivant = false;
        if ((position & CommentUI.FORM_BITVALUE) != 0) {
            suivant = true;
            printPosition(CommentUIChangeCommand.LOCATION_FORM_PARAMVALUE);
        }
        if ((position & CommentUI.AFFICHAGE_BITVALUE) != 0) {
            if (suivant) {
                printSuivant();
            } else {
                suivant = true;
            }
            printPosition(CommentUIChangeCommand.LOCATION_AFFICHAGE_PARAMVALUE);
        }
    }

    private void printSuivant() {
        append(", ");
    }

    private void printPosition(String val) {
        localize("corpus_commentlocation_" + val);
    }

    private void printSuppBox(CommentUI commentUI) {
        String name = commentUI.getName();
        BdfHtmlUtils.printDeploieTitle(this, "displaySupp('" + name + "');", "suppbox_" + name, "global_link_remove");
        BdfHtmlUtils.startDeployBlock(this, "suppblock_" + name);
        beginFormBlock(corpus, CommentUIRemoveCommand.getCommandString(), "");
        addParameter(CommentUIRemoveCommand.NAME_PARAMNAME, name);
        endFormWithConfirm("corpus_submit_commentremove");
        endDiv();

    }

}
