/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import net.mapeadores.basedefiches.Intitule;
import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class IntitulesFormPage extends CorpusHtmlPage {

    public IntitulesFormPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        customInit();
    }

    public boolean needWorkingLangArray() {
        return true;
    }

    public boolean needCorpusUI() {
        return false;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        beginFormBlock(corpus, IntitulesCommand.getCommandString(), "corpus_title_intitules");
        printFixedSubmit("corpus_submit_intitules");
        appendCorpusIntituleTable(CORPUS);
        appendCorpusIntituleTable(FICHE_EXISTING);
        appendCorpusIntituleTable(FICHE_NEW);
        appendH3("corpus_lib_intitules_special");
        List<CorpusField> specialList = CorpusMetadataUtils.getSpecialCorpusFieldList(corpusMetadata, true);
        for (CorpusField corpusField : specialList) {
            appendCorpusFieldTable(corpusField);
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        if (proprieteCount > 0) {
            appendH3("corpus_lib_intitules_propriete");
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                appendCorpusFieldTable(corpusField);
            }
        }
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        if (annexeCount > 0) {
            appendH3("corpus_lib_intitules_annexe");
            for (int i = 0; i < annexeCount; i++) {
                CorpusField corpusField = corpusMetadata.getAnnexeField(i);
                appendCorpusFieldTable(corpusField);
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        if (informationCount > 0) {
            appendH3("corpus_lib_intitules_information");
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                appendCorpusFieldTable(corpusField);
            }
        }
        endForm("corpus_submit_intitules");
        end();
    }

    private void appendCorpusFieldList(String key, List list) {
        if (list.size() == 0) {
            return;
        }
        appendH3("corpus_lib_intitules_" + key);
        for (Iterator it = list.iterator(); it.hasNext();) {
            CorpusField corpusField = (CorpusField) it.next();
            appendCorpusFieldTable(corpusField);
        }
    }

    private void appendCorpusFieldTable(CorpusField corpusField) {
        startP();
        startCode();
        append("[");
        append(corpusField.getFieldKey().toString());
        append("]");
        endCode();
        endP();
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String intitule = corpusField.getLibelleString(langInteger, false);
            if (intitule == null) {
                intitule = "";
            }
            printLangFieldRow(corpusField.getFieldKey().toString(), langInteger, intitule);
        }
        endTable();
    }

    private void appendCorpusIntituleTable(String name) {
        appendP(null, "corpus_corpusintitule_" + name);
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String intitule = getLibelleString(corpusMetadata.getIntituleByName(name), langInteger);
            printLangFieldRow("intitule_" + name, langInteger, intitule);
        }
        endTable();
    }

    private String getLibelleString(Intitule corpusIntitule, int langInteger) {
        if (corpusIntitule == null) {
            return "";
        }
        String s = corpusIntitule.getLibelleString(langInteger, false);
        if (s == null) {
            return "";
        }
        return s;
    }

}
