/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.album.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.basedefiches.format.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.html.*;


/**
 *
 * @author Vincent Calame
 */
public class OptionsFormPage extends CorpusHtmlPage {

    private List<CorpusField> geopointFieldList = new ArrayList<CorpusField>();

    public OptionsFormPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setCommandString(OptionsCommand.getCommandString());
        commandBox.setCommandKey(OptionsCommand.getCommandKey());
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        startCommandBox(commandBox);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        if (proprieteCount > 0) {
            appendH3("corpus_title_options_propriete");
            startUl();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                printCorpusFieldLi(corpusField);
            }
            endUl();
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        if (informationCount > 0) {
            appendH3("corpus_title_options_information");
            startUl();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                printCorpusFieldLi(corpusField);
            }
            endUl();
        }
        appendH3("corpus_title_options_titregeneration");
        CorpusField titreField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
        BasedefichesFormatDef ficheFormatDef = (BasedefichesFormatDef) titreField.getOption(FieldOptions.TITREGENERATIONFORMAT_OPTION);
        printTitreGenerationFormat(ficheFormatDef);
        if (geopointFieldList.size() > 0) {
            appendH3("corpus_title_options_geolocalisation");
            printGeolocalisationFieldSelect();
        }
        endCommandBox("corpus_submit_options");
        end();
    }

    private void printCorpusFieldLi(CorpusField corpusField) {
        startLi();
        printIntitule(corpusField);
        printDefaultSphereKeyList(corpusField);
        printDisplay(corpusField);
        printCurrencyArray(corpusField);
        printAddressFieldArray(corpusField);
        printBaseUrl(corpusField);
        printImageSource(corpusField);
        printAlbumKey(corpusField);
        printSubfieldDisplay(corpusField);
        endLi();
    }

    private void printTitreGenerationFormat(BasedefichesFormatDef ficheFormatDef) {
        String formatSource = null;
        String formatPattern = null;
        String formatInstruction = null;
        if (ficheFormatDef != null) {
            String[] array = ficheFormatDef.toStringArray();
            formatSource = array[0];
            formatPattern = array[1];
            formatInstruction = array[2];
        }
        startTable();
        startRow();
        printLibCells("corpus_lib_options_titregeneration_source", false);
        startCell();
        appendLargeInput(OptionsCommand.TITREGENERATION_SOURCE_PARAMNAME, formatSource);
        endCell();
        endRow();
        startRow();

        printLibCells("corpus_lib_options_titregeneration_pattern", false);
        startCell();
        appendLargeInput(OptionsCommand.TITREGENERATION_PATTERN_PARAMNAME, formatPattern);
        endCell();
        endRow();
        startRow();
        printLibCells("corpus_lib_options_titregeneration_instruction", false);
        startCell();
        appendLargeInput(OptionsCommand.TITREGENERATION_INSTRUCTION_PARAMNAME, formatInstruction);
        endCell();
        endRow();

        endTable();
    }

    private void printGeolocalisationFieldSelect() {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        CorpusField currentField = corpusMetadata.getGeolocalisationField();
        startP();
        printLibColon("corpus_lib_options_geolocalisation_field");
        startSelect(OptionsCommand.GEOLOCALISATIONFIELD_PARAMNAME);
        htmlAttributes.setValue("");
        htmlAttributes.setSelected((currentField == null));
        startOption(htmlAttributes);
        localize("global_lib_none");
        endOption();
        for (CorpusField otherCorpusField : geopointFieldList) {
            FieldKey fieldKey = otherCorpusField.getFieldKey();
            htmlAttributes.setValue(fieldKey.toString());
            boolean selected = ((currentField != null) && (currentField.equals(otherCorpusField)));
            htmlAttributes.setSelected(selected);
            startOption(htmlAttributes);
            printIntituleValue(otherCorpusField);
            endOption();
        }
        endSelect();
        endP();
    }

    private void printIntitule(CorpusField corpusField) {
        startP();
        printIntituleValue(corpusField);
        endP();
        startP(CssClass.P_SUBTITLE);
        printLibColon("corpus_lib_fieldcreation_ficheitemtype");
        append(" ");
        String ficheItemString = CorpusMetadataUtils.ficheItemTypeToString(corpusField.getFicheItemType());
        localize("corpus_ficheitem_" + ficheItemString);
        endP();
    }

    private void printIntituleValue(CorpusField corpusField) {
        append("[");
        append(corpusField.getFieldKey().toString());
        append("] - ");
        String lib = corpusField.getLibelleString(workingLangInteger, false);
        if (lib != null) {
            escape(lib);
        }
    }

    private void printImageSource(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.IMAGESOURCE_OPTION, corpusField)) {
            return;
        }
        String value = corpusField.getStringOption(FieldOptions.IMAGESOURCE_OPTION);
        if (value == null) {
            value = "";
        }
        String paramName = corpusField.getFieldKey().toString() + OptionsCommand.IMAGESOURCE_SUFFIX;
        startP();
        startSmall();
        printLibColon("corpus_lib_options_imagesource");
        appendLargeInput(paramName, value);
        endSmall();
        endP();
    }

    private void printAlbumKey(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.ALBUMKEY_OPTION, corpusField)) {
            return;
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int albumCount = basedefiches.getAlbumCount();
        if (albumCount == 0) {
            return;
        }
        SubsetKey currentAlbumKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
        startP();
        startSmall();
        printLibColon("corpus_lib_options_albumkey");
        startSelect(corpusField.getFieldKey().toString() + OptionsCommand.ALBUMKEY_SUFFIX);
        appendOption("", (currentAlbumKey == null), "global_lib_none");
        for (int i = 0; i < albumCount; i++) {
            Album album = basedefiches.getAlbum(i);
            SubsetKey albumKey = album.getSubsetKey();
            boolean selected = ((currentAlbumKey != null) && (currentAlbumKey.equals(albumKey)));
            startOption(albumKey.getSubsetName(), selected);
            append("[");
            append(albumKey.toString());
            append("]");
            printTiret();
            escape(AlbumUtils.getAlbumIntitule(album, workingLangInteger));
            endOption();
        }
        endSelect();
        endSmall();
        endP();
    }

    private void printBaseUrl(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.BASEURL_OPTION, corpusField)) {
            return;
        }
        String value = corpusField.getStringOption(FieldOptions.BASEURL_OPTION);
        if (value == null) {
            value = "";
        }
        String paramName = corpusField.getFieldKey().toString() + OptionsCommand.BASEURL_SUFFIX;
        startP();
        startSmall();
        printLibColon("corpus_lib_options_baseurl");
        appendLargeInput(paramName, value);
        endSmall();
        endP();
    }

    private void printDefaultSphereKeyList(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.DEFAULTSPHEREKEY_OPTION, corpusField)) {
            return;
        }
        SubsetKey currentDefault = (SubsetKey) corpusField.getOption(FieldOptions.DEFAULTSPHEREKEY_OPTION);
        startP();
        startSmall();
        printLibColon("corpus_lib_options_defaultsphere");
        startSelect(corpusField.getFieldKey().toString() + OptionsCommand.DEFAULTSPHERE_SUFFIX);
        appendOption("", (currentDefault == null), "corpus_lib_options_defaultsphere_user");
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int sphereCount = basedefiches.getSphereCount();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = basedefiches.getSphere(i);
            SubsetKey sphereKey = sphere.getSubsetKey();
            boolean selected = ((currentDefault != null) && (currentDefault.equals(sphereKey)));
            startOption(sphereKey.getSubsetName(), selected);
            append("[");
            append(sphereKey.toString());
            append("]");
            printTiret();
            escape(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
            endOption();
        }
        endSelect();
        endSmall();
        endP();
    }

    private void printSubfieldDisplay(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.SUBFIELDDISPLAY_OPTION, corpusField)) {
            return;
        }
        String paramName = corpusField.getFieldKey().toString() + OptionsCommand.SUBFIELDDISPLAY_SUFFIX;
        boolean checked = false;
        Object obj = corpusField.getOption(FieldOptions.SUBFIELDDISPLAY_OPTION);
        if ((obj != null) && (obj.equals(Boolean.TRUE))) {
            checked = true;
        }
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(paramName);
        htmlAttributes.setValue("1");
        htmlAttributes.setChecked(checked);
        startP();
        startSmall();
        addCheck(htmlAttributes, "corpus_lib_options_subfielddisplay", null);
        endSmall();
        endP();
    }

    private void printCurrencyArray(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.CURRENCYARRAY_OPTION, corpusField)) {
            return;
        }
        String paramName = corpusField.getFieldKey().toString() + OptionsCommand.CURRENCYARRAY_SUFFIX;
        Object obj = corpusField.getOption(FieldOptions.CURRENCYARRAY_OPTION);
        String value = "";
        if ((obj != null) && (obj instanceof int[])) {
            int[] currencyArray = (int[]) obj;
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < currencyArray.length; i++) {
                int currencyInteger = currencyArray[i];
                buf.append(CurrencyInteger.toString(currencyInteger));
                buf.append("; ");
            }
            value = buf.toString();
        }
        startP();
        startSmall();
        printLibColon("corpus_lib_options_currencyarray");
        appendMediumInput(paramName, value);
        endSmall();
        endP();
    }

    private void printAddressFieldArray(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.ADDRESSFIELDARRAY_OPTION, corpusField)) {
            return;
        }
        geopointFieldList.add(corpusField);
        String paramName = corpusField.getFieldKey().toString() + OptionsCommand.ADDRESSFIELDARRAY_SUFFIX;
        Object obj = corpusField.getOption(FieldOptions.ADDRESSFIELDARRAY_OPTION);
        String value = "";
        if ((obj != null) && (obj instanceof CorpusField[])) {
            CorpusField[] corpusFieldArray = (CorpusField[]) obj;
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < corpusFieldArray.length; i++) {
                CorpusField otherCorpusField = corpusFieldArray[i];
                if (i > 0) {
                    buf.append(", ");
                }
                buf.append(otherCorpusField.getFieldKey().toString());
            }
            value = buf.toString();
        }
        startP();
        startSmall();
        printLibColon("corpus_lib_options_addressfieldarray");
        appendLargeInput(paramName, value);
        endSmall();
        endP();
    }

    private void printDisplay(CorpusField corpusField) {
        if (!FieldOptions.testFieldOption(FieldOptions.INFORMATIONDISPLAY_OPTION, corpusField)) {
            return;
        }
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(corpusField.getFieldKey().toString() + OptionsCommand.BLOCKDISPLAY_SUFFIX);
        htmlAttributes.setValue("1");
        htmlAttributes.setChecked(CorpusMetadataUtils.isBlockDisplayInformationField(corpusField));
        startP();
        startSmall();
        addCheck(htmlAttributes, "corpus_lib_options_display", null);
        endSmall();
        endP();
    }

}
