/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.corpus.htmlpages;

import fr.exemole.bdfserver.response.CorpusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.corpus.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.util.html.*;


/**
 *
 * @author Vincent Calame
 */
public class UIFormPage extends CorpusHtmlPage {

    CorpusUI corpusUI;

    public UIFormPage(CorpusResponseParameters responseParameters) {
        super(responseParameters);
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre();
        beginFormBlock(corpus, ComponentUIPositionCommand.getCommandString(), "corpus_title_componentposition");
        startTextArea(ComponentUIPositionCommand.COMPONENTPOSITION_PARAMNAME, 60, 20);
        String[] array = corpusUI.getPositionArray();
        for (int i = 0; i < array.length; i++) {
            append(array[i]);
            append("\n");
        }
        endTextArea();
        endForm("corpus_submit_componentposition");
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);
        beginFormBlock(corpus, ComponentUIInputCommand.getCommandString(), "corpus_title_componentinput");
        startUl();
        for (Iterator it = list.iterator(); it.hasNext();) {
            Object obj = it.next();
            if (obj instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) obj;
                if (!fieldUI.getCorpusField().getFieldKey().equals(FieldKey.LANG)) {
                    printLi(fieldUI.getCorpusField().getFieldKey().toString(), fieldUI.getInputOption(), fieldUI.getValueOption(), false);
                }
            } else if (obj instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) obj;
                FilteredIncludeKey filteredIncludeKey = includeUI.getFiteredIncludeKey();
                String key = filteredIncludeKey.toString();
                IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
                if (includeKey.isIndexationIncludeKey()) {
                    printLi(key, includeUI.getInputOption(), includeUI.getValueOption(), true);
                } else if (includeKey.equals(IncludeKey.LIAGE)) {
                    printLi(key, includeUI.getLimitOption());
                }
            }
        }
        endUl();
        endForm("corpus_submit_componentinput");
        end();
    }

    private void printLi(String key, InputOption inputOption, ValueOption valueOption, boolean allInputAllowed) {
        startLi();
        startP();
        startCode();
        if (allInputAllowed) {
            append("[");
        }
        append(key);
        if (allInputAllowed) {
            append("]");
        }
        endCode();
        endP();
        startSmall();
        startUl();
        startLi();
        startP();
        short currentWidthType = inputOption.getWidthType();
        printLibColon("corpus_lib_size");
        startSelect(key + ComponentUIInputCommand.WIDTH_PARAMSUFFIX);
        printWidthOption(InputOption.WIDTH_SMALL, currentWidthType);
        printWidthOption(InputOption.WIDTH_MEDIUM, currentWidthType);
        printWidthOption(InputOption.WIDTH_LARGE, currentWidthType);
        endSelect();
        endP();
        endLi();
        startLi();
        startP();
        printLibColon("corpus_lib_rows");
        appendSmallestInput(key + ComponentUIInputCommand.ROWS_PARAMSUFFIX, String.valueOf(inputOption.getRows()));
        endP();
        endLi();
        if (allInputAllowed) {
            short currentValue = inputOption.getInputType();
            startLi();
            startP();
            printLibColon("corpus_lib_inputtype");
            startSelect(key + ComponentUIInputCommand.INPUT_PARAMSUFFIX);
            printInputOption(InputOption.INPUT_TEXT, currentValue);
            printInputOption(InputOption.INPUT_SELECT, currentValue);
            printInputOption(InputOption.INPUT_CHECK, currentValue);
            printInputOption(InputOption.INPUT_RADIO, currentValue);
            endSelect();
            endP();
            endLi();
        }
        if (valueOption != null) {
            startLi();
            startP();
            HTMLAttributes checkAttributes = new HTMLAttributes();
            checkAttributes.setName(key + ComponentUIInputCommand.MANDATORY_PARAMSUFFIX);
            checkAttributes.setValue("1");
            checkAttributes.setChecked(valueOption.isMandatory());
            addCheck(checkAttributes, "corpus_lib_mandatoryvalue", null);
            endP();
            endLi();
            startLi();
            startP();
            printLibColon("corpus_lib_defaulvalue");
            appendLargeInput(key + ComponentUIInputCommand.DEFAULT_PARAMSUFFIX, valueOption.getDefaultValue());
            endP();
            endLi();
        }
        endUl();
        endSmall();
        endLi();

    }

    private void printLi(String key, LimitOption limitOption) {
        startLi();
        startP();
        startCode();
        append("[");
        append(key);
        append("]");
        endCode();
        endP();
        startSmall();
        startUl();
        startLi();
        startP();
        HTMLAttributes nonewAttributes = new HTMLAttributes();
        nonewAttributes.setChecked(limitOption.isNoNewAllowed());
        nonewAttributes.setValue("1");
        nonewAttributes.setName(key + ComponentUIInputCommand.NONEW_PARAMSUFFIX);
        addCheck(nonewAttributes, "corpus_lib_nonew", null);
        endP();
        endLi();
        endUl();
        endSmall();
        endLi();
    }

    private void printInputOption(short value, short currentValue) {
        String string = DefaultInputOption.inputTypeToString(value);
        boolean selected = (value == currentValue);
        appendOption(string, selected, "corpus_inputtype_" + string);
    }

    private void printWidthOption(short value, short currentValue) {
        String string = DefaultInputOption.widthTypeToString(value);
        boolean selected = (value == currentValue);
        appendOption(string, selected, "corpus_size_" + string);
    }

}
