/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.envoi;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.sphere.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractEnvoiCommand extends AbstractBdfCommand {

    protected CourrielBean fromCourrielBean;
    protected short envoiType;
    protected AbregeFiche uniqueAbregeFiche;

    public AbstractEnvoiCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        Redacteur redacteur = BdfUserUtils.getRedacteur(bdfUser);
        if (redacteur == null) {
            throw new ParameterException("no redacteur matches");
        }
        fromCourrielBean = redacteur.getCourrielBean();
        if (fromCourrielBean == null) {
            throw new ParameterException("you don't have any email defined yet");
        }
        String param_envoitype = requestMap.getParameter("type");
        if (param_envoitype == null) {
            envoiType = EnvoiResponseParameters.ENVOI_FICHEUNIQUE;
        } else {
            try {
                envoiType = EnvoiResponseParameters.typeToShort(param_envoitype);
            } catch (IllegalArgumentException iae) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "type" + " = " + param_envoitype);
            }
        }
        if (envoiType == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            uniqueAbregeFiche = RequestUtils.getAbregeFiche(requestMap, bdfServer.getBasedefiches());
        }
        return doEnvoiCommand();
    }

    protected abstract EnvoiResponseParameters doEnvoiCommand() throws ParameterException;

    public EnvoiResponseParameters initResponseParameters() {
        EnvoiResponseParameters responseParameters = new EnvoiResponseParameters(bdfServer, bdfUser);
        responseParameters.setEnvoiType(envoiType);
        if (uniqueAbregeFiche != null) {
            responseParameters.putParameter(EnvoiResponseParameters.UNIQUEABREGEFICHE_PARAMNAME, uniqueAbregeFiche);
        }
        return responseParameters;
    }

    public boolean needSynchronisation() {
        return false;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new EnvoiResponsePrinterFactory();
    }

}
