/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.envoi;

import net.mapeadores.util.email.EmailException;
import net.mapeadores.util.email.MultiPartEmail;
import fr.exemole.bdfserver.response.*;
import net.mapeadores.util.email.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.corpus.*;
import javax.activation.DataSource;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.*;
import java.io.*;
import net.mapeadores.basedefiches.xml.extraction.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.transformation.*;


/**
 *
 * @author Vincent Calame
 */
public class EmailSender {

    BdfServer bdfServer;
    BdfUser bdfUser;
    EnvoiResponseParameters envoiResponseParameters;
    MultiPartEmail email;
    CourrielBean fromCourrielBean;

    public EmailSender(CourrielBean fromCourrielBean, EnvoiResponseParameters envoiResponseParameters) {
        this.envoiResponseParameters = envoiResponseParameters;
        this.fromCourrielBean = fromCourrielBean;
        this.bdfServer = envoiResponseParameters.getBdfServer();
        this.bdfUser = envoiResponseParameters.getBdfUser();
    }

    public void sendMail() throws EmailException {
        email = new MultiPartEmail();
        /*email.setHostName("ns354147.ovh.net");
        email.setPopBeforeSmtp(true,"ns354147.ovh.net","expediteur.fiches@basedefiches.net","expedit");
         */
        email.setHostName("smtp.fr.oleane.com");
        email.setAuthentication("smtp@fph.fr", "75011paris");
        List toList = envoiResponseParameters.getToList();
        for (Iterator it = toList.iterator(); it.hasNext();) {
            Object obj = it.next();
            CourrielBean courrielBean = null;
            if (obj instanceof CourrielBean) {
                courrielBean = (CourrielBean) obj;
            } else if (obj instanceof Redacteur) {
                courrielBean = ((Redacteur) obj).getCourrielBean();
            }
            if (courrielBean != null) {
                addTo(courrielBean);
            }
        }
        setFrom();
        setSubject();
        setMsg();
        switch (envoiResponseParameters.getEnvoiType()) {
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                CompilationSource compilationSource = new CompilationSource();
                email = email.attach(compilationSource, compilationSource.getName(), "");
                break;
            case EnvoiResponseParameters.ENVOI_SELECTION:
                Set abregeSet = (Set) envoiResponseParameters.getParameter(EnvoiResponseParameters.ABREGEFICHESET_PARAMNAME);
                for (Iterator it = abregeSet.iterator(); it.hasNext();) {
                    AbregeFiche abrege = (AbregeFiche) it.next();
                    FicheSource fichesource = new FicheSource(abrege);
                    email = email.attach(fichesource, fichesource.getName(), "");
                }
                break;
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                AbregeFiche abregeFiche = (AbregeFiche) envoiResponseParameters.getParameter(EnvoiResponseParameters.UNIQUEABREGEFICHE_PARAMNAME);
                FicheSource fichesource = new FicheSource(abregeFiche);
                email = email.attach(fichesource, fichesource.getName(), "");
                break;

        }
        email.send();
    }

    private void setFrom() throws EmailException {
        String nomcomplet = fromCourrielBean.getNomComplet();
        checkCharset(nomcomplet);
        if (envoiResponseParameters.isBcc()) {
            email.addBcc(fromCourrielBean.getAddrSpec(), nomcomplet);
        }
        email.setFrom(fromCourrielBean.getAddrSpec(), nomcomplet);
    }

    private void addTo(CourrielBean courrielBean) throws EmailException {
        String nomcomplet = courrielBean.getNomComplet();
        if (nomcomplet == null) {
            email.addTo(courrielBean.getAddrSpec());
        } else {
            checkCharset(nomcomplet);
            email.addTo(courrielBean.getAddrSpec(), nomcomplet);
        }
    }

    private void setMsg() throws EmailException {
        String msg = envoiResponseParameters.getMessage();
        if ((msg != null) && (msg.length() > 0)) {
            checkCharset(msg);
            email.setMsg(msg);
        }
    }

    private void setSubject() throws EmailException {
        String subject = envoiResponseParameters.getSubject();
        checkCharset(subject);
        boolean utf8 = testUTF8(subject);
        if (utf8) {
            email.setCharsetSubject("UTF-8");
        } else {
            email.setCharsetSubject("ISO-8859-1");
        }
        email.setSubject(subject);
    }

    void checkCharset(String s) {
        boolean utf8 = testUTF8(s);
        if (utf8) {
            email.setCharset("UTF-8");
        } else {
            email.setCharset("ISO-8859-1");
        }
    }

    static private boolean testUTF8(String s) {
        if (s == null) {
            return false;
        }
        for (int i = 0; i < s.length(); i++) {
            if (s.charAt(i) > '\u00FF') {
                return true;
            }
        }
        return false;
    }


    abstract class ExtractionSource implements DataSource {

        String contenttype;
        Transformer transformer;
        Extraction extraction;
        Object extractionSource;

        ExtractionSource() {
            extraction = Extraction.newDefaultFicheInstance(bdfServer.getBasedefiches(), LocalisationLists.newInstance(bdfUser.getLocalisationInfo()), null);
        }

        public String getContentType() {
            return contenttype;
        }

        void initTransformer(TransformationKey transformationKey) {
            XslTransformerKey templateKey = bdfUser.getXslTransformerKey(transformationKey);
            transformer = bdfServer.getTransformationManager().getXslTransformer(templateKey);
            TransformerParameters transformerParameters = new TransformerParameters();
            transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
            transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, envoiResponseParameters.getBaseDeFichesUrl());
            transformerParameters.initParameters(transformer);
            String method = transformer.getOutputProperty("method");
            if (method == null) {
                method = "xml";
            }
            String encoding = transformer.getOutputProperty("encoding");
            if (encoding == null) {
                encoding = "UTF-8";
            }
            if (method.equals("xml")) {
                contenttype = "text/xml;charset=" + encoding;
            } else if (method.equals("html")) {
                contenttype = "text/html;charset=" + encoding;
            } else {
                contenttype = "text/plain;charset=" + encoding;
            }
        }

        public InputStream getInputStream() throws IOException {
            try {
                ByteArrayOutputStream byteoutput = new ByteArrayOutputStream();
                transformer.transform(ExtractionXMLWriter.toStreamSource(extraction, extractionSource), new StreamResult(byteoutput));
                return new ByteArrayInputStream(byteoutput.toByteArray());
            } catch (TransformerException te) {
                throw new NestedTransformerException(te);
            }
        }

        public OutputStream getOutputStream() throws IOException {
            throw new IOException("no editable source");
        }

    }


    class FicheSource extends ExtractionSource {

        AbregeFiche abrege;

        FicheSource(AbregeFiche abrege) {
            this.abrege = abrege;
            extractionSource = abrege;
            initTransformer(new TransformationKey(abrege.getCorpus().getSubsetKey()));
        }

        public String getName() {
            return abrege.getCorpus().getSubsetKey().getSubsetName() + "-" + String.valueOf(abrege.getIdcorpus()) + ".html";
        }

    }


    class CompilationSource extends ExtractionSource {

        CompilationSource() {
            initTransformer(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
            extractionSource = bdfUser.getAbregeFicheSortedList();
        }

        public String getName() {
            return "compilation.html";
        }

    }

}
