/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.envoi;

import fr.exemole.bdfserver.response.EnvoiResponseParameters;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.corpus.*;
import java.util.*;
import java.text.ParseException;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.sphere.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.email.EmailException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SendCommand extends AbstractEnvoiCommand {

    Sphere sphere;
    List listeTo;
    List errorList;

    public SendCommand() {
    }

    protected EnvoiResponseParameters doEnvoiCommand() throws ParameterException {
        sphere = BdfUserUtils.getSphere(bdfUser);
        listeTo = new ArrayList();
        errorList = new ArrayList();
        EnvoiResponseParameters envoiResponseParameters = initResponseParameters();
        fillResponseParameters(envoiResponseParameters);
        boolean test = testResponseParameters(envoiResponseParameters);
        if (test) {
            try {
                EmailSender emailSender = new EmailSender(fromCourrielBean, envoiResponseParameters);
                emailSender.sendMail();
                envoiResponseParameters.setHtmlPageCode(EnvoiResponseParameters.CONFIRMATION_PAGE);
            } catch (EmailException ee) {
                envoiResponseParameters.setHtmlPageCode(EnvoiResponseParameters.ECHEC_PAGE);
                envoiResponseParameters.putParameter(EnvoiResponseParameters.MAILEXCEPTION_PARAMNAME, ee);
            }
        } else {
            envoiResponseParameters.setHtmlPageCode(EnvoiResponseParameters.ENVOIFORM_PAGE);
        }
        return envoiResponseParameters;
    }

    private void fillResponseParameters(EnvoiResponseParameters envoiResponseParameters) throws ParameterException {
        String param_bcc = requestMap.getParameter("bcc");
        if ((param_bcc != null) && (param_bcc.equals("1"))) {
            envoiResponseParameters.setBcc(true);
        } else {
            envoiResponseParameters.setBcc(false);
        }
        String param_msg = requestMap.getParameter("msg");
        envoiResponseParameters.setMessage(param_msg);
        String param_subject = requestMap.getParameter("subject");
        if (param_subject != null) {
            envoiResponseParameters.setSubject(StringUtils.cleanString(param_subject));
        }
        String param_to = requestMap.getParameter("to");
        if (param_to != null) {
            parseTo(param_to);
        }
        if (envoiType == EnvoiResponseParameters.ENVOI_SELECTION) {
            Set setAbreges = new LinkedHashSet();
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                String[] param_selection = requestMap.getParameterValues("selection_" + corpus.getSubsetKey().getSubsetName());
                if ((param_selection != null) && (param_selection.length > 0)) {
                    for (int j = 0; j < param_selection.length; j++) {
                        try {
                            int idcorpus = Integer.parseInt(param_selection[j]);
                            AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                            if (abregeFiche != null) {
                                setAbreges.add(abregeFiche);
                            }
                        } catch (NumberFormatException nfe) {
                        }
                    }
                }
            }
            envoiResponseParameters.putParameter(EnvoiResponseParameters.ABREGEFICHESET_PARAMNAME, setAbreges);
        }
        String param_err = requestMap.getParameter("err");
        if (param_err != null) {
            try {
                int errsize = Integer.parseInt(param_err);
                if (errsize < 1) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "err" + " = " + param_err);
                }
                for (int i = 1; i <= errsize; i++) {
                    String param_errtok = requestMap.getParameter("err_" + String.valueOf(i));
                    if (param_errtok != null) {
                        param_errtok = param_errtok.trim();
                        if (param_errtok.length() > 0) {
                            parseToken(param_errtok);
                        }
                    }
                }
            } catch (NumberFormatException nfe) {
                throw new ParameterException(ParameterException.PARAMETER_NOTINTEGER, "err");
            }
        }
        envoiResponseParameters.setToList(listeTo);
    }

    private boolean testResponseParameters(EnvoiResponseParameters envoiResponseParameters) {
        if (!envoiResponseParameters.hasSubject()) {
            envoiResponseParameters.setErrorMessage(EnvoiResponseParameters.NOSUBJECT_ERRORKEY, null);
            return false;
        }
        if (errorList.size() > 0) {
            envoiResponseParameters.setErrorMessage(EnvoiResponseParameters.WRONGADRESSES_ERRORKEY, null);
            envoiResponseParameters.putParameter(EnvoiResponseParameters.WRONGADRESSES_PARAMNAME, errorList);
            return false;
        }
        if (!envoiResponseParameters.hasToList()) {
            envoiResponseParameters.setErrorMessage(EnvoiResponseParameters.NOADDRESS_ERRORKEY, null);
            return false;
        }
        return true;
    }

    private void parseTo(String s) {
        String[] split = s.split("[,;\n\r]");
        for (int i = 0; i < split.length; i++) {
            String token = split[i].trim();
            if (token.length() > 0) {
                parseToken(token);
            }
        }

    }

    private void parseToken(String token) {
        RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(token, sphere.getSubsetKey());
        if (redacteurKey != null) {
            Redacteur redacteur = sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString());
            if (redacteur == null) {
                errorList.add(new RedacteurError(redacteurKey, "unknown"));
            } else if (!redacteur.isActif()) {
                errorList.add(new RedacteurError(redacteurKey, "inactif"));
            } else if (redacteur.getCourrielBean() == null) {
                errorList.add(new RedacteurError(redacteurKey, "inconnu"));
            } else {
                listeTo.add(redacteur);

            }
        } else {
            try {
                CourrielBean courrielBean = CourrielBean.parse(token);
                listeTo.add(courrielBean);
            } catch (ParseException pe) {
                errorList.add(token);
            }
        }
    }

    public static String getCommandString() {
        return "Send";
    }

}


