/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.envoi.htmlpages;

import fr.exemole.bdfserver.response.EnvoiResponseParameters;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.basedefiches.corpus.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.sphere.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class ConfirmationPage extends EnvoiHtmlPage {

    public ConfirmationPage(EnvoiResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendP(null, getDoneKey());
        appendP(null, "envoi_msg_rappel");
        startUl();
        if (typeEnvoi == EnvoiResponseParameters.ENVOI_COMPILATION) {
            startLi();
            appendP(null, "envoi_msg_rpcompil");
            startUl();
            AbregeFicheSortedList triAbregesFiches = bdfUser.getAbregeFicheSortedList();
            for (int i = 0; i < triAbregesFiches.getCorpusCount(); i++) {
                Corpus corpus = triAbregesFiches.getCorpus(i);
                SubsetItemList abregeFicheList = triAbregesFiches.getCorpusAbregeFicheList(i);
                startLi();
                startH3();
                escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
                endH3();
                startUl();
                int ficheCount = abregeFicheList.getSubsetItemCount();
                for (int j = 0; j < ficheCount; j++) {
                    AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                    startLi();
                    escape(BdfUserUtils.getAbregeFicheString(bdfUser, abrege));
                    endLi();
                }
                endUl();
            }
            endUl();
            endLi();
        } else if (typeEnvoi == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            startLi();
            appendP(null, "envoi_msg_rpfiches");
            startUl();
            startLi();
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, getUniqueAbregeFiche()));
            endLi();
        } else {
            Set setAbreges = getAbregeFicheSet();
            if ((setAbreges == null) || (setAbreges.size() == 0)) {
                startLi();
                appendP(null, "envoi_msg_nofiche");
                endLi();
            } else {
                startLi();
                appendP(null, "envoi_msg_rpfiches");
                startUl();
                for (Iterator it = setAbreges.iterator(); it.hasNext();) {
                    AbregeFiche abrege = (AbregeFiche) it.next();
                    startLi();
                    escape(BdfUserUtils.getAbregeFicheString(bdfUser, abrege));
                    endLi();
                }
                endUl();
                endLi();
            }
        }

        startLi();
        appendP(null, "envoi_msg_rpto");
        startUl();
        List listeTo = responseParameters.getToList();
        for (Iterator it = listeTo.iterator(); it.hasNext();) {
            Object obj = it.next();
            startLi();
            if (obj instanceof CourrielBean) {
                escape(obj.toString());
            } else if (obj instanceof Redacteur) {
                escape(((Redacteur) obj).getCourrielBean().toString());
                append(" (");
                escape(((Redacteur) obj).getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE));
                append(")");
                endLi();
            }
        }
        endUl();
        endLi();
        endUl();
        end();
    }

    private String getDoneKey() {
        switch (typeEnvoi) {
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                return "envoi_msg_donefiche";
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                return "envoi_msg_donecompilation";
            case EnvoiResponseParameters.ENVOI_SELECTION:
                return "envoi_msg_doneselection";
            default:
                return null;
        }
    }

}
