/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.envoi.htmlpages;

import fr.exemole.bdfserver.response.EnvoiResponseParameters;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.servlets.envoi.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.basedefiches.corpus.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.util.html.HTMLUtils;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class EnvoiFormPage extends EnvoiHtmlPage {

    public EnvoiFormPage(EnvoiResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1(getTitleKey());
        startPostForm(SendCommand.getCommandString());
        addParameter("type", EnvoiResponseParameters.typeToString(typeEnvoi));
        if (typeEnvoi == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            append(RequestUtils.toHiddenParameter(getUniqueAbregeFiche()));
        }
        printMessage();
        startP();
        localize("envoi_msg_to");
        pw.print("<br><textarea name=\"to\" cols=\"60\" rows=\"4\">");
        if (responseParameters.hasToList()) {
            List listeTo = responseParameters.getToList();
            for (int i = 0; i < listeTo.size(); i++) {
                if (i > 0) {
                    pw.print("\n");
                }
                Object obj = listeTo.get(i);
                if (obj instanceof CourrielBean) {
                    escape(obj.toString());
                } else if (obj instanceof Redacteur) {
                    escape(((Redacteur) obj).getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE));
                }
            }
        }
        endTextArea();
        endP();
        startP();
        pw.print("<small><input type=\"checkbox\" name=\"bcc\" value=\"1\"");
        if (responseParameters.isBcc()) {
            pw.print(" checked");
        }
        pw.print(">");
        localize("envoi_msg_bcc");
        pw.print("</small></input>");
        endP();
        startP();
        printLib("envoi_msg_subject");
        appendLargeInput("subject", getSubject());
        endP();
        startP();
        localize("envoi_msg_msg");
        appendBr();
        pw.print("<textarea name=\"msg\" cols=\"72\" rows=\"5\">");
        if (responseParameters.hasMessage()) {
            HTMLUtils.printPreformattedHTML(pw, responseParameters.getMessage());
        }
        endTextArea();
        endP();
        if (typeEnvoi == EnvoiResponseParameters.ENVOI_COMPILATION) {
            XslTransformerKey templateKey = bdfUser.getXslTransformerKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
            String templateName = templateKey.getTransformerName();
            pw.print("<p><br>");
            if (templateName == null) {
                localize("envoi_msg_modeledefaut");
            } else {
                localize("envoi_msg_modelexslt", templateName);
            }
            endP();
        }
        appendH2(getMessageKey());
        startUl();
        if (typeEnvoi == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            startLi();
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, getUniqueAbregeFiche()));
            endLi();

        } else {
            AbregeFicheSortedList triAbregesFiches = bdfUser.getAbregeFicheSortedList();
            Set setAbreges = null;
            boolean selection = (typeEnvoi == EnvoiResponseParameters.ENVOI_SELECTION);
            if (selection) {
                setAbreges = getAbregeFicheSet();
            }
            for (int i = 0; i < triAbregesFiches.getCorpusCount(); i++) {
                Corpus corpus = triAbregesFiches.getCorpus(i);
                SubsetItemList abregeFicheList = triAbregesFiches.getCorpusAbregeFicheList(i);
                startLi();
                pw.print("<p><strong>");
                escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
                pw.print("</strong></p>");
                startUl();
                int ficheCount = abregeFicheList.getSubsetItemCount();
                for (int j = 0; j < ficheCount; j++) {
                    AbregeFiche abrg = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                    startLi();
                    if (selection) {
                        pw.print("<input type=\"checkbox\" name=\"selection_");
                        pw.print(corpus.getSubsetKey().getSubsetName());
                        pw.print("\" value=\"");
                        pw.print(String.valueOf(abrg.getIdcorpus()));
                        pw.print("\"");
                        if ((setAbreges != null) && (setAbreges.contains(abrg))) {
                            pw.print(" checked");
                        }
                        pw.print("> ");
                    }
                    append(String.valueOf(abrg.getIdcorpus()));
                    printTiret();
                    escape(abrg.getTitre());
                    endLi();
                }
                endUl();
                endLi();
            }
        }
        endUl();
        endForm(getSubmitKey());
        end();
    }

    private String getSubject() {
        if (responseParameters.hasSubject()) {
            return responseParameters.getSubject();
        }
        if (responseParameters.getEnvoiType() == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            return BdfUserUtils.getAbregeFicheString(bdfUser, getUniqueAbregeFiche());
        }
        return "";
    }

    private String getSubmitKey() {
        switch (typeEnvoi) {
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                return "envoi_bt_envoifiche";
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                return "envoi_bt_envoicompilation";
            case EnvoiResponseParameters.ENVOI_SELECTION:
                return "envoi_bt_envoiselection";
            default:
                return null;
        }
    }

    private String getMessageKey() {
        switch (typeEnvoi) {
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                return "envoi_msg_attach";
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                return "envoi_msg_compilation";
            case EnvoiResponseParameters.ENVOI_SELECTION:
                return "envoi_msg_attachs";
            default:
                return null;
        }
    }

    private String getTitleKey() {
        switch (typeEnvoi) {
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                return "envoi_titre_ficheunique";
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                return "envoi_titre_compilation";
            case EnvoiResponseParameters.ENVOI_SELECTION:
                return "envoi_titre_selection";
            default:
                return null;
        }
    }

    protected void printMessage() {
        if (message != null) {
            super.printMessage();
        }
        List errorList = (List) responseParameters.getParameter(EnvoiResponseParameters.WRONGADRESSES_PARAMNAME);
        if (errorList != null) {
            appendHiddenInput("err", String.valueOf(errorList.size()));
            startTable();
            for (int i = 0; i < errorList.size(); i++) {
                String codeerr = null;
                String txterr = null;
                Object obj = errorList.get(i);
                if (obj instanceof String) {
                    codeerr = "envoi_err_incorrect";
                    txterr = obj.toString();
                } else if (obj instanceof RedacteurError) {
                    RedacteurError error = (RedacteurError) obj;
                    codeerr = "envoi_err_" + error.getType();
                    txterr = error.getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE);
                }
                startRow();
                pw.print("<td style=\"color: blue;\">");
                localize(codeerr);
                endCell();
                startCell();
                appendLargeInput("err_" + String.valueOf(i + 1), txterr);
                endCell();
                endRow();
            }
            endTable();
        }
    }

}
