/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.BdfServer;
import javax.servlet.http.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.scrutariexport.CorpusExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExport;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusExportDef;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.parsers.*;
import java.net.*;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportChangeCommand extends AbstractExportationCommand {

    public final static String BASEICON_PARAMNAME = "baseicon";
    public final static String FICHEGLOBALPATTERN_PARAMNAME = "ficheglobalpattern";
    public final static String MOTCLEGLOBALPATTERN_PARAMNAME = "motcleglobalpattern";
    public final static String BASENAMEACTIVATION_PARAMNAME = "basenameactivation";
    public final static String BASENAMEVALUE_PARAMNAME = "basenamevalue";
    public final static String LANGSUI_PARAMNAME = "langsui";
    public final static String COMPLEMENTKEYLIST_PARAMSUFFIX = "|complementkeylist";
    public final static String DATEKEY_PARAMSUFFIX = "|datekey";
    public final static String SELECTED_PARAMSUFFIX = "|selected";
    public final static String LIBELLEPATTERN_PARAMSUFFIX = "|libellepattern";
    private ScrutariExport scrutariExport;
    private String baseIconString;
    private int[] array;
    private List<CorpusExportDef> corpusDefList;
    private List<ThesaurusExportDef> thesaurusDefList;
    private AccoladePattern ficheGlobalPattern;
    private AccoladePattern motcleGlobalPattern;
    private String activationParam;
    private String baseName;

    public ScrutariExportChangeCommand() {
    }

    protected void doExportationCommand() {
        synchronized (bdfServer) {
            updateScrutariExport();
            bdfServer.saveScrutariExportManager();
        }
        responseParameters.setScrutariExport(scrutariExport);
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE);
        responseParameters.setDoneMessage("exportation_done_scrutariexportchange", null);
    }

    protected void checkExportationParameters() throws ParameterException {
        scrutariExport = RequestUtils.getScrutariExport(requestMap, bdfServer.getScrutariExportManager());
        baseIconString = getMandatoryParameter(BASEICON_PARAMNAME);
        String langsUi = getMandatoryParameter(LANGSUI_PARAMNAME);
        array = LangIntegerUtils.toCleanLangIntegerArray(langsUi);
        ficheGlobalPattern = getPattern(FICHEGLOBALPATTERN_PARAMNAME);
        motcleGlobalPattern = getPattern(MOTCLEGLOBALPATTERN_PARAMNAME);
        String activationParam = getMandatoryParameter(BASENAMEACTIVATION_PARAMNAME);
        if (activationParam.equals("1")) {
            baseName = getMandatoryParameter(BASENAMEVALUE_PARAMNAME);
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        corpusDefList = new ArrayList<CorpusExportDef>();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusExportDef exportCorpusDef = getCorpusExportDef(corpus);
            if (exportCorpusDef != null) {
                corpusDefList.add(exportCorpusDef);
            }
        }
        thesaurusDefList = new ArrayList<ThesaurusExportDef>();
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            ThesaurusExportDef thesaurusExportDef = getThesaurusExportDef(thesaurus);
            if (thesaurusExportDef != null) {
                thesaurusDefList.add(thesaurusExportDef);
            }
        }
    }

    public static String getCommandString() {
        return "ScrutariExportChange";
    }

    void updateScrutariExport() {
        scrutariExport.setBaseIcon(baseIconString);
        scrutariExport.setFicheGlobalPattern(ficheGlobalPattern);
        scrutariExport.setMotcleGlobalPattern(motcleGlobalPattern);
        try {
            scrutariExport.setBaseName(baseName);
        } catch (java.text.ParseException pe) {
        }
        scrutariExport.setLangUiArray(array);

        for (CorpusExportDef corpusExportDef : corpusDefList) {
            scrutariExport.putCorpusExportDef(corpusExportDef);
        }
        for (ThesaurusExportDef thesaurusExportDef : thesaurusDefList) {
            scrutariExport.putThesaurusExportDef(thesaurusExportDef);
        }
        scrutariExport.setFicheSelectOptions(FicheSelectParser.parse(requestMap, bdfServer, bdfUser));
    }

    AccoladePattern getPattern(String paramName) throws ParameterException {
        String patternParam = requestMap.getParameter(paramName);
        if (patternParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, paramName);
        }
        patternParam = patternParam.trim();
        if (patternParam.length() == 0) {
            return null;
        }
        try {
            AccoladePattern pattern = new AccoladePattern(patternParam);
            return pattern;
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    CorpusExportDef getCorpusExportDef(Corpus corpus) throws ParameterException {
        SubsetKey corpusKey = corpus.getSubsetKey();
        String dateKeyParam = requestMap.getParameter(corpusKey.toString() + DATEKEY_PARAMSUFFIX);
        String fieldListParam = requestMap.getParameter(corpusKey.toString() + COMPLEMENTKEYLIST_PARAMSUFFIX);
        if ((fieldListParam == null) && (dateKeyParam == null)) {
            return null;
        }
        if (dateKeyParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, corpusKey.toString() + DATEKEY_PARAMSUFFIX);
        }
        if (fieldListParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, corpusKey.toString() + COMPLEMENTKEYLIST_PARAMSUFFIX);
        }
        CorpusExportDef corpusExportDef = new CorpusExportDef(corpusKey);
        String[] tokens = StringUtils.getTechnicalTokens(fieldListParam);
        for (int i = 0; i < tokens.length; i++) {
            Object key = IncludeUtils.parseKey(tokens[i]);
            if (key != null) {
                corpusExportDef.addComplementKey(key);
            }
        }
        Object obj = CorpusExportDef.parseDateKey(dateKeyParam);
        if (obj == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, corpusKey.toString() + DATEKEY_PARAMSUFFIX + " = " + dateKeyParam);
        }
        if (obj instanceof FieldKey) {
            List corpusFieldList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpus.getCorpusMetadata(), CorpusField.DATATION_FIELD);
            boolean present = false;
            for (Iterator it = corpusFieldList.iterator(); it.hasNext();) {
                CorpusField corpusField = (CorpusField) it.next();
                if (corpusField.getFieldKey().equals(obj)) {
                    present = true;
                    break;
                }
            }
            if (!present) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, corpusKey.toString() + DATEKEY_PARAMSUFFIX + " = " + dateKeyParam);
            }
        }
        corpusExportDef.setDateKey(obj);
        return corpusExportDef;
    }

    ThesaurusExportDef getThesaurusExportDef(Thesaurus thesaurus) {
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        ThesaurusExportDef thesaurusExportDef = new ThesaurusExportDef(thesaurusKey);
        String selectedParam = requestMap.getParameter(thesaurusKey.toString() + SELECTED_PARAMSUFFIX);
        boolean selected = false;
        if ((selectedParam != null) && (selectedParam.equals("1"))) {
            selected = true;
        }
        thesaurusExportDef.setSelected(selected);
        String patternParam = requestMap.getParameter(thesaurusKey.toString() + LIBELLEPATTERN_PARAMSUFFIX);
        if (patternParam != null) {
            patternParam = patternParam.trim();
            if (patternParam.length() == 0) {
                thesaurusExportDef.setLibellePattern(null);
            } else {
                try {
                    AccoladePattern libellePattern = new AccoladePattern(patternParam);
                    thesaurusExportDef.setLibellePattern(libellePattern);
                } catch (java.text.ParseException pe) {
                }
            }
        }
        return thesaurusExportDef;
    }

}
