/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.sqlexport.*;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import net.mapeadores.util.beans.*;
import net.mapeadores.util.text.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportChangeCommand extends AbstractExportationCommand {

    public final static String CUSTOMCLASSNAME_PARAMNAME = "customclassname";
    public final static String PATH_PARAMNAME = "path";
    public final static String PARAMS_PARAMNAME = "params";
    public final static String URL_PARAMNAME = "url";
    public final static String TABLEEXPORTNAME_PARAMNAME = "tableexportname";
    public final static String POSTCOMMAND_PARAMNAME = "postcommand";
    private SqlExportBean sqlExportBean;

    public SqlExportChangeCommand() {
    }

    protected void doExportationCommand() {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        synchronized (bdfServer) {
            sqlExportManager.putSqlExportBean(sqlExportBean);
            bdfServer.saveSqlExportManager();
        }
        responseParameters.setSqlExportBean(sqlExportBean);
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTFORM_PAGE);
        responseParameters.setDoneMessage("exportation_done_sqlexportchange", sqlExportBean.getName());
    }

    protected void checkExportationParameters() throws ParameterException {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        sqlExportBean = RequestUtils.getSqlExport(requestMap, sqlExportManager);
        String customClassName = getMandatoryParameter(CUSTOMCLASSNAME_PARAMNAME).trim();
        sqlExportBean.setSqlExportClassName(customClassName);
        String path = getMandatoryParameter(PATH_PARAMNAME).trim();
        sqlExportBean.setPath(path);
        String url = getMandatoryParameter(URL_PARAMNAME).trim();
        sqlExportBean.setUrl(url);
        String postCommand = getMandatoryParameter(POSTCOMMAND_PARAMNAME).trim();
        sqlExportBean.setPostCommand(postCommand);
        String tableExportName = getMandatoryParameter(TABLEEXPORTNAME_PARAMNAME).trim();
        if (tableExportName.length() == 0) {
            sqlExportBean.setTableExportName(null);
        } else {
            sqlExportBean.setTableExportName(tableExportName);
        }
        String params = getMandatoryParameter(PARAMS_PARAMNAME);
        String[] tokens = StringUtils.getLineTokens(params, StringUtils.EMPTY_EXCLUDE);
        List paramBeanList = new ArrayList();
        for (int i = 0; i < tokens.length; i++) {
            String token = tokens[i];
            int idx = token.indexOf('=');
            if (idx == -1) {
                continue;
            }
            String name = token.substring(0, idx).trim();
            if (name.length() > 0) {
                String value = token.substring(idx + 1).trim();
                paramBeanList.add(new ParamBean(name, value));
            }
        }
        int size = paramBeanList.size();
        if (size == 0) {
            sqlExportBean.setParamBeanArray(null);
        } else {
            ParamBean[] array = new ParamBean[size];
            for (int i = 0; i < size; i++) {
                array[i] = (ParamBean) paramBeanList.get(i);
            }
            sqlExportBean.setParamBeanArray(array);
        }
    }

    public static String getCommandString() {
        return "SqlExportChange";
    }

}
