/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.sqlexport.*;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportCreationCommand extends AbstractExportationCommand {

    public final static String NEWSQLEXPORT_PARAMNAME = "newsqlexport";
    private String newName;

    public SqlExportCreationCommand() {
    }

    protected void doExportationCommand() {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        SqlExportBean sqlExportBean = new SqlExportBean();
        sqlExportBean.setName(newName);
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTFORM_PAGE);
        responseParameters.setDoneMessage("exportation_done_sqlexportcreation", newName);
        responseParameters.setSqlExportBean(sqlExportBean);
        synchronized (bdfServer) {
            sqlExportManager.putSqlExportBean(sqlExportBean);
            bdfServer.saveSqlExportManager();
        }
    }

    protected void checkExportationParameters() throws ParameterException {
        newName = getMandatoryParameter(NEWSQLEXPORT_PARAMNAME);
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        SqlExportBean sqlExportBean = sqlExportManager.getSqlExportBeanByName(newName);
        if (sqlExportBean != null) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTCREATION_PAGE);
            responseParameters.setErrorMessage("exportation_err_existingsqlexportname", newName);
            return;
        }
        if (!StringUtils.isTechnicalName(newName, true)) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTCREATION_PAGE);
            responseParameters.setErrorMessage("exportation_err_wrongexportname", newName);
            return;
        }
    }

    public static String getCommandString() {
        return "SqlExportCreation";
    }

}
