/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import net.mapeadores.basedefiches.tableexport.*;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import fr.exemole.bdfserver.servlets.util.*;


/**
 *
 * @author Vincent Calame
 */
public class TableStreamCommand extends AbstractExportationCommand {

    public final static String DEFAULTEXPORT_PARAMNAME = "defaultexport";
    private TableExportParametersBean tableExportParametersBean;

    public TableStreamCommand() {
    }

    protected void doExportationCommand() {
        TableExportManager tableExportManager = bdfServer.getTableExportManager();
        bdfUser.setTableExportParametersBean(tableExportParametersBean);
        BdfUserUtils.saveBdfUser(bdfUser);
        String name = tableExportParametersBean.getTableExportName();
        TableExport tableExport = null;
        SubsetTableExport[] corpusTableExportArray = null;
        if (name != null) {
            tableExport = tableExportManager.getTableExport(name);
        }
        if (tableExport == null) {
            tableExport = tableExportManager.getDefaultTableExport(tableExportParametersBean);
        }
        corpusTableExportArray = tableExport.getSubsetTableExportArray(bdfUser.getLocalisationInfo());
        responseParameters.setAbregeFicheSortedList(bdfUser.getAbregeFicheSortedList());
        responseParameters.setContentStreamCode(ExportationResponseParameters.CORPUSTABLE_ODS_CONTENTSTREAM);
        responseParameters.setCorpusTableExportArray(corpusTableExportArray);
    }

    protected void checkExportationParameters() throws ParameterException {
        tableExportParametersBean = getTableExportParametersBean();
    }

    private TableExportParametersBean getTableExportParametersBean() throws ParameterException {
        String defaultExportParam = requestMap.getParameter(DEFAULTEXPORT_PARAMNAME);
        boolean mandatoryName = ((defaultExportParam != null) && (defaultExportParam.equals("0")));
        TableExportParametersBean tableExportParametersBean = TableExportParameters.getTableExportParametersBean(requestMap, mandatoryName, true);
        if (mandatoryName) {
            String name = tableExportParametersBean.getTableExportName();
            boolean present = bdfServer.getTableExportManager().getTableExportNameSet().contains(name);
            if (!present) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TableExportParameters.TABLEEXPORTNAME_PARAMNAME + " = " + name);
            }
        }

        return tableExportParametersBean;
    }

    public static String getCommandString() {
        return "TableStream";
    }

}
