/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.htmlpages;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.servlets.exportation.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import fr.exemole.bdfserver.request.RequestUtils;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExport;
import net.mapeadores.basedefiches.sqlexport.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import fr.exemole.bdfserver.balayages.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class ExportationHtmlPage extends BdfHtmlPage {

    protected ScrutariExport scrutariExport;
    protected SqlExportBean sqlExportBean;
    protected short pageType;

    public ExportationHtmlPage(ExportationResponseParameters responseParameters) {
        super(responseParameters);
        this.scrutariExport = responseParameters.getScrutariExport();
        this.sqlExportBean = responseParameters.getSqlExportBean();
        this.pageType = responseParameters.getResponseCode();
    }

    public static ResponseHandler getInstance(ExportationResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case ExportationResponseParameters.STREAMFORM_PAGE:
                return new StreamFormPage(responseParameters);
            case ExportationResponseParameters.SCRUTARIEXPORTINDEX_PAGE:
                return new ScrutariExportIndexPage(responseParameters);
            case ExportationResponseParameters.SCRUTARIEXPORTCREATION_PAGE:
                return new ScrutariExportCreationPage(responseParameters);
            case ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE:
                return new ScrutariExportFormPage(responseParameters);
            case ExportationResponseParameters.SCRUTARIEXPORTRUN_PAGE:
                return new ScrutariExportRunPage(responseParameters);
            case ExportationResponseParameters.SCRUTARIEXPORTREMOVE_PAGE:
                return new ScrutariExportRemovePage(responseParameters);
            case ExportationResponseParameters.SQLEXPORTINDEX_PAGE:
                return new SqlExportIndexPage(responseParameters);
            case ExportationResponseParameters.SQLEXPORTCREATION_PAGE:
                return new SqlExportCreationPage(responseParameters);
            case ExportationResponseParameters.SQLEXPORTFORM_PAGE:
                return new SqlExportFormPage(responseParameters);
            case ExportationResponseParameters.SQLRUN_PAGE:
                return new SqlRunPage(responseParameters);
            case ExportationResponseParameters.SQLEXPORTREMOVE_PAGE:
                return new SqlExportRemovePage(responseParameters);
            case ExportationResponseParameters.BALAYAGEERROR_PAGE:
                return new BalayageErrorPage(responseParameters);
            case ExportationResponseParameters.BALAYAGE_PAGE:
                return new BalayagePage(responseParameters);
            case ExportationResponseParameters.THESAURUSFORM_PAGE:
                return new ThesaurusFormPage(responseParameters);
        }
        return null;
    }

    public String getTitle() {
        return "Exportation";
    }

    public String getCssThemeFile() {
        return "exportation.css";
    }

    public String getFormAction() {
        return "Exportation";
    }

    protected void printScrutariTitre() {
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Exportation?page=" + DefaultCommand.SCRUTARIEXPORTINDEX_PAGE_PARAMVALUE, "exportation_title_scrutariexportindex");
        printTiret();
        printScrutariLinks(scrutariExport);
        endP();
        startH1();
        printLibColon("exportation_lib_scrutariexport");
        append(scrutariExport.getName());
        endH1();
        printMessage();
    }

    protected void printScrutariLinks(ScrutariExport scrutariExport) {
        String href = "Exportation?" + RequestUtils.toParameterString(scrutariExport);
        if (pageType == ExportationResponseParameters.SCRUTARIEXPORTRUN_PAGE) {
            printNoLink("global_link_exportrun");
        } else {
            appendA(href + "&run=" + DefaultCommand.SCRUTARI_RUN_PARAMVALUE, "global_link_exportrun");
        }
        printTiret();
        if (pageType == ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE) {
            printNoLink("global_link_change");
        } else {
            appendA(href, "global_link_change");
        }
        printTiret();
        if (pageType == ExportationResponseParameters.SCRUTARIEXPORTREMOVE_PAGE) {
            printNoLink("global_link_remove");
        } else {
            appendA(href + "&amp;page=" + DefaultCommand.SCRUTARIEXPORTREMOVE_PAGE_PARAMVALUE, "global_link_remove");
        }
    }

    protected void printSqlTitre() {
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Exportation?page=" + DefaultCommand.SQLEXPORTINDEX_PAGE_PARAMVALUE, "exportation_title_sqlexportindex");
        printTiret();
        printSqlLinks(sqlExportBean.getName());
        endP();
        startH1();
        printLibColon("exportation_lib_sqlexport");
        append(sqlExportBean.getName());
        endH1();
        printMessage();
    }

    protected void printSqlLinks(String sqlExportName) {
        String href = "Exportation?" + RequestUtils.SQLEXPORT_PARAMNAME + "=" + sqlExportName;
        appendA(href + "&run=" + DefaultCommand.SQLEXPORT_RUN_PARAMVALUE, "global_link_exportrun");
        printTiret();
        if (pageType == ExportationResponseParameters.SQLEXPORTFORM_PAGE) {
            printNoLink("global_link_change");
        } else {
            appendA(href, "global_link_change");
        }
        printTiret();
        if (pageType == ExportationResponseParameters.SQLEXPORTREMOVE_PAGE) {
            printNoLink("global_link_remove");
        } else {
            appendA(href + "&amp;page=" + DefaultCommand.SQLEXPORTREMOVE_PAGE_PARAMVALUE, "global_link_remove");
        }
    }

    protected void printNoLink(String key) {
        startSpan(CssClass.SPAN_CURRENTPAGE);
        localize(key);
        endSpan();
    }

    public void printBalayageLinks(Balayage balayage) {
        printBalayageLinks(this, balayage);
    }

    public static void printBalayageLinks(BdfHtmlPage bdfHtmlPage, Balayage balayage) {
        String hrefBal = "Exportation?run=" + DefaultCommand.BALAYAGE_RUN_PARAMVALUE + "&amp;" + RequestUtils.BALAYAGE_PARAMNAME + "=" + balayage.getName();
        bdfHtmlPage.startA(hrefBal);
        bdfHtmlPage.append(balayage.getName());
        bdfHtmlPage.endA();
        int count = balayage.getBalayageModeCount();
        if (count > 0) {
            bdfHtmlPage.space();
            bdfHtmlPage.startSmall();
            bdfHtmlPage.append("[");
            for (int i = 0; i < count; i++) {
                BalayageMode balayageMode = balayage.getBalayageMode(i);
                String modeName = balayageMode.getName();
                if (i != 0) {
                    bdfHtmlPage.append(" | ");
                }
                bdfHtmlPage.startA(hrefBal + "&amp;mode=" + modeName);
                bdfHtmlPage.append(modeName);
                bdfHtmlPage.endA();
            }
            bdfHtmlPage.append("]");
            bdfHtmlPage.endSmall();
        }
    }

}
