/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.htmlpages;

import fr.exemole.bdfserver.html.catalogs.SelectFormCatalog;
import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.servlets.util.FicheSelectWriter;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.exportation.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.scrutariexport.CorpusExportDef;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusExportDef;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportFormPage extends ExportationHtmlPage {

    public ScrutariExportFormPage(ExportationResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(SelectFormCatalog.SELECTFORM_JAVASCRIPT_LIB);
        addJavascript(SelectFormCatalog.SELECTFORM_FICHE);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        initResponse(response);
        start();
        printScrutariTitre();

        startForm(scrutariExport, ScrutariExportChangeCommand.getCommandString());
        printFixedSubmit("exportation_submit_scrutariexportchange");
        appendH2("exportation_title_generaloptions");
        printBaseName();
        printBaseIcon();
        printPattern(ScrutariExportChangeCommand.FICHEGLOBALPATTERN_PARAMNAME, scrutariExport.getFicheGlobalPattern());
        printPattern(ScrutariExportChangeCommand.MOTCLEGLOBALPATTERN_PARAMNAME, scrutariExport.getMotcleGlobalPattern());
        String langsUi = getArray(scrutariExport.getLangUiArray());
        startP();
        printLib("exportation_lib_langsui");
        appendSmallInput(ScrutariExportChangeCommand.LANGSUI_PARAMNAME, langsUi);
        endP();
        appendH2("exportation_title_corpusoptions");
        startTable("100%");
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            printCorpus(basedefiches.getCorpus(i));
        }
        endTable();
        appendH2("exportation_title_thesaurusoptions");
        startTable("100%");
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            printThesaurus(basedefiches.getThesaurus(i));
        }
        endTable();
        appendH2("exportation_title_selectionoptions");

        FicheSelectWriter selectionFormWriter = new FicheSelectWriter(bdfServer, this, workingLangInteger);
        selectionFormWriter.setFicheSelectOptions(scrutariExport.getFicheSelectOptions());
        selectionFormWriter.printFieldSets();

        endForm("exportation_submit_scrutariexportchange");
        end();
    }

    void printBaseName() {
        String baseName = scrutariExport.getBaseName();
        startP();
        appendRadioInput(ScrutariExportChangeCommand.BASENAMEACTIVATION_PARAMNAME, "0", (baseName == null), null);
        localize("exportation_lib_basename_default", scrutariExport.getBasedefiches().getBasedefichesMetadata().getBasedefichesName());
        endP();
        startP();
        appendRadioInput(ScrutariExportChangeCommand.BASENAMEACTIVATION_PARAMNAME, "1", (baseName != null), "exportation_lib_basename_activation");
        space();
        appendSmallInput(ScrutariExportChangeCommand.BASENAMEVALUE_PARAMNAME, (baseName != null) ? baseName : "");
        endP();
    }

    void printBaseIcon() {
        String baseIcon = scrutariExport.getBaseIcon();
        startP();
        printLib("configuration_title_icon");
        append(" : ");
        appendLargeInput(ScrutariExportChangeCommand.BASEICON_PARAMNAME, baseIcon);
        endP();
    }

    void printPattern(String paramName, AccoladePattern pattern) {
        String value = (pattern != null) ? pattern.toString() : "";
        startP();
        printLib("exportation_lib_" + paramName);
        appendLargeInput(paramName, value);
        endP();
    }

    void printThesaurus(Thesaurus thesaurus) {
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        ThesaurusExportDef thesaurusExportDef = scrutariExport.getThesaurusExportDef(thesaurusKey);
        if (thesaurusExportDef == null) {
            thesaurusExportDef = new ThesaurusExportDef(thesaurusKey);
        }
        AccoladePattern libellePattern = thesaurusExportDef.getLibellePattern();
        String patternString = (libellePattern != null) ? libellePattern.toString() : "";
        startRow();
        startCell(3, 0);
        startH3();
        appendCheckInput(thesaurusKey.toString() + ScrutariExportChangeCommand.SELECTED_PARAMSUFFIX, "1", thesaurusExportDef.isSelected(), null);
        space();
        append(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        endH3();
        endCell();
        endRow();
        startRow();
        startCell(3, 0);
        startP();
        startSmall();
        appendMediumInput(thesaurusKey.toString() + ScrutariExportChangeCommand.LIBELLEPATTERN_PARAMSUFFIX, patternString);
        endSmall();
        endP();
        endCell();
        endRow();
    }

    void printCorpus(Corpus corpus) {
        SubsetKey corpusKey = corpus.getSubsetKey();
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        CorpusUI corpusUI = CorpusUIUtils.getCorpusUI(bdfServer.getUIManager(), corpusMetadata);
        CorpusExportDef corpusExportDef = scrutariExport.getCorpusExportDef(corpusKey);
        if (corpusExportDef == null) {
            corpusExportDef = new CorpusExportDef(corpusKey);
        }
        startRow();
        startCell(3, 0);
        startH3();
        append(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endH3();
        endCell();
        endRow();
        startRow();
        startCellWithTopAlign();
        startP();
        startSmall();
        localize("exportation_lib_complementfieldlist");
        endSmall();
        endP();
        endCell();
        startCell();
        startTextArea(corpus.getSubsetKey().toString() + ScrutariExportChangeCommand.COMPLEMENTKEYLIST_PARAMSUFFIX, 30, 3);
        List list = corpusExportDef.getComplementKeyList();
        for (Iterator it = list.iterator(); it.hasNext();) {
            Object key = it.next();
            append(key.toString());
            append("\n");
        }
        endTextArea();
        endCell();
        startCellWithTopAlign();
        startP();
        startCode();
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            append(corpusField.getFieldKey().toString());
            doublespace();
        }
        append(FieldKey.REDACTEURS.toString());
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            doublespace();
            append(corpusField.getFieldKey().toString());
        }
        List liageList = corpusUI.getOrderedComponentUIList(CorpusUI.LIAGE_INCLUDEUI_LIST);
        for (Iterator it = liageList.iterator(); it.hasNext();) {
            doublespace();
            IncludeUI includeUI = (IncludeUI) it.next();
            append(includeUI.getFiteredIncludeKey().toString());
        }
        endCode();
        endP();
        endCell();
        endRow();
        startRow();
        startCell();
        startP();
        startSmall();
        printLib("exportation_lib_datekey");
        endSmall();
        endP();
        endCell();
        startCell(2, 0);
        startP();
        Object obj = corpusExportDef.getDateKey();
        startSelect(corpus.getSubsetKey().toString() + ScrutariExportChangeCommand.DATEKEY_PARAMSUFFIX);
        printOption(IncludeKey.DATE_CREATION, obj);
        printOption(IncludeKey.DATE_MODIFICATION, obj);
        List fieldList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpusMetadata, CorpusField.DATATION_FIELD);
        for (Iterator it = fieldList.iterator(); it.hasNext();) {
            CorpusField corpusField = (CorpusField) it.next();
            printOption(corpusField.getFieldKey(), obj);
        }
        endSelect();
        endP();
        endCell();
        endRow();
    }

    private void printOption(Object key, Object current) {
        startOption(key.toString(), (current.equals(key)));
        append(key.toString());
        endOption();
    }

    private String getArray(int[] array) {
        if (array == null) {
            return "";
        }
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                buf.append(';');
            }
            LangInteger.appendString(buf, array[i]);
        }
        return buf.toString();
    }

    public String getCssThemeFile() {
        return "exportation_scrutari.css";
    }

}
