/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.run;

import net.mapeadores.basedefiches.tools.scrutariexport.*;
import net.mapeadores.basedefiches.scrutariexport.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.xml.*;
import java.io.*;
import javax.servlet.*;
import fr.exemole.bdfserver.*;


/**
 *
 * @author Vincent Calame
 */
public class ScutariExportRun {

    public final static String INFO_SUFFIX = ".scrutari-info";
    public final static String DATA_SUFFIX = ".scrutari-data";

    private ScutariExportRun() {
    }

    public static void runExport(ScrutariExport scrutariExport, BdfServer bdfServer, ServletContext servletContext) throws IOException {
        ExportRunner runner = new ExportRunner(scrutariExport, bdfServer, servletContext);
        runner.run();
    }


    private static class ExportRunner {

        BdfServer bdfServer;
        ServletContext servletContext;
        ScrutariExport scrutariExport;

        ExportRunner(ScrutariExport scrutariExport, BdfServer bdfServer, ServletContext servletContext) {
            this.scrutariExport = scrutariExport;
            this.bdfServer = bdfServer;
            this.servletContext = servletContext;
        }

        void run() throws IOException {
            writeHeader();
            writeBase();
        }

        private File getFile(String suffix) {
            File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PUBLIC_OUTPUTDIRECTORY);
            File scrutariDirectory = new File(outputDirectory, "scrutari");
            scrutariDirectory.mkdir();
            String fileName = scrutariExport.getName() + suffix + ".xml";
            File file = new File(scrutariDirectory, fileName);
            return file;
        }

        private void writeHeader() throws IOException {
            File file = getFile(INFO_SUFFIX);
            BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            XMLUtils.writeXMLDeclaration(bufwriter);
            HeaderWriter headerWriter = new HeaderWriter(bufwriter);
            headerWriter.write();
            bufwriter.close();
        }

        private void writeBase() throws IOException {
            File file = getFile(DATA_SUFFIX);
            BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            ScrutariExportXMLWriter xmlWriter = new ScrutariExportXMLWriter(bdfServer.getBasedefiches(), bufwriter, -999, bdfServer.getIncludeLibelleManager());
            xmlWriter.appendXMLDeclaration();
            xmlWriter.addBase(scrutariExport, bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray());
            bufwriter.close();
        }


        private class HeaderWriter extends DefaultXMLWriter {

            HeaderWriter(Appendable appendable) {
                super();
                setAppendable(appendable);
                setIndentLength(-999);
            }

            void write() throws IOException {
                openTag("header");
                addSimpleElement("date", DateInteger.toString(DateInteger.current()));
                addSimpleElement("path", scrutariExport.getName() + DATA_SUFFIX + ".xml");
                closeTag("header");
            }

        }

    }

}
