/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.run;

import fr.exemole.bdfserver.sqlexport.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import net.mapeadores.basedefiches.sqlexport.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import net.mapeadores.basedefiches.tableexport.croisement.*;
import net.mapeadores.util.beans.*;
import java.io.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.tableexport.*;
import net.mapeadores.util.logging.*;
import fr.exemole.bdfserver.externalscript.*;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportRun {

    public SqlExportRun() {
    }

    public static CommandMessage runExport(BdfServer bdfServer, SqlExportBean sqlExportBean) throws IOException {
        LocalisationInfo localisationInfo = null;
        String className = sqlExportBean.getSqlExportClassName();
        if (className.length() == 0) {
            className = "net.mapeadores.basedefiches.tools.sqlexport.DefaultSqlExport";
        }
        SqlExport sqlExport = null;
        try {
            Class classe = Class.forName(className);
            Object obj = classe.newInstance();
            sqlExport = (SqlExport) obj;
        } catch (ClassNotFoundException cnfe) {
            return CommandMessage.error("exportation_err_sqlexport_classnotfound", className);
        } catch (IllegalAccessException iae) {
            iae.printStackTrace();
            return CommandMessage.error("exportation_err_sqlexport_illegalaccess", className);
        } catch (InstantiationException ie) {
            ie.printStackTrace();
            return CommandMessage.error("exportation_err_sqlexport_instantiation", className);
        } catch (ClassCastException cce) {
            return CommandMessage.error("exportation_err_sqlexport_classcast", className);
        }
        String tableExportName = sqlExportBean.getTableExportName();
        if (tableExportName != null) {
            TableExport tableExport = bdfServer.getTableExportManager().getTableExport(tableExportName);
            if (tableExport != null) {
                SubsetTableExport[] subsetTableExportArray = tableExport.getSubsetTableExportArray(localisationInfo);
                sqlExport.setSubsetTableExportArray(subsetTableExportArray);
                CroisementTableExport[] croisementTableExport = tableExport.getCroisementTableExportArray();
                sqlExport.setCroisementTableExportArray(croisementTableExport);
            }
        }
        ParamBean[] array = sqlExportBean.getParamBeanArray();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                ParamBean paramBean = array[i];
                sqlExport.setParameter(paramBean.getName(), paramBean.getValue());
            }
        }
        String path = sqlExportBean.getPath();
        OutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(new File(path));
        } catch (IOException ioe) {
            return CommandMessage.error(new ResourceItem("exportation_err_sqlexport_path", path));
        }
        sqlExport.setBasedefiches(bdfServer.getBasedefiches());
        if (sqlExport instanceof BdfServerSqlExport) {
            ((BdfServerSqlExport) sqlExport).setBdfServer(bdfServer);
        }
        ProcessLog processLog = sqlExport.exportDump(outputStream);
        outputStream.close();
        String postCommand = sqlExportBean.getPostCommand();
        if (postCommand != null) {
            ExternalScript externalScript = bdfServer.getExternalScriptManager().getExternalScript(postCommand);
            if (externalScript != null) {
                try {
                    externalScript.exec();
                } catch (IOException ioe) {
                }
            }
        }
        return CommandMessage.log(new ResourceItem("exportation_done_sqlrun"), processLog);
    }

}
