/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.tableparser.*;
import net.mapeadores.util.text.coupleparser.*;
import net.mapeadores.util.text.*;
import java.util.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractMassCommand extends AbstractBdfCommand {

    public final static String PARSE_STEP_PARAMVALUE = "parse";
    public final static String EXECUTE_STEP_PARAMVALUE = "do";
    public final static String TEXT_PARAMNAME = "text";
    public final static String TYPEID_PARAMNAME = "typeid";
    public final static String IDTHS_TYPEID_PARAMVALUE = "idths";
    public final static String IDALPHA_TYPEID_PARAMVALUE = "idalpha";
    public final static String OTHERTHESAURUS_PARAMNAME = "otherthesaurus";
    public final static String CROISEMENTUPDATE_PARAMNAME = "croisementupdate";
    public final static String CLEAR_PARAMVALUE = "clear";
    public final static String APPEND_REPLACE_PARAMVALUE = "append_replace";
    public final static String APPEND_MAX_PARAMVALUE = "append_max";
    public final static String APPEND_NO_PARAMVALUE = "append_no";
    public final static String PARSETYPE_PARAMNAME = "parsetype";
    public final static String OOCOPY_PARAMVALUE = "oocopy";
    public final static String CSV_PARAMVALUE = "csv";
    public final static String MYSQL_PARAMVALUE = "mysql";
    public final static String FIELDS_PARAMNAME = "fields";
    public final static String MYSQL_LINESTERMINATEDBY_PARAMNAME = "mysql_linesterminatedby";
    public final static String MYSQL_FIELDSTERMINATEDBY_PARAMNAME = "mysql_fieldsterminatedby";
    public final static String CSV_DELIMITER_PARAMNAME = "csv_delimiter";
    public final static String CSV_QUOTE_PARAMNAME = "csv_quote";
    public final static String CSV_EXCEL_PARAMNAME = "csv_excel";
    public final static String COUPLESEPARATEUR_PARAMNAME = "coupleseparateur";
    public final static String FUSIONDOUBLON_PARAMNAME = "fusiondoublon";
    public final static String LANG_PARAMNAME = "lang";
    public final static String FIELDKEY_PARAMNAME = "fieldkey";
    public final static String OTHERCORPUS_PARAMNAME = "othercorpus";
    protected Corpus corpus;
    protected Thesaurus thesaurus;
    protected int workingLangInteger;
    protected ImportationResponseParameters responseParameters;
    protected Integer langInteger;
    boolean executeStep = false;
    private String text;
    protected char coupleSeparateur;
    protected boolean fusionDoublon;
    protected Thesaurus otherThesaurus;
    protected String typeId;
    protected CorpusField corpusField;
    protected short updateType;
    protected Corpus otherCorpus;

    public AbstractMassCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        checkParameters();
        if (responseParameters.mayBeDone()) {
            this.workingLangInteger = bdfUser.getWorkingLangInteger();
            if (executeStep) {
                doExecuteStep();
            } else {
                doParseStep();
            }
        }
        return responseParameters;
    }

    protected CommandTest checkParameters() throws ParameterException {
        responseParameters = new ImportationResponseParameters(bdfServer, bdfUser);
        if (needThesaurus()) {
            thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
            responseParameters.setThesaurus(thesaurus);
        }
        if (needCorpus()) {
            corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
            responseParameters.setCorpus(corpus);
        }
        String step = getMandatoryParameter("step");
        if (step.equals(PARSE_STEP_PARAMVALUE)) {
            executeStep = false;
        } else if (step.equals(EXECUTE_STEP_PARAMVALUE)) {
            executeStep = true;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "step = " + step);
        }
        if (!executeStep) {
            text = getMandatoryParameter(TEXT_PARAMNAME);
            text = text.trim();
            responseParameters.putParameter(TEXT_PARAMNAME, text);
            initSuppParam(getParseStepParamArray());
        } else {
            initSuppParam(getExecuteStepParamArray());
        }
        return responseParameters;
    }

    private void initSuppParam(String[] paramArray) throws ParameterException {
        for (int i = 0; i < paramArray.length; i++) {
            String param = paramArray[i];
            if (param.equals(LANG_PARAMNAME)) {
                String value = getMandatoryParameter(LANG_PARAMNAME);
                langInteger = initLang(responseParameters, value);
            } else if (param.equals(COUPLESEPARATEUR_PARAMNAME)) {
                String value = getMandatoryParameter(COUPLESEPARATEUR_PARAMNAME);
                coupleSeparateur = initCoupleSeparateur(responseParameters, value);
            } else if (param.equals(FUSIONDOUBLON_PARAMNAME)) {
                String value = requestMap.getParameter(FUSIONDOUBLON_PARAMNAME);
                fusionDoublon = initFusionDoublon(responseParameters, value);
            } else if (param.equals(OTHERTHESAURUS_PARAMNAME)) {
                String value = getMandatoryParameter(OTHERTHESAURUS_PARAMNAME);
                otherThesaurus = initOtherThesaurus(responseParameters, value);
            } else if (param.equals(TYPEID_PARAMNAME)) {
                String value = getTypeIdParameter();
                typeId = initTypeId(responseParameters, value);
            } else if (param.equals(FIELDKEY_PARAMNAME)) {
                String value = getMandatoryParameter(FIELDKEY_PARAMNAME);
                corpusField = initCorpusField(responseParameters, value);
            } else if (param.equals(CROISEMENTUPDATE_PARAMNAME)) {
                String value = getMandatoryParameter(CROISEMENTUPDATE_PARAMNAME);
                updateType = initCroisementUpdate(responseParameters, value);
            } else if (param.equals(OTHERCORPUS_PARAMNAME)) {
                String value = getMandatoryParameter(OTHERCORPUS_PARAMNAME);
                otherCorpus = initOtherCorpus(responseParameters, value);
            }
        }
    }

    private String getTypeIdParameter() throws ParameterException {
        Thesaurus thesaurus = responseParameters.getThesaurus();
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            return null;
        } else {
            return getMandatoryParameter(TYPEID_PARAMNAME);
        }
    }

    private void doParseStep() throws ParameterException {
        boolean init = initSupplementaryParseParameters(responseParameters);
        if (!init) {
            responseParameters.setHtmlPageCode(getFormPage());
        } else if (text.length() == 0) {
            responseParameters.setHtmlPageCode(getFormPage());
            responseParameters.setErrorMessage("saisie_err_textparse_empty", null);
        } else {
            ParseResult parseResult = doParseStep(text);
            responseParameters.putParameter(ImportationResponseParameters.PARSERESULT_KEY, parseResult);
            if (parseResult.hasOnlyParseError()) {
                responseParameters.setHtmlPageCode(getFormPage());
                responseParameters.setErrorMessage("saisie_err_textparse_allwrong", null);
            } else {
                responseParameters.setHtmlPageCode(getConfirmPage());
                if (parseResult.hasError()) {
                    responseParameters.setErrorMessage("saisie_err_textparse_some", null);
                }
            }
        }
    }

    protected boolean initSupplementaryParseParameters(ImportationResponseParameters responseParameters) throws ParameterException {
        return true;
    }

    protected abstract ParseResult doParseStep(String text);

    protected abstract void doExecuteStep() throws ParameterException;

    public boolean needInit(String param) {
        return false;
    }

    public String[] getParseStepParamArray() {
        String[] res = new String[0];
        return res;
    }

    public String[] getExecuteStepParamArray() {
        String[] res = new String[0];
        return res;
    }

    /*public boolean needAdmin() {
    return true;
    }*/
    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ImportationResponsePrinterFactory();
    }

    public boolean needCorpus() {
        return false;
    }

    public boolean needThesaurus() {
        return false;
    }

    protected short getCroisementUpdate(ImportationResponseParameters responseParameters) throws ParameterException {
        String updatetypeString = requestMap.getParameter(CROISEMENTUPDATE_PARAMNAME);
        if (updatetypeString == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, CROISEMENTUPDATE_PARAMNAME);
        }
        return initCroisementUpdate(responseParameters, updatetypeString);
    }

    protected String[] getFieldsArray(ImportationResponseParameters responseParameters) throws ParameterException {
        String value = requestMap.getParameter(FIELDS_PARAMNAME);
        if (value == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FIELDS_PARAMNAME);
        }
        return initFieldsParam(responseParameters, value);
    }

    protected TableParser.Parameters getTableParserParameters(ImportationResponseParameters responseParameters) throws ParameterException {
        String parserType = requestMap.getParameter(PARSETYPE_PARAMNAME);
        if (parserType == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, PARSETYPE_PARAMNAME);
        }
        return initTableParserParameters(parserType, requestMap, responseParameters);
    }

    public abstract short getFormPage();

    public abstract short getConfirmPage();

    static void checkCommonParameters(ImportationResponseParameters fusionParameters, RequestMap requestMap, String[] paramArray) throws ParameterException {
        String text = requestMap.getParameter(TEXT_PARAMNAME);
        if (text != null) {
            fusionParameters.putParameter(TEXT_PARAMNAME, text);
        }
        Thesaurus thesaurus = fusionParameters.getThesaurus();
        if ((thesaurus != null) && (ThesaurusUtils.isIdalphaType(thesaurus))) {
            String parseType = requestMap.getParameter(TYPEID_PARAMNAME);
            if (parseType != null) {
                if ((parseType.equals(IDTHS_TYPEID_PARAMVALUE)) || (parseType.equals(IDALPHA_TYPEID_PARAMVALUE))) {
                    fusionParameters.putParameter(TYPEID_PARAMNAME, parseType);
                } else {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TYPEID_PARAMNAME + " = " + parseType);
                }
            }
        }
        if (paramArray == null) {
            return;
        }
        for (int i = 0; i < paramArray.length; i++) {
            String param = paramArray[i];
            String value = requestMap.getParameter(param);
            if (value == null) {
                continue;
            }
            if (param.equals(LANG_PARAMNAME)) {
                initLang(fusionParameters, value);
            } else if (param.equals(COUPLESEPARATEUR_PARAMNAME)) {
                initCoupleSeparateur(fusionParameters, value);
            } else if (param.equals(FUSIONDOUBLON_PARAMNAME)) {
                initFusionDoublon(fusionParameters, value);
            } else if (param.equals(OTHERTHESAURUS_PARAMNAME)) {
                initOtherThesaurus(fusionParameters, value);
            } else if (param.equals(TYPEID_PARAMNAME)) {
                initTypeId(fusionParameters, value);
            } else if (param.equals(FIELDKEY_PARAMNAME)) {
                initCorpusField(fusionParameters, value);
            } else if (param.equals(CROISEMENTUPDATE_PARAMNAME)) {
                initCroisementUpdate(fusionParameters, value);
            } else if (param.equals(OTHERCORPUS_PARAMNAME)) {
                initOtherCorpus(fusionParameters, value);
            }
        }
    }

    static void checkCroisementUpdateParameter(ImportationResponseParameters parameters, RequestMap requestMap) throws ParameterException {
        String updatetypeString = requestMap.getParameter(CROISEMENTUPDATE_PARAMNAME);
        if (updatetypeString != null) {
            initCroisementUpdate(parameters, updatetypeString);
        }
    }

    public static void checkTableParserParameters(ImportationResponseParameters parameters, RequestMap requestMap) throws ParameterException {
        String paramname = requestMap.getParameter(PARSETYPE_PARAMNAME);
        if (paramname == null) {
            return;
        }
        initTableParserParameters(paramname, requestMap, parameters);
    }

    public static void checkFieldsParameters(ImportationResponseParameters parameters, RequestMap requestMap) {
        String fieldsParam = requestMap.getParameter(FIELDS_PARAMNAME);
        if (fieldsParam == null) {
            return;
        }
        initFieldsParam(parameters, fieldsParam);
    }

    private static TableParser.Parameters initTableParserParameters(String parseType, RequestMap requestMap, ImportationResponseParameters responseParameters) throws ParameterException {
        TableParser.Parameters params = null;
        if (parseType.equals(OOCOPY_PARAMVALUE)) {
            params = new OOCopyParameters();
        } else if (parseType.equals(CSV_PARAMVALUE)) {
            CsvParameters csvParameters = new CsvParameters();
            String delimParam = requestMap.getParameter(CSV_DELIMITER_PARAMNAME);
            if (delimParam != null) {
                try {
                    char delim = StringUtils.stringToChar(delimParam);
                    csvParameters.setDelimiter(delim);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CSV_DELIMITER_PARAMNAME + " = " + delimParam);
                }
            }
            String quoteParam = requestMap.getParameter(CSV_QUOTE_PARAMNAME);
            if (quoteParam != null) {
                try {
                    char quote = StringUtils.stringToChar(quoteParam);
                    csvParameters.setQuote(quote);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CSV_QUOTE_PARAMNAME + " = " + quoteParam);
                }
            }
            String excelParam = requestMap.getParameter(CSV_EXCEL_PARAMNAME);
            if (excelParam != null) {
                if (excelParam.equals("1")) {
                    csvParameters.setExcelCSV(true);
                } else if (!excelParam.equals("0")) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CSV_EXCEL_PARAMNAME + " = " + excelParam);
                }
            }
            params = csvParameters;
        } else if (parseType.equals(MYSQL_PARAMVALUE)) {
            MysqlParameters mysqlParameters = new MysqlParameters();
            String fieldsParam = requestMap.getParameter(MYSQL_FIELDSTERMINATEDBY_PARAMNAME);
            if (fieldsParam != null) {
                try {
                    char fieldsby = StringUtils.stringToChar(fieldsParam);
                    mysqlParameters.setFieldsTerminatedBy(fieldsby);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, MYSQL_FIELDSTERMINATEDBY_PARAMNAME + " = " + fieldsParam);
                }
            }
            String linesParam = requestMap.getParameter(MYSQL_LINESTERMINATEDBY_PARAMNAME);
            if (linesParam != null) {
                try {
                    char linesby = StringUtils.stringToChar(linesParam);
                    mysqlParameters.setLinesTerminatedBy(linesby);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, MYSQL_LINESTERMINATEDBY_PARAMNAME + " = " + linesParam);
                }
            }
            params = mysqlParameters;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, PARSETYPE_PARAMNAME + "=" + parseType);
        }
        responseParameters.putParameter(PARSETYPE_PARAMNAME, params);
        return params;
    }

    private static String[] initFieldsParam(ImportationResponseParameters parameters, String value) {
        String[] fieldsArray = StringUtils.getTechnicalTokens(value);
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < fieldsArray.length; i++) {
            if (i > 0) {
                buf.append("; ");
            }
            buf.append(fieldsArray[i]);
        }
        parameters.putParameter(FIELDS_PARAMNAME, buf.toString());
        return fieldsArray;
    }

    private static char initCoupleSeparateur(ImportationResponseParameters parameters, String value) throws ParameterException {
        try {
            char c = StringUtils.stringToChar(value);
            parameters.putParameter(COUPLESEPARATEUR_PARAMNAME, new Character(c));
            return c;
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, COUPLESEPARATEUR_PARAMNAME + " = " + value);
        }
    }

    private static String initTypeId(ImportationResponseParameters parameters, String value) throws ParameterException {
        Thesaurus thesaurus = parameters.getThesaurus();
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            return null;
        }
        if ((value.equals(IDTHS_TYPEID_PARAMVALUE)) || (value.equals(IDALPHA_TYPEID_PARAMVALUE))) {
            parameters.putParameter(TYPEID_PARAMNAME, value);
            return value;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TYPEID_PARAMNAME + " = " + value);
        }
    }

    private static Integer initLang(ImportationResponseParameters responseParameters, String value) throws ParameterException {
        try {
            int lang = LangInteger.parse(value);
            Integer langInteger = new Integer(lang);
            responseParameters.putParameter(LANG_PARAMNAME, langInteger);
            return langInteger;
        } catch (LangIntegerException lie) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, LANG_PARAMNAME + " = " + value);
        }
    }

    private static boolean initFusionDoublon(ImportationResponseParameters parameters, String value) throws ParameterException {
        boolean fusion = false;
        if (value == null) {
            fusion = false;
        } else if (value.equals(YES_PARAMVALUE)) {
            fusion = true;
        } else if (value.equals(NO_PARAMVALUE)) {
            fusion = false;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FUSIONDOUBLON_PARAMNAME + " = " + value);
        }
        parameters.putParameter(FUSIONDOUBLON_PARAMNAME, new Boolean(fusion));
        return fusion;
    }

    private static CorpusField initCorpusField(ImportationResponseParameters parameters, String fieldkeyString) throws ParameterException {
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldkeyString);
        } catch (java.text.ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELDKEY_PARAMNAME + " = " + fieldkeyString);
        }
        Corpus corpus = parameters.getCorpus();
        CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FIELDKEY_PARAMNAME + " = " + fieldkeyString);
        }
        parameters.putParameter(FIELDKEY_PARAMNAME, fieldKey);
        return corpusField;
    }

    private static Thesaurus initOtherThesaurus(ImportationResponseParameters parameters, String value) throws ParameterException {
        Thesaurus otherThesaurus = BasedefichesUtils.getThesaurus(parameters.getBdfServer().getBasedefiches(), value);
        if (otherThesaurus == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, OTHERTHESAURUS_PARAMNAME + " = " + value);
        }
        parameters.putParameter(OTHERTHESAURUS_PARAMNAME, otherThesaurus);
        return otherThesaurus;
    }

    private static Corpus initOtherCorpus(ImportationResponseParameters parameters, String value) throws ParameterException {
        Corpus otherCorpus = BasedefichesUtils.getCorpus(parameters.getBdfServer().getBasedefiches(), value);
        if (otherCorpus == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, OTHERCORPUS_PARAMNAME + " = " + value);
        }
        parameters.putParameter(OTHERCORPUS_PARAMNAME, otherCorpus);
        return otherCorpus;
    }

    public static short initCroisementUpdate(ImportationResponseParameters parameters, String updateType) throws ParameterException {
        short ut = 0;
        if (updateType.equals(CLEAR_PARAMVALUE)) {
            ut = CroisementEditor.CLEAR_EXISTING;
        } else if (updateType.equals(APPEND_REPLACE_PARAMVALUE)) {
            ut = CroisementEditor.APPEND_AND_POIDS_REPLACE;
        } else if (updateType.equals(APPEND_MAX_PARAMVALUE)) {
            ut = CroisementEditor.APPEND_AND_POIDS_MAX;
        } else if (updateType.equals(APPEND_NO_PARAMVALUE)) {
            ut = CroisementEditor.APPEND_AND_POIDS_NOCHANGE;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CROISEMENTUPDATE_PARAMNAME + " = " + updateType);
        }
        parameters.putParameter(CROISEMENTUPDATE_PARAMNAME, new Short(ut));
        return ut;
    }

    public static String croisementUpdateToString(short updateType) {
        switch (updateType) {
            case CroisementEditor.CLEAR_EXISTING:
                return CLEAR_PARAMVALUE;
            case CroisementEditor.APPEND_AND_POIDS_REPLACE:
                return APPEND_REPLACE_PARAMVALUE;
            case CroisementEditor.APPEND_AND_POIDS_MAX:
                return APPEND_MAX_PARAMVALUE;
            case CroisementEditor.APPEND_AND_POIDS_NOCHANGE:
                return APPEND_NO_PARAMVALUE;
            default:
                throw new IllegalArgumentException("wrong udpateType = " + updateType);
        }
    }

    protected String listToString(List list) {
        if (list.isEmpty()) {
            return null;
        }
        StringBuffer buf = new StringBuffer();
        int size = list.size();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                buf.append(", ");
            }
            buf.append(list.get(i).toString());
        }
        return buf.toString();
    }


    public static abstract class CoupleCorpusParser {

        protected Corpus corpus;
        protected ParseResult parseResult = new ParseResult(ParseResult.LINES);
        private CoupleParser coupleParser;

        CoupleCorpusParser(Corpus corpus, char coupleSeparateur) {
            this.corpus = corpus;
            coupleParser = new CoupleParser(CoupleParser.INTEGER_MODE);
            coupleParser.setCoupleSeparateur(coupleSeparateur);
        }

        public ParseResult getParseResult() {
            return parseResult;
        }

        public void parse(String text) {
            Couple[] coupleArray = coupleParser.parse(text);
            int length = coupleArray.length;
            for (int i = 0; i < length; i++) {
                parseCouple(coupleArray[i]);
            }
        }

        private void parseCouple(Couple couple) {
            if (couple instanceof ErrorCouple) {
                parseResult.addParseError(couple.getValue(), couple.getLineNumber());
                return;
            }
            IntegerCouple integerCouple = (IntegerCouple) couple;
            int idcorpus = integerCouple.getInteger();
            AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche == null) {
                parseResult.addBdfError("UNKNOWN IDCORPUS", String.valueOf(idcorpus), couple.getLineNumber());
                return;
            }
            parseValue(abregeFiche, couple.getValue(), couple.getLineNumber());
        }

        protected abstract void parseValue(AbregeFiche abregeFiche, String value, int lineNumber);

        protected void addResultWithNoDoublon(int id, Object key, Object value, int lineNumber) {
            if (parseResult.containsId(id)) {
                parseResult.checkDoublon(id, lineNumber);
            } else {
                parseResult.addResult(id, key, value, lineNumber);
            }
        }

    }


    public static abstract class CoupleThesaurusParser {

        ParseResult parseResult = new ParseResult(ParseResult.LINES);
        Thesaurus thesaurus;
        private CoupleParser coupleParser;
        private boolean idalphaParser = false;

        public CoupleThesaurusParser(Thesaurus thesaurus, String parseType, char coupleSeparateur) {
            this.thesaurus = thesaurus;
            short mode = 0;
            if ((parseType == null) || (parseType.equals(IDTHS_TYPEID_PARAMVALUE))) {
                mode = CoupleParser.INTEGER_MODE;
                idalphaParser = false;
            } else {
                mode = CoupleParser.IDALPHA_MODE;
                idalphaParser = true;
            }
            coupleParser = new CoupleParser(mode);
            coupleParser.setCoupleSeparateur(coupleSeparateur);
        }

        public ParseResult getParseResult() {
            return parseResult;
        }

        public void parse(String text) {
            Couple[] coupleArray = coupleParser.parse(text);
            int length = coupleArray.length;
            if (idalphaParser) {
                for (int i = 0; i < length; i++) {
                    parseIdalphaCouple(coupleArray[i]);
                }
            } else {
                for (int i = 0; i < length; i++) {
                    parseIdthsCouple(coupleArray[i]);
                }
            }
            endParse();
        }

        private void parseIdalphaCouple(Couple couple) {
            if (couple instanceof ErrorCouple) {
                parseResult.addParseError(couple.getValue(), couple.getLineNumber());
                return;
            }
            IdalphaCouple idalphaCouple = (IdalphaCouple) couple;
            String idalpha = idalphaCouple.getIdalpha();
            Motcle motcle = (Motcle) thesaurus.getMotcleByIdalpha(idalpha);
            if (motcle == null) {
                parseResult.addBdfError("UNKNOWN IDALPHA", idalpha, couple.getLineNumber());
                return;
            }
            parseValue(motcle, couple.getValue(), couple.getLineNumber(), true);
        }

        private void parseIdthsCouple(Couple couple) {
            if (couple instanceof ErrorCouple) {
                parseResult.addParseError(couple.getValue(), couple.getLineNumber());
                return;
            }
            IntegerCouple integerCouple = (IntegerCouple) couple;
            int idths = integerCouple.getInteger();
            Motcle motcle = thesaurus.getMotcleByIdths(idths);
            if (motcle == null) {
                parseResult.addBdfError("UNKNOWN IDTHS", String.valueOf(idths), couple.getLineNumber());
                return;
            }
            parseValue(motcle, couple.getValue(), couple.getLineNumber(), false);
        }

        protected abstract void parseValue(Motcle motcle, String value, int lineNumber, boolean fromIdalpha);

        protected abstract void endParse();

        protected void addResultWithNoDoublon(int id, Object key, Object value, int lineNumber) {
            if (parseResult.containsId(id)) {
                parseResult.checkDoublon(id, lineNumber);
            } else {
                parseResult.addResult(id, key, value, lineNumber);
            }
        }

    }

}
