/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractBdfCommand {

    public static final String MASSFIELDFORM_PAGE_PARAMVALUE = "massfieldform";
    public static final String MASSLIBFORM_PAGE_PARAMVALUE = "masslibform";
    public static final String MASSMOTCLEREMOVEFORM_PAGE_PARAMVALUE = "massmotcleremoveform";
    public static final String MASSMOTCLEMERGEFORM_PAGE_PARAMVALUE = "massmotclemergeform";
    public static final String MASSMOTCLEMOVEFORM_PAGE_PARAMVALUE = "massmotclemoveform";
    public static final String MASSMOTCLECREATIONFORM_PAGE_PARAMVALUE = "massmotclecreationform";
    public static final String MASSFIELDTRANSFERFORM_PAGE_PARAMVALUE = "massfieldtransferform";
    public static final String MASSLIAGEFORM_PAGE_PARAMVALUE = "massliageform";
    public static final String MASSINDEXATIONFICHEFORM_PAGE_PARAMVALUE = "massindexationficheform";
    public static final String MASSFICHECREATIONFORM_PAGE_PARAMVALUE = "massfichecreationform";
    public static final String MASSUIFORM_PAGE_PARAMVALUE = "massuiform";
    public static final String MASSFICHEREMOVEFORM_PAGE_PARAMVALUE = "massficheremove";
    ImportationResponseParameters responseParameters;

    public DefaultCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        responseParameters = new ImportationResponseParameters(bdfServer, bdfUser);
        short pageType = getPageType();
        switch (pageType) {
            case ImportationResponseParameters.MASSFIELDFORM_PAGE:
            case ImportationResponseParameters.MASSLIAGEFORM_PAGE:
            case ImportationResponseParameters.MASSINDEXATIONFICHEFORM_PAGE:
            case ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE:
            case ImportationResponseParameters.MASSFICHEREMOVEFORM_PAGE:
                initCorpusSelection();
                break;
            case ImportationResponseParameters.MASSLIBFORM_PAGE:
            case ImportationResponseParameters.MASSMOTCLEMOVEFORM_PAGE:
            case ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE:
            case ImportationResponseParameters.MASSMOTCLEREMOVEFORM_PAGE:
            case ImportationResponseParameters.MASSMOTCLEMERGEFORM_PAGE:
                initThesaurusSelection();
                break;
        }
        responseParameters.setHtmlPageCode(pageType);
        if (pageType != ImportationResponseParameters.MASSINDEX_PAGE) {
            AbstractMassCommand.checkCommonParameters(responseParameters, requestMap, getParamArray(pageType));
        }

        switch (pageType) {
            case ImportationResponseParameters.MASSLIBFORM_PAGE:
                MassLibCommand.checkExistingParameters(responseParameters, requestMap);
                break;
            case ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE:
                MassMotcleCreationCommand.checkExistingParameters(responseParameters, requestMap);
                break;
            case ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE:
                MassFicheCreationCommand.checkExistingParameters(responseParameters, requestMap);
                break;
            default:

        }
        return responseParameters;
    }

    private String[] getParamArray(short pageType) {
        switch (pageType) {
            case ImportationResponseParameters.MASSFIELDFORM_PAGE:
                return MassFieldCommand.getSuppParamArray();
            case ImportationResponseParameters.MASSLIBFORM_PAGE:
                return MassLibCommand.getSuppParamArray();
            case ImportationResponseParameters.MASSMOTCLEREMOVEFORM_PAGE:
                return MassMotcleRemoveCommand.getSuppParamArray();
            case ImportationResponseParameters.MASSMOTCLEMERGEFORM_PAGE:
                return MassMotcleMergeCommand.getSuppParamArray();
            case ImportationResponseParameters.MASSLIAGEFORM_PAGE:
                return MassLiageCommand.getSuppParamArray();
            case ImportationResponseParameters.MASSINDEXATIONFICHEFORM_PAGE:
                return MassIndexationFicheCommand.getSuppParamArray();
            /*case ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE :
            return MassFicheCreationCommand;*/
            /*case ImportationResponseParameters.MASSMOTCLEMOVEFORM_PAGE :
            return MassMotcleMoveCommand;*/
            /*case ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE :
            return MassMotcleCreationCommand.;*/
            case ImportationResponseParameters.MASSUIFORM_PAGE:
                return MassUICommand.getSuppParamArray();
            default:
                return null;
        }
    }

    private void initThesaurusSelection() throws ParameterException {
        Thesaurus thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
        responseParameters.setThesaurus(thesaurus);
        bdfUser.setParameterValue("saisie.thesaurusselection", thesaurus.getSubsetKey());
    }

    private void initCorpusSelection() throws ParameterException {
        Corpus corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
        responseParameters.setCorpus(corpus);
        bdfUser.setParameterValue("saisie.corpusselection", corpus.getSubsetKey());
    }

    public static String getCommandString() {
        return null;
    }

    public boolean needSynchronisation() {
        return false;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            return ImportationResponseParameters.MASSINDEX_PAGE;
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(MASSFIELDFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSFIELDFORM_PAGE;
        } else if (page_param.equals(MASSLIBFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSLIBFORM_PAGE;
        } else if (page_param.equals(MASSMOTCLEREMOVEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSMOTCLEREMOVEFORM_PAGE;
        } else if (page_param.equals(MASSMOTCLEMERGEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSMOTCLEMERGEFORM_PAGE;
        } else if (page_param.equals(MASSMOTCLEMOVEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSMOTCLEMOVEFORM_PAGE;
        } else if (page_param.equals(MASSMOTCLECREATIONFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE;
        } else if (page_param.equals(MASSFIELDTRANSFERFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSFIELDTRANSFERFORM_PAGE;
        } else if (page_param.equals(MASSLIAGEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSLIAGEFORM_PAGE;
        } else if (page_param.equals(MASSINDEXATIONFICHEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSINDEXATIONFICHEFORM_PAGE;
        } else if (page_param.equals(MASSFICHECREATIONFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE;
        } else if (page_param.equals(MASSUIFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSUIFORM_PAGE;
        } else if (page_param.equals(MASSFICHEREMOVEFORM_PAGE_PARAMVALUE)) {
            return ImportationResponseParameters.MASSFICHEREMOVEFORM_PAGE;
        }
        throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ImportationResponsePrinterFactory();
    }

}
