/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.tableparser.*;
import net.mapeadores.basedefiches.misc.*;
import fr.exemole.bdfserver.request.parsers.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFicheCreationCommand extends AbstractMassCommand {

    public final static String SEPARATEUR_PARAMNAME = "separateur";
    public final static String KEYARRAY_PARAM = "keyarray";
    public final static String IDCORPUSARRAY_PARAMNAME = "idcorpusarray";
    private CorpusFieldAndIncludeList corpusFieldAndIncludeList;
    private char separateur;
    private TableParser.Parameters tableParserParameters;

    public MassFicheCreationCommand() {
    }

    public boolean needCorpus() {
        return true;
    }

    public static String getCommandString() {
        return "MassFicheCreation";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSFICHECREATIONCONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(corpus, corpusFieldAndIncludeList);
        parser.parse(text, tableParserParameters);
        return parser.getParseResult();
    }

    protected boolean initSupplementaryParseParameters(ImportationResponseParameters saisieResponseParameters) throws ParameterException {
        tableParserParameters = getTableParserParameters(saisieResponseParameters);
        String[] fieldsArray = getFieldsArray(saisieResponseParameters);
        if (fieldsArray.length == 0) {
            saisieResponseParameters.setErrorMessage("saisie_err_fields_empty", null);
            return false;
        }
        corpusFieldAndIncludeList = new CorpusFieldAndIncludeList(corpus.getCorpusMetadata());
        corpusFieldAndIncludeList.parse(fieldsArray);
        if (corpusFieldAndIncludeList.hasError()) {
            List errList = new ArrayList();
            String wrong = listToString(corpusFieldAndIncludeList.getParseErrorList());
            if (wrong != null) {
                errList.add(new ResourceItem("saisie_err_fields_wrong", wrong));
            }
            String doublon = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.UNAUTHORIZED_DOUBLON_ERROR));
            if (doublon != null) {
                errList.add(new ResourceItem("saisie_err_fields_doublon", doublon));
            }
            String unknowncorpus = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.UNKNOWN_CORPUS_ERROR));
            if (unknowncorpus != null) {
                errList.add(new ResourceItem("saisie_err_fields_unknowncorpus", unknowncorpus));
            }
            String unknownthesaurus = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.UNKNOWN_THESAURUS_ERROR));
            if (unknownthesaurus != null) {
                errList.add(new ResourceItem("saisie_err_fields_unknownthesaurus", unknownthesaurus));
            }
            String unknownfield = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.UNKNOWN_FIELD_ERROR));
            if (unknownfield != null) {
                errList.add(new ResourceItem("saisie_err_fields_unknownfield", unknownfield));
            }
            String forbiddenSubfield = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.FORBIDDEN_SUBFIELD_ERROR));
            if (forbiddenSubfield != null) {
                errList.add(new ResourceItem("saisie_err_fields_forbiddensubfield", forbiddenSubfield));
            }
            String fieldWithSubfield = listToString(corpusFieldAndIncludeList.getBdfErrorList(CorpusFieldAndIncludeList.FIELD_WITH_SUBFIELD_ERROR));
            if (fieldWithSubfield != null) {
                errList.add(new ResourceItem("saisie_err_fields_fieldwithsubfield", fieldWithSubfield));
            }
            saisieResponseParameters.setMultiErrorMessage("saisie_err_fields", errList);
            return false;
        }
        saisieResponseParameters.putParameter(KEYARRAY_PARAM, corpusFieldAndIncludeList.getKeyArray());
        return true;
    }

    protected void doExecuteStep() throws ParameterException {
        String[] idcorpusArray = requestMap.getParameterValues(IDCORPUSARRAY_PARAMNAME);
        if (idcorpusArray == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, IDCORPUSARRAY_PARAMNAME);
        }
        CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
        int count = 0;
        SaisieEngine saisieEngine = new SaisieEngine(bdfServer, bdfUser, requestMap, SaisieEngine.ALL_MASK);
        for (int i = 0; i < idcorpusArray.length; i++) {
            try {
                int idcorpus = Integer.parseInt(idcorpusArray[i]);
                if (idcorpus < 1) {
                    idcorpus = -1;
                }
                AbregeFiche abregeFiche = corpusEditor.createFiche(idcorpus);
                corpusEditor.setDate(abregeFiche, DateInteger.current(), false);
                saisieEngine.updateFiche(abregeFiche, idcorpusArray[i]);
                count++;
            } catch (NumberFormatException nfe) {
                nfe.printStackTrace();
            } catch (ExistingIdException eii) {
                eii.printStackTrace();
            } catch (NoMasterIdsubsetException nmie) {
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_fichecreation", new Integer(count));
        }
    }

    public static void checkExistingParameters(ImportationResponseParameters parameters, RequestMap requestMap) throws ParameterException {
        checkTableParserParameters(parameters, requestMap);
        checkFieldsParameters(parameters, requestMap);
    }


    private static class Parser {
        //private Object[] keyArray;

        private int realFieldCount;
        private FieldProperty[] fieldPropertyArray;
        private boolean[] cleanAtLast;
        private ParseResult parseResult = new ParseResult(ParseResult.LINES);
        private Corpus corpus;
        private int idcorpusIndex = -1;
        private int langIndex = -1;
        private int allFieldCount;

        Parser(Corpus corpus, CorpusFieldAndIncludeList corpusFieldAndIncludeList) {
            this.corpus = corpus;
            realFieldCount = corpusFieldAndIncludeList.getKeyCount();
            cleanAtLast = new boolean[realFieldCount];
            Object[] allArray = corpusFieldAndIncludeList.getAllArray();
            allFieldCount = allArray.length;
            fieldPropertyArray = new FieldProperty[allFieldCount];
            for (int i = 0; i < allFieldCount; i++) {
                Object obj = allArray[i];
                if (obj instanceof CorpusFieldAndIncludeList.DoublonItem) {
                    FieldProperty fieldProperty = new FieldProperty((CorpusFieldAndIncludeList.DoublonItem) obj);
                    fieldPropertyArray[i] = fieldProperty;
                } else {
                    FieldProperty fieldProperty = new FieldProperty((CorpusFieldAndIncludeList.KeyItem) obj);
                    Object key = fieldProperty.getKey();
                    if (key instanceof FieldKey) {
                        FieldKey fieldKey = (FieldKey) key;
                        if (fieldKey.equals(FieldKey.IDCORPUS)) {
                            idcorpusIndex = i;
                        } else if (fieldKey.equals(FieldKey.LANG)) {
                            langIndex = i;
                        } else if (fieldKey.ownsToEntete()) {
                            cleanAtLast[fieldProperty.getRealIndex()] = true;
                        }
                    }
                    fieldPropertyArray[i] = fieldProperty;
                }

            }
        }

        ParseResult getParseResult() {
            return parseResult;
        }

        private void parse(String s, TableParser.Parameters tableParserParameters) {
            String[][] lines = TableParser.parse(s, tableParserParameters);
            for (int i = 0; i < lines.length; i++) {
                String[] line = lines[i];
                boolean done = testLine(i + 1, line);
                if (done) {
                }
            }
        }

        private boolean testLine(int lineNumber, String[] line) {
            int length = line.length;
            if (length == 0) {
                return false;
            }
            if (length > allFieldCount) {
                parseResult.addParseError("TOO MANY FIELDS", lineNumber);
                return false;
            }
            int idcorpus = -lineNumber - 1;
            if (idcorpusIndex != -1) {
                if (idcorpusIndex < length) {
                    String idcorpusString = line[idcorpusIndex].trim();
                    try {
                        idcorpus = Integer.parseInt(idcorpusString);
                        if (idcorpus < 1) {
                            parseResult.addParseError("IDCORPUS < 1", lineNumber);
                            return false;
                        }
                        if (corpus.getAbregeFicheByIdcorpus(idcorpus) != null) {
                            parseResult.addBdfError("EXISTING IDCORPUS", idcorpusString, lineNumber);
                            return false;
                        }
                        line[idcorpusIndex] = idcorpusString;
                    } catch (NumberFormatException nfe) {
                        parseResult.addParseError("IDCORPUS NOT INTEGER", lineNumber);
                        return false;
                    }
                }
            }
            if (langIndex != -1) {
                if (langIndex < length) {
                    String langString = line[langIndex].trim();
                    if (langString.length() > 0) {
                        try {
                            int lg = LangInteger.parse(langString);
                            langString = LangInteger.toString(lg);
                        } catch (LangIntegerException lie) {
                            parseResult.addParseError("WRONG LANG CODE", lineNumber);
                            return false;
                        }
                    }
                    line[langIndex] = langString;
                }
            }
            Integer itg = new Integer(idcorpus);
            if (parseResult.containsId(idcorpus)) {
                parseResult.checkDoublon(idcorpus, lineNumber);
            } else {
                String[] newLine = checkLine(line);
                parseResult.addResult(idcorpus, itg, newLine, lineNumber);
            }
            return true;
        }

        private String[] checkLine(String[] line) {
            String[] newLine = new String[realFieldCount];
            int length = line.length;
            int p = 0;
            for (int i = 0; i < length; i++) {
                FieldProperty fieldProperty = fieldPropertyArray[i];
                int realIndex = fieldProperty.getRealIndex();
                String value = line[i];
                if (fieldProperty.cleanField()) {
                    value = StringUtils.cleanString(value);
                }
                if (fieldProperty.isDoublon()) {
                    String current = newLine[realIndex];
                    if (current.length() == 0) {
                        newLine[realIndex] = value;
                    } else {
                        String separator = fieldProperty.getSeparator();
                        newLine[realIndex] = current + separator + value;
                    }
                } else {
                    newLine[realIndex] = value;
                    p++;
                }
            }
            for (int i = p + 1; i < realFieldCount; i++) {
                newLine[i] = "";
            }
            for (int i = 0; i < realFieldCount; i++) {
                if (cleanAtLast[i]) {
                    newLine[i] = StringUtils.cleanString(newLine[i]);
                }
            }
            return newLine;
        }


        private static class FieldProperty {

            Object key;
            boolean cleanField;
            boolean isDoublon;
            int realIndex;
            String separator;

            FieldProperty(CorpusFieldAndIncludeList.KeyItem keyItem) {
                this.isDoublon = false;
                this.key = keyItem.getKey();
                this.realIndex = keyItem.getKeyIndex();
                cleanField = true;
                if (key instanceof FieldKey) {
                    FieldKey fk = (FieldKey) key;
                    if ((fk.ownsToCorpsdefiche()) || (fk.ownsToEntete())) {
                        cleanField = false;
                    }
                }
            }

            FieldProperty(CorpusFieldAndIncludeList.DoublonItem doublonItem) {
                this.isDoublon = true;
                CorpusFieldAndIncludeList.KeyItem keyItem = doublonItem.getKeyItem();
                this.key = keyItem.getKey();
                this.realIndex = keyItem.getKeyIndex();
                cleanField = true;
                separator = "; ";
                if (key instanceof FieldKey) {
                    FieldKey fk = (FieldKey) key;
                    if (fk.ownsToCorpsdefiche()) {
                        cleanField = false;
                        separator = "\n";
                    } else if (fk.ownsToEntete()) {
                        cleanField = false;
                        separator = "";
                    }
                }
            }

            Object getKey() {
                return key;
            }

            boolean isDoublon() {
                return isDoublon;
            }

            public int getRealIndex() {
                return realIndex;
            }

            boolean cleanField() {
                return cleanField;
            }

            String getSeparator() {
                return separator;
            }

        }

    }

}
