/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.croisement.*;
import fr.exemole.bdfserver.request.parsers.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class MassIndexationFicheCommand extends AbstractMassCommand {

    public final static String[] paramArray = {COUPLESEPARATEUR_PARAMNAME, CROISEMENTUPDATE_PARAMNAME, OTHERTHESAURUS_PARAMNAME, FUSIONDOUBLON_PARAMNAME};
    public final static String[] executeParamArray = {CROISEMENTUPDATE_PARAMNAME, OTHERTHESAURUS_PARAMNAME};

    public MassIndexationFicheCommand() {
    }

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public boolean needCorpus() {
        return true;
    }

    public static String getCommandString() {
        return "MassIndexationFiche";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSINDEXATIONFICHEFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSINDEXATIONFICHECONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(corpus, coupleSeparateur, fusionDoublon, otherThesaurus, workingLangInteger);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        CroisementEditor croisementsEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        int count = 0;
        for (Iterator it = requestMap.getParameterNameSet().iterator(); it.hasNext();) {
            String name = (String) it.next();
            try {
                int idcorpus = Integer.parseInt(name);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    DefaultMotcleCroisementList list = null;
                    if (requestMap.getParameter(name).length() == 0) {
                        list = new DefaultMotcleCroisementList(otherThesaurus.getSubsetKey(), 0);
                    } else {
                        String[] values = requestMap.getParameterValues(name);
                        list = new DefaultMotcleCroisementList(otherThesaurus.getSubsetKey(), values.length);
                        for (int i = 0; i < values.length; i++) {
                            PoidsIntegerToken poidsIntegerToken = PoidsIntegerToken.parse(values[i]);
                            if (poidsIntegerToken != null) {
                                Motcle motcle = otherThesaurus.getMotcleByIdths(poidsIntegerToken.getInteger());
                                if (motcle != null) {
                                    list.add(motcle, poidsIntegerToken.getPoids());
                                }
                            }
                        }
                    }
                    count++;
                    croisementsEditor.updateCroisement(abregeFiche, list, updateType);
                }
            } catch (NumberFormatException nfe) {
            }
        }


        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_indexationfiche", new Integer(count));
        }
    }


    public static class Parser extends CoupleCorpusParser {

        Thesaurus indexationThesaurus;
        boolean withIdalpha;
        private int thesaurusLangInteger;
        boolean fusionDoublon;

        Parser(Corpus corpus, char coupleSeparateur, boolean fusionDoublon, Thesaurus indexationThesaurus, int workingLangInteger) {
            super(corpus, coupleSeparateur);
            this.fusionDoublon = fusionDoublon;
            this.indexationThesaurus = indexationThesaurus;
            withIdalpha = ThesaurusUtils.isIdalphaType(indexationThesaurus);
            thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(indexationThesaurus, workingLangInteger);
        }

        protected void parseValue(AbregeFiche abregeFiche, String value, int lineNumber) {
            String[] idtokens = StringUtils.getCleanedTokens(value, ';');
            int length = idtokens.length;
            DefaultMotcleCroisementList motcleList = new DefaultMotcleCroisementList(indexationThesaurus.getSubsetKey(), length);
            boolean error = false;
            for (int i = 0; i < length; i++) {
                PoidsMotcleToken poidsIntegerToken = PoidsMotcleToken.parse(idtokens[i], withIdalpha);
                if (poidsIntegerToken == null) {
                    parseResult.addParseError(idtokens[i], lineNumber);
                    error = true;
                } else {
                    Motcle motcle = PoidsMotcleToken.getMotcle(poidsIntegerToken, indexationThesaurus, thesaurusLangInteger);
                    if (motcle == null) {
                        if (poidsIntegerToken.isIdthsBundle()) {
                            parseResult.addBdfError("UNKNOWN INDEXATION IDTHS", String.valueOf(poidsIntegerToken.getIdths()), lineNumber);
                        } else if (withIdalpha) {
                            parseResult.addBdfError("UNKNOWN INDEXATION IDALPHA", poidsIntegerToken.getText(), lineNumber);
                        } else {
                            parseResult.addBdfError("UNKNOWN INDEXATION LIB", poidsIntegerToken.getText(), lineNumber);
                        }
                        error = true;
                    } else {
                        motcleList.add(motcle, poidsIntegerToken.getPoids());
                    }
                }
            }
            if (!error) {
                int id = abregeFiche.getIdcorpus();
                if (parseResult.containsId(id)) {
                    if (fusionDoublon) {
                        ParseResult.ResultItem resultItem = parseResult.getResultItem(id);
                        DefaultMotcleCroisementList currentMotcleList = (DefaultMotcleCroisementList) resultItem.getValueObject();
                        currentMotcleList.append(motcleList);
                    } else {
                        parseResult.checkDoublon(id, lineNumber);
                    }
                } else {
                    parseResult.addResult(id, abregeFiche, motcleList, lineNumber);
                }
            }
        }

    }

}
