/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class MassLibCommand extends AbstractMassCommand {

    public final static String[] paramArray = {LANG_PARAMNAME, COUPLESEPARATEUR_PARAMNAME, TYPEID_PARAMNAME};
    public final static String[] executeParamArray = {LANG_PARAMNAME};
    public static String VONEW_PARAMNAME = "vonew";

    public MassLibCommand() {
    }

    @Override
    public boolean needThesaurus() {
        return true;
    }

    @Override
    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    @Override
    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public static String getCommandString() {
        return "MassLib";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSLIBFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSLIBCONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(thesaurus, typeId, coupleSeparateur);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        ThesaurusEditor thesaurusEditor = bdfServer.getBasedefichesEditor().getThesaurusEditor(thesaurus.getSubsetKey());
        boolean done = false;
        TextParser textParser = new TextParser();
        TypoOptions defaultTypoOptions = null;
        if (langInteger != null) {
            defaultTypoOptions = TypoOptions.getTypoOptions(langInteger.intValue());
            textParser.setQuoteOptions(defaultTypoOptions.getQuoteOptions());
        }
        for (Iterator it = requestMap.getParameterNameSet().iterator(); it.hasNext();) {
            String name = (String) it.next();
            try {
                int idths = Integer.parseInt(name);
                String value = requestMap.getParameter(name);
                Motcle motcle = thesaurus.getMotcleByIdths(idths);
                if (motcle != null) {
                    int li = -1;
                    if (langInteger != null) {
                        li = langInteger.intValue();
                    } else {
                        Libelle libelle = motcle.getLibelleByIndex(0);
                        li = libelle.getLangInteger();
                        textParser.setQuoteOptions(TypoOptions.getTypoOptions(li).getQuoteOptions());
                    }
                    thesaurusEditor.setLibelle(motcle, textParser.parseString(value), li);
                    done = true;
                }
            } catch (NumberFormatException nfe) {
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (done) {
            responseParameters.setDoneMessage("mass_done_lib", null);
        }
    }

    protected boolean initSupplementaryParseParameters(ImportationResponseParameters saisieResponseParameters) throws ParameterException {
        return true;
    }

    public static void checkExistingParameters(ImportationResponseParameters parameters, RequestMap requestMap) throws ParameterException {
    }


    private static class Parser extends CoupleThesaurusParser {

        Parser(Thesaurus thesaurus, String parseType, char coupleSeparateur) {
            super(thesaurus, parseType, coupleSeparateur);
        }

        protected void parseValue(Motcle motcle, String value, int lineNumber, boolean fromIdalpha) {
            addResultWithNoDoublon(motcle.getIdths(), motcle, value, lineNumber);
        }

        protected void endParse() {
        }

    }

}
