/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.text.tableparser.*;
import java.text.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class MassMotcleCreationCommand extends AbstractMassCommand {

    private String[] fieldsArray;
    private TableParser.Parameters tableParserParameters;

    public MassMotcleCreationCommand() {
    }

    public boolean needThesaurus() {
        return true;
    }

    public static String getCommandString() {
        return "MassMotcleCreation";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSMOTCLECREATIONCONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(thesaurus, fieldsArray);
        parser.parse(text, tableParserParameters);
        return parser.getParseResult();
    }

    protected boolean initSupplementaryParseParameters(ImportationResponseParameters saisieResponseParameters) throws ParameterException {
        tableParserParameters = getTableParserParameters(saisieResponseParameters);
        String[] fieldsArray = getFieldsArray(saisieResponseParameters);
        int length = fieldsArray.length;
        if (length == 0) {
            saisieResponseParameters.setErrorMessage("saisie_err_fields_empty", null);
            return false;
        }
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            boolean here = false;
            for (int i = 0; i < length; i++) {
                if (fieldsArray[i].equals("idalpha")) {
                    here = true;
                    break;
                }
            }
            if (!here) {
                saisieResponseParameters.setErrorMessage("saisie_err_fields_idalphamissing", null);
                return false;
            }
        }
        List wrongList = new ArrayList();
        List doublonList = new ArrayList();
        String[] validFields = getValidFieldsArray(thesaurus);
        for (int i = 0; i < length; i++) {
            String field = fieldsArray[i];
            if (field.equals("idths") || field.equals("parent_idths")) {
                boolean doublon = false;
                for (int k = i + 1; k < length; k++) {
                    if (fieldsArray[k].equals(field)) {
                        doublon = true;
                        break;
                    }
                }
                if (doublon) {
                    doublonList.add(field);
                    continue;
                }
            }
            boolean valid = false;
            for (int k = 0; k < validFields.length; k++) {
                if (validFields[k].equals(field)) {
                    valid = true;
                    break;
                }
            }
            if (!valid) {
                wrongList.add(field);
            }
        }
        if ((!doublonList.isEmpty()) || (!wrongList.isEmpty())) {
            List errList = new ArrayList();
            String wrong = listToString(wrongList);
            if (wrong != null) {
                errList.add(new ResourceItem("saisie_err_fields_wrong", wrong));
            }
            String doublon = listToString(doublonList);
            if (doublon != null) {
                errList.add(new ResourceItem("saisie_err_fields_doublon", doublon));
            }
            saisieResponseParameters.setMultiErrorMessage("saisie_err_fields", errList);
            return false;
        }
        this.fieldsArray = fieldsArray;
        return true;
    }

    protected void doExecuteStep() throws ParameterException {
        ParameterExceptionLog parameterExceptionLog = new ParameterExceptionLog();
        int count = 0;
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            count = doIdalphaExecuteStep(parameterExceptionLog);
        } else if (ThesaurusUtils.isBabelienType(thesaurus)) {
            count = doBabelienExecuteStep(parameterExceptionLog);
        } else {
            count = doMultiExecuteStep(parameterExceptionLog);
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_motclecreation", new Integer(count));
        }
    }

    protected int doIdalphaExecuteStep(ParameterExceptionLog parameterExceptionLog) throws ParameterException {
        BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
        ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        int[] langArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
        int langLength = langArray.length;
        int p = 0;
        int count = 0;
        TextParser textParser = new TextParser();
        TypoOptions[] typoArray = new TypoOptions[langLength];
        for (int i = 0; i < langLength; i++) {
            typoArray[i] = TypoOptions.getTypoOptions(langArray[i]);
        }
        while (true) {
            String prefix = String.valueOf(p) + "_";
            String param = prefix + "idalpha";
            String idalpha = requestMap.getParameter(param);
            if (idalpha == null) {
                break;
            }
            p++;
            Motcle motcle = null;
            try {
                motcle = thesaurusEditor.createMotcle(-1, idalpha);
            } catch (ExistingIdException e) {
                parameterExceptionLog.addParameterException("ExistingIdException", param, idalpha);
                continue;
            } catch (ParseException e) {
                parameterExceptionLog.addParameterException("ParseException", param, idalpha);
                continue;
            } catch (IllegalArgumentException e) {
                parameterExceptionLog.addParameterException("IllegalArgumentException", param, idalpha);
                continue;
            }
            count++;
            String parentParamName = prefix + "parent_idalpha";
            String parentIdalpha = requestMap.getParameter(parentParamName);
            if (parentIdalpha != null) {
                Motcle parent = thesaurus.getMotcleByIdalpha(parentIdalpha);
                if (parent == null) {
                    parameterExceptionLog.addParameterException("unknwonParent", parentParamName, parentIdalpha);
                } else {
                    try {
                        thesaurusEditor.setParent(motcle, parent);
                    } catch (ParentRecursivityException pre) {
                        parameterExceptionLog.addParameterException("ParentRecursivityException", parentParamName, parentIdalpha);
                    }
                }
            }
            for (int i = 0; i < langLength; i++) {
                String paramName = prefix + "lib_" + LangInteger.toString(langArray[i]);
                String value = requestMap.getParameter(paramName);
                if (value != null) {
                    textParser.setQuoteOptions(typoArray[i].getQuoteOptions());
                    thesaurusEditor.setLibelle(motcle, textParser.parseString(value), langArray[i]);
                }
            }
        }
        return count;
    }

    protected int doMultiExecuteStep(ParameterExceptionLog parameterExceptionLog) throws ParameterException {
        BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
        ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        int[] langArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
        int langLength = langArray.length;
        int p = 0;
        int count = 0;
        TextParser textParser = new TextParser();
        TypoOptions[] typoArray = new TypoOptions[langLength];
        for (int i = 0; i < langLength; i++) {
            typoArray[i] = TypoOptions.getTypoOptions(langArray[i]);
        }
        while (true) {
            String prefix = String.valueOf(p) + "_";
            String idthsString = requestMap.getParameter(prefix + "idths");
            String idthsParentString = requestMap.getParameter(prefix + "parent_idths");
            if ((idthsString == null) && (idthsParentString == null)) {
                boolean here = false;
                for (int i = 0; i < langLength; i++) {
                    String paramName = prefix + "lib_" + LangInteger.toString(langArray[i]);
                    String value = requestMap.getParameter(paramName);
                    if (value != null) {
                        here = true;
                        break;
                    }
                }
                if (!here) {
                    break;
                }
            }
            p++;
            Motcle motcle = getMotcle(prefix, thesaurusEditor, parameterExceptionLog);
            if (motcle == null) {
                continue;
            }
            count++;
            for (int i = 0; i < langLength; i++) {
                String paramName = prefix + "lib_" + LangInteger.toString(langArray[i]);
                String value = requestMap.getParameter(paramName);
                if (value != null) {
                    textParser.setQuoteOptions(typoArray[i].getQuoteOptions());
                    thesaurusEditor.setLibelle(motcle, textParser.parseString(value), langArray[i]);
                }
            }
        }
        return count;
    }

    protected int doBabelienExecuteStep(ParameterExceptionLog parameterExceptionLog) throws ParameterException {
        BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
        ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        int p = 0;
        int count = 0;
        TextParser textParser = new TextParser();
        TypoOptions typoOptions = TypoOptions.getTypoOptions(workingLangInteger);
        textParser.setQuoteOptions(typoOptions.getQuoteOptions());
        while (true) {
            String prefix = String.valueOf(p) + "_";
            String idthsString = requestMap.getParameter(prefix + "idths");
            String idthsParentString = requestMap.getParameter(prefix + "parent_idths");
            String lib = requestMap.getParameter(prefix + "lib");
            if ((idthsString == null) && (idthsParentString == null) && (lib == null)) {
                break;
            }
            p++;
            Motcle motcle = getMotcle(prefix, thesaurusEditor, parameterExceptionLog);
            if (motcle == null) {
                continue;
            }
            count++;
            if (lib != null) {
                thesaurusEditor.setLibelle(motcle, textParser.parseString(lib), workingLangInteger);
            }
        }
        return count;
    }

    private Motcle getMotcle(String prefix, ThesaurusEditor thesaurusEditor, ParameterExceptionLog parameterExceptionLog) {
        String paramIdths = prefix + "idths";
        String paramParent = prefix + "parent_idths";
        String idthsString = requestMap.getParameter(paramIdths);
        String idthsParentString = requestMap.getParameter(paramParent);
        int idths = -1;
        if (idthsString != null) {
            try {
                idths = Integer.parseInt(idthsString);
            } catch (NumberFormatException nfe) {
                parameterExceptionLog.addParameterException("NumberFormatException", paramIdths, idthsString);
                return null;
            }
        }
        Motcle motcle = null;
        try {
            motcle = thesaurusEditor.createMotcle(idths, null);
        } catch (ExistingIdException e) {
            parameterExceptionLog.addParameterException("ExistingIdException", paramIdths, idthsString);
            return null;
        } catch (ParseException e) {
            throw new ImplementationException(e);
        }
        if (idthsParentString != null) {
            try {
                int idthsParent = Integer.parseInt(idthsParentString);
                Motcle parent = thesaurus.getMotcleByIdths(idthsParent);
                if (parent == null) {
                    parameterExceptionLog.addParameterException("unknwonParent", paramParent, idthsParentString);
                } else {
                    try {
                        thesaurusEditor.setParent(motcle, parent);
                    } catch (ParentRecursivityException pre) {
                        parameterExceptionLog.addParameterException("ParentRecursivityException", paramParent, idthsParentString);
                    }
                }
            } catch (NumberFormatException nfe) {
                parameterExceptionLog.addParameterException("NumberFormatException", paramParent, idthsParentString);
            }
        }
        return motcle;
    }

    public static void checkExistingParameters(ImportationResponseParameters parameters, RequestMap requestMap) throws ParameterException {
        checkTableParserParameters(parameters, requestMap);
        checkFieldsParameters(parameters, requestMap);
    }

    public static String[] getValidFieldsArray(Thesaurus thesaurus) {
        if (ThesaurusUtils.isBabelienType(thesaurus)) {
            String[] result = {"idths", "parent_idths", "lib"};
            return result;
        }
        int[] langArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
        String[] result = new String[langArray.length + 2];
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            result[0] = "idalpha";
            result[1] = "parent_idalpha";
        } else {
            result[0] = "idths";
            result[1] = "parent_idths";
        }
        for (int i = 0; i < langArray.length; i++) {
            result[i + 2] = "lib_" + LangInteger.toString(langArray[i]);
        }
        return result;
    }


    private static class Parser {

        ParseResult parseResult = new ParseResult(ParseResult.LINES);
        Thesaurus thesaurus;
        String[] fieldsArray;
        LineTest lineTest;
        HashSet idSet = new HashSet();

        Parser(Thesaurus thesaurus, String[] fieldsArray) {
            this.thesaurus = thesaurus;
            this.fieldsArray = fieldsArray;
            if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                lineTest = new IdalphaTest();
            } else if (ThesaurusUtils.isBabelienType(thesaurus)) {
                lineTest = new BabelienTest();
            } else {
                lineTest = new MultiTest();
            }
        }

        ParseResult getParseResult() {
            return parseResult;
        }

        private void parse(String s, TableParser.Parameters tableParserParameters) {
            String[][] lines = TableParser.parse(s, tableParserParameters);
            for (int i = 0; i < lines.length; i++) {
                String[] line = lines[i];
                if (line.length != 0) {
                    lineTest.reinit(line, i + 1);
                    lineTest.parse();
                }
            }
        }


        private abstract class LineTest {

            int lineNumber;
            String[] tokens;
            Object parentId;
            Object objectId;

            void reinit(String[] line, int lineNumber) {
                this.lineNumber = lineNumber;
                tokens = line;
            }

            abstract void parse();

            boolean testLib(String field, String value, LinkedIntHashMap libMap) {
                if (!field.startsWith("lib_")) {
                    return false;
                }
                try {
                    int langInteger = LangInteger.parse(field.substring(4));
                    String exist = (String) libMap.get(langInteger);
                    if (exist == null) {
                        exist = value;
                    } else {
                        exist = exist + value;
                    }
                    libMap.put(langInteger, exist);
                } catch (LangIntegerException lie) {
                    throw new IllegalStateException("should not occur", lie);
                }
                return true;
            }

            boolean testParentIdths(String value) {
                if (value.length() == 0) {
                    parentId = null;
                    return true;
                }
                try {
                    int parentIdths = Integer.parseInt(value);
                    if (parentIdths >= 0) {
                        if ((thesaurus.getMotcleByIdths(parentIdths) == null) && (!idSet.contains(new Integer(parentIdths)))) {
                            parseResult.addBdfError("UNKNOWN PARENT IDTHS", value, lineNumber);
                            return false;
                        } else {
                            parentId = new Integer(parentIdths);
                        }
                    } else {
                        parentId = null;
                    }
                    return true;
                } catch (NumberFormatException nfe) {
                    parseResult.addBdfError("UNKNOWN PARENT IDTHS", value, lineNumber);
                    return false;
                }
            }

            boolean testIdths(String value) {
                try {
                    int idths = Integer.parseInt(value);
                    if (idths >= 0) {
                        Motcle current = thesaurus.getMotcleByIdths(idths);
                        if (current != null) {
                            parseResult.addBdfError("EXISTING IDTHS", value, lineNumber);
                            return false;
                        } else {
                            objectId = new Integer(idths);
                        }
                    }
                    return true;
                } catch (NumberFormatException nfe) {
                    parseResult.addParseError(value, lineNumber);
                    return false;
                }
            }

            void cleanLibMap(LinkedIntHashMap libMap) {
                int size = libMap.size();
                int[] keys = new int[size];
                for (int i = 0; i < size; i++) {
                    keys[i] = libMap.getKeyAt(i);
                }
                for (int i = 0; i < size; i++) {
                    int key = keys[i];
                    String value = (String) libMap.get(key);
                    value = StringUtils.cleanString(value);
                    if (value.length() == 0) {
                        libMap.remove(key);
                    } else {
                        libMap.put(key, value);
                    }
                }
            }

        }


        private class IdalphaTest extends LineTest {

            void parse() {
                String idalpha = "";
                String parentIdalpha = "";
                LinkedIntHashMap libMap = new LinkedIntHashMap();
                int length = Math.min(tokens.length, fieldsArray.length);
                boolean done = true;
                for (int i = 0; i < length; i++) {
                    String value = tokens[i];
                    String field = fieldsArray[i];
                    if (field.equals("idalpha")) {
                        idalpha = idalpha + value;
                    } else if (field.equals("parent_idalpha")) {
                        parentIdalpha = parentIdalpha + value;
                    } else {
                        testLib(field, value, libMap);
                    }
                }
                idalpha = StringUtils.cleanString(idalpha);
                parentIdalpha = StringUtils.cleanString(parentIdalpha);
                if (idalpha.length() == 0) {
                    parseResult.addBdfError("MISSING IDALPHA", "", lineNumber);
                } else if (testIdalpha(idalpha)) {
                    if (parentIdalpha.length() == 0) {
                        parentIdalpha = null;
                    } else if ((thesaurus.getMotcleByIdalpha(parentIdalpha) == null) && (!idSet.contains(parentIdalpha))) {
                        parseResult.addBdfError("UNKNOWN PARENT IDALPHA", parentIdalpha, lineNumber);
                        return;
                    }
                    if (idalpha == null) {
                        parseResult.addBdfError("MISSING IDALPHA", "", lineNumber);
                    } else {
                        idSet.add(idalpha);
                        CreationValue creationValue = new CreationValue(idalpha, parentIdalpha);
                        cleanLibMap(libMap);
                        creationValue.setLibMap(libMap);
                        parseResult.addResult(lineNumber, null, creationValue, lineNumber);
                    }
                }
            }

            boolean testIdalpha(String value) {
                if (!ThesaurusUtils.isValidIdalpha(value)) {
                    parseResult.addParseError(value, lineNumber);
                    return false;
                }
                if (thesaurus.getMotcleByIdalpha(value) != null) {
                    parseResult.addBdfError("EXISTING IDALPHA", value, lineNumber);
                    return false;
                }
                if (idSet.contains(value)) {
                    parseResult.addBdfError("DOUBLON IDALPHA", value, lineNumber);
                    return false;
                }
                return true;
            }

        }


        private class MultiTest extends LineTest {

            void parse() {
                objectId = null;
                parentId = null;
                LinkedIntHashMap libMap = new LinkedIntHashMap();
                int length = Math.min(tokens.length, fieldsArray.length);
                boolean done = true;
                for (int i = 0; i < length; i++) {
                    String value = tokens[i];
                    String field = fieldsArray[i];
                    if (field.equals("idths")) {
                        value = StringUtils.cleanString(value);
                        done = testIdths(value);
                        if (!done) {
                            break;
                        }
                    } else if (field.equals("parent_idths")) {
                        value = StringUtils.cleanString(value);
                        done = testParentIdths(value);
                        if (!done) {
                            break;
                        }
                    } else {
                        testLib(field, value, libMap);
                    }
                }
                if (done) {
                    int idths = -1;
                    if (objectId != null) {
                        idSet.add(objectId);
                        idths = ((Integer) objectId).intValue();
                    }
                    CreationValue creationValue = new CreationValue(objectId, parentId);
                    cleanLibMap(libMap);
                    creationValue.setLibMap(libMap);
                    parseResult.addResult(lineNumber, null, creationValue, lineNumber);
                }
            }

        }


        private class BabelienTest extends LineTest {

            void parse() {
                objectId = null;
                parentId = null;
                String lib = "";
                int length = Math.min(tokens.length, fieldsArray.length);
                boolean done = true;
                for (int i = 0; i < length; i++) {
                    String value = tokens[i];
                    String field = fieldsArray[i];
                    if (field.equals("idths")) {
                        value = StringUtils.cleanString(value);
                        done = testIdths(value);
                        if (!done) {
                            break;
                        }
                    } else if (field.equals("parent_idths")) {
                        value = StringUtils.cleanString(value);
                        done = testParentIdths(value);
                        if (!done) {
                            break;
                        }
                    } else if (field.equals("lib")) {
                        lib = lib + value;
                    }
                }
                if (done) {
                    lib = StringUtils.cleanString(lib);
                    int idths = -1;
                    if (objectId != null) {
                        idSet.add(objectId);
                        idths = ((Integer) objectId).intValue();
                    }
                    CreationValue creationValue = new CreationValue(objectId, parentId);
                    creationValue.setLib(lib);
                    parseResult.addResult(lineNumber, null, creationValue, lineNumber);
                }
            }

        }

    }


    public static class CreationValue {

        Object idObject;
        Object parentIdObject;
        LinkedIntHashMap libMap;
        String lib;

        CreationValue(Object idObject, Object parentIdObject) {
            this.idObject = idObject;
            this.parentIdObject = parentIdObject;
        }

        public Object getIdObject() {
            return idObject;
        }

        public Object getParentIdObject() {
            return parentIdObject;
        }

        void setLibMap(LinkedIntHashMap libMap) {
            this.libMap = libMap;
        }

        public LinkedIntHashMap getLibMap() {
            return libMap;
        }

        void setLib(String lib) {
            this.lib = lib;
        }

        public String getLib() {
            return lib;
        }

    }

}
