/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class MassMotcleMergeCommand extends AbstractMassCommand {

    public final static String[] paramArray = {COUPLESEPARATEUR_PARAMNAME, TYPEID_PARAMNAME, OTHERTHESAURUS_PARAMNAME};
    public final static String[] executeParamArray = {OTHERTHESAURUS_PARAMNAME};
    public final static String MERGELIST_PARAMNAME = "mergelist";

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public MassMotcleMergeCommand() {
    }

    public boolean needThesaurus() {
        return true;
    }

    public static String getCommandString() {
        return "MassMotcleMerge";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSMOTCLEMERGEFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSMOTCLEMERGECONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(thesaurus, typeId, coupleSeparateur, otherThesaurus);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        String[] idthsList = requestMap.getParameterValues(MERGELIST_PARAMNAME);
        if (idthsList == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, MERGELIST_PARAMNAME);
        }
        BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
        int count = 0;
        for (int i = 0; i < idthsList.length; i++) {
            String s = idthsList[i];
            int idx = s.indexOf('=');
            if (idx == -1) {
                continue;
            }
            try {
                int idths1 = Integer.parseInt(s.substring(0, idx));
                int idths2 = Integer.parseInt(s.substring(idx + 1));
                Motcle motcle1 = thesaurus.getMotcleByIdths(idths1);
                Motcle motcle2 = otherThesaurus.getMotcleByIdths(idths2);
                if ((motcle1 != null) && (motcle2 != null)) {
                    try {
                        basedefichesEditor.merge(motcle1, motcle2);
                        count++;
                    } catch (ParentRecursivityException pre) {
                    }
                }
            } catch (NumberFormatException nfe) {
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_motclemerge", new Integer(count));
        }
    }


    private static class Parser extends CoupleThesaurusParser {

        Thesaurus thesaurusDestination;

        Parser(Thesaurus thesaurus, String parseType, char coupleSeparateur, Thesaurus thesaurusDestination) {
            super(thesaurus, parseType, coupleSeparateur);
            this.thesaurusDestination = thesaurusDestination;
        }

        protected void parseValue(Motcle motcle, String value, int lineNumber, boolean fromIdalpha) {
            value = value.trim();
            boolean secondIdalpha = ThesaurusUtils.isIdalphaType(thesaurusDestination);
            if ((fromIdalpha) && (!secondIdalpha)) {
                fromIdalpha = false;
            }
            if (fromIdalpha) {
                Motcle motcle2 = thesaurusDestination.getMotcleByIdalpha(value);
                if (motcle2 == null) {
                    parseResult.addBdfError("UNKNOWN IDALPHA 2", value, lineNumber);
                } else {
                    finalTest(motcle, motcle2, lineNumber, true);
                }
            } else {
                try {
                    int idths2 = Integer.parseInt(value);
                    Motcle motcle2 = thesaurusDestination.getMotcleByIdths(idths2);
                    if (motcle2 == null) {
                        parseResult.addBdfError("UNKNOWN IDTHS 2", value, lineNumber);
                    } else {
                        finalTest(motcle, motcle2, lineNumber, false);
                    }
                } catch (NumberFormatException nfe) {
                    parseResult.addParseError(value, lineNumber);
                }
            }
        }

        private void finalTest(Motcle motcle1, Motcle motcle2, int lineNumber, boolean fromIdalpha) {
            if (thesaurus.equals(thesaurusDestination)) {
                if (motcle1.equals(motcle2)) {
                    if (fromIdalpha) {
                        parseResult.addBdfError("SAME IDALPHA", motcle1.getIdalpha(), lineNumber);
                    } else {
                        parseResult.addBdfError("SAME IDTHS", String.valueOf(motcle1.getIdths()), lineNumber);
                    }

                } else if (ThesaurusUtils.isDescendant(motcle2, motcle1)) {
                    if (fromIdalpha) {
                        parseResult.addBdfError("DESCENDANT IDALPHA", motcle2.getIdalpha() + " / " + motcle1.getIdalpha(), lineNumber);
                    } else {
                        parseResult.addBdfError("DESCENDANT IDTHS", motcle2.getIdths() + " / " + motcle1.getIdths(), lineNumber);
                    }
                } else {
                    addResultWithNoDoublon(motcle1.getIdths(), motcle1, motcle2, lineNumber);
                }
            } else {
                addResultWithNoDoublon(motcle1.getIdths(), motcle1, motcle2, lineNumber);
            }
        }

        protected void endParse() {
            if (!thesaurus.equals(thesaurusDestination)) {
                return;
            }
            List removeIdList = new ArrayList();
            List list = parseResult.getResultItemList();
            for (Iterator it = list.iterator(); it.hasNext();) {
                ParseResult.ResultItem resultItem = (ParseResult.ResultItem) it.next();
                Motcle key = (Motcle) resultItem.getKeyObject();
                ParseResult.ResultItem potentiel = nextResultItem(resultItem);
                if (potentiel != null) {
                    HashSet existant = new HashSet();
                    while (true) {
                        Motcle lastValue = (Motcle) potentiel.getValueObject();
                        if (existant.contains(lastValue)) {
                            String type = (existant.contains(key)) ? "LOOP" : "GOTO LOOP";
                            String val = key.getIdalpha();
                            if (val == null) {
                                val = String.valueOf(key.getIdths());
                            }
                            parseResult.addBdfError(type, val, resultItem.getLineNumber());
                            removeIdList.add(new Integer(key.getIdths()));
                            break;
                        }
                        existant.add(potentiel.getKeyObject());
                        potentiel = nextResultItem(potentiel);
                        if (potentiel == null) {
                            if (ThesaurusUtils.isDescendant(lastValue, key)) {
                                String val = key.getIdalpha();
                                if (val == null) {
                                    val = String.valueOf(key.getIdths());
                                }
                                parseResult.addBdfError("GOTO DESCENDANT", val, resultItem.getLineNumber());
                                removeIdList.add(new Integer(key.getIdths()));
                            } else {
                                resultItem.setValueObject(lastValue);
                            }
                            break;
                        }
                    }
                }
            }
            for (Iterator it = removeIdList.iterator(); it.hasNext();) {
                Integer itg = (Integer) it.next();
                parseResult.removeResultItem(itg.intValue());
            }
        }

        private ParseResult.ResultItem nextResultItem(ParseResult.ResultItem resultItem) {
            int id = ((Motcle) resultItem.getValueObject()).getIdths();
            return parseResult.getResultItem(id);
        }

    }

}
