/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import net.mapeadores.basedefiches.Intitule;
import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.coupleparser.*;
import net.mapeadores.basedefiches.*;
import java.text.*;
import fr.exemole.bdfserver.ui.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class MassUICommand extends AbstractMassCommand {

    public final static String[] paramArray = {LANG_PARAMNAME, COUPLESEPARATEUR_PARAMNAME};
    public final static String[] executeParamArray = {LANG_PARAMNAME};

    public MassUICommand() {
    }

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public static String getCommandString() {
        return "MassUI";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSUIFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSUICONFIRM_PAGE;
    }

    protected void doExecuteStep() throws ParameterException {
        boolean done = false;
        LibBundle libBundle = new LibBundle(bdfServer, langInteger.intValue());
        for (Iterator it = requestMap.getParameterNameSet().iterator(); it.hasNext();) {
            String name = (String) it.next();
            if (!name.startsWith("key_")) {
                continue;
            }
            String key = name.substring(4);
            String value = requestMap.getParameter(name);
            libBundle.addCouple(key, value);
        }
        done = libBundle.update();
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (done) {
            bdfServer.saveUIManager();
            responseParameters.setDoneMessage("mass_done_lib", null);
        }
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(bdfServer, langInteger.intValue(), coupleSeparateur);
        parser.parse(text);
        return parser.getParseResult();
    }


    private static class Parser {

        LibBundle libBundle;
        ParseResult parseResult = new ParseResult(ParseResult.LINES);
        private CoupleParser coupleParser;
        Basedefiches basedefiches;

        Parser(BdfServer bdfServer, int langInteger, char coupleSeparateur) {
            coupleParser = new CoupleParser(CoupleParser.IDALPHA_MODE);
            coupleParser.setCoupleSeparateur(coupleSeparateur);
            libBundle = new LibBundle(bdfServer, langInteger);
        }

        private void parse(String text) {
            Couple[] coupleArray = coupleParser.parse(text);
            int length = coupleArray.length;
            for (int i = 0; i < length; i++) {
                parseIdalphaCouple(coupleArray[i]);
            }
        }

        private void parseIdalphaCouple(Couple couple) {
            if (couple instanceof ErrorCouple) {
                parseResult.addParseError(couple.getValue(), couple.getLineNumber());
                return;
            }
            IdalphaCouple idalphaCouple = (IdalphaCouple) couple;
            String idalpha = idalphaCouple.getIdalpha();
            int lineNumber = couple.getLineNumber();
            int currentId = libBundle.getId(idalpha);
            if (currentId != -1) {
                parseResult.checkDoublon(currentId, lineNumber);
                return;
            }
            String value = idalphaCouple.getValue();
            int result = libBundle.addCouple(idalpha, value);
            if (result < 0) {
                switch (result) {
                    case -1:
                        parseResult.addBdfError("MALFORMED KEY", idalpha, lineNumber);
                        break;
                    case -2:
                        parseResult.addBdfError("MISSING KEY FIRST PART", idalpha, lineNumber);
                        break;
                    case -3:
                        parseResult.addBdfError("MISSING KEY SECOND PART", idalpha, lineNumber);
                        break;
                    case - 4:
                        parseResult.addBdfError("UNKNOWN KEY FIRST PART", idalpha, lineNumber);
                        break;
                    case -5:
                        parseResult.addBdfError("NOT APPLICABLE SUBSET", idalpha, lineNumber);
                        break;
                    case -6:
                        parseResult.addBdfError("UNKNWON SUBSET", idalpha, lineNumber);
                        break;
                    case -7:
                        parseResult.addBdfError("NOT INTITULE NAME", idalpha, lineNumber);
                        break;
                    case -8:
                        parseResult.addBdfError("WRONG INTITULE NAME", idalpha, lineNumber);
                        break;
                    case -9:
                        parseResult.addBdfError("WRONG INTITULE NAME OR FIELDKEY", idalpha, lineNumber);
                        break;
                    case -10:
                        parseResult.addBdfError("NOT AVAILABLE FIELDKEY", idalpha, lineNumber);
                        break;
                    case -11:
                        parseResult.addBdfError("NOT AVAILABLE INCLUDEKEY", idalpha, lineNumber);
                        break;
                }
            } else {
                String[] valueArray = new String[2];
                valueArray[0] = value;
                valueArray[1] = libBundle.lastOldValue();
                parseResult.addResult(libBundle.lastId(), idalpha, valueArray, lineNumber);
            }

        }

        ParseResult getParseResult() {
            return parseResult;
        }

    }


    private static class LibBundle {

        String oldValue = "";
        HashMap keyIdMap = new HashMap();
        SubsetMap basedefichesMap;
        HashMap mapOfSubsetMap = new HashMap();
        int id = 0;
        BdfServer bdfServer;
        Basedefiches basedefiches;
        int langInteger;

        LibBundle(BdfServer bdfServer, int langInteger) {
            this.bdfServer = bdfServer;
            this.basedefiches = bdfServer.getBasedefiches();
            this.langInteger = langInteger;
            basedefichesMap = new SubsetMap(bdfServer, null);
        }

        int getId(String key) {
            Integer itg = (Integer) keyIdMap.get(key);
            if (itg == null) {
                return -1;
            }
            return itg.intValue();
        }

        int addCouple(String key, String value) {
            id++;
            keyIdMap.put(key, new Integer(id));
            oldValue = "";
            int idx = key.indexOf('/');
            if (idx < 0) {
                return -1;
            }
            String part1 = key.substring(0, idx);
            String part2 = key.substring(idx + 1);
            if (part1.length() == 0) {
                return -2;
            }
            if (part2.length() == 0) {
                return -3;
            }
            if (part1.equals("basedefiches")) {
                String intitule = getIntitule(part2);
                if (intitule == null) {
                    return -7;
                }
                oldValue = basedefichesMap.putIntituleValue(intitule, value);
            } else {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(part1);
                    if ((!subsetKey.isCorpusSubset()) && (!subsetKey.isThesaurusSubset()) && (!subsetKey.isSphereSubset())) {
                        return -5;
                    }
                    Subset subset = (Subset) basedefiches.getSubset(subsetKey);
                    if (subset == null) {
                        return -6;
                    }
                    SubsetMap subsetMap = getSubsetMap(subset);
                    String intitule = getIntitule(part2);
                    if (intitule != null) {
                        oldValue = subsetMap.putIntituleValue(intitule, value);
                    } else if (subsetKey.isCorpusSubset()) {
                        try {
                            FieldKey fieldKey = FieldKey.parse(part2);
                            oldValue = subsetMap.putFieldKey(fieldKey, value);
                            if (oldValue == null) {
                                return -10;
                            }
                        } catch (ParseException pe) {
                            try {
                                FilteredIncludeKey filIncKey = FilteredIncludeKey.parse(part2);
                                oldValue = subsetMap.putIncludeKey(filIncKey, value);
                                if (oldValue == null) {
                                    return -11;
                                }
                            } catch (ParseException pe2) {
                                return -9;
                            }
                        }
                    } else {
                        return -7;
                    }
                } catch (java.text.ParseException pe) {
                    return -4;
                }
            }
            return 1;
        }

        int lastId() {
            return id;
        }

        String lastOldValue() {
            return oldValue;
        }

        private String getIntitule(String s) {
            if (s.length() < 10) {
                return null;
            }
            if (!s.startsWith("intitule_")) {
                return null;
            }
            return s.substring(9);
        }

        private boolean update() {
            boolean done = false;
            boolean donet = basedefichesMap.update();
            if (donet) {
                done = true;
            }
            for (Iterator it = mapOfSubsetMap.values().iterator(); it.hasNext();) {
                SubsetMap subsetMap = (SubsetMap) it.next();
                donet = subsetMap.update();
                if (donet) {
                    done = true;
                }
            }
            return done;
        }

        private SubsetMap getSubsetMap(Subset subset) {
            SubsetMap subsetMap = (SubsetMap) mapOfSubsetMap.get(subset.getSubsetKey());
            if (subsetMap == null) {
                subsetMap = new SubsetMap(bdfServer, subset);
                mapOfSubsetMap.put(subset.getSubsetKey(), subsetMap);
            }
            return subsetMap;
        }


        class SubsetMap {

            Basedefiches basedefiches;
            BasedefichesEditor basedefichesEditor;
            Subset subset;
            Map intituleMap = new HashMap();
            List corpusFieldList = new ArrayList();
            List includeUIList = new ArrayList();
            Map keyMap = new HashMap();
            Metadata metadata;
            CorpusUI corpusUi;
            ComponentUIEditor componentUIEditor;

            SubsetMap(BdfServer bdfServer, Subset subset) {
                this.subset = subset;
                this.basedefiches = bdfServer.getBasedefiches();
                metadata = getMetadata();
                if ((subset != null) && (subset instanceof Corpus)) {
                    corpusUi = bdfServer.getUIManager().getCorpusUI((Corpus) subset);
                    componentUIEditor = corpusUi.getComponentUIEditor();
                }
            }

            boolean update() {
                basedefichesEditor = bdfServer.getBasedefichesEditor();
                boolean done = false;
                boolean done2 = false;
                done2 = updateMetadata();
                if (done2) {
                    done = true;
                }
                if (!corpusFieldList.isEmpty()) {
                    done2 = updateCorpusField();
                    if (done2) {
                        done = true;
                    }
                }
                if (!includeUIList.isEmpty()) {
                    done2 = updateIncludeUI();
                    if (done2) {
                        done = true;
                    }
                }
                return true;
            }

            boolean updateMetadata() {
                boolean done = false;
                boolean done2 = false;
                MetadataEditor metadataEditor = getMetadataEditor();
                for (Iterator it = intituleMap.entrySet().iterator(); it.hasNext();) {
                    Map.Entry mapEntry = (Map.Entry) it.next();
                    String name = (String) mapEntry.getKey();
                    String value = (String) mapEntry.getValue();
                    Intitule intitule = metadata.getIntituleByName(name);
                    if (intitule == null) {
                        intitule = metadataEditor.createIntitule(name);
                    }
                    done2 = metadataEditor.setIntituleLibelle(intitule, value, langInteger);
                    if (done2) {
                        done = true;
                    }
                }
                return done;
            }

            boolean updateCorpusField() {
                boolean done = false;
                boolean done2 = false;
                CorpusMetadataEditor cme = (CorpusMetadataEditor) getMetadataEditor();
                for (Iterator it = corpusFieldList.iterator(); it.hasNext();) {
                    Object[] array = (Object[]) it.next();
                    CorpusField corpusField = (CorpusField) array[0];
                    String value = (String) array[1];
                    done2 = cme.setFieldLibelle(corpusField, value, langInteger);
                    if (done2) {
                        done = true;
                    }
                }
                return done;
            }

            boolean updateIncludeUI() {
                boolean done = false;
                boolean done2 = false;
                for (Iterator it = includeUIList.iterator(); it.hasNext();) {
                    Object[] array = (Object[]) it.next();
                    IncludeUI includeUI = (IncludeUI) array[0];
                    String value = (String) array[1];
                    if (includeUI.useDefaultLibelle()) {
                        componentUIEditor.setDefaultLibelleUsage(includeUI, false);
                    }
                    done2 = componentUIEditor.setLibelleString(includeUI, langInteger, value);
                    if (done2) {
                        done = true;
                    }
                }
                return done;
            }

            String putIntituleValue(String intitule, String value) {
                intituleMap.put(intitule, value);
                String oldV = "";
                Intitule obj = metadata.getIntituleByName(intitule);
                if (obj != null) {
                    oldV = obj.getLibelleString(langInteger, false);
                }
                if (oldV == null) {
                    oldV = "";
                }
                return oldV;
            }

            String putFieldKey(FieldKey fieldKey, String value) {
                CorpusField corpusField = ((CorpusMetadata) metadata).getCorpusFieldByFieldKey(fieldKey);
                if (corpusField == null) {
                    return null;
                }
                Object[] array = {corpusField, value};
                corpusFieldList.add(array);
                String oldV = corpusField.getLibelleString(langInteger, false);
                if (oldV == null) {
                    return "";
                }
                return oldV;
            }

            String putIncludeKey(FilteredIncludeKey fik, String value) {
                IncludeUI includeUI = corpusUi.getIncludeUI(fik);
                if (includeUI == null) {
                    return null;
                }
                Object[] array = {includeUI, value};
                includeUIList.add(array);
                LibelleHolder libelleHolder = includeUI.getCustomLibelle();
                if (libelleHolder == null) {
                    return "";
                }
                String oldV = libelleHolder.getLibelleString(langInteger, false);
                if (oldV == null) {
                    return "";
                }
                return oldV;
            }

            MetadataEditor getMetadataEditor() {
                if (subset == null) {
                    return basedefichesEditor.getBasedefichesMetadataEditor();
                } else if (subset instanceof Thesaurus) {
                    return basedefichesEditor.getThesaurusEditor(subset.getSubsetKey()).getThesaurusMetadataEditor();
                } else if (subset instanceof Sphere) {
                    return basedefichesEditor.getSphereEditor(subset.getSubsetKey()).getSphereMetadataEditor();
                } else if (subset instanceof Corpus) {
                    return basedefichesEditor.getCorpusEditor(subset.getSubsetKey()).getCorpusMetadataEditor();
                } else {
                    return null;
                }
            }

            Metadata getMetadata() {
                if (subset == null) {
                    return basedefiches.getBasedefichesMetadata();
                } else if (subset instanceof Thesaurus) {
                    return ((Thesaurus) subset).getThesaurusMetadata();
                } else if (subset instanceof Sphere) {
                    return ((Sphere) subset).getSphereMetadata();
                } else if (subset instanceof Corpus) {
                    return ((Corpus) subset).getCorpusMetadata();
                } else {
                    return null;
                }
            }

        }

    }

}
