/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import java.util.*;
import net.mapeadores.util.collections.*;


/**
 *
 * @author Vincent Calame
 */
public class ParseResult {

    public final static String LINES = "lines";
    public final static String TOKENS = "tokens";
    private String type;
    List parseErrorList = new ArrayList();
    List bdfErrorList = new ArrayList();
    LinkedIntHashMap resultMap = new LinkedIntHashMap();

    public ParseResult(String type) {
        this.type = type;
    }

    public String getType() {
        return type;
    }

    public List getParseErrorList() {
        return parseErrorList;
    }

    public List getBdfErrorList() {
        return bdfErrorList;
    }

    public boolean hasOnlyParseError() {
        return (resultMap.size() == 0) && (bdfErrorList.size() == 0);
    }

    public List getResultItemList() {
        return resultMap.getValueList();
    }

    public ResultItem getResultItem(int id) {
        return (ResultItem) resultMap.get(id);
    }

    public void removeResultItem(int id) {
        resultMap.remove(id);
    }

    public boolean hasError() {
        return (bdfErrorList.size() != 0) || (parseErrorList.size() != 0);
    }

    void addParseError(String parseError, int lineNumber) {
        parseErrorList.add(new ParseError(parseError, lineNumber));
    }

    void addBdfError(String key, String text, int lineNumber) {
        bdfErrorList.add(new BdfError(key, text, lineNumber));
    }

    void addResult(int id, Object key, Object value, int lineNumber) {
        ResultItem resultItem = new ResultItem(key, value, lineNumber);
        resultMap.put(id, resultItem);
    }

    void checkDoublon(int id, int lineNumber) {
        ResultItem resultItem = (ResultItem) resultMap.get(id);
        addBdfError("DOUBLON", String.valueOf(resultItem.getLineNumber()), lineNumber);
    }

    boolean containsId(int id) {
        return resultMap.containsKey(id);
    }


    public static class ParseError {

        String rawText;
        int lineNumber;

        ParseError(String rawText, int lineNumber) {
            this.rawText = rawText;
            this.lineNumber = lineNumber;
        }

        public String getRawText() {
            return rawText;
        }

        public int getLineNumber() {
            return lineNumber;
        }

    }


    public static class BdfError {

        String key;
        String text;
        int lineNumber;

        BdfError(String key, String text, int lineNumber) {
            this.key = key;
            this.text = text;
            this.lineNumber = lineNumber;
        }

        public String getKey() {
            return key;
        }

        public String getText() {
            return text;
        }

        public int getLineNumber() {
            return lineNumber;
        }

    }


    public static class ResultItem {

        Object keyObject;
        Object valueObject;
        int lineNumber;

        ResultItem(Object keyObject, Object valueObject, int lineNumber) {
            this.keyObject = keyObject;
            this.valueObject = valueObject;
            this.lineNumber = lineNumber;
        }

        public Object getKeyObject() {
            return keyObject;
        }

        public Object getValueObject() {
            return valueObject;
        }

        public int getLineNumber() {
            return lineNumber;
        }

        public void setValueObject(Object obj) {
            this.valueObject = obj;
        }

    }

}
