/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.bdfserver.users.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFicheCreationFormPage extends ImportationHtmlPage {

    public MassFicheCreationFormPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        //this.htmlFormWriter = new HtmlFormWriter(BdfUserUtils.getFicheItemWriterParameters(bdfUser));
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massfichecreation");
        printMessage();
        startParseCorpusForm(MassFicheCreationCommand.getCommandString());
        printFields();
        printParseTypeRadiosRow();
        printTextArea();
        endParseForm();
        end();
    }

    protected void printPotentialFields() {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        append(FieldKey.IDCORPUS.toString());
        append("; ");
        append(FieldKey.LANG.toString());
        append("; ");
        append(FieldKey.TITRE.toString());
        append("; ");
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            append(FieldKey.SOUSTITRE.toString());
            append("; ");
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            FieldKey fieldKey = corpusField.getFieldKey();
            if (CorpusMetadataUtils.hasPersonneSubfield(corpusField)) {
                append("(");
                append(fieldKey.toString());
                space();
                localize("global_lib_or_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.NOM_SUBTYPE)).toString());
                append(", ");
                append((new SubfieldKey(fieldKey, SubfieldKey.PRENOM_SUBTYPE)).toString());
                append(", ");
                append((new SubfieldKey(fieldKey, SubfieldKey.NOMAVANT_SUBTYPE)).toString());
                space();
                localize("global_lib_and_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.ORIGINAL_SUBTYPE)).toString());
                append(")");
            } else if (CorpusMetadataUtils.isCurrencyArrayField(corpusField)) {
                append("(");
                append(fieldKey.toString());
                space();
                localize("global_lib_or_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.VALUE_SUBTYPE)).toString());
                space();
                localize("global_lib_and_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.CUR_SUBTYPE)).toString());
                append(")");
            } else if (CorpusMetadataUtils.hasGeopointSubfield(corpusField)) {
                append("(");
                append(fieldKey.toString());
                space();
                localize("global_lib_or_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.LATITUDE_SUBTYPE)).toString());
                localize("global_lib_and_min");
                space();
                append((new SubfieldKey(fieldKey, SubfieldKey.LONGITUDE_SUBTYPE)).toString());
                append(")");
            } else {
                append(fieldKey.toString());
            }
            append("; ");
        }
        if (CorpusMetadataUtils.hasTexteField(corpusMetadata)) {
            append(FieldKey.TEXTE.toString());
            append("; ");
        }
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        for (int i = 0; i < annexeCount; i++) {
            CorpusField corpusField = corpusMetadata.getAnnexeField(i);
            append(corpusField.getFieldKey().toString());
            append("; ");
        }
        append(FieldKey.REDACTEURS.toString());
        append("; ");
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            append(corpusField.getFieldKey().toString());
            append("; ");
        }
        appendBr();
        appendBr();
        append(IncludeKey.LIAGE.toString());
        append("; ");
        appendBr();
        appendBr();
        localize("saisie_info_includerappel");
    }

}
