/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.servlets.importation.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.formfactory.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFieldConfirmPage extends ImportationHtmlPage {

    FieldKey fieldKey;
    CorpusField corpusField;
    private FicheItemToString.Parameters userParameters;
    protected FicheFormParameters ficheFormParameters;

    public MassFieldConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        fieldKey = (FieldKey) responseParameters.getParameter(MassFieldCommand.FIELDKEY_PARAMNAME);
        corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        ficheFormParameters = FicheFormParameters.newInstance(bdfServer, bdfUser, new FichePointeur(corpus, false));
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massfieldconfirm");
        startTable();
        printSelectedCorpusRow();
        startRow();
        printLibCells("saisie_lib_fieldkey", false);
        startCell();
        escape(corpusField.getLibelleString(workingLangInteger, true));
        space();
        startCode();
        append("[");
        append(fieldKey.toString());
        append("]");
        endCode();
        endCell();
        endRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSFIELDFORM_PAGE_PARAMVALUE;
    }

    protected void addSupplementaryParameters() {
        addParameter(MassFieldCommand.FIELDKEY_PARAMNAME, fieldKey.toString());
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        String keyString = fieldKey.toString();
        boolean corpsdefiche = fieldKey.ownsToCorpsdefiche();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        printResultItemListTitle("saisie_title_list_fiches", size);
        startPostForm(corpus, MassFieldCommand.getCommandString());
        addExecuteStepParameter();
        addParameter(MassFieldCommand.FIELDKEY_PARAMNAME, keyString);
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            AbregeFiche abregeFiche = (AbregeFiche) resultItem.getKeyObject();
            fichePointeur.setCurrent(abregeFiche);
            String idS = String.valueOf(abregeFiche.getIdcorpus());
            String paramName = idS + "_" + keyString;
            startLi();
            startP();
            appendHiddenInput(MassFieldCommand.IDCORPUSARRAY_PARAMNAME, idS);
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
            endP();
            startUl();
            if (corpsdefiche) {
                startP();
                printLib("saisie_lib_newvalue");
                appendBr();
                startTextArea(paramName, 60, 4);
                escape((String) resultItem.getValueObject(), true);
                endTextArea();
                endP();
            } else {
                startP();
                printLib("saisie_lib_oldvalue");
                startCode();
                escape(FicheFormUtils.getStringValue(corpusField, ficheFormParameters));
                endCode();
                endP();
                startP();
                printLib("saisie_lib_newvalue");
                appendLargeInput(paramName, (String) resultItem.getValueObject());
                endP();
            }
            endUl();
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

}
