/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.thesaurus.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class MassIndexationFicheConfirmPage extends ImportationHtmlPage {

    short updateType;
    String updateTypeString;
    private int thesaurusLangInteger;

    public MassIndexationFicheConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        updateType = ((Short) responseParameters.getParameter(MassIndexationFicheCommand.CROISEMENTUPDATE_PARAMNAME)).shortValue();
        updateTypeString = MassLiageCommand.croisementUpdateToString(updateType);
        thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(otherThesaurus, workingLangInteger);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massindexationficheconfirm");
        startTable();
        printSelectedCorpusRow();
        startRow();
        printLibCells("saisie_lib_indexationthesaurus", false);
        startCell();
        escape(ThesaurusUtils.getThesaurusIntitule(otherThesaurus, workingLangInteger));
        space();
        startCode();
        append("[");
        append(otherThesaurus.getSubsetKey().toString());
        append("]");
        endCode();
        endCell();
        endRow();
        startRow();
        printLibCells("saisie_lib_updatetype", false);
        startCell();
        localize("saisie_lib_updatetype_" + updateTypeString);
        endCell();
        endRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSINDEXATIONFICHEFORM_PAGE_PARAMVALUE;
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), otherThesaurus.getSubsetKey());
        printResultItemListTitle("saisie_title_list_fiches", size);
        startPostForm(corpus, MassIndexationFicheCommand.getCommandString());
        addExecuteStepParameter();
        addOtherThesaurusParameter();
        addParameter(MassIndexationFicheCommand.CROISEMENTUPDATE_PARAMNAME, updateTypeString);
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            AbregeFiche abregeFiche = (AbregeFiche) resultItem.getKeyObject();

            startLi();
            startP();
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
            endP();
            startUl();

            startLi();
            appendH3("saisie_lib_existingindexation");
            startUl();
            CroisementItem croisementItem = croisement.getCroisementItem(corpus.getSubsetKey(), abregeFiche.getTicket());
            int existLength = croisementItem.getCount();
            if (existLength == 0) {
                printNoMotcleLi();
            } else {
                for (int k = 0; k < existLength; k++) {
                    int ticket = croisementItem.getTicket(k);
                    int poids = croisementItem.getPoids(k);
                    Motcle abr2 = (Motcle) otherThesaurus.getSubsetItemByTicket(ticket);
                    if (abr2 != null) {
                        printMotcleLi(abr2, poids);
                    }
                }
            }
            endUl();
            endLi();

            startLi();
            if (updateType == CroisementEditor.CLEAR_EXISTING) {
                appendH3("saisie_lib_newindexation_clear");
            } else {
                appendH3("saisie_lib_newindexation_append");
            }
            startUl();
            MotcleCroisementList motcleList = (MotcleCroisementList) resultItem.getValueObject();
            int abrgSize = motcleList.getSubsetItemCount();
            if (abrgSize == 0) {
                if (updateType == CroisementEditor.CLEAR_EXISTING) {
                    appendHiddenInput(String.valueOf(abregeFiche.getIdcorpus()), "");
                    printNoMotcleLi();
                } else {
                    printNoAppendMotcleLi();
                }
            } else {
                for (int j = 0; j < abrgSize; j++) {
                    Motcle motcle = motcleList.getMotcle(j);
                    int poids = motcleList.getPoids(j);
                    StringBuffer buf = new StringBuffer();
                    buf.append(motcle.getIdths());

                    if (poids > 1) {
                        buf.append('+');
                        buf.append(poids - 1);
                    }
                    appendHiddenInput(String.valueOf(abregeFiche.getIdcorpus()), buf.toString());
                    printMotcleLi(motcle, poids);
                }
            }
            endUl();
            endLi();

            endUl();
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

    private void printNoMotcleLi() {
        startLi();
        startP();
        startEm();
        localize("saisie_info_nomotcle");
        endEm();
        endP();
        endLi();
    }

    private void printMotcleLi(Motcle motcle, int poids) {
        startLi();
        startP();
        startCode();
        if (poids > 1) {
            append("(+");
            append(String.valueOf(poids - 1));
            append(")");
            space();
        }
        printMotcleLibelle(motcle, thesaurusLangInteger);
        endCode();
        endP();
        endLi();
    }

    private void printNoAppendMotcleLi() {
        startLi();
        startP();
        startEm();
        localize("saisie_info_noappendmotcle");
        endEm();
        endP();
        endLi();
    }

}
