/* @@@@@APPLICATION@@@@@ - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets.instructions;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.request.ParameterException;
import fr.exemole.bdfserver.response.BdfResponseParameters;
import fr.exemole.bdfserver.servlets.instructions.actions.CommandInstruction;
import fr.exemole.bdfserver.servlets.instructions.actions.PiocheInstruction;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public abstract class ActionBdfInstruction implements BdfInstruction, BdfInstructionConstants, ResponseDataType {

    protected final BdfContext bdfContext;
    protected final BdfServer bdfServer;
    protected final BdfRequestMap requestMap;
    protected BdfResponseParameters responseParameters;
    protected BdfUser bdfUser;
    protected LocalisationInfo localisationInfo;

    public ActionBdfInstruction(BdfContext bdfContext, BdfRequestMap requestMap) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.requestMap = requestMap;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
        if (bdfUser != null) {
            localisationInfo = bdfUser.getLocalisationInfo();
        } else {
            localisationInfo = fr.exemole.bdfserver.instruction.InstructionUtils.getLocalisationInfo(requestMap, bdfServer);
        }
        responseParameters = new BdfResponseParameters(bdfServer, bdfUser);
        responseParameters.setDataType(requestMap.getResponseDataType());
        try {
            return run();
        } catch (ParameterException pe) {
            return null;
        }
    }

    public static BdfInstruction initAction(BdfContext bdfContext, BdfRequestMap requestMap, String actionName) {
        ActionBdfInstruction actionBdfInstruction = null;
        if (actionName.equals("Pioche")) {
            actionBdfInstruction = new PiocheInstruction(bdfContext, requestMap);
        } else if (CommandInstruction.isValidActionName(actionName)) {
            actionBdfInstruction = new CommandInstruction(bdfContext, requestMap, actionName);
        } else {
            return null;
        }
        return actionBdfInstruction;
    }

    public abstract ResponseHandler run() throws ParameterException;

    public String getMandatoryParameter(String paramName) throws ParameterException {
        String s = requestMap.getParameter(paramName);
        if (s == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, paramName);
        }
        return s;
    }

}
