/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions;

import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.servlets.instructions.affichage.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class DirBdfInstruction implements DirConstants, BdfInstruction, BdfInstructionConstants {

    protected short dirType;
    protected String filePath;
    protected BdfContext bdfContext;
    protected BdfServer bdfServer;
    protected BdfUser bdfUser;

    public DirBdfInstruction(short dirType) {
        this.dirType = dirType;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
        return runInstruction();
    }

    protected abstract ResponseHandler runInstruction();

    protected abstract boolean initDirInstruction();

    public static BdfInstruction initInstruction(BdfContext bdfContext, String dirPath, String filePath) {
        if (filePath.length() == 0) {
            return null;
        }
        DirBdfInstruction dirBdfInstruction = null;
        if (dirPath.equals("js")) {
            dirBdfInstruction = new StreamInstruction(JS);
        } else if (dirPath.equals("xslt")) {
            dirBdfInstruction = new StreamInstruction(XSLT);
        } else if (dirPath.equals("docs")) {
            dirBdfInstruction = new StreamInstruction(DOCS);
        } else if (dirPath.equals("prv")) {
            dirBdfInstruction = new StreamInstruction(PRV);
        } else if (dirPath.equals("pub")) {
            dirBdfInstruction = new StreamInstruction(PUB);
        } else if (dirPath.equals("output")) {
            dirBdfInstruction = new StreamInstruction(OUTPUT);
        } else if (dirPath.equals("rsc")) {
            dirBdfInstruction = new StreamInstruction(RSC);
        } else if (dirPath.equals("theme")) {
            dirBdfInstruction = new StreamInstruction(THEME);
        } else if (dirPath.equals("jquery")) {
            dirBdfInstruction = new StreamInstruction(JQUERY);
        } else if (dirPath.equals("css")) {
            dirBdfInstruction = new StreamInstruction(CSS);
        } else if (dirPath.equals("balayages")) {
            dirBdfInstruction = new BalayagesInstruction();
        } else if (dirPath.equals("iso")) {
            dirBdfInstruction = new IsoInstruction();
        } else if (dirPath.equals("redacteurs")) {
            dirBdfInstruction = new RedacteursInstruction();
        } else if (dirPath.equals("admin")) {
            dirBdfInstruction = new AdminInstruction();
        } else if (dirPath.equals("fiches")) {
            dirBdfInstruction = new FichesInstruction();
        } else if (dirPath.equals("motscles")) {
            dirBdfInstruction = new MotsclesInstruction();
        } else if (dirPath.equals("croisements")) {
            dirBdfInstruction = new CroisementsInstruction();
        } else if (dirPath.equals("images")) {
            dirBdfInstruction = new ImagesInstruction();
        } else {
            return null;
        }
        dirBdfInstruction.initBdfContext(bdfContext);
        dirBdfInstruction.setFilePath(filePath);
        boolean init = dirBdfInstruction.initDirInstruction();
        if (!init) {
            return null;
        } else {
            return dirBdfInstruction;
        }
    }

    private void setFilePath(String filePath) {
        this.filePath = filePath;
    }

    private void initBdfContext(BdfContext bdfContext) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
    }

}
