/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.actions;

import fr.exemole.bdfserver.instruction.BdfRequestMap;
import net.mapeadores.util.request.FileValue;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.response.PiocheParameters;
import fr.exemole.bdfserver.servlets.instructions.*;
import fr.exemole.bdfserver.servlets.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.text.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.servlets.pioche.htmlpages.*;
import fr.exemole.bdfserver.servlets.pioche.data.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;
import java.text.*;
import fr.exemole.bdfserver.imagesource.*;
import net.mapeadores.basedefiches.selection.util.*;
import net.mapeadores.basedefiches.selection.*;
import fr.exemole.bdfserver.*;


/**
 *
 * @author Vincent Calame
 */
public class PiocheInstruction extends ActionBdfInstruction implements PiocheConstants {

    private Basedefiches basedefiches;
    private short piocheCategory;

    public PiocheInstruction(BdfContext bdfContext, BdfRequestMap requestMap) {
        super(bdfContext, requestMap);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler run() throws ParameterException {
        init();
        ResponseHandler responsePrinter = InstructionUtils.testJsonResponsePrinter(responseParameters);
        if (responsePrinter != null) {
            return responsePrinter;
        }
        short category = responseParameters.getResponseCodeCategory();
        if (category == BdfResponseParameters.HTMLPAGE_CATEGORY) {
            return PiocheHtmlPage.getInstance(responseParameters);
        } else {
            throw new SwitchException("category = " + responseParameters.getResponseCodeCategory());
        }
    }

    private void init() throws ParameterException {
        basedefiches = bdfServer.getBasedefiches();
        String catString = getMandatoryParameter(CATEGORY_PARAMNAME);
        try {
            piocheCategory = PiocheParameters.categoryToShort(catString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, CATEGORY_PARAMNAME + " = " + catString);
        }
        if (piocheCategory == CATEGORY_GEOCODAGE) {
            initGeocodage();
            return;
        }
        if (piocheCategory == CATEGORY_IMAGESOURCE) {
            initImageSource();
            return;
        }
        if (requestMap.getParameter(RECHERCHE_PARAMNAME) != null) {
            initPiocheData();
        } else {
            initHtmlPage();
        }
    }

    private void initPiocheData() throws ParameterException {
        short dataType = responseParameters.getDataType();
        if ((dataType != JSON_DATATYPE) && (dataType != JSON_IN_HTML_DATATYPE)) {
            responseParameters.setDataType(JSON_DATATYPE);
        }
        int langInteger = bdfUser.getWorkingLangInteger();
        try {
            langInteger = LangInteger.parse(requestMap.getParameter("lang"));
        } catch (LangIntegerException mcle) {
        }
        String param_search = requestMap.getParameter("recherche").trim();
        ConditionGroup conditionGroup = null;
        if (param_search.length() > 0) {
            conditionGroup = Conditions.parseSimpleConditionGroup(param_search);
        }
        if (piocheCategory == CATEGORY_CORPUS) {
            List<Subset> list = getCorpusList();
            FichePiocheData fichePiocheData = new FichePiocheData(basedefiches, langInteger);
            fichePiocheData.setSubsetList(list);
            fichePiocheData.setFicheEligibility(bdfServer.getPermissionManager().getFicheEligibility(bdfUser));
            fichePiocheData.setConditionGroup(conditionGroup);
            String motcleSelectString = requestMap.getParameter(PIOCHESELECT_PARAMNAME);
            if (motcleSelectString != null) {
                MotcleSelectOptions motcleSelectOptions = SelectOptionsParser.parse(motcleSelectString);
                if (motcleSelectOptions != null) {
                    MotcleSelectOptions[] array = {motcleSelectOptions};
                    fichePiocheData.setMotcleSelectOptionsArray(array);
                }
            }
            fichePiocheData.init();
            responseParameters.setJsonSource(fichePiocheData);
        } else if (piocheCategory == CATEGORY_THESAURUS) {
            List<Subset> list = getThesaurusList();
            if ((list != null) && (list.size() == 1)) {
                Thesaurus thesaurus = (Thesaurus) list.get(0);
                langInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, langInteger);
            }
            MotclePiocheData motclePiocheData = new MotclePiocheData(basedefiches, langInteger);
            motclePiocheData.setSubsetList(list);
            motclePiocheData.setConditionGroup(conditionGroup);
            motclePiocheData.init();
            responseParameters.setJsonSource(motclePiocheData);
        } else if (piocheCategory == CATEGORY_SPHERE) {
            List<Subset> list = getSphereList();
            RedacteurPiocheData redacteurPiocheData = new RedacteurPiocheData(basedefiches, langInteger);
            redacteurPiocheData.setSubsetList(list);
            redacteurPiocheData.setConditionGroup(conditionGroup);
            redacteurPiocheData.setDefaultSphereKey(getDefaultSphereKey());
            redacteurPiocheData.init();
            responseParameters.setJsonSource(redacteurPiocheData);
        } else if (piocheCategory == CATEGORY_LANGUE) {
            LanguePiocheData languePiocheData = new LanguePiocheData(responseParameters.getLocalisationInfo().getFormatLocale());
            languePiocheData.setConditionGroup(conditionGroup);
            languePiocheData.init();
            responseParameters.setJsonSource(languePiocheData);
        } else if (piocheCategory == CATEGORY_PAYS) {
            PaysPiocheData paysPiocheData = new PaysPiocheData(responseParameters.getLocalisationInfo().getFormatLocale());
            paysPiocheData.setConditionGroup(conditionGroup);
            paysPiocheData.init();
            responseParameters.setJsonSource(paysPiocheData);
        }
    }

    private void initGeocodage() throws ParameterException {
        String appelant = getMandatoryParameter(APPELANT_PARAMNAME);
        responseParameters.putParameter(APPELANT_PARAMNAME, appelant);
        String latString = getMandatoryParameter(LAT_PARAMNAME).trim();
        String lonString = getMandatoryParameter(LONG_PARAMNAME).trim();
        DegreDecimal latDecimal = null;
        DegreDecimal longDecimal = null;
        if ((latString.length() > 0) && (lonString.length() > 0)) {
            List errorList = new ArrayList();

            try {
                latDecimal = new DegreDecimal(StringUtils.parseDecimal(latString));
            } catch (NumberFormatException nfe) {
                errorList.add(new ResourceItem("saisie_err_geopoint_lat", latString));
            }
            try {
                longDecimal = new DegreDecimal(StringUtils.parseDecimal(lonString));
            } catch (NumberFormatException nfe) {
                errorList.add(new ResourceItem("saisie_err_geopoint_lon", lonString));
            }
            if (errorList.size() > 0) {
                responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
                responseParameters.setMultiErrorMessage("saisie_err_geopoint", errorList);
                return;
            }
            responseParameters.putParameter(LAT_PARAMNAME, latDecimal);
            responseParameters.putParameter(LONG_PARAMNAME, longDecimal);
        }
        responseParameters.setHtmlPageCode(GEOCODAGE_PAGE);
    }

    private void initImageSource() throws ParameterException {
        PiocheParameters piocheParameters = new PiocheParameters();
        responseParameters.putParameter(PIOCHEPARAMETERS_OBJ, piocheParameters);
        short action = 0;
        String actionString = getMandatoryParameter(ACTION_PARAMNAME);
        String imageField = getMandatoryParameter(IMAGEFIELD_PARAMNAME);
        responseParameters.putParameter(ACTION_PARAMNAME, actionString);
        responseParameters.putParameter(IMAGEFIELD_PARAMNAME, imageField);
        boolean form = false;
        if (actionString.startsWith("form-")) {
            form = true;
            actionString = actionString.substring(5);
        }
        try {
            action = ImageSourceUtils.actionToShort(actionString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, ACTION_PARAMNAME + "=" + actionString);
        }
        if (form) {
            String appelant = getMandatoryParameter(APPELANT_PARAMNAME);
            piocheParameters.setAppelant(appelant);
            responseParameters.putParameter(ACTION_PARAMNAME, actionString);
            responseParameters.setHtmlPageCode(IMAGESOURCE_PAGE);
        } else {
            FileValue fichier = null;

            CorpusField corpusField = getImageField(imageField);
            if (corpusField == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, IMAGEFIELD_PARAMNAME + "=" + imageField);
            }
            ImageSourceConnector imageSourceConnector = new ImageSourceConnector(bdfServer, corpusField, action);
            if (imageSourceConnector.needFile()) {
                fichier = requestMap.getFileValue(FICHIER_PARAMNAME);
                if (fichier == null) {
                    throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FICHIER_PARAMNAME);
                }
                imageSourceConnector.setFile(fichier);
            }
            /*if (imageSourceConnector.needFileNumber()) {
            String fileNumberString = getMandatoryParameter(requestMap, FILENUMBER_PARAMNAME);
            try {
            int fileNumber = Integer.parseInt(fileNumberString);
            imageSourceConnector.setFileNumber(fileNumber);
            } catch (NumberFormatException nfe) {
            throw new ParameterException(ParameterException.PARAMETER_NOTINTEGER, FILENUMBER_PARAMNAME + "=" + fileNumberString);
            }
            }*/
            if (imageSourceConnector.needTempFileName()) {
                String tempFileName = getMandatoryParameter(TEMPFILENAME_PARAMNAME);
                imageSourceConnector.setTempFileName(tempFileName);
            }
            ActionResult actionResult = imageSourceConnector.doAction();
            if (actionResult.getState() == ImageSourceConstants.ERROR_STATE) {
                responseParameters.setCommandMessage(actionResult.getCommandMessage());
            } else {
                responseParameters.setJsonSource(new ActionResultToJson(actionResult, ServletCommons.getResourceLocalisation(responseParameters.getLocalisationInfo().getFormatLocale())));
            }
            if (fichier != null) {
                fichier.free();
            }
        }
    }

    private void initHtmlPage() throws ParameterException {
        PiocheParameters piocheParameters = new PiocheParameters();
        responseParameters.putParameter(PIOCHEPARAMETERS_OBJ, piocheParameters);
        piocheParameters.setAppelant(getMandatoryParameter(APPELANT_PARAMNAME));
        piocheParameters.setLimite(getLimite());
        if (piocheCategory == CATEGORY_CORPUS) {
            piocheParameters.setSubsetList(getCorpusList());
            String motcleSelectString = requestMap.getParameter(PIOCHESELECT_PARAMNAME);
            if (motcleSelectString != null) {
                piocheParameters.setMotcleSelectString(motcleSelectString);
            }
            responseParameters.setHtmlPageCode(FICHE_PAGE);
        } else if (piocheCategory == CATEGORY_THESAURUS) {
            String param = requestMap.getParameter(PARAM_PARAMNAME);
            if ((param != null) && (param.equals(IDTHS_PARAMVALUE))) {
                piocheParameters.setParam(PiocheParameters.IDTHS_PARAMVALUE);
            }
            piocheParameters.setSubsetList(getThesaurusList());
            responseParameters.setHtmlPageCode(MOTCLE_PAGE);
        } else if (piocheCategory == CATEGORY_SPHERE) {
            piocheParameters.setParam(getDefaultSphereKey());
            piocheParameters.setSubsetList(getSphereList());
            responseParameters.setHtmlPageCode(REDACTEUR_PAGE);
        } else if (piocheCategory == CATEGORY_LANGUE) {
            responseParameters.setHtmlPageCode(LANGUE_PAGE);
        } else if (piocheCategory == CATEGORY_PAYS) {
            responseParameters.setHtmlPageCode(PAYS_PAGE);
        }
    }

    private int getLimite() {
        String param_limite = requestMap.getParameter(LIMITE_PARAMNAME);
        int limite = 5;
        if (param_limite != null) {
            try {
                limite = Integer.parseInt(param_limite);
                if (limite < 1) {
                    limite = 5;
                }
            } catch (NumberFormatException nfe) {
            }
        }
        return limite;
    }

    private SubsetKey getDefaultSphereKey() {
        String param = requestMap.getParameter(PARAM_PARAMNAME);
        if (param == null) {
            return null;
        }
        try {
            SubsetKey sphereKey = SubsetKey.parse(param);
            if (sphereKey.isSphereSubset()) {
                return sphereKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private List<Subset> getThesaurusList() throws ParameterException {
        String thesaurusParam = getMandatoryParameter("subsets");
        if (thesaurusParam.length() == 0) {
            return null;
        } else {
            String[] thesaurusArray = StringUtils.getTechnicalTokens(thesaurusParam);
            ArrayList<Subset> thsList = new ArrayList<Subset>();
            for (int i = 0; i < thesaurusArray.length; i++) {
                Subset subset = (Thesaurus) BasedefichesUtils.getThesaurus(basedefiches, thesaurusArray[i]);
                if (subset != null) {
                    thsList.add(subset);
                }
            }
            if (thsList.size() == 0) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "subsets = " + thesaurusParam);
            }
            return thsList;
        }
    }

    private List<Subset> getCorpusList() throws ParameterException {
        String subsetsParam = getMandatoryParameter("subsets");
        if (subsetsParam.length() == 0) {
            return null;
        } else {
            String[] corpusArray = StringUtils.getTechnicalTokens(subsetsParam);
            ArrayList<Subset> crpList = new ArrayList<Subset>();
            for (int i = 0; i < corpusArray.length; i++) {
                Subset subset = BasedefichesUtils.getCorpus(basedefiches, corpusArray[i]);
                if (subset != null) {
                    crpList.add(subset);
                }
            }
            if (crpList.size() == 0) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "subsets = " + subsetsParam);
            }
            return crpList;
        }
    }

    private List<Subset> getSphereList() throws ParameterException {
        String subsetsParam = getMandatoryParameter("subsets");
        if (subsetsParam.length() == 0) {
            return null;
        } else {
            String[] array = StringUtils.getTechnicalTokens(subsetsParam);
            ArrayList<Subset> sphList = new ArrayList<Subset>();
            for (int i = 0; i < array.length; i++) {
                Subset subset = BasedefichesUtils.getSphere(basedefiches, array[i]);
                if (subset != null) {
                    sphList.add(subset);
                }
            }
            if (sphList.size() == 0) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "subsets = " + subsetsParam);
            }
            return sphList;
        }
    }

    private CorpusField getImageField(String imageField) {
        int idx = imageField.indexOf('/');
        if (idx == -1) {
            return null;
        }
        SubsetKey corpusKey = null;
        try {
            corpusKey = SubsetKey.parse(imageField.substring(0, idx));
        } catch (ParseException pe) {
            return null;
        }
        if (!corpusKey.isCorpusSubset()) {
            return null;
        }
        Corpus corpus = (Corpus) basedefiches.getSubset(corpusKey);
        if (corpus == null) {
            return null;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(imageField.substring(idx + 1));
        } catch (ParseException pe) {
            return null;
        }
        CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            return null;
        }
        if (corpusField.getFicheItemType() != CorpusField.IMAGE_FIELD) {
            return null;
        }
        //if (corpusField.getStringOption(FieldOptions.IMAGESOURCE_OPTION) == null) {
        SubsetKey albumKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
        if (albumKey == null) {
            return null;
        }
        Album album = (Album) basedefiches.getSubset(albumKey);
        if (album == null) {
            return null;
        }
        return corpusField;
    }

}
