/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import net.mapeadores.util.servlets.handlers.RedirectResponseHandler;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.managers.transformation.*;
import fr.exemole.bdfserver.servlets.instructions.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.affichage.htmlpages.*;
import fr.exemole.bdfserver.servlets.affichage.contentstreams.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractAffichageInstruction extends DirBdfInstruction implements AffichageConstants {

    protected String extension;
    protected String baseName;
    TransformationKey redirectTransformationKey;

    public AbstractAffichageInstruction(short dirType) {
        super(dirType);
    }

    public boolean needBdfUser() {
        return true;
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    protected ResponseHandler runInstruction() {
        if (redirectTransformationKey != null) {
            return getRedirectResponsePrinter();
        }
        BdfResponseParameters responseParameters = new BdfResponseParameters(bdfServer, bdfUser);
        checkResponseParameters(responseParameters);
        switch (responseParameters.getResponseCategory()) {
            case ResponseParameters.HTMLPAGE_CATEGORY:
                return AffichageHtmlPage.getInstance(responseParameters);
            case ResponseParameters.CONTENTSTREAM_CATEGORY:
                return AffichageContentStream.getInstance(responseParameters);
            default:
                throw new SwitchException("category = " + responseParameters.getResponseCategory());
        }
    }

    public abstract void checkResponseParameters(BdfResponseParameters responseParameters);

    protected boolean initSimpleFile() {
        return initSimpleFile(filePath);
    }

    protected boolean initSimpleFile(String fileName) {
        int idx = fileName.indexOf('/');
        if (idx != -1) {
            return false;
        }
        int idx2 = fileName.indexOf('.');
        if ((idx2 < 1) || (idx2 > (fileName.length() - 2))) {
            return false;
        }
        baseName = fileName.substring(0, idx2);
        extension = fileName.substring(idx2 + 1);
        return true;
    }

    protected void setXslRedirect(TransformationKey redirectTransformationKey) {
        this.redirectTransformationKey = redirectTransformationKey;
    }

    private ResponseHandler getRedirectResponsePrinter() {
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(redirectTransformationKey);
        if (!xslTransformerKey.isDefaultTransformerKey()) {
            TransformationInfo transformationInfo = bdfServer.getTransformationManager().getTransformationInfo(xslTransformerKey.getTransformationKey());
            if (!transformationInfo.containsXslTransformer(xslTransformerKey.getTransformerName())) {
                xslTransformerKey = xslTransformerKey.toDefaultTransformerKey();
            }
        }
        return new RedirectResponseHandler("../" + ServletCommons.getTemplatePath(xslTransformerKey));
    }

}
