/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.balayages.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.servlets.affichage.contentstreams.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayagesInstruction extends AbstractAffichageInstruction {

    private String extension;
    private Balayage balayage;
    private BalayageUnit balayageUnit;
    private SubsetKey subsetKey;
    private Subset subset;
    private Output output;
    private int subsetItemId;
    private SubsetItem subsetItem;
    private int langInteger;
    private boolean xmlVersion = false;

    public BalayagesInstruction() {
        super(BALAYAGES);
    }

    public boolean needBdfUser() {
        return false;
    }

    public void checkResponseParameters(BdfResponseParameters responseParameters) {
        responseParameters.setProfondeur(3);
        responseParameters.setType(BALAYAGEUNIT_OCTET_STREAM);
        BalayageUnitStream.Parameters parameters = new BalayageUnitStream.Parameters();
        parameters.setBalayage(balayage);
        parameters.setBalayageUnit(balayageUnit);
        parameters.setSubset(subset);
        parameters.setSubsetItem(subsetItem);
        parameters.setLangInteger(langInteger);
        parameters.setOutput(output);
        parameters.setXmlVersion(xmlVersion);
        responseParameters.putParameter(OBJECT_OBJ, parameters);
    }

    public boolean initDirInstruction() {
        int ln = filePath.length();
        int idx1 = filePath.indexOf('/');
        if (idx1 == -1) {
            return false;
        }
        int idx2 = filePath.indexOf('/', idx1 + 1);
        if ((idx2 == -1) || (idx2 > (ln - 5))) {
            return false;
        }
        String balayageName = filePath.substring(0, idx1);
        boolean test = initBalayage(balayageName);
        if (!test) {
            return false;
        }
        String balayageUnitName = filePath.substring(idx1 + 1, idx2);
        test = initBalayageUnit(balayageUnitName);
        if (!test) {
            return false;
        }
        baseName = filePath.substring(idx2 + 1);
        test = initSubset(baseName);
        if (!test) {
            return false;
        }
        return true;
    }

    private boolean initBalayage(String balayageName) {
        int idxtiret = balayageName.indexOf('-');
        if (idxtiret != -1) {
            try {
                langInteger = LangInteger.parse(balayageName.substring(idxtiret + 1));
            } catch (LangIntegerException lie) {
                return false;
            }
            balayageName = balayageName.substring(0, idxtiret);
        }
        balayage = bdfServer.getBalayageManager().getBalayage(balayageName);
        if (balayage == null) {
            return false;
        }
        return true;
    }

    private boolean initBalayageUnit(String balayageUnitName) {
        int idxtiret = balayageUnitName.indexOf('-');
        String outputName = null;
        if (idxtiret != -1) {
            if (idxtiret < 2) {
                return false;
            }
            outputName = balayageUnitName.substring(idxtiret + 1);
            balayageUnitName = balayageUnitName.substring(0, idxtiret);
        }
        balayageUnit = BalayageUtils.getBalayageUnit(balayage, balayageUnitName);
        if (balayageUnit == null) {
            return false;
        }
        if (outputName != null) {
            output = BalayageUtils.getOutput(balayageUnit, outputName);
            if (output == null) {
                return false;
            }
        }
        return true;
    }

    private boolean initSubset(String baseName) {
        if (baseName.endsWith(".xml")) {
            xmlVersion = true;
            baseName = baseName.substring(0, baseName.length() - 4);
        }
        int idxtiret = baseName.indexOf('-');
        boolean needId = false;
        try {
            switch (balayageUnit.getType()) {
                case BalayageConstants.UNIQUE_TYPE:
                    if (baseName.equals("unique")) {
                        return true;
                    } else {
                        return false;
                    }
                case BalayageConstants.CORPUS_TYPE:
                    if (idxtiret != -1) {
                        return false;
                    }
                    subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName);
                    break;
                case BalayageConstants.THESAURUS_TYPE:
                    if (idxtiret != -1) {
                        return false;
                    }
                    subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName);
                    break;
                case BalayageConstants.FICHE_TYPE:
                    if (idxtiret < 2) {
                        return false;
                    }
                    subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idxtiret));
                    needId = true;
                    break;
                case BalayageConstants.MOTCLE_TYPE:
                    if (idxtiret < 2) {
                        return false;
                    }
                    subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName.substring(0, idxtiret));
                    needId = true;
                    break;
                default:
                    return false;
            }
        } catch (java.text.ParseException pe) {
            return false;
        }
        subset = bdfServer.getBasedefiches().getSubset(subsetKey);
        if (subset == null) {
            return false;
        }
        if (needId) {
            try {
                int subsetItemId = Integer.parseInt(baseName.substring(idxtiret + 1));
                if (subsetItemId < 1) {
                    return false;
                }
                subsetItem = subset.getSubsetItemById(subsetItemId);
                if (subsetItem == null) {
                    return false;
                }
            } catch (NumberFormatException nfe) {
                return false;
            }
        }
        return true;
    }

}
