/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;


/**
 *
 * @author Vincent Calame
 */
public class CroisementsInstruction extends AbstractAffichageInstruction {

    Corpus corpus;
    Thesaurus thesaurus;

    public CroisementsInstruction() {
        super(CROISEMENTS);
    }

    public boolean initDirInstruction() {
        int idx1 = filePath.indexOf('/');
        if (idx1 == -1) {
            return false;
        }
        String subDir = filePath.substring(0, idx1);
        if (!subDir.equals("corpus_thesaurus")) {
            return false;
        }
        String fileName = filePath.substring(idx1 + 1);
        if (!initSimpleFile(fileName)) {
            return false;
        }
        if (!extension.equals("html")) {
            return false;
        }
        int idx = baseName.indexOf('_');
        if ((idx < 1) || (idx == (baseName.length() - 1))) {
            return false;
        }
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idx));
            corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
            if (corpus == null) {
                return false;
            }
        } catch (java.text.ParseException pe) {
            return false;
        }
        try {
            SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName.substring(idx + 1));
            thesaurus = (Thesaurus) bdfServer.getBasedefiches().getSubset(thesaurusKey);
            if (thesaurus == null) {
                return false;
            }
        } catch (java.text.ParseException pe) {
            return false;
        }
        return true;
    }

    public void checkResponseParameters(BdfResponseParameters responseParameters) {
        responseParameters.setProfondeur(2);
        Object[] objectArray = new Object[2];
        objectArray[0] = corpus;
        objectArray[1] = thesaurus;
        responseParameters.putParameter(OBJECTARRAY_OBJ, objectArray);
        responseParameters.setType(CORPUSTHESAURUS_PAGE);
    }

}
