/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.actions.*;
import fr.exemole.bdfserver.servlets.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class FichesInstruction extends AbstractAffichageInstruction implements BdfConstants {

    private short type;
    private boolean getSelection = false;
    private Corpus corpus;
    private AbregeFiche abregeFiche;

    public FichesInstruction() {
        super(FICHES);
    }

    public boolean initDirInstruction() {
        if (!initSimpleFile()) {
            return false;
        }
        if (baseName.equals("geolocalisation")) {
            return initGeolocalisation();
        }
        if (baseName.equals("compilation")) {
            return initCompilation();
        }
        int idxtiret = baseName.indexOf('-');
        if (idxtiret < 2) {
            return false;
        }
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idxtiret));
            corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
        } catch (java.text.ParseException pe) {
            return false;
        }
        if (corpus == null) {
            return false;
        }
        try {
            int idcorpus = Integer.parseInt(baseName.substring(idxtiret + 1));
            if (idcorpus < 1) {
                return false;
            }
            abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche == null) {
                return false;
            }
        } catch (NumberFormatException nfe) {
            return false;
        }
        return initAbregeFiche(abregeFiche);
    }

    public void checkResponseParameters(BdfResponseParameters responseParameters) {
        responseParameters.setProfondeur(1);
        responseParameters.putParameter(EXTENSION_PARAMNAME, extension);
        if (getSelection) {
            responseParameters.putParameter(ABREGEFICHESORTEDLIST_OBJ, bdfUser.getAbregeFicheSortedList());
        } else if (abregeFiche != null) {
            responseParameters.putParameter(ABREGEFICHE_OBJ, abregeFiche);
            ReadFiche readAction = FicheAction.getReadFiche(abregeFiche);
            boolean canRead = bdfServer.getPermissionManager().canDo(bdfUser, readAction);
            if (!canRead) {
                if (extension.equals("html")) {
                    responseParameters.setHtmlPageCode(MESSAGE_SERVLETPAGE);
                    responseParameters.setErrorMessage("global_err_readfiche", CorpusUtils.toKeyString(abregeFiche));
                    return;
                } else {
                    throw new HttpAccessException(HttpAccessException.FORBIDDEN_ACCESS);
                }
            }
        }
        responseParameters.setType(type);
    }

    private boolean initGeolocalisation() {
        getSelection = true;
        if (extension.equals("html")) {
            type = GEOLOCALISATION_PAGE;
            return true;
        } else {
            return false;
        }
    }

    private boolean initCompilation() {
        if (extension.equals("xsl")) {
            setXslRedirect(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
            return true;
        }
        getSelection = true;
        if (extension.equals("xml")) {
            type = COMPILATION_XML_STREAM;
        } else if (extension.equals("html")) {
            type = COMPILATION_HTML_STREAM;
        } else {
            type = COMPILATION_OCTET_STREAM;
        }
        return true;
    }

    private boolean initAbregeFiche(AbregeFiche abregeFiche) {
        if (extension.equals("xsl")) {
            setXslRedirect(new TransformationKey(abregeFiche.getCorpus().getSubsetKey()));
            return true;
        }
        if (extension.equals("xml")) {
            type = FICHE_XML_STREAM;
        } else if (extension.equals("xml+xsl")) {
            type = FICHE_XMLXSL_STREAM;
        } else if (extension.equals("html")) {
            type = FICHE_HTML_STREAM;
        } else {
            type = FICHE_OCTET_STREAM;
        }
        return true;
    }

}
