/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class ImagesInstruction extends AbstractAffichageInstruction {

    private Album album;
    private AlbumImage albumImage;
    private AlbumDim albumDim;

    public ImagesInstruction() {
        super(IMAGES);
    }

    public boolean initDirInstruction() {
        int idx1 = filePath.indexOf('/');
        if (idx1 == -1) {
            return false;
        }
        String subDir = filePath.substring(0, idx1);
        SubsetKey albumKey = null;
        try {
            albumKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, subDir);
        } catch (ParseException pe) {
            return false;
        }
        album = (Album) bdfServer.getBasedefiches().getSubset(albumKey);
        if (album == null) {
            return false;
        }
        String fileName = filePath.substring(idx1 + 1);
        int idx2 = fileName.indexOf('/');
        if (idx2 != -1) {
            String dimName = fileName.substring(0, idx2);
            albumDim = album.getAlbumMetadata().getAlbumDimByName(dimName);
            if (albumDim == null) {
                return false;
            }
            fileName = fileName.substring(idx2 + 1);
        }
        if (!initSimpleFile(fileName)) {
            return false;
        }
        short type = 0;
        try {
            type = AlbumUtils.imageTypeToShort(extension);
        } catch (IllegalArgumentException iae) {
            return false;
        }

        albumImage = album.getAlbumImageByName(baseName);
        if (albumImage == null) {
            return false;
        }
        if (albumImage.getType() != type) {
            return false;
        }
        return true;
    }

    public void checkResponseParameters(BdfResponseParameters responseParameters) {
        int profondeur = (albumDim != null) ? 3 : 2;
        responseParameters.setProfondeur(profondeur);
        responseParameters.putParameter(ALBUMIMAGE_OBJ, albumImage);
        if (albumDim != null) {
            responseParameters.putParameter(ALBUMDIM_OBJ, albumDim);
        }
        responseParameters.setType(IMAGE_OCTET_STREAM);
    }

}
