/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.instructions.affichage;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesInstruction extends AbstractAffichageInstruction {

    public final static String STATS_OPTION = "stats";
    public final static String STATSSEL_OPTION = "statssel";
    public final static String INVERSE_OPTION = "inverse";
    public final static String LIBS_OPTION = "libs";
    public final static String TABLE_OPTION = "table";
    public final static String MOTCLE_OPTION = "motcle";
    private Thesaurus thesaurus;
    private Motcle motcle;
    private String outputOption = "";
    private int langInteger;
    private short type;

    public MotsclesInstruction() {
        super(MOTSCLES);
    }

    public boolean initDirInstruction() {
        if (!initSimpleFile()) {
            return false;
        }
        boolean firstStep = firstStep();
        if (!firstStep) {
            return false;
        }
        if (extension.equals("xsl")) {
            TransformationKey transformationKey = getTransformationKey(outputOption);
            if (transformationKey != null) {
                setXslRedirect(transformationKey);
                return true;
            } else {
                return false;
            }
        } else if (extension.equals("xml")) {
            type = getXMLContentStreamType(outputOption);
            return (type != 0);
        } else if (extension.equals("html")) {
            type = getHTMLContentStreamType(outputOption);
            return (type != 0);
        } else if (extension.equals("ods")) {
            type = getOdsContentStreamType(outputOption);
            return (type != 0);
        } else {
            return false;
        }
    }

    private boolean firstStep() {
        int idxLang = baseName.indexOf('_');
        if (idxLang != -1) {
            try {
                langInteger = LangInteger.parse(baseName.substring(idxLang + 1));
            } catch (LangIntegerException lie) {
                return false;
            }
            baseName = baseName.substring(0, idxLang);
        }
        int idx = baseName.indexOf('-');
        String outputString = null;
        if (idx != -1) {
            outputString = baseName.substring(idx + 1);
            baseName = baseName.substring(0, idx);

        }
        thesaurus = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), baseName);
        if ((thesaurus == null) || (outputString == null)) {
            return false;
        }
        if ((outputString.equals(STATS_OPTION)) || (outputString.equals(STATSSEL_OPTION)) || (outputString.equals(INVERSE_OPTION)) || (outputString.equals(TABLE_OPTION))) {
            outputOption = outputString;
            return true;
        }
        if (outputString.equals(LIBS_OPTION)) {
            if (langInteger == 0) {
                return false;
            } else {
                outputOption = LIBS_OPTION;
                return true;
            }
        }
        try {
            int idths = Integer.parseInt(outputString);
            motcle = thesaurus.getMotcleByIdths(idths);
            if (motcle != null) {
                outputOption = MOTCLE_OPTION;
                return true;
            } else {
                return false;
            }
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

    public void checkResponseParameters(BdfResponseParameters responseParameters) {
        responseParameters.setProfondeur(1);
        if (motcle != null) {
            responseParameters.putParameter(MOTCLE_OBJ, motcle);
        } else {
            responseParameters.putParameter(THESAURUS_OBJ, thesaurus);
        }
        responseParameters.setType(type);
    }

    private static short getXMLContentStreamType(String outputOption) {
        if (outputOption.equals(STATS_OPTION)) {
            return THS_STATS_XML_STREAM;
        }
        if (outputOption.equals(STATSSEL_OPTION)) {
            return THS_STATSSEL_XML_STREAM;
        }
        if (outputOption.equals(INVERSE_OPTION)) {
            return THS_INVERSE_XML_STREAM;
        }
        if (outputOption.equals(LIBS_OPTION)) {
            return THS_LIBS_XML_STREAM;
        }
        if (outputOption.equals(MOTCLE_OPTION)) {
            return MOTCLE_XML_STREAM;
        }
        return 0;
    }

    private static short getHTMLContentStreamType(String outputOption) {
        if (outputOption.equals(STATS_OPTION)) {
            return THS_STATS_HTML_STREAM;
        }
        if (outputOption.equals(STATSSEL_OPTION)) {
            return THS_STATSSEL_HTML_STREAM;
        }
        if (outputOption.equals(INVERSE_OPTION)) {
            return THS_INVERSE_HTML_STREAM;
        }
        if (outputOption.equals(MOTCLE_OPTION)) {
            return MOTCLE_HTML_STREAM;
        }
        return 0;
    }

    private static short getOdsContentStreamType(String outputOption) {
        if (outputOption.equals(TABLE_OPTION)) {
            return THESAURUS_ODS_STREAM;
        }
        if (outputOption.equals(STATS_OPTION)) {
            return THS_STATS_ODS_STREAM;
        }
        if (outputOption.equals(STATSSEL_OPTION)) {
            return THS_STATSSEL_ODS_STREAM;
        }
        if (outputOption.equals(INVERSE_OPTION)) {
            return THS_INVERSE_ODS_STREAM;
        }
        return 0;
    }

    private static TransformationKey getTransformationKey(String outputOption) {
        if (outputOption.equals(STATS_OPTION)) {
            return TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY;
        }
        if (outputOption.equals(STATSSEL_OPTION)) {
            return TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY;
        }
        if (outputOption.equals(INVERSE_OPTION)) {
            return TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY;
        }
        if (outputOption.equals(MOTCLE_OPTION)) {
            return TransformationKey.MOTCLE_TRANSFORMATIONKEY;
        }
        return null;
    }

}
