/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.data;

import java.io.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import java.util.*;
import net.mapeadores.util.json.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class LanguePiocheData implements JsonProducer {

    private ConditionGroup conditionGroup;
    private LocaleItem localeItem;
    private int[] result;

    public LanguePiocheData(Locale locale) {
        this.localeItem = LangIntegerLocalisation.getLocaleItem(locale);
    }

    public void setConditionGroup(ConditionGroup conditionGroup) {
        this.conditionGroup = conditionGroup;
    }

    public void init() {
        if (conditionGroup != null) {
            result = selection();
        } else {
            result = localeItem.getLangIntegerArray();
        }
    }

    private int[] selection() {
        ContentConditionBean[] array = conditionGroup.getIncludingContentConditionBeanArray();
        String texte = array[0].getText();
        return localeItem.getLangIntegerArray(texte, TextConstants.CONTAINS);
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.length;
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (int i = 0; i < length; i++) {
                    int id = result[i];
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(id);
                    jsonWriter.key("idalphakey");
                    jsonWriter.value(LangInteger.getLanguage(id));
                    String lib = localeItem.getLangLibelle(id, false);
                    if (lib.length() > 0) {
                        jsonWriter.key("text");
                        jsonWriter.value(lib);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
