/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.htmlpages;

import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.util.HTMLPrinterUtils;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class GeocodagePiochePage extends PiocheHtmlPage implements ValueResolver {

    DegreDecimal latDecimal;
    DegreDecimal lonDecimal;
    String appelant;
    CorpusField corpusField;

    public GeocodagePiochePage(BdfResponseParameters responseParameters) {
        super(responseParameters);
        appelant = (String) responseParameters.getParameter(APPELANT_PARAMNAME);
        latDecimal = (DegreDecimal) responseParameters.getParameter(LAT_PARAMNAME);
        lonDecimal = (DegreDecimal) responseParameters.getParameter(LONG_PARAMNAME);
        corpusField = (CorpusField) responseParameters.getParameter(FIELD_PARAMNAME);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        HTMLPrinterUtils.printTemplate(getPrintWriter(), ServletCommons.getTemplateInputStream("pioche_geocodage"), this);
    }

    public String getValue(AccoladeArgument accoladeArgument) {
        String key = accoladeArgument.getName();
        if (key.equals("TITRE")) {
            return getEscapedLocalization("pioche_title_geocodage", null);
        } else if (key.equals("APPELANT")) {
            return appelant;
        } else if (key.equals("LAT")) {
            if (latDecimal != null) {
                return latDecimal.toString();
            } else {
                return "-999";
            }
        } else if (key.equals("LON")) {
            if (lonDecimal != null) {
                return lonDecimal.toString();
            } else {
                return "-999";
            }
        } else if (key.equals("IDPREFIX")) {
            return FicheFormCatalog.ID_FF_CHAMP_PREFIX;
        } else if (key.equals("ICONPATH")) {
            return "theme/icons/48x16/";
        } else {
            return getEscapedLocalization(key, null);
        }
    }

}
