/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.htmlpages;

import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import fr.exemole.bdfserver.servlets.util.*;


/**
 *
 * @author Vincent Calame
 */
public class MotclePiochePage extends PiocheHtmlPage {

    List<Subset> thesaurusList;
    String param;
    boolean wantedIdths = false;

    public MotclePiochePage(BdfResponseParameters piocheResponseParameters) {
        super(piocheResponseParameters);
        thesaurusList = piocheParameters.getSubsetList();
        param = (String) piocheParameters.getParam();
        if ((param != null) && (param.equals(IDTHS_PARAMVALUE))) {
            wantedIdths = true;
        }
        addJqueryExtensions("form");
        addJavascript(FICHEFORM_JAVASCRIPT_CORE);
        addJavascript(FICHEFORM_JAVASCRIPT_RECHERCHE);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        int total = getTotal();
        initResponse(response);
        start();
        printScriptVar(total, " – ", (wantedIdths) ? IDALPHAKEY_INTKEY_WANTED : IDALPHAKEY_TEXT_WANTED);
        printRechercheDiv("thesaurus", AppelantWriter.getSubsetsString(thesaurusList), null);
        printSelectZone(total);
        printButtonDiv();
        end();
    }

    public int getTotal() {
        int count = 0;
        if (thesaurusList == null) {
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                count = count + basedefiches.getThesaurus(i).size();
            }
        } else {
            for (Subset subset : thesaurusList) {
                count = count + subset.size();
            }
        }
        return count;
    }

    public String getTitle() {
        if ((thesaurusList != null) && (thesaurusList.size() == 1)) {
            Thesaurus thesaurus = (Thesaurus) thesaurusList.get(0);
            return getLocalization("pioche_title_motcle_one", ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        } else {
            return getLocalization("pioche_title_motcle", null);
        }
    }

}
