/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.htmlpages;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class PiocheHtmlPage extends BdfHtmlPage implements PiocheConstants, PiocheCatalog {

    protected PiocheParameters piocheParameters;

    public PiocheHtmlPage(BdfResponseParameters piocheResponseParameters) {
        super(piocheResponseParameters);
        this.piocheParameters = (PiocheParameters) piocheResponseParameters.getParameter(PIOCHEPARAMETERS_OBJ);
    }

    public static ResponseHandler getInstance(BdfResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case MESSAGE_SERVLETPAGE:
                return new MessagePage(responseParameters);
            case MOTCLE_PAGE:
                return new MotclePiochePage(responseParameters);
            case FICHE_PAGE:
                return new FichePiochePage(responseParameters);
            case REDACTEUR_PAGE:
                return new RedacteurPiochePage(responseParameters);
            case PAYS_PAGE:
                return new PaysPiochePage(responseParameters);
            case LANGUE_PAGE:
                return new LanguePiochePage(responseParameters);
            case GEOCODAGE_PAGE:
                return new GeocodagePiochePage(responseParameters);
            case IMAGESOURCE_PAGE:
                return new ImageSourcePiochePage(responseParameters);
        }
        return null;
    }

    public String getTitle() {
        return null;
    }

    public String getFormAction() {
        return "Pioche";
    }

    public String getCssThemeFile() {
        return "pioche.css";
    }

    public String getSelectSize() {
        int limite = piocheParameters.getLimite();
        if (limite == 1) {
            return "2";
        } else {
            return String.valueOf(limite);
        }
    }


    /*public void printAppelant(String appelant) {
    append("PCH_setAppelant(\"");
    append("\");");
    append(FicheFormCatalog.ID_FF_CHAMP_PREFIX);
    append("\",\"");
    append(appelant);
    append("\");");
    }*/
    public void printAppelant() {
        append("PCH_Core.setAppelantId(\"");
        append(piocheParameters.getAppelant());
        append("\");");
    }

    public void printLimite() {
        append("PCH_Core.setLimite(");
        append(String.valueOf(piocheParameters.getLimite()));
        append(");");
    }

    public void printTotal(int total) {
        append("PCH_Core.setTotal(");
        append(String.valueOf(total));
        append(");");
    }

    public void printLoc(String key, Object value) {
        append("PCH_Core.setLoc(\"");
        append(key);
        append("\",\"");
        append(StringUtils.escapeDoubleQuote(getLocalization(key, value)));
        append("\");");
    }

    public void printSeparator(String value) {
        append("PCH_Core.setSeparator(\"");
        append(StringUtils.escapeDoubleQuote(value));
        append("\");");
    }

    public void printWanted(String value) {
        append("PCH_Core.setWanted(\"");
        append(StringUtils.escapeDoubleQuote(value));
        append("\");");
    }

    public void printParam(String key, String value) {
        append("PCH_Core.setParam(\"");
        append(key);
        append("\",\"");
        append(StringUtils.escapeDoubleQuote(value));
        append("\");");
    }

    public void printScriptVar(int total, String separator, String wanted) {
        startScript(null);
        printAppelant();
        printLimite();
        printLoc("pioche_info_aucunresultat", null);
        printSeparator(separator);
        printTotal(total);
        printWanted(wanted);
        endScript();
    }

    public void printRechercheDiv(String category, String subsetsString, String param) {
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId("ID_PIOCHE_FORMDIV");
        startDiv(divAttributes);
        startForm(null);
        appendHiddenInput(CATEGORY_PARAMNAME, category);
        appendHiddenInput(SUBSETS_PARAMNAME, subsetsString);
        if (param != null) {
            appendHiddenInput(PARAM_PARAMNAME, param);
        }
        String motcleSelectString = piocheParameters.getMotcleSelectString();
        if (motcleSelectString != null) {
            appendHiddenInput(PIOCHESELECT_PARAMNAME, motcleSelectString);
        }
        startP();
        printLibColon("pioche_lib_sequence");
        appendBr();
        HTMLAttributes rechercheAttributes = new HTMLAttributes();
        rechercheAttributes.setId("ID_PIOCHE_RECHERCHE");
        rechercheAttributes.setSize("35");
        rechercheAttributes.setType("text");
        rechercheAttributes.setName(RECHERCHE_PARAMNAME);
        appendInput(rechercheAttributes);
        space();
        addSubmit("global_submit_search", false);
        endP();

        endForm();
        endDiv();
    }

    public void printSelectZone(int total) {
        HTMLAttributes totalAttributes = new HTMLAttributes();
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setStyle("width: 100%;");
        startP();
        printLibColon("pioche_lib_proposition");
        space();
        append("[");
        totalAttributes.setId("ID_PIOCHE_PROP_TOTAL");
        startSpan(totalAttributes);
        append("0");
        endSpan();
        append("/");
        append(String.valueOf(total));
        append("]");
        HTMLAttributes ongletAttributes = new HTMLAttributes();
        ongletAttributes.setId("ID_PIOCHE_PROP_ONGLETS");
        startSpan(ongletAttributes);
        endSpan();
        appendBr();
        selectAttributes.setId("ID_PIOCHE_PROP_SELECT");
        selectAttributes.setSize("11");
        startSelect(selectAttributes);
        endSelect();
        endP();
        startP();
        printLibColon("pioche_lib_selection");
        space();
        append("[");
        totalAttributes.setId("ID_PIOCHE_SEL_TOTAL");
        startSpan(totalAttributes);
        append("0");
        endSpan();
        append("]");
        appendBr();
        selectAttributes.setId("ID_PIOCHE_SEL_SELECT");
        selectAttributes.setSize(getSelectSize());
        startSelect(selectAttributes);
        endSelect();
        endP();
    }

    public void printButtonDiv() {
        HTMLAttributes buttonsAttributes = new HTMLAttributes();
        buttonsAttributes.addClass(PCH_BUTTONSDIV);
        startDiv(buttonsAttributes);
        HTMLAttributes buttonAttributes = new HTMLAttributes();
        buttonAttributes.setType("button");

        buttonAttributes.setId("ID_PIOCHE_OK");
        buttonAttributes.setValue(getLocalization("global_submit_ok"));
        appendInput(buttonAttributes);
        space();

        buttonAttributes.setId("ID_PIOCHE_CANCEL");
        buttonAttributes.setValue(getLocalization("global_submit_cancel"));
        appendInput(buttonAttributes);
        endDiv();
    }

}
