/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractThesaurusCommand extends AbstractBdfCommand {

    protected Thesaurus thesaurus;
    protected ThesaurusMetadata thesaurusMetadata;
    protected Motcle motcle;
    protected ThesaurusEditor thesaurusEditor;
    protected int thesaurusLangInteger;
    protected ThesaurusResponseParameters responseParameters;

    public AbstractThesaurusCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        checkParameters();
        if (responseParameters.mayBeDone()) {
            thesaurusEditor = bdfServer.getBasedefichesEditor().getThesaurusEditor(thesaurus.getSubsetKey());
            doThesaurusCommand();
        }
        return responseParameters;
    }

    protected CommandTest checkParameters() throws ParameterException {
        thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
        thesaurusMetadata = thesaurus.getThesaurusMetadata();
        thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, bdfUser.getWorkingLangInteger());
        responseParameters = new ThesaurusResponseParameters(bdfServer, bdfUser, thesaurusLangInteger);
        if (needMotcleInit()) {
            initMotcle();
        } else {
            responseParameters.setThesaurus(thesaurus);
        }
        checkThesaurusParameters();
        return responseParameters;
    }

    protected abstract void doThesaurusCommand();

    protected abstract void checkThesaurusParameters() throws ParameterException;

    protected void initMotcle() throws ParameterException {
        motcle = RequestUtils.getMotcle(requestMap, thesaurus);
        responseParameters.setMotcle(motcle);
    }

    public abstract boolean needMotcleInit();

    public boolean checkConfirmPassword() throws ParameterException {
        return RequestUtils.checkConfirmPassword(requestMap, bdfServer);
    }

    public int getStepNumber() throws ParameterException {
        String param_step = requestMap.getParameter("step");
        if (param_step == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, "step");
        }
        try {
            int step = Integer.parseInt(param_step);
            if ((step < 1) || (step > 2)) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "step" + " = " + param_step);
            }
            return step;
        } catch (NumberFormatException nfe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "step" + " = " + param_step);
        }
    }

    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ThesaurusResponsePrinterFactory();
    }

    public String getCompleteLibelle(Motcle motcle, int langInteger) {
        StringBuffer buf = new StringBuffer();
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            buf.append(idalpha);
            buf.append(" \u2013 ");
        }
        String lib = motcle.getLibelleString(langInteger, ThesaurusUtils.isBabelienType(motcle.getThesaurus()));
        if (lib == null) {
            buf.append("?");
        } else {
            buf.append(lib);
        }
        if (idalpha == null) {
            buf.append(" (idths=");
            buf.append(String.valueOf(motcle.getIdths()));
            buf.append(")");
        }
        return buf.toString();
    }

    public boolean isSlaveCorpus(Corpus corpus) {
        Subset masterSubset = corpus.getMasterSubset();
        if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
            return true;
        }
        return false;
    }

}
