/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import net.mapeadores.basedefiches.tools.thesaurus.RecouvrementEngine;
import net.mapeadores.basedefiches.tools.thesaurus.RecouvrementUnit;
import net.mapeadores.basedefiches.tools.thesaurus.ComparaisonEngine;
import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class AnalyseCommand extends AbstractThesaurusCommand {

    public final short COMPARAISON_ANALYSE = 1;
    public final short RECOUVREMENT_ANALYSE = 2;
    public final static String TYPE_PARAMNAME = "type";
    public final static String OTHERTHESAURUS_PARAMNAME = "otherthesaurus";
    public final static String LANG_PARAMNAME = "lang";
    private short typeAnalyse;
    private Thesaurus otherThesaurus;
    private int lang = LangInteger.FRENCH;

    public AnalyseCommand() {
    }

    protected void doThesaurusCommand() {
        switch (typeAnalyse) {
            case COMPARAISON_ANALYSE:
                List comparaisonList = ComparaisonEngine.getComparaisonItemList(thesaurus, otherThesaurus, lang);
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.COMPARAISON_PAGE);
                responseParameters.putParameter("Comparaison", comparaisonList);
                responseParameters.putParameter("Lang", new Integer(lang));
                break;
            case RECOUVREMENT_ANALYSE:
                List<RecouvrementUnit> recouvrementList = RecouvrementEngine.getRecouvrementItemList(thesaurus, lang);
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.RECOUVREMENT_PAGE);
                responseParameters.putParameter("Recouvrement", recouvrementList);
                responseParameters.putParameter("Lang", new Integer(lang));
                break;
        }
    }

    public static String getCommandString() {
        return "Analyse";
    }

    public boolean needMotcleInit() {
        return false;
    }

    protected void checkThesaurusParameters() throws ParameterException {
        typeAnalyse = getAnalyseType();
        switch (typeAnalyse) {
            case COMPARAISON_ANALYSE:
                lang = getLangParameter(LANG_PARAMNAME, true);
                String param_destination = getMandatoryParameter(OTHERTHESAURUS_PARAMNAME);
                otherThesaurus = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), param_destination);
                if (otherThesaurus == null) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, OTHERTHESAURUS_PARAMNAME + " = " + param_destination);
                }
                break;
            case RECOUVREMENT_ANALYSE:
                lang = getLangParameter(LANG_PARAMNAME, true);
                break;
        }
    }

    private short getAnalyseType() throws ParameterException {
        String typeParam = getMandatoryParameter(TYPE_PARAMNAME);
        if (typeParam.equals("comp")) {
            return COMPARAISON_ANALYSE;
        } else if (typeParam.equals("rec")) {
            return RECOUVREMENT_ANALYSE;
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TYPE_PARAMNAME + " = " + typeParam);
        }
    }

}
