/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.managers.policies.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class CleaningCommand extends AbstractThesaurusCommand {

    public CleaningCommand() {
    }

    protected void doThesaurusCommand() {
        Integer[] result = clean();
        responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
        responseParameters.setDoneMessage("thesaurus_msg_nettoyage2", result);
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (!checkConfirmPassword()) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.CLEANINGCONFIRMATION_PAGE);
            responseParameters.setErrorMessage("global_err_password", null);
            return;
        }
    }

    public static String getCommandString() {
        return "Cleaning";
    }

    public boolean needMotcleInit() {
        return false;
    }

    private Integer[] clean() {
        int nbretransfert = 0;
        ThesaurusPolicy thesaurusPolicy = bdfServer.getPolicyManager().getThesaurusPolicy(thesaurus.getSubsetKey());
        if (thesaurusPolicy.getAjoutPolicy() == ThesaurusPolicy.AJOUT_VERIFICATION) {
            Thesaurus[] verifThesaurusArray = PolicyUtils.getVerifThesaurusArray(bdfServer.getBasedefiches(), thesaurusPolicy);
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle != null) {
                    boolean done = transfer(motcle, verifThesaurusArray);
                    if (done) {
                        nbretransfert++;
                    }
                }
            }
        }
        int nbresuppression = ThesaurusUtils.removeRemoveableMotcle(thesaurusEditor);
        int nbrefusion = 0;
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle != null) {
                    boolean done = merge(motcle);
                    if (done) {
                        nbrefusion++;
                    }
                }
            }
        }
        Integer[] result = new Integer[3];
        result[0] = new Integer(nbretransfert);
        result[1] = new Integer(nbresuppression);
        result[2] = new Integer(nbrefusion);
        return result;
    }

    boolean merge(Motcle motcle) {
        int libelleCount = motcle.getLibelleCount();
        if (libelleCount != 1) {
            return false;
        }
        Libelle libelle = motcle.getLibelleByIndex(0);
        Motcle destination = thesaurus.getMotcleByLibelle(libelle.getLibelleString(), libelle.getLangInteger());
        if (destination.equals(motcle)) {
            return false;
        }
        Motcle source = motcle;
        if (ThesaurusUtils.isDescendant(destination, motcle)) {
            source = destination;
            destination = motcle;
        }
        try {
            bdfServer.getBasedefichesEditor().merge(source, destination);
        } catch (ParentRecursivityException pre) {
            throw new ImplementationException(pre);
        }
        return true;
    }

    boolean transfer(Motcle motcle, Thesaurus[] verifThesaurusArray) {
        int libelleCount = motcle.getLibelleCount();
        if (libelleCount != 1) {
            return false;
        }
        Libelle libelle = motcle.getLibelleByIndex(0);
        for (int i = 0; i < verifThesaurusArray.length; i++) {
            Motcle destination = verifThesaurusArray[i].getMotcleByLibelle(libelle.getLibelleString(), libelle.getLangInteger());
            if (destination != null) {
                try {
                    bdfServer.getBasedefichesEditor().merge(motcle, destination);
                } catch (ParentRecursivityException pre) {
                    throw new ImplementationException(pre);
                }
                return true;
            }
        }
        return false;
    }

}
