/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import java.util.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.response.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class CloneCommand extends AbstractThesaurusCommand {

    public static String NEWTHESAURUS_PARAMNAME = "newthesaurus";
    private SubsetKey newThesaurusKey;
    ThesaurusEditor newThesaurusEditor;
    ThesaurusMetadataEditor newThesaurusMetadataEditor;

    public CloneCommand() {
    }

    protected void doThesaurusCommand() {
        bdfServer.bufferBasedefichesListener();
        try {
            newThesaurusEditor = bdfServer.getBasedefichesEditor().createThesaurus(newThesaurusKey, thesaurus.getThesaurusMetadata().getThesaurusType());
            newThesaurusMetadataEditor = newThesaurusEditor.getThesaurusMetadataEditor();
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
        int[] langArray = thesaurusMetadata.getAuthorizedLangIntegerArray();
        if (langArray != null) {
            newThesaurusMetadataEditor.setAuthorizedLangIntegerArray(langArray);
        }
        int intituleCount = thesaurusMetadata.getIntituleCount();
        for (int i = 0; i < intituleCount; i++) {
            Intitule intitule = thesaurusMetadata.getIntitule(i);
            Intitule newIntitule = newThesaurusMetadataEditor.createIntitule(intitule.getName());
            int libelleCount = intitule.getLibelleCount();
            for (int j = 0; j < libelleCount; j++) {
                Libelle libelle = intitule.getLibelleByIndex(j);
                String newLib = libelle.getLibelleString() + " (2)";
                newThesaurusMetadataEditor.setIntituleLibelle(newIntitule, newLib, libelle.getLangInteger());
            }
        }
        MotcleList list = thesaurus.getFirstLevelList();
        for (int i = 0; i < list.getMotcleCount(); i++) {
            cloneMotcle(list.getMotcle(i), null);
        }
        bdfServer.flushBasedefichesListener();
        responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
        String[] val = new String[2];
        val[0] = thesaurus.getSubsetKey().toString();
        val[1] = newThesaurusKey.toString();
        responseParameters.setDoneMessage("thesaurus_done_clone", val);
    }

    protected void checkThesaurusParameters() throws ParameterException {
        String newThesaurusName = getMandatoryParameter(NEWTHESAURUS_PARAMNAME);
        try {
            newThesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, newThesaurusName);
            if (bdfServer.getBasedefiches().getSubset(newThesaurusKey) != null) {
                responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
                responseParameters.setErrorMessage("thesaurus_err_wrongthesaurusname", newThesaurusKey.toString());
                return;
            }
        } catch (java.text.ParseException pe) {
            responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
            responseParameters.setErrorMessage("thesaurus_err_wrongthesaurusname", newThesaurusName);
            return;
        }
    }

    public static String getCommandString() {
        return "Clone";
    }

    public boolean needMotcleInit() {
        return false;
    }

    private void cloneMotcle(Motcle motcle, Motcle parentMotcle) {
        Motcle newMotcle = null;
        try {
            newMotcle = newThesaurusEditor.createMotcle(motcle.getIdths(), motcle.getIdalpha());
        } catch (ExistingIdException eie) {
            throw new ShouldNotOccurException("clone motcle");
        } catch (ParseException pe) {
            throw new ShouldNotOccurException("clone motcle");
        }
        if (parentMotcle != null) {
            try {
                newThesaurusEditor.setParent(newMotcle, parentMotcle);
            } catch (ParentRecursivityException pre) {
                throw new ShouldNotOccurException("clone motcle");
            }
        }
        int libelleCount = motcle.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcle.getLibelleByIndex(i);
            newThesaurusEditor.setLibelle(newMotcle, libelle.getLibelleString(), libelle.getLangInteger());
        }
        MotcleList children = motcle.getChildList();
        for (int i = 0; i < children.getMotcleCount(); i++) {
            cloneMotcle(children.getMotcle(i), newMotcle);
        }
    }

}
