/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractBdfCommand {

    public static final String ADVANCED_PAGE_PARAMVALUE = "advanced";
    public static final String MOTCLECREATIONFORM_PAGE_PARAMVALUE = "motclecreation";
    public static final String CLEANING_PAGE_PARAMVALUE = "cleaning";
    public static final String FORM_PAGE_PARAMVALUE = "form";
    public static final String MOTCLEINDEXATION_PAGE_PARAMVALUE = "motcleindexation";
    public static final String THESAURUSCREATIONFORM_PAGE_PARAMVALUE = "thesauruscreation";
    public static final String PONDERATION_PAGE_PARAMVALUE = "ponderation";
    public static final String IDALPHASORT_PAGE_PARAMVALUE = "idalphasort";
    public static final String CORPUSASC_TRI_PARAMVALUE = "corpusasc";
    public static final String CORPUSDESC_TRI_PARAMVALUE = "corpusdesc";
    public static final String ASC_TRI_PARAMVALUE = "asc";
    public static final String DESC_TRI_PARAMVALUE = "desc";
    public static final String TRI_PARAMNAME = "tri";
    public static final String FILTRE_PARAMNAME = "filtre";
    public static final String LIBLANG_PARAMNAME = "liblang";
    public final static String NOLIB_OPTION = "nolib";
    public final static String LIB_OPTION = "lib";
    private Thesaurus thesaurus;
    private Motcle motcle;
    private String triParamValue;
    private String filtreParamValue;
    private Integer liblangParamValue;

    public DefaultCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        short pageType = getPageType();
        int thesaurusLangInteger = bdfUser.getWorkingLangInteger();
        if (thesaurus != null) {
            thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, thesaurusLangInteger);
        }
        ThesaurusResponseParameters responseParameters = new ThesaurusResponseParameters(bdfServer, bdfUser, thesaurusLangInteger);
        if (motcle != null) {
            responseParameters.setMotcle(motcle);
        } else if (thesaurus != null) {
            responseParameters.setThesaurus(thesaurus);
        }
        if (triParamValue != null) {
            responseParameters.putParameter(TRI_PARAMNAME, triParamValue);
        }
        if (filtreParamValue != null) {
            responseParameters.putParameter(FILTRE_PARAMNAME, filtreParamValue);
        }
        if (liblangParamValue != null) {
            responseParameters.putParameter(LIBLANG_PARAMNAME, liblangParamValue);
        }
        responseParameters.setHtmlPageCode(pageType);
        return responseParameters;
    }

    public static String getCommandString() {
        return null;
    }

    private void initThesaurus() throws ParameterException {
        thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
    }

    private void initMotcle() throws ParameterException {
        thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
        motcle = RequestUtils.getMotcle(requestMap, thesaurus);
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            if (RequestUtils.hasMotcleParameter(requestMap)) {
                initMotcle();
                return ThesaurusResponseParameters.MOTCLE_PAGE;
            } else {
                initThesaurus();
                initFiltreParameter();
                initLibLangParameter();
                return ThesaurusResponseParameters.THESAURUS_PAGE;
            }
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(ADVANCED_PAGE_PARAMVALUE)) {
            initMotcle();
            return ThesaurusResponseParameters.ADVANCED_PAGE;
        } else if (page_param.equals(MOTCLEINDEXATION_PAGE_PARAMVALUE)) {
            initMotcle();
            return ThesaurusResponseParameters.MOTCLEINDEXATION_PAGE;
        } else if (page_param.equals(MOTCLECREATIONFORM_PAGE_PARAMVALUE)) {
            initThesaurus();
            if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                return ThesaurusResponseParameters.NVMTCLBYIDALPHA_PAGE;
            } else {
                return ThesaurusResponseParameters.NVIDTHS_PAGE;
            }
        } else if (page_param.equals(CLEANING_PAGE_PARAMVALUE)) {
            initThesaurus();
            return ThesaurusResponseParameters.CLEANINGCONFIRMATION_PAGE;
        } else if (page_param.equals(FORM_PAGE_PARAMVALUE)) {
            initThesaurus();
            return ThesaurusResponseParameters.THESAURUSFORM_PAGE;
        } else if (page_param.equals(THESAURUSCREATIONFORM_PAGE_PARAMVALUE)) {
            return ThesaurusResponseParameters.THESAURUSCREATIONFORM_PAGE;
        } else if (page_param.equals(PONDERATION_PAGE_PARAMVALUE)) {
            initMotcle();
            triParamValue = getTriParamValue(requestMap);
            return ThesaurusResponseParameters.PONDERATION_PAGE;
        } else if (page_param.equals(IDALPHASORT_PAGE_PARAMVALUE)) {
            initThesaurus();
            return ThesaurusResponseParameters.IDALPHASORT_PAGE;
        } else {
            throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
        }
    }

    public static String getPageTypeString(short pageType) {
        switch (pageType) {
            case ThesaurusResponseParameters.MOTCLE_PAGE:
                return "";
            case ThesaurusResponseParameters.THESAURUSFORM_PAGE:
                return "";
            case ThesaurusResponseParameters.ADVANCED_PAGE:
                return ADVANCED_PAGE_PARAMVALUE;
            case ThesaurusResponseParameters.MOTCLEINDEXATION_PAGE:
                return MOTCLEINDEXATION_PAGE_PARAMVALUE;
            case ThesaurusResponseParameters.THESAURUSCREATIONFORM_PAGE:
                return THESAURUSCREATIONFORM_PAGE_PARAMVALUE;
            case ThesaurusResponseParameters.PONDERATION_PAGE:
                return PONDERATION_PAGE_PARAMVALUE;
            default:
                throw new SwitchException("pageType = " + pageType);
        }
    }

    public static String getTriParamValue(RequestMap requestMap) throws ParameterException {
        String value = requestMap.getParameter(TRI_PARAMNAME);
        if (value == null) {
            return CORPUSASC_TRI_PARAMVALUE;
        }
        if (value.equals(CORPUSASC_TRI_PARAMVALUE)) {
            return value;
        }
        if (value.equals(CORPUSDESC_TRI_PARAMVALUE)) {
            return value;
        }
        if (value.equals(ASC_TRI_PARAMVALUE)) {
            return value;
        }
        if (value.equals(DESC_TRI_PARAMVALUE)) {
            return value;
        }
        throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TRI_PARAMNAME + " = " + value);
    }

    public static String getFiltreParamValue(RequestMap requestMap) throws ParameterException {
        String value = requestMap.getParameter(FILTRE_PARAMNAME);
        if (value == null) {
            return null;
        }
        if (value.equals(NOLIB_OPTION)) {
            return value;
        }
        if (value.equals(LIB_OPTION)) {
            return value;
        }
        throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, FILTRE_PARAMNAME + " = " + value);
    }

    public boolean needSynchronisation() {
        return false;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ThesaurusResponsePrinterFactory();
    }

    private void initFiltreParameter() throws ParameterException {
        filtreParamValue = getFiltreParamValue(requestMap);
    }

    private void initLibLangParameter() throws ParameterException {
        String paramString = requestMap.getParameter(LIBLANG_PARAMNAME);
        if (paramString == null) {
            return;
        }
        try {
            int langInteger = LangInteger.parse(paramString);
            liblangParamValue = new Integer(langInteger);
        } catch (LangIntegerException lie) {
            throw new ParameterException(ParameterException.PARAMETER_NOTLANG, LIBLANG_PARAMNAME + " = " + paramString);
        }
    }

}
