/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import net.mapeadores.basedefiches.ExistingIdException;
import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class IdalphaCommand extends AbstractThesaurusCommand {

    public final static String IDALPHA_PARAMNAME = "idalpha";
    String param_nvidalpha;

    public IdalphaCommand() {
    }

    protected void doThesaurusCommand() {
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
        try {
            boolean done = thesaurusEditor.setIdalpha(motcle, param_nvidalpha);
            if (done) {
                responseParameters.setDoneMessage("thesaurus_done_idalpha", null);
            }
        } catch (ExistingIdException eia) {
            throw new ShouldNotOccurException("test done before");
        } catch (java.text.ParseException pe) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            throw new ParameterException(ParameterException.IDALPHA_NOTAVAILABLE, thesaurus.getSubsetKey().getSubsetName());
        }
        param_nvidalpha = requestMap.getParameter(IDALPHA_PARAMNAME);
        if (param_nvidalpha == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, IDALPHA_PARAMNAME);
        }
        param_nvidalpha = StringUtils.cleanString(param_nvidalpha);
        if (param_nvidalpha.length() == 0) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
            responseParameters.setErrorMessage("tthesaurus_err_idalphavide", null);
            return;
        }
        if (!ThesaurusUtils.isValidIdalpha(param_nvidalpha)) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_wrongidalpha", param_nvidalpha);
            return;
        }
        Motcle other = thesaurus.getMotcleByIdalpha(param_nvidalpha);
        if (other != null) {
            if (!other.equals(motcle)) {
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
                responseParameters.setErrorMessage("thesaurus_err_idalphadoublon", param_nvidalpha);
                return;
            }
        }
    }

    public static String getCommandString() {
        return "Idalpha";
    }

    public boolean needMotcleInit() {
        return true;
    }

}
