/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class MergeCommand extends AbstractThesaurusCommand {

    public final static String DESTINATIONTHESAURUS_PARAMNAME = "destinationthesaurus";
    public final static String DESTINATIONIDMTCL_PARAMNAME = "destinationidmtcl";
    protected Thesaurus destinationThesaurus;
    protected Motcle destinationMotcle;
    int step;

    public MergeCommand() {
    }

    protected void doThesaurusCommand() {
        bdfUser.setParameterValue("thesaurus.merge.last", destinationThesaurus.getSubsetKey());
        if (step == 1) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MERGECONFIRMATION_PAGE);
        } else if (step == 2) {
            try {
                int langInteger = bdfUser.getWorkingLangInteger();
                Object[] objs = new Object[2];
                objs[0] = getCompleteLibelle(motcle, langInteger);
                objs[1] = getCompleteLibelle(destinationMotcle, langInteger);
                bdfServer.getBasedefichesEditor().merge(motcle, destinationMotcle);
                String messageKey = (destinationThesaurus.equals(thesaurus)) ? "thesaurus_done_merge_interne" : "thesaurus_done_merge_externe";
                responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
                responseParameters.setDoneMessage(messageKey, objs);
            } catch (ParentRecursivityException pre) {
                throw new ShouldNotOccurException("test done before");
            }
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        step = getStepNumber();
        String param_destination = getMandatoryParameter(DESTINATIONTHESAURUS_PARAMNAME);
        destinationThesaurus = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), param_destination);
        if (destinationThesaurus == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, DESTINATIONTHESAURUS_PARAMNAME + " = " + param_destination);
        }
        responseParameters.putParameter(DESTINATIONTHESAURUS_PARAMNAME, destinationThesaurus);
        try {
            destinationMotcle = RequestUtils.getMtclByIdmtcl(requestMap, destinationThesaurus, DESTINATIONIDMTCL_PARAMNAME);
            responseParameters.putParameter(DESTINATIONIDMTCL_PARAMNAME, destinationMotcle);
        } catch (ParameterException pe) {
            short type = pe.getType();
            if ((type != ParameterException.PARAMETER_WRONGVALUE) && (type != ParameterException.PARAMETER_NOTINTEGER)) {
                throw pe;
            }
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.ADVANCED_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_idmtclfusion", requestMap.getParameter(DESTINATIONIDMTCL_PARAMNAME));
            return;
        }
        if (ThesaurusUtils.isDescendant(destinationMotcle, motcle)) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.ADVANCED_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_fusionrecursive", requestMap.getParameter(DESTINATIONIDMTCL_PARAMNAME));
            return;
        }
        if (step == 2) {
            if (!checkConfirmPassword()) {
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MERGECONFIRMATION_PAGE);
                responseParameters.setErrorMessage("global_err_password", null);
                return;
            }
        }

    }

    public static String getCommandString() {
        return "Merge";
    }

    public boolean needMotcleInit() {
        return true;
    }

}
