/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.managers.policies.*;
import net.mapeadores.basedefiches.thesaurus.*;


/**
 *
 * @author Vincent Calame
 */
public class MotcleRemoveCommand extends AbstractThesaurusCommand {

    boolean isLaxPolicy;

    public MotcleRemoveCommand() {
    }

    protected void doThesaurusCommand() {
        if (isLaxPolicy) {
            ThesaurusUtils.prepareToRemove(bdfServer.getBasedefichesEditor(), motcle);
        }
        thesaurusEditor.removeMotcle(motcle);
        responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
        responseParameters.setDoneMessage("thesaurus_msg_suppressionoui", null);
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (!checkConfirmPassword()) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.ADVANCED_PAGE);
            responseParameters.setErrorMessage("global_err_password", null);
            return;
        }
        PolicyManager policyManager = bdfServer.getPolicyManager();
        ThesaurusPolicy thesaurusPolicy = policyManager.getThesaurusPolicy(thesaurus.getSubsetKey());
        isLaxPolicy = ThesaurusPolicy.isLaxPolicy(thesaurusPolicy.getAjoutPolicy());
        if ((!isLaxPolicy) && (!bdfServer.getBasedefiches().isRemoveable(motcle))) {
            throw new ParameterException(ParameterException.MOTCLE_NOTREMOVEABLE, null);
        }
    }

    public static String getCommandString() {
        return "MotcleRemove";
    }

    public boolean needMotcleInit() {
        return true;
    }

}
