/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class MoveCommand extends AbstractThesaurusCommand {

    public final static String DESTINATIONTHESAURUS_PARAMNAME = "destinationthesaurus";
    protected Thesaurus destination;
    int step;

    public MoveCommand() {
    }

    protected void doThesaurusCommand() {
        bdfUser.setParameterValue("thesaurus.move.last", destination.getSubsetKey());
        responseParameters.putParameter(DESTINATIONTHESAURUS_PARAMNAME, destination);
        if (step == 1) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOVECONFIRMATION_PAGE);
        } else if (step == 2) {
            int langInteger = bdfUser.getWorkingLangInteger();
            Object[] objs = new Object[3];
            objs[0] = getCompleteLibelle(motcle, langInteger);
            objs[1] = ThesaurusUtils.getThesaurusIntitule(destination, langInteger);

            Motcle nouveau = ThesaurusUtils.move(bdfServer.getBasedefichesEditor(), motcle, destination);
            objs[2] = new Integer(nouveau.getIdths());
            responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
            responseParameters.setDoneMessage("thesaurus_done_move", objs);
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        step = getStepNumber();
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            throw new ParameterException(ParameterException.CMD_UNVALAIBLE_IDALPHA, getCommandString());
        }
        String param_destination = requestMap.getParameter(DESTINATIONTHESAURUS_PARAMNAME);
        if (param_destination == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, DESTINATIONTHESAURUS_PARAMNAME);
        }
        destination = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), param_destination);
        if (destination == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, DESTINATIONTHESAURUS_PARAMNAME + " = " + param_destination);
        }
        if (destination.equals(thesaurus)) {
            throw new ParameterException(ParameterException.PARAMETER_EQUALSTHESAURUS, DESTINATIONTHESAURUS_PARAMNAME);
        }

        if (step == 2) {
            if (!checkConfirmPassword()) {
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOVECONFIRMATION_PAGE);
                responseParameters.setErrorMessage("global_err_password", null);
                return;
            }
        }
    }

    public static String getCommandString() {
        return "Move";
    }

    public boolean needMotcleInit() {
        return true;
    }

}
