/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.thesaurus.*;


/**
 *
 * @author Vincent Calame
 */
public class ParentCommand extends AbstractThesaurusCommand {

    public final static String PARENT_PARAMNAME = "parent";
    Motcle parentMotcle;

    public ParentCommand() {
    }

    protected void doThesaurusCommand() {
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
        try {
            boolean done = thesaurusEditor.setParent(motcle, parentMotcle);
            if (done) {
                responseParameters.setDoneMessage("thesaurus_msg_modifparent", null);
            }
        } catch (ParentRecursivityException pre) {
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        try {
            parentMotcle = RequestUtils.getMtclByIdmtcl(requestMap, thesaurus, PARENT_PARAMNAME);
        } catch (ParameterException pe) {
            short type = pe.getType();
            if ((type != ParameterException.PARAMETER_WRONGVALUE) && (type != ParameterException.PARAMETER_NOTINTEGER)) {
                throw pe;
            }
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_idmtclparent", requestMap.getParameter(PARENT_PARAMNAME));
            return;
        }
        if ((parentMotcle != null) && (parentMotcle.equals(motcle))) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_parentluimeme", null);
            return;
        }
        if (parentMotcle != null) {
            if (ThesaurusUtils.isDescendant(parentMotcle, motcle)) {
                Object[] objserr = new Object[2];
                if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                    objserr[0] = motcle.getIdalpha();
                    objserr[1] = parentMotcle.getIdalpha();
                } else {
                    objserr[0] = new Integer(motcle.getIdths());
                    objserr[1] = new Integer(parentMotcle.getIdths());
                }
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
                responseParameters.setErrorMessage("thesaurus_err_parentrecursif", objserr);
                return;
            }
        }
    }

    public static String getCommandString() {
        return "Parent";
    }

    public boolean needMotcleInit() {
        return true;
    }

}
