/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.croisement.*;


/**
 *
 * @author Vincent Calame
 */
public class PonderationCommand extends AbstractThesaurusCommand {

    Map<SubsetKey, DefaultFicheCroisementList> listMap = new HashMap<SubsetKey, DefaultFicheCroisementList>();

    public PonderationCommand() {
    }

    protected void doThesaurusCommand() {
        updatePonderation();
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.PONDERATION_PAGE);
        responseParameters.setDoneMessage("thesaurus_done_ponderation", null);
        responseParameters.setMajLinkNeeded(false);
    }

    protected void checkThesaurusParameters() throws ParameterException {
        responseParameters.putParameter(DefaultCommand.TRI_PARAMNAME, DefaultCommand.getTriParamValue(requestMap));
    }

    public static String getCommandString() {
        return "Ponderation";
    }

    public boolean needMotcleInit() {
        return true;
    }

    private void updatePonderation() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        Set set = requestMap.getParameterNameSet();
        for (Iterator it = set.iterator(); it.hasNext();) {
            String paramName = (String) it.next();
            int idx = paramName.indexOf('_');
            if (idx == -1) {
                continue;
            }
            try {
                int idcorpus = Integer.parseInt(paramName.substring(idx + 1));
                String poidsString = requestMap.getParameter(paramName);
                int poids = 1;
                if (poidsString.length() > 0) {
                    poids = Integer.parseInt(poidsString);
                }
                if (poids < 1) {
                    continue;
                }
                Corpus corpus = BasedefichesUtils.getCorpus(basedefiches, paramName.substring(0, idx));
                if (isSlaveCorpus(corpus)) {
                    continue;
                }
                if (corpus != null) {
                    AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                    if (abregeFiche != null) {
                        DefaultFicheCroisementList abregeFicheList = getAbregeFicheList(corpus.getSubsetKey());
                        abregeFicheList.add(abregeFiche, poids);
                    }
                }
            } catch (NumberFormatException nfe) {
            }
        }
        CroisementEditor croisementsEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        for (DefaultFicheCroisementList abregeFicheList : listMap.values()) {
            croisementsEditor.updateCroisement(motcle, abregeFicheList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
        }
    }

    private DefaultFicheCroisementList getAbregeFicheList(SubsetKey corpusKey) {
        DefaultFicheCroisementList abregeFicheList = listMap.get(corpusKey);
        if (abregeFicheList == null) {
            abregeFicheList = new DefaultFicheCroisementList(corpusKey, 30);
            listMap.put(corpusKey, abregeFicheList);
        }
        return abregeFicheList;
    }

}
