/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.saisie.SaisieHtmlConstants;
import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.util.text.Libelle;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class MotclePage extends ThesaurusHtmlPage {

    public MotclePage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);

        ThesaurusMetadata thsmeta = thesaurus.getThesaurusMetadata();
        short thesaurusType = thsmeta.getThesaurusType();
        boolean withIdalpha = (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE);
        boolean isBabelien = (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE);
        int[] array = thsmeta.getAuthorizedLangIntegerArray();
        boolean multilangues = ((!isBabelien) && (array.length > 1));
        Motcle parentMotcle = motcle.getParent();
        start();
        printTitre();

        if (withIdalpha) {
            startInLineForm(motcle, IdalphaCommand.getCommandString());
            printLib("thesaurus_lib_idalpha");
            appendMediumInput(IdalphaCommand.IDALPHA_PARAMNAME, motcle.getIdalpha());
            endInLineForm("thesaurus_submit_idalpha");
            appendHr();
        }

        startInLineForm(motcle, ParentCommand.getCommandString());
        if (withIdalpha) {
            printLib("thesaurus_lib_parent_idalpha");
        } else {
            printLib("thesaurus_lib_parent_idths");
        }
        String parentValue = "";
        if (parentMotcle != null) {
            if (withIdalpha) {
                parentValue = parentMotcle.getIdalpha();
            } else {
                parentValue = String.valueOf(parentMotcle.getIdths());
            }
        }
        appendSmallInput(ParentCommand.PARENT_PARAMNAME, parentValue);
        endInLineForm("thesaurus_submit_parent");

        appendHr();

        startForm(motcle, LibelleCommand.getCommandString());
        appendH2((multilangues) ? "thesaurus_title_libelle_many" : "thesaurus_title_libelle_one");
        if (isBabelien) {
            int motcleLangInteger = ThesaurusUtils.getDefaultPreferredLangInteger(motcle);
            startP();
            startTextArea("mtcl_lib", 50, 3);
            escape(TextParser.escapeString(motcle.getLibelleString(motcleLangInteger, false)));
            endTextArea();
            appendBr();
            localize("thesaurus_frm_languemtcl");
            String value = "";
            if (motcleLangInteger != LangInteger.UNDETERMINED) {
                value = LangInteger.toString(motcleLangInteger);
            }
            appendSmallInput("mtcl_lgpref", value);
            endP();
        } else {
            startTable();
            for (int i = 0; i < array.length; i++) {
                int codelg = array[i];
                startRow();
                startCellWithTopAlign();
                if (multilangues) {
                    LangInteger.appendString(pw, codelg);
                    pw.print(" : ");
                } else {
                    doublespace();
                    doublespace();
                }
                endCell();
                startCell();
                startTextArea("mtcl_lib_" + LangInteger.toString(codelg), 50, 3);
                Libelle libelle = motcle.getLibelleByLang(codelg);
                if (libelle != null) {
                    escape(TextParser.escapeString(libelle.getLibelleString()));
                }
                endTextArea();
                endCell();
                endRow();
            }
            endTable();
        }
        endForm((multilangues) ? "thesaurus_submit_libelle_many" : "thesaurus_submit_libelle_one");
        appendHr();
        printFicheList(pw);
        end();
    }

    /**
     * Appelé par printMotclePage, écrit la liste des fiches.
     */
    protected void printFicheList(PrintWriter pw) {
        int count = 0;
        StringBuffer buf = new StringBuffer(128);
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        String thesaurusName = thesaurusKey.getSubsetName();
        Basedefiches basedefiches = thesaurus.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int j = 0; j < corpusCount; j++) {
            Corpus corpus = basedefiches.getCorpus(j);
            Croisement croisement = thesaurus.getBasedefiches().getCroisement(thesaurusKey, corpus.getSubsetKey());
            CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
            int ciCount = croisementItem.getCount();
            if (ciCount == 0) {
                continue;
            }
            boolean slave = false;
            Subset masterSubset = corpus.getMasterSubset();
            if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                slave = true;
            }
            SubsetKey corpusKey = corpus.getSubsetKey();
            count = count + ciCount;
            buf.append("<li><p>");
            buf.append(corpusKey.getSubsetName());
            buf.append("</p><ul>");
            for (int i = 0; i < ciCount; i++) {
                SubsetItem subsetItem = corpus.getSubsetItemByTicket(croisementItem.getTicket(i));
                int idcorpus = subsetItem.getIdsubset();
                buf.append("<li><p>");
                buf.append(corpusKey.getSubsetName());
                buf.append("/");
                buf.append(idcorpus);
                buf.append(" <span class=\"");
                buf.append(CssClass.SPAN_SMALLLINKS);
                buf.append("\">");
                if (!slave) {
                    buf.append("[");
                    String conditioncorpus = RequestUtils.toParameterString(corpusKey, idcorpus);
                    buf.append("<a href=\"Saisie?");
                    buf.append(conditioncorpus);
                    buf.append("&amp;page=");
                    buf.append(SaisieHtmlConstants.FICHEINDEXATION_PAGE);
                    buf.append("&amp;thesaurus=");
                    buf.append(thesaurusName);
                    buf.append("\">");
                    buf.append(getEscapedLocalization("listefiches_modifier", null));
                    buf.append("</a>");
                    /*buf.append("<a href=\"Saisie?");
                    buf.append(conditioncorpus);
                    buf.append("\">");
                    HTMLUtils.appendHTML(buf,servletsBundle,"listefiches_modifier");
                    buf.append("</a>");
                    buf.append(" - <a href=\"fiches/");
                    buf.append(corpusKey.getSubId());
                    buf.append("-");
                    buf.append(idcorpus);
                    buf.append(".html");
                    buf.append("\">");
                    HTMLUtils.appendHTML(buf,servletsBundle,"listefiches_afficher");
                    buf.append("</a>");
                    buf.append(" - <a href=\"Saisie?");
                    buf.append(conditioncorpus);
                    buf.append("&amp;page=");
                    buf.append(fr.exemole.bdfserver.servlets.saisie.DefaultCommand.INDEXATION_PAGE_PARAMVALUE);
                    buf.append("\">");
                    HTMLUtils.appendHTML(buf,servletsBundle,"listefiches_indexer");
                    buf.append("</a>");*/
                    buf.append("]");
                }
                buf.append("</span></p></li>");
            }
            buf.append("</ul></li>");
        }
        if (count == 0) {
            pw.print("<p style=\"color: red;\">");
            localize("thesaurus_msg_aucuneindexation");
            pw.print("</p>");
        } else {
            pw.print("<p>");
            if (count == 1) {
                localize("thesaurus_msg_uneficheindexee");
            } else {
                localize("thesaurus_msg_fichesindexees", new Integer(count));
            }
            pw.print("</p><ul>");
            pw.print(buf.toString());
            pw.print("</ul>");
        }
    }

}
