/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.saisie.SaisieHtmlConstants;
import net.mapeadores.basedefiches.tools.thesaurus.RecouvrementUnit;
import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class RecouvrementPage extends ThesaurusHtmlPage {

    private int lang;
    private List recouvrementList;

    public RecouvrementPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        recouvrementList = (List) responseParameters.getParameter("Recouvrement");
        lang = ((Integer) responseParameters.getParameter("Lang")).intValue();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        startTable();
        for (Iterator it = recouvrementList.iterator(); it.hasNext();) {
            RecouvrementUnit recouvrementUnit = (RecouvrementUnit) it.next();
            Motcle motcle = recouvrementUnit.getMotcle();
            String lib = motcle.getLibelleString(lang, false);
            int statCount = recouvrementUnit.getStatCount();
            List motcleList = recouvrementUnit.getMotcleList();
            boolean premier = true;
            String href = "Saisie?thesaurus=" + motcle.getThesaurus().getSubsetKey().getSubsetName() + "&amp;page=" + SaisieHtmlConstants.FICHEINDEXATION_PAGE + "&amp;";
            for (Iterator it2 = motcleList.iterator(); it2.hasNext();) {
                RecouvrementUnit.MotcleItem other = (RecouvrementUnit.MotcleItem) it2.next();
                Motcle otherMotcle = other.getMotcle();
                int otherStat = other.getStatCount();
                Set commonSet = other.getCommonSet();
                startRow();

                if (premier) {
                    startCell();
                    escape(lib);
                    endCell();
                    startCell();
                    append(String.valueOf(statCount));
                    endCell();
                    premier = false;
                } else {
                    startCell();
                    endCell();
                    startCell();
                    endCell();
                }
                startCell();
                escape(otherMotcle.getLibelleString(lang, false));
                endCell();
                startCell();
                append(String.valueOf(otherStat));
                endCell();
                startCell();
                append(String.valueOf(commonSet.size()));
                endCell();

                startCell();
                escape(lib);
                endCell();
                startCell();
                append(String.valueOf(statCount));
                endCell();

                startCell();

                for (Iterator it3 = commonSet.iterator(); it3.hasNext();) {
                    AbregeFiche abregeFiche = (AbregeFiche) it3.next();
                    space();
                    startA(href + RequestUtils.toParameterString(abregeFiche), "_New");
                    append(abregeFiche.getCorpus().getSubsetKey().getSubsetName());
                    append("-");
                    append(String.valueOf(abregeFiche.getIdcorpus()));
                    endA();
                }
                endCell();

                endRow();
            }
        }
        endTable();
        end();
    }

}
