/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.managers.policies.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusFormPage extends ThesaurusHtmlPage {

    int[] workingArray;
    ThesaurusPolicy thesaurusPolicy;

    public ThesaurusFormPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(BDF_JS);
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        workingArray = langConfig.getWorkingLangIntegerArray();
        thesaurusPolicy = bdfServer.getPolicyManager().getThesaurusPolicy(thesaurus.getSubsetKey());
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();

        printTitre();
        startPostForm(thesaurus, ThesaurusIntituleCommand.getCommandString());
        printThesaurusIntituleTable(ThesaurusConstants.THESAURUS);
        endForm("thesaurus_submit_intitules");
        appendH2("thesaurus_title_special");
        startUl();
        printSpecialCommandLi(DefaultCommand.CLEANING_PAGE_PARAMVALUE, "thesaurus_title_cleaning");
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            printSpecialCommandLi(DefaultCommand.IDALPHASORT_PAGE_PARAMVALUE, "thesaurus_title_idalphasort");
        }
        endUl();

        if (!ThesaurusUtils.isBabelienType(thesaurus)) {
            appendH2("thesaurus_title_langlist");
            startPostForm(thesaurus, LangArrayCommand.getCommandString());
            startP();
            printLib("thesaurus_lib_langlist");
            int[] array = thesaurusMetadata.getAuthorizedLangIntegerArray();
            appendSmallInput(LangArrayCommand.LANGARRAY_PARAMNAME, LangIntegerUtils.toString(array, ';'));
            endP();
            endForm("thesaurus_submit_langlist");
        }
        printPolicy();
        if (BasedefichesUtils.isRemoveable(thesaurus)) {
            appendHr();
            startPostForm(thesaurus, ThesaurusRemoveCommand.getCommandString());
            appendH2("thesaurus_title_thesaurusremove");
            endFormWithConfirm("thesaurus_submit_thesaurusremove");
        }
        end();
    }

    protected void printTitre() {
        startH1();
        startCode();
        append("[");
        append(thesaurus.getSubsetKey().toString());
        append("]");
        endCode();
        if (isWithJQuery()) {
            printTiret();
            BdfHtmlUtils.printChangeSubsetSelect(this, thesaurus, DefaultCommand.FORM_PAGE_PARAMVALUE);
        }
        endH1();
        String thesaurusType = ThesaurusUtils.thesaurusTypeToString(thesaurusMetadata.getThesaurusType());
        appendP(CssClass.P_SUBTITLE, "thesaurus_thesaurustype_" + thesaurusType);
        printMessage();
    }

    private void printThesaurusIntituleTable(String name) {
        appendP(null, "thesaurus_thesaurusintitule_" + name);
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String intitule = getLibelleString(thesaurusMetadata.getIntituleByName(name), langInteger);
            printLangFieldRow("intitule_" + name, langInteger, intitule);
        }
        endTable();
    }

    private String getLibelleString(Intitule sphereIntitule, int langInteger) {
        if (sphereIntitule == null) {
            return "";
        }
        String s = sphereIntitule.getLibelleString(langInteger, false);
        if (s == null) {
            return "";
        }
        return s;
    }

    private void printSpecialCommandLi(String pageName, String titleKey) {
        String href = "Thesaurus?" + RequestUtils.toParameterString(thesaurus) + "&amp;page=" + pageName;
        startLi();
        startP();
        appendA(href, titleKey);
        endP();
        endLi();
    }

    private void printPolicy() {
        short thesaurusType = thesaurus.getThesaurusMetadata().getThesaurusType();
        if (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE) {
            return;
        } else if (thesaurusType == ThesaurusMetadata.MULTI_TYPE) {
            printMultiPolicy();
        } else if (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE) {
            printBabelienPolicy();
        }
    }

    private void printMultiPolicy() {
        List<SubsetKey> babelienList = ThesaurusUtils.getBabelienThesaurusList(bdfServer.getBasedefiches());
        if (babelienList.size() == 0) {
            return;
        }
        appendH2("thesaurus_title_policy");
        startPostForm(thesaurus, ThesaurusPolicyCommand.getCommandString());
        short currentAjoutPolicy = thesaurusPolicy.getAjoutPolicy();
        printPolicyRadio(ThesaurusPolicy.AJOUT_AUCUN, currentAjoutPolicy);
        printTransferRadio(thesaurusPolicy.getTransferThesaurusKey(), babelienList);
        endForm("thesaurus_submit_policy");
    }

    private void printBabelienPolicy() {
        List<SubsetKey> babelienList = ThesaurusUtils.getBabelienThesaurusList(bdfServer.getBasedefiches());
        List<SubsetKey> noIdalphaList = ThesaurusUtils.getNoIdalphaThesaurusList(bdfServer.getBasedefiches());
        appendH2("thesaurus_title_policy");
        startPostForm(thesaurus, ThesaurusPolicyCommand.getCommandString());
        short currentAjoutPolicy = thesaurusPolicy.getAjoutPolicy();
        printPolicyRadio(ThesaurusPolicy.AJOUT_AUCUN, currentAjoutPolicy);
        if (babelienList.size() >= 0) {
            printTransferRadio(thesaurusPolicy.getTransferThesaurusKey(), babelienList);
        }
        printPolicyRadio(ThesaurusPolicy.AJOUT_AUTORISE, currentAjoutPolicy);
        if (noIdalphaList.size() >= 0) {
            printVerifRadio(thesaurusPolicy.getVerifThesaurusKeyList(), noIdalphaList);
        }
        endForm("thesaurus_submit_policy");

    }

    private void printPolicyRadio(short ajoutPolicy, short checkedAjoutPolicy) {
        String name = ThesaurusPolicy.ajoutPolicyToString(ajoutPolicy);
        startP();
        appendRadioInput(ThesaurusPolicyCommand.POLICY_PARAMNAME, name, (ajoutPolicy == checkedAjoutPolicy), "thesaurus_policy_" + name);
        endP();
    }

    private void printTransferRadio(SubsetKey currentTransferKey, List<SubsetKey> transferList) {
        String name = ThesaurusPolicy.ajoutPolicyToString(ThesaurusPolicy.AJOUT_TRANSFERT);
        startP();
        appendRadioInput(ThesaurusPolicyCommand.POLICY_PARAMNAME, name, (currentTransferKey != null), "thesaurus_policy_" + name);
        space();
        startSelect(ThesaurusPolicyCommand.TRANSFERKEY_PARAMNAME);
        for (SubsetKey thesaurusKey : transferList) {
            startOption(thesaurusKey.getSubsetName(), (currentTransferKey != null) && (thesaurusKey.equals(currentTransferKey)));
            append(thesaurusKey.toString());
            endOption();
        }
        endSelect();
        endP();
    }

    private void printVerifRadio(List currentList, List<SubsetKey> availableList) {
        String name = ThesaurusPolicy.ajoutPolicyToString(ThesaurusPolicy.AJOUT_VERIFICATION);
        startP();
        appendRadioInput(ThesaurusPolicyCommand.POLICY_PARAMNAME, name, (currentList != null), "thesaurus_policy_" + name);
        space();
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setName(ThesaurusPolicyCommand.VERIFKEYLIST_PARAMNAME);
        selectAtt.setSize("5");
        selectAtt.setMultiple(true);
        startSelect(selectAtt);
        for (SubsetKey thesaurusKey : availableList) {
            startOption(thesaurusKey.getSubsetName(), ((currentList != null) && (currentList.contains(thesaurusKey))));
            append(thesaurusKey.toString());
            endOption();
        }
        endSelect();
        endP();
    }

}
