/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.util;

import fr.exemole.bdfserver.html.utils.SubsetGroupingPrinter;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.*;
import java.util.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.conditions.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.html.forms.CommonFormHtml;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.servlets.BdfHtmlPage;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectWriter {

    public final static String DETAIL_CLASS = "FS_Detail";
    public final static String ANDOR_CLASS = "FS_AndOr";
    private BdfHtmlPage hp;
    private FicheSelectOptions ficheSelectOptions;
    private Basedefiches basedefiches;
    private BdfServer bdfServer;
    private int workingLangInteger;
    private HTMLAttributes labelAttributes;
    private HTMLAttributes col3Attributes;
    private HTMLAttributes textAttributes;
    private BdfUser bdfUser;
    private SubsetGroupingPrinter stw;
    private HTMLAttributes groupTitleAttributes;
    private HTMLAttributes groupDivAttributes;

    public FicheSelectWriter(BdfServer bdfServer, BdfHtmlPage htmlPrinter, int workingLangInteger) {
        this.hp = htmlPrinter;
        this.basedefiches = bdfServer.getBasedefiches();
        this.workingLangInteger = workingLangInteger;
        this.bdfServer = bdfServer;
        labelAttributes = new HTMLAttributes();
        labelAttributes.setClasses("FS_Label");
        col3Attributes = new HTMLAttributes();
        col3Attributes.setClasses("FS_Col3");
        textAttributes = new HTMLAttributes();
        textAttributes.setClasses("FS_Input");
        groupTitleAttributes = new HTMLAttributes();
        groupTitleAttributes.setClasses("FS_GroupTitle");
        groupDivAttributes = new HTMLAttributes();
        groupDivAttributes.setClasses("FS_GroupDiv");
        stw = new SubsetGroupingPrinter(basedefiches, hp, workingLangInteger);
   }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.ficheSelectOptions = ficheSelectOptions;
    }

    public void setBdfUser(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
    }

    public void printFieldSets() {
        if (hp.isWithJQuery()) {
            CommonFormHtml.printSLCCoreLocValues(hp);
        }
        hp.startDiv(col3Attributes);
        printIdcorpusFieldSet();
        printCorpusFieldSet();
        hp.endDiv();
        hp.startDiv(col3Attributes);
        printTitreFieldSet();
        printRedacteursFieldSet();
        hp.endDiv();
        hp.startDiv(col3Attributes);
        printMotsclesFieldSet();
        hp.endDiv();
    }

    public void printIdcorpusFieldSet() {
        String idcorpusValue = "";
        RangeList rangeList = ficheSelectOptions.getRangeList();
        if (rangeList != null) {
            idcorpusValue = RangeUtils.positiveRangetoString(rangeList);
        }
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_ParIdcorpus");
        hp.startFieldset(att);
        hp.appendLegend("selection_lib_idcorpus");
        addInput(FicheSelectParser.IDCORPUS_PARAMNAME, idcorpusValue);
        hp.endFieldset();
    }

    public void printTitreFieldSet() {
        String titreValue = "";
        ConditionGroup titreConditionGroup = ficheSelectOptions.getTitreConditionGroup();
        if (titreConditionGroup != null) {
            titreValue = Conditions.conditionGroupToString(titreConditionGroup);
        }
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_ParTitre");
        hp.startFieldset(att);
        hp.appendLegend("selection_lib_titre");
        addInput(FicheSelectParser.TITRE_PARAMNAME, titreValue);
        hp.endFieldset();
    }

    public void printSeparator() {
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_Separator");
        hp.appendHr(att);
    }

    public void printCorpusFieldSet() {
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        boolean empty = (corpusKeyArray == null);
        String detailId = "corpusList";
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_ParCorpus");
        hp.startFieldset(att);
        hp.appendLegend("selection_lib_corpus");

        addRadios(FicheSelectParser.CORPUS_ALL_PARAMNAME, detailId, empty);

        HashSet<SubsetKey> selectedSet = new HashSet<SubsetKey>();
        if (!empty) {
            for (int j = 0; j < corpusKeyArray.length; j++) {
                selectedSet.add(corpusKeyArray[j]);
            }
        }
        SubsetGrouping subsetGrouping = SubsetGroupingUtils.checkCorpusTree(bdfUser, bdfServer.getUIManager().getCorpusSubsetKeyTree());
        stw.setSubsetGrouping(subsetGrouping);
        stw.setSelectedSet(selectedSet);
        stw.setDisabledSet(null);
        stw.setOnlySubIdValues(true);
        stw.setSubsetStringPrint(false);
        HTMLAttributes checkAtt = new HTMLAttributes();
        checkAtt.setName(FicheSelectParser.CORPUS_SELECTION_PARAMNAME);
        stw.printCheckBoxList(checkAtt, labelAttributes, groupTitleAttributes, groupDivAttributes);
        hp.endDiv();
        hp.endFieldset();

    }

    public void printRedacteursFieldSet() {
        String appelantId = "appelantRedacteurs";
        RedacteurKey[] redacteurKeyArray = ficheSelectOptions.getRedacteurKeyArray();
        SubsetKey[] sphereKeyArray = ficheSelectOptions.getSphereKeyArray();
        String detailId = "redacteursList";
        boolean empty = ((redacteurKeyArray == null) && (sphereKeyArray == null));
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_ParRedacteurs");
        hp.startFieldset(att);
        hp.appendLegend("selection_lib_redacteurs");

        addRadios(FicheSelectParser.REDACTEURS_ALL_PARAMNAME, detailId, empty);
        textAttributes.setId(appelantId);
        addInput(FicheSelectParser.REDACTEURS_SELECTION_PARAMNAME, getRedacteursString());
        textAttributes.setId(null);
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setClasses(ANDOR_CLASS);
        hp.startDiv(divAtt);
        if (hp.isWithJQuery()) {
            HTMLAttributes toolAttributes = new HTMLAttributes();
            toolAttributes.addClass("JQ_SLC_PIOCHE");
            toolAttributes.setId("ID_JQ_TOOL_" + appelantId);
            hp.startSpan(toolAttributes);
            hp.endSpan();
            hp.startScript(null);
            CommonFormHtml.printSLCPiocheAdd(hp, appelantId, "ID_JQ_TOOL_" + appelantId);
            CommonFormHtml.printSLCCoreTitleKeyValue(hp, appelantId, "global_link_pioche_redacteur");
            CommonFormHtml.printSLCCoreCategoryValue(hp, appelantId, "sphere");
            CommonFormHtml.printSLCCoreLimiteValue(hp, appelantId, "-1");
            if (bdfUser != null) {
                CommonFormHtml.printSLCCoreParamValue(hp, appelantId, bdfUser.getRedacteurKey().getSphereKey().toString());
            }
            hp.breakLine();
            hp.endScript();
        }
        hp.endDiv();
        hp.endDiv();
        hp.endFieldset();
    }

    private boolean containsKey(SubsetKey subsetKey, SubsetKey[] array) {
        if (array == null) {
            return false;
        }
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(subsetKey)) {
                return true;
            }
        }
        return false;
    }

    private String getRedacteursString() {
        RedacteurKey[] redacteurKeyArray = ficheSelectOptions.getRedacteurKeyArray();
        SubsetKey[] sphereKeyArray = ficheSelectOptions.getSphereKeyArray();
        if ((redacteurKeyArray == null) && (sphereKeyArray == null)) {
            return "";
        }
        SubsetKey defaultSphereKey = null;
        if (bdfUser != null) {
            defaultSphereKey = bdfUser.getRedacteurKey().getSphereKey();
        }
        StringBuffer bufredacs = new StringBuffer();
        if (redacteurKeyArray != null) {
            for (int i = 0; i < redacteurKeyArray.length; i++) {
                RedacteurKey redacteurKey = redacteurKeyArray[i];
                if ((defaultSphereKey == null) || (!defaultSphereKey.equals(redacteurKey.getSphereKey()))) {
                    bufredacs.append(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                } else {
                    bufredacs.append(redacteurKey.getIdsphereString());
                }
                bufredacs.append("; ");
            }
        }
        if (sphereKeyArray != null) {
            int length = sphereKeyArray.length;
            for (int i = 0; i < length; i++) {
                bufredacs.append('[');
                bufredacs.append(sphereKeyArray[i].getSubsetName());
                bufredacs.append(']');
                bufredacs.append("; ");
            }
        }
        return bufredacs.toString();
    }

    public void printMotsclesFieldSet() {
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId("FS_motsclesFieldSets");
        HTMLAttributes scriptAtt = new HTMLAttributes();
        scriptAtt.setType("text/javascript");
        hp.startDiv(divAttributes);
        short mode = ficheSelectOptions.getMotcleSelectMode();
        MotcleSelectOptions[] array = ficheSelectOptions.getMotcleSelectOptionsArray();
        int size = 0;
        if ((array == null) || (array.length == 0)) {
            printMotcleConditionGroupFieldSet(null, "1");
        } else {
            size = array.length;
            for (int i = 0; i < size; i++) {
                printMotcleConditionGroupFieldSet(array[i], String.valueOf(i + 1));
            }
        }
        printMotcleConditionGroupFieldSet(null, "nnn");
        printMotcleMode(mode, size);
        hp.startSpan("FS_ScriptLink");
        hp.startA("javascript:SLC_Fiche.addNewSelection()");
        hp.localize("selection_link_motsclesadd");
        hp.endA();
        hp.endSpan();
        hp.endDiv();
    }

    private void printMotcleMode(short mode, int size) {
        HTMLAttributes att = new HTMLAttributes();
        att.setClasses("FS_ScriptLink");
        att.setId("FS_motsclesMode");
        if (size < 2) {
            att.setStyle("display: none;");
        }
        hp.startSpan(att);
        hp.localize("selection_lib_motscles_mode");
        hp.appendBr();
        printAndOr(FicheSelectParser.MOTCLE_MODE_PARAMNAME, mode);
        hp.endSpan();
    }

    public void printMotcleConditionGroupFieldSet(MotcleSelectOptions motcleSelectOptions, String suffix) {
        short typeMode = Conditions.MODE_AND;
        ConditionGroup motcleConditionGroup = null;
        boolean thesaurusSelection = false;
        SubsetKey[] thesaurusKeyArray = null;
        List<Subset> subsetList = null;
        if (motcleSelectOptions != null) {
            motcleConditionGroup = motcleSelectOptions.getSourceConditionGroup();
            if (motcleConditionGroup != null) {
                typeMode = motcleConditionGroup.getMode();
            }
            thesaurusKeyArray = motcleSelectOptions.getThesaurusKeyArray();
            if (thesaurusKeyArray != null) {
                thesaurusSelection = true;
                subsetList = new ArrayList<Subset>();
                for (int i = 0; i < thesaurusKeyArray.length; i++) {
                    SubsetKey thesaurusKey = thesaurusKeyArray[i];
                    Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
                    if (thesaurus != null) {
                        subsetList.add(thesaurus);
                    }
                }
                if (subsetList.size() == 0) {
                    subsetList = null;
                    thesaurusSelection = false;
                }
            }
        }
        String detailId = "thesaurusList_" + suffix;
        HTMLAttributes att = new HTMLAttributes();
        att.setId("FS_parMotcle_" + suffix);
        att.setClasses("FS_ParThesaurus");
        if (suffix.equals("nnn")) {
            att.setStyle("display: none");
        }
        hp.startFieldset(att);
        hp.startLegend(null);
        hp.localize("selection_lib_motscles", suffix);
        hp.endLegend();

        String value = (motcleConditionGroup != null) ? Conditions.conditionGroupToString(motcleConditionGroup) : "";
        textAttributes.setId(FicheSelectParser.MOTCLE_INPUT_PREFIX + suffix);
        addInput(FicheSelectParser.MOTCLE_INPUT_PREFIX + suffix, value);
        textAttributes.setId(null);

        printAndORDiv(suffix, typeMode, subsetList);

        hp.startSpan("FS_Lib");
        hp.localize("selection_lib_thesaurus");
        hp.endSpan();
        addRadios(FicheSelectParser.MOTCLE_ALL_PREFIX + suffix, detailId, !thesaurusSelection, "_motscles");
        HashSet<SubsetKey> disabledSet = new HashSet<SubsetKey>();
        if (thesaurusSelection) {
            HTMLAttributes checkAtt = new HTMLAttributes();
            checkAtt.setName(FicheSelectParser.MOTCLE_SELECTION_PREFIX + suffix);
            checkAtt.setChecked(true);
            checkAtt.setOnClick("SLC_Fiche.checkPiocheurSelection(this);");
            for (int i = 0; i < subsetList.size(); i++) {
                Thesaurus thesaurus = (Thesaurus) subsetList.get(i);
                SubsetKey thesaurusKey = thesaurus.getSubsetKey();
                checkAtt.setValue(thesaurusKey.getSubsetName());
                hp.addCheck(checkAtt, ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger), labelAttributes);
                disabledSet.add(thesaurusKey);
            }
        }
        if (thesaurusSelection) {
            HTMLAttributes linkAtt = new HTMLAttributes();
            linkAtt.setClasses("FS_ScriptLink");
            linkAtt.setId("FS_thesaurusSelectLink_" + suffix);
            linkAtt.setHref("javascript:SLC_Fiche.displayThesaurusSelect('" + suffix + "');");
            hp.startA(linkAtt);
            hp.localize("selection_link_thesaurusselect");
            hp.endA();
        }
        HTMLAttributes selAtt = new HTMLAttributes();
        selAtt.setOnChange("SLC_Fiche.checkPiocheurSelection(this);");
        selAtt.setMultiple(true);
        selAtt.setSize("10");
        selAtt.setName(FicheSelectParser.MOTCLE_SELECTION_PREFIX + suffix);
        selAtt.setClasses("FS_ThesaurusSelect");
        selAtt.setId("FS_thesaurusSelect_" + suffix);
        if (thesaurusSelection) {
            selAtt.setStyle("display: none");
        }
        stw.setSubsetGrouping(bdfServer.getUIManager().getThesaurusSubsetKeyTree());
        stw.setOnlySubIdValues(true);
        stw.setDisabledSet(disabledSet);
        stw.printTreeSelect(selAtt);
        hp.endDiv();

        hp.endFieldset();
    }

    private void addInput(String name, String value) {
        textAttributes.setName(name);
        textAttributes.setValue(value);
        hp.appendTextInput(textAttributes);
    }

    private void addRadios(String name, String detailId, boolean empty) {
        addRadios(name, detailId, empty, detailId);
    }

    private void addRadios(String name, String detailId, boolean empty, String scriptKey) {
        HTMLAttributes radioAtt = new HTMLAttributes();
        radioAtt.setName(name);
        radioAtt.setChecked(empty);
        radioAtt.setValue("1");
        radioAtt.setOnClick("SLC_Fiche.displayDetailDivFS(this,'" + scriptKey + "')");
        hp.addRadio(radioAtt, "selection_lib_list_all", labelAttributes);
        radioAtt.setChecked(!empty);
        radioAtt.setValue("0");
        radioAtt.setId(name);
        hp.addRadio(radioAtt, "selection_lib_list_selection", labelAttributes);
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setId("FS_" + detailId);
        divAtt.setStyle((empty) ? "display: none;" : "display: block;");
        divAtt.setClasses(DETAIL_CLASS);
        hp.startDiv(divAtt);
    }

    private void printAndORDiv(String suffix, short mode, List<Subset> subsetList) {
        String appelantId = FicheSelectParser.MOTCLE_INPUT_PREFIX + suffix;
        // piocheParameters.setLinkId("FS_motclePiocheur_" + suffix);
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setClasses(ANDOR_CLASS);
        hp.startDiv(divAtt);
        printAndOr(FicheSelectParser.MOTCLE_MODE_PREFIX + suffix, mode);
        hp.space();
        if (hp.isWithJQuery()) {
            HTMLAttributes toolAttributes = new HTMLAttributes();
            toolAttributes.addClass("JQ_SLC_PIOCHE");
            toolAttributes.setId("ID_JQ_TOOL_" + appelantId);
            hp.startSpan(toolAttributes);
            hp.endSpan();
            if (!suffix.equals("nnn")) {
                hp.startScript(null);
                CommonFormHtml.printSLCPiocheAdd(hp, appelantId, "ID_JQ_TOOL_" + appelantId);
                CommonFormHtml.printSLCCoreCategoryValue(hp, appelantId, "thesaurus");
                CommonFormHtml.printSLCCoreLimiteValue(hp, appelantId, "-1");
                if (subsetList != null) {
                    StringBuilder buf = new StringBuilder();
                    int size = subsetList.size();
                    for (int i = 0; i < size; i++) {
                        Subset subset = (Subset) subsetList.get(i);
                        if (i > 0) {
                            buf.append(",");
                        }
                        buf.append(subset.getSubsetKey().getSubsetName());
                    }
                    CommonFormHtml.printSLCCoreSubsetsValue(hp, appelantId, buf.toString());
                    CommonFormHtml.printSLCCoreTitleKeyValue(hp, appelantId, "global_link_pioche_motcle_selection");
                } else {
                    CommonFormHtml.printSLCCoreTitleKeyValue(hp, appelantId, "global_link_pioche_motcle_all");
                }
                hp.breakLine();
                hp.endScript();
            }
        }
        hp.endDiv();
    }

    private void printAndOr(String name, short mode) {
        HTMLAttributes radioAtt = new HTMLAttributes();
        radioAtt.setChecked((mode == Conditions.MODE_AND));
        radioAtt.setValue("and");
        radioAtt.setName(name);
        hp.addRadio(radioAtt, "global_lib_and", null);
        hp.space();
        radioAtt.setChecked((mode == Conditions.MODE_OR));
        radioAtt.setValue("or");
        hp.addRadio(radioAtt, "global_lib_or", null);
    }

}
