/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.util;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.response.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class PiocheParameters extends AppelantParameters implements PiocheConstants {

    List subsetList;
    int limite = -1;
    short category;
    Object param;

    public PiocheParameters() {
    }

    /**
     * null ou contient au moins un élément
     */
    public List getSubsetList() {
        return subsetList;
    }

    public void setSubsetList(List subsetList) {
        this.subsetList = subsetList;
    }

    /**
     * inutile d'invoquer setCategory.
     */
    public void setSubset(Corpus corpus) {
        category = CATEGORY_CORPUS;
        subsetList = Collections.singletonList(corpus);
    }

    public void setSubset(Thesaurus thesaurus) {
        category = CATEGORY_THESAURUS;
        subsetList = Collections.singletonList(thesaurus);
    }

    public void setCategory(short category) {
        this.category = category;
    }

    public short getCategory() {
        return category;
    }

    public int getLimite() {
        return limite;
    }

    public void setLimite(int limite) {
        if (limite < 1) {
            this.limite = -1;
        } else {
            this.limite = limite;
        }
    }

    public Object getParam() {
        return param;
    }

    public void setParam(Object param) {
        this.param = param;
    }

    public static PiocheParameters newInstance(Basedefiches basedefiches, CorpusField corpusField, SubsetKey defaultSphereKey) {
        short category = getCategory(corpusField.getFicheItemType());
        if (category == 0) {
            return null;
        }
        PiocheParameters params = new PiocheParameters();
        params.setCategory(category);
        FieldKey fieldKey = corpusField.getFieldKey();
        if (fieldKey.isProprieteFieldKey()) {
            params.setLimite(1);
        } else {
            params.setLimite(-1);
        }
        if (category == CATEGORY_SPHERE) {
            SubsetKey subsetKey = (SubsetKey) corpusField.getOption(FieldOptions.DEFAULTSPHEREKEY_OPTION);
            if (subsetKey == null) {
                subsetKey = defaultSphereKey;
            }
            params.setParam(subsetKey);
        }
        return params;
    }

    private static short getCategory(int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                return CATEGORY_SPHERE;
            case CorpusField.LANGUE_FIELD:
                return CATEGORY_LANGUE;
            /*case CorpusField.DATATION_FIELD :
            return SubsetKey.CATEGORY_DATATION;*/
            case CorpusField.PAYS_FIELD:
                return CATEGORY_PAYS;
            default:
                return 0;
        }
    }

    public static String categoryToString(short category) {
        switch (category) {
            case CATEGORY_CORPUS:
                return "corpus";
            case CATEGORY_THESAURUS:
                return "thesaurus";
            case CATEGORY_SPHERE:
                return "sphere";
            /*case CATEGORY_DOCUMENT :
            return "document";*/
            case CATEGORY_PAYS:
                return "pays";
            case CATEGORY_LANGUE:
                return "langue";
            case CATEGORY_GEOCODAGE:
                return "geocodage";
            case CATEGORY_IMAGESOURCE:
                return "imagesource";
            default:
                throw new IllegalArgumentException("wrong croisementCategory value");
        }
    }

    public static short categoryToShort(String s) {
        if (s.equals("corpus")) {
            return CATEGORY_CORPUS;
        }
        if (s.equals("thesaurus")) {
            return CATEGORY_THESAURUS;
        }
        if (s.equals("sphere")) {
            return CATEGORY_SPHERE;
        }
        //if (s.equals("document")) return CATEGORY_DOCUMENT;
        if (s.equals("pays")) {
            return CATEGORY_PAYS;
        }
        if (s.equals("langue")) {
            return CATEGORY_LANGUE;
        }
        if (s.equals("geocodage")) {
            return CATEGORY_GEOCODAGE;
        }
        if (s.equals("imagesource")) {
            return CATEGORY_IMAGESOURCE;
        }
        throw new IllegalArgumentException("Wrong category string");
    }

}
