/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches;

import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.association.DocumentSubsetEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.ParentRecursivityException;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.Album;


/**
 *
 * @author Vincent Calame
 */
public interface BasedefichesEditor {

    public Basedefiches getBasedefiches();

    public BasedefichesMetadataEditor getBasedefichesMetadataEditor();

    public CorpusEditor getCorpusEditor(SubsetKey corpusKey);

    public SphereEditor getSphereEditor(SubsetKey sphereKey);

    public ThesaurusEditor getThesaurusEditor(SubsetKey thesaurusKey);

    public AlbumEditor getAlbumEditor(SubsetKey albumKey);

    public DocumentSubsetEditor getDocumentSubsetEditor();

    public CroisementEditor getCroisementEditor();

    /**
     *
     *@throws IllegalArgumentException si corpusKey est nul ou d'un type incorrect
     */
    public CorpusEditor createCorpus(SubsetKey corpusKey, Subset masterSubset) throws ExistingSubsetException;

    /**
     *
     *@throws IllegalArgumentException si sphereKey est nul ou d'un type incorrect
     */
    public SphereEditor createSphere(SubsetKey sphereKey) throws ExistingSubsetException;

    /**
     * Crèe un nouveau thésaurus de type indiqué.
     * Le type ne peut pas être modifié après création du thésaurus.
     *
     *@throws IllegalArgumentException si thesaurusKey est nul ou d'un type incorrect
     *@throws IllegalArgumentException si thesaurusType est incorrect
     */
    public ThesaurusEditor createThesaurus(SubsetKey thesaurusKey, short thesaurusType) throws ExistingSubsetException;

    /**
     *
     *@throws IllegalArgumentException si albumKey est nul ou d'un type incorrect
     */
    public AlbumEditor createAlbum(SubsetKey albumKey) throws ExistingSubsetException;

    public void merge(Motcle source, Motcle destination) throws ParentRecursivityException;

    /**
     *Supprime la sphère. Une sphère ne peut être supprimée que si elle ne contient aucun rédacteur.
     *
     *@throws IllegalArgumentException si sphere n'appartient pas à la base de fiches
     */
    public void removeSphere(Sphere sphere) throws NoRemoveableSubsetException;

    /**
     *Supprime le thésaurus. Un thésaurus ne peut être supprimé que s'il ne contient aucun mot-clé.
     *
     *@throws IllegalArgumentException si le thésaurus n'appartient pas à la base de fiches
     */
    public void removeThesaurus(Thesaurus thesaurus) throws NoRemoveableSubsetException;

    /**
     *Supprime le corpus. Un corpus ne peut être supprimé que s'il ne contient aucune fiche.
     *
     *@throws IllegalArgumentException si le corpus n'appartient pas à la base de fiches
     */
    public void removeCorpus(Corpus corpus) throws NoRemoveableSubsetException;

    /**
     *Supprime l'album. Un album ne peut être supprimé que s'il ne contient aucune fiche.
     *
     *@throws IllegalArgumentException si le corpus n'appartient pas à la base de fiches
     */
    public void removeAlbum(Album album) throws NoRemoveableSubsetException;

}
