/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches;

import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.text.StringUtils;
import java.util.Set;
import java.util.HashSet;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesUtils {

    private BasedefichesUtils() {
    }

    public static String getSubsetIntitule(Basedefiches basedefiches, SubsetKey subsetKey, int langInteger) {
        if (subsetKey.isCorpusSubset()) {
            Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
            if (corpus == null) {
                return "?";
            }
            return CorpusMetadataUtils.getCorpusIntituleString(corpus, langInteger);
        } else if (subsetKey.isThesaurusSubset()) {
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(subsetKey);
            if (thesaurus == null) {
                return "?";
            }
            return ThesaurusUtils.getThesaurusIntitule(thesaurus, langInteger);
        } else if (subsetKey.isSphereSubset()) {
            Sphere sphere = (Sphere) basedefiches.getSubset(subsetKey);
            if (sphere == null) {
                return "?";
            }
            return SphereUtils.getSphereIntitule(sphere, langInteger);
        }
        return "?";
    }

    public static boolean isRemoveable(Subset subset) {
        return subset.getBasedefiches().isRemoveable(subset);
    }

    public static Sphere[] getSphereArray(Basedefiches basedefiches) {
        int sphereCount = basedefiches.getSphereCount();
        Sphere[] sphereArray = new Sphere[sphereCount];
        for (int i = 0; i < sphereCount; i++) {
            sphereArray[i] = basedefiches.getSphere(i);
        }
        return sphereArray;
    }

    public static Thesaurus[] getThesaurusArray(Basedefiches basedefiches) {
        int thesaurusCount = basedefiches.getThesaurusCount();
        Thesaurus[] thesaurusArray = new Thesaurus[thesaurusCount];
        for (int i = 0; i < thesaurusCount; i++) {
            thesaurusArray[i] = basedefiches.getThesaurus(i);
        }
        return thesaurusArray;
    }

    public static Corpus[] getCorpusArray(Basedefiches basedefiches) {
        int corpusCount = basedefiches.getCorpusCount();
        Corpus[] corpusArray = new Corpus[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            corpusArray[i] = basedefiches.getCorpus(i);
        }
        return corpusArray;
    }

    public static Corpus getCorpus(Basedefiches basedefiches, String corpusId) {
        try {
            SubsetKey subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusId);
            return (Corpus) basedefiches.getSubset(subsetKey);
        } catch (ParseException pe) {
            return null;
        }
    }

    public static Thesaurus getThesaurus(Basedefiches basedefiches, String thesaurusId) {
        try {
            SubsetKey subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, thesaurusId);
            return (Thesaurus) basedefiches.getSubset(subsetKey);
        } catch (ParseException pe) {
            return null;
        }
    }

    public static Sphere getSphere(Basedefiches basedefiches, String sphereId) {
        try {
            SubsetKey subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereId);
            return (Sphere) basedefiches.getSubset(subsetKey);
        } catch (ParseException pe) {
            return null;
        }
    }

    public static SubsetKey[] toSubsetKeyArray(short category, String s) {
        Set<String> stringSet = new HashSet<String>();
        String[] array = StringUtils.getTechnicalTokens(s, true);
        int size = array.length;
        if (size == 0) {
            return null;
        }
        SubsetKey[] subsetKeyArray = new SubsetKey[size];
        int p = 0;
        for (int i = 0; i < size; i++) {
            String key = array[i];
            if (!stringSet.contains(key)) {
                stringSet.add(key);
                try {
                    SubsetKey corpusKey = SubsetKey.parse(category, key);
                    subsetKeyArray[p] = corpusKey;
                    p++;
                } catch (ParseException pe) {
                }
            }
        }
        if (p == 0) {
            return null;
        }
        if (p < size) {
            SubsetKey[] tempArray = new SubsetKey[p];
            System.arraycopy(subsetKeyArray, 0, tempArray, 0, p);
            return tempArray;
        } else {
            return subsetKeyArray;
        }
    }

    public static Intitule getOrCreateIntitule(MetadataEditor metadataEditor, String name) {
        Intitule intitule = metadataEditor.getMetadata().getIntituleByName(name);
        if (intitule == null) {
            intitule = metadataEditor.createIntitule(name);
        }
        return intitule;
    }

    /**
     * Passerelle entre l'interface Subset et l'interface SubsetItemList.
     * L'objet retourné est une copie et n'est pas synchronisé avec les évolutions
     * ultérieures du contenu de l'objet Subset.
     */
    public static SubsetItemList toSubsetItemList(Subset subset) {
        int length = subset.getTicketLength();
        int size = subset.size();
        SubsetItem[] array = new SubsetItem[size];
        int p = 0;
        for (int i = 0; i < length; i++) {
            SubsetItem subsetItem = subset.getSubsetItemByTicket(i);
            if (subsetItem != null) {
                array[p] = subsetItem;
                p++;
            }
        }
        return toSubsetItemList(array);
    }

    /**
     * Méthode qui encapsule un tableau d'instances de SubsetItem dans une instance de SubsetItemList
     * Les modifications apportées au tableau sont répercutés dans l'instance de SubsetItemList
     */
    public static SubsetItemList toSubsetItemList(SubsetItem[] subsetItemArray) {
        return new ArraySubsetItemList(subsetItemArray);
    }


    private static class ArraySubsetItemList implements SubsetItemList {

        private SubsetItem[] array;

        private ArraySubsetItemList(SubsetItem[] array) {
            this.array = array;
        }

        public int getSubsetItemCount() {
            return array.length;
        }

        public SubsetItem getSubsetItem(int i) {
            return array[i];
        }

    }

}
