/* BasedefichesLib_API - Copyright (c) 20010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.album;

import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.text.ParseException;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.regex.Matcher;


/**
 *
 * @author Vincent Calame
 */
public class AlbumUtils implements AlbumConstants {

    private static final Pattern ALBUMDIMNAME_PATTERN;
    private static final Pattern IMAGENAME_PATTERN;

    static {
        try {
            ALBUMDIMNAME_PATTERN = Pattern.compile("^[a-z][a-z0-9_]*$");
            IMAGENAME_PATTERN = Pattern.compile("^[-_a-zA-Z0-9.]+$");
        } catch (PatternSyntaxException pse) {
            throw new ShouldNotOccurException(pse);
        }
    }

    private AlbumUtils() {
    }

    public static String getAlbumIntitule(Album album, int langInteger) {
        Intitule albumIntitule = album.getAlbumMetadata().getIntituleByName(ALBUM);
        if (albumIntitule == null) {
            return album.getSubsetKey().toString();
        }
        return albumIntitule.getLibelleString(langInteger, true);
    }

    public static String toFileName(AlbumImage albumImage) {
        return albumImage.getName() + "." + imageTypeToString(albumImage.getType());
    }

    public static short imageTypeToShort(String typeString) {
        typeString = typeString.toLowerCase();
        if (typeString.equals("png")) {
            return PNG;
        }
        if (typeString.equals("jpg")) {
            return JPG;
        }
        if (typeString.equals("jpeg")) {
            return JPG;
        }
        throw new IllegalArgumentException();
    }

    public static String imageTypeToString(short type) {
        switch (type) {
            case PNG:
                return "png";
            case JPG:
                return "jpg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static short dimTypeToShort(String typeString) {
        typeString = typeString.toLowerCase();
        if (typeString.equals("fixed-width")) {
            return FIXED_WIDTH;
        }
        if (typeString.equals("fixed-height")) {
            return FIXED_HEIGHT;
        }
        if (typeString.equals("max-width")) {
            return MAX_WIDTH;
        }
        if (typeString.equals("max-height")) {
            return MAX_HEIGHT;
        }
        if (typeString.equals("max-dim")) {
            return MAX_DIM;
        }
        throw new IllegalArgumentException();
    }

    public static String dimTypeToString(short type) {
        switch (type) {
            case FIXED_WIDTH:
                return "fixed-width";
            case FIXED_HEIGHT:
                return "fixed-height";
            case MAX_WIDTH:
                return "max-width";
            case MAX_HEIGHT:
                return "max-height";
            case MAX_DIM:
                return "max-dim";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static String getMimeType(short type) {
        switch (type) {
            case PNG:
                return "image/png";
            case JPG:
                return "image/jpeg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static String getFormatString(short type) {
        switch (type) {
            case PNG:
                return "png";
            case JPG:
                return "jpeg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static boolean isValidImageType(short type) {
        switch (type) {
            case PNG:
            case JPG:
                return true;
            default:
                return false;
        }
    }

    public static boolean isValidShortType(short type) {
        switch (type) {
            case FIXED_WIDTH:
            case FIXED_HEIGHT:
            case MAX_WIDTH:
            case MAX_HEIGHT:
            case MAX_DIM:
                return true;
            default:
                return false;
        }
    }

    public static boolean needWidth(short type) {
        switch (type) {
            case FIXED_WIDTH:
            case MAX_WIDTH:
            case MAX_DIM:
                return true;
            default:
                return false;
        }
    }

    public static boolean needHeight(short type) {
        switch (type) {
            case FIXED_HEIGHT:
            case MAX_HEIGHT:
            case MAX_DIM:
                return true;
            default:
                return false;
        }
    }

    public static boolean isValidImageName(String s) {
        Matcher matcher = IMAGENAME_PATTERN.matcher(s);
        return matcher.matches();
    }

    public static void testAlbumDimName(String albumDimName) throws ParseException {
        Matcher matcher = ALBUMDIMNAME_PATTERN.matcher(albumDimName);
        if (!matcher.matches()) {
            throw new ParseException(albumDimName, 0);
        }
    }

    public static boolean isValidAlbumDimName(String albumDimName) {
        Matcher matcher = ALBUMDIMNAME_PATTERN.matcher(albumDimName);
        return matcher.matches();
    }

}
