/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.association;

import java.text.ParseException;
import net.mapeadores.util.exceptions.ImplementationException;


/**
 *
 * @author Vincent Calame
 */
public class AssociationUtils {

    private AssociationUtils() {
    }

    public static Version getOrCreateVersion(Document document, DocumentSubsetEditor documentSubsetEditor, String extension) {
        Version version = document.getVersionByExtension(extension);
        if (version == null) {
            try {
                version = documentSubsetEditor.createVersion(document, extension);
            } catch (ExistingVersionException eve) {
                throw new ImplementationException(eve);
            }
        }
        return version;
    }

    /**
     * Un nom de document doit contenir que des minuscules de 'a' à 'z', il peut contenir 
     * '.', '_', '-' ainsi que des chiffres à condition de ne pas commencer par eux.
     *
     *@throws IllegalArgumentException si basename est nul ou de longueur nulle
     *@throws ParseException si idalpha possède des caractères incorrects.
     */
    public static void testBasename(String basename) throws ParseException {
        if (basename == null) {
            throw new IllegalArgumentException("basename is null");
        }
        if (basename.length() == 0) {
            throw new IllegalArgumentException("basename is empty");
        }
        int length = basename.length();
        for (int i = 0; i < length; i++) {
            char carac = basename.charAt(i);
            if (!charCharacter(carac)) {
                if (notBeginCharacter(carac)) {
                    if (i == 0) {
                        throw new ParseException("starting with wrong character = " + carac, i);
                    }
                } else {
                    throw new ParseException("Wrong character = " + carac, i);
                }
            }
        }
    }

    private static boolean charCharacter(char c) {
        return ((c <= 'z') && (c >= 'a'));
    }

    private static boolean notBeginCharacter(char c) {
        switch (c) {
            case '_':
            case '.':
            case '-':
                return true;
            default:
                if ((c < '0') || (c > '9')) {
                    return false;
                } else {
                    return true;
                }
        }
    }

    /**
     * Vérifie la validité du nom ainsi que son existence dans la base.
     * Les caractères accentués sont remplacés par des « _ », un numéro est rajouté si un document du même 
     * nom existe déjà.
     */
    public static String checkBasename(String basename, DocumentSubset documentSubset) {
        basename = checkBasename(basename);
        String result = basename;
        int p = 1;
        while (true) {
            if (documentSubset.getDocumentByBasename(result) == null) {
                break;
            }
            result = basename + String.valueOf(p);
            p++;
        }
        return result;
    }

    public static String checkBasename(String s) {
        StringBuffer buf = new StringBuffer();
        boolean chgt = false;
        if ((s == null) || (s.length() == 0)) {
            return "_";
        }
        for (int i = 0; i < s.length(); i++) {
            char carac = s.charAt(i);
            if (charCharacter(carac)) {
                buf.append(carac);
            } else if (notBeginCharacter(carac)) {
                if (i == 0) {
                    buf.append('a');
                    chgt = true;
                } else {
                    buf.append(carac);
                }
            } else {
                if (i == 0) {
                    buf.append('a');
                } else {
                    buf.append('_');
                }
                chgt = true;
            }
        }
        if (chgt) {
            return buf.toString();
        } else {
            return s;
        }
    }

    /**
     * Indique si l'extension est valide. Une extension est valide si elle ne contient
     * que des caractères de 0 à 9 ou de a à z (uniquement des minuscules).
     */
    public static boolean validateExtension(String s) {
        if ((s == null) || (s.length() == 0)) {
            return false;
        }
        for (int i = 0; i < s.length(); i++) {
            char carac = s.charAt(i);
            if ((carac >= '0') && (carac <= '9')) {
                continue;
            }
            if (((carac >= 'a') && (carac <= 'z'))) {
                continue;
            }
            return false;
        }
        return true;
    }

    /**
     * Un nom de document doit contenir que des minuscules de 'a' à 'z', il peut contenir 
     * '.', '_', '-' ainsi que des chiffres à condition de ne pas commencer par eux.
     *
     *Ne retourne pas d'exception mais false si basename est incorrect
     */
    public static boolean validateBasename(String basename) {
        if (basename == null) {
            return false;
        }
        if (basename.length() == 0) {
            return false;
        }
        int length = basename.length();
        for (int i = 0; i < length; i++) {
            char carac = basename.charAt(i);
            if (!charCharacter(carac)) {
                if (notBeginCharacter(carac)) {
                    if (i == 0) {
                        return false;
                    }
                } else {
                    return false;
                }
            }
        }
        return true;
    }

}
