/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.beans;

import java.io.Serializable;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class PersonneBean implements Serializable {

    private static final long serialVersionUID = 1L;
    /**
     * si nom original existe, devant suivi de la traduction entre parenthèse, sinon prénom + nom ou nom + prénom suivant l'usage.
     */
    public final static short NORMAL_STYLE = 1;
    /**
     * Nom séparé du prénom par un espace, nom original entre parenthèse.
     */
    public final static short ANNUAIRE_STYLE = 2;
    /**
     * Idem, nom en majuscule.
     */
    public final static short ANNUAIRE_UP_STYLE = 3;
    /**
     * Nom séparé du prénom par une virgule, nom original entre parenthèse.
     */
    public final static short BIBLIO_STYLE = 4;
    /**
     * Idem, le nom est en majuscule
     */
    public final static short BIBLIO_UP_STYLE = 5;
    private String nom = "";
    private String prenom = "";
    private String original = "";
    private boolean nomAvant = false;

    public PersonneBean() {
    }

    public PersonneBean(PersonneBean personneBean) {
        this.nom = personneBean.nom;
        this.prenom = personneBean.prenom;
        this.original = personneBean.original;
        this.nomAvant = personneBean.nomAvant;
    }

    /*
     *
     *@throws IllegalArgumentException si nom ou prenom est null
     */
    public PersonneBean(String nom, String prenom) {
        setNom(nom);
        setPrenom(prenom);
    }

    /**
     * N'est jamais <em>null</em>.
     */
    public String getNom() {
        return nom;
    }

    /*
     *
     *@throws IllegalArgumentException si nom est null
     */
    public void setNom(String nom) {
        if (nom == null) {
            throw new IllegalArgumentException("nom is null");
        }
        this.nom = StringUtils.cleanString(nom);
    }

    /**
     * N'est jamais <em>null</em>.
     */
    public String getPrenom() {
        return prenom;
    }

    /*
     *
     *@throws IllegalArgumentException si prenom est null
     */
    public void setPrenom(String prenom) {
        if (prenom == null) {
            throw new IllegalArgumentException("prenom is null");
        }
        this.prenom = StringUtils.cleanString(prenom);
    }

    /**
     * N'est jamais <em>null</em>.
     */
    public String getOriginal() {
        return original;
    }

    /*
     *
     *@throws IllegalArgumentException si nom est null
     */
    public void setOriginal(String original) {
        if (original == null) {
            throw new IllegalArgumentException("original is null");
        }
        this.original = StringUtils.cleanString(original);
    }

    public boolean isNomAvant() {
        return nomAvant;
    }

    public void setNomAvant(boolean nomAvant) {
        this.nomAvant = nomAvant;
    }

    public String getNomComplet(short style) {
        switch (style) {
            case NORMAL_STYLE:
                return getNormalStyle();
            case ANNUAIRE_STYLE:
                return getAnnuaireStyle(false);
            case ANNUAIRE_UP_STYLE:
                return getAnnuaireStyle(true);
            case BIBLIO_STYLE:
                return getBiblioStyle(false);
            case BIBLIO_UP_STYLE:
                return getBiblioStyle(true);
            default:
                throw new IllegalArgumentException("wrong style value");
        }
    }

    private String getNormalStyle() {
        String latin = getLatin();
        if (original.length() == 0) {
            return latin;
        } else {
            if (latin.length() == 0) {
                return original;
            }
            return original + " (" + latin + ")";
        }
    }

    private String getAnnuaireStyle(boolean uppercase) {
        StringBuffer buf = new StringBuffer();
        if (uppercase) {
            buf.append(nom.toUpperCase());
        } else {
            buf.append(nom);
        }
        if (prenom.length() > 0) {
            buf.append(" ");
            buf.append(prenom);
        }
        if (original.length() > 0) {
            buf.append(" (");
            buf.append(original);
            buf.append(")");
        }
        return buf.toString();
    }

    private String getBiblioStyle(boolean uppercase) {
        StringBuffer buf = new StringBuffer();
        if (uppercase) {
            buf.append(nom.toUpperCase());
        } else {
            buf.append(nom);
        }
        if (prenom.length() > 0) {
            buf.append(", ");
            buf.append(prenom);
        }
        if (original.length() > 0) {
            buf.append(" (");
            buf.append(original);
            buf.append(")");
        }
        return buf.toString();
    }

    private String getLatin() {
        if (prenom.length() == 0) {
            if (nom.length() == 0) {
                return "";
            }
            return nom;
        } else if (nom.length() == 0) {
            return prenom;
        } else if (nomAvant) {
            return nom + " " + prenom;
        } else {
            return prenom + " " + nom;
        }
    }

    @Override
    public int hashCode() {
        return nom.hashCode() + prenom.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof PersonneBean)) {
            return false;
        }
        PersonneBean other = (PersonneBean) obj;
        if (other.nomAvant != this.nomAvant) {
            return false;
        }
        if (!other.nom.equals(this.nom)) {
            return false;
        }
        if (!other.prenom.equals(this.prenom)) {
            return false;
        }
        if (!other.original.equals(this.original)) {
            return false;
        }
        return true;
    }

    public static short styleToShort(String style) {
        if (style.equals("normal")) {
            return PersonneBean.NORMAL_STYLE;
        } else if (style.equals("annu")) {
            return PersonneBean.ANNUAIRE_STYLE;
        } else if (style.equals("annu_up")) {
            return PersonneBean.ANNUAIRE_UP_STYLE;
        } else if (style.equals("biblio")) {
            return PersonneBean.BIBLIO_STYLE;
        } else if (style.equals("biblio_up")) {
            return PersonneBean.BIBLIO_UP_STYLE;
        } else {
            throw new IllegalArgumentException("unknown style : " + style);
        }
    }

}
