/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import java.util.Set;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Collections;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUtils {

    private CorpusUtils() {
    }

    /**
     * Retourne le nombre total de fiches de la base.
     */
    public static int getFicheCount(Basedefiches basedefiches) {
        int corpusCount = basedefiches.getCorpusCount();
        int count = 0;
        for (int i = 0; i < corpusCount; i++) {
            count = count + basedefiches.getCorpus(i).size();
        }
        return count;
    }

    public static String toKeyString(AbregeFiche abregeFiche) {
        String separator = "/";
        return abregeFiche.getCorpus().getSubsetKey().getSubsetName() + separator + String.valueOf(abregeFiche.getIdcorpus());
    }

    public static AbregeFiche parse(String key, char separateur, Basedefiches basedefiches, Corpus defaultCorpus) {
        int idx = key.indexOf(separateur);
        if (idx == 0) {
            return null;
        }
        if (idx == -1) {
            if (defaultCorpus == null) {
                return null;
            }
            try {
                int idcorpus = Integer.parseInt(key);
                return defaultCorpus.getAbregeFicheByIdcorpus(idcorpus);
            } catch (NumberFormatException nfe) {
                return null;
            }
        } else {
            try {
                int idcorpus = Integer.parseInt(key.substring(idx + 1));
                Corpus otherCorpus = BasedefichesUtils.getCorpus(basedefiches, key.substring(0, idx));
                if (otherCorpus == null) {
                    return null;
                }
                return otherCorpus.getAbregeFicheByIdcorpus(idcorpus);
            } catch (NumberFormatException nfe) {
                return null;
            }
        }
    }

    public static void replaceInField(CorpusEditor corpusEditor, int idcorpus, CorpusUtils.FieldSelection fieldSelection, FicheItem oldFicheItem, FicheItem newFicheItem) {
        Corpus corpus = corpusEditor.getCorpus();
        AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
        if (abregeFiche == null) {
            return;
        }
        Fiche fiche = corpus.getFiche(abregeFiche);
        boolean done = replaceInField(fiche, fieldSelection, oldFicheItem, newFicheItem);
        if (done) {
            corpusEditor.saveFiche(abregeFiche, fiche);
        }
    }

    public static boolean replaceInField(Fiche fiche, CorpusUtils.FieldSelection fieldSelection, FicheItem oldFicheItem, FicheItem newFicheItem) {
        boolean done = false;
        if (fieldSelection.withRedacteurs()) {
            FicheItemList newList = replaceInFicheItemList(fiche.getRedacteurs(), oldFicheItem, newFicheItem);
            if (newList != null) {
                fiche.setRedacteurs(newList);
                done = true;
            }
        }
        if (fieldSelection.withInformation()) {
            int informationCount = fiche.getInformationCount();
            for (int i = 0; i < informationCount; i++) {
                Information information = fiche.getInformation(i);
                if (fieldSelection.containsInformation(information.getFieldKey())) {
                    FicheItemList newList = replaceInFicheItemList(information, oldFicheItem, newFicheItem);
                    if (newList != null) {
                        fiche.setInformation(information.getFieldKey(), newList);
                        done = true;
                    }
                }
            }
        }
        if (fieldSelection.withPropriete()) {
            int proprieteCount = fiche.getProprieteCount();
            for (int i = 0; i < proprieteCount; i++) {
                Propriete propriete = fiche.getPropriete(i);
                FieldKey fieldKey = propriete.getFieldKey();
                if (fieldSelection.containsPropriete(propriete.getFieldKey())) {
                    FicheItem current = propriete.getFicheItem();
                    if (current.equals(oldFicheItem)) {
                        fiche.setPropriete(fieldKey, newFicheItem);
                        done = true;
                    }
                }
            }
        }
        return done;
    }

    /**
     * Return null si aucun remplacement n'est fait.
     */
    private static FicheItemList replaceInFicheItemList(FicheItemList currentList, FicheItem oldFicheItem, FicheItem newFicheItem) {
        MutableFicheItemList newList = new MutableFicheItemList();
        int count = currentList.getFicheItemCount();
        boolean done = false;
        for (int i = 0; i < count; i++) {
            FicheItem ficheItem = currentList.getFicheItem(i);
            if (ficheItem.equals(oldFicheItem)) {
                newList.addFicheItem(newFicheItem);
                done = true;
            } else {
                newList.addFicheItem(ficheItem);
            }
        }
        if (done) {
            return newList;
        } else {
            return null;
        }
    }

    public static Set<SubsetKey> getRattachementSet(Corpus corpus) {
        Subset masterSubset = corpus.getMasterSubset();
        Subset[] slaveSubsetArray = corpus.getSlaveSubsetArray();
        if (masterSubset != null) {
            return Collections.singleton(masterSubset.getSubsetKey());
        } else if (slaveSubsetArray != null) {
            Set<SubsetKey> rattachementSet = new LinkedHashSet<SubsetKey>();
            for (int i = 0; i < slaveSubsetArray.length; i++) {
                rattachementSet.add(slaveSubsetArray[i].getSubsetKey());
            }
            return rattachementSet;
        }
        return Collections.emptySet();
    }

    public static FieldSelection getPersonneFieldSelection(Corpus corpus) {
        FieldSelection fieldSelection = new FieldSelection();
        fieldSelection.addFieldKey(FieldKey.REDACTEURS);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
                fieldSelection.addFieldKey(corpusField.getFieldKey());
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
                fieldSelection.addFieldKey(corpusField.getFieldKey());
            }
        }
        return fieldSelection;
    }


    public static class FieldSelection {

        private boolean withRedacteurs = false;
        private Set<FieldKey> proprieteSet = new HashSet<FieldKey>();
        private Set<FieldKey> informationSet = new HashSet<FieldKey>();
        private Set<String> annexeSet = new HashSet<String>();

        private void addFieldKey(FieldKey fieldKey) {
            if (fieldKey.isProprieteFieldKey()) {
                proprieteSet.add(fieldKey);
            } else if (fieldKey.isInformationFieldKey()) {
                informationSet.add(fieldKey);
            } else if (fieldKey.isAnnexeFieldKey()) {
                annexeSet.add(fieldKey.getTypeString());
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                withRedacteurs = true;
            }
        }

        public boolean withPropriete() {
            return !proprieteSet.isEmpty();
        }

        public boolean withInformation() {
            return !informationSet.isEmpty();
        }

        public boolean withAnnexe() {
            return !annexeSet.isEmpty();
        }

        public boolean withRedacteurs() {
            return withRedacteurs;
        }

        public boolean containsPropriete(FieldKey fieldKey) {
            return proprieteSet.contains(fieldKey);
        }

        public boolean containsInformation(FieldKey fieldKey) {
            return informationSet.contains(fieldKey);
        }

        public boolean containsAnnexe(String type) {
            return annexeSet.contains(type);
        }

    }

}
