/* BasedefichesLib_API - Copyright (c) 2006-2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;


/**
 *
 * @author Vincent Calame
 */
public class Code extends ZoneBlock implements FicheBlockList {

    private static final long serialVersionUID = 3L;
    public final static short SANS_TYPE = 0;
    public final static short TYPE_XML = 1;
    public final static short TYPE_SCRIPT = 2;
    private short type = SANS_TYPE;
    private int lnCount = 0;
    private Ln[] lnArray = new Ln[8];

    /**
     *
     *@throws IllegalArgumentException
     */
    public Code(short type) {
        if ((type < 0) || (type > 2)) {
            throw new IllegalArgumentException("wrong type value");
        }
        this.type = type;
    }

    public short getType() {
        return type;
    }

    public void clear() {
        super.clear();
        for (int i = 0; i < lnCount; i++) {
            lnArray[i].clear();
            lnArray[i] = null;
        }
        lnCount = 0;
    }

    public Ln getLn(int index) {
        if (index >= lnCount) {
            throw new IndexOutOfBoundsException();
        }
        return lnArray[index];
    }

    public FicheBlock getFicheBlock(int index) {
        return getLn(index);
    }

    public int getFicheBlockCount() {
        return lnCount;
    }

    public void addLn(Ln ln) {
        if (ln == null) {
            throw new IllegalArgumentException("ln is null");
        }
        checkLength();
        lnArray[lnCount] = ln;
        lnCount++;
    }

    public boolean isValidFicheBlock(FicheBlock ficheBlock) {
        if (ficheBlock == null) return false;
        return (ficheBlock instanceof Ln);
    }

    private void checkLength() {
        if (lnCount == lnArray.length) {
            Ln[] nv = new Ln[lnCount * 2];
            System.arraycopy(lnArray, 0, nv, 0, lnCount);
            lnArray = nv;
        }
    }

    public static short typeToShort(String strg_type) {
        if ((strg_type == null) || (strg_type.length() == 0)) {
            return SANS_TYPE;
        }
        if (strg_type.equals("xml")) {
            return TYPE_XML;
        }
        if (strg_type.equals("script")) {
            return TYPE_SCRIPT;
        }
        return SANS_TYPE;
    }

    public static String typeToString(short type) {
        switch (type) {
            case TYPE_XML:
                return "xml";
            case TYPE_SCRIPT:
                return "script";
            default:
                return "";
        }
    }

    static public char typeToInitiale(short type) {
        switch (type) {
            case TYPE_XML:
                return 'x';
            case TYPE_SCRIPT:
                return 's';
            default:
                return '0';
        }
    }

    static public short typeToShort(char initiale) {
        switch (initiale) {
            case 'x':
                return TYPE_XML;
            case 's':
                return TYPE_SCRIPT;
            default:
                return SANS_TYPE;
        }
    }

}
