/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.util.Arrays;
import java.io.Serializable;

/**
 *
 * @author Vincent Calame
 */
public class DefaultFicheItemList implements FicheItemList, Serializable {

    private static final long serialVersionUID = 3L;
    private int size = 0;
    private FicheItem[] items;

    public DefaultFicheItemList() {
        this(4);
    }

    public DefaultFicheItemList(int size) {
        items = new FicheItem[size];
    }

    public DefaultFicheItemList(FicheItem ficheItem) {
        if (ficheItem == null) {
            throw new IllegalArgumentException("null not allowed");
        }
        items = new FicheItem[2];
        size = 1;
        items[0] = ficheItem;
    }

    public FicheItem getFicheItem(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return items[index];
    }

    public int getFicheItemCount() {
        return size;
    }

    protected void innerClear() {
        Arrays.fill(items, null);
        size = 0;
    }

    /**
     *
     *
     *@throws IllegalArgumentException si ficheItem est nul
     */
    protected void innerAddFicheItem(FicheItem ficheItem) {
        if (ficheItem == null) {
            throw new IllegalArgumentException("null not allowed");
        }
        addInterne(ficheItem);
    }

    protected void innerAddFicheItemList(FicheItemList ficheItemList) {
        int othersize = ficheItemList.getFicheItemCount();
        for (int i = 0; i < othersize; i++) {
            addInterne(ficheItemList.getFicheItem(i));
        }
    }

    protected boolean innerReplaceFicheItem(FicheItem oldFicheItem, FicheItem newFicheItem) {
        boolean here = false;
        boolean remove = false;
        for (int i = 0; i < size; i++) {
            FicheItem ficheitem = items[i];
            if (ficheitem.equals(oldFicheItem)) {
                if (here) {
                    remove = true;
                } else {
                    here = true;
                    items[i] = newFicheItem;
                }
            } else if (ficheitem.equals(newFicheItem)) {
                if (here) {
                    remove = true;
                } else {
                    here = true;
                }
            } else if (remove) {
                items[i - 1] = items[i];
            }
        }
        if (remove) {
            items[size - 1] = null;
            size = size - 1;
        }
        return here;
    }

    protected boolean innerReplaceFicheItem(FicheItem oldFicheItem, FicheItem[] newFicheItemArray) {
        int oldIndex = -1;
        for (int i = 0; i < size; i++) {
            FicheItem ficheitem = items[i];
            if (ficheitem.equals(oldFicheItem)) {
                oldIndex = i;
                break;
            }
        }
        if (oldIndex == -1) {
            return false;
        }
        FicheItem[] newArray = new FicheItem[newFicheItemArray.length];
        int p = 0;
        for (int j = 0; j < newFicheItemArray.length; j++) {
            FicheItem newFicheItem = newFicheItemArray[j];
            boolean here = false;
            for (int i = 0; i < size; i++) {
                FicheItem ficheitem = items[i];
                if (ficheitem.equals(newFicheItem)) {
                    here = true;
                    break;
                }
            }
            if (!here) {
                newArray[p] = newFicheItem;
                p++;
            }
        }
        if (p == 0) {
            return false;
        }
        if ((size + p - 1) > items.length) {
            FicheItem[] nv = new FicheItem[size + p - 1];
            System.arraycopy(items, 0, nv, 0, oldIndex);
            System.arraycopy(items, oldIndex + 1, nv, oldIndex + p, (size - oldIndex - 1));
            items = nv;
        } else {
            System.arraycopy(items, oldIndex + 1, items, oldIndex + p, (size - oldIndex - 1));
        }
        for (int i = 0; i < p; i++) {
            items[oldIndex + i] = newArray[i];
        }
        size = size + p - 1;
        return true;
    }

    private boolean addInterne(FicheItem item) {
        for (int i = 0; i < size; i++) {
            if (item.equals(items[i])) {
                return false;
            }
        }
        checkLength();
        items[size] = item;
        size++;
        return true;
    }

    private void checkLength() {
        if (size == items.length) {
            FicheItem[] nv = new FicheItem[size * 2];
            System.arraycopy(items, 0, nv, 0, size);
            items = nv;
        }
    }

}
