/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.langinteger.LangIntegerException;
import java.util.Locale;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class Langue implements FicheItem, Serializable {

    private static final long serialVersionUID = 2L;
    private int langInteger;

    public Langue(int langInteger) {
        this.langInteger = langInteger;
    }

    public int getLangInteger() {
        return langInteger;
    }

    @Override
    public String toString() {
        return LangInteger.toString(langInteger);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Langue)) {
            return false;
        }
        return (((Langue) obj).langInteger == langInteger);
    }

    @Override
    public int hashCode() {
        return langInteger;
    }

    public static Langue parseLangue(String s, Locale locale) {
        try {
            int langInteger = LangInteger.parse(s, true);
            return new Langue(langInteger);
        } catch (LangIntegerException mcle) {
            if (locale != null) {
                Integer itg = LangIntegerLocalisation.getLangInteger(s, locale);
                if (itg == null) {
                    return null;
                }
                int langInteger = itg.intValue();
                if (LangInteger.getLangPart(langInteger) == LangInteger.NO_LINGUISTIC_CONTENT) {
                    return null;
                }
                return new Langue(langInteger);
            } else {
                return null;
            }
        }
    }

}
