/* BasedefichesLib_API - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class Para implements FicheItem, TextContentList, Serializable {

    private static final long serialVersionUID = 2L;
    private Object[] contentArray;

    private Para(Object[] contentArray) {
        this.contentArray = contentArray;
    }

    public int getTextContentCount() {
        return contentArray.length;
    }

    public Object getTextContent(int i) {
        return contentArray[i];
    }

    public String toRawString() {
        StringBuilder buf = new StringBuilder();
        int size = contentArray.length;
        for (int i = 0; i < size; i++) {
            Object obj = contentArray[i];
            if (obj instanceof String) {
                buf.append((String) obj);
            } else if (obj instanceof S) {
                S s = (S) obj;
                buf.append(s.getValue());
            }
        }
        return buf.toString();
    }


    public static class Builder implements TextContentBuilder {

        private List<Object> contentList = new ArrayList<Object>();

        public void addText(String text) {
            if (text.length() == 0) {
                return;
            }
            int size = contentList.size();
            boolean merge = false;
            if (size > 0) {
                Object obj = contentList.get(size - 1);
                if (obj instanceof String) {
                    contentList.set(size - 1, ((String) obj) + text);
                    merge = true;
                }
            }
            if (!merge) {
                contentList.add(text);
            }
        }

        public void addS(S s) {
            contentList.add(s);
        }

        public Para toPara() {
            return new Para(contentList.toArray());
        }

    }

}
