/* BasedefichesLib_API - Copyright (c) 2006-2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.util.text.StringUtils;
import java.io.Serializable;


/**
 * Il y a deux types de personnes, celles construite avec une clé de rédacteur et celle construite directement avec un PersonneBean.
 *
 * @author Vincent Calame
 */
public final class Personne implements RedacteurItem, Serializable {

    private static final long serialVersionUID = 2L;
    private PersonneBean personneBean;
    private String redacteurKeyString;
    private String organisme;

    public Personne(String redacteurKeyString) {
        if (redacteurKeyString == null) {
            throw new IllegalArgumentException("redacteurKeyString is null");
        }
        this.redacteurKeyString = redacteurKeyString;
    }

    /**
     */
    public Personne(PersonneBean personneBean, String organisme) {
        if (personneBean == null) {
            throw new IllegalArgumentException("personneBean is null");
        }
        this.personneBean = new PersonneBean(personneBean);
        organisme = StringUtils.cleanString(organisme);
        if ((organisme != null) && (organisme.length() == 0)) {
            organisme = null;
        }
        this.organisme = organisme;
    }

    public boolean isRedacteurSphere() {
        return (redacteurKeyString != null);
    }

    public String getRedacteurKeyString() {
        return redacteurKeyString;
    }

    public PersonneBean getPersonneBean() {
        return new PersonneBean(personneBean);
    }

    /**
     * Retourne l'organisme de la personne.
     */
    public String getOrganisme() {
        return organisme;
    }

    public boolean isNomAvant() {
        return personneBean.isNomAvant();
    }

    /**
     * Retourne le nom de la personne.
     *
     *@throws UnsupportedOperationException
     */
    public String getNom() {
        if (personneBean == null) {
            throw new UnsupportedOperationException();
        }
        return personneBean.getNom();
    }

    /**
     * Retourne le prénom de la personne.
     *
     *@throws UnsupportedOperationException
     */
    public String getPrenom() {
        if (personneBean == null) {
            throw new UnsupportedOperationException();
        }
        return personneBean.getPrenom();
    }

    /*
     *
     **@throws UnsupportedOperationException
     */
    public String getOriginal() {
        if (personneBean == null) {
            throw new UnsupportedOperationException();
        }
        return personneBean.getOriginal();
    }

    @Override
    public String toString() {
        if (redacteurKeyString != null) {
            return redacteurKeyString;
        }
        return personneBean.getNomComplet(PersonneBean.BIBLIO_STYLE);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Personne)) {
            return false;
        }
        Personne other = (Personne) obj;
        if (other.redacteurKeyString != null) {
            if (this.redacteurKeyString == null) {
                return false;
            }
            return other.redacteurKeyString.equals(this.redacteurKeyString);
        } else {
            if (this.redacteurKeyString != null) {
                return false;
            }
            boolean result = this.personneBean.equals(other.personneBean);
            if (!result) {
                return false;
            }
            if (other.organisme == null) {
                return (this.organisme == null);
            } else {
                if (this.organisme == null) {
                    return false;
                }
                return other.organisme.equals(this.organisme);
            }
        }
    }

    @Override
    public int hashCode() {
        if (redacteurKeyString != null) {
            return redacteurKeyString.hashCode();
        }
        return personneBean.hashCode();
    }

}
