/* BasedefichesLib_API - Copyright (c) 2006-2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;

import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.l10n.BasedefichesL10n;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.ImplementationException;
import java.util.Locale;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusMetadataUtils implements CorpusConstants {

    private CorpusMetadataUtils() {
    }

    public static boolean isTitreGenerated(CorpusMetadata corpusMetadata) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
        return isTitreGenerated(corpusField);
    }

    public static boolean isTitreGenerated(CorpusField titreField) {
        Object pattern = titreField.getOption(FieldOptions.TITREGENERATIONFORMAT_OPTION);
        return (pattern != null);
    }

    public static boolean hasSoustitreField(CorpusMetadata corpusMetadata) {
        return (corpusMetadata.getCorpusFieldByFieldKey(FieldKey.SOUSTITRE) != null);
    }

    public static boolean hasTexteField(CorpusMetadata corpusMetadata) {
        return (corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TEXTE) != null);
    }

    public static boolean isBlockDisplayInformationField(CorpusField corpusField) {
        if (!(corpusField.getFieldKey().isInformationFieldKey())) {
            return false;
        }
        Object obj = corpusField.getOption(FieldOptions.INFORMATIONDISPLAY_OPTION);
        if (obj == null) {
            throw new ImplementationException("options CorpusField.INFORMATIONDISPLAY_OPTION is null");
        }
        Short shortValue = (Short) obj;
        return (shortValue.shortValue() == FieldOptions.BLOCK_DISPLAY);
    }

    public static boolean hasPersonneSubfield(CorpusField corpusField) {
        if (!(corpusField.getFieldKey().isProprieteFieldKey())) {
            return false;
        }
        if (corpusField.getFicheItemType() != CorpusField.PERSONNE_FIELD) {
            return false;
        }
        Object obj = corpusField.getOption(FieldOptions.SUBFIELDDISPLAY_OPTION);
        if ((obj != null) && (obj.equals(Boolean.TRUE))) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean hasGeopointSubfield(CorpusField corpusField) {
        if (!(corpusField.getFieldKey().isProprieteFieldKey())) {
            return false;
        }
        if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
            return false;
        }
        Object obj = corpusField.getOption(FieldOptions.SUBFIELDDISPLAY_OPTION);
        if ((obj != null) && (obj.equals(Boolean.TRUE))) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isCurrencyArrayField(CorpusField corpusField) {
        if (!(corpusField.getFieldKey().isProprieteFieldKey())) {
            return false;
        }
        Object obj = corpusField.getOption(FieldOptions.CURRENCYARRAY_OPTION);
        if (obj != null) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isImageField(CorpusField corpusField) {
        if (!(corpusField.getFieldKey().isProprieteFieldKey())) {
            return false;
        }
        return (corpusField.getFicheItemType() == CorpusField.IMAGE_FIELD);
    }

    public static String getLibelleString(Corpus corpus, FieldKey fieldKey, int langInteger) {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            return "?";
        }
        return corpusField.getLibelleString(langInteger, true);
    }

    /**
     * Ne s'applique pas au clé de type thésaurus.
     */
    public static String getLibelleString(IncludeKey includeKey, int langInteger) {
        Libelle libelle = BasedefichesL10n.getLibelle(includeKey, langInteger);
        if (libelle == null) {
            return includeKey.toString();
        }
        return libelle.getLibelleString();
    }

    public static String getIntituleString(CorpusMetadata corpusMetadata, String name, int langInteger, boolean defaut) {
        Intitule corpusIntitule = corpusMetadata.getIntituleByName(name);
        if (corpusIntitule == null) {
            if (defaut) {
                return corpusMetadata.getCorpus().getSubsetKey().getSubsetName();
            } else {
                return null;
            }
        }
        return corpusIntitule.getLibelleString(langInteger, defaut);
    }

    public static String getFicheIntituleString(AbregeFiche abregeFiche, LocalisationInfo localisationInfo) {
        return getFicheNumberString(abregeFiche, localisationInfo) + " – " + abregeFiche.getTitre();
    }

    public static String getFicheNumberString(AbregeFiche abregeFiche, LocalisationInfo localisationInfo) {
        return getFicheNumberString(abregeFiche.getCorpus(), abregeFiche.getIdcorpus(), localisationInfo.getWorkingLangInteger(), localisationInfo.getFormatLocale());
    }

    public static String getFicheNumberString(Corpus corpus, int idcorpus, LocalisationInfo localisationInfo) {
        return getFicheNumberString(corpus, idcorpus, localisationInfo.getWorkingLangInteger(), localisationInfo.getFormatLocale());
    }

    public static String getFicheNumberString(AbregeFiche abregeFiche, int workingLangInteger, Locale formatLocale) {
        return getFicheNumberString(abregeFiche.getCorpus(), abregeFiche.getIdcorpus(), workingLangInteger, formatLocale);
    }

    public static String getFicheNumberString(Corpus corpus, int idcorpus, int workingLangInteger, Locale formatLocale) {
        Intitule corpusIntitule = corpus.getCorpusMetadata().getIntituleByName(FICHE_EXISTING);
        String intituleFiche = (corpusIntitule != null) ? corpusIntitule.getLibelleString(workingLangInteger, true) : corpus.getSubsetKey().getSubsetName() + "/";
        return intituleFiche + StringUtils.toString(idcorpus, formatLocale);
    }

    public static String getCorpusIntituleString(Corpus corpus, int langInteger) {
        return getIntituleString(corpus.getCorpusMetadata(), CORPUS, langInteger, true);
    }

    /**
     * Compile les différents champs d'une fiche en une seule liste. Cette liste commence par
     * les champs obligatoires. En particulier, le premier élément est le champ de l'identifiant de corpus.
     */
    public static List<CorpusField> getCorpusFieldList(CorpusMetadata corpusMetadata) {
        List<CorpusField> list = getSpecialCorpusFieldList(corpusMetadata, true);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            list.add(corpusMetadata.getProprieteField(i));
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            list.add(corpusMetadata.getInformationField(i));
        }
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        for (int i = 0; i < annexeCount; i++) {
            list.add(corpusMetadata.getAnnexeField(i));
        }
        return list;
    }

    /**
     * Compile dans une seule liste les champs spéciaux d'une fiche. Le
     *premier élément est le champ de l'identifiant de corpus.
     */
    public static List<CorpusField> getSpecialCorpusFieldList(CorpusMetadata corpusMetadata, boolean idcorpusInclude) {
        List<CorpusField> list = new ArrayList<CorpusField>();
        if (idcorpusInclude) {
            list.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.IDCORPUS));
        }
        list.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.LANG));
        list.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE));
        list.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.REDACTEURS));
        CorpusField soustitre = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.SOUSTITRE);
        if (soustitre != null) {
            list.add(soustitre);
        }
        CorpusField texte = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TEXTE);
        if (texte != null) {
            list.add(texte);
        }
        return list;
    }

    /**
     * Tente de reconstuire une instance de FieldKey ou d'IncludeKey.
     * Retourne <em>null</em> si la chaîne est incorrecte.
     */
    public static Object parseKey(String s) {
        try {
            FieldKey fieldKey = FieldKey.parse(s);
            return fieldKey;
        } catch (java.text.ParseException pe) {
            try {
                IncludeKey includeKey = IncludeKey.parse(s);
                return includeKey;
            } catch (java.text.ParseException pe2) {
                return null;
            }
        }
    }

    /**
     * Indique si le corpus possède au moins un champ susceptible d'être supprimé.
     */
    public static boolean containsRemoveableField(CorpusMetadata corpusMetadata) {
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            return true;
        }
        if (CorpusMetadataUtils.hasTexteField(corpusMetadata)) {
            return true;
        }
        if (corpusMetadata.getProprieteFieldCount() > 0) {
            return true;
        }
        if (corpusMetadata.getAnnexeFieldCount() > 0) {
            return true;
        }
        if (corpusMetadata.getInformationFieldCount() > 0) {
            return true;
        }
        return false;
    }

    public static Class getClass(int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.ITEM_FIELD:
                return Item.class;
            case CorpusField.PERSONNE_FIELD:
                return Personne.class;
            case CorpusField.LANGUE_FIELD:
                return Langue.class;
            case CorpusField.PAYS_FIELD:
                return Pays.class;
            case CorpusField.DATATION_FIELD:
                return Datation.class;
            case CorpusField.COURRIEL_FIELD:
                return Courriel.class;
            case CorpusField.LINK_FIELD:
                return Link.class;
            case CorpusField.NOMBRE_FIELD:
                return Nombre.class;
            case CorpusField.MONTANT_FIELD:
                return Montant.class;
            case CorpusField.GEOPOINT_FIELD:
                return Geopoint.class;
            case CorpusField.PARA_FIELD:
                return Para.class;
            case CorpusField.IMAGE_FIELD:
                return Image.class;
            default:
                throw new IllegalArgumentException("Unknown ficheItemType : " + ficheItemType);
        }
    }

    public static String ficheItemTypeToString(short ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                return "personne";
            case CorpusField.LANGUE_FIELD:
                return "langue";
            case CorpusField.DATATION_FIELD:
                return "datation";
            case CorpusField.PAYS_FIELD:
                return "pays";
            case CorpusField.COURRIEL_FIELD:
                return "courriel";
            case CorpusField.LINK_FIELD:
                return "link";
            case CorpusField.ITEM_FIELD:
                return "item";
            case CorpusField.NOMBRE_FIELD:
                return "nombre";
            case CorpusField.MONTANT_FIELD:
                return "montant";
            case CorpusField.GEOPOINT_FIELD:
                return "geopoint";
            case CorpusField.PARA_FIELD:
                return "para";
            case CorpusField.IMAGE_FIELD:
                return "image";
            default:
                return "";
        }
    }

    public static short ficheItemTypeToShort(String s) {
        if (s.equals("item")) {
            return CorpusField.ITEM_FIELD;
        }
        if (s.equals("personne")) {
            return CorpusField.PERSONNE_FIELD;
        }
        if (s.equals("langue")) {
            return CorpusField.LANGUE_FIELD;
        }
        if (s.equals("pays")) {
            return CorpusField.PAYS_FIELD;
        }
        if (s.equals("datation")) {
            return CorpusField.DATATION_FIELD;
        }
        if (s.equals("courriel")) {
            return CorpusField.COURRIEL_FIELD;
        }
        if (s.equals("link")) {
            return CorpusField.LINK_FIELD;
        }
        if (s.equals("nombre")) {
            return CorpusField.NOMBRE_FIELD;
        }
        if (s.equals("montant")) {
            return CorpusField.MONTANT_FIELD;
        }
        if (s.equals("geopoint")) {
            return CorpusField.GEOPOINT_FIELD;
        }
        if (s.equals("para")) {
            return CorpusField.PARA_FIELD;
        }
        if (s.equals("image")) {
            return CorpusField.IMAGE_FIELD;
        }
        throw new IllegalArgumentException("unkonwn type : " + s);
    }

    public static short checkFicheItem(FieldKey fieldKey, short ficheItemType) {
        if (fieldKey.equals(FieldKey.LANG)) {
            return CorpusField.LANGUE_FIELD;
        }
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return CorpusField.PERSONNE_FIELD;
        }
        if (fieldKey.isFicheItemField(0)) {
            switch (ficheItemType) {
                case CorpusField.PERSONNE_FIELD:
                case CorpusField.LANGUE_FIELD:
                case CorpusField.DATATION_FIELD:
                case CorpusField.PAYS_FIELD:
                case CorpusField.COURRIEL_FIELD:
                case CorpusField.LINK_FIELD:
                case CorpusField.ITEM_FIELD:
                case CorpusField.NOMBRE_FIELD:
                case CorpusField.MONTANT_FIELD:
                case CorpusField.GEOPOINT_FIELD:
                case CorpusField.PARA_FIELD:
                case CorpusField.IMAGE_FIELD:
                    return ficheItemType;
                case CorpusField.NO_FICHEITEM_FIELD:
                    throw new IllegalArgumentException("FicheItemType cannot be NO_FICHEITEM_FIELD");
                default:
                    throw new IllegalArgumentException("Wrong ficheItemType value : " + ficheItemType);
            }
        }
        return CorpusField.NO_FICHEITEM_FIELD;
    }

    public static List<CorpusField> getCorpusFieldListByFicheItemType(CorpusMetadata corpusMetadata, short ficheItemType) {
        List<CorpusField> liste = new ArrayList<CorpusField>();
        if (ficheItemType == CorpusField.PERSONNE_FIELD) {
            liste.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.REDACTEURS));
        }
        if (ficheItemType == CorpusField.LANGUE_FIELD) {
            liste.add(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.LANG));
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            if (corpusField.getFicheItemType() == ficheItemType) {
                liste.add(corpusField);
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            if (corpusField.getFicheItemType() == ficheItemType) {
                liste.add(corpusField);
            }
        }
        return liste;
    }

}
