/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;

import net.mapeadores.basedefiches.SubsetKey;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class IncludeKey {

    public static final short SPECIAL_CATEGORY = 10;
    public static final short INDEXATION_CATEGORY = 11;
    public static final short CHRONO_CATEGORY = 12;
    public static final short LIAGE_CATEGORY = 13;
    public final static IncludeKey ASSOCIATION = new IncludeKey("association", SPECIAL_CATEGORY);
    public final static IncludeKey LIAGE = new IncludeKey("liage", SPECIAL_CATEGORY);
    public final static IncludeKey RATTACHEMENT = new IncludeKey("rattachement", SPECIAL_CATEGORY);
    public final static IncludeKey DATE_CREATION = new IncludeKey("date_creation", CHRONO_CATEGORY);
    public final static IncludeKey DATE_MODIFICATION = new IncludeKey("date_modification", CHRONO_CATEGORY);
    private String includeKey;
    private SubsetKey subsetKey;
    private short category;

    private IncludeKey(String includeKey, short category) {
        this.includeKey = includeKey;
        this.subsetKey = null;
        this.category = category;
    }

    private IncludeKey(SubsetKey subsetKey) {
        this.subsetKey = subsetKey;
        this.includeKey = subsetKey.toString();
        if (subsetKey.isCorpusSubset()) {
            this.category = LIAGE_CATEGORY;
        } else {
            this.category = INDEXATION_CATEGORY;
        }
    }

    /**
     *
     *@throws IllegalArgumentException s'il ne s'agit pas d'une clé de thésaurus
     */
    public static IncludeKey createIndexationInstance(SubsetKey thesaurusKey) {
        if (!thesaurusKey.isThesaurusSubset()) {
            throw new IllegalArgumentException("thesaurusKey.isThesaurusSubset() == false");
        }
        return new IncludeKey(thesaurusKey);
    }

    /**
     *
     *@throws IllegalArgumentException s'il ne s'agit pas d'une clé de corpus
     */
    public static IncludeKey createLiageInstance(SubsetKey corpusKey) {
        if (!corpusKey.isCorpusSubset()) {
            throw new IllegalArgumentException("thesaurusKey.isThesaurusSubset() == false");
        }
        return new IncludeKey(corpusKey);
    }

    public static IncludeKey parse(String s) throws ParseException {
        if (s.equals("association")) {
            return ASSOCIATION;
        }
        if (s.equals("liage")) {
            return LIAGE;
        }
        if (s.equals("rattachement")) {
            return RATTACHEMENT;
        }
        if (s.equals("date_creation")) {
            return DATE_CREATION;
        }
        if (s.equals("date_modification")) {
            return DATE_MODIFICATION;
        }
        if (s.startsWith("thesaurus_")) {
            SubsetKey thesaurusKey = SubsetKey.parse(s);
            return createIndexationInstance(thesaurusKey);
        }
        if (s.startsWith("corpus_")) {
            SubsetKey corpusKey = SubsetKey.parse(s);
            return createLiageInstance(corpusKey);
        }
        throw new ParseException("unkown fieldkey", 0);
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static IncludeKey build(String s) {
        try {
            return parse(s);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    public boolean isSpecialIncludeKey() {
        return (category == SPECIAL_CATEGORY);
    }

    public boolean isChronoIncludeKey() {
        return (category == CHRONO_CATEGORY);
    }

    public boolean isIndexationIncludeKey() {
        return (category == INDEXATION_CATEGORY);
    }

    public boolean isLiageIncludeKey() {
        return (category == LIAGE_CATEGORY);
    }

    public SubsetKey getSubsetKey() {
        return subsetKey;
    }

    public boolean acceptPoidsFilter() {
        return (category == INDEXATION_CATEGORY) || (category == LIAGE_CATEGORY);
    }

    /**
     * Est égal à thesaurusKey.toString() si thesaurusKey n'est pas nul.
     */
    @Override
    public String toString() {
        return includeKey;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof IncludeKey)) {
            return false;
        }
        return ((IncludeKey) obj).includeKey.equals(this.includeKey);
    }

    @Override
    public int hashCode() {
        return includeKey.hashCode();
    }

}
