/* BasedefichesLib_API - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;


/**
 *
 * @author Vincent Calame
 */
public class DefaultMotcleCroisementList implements MotcleCroisementList {

    private SubsetKey thesaurusKey;
    private Motcle[] motcleArray;
    private int[] poidsArray;
    private int size;

    public DefaultMotcleCroisementList(SubsetKey thesaurusKey, int initialCapacity) {
        this.thesaurusKey = thesaurusKey;
        motcleArray = new Motcle[initialCapacity];
        poidsArray = new int[initialCapacity];
    }

    /**
     *
     *@throws IllegalArgumentException si poids < 1
     */
    public DefaultMotcleCroisementList(Motcle motcle, int poids) {
        if (poids < 1) {
            throw new IllegalArgumentException("poids < 1");
        }
        thesaurusKey = motcle.getThesaurus().getSubsetKey();
        motcleArray = new Motcle[1];
        poidsArray = new int[1];
        size = 1;
        motcleArray[0] = motcle;
        poidsArray[0] = poids;
    }

    public int getSubsetItemCount() {
        return size;
    }

    public Motcle getMotcle(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return motcleArray[index];
    }

    public int getPoids(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return poidsArray[index];
    }

    public int getTicket(int index) {
        return getMotcle(index).getTicket();
    }

    /**
     *
     *@throws IllegalArgumentException si poids < 1
     */
    public void add(Motcle motcle, int poids) {
        if (!motcle.getThesaurus().getSubsetKey().equals(thesaurusKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        if (poids < 1) {
            throw new IllegalArgumentException("poids < 1");
        }
        for (int i = 0; i < size; i++) {
            if (motcleArray[i].equals(motcle)) {
                poidsArray[i] = Math.max(poidsArray[i], poids);
                return;
            }
        }
        checkLength();
        motcleArray[size] = motcle;
        poidsArray[size] = poids;
        size++;
    }

    public void append(MotcleCroisementList motcleList) {
        if (!motcleList.getSubsetKey().equals(thesaurusKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        int currentSize = size;
        int otherSize = motcleList.getSubsetItemCount();
        int newsize = currentSize + otherSize;
        if (newsize > motcleArray.length) {
            extendsLength(newsize);
        }
        for (int i = 0; i < otherSize; i++) {
            Motcle otherMotcle = motcleList.getMotcle(i);
            int otherPoids = motcleList.getPoids(i);
            boolean here = false;
            for (int j = 0; j < currentSize; j++) {
                if (motcleArray[j].equals(otherMotcle)) {
                    poidsArray[j] = Math.max(poidsArray[j], otherPoids);
                    here = true;
                    break;
                }
            }
            if (!here) {
                motcleArray[size] = otherMotcle;
                poidsArray[size] = otherPoids;
                size++;
            }
        }
    }

    public SubsetKey getSubsetKey() {
        return thesaurusKey;
    }

    private void checkLength() {
        if (size == motcleArray.length) {
            extendsLength(Math.max(size * 2, 4));
        }
    }

    private void extendsLength(int newLength) {
        Motcle[] nv = new Motcle[newLength];
        System.arraycopy(motcleArray, 0, nv, 0, size);
        motcleArray = nv;
        int[] nvi = new int[newLength];
        System.arraycopy(poidsArray, 0, nvi, 0, size);
        poidsArray = nvi;
    }

}

