/* BasedefichesLib_API - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.format.FormatDef;
import java.util.List;
import java.util.ArrayList;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesFormatDef extends FormatDef {

    private List<FormatSourceKey> formatSourceKeyList = new ArrayList<FormatSourceKey>();
    private FieldKey defaultProprieteKey;
    private int noItem = -9;

    private BasedefichesFormatDef() {
    }

    public BasedefichesFormatDef(FilteredIncludeKey filteredIncludeKey) {
        if (filteredIncludeKey == null) {
            throw new IllegalArgumentException("filteredIncludeKey argument is null");
        }
        formatSourceKeyList.add(new FormatSourceKey(filteredIncludeKey));
    }

    public BasedefichesFormatDef(FieldKey fieldKey) {
        if (fieldKey == null) {
            throw new IllegalArgumentException("fieldKey argument is null");
        }
        formatSourceKeyList.add(new FormatSourceKey(fieldKey));
    }

    public BasedefichesFormatDef(SubfieldKey subfieldKey) {
        if (subfieldKey == null) {
            throw new IllegalArgumentException("fieldKey argument is null");
        }
        formatSourceKeyList.add(new FormatSourceKey(subfieldKey));
    }

    private void add(FormatSourceKey formatSourceKey) {
        formatSourceKeyList.add(formatSourceKey);
    }

    public FieldKey getDefaultProprieteKey() {
        return defaultProprieteKey;
    }

    public void setDefaultProprieteKey(FieldKey defaultProprieteKey) {
        if ((defaultProprieteKey != null) && (!defaultProprieteKey.isProprieteFieldKey())) {
            throw new IllegalArgumentException("defaultProprieteKey is not a Propriete : " + defaultProprieteKey);
        }
        this.defaultProprieteKey = defaultProprieteKey;
    }

    public int getFormatSourceKeyCount() {
        return formatSourceKeyList.size();
    }

    public FormatSourceKey getFormatSourceKey(int i) {
        return formatSourceKeyList.get(i);
    }

    public void setNoItem(boolean noItemB) {
        if (noItemB) {
            noItem = 1;
        } else {
            noItem = 0;
        }
    }

    public boolean isNoItem() {
        return (noItem == 1);
    }

    public boolean isSame(BasedefichesFormatDef other) {
        return (other.toString().equals(this.toString()));
    }

    public String toString() {
        String[] rest = toStringArray();
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < rest.length; i++) {
            if (i > 0) {
                buf.append('|');
            }
            buf.append(rest[i]);
        }
        return buf.toString();
    }

    public String[] toStringArray() {
        StringBuilder buf = new StringBuilder();
        int size = formatSourceKeyList.size();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                buf.append(',');
            }
            buf.append(formatSourceKeyList.get(i).toString());
        }
        String[] result = new String[3];
        result[0] = buf.toString();
        result[1] = formatPatternToString();
        result[2] = paramToString();
        return result;
    }

    public void appendCustomParams(Appendable buf, boolean stillParams) throws IOException {
        if (defaultProprieteKey != null) {
            if (stillParams) {
                buf.append(',');
            } else {
                stillParams = true;
            }
            buf.append("defprop=");
            buf.append(defaultProprieteKey.toString());
        }
        if (noItem == 1) {
            if (stillParams) {
                buf.append(',');
            } else {
                stillParams = true;
            }
            buf.append("noitem");
        }
    }


    public static class Builder {

        BasedefichesFormatDef basedefichesFormatDef;

        public Builder() {
            basedefichesFormatDef = new BasedefichesFormatDef();
        }

        public void add(FieldKey fieldKey, SubsetKey slaveCorpusKey) {
            if (fieldKey == null) {
                throw new IllegalArgumentException("fieldKey is null");
            }
            basedefichesFormatDef.add(new FormatSourceKey(fieldKey, slaveCorpusKey));
        }

        public void add(FilteredIncludeKey filteredIncludeKey, SubsetKey slaveCorpusKey) {
            if (filteredIncludeKey == null) {
                throw new IllegalArgumentException("filteredIncludeKey is null");
            }
            basedefichesFormatDef.add(new FormatSourceKey(filteredIncludeKey, slaveCorpusKey));
        }

        public void add(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
            if (subfieldKey == null) {
                throw new IllegalArgumentException("subfieldKey is null");
            }
            basedefichesFormatDef.add(new FormatSourceKey(subfieldKey, slaveCorpusKey));
        }

        public void add(ThesaurusFieldKey thesaurusFieldKey) {
            if (thesaurusFieldKey == null) {
                throw new IllegalArgumentException("thesaurusFieldKey is null");
            }
            basedefichesFormatDef.add(new FormatSourceKey(thesaurusFieldKey));
        }

        public BasedefichesFormatDef getBasedefichesFormatDef() {
            if (basedefichesFormatDef.formatSourceKeyList.size() == 0) {
                return null;
            }
            return basedefichesFormatDef;
        }

    }

}
