/* BasedefichesLib_API - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class FormatSourceKey {

    private FieldKey fieldKey;
    private SubfieldKey subfieldKey;
    private FilteredIncludeKey filteredIncludeKey;
    private SubsetKey slaveCorpusKey;
    private ThesaurusFieldKey thesaurusFieldKey;

    public FormatSourceKey(FieldKey fieldKey) {
        this(fieldKey, null);
    }

    public FormatSourceKey(FieldKey fieldKey, SubsetKey slaveCorpusKey) {
        if (fieldKey == null) {
            throw new IllegalArgumentException("fieldKey is null");
        }
        this.fieldKey = fieldKey;
        this.slaveCorpusKey = slaveCorpusKey;
    }

    public FormatSourceKey(SubfieldKey subfieldKey) {
        this(subfieldKey, null);
    }

    public FormatSourceKey(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
        if (subfieldKey == null) {
            throw new IllegalArgumentException("subfieldKey is null");
        }
        this.subfieldKey = subfieldKey;
        this.slaveCorpusKey = slaveCorpusKey;
    }

    public FormatSourceKey(FilteredIncludeKey filteredIncludeKey) {
        this(filteredIncludeKey, null);
    }

    public FormatSourceKey(FilteredIncludeKey filteredIncludeKey, SubsetKey slaveCorpusKey) {
        if (filteredIncludeKey == null) {
            throw new IllegalArgumentException("filteredIncludeKey is null");
        }
        this.filteredIncludeKey = filteredIncludeKey;
        this.slaveCorpusKey = slaveCorpusKey;
    }

    public FormatSourceKey(ThesaurusFieldKey thesaurusFieldKey) {
        this.thesaurusFieldKey = thesaurusFieldKey;
    }

    public boolean isFieldKey() {
        return (fieldKey != null);
    }

    public boolean isSubfieldKey() {
        return (subfieldKey != null);
    }

    public boolean isFilteredIncludeKey() {
        return (filteredIncludeKey != null);
    }

    public boolean isThesaurusFieldKey() {
        return (thesaurusFieldKey != null);
    }

    public FieldKey getFieldKey() {
        return fieldKey;
    }

    public SubfieldKey getSubfieldKey() {
        return subfieldKey;
    }

    public FilteredIncludeKey getFilteredIncludeKey() {
        return filteredIncludeKey;
    }

    public boolean fromSlaveCorpusKey() {
        return (slaveCorpusKey != null);
    }

    public SubsetKey getSlaveCorpusKey() {
        return slaveCorpusKey;
    }

    public ThesaurusFieldKey getThesaurusFieldKey() {
        return thesaurusFieldKey;
    }

    public String toString() {
        if (fieldKey != null) {
            return fieldKey.toString();
        }
        if (subfieldKey != null) {
            return subfieldKey.toString();
        }
        if (filteredIncludeKey != null) {
            return filteredIncludeKey.toString();
        }
        if (thesaurusFieldKey != null) {
            return thesaurusFieldKey.toString();
        }
        throw new SwitchException("All key are null !");
    }

}
