/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class AbregeFicheFormat extends AbstractFormat {

    private static final int IDCORPUS_SUBTYPE = 1;
    private static final int TITRE_SUBTYPE = 2;
    private static final int CORPUS_SUBTYPE = 3;
    private static final int POIDS_SUBTYPE = 4;
    private static final int PLUS_POIDS_SUBTYPE = 5;

    public AbregeFicheFormat() {
    }

    /**
     * LocalisationInfo n'est pas utilisé.
     */
    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        int poids = 1;
        if (obj instanceof PoidsObject) {
            poids = ((PoidsObject) obj).getPoids();
            obj = ((PoidsObject) obj).getObject();
        }
        if (obj instanceof AbregeFiche) {
            return formatAbregeFiche((AbregeFiche) obj, poids);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatAbregeFiche(AbregeFiche abregeFiche, int poids) {
        if (abregeFiche == null) {
            return null;
        }
        return processFormat(new AbregeFicheResolver(abregeFiche, poids));
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("idcorpus")) {
            return new FormatPart(IDCORPUS_SUBTYPE, null);
        } else if (key.equals("titre")) {
            return new FormatPart(TITRE_SUBTYPE, null);
        } else if (key.equals("corpus")) {
            return new FormatPart(CORPUS_SUBTYPE, null);
        } else if (key.equals("poids")) {
            return new FormatPart(POIDS_SUBTYPE, null);
        } else if (key.equals("plus_poids")) {
            return new FormatPart(PLUS_POIDS_SUBTYPE, null);
        } else {
            return null;
        }
    }

    public static AbregeFicheFormat newDefaultInstance() {
        try {
            AbregeFicheFormat abregeFicheFormat = new AbregeFicheFormat();
            abregeFicheFormat.setPattern(getDefaultPattern());
            return abregeFicheFormat;
        } catch (java.text.ParseException pe) {
            throw new ShouldNotOccurException(pe);
        }
    }

    public static String getDefaultPattern() {
        return "{titre}";
    }


    private class AbregeFicheResolver implements ObjectFormatPartResolver {

        AbregeFiche abregeFiche;
        int poids;

        AbregeFicheResolver(AbregeFiche abregeFiche, int poids) {
            this.abregeFiche = abregeFiche;
            this.poids = poids;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case CORPUS_SUBTYPE:
                    buf.append(abregeFiche.getCorpus().getSubsetKey().getSubsetName());
                    break;
                case TITRE_SUBTYPE:
                    buf.append(abregeFiche.getTitre());
                    break;
                case IDCORPUS_SUBTYPE:
                    buf.append(String.valueOf(abregeFiche.getIdcorpus()));
                    break;
                case POIDS_SUBTYPE:
                    buf.append(String.valueOf(poids));
                    break;
                case PLUS_POIDS_SUBTYPE:
                    if (poids > 1) {
                        buf.append('+');
                        buf.append(String.valueOf(poids - 1));
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
