/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.util.Locale;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class CodeFormat extends FicheItemFormat {

    private static final int CODE_SUBTYPE = 1;
    private static final int LIB_SUBTYPE = 2;

    public CodeFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Langue) {
            return formatLangue((Langue) obj, localisationInfo);
        } else if (obj instanceof Pays) {
            return formatPays((Pays) obj, localisationInfo);

        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatLangue(Langue langue, LocalisationInfo localisationInfo) {
        if (langue == null) {
            return null;
        }
        return processFormat(new LangueResolver(langue, localisationInfo));
    }

    public String formatPays(Pays pays, LocalisationInfo localisationInfo) {
        if (pays == null) {
            return null;
        }
        return processFormat(new PaysResolver(pays, localisationInfo));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("code")) {
            return new FormatPart(CODE_SUBTYPE, null);
        } else if (key.equals("lib")) {
            return new FormatPart(LIB_SUBTYPE, new LibParameters(instructionArray));
        } else {
            return null;
        }
    }


    private class PaysResolver implements ObjectFormatPartResolver {

        Pays pays;
        LocalisationInfo localisationInfo;

        PaysResolver(Pays pays, LocalisationInfo localisationInfo) {
            this.pays = pays;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case CODE_SUBTYPE:
                    buf.append(pays.getCountryCode());
                    break;
                case LIB_SUBTYPE:
                    int countryInteger = pays.getCountryInteger();
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        appendLocale(buf, countryInteger, getFormatLocale(localisationInfo));
                    } else {
                        for (int i = 0; i < array.length; i++) {
                            if (i > 0) {
                                buf.append(libParameters.getSeparator());
                            }
                            appendLocale(buf, countryInteger, LangInteger.getLocale(array[i]));
                        }
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

        private void appendLocale(Appendable buf, int countryInteger, Locale locale) throws IOException {
            buf.append(LangIntegerLocalisation.getCountryLibelle(countryInteger, locale));
        }

    }


    private class LangueResolver implements ObjectFormatPartResolver {

        Langue langue;
        LocalisationInfo localisationInfo;

        LangueResolver(Langue langue, LocalisationInfo localisationInfo) {
            this.langue = langue;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case CODE_SUBTYPE:
                    LangInteger.appendString(buf, langue.getLangInteger());
                    break;
                case LIB_SUBTYPE:
                    int langInteger = langue.getLangInteger();
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        appendLocale(buf, langInteger, getFormatLocale(localisationInfo));
                    } else {
                        for (int i = 0; i < array.length; i++) {
                            if (i > 0) {
                                buf.append(libParameters.getSeparator());
                            }
                            appendLocale(buf, langInteger, LangInteger.getLocale(array[i]));
                        }
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

        private void appendLocale(Appendable buf, int langInteger, Locale locale) throws IOException {
            buf.append(LangIntegerLocalisation.getLangLibelle(langInteger, locale));
        }

    }

}
