/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.DateFormatBundle;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.util.Locale;
import java.io.IOException;


/**
 *Accepté {iso}, {value}, {a}
 *
 * @author Vincent Calame
 */
public class DatationFormat extends FicheItemFormat {

    private static final int CODE_SUBTYPE = 1;
    private static final int ISO_SUBTYPE = 2;
    private static final int ANNEE_SUBTYPE = 3;
    private static final int LIB_SUBTYPE = 4;
    private static final int MOIS_SUBTYPE = 5;
    private static final int DERNIERMOIS_SUBTYPE = 6;
    private static final int ISOLAST_SUBTYPE = 7;

    public DatationFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Datation) {
            return formatDatation((Datation) obj, localisationInfo);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatDatation(Datation datation, LocalisationInfo localisationInfo) {
        if (datation == null) {
            return null;
        }
        return processFormat(new DatationResolver(datation, localisationInfo));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("code")) {
            return new FormatPart(CODE_SUBTYPE, null);
        } else if (key.equals("iso")) {
            return new FormatPart(ISO_SUBTYPE, null);
        } else if (key.equals("iso_last")) {
            return new FormatPart(ISOLAST_SUBTYPE, null);
        } else if (key.equals("a")) {
            return new FormatPart(ANNEE_SUBTYPE, null);
        } else if (key.equals("a-m")) {
            return new FormatPart(MOIS_SUBTYPE, null);
        } else if (key.equals("a-m_last")) {
            return new FormatPart(DERNIERMOIS_SUBTYPE, null);
        } else if (key.equals("lib")) {
            return new FormatPart(LIB_SUBTYPE, new LibParameters(instructionArray));
        } else {
            return null;
        }
    }


    private class DatationResolver implements ObjectFormatPartResolver {

        int dateInteger;
        LocalisationInfo localisationInfo;

        DatationResolver(Datation datation, LocalisationInfo localisationInfo) {
            this.dateInteger = datation.getDateInteger();
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case CODE_SUBTYPE:
                    DateInteger.appendString(buf, dateInteger);
                    break;
                case ISO_SUBTYPE:
                    DateInteger.appendCompleteString(buf, dateInteger, true, false);
                    break;
                case ISOLAST_SUBTYPE:
                    DateInteger.appendCompleteString(buf, dateInteger, true, true);
                    break;
                case ANNEE_SUBTYPE:
                    buf.append(String.valueOf(DateInteger.getAnnee(dateInteger)));
                    break;
                case LIB_SUBTYPE:
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        appendLocale(buf, dateInteger, getFormatLocale(localisationInfo));
                    } else {
                        for (int i = 0; i < array.length; i++) {
                            if (i > 0) {
                                buf.append(libParameters.getSeparator());
                            }
                            appendLocale(buf, dateInteger, LangInteger.getLocale(array[i]));
                        }
                    }
                    break;
                case MOIS_SUBTYPE:
                    DateInteger.appendCompleteString(buf, dateInteger, false, false);
                    break;
                case DERNIERMOIS_SUBTYPE:
                    DateInteger.appendCompleteString(buf, dateInteger, false, true);
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

        private void appendLocale(Appendable buf, int dateInteger, Locale locale) throws IOException {
            DateFormatBundle dateFormatBundle = DateFormatBundle.getDateFormatBundle(locale);
            buf.append(DateInteger.getDateLibelle(dateInteger, dateFormatBundle));
        }

    }

}
