/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class DocumentFormat extends AbstractFormat {

    private static final int BASENAME_SUBTYPE = 1;
    private static final int EXTENSIONS_SUBTYPE = 2;

    public DocumentFormat() {
    }

    /**
     * LocalisationInfo n'est pas utilisé.
     */
    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Document) {
            return formatDocument((Document) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatDocument(Document document) {
        if (document == null) {
            return null;
        }
        return processFormat(new DocumentResolver(document));
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("basename")) {
            return new FormatPart(BASENAME_SUBTYPE, null);
        } else if (key.equals("extensions")) {
            return new FormatPart(EXTENSIONS_SUBTYPE, null);
        } else {
            return null;
        }
    }

    public static String getDefaultPattern() {
        return "{basename}{extensions}";
    }


    private class DocumentResolver implements ObjectFormatPartResolver {

        private Document document;

        DocumentResolver(Document document) {
            this.document = document;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case BASENAME_SUBTYPE:
                    buf.append(document.getBasename());
                    break;
                case EXTENSIONS_SUBTYPE:
                    int count = document.getVersionCount();
                    if (count == 1) {
                        buf.append('.');
                        buf.append(document.getVersion(0).getVersionName().getExtension());
                    } else {
                        for (int i = 0; i < count; i++) {
                            buf.append('[');
                            buf.append('.');
                            buf.append(document.getVersion(i).getVersionName().getExtension());
                            buf.append(']');
                        }
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
