/* BasedefichesLib_API - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.ParagraphBlock;
import net.mapeadores.basedefiches.corpus.fiche.P;
import net.mapeadores.basedefiches.corpus.fiche.H;
import net.mapeadores.basedefiches.corpus.fiche.Code;
import net.mapeadores.basedefiches.corpus.fiche.Table;
import net.mapeadores.basedefiches.corpus.fiche.Div;
import net.mapeadores.basedefiches.corpus.fiche.Ln;
import net.mapeadores.basedefiches.corpus.fiche.Td;
import net.mapeadores.basedefiches.corpus.fiche.Li;
import net.mapeadores.basedefiches.corpus.fiche.Ul;
import net.mapeadores.basedefiches.corpus.fiche.Tr;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class FicheBlockFormat {

    public FicheBlockFormat() {
    }

    public static List<String> formatFicheBlockList(FicheBlockList ficheBlockList) {
        FicheBlockProcess process = new FicheBlockProcess();
        process.addAll(ficheBlockList);
        return process.resultList;
    }

    public static List<String> formatFicheBlock(FicheBlock ficheBlock) {
        FicheBlockProcess process = new FicheBlockProcess();
        process.add(ficheBlock);
        return process.resultList;
    }


    static class FicheBlockProcess {

        List<String> resultList = new ArrayList<String>();

        void addAll(FicheBlockList ficheBlockList) {
            int size = ficheBlockList.getFicheBlockCount();
            for (int i = 0; i < size; i++) {
                FicheBlock ficheBlock = ficheBlockList.getFicheBlock(i);
                add(ficheBlock);
            }

        }

        void add(FicheBlock ficheBlock) {
            if (ficheBlock instanceof P) {
                addP((P) ficheBlock);
            }
            if (ficheBlock instanceof H) {
                addH((H) ficheBlock);
            } else if (ficheBlock instanceof Code) {
                Code code = (Code) ficheBlock;
                int size2 = code.getFicheBlockCount();
                resultList.add("");
                resultList.add("+++++");
                for (int j = 0; j < size2; j++) {
                    addLn(code.getLn(j));
                }
                resultList.add("+++++");
                resultList.add("");
            } else if (ficheBlock instanceof Table) {
                Table table = (Table) ficheBlock;
                resultList.add("");
                resultList.add("=====");
                int size2 = table.getTrCount();
                for (int j = 0; j < size2; j++) {
                    Tr tr = table.getTr(j);
                    addTr(tr);
                }
                resultList.add("=====");
                resultList.add("");
            } else if (ficheBlock instanceof Ul) {
                addUl((Ul) ficheBlock, 0);
            } else if (ficheBlock instanceof Div) {
                resultList.add("");
                resultList.add("!!!!!");
                addAll(((Div) ficheBlock).getFicheBlockList());
                resultList.add("!!!!!");
                resultList.add("");
            } else if (ficheBlock instanceof Ln) {
                addLn((Ln) ficheBlock);
            }
        }

        void addP(P p) {
            addParagraphBlock(p, "");
        }

        void addP(P p, String pre) {
            addParagraphBlock(p, pre);
        }

        void addH(H h) {
            resultList.add("");
            addParagraphBlock(h, "");
            resultList.add("");
        }

        void addLn(Ln ln) {
            StringBuilder buf = new StringBuilder();
            int indentation = ln.getIndentation();
            for (int i = 0; i < indentation; i++) {
                buf.append('\u00A0');
            }
            buf.append(ln.getValue());
            resultList.add(buf.toString());
        }

        void addZoneBlock(FicheBlockList ficheBlockList, String ligne) {
            resultList.add("");
            resultList.add(ligne);
            addAll(ficheBlockList);
            resultList.add(ligne);
            resultList.add("");
        }

        void addParagraphBlock(ParagraphBlock paragraphBlock, String pre) {
            StringBuilder buf = new StringBuilder();
            buf.append(pre);
            int count = paragraphBlock.getTextContentCount();
            for (int i = 0; i < count; i++) {
                Object obj = paragraphBlock.getTextContent(i);
                buf.append(obj.toString());
            }
            resultList.add(buf.toString());
        }

        void addUl(Ul ul, int depth) {
            int size2 = ul.getLiCount();
            for (int j = 0; j < size2; j++) {
                Li li = ul.getLi(j);
                addLi(li, depth);
            }
        }

        void addTr(Tr tr) {
            int size = tr.getFicheBlockCount();
            StringBuilder buf = new StringBuilder();
            buf.append('|');
            for (int i = 0; i < size; i++) {
                Td td = tr.getTd(i);
                int count = td.getTextContentCount();
                for (int j = 0; j < count; j++) {
                    Object obj = td.getTextContent(j);
                    buf.append(obj.toString());
                }
                buf.append('|');
            }
            resultList.add(buf.toString());
        }

        void addLi(Li li, int depth) {
            int liSize = li.getFicheBlockCount();
            boolean premier = true;
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < depth; i++) {
                buf.append('\u00A0');
            }
            String pre = buf.toString();
            for (int j = 0; j < liSize; j++) {
                FicheBlock prg = li.getFicheBlock(j);
                if (prg instanceof P) {
                    if (premier) {
                        premier = false;
                        addParagraphBlock((P) prg, pre + "– ");
                    } else {
                        addParagraphBlock((P) prg, pre + "\u00A0 ");
                    }
                } else if (prg instanceof Ul) {
                    addUl((Ul) prg, depth + 1);
                }
            }
        }

    }

}
