/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;


/**
 *
 * @author Vincent Calame
 */
public class FormatUtils {

    private FormatUtils() {
    }

    /**
     * ficheItemFormat
     */
    public static String formatSubfield(short subfieldType, FicheItem ficheItem, Basedefiches basedefiches) {
        PersonneBean personneBean = null;
        if (ficheItem instanceof Personne) {
            Personne personne = (Personne) ficheItem;
            if (personne.isRedacteurSphere()) {
                try {
                    RedacteurKey redacteurKey = RedacteurKey.parse(personne.getRedacteurKeyString());
                    personneBean = SphereUtils.getPersonneBean(basedefiches, redacteurKey);
                    if (personneBean == null) {
                        personneBean = new PersonneBean();
                        personneBean.setNom(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    }
                } catch (java.text.ParseException pe) {
                    personneBean = new PersonneBean();
                    personneBean.setNom(personne.getRedacteurKeyString());
                }
            } else {
                personneBean = personne.getPersonneBean();
            }
        }
        switch (subfieldType) {
            case SubfieldKey.NOM_SUBTYPE:
                if (personneBean != null) {
                    return personneBean.getNom();
                } else if (ficheItem instanceof Item) {
                    return ((Item) ficheItem).getValue();
                }
                return null;
            case SubfieldKey.PRENOM_SUBTYPE:
                if (personneBean != null) {
                    return personneBean.getPrenom();
                } else {
                    return null;
                }
            case SubfieldKey.ORIGINAL_SUBTYPE:
                if (personneBean != null) {
                    return personneBean.getOriginal();
                } else {
                    return null;
                }
            case SubfieldKey.NOMAVANT_SUBTYPE:
                if (personneBean != null) {
                    boolean b = personneBean.isNomAvant();
                    return (b) ? "1" : "0";
                } else {
                    return null;
                }
            default:
                throw new IllegalArgumentException("Unknown subfieldType : " + subfieldType);


        }
    }

    public static Integer getLangInteger(String value) {
        if (value == null) {
            return null;
        }
        value = value.trim();
        try {
            int langInteger = LangInteger.parse(value);
            return new Integer(langInteger);
        } catch (LangIntegerException lie) {
            return null;
        }

    }

}
