/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *Comprend {href}, {title}, {comment}
 *
 * @author Vincent Calame
 */
public class LinkFormat extends FicheItemFormat {

    private static final int HREF_SUBTYPE = 1;
    private static final int TITLE_SUBTYPE = 2;
    private static final int COMMENT_SUBTYPE = 3;

    public LinkFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Link) {
            return formatLink((Link) obj);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatLink(Link link) {
        if (link == null) {
            return null;
        }
        return processFormat(new LinkResolver(link));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("href")) {
            return new FormatPart(HREF_SUBTYPE, null);
        } else if (key.equals("title")) {
            return new FormatPart(TITLE_SUBTYPE, null);
        } else if (key.equals("comment")) {
            return new FormatPart(COMMENT_SUBTYPE, null);
        } else {
            return null;
        }
    }


    private class LinkResolver implements ObjectFormatPartResolver {

        Link link;

        LinkResolver(Link link) {
            this.link = link;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case HREF_SUBTYPE:
                    buf.append(link.getHref());
                    break;
                case TITLE_SUBTYPE:
                    buf.append(link.getTitle());
                    break;
                case COMMENT_SUBTYPE:
                    buf.append(link.getComment());
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
