/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class MotcleFormat extends AbstractFormat {

    private static final int IDALPHA_SUBTYPE = 1;
    private static final int LIB_SUBTYPE = 2;
    private static final int IDTHS_SUBTYPE = 3;
    private static final int POIDS_SUBTYPE = 4;
    private static final int PLUS_POIDS_SUBTYPE = 5;
    private static final int THESAURUS_SUBTYPE = 6;
    private static final int POSITION_LOCAL_SUBTYPE = 7;
    private static final int POSITION_GLOBAL_SUBTYPE = 8;
    private static final int SAISIE_POIDS_SUBTYPE = 9;
    private static final int PARENT_IDTHS_SUBTYPE = 10;
    private static final int PARENT_IDALPHA_SUBTYPE = 11;

    public MotcleFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        int poids = 1;
        if (obj instanceof PoidsObject) {
            poids = ((PoidsObject) obj).getPoids();
            obj = ((PoidsObject) obj).getObject();
        }
        if (obj instanceof Motcle) {
            return formatMotcle((Motcle) obj, poids, localisationInfo);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatMotcle(Motcle motcle, int poids, LocalisationInfo localisationInfo) {
        if (motcle == null) {
            return null;
        }
        return processFormat(new MotcleResolver(motcle, poids, localisationInfo));
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("idalpha")) {
            return new FormatPart(IDALPHA_SUBTYPE, null);
        } else if (key.equals("lib")) {
            return new FormatPart(LIB_SUBTYPE, new LibParameters(instructionArray));
        } else if (key.equals("idths")) {
            return new FormatPart(IDTHS_SUBTYPE, null);
        } else if (key.equals("poids")) {
            return new FormatPart(POIDS_SUBTYPE, null);
        } else if (key.equals("plus_poids")) {
            return new FormatPart(PLUS_POIDS_SUBTYPE, null);
        } else if (key.equals("thesaurus")) {
            return new FormatPart(THESAURUS_SUBTYPE, null);
        } else if (key.equals("pos_loc")) {
            return new FormatPart(POSITION_LOCAL_SUBTYPE, null);
        } else if (key.equals("pos_glob")) {
            return new FormatPart(POSITION_GLOBAL_SUBTYPE, null);
        } else if (key.equals("saisie_poids")) {
            return new FormatPart(SAISIE_POIDS_SUBTYPE, null);
        } else if (key.equals("parent_idths")) {
            return new FormatPart(PARENT_IDTHS_SUBTYPE, null);
        } else if (key.equals("parent_idalpha")) {
            return new FormatPart(PARENT_IDALPHA_SUBTYPE, null);
        } else {
            return null;
        }
    }

    public static MotcleFormat newDefaultInstance(boolean withIdalpha) {
        try {
            MotcleFormat motcleFormat = new MotcleFormat();
            motcleFormat.setPattern(getDefaultPattern(withIdalpha));
            return motcleFormat;
        } catch (java.text.ParseException pe) {
            throw new ShouldNotOccurException(pe);
        }
    }

    public static String getDefaultPattern(boolean withIdalpha) {
        if (withIdalpha) {
            return "{idalpha}";
        } else {
            return "{lib}";
        }
    }


    private class MotcleResolver implements ObjectFormatPartResolver {

        Motcle motcle;
        int poids;
        LocalisationInfo localisationInfo;

        MotcleResolver(Motcle motcle, int poids, LocalisationInfo localisationInfo) {
            this.motcle = motcle;
            this.poids = poids;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case IDALPHA_SUBTYPE:
                    buf.append(motcle.getIdalpha());
                    break;
                case LIB_SUBTYPE:
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        int workingLangInteger = getWorkingLangInteger(localisationInfo);
                        buf.append(motcle.getLibelleString(workingLangInteger, true));
                    } else {
                        boolean premier = true;
                        for (int i = 0; i < array.length; i++) {
                            String lib = motcle.getLibelleString(array[i], false);
                            if (lib != null) {
                                if (premier) {
                                    premier = false;
                                } else {
                                    buf.append(libParameters.getSeparator());
                                }
                                buf.append(lib);
                            }
                        }
                    }
                    break;
                case IDTHS_SUBTYPE:
                    buf.append(String.valueOf(motcle.getIdths()));
                    break;
                case POIDS_SUBTYPE:
                    buf.append(String.valueOf(poids));
                    break;
                case PLUS_POIDS_SUBTYPE:
                    if (poids > 1) {
                        buf.append('+');
                        buf.append(String.valueOf(poids - 1));
                    }
                    break;
                case THESAURUS_SUBTYPE:
                    buf.append(motcle.getThesaurus().getSubsetKey().getSubsetName());
                    break;
                case POSITION_LOCAL_SUBTYPE:
                    buf.append(String.valueOf(motcle.getChildIndex() + 1));
                    break;
                case POSITION_GLOBAL_SUBTYPE:
                    ThesaurusUtils.appendGlobalPosition(buf, motcle);
                case SAISIE_POIDS_SUBTYPE:
                    if (poids > 1) {
                        buf.append(" <");
                        buf.append(String.valueOf(poids));
                        buf.append(">");
                    }
                    break;
                case PARENT_IDTHS_SUBTYPE:
                    Motcle parentidths = motcle.getParent();
                    if (parentidths != null) {
                        buf.append(String.valueOf(parentidths.getIdths()));
                    }
                    break;
                case PARENT_IDALPHA_SUBTYPE:
                    Motcle parentidalpha = motcle.getParent();
                    if (parentidalpha != null) {
                        buf.append(parentidalpha.getIdalpha());
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
