/* BasedefichesLib_API - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class ParaFormat extends FicheItemFormat {

    private static final int RAW_SUBTYPE = 1;

    public ParaFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Para) {
            return formatPara((Para) obj);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatPara(Para para) {
        if (para == null) {
            return null;
        }
        return processFormat(new ParaResolver(para));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("raw")) {
            return new FormatPart(RAW_SUBTYPE, null);
        } else {
            return null;
        }
    }


    private class ParaResolver implements ObjectFormatPartResolver {

        Para para;

        ParaResolver(Para para) {
            this.para = para;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case RAW_SUBTYPE:
                    buf.append(para.toRawString());
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
