/* BasedefichesLib_API - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.include;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.l10n.BasedefichesL10n;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class DefaultIncludeLibelleMap implements IncludeLibelleMap {

    private Basedefiches basedefiches;

    public DefaultIncludeLibelleMap(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public String getLibelleString(FilteredIncludeKey filteredIncludeKey, int langInteger) {
        IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(includeKey.getSubsetKey());
            if (thesaurus == null) {
                return appendPoidsFilter("?", filteredIncludeKey.getPoidsFilter());
            } else {
                String result = ThesaurusUtils.getThesaurusIntitule(thesaurus, langInteger);
                return appendPoidsFilter(result, filteredIncludeKey.getPoidsFilter());
            }
        } else if (includeKey.isLiageIncludeKey()) {
            Corpus corpus = (Corpus) basedefiches.getSubset(includeKey.getSubsetKey());
            if (corpus == null) {
                return appendPoidsFilter("?", filteredIncludeKey.getPoidsFilter());
            } else {
                String result = CorpusMetadataUtils.getCorpusIntituleString(corpus, langInteger);
                return appendPoidsFilter(result, filteredIncludeKey.getPoidsFilter());
            }
        } else {
            Libelle libelle = BasedefichesL10n.getLibelle(includeKey, langInteger);
            if (libelle == null) {
                return includeKey.toString();
            } else {
                return libelle.getLibelleString();
            }
        }
    }

    private String appendPoidsFilter(String result, int poidsFilter) {
        if (poidsFilter < 1) {
            return result;
        }
        return result + " <" + String.valueOf(poidsFilter) + ">";
    }

}
