/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.include;

import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class FilteredIncludeKey {

    private IncludeKey includeKey;
    private int poidsFilter = -1;

    public FilteredIncludeKey(IncludeKey includeKey) {
        this.includeKey = includeKey;
    }

    public FilteredIncludeKey(IncludeKey includeKey, int poidsFilter) {
        this.includeKey = includeKey;
        if ((poidsFilter == 0) || (poidsFilter < -1)) {
            throw new IllegalArgumentException();
        }
        if ((poidsFilter != -1) && (!includeKey.acceptPoidsFilter())) {
            throw new IllegalArgumentException();
        }
        this.poidsFilter = poidsFilter;
    }

    public IncludeKey getIncludeKey() {
        return includeKey;
    }

    public int getPoidsFilter() {
        return poidsFilter;
    }

    public boolean hasPoidsFilter() {
        return (poidsFilter != -1);
    }

    @Override
    public String toString() {
        return toString(includeKey, poidsFilter);
    }

    public static FilteredIncludeKey parse(String key) throws ParseException {
        int lastIdx = -1;
        int firstIdx = - 1;
        int length = key.length();
        for (int i = 0; i < length; i++) {
            char carac = key.charAt(i);
            switch (carac) {
                case ':':
                    return oldParse(key, i);
                case '_':
                    if (lastIdx == -1) {
                        lastIdx = i;
                    } else if (firstIdx == -1) {
                        firstIdx = lastIdx;
                        lastIdx = i;
                    } else {
                        throw new ParseException("Too many undescore", i);
                    }
            }
        }
        int poidsFilter = -1;
        String trueKey = key;
        if (lastIdx != -1) {
            try {
                poidsFilter = Integer.parseInt(key.substring(lastIdx + 1));
                if (poidsFilter < 1) {
                    throw new ParseException("negative integer", lastIdx + 1);
                }
                trueKey = key.substring(0, lastIdx);
            } catch (NumberFormatException nfe) {
            }
        }
        IncludeKey includeKey = IncludeKey.parse(trueKey);
        if ((poidsFilter != -1) && (!includeKey.acceptPoidsFilter())) {
            throw new ParseException("unexpected poids filter", lastIdx + 1);
        }
        return new FilteredIncludeKey(includeKey, poidsFilter);
    }

    private static FilteredIncludeKey oldParse(String key, int idx) throws ParseException {
        String trueKey = key.substring(0, idx);
        String part = key.substring(idx + 1);
        try {
            int poidsFilter = Integer.parseInt(part) + 1;
            if (poidsFilter < 1) {
                throw new ParseException(key + " has bad poidsFilter part value", idx);
            }
            IncludeKey includeKey = IncludeKey.parse(trueKey);
            try {
                return new FilteredIncludeKey(includeKey, poidsFilter);
            } catch (IllegalArgumentException iae) {
                throw new ParseException(key + " has bad poidsFilter part value", idx);
            }
        } catch (NumberFormatException nfe) {
            throw new ParseException(key + " has bad poidsFilter part value", idx);
        }
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static FilteredIncludeKey build(String key) {
        try {
            return parse(key);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    public static String toString(IncludeKey includeKey, int poidsFilter) {
        if (poidsFilter < 1) {
            return includeKey.toString();
        }
        return includeKey.toString() + "_" + String.valueOf(poidsFilter);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof FilteredIncludeKey)) {
            return false;
        }
        FilteredIncludeKey other = (FilteredIncludeKey) obj;
        if (other.poidsFilter != this.poidsFilter) {
            return false;
        }
        return other.includeKey.equals(this.includeKey);
    }

    @Override
    public int hashCode() {
        return includeKey.hashCode() + poidsFilter;
    }

}
