/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.include;

import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.util.primitives.Range;
import net.mapeadores.util.primitives.RangeList;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class IncludeSummaryBuilder {

    InternalIncludeSummary internal = new InternalIncludeSummary();

    public IncludeSummaryBuilder() {
    }

    public IncludeSummary getIncludeSummary() {
        return internal;
    }

    public void append(FilteredIncludeKey filteredIncludeKey) {
        internal.add(filteredIncludeKey);
    }


    private class InternalIncludeSummary implements IncludeSummary {

        List<InternalIncludeKeyInfo> liageList = new ArrayList<InternalIncludeKeyInfo>();
        List<InternalIncludeKeyInfo> indexationList = new ArrayList<InternalIncludeKeyInfo>();
        Map<IncludeKey, InternalIncludeKeyInfo> keyInfoMap = new HashMap<IncludeKey, InternalIncludeKeyInfo>();

        public int getIncludeKeyInfoCount(short type) {
            switch (type) {
                case LIAGE_TYPE:
                    return liageList.size();
                case INDEXATION_TYPE:
                    return indexationList.size();
                default:
                    throw new IllegalArgumentException("Unknown Type");
            }
        }

        public IncludeKeyInfo getIncludeKeyInfo(short type, int i) {
            switch (type) {
                case LIAGE_TYPE:
                    return liageList.get(i);
                case INDEXATION_TYPE:
                    return indexationList.get(i);
                default:
                    throw new IllegalArgumentException("Unknown Type");
            }
        }

        public IncludeKeyInfo getIncludeKeyInfo(IncludeKey includeKey) {
            return keyInfoMap.get(includeKey);
        }

        private void add(FilteredIncludeKey filteredIncludeKey) {
            IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
            InternalIncludeKeyInfo ifi = keyInfoMap.get(includeKey);
            if (ifi == null) {
                ifi = new InternalIncludeKeyInfo(includeKey);
                keyInfoMap.put(includeKey, ifi);
                if (includeKey.isIndexationIncludeKey()) {
                    indexationList.add(ifi);
                } else if (includeKey.isLiageIncludeKey()) {
                    liageList.add(ifi);
                }
            }
            ifi.add(filteredIncludeKey.getPoidsFilter());
        }


        private class InternalIncludeKeyInfo implements IncludeKeyInfo {

            private boolean anyPoids = false;
            private RangeList rangeList = null;
            private IncludeKey includeKey;

            InternalIncludeKeyInfo(IncludeKey includeKey) {
                this.includeKey = includeKey;
            }

            public IncludeKey getIncludeKey() {
                return includeKey;
            }

            public boolean hasPoidsFilter() {
                return (rangeList != null);
            }

            public boolean hasAnyPoidsInclude() {
                return anyPoids;
            }

            public RangeList getPoidsFilterRangeList() {
                return rangeList;
            }

            private void add(int poidsFilter) {
                if (poidsFilter < 1) {
                    anyPoids = true;
                } else {
                    Range range = new Range(poidsFilter, poidsFilter);
                    if (rangeList == null) {
                        rangeList = new RangeList();
                    }
                    rangeList.addRange(range);
                }
            }

        }

    }

}
