/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.l10n;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleListMap;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.List;
import java.util.Properties;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesL10n {

    static LibelleListMap libelleListMap = new LibelleListMap();

    static {
        staticInit();
    }

    private static void staticInit() {
        Properties defaultProperties = staticInit("fr", null);
        staticInit("es", defaultProperties);
        staticInit("en", defaultProperties);
        staticInit("ar", defaultProperties);
        staticInit("zh", defaultProperties);
        staticInit("pt", defaultProperties);
    }

    private static Properties staticInit(String langString, Properties defaultProperties) {
        Properties properties = new Properties(defaultProperties);
        try {
            properties.load(BasedefichesL10n.class.getResourceAsStream("basedefiches_" + langString + ".properties"));
            Initializer initializer = new Initializer(properties, LangInteger.parse(langString));
            initializer.init();
            return properties;
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException ile) {
            throw new InternalResourceException(ile);
        }
    }

    private BasedefichesL10n() {
    }

    public static List<Libelle> getSpecialFieldKeyLibelle(FieldKey fieldKey) {
        if (!fieldKey.isSpecialFieldKey()) {
            throw new IllegalArgumentException("fieldKey is not a special key");
        }
        List<Libelle> list = libelleListMap.getLibelleList(fieldKey);
        if (list == null) {
            throw new IllegalArgumentException("libelleList not defined for fieldKey = " + fieldKey.toString());
        }
        return list;
    }

    public static List<Libelle> getIncludeKeyLibelle(IncludeKey includeKey) {
        if (!includeKey.isChronoIncludeKey() && !includeKey.isSpecialIncludeKey()) {
            throw new IllegalArgumentException("fieldKey is not a special key");
        }
        List<Libelle> list = libelleListMap.getLibelleList(includeKey);
        if (list == null) {
            throw new IllegalArgumentException("libelleList not defined for fieldKey = " + includeKey.toString());
        }
        return list;
    }

    public static Libelle getLibelle(IncludeKey includeKey, int langInteger) {
        Libelle libelle = libelleListMap.getLibelle(includeKey, langInteger);
        if (libelle != null) {
            return libelle;
        }
        int langPart = LangInteger.getLangPart(langInteger);
        if (langPart != langInteger) {
            return libelleListMap.getLibelle(includeKey, langInteger);
        }
        return null;
    }


    private static class Initializer {

        Properties properties;
        int langInteger;

        Initializer(Properties properties, int langInteger) {
            this.properties = properties;
            this.langInteger = langInteger;
        }

        void init() {
            init(FieldKey.IDCORPUS);
            init(FieldKey.LANG);
            init(FieldKey.TITRE);
            init(FieldKey.SOUSTITRE);
            init(FieldKey.REDACTEURS);
            init(FieldKey.TEXTE);
            init(IncludeKey.ASSOCIATION);
            init(IncludeKey.LIAGE);
            init(IncludeKey.RATTACHEMENT);
            init(IncludeKey.DATE_CREATION);
            init(IncludeKey.DATE_MODIFICATION);
        }

        void init(FieldKey fieldKey) {
            String text = properties.getProperty(fieldKey.toString());
            libelleListMap.setLibelle(fieldKey, langInteger, text);
        }

        void init(IncludeKey includeKey) {
            String text = properties.getProperty(includeKey.toString());
            libelleListMap.setLibelle(includeKey, langInteger, text);
        }

    }

}
