/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.patterns;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.ValueResolver;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public class FicheValueResolver implements ValueResolver {

    private AbregeFiche abregeFiche;
    private Corpus corpus;
    private Fiche fiche;

    public FicheValueResolver(AbregeFiche abregeFiche) {
        this.abregeFiche = abregeFiche;
        this.corpus = abregeFiche.getCorpus();
    }

    public String getValue(AccoladeArgument accoladeArgument) {
        if (accoladeArgument.getName().equals(FichAccoladeArgument.CORPUS)) {
            return corpus.getSubsetKey().getSubsetName();
        }
        if (accoladeArgument instanceof FichAccoladeArgument) {
            return getValue((FichAccoladeArgument) accoladeArgument);
        } else {
            FichAccoladeArgument fichePatternArgument = FichAccoladeArgument.parse(accoladeArgument.getName(), accoladeArgument.getMode());
            if (fichePatternArgument != null) {
                return getValue(fichePatternArgument);
            } else {
                return AccoladePattern.toString(accoladeArgument);
            }
        }
    }

    public String getValue(FichAccoladeArgument ficheAccoladeArgument) {
        FieldKey fieldKey = ficheAccoladeArgument.getFieldKey();
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            return String.valueOf(abregeFiche.getIdcorpus());
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return LangInteger.toString(abregeFiche.getLangInteger());
        }
        if (fieldKey.equals(FieldKey.TITRE)) {
            return abregeFiche.getTitre();
        }
        if (fieldKey.isProprieteFieldKey()) {
            if (fiche == null) {
                fiche = corpus.getEnteteFiche(abregeFiche);
            }
            Propriete propriete = fiche.getPropriete(fieldKey);
            if (propriete == null) {
                return "";
            }
            FicheItem ficheItem = propriete.getFicheItem();
            if (ficheItem instanceof Item) {
                return ((Item) ficheItem).getValue();
            }
            return AccoladePattern.toString(ficheAccoladeArgument);
        }
        return AccoladePattern.toString(ficheAccoladeArgument);
    }

}
