/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.thesaurus;

import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import java.util.Set;


/**
 *Lorqu'un thésaurus utilise des identifiants alphanumériques, ceux-ci ne doivent être constitué
 * que des caractères suivants : 0-9, A-Z, a-z, _-.()[]/@ et l'espace ' '. S'il est possible d'utiliser des miniscules et des majuscules
 * les méthodes getMotcleByIdalpha et getMotcleSetByIdalpha doivent être insensibles à la casse.
 *<p>Autrement dit, un thésaurus peut très bien avoir 'C' et 'd' comme identifiant alphanumérique mais il ne peut avoir
 * 'D' et 'd'.
 *<p>L'espace ' ' est autorisé sauf en début et en fin de chaîne et il ne peut y avoir deux espaces à la suite.
 *L'espace est ignoré pour établir l'égalité : 'c d' et 'CD' sont deux identifiants égaux.
 *<p>Il n'y a pas de limitation à la taille d'un identifiant alphanumérique.
 *  Les caractères [] et _ sont susceptibles d'avoir des significations particulières.
 *
 * @author  Vincent Calame
 */
public interface Thesaurus extends Subset {

    public MotcleList getFirstLevelList();

    public ThesaurusMetadata getThesaurusMetadata();

    /**
     * Résultat équivalent à (Motcle) getSubsetItemById()
     */
    public Motcle getMotcleByIdths(int idths);

    /**
     *
     *@throws UnsupportedOperationException si le thésaurus n'a pas d'idalpha
     */
    public Motcle getMotcleByIdalpha(String idalpha);

    /**
     *
     *
     *@throws UnsupportedOperationException si le thésaurus n'a pas d'idalpha.
     *@throws IllegalArgumentException si la condition n'est pas valide (voir TextConstants)
     */
    public Set<Motcle> getMotcleSetByIdalpha(String s, int condition);

    public Motcle getMotcleByLibelle(String libelle, int intlang);

    /**
     *
     *@throws IllegalArgumentException si la condition n'est pas valide (voir TextConstants)
     */
    public Set<Motcle> getMotcleSetByLibelle(String s, int condition, int intlang);

}
