/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.thesaurus;

import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadataEditor;
import java.text.ParseException;


/**
 *
 * @author  Vincent Calame
 */
public interface ThesaurusEditor {

    /**
     * Retourne le thésaurus auquel est associé l'instance de ThesaurusEditor
     */
    public Thesaurus getThesaurus();

    public ThesaurusMetadataEditor getThesaurusMetadataEditor();

    /**
     *si idths &lt; 1, un nouvel identifiant est attribué
     *
     *@throws IllegalArgumentException si le thésaurus utilise les idalpha et idalpha est nul ou vide (après suppression des espaces blancs superflus).
     *@throws ParseException si l'idalpha est incorrect
     */
    public Motcle createMotcle(int idths, String idalpha) throws ExistingIdException, ParseException;

    /**
     * Modifie le libellé du mot-clé dans une langue donnée.
     * Dans le cas d'un thésaurus babélien, le libellé précédent est remplacé et la langue est modifiée.
     *<p>Le libellé est nettoyé des espaces blancs superflus. Si <code>nvlibelle</code> est nul ou vide,
     *cela correspond à la suppression du libellé pour la langue en question. Dans le cas d'un thésaurus babélien,
     *la langue devient alors non définie et le libellé vide.
     *
     */
    public boolean setLibelle(Motcle motcle, String nvlibelle, int langInteger);

    /**
     * Modifie le parent du mot-clé. Le mot-clé est placé à la fin de la liste des enfants du mot-clé parent.
     * Si parentMotcle a la valeur <em>null</em>, le mot-clé est placé au premier niveau.
     * Retourne <em>true</em> si le parent a été effectivement modifié.
     */
    public boolean setParent(Motcle motcle, Motcle parentMotcle) throws ParentRecursivityException;

    /**
     * Modifie l'idalpha du motcle.
     *
     *@throws UnsupportedOperationException si le thésaurus n'est pas avec idalpha
     *@throws IllegalArgumentException si nvidalpha est nul ou vide (après suppression des espaces blancs superflus).
     *@throws ParseException si l'idalpha est incorrect
     */
    public boolean setIdalpha(Motcle motcle, String newIdalpha) throws ExistingIdException, ParseException;

    /**
     * Modifie la position du mot-clé dans sa fratrie.
     * Retourne <em>true</em> s'il y a bien eu modification (autrement dit, si la nouvelle valeur de l'index est différente de l'ancienne).
     *
     *@throws IndexOutOfBoundsException si index < 0 ou >= getParent().getChildren().size()
     */
    public boolean setChildIndex(Motcle motcle, int index);

    /**
     *Supprime le mot-clé du thésaurus. Retourne <em>false</em> si le mot-clé
     *ne peut pas être supprimé. Pour pouvoir être supprimé un mot-clé ne doit posséder aucun lien :
     *(avec des fiches ou d'autres mots-clés). La suppression est donc dépendante de la situation du
     *mot-clé dans la base de fiches générale.
     */
    public boolean removeMotcle(Motcle motcle);

}
