/* BasedefichesLib_Explore
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.exceptions.*;
import java.text.*;
import net.mapeadores.util.logging.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheConverter {

    MutableProcessLog mutableProcessLog = new MutableProcessLog();

    public FicheConverter() {
    }

    public static FicheConverter getInstance(CorpusField source, CorpusField destination) {
        FieldKey sourceKey = source.getFieldKey();
        FieldKey destinationKey = destination.getFieldKey();
        int type = 0;
        if (destinationKey.isFicheItemField(0)) {
            type = 1;
        } else if (destinationKey.isAnnexeFieldKey()) {
            type = 2;
        } else {
            throw new IllegalArgumentException("wrong destination : " + destinationKey.toString());
        }
        if (sourceKey.isProprieteFieldKey()) {
            if (type == 1) {
                return new ProprieteToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                throw new IllegalArgumentException("No annexe Key : " + destinationKey.toString());
            }
        } else if ((sourceKey.isInformationFieldKey()) || (sourceKey.equals(FieldKey.REDACTEURS))) {
            if (type == 1) {
                return new InformationToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                throw new IllegalArgumentException("No annexe Key : " + destinationKey.toString());
            }
        } else if ((sourceKey.isAnnexeFieldKey()) || (sourceKey.equals(FieldKey.TEXTE))) {
            if (type == 1) {
                return new AnnexeToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                return new AnnexeToFicheBlockField(sourceKey, destinationKey);
            }
        } else if (sourceKey.isSpecialFieldKey()) {
            if (type == 1) {
                return new SimpleToFicheItemField(sourceKey, destinationKey);
            } else {
                throw new IllegalArgumentException("No annexe Key : " + destinationKey.toString());
            }
        }
        throw new SwitchException("wrong category : " + sourceKey.toString());
    }

    public void convert(AbregeFiche abregeFiche, Fiche fiche) {
        String uri = abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "/" + abregeFiche.getIdcorpus();
        mutableProcessLog.setCurrentURI(ProcessURI.fromString(uri));
        convertContent(abregeFiche, fiche);
    }

    public ProcessLog getProcessLog() {
        return mutableProcessLog;
    }

    abstract void convertContent(AbregeFiche abregeFiche, Fiche fiche);

    void addLog(String category, String key, Object object) {
        ProcessLogItem logItem = mutableProcessLog.newLogItemInstance();
        logItem.setKey(key);
        logItem.setCategory(category);
        logItem.setObject(object);
    }

}
