/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.extraction.api.*;
import net.mapeadores.basedefiches.extraction.builders.*;


/**
 *
 * @author Vincent Calame
 */
public class Extraction implements ExtractionConstants {

    Basedefiches basedefiches;
    CorpusExtractorList staticCorpusExtractorList = CorpusExtractorListBuilder.EMPTY_CORPUSEXTRACTORLIST;
    ThesaurusExtractor staticThesaurusExtractor;
    ExtraitCorpusDef dynamicExtraitCorpusDef;
    MotcleFilterWrapper dynamicMotcleFilterWrapper;
    FicheEligibility ficheEligibility;
    LocalisationLists localisationLists;

    public Extraction(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        this.basedefiches = basedefiches;
        this.ficheEligibility = ficheEligibility;
        this.localisationLists = localisationLists;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public LocalisationLists getLocalisationLists() {
        return localisationLists;
    }

    public FicheEligibility getFicheEligibility() {
        return ficheEligibility;
    }

    public ExtraitCorpusDef getDynamicExtraitCorpusDef() {
        return dynamicExtraitCorpusDef;
    }

    public void setDynamicExtraitCorpusDef(ExtraitCorpusDef extraitCorpusDef) {
        this.dynamicExtraitCorpusDef = extraitCorpusDef;
    }

    public MotcleFilterWrapper getDynamicMotcleFilterWrapper() {
        return dynamicMotcleFilterWrapper;
    }

    public void setDynamicMotcleFilterWrapper(MotcleFilterWrapper wrapper) {
        this.dynamicMotcleFilterWrapper = wrapper;
    }

    public CorpusExtractorList getStaticCorpusExtractorList() {
        return staticCorpusExtractorList;
    }

    public ThesaurusExtractor getStaticThesaurusExtractor() {
        return staticThesaurusExtractor;
    }

    public void initStaticCorpusExtractorList(ExtraitCorpusDefList staticExtraitCorpusDefList) {
        int count = staticExtraitCorpusDefList.getExtraitCorpusDefCount();
        if (count == 0) {
            this.staticCorpusExtractorList = CorpusExtractorListBuilder.EMPTY_CORPUSEXTRACTORLIST;
            return;
        }
        CorpusExtractorListBuilder builder = new CorpusExtractorListBuilder();
        for (int k = 0; k < count; k++) {
            ExtraitCorpusDef extraitCorpusDef = staticExtraitCorpusDefList.getExtraitCorpusDef(k);
            FicheSelector ficheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            ficheSelector.setFicheEligibility(ficheEligibility);
            ficheSelector.addFicheSelectOptionsList(extraitCorpusDef);
            int corpusCount = ficheSelector.getCorpusCount();
            if (corpusCount == 0) {
                continue;
            }
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef, null);
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = ficheSelector.getCorpus(i);
                for (int j = 0; j < corpus.getTicketLength(); j++) {
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                    if (abregeFiche != null) {
                        if (ficheSelector.isSelected(abregeFiche)) {
                            corpusExtractor.add(abregeFiche, 0);
                        }
                    }
                }
            }
            builder.addCorpusExtractor(corpusExtractor);
        }
        this.staticCorpusExtractorList = builder.toCorpusExtractorList();
    }

    public void initStaticThesaurusExtractor(List staticWrapperList) {
        if ((staticWrapperList == null) || (staticWrapperList.size() == 0)) {
            return;
        }
        MotcleSelector motcleSelector = new MotcleSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
        for (Iterator it = staticWrapperList.iterator(); it.hasNext();) {
            MotcleFilterWrapper wrapper = (MotcleFilterWrapper) it.next();
            motcleSelector.addMotcleFilterWrapper(wrapper);
        }
        int thesaurusCount = motcleSelector.getThesaurusCount();
        if (thesaurusCount == 0) {
            return;
        }
        staticThesaurusExtractor = new ThesaurusExtractor(basedefiches, null);
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = motcleSelector.getThesaurus(i);
            for (int j = 0; j < thesaurus.getTicketLength(); j++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(j);
                if (motcle != null) {
                    MotcleFilter motcleFilter = motcleSelector.getMotcleFilter(motcle);
                    if (motcleFilter != null) {
                        staticThesaurusExtractor.add(motcle, motcleFilter, 0);
                    }
                }
            }
        }
    }

    /**
     * Instance d'extraction proposée comme instance par défaut lorsque l'objet dynamique de l'extraction est un corpus.
     */
    public final static Extraction newDefaultCorpusInstance(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        extraction.setDynamicExtraitCorpusDef(ExtraitCorpusDefBuilder.getExtraitCorpusDef(FicheFilterBuilder.newAbregeFicheInstance(ficheEligibility, null), null));
        return extraction;
    }

    /**
     * Instance d'extraction proposée comme instance par défaut lorsque l'objet dynamique de l'extraction est une fiche ou un ensemble de fiches.
     */
    public final static Extraction newDefaultFicheInstance(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        FicheFilter ficheFilter = FicheFilterBuilder.newFicheInstance(basedefiches, ficheEligibility, null);
        extraction.setDynamicExtraitCorpusDef(ExtraitCorpusDefBuilder.getExtraitCorpusDef(ficheFilter, IntituleClauseBuilder.DEFAULT));
        return extraction;
    }

    /**
     * Instance d'extraction proposée comme instance par défaut lorsque l'objet dynamique de l'extraction est un mot-clé.
     */
    public final static Extraction newDefaultMotcleInstance(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        MotcleFilterWrapper wrapper = new MotcleFilterWrapper(MotcleFilter.getAloneInstance(ficheEligibility));
        extraction.setDynamicMotcleFilterWrapper(wrapper);
        return extraction;
    }

    /**
     * Instance d'extraction proposée comme instance par défaut lorsque l'objet dynamique de l'extraction est un thésaurus.
     */
    public final static Extraction newDefaultThesaurusInstance(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        MotcleFilterWrapper wrapper = new MotcleFilterWrapper(MotcleFilter.getThesaurusInstance(ficheEligibility));
        extraction.setDynamicMotcleFilterWrapper(wrapper);
        return extraction;
    }

    /**
     * Instance d'extraction proposée comme instance par défaut lorsqu'il n'y a aucun objet dynamique.
     */
    public final static Extraction newDefaultUniqueInstance(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        return extraction;
    }

}
