/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.extraction.api.*;
import net.mapeadores.basedefiches.extraction.builders.*;


/**
 *
 * @author Vincent Calame
 */
public class MotcleFilter {

    private FicheEligibility ficheEligibility;
    private ExtraitCorpusDefList indexationExtraitCorpusDefList = null;
    private boolean withLevel, withLib;
    private MotcleFilter previousFilter;
    private MotcleFilter nextFilter;
    private MotcleFilter parentFilter;
    private MotcleFilter childrenFilter;
    private boolean withIntituleThesaurus = false;
    private String tagName;

    public MotcleFilter() {
    }

    public boolean isNoneFiltering() {
        if (withLib) {
            return false;
        }
        if (withLevel) {
            return false;
        }
        if (withChildren()) {
            return false;
        }
        if (this.withNext()) {
            return false;
        }
        if (this.withPrevious()) {
            return false;
        }
        if (this.withParent()) {
            return false;
        }
        if (this.includeIndexation()) {
            return false;
        }
        return true;
    }

    public boolean includeIndexation() {
        return ((indexationExtraitCorpusDefList != null) && (indexationExtraitCorpusDefList.getExtraitCorpusDefCount() > 0));
    }

    public boolean withLevel() {
        return withLevel;
    }

    public void setWithLevel(boolean withLevel) {
        this.withLevel = withLevel;
    }

    public boolean withLib() {
        return withLib;
    }

    public void setWithLib(boolean withLib) {
        this.withLib = withLib;
    }

    public boolean withChildren() {
        return (childrenFilter != null);
    }

    public MotcleFilter getChildrenFilter() {
        return childrenFilter;
    }

    public void setChildrenFilter(MotcleFilter childrenFilter) {
        this.childrenFilter = childrenFilter;
    }

    public boolean withParent() {
        return (parentFilter != null);
    }

    public MotcleFilter getParentFilter() {
        return parentFilter;
    }

    public void setParentFilter(MotcleFilter parentFilter) {
        this.parentFilter = parentFilter;
    }

    public boolean withNext() {
        return (nextFilter != null);
    }

    public MotcleFilter getNextFilter() {
        return nextFilter;
    }

    public void setNextFilter(MotcleFilter nextFilter) {
        this.nextFilter = nextFilter;
    }

    public boolean withPrevious() {
        return (previousFilter != null);
    }

    public MotcleFilter getPreviousFilter() {
        return previousFilter;
    }

    public void setPreviousFilter(MotcleFilter previousFilter) {
        this.previousFilter = previousFilter;
    }

    /**
     * Propose une instance par défaut pour les cas où le mot-clé est demandé seul.
     */
    public static MotcleFilter getAloneInstance(FicheEligibility ficheEligibility) {
        MotcleFilter motcleFilter = new MotcleFilter();
        motcleFilter.setWithLib(true);
        motcleFilter.setFicheEligibility(ficheEligibility);
        motcleFilter.setIndexationExtraitCorpusDefList(getDefaultExtraitCorpusDefList());
        return motcleFilter;
    }

    /**
     * Propose une instance par défaut pour les cas où le mot-clé est demandé à l'intérieur d'un thésaurus.
     */
    public static MotcleFilter getThesaurusInstance(FicheEligibility ficheEligibility) {
        MotcleFilter motcleFilter = new MotcleFilter();
        motcleFilter.setWithLib(true);
        motcleFilter.setFicheEligibility(ficheEligibility);
        motcleFilter.setIndexationExtraitCorpusDefList(getDefaultExtraitCorpusDefList());
        motcleFilter.childrenFilter = motcleFilter;
        motcleFilter.setWithIntituleThesaurus(true);
        return motcleFilter;
    }

    public boolean isWithIntituleThesaurus() {
        return withIntituleThesaurus;
    }

    public void setWithIntituleThesaurus(boolean withIntituleThesaurus) {
        this.withIntituleThesaurus = withIntituleThesaurus;
    }

    public String getTagName() {
        return tagName;
    }

    public void setTagName(String tagName) {
        this.tagName = tagName;
    }

    public FicheEligibility getFicheEligibility() {
        return ficheEligibility;
    }

    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        this.ficheEligibility = ficheEligibility;
    }

    public ExtraitCorpusDefList getIndexationExtraitCorpusDefList() {
        return indexationExtraitCorpusDefList;
    }

    public void setIndexationExtraitCorpusDefList(ExtraitCorpusDefList indexationExtraitCorpusDefList) {
        this.indexationExtraitCorpusDefList = indexationExtraitCorpusDefList;
    }

    public static ExtraitCorpusDefList getDefaultExtraitCorpusDefList() {
        FicheFilter indexationFicheFilter = FicheFilterBuilder.newAbregeFicheInstance(null, null);
        ExtraitCorpusDef extraitCorpusDef = ExtraitCorpusDefBuilder.getExtraitCorpusDef(indexationFicheFilter, IntituleClauseBuilder.DEFAULT);
        return ExtraitCorpusDefListBuilder.getSingletonExtraitCorpusDefList(extraitCorpusDef);
    }

}
