/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusExtractor {

    private Basedefiches basedefiches;
    private HashMap thesaurusKeyMap = new HashMap();
    private ArrayList thesaurusList = new ArrayList();
    private Subset filterSubset;

    public ThesaurusExtractor(Basedefiches basedefiches, Subset filterSubset) {
        this.basedefiches = basedefiches;
        this.filterSubset = filterSubset;
    }

    public void add(Motcle motcle, MotcleFilter motcleFilter, int poids) {
        if (motcleFilter == null) {
            throw new IllegalArgumentException("motcleFilter is null");
        }
        if (poids < 0) {
            poids = 0;
        }
        Thesaurus thesaurus = motcle.getThesaurus();
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusKeyMap.get(thesaurus.getSubsetKey());
        if (bundle == null) {
            bundle = new ThesaurusExtractBundle(thesaurus);
            bundle.setWithIntituleThesaurus(motcleFilter.isWithIntituleThesaurus());
            bundle.setTagName(motcleFilter.getTagName());
            thesaurusKeyMap.put(thesaurus.getSubsetKey(), bundle);
            thesaurusList.add(bundle);
        }
        bundle.add(new MotcleExtractBundle(motcle, motcleFilter, poids));
    }

    public boolean containsMotcle() {
        return (thesaurusKeyMap.size() != 0);
    }

    public int getThesaurusCount() {
        return thesaurusList.size();
    }

    public Thesaurus getThesaurus(int index) {
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusList.get(index);
        return bundle.thesaurus;
    }

    public String getRattachementType(int index) {
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusList.get(index);
        return bundle.rattachementType;
    }

    public String getTagName(int index) {
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusList.get(index);
        return bundle.getTagName();
    }

    public MotcleExtractInfo[] getMotcleExtractInfoArray(int index) {
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusList.get(index);
        return bundle.getMotcleExtractInfoArray();
    }

    public boolean isWithIntituleThesaurus(int index) {
        ThesaurusExtractBundle bundle = (ThesaurusExtractBundle) thesaurusList.get(index);
        return bundle.isWithIntituleThesaurus();
    }


    private class ThesaurusExtractBundle {

        String rattachementType = null;
        Thesaurus thesaurus;
        ArrayList bundleList = new ArrayList();
        boolean withIntituleThesaurus;
        private String tagName;

        ThesaurusExtractBundle(Thesaurus thesaurus) {
            this.thesaurus = thesaurus;
            if (filterSubset != null) {
                Subset masterSubset = filterSubset.getMasterSubset();
                if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                    rattachementType = "master";
                }
            }
        }

        void add(MotcleExtractBundle motcleExtractBundle) {
            bundleList.add(motcleExtractBundle);
        }

        MotcleExtractInfo[] getMotcleExtractInfoArray() {
            int size = bundleList.size();
            MotcleExtractInfo[] array = new MotcleExtractInfo[size];
            for (int i = 0; i < size; i++) {
                array[i] = ((MotcleExtractBundle) bundleList.get(i)).toMotcleExtractInfo();
            }
            return array;
        }

        public boolean isWithIntituleThesaurus() {
            return withIntituleThesaurus;
        }

        public void setWithIntituleThesaurus(boolean withIntituleThesaurus) {
            this.withIntituleThesaurus = withIntituleThesaurus;
        }

        public String getTagName() {
            return tagName;
        }

        public void setTagName(String tagName) {
            this.tagName = tagName;
        }

    }


    private class MotcleExtractBundle {

        Motcle motcle;
        MotcleFilter motcleFilter;
        int poids;

        MotcleExtractBundle(Motcle motcle, MotcleFilter motcleFilter, int poids) {
            this.motcle = motcle;
            this.motcleFilter = motcleFilter;
            this.poids = poids;
        }

        MotcleExtractInfo toMotcleExtractInfo() {
            MotcleExtractInfo motcleExtractInfo = new MotcleExtractInfo(motcle, motcleFilter);
            motcleExtractInfo.setPoids(poids);
            return motcleExtractInfo;
        }

    }

}
