/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.extraction.api.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.extraction.filterinfo.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.extraction.filterinfo.*;
import java.util.*;
import net.mapeadores.basedefiches.extraction.api.*;
import net.mapeadores.basedefiches.extraction.builders.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheFilterBuilder implements ExtractionConstants {

    InternalFicheFilter internalFicheFilter;

    public FicheFilterBuilder() {
        internalFicheFilter = new InternalFicheFilter();
    }

    public FicheFilter toFicheFilter() {
        return internalFicheFilter;
    }

    public boolean parseTagName(String tagName) {
        tagName = BuilderUtils.testTagName(tagName, FICHE_DEFAULT_TAGNAME);
        internalFicheFilter.setTagName(tagName);
        return true;
    }

    public void updateFilterInfoList(FilterInfoList filterInfoList) {
        internalFicheFilter.updateFilterInfoList(filterInfoList);
    }

    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        internalFicheFilter.setFicheEligibility(ficheEligibility);
    }

    public void setLiageExtraitCorpusDefList(ExtraitCorpusDefList liageExtraitCorpusDefList) {
        internalFicheFilter.setLiageExtraitCorpusDefList(liageExtraitCorpusDefList);
    }

    public void setAssociationInclude(boolean includeAssociation) {
        internalFicheFilter.setAssociationInclude(includeAssociation);
    }

    public void setLangueInclude(boolean includeLangue) {
        internalFicheFilter.setLangueInclude(includeLangue);
    }

    public void setIntituleFicheInclude(boolean includeIntituleFiche) {
        internalFicheFilter.setIntituleFicheInclude(includeIntituleFiche);
    }

    public void setIndexationMotcleSelector(MotcleSelector indexationMotcleSelector) {
        internalFicheFilter.setIndexationMotcleSelector(indexationMotcleSelector);
    }

    public static FicheFilter newFicheInstance(Basedefiches basedefiches, FicheEligibility ficheEligibility, String tagName) {
        FilterInfoListBuilder infoListBuilder = new FilterInfoListBuilder();
        infoListBuilder.addEnteteFilter();
        infoListBuilder.addCorpsdeficheFilter();
        infoListBuilder.addChronoFilter();
        infoListBuilder.addFieldKey(FieldKey.LANG);

        FicheFilter liageFicheFilter = newAbregeFicheInstance(ficheEligibility, null);
        ExtraitCorpusDef extraitCorpusDef = ExtraitCorpusDefBuilder.getExtraitCorpusDef(liageFicheFilter, IntituleClauseBuilder.DEFAULT);

        MotcleSelector motcleSelector = new MotcleSelector(basedefiches, 0);
        MotcleFilter motcleFilter = new MotcleFilter();
        motcleFilter.setWithLib(true);
        motcleSelector.addMotcleFilterWrapper(new MotcleFilterWrapper(motcleFilter));

        FicheFilterBuilder builder = new FicheFilterBuilder();
        builder.setFicheEligibility(ficheEligibility);
        builder.parseTagName(tagName);
        builder.updateFilterInfoList(infoListBuilder.toFilterInfoList());
        builder.setLiageExtraitCorpusDefList(ExtraitCorpusDefListBuilder.getSingletonExtraitCorpusDefList(extraitCorpusDef));
        builder.setAssociationInclude(true);
        builder.setIntituleFicheInclude(true);
        builder.setIndexationMotcleSelector(motcleSelector);
        return builder.toFicheFilter();
    }

    public static FicheFilter newIdFicheInstance(String tagName) {
        FicheFilterBuilder builder = new FicheFilterBuilder();
        builder.parseTagName(tagName);
        return builder.toFicheFilter();
    }

    public static FicheFilter newAbregeFicheInstance(FicheEligibility ficheEligibility, String tagName) {
        FilterInfoListBuilder infoListBuilder = new FilterInfoListBuilder();
        infoListBuilder.addFieldKey(FieldKey.TITRE);
        infoListBuilder.addFieldKey(FieldKey.SOUSTITRE);
        FicheFilterBuilder builder = new FicheFilterBuilder();
        builder.setFicheEligibility(ficheEligibility);
        builder.parseTagName(tagName);
        builder.updateFilterInfoList(infoListBuilder.toFilterInfoList());
        builder.setIntituleFicheInclude(true);
        return builder.toFicheFilter();
    }


    private static class InternalFicheFilter implements FicheFilter {

        private FilterInfo[] filterInfoArray;
        private boolean withCorpsdefiche = false;
        private String tagName = FICHE_DEFAULT_TAGNAME;
        private FicheEligibility ficheEligibility;
        private ExtraitCorpusDefList liageExtraitCorpusDefList = BuilderUtils.EMPTY_EXTRAITCORPUSDEFLIST;
        private boolean includeAssociation = false;
        private boolean includeLangue = false;
        private boolean includeIntituleFiche = false;
        private MotcleSelector indexationMotcleSelector = null;

        InternalFicheFilter() {
        }

        public boolean isEmptyFicheFilter() {
            if (filterInfoArray != null) {
                return false;
            }
            if (liageExtraitCorpusDefList.getExtraitCorpusDefCount() > 0) {
                return false;
            }
            if (includeAssociation) {
                return false;
            }
            if (includeLangue) {
                return false;
            }
            if (includeIntituleFiche) {
                return false;
            }
            if (indexationMotcleSelector != null) {
                return false;
            }
            return false;
        }

        public boolean isWithCorpsdefiche() {
            return withCorpsdefiche;
        }

        public int getFilterInfoCount() {
            if (filterInfoArray == null) {
                return 0;
            }
            return filterInfoArray.length;
        }

        public FilterInfo getFilterInfo(int i) {
            return filterInfoArray[i];
        }

        public String getTagName() {
            return tagName;
        }

        public FicheEligibility getFicheEligibility() {
            return ficheEligibility;
        }

        public ExtraitCorpusDefList getLiageExtraitCorpusDefList() {
            return liageExtraitCorpusDefList;
        }

        public boolean includeAssociation() {
            return includeAssociation;
        }

        public boolean includeLangue() {
            return includeLangue;
        }

        public boolean includeIntituleFiche() {
            return includeIntituleFiche;
        }

        public MotcleSelector getIndexationMotcleSelector() {
            return indexationMotcleSelector;
        }

        private void setIndexationMotcleSelector(MotcleSelector indexationMotcleSelector) {
            this.indexationMotcleSelector = indexationMotcleSelector;
        }

        private void setLangueInclude(boolean includeLangue) {
            this.includeLangue = includeLangue;
        }

        private void setIntituleFicheInclude(boolean includeIntituleFiche) {
            this.includeIntituleFiche = includeIntituleFiche;
        }

        private void setAssociationInclude(boolean includeAssociation) {
            this.includeAssociation = includeAssociation;
        }

        private void setLiageExtraitCorpusDefList(ExtraitCorpusDefList liageExtraitCorpusDefList) {
            if (liageExtraitCorpusDefList == null) {
                this.liageExtraitCorpusDefList = BuilderUtils.EMPTY_EXTRAITCORPUSDEFLIST;
            } else {
                this.liageExtraitCorpusDefList = liageExtraitCorpusDefList;
            }
        }

        private void setFicheEligibility(FicheEligibility ficheEligibility) {
            this.ficheEligibility = ficheEligibility;
        }

        private void setTagName(String tagName) {
            this.tagName = tagName;
        }

        private void updateFilterInfoList(FilterInfoList filterInfoList) {
            if (filterInfoList == null) {
                filterInfoArray = null;
            }
            int count = filterInfoList.getFilterInfoCount();
            filterInfoArray = new FilterInfo[count];
            for (int i = 0; i < count; i++) {
                FilterInfo filterInfo = filterInfoList.getFilterInfo(i);
                if (filterInfo instanceof CorpsdeficheFilterInfo) {
                    withCorpsdefiche = true;
                } else if (filterInfo instanceof FieldKeyFilterInfo) {
                    FieldKey fieldKey = ((FieldKeyFilterInfo) filterInfo).getFieldKey();
                    if (fieldKey.ownsToCorpsdefiche()) {
                        withCorpsdefiche = true;
                    }
                } else if (filterInfo instanceof TypePrefixFilterInfo) {
                    short category = ((TypePrefixFilterInfo) filterInfo).getCategory();
                    if (category == FieldKey.ANNEXE_CATEGORY) {
                        withCorpsdefiche = true;
                    }
                }
                filterInfoArray[i] = filterInfo;
            }
        }

    }

}
