/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.extraction.api.*;


/**
 *
 * @author Vincent Calame
 */
public class PackClauseBuilder implements ExtractionConstants {

    private InternalPackClause internalPackClause;

    public PackClauseBuilder(int packSize) {
        if (packSize < 1) {
            packSize = 1;
        }
        internalPackClause = new InternalPackClause(packSize);
    }

    public PackClause toPackClause() {
        return internalPackClause;
    }

    public boolean parseTagName(String tagName) {
        tagName = BuilderUtils.testTagName(tagName, PACK_DEFAULT_TAGNAME);
        internalPackClause.setTagName(tagName);
        return true;
    }

    public boolean parseModulo(String moduloString) {
        if ((moduloString == null) || (moduloString.length() == 0)) {
            return false;
        }
        try {
            int modulo = Integer.parseInt(moduloString);
            internalPackClause.setModulo(modulo);
            return true;
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

    public static PackClauseBuilder newInstance(String packSizeString) {
        if ((packSizeString == null) || (packSizeString.length() == 0)) {
            return null;
        }
        try {
            int packSize = Integer.parseInt(packSizeString);
            return new PackClauseBuilder(packSize);
        } catch (NumberFormatException nfe) {
            return null;
        }
    }


    private static class InternalPackClause implements PackClause {

        private int packSize;
        private int modulo = 0;
        private String tagName = PACK_DEFAULT_TAGNAME;

        InternalPackClause(int packSize) {
            this.packSize = packSize;
        }

        public String getTagName() {
            return tagName;
        }

        private void setTagName(String tagName) {
            this.tagName = tagName;
        }

        public int getModulo() {
            return modulo;
        }

        private void setModulo(int modulo) {
            if (modulo < 2) {
                modulo = 0;
            }
            this.modulo = modulo;
        }

        public int getPackSize() {
            return packSize;
        }

    }

}
