/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.filterinfo;

import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class FilterInfoListBuilder {

    private List filterInfoList = new ArrayList();

    public FilterInfoListBuilder() {
    }

    public void addFieldKey(FieldKey fieldKey) {
        FilterInfo filterInfo = new DefaultFieldKeyFilterInfo(fieldKey);
        filterInfoList.add(filterInfo);
    }

    public void addTypeFilter(short category, String type) {
        if (type.endsWith("*")) {
            int length = type.length();
            if (length == 1) {
                return;
            }
            String prefix = type.substring(0, length - 1);
            TypePrefixFilterInfo info = new DefaultTypePrefixFilterInfo(category, prefix);
            filterInfoList.add(info);
        } else {
            try {
                FieldKey fieldKey = FieldKey.parse(category, type);
                addFieldKey(fieldKey);
            } catch (java.text.ParseException pe) {
            }
        }
    }

    public void addLotFilter(String tagName, FilterInfoList fil) {
        if (tagName != null) {
            tagName = tagName.trim();
            if (tagName.length() == 0) {
                tagName = null;
            }
        }
        FilterInfo filterInfo = new DefaultLotFilterInfo(tagName, fil);
        filterInfoList.add(filterInfo);
    }

    public void addEnteteFilter() {
        filterInfoList.add(new DefaultEnteteFilterInfo());
    }

    public void addCorpsdeficheFilter() {
        filterInfoList.add(new DefaultCorpsdeficheFilterInfo());
    }

    public void addChronoFilter() {
        filterInfoList.add(new DefaultChronoFilterInfo());
    }

    public FilterInfoList toFilterInfoList() {
        return new InternalFicheInfoList(filterInfoList);
    }


    private static class InternalFicheInfoList implements FilterInfoList {

        private List filterInfoList;

        InternalFicheInfoList(List filterInfoList) {
            this.filterInfoList = filterInfoList;
        }

        public int getFilterInfoCount() {
            return filterInfoList.size();
        }

        public FilterInfo getFilterInfo(int i) {
            return (FilterInfo) filterInfoList.get(i);
        }

    }

}
