/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.groups;

import net.mapeadores.basedefiches.corpus.fiche.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.text.collation.group.*;
import net.mapeadores.basedefiches.extraction.*;


/**
 *
 * @author Vincent Calame
 */
public class CollationGroupBasedGroup implements FicheGroup {

    private Map attributesMap = new HashMap();
    private CollationGroup collationGroup;
    private String name;
    private boolean asc;
    private String tagName;

    public CollationGroupBasedGroup(CollationGroup collationGroup, boolean asc, String tagName) {
        this.collationGroup = collationGroup;
        char[] array = new char[1];
        array[0] = collationGroup.getInitial();
        name = new String(array);
        this.asc = asc;
        int byInteger = getByInteger(array[0]);
        attributesMap.put("order", String.valueOf(byInteger));
        this.tagName = tagName;
    }

    private int getByInteger(char c) {
        if (c == CollationGroup.NOLETTER_INITIAL) {
            return 10;
        }
        if (c == CollationGroup.NOEUROPEAN_INITIAL) {
            return 37;
        }
        if (c >= '0' && c <= '9') {
            return (int) (c - 48);
        }
        if (c >= 'A' && c <= 'Z') {
            return (int) (c - 54);
        }
        throw new SwitchException("char = " + c);
    }

    public String getName() {
        return name;
    }

    public String getTagName() {
        return tagName;
    }

    public FicheItem[] getMatchingFicheItemArray() {
        return null;
    }

    public Map getAttributesMap() {
        return attributesMap;
    }

    public FicheExtractInfo[] getFicheExtractInfoArray() {
        List list = collationGroup.getSortedObjectWrapperList();
        int size = list.size();
        FicheExtractInfo[] array = new FicheExtractInfo[size];
        for (int i = 0; i < size; i++) {
            ObjectWrapper objectWrapper = (ObjectWrapper) list.get(i);
            int p = (asc) ? i : (size - i - 1);
            array[p] = ((FicheExtractBundle) objectWrapper.getObject()).toFicheExtractInfo();
        }
        return array;
    }

    public FicheGroup[] getSubgroupArray() {
        return null;
    }

    public boolean isBottomGroup() {
        return true;
    }

}
