/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.groups;

import net.mapeadores.basedefiches.corpus.fiche.*;
import java.util.*;
import net.mapeadores.basedefiches.extraction.*;


/**
 *
 * @author Vincent Calame
 */
public class InnerSortFicheGroup implements MutableFicheGroup {

    private String name;
    private String tagName;
    private ArrayList bundleList = new ArrayList();
    private FicheItem[] matchingFicheItemArray;
    private boolean asc;

    public InnerSortFicheGroup(String name, String tagName, boolean asc) {
        this.name = name;
        this.tagName = tagName;
        this.asc = asc;
    }

    public String getName() {
        return name;
    }

    public String getTagName() {
        return tagName;
    }

    public void setMatchingFicheItemArray(FicheItem[] matchingFicheItemArray) {
        this.matchingFicheItemArray = matchingFicheItemArray;
    }

    public FicheItem[] getMatchingFicheItemArray() {
        return matchingFicheItemArray;
    }

    public boolean isBottomGroup() {
        return true;
    }

    public Map getAttributesMap() {
        return null;
    }

    public FicheExtractInfo[] getFicheExtractInfoArray() {
        Collections.sort(bundleList, new InternalComparator(asc));
        int size = bundleList.size();
        FicheExtractInfo[] array = new FicheExtractInfo[size];
        for (int i = 0; i < size; i++) {
            array[i] = ((FicheExtractBundle) bundleList.get(i)).toFicheExtractInfo();
        }
        return array;
    }

    public void add(FicheExtractBundle ficheExtractBundle) {
        bundleList.add(ficheExtractBundle);
    }

    public FicheGroup[] getSubgroupArray() {
        return null;
    }


    private static class InternalComparator implements Comparator {

        boolean asc;

        InternalComparator(boolean asc) {
            this.asc = asc;
        }

        public int compare(Object obj1, Object obj2) {
            int result = internalCompare(obj1, obj2);
            if (!asc) {
                return -result;
            }
            return result;
        }

        private int internalCompare(Object obj1, Object obj2) {
            FicheExtractBundle feb1 = (FicheExtractBundle) obj1;
            FicheExtractBundle feb2 = (FicheExtractBundle) obj2;
            int in1 = feb1.getInnerSortValue();
            int in2 = feb2.getInnerSortValue();
            if (in1 < in2) {
                return -1;
            }
            if (in1 > in2) {
                return 1;
            }
            int id1 = feb1.getAbregeFiche().getIdcorpus();
            int id2 = feb2.getAbregeFiche().getIdcorpus();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
