/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction.params;

import net.mapeadores.util.text.insertpattern.api.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import java.text.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class GroupParamsParser {

    private static String ALPHABET_MODE = "alphabet";
    private static String MODE_KEY = "mode";
    private static String FIELDS_KEY = "fields";
    private static String FIELD_KEY = "field";

    public static FieldsGroupParams parseFieldsGroupParams(Instruction instruction) {
        return internalParseFieldsGroupParams(instruction, true);
    }

    public static FieldsGroupParams parseTitreGroupParams(Instruction instruction) {
        return internalParseFieldsGroupParams(instruction, false);
    }

    public static FicheItemGroupParams parseFicheItemGroupParams(Instruction instruction) {
        FieldKey fieldKey = null;
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            String value = argument.getValue();
            if (key.equals(FIELD_KEY)) {
                if (value != null) {
                    try {
                        fieldKey = FieldKey.parse(value);
                    } catch (java.text.ParseException pe) {
                    }
                }
            } else if (value == null) {
                try {
                    fieldKey = FieldKey.parse(key);
                } catch (java.text.ParseException pe) {
                }
            }
        }
        if (fieldKey == null) {
            return null;
        }
        if (!fieldKey.isFicheItemField(2)) {
            return null;
        }
        return new InternalFicheItemGroupParams(fieldKey);
    }

    private static FieldsGroupParams internalParseFieldsGroupParams(Instruction instruction, boolean testFields) {
        short mode = FieldsGroupParams.DEFAULT_MODE;
        FieldKey[] fieldKeyArray = null;
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            String value = argument.getValue();
            if (key.equals(ALPHABET_MODE)) {
                mode = FieldsGroupParams.ALPHABET_MODE;
            } else if (key.equals(MODE_KEY)) {
                if (value != null) {
                    if (value.equals(ALPHABET_MODE)) {
                        mode = FieldsGroupParams.ALPHABET_MODE;
                    } else {
                        mode = FieldsGroupParams.DEFAULT_MODE;
                    }
                }
            } else if (key.equals(FIELDS_KEY)) {
                if ((testFields) && (value != null)) {
                    fieldKeyArray = parseFieldArray(value);
                }
            }
        }
        if (!testFields) {
            fieldKeyArray = new FieldKey[1];
            fieldKeyArray[0] = FieldKey.TITRE;
        }
        if (fieldKeyArray == null) {
            return null;
        }
        return new InternalFieldsGroupParams(mode, fieldKeyArray);
    }

    private static FieldKey[] parseFieldArray(String s) {
        String[] tokenArray = StringUtils.getTechnicalTokens(s, true);
        int length = tokenArray.length;
        if (length == 0) {
            return null;
        }
        FieldKey[] fieldArray = new FieldKey[length];
        for (int i = 0; i < length; i++) {
            try {
                FieldKey fieldKey = FieldKey.parse(tokenArray[i]);
                if (testFieldKey(fieldKey)) {
                    fieldArray[i] = fieldKey;
                } else {
                    return null;
                }
            } catch (ParseException pe) {
                return null;
            }
        }
        return fieldArray;
    }

    private static boolean testFieldKey(FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.TITRE)) {
            return true;
        }
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            return true;
        }
        if (fieldKey.isProprieteFieldKey()) {
            return true;
        }
        if (fieldKey.isInformationFieldKey()) {
            return true;
        }
        return false;
    }


    private static class InternalFieldsGroupParams implements FieldsGroupParams {

        private short mode;
        private FieldKey[] fieldKeyArray;

        InternalFieldsGroupParams(short mode, FieldKey[] fieldKeyArray) {
            this.mode = mode;
            this.fieldKeyArray = fieldKeyArray;
        }

        public short getMode() {
            return mode;
        }

        public FieldKey[] getFieldKeyArray() {
            return fieldKeyArray;
        }

    }


    private static class InternalFicheItemGroupParams implements FicheItemGroupParams {

        private FieldKey fieldKey;

        InternalFicheItemGroupParams(FieldKey fieldKey) {
            this.fieldKey = fieldKey;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

    }

}
