/* BasedefichesLib_Explore
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute;

import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.format.*;
import net.mapeadores.basedefiches.format.patterns.*;
import net.mapeadores.basedefiches.pointeurs.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import java.text.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.format.compute.sources.*;
import net.mapeadores.basedefiches.format.compute.resolvers.*;


/**
 *
 * @author Vincent Calame
 */
public class ComputedBasedefichesFormatDef {

    private BasedefichesFormatDef basedefichesFormatDef;
    private Subset currentSubset;
    private List resolverList = new ArrayList();
    private FicheItemFormat defaultPropItemFormat;

    private ComputedBasedefichesFormatDef(Subset currentSubset, BasedefichesFormatDef basedefichesFormatDef) {
        this.basedefichesFormatDef = basedefichesFormatDef;
        this.currentSubset = currentSubset;
    }

    public BasedefichesFormatDef getBasedefichesFormatDef() {
        return basedefichesFormatDef;
    }

    public static ComputedBasedefichesFormatDef compute(Subset subset, BasedefichesFormatDef basedefichesFormatDef, ErrorHandler errorHandler) {
        ComputedBasedefichesFormatDef cffd = new ComputedBasedefichesFormatDef(subset, basedefichesFormatDef);
        boolean done = cffd.init(errorHandler);
        if (done) {
            return cffd;
        } else {
            return null;
        }
    }

    private boolean init(ErrorHandler errorHandler) {
        Initializer initializer = new Initializer(errorHandler);
        return initializer.init();
    }


    private class Initializer {

        private ErrorHandler errorHandler;
        private Corpus currentCorpus;
        private SubsetKey slaveCorpusKey;
        private Basedefiches basedefiches;

        Initializer(ErrorHandler errorHandler) {
            this.errorHandler = errorHandler;
            basedefiches = currentSubset.getBasedefiches();
        }

        boolean init() {
            int size = basedefichesFormatDef.getFormatSourceKeyCount();
            for (int i = 0; i < size; i++) {
                FormatSourceKey formatSourceKey = basedefichesFormatDef.getFormatSourceKey(i);
                String formatPattern = basedefichesFormatDef.getFormatPattern(i);
                if (!init(formatSourceKey, formatPattern)) {
                    return false;
                }
            }
            if (currentSubset instanceof Corpus) {
                FieldKey defaultPropKey = basedefichesFormatDef.getDefaultProprieteKey();
                if (defaultPropKey != null) {
                    CorpusField corpusField = ((Corpus) currentSubset).getCorpusMetadata().getCorpusFieldByFieldKey(defaultPropKey);
                    if (corpusField != null) {
                        int ficheItemType = corpusField.getFicheItemType();
                        Object obj = resolverList.get(0);
                        if (obj instanceof FicheItemFormatResolver) {
                            FicheItemFormatResolver fifr = (FicheItemFormatResolver) obj;
                            if (fifr.isSameType(ficheItemType)) {
                                defaultPropItemFormat = fifr.getFicheItemFormat();
                            } else {
                                defaultPropItemFormat = FicheItemFormat.newDefaultInstance(ficheItemType);
                            }
                        } else {
                            defaultPropItemFormat = FicheItemFormat.newDefaultInstance(ficheItemType);
                        }

                    }
                }
            }
            return true;
        }

        private boolean init(FormatSourceKey formatSourceKey, String formatPattern) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            if (formatSourceKey.isFieldKey()) {
                return initFieldKey(formatSourceKey, formatPattern);
            } else if (formatSourceKey.isSubfieldKey()) {
                return initSubfieldKey(formatSourceKey, formatPattern);
            } else if (formatSourceKey.isFilteredIncludeKey()) {
                return initFilteredIncludeKey(formatSourceKey, formatPattern);
            } else if (formatSourceKey.isThesaurusFieldKey()) {
                return initThesaurusFieldKey(formatSourceKey, formatPattern);
            } else {
                throw new SwitchException("FormatSourceKey ?");
            }
        }

        private boolean initCurrentCorpus(FormatSourceKey formatSourceKey) {
            currentCorpus = null;
            slaveCorpusKey = formatSourceKey.getSlaveCorpusKey();
            if (slaveCorpusKey != null) {
                currentCorpus = (Corpus) basedefiches.getSubset(slaveCorpusKey);
                if (currentCorpus == null) {
                    errorHandler.unknownSubsetKey(slaveCorpusKey);
                    return false;
                }
                Subset masterSubset = currentCorpus.getMasterSubset();
                if ((masterSubset == null) || (!masterSubset.equals(currentSubset))) {
                    currentCorpus = null;
                    errorHandler.notASlaveCorpusKey(slaveCorpusKey);
                    return false;
                }
            } else {
                if (currentSubset instanceof Corpus) {
                    currentCorpus = (Corpus) currentSubset;
                }
            }
            return true;
        }

        private boolean initThesaurusFieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
            if (!(currentSubset instanceof Thesaurus)) {
                errorHandler.illegalFormatSourceKey(formatSourceKey.toString());
                return false;
            }

            /*MotcleFormat motcleFormat = initMotcleFormat(thesaurus, formatPattern, errorHandler);
            if (motcleFormat == null) {
            return false;
            }*/
            addFormatResolver(new ThesaurusFieldFormatResolver((Thesaurus) currentSubset, formatSourceKey.getThesaurusFieldKey()));
            return true;
        }

        private boolean initFieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
            FieldKey fieldKey = formatSourceKey.getFieldKey();
            if (!testCurrentCorpus(fieldKey)) {
                return false;
            }
            CorpusField corpusField = currentCorpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                errorHandler.unknownFieldKey(fieldKey);
                return false;
            }
            if (fieldKey.ownsToCorpsdefiche()) {
                addFormatResolver(new FicheBlockFormatResolver(corpusField, slaveCorpusKey));
            } else if ((fieldKey.equals(FieldKey.IDCORPUS)) || (fieldKey.equals(FieldKey.TITRE))) {
                addFormatResolver(new SimpleValueFormatResolver(corpusField, slaveCorpusKey));
            } else {
                FicheItemFormat ficheItemFormat = null;
                try {
                    ficheItemFormat = FicheItemFormat.newInstance(corpusField.getFicheItemType(), formatPattern);
                    if (ficheItemFormat instanceof PersonneFormat) {
                        ((PersonneFormat) ficheItemFormat).setBasedefiches(currentSubset.getBasedefiches());
                    }
                } catch (ParseException pe) {
                    errorHandler.wrongFormatPattern(formatPattern);
                    return false;
                }
                FormatResolver formatResolver = createFicheItemFormatResolver(corpusField, ficheItemFormat);
                addFormatResolver(formatResolver);
            }
            return true;
        }

        private FormatResolver createFicheItemFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat) {
            FicheItemFormatResolver formatResolver = FicheItemFormatResolver.newInstance(corpusField, ficheItemFormat, slaveCorpusKey);
            formatResolver.setBasedefichesFormatDef(basedefichesFormatDef);
            return formatResolver;
        }

        private boolean initSubfieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
            SubfieldKey subfieldKey = formatSourceKey.getSubfieldKey();
            if (!testCurrentCorpus(subfieldKey)) {
                return false;
            }
            FieldKey fieldKey = subfieldKey.getFieldKey();
            CorpusField corpusField = currentCorpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                errorHandler.unknownSubfieldKey(subfieldKey);
                return false;
            }
            if (!SubfieldKey.isLegalSubfield(subfieldKey.getSubtype(), corpusField.getFicheItemType())) {
                errorHandler.illegalSubfieldKey(subfieldKey);
                return false;
            }
            SubfieldFormatResolver formatResolver = SubfieldFormatResolver.newInstance(subfieldKey, slaveCorpusKey);
            formatResolver.setBasedefiches(basedefiches);
            formatResolver.setBasedefichesFormatDef(basedefichesFormatDef);
            addFormatResolver(formatResolver);
            return true;
        }

        private boolean testCurrentCorpus(Object obj) {
            if (currentCorpus == null) {
                errorHandler.illegalFormatSourceKey(obj.toString());
                return false;
            } else {
                return true;
            }
        }

        private boolean initFilteredIncludeKey(FormatSourceKey formatSourceKey, String formatPattern) {
            FilteredIncludeKey filteredIncludeKey = formatSourceKey.getFilteredIncludeKey();
            IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
            if (includeKey.isChronoIncludeKey()) {
                if (!testCurrentCorpus(includeKey)) {
                    return false;
                }
                FicheItemFormat ficheItemFormat;
                try {
                    ficheItemFormat = FicheItemFormat.newInstance(CorpusField.DATATION_FIELD, formatPattern);
                } catch (ParseException pe) {
                    errorHandler.wrongFormatPattern(formatPattern);
                    return false;
                }
                addFormatResolver(new ChronoFormatResolver(includeKey, ficheItemFormat, slaveCorpusKey));
            } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                if (!initCurrentCorpus(formatSourceKey)) {
                    return false;
                }
                DocumentFormat documentFormat = null;
                if (formatPattern == null) {
                    formatPattern = DocumentFormat.getDefaultPattern();
                }
                try {
                    documentFormat = new DocumentFormat();
                    documentFormat.setPattern(formatPattern);
                } catch (ParseException pe) {
                    errorHandler.wrongFormatPattern(formatPattern);
                    return false;
                }
                addFormatResolver(new AssociationFormatResolver(documentFormat, slaveCorpusKey));
            } else if (includeKey.equals(IncludeKey.LIAGE)) {
                if (!initCurrentCorpus(formatSourceKey)) {
                    return false;
                }
                AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                if (abregeFicheFormat == null) {
                    return false;
                }
                addFormatResolver(new SpecialLiageResolver(abregeFicheFormat, slaveCorpusKey));
            } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                if (!initCurrentCorpus(formatSourceKey)) {
                    return false;
                }
                Subset masterSubset = currentSubset.getMasterSubset();
                Subset[] slaveSubsetArray = currentSubset.getSlaveSubsetArray();
                if (masterSubset != null) {
                    if (masterSubset instanceof Corpus) {
                        AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                        if (abregeFicheFormat == null) {
                            return false;
                        }
                        addFormatResolver(new CorpusFormatResolver((Corpus) masterSubset, abregeFicheFormat, -1, slaveCorpusKey));
                    } else if (masterSubset instanceof Thesaurus) {
                        Thesaurus thesaurus = (Thesaurus) masterSubset;
                        MotcleFormat motcleFormat = initMotcleFormat(thesaurus, formatPattern, errorHandler);
                        if (motcleFormat == null) {
                            return false;
                        }
                        addFormatResolver(new ThesaurusFormatResolver(thesaurus, motcleFormat, filteredIncludeKey.getPoidsFilter(), slaveCorpusKey));
                    }
                } else if (slaveSubsetArray != null) {
                    AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                    if (abregeFicheFormat == null) {
                        return false;
                    }
                    addFormatResolver(new SpecialSlaveCorpusResolver(abregeFicheFormat, slaveCorpusKey));
                } else {
                    addFormatResolver(new EmptyFormatResolver());
                }

            } else if (includeKey.isLiageIncludeKey()) {
                if (!initCurrentCorpus(formatSourceKey)) {
                    return false;
                }
                Corpus corpus = (Corpus) basedefiches.getSubset(includeKey.getSubsetKey());
                if (corpus == null) {
                    errorHandler.unknownSubsetKey(includeKey.getSubsetKey());
                    return false;
                }
                AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                if (abregeFicheFormat == null) {
                    return false;
                }
                addFormatResolver(new CorpusFormatResolver(corpus, abregeFicheFormat, filteredIncludeKey.getPoidsFilter(), slaveCorpusKey));
            } else if (includeKey.isIndexationIncludeKey()) {
                if (!initCurrentCorpus(formatSourceKey)) {
                    return false;
                }
                Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(includeKey.getSubsetKey());
                if (thesaurus == null) {
                    errorHandler.unknownSubsetKey(includeKey.getSubsetKey());
                    return false;
                }
                MotcleFormat motcleFormat = initMotcleFormat(thesaurus, formatPattern, errorHandler);
                if (motcleFormat == null) {
                    return false;
                }
                addFormatResolver(new ThesaurusFormatResolver(thesaurus, motcleFormat, filteredIncludeKey.getPoidsFilter(), slaveCorpusKey));
            }
            return true;
        }

    }

    private AbregeFicheFormat initAbregeFicheFormat(String formatPattern, ErrorHandler errorHandler) {
        AbregeFicheFormat abregeFicheFormat = null;
        if (formatPattern == null) {
            formatPattern = AbregeFicheFormat.getDefaultPattern();
        }
        try {
            abregeFicheFormat = new AbregeFicheFormat();
            abregeFicheFormat.setPattern(formatPattern);
            return abregeFicheFormat;
        } catch (ParseException pe) {
            errorHandler.wrongFormatPattern(formatPattern);
            return null;
        }
    }

    private MotcleFormat initMotcleFormat(Thesaurus thesaurus, String formatPattern, ErrorHandler errorHandler) {
        MotcleFormat motcleFormat = null;
        if (formatPattern == null) {
            formatPattern = MotcleFormat.getDefaultPattern(ThesaurusUtils.isIdalphaType(thesaurus));
        }
        try {
            motcleFormat = new MotcleFormat();
            motcleFormat.setPattern(formatPattern);
            return motcleFormat;
        } catch (ParseException pe) {
            errorHandler.wrongFormatPattern(formatPattern);
            return null;
        }
    }

    private void addFormatResolver(FormatResolver formatResolver) {
        resolverList.add(formatResolver);
    }

    private String getDefaultString(SubsetItemPointeur subsetItemPointeur, LocalisationInfo localisationInfo) {
        String defaultString = null;
        if (defaultPropItemFormat != null) {
            FieldKey defaultPropKey = basedefichesFormatDef.getDefaultProprieteKey();
            Object defObject = ((FichePointeur) subsetItemPointeur).getValue(defaultPropKey);
            if (defObject != null) {
                defaultString = defaultPropItemFormat.formatFicheItem((FicheItem) defObject, localisationInfo);
            }
        }
        if (defaultString == null) {
            defaultString = basedefichesFormatDef.getDefaultValue();
        }
        return defaultString;
    }

    /**
     * Retourne une value nulle ou une liste avec au moins un élement non nul.
     */
    public List<String> formatToList(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        testSubsetItemPointeur(subsetItemPointeur);
        FormatResult formatResult = resolve(subsetItemPointeur, includeSummaryMap, localisationInfo);
        List<String> result = formatResult.getResultList();
        if (result == null) {
            String s = getDefaultString(subsetItemPointeur, localisationInfo);
            if (s != null) {
                result = Collections.singletonList(s);
            }
        }
        return result;
    }

    public String formatToString(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        testSubsetItemPointeur(subsetItemPointeur);
        FormatResult formatResult = resolve(subsetItemPointeur, includeSummaryMap, localisationInfo);
        String result = formatResult.getResultString();
        if (result == null) {
            result = getDefaultString(subsetItemPointeur, localisationInfo);
        }
        return result;
    }

    private void testSubsetItemPointeur(SubsetItemPointeur subsetItemPointeur) {
        if (currentSubset instanceof Corpus) {
            if (!(subsetItemPointeur instanceof FichePointeur)) {
                throw new IllegalArgumentException("subsetItemPointeur must be an instance of FichePointeur");
            }
        }
    }

    private FormatResult resolve(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        FormatSource formatSource = FormatComputeUtils.toFormatSource(subsetItemPointeur, includeSummaryMap, localisationInfo);
        int position = basedefichesFormatDef.getPosition();
        if (position != -1) {
            ImplPositionFormatSource positionFormatSource = new ImplPositionFormatSource(formatSource, position, basedefichesFormatDef.fillByLast());
            positionFormatSource.setResolverList(resolverList);
            return positionFormatSource.resolve();
        } else {
            ImplLimitFormatSource limitFormatSource = new ImplLimitFormatSource(formatSource);
            limitFormatSource.setFormatDef(basedefichesFormatDef);
            limitFormatSource.setResolverList(resolverList);
            return limitFormatSource.resolve();
        }
    }


    public static interface ErrorHandler {

        public void unknownSubfieldKey(SubfieldKey subfieldKey);

        public void illegalSubfieldKey(SubfieldKey subfieldKey);

        public void wrongFormatPattern(String formatPattern);

        public void unknownFieldKey(FieldKey fieldKey);

        public void unknownSubsetKey(SubsetKey subsetKey);

        public void notASlaveCorpusKey(SubsetKey subsetKey);

        public void illegalFormatSourceKey(String keyString);

    }

}
