/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.format.patterns.*;
import net.mapeadores.basedefiches.pointeurs.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.format.compute.*;


/**
 *
 * @author Vincent Calame
 */
public class CorpusFormatResolver extends AbstractFormatResolver {

    private Corpus corpus;

    ;
    private AbregeFicheFormat abregeFicheFormat;
    private int poidsFilter;

    public CorpusFormatResolver(Corpus corpus, AbregeFicheFormat abregeFicheFormat, int poidsFilter, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.corpus = corpus;
        this.abregeFicheFormat = abregeFicheFormat;
        this.poidsFilter = poidsFilter;
    }

    public void getInPosition(PositionFormatBuilder formatBuilder) {
        FicheCroisementList abregeFicheList = getAbregeFicheList(formatBuilder);
        int size = abregeFicheList.getSubsetItemCount();
        int position = formatBuilder.checkPosition(size);
        if (position != -1) {
            formatBuilder.set(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(position), abregeFicheList.getPoids(position)));
        }
    }

    public void addInList(LimitFormatBuilder formatBuilder) {
        FicheCroisementList abregeFicheList = getAbregeFicheList(formatBuilder);
        int size = abregeFicheList.getSubsetItemCount();
        for (int i = 0; i < size; i++) {
            boolean cont = formatBuilder.add(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(i), abregeFicheList.getPoids(i)));
            if (!cont) {
                break;
            }
        }
    }

    private FicheCroisementList getAbregeFicheList(FormatBuilder formatBuilder) {
        CorpusCroisementAccess acces = getFichePointeur(formatBuilder).getCorpusCroisementAccess(corpus);
        if (poidsFilter > -1) {
            return acces.getFicheCroisement(poidsFilter);
        } else {
            IncludeSummary includeSummary = getIncludeSummary(formatBuilder);
            RangeList rangeList = null;
            if (includeSummary != null) {
                IncludeSummary.IncludeKeyInfo info = includeSummary.getIncludeKeyInfo(IncludeKey.createLiageInstance(corpus.getSubsetKey()));
                if (info != null) {
                    rangeList = info.getPoidsFilterRangeList();
                }
            }
            return acces.getFicheCroisement(rangeList);
        }
    }

}
