/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.format.*;
import net.mapeadores.basedefiches.format.patterns.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.format.compute.*;
import net.mapeadores.util.localisation.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheItemFormatResolver extends AbstractFormatResolver {

    CorpusField corpusField;
    FicheItemFormat ficheItemFormat;
    BasedefichesFormatDef basedefichesFormatDef;

    private FicheItemFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.corpusField = corpusField;
        this.ficheItemFormat = ficheItemFormat;
    }

    public void setBasedefichesFormatDef(BasedefichesFormatDef basedefichesFormatDef) {
        this.basedefichesFormatDef = basedefichesFormatDef;
    }

    String formatFicheItem(FicheItem ficheItem, LocalisationInfo localisationInfo) {
        if ((basedefichesFormatDef.isNoItem()) && (ficheItem instanceof Item)) {
            if (corpusField.getFicheItemType() != CorpusField.ITEM_FIELD) {
                return null;
            }
        }
        return ficheItemFormat.formatFicheItem(ficheItem, localisationInfo);
    }

    public boolean isSameType(int ficheItemType) {
        return (corpusField.getFicheItemType() == ficheItemType);
    }

    public FicheItemFormat getFicheItemFormat() {
        return ficheItemFormat;
    }

    public static FicheItemFormatResolver newInstance(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if ((fieldKey.isProprieteFieldKey()) || (fieldKey.equals(FieldKey.LANG)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return new ProprieteFormatResolver(corpusField, ficheItemFormat, slaveCorpusKey);
        } else {
            return new InformationFormatResolver(corpusField, ficheItemFormat, slaveCorpusKey);
        }
    }


    private static class ProprieteFormatResolver extends FicheItemFormatResolver {

        private ProprieteFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey) {
            super(corpusField, ficheItemFormat, slaveCorpusKey);
        }

        public void getInPosition(PositionFormatBuilder positionFormatBuilder) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(positionFormatBuilder).getValue(corpusField.getFieldKey());
            if (ficheItem != null) {
                int position = positionFormatBuilder.checkPosition(1);
                if (position != -1) {
                    positionFormatBuilder.set(formatFicheItem(ficheItem, getLocalisationInfo(positionFormatBuilder)));
                }
            }
        }

        public void addInList(LimitFormatBuilder limitFormatBuilder) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(limitFormatBuilder).getValue(corpusField.getFieldKey());
            if (ficheItem != null) {
                limitFormatBuilder.add(formatFicheItem(ficheItem, getLocalisationInfo(limitFormatBuilder)));
            }
        }

    }


    private static class InformationFormatResolver extends FicheItemFormatResolver {

        private InformationFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey) {
            super(corpusField, ficheItemFormat, slaveCorpusKey);
        }

        public void getInPosition(PositionFormatBuilder positionFormatBuilder) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(positionFormatBuilder).getValue(corpusField.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            int position = positionFormatBuilder.checkPosition(size);
            if (position != -1) {
                positionFormatBuilder.set(formatFicheItem(ficheItemList.getFicheItem(position), getLocalisationInfo(positionFormatBuilder)));
            }

        }

        public void addInList(LimitFormatBuilder limitFormatBuilder) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(limitFormatBuilder).getValue(corpusField.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            for (int i = 0; i < size; i++) {
                boolean cont = limitFormatBuilder.add(formatFicheItem(ficheItemList.getFicheItem(i), getLocalisationInfo(limitFormatBuilder)));
                if (!cont) {
                    break;
                }
            }

        }

    }

}
