/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.format.patterns.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.format.compute.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class SpecialCorpusArrayResolver extends AbstractFormatResolver {

    AbregeFicheFormat abregeFicheFormat;

    public SpecialCorpusArrayResolver(AbregeFicheFormat abregeFicheFormat, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.abregeFicheFormat = abregeFicheFormat;
    }

    public void getInPosition(PositionFormatBuilder formatBuilder) {
        FicheCroisementList[] array = getArray(formatBuilder);
        int size = getSize(array);
        int position = formatBuilder.checkPosition(size);
        if (position != -1) {
            int length = array.length;
            for (int i = 0; i < length; i++) {
                FicheCroisementList abregeFicheList = array[i];
                if (position >= abregeFicheList.getSubsetItemCount()) {
                    position = position - abregeFicheList.getSubsetItemCount();
                } else {
                    formatBuilder.set(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(position), abregeFicheList.getPoids(position)));
                    break;
                }
            }
        }
    }

    public void addInList(LimitFormatBuilder formatBuilder) {
        FicheCroisementList[] array = getArray(formatBuilder);
        int size = getSize(array);
        int length = array.length;
        int p = 0;
        for (int i = 0; i < length; i++) {
            FicheCroisementList abregeFicheList = array[i];
            int curSize = abregeFicheList.getSubsetItemCount();
            curSize = Math.min(size - p, curSize);
            boolean brekCont = false;
            for (int j = 0; j < curSize; j++) {
                boolean cont = formatBuilder.add(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(j), abregeFicheList.getPoids(j)));
                if (!cont) {
                    brekCont = true;
                    break;
                }
                p++;
            }
            if (brekCont) {
                break;
            }
        }
    }

    protected abstract FicheCroisementList[] getArray(FormatBuilder formatBuilder);

    private int getSize(FicheCroisementList[] array) {
        int size = 0;
        int length = array.length;
        for (int i = 0; i < length; i++) {
            size = size + array[i].getSubsetItemCount();
        }
        return size;
    }

}
