/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.format.*;
import net.mapeadores.basedefiches.format.patterns.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.format.compute.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class SubfieldFormatResolver extends AbstractFormatResolver {

    BasedefichesFormatDef basedefichesFormatDef;
    Basedefiches basedefiches;
    SubfieldKey subfieldKey;

    private SubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.subfieldKey = subfieldKey;
    }

    public void setBasedefichesFormatDef(BasedefichesFormatDef basedefichesFormatDef) {
        this.basedefichesFormatDef = basedefichesFormatDef;
    }

    public void setBasedefiches(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    protected String format(FicheItem ficheItem) {
        if ((basedefichesFormatDef.isNoItem()) && (ficheItem instanceof Item)) {
            return null;
        }
        return FormatUtils.formatSubfield(subfieldKey.getSubtype(), ficheItem, basedefiches);
    }

    public static SubfieldFormatResolver newInstance(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
        FieldKey fieldKey = subfieldKey.getFieldKey();
        if (fieldKey.isProprieteFieldKey()) {
            return new ProprieteSubfieldFormatResolver(subfieldKey, slaveCorpusKey);
        } else {
            return new InformationSubfieldFormatResolver(subfieldKey, slaveCorpusKey);
        }
    }


    private static class ProprieteSubfieldFormatResolver extends SubfieldFormatResolver {

        private ProprieteSubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
            super(subfieldKey, slaveCorpusKey);

        }

        public void getInPosition(PositionFormatBuilder formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItem != null) {
                int position = formatSource.checkPosition(1);
                if (position != -1) {
                    formatSource.set(format(ficheItem));
                }
            }
        }

        public void addInList(LimitFormatBuilder formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItem != null) {
                formatSource.add(format(ficheItem));
            }
        }

    }


    private static class InformationSubfieldFormatResolver extends SubfieldFormatResolver {

        private InformationSubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
            super(subfieldKey, slaveCorpusKey);
        }

        public void getInPosition(PositionFormatBuilder formatBuilder) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatBuilder).getValue(subfieldKey.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            int position = formatBuilder.checkPosition(size);
            if (position != -1) {
                formatBuilder.set(format(ficheItemList.getFicheItem(position)));
            }
        }

        public void addInList(LimitFormatBuilder formatBuilder) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatBuilder).getValue(subfieldKey.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            for (int i = 0; i < size; i++) {
                boolean cont = formatBuilder.add(format(ficheItemList.getFicheItem(i)));
                if (!cont) {
                    break;
                }
            }
        }

    }

}
