/* BasedefichesLib_Explore - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.format.compute.FormatBuilder;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.pointeurs.ThesaurusCroisementAccess;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusFormatResolver extends AbstractFormatResolver {

    private Thesaurus thesaurus;
    private MotcleFormat motcleFormat;
    private int poidsFilter;

    public ThesaurusFormatResolver(Thesaurus thesaurus, MotcleFormat motcleFormat, int poidsFilter, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.thesaurus = thesaurus;
        this.motcleFormat = motcleFormat;
        this.poidsFilter = poidsFilter;
    }

    public void getInPosition(PositionFormatBuilder positionFormatBuilder) {
        MotcleCroisementList motcleList = getMotcleList(positionFormatBuilder);
        int size = motcleList.getSubsetItemCount();
        int position = positionFormatBuilder.checkPosition(size);
        if (position != -1) {
            positionFormatBuilder.set(motcleFormat.formatMotcle(motcleList.getMotcle(position), motcleList.getPoids(position), getLocalisationInfo(positionFormatBuilder)));
        }
    }

    public void addInList(LimitFormatBuilder limitFormatBuilder) {
        MotcleCroisementList motcleList = getMotcleList(limitFormatBuilder);
        int size = motcleList.getSubsetItemCount();
        for (int i = 0; i < size; i++) {
            boolean cont = limitFormatBuilder.add(motcleFormat.formatMotcle(motcleList.getMotcle(i), motcleList.getPoids(i), getLocalisationInfo(limitFormatBuilder)));
            if (!cont) {
                break;
            }
        }
    }

    private MotcleCroisementList getMotcleList(FormatBuilder formatSource) {
        ThesaurusCroisementAccess acces = getFichePointeur(formatSource).getThesaurusCroisementAccess(thesaurus);
        if (poidsFilter > -1) {
            return acces.getMotcleCroisementList(poidsFilter);
        } else {
            RangeList rangeList = null;
            IncludeSummary includeSummary = getIncludeSummary(formatSource);
            if (includeSummary != null) {
                IncludeSummary.IncludeKeyInfo info = includeSummary.getIncludeKeyInfo(IncludeKey.createIndexationInstance(thesaurus.getSubsetKey()));
                if (info != null) {
                    rangeList = info.getPoidsFilterRangeList();
                }
            }
            return acces.getMotcleCroisementList(rangeList);
        }
    }

}
