/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.sources;

import java.util.*;
import net.mapeadores.util.format.*;
import net.mapeadores.basedefiches.format.compute.*;


/**
 *
 * @author Vincent Calame
 */
public class ImplLimitFormatSource extends ImplFormatSource implements LimitFormatBuilder {

    private int limit;
    private List<String> arrayList = new ArrayList<String>();
    private List positionList = new ArrayList();
    private boolean resolved = false;
    private Integer currentPosition = null;
    private FormatDef formatDef;

    public ImplLimitFormatSource(FormatSource formatSource) {
        super(formatSource);
    }

    public void setFormatDef(FormatDef formatDef) {
        this.formatDef = formatDef;
        this.limit = formatDef.getLimit();
    }

    public boolean add(String s) {
        if (resolved) {
            return false;
        }
        if (s == null) {
            return true;
        }
        positionList.add(currentPosition);
        arrayList.add(s);
        if (limit != -1) {
            if (arrayList.size() == limit) {
                resolved = true;
                return false;
            }
        }
        return true;
    }

    public FormatResult resolve() {
        int size = resolverList.size();
        for (int i = 0; i < size; i++) {
            FormatResolver formatResolver = (FormatResolver) resolverList.get(i);
            currentPosition = new Integer(i);
            formatResolver.addInList(this);
            if (resolved) {
                break;
            }
        }
        return new InternalFormatResult();
    }


    private class InternalFormatResult implements FormatResult {

        public List<String> getResultList() {
            if (arrayList.size() == 0) {
                return null;
            }
            return arrayList;
        }

        public String getResultString() {
            if (arrayList.size() == 0) {
                return null;
            }
            StringBuffer buf = new StringBuffer();
            int size = arrayList.size();
            String separator = formatDef.getSeparator();
            Integer previousPosition = null;
            for (int i = 0; i < size; i++) {
                Integer position = (Integer) positionList.get(i);
                if (i > 0) {
                    if (position.equals(previousPosition)) {
                        buf.append(formatDef.getInternalSeparator(position.intValue()));
                    } else {
                        buf.append(formatDef.getSourceSeparator(previousPosition.intValue(), position.intValue()));
                    }
                }
                previousPosition = position;
                buf.append((String) arrayList.get(i));
            }
            return buf.toString();
        }

    }

}
