/* BasedefichesLib_Explore
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.format.compute.sources;

import java.util.*;
import net.mapeadores.util.format.*;
import net.mapeadores.basedefiches.format.compute.*;


/**
 *
 * @author Vincent Calame
 */
public class ImplPositionFormatSource extends ImplFormatSource implements PositionFormatBuilder {

    private int position;
    private String result;
    private boolean resolved = false;
    private boolean fillByLast;

    public ImplPositionFormatSource(FormatSource formatSource, int position, boolean fillByLast) {
        super(formatSource);
        this.position = position;
        this.fillByLast = fillByLast;
    }

    public FormatResult resolve() {
        if (position != FormatDef.MAX_POSITION) {
            int size = resolverList.size();
            for (int i = 0; i < size; i++) {
                FormatResolver formatResolver = (FormatResolver) resolverList.get(i);
                formatResolver.getInPosition(this);
                if (resolved) {
                    break;
                }
            }
            if ((!resolved) && (fillByLast)) {
                position = FormatDef.MAX_POSITION;
                resolveLastPosition();
            }
        } else {
            resolveLastPosition();
        }
        return new InternalFormatResult();
    }

    private void resolveLastPosition() {
        int size = resolverList.size();
        for (int i = (size - 1); i >= 0; i--) {
            FormatResolver formatResolver = (FormatResolver) resolverList.get(i);
            formatResolver.getInPosition(this);
            if (resolved) {
                break;
            }
        }
    }

    public int getPosition() {
        return position;
    }

    public int checkPosition(int length) {
        if (resolved) {
            return -1;
        }
        if (position == FormatDef.MAX_POSITION) {
            return (length - 1);
        }
        if (position < length) {
            return position;
        } else {
            position = position - length;
            return -1;
        }
    }

    public void set(String result) {
        this.result = result;
        resolved = true;
    }


    private class InternalFormatResult implements FormatResult {

        public List<String> getResultList() {
            if (result == null) {
                return null;
            }
            return Collections.singletonList(result);
        }

        public String getResultString() {
            return result;
        }

    }

}
