/* BasedefichesLib_Explore - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.misc;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public class CorpusFieldAndIncludeList {

    public final static short UNKNOWN_THESAURUS_ERROR = 1;
    public final static short UNKNOWN_CORPUS_ERROR = 2;
    public final static short UNAUTHORIZED_DOUBLON_ERROR = 3;
    public final static short UNKNOWN_FIELD_ERROR = 4;
    public final static short FORBIDDEN_SUBFIELD_ERROR = 5;
    public final static short FIELD_WITH_SUBFIELD_ERROR = 6;
    private List<ListItem> itemList = new ArrayList<ListItem>();
    private List<ListItem> parseErrorList = new ArrayList<ListItem>();
    private List<Object> keyList = new ArrayList<Object>();
    private List<ListItem>[] bdfErrorListArray = new List[6];

    {
        for (int i = 0; i < bdfErrorListArray.length; i++) {
            bdfErrorListArray[i] = new ArrayList<ListItem>();
        }
    }

    private Set<FieldKey> fieldWithSubfieldSet = new HashSet<FieldKey>();
    private Map<Object, KeyItem> keyMap = new HashMap<Object, KeyItem>();
    private CorpusMetadata corpusMetadata;
    private Basedefiches basedefiches;

    public CorpusFieldAndIncludeList(CorpusMetadata corpusMetadata) {
        this.corpusMetadata = corpusMetadata;
        basedefiches = corpusMetadata.getCorpus().getBasedefiches();
    }

    public void parse(String[] array) {
        int length = array.length;
        for (int i = 0; i < length; i++) {
            String token = array[i];
            if (!testSubfieldKey(i, token)) {
                if (!testFieldKey(i, token)) {
                    if (!testFilteredIncludeKey(i, token)) {
                        addParseError(i, token);
                    }
                }
            }
        }
    }

    public boolean hasError() {
        if (!parseErrorList.isEmpty()) {
            return true;
        }
        for (int i = 0; i < bdfErrorListArray.length; i++) {
            if (!bdfErrorListArray[i].isEmpty()) {
                return true;
            }
        }
        return false;
    }

    public List<ListItem> getParseErrorList() {
        return parseErrorList;
    }

    public List<ListItem> getBdfErrorList(short type) {
        return bdfErrorListArray[((short) type) - 1];
    }

    public Object[] getAllArray() {
        return itemList.toArray();
    }

    public int getKeyCount() {
        return keyList.size();
    }

    public Object[] getKeyArray() {
        return keyList.toArray();
    }

    private boolean testFilteredIncludeKey(int index, String name) {
        try {
            FilteredIncludeKey filteredIncludeKey = FilteredIncludeKey.parse(name);
            IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
            if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                return false;
            }
            if (includeKey.isLiageIncludeKey()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(includeKey.getSubsetKey());
                if (corpus == null) {
                    addBdfError(index, UNKNOWN_CORPUS_ERROR, filteredIncludeKey);
                } else {
                    addKeyItem(index, filteredIncludeKey);
                }
                return true;
            }
            if (includeKey.isIndexationIncludeKey()) {
                Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(includeKey.getSubsetKey());
                if (thesaurus == null) {
                    addBdfError(index, UNKNOWN_THESAURUS_ERROR, filteredIncludeKey);
                } else {
                    addKeyItem(index, filteredIncludeKey);
                }
                return true;
            }
            if (includeKey.equals(IncludeKey.LIAGE)) {
                addKeyItem(index, filteredIncludeKey);
            }
            return true;
        } catch (ParseException pe) {
            return false;
        }
    }

    private boolean testFieldKey(int index, String name) {
        try {
            FieldKey fieldKey = FieldKey.parse(name);
            if (fieldWithSubfieldSet.contains(fieldKey)) {
                addBdfError(index, FIELD_WITH_SUBFIELD_ERROR, fieldKey);
                return true;
            }
            CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                addBdfError(index, UNKNOWN_FIELD_ERROR, fieldKey);
            } else {
                addKeyItem(index, fieldKey);
            }
            return true;
        } catch (ParseException pe) {
            return false;
        }
    }

    private boolean testSubfieldKey(int index, String name) {
        try {
            SubfieldKey subfieldKey = SubfieldKey.parse(name);
            FieldKey fieldKey = subfieldKey.getFieldKey();
            fieldWithSubfieldSet.add(fieldKey);
            if (keyMap.containsKey(fieldKey)) {
                addBdfError(index, FIELD_WITH_SUBFIELD_ERROR, subfieldKey);
                return true;
            }
            CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                addBdfError(index, UNKNOWN_FIELD_ERROR, subfieldKey);
            } else if (!SubfieldKey.isLegalSubfield(subfieldKey.getSubtype(), corpusField.getFicheItemType())) {
                addBdfError(index, FORBIDDEN_SUBFIELD_ERROR, subfieldKey);
            } else {
                addKeyItem(index, subfieldKey);
            }
            return true;
        } catch (ParseException pe) {
            return false;
        }
    }

    private void addBdfError(int index, short type, Object key) {
        BdfErrorItem bdfErrorItem = new BdfErrorItem(index, key, type);
        getBdfErrorList(type).add(bdfErrorItem);
        itemList.add(bdfErrorItem);
    }

    private void addParseError(int index, String token) {
        ParseErrorItem pei = new ParseErrorItem(index, token);
        parseErrorList.add(pei);
        itemList.add(pei);
    }

    private void addKeyItem(int index, Object key) {
        if (keyMap.containsKey(key)) {
            if ((key.equals(FieldKey.IDCORPUS)) || (key.equals(FieldKey.LANG))) {
                addBdfError(index, UNAUTHORIZED_DOUBLON_ERROR, key);
            } else {
                DoublonItem doublonItem = new DoublonItem(index, keyMap.get(key));
                itemList.add(doublonItem);
            }
        } else {
            KeyItem keyItem = new KeyItem(index, key, keyList.size());
            keyMap.put(key, keyItem);
            itemList.add(keyItem);
            keyList.add(key);
        }
    }


    public static abstract class ListItem {

        private int index;

        ListItem(int index) {
            this.index = index;
        }

        public int getIndex() {
            return index;
        }

    }


    public static class KeyItem extends ListItem {

        private Object key;
        private int keyIndex;

        KeyItem(int index, Object key, int keyIndex) {
            super(index);
            this.key = key;
            this.keyIndex = keyIndex;
        }

        public Object getKey() {
            return key;
        }

        @Override
        public String toString() {
            return key.toString();
        }

        public int getKeyIndex() {
            return keyIndex;
        }

    }


    public static class DoublonItem extends ListItem {

        private KeyItem keyItem;

        DoublonItem(int index, KeyItem keyItem) {
            super(index);
            this.keyItem = keyItem;
        }

        public KeyItem getKeyItem() {
            return keyItem;
        }

        @Override
        public String toString() {
            return keyItem.toString();
        }

    }


    public static class ParseErrorItem extends ListItem {

        private String token;

        ParseErrorItem(int index, String token) {
            super(index);
            this.token = token;
        }

        public String getToken() {
            return token;
        }

        @Override
        public String toString() {
            return token;
        }

    }


    public static class BdfErrorItem extends ListItem {

        private Object key;
        private short type;

        BdfErrorItem(int index, Object key, short type) {
            super(index);
            this.key = key;
            this.type = type;
        }

        public Object getKey() {
            return key;
        }

        public short getType() {
            return type;
        }

        @Override
        public String toString() {
            return key.toString();
        }

    }

}
