/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.misc;

import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.text.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusResourceLocalisation implements ResourceLocalisation {

    private Thesaurus thesaurus;
    private int workingLangInteger;
    private ResourceLocalisation defaultResourceLocalisation;

    public ThesaurusResourceLocalisation(Thesaurus thesaurus, int workingLangInteger) {
        this.thesaurus = thesaurus;
        this.workingLangInteger = workingLangInteger;
    }

    public String toString(ResourceItem resourceItem) {
        return toString(resourceItem.getResourceKey(), resourceItem.getValue());
    }

    public String toString(String resourceKey, Object value) {
        Motcle motcle = thesaurus.getMotcleByIdalpha(resourceKey);
        if (motcle != null) {
            String lib = motcle.getLibelleString(workingLangInteger, false);
            if (lib == null) {
                return resourceKey;
            } else {
                return StringUtils.getMessage(lib, value);
            }
        } else if (defaultResourceLocalisation != null) {
            try {
                return defaultResourceLocalisation.toString(resourceKey, value);
            } catch (MissingResourceException mre) {
                return "?" + resourceKey + "?";
            }
        } else {
            return "?" + resourceKey + "?";
        }
    }

    public boolean containsKey(String resourceKey) {
        Motcle motcle = thesaurus.getMotcleByIdalpha(resourceKey);
        if (motcle != null) {
            return true;
        } else if (defaultResourceLocalisation != null) {
            return defaultResourceLocalisation.containsKey(resourceKey);
        } else {
            return false;
        }
    }

    public ResourceLocalisation getDefaultResourceLocalisation() {
        return defaultResourceLocalisation;
    }

    public void setDefaultResourceLocalisation(ResourceLocalisation defaultResourceLocalisation) {
        this.defaultResourceLocalisation = defaultResourceLocalisation;
    }

}
