/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.pointeurs;

import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCorpusCroisementAccess extends AbstractCroisementAccess implements CorpusCroisementAccess {

    Corpus otherCorpus;

    public DefaultCorpusCroisementAccess(Subset mainSubset, Corpus otherCorpus) {
        super(mainSubset, otherCorpus);
        this.otherCorpus = otherCorpus;
    }

    public AbregeFiche getFirstFiche() {
        int length = croisementItem.getCount();
        if (length == 0) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            int ticket = croisementItem.getTicket(i);
            AbregeFiche otherAF = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
            if (otherAF != null) {
                return otherAF;
            }
        }
        return null;
    }

    public AbregeFiche getFirstFiche(int poidsFilter) {
        if (poidsFilter == -1) {
            return getFirstFiche();
        }
        int length = croisementItem.getCount();
        if (length == 0) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (pds != poidsFilter) {
                continue;
            }
            AbregeFiche otherAF = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
            if (otherAF != null) {
                return otherAF;
            }
        }
        return null;
    }

    public FicheCroisementList getFicheCroisementList() {
        int length = croisementItem.getCount();
        DefaultFicheCroisementList result = new DefaultFicheCroisementList(otherCorpus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            AbregeFiche otherAF = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
            if (otherAF != null) {
                result.add(otherAF, pds);
            }
        }
        return result;
    }

    public FicheCroisementList getFicheCroisement(int poidsFilter) {
        if (poidsFilter == -1) {
            return getFicheCroisementList();
        }
        int length = croisementItem.getCount();
        DefaultFicheCroisementList result = new DefaultFicheCroisementList(otherCorpus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (pds != poidsFilter) {
                continue;
            }
            AbregeFiche otherAF = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
            if (otherAF != null) {
                result.add(otherAF, pds);
            }
        }
        return result;
    }

    public FicheCroisementList getFicheCroisement(RangeList excludePoidsFilter) {
        if (excludePoidsFilter == null) {
            return getFicheCroisementList();
        }
        int length = croisementItem.getCount();
        DefaultFicheCroisementList result = new DefaultFicheCroisementList(otherCorpus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (excludePoidsFilter.contains(pds)) {
                continue;
            }
            AbregeFiche otherAF = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
            if (otherAF != null) {
                result.add(otherAF, pds);
            }
        }
        return result;
    }

}
