/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.pointeurs;

import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultThesaurusCroisementAccess extends AbstractCroisementAccess implements ThesaurusCroisementAccess {

    Thesaurus thesaurus;

    DefaultThesaurusCroisementAccess(Subset mainSubset, Thesaurus thesaurus) {
        super(mainSubset, thesaurus);
        this.thesaurus = thesaurus;
    }

    public Motcle getFirstMotcle(int poidsFilter) {
        if (poidsFilter == -1) {
            return getFirstMotcle();
        }
        int length = croisementItem.getCount();
        if (length == 0) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (pds != poidsFilter) {
                continue;
            }
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                return motcle;
            }
        }
        return null;
    }

    public Motcle getFirstMotcle() {
        int length = croisementItem.getCount();
        if (length == 0) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            int ticket = croisementItem.getTicket(i);
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                return motcle;
            }
        }
        return null;
    }

    public MotcleCroisementList getMotcleCroisementList() {
        int length = croisementItem.getCount();
        DefaultMotcleCroisementList result = new DefaultMotcleCroisementList(thesaurus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                result.add(motcle, pds);
            }
        }
        return result;
    }

    public MotcleCroisementList getMotcleCroisementList(int poidsFilter) {
        if (poidsFilter == -1) {
            return getMotcleCroisementList();
        }
        int length = croisementItem.getCount();
        DefaultMotcleCroisementList result = new DefaultMotcleCroisementList(thesaurus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (pds != poidsFilter) {
                continue;
            }
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                result.add(motcle, pds);
            }
        }
        return result;
    }

    public MotcleCroisementList getMotcleCroisementList(RangeList excludePoidsFilter) {
        if (excludePoidsFilter == null) {
            return getMotcleCroisementList();
        }
        int length = croisementItem.getCount();
        DefaultMotcleCroisementList result = new DefaultMotcleCroisementList(thesaurus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int pds = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            if (excludePoidsFilter.contains(pds)) {
                continue;
            }
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                result.add(motcle, pds);
            }
        }
        return result;
    }

}
