/* BasedefichesLib_Explore - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.pointeurs;

import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import java.util.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FichePointeur implements SubsetItemPointeur {

    private boolean ficheComplete;
    private boolean withSlaveCorpusPointeur;
    private Corpus corpus;
    private Fiche fiche;
    private AbregeFiche abregeFiche;
    private boolean isSlaveInit = false;
    private HashMap<SubsetKey, AbstractCroisementAccess> accessMap = new HashMap<SubsetKey, AbstractCroisementAccess>();
    private DefaultDocumentCroisementAccess documentCroisementAccess;
    private Map<SubsetKey, FichePointeur> slaveFichePointeurMap;
    private Subset masterSubset;
    private Map<FieldKey, Object> fieldBufferMap = new HashMap<FieldKey, Object>();
    private Corpus[] rattachementCorpusArray;
    private boolean cacheEnable = false;
    private Map<Integer, Fiche> cacheFicheMap;

    public FichePointeur(Corpus corpus) {
        this(corpus, false, false);
    }

    public FichePointeur(Corpus corpus, boolean ficheComplete) {
        this(corpus, ficheComplete, false);
    }

    public FichePointeur(Corpus corpus, boolean ficheComplete, boolean withSlaveCorpusPointeur) {
        this.ficheComplete = ficheComplete;
        this.withSlaveCorpusPointeur = withSlaveCorpusPointeur;
        this.corpus = corpus;
        this.masterSubset = corpus.getMasterSubset();
    }

    public FichePointeur(AbregeFiche abregeFiche) {
        this(abregeFiche, false, false);
    }

    public FichePointeur(AbregeFiche abregeFiche, boolean ficheComplete) {
        this(abregeFiche, ficheComplete, false);
    }

    public FichePointeur(AbregeFiche abregeFiche, boolean ficheComplete, boolean withSlaveCorpusPointeur) {
        this(abregeFiche.getCorpus(), ficheComplete, withSlaveCorpusPointeur);
        setCurrent(abregeFiche);
    }

    public Corpus getCurrentCorpus() {
        return corpus;
    }

    public AbregeFiche getCurrentAbregeFiche() {
        return abregeFiche;
    }

    public void setCurrentSubsetItem(SubsetItem subsetItem) {
        setCurrent((AbregeFiche) subsetItem);
    }

    public void setCurrent(AbregeFiche abregeFiche) {
        setCurrent(abregeFiche, null);
    }

    public void setCurrent(AbregeFiche abregeFiche, Fiche fiche) {
        fieldBufferMap.clear();
        this.fiche = fiche;
        this.abregeFiche = abregeFiche;
        initAbregeFicheSlave();
        initCroisementAccess();
    }

    public AbregeFiche setCurrentByTicket(int ticket) {
        AbregeFiche nvAbregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
        setCurrent(nvAbregeFiche);
        return nvAbregeFiche;
    }

    public AbregeFiche setCurrentById(int id) {
        AbregeFiche nvAbregeFiche = corpus.getAbregeFicheByIdcorpus(id);
        setCurrent(nvAbregeFiche);
        return nvAbregeFiche;
    }

    public boolean isFicheComplete() {
        return ficheComplete;
    }

    public void enableCache(boolean enable) {
        if (cacheEnable == enable) return;
        this.cacheEnable = enable;
        if (enable) {
            cacheFicheMap = new HashMap<Integer, Fiche>();
        } else {
            cacheFicheMap.clear();
            cacheFicheMap = null;
        }
    }

    private void initCroisementAccess() {
        if (documentCroisementAccess != null) {
            documentCroisementAccess.setCurrent(abregeFiche);
        }
        for (AbstractCroisementAccess aca : accessMap.values()) {
            aca.setCurrent(abregeFiche);
        }
    }

    private void initAbregeFicheSlave() {
        if (!isSlaveInit) return;
        if (slaveFichePointeurMap == null) {
            return;
        }
        int idcorpus = -1;
        if (abregeFiche != null) idcorpus = abregeFiche.getIdcorpus();
        for (FichePointeur fichePointeur : slaveFichePointeurMap.values()) {
            if (abregeFiche == null) {
                fichePointeur.setCurrent(null);
            } else {
                AbregeFiche slaveAbregeFiche = fichePointeur.getCorpus().getAbregeFicheByIdcorpus(idcorpus);
                fichePointeur.setCurrent(slaveAbregeFiche);
            }
        }
    }

    public Subset getMasterSubset() {
        return masterSubset;
    }

    public Fiche getFiche() {
        return fiche;
    }

    public AbregeFiche getAbregeFiche() {
        return abregeFiche;
    }

    public Corpus getCorpus() {
        return corpus;
    }

    public CorpusField getCorpusField(FieldKey fieldKey) {
        return corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
    }

    public FichePointeur getSlaveFichePointeur(SubsetKey slaveCorpusKey) {
        testSlaveInit();
        if (slaveFichePointeurMap == null) {
            return null;
        }
        return slaveFichePointeurMap.get(slaveCorpusKey);
    }

    public boolean isRattachementCorpus(Corpus corpus) {
        testSlaveInit();
        if (rattachementCorpusArray == null) {
            return false;
        }
        int size = rattachementCorpusArray.length;
        for (int i = 0; i < size; i++) {
            if (rattachementCorpusArray[i].equals(corpus)) {
                return true;
            }
        }
        return false;
    }

    public int getRattachementCorpusCount() {
        testSlaveInit();
        if (rattachementCorpusArray == null) {
            return 0;
        }
        return rattachementCorpusArray.length;
    }

    public Corpus getRattachementCorpus(int i) {
        testSlaveInit();
        return rattachementCorpusArray[i];
    }

    private void testSlaveInit() {
        if(isSlaveInit) return;
        isSlaveInit = true;
        Subset[] slaveCorpusArray = corpus.getSlaveSubsetArray();
        if (masterSubset != null) {
            if (masterSubset instanceof Corpus) {
                rattachementCorpusArray = new Corpus[1];
                rattachementCorpusArray[0] = (Corpus) masterSubset;
            }
        } else if (slaveCorpusArray != null) {
            if (withSlaveCorpusPointeur) {
                slaveFichePointeurMap = new HashMap<SubsetKey, FichePointeur>();
            }
            int length = slaveCorpusArray.length;
            rattachementCorpusArray = new Corpus[length];
            for (int i = 0; i < length; i++) {
                Corpus slaveCorpus = (Corpus) slaveCorpusArray[i];
                rattachementCorpusArray[i] = slaveCorpus;
                if (withSlaveCorpusPointeur) {
                    slaveFichePointeurMap.put(slaveCorpus.getSubsetKey(), new FichePointeur(slaveCorpus, ficheComplete));
                }
            }
        }
        initAbregeFicheSlave();
    }

    public SubsetItem getMasterSubsetItem() {
        if (abregeFiche == null) {
            return null;
        }
        if (masterSubset == null) {
            return null;
        }
        return masterSubset.getSubsetItemById(abregeFiche.getIdcorpus());
    }

    public Datation getChronoToDatationDate(IncludeKey includeKey) {
        if (abregeFiche == null) {
            return null;
        }
        int dateInteger = 0;
        if (includeKey.equals(IncludeKey.DATE_CREATION)) {
            dateInteger = abregeFiche.getCreationDate();
        } else if (includeKey.equals(IncludeKey.DATE_MODIFICATION)) {
            dateInteger = abregeFiche.getModificationDate();
        } else {
            throw new IllegalArgumentException("wrong includeKey : " + includeKey.toString());
        }
        if (dateInteger == 0) {
            return null;
        }
        return new Datation(dateInteger);
    }

    /**
     * Valeurs possibles : String, FicheItem, FicheItemList (avec au moins un élément), FicheBlockList (avec au moins un élément), Integer (si idcorpus).
     * Si LANG, c'est un ficheItem de type Langue
     *
     */
    public Object getValue(FieldKey fieldKey) {
        if (abregeFiche == null) {
            return null;
        }
        Object obj = fieldBufferMap.get(fieldKey);
        if (obj != null) {
            if (obj.equals(Boolean.FALSE)) {
                return null;
            } else {
                return obj;
            }
        }
        initFiche();
        obj = getValue(abregeFiche, fiche, fieldKey);
        if (obj == null) {
            fieldBufferMap.put(fieldKey, Boolean.FALSE);
        } else {
            fieldBufferMap.put(fieldKey, obj);
        }
        return obj;
    }

    private void initFiche() {
        if (fiche != null) return;
        if (cacheEnable) {
            Fiche cacheFiche = cacheFicheMap.get(abregeFiche.getIdcorpus());
            if (cacheFiche != null) {
                fiche = cacheFiche;
                return;
            }
        }
        if (ficheComplete) {
            fiche = corpus.getFiche(abregeFiche);
        } else {
            fiche = corpus.getEnteteFiche(abregeFiche);
        }
        if (cacheEnable) {
            cacheFicheMap.put(abregeFiche.getIdcorpus(), fiche);
        }
    }

    public Document[] getDocumentArray() {
        if (documentCroisementAccess == null) {
            documentCroisementAccess = new DefaultDocumentCroisementAccess(corpus, (DocumentSubset) corpus.getBasedefiches().getSubset(SubsetKey.DOCUMENT_SUBSET));
            documentCroisementAccess.setCurrent(abregeFiche);
        }
        return documentCroisementAccess.getDocumentArray();
    }

    public ThesaurusCroisementAccess getThesaurusCroisementAccess(Thesaurus thesaurus) {
        DefaultThesaurusCroisementAccess tca = (DefaultThesaurusCroisementAccess) accessMap.get(thesaurus.getSubsetKey());
        if (tca == null) {
            tca = new DefaultThesaurusCroisementAccess(corpus, thesaurus);
            accessMap.put(thesaurus.getSubsetKey(), tca);
            tca.setCurrent(abregeFiche);
        }
        return tca;
    }

    public CorpusCroisementAccess getCorpusCroisementAccess(Corpus otherCorpus) {
        DefaultCorpusCroisementAccess cca = (DefaultCorpusCroisementAccess) accessMap.get(otherCorpus.getSubsetKey());
        if (cca == null) {
            cca = new DefaultCorpusCroisementAccess(corpus, otherCorpus);
            accessMap.put(otherCorpus.getSubsetKey(), cca);
            cca.setCurrent(abregeFiche);
        }
        return cca;
    }

    public static Object getValue(AbregeFiche abregeFiche, FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            return new Integer(abregeFiche.getIdcorpus());
        }
        if (fieldKey.equals(FieldKey.TITRE)) {
            return abregeFiche.getTitre();
        }
        Fiche fiche = null;
        if (fieldKey.ownsToCorpsdefiche()) {
            fiche = abregeFiche.getCorpus().getFiche(abregeFiche);
        } else {
            fiche = abregeFiche.getCorpus().getEnteteFiche(abregeFiche);
        }
        return getValue(fiche, fieldKey);
    }

    public static Object getValue(AbregeFiche abregeFiche, Fiche fiche, FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            return new Integer(abregeFiche.getIdcorpus());
        }
        if (fieldKey.equals(FieldKey.TITRE)) {
            return abregeFiche.getTitre();
        }
        return getValue(fiche, fieldKey);
    }

    private static Object getValue(Fiche fiche, FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            return fiche.getSoustitre();
        } else if (fieldKey.equals(FieldKey.TEXTE)) {
            return fiche.getTexte();
        } else if (fieldKey.equals(FieldKey.LANG)) {
            int langInteger = fiche.getLangInteger();
            if (langInteger == LangInteger.UNDETERMINED) {
                return null;
            } else {
                return new Langue(langInteger);
            }
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return fiche.getRedacteurs();
        } else if (fieldKey.isProprieteFieldKey()) {
            Propriete prop = fiche.getPropriete(fieldKey);
            if (prop == null) {
                return null;
            }
            return prop.getFicheItem();
        } else if (fieldKey.isInformationFieldKey()) {
            Information information = fiche.getInformation(fieldKey);
            if (information == null) {
                return null;
            }
            if (information.getFicheItemCount() == 0) {
                return null;
            }
            return information;
        } else if (fieldKey.isAnnexeFieldKey()) {
            Annexe annexe = fiche.getAnnexe(fieldKey);
            if (annexe == null) {
                return null;
            }
            if (annexe.getFicheBlockCount() == 0) {
                return null;
            }
            return annexe;
        } else {
            throw new SwitchException("unknown fieldKey = " + fieldKey.toString());
        }
    }

}
