/* BasedefichesLib_Explore
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.pointeurs;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.include.*;


/**
 *
 * @author Vincent Calame
 */
public class FichePointeurUtils {

    private FichePointeurUtils() {
    }

    public static FicheCroisementList[] getNoList(FichePointeur fichePointeur, IncludeSummary includeSummary) {
        List<FicheCroisementList> listOfLists = new ArrayList<FicheCroisementList>();
        Basedefiches basedefiches = fichePointeur.getCorpus().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus otherCorpus = basedefiches.getCorpus(i);
            if (fichePointeur.isRattachementCorpus(otherCorpus)) {
                continue;
            }
            IncludeKey includeKey = IncludeKey.createLiageInstance(otherCorpus.getSubsetKey());
            CorpusCroisementAccess acces = fichePointeur.getCorpusCroisementAccess(otherCorpus);
            FicheCroisementList list = null;
            if (includeSummary == null) {
                list = acces.getFicheCroisementList();
            } else {
                IncludeSummary.IncludeKeyInfo info = includeSummary.getIncludeKeyInfo(includeKey);
                if (info == null) {
                    list = acces.getFicheCroisementList();
                } else if (!info.hasAnyPoidsInclude()) {
                    RangeList rangeList = info.getPoidsFilterRangeList();
                    list = acces.getFicheCroisement(rangeList);
                }
            }
            if ((list != null) && (list.getSubsetItemCount() > 0)) {
                listOfLists.add(list);
            }
        }
        int size = listOfLists.size();
        FicheCroisementList[] result = new FicheCroisementList[size];
        for (int i = 0; i < size; i++) {
            result[i] = listOfLists.get(i);
        }
        return result;
    }

    public static FicheCroisementList[] getRattachementList(FichePointeur fichePointeur) {
        int size = fichePointeur.getRattachementCorpusCount();
        FicheCroisementList[] result = new FicheCroisementList[size];
        if (size == 0) {
            return result;
        }
        int p = 0;
        for (int i = 0; i < size; i++) {
            Corpus otherCorpus = fichePointeur.getRattachementCorpus(i);
            CorpusCroisementAccess acces = fichePointeur.getCorpusCroisementAccess(otherCorpus);
            FicheCroisementList list = acces.getFicheCroisementList();
            if (list.getSubsetItemCount() > 0) {
                result[p] = list;
                p++;
            }
        }
        if (p < size) {
            FicheCroisementList[] newResult = new FicheCroisementList[p];
            System.arraycopy(result, 0, newResult, 0, p);
            return newResult;
        } else {
            return result;
        }
    }

}
