/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.pointeurs;

import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class MotclePointeur implements SubsetItemPointeur {

    private Thesaurus thesaurus;
    private Motcle motcle;
    private Map<SubsetKey, FichePointeur> slaveFichePointeurMap;
    private Corpus[] rattachementCorpusArray;
    private HashMap<SubsetKey, AbstractCroisementAccess> accessMap = new HashMap<SubsetKey, AbstractCroisementAccess>();

    public MotclePointeur(Motcle motcle) {
        this(motcle.getThesaurus());
        setCurrent(motcle);
    }

    public MotclePointeur(Thesaurus thesaurus) {
        this.thesaurus = thesaurus;
        Subset[] slaveCorpusArray = thesaurus.getSlaveSubsetArray();
        if (slaveCorpusArray != null) {
            slaveFichePointeurMap = new HashMap<SubsetKey, FichePointeur>();
            int length = slaveCorpusArray.length;
            rattachementCorpusArray = new Corpus[length];
            for (int i = 0; i < length; i++) {
                Corpus slaveCorpus = (Corpus) slaveCorpusArray[i];
                rattachementCorpusArray[i] = slaveCorpus;
                slaveFichePointeurMap.put(slaveCorpus.getSubsetKey(), new FichePointeur(slaveCorpus, false));
            }
        }
    }

    public Thesaurus getThesaurus() {
        return thesaurus;
    }

    public Motcle getMotcle() {
        return motcle;
    }

    public void setCurrentSubsetItem(SubsetItem subsetItem) {
        setCurrent((Motcle) subsetItem);
    }

    public void setCurrent(Motcle motcle) {
        this.motcle = motcle;
        initSlave(motcle);
        initCroisementAccess(motcle);
    }

    public Object getFieldValue(ThesaurusFieldKey thesaurusFieldKey) {
        if (motcle == null) {
            return null;
        }
        short category = thesaurusFieldKey.getCategory();
        if (category == ThesaurusFieldKey.SPECIAL_CATEGORY) {
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.IDTHS)) {
                return new Integer(motcle.getIdths());
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.IDALPHA)) {
                return motcle.getIdalpha();
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.PARENT_IDTHS)) {
                Motcle parent = motcle.getParent();
                if (parent == null) {
                    return null;
                }
                return new Integer(parent.getIdths());
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.PARENT_IDALPHA)) {
                Motcle parent = motcle.getParent();
                if (parent == null) {
                    return null;
                }
                return parent.getIdalpha();
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.POSITION_LOCAL)) {
                return new Integer(motcle.getChildIndex() + 1);
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.POSITION_GLOBAL)) {
                return ThesaurusUtils.getGlobalPosition(motcle);
            }
            throw new IllegalArgumentException("Unkwnon special thesaurusFieldKey : " + thesaurusFieldKey.toString());
        }
        if (category == ThesaurusFieldKey.LIB_CATEGORY) {
            return motcle.getLibelleString(thesaurusFieldKey.getLangInteger(), false);
        }
        throw new IllegalArgumentException("Unkwnon thesaurusFieldKey : " + thesaurusFieldKey.toString());
    }

    private void initCroisementAccess(Motcle motcle) {
        for (AbstractCroisementAccess aca : accessMap.values()) {
            aca.setCurrent(motcle);
        }
    }

    private void initSlave(Motcle motcle) {
        if (slaveFichePointeurMap == null) {
            return;
        }
        for (FichePointeur fichePointeur : slaveFichePointeurMap.values()) {
            if (motcle == null) {
                fichePointeur.setCurrent(null);
            } else {
                AbregeFiche slaveAbregeFiche = fichePointeur.getCorpus().getAbregeFicheByIdcorpus(motcle.getIdths());
                fichePointeur.setCurrent(slaveAbregeFiche);
            }
        }
    }

    public FichePointeur getSlaveFichePointeur(SubsetKey slaveCorpusKey) {
        if (slaveFichePointeurMap == null) {
            return null;
        }
        return slaveFichePointeurMap.get(slaveCorpusKey);
    }

    public CorpusCroisementAccess getCorpusCroisementAccess(Corpus otherCorpus) {
        DefaultCorpusCroisementAccess cca = (DefaultCorpusCroisementAccess) accessMap.get(otherCorpus.getSubsetKey());
        if (cca == null) {
            cca = new DefaultCorpusCroisementAccess(thesaurus, otherCorpus);
            accessMap.put(otherCorpus.getSubsetKey(), cca);
        }
        cca.setCurrent(motcle);
        return cca;
    }

}
