/* BasedefichesLib_Explore
 *
 * Copyright (C) 2007-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.selection.util.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheByMotcleSelector {

    Basedefiches basedefiches;
    int langInteger;
    short optionsMode = Conditions.MODE_OR;
    OptionBundle[] bundleArray = new OptionBundle[4];
    boolean impossible = false;
    int size = 0;

    public FicheByMotcleSelector(Basedefiches basedefiches, int langInteger, short mode) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
        this.optionsMode = mode;
    }

    public void addOptions(MotcleSelectOptions motcleSelectOptions) {
        if (impossible) {
            return;
        }
        if (motcleSelectOptions.isEmpty()) {
            return;
        }
        if (motcleSelectOptions.isImpossible()) {
            if (optionsMode == Conditions.MODE_AND) {
                impossible = true;
            }
            return;
        }
        addBundle(new OptionBundle(motcleSelectOptions));
    }

    private void addBundle(OptionBundle bundle) {
        if (size == bundleArray.length) {
            OptionBundle[] temp = new OptionBundle[size * 2];
            System.arraycopy(bundleArray, 0, temp, 0, size);
            bundleArray = temp;
        }
        bundleArray[size] = bundle;
        size++;
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        if (impossible) {
            return false;
        }
        if (size == 0) {
            return true;
        }
        if (optionsMode == Conditions.MODE_OR) {
            for (int i = 0; i < size; i++) {
                if (bundleArray[i].isSelected(abregeFiche)) {
                    return true;
                }
            }
            return false;
        } else {
            for (int i = 0; i < size; i++) {
                if (!bundleArray[i].isSelected(abregeFiche)) {
                    return false;
                }
            }
            return true;
        }

    }


    class OptionBundle {

        short contentMode;
        boolean withContent = false;
        MotcleSelector motcleSelector;
        int completelyAcceptedMask = 0;
        int completelyRejectedMask = 0;
        ConditionGroup contentConditionGroup;
        short indexationType;
        HashMap croisementsMap = new HashMap();

        OptionBundle(MotcleSelectOptions motcleSelectOptions) {
            motcleSelector = new MotcleSelector(basedefiches, langInteger);
            motcleSelector.addMotcleSelectOptions(motcleSelectOptions);
            contentConditionGroup = motcleSelectOptions.getContentConditionGroup();
            if (contentConditionGroup != null) {
                ContentConditionBean[] includingArray = contentConditionGroup.getIncludingContentConditionBeanArray();
                if ((includingArray != null) && (includingArray.length > 0)) {
                    completelyAcceptedMask = ((int) Math.pow(2, includingArray.length) - 1);
                }
                ContentConditionBean[] excludingArray = contentConditionGroup.getExcludingContentConditionBeanArray();
                if (excludingArray.length > 0) {
                    completelyRejectedMask = ((int) Math.pow(2, excludingArray.length) - 1);
                }
                contentMode = contentConditionGroup.getMode();
                withContent = true;
            }
            indexationType = motcleSelectOptions.getIndexationType();
        }

        boolean isSelected(AbregeFiche abregeFiche) {
            if (withContent) {
                return isContentSelected(abregeFiche);
            }
            switch (indexationType) {
                case MotcleSelectOptions.NOINDEXATION_ONLY:
                    return !isIndexationSelected(abregeFiche);
                case MotcleSelectOptions.INDEXATION_ONLY:
                    return isIndexationSelected(abregeFiche);
                case MotcleSelectOptions.NOINDEXATION_ALLOWED:
                    throw new SwitchException("should not occur : NOINDEXATION_ALLOWED associated to contentConditionGroup");
                case MotcleSelectOptions.IMPOSSIBLE:
                    throw new SwitchException("should not occur : test before");
                default:
                    throw new SwitchException("should not occur : wrong indexation type = " + indexationType);
            }
        }

        boolean isIndexationSelected(AbregeFiche abregeFiche) {
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            Croisement[] croisements = getCroisementArray(corpusKey);
            for (int i = 0; i < croisements.length; i++) {
                Croisement croisement = croisements[i];
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, abregeFiche.getTicket());
                int count = croisementItem.getCount();
                Thesaurus thesaurus = motcleSelector.getThesaurus(i);
                for (int j = 0; j < count; j++) {
                    int ticket = croisementItem.getTicket(j);
                    Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
                    if (motcleSelector.isSelected(motcle)) {
                        return true;
                    }
                }
            }
            return false;
        }

        boolean isContentSelected(AbregeFiche abregeFiche) {
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            Croisement[] croisements = getCroisementArray(corpusKey);
            boolean withEmpty = (indexationType == MotcleSelectOptions.NOINDEXATION_ALLOWED);
            boolean withAcceptClause = (completelyAcceptedMask != 0);
            boolean withRejectClause = (completelyRejectedMask != 0);
            int ficheTicket = abregeFiche.getTicket();
            int acceptMask = 0;
            int rejectMask = 0;
            boolean isEmpty = true;
            for (int i = 0; i < croisements.length; i++) {
                Thesaurus thesaurus = motcleSelector.getThesaurus(i);
                Croisement croisement = croisements[i];
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ficheTicket);
                int count = croisementItem.getCount();
                if ((isEmpty) && (count > 0)) {
                    isEmpty = false;
                }
                boolean breakIteration = false;
                for (int j = 0; j < count; j++) {
                    int ticket = croisementItem.getTicket(j);
                    Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
                    int mask = motcleSelector.getPMask(motcle);
                    if (mask == 0) { // rien de précis sur le mot-clé
                        continue;
                    } else if (mask < 0) { // le mot-clé doit être rejeté
                        if (contentMode == Conditions.MODE_AND) {
                            rejectMask = -1; //Inutile d'aller plus loin
                            breakIteration = true;
                            break;
                        } else {
                            rejectMask = rejectMask | mask;
                            if (rejectMask == completelyRejectedMask) {
                                if (!withAcceptClause) {
                                    breakIteration = true;
                                    break;
                                } /*else if (!selection.containsAcceptedMotcle()) {
                                break;
                                }*/
                            }
                        }
                    } else if (mask > 0) { // le mot-clé doit être accepté
                        if (contentMode == Conditions.MODE_OR) {
                            acceptMask = 1; //une valeur positive suffit
                            breakIteration = true;
                            break;
                        } else {
                            acceptMask = acceptMask | mask;
                            if (acceptMask == completelyAcceptedMask) {
                                if (!withRejectClause) {
                                    breakIteration = true;
                                    break;
                                }/* else if (!selection.containsRejectedMotcle()) {
                                break;
                                }*/
                            }
                        }
                    }
                }
                if (breakIteration) {
                    break;
                }
            }
            if (isEmpty) { // isEmpty == true si aucun mot-clé n'indexe la fiche
                return withEmpty;
            } else {
                if (contentMode == Conditions.MODE_OR) {
                    if ((acceptMask > 0) || (rejectMask != completelyRejectedMask)) {
                        return true;
                    }
                } else if (contentMode == Conditions.MODE_AND) {
                    {
                        if ((rejectMask == 0) && (acceptMask == completelyAcceptedMask)) {
                            return true;
                        }
                    }
                }

            }
            return false;
        }

        private Croisement[] getCroisementArray(SubsetKey corpusKey) {
            Croisement[] croisements = (Croisement[]) croisementsMap.get(corpusKey);
            if (croisements == null) {
                croisements = new Croisement[motcleSelector.getThesaurusCount()];
                for (int i = 0; i < croisements.length; i++) {
                    Thesaurus thesaurus = motcleSelector.getThesaurus(i);
                    croisements[i] = basedefiches.getCroisement(corpusKey, thesaurus.getSubsetKey());
                }
                croisementsMap.put(corpusKey, croisements);
            }
            return croisements;
        }

    }

}
