/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.util.primitives.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectOptions {

    private RangeList rangeList;
    private MotcleSelectOptions[] motcleSelectOptionsArray;
    private ConditionGroup titreConditionGroup;
    private RedacteurKey[] redacteurKeyArray;
    private SubsetKey[] sphereKeyArray;
    private SubsetKey[] corpusKeyArray;
    private short motcleSelectMode = ConditionsConstants.MODE_AND;

    public FicheSelectOptions() {
    }

    public FicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.rangeList = ficheSelectOptions.rangeList;
        this.motcleSelectOptionsArray = ficheSelectOptions.motcleSelectOptionsArray;
        this.titreConditionGroup = ficheSelectOptions.titreConditionGroup;
        this.redacteurKeyArray = ficheSelectOptions.redacteurKeyArray;
        this.sphereKeyArray = ficheSelectOptions.sphereKeyArray;
        this.corpusKeyArray = ficheSelectOptions.corpusKeyArray;
        this.motcleSelectMode = ficheSelectOptions.motcleSelectMode;
    }

    public RangeList getRangeList() {
        return rangeList;
    }

    /**
     *
     *@throws IllegalArgumentException si rangeList n'est pas positif
     */
    public void setRangeList(RangeList rangeList) {
        if ((rangeList != null) && (!RangeUtils.isPositiveRangeList(rangeList))) {
            throw new IllegalArgumentException("rangeList is not positive");
        }
        this.rangeList = rangeList;
    }

    public MotcleSelectOptions[] getMotcleSelectOptionsArray() {
        return motcleSelectOptionsArray;
    }

    /**
     * Si motcleSelectOptions est vide, cela revient à donner la valeur nulle.
     */
    public void setMotcleSelectOptionsArray(MotcleSelectOptions[] motcleSelectOptionsArray) {
        if ((motcleSelectOptionsArray == null) || (motcleSelectOptionsArray.length == 0)) {
            this.motcleSelectOptionsArray = null;
        } else {
            this.motcleSelectOptionsArray = motcleSelectOptionsArray;
        }
    }

    public void setMotcleSelectMode(short motcleSelectMode) {
        this.motcleSelectMode = motcleSelectMode;
    }

    public short getMotcleSelectMode() {
        return motcleSelectMode;
    }

    public ConditionGroup getTitreConditionGroup() {
        return titreConditionGroup;
    }

    public void setTitreConditionGroup(ConditionGroup titreConditionGroup) {
        this.titreConditionGroup = titreConditionGroup;
    }

    public RedacteurKey[] getRedacteurKeyArray() {
        return redacteurKeyArray;
    }

    public void setRedacteurKeyArray(RedacteurKey[] redacteurKeyArray) {
        this.redacteurKeyArray = redacteurKeyArray;
    }

    public SubsetKey[] getSphereKeyArray() {
        return sphereKeyArray;
    }

    public void setSphereKeyArray(SubsetKey[] sphereKeyArray) {
        this.sphereKeyArray = sphereKeyArray;
    }

    public SubsetKey[] getCorpusKeyArray() {
        return corpusKeyArray;
    }

    public void setCorpusKeyArray(SubsetKey[] corpusKeyArray) {
        this.corpusKeyArray = corpusKeyArray;
    }

    /**
     * Retourne <em>true</em> s'il y a au moins une option.
     */
    public boolean containsOption() {
        if (rangeList != null) {
            return true;
        }
        if (motcleSelectOptionsArray != null) {
            return true;
        }
        if (titreConditionGroup != null) {
            return true;
        }
        if (redacteurKeyArray != null) {
            return true;
        }
        if (getSphereKeyArray() != null) {
            return true;
        }
        if (corpusKeyArray != null) {
            return true;
        }
        return false;
    }

}
