/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.selection.util.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectUnit {

    private SubsetKey corpusKey;
    private Basedefiches basedefiches;
    private FicheByMotcleSelector ficheyMotcleSelector;
    private RangeList rangeList;
    private ConditionGroup titreConditionGroup;
    private StringPASelector titrePASelector;
    private RedacteurKey[] redacteurKeyArray;
    private SubsetKey[] sphereKeyArray;

    public FicheSelectUnit(Basedefiches basedefiches, SubsetKey corpusKey) {
        this.basedefiches = basedefiches;
        this.corpusKey = corpusKey;
    }

    public void setFicheByMotcleSelector(FicheByMotcleSelector ficheyMotcleSelector) {
        this.ficheyMotcleSelector = ficheyMotcleSelector;
    }

    public void setTitreConditionGroup(ConditionGroup titreConditionGroup, int langInteger) {
        this.titreConditionGroup = titreConditionGroup;
        titrePASelector = StringPASelector.newInstance(titreConditionGroup, langInteger);
    }

    public void setRedacteurKeyArray(RedacteurKey[] redacteurKeyArray) {
        this.redacteurKeyArray = redacteurKeyArray;
    }

    public void setSphereKeyArray(SubsetKey[] sphereKeyArray) {
        this.sphereKeyArray = sphereKeyArray;
    }

    public void setRangeList(RangeList rangeList) {
        this.rangeList = rangeList;
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        if (rangeList != null) {
            if (!rangeList.contains(abregeFiche.getIdcorpus())) {
                return false;
            }
        }
        if ((redacteurKeyArray != null) || (sphereKeyArray != null)) {
            if (!isRedacteurSelected(abregeFiche)) {
                return false;
            }
        }
        if (titreConditionGroup != null) {
            if (!isTitreSelected(abregeFiche)) {
                return false;
            }
        }
        if (ficheyMotcleSelector != null) {
            if (!ficheyMotcleSelector.isSelected(abregeFiche)) {
                return false;
            }
        }
        return true;
    }

    public boolean isRedacteurSelected(AbregeFiche abregeFiche) {
        if (sphereKeyArray != null) {
            int length = sphereKeyArray.length;
            for (int i = 0; i < length; i++) {
                SubsetKey subsetKey = sphereKeyArray[i];
                if (abregeFiche.hasRedacteurInSphere(subsetKey)) {
                    return true;
                }
            }
        }
        if (redacteurKeyArray != null) {
            int length = redacteurKeyArray.length;
            for (int i = 0; i < length; i++) {
                RedacteurKey redacteurKey = redacteurKeyArray[i];
                if (abregeFiche.hasRedacteur(redacteurKey)) {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean isTitreSelected(AbregeFiche abregeFiche) {
        short state = titreConditionGroup.getState();
        switch (state) {
            case Conditions.X:
                return false;
            case Conditions.N:
            case Conditions.U:
                return true;
            case Conditions.T:
                return hasTitre(abregeFiche);
            case Conditions.A:
                return !hasTitre(abregeFiche);
            case Conditions.P:
            case Conditions.PA:
                return titrePASelector.isSelected(abregeFiche.getTitre(), (state == Conditions.PA));
            default:
                throw new SwitchException("state = " + state);
        }
    }

    private boolean hasTitre(AbregeFiche abregeFiche) {
        return (abregeFiche.getTitre().length() > 0);
    }

}
