/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import java.util.*;
import net.mapeadores.basedefiches.sphere.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelector {

    private FicheEligibility ficheEligibility;
    private Basedefiches basedefiches;
    private HashMap corpusSelectBundleMap = new HashMap();
    private ArrayList corpusSelectBundleList = new ArrayList();
    private int langInteger;

    public FicheSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    /**
     * Précise le filtre d'éligibilité.
     */
    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        this.ficheEligibility = ficheEligibility;
    }

    public void addFicheSelectOptionsList(FicheSelectOptionsList ficheSelectOptionsList) {
        int count = ficheSelectOptionsList.getFicheSelectOptionsCount();
        if (count == 0) {
            add(new FicheSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(ficheSelectOptionsList.getFicheSelectOptions(i));
            }
        }
    }

    public void addFicheSelectOptions(FicheSelectOptions selectOptions) {
        add(selectOptions);
    }

    private void add(FicheSelectOptions selectOptions) {
        FicheByMotcleSelector ficheByMotcleSelector = null;
        MotcleSelectOptions[] motcleSelectOptionsArray = selectOptions.getMotcleSelectOptionsArray();
        if (motcleSelectOptionsArray != null) {
            ficheByMotcleSelector = new FicheByMotcleSelector(basedefiches, langInteger, selectOptions.getMotcleSelectMode());
            for (int i = 0; i < motcleSelectOptionsArray.length; i++) {
                ficheByMotcleSelector.addOptions(motcleSelectOptionsArray[i]);
            }
        }
        RedacteurKey[] redacteurKeyArray = selectOptions.getRedacteurKeyArray();
        SubsetKey[] subsetKeyArray = selectOptions.getSphereKeyArray();
        SubsetKey[] corpusKeyArray = selectOptions.getCorpusKeyArray();
        if (corpusKeyArray != null) {
            for (int i = 0; i < corpusKeyArray.length; i++) {
                CorpusSelectBundle corpusSelectBundle = getCorpusSelectBundle(corpusKeyArray[i]);
                if (corpusSelectBundle != null) {
                    corpusSelectBundle.add(ficheByMotcleSelector, selectOptions, redacteurKeyArray, subsetKeyArray);
                }
            }
        } else {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                CorpusSelectBundle corpusSelectBundle = getCorpusSelectBundle(corpus);
                corpusSelectBundle.add(ficheByMotcleSelector, selectOptions, redacteurKeyArray, subsetKeyArray);
            }
        }
    }

    public boolean containsCorpus(SubsetKey corpusKey) {
        return corpusSelectBundleMap.containsKey(corpusKey);
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        if (ficheEligibility != null) {
            if (!(ficheEligibility.accept(abregeFiche))) {
                return false;
            }
        }
        CorpusSelectBundle corpusSelectBundle = (CorpusSelectBundle) corpusSelectBundleMap.get(abregeFiche.getCorpus().getSubsetKey());
        if (corpusSelectBundle == null) {
            return false;
        }
        return corpusSelectBundle.isSelected(abregeFiche);
    }

    public int getCorpusCount() {
        return corpusSelectBundleList.size();
    }

    public Corpus getCorpus(int index) {
        CorpusSelectBundle corpusSelectBundle = (CorpusSelectBundle) corpusSelectBundleList.get(index);
        return corpusSelectBundle.corpus;
    }

    private CorpusSelectBundle getCorpusSelectBundle(SubsetKey corpusKey) {
        CorpusSelectBundle corpusSelectBundle = (CorpusSelectBundle) corpusSelectBundleMap.get(corpusKey);
        if (corpusSelectBundle == null) {
            Corpus corpus = (Corpus) basedefiches.getSubset(corpusKey);
            if (corpus == null) {
                return null;
            }
            return createCorpusSelectBundle(corpus);
        }
        return corpusSelectBundle;
    }

    private CorpusSelectBundle getCorpusSelectBundle(Corpus corpus) {
        SubsetKey corpusKey = corpus.getSubsetKey();
        CorpusSelectBundle corpusSelectBundle = (CorpusSelectBundle) corpusSelectBundleMap.get(corpusKey);
        if (corpusSelectBundle == null) {
            return createCorpusSelectBundle(corpus);
        }
        return corpusSelectBundle;
    }

    private CorpusSelectBundle createCorpusSelectBundle(Corpus corpus) {
        CorpusSelectBundle corpusSelectBundle = new CorpusSelectBundle(corpus);
        corpusSelectBundleMap.put(corpus.getSubsetKey(), corpusSelectBundle);
        corpusSelectBundleList.add(corpusSelectBundle);
        return corpusSelectBundle;
    }


    private class CorpusSelectBundle {

        private Corpus corpus;
        private FicheSelectUnit[] ficheSelectUnitArray = new FicheSelectUnit[8];
        private int size = 0;

        CorpusSelectBundle(Corpus corpus) {
            this.corpus = corpus;
        }

        void add(FicheByMotcleSelector ficheByMotcleSelector, FicheSelectOptions ficheSelectOptions, RedacteurKey[] redacteurKeyArray, SubsetKey[] sphereKeyArray) {
            if (size == ficheSelectUnitArray.length) {
                FicheSelectUnit[] nvFSU = new FicheSelectUnit[size * 2];
                System.arraycopy(ficheSelectUnitArray, 0, nvFSU, 0, size);
                ficheSelectUnitArray = nvFSU;
            }
            FicheSelectUnit ficheSelectUnit = new FicheSelectUnit(basedefiches, corpus.getSubsetKey());
            ficheSelectUnit.setFicheByMotcleSelector(ficheByMotcleSelector);
            ficheSelectUnit.setTitreConditionGroup(ficheSelectOptions.getTitreConditionGroup(), langInteger);
            ficheSelectUnit.setRangeList(ficheSelectOptions.getRangeList());
            ficheSelectUnit.setRedacteurKeyArray(redacteurKeyArray);
            ficheSelectUnit.setSphereKeyArray(sphereKeyArray);
            ficheSelectUnitArray[size] = ficheSelectUnit;
            size++;
        }

        boolean isSelected(AbregeFiche abregeFiche) {
            for (int i = 0; i < size; i++) {
                if (ficheSelectUnitArray[i].isSelected(abregeFiche)) {
                    return true;
                }
            }
            return false;
        }

    }

}
