/* BasedefichesLib_Explore - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.text.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSelectUnit implements ConditionsConstants {

    Thesaurus thesaurus;
    RangeList idthsRangeList;
    RangeList levelRangeList;
    boolean idalphaOnly;
    boolean impossible = false;
    int langInteger;
    ContentSelection contentSelection;

    public MotcleSelectUnit(Thesaurus thesaurus, int langInteger) {
        this.thesaurus = thesaurus;
        this.langInteger = langInteger;
    }

    public void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        impossible = motcleSelectOptions.isImpossible();
        if (!impossible) {
            this.levelRangeList = motcleSelectOptions.getLevelRangeList();
            this.idthsRangeList = motcleSelectOptions.getIdthsRangeList();
            this.idalphaOnly = motcleSelectOptions.isIdalphaOnly();
            ConditionGroup cg = motcleSelectOptions.getContentConditionGroup();
            if (cg != null) {
                contentSelection = new ContentSelection(cg);
            } else {
                contentSelection = null;
            }
        }
    }

    /**
     * Retourne 0 si le mot-clé n'est pas sélectionné, un nombre négatif s'il est rejeté, un nombre positif s'il est accepté.
     */
    public int isSelected(Motcle motcle) {
        if (impossible) {
            return 0;
        }
        if (levelRangeList != null) {
            if (!levelRangeList.contains(motcle.getLevel())) {
                return 0;
            }
        }
        if (idthsRangeList != null) {
            if (!idthsRangeList.contains(motcle.getIdths())) {
                return 0;
            }
        }
        if (contentSelection != null) {
            return contentSelection.getMask(motcle);
        }
        return 1;
    }

    /**
     * Retourne null s'il n'y a pas de condition partielle
     * Implémente RandomAccess
     */
    public List getAcceptedMotcleList() {
        if (contentSelection == null) {
            return null;
        }
        return contentSelection.acceptedMotcleList;
    }

    public List getRejectedMotcleList() {
        if (contentSelection == null) {
            return null;
        }
        return contentSelection.rejectedMotcleList;
    }


    private class ContentSelection {

        private boolean avecIdalpha;
        int completelyAcceptedMask = 0;
        int completelyRejectedMask = 0;
        private List acceptedMotcleList = new ArrayList();
        private List rejectedMotcleList = new ArrayList();
        private HashMap motcleMap = new HashMap();
        short normalMode, inverseMode;

        private ContentSelection(ConditionGroup conditionGroup) {
            avecIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
            normalMode = conditionGroup.getMode();
            inverseMode = Conditions.getModeInverse(normalMode);
            ContentConditionBean[] excludingArray = conditionGroup.getExcludingContentConditionBeanArray();
            if (excludingArray.length > 0) {
                completelyRejectedMask = ((int) Math.pow(2, excludingArray.length) - 1);

                for (int i = 0; i < excludingArray.length; i++) {
                    ContentConditionBean excludingOpposite = Conditions.toOpposite(excludingArray[i]);
                    Set exclusionSet = getSetByContent(excludingOpposite);
                    int mask = BitmaskUtils.getBitValue(i) | BitmaskUtils.getBitValue(31);
                    for (Iterator it = exclusionSet.iterator(); it.hasNext();) {
                        addMask((Motcle) it.next(), mask);
                    }
                }
            }
            ContentConditionBean[] includingArray = conditionGroup.getIncludingContentConditionBeanArray();
            if (includingArray.length > 0) {
                completelyAcceptedMask = ((int) Math.pow(2, includingArray.length) - 1);
                for (int i = 0; i < includingArray.length; i++) {
                    Set inclusionSet = getSetByContent(includingArray[i]);
                    int mask = BitmaskUtils.getBitValue(i);
                    for (Iterator it = inclusionSet.iterator(); it.hasNext();) {
                        addMask((Motcle) it.next(), mask);
                    }
                }
            }
        }

        private Set getSetByContent(ContentConditionBean contentConditionBean) {
            short typeCondition = contentConditionBean.getConditionType();
            String text = contentConditionBean.getText();
            if ((typeCondition == CORRESPOND_A) && (!avecIdalpha)) {
                try {
                    int idths = Integer.parseInt(text);
                    Motcle motcle = thesaurus.getMotcleByIdths(idths);
                    if (motcle != null) {
                        return Collections.singleton(motcle);
                    } else {
                        return Collections.EMPTY_SET;
                    }
                } catch (NumberFormatException nfe) {
                }
            }
            int textConstants = Conditions.toTextConstants(typeCondition);
            if (avecIdalpha) {
                Set<Motcle> candidatsSet = thesaurus.getMotcleSetByIdalpha(text, textConstants);
                if ((!idalphaOnly) && ((textConstants & TextConstants.STARTSWITH) != 0)) {
                    Set<Motcle> newset = thesaurus.getMotcleSetByLibelle(text, textConstants, langInteger);
                    if (candidatsSet.size() == 0) {
                        candidatsSet = newset;
                    } else {
                        candidatsSet.addAll(newset);
                    }
                }
                return candidatsSet;

            } else {
                return thesaurus.getMotcleSetByLibelle(text, textConstants, langInteger);
            }
        }

        void addMask(Motcle motcle, int mask) {
            MaskBundle maskBundle = (MaskBundle) motcleMap.get(motcle);
            if (maskBundle == null) {
                maskBundle = new MaskBundle(motcle, mask);
                motcleMap.put(motcle, maskBundle);
            } else {
                maskBundle.addMask(mask);
            }
        }

        /*
         *
         */
        int getMask(Motcle motcle) {
            MaskBundle maskBundle = (MaskBundle) motcleMap.get(motcle);
            if (maskBundle == null) {
                return 0;
            }
            return maskBundle.mask;
        }


        class MaskBundle {

            Motcle motcle;
            int mask;

            MaskBundle(Motcle motcle, int mask) {
                this.motcle = motcle;
                this.mask = mask;
                if (mask > 0) {
                    acceptedMotcleList.add(motcle);
                } else {
                    rejectedMotcleList.add(motcle);
                }
            }

            void addMask(int addedMask) {
                if ((mask > 0) && (addedMask < 0)) {
                    if (normalMode == MODE_AND) {
                        mask = addedMask;
                        acceptedMotcleList.remove(motcle);
                        rejectedMotcleList.add(motcle);
                    }
                } else if ((mask < 0) && (addedMask > 0)) {
                    if (normalMode == MODE_OR) {
                        mask = addedMask;
                        acceptedMotcleList.add(motcle);
                        rejectedMotcleList.remove(motcle);
                    }
                } else {
                    mask = mask | addedMask;
                }
            }

        }

    }

}
