/* BasedefichesLib_Explore
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.*;
import java.util.*;
import net.mapeadores.basedefiches.sphere.*;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurSelector {

    private Basedefiches basedefiches;
    private RedacteurEligibility redacteurEligibility;
    private int langInteger;
    private Map<SubsetKey, SphereSelectBundle> sphereSelectBundleMap = new HashMap<SubsetKey, SphereSelectBundle>();

    public RedacteurSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    public void setRedacteurEligibility(RedacteurEligibility redacteurEligibility) {
        this.redacteurEligibility = redacteurEligibility;
    }

    public void addRedacteurSelectOptions(RedacteurSelectOptions selectOptions) {
        int sphereCount = basedefiches.getSphereCount();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = basedefiches.getSphere(i);
            SphereSelectBundle sphereSelectBundle = getSphereSelectBundle(sphere);
            sphereSelectBundle.add(selectOptions);
        }
    }

    public boolean isSelected(Redacteur redacteur) {
        if (redacteurEligibility != null) {
            if (!(redacteurEligibility.accept(redacteur))) {
                return false;
            }
        }
        SphereSelectBundle sphereSelectBundle = sphereSelectBundleMap.get(redacteur.getSphere().getSubsetKey());
        if (sphereSelectBundle == null) {
            return false;
        }
        return sphereSelectBundle.isSelected(redacteur);
    }

    private SphereSelectBundle getSphereSelectBundle(Sphere sphere) {
        SubsetKey sphereKey = sphere.getSubsetKey();
        SphereSelectBundle sphereSelectBundle = sphereSelectBundleMap.get(sphereKey);
        if (sphereSelectBundle == null) {
            return createSphereSelectBundle(sphere);
        }
        return sphereSelectBundle;
    }

    private SphereSelectBundle createSphereSelectBundle(Sphere sphere) {
        SphereSelectBundle sphereSelectBundle = new SphereSelectBundle(sphere);
        sphereSelectBundleMap.put(sphere.getSubsetKey(), sphereSelectBundle);
        return sphereSelectBundle;
    }


    class SphereSelectBundle {

        Sphere sphere;
        RedacteurSelectUnit[] redacteurSelectUnitArray = new RedacteurSelectUnit[2];
        int size = 0;

        SphereSelectBundle(Sphere sphere) {
            this.sphere = sphere;
        }

        void add(RedacteurSelectOptions selectOptions) {
            if (size == redacteurSelectUnitArray.length) {
                RedacteurSelectUnit[] nvFSU = new RedacteurSelectUnit[size * 2];
                System.arraycopy(redacteurSelectUnitArray, 0, nvFSU, 0, size);
                redacteurSelectUnitArray = nvFSU;
            }
            RedacteurSelectUnit redacteurSelectUnit = new RedacteurSelectUnit(basedefiches);
            redacteurSelectUnit.setNomcompletConditionGroup(selectOptions.getNomcompletConditionGroup(), langInteger);
            redacteurSelectUnitArray[size] = redacteurSelectUnit;
            size++;
        }

        boolean isSelected(Redacteur redacteur) {
            for (int i = 0; i < size; i++) {
                if (redacteurSelectUnitArray[i].isSelected(redacteur)) {
                    return true;
                }
            }
            return false;
        }

    }

}
