/* @@@@@APPLICATION@@@@@ - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.AlbumList;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.selection.util.MutableAlbumList;
import net.mapeadores.basedefiches.selection.util.MutableCorpusList;
import net.mapeadores.basedefiches.selection.util.MutableThesaurusList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.langinteger.LangInteger;
import java.text.Collator;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class SelectionUtils {

    private SelectionUtils() {
    }

    public static AbregeFicheSortedList getAbregeFicheSortedList(FicheSelector ficheSelector, short sortType) {
        if (!AbregeFicheSortedList.isValideShortType(sortType)) {
            throw new IllegalArgumentException("shortType argument is not valid");
        }
        List<AbregeFiche> list = new ArrayList<AbregeFiche>();
        for (int i = 0; i < ficheSelector.getCorpusCount(); i++) {
            Corpus corpus = ficheSelector.getCorpus(i);
            for (int j = 0; j < corpus.getTicketLength(); j++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                if (abregeFiche != null) {
                    if (ficheSelector.isSelected(abregeFiche)) {
                        list.add(abregeFiche);
                    }
                }
            }
        }
        Collator collator = null;
        if (sortType == AbregeFicheSortedList.TITRE_SORTTYPE) {
            collator = Collator.getInstance(LangInteger.getLocale(ficheSelector.getLangInteger()));
            collator.setStrength(Collator.PRIMARY);
        }
        return new AbregeFicheSortedList(list, sortType, collator);
    }

    public static ThesaurusList getThesaurusList(Basedefiches basedefiches, MotcleSelectOptions motcleSelectOptions) {
        if (motcleSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] thesaurusKeyArray = motcleSelectOptions.getThesaurusKeyArray();
        if (thesaurusKeyArray == null) {
            return basedefiches;
        }
        MutableThesaurusList list = new MutableThesaurusList();
        int length = thesaurusKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey thesaurusKey = thesaurusKeyArray[i];
            if (thesaurusKey == null) {
                continue;
            }
            if (!thesaurusKey.isThesaurusSubset()) {
                continue;
            }
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
            if (thesaurus != null) {
                list.add(thesaurus);
            }
        }
        return list;
    }

    public static CorpusList getCorpusList(Basedefiches basedefiches, FicheSelectOptions ficheSelectOptions) {
        if (ficheSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        if (corpusKeyArray == null) {
            return basedefiches;
        }
        MutableCorpusList list = new MutableCorpusList();
        int length = corpusKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey corpusKey = corpusKeyArray[i];
            if (corpusKey == null) {
                continue;
            }
            if (!corpusKey.isCorpusSubset()) {
                continue;
            }
            Corpus corpus = (Corpus) basedefiches.getSubset(corpusKey);
            if (corpus != null) {
                list.add(corpus);
            }
        }
        return list;
    }

    public static AlbumList getAlbumList(Basedefiches basedefiches, AlbumSelectOptions albumSelectOptions) {
        if (albumSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] albumKeyArray = albumSelectOptions.getAlbumKeyArray();
        if (albumKeyArray == null) {
            return basedefiches;
        }
        MutableAlbumList list = new MutableAlbumList();
        int length = albumKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey albumKey = albumKeyArray[i];
            if (albumKey == null) {
                continue;
            }
            if (!albumKey.isAlbumSubset()) {
                continue;
            }
            Album album = (Album) basedefiches.getSubset(albumKey);
            if (album != null) {
                list.add(album);
            }
        }
        return list;
    }

}
