/* BasedefichesLib_Explore - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection.beans;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.util.conditions.ConditionGroup;
import net.mapeadores.util.conditions.ConditionGroupBean;
import net.mapeadores.util.conditions.Conditions;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public class SelectionBeanStorage {

    private SelectionBeanStorage() {
    }

    public static FicheSelectOptionsBean toBean(FicheSelectOptions ficheSelectOptions) {
        FicheSelectOptionsBean bean = new FicheSelectOptionsBean();
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        if (corpusKeyArray != null) {
            bean.setCorpusSubIdArray(toSubsetNameArray(corpusKeyArray));
        }
        ConditionGroup titreConditionGroup = ficheSelectOptions.getTitreConditionGroup();
        if (titreConditionGroup != null) {
            bean.setTitreConditionGroupBean(titreConditionGroup.toConditionGroupBean());
        }
        MotcleSelectOptions[] motcleSelectOptionsArray = ficheSelectOptions.getMotcleSelectOptionsArray();
        if (motcleSelectOptionsArray != null) {
            int length = motcleSelectOptionsArray.length;
            MotcleSelectOptionsBean[] beanArray = new MotcleSelectOptionsBean[length];
            for (int i = 0; i < length; i++) {
                beanArray[i] = toBean(motcleSelectOptionsArray[i]);
            }
            bean.setMotcleSelectOptionsBeanArray(beanArray);
        }
        RedacteurKey[] redacteurKeyArray = ficheSelectOptions.getRedacteurKeyArray();
        if (redacteurKeyArray != null) {
            String[] redacteurArray = new String[redacteurKeyArray.length];
            for (int i = 0; i < redacteurKeyArray.length; i++) {
                redacteurArray[i] = redacteurKeyArray[i].toString(RedacteurKey.BRACKET_STYLE);
            }
            bean.setRedacteurArray(redacteurArray);
        }
        RangeList rangeList = ficheSelectOptions.getRangeList();
        if (rangeList != null) {
            bean.setRange(RangeUtils.positiveRangetoString(rangeList));
        }
        bean.setMotcleMode(Conditions.modeToString(ficheSelectOptions.getMotcleSelectMode()));
        return bean;
    }

    public static MotcleSelectOptionsBean toBean(MotcleSelectOptions motcleSelectOptions) {
        MotcleSelectOptionsBean motcleSelectOptionsBean = new MotcleSelectOptionsBean();
        RangeList idthsRangeList = motcleSelectOptions.getIdthsRangeList();
        if (idthsRangeList != null) {
            motcleSelectOptionsBean.setIdthsRange(RangeUtils.positiveRangetoString(idthsRangeList));
        }
        RangeList levelRangeList = motcleSelectOptions.getLevelRangeList();
        if (levelRangeList != null) {
            motcleSelectOptionsBean.setLevelRange(RangeUtils.positiveRangetoString(levelRangeList));
        }
        SubsetKey[] thesaurusKeyArray = motcleSelectOptions.getThesaurusKeyArray();
        if (thesaurusKeyArray != null) {
            motcleSelectOptionsBean.setThesaurusSubIdArray(toSubsetNameArray(thesaurusKeyArray));
        }
        ConditionGroup conditionGroup = motcleSelectOptions.getSourceConditionGroup();
        if (conditionGroup != null) {
            motcleSelectOptionsBean.setConditionGroupBean(conditionGroup.toConditionGroupBean());
        }
        motcleSelectOptionsBean.setIdalphaOnly(motcleSelectOptions.isIdalphaOnly());
        return motcleSelectOptionsBean;
    }

    public static void fill(MotcleSelectOptions options, MotcleSelectOptionsBean bean) {
        RangeList idthsRangeList = getRangeList(bean.getIdthsRange());
        if (idthsRangeList != null) {
            options.setIdthsRangeList(idthsRangeList);
        }
        RangeList levelRangeList = getRangeList(bean.getLevelRange());
        if (levelRangeList != null) {
            options.setLevelRangeList(levelRangeList);
        }
        ConditionGroupBean conditionGroupeBean = bean.getConditionGroupBean();
        if (conditionGroupeBean != null) {
            options.initConditionGroup(new ConditionGroup(conditionGroupeBean));
        }
        options.setIdalphaOnly(bean.isIdalphaOnly());
        String[] thesaurusSubIdArray = bean.getThesaurusSubIdArray();
        if (thesaurusSubIdArray != null) {
            int length = thesaurusSubIdArray.length;
            SubsetKey[] thesaurusKeyArray = new SubsetKey[length];
            int p = 0;
            for (int i = 0; i < length; i++) {
                String subId = thesaurusSubIdArray[i];
                if ((subId != null) && (subId.length() > 0)) {
                    try {
                        thesaurusKeyArray[i] = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, subId);
                        p++;
                    } catch (ParseException pe) {
                    }
                }
            }
            thesaurusKeyArray = compare(thesaurusKeyArray, p);
            if (thesaurusKeyArray.length > 0) {
                options.setThesaurusKeyArray(thesaurusKeyArray);
            }
        }
    }

    public static void fill(FicheSelectOptions options, FicheSelectOptionsBean bean) {
        options.setMotcleSelectMode(Conditions.modeToShort(bean.getMotcleMode(), options.getMotcleSelectMode()));
        MotcleSelectOptionsBean[] motcleSelectOptionsBeanArray = bean.getMotcleSelectOptionsBeanArray();
        if (motcleSelectOptionsBeanArray != null) {
            int length = motcleSelectOptionsBeanArray.length;
            MotcleSelectOptions[] motcleSelectOptionsArray = new MotcleSelectOptions[length];
            for (int i = 0; i < length; i++) {
                motcleSelectOptionsArray[i] = new MotcleSelectOptions();
                fill(motcleSelectOptionsArray[i], motcleSelectOptionsBeanArray[i]);
            }
            options.setMotcleSelectOptionsArray(motcleSelectOptionsArray);
        }
        ConditionGroupBean titreBean = bean.getTitreConditionGroupBean();
        if (titreBean != null) {
            options.setTitreConditionGroup(new ConditionGroup(titreBean));
        }
        String[] corpusSubIdArray = bean.getCorpusSubIdArray();
        if (corpusSubIdArray != null) {
            SubsetKey[] corpusKeyArray = new SubsetKey[corpusSubIdArray.length];
            int p = 0;
            for (int i = 0; i < corpusKeyArray.length; i++) {
                String corpusSubId = corpusSubIdArray[i];
                if ((corpusSubId != null) && (corpusSubId.length() > 0)) {
                    try {
                        corpusKeyArray[i] = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusSubId);
                        p++;
                    } catch (ParseException pe) {
                    }
                }
            }
            corpusKeyArray = compare(corpusKeyArray, p);
            if (corpusKeyArray.length > 0) {
                options.setCorpusKeyArray(corpusKeyArray);
            }
        }
        String[] redacteurArray = bean.getRedacteurArray();
        if (redacteurArray != null) {
            RedacteurKey[] redacteurKeyArray = new RedacteurKey[redacteurArray.length];
            int p = 0;
            for (int i = 0; i < redacteurKeyArray.length; i++) {
                try {
                    RedacteurKey redacteurKey = RedacteurKey.parse(redacteurArray[i]);
                    redacteurKeyArray[p] = redacteurKey;
                    p++;
                } catch (java.text.ParseException pe) {
                }
            }
            if (p < redacteurKeyArray.length) {
                RedacteurKey[] nv = new RedacteurKey[p];
                System.arraycopy(redacteurKeyArray, 0, nv, 0, p);
                redacteurKeyArray = nv;
            }
            if (redacteurKeyArray.length > 0) {
                options.setRedacteurKeyArray(redacteurKeyArray);
            }
        }
        RangeList rangeList = getRangeList(bean.getRange());
        if (rangeList != null) {
            options.setRangeList(rangeList);
        }
    }

    private static RangeList getRangeList(String s) {
        if (s == null) {
            return null;
        }
        return RangeUtils.positiveRangeParse(s);
    }

    private static String[] toSubsetNameArray(SubsetKey[] subsetKeyArray) {
        int length = subsetKeyArray.length;
        String[] subsetNameArray = new String[length];
        for (int i = 0; i < length; i++) {
            subsetNameArray[i] = subsetKeyArray[i].getSubsetName();
        }
        return subsetNameArray;
    }

    private static SubsetKey[] compare(SubsetKey[] subsetKeyArray, int p) {
        if (p == subsetKeyArray.length) {
            return subsetKeyArray;
        }
        SubsetKey[] array = new SubsetKey[p];
        System.arraycopy(subsetKeyArray, 0, array, 0, p);
        return array;
    }

}
