/* BasedefichesLib_Explore
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection.util;

import net.mapeadores.util.text.*;
import net.mapeadores.util.text.insertpattern.api.*;
import net.mapeadores.util.text.insertpattern.impl.*;
import net.mapeadores.basedefiches.selection.*;
import java.util.*;
import java.text.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.conditions.*;


/**
 *
 * @author Vincent Calame
 */
public class SelectOptionsParser {

    public static boolean isMotcleSelectOptionsInstruction(Instruction instruction) {
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            if (key.equals("thesaurus")) {
                return true;
            }
            if (key.equals("motcle-q")) {
                return true;
            }
            if (key.equals("motcle-q-a")) {
                return true;
            }
        }
        return false;
    }

    public static MotcleSelectOptions parseInstruction(Instruction instruction) {
        Set<SubsetKey> thesaurusKeySet = new LinkedHashSet<SubsetKey>();
        ConditionGroup conditionGroup = null;
        ;
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            String value = argument.getValue();
            if (key.equals("thesaurus")) {
                if (value == null) {
                    continue;
                }
                String[] tokens = StringUtils.getTechnicalTokens(value, true);
                for (int j = 0; j < tokens.length; j++) {
                    try {
                        SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, tokens[i]);
                        thesaurusKeySet.add(thesaurusKey);
                    } catch (ParseException pe) {
                    }
                }
            } else if (key.equals("motcle-q")) {
                if (value != null) {
                    conditionGroup = Conditions.parseConditionGroup(value, ConditionGroup.MODE_OR);
                }
            } else if (key.equals("motcle-q-a")) {
                if (value != null) {
                    conditionGroup = Conditions.parseConditionGroup(value, ConditionGroup.MODE_AND);
                }
            }
        }
        int thesaurusSize = thesaurusKeySet.size();
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        if (thesaurusSize > 0) {
            SubsetKey[] thesaurusKeyArray = new SubsetKey[thesaurusSize];
            int p = 0;
            for (SubsetKey thesaurusKey : thesaurusKeySet) {
                thesaurusKeyArray[p] = thesaurusKey;
                p++;
            }
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
        }
        motcleSelectOptions.initConditionGroup(conditionGroup);
        return motcleSelectOptions;
    }

    public static MotcleSelectOptions parse(String s) {
        Instruction instruction = InstructionParser.parse(s, null);
        if (instruction == null) {
            return null;
        }
        return parseInstruction(instruction);
    }

}
