/* BasedefichesLib_Explore - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection.util;

import net.mapeadores.util.conditions.ConditionGroup;
import net.mapeadores.util.conditions.ContentConditionBean;
import net.mapeadores.util.conditions.Conditions;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.langinteger.LangInteger;
import java.text.Collator;


/**
 * Sélectionne une chaîne
 *
 * @author Vincent Calame
 */
public class StringPASelector {

    private ContentConditionBean[] excludingArray;
    private ContentConditionBean[] includingArray;
    private Collator collator;
    private short mode;

    private StringPASelector(ConditionGroup conditionGroup, int langInteger) {
        collator = Collator.getInstance(LangInteger.getLocale(langInteger));
        collator.setStrength(Collator.PRIMARY);
        excludingArray = conditionGroup.getExcludingContentConditionBeanArray();
        includingArray = conditionGroup.getIncludingContentConditionBeanArray();
        mode = conditionGroup.getMode();
    }

    public boolean isSelected(String s, boolean withEmpty) {
        if ((s == null) || (s.length() == 0)) {
            if (withEmpty) {
                return true;
            } else {
                return false;
            }
        }
        if (mode == ConditionsConstants.MODE_AND) {
            return isAndAccepted(s);
        } else {
            return isOrAccepted(s);
        }
    }

    private boolean isAndAccepted(String s) {
        for (int i = 0; i < includingArray.length; i++) {
            if (!Conditions.accept(includingArray[i], s, collator)) {
                return false;
            }
        }
        for (int i = 0; i < excludingArray.length; i++) {
            if (!Conditions.accept(excludingArray[i], s, collator)) {
                return false;
            }
        }
        return true;
    }

    private boolean isOrAccepted(String s) {
        for (int i = 0; i < includingArray.length; i++) {
            if (Conditions.accept(includingArray[i], s, collator)) {
                return true;
            }
        }
        for (int i = 0; i < excludingArray.length; i++) {
            if (Conditions.accept(excludingArray[i], s, collator)) {
                return true;
            }
        }
        return false;
    }

    public static StringPASelector newInstance(ConditionGroup conditionGroup, int langInteger) {
        if (conditionGroup == null) {
            return null;
        }
        if (!Conditions.isPartState(conditionGroup.getState())) {
            return null;
        }
        return new StringPASelector(conditionGroup, langInteger);
    }

}
