/* BasedefichesLib_Export - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;


/**
 *
 * @author Vincent Calame
 */
public class CorpusExportDef {

    SubsetKey corpusKey;
    List<Object> complementList = new ArrayList<Object>();
    List<Object> unmodifiableList = Collections.unmodifiableList(complementList);
    Object dateKey = IncludeKey.DATE_CREATION;

    public CorpusExportDef(SubsetKey corpusKey) {
        this.corpusKey = corpusKey;
    }

    public SubsetKey getCorpusKey() {
        return corpusKey;
    }

    public void addComplementKey(Object key) {
        if (key == null) {
            throw new IllegalArgumentException("fieldKey cannot be null");
        }
        if (key instanceof FieldKey) {
            FieldKey fieldKey = (FieldKey) key;
            if (fieldKey.isAnnexeFieldKey()) {
                return;
            }
            if ((fieldKey.isSpecialFieldKey()) && (!fieldKey.equals(FieldKey.REDACTEURS))) {
                return;
            }
        } else if (key instanceof FilteredIncludeKey) {
            IncludeKey includeKey = ((FilteredIncludeKey) key).getIncludeKey();
            if (!includeKey.isLiageIncludeKey()) {
                return;
            }
        } else {
            throw new ClassCastException("key must be an instance of FieldKey or IncludeBundle");
        }
        if (!complementList.contains(key)) {
            complementList.add(key);
        }
    }

    /**
     * Contient des instances de FieldKey ou de FilteredIncludeKey
     */
    public List<Object> getComplementKeyList() {
        return unmodifiableList;
    }

    public Object getDateKey() {
        return dateKey;
    }

    /**
     *@throws IllegalArgumentException si dateKey est nul
     */
    public void setDateKey(Object dateKey) {
        if (dateKey == null) {
            throw new IllegalArgumentException("dateKey argument is null");
        }
        this.dateKey = dateKey;
    }

    public static Object parseDateKey(String s) {
        try {
            IncludeKey includeKey = IncludeKey.parse(s);
            if (includeKey.isChronoIncludeKey()) {
                return includeKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            try {
                FieldKey fieldKey = FieldKey.parse(s);
                if ((!fieldKey.isProprieteFieldKey()) && (!fieldKey.isInformationFieldKey())) {
                    return null;
                }
                return fieldKey;
            } catch (java.text.ParseException pe2) {
                return null;
            }
        }
    }

}
