/* BasedefichesLib_Export - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.StringUtils;
import java.util.Map;
import java.util.HashMap;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExport {

    private String name;
    private Basedefiches basedefiches;
    private String baseName = null;
    private String baseIcon = null;
    private FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
    private Map<SubsetKey, Object> exportDefMap = new HashMap<SubsetKey, Object>();
    private AccoladePattern ficheGlobalPattern;
    private AccoladePattern motcleGlobalPattern;
    private int[] langUiArray;

    public ScrutariExport(Basedefiches basedefiches, String name) throws ParseException {
        StringUtils.checkTechnicalName(name, true);
        this.name = name;
        this.basedefiches = basedefiches;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public String getName() {
        return name;
    }

    public String getBaseName() {
        return baseName;
    }

    public void setBaseName(String baseName) throws ParseException {
        if (baseName == null) {
            this.baseName = null;
        } else {
            StringUtils.checkTechnicalName(baseName, true);
            this.baseName = baseName;
        }
    }

    public String getBaseIcon() {
        return baseIcon;
    }

    public void setBaseIcon(String baseIcon) {
        this.baseIcon = baseIcon;
    }

    public FicheSelectOptions getFicheSelectOptions() {
        return ficheSelectOptions;
    }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.ficheSelectOptions = ficheSelectOptions;
    }

    public CorpusExportDef getCorpusExportDef(SubsetKey corpusKey) {
        return (CorpusExportDef) exportDefMap.get(corpusKey);
    }

    public void putCorpusExportDef(CorpusExportDef corpusExportDef) {
        exportDefMap.put(corpusExportDef.getCorpusKey(), corpusExportDef);
    }

    public ThesaurusExportDef getThesaurusExportDef(SubsetKey thesaurusKey) {
        return (ThesaurusExportDef) exportDefMap.get(thesaurusKey);
    }

    public void putThesaurusExportDef(ThesaurusExportDef thesaurusExportDef) {
        exportDefMap.put(thesaurusExportDef.getThesaurusKey(), thesaurusExportDef);
    }

    public AccoladePattern getFicheGlobalPattern() {
        return ficheGlobalPattern;
    }

    public void setFicheGlobalPattern(AccoladePattern ficheGlobalPattern) {
        this.ficheGlobalPattern = ficheGlobalPattern;
    }

    public AccoladePattern getMotcleGlobalPattern() {
        return motcleGlobalPattern;
    }

    public void setMotcleGlobalPattern(AccoladePattern motcleGlobalPattern) {
        this.motcleGlobalPattern = motcleGlobalPattern;
    }

    /**
     * Peut être égal à <em>null</em> mais n'est jamais de longueur nulle.
     */
    public int[] getLangUiArray() {
        return langUiArray;
    }

    public void setLangUiArray(int[] array) {
        if ((array == null) || (array.length == 0)) {
            this.langUiArray = null;
        } else {
            int length = array.length;
            langUiArray = new int[length];
            System.arraycopy(array, 0, langUiArray, 0, length);
        }
    }

}
