/* BasedefichesLib_Export - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport.beans;

import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeUtils;
import net.mapeadores.basedefiches.scrutariexport.CorpusExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExport;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusExportDef;
import net.mapeadores.basedefiches.selection.beans.FicheSelectOptionsBean;
import net.mapeadores.basedefiches.selection.beans.SelectionBeanStorage;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportBeanStorage {

    private ScrutariExportBeanStorage() {
    }

    public static RchfExportBean toBean(ScrutariExport scrutariExport) {
        Basedefiches basedefiches = scrutariExport.getBasedefiches();
        RchfExportBean bean = new RchfExportBean();
        bean.setName(scrutariExport.getName());
        bean.setBaseName(scrutariExport.getBaseName());
        bean.setFicheSelectOptionsBean(SelectionBeanStorage.toBean(scrutariExport.getFicheSelectOptions()));
        bean.setLangUiArray(scrutariExport.getLangUiArray());
        int corpusCount = basedefiches.getCorpusCount();
        CorpusRchfInfoBean[] corpusRchfInfoBeanArray = new CorpusRchfInfoBean[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusExportDef corpusInfo = scrutariExport.getCorpusExportDef(corpus.getSubsetKey());
            if (corpusInfo == null) {
                CorpusRchfInfoBean corpusRchfInfoBean = new CorpusRchfInfoBean();
                corpusRchfInfoBean.setCorpusSubId(corpus.getSubsetKey().getSubsetName());
                corpusRchfInfoBeanArray[i] = corpusRchfInfoBean;
            } else {
                corpusRchfInfoBeanArray[i] = toBean(corpusInfo);
            }
        }
        bean.setCorpusRchfInfoBeanArray(corpusRchfInfoBeanArray);
        int thesaurusCount = basedefiches.getThesaurusCount();
        ThesaurusRchfInfoBean[] thesaurusRchfInfoBeanArray = new ThesaurusRchfInfoBean[thesaurusCount];
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            ThesaurusExportDef thesaurusInfo = scrutariExport.getThesaurusExportDef(thesaurus.getSubsetKey());
            if (thesaurusInfo == null) {
                ThesaurusRchfInfoBean thsbean = new ThesaurusRchfInfoBean();
                thsbean.setThesaurusSubId(thesaurus.getSubsetKey().getSubsetName());
                thesaurusRchfInfoBeanArray[i] = thsbean;
            } else {
                thesaurusRchfInfoBeanArray[i] = toBean(thesaurusInfo);
            }
        }
        bean.setThesaurusRchfInfoBeanArray(thesaurusRchfInfoBeanArray);
        AccoladePattern ficheGlobalPattern = scrutariExport.getFicheGlobalPattern();
        if (ficheGlobalPattern != null) {
            bean.setFicheGlobalPattern(ficheGlobalPattern.toString());
        }
        AccoladePattern motcleGlobalPattern = scrutariExport.getMotcleGlobalPattern();
        if (motcleGlobalPattern != null) {
            bean.setMotcleGlobalPattern(motcleGlobalPattern.toString());
        }
        bean.setBaseIcon(scrutariExport.getBaseIcon());
        return bean;
    }

    private static CorpusRchfInfoBean toBean(CorpusExportDef corpusRchfInfo) {
        CorpusRchfInfoBean bean = new CorpusRchfInfoBean();
        bean.setCorpusSubId(corpusRchfInfo.getCorpusKey().getSubsetName());
        List list = corpusRchfInfo.getComplementKeyList();
        int size = list.size();
        String[] keyArray = new String[size];
        for (int i = 0; i < size; i++) {
            keyArray[i] = list.get(i).toString();
        }
        bean.setComplementKeyArray(keyArray);
        bean.setDateKey(corpusRchfInfo.getDateKey().toString());
        return bean;
    }

    private static ThesaurusRchfInfoBean toBean(ThesaurusExportDef thesaurusRchfInfo) {
        ThesaurusRchfInfoBean bean = new ThesaurusRchfInfoBean();
        bean.setThesaurusSubId(thesaurusRchfInfo.getThesaurusKey().getSubsetName());
        bean.setSelected(thesaurusRchfInfo.isSelected());
        AccoladePattern libellePattern = thesaurusRchfInfo.getLibellePattern();
        if (libellePattern != null) {
            bean.setLibellePattern(libellePattern.toString());
        }
        return bean;
    }

    public static void fill(ScrutariExport rchfExport, RchfExportBean rchfExportBean) {
        try {
            rchfExport.setBaseName(rchfExportBean.getBaseName());
        } catch (java.text.ParseException pe) {
        }
        rchfExport.setLangUiArray(rchfExportBean.getLangUiArray());
        FicheSelectOptionsBean optionsBean = rchfExportBean.getFicheSelectOptionsBean();
        if (optionsBean != null) {
            SelectionBeanStorage.fill(rchfExport.getFicheSelectOptions(), optionsBean);
        }
        CorpusRchfInfoBean[] corpusRchfInfoBeanArray = rchfExportBean.getCorpusRchfInfoBeanArray();
        if (corpusRchfInfoBeanArray != null) {
            for (int i = 0; i < corpusRchfInfoBeanArray.length; i++) {
                CorpusRchfInfoBean corpusRchfInfoBean = corpusRchfInfoBeanArray[i];
                if (corpusRchfInfoBean != null) {
                    String corpusSubId = corpusRchfInfoBean.getCorpusSubId();
                    if (corpusSubId != null) {
                        Corpus corpus = BasedefichesUtils.getCorpus(rchfExport.getBasedefiches(), corpusSubId);
                        if (corpus != null) {
                            CorpusExportDef corpusRchfInfo = new CorpusExportDef(corpus.getSubsetKey());
                            fill(corpusRchfInfo, corpusRchfInfoBean);
                            rchfExport.putCorpusExportDef(corpusRchfInfo);
                        }
                    }
                }
            }
        }
        ThesaurusRchfInfoBean[] thesaurusRchfInfoBeanArray = rchfExportBean.getThesaurusRchfInfoBeanArray();
        if (thesaurusRchfInfoBeanArray != null) {
            for (int i = 0; i < thesaurusRchfInfoBeanArray.length; i++) {
                ThesaurusRchfInfoBean thesaurusRchfInfoBean = thesaurusRchfInfoBeanArray[i];
                if (thesaurusRchfInfoBean != null) {
                    String thesaurusSubId = thesaurusRchfInfoBean.getThesaurusSubId();
                    if (thesaurusSubId != null) {
                        Thesaurus thesaurus = BasedefichesUtils.getThesaurus(rchfExport.getBasedefiches(), thesaurusSubId);
                        if (thesaurus != null) {
                            ThesaurusExportDef thesaurusRchfInfo = new ThesaurusExportDef(thesaurus.getSubsetKey());
                            fill(thesaurusRchfInfo, thesaurusRchfInfoBean);
                            rchfExport.putThesaurusExportDef(thesaurusRchfInfo);
                        }
                    }
                }
            }
        }
        String fichePatternString = rchfExportBean.getFicheGlobalPattern();
        if (fichePatternString != null) {
            try {
                rchfExport.setFicheGlobalPattern(new AccoladePattern(fichePatternString));
            } catch (java.text.ParseException pe) {
            }
        }
        String motclePatternString = rchfExportBean.getMotcleGlobalPattern();
        if (motclePatternString != null) {
            try {
                rchfExport.setMotcleGlobalPattern(new AccoladePattern(motclePatternString));
            } catch (java.text.ParseException pe) {
            }
        }
        rchfExport.setBaseIcon(rchfExportBean.getBaseIcon());
    }

    private static void fill(CorpusExportDef corpusRchfInfo, CorpusRchfInfoBean corpusRchfInfoBean) {
        String[] keyArray = corpusRchfInfoBean.getComplementKeyArray();
        if (keyArray != null) {
            for (int i = 0; i < keyArray.length; i++) {
                Object key = IncludeUtils.parseKey(keyArray[i]);
                if (key != null) {
                    corpusRchfInfo.addComplementKey(key);
                }
            }
        }
        String dateKeyString = corpusRchfInfoBean.getDateKey();
        if (dateKeyString != null) {
            Object dateKey = CorpusExportDef.parseDateKey(dateKeyString);
            if (dateKey != null) {
                corpusRchfInfo.setDateKey(dateKey);
            }
        }
    }

    private static void fill(ThesaurusExportDef thesaurusRchfInfo, ThesaurusRchfInfoBean thesaurusRchfInfoBean) {
        thesaurusRchfInfo.setSelected(thesaurusRchfInfoBean.isSelected());
        String pat = thesaurusRchfInfoBean.getLibellePattern();
        if (pat != null) {
            try {
                thesaurusRchfInfo.setLibellePattern(new AccoladePattern(pat));
            } catch (java.text.ParseException pe) {
            }
        }
    }

}
