/* BasedefichesLib_Export - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tableexport;

import java.io.IOException;
import java.io.LineNumberReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefParser;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class ColDefListParser {

    private InternalColDefList result = new InternalColDefList();
    private ColDefErrorHandler errorHandler;
    private ColDefTestHandler testHandler;
    private String currentColLine = null;
    private String currentFormatSource = null;
    private String currentFormatPattern = null;
    private String currentFormatInstruction = null;
    private List<String> customLineList = new ArrayList<String>();
    private int currentLineNumber;
    private int firstLineNumber;
    private boolean ficheFormatLine = false;

    private ColDefListParser(ColDefTestHandler testHandler, ColDefErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.testHandler = testHandler;
    }

    public static ColDefList parse(Reader reader, ColDefTestHandler testHandler, ColDefErrorHandler errorHandler) throws IOException {
        LineNumberReader lineNumberReader = new LineNumberReader(reader);
        String line = null;
        ColDefListParser colDefListParser = new ColDefListParser(testHandler, errorHandler);
        boolean debut = true;
        while ((line = lineNumberReader.readLine()) != null) {
            int lineNumber = lineNumberReader.getLineNumber();
            line = line.trim();
            if (line.length() == 0) {
                colDefListParser.flush();
                debut = true;
            } else if (line.startsWith("#")) {
                continue;
            } else if ((debut) && (line.startsWith("!"))) {
                colDefListParser.addParameter(line, lineNumber);
            } else {
                debut = false;
                colDefListParser.addLine(line, lineNumber);
            }
        }
        colDefListParser.flush();
        return colDefListParser.result;
    }

    public static ColDefList parse(String s, ColDefTestHandler testHandler, ColDefErrorHandler errorHandler) throws IOException {
        try {
            StringReader reader = new StringReader(s);
            return parse(reader, testHandler, errorHandler);
        } catch (IOException ioe) {
            throw new ShouldNotOccurException();
        }
    }

    private void addParameter(String line, int currentLineNumber) {
        if (line.length() == 1) {
            errorHandler.wrongParameterLine(currentLineNumber, line, "emptyParameterLine");
            return;
        }
        int idx = line.indexOf('=');
        if (idx == -1) {
            errorHandler.wrongParameterLine(currentLineNumber, line, "missingParameterValue");
            return;
        }
        String paramName = line.substring(1, idx).trim();
        if (paramName.length() == 0) {
            errorHandler.wrongParameterLine(currentLineNumber, line, "missingParameterName");
            return;
        }
        String paramValue = line.substring(idx + 1).trim();
        result.addParameter(paramName, paramValue);
    }

    private void addLine(String line, int currentLineNumber) {
        this.currentLineNumber = currentLineNumber;
        if (currentColLine == null) {
            firstLineNumber = currentLineNumber;
            if (line.startsWith("%")) {
                if (line.length() == 1) {
                    line = "NULL";
                } else {
                    line = line.substring(1);
                }
                ficheFormatLine = false;
            } else {
                ficheFormatLine = true;
            }
            currentColLine = line;
        } else {
            if (ficheFormatLine) {
                if (currentFormatSource == null) {
                    currentFormatSource = line;
                } else if (currentFormatPattern == null) {
                    currentFormatPattern = line;
                } else if (currentFormatInstruction == null) {
                    currentFormatInstruction = line;
                } else {
                    errorHandler.tooManyLineWarning(currentLineNumber, line);
                }
            } else {
                customLineList.add(line);
            }
        }
    }

    private void flush() {
        if (currentColLine != null) {
            String colName = null;
            String colLib = null;
            int idx = currentColLine.indexOf('>');
            if (idx > 0) {
                colName = currentColLine.substring(0, idx).trim();
                colLib = currentColLine.substring(idx + 1).trim();
            } else {
                colName = currentColLine;
            }
            if (ficheFormatLine) {
                if (currentFormatSource == null) {
                    errorHandler.missingFormatSourceLineError(firstLineNumber + 2);
                    clear();
                    return;
                }
                if ((currentFormatPattern != null) && (currentFormatPattern.equals("-"))) {
                    currentFormatPattern = null;
                }
                BasedefichesFormatDefParser.ErrorHandler eh = new BasedefichesFormatDefErrorHandler(firstLineNumber + 1);
                BasedefichesFormatDef basedefichesFormatDef = BasedefichesFormatDefParser.parse(currentFormatSource, currentFormatPattern, currentFormatInstruction, eh);
                if (basedefichesFormatDef != null) {
                    if ((testHandler == null) || (testHandler.testBasedefichesFormatDef(firstLineNumber + 1, basedefichesFormatDef))) {
                        ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(colName, colLib, basedefichesFormatDef);
                        result.addColDef(colDef);

                    }
                }
            } else {
                ColDef colDef = ColDefUtils.newExtensionInstance(colName, colLib, customLineList.toArray(new String[customLineList.size()]));
                result.addColDef(colDef);
            }
            clear();
        }
    }

    private void clear() {
        currentColLine = null;
        currentFormatSource = null;
        currentFormatPattern = null;
        currentFormatInstruction = null;
        customLineList.clear();
    }


    private class BasedefichesFormatDefErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        int sourceLineNumber;

        BasedefichesFormatDefErrorHandler(int sourceLineNumber) {
            this.sourceLineNumber = sourceLineNumber;
        }

        public void nullFormatSourceError() {
            throw new ShouldNotOccurException();
        }

        public void wrongFormatSourceError() {
            errorHandler.formatSourceError(sourceLineNumber, "wrongFormatSource", currentFormatSource);
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            errorHandler.formatSourceError(sourceLineNumber, "wrongFormatSourcePart", formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            errorHandler.formatInstructionError(sourceLineNumber + 2, "invalidAsciiCharacter", formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            errorHandler.formatInstructionError(sourceLineNumber + 2, "invalidEndCharacter", formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            errorHandler.formatInstructionError(sourceLineNumber + 2, "invalidSeparatorCharacter", formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            errorHandler.formatInstructionError(sourceLineNumber + 2, "unknownArgumentKey", argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            errorHandler.formatInstructionError(sourceLineNumber + 2, "missingArgumentValue", argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            String[] res = {argumentKey, argumentValue};
            errorHandler.formatInstructionError(sourceLineNumber + 2, "wrongArgumentValue", res);
        }

    }


    private static class InternalColDefList implements ColDefList {

        private List<ColDef> colDefList = new ArrayList<ColDef>();
        private Map<String, String> parametersMap = new HashMap<String, String>();

        private InternalColDefList() {
        }

        public int getColDefCount() {
            return colDefList.size();
        }

        public ColDef getColDef(int i) {
            return colDefList.get(i);
        }

        public String getParameterValue(String paramName) {
            return parametersMap.get(paramName);
        }

        private void addParameter(String paramName, String paramValue) {
            parametersMap.put(paramName, paramValue);
        }

        private void addColDef(ColDef colDef) {
            colDefList.add(colDef);
        }

    }

}
