/* BasedefichesLib_Impl - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumImage;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadata;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplAlbum extends AbstractIndexAndIntKeyMap implements Album {

    private BdfimplBasedefiches basedefiches;
    private BdfimplAlbumMetadata albumMetadata;
    private Map<String, BdfimplAlbumImage> imageByNameMap = new HashMap<String, BdfimplAlbumImage>();
    private SubsetKey albumKey;
    private BdfimplAlbumEditor bdfimplAlbumEditor = new BdfimplAlbumEditor();

    public BdfimplAlbum(SubsetKey albumKey, BdfimplBasedefiches basedefiches) {
        this.basedefiches = basedefiches;
        this.albumKey = albumKey;
        this.albumMetadata = new BdfimplAlbumMetadata(this);
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public AlbumMetadata getAlbumMetadata() {
        return albumMetadata;
    }

    public AlbumImage getAlbumImageByName(String name) {
        return imageByNameMap.get(name);
    }

    public SubsetKey getSubsetKey() {
        return albumKey;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    public Subset getMasterSubset() {
        return null;
    }

    public Subset[] getSlaveSubsetArray() {
        return null;
    }

    protected AlbumEditor getAlbumEditor() {
        return bdfimplAlbumEditor;
    }

    protected void fireAlbumChange() {
        basedefiches.fireSubsetChange(albumKey);
    }

    protected void disable() {
        albumMetadata.disable();
        imageByNameMap.clear();
    }


    private class BdfimplAlbumEditor implements AlbumEditor {

        public Album getAlbum() {
            return BdfimplAlbum.this;
        }

        public AlbumMetadataEditor getAlbumMetadataEditor() {
            return albumMetadata.getAlbumMetadataEditor();
        }

        public AlbumImage createAlbumImage(short type, String imageName) throws ExistingNameException {
            if (!AlbumUtils.isValidImageType(type)) {
                throw new IllegalArgumentException("wrong type " + type);
            }
            int idalbum = BdfimplAlbum.this.getTicketLength() + 1;
            if (imageName != null) {
                if (imageByNameMap.containsKey(imageName)) {
                    throw new ExistingNameException();
                }
            } else {
                imageName = getNewName(idalbum, 1);
            }
            BdfimplAlbumImage bdfimplAlbumImage = new BdfimplAlbumImage(type, imageName, idalbum);
            bdfimplAlbumImage.ticket = BdfimplAlbum.this.putFromKey(idalbum, bdfimplAlbumImage);
            imageByNameMap.put(imageName, bdfimplAlbumImage);
            return bdfimplAlbumImage;
        }

        public void updateAlbumImage(AlbumImage albumImage, InputStream inputStream) throws IOException {
            basedefiches.getAlbumDataSource().update(albumImage, inputStream);
        }

        private String getNewName(int idalbum, int increment) {
            String prefix = "bdf";
            if (increment > 1) {
                prefix = prefix + increment;
            }
            String name = prefix + "-" + idalbum;
            if (!imageByNameMap.containsKey(name)) {
                return name;
            } else {
                return getNewName(idalbum, increment + 1);
            }
        }

    }


    private class BdfimplAlbumImage implements AlbumImage {

        int ticket;
        int idalbum;
        String name;
        short type;

        private BdfimplAlbumImage(short type, String name, int idalbum) {
            this.type = type;
            this.name = name;
            this.idalbum = idalbum;
        }

        public Subset getSubset() {
            return BdfimplAlbum.this;
        }

        public int getIdsubset() {
            return idalbum;
        }

        public int getTicket() {
            return ticket;
        }

        public Album getAlbum() {
            return BdfimplAlbum.this;
        }

        public String getName() {
            return name;
        }

        public short getType() {
            return type;
        }

        public InputStream getInputStream(AlbumDim albumDim) {
            if (albumDim != null) {
                if (!albumMetadata.isFromAlbum(albumDim)) {
                    throw new IllegalArgumentException("albumDim is not linked to this Album");
                }
            }
            return basedefiches.getAlbumDataSource().getInputStream(this, albumDim);
        }

    }

}
