/* BasedefichesLib_Impl - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadata;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplAlbumMetadata implements AlbumMetadata {

    private BdfimplAlbum album;
    private BdfimplAlbumMetadataEditor albumMetadataEditor = new BdfimplAlbumMetadataEditor();
    private Map<String, BdfimplAlbumDim> albumDimMap = new HashMap<String, BdfimplAlbumDim>();
    private List<BdfimplAlbumDim> albumDimList = new ArrayList<BdfimplAlbumDim>();
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();
    private Map<String, BdfimplAlbumDim> albumDimUsageMap = new LinkedHashMap<String, BdfimplAlbumDim>();

    public BdfimplAlbumMetadata(BdfimplAlbum album) {
        this.album = album;
    }

    public Album getAlbum() {
        return album;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    protected AlbumMetadataEditor getAlbumMetadataEditor() {
        return albumMetadataEditor;
    }

    public int getAlbumDimCount() {
        return albumDimList.size();
    }

    public AlbumDim getAlbumDim(int i) {
        return albumDimList.get(i);
    }

    public AlbumDim getAlbumDimByName(String name) {
        return albumDimMap.get(name);
    }

    public AlbumDim getAlbumDimForUsage(String usageName) {
        return albumDimUsageMap.get(usageName);
    }

    public String[] getUsageNameArray() {
        int size = albumDimUsageMap.size();
        String[] result = new String[size];
        if (size > 0) {
            result = albumDimUsageMap.keySet().toArray(result);
        }
        return result;
    }

    protected void fireChange() {
        album.fireAlbumChange();
    }

    protected void disable() {
        albumDimMap.clear();
        albumDimList.clear();
        intituleMap.clear();
        intituleList.clear();
        albumDimUsageMap.clear();
    }


    private class BdfimplAlbumMetadataEditor implements AlbumMetadataEditor {

        public AlbumMetadata getAlbumMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public AlbumDim createAlbumDim(String name, short type) throws ExistingNameException, ParseException {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (albumDimMap.containsKey(name)) {
                throw new ExistingNameException();
            }
            AlbumUtils.testAlbumDimName(name);
            BdfimplAlbumDim bdfimplAlbumDim = new BdfimplAlbumDim(name, type);
            albumDimList.add(bdfimplAlbumDim);
            albumDimMap.put(name, bdfimplAlbumDim);
            fireChange();
            return bdfimplAlbumDim;
        }

        public boolean setDim(AlbumDim albumDim, int width, int height) {
            BdfimplAlbumDim bdfimplAlbumDim = testAlbumDim(albumDim);
            boolean done = false;
            if (bdfimplAlbumDim.setWidth(width)) {
                done = true;
            }
            if (bdfimplAlbumDim.setHeight(height)) {
                done = true;
            }
            if (done) {
                fireChange();
            }
            return done;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplAlbumMetadata.this, album.getSubsetKey().toString());
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            fireChange();
            return intitule;
        }

        public Metadata getMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public boolean setIntituleLibelle(Intitule intitule, String nvlibelle, int langInteger) {
            BdfimplIntitule bti = BdfimplUtils.testIntitule(BdfimplAlbumMetadata.this, intitule);
            boolean done = bti.internalSetLibelle(nvlibelle, langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean setAlbumDimForUsage(String usageName, AlbumDim albumDim) {
            BdfimplAlbumDim current = albumDimUsageMap.get(usageName);
            if (albumDim == null) {
                if (current == null) {
                    return false;
                } else {
                    albumDimUsageMap.remove(usageName);
                    return true;
                }
            }
            BdfimplAlbumDim bdfimplAlbumDim = testAlbumDim(albumDim);
            if ((current != null) && (current.equals(bdfimplAlbumDim))) {
                return false;
            }
            albumDimUsageMap.put(usageName, bdfimplAlbumDim);
            return true;
        }

    }


    private class BdfimplAlbumDim implements AlbumDim {

        private String name;
        private short type;
        private int width = 0;
        private int height = 0;

        private BdfimplAlbumDim(String name, short type) {
            this.name = name;
            this.type = type;
            if (AlbumUtils.needWidth(type)) {
                width = 100;
            }
            if (AlbumUtils.needHeight(type)) {
                height = 100;
            }
        }

        public AlbumMetadata getAlbumMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public String getName() {
            return name;
        }

        public short getType() {
            return type;
        }

        public int getWidth() {
            return width;
        }

        public int getHeight() {
            return height;
        }

        boolean setWidth(int width) {
            if (!AlbumUtils.needWidth(type)) {
                return false;
            }
            if (width < 1) {
                return false;
            }
            if (this.width == width) {
                return false;
            }
            this.width = width;
            return true;
        }

        boolean setHeight(int height) {
            if (!AlbumUtils.needHeight(type)) {
                return false;
            }
            if (height < 1) {
                return false;
            }
            if (this.height == height) {
                return false;
            }
            this.height = height;
            return true;
        }

    }

    protected boolean isFromAlbum(AlbumDim albumDim) {
        try {
            testAlbumDim(albumDim);
            return true;
        } catch (IllegalArgumentException iae) {
            return false;
        }
    }

    private BdfimplAlbumDim testAlbumDim(AlbumDim albumDim) throws IllegalArgumentException {
        if (!(albumDim instanceof BdfimplAlbumDim)) {
            throw new IllegalArgumentException("albumDim argument does not come from this AlbumMetadata instance");
        }
        if (albumDim.getAlbumMetadata() != this) {
            throw new IllegalArgumentException("albumDim argument does not come from this AlbumMetadata instance");
        }
        return (BdfimplAlbumDim) albumDim;
    }

}
