/* BasedefichesLib_Impl - Copyright (c) 2068-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplBasedefichesMetadata implements BasedefichesMetadata {

    private BdfimplBasedefiches basedefiches;
    private String authorityUUID = "e17a05b0-c45e-11d8-9669-0800200c9a66";
    private String basedefichesName = "basedefiches";
    private BdfimplBasedefichesMetadataEditor basedefichesMetadataEditor = new BdfimplBasedefichesMetadataEditor();
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();

    public BdfimplBasedefichesMetadata(BdfimplBasedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public String getAuthorityUUID() {
        return authorityUUID;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public String getBasedefichesName() {
        return basedefichesName;
    }

    protected BasedefichesMetadataEditor getBasedefichesMetadataEditor() {
        return basedefichesMetadataEditor;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }


    private class BdfimplBasedefichesMetadataEditor implements BasedefichesMetadataEditor {

        public Metadata getMetadata() {
            return BdfimplBasedefichesMetadata.this;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplBasedefichesMetadata.this, basedefichesName);
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            basedefiches.fireBasedefichesMetadataChange();
            return intitule;
        }

        public boolean setAuthorityUUID(String s) {
            if (!StringUtils.checkUUID(s)) {
                throw new IllegalArgumentException("not a UUID : " + s);
            }
            boolean done = false;
            if (!s.equals(authorityUUID)) {
                authorityUUID = s;
                done = true;
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

        public boolean setBasedefichesName(String s) {
            if (!StringUtils.isTechnicalName(s, true)) {
                throw new IllegalArgumentException("not a technical name : " + s);
            }
            boolean done = false;
            if (!s.equals(basedefichesName)) {
                basedefichesName = s;
                done = true;
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

        public boolean setIntituleLibelle(Intitule intitule, String nvlibelle, int langInteger) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplBasedefichesMetadata.this, intitule);
            boolean done = def.internalSetLibelle(nvlibelle, langInteger);
            if (done) {
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

    }

}
