/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.corpus.metadata.MandatoryFieldException;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.text.AbstractLibelleHolder;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplCorpusMetadata implements CorpusMetadata {

    private BdfimplCorpus corpus;
    private BdfimplCorpusMetadataEditor corpusMetadataEditor;
    private CorpusField geolocalisationField;
    private Map<FieldKey, CorpusField> corpusFieldMap = new HashMap<FieldKey, CorpusField>();
    private List<BdfimplCorpusField> annexeFieldList = new ArrayList<BdfimplCorpusField>();
    private List<BdfimplCorpusField> proprieteFieldList = new ArrayList<BdfimplCorpusField>();
    private List<BdfimplCorpusField> informationFieldList = new ArrayList<BdfimplCorpusField>();
    private BasedefichesFormatDef titreFormatDef;
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();

    public BdfimplCorpusMetadata(BdfimplCorpus corpus) {
        this.corpus = corpus;
        corpusMetadataEditor = new BdfimplCorpusMetadataEditor();
        initField(FieldKey.IDCORPUS);
        initField(FieldKey.LANG);
        initField(FieldKey.TITRE);
        initField(FieldKey.REDACTEURS);
    }

    private void initField(FieldKey fieldKey) {
        corpusMetadataEditor.internalCreateCorpusField(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public Corpus getCorpus() {
        return corpus;
    }

    public CorpusField getCorpusFieldByFieldKey(FieldKey fieldKey) {
        return corpusFieldMap.get(fieldKey);
    }

    public int getProprieteFieldCount() {
        return proprieteFieldList.size();
    }

    public CorpusField getProprieteField(int i) {
        return proprieteFieldList.get(i);
    }

    public int getInformationFieldCount() {
        return informationFieldList.size();
    }

    public CorpusField getInformationField(int i) {
        return informationFieldList.get(i);
    }

    public int getAnnexeFieldCount() {
        return annexeFieldList.size();
    }

    public CorpusField getAnnexeField(int i) {
        return annexeFieldList.get(i);
    }

    public CorpusField getGeolocalisationField() {
        return geolocalisationField;
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    protected CorpusMetadataEditor getCorpusMetadataEditor() {
        return corpusMetadataEditor;
    }

    protected void disable() {
        annexeFieldList.clear();
        proprieteFieldList.clear();
        informationFieldList.clear();
        corpusFieldMap.clear();
        intituleMap.clear();
        intituleList.clear();
    }

    protected ListenerList getListenerList() {
        return corpus.getListenerList();
    }

    BasedefichesFormatDef getFicheFormatDef() {
        return titreFormatDef;
    }

    private void fireChange() {
        corpus.basedefiches.fireSubsetChange(corpus.getSubsetKey());
    }


    private class BdfimplCorpusField extends AbstractLibelleHolder implements CorpusField {

        private FieldKey fieldKey;
        private short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;
        private short informationDisplay = FieldOptions.INLINE_DISPLAY;
        private SubsetKey sphereKey;
        private boolean subfieldDisplay = false;
        private int[] currencyArray = null;
        private CorpusField[] addressFieldsArray = null;
        private Map<String, String> stringOptions = new HashMap<String, String>();
        private SubsetKey albumKey;

        BdfimplCorpusField(FieldKey fieldKey) {
            this(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
        }

        BdfimplCorpusField(FieldKey fieldKey, short ficheItemType) {
            super(fieldKey.toString());
            this.fieldKey = fieldKey;
            this.ficheItemType = ficheItemType;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

        public CorpusMetadata getCorpusMetadata() {
            return BdfimplCorpusMetadata.this;
        }

        boolean internalSetLibelle(String nvlibelle, int langInteger) {
            return this.setLibelle(nvlibelle, langInteger);
        }

        public short getFicheItemType() {
            return ficheItemType;
        }

        public String getStringOption(String optionName) {
            return stringOptions.get(optionName);
        }

        public String[] getStringOptionNameArray() {
            int size = stringOptions.size();
            String[] result = new String[size];
            if (size == 0) {
                return result;
            }
            int p = 0;
            for (String key : stringOptions.keySet()) {
                result[p] = key;
                p++;
            }
            return result;
        }

        boolean setOption(String optionName, Object optionValue) throws FieldOptionException {
            FieldOptions.checkFieldOption(optionName, fieldKey, ficheItemType);
            if (optionName.equals(FieldOptions.INFORMATIONDISPLAY_OPTION)) {
                if (!(optionValue instanceof Short)) {
                    throw new FieldOptionException("optionValue must be a short");
                }
                Short sh = (Short) optionValue;
                return setInformationDisplay(sh.shortValue());
            } else if (optionName.equals(FieldOptions.DEFAULTSPHEREKEY_OPTION)) {
                if (optionValue == null) {
                    if (sphereKey == null) {
                        return false;
                    } else {
                        sphereKey = null;
                        return true;
                    }
                } else {
                    if (!(optionValue instanceof SubsetKey)) {
                        throw new FieldOptionException("optionValue must be a SubsetKey instance");
                    }
                    SubsetKey subsetKey = (SubsetKey) optionValue;
                    if (!subsetKey.isSphereSubset()) {
                        throw new FieldOptionException("optionValue must be a sphere SubsetKey");
                    }
                    if ((sphereKey != null) && (sphereKey.equals(subsetKey))) {
                        return false;
                    }
                    sphereKey = subsetKey;
                    return true;
                }
            } else if (optionName.equals(FieldOptions.SUBFIELDDISPLAY_OPTION)) {
                if (optionValue == null) {
                    if (subfieldDisplay) {
                        subfieldDisplay = false;
                        return true;
                    } else {
                        return false;
                    }
                } else if (optionValue.equals(Boolean.TRUE)) {
                    if (!subfieldDisplay) {
                        subfieldDisplay = true;
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    throw new FieldOptionException("optionValue must be null or equals to Boolean.TRUE");
                }
            } else if (optionName.equals(FieldOptions.TITREGENERATIONFORMAT_OPTION)) {
                if (!fieldKey.equals(FieldKey.TITRE)) {
                    throw new FieldOptionException("titreGenerationPattern Option applies only to Titre Field");
                }
                boolean returnVal = false;
                if (optionValue == null) {
                    if (titreFormatDef != null) {
                        titreFormatDef = null;
                        returnVal = true;
                    }
                } else {
                    if (!(optionValue instanceof BasedefichesFormatDef)) {
                        throw new FieldOptionException("optionValue must be an instance of FicheFormatDef");
                    }
                    BasedefichesFormatDef ficheFormatDef = (BasedefichesFormatDef) optionValue;
                    if (titreFormatDef == null) {
                        titreFormatDef = ficheFormatDef;
                        returnVal = true;
                    } else {
                        if (ficheFormatDef.isSame(titreFormatDef)) {
                            returnVal = false;
                        } else {
                            titreFormatDef = ficheFormatDef;
                            returnVal = true;
                        }
                    }
                }
                return returnVal;
            } else if (optionName.equals(FieldOptions.CURRENCYARRAY_OPTION)) {
                if (optionValue == null) {
                    if (currencyArray == null) {
                        return false;
                    } else {
                        currencyArray = null;
                        return true;
                    }
                }
                if (!(optionValue instanceof int[])) {
                    throw new FieldOptionException("optionValue must be an instance of int[]");
                }
                int[] newValue = (int[]) optionValue;
                int length = newValue.length;
                if (length == 0) {
                    if (currencyArray == null) {
                        return false;
                    } else {
                        currencyArray = null;
                        return true;
                    }
                }
                if ((currencyArray != null) && (currencyArray.length == length)) {
                    boolean returnVal = false;
                    for (int i = 0; i < length; i++) {
                        if (newValue[i] != currencyArray[i]) {
                            returnVal = true;
                        }
                    }
                    if (!returnVal) {
                        return false;
                    }
                }
                currencyArray = new int[length];
                System.arraycopy(newValue, 0, currencyArray, 0, length);
                return true;
            } else if (optionName.equals(FieldOptions.ADDRESSFIELDARRAY_OPTION)) {
                if (optionValue == null) {
                    if (addressFieldsArray == null) {
                        return false;
                    } else {
                        addressFieldsArray = null;
                        return true;
                    }
                }
                if (!(optionValue instanceof CorpusField[])) {
                    throw new FieldOptionException("optionValue must be an instance of CorpusField[]");
                }
                CorpusField[] newValue = (CorpusField[]) optionValue;
                int length = newValue.length;
                if (length == 0) {
                    if (addressFieldsArray == null) {
                        return false;
                    } else {
                        addressFieldsArray = null;
                        return true;
                    }
                }
                if ((addressFieldsArray != null) && (addressFieldsArray.length == length)) {
                    boolean egalite = true;
                    for (int i = 0; i < length; i++) {
                        if (!addressFieldsArray[i].equals(newValue[i])) {
                            egalite = false;
                            break;
                        }
                    }
                    if (egalite) {
                        return false;
                    }
                }
                addressFieldsArray = new CorpusField[length];
                for (int i = 0; i < length; i++) {
                    CorpusField newCorpusField = newValue[i];
                    try {
                        testCorpusField(newCorpusField);
                        addressFieldsArray[i] = newCorpusField;
                    } catch (IllegalArgumentException iae) {
                        throw new FieldOptionException("wrong corpusField in optionValue = " + newCorpusField.getFieldKey().toString());
                    }
                }
                return true;
            } else if (optionName.equals(FieldOptions.ALBUMKEY_OPTION)) {
                if (optionValue == null) {
                    if (albumKey == null) {
                        return false;
                    } else {
                        albumKey = null;
                        return true;
                    }
                } else {
                    if (!(optionValue instanceof SubsetKey)) {
                        throw new FieldOptionException("optionValue must be a SubsetKey instance");
                    }
                    SubsetKey subsetKey = (SubsetKey) optionValue;
                    if (!subsetKey.isAlbumSubset()) {
                        throw new FieldOptionException("optionValue must be a album SubsetKey");
                    }
                    if ((albumKey != null) && (albumKey.equals(subsetKey))) {
                        return false;
                    }
                    albumKey = subsetKey;
                    return true;
                }
            } else {
                if ((optionValue != null) && (!(optionValue instanceof String))) {
                    throw new FieldOptionException("optionValue must be null or a String instance");
                }
                return setStringOption(optionName, (String) optionValue);
            }
        }

        private boolean setStringOption(String optionName, String optionValue) {
            if (optionValue != null) {
                optionValue = optionValue.trim();
                if (optionValue.length() == 0) {
                    optionValue = null;
                }
            }
            if (optionValue == null) {
                if (stringOptions.containsKey(optionName)) {
                    stringOptions.remove(optionName);
                    return true;
                } else {
                    return false;
                }
            }
            String currentValue = stringOptions.get(optionName);
            if (currentValue == null) {
                stringOptions.put(optionName, optionValue);
                return true;
            } else if (currentValue.equals(optionValue)) {
                return false;
            } else {
                stringOptions.put(optionName, optionValue);
                return true;
            }
        }

        public Object getOption(String optionName) {
            if (optionName.equals(FieldOptions.INFORMATIONDISPLAY_OPTION)) {
                return new Short(informationDisplay);
            } else if (optionName.equals(FieldOptions.DEFAULTSPHEREKEY_OPTION)) {
                return sphereKey;
            } else if (optionName.equals(FieldOptions.SUBFIELDDISPLAY_OPTION)) {
                if (subfieldDisplay) {
                    return Boolean.TRUE;
                } else {
                    return null;
                }
            } else if (optionName.equals(FieldOptions.TITREGENERATIONFORMAT_OPTION)) {
                if (fieldKey.equals(FieldKey.TITRE)) {
                    return titreFormatDef;
                } else {
                    return null;
                }
            } else if (optionName.equals(FieldOptions.CURRENCYARRAY_OPTION)) {
                return currencyArray;
            } else if (optionName.equals(FieldOptions.ADDRESSFIELDARRAY_OPTION)) {
                return addressFieldsArray;
            } else if (optionName.equals(FieldOptions.ALBUMKEY_OPTION)) {
                return albumKey;
            } else {
                return stringOptions.get(optionName);
            }
        }

        boolean setInformationDisplay(short informationDisplay) throws FieldOptionException {
            switch (informationDisplay) {
                case FieldOptions.INLINE_DISPLAY:
                case FieldOptions.BLOCK_DISPLAY:
                    if (this.informationDisplay != informationDisplay) {
                        this.informationDisplay = informationDisplay;
                        return true;
                    } else {
                        return false;
                    }
                default:
                    throw new FieldOptionException();
            }
        }

    }


    private class BdfimplCorpusMetadataEditor implements CorpusMetadataEditor {

        public Metadata getMetadata() {
            return BdfimplCorpusMetadata.this;
        }

        public CorpusField createCorpusField(FieldKey fieldKey, short ficheItemType) throws ExistingFieldKeyException {
            if (corpusFieldMap.containsKey(fieldKey)) {
                throw new ExistingFieldKeyException();
            }
            CorpusField corpusField = internalCreateCorpusField(fieldKey, ficheItemType);
            fireChange();
            getListenerList().fireCorpusFieldCreated(corpus, corpusField);
            return corpusField;
        }

        private CorpusField internalCreateCorpusField(FieldKey fieldKey, short ficheItemType) {
            if (!fieldKey.isSpecialFieldKey()) {
                if (!StringUtils.isTechnicalName(fieldKey.getTypeString(), false)) {
                    throw new IllegalArgumentException("fieldKey.getTypeString() is not valid : " + fieldKey.getTypeString());
                }
            }
            BdfimplCorpusField corpusField = null;
            if ((fieldKey.equals(FieldKey.IDCORPUS)) || (fieldKey.equals(FieldKey.TITRE))) {
                corpusField = new BdfimplCorpusField(fieldKey);
            } else if (fieldKey.ownsToCorpsdefiche()) {
                corpusField = new BdfimplCorpusField(fieldKey);
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.PERSONNE_FIELD);
            } else if (fieldKey.equals(FieldKey.LANG)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.LANGUE_FIELD);
            } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.PARA_FIELD);
            } else {
                ficheItemType = CorpusMetadataUtils.checkFicheItem(fieldKey, ficheItemType);
                corpusField = new BdfimplCorpusField(fieldKey, ficheItemType);
            }
            addInList(corpusField);
            corpusFieldMap.put(fieldKey, corpusField);
            return corpusField;
        }

        private void addInList(BdfimplCorpusField corpusField) {
            FieldKey fieldKey = corpusField.getFieldKey();
            if (fieldKey.isAnnexeFieldKey()) {
                annexeFieldList.add(corpusField);
            } else if (fieldKey.isProprieteFieldKey()) {
                proprieteFieldList.add(corpusField);
            } else if (fieldKey.isInformationFieldKey()) {
                informationFieldList.add(corpusField);
            }
        }

        public boolean setFieldLibelle(CorpusField corpusField, String nvlibelle, int langInteger) {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            boolean done = defcorpusfield.internalSetLibelle(nvlibelle, langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            String defaultLib = corpus.getSubsetKey().getSubsetName();
            if (name.equals(CorpusConstants.FICHE_EXISTING)) {
                defaultLib = defaultLib + "/";
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplCorpusMetadata.this, defaultLib);
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            fireChange();
            return intitule;
        }

        public boolean setIntituleLibelle(Intitule intitule, String nvlibelle, int langInteger) {
            BdfimplIntitule defIntitule = BdfimplUtils.testIntitule(BdfimplCorpusMetadata.this, intitule);
            boolean done = defIntitule.internalSetLibelle(nvlibelle, langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean setFieldOption(CorpusField corpusField, String optionName, Object optionValue) throws FieldOptionException {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            boolean done = defcorpusfield.setOption(optionName, optionValue);
            if (done) {
                fireChange();
            }
            return done;
        }

        public void removeCorpusField(CorpusField corpusField) throws MandatoryFieldException {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            FieldKey fieldKey = defcorpusfield.getFieldKey();
            if (fieldKey.isMandatoryFieldKey()) {
                throw new MandatoryFieldException();
            }
            corpusFieldMap.remove(fieldKey);
            if (fieldKey.isProprieteFieldKey()) {
                proprieteFieldList.remove(defcorpusfield);
                if ((geolocalisationField != null) && (geolocalisationField.equals(defcorpusfield))) {
                    geolocalisationField = null;
                }
            } else if (fieldKey.isAnnexeFieldKey()) {
                annexeFieldList.remove(defcorpusfield);
            } else if (fieldKey.isInformationFieldKey()) {
                informationFieldList.remove(defcorpusfield);
            }
            for (BdfimplCorpusField otherCorpusfield : proprieteFieldList) {
                if (otherCorpusfield.addressFieldsArray != null) {
                    int length = otherCorpusfield.addressFieldsArray.length;
                    int index = ArrayUtils.indexOf(otherCorpusfield.addressFieldsArray, otherCorpusfield);
                    if (index != -1) {
                        if (length == 1) {
                            otherCorpusfield.addressFieldsArray = null;
                        } else {
                            otherCorpusfield.addressFieldsArray = (CorpusField[]) ArrayUtils.removeUnique(otherCorpusfield.addressFieldsArray, otherCorpusfield, new CorpusField[length - 1]);
                        }
                    }
                }
            }
            fireChange();
            getListenerList().fireCorpusFieldRemoved(corpus, fieldKey);
        }

        public boolean setGeolocalisationField(CorpusField corpusField) {
            boolean done = false;
            if (corpusField == null) {
                if (geolocalisationField != null) {
                    geolocalisationField = null;
                    done = true;
                }
            } else {
                BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
                FieldKey fieldKey = defcorpusfield.getFieldKey();
                if (!fieldKey.isProprieteFieldKey()) {
                    throw new IllegalArgumentException("not a Propriete : " + fieldKey.toString());
                }
                if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
                    throw new IllegalArgumentException("not a GeoPoint Type : " + corpusField.getFicheItemType());
                }
                if (geolocalisationField == null) {
                    geolocalisationField = defcorpusfield;
                    done = true;
                } else if (!geolocalisationField.equals(defcorpusfield)) {
                    geolocalisationField = defcorpusfield;
                    done = true;
                }
            }
            if (done) {
                fireChange();
            }
            return done;
        }

    }

    private BdfimplCorpusField testCorpusField(CorpusField corpusField) throws IllegalArgumentException {
        if (!(corpusField instanceof BdfimplCorpusField)) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        BdfimplCorpusField def = (BdfimplCorpusField) corpusField;
        if (def.getCorpusMetadata() != this) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        return def;
    }

}
