/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.AssociationUtils;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.DocumentSubset;
import net.mapeadores.basedefiches.association.DocumentSubsetEditor;
import net.mapeadores.basedefiches.association.ExistingBasenameException;
import net.mapeadores.basedefiches.association.ExistingVersionException;
import net.mapeadores.basedefiches.association.FileLength;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.association.VersionName;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplDocumentSubset extends AbstractIndexAndIntKeyMap implements DocumentSubset {

    private BdfimplBasedefiches basedefiches;
    private int availableIdsubset = 1;
    private Map<String, BdfimplDocument> documentByBasenameMap = new HashMap<String, BdfimplDocument>();
    BdfimplDocumentSubsetEditor bdfimplDocumentSubsetEditor = new BdfimplDocumentSubsetEditor();
    DocumentDataSource documentDataSource;

    public BdfimplDocumentSubset(BdfimplBasedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    protected void setDocumentDataSource(DocumentDataSource documentDataSource) {
        this.documentDataSource = documentDataSource;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public Document getDocumentByBasename(String basename) {
        return documentByBasenameMap.get(basename);
    }

    public SubsetKey getSubsetKey() {
        return SubsetKey.DOCUMENT_SUBSET;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    public Subset getMasterSubset() {
        return null;
    }

    public Subset[] getSlaveSubsetArray() {
        return null;
    }

    protected DocumentSubsetEditor getDocumentSubsetEditor() {
        return bdfimplDocumentSubsetEditor;
    }

    protected void fireDocumentSubsetChange() {
        basedefiches.fireSubsetChange(SubsetKey.DOCUMENT_SUBSET);
    }


    private class BdfimplDocumentSubsetEditor implements DocumentSubsetEditor {

        public boolean removeDocument(Document document) {
            BdfimplDocument bdfimpldoc = testDocument(document);
            if (CroisementUtils.hasAssociation(document)) {
                return false;
            }
            boolean done = documentDataSource.deleteDocument(bdfimpldoc.getIdsubset());
            if (!done) {
                return false;
            }
            documentByBasenameMap.remove(bdfimpldoc.getBasename());
            BdfimplDocumentSubset.this.disable(bdfimpldoc.getTicket());
            fireDocumentSubsetChange();
            return true;
        }

        public DocumentSubset getDocumentSubset() {
            return BdfimplDocumentSubset.this;
        }

        public boolean removeVersion(Version version) {
            BdfimplDocument.BdfimplVersion bdfimplversion = testVersion(version);
            boolean done = bdfimplversion.delete();
            return done;
        }

        public Document createDocument(int idsubset, String[] extensionArray, long[] lengthArray) throws ExistingIdException {
            if (getValueByKey(idsubset) != null) {
                throw new ExistingIdException();
            }
            availableIdsubset = Math.max(availableIdsubset, idsubset + 1);
            BdfimplDocument document = new BdfimplDocument(idsubset);
            document.ticket = putFromKey(idsubset, document);
            documentByBasenameMap.put(document.getBasename(), document);
            int vc = extensionArray.length;
            for (int j = 0; j < vc; j++) {
                document.addVersion(extensionArray[j], lengthArray[j]);
            }
            fireDocumentSubsetChange();
            return document;
        }

        public Document createDocument(VersionName versionName) throws ExistingBasenameException {
            String basename = versionName.getBasename();
            if (documentByBasenameMap.containsKey(basename)) {
                throw new ExistingBasenameException();
            }
            BdfimplDocument document = new BdfimplDocument(availableIdsubset);
            document.ticket = putFromKey(availableIdsubset, document);
            document.setBasename(versionName.getBasename());
            documentByBasenameMap.put(document.getBasename(), document);
            document.addVersion(versionName.getExtension(), -1);
            availableIdsubset++;
            fireDocumentSubsetChange();
            return document;
        }

        public boolean setBasename(Document document, String newName) throws ExistingBasenameException, ParseException {
            BdfimplDocument bdfimpldoc = testDocument(document);
            String oldName = bdfimpldoc.getBasename();
            if (oldName.equals(newName)) {
                return false;
            }
            if (documentByBasenameMap.containsKey(newName)) {
                throw new ExistingBasenameException();
            }
            AssociationUtils.testBasename(newName);
            documentByBasenameMap.remove(oldName);
            bdfimpldoc.setBasename(newName);
            documentByBasenameMap.put(newName, bdfimpldoc);
            fireDocumentSubsetChange();
            return true;
        }

        public Version createVersion(Document document, String extension) throws ExistingVersionException {
            BdfimplDocument bdfimpldoc = testDocument(document);
            if (document.getVersionByExtension(extension) != null) {
                throw new ExistingVersionException();
            }
            BdfimplDocument.BdfimplVersion version = bdfimpldoc.addVersion(extension, -1);
            return version;
        }

        public boolean updateVersion(Version version, InputStream inputStream) throws IOException {
            BdfimplDocument.BdfimplVersion bdfimplversion = testVersion(version);
            return bdfimplversion.update(inputStream);
        }

    }


    private class BdfimplDocument implements Document {

        int idsubset;
        int ticket;
        VersionName emptyVersionName;
        List<BdfimplVersion> versionList = new ArrayList<BdfimplVersion>();

        BdfimplDocument(int idsubset) {
            this.idsubset = idsubset;
            emptyVersionName = VersionName.getExtensionEmptyInstance(idsubset);
        }

        public DocumentSubset getDocumentSubset() {
            return BdfimplDocumentSubset.this;
        }

        public Subset getSubset() {
            return BdfimplDocumentSubset.this;
        }

        public int getIdsubset() {
            return idsubset;
        }

        public int getTicket() {
            return ticket;
        }

        public String getBasename() {
            return emptyVersionName.getBasename();
        }

        public int getVersionCount() {
            return versionList.size();
        }

        public Version getVersion(int index) {
            return versionList.get(index);
        }

        public Version getVersionByExtension(String extension) {
            int size = versionList.size();
            for (int i = 0; i < size; i++) {
                Version version = versionList.get(i);
                if (version.getVersionName().getExtension().equals(extension)) {
                    return version;
                }
            }
            return null;
        }

        void setBasename(String basename) {
            this.emptyVersionName = VersionName.getExtensionEmptyInstance(basename);
            for (int i = 0; i < versionList.size(); i++) {
                BdfimplVersion bdfimplVersion = versionList.get(i);
                bdfimplVersion.updateVersionName();
            }
        }

        /*
         *@throws IllegalArgumentException
         */
        BdfimplVersion addVersion(String extension, long filelength) {
            VersionName newVersion = emptyVersionName.derive(extension);
            BdfimplVersion version = new BdfimplVersion(versionList.size(), newVersion, filelength);
            versionList.add(version);
            return version;
        }


        private class BdfimplVersion implements Version {

            int versionIndex;
            FileLength fileLength;
            VersionName versionName;

            BdfimplVersion(int versionIndex, VersionName versionName, long filelength) {
                this.versionName = versionName;
                this.versionIndex = versionIndex;
                if (filelength == -1) {
                    byte[] buf = new byte[0];
                    update(new ByteArrayInputStream(buf));
                } else {
                    setFileLength(filelength);
                }
            }

            boolean update(InputStream inputStream) {
                if (versionIndex == -1) {
                    return false;
                }
                long filelength = documentDataSource.saveVersion(idsubset, versionName.getExtension(), inputStream);
                setFileLength(filelength);
                return true;
            }

            boolean delete() {
                if (versionList.size() == 1) {
                    return false;
                }
                if (versionIndex == -1) {
                    return false;
                }
                documentDataSource.deleteVersion(idsubset, versionName.getExtension());
                versionList.remove(versionIndex);
                versionIndex = -1;
                return true;
            }

            void updateVersionName() {
                versionName = emptyVersionName.derive(versionName);
            }

            public Document getDocument() {
                return BdfimplDocument.this;
            }

            public FileLength getFileLength() {
                return fileLength;
            }

            void setFileLength(long filelength) {
                fileLength = new FileLength(filelength);
            }

            public VersionName getVersionName() {
                return versionName;
            }

            public InputStream getInputStream() {
                return documentDataSource.getInputStream(idsubset, versionName.getExtension());
            }

            public boolean linkTo(File file, boolean symbolicLink) throws IOException {
                return documentDataSource.linkTo(idsubset, versionName.getExtension(), file, symbolicLink);
            }

        }

    }

    private BdfimplDocument.BdfimplVersion testVersion(Version version) {
        if (version == null) {
            throw new IllegalArgumentException("version argument cannot be null");
        }
        try {
            BdfimplDocument.BdfimplVersion bdfimpl = (BdfimplDocument.BdfimplVersion) version;
            if (bdfimpl.getDocument().getDocumentSubset() != this) {
                throw new IllegalArgumentException("version argument does not come from this documentSubse");
            }
            return bdfimpl;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("version argument does not come from this documentSubset");
        }
    }

    private BdfimplDocument testDocument(Document document) {
        if (document == null) {
            throw new IllegalArgumentException("document argument cannot be null");
        }
        try {
            BdfimplDocument bdfimpl = (BdfimplDocument) document;
            if (bdfimpl.getDocumentSubset() != this) {
                throw new IllegalArgumentException("document argument does not come from this documentSubse");
            }
            return bdfimpl;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("document argument does not come from this documentSubset");
        }
    }

}
