/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.text.ParseException;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Idsphere;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadata;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadataEditor;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplSphere extends AbstractIndexAndIntKeyMap implements Sphere {

    SubsetKey sphereKey;
    SphereEditor sphereEditor = new DefaultSphereEditor();
    BdfimplBasedefiches basedefiches;
    BdfimplSphereMetadata sphereMetadata;

    public BdfimplSphere(SubsetKey sphereKey, BdfimplBasedefiches basedefiches) {
        this.sphereKey = sphereKey;
        this.basedefiches = basedefiches;
        sphereMetadata = new BdfimplSphereMetadata(this);
    }

    public SubsetKey getSubsetKey() {
        return sphereKey;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    protected SphereEditor getSphereEditor() {
        return sphereEditor;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public SphereMetadata getSphereMetadata() {
        return sphereMetadata;
    }

    public Redacteur getRedacteurByIdsphere(int idsphere) {
        return (Redacteur) getValueByKey(idsphere);
    }

    public Redacteur getRedacteurByIdsphere(String idsphereString) {
        try {
            return (Redacteur) getValueByKey(Idsphere.parse(idsphereString));
        } catch (ParseException pe) {
            return null;
        }
    }

    public Subset getMasterSubset() {
        return null;
    }

    public Subset[] getSlaveSubsetArray() {
        return null;
    }

    protected void fireSphereChange() {
        basedefiches.fireSubsetChange(sphereKey);
    }

    protected void disable() {
        sphereMetadata.disable();
    }

    protected ListenerList getListenerList() {
        return basedefiches.getListenerList();
    }


    private class BdfimplRedacteur implements Redacteur {

        int ticket;
        RedacteurKey redacteurKey;
        PersonneBean personneBean = new PersonneBean();
        CourrielBean courrielBean;
        boolean actif = true;

        BdfimplRedacteur(int idsphere) {
            try {
                this.redacteurKey = RedacteurKey.parse(sphereKey.getSubsetName() + "_" + Idsphere.toString(idsphere));
            } catch (java.text.ParseException pe) {
                throw new ShouldNotOccurException();
            }
        }

        public int getIdsphere() {
            return redacteurKey.getIdsphere();
        }

        public Sphere getSphere() {
            return BdfimplSphere.this;
        }

        public Subset getSubset() {
            return BdfimplSphere.this;
        }

        public int getIdsubset() {
            return getIdsphere();
        }

        public int getTicket() {
            return ticket;
        }

        public boolean isActif() {
            return actif;
        }

        private boolean setActif(boolean newActif) {
            if (newActif == actif) {
                return false;
            }
            actif = newActif;
            return true;
        }

        private boolean setPersonneBean(PersonneBean pb) {
            if (pb == null) {
                throw new IllegalArgumentException("personneBean argument is null");
            }
            if (!(pb.equals(personneBean))) {
                PersonneBean oldPersonneBean = personneBean;
                personneBean = new PersonneBean(pb);
                CourrielBean oldCourrielBean = null;
                if (courrielBean != null) {
                    String oldNomComplet = oldPersonneBean.getNomComplet(PersonneBean.NORMAL_STYLE);
                    if (courrielBean.getNomComplet().equals(oldNomComplet)) {
                        String newNomComplet = personneBean.getNomComplet(PersonneBean.NORMAL_STYLE);
                        if (!newNomComplet.equals(oldNomComplet)) {
                            oldCourrielBean = courrielBean;
                            courrielBean = new CourrielBean(oldCourrielBean);
                            courrielBean.setNomComplet(newNomComplet);
                        }
                    }
                }
                return true;
            } else {
                return false;
            }
        }

        private boolean setCourrielBean(CourrielBean cb) {
            CourrielBean oldBean = null;
            boolean done = false;
            if (cb == null) {
                if (courrielBean != null) {
                    oldBean = courrielBean;
                    courrielBean = null;
                    done = true;
                }
            } else {
                if (courrielBean == null) {
                    courrielBean = new CourrielBean(cb);
                    if (courrielBean.getNomComplet().length() == 0) {
                        courrielBean.setNomComplet(personneBean.getNomComplet(PersonneBean.NORMAL_STYLE));
                    }
                    done = true;
                } else if (!cb.equals(courrielBean)) {
                    oldBean = courrielBean;
                    courrielBean = new CourrielBean(cb);
                    if (courrielBean.getNomComplet().length() == 0) {
                        courrielBean.setNomComplet(personneBean.getNomComplet(PersonneBean.NORMAL_STYLE));
                    }
                    done = true;
                }
            }
            return done;
        }

        public RedacteurKey getRedacteurKey() {
            return redacteurKey;
        }

        public CourrielBean getCourrielBean() {
            if (courrielBean == null) {
                return null;
            }
            return new CourrielBean(courrielBean);
        }

        public PersonneBean getPersonneBean() {
            return new PersonneBean(personneBean);
        }

    }


    private class DefaultSphereEditor implements SphereEditor {

        public Redacteur createRedacteur(int idsphere) throws ExistingIdException {
            if (getIndexByKey(idsphere) != -1) {
                throw new ExistingIdException();
            }
            BdfimplRedacteur defredac = new BdfimplRedacteur(idsphere);
            defredac.ticket = putFromKey(idsphere, defredac);
            fireSphereChange();
            return defredac;
        }

        public Sphere getSphere() {
            return BdfimplSphere.this;
        }

        public SphereMetadataEditor getSphereMetadataEditor() {
            return sphereMetadata.getSphereMetadataEditor();
        }

        public boolean removeRedacteur(Redacteur redacteur) {
            if (!basedefiches.isRemoveable(redacteur)) {
                return false;
            }
            BdfimplRedacteur defredac = testRedacteur(redacteur);
            RedacteurKey redacteurKey = defredac.getRedacteurKey();
            boolean done = removeFromKey(defredac.getIdsphere());
            if (done) {
                fireSphereChange();
                getListenerList().fireRedacteurRemoved(redacteurKey);
            }
            return done;
        }

        public boolean setCourrielBean(Redacteur redacteur, CourrielBean courrielBean) {
            BdfimplRedacteur defredac = testRedacteur(redacteur);
            boolean done = defredac.setCourrielBean(courrielBean);
            if (done) {
                fireSphereChange();
            }
            return done;
        }

        public boolean setPersonneBean(Redacteur redacteur, PersonneBean personneBean) {
            BdfimplRedacteur defredac = testRedacteur(redacteur);
            boolean done = defredac.setPersonneBean(personneBean);
            if (done) {
                fireSphereChange();
            }
            return done;
        }

        public boolean setActif(Redacteur redacteur, boolean actif) {
            BdfimplRedacteur defredac = testRedacteur(redacteur);
            boolean done = defredac.setActif(actif);
            if (done) {
                fireSphereChange();
            }
            return done;
        }

    }

    private BdfimplRedacteur testRedacteur(Redacteur redacteur) {
        if (redacteur == null) {
            throw new IllegalArgumentException("redacteur argument cannot be null");
        }
        try {
            BdfimplRedacteur defredacteur = (BdfimplRedacteur) redacteur;
            if (defredacteur.getSphere() != this) {
                throw new IllegalArgumentException("redacteur argument does not come from this sphere");
            }
            return defredacteur;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("redacteur argument does not come from this sphere");
        }
    }

}
