/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadataEditor;
import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplThesaurusMetadata implements ThesaurusMetadata {

    private BdfimplThesaurus thesaurus;
    private BdfimplThesaurusMetadataEditor thesaurusMetadataEditor = new BdfimplThesaurusMetadataEditor();
    private int[] langIntArray;
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();

    public BdfimplThesaurusMetadata(BdfimplThesaurus thesaurus) {
        this.thesaurus = thesaurus;
        if (thesaurus.thesaurusType != BABELIEN_TYPE) {
            langIntArray = new int[1];
            langIntArray[0] = LangInteger.FRENCH;
        }
    }

    public Thesaurus getThesaurus() {
        return thesaurus;
    }

    public short getThesaurusType() {
        return thesaurus.thesaurusType;
    }

    public int[] getAuthorizedLangIntegerArray() {
        if (langIntArray == null) {
            return null;
        }
        int length = langIntArray.length;
        int[] result = new int[length];
        System.arraycopy(langIntArray, 0, result, 0, length);
        return result;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    protected ThesaurusMetadataEditor getThesaurusMetadataEditor() {
        return thesaurusMetadataEditor;
    }

    protected void disable() {
        intituleMap.clear();
        intituleList.clear();
    }


    private class BdfimplThesaurusMetadataEditor implements ThesaurusMetadataEditor {

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplThesaurusMetadata.this, thesaurus.getSubsetKey().toString());
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            thesaurus.fireThesaurusChange();
            return intitule;
        }

        public Metadata getMetadata() {
            return BdfimplThesaurusMetadata.this;
        }

        public boolean setAuthorizedLangIntegerArray(int[] array) {
            if (thesaurus.thesaurusType == BABELIEN_TYPE) {
                throw new UnsupportedOperationException("setAuthorizedLangIntegerList() cannot be invoked when getThesaurusType() == BABELIEN ");
            }
            if (array == null) {
                throw new IllegalArgumentException("intList == null");
            }
            if (array.length == 0) {
                throw new IllegalArgumentException("intList.size() == 0");
            }
            int[] cleanCopy = ArrayUtils.cleanCopy(array);
            boolean done = !ArrayUtils.compare(cleanCopy, langIntArray);
            if (done) {
                langIntArray = cleanCopy;
                thesaurus.fireThesaurusChange();
            }
            return done;
        }

        public boolean setIntituleLibelle(Intitule intitule, String nvlibelle, int langInteger) {
            BdfimplIntitule bti = BdfimplUtils.testIntitule(BdfimplThesaurusMetadata.this, intitule);
            boolean done = bti.internalSetLibelle(nvlibelle, langInteger);
            if (done) {
                thesaurus.fireThesaurusChange();
            }
            return done;
        }

    }

}
