/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.Libelle;




/**
 *
 * @author Vincent Calame
 */
public class MultiMotcle extends AbstractMotcle {
    private LinkedIntHashMap langIntMap = new LinkedIntHashMap();
    
    public MultiMotcle(BdfimplThesaurus thesaurus,int idths) {
        super(thesaurus,idths);
    }

    public Libelle getLibelleByLang(int langInteger) {
       return getInternalLibelle(langInteger);
    }

    public int getLibelleCount() {
        return langIntMap.size();
    }

    public Libelle getLibelleByIndex(int i) {
        return (Libelle) langIntMap.getValueAt(i);
    }

    public String getLibelleString(int langInteger, boolean defaut) {
        InternalMotcleLibelle libelle = getInternalLibelle(langInteger);
        if (libelle != null) return libelle.getLibelleString();
        if (defaut) {
            int[] langArray = this.getThesaurus().getThesaurusMetadata().getAuthorizedLangIntegerArray();
            for (int i = 0; i < langArray.length; i++) {
                int lg2 = langArray[i];
                if (lg2 != langInteger) {
                    libelle = getInternalLibelle(lg2);
                    if (libelle != null) return libelle.getLibelleString();
                }
            }
            return "?";
        }
        else return null;
    }
    
    boolean setLibelle(String libelleString, int langInteger) {
        InternalMotcleLibelle libelle = getInternalLibelle(langInteger);
        if (libelleString == null) {
            if (libelle == null) return false;
            else {
                langIntMap.remove(langInteger);
                return true;
            }
        }
        else {
            if (libelle == null) {
                libelle = new InternalMotcleLibelle();
                libelle.setLangInteger(langInteger);
                libelle.setLibelleString(libelleString);
                langIntMap.put(langInteger, libelle);
                return true;
            }
            else {
                boolean done = false;;
                if (libelle.setLangInteger(langInteger)) done = true;
                if (libelle.setLibelleString(libelleString)) done = true;
                return done;
            }
        }
    }
    
    InternalMotcleLibelle getInternalLibelle(int langInteger) {
        return (InternalMotcleLibelle) langIntMap.get(langInteger);
    }
    
    class InternalMotcleLibelle implements Libelle {
        int langInteger;
        String libelleString;

        public int getLangInteger() {
            return langInteger;
        }

        public String getLibelleString() {
            return libelleString;
        }
        
        boolean setLangInteger(int newLangInteger) {
            if (newLangInteger == langInteger) return false;
            langInteger = newLangInteger;
            return true;
        }
        
        boolean setLibelleString(String newLibelleString) {
            if (newLibelleString.equals(libelleString)) return false;
            libelleString = newLibelleString;
            return true;
        }
    }
    

}
