/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class UniqueCroisementItemProvider extends CroisementItemProvider {

    private ImplCroisementItem[] croisementItemArray = new ImplCroisementItem[64];
    private SubsetKey subsetKey;

    public UniqueCroisementItemProvider(SubsetKey subsetKey) {
        this.subsetKey = subsetKey;
    }

    public ImplCroisementItem getCroisementItem(SubsetKey subsetKey, int ticket) {
        return getCroisementItem(croisementItemArray, ticket);
    }

    public ImplCroisementItem getOrCreateCroisementItem(SubsetKey subsetKey, int ticket) {
        croisementItemArray = testSize(croisementItemArray, ticket);
        return getOrCreateCroisementItem(croisementItemArray, ticket);
    }

    public boolean isTicketForbidden() {
        return true;
    }

    public SubsetKey getOtherSubsetKey(SubsetKey subsetKey) {
        return subsetKey;
    }

    public CroisementItemProviderBuilder getCroisementItemProviderBuilder(Basedefiches basedefiches) {
        Subset subset = basedefiches.getSubset(subsetKey);
        return new UniqueCroisementItemProviderBuilder(subset);
    }


    private class UniqueCroisementItemProviderBuilder extends CroisementItemProviderBuilder {

        private Subset subset;
        private Map<Integer, CroisementItemBuilder> map = new HashMap<Integer, CroisementItemBuilder>();

        UniqueCroisementItemProviderBuilder(Subset subset) {
            this.subset = subset;
        }

        public CroisementItemBuilder getCroisementItemBuilder1(int ticket1) {
            return getCroisementItemBuilder(map, ticket1);
        }

        public CroisementItemBuilder getCroisementItemBuilder2(int ticket2) {
            return getCroisementItemBuilder(map, ticket2);
        }

        public int getTicket1(int idsubset1) {
            return getTicket(subset, idsubset1);
        }

        public int getTicket2(int idsubset2) {
            return getTicket(subset, idsubset2);
        }

        public void build() {
            int max = getMaxTicket(map);
            croisementItemArray = new ImplCroisementItem[max + 1];
            for (CroisementItemBuilder croisementItemBuilder : map.values()) {
                int ticket = croisementItemBuilder.ticket;
                ImplCroisementItem croisementItem = croisementItemBuilder.build();
                croisementItemArray[ticket] = croisementItem;
            }
        }

    }

}
