/* BasedefichesLib_Tools - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.misc;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.tableexport.thesaurus.ThesaurusTableExport;
import net.mapeadores.basedefiches.tableexport.thesaurus.ThesaurusTableWriter;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.sxio.SXContentSource;
import net.mapeadores.util.sxio.SXIO;
import net.mapeadores.util.sxio.SXIOException;
import net.mapeadores.util.sxio.SXUtils;
import net.mapeadores.util.sxio.StringSXStylesSource;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusOOWriter {

    private ThesaurusOOWriter() {
    }

    public static void exportSXC(OutputStream outputStream, ThesaurusTableExport thesaurusTableExport) throws IOException {
        ThesaurusContentSource thesaurusContentSource = new ThesaurusContentSource(thesaurusTableExport);
        try {
            SXIO.write(outputStream, thesaurusContentSource, new StringSXStylesSource("<office:style></office:style>"));
        } catch (SXIOException sxioe) {
        }
    }


    static class ThesaurusContentSource implements SXContentSource {

        ThesaurusTableExport thesaurusTableExport;
        SubsetKey thesaurusKey;
        Thesaurus thesaurus;
        boolean withIdalpha;
        boolean isBabelien;
        int[] codeslangues;
        Croisement[] croisements;
        List motcleList;

        ThesaurusContentSource(ThesaurusTableExport thesaurusTableExport) throws IOException {
            this.thesaurusTableExport = thesaurusTableExport;
            this.thesaurus = thesaurusTableExport.getThesaurus();
            thesaurusKey = thesaurus.getSubsetKey();
            ThesaurusMetadata thesaurusMetaData = thesaurus.getThesaurusMetadata();
            short thesaurusType = thesaurusMetaData.getThesaurusType();
            withIdalpha = (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE);
            isBabelien = (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE);
            if (!isBabelien) {
                codeslangues = thesaurusMetaData.getAuthorizedLangIntegerArray();
            }
            croisements = CroisementUtils.getCroisementWithCorpusArray(thesaurus);
        }

        public void writeContent(OutputStream outputStream) throws SXIOException {
            try {
                BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                /*bufwriter.write("<office:automatic-styles>");
                /*bufwriter.write("<number:date-style style:name=\"i0\" style:family=\"data-style\" number:automatic-order=\"true\">");
                bufwriter.newLine();
                bufwriter.write("<number:year number:style=\"long\"/><number:text>-</number:text><number:month number:style=\"long\"/><number:text>-</number:text><number:day number:style=\"long\"/>");
                bufwriter.newLine();
                bufwriter.write("</number:date-style>");
                bufwriter.newLine();
                bufwriter.write("<style:style style:name=\"c0\" style:family=\"table-cell\" style:parent-style-name=\"Default\" style:data-style-name=\"i0\"/>");
                bufwriter.newLine();
                bufwriter.write("</office:automatic-styles>");*/
                bufwriter.write("<office:body>");
                bufwriter.newLine();
                writeThesaurusTable(bufwriter);
                bufwriter.write("</office:body>");
                bufwriter.newLine();
                bufwriter.flush();
            } catch (IOException ioe) {
                throw new SXIOException(ioe);
            }

        }

        void writeThesaurusTable(BufferedWriter bufwriter) throws IOException {
            bufwriter.write("<table:table table:name=\"");
            XMLUtils.writeEscape(thesaurusKey.getSubsetName(), bufwriter);
            bufwriter.write("\">");
            bufwriter.newLine();

            bufwriter.write("<table:table-row>");
            bufwriter.newLine();

            ThesaurusFieldKey[] array = thesaurusTableExport.getThesaurusFieldKeyArray();
            int length = array.length;
            for (int i = 0; i < length; i++) {
                SXUtils.writeStringCell(bufwriter, array[i].toString());
            }

            /*for (int i = 0; i < croisements.length;i++) {
            SXUtils.writeStringCell(bufwriter, croisements[i].getSubsetCouple().getSubsetKey1().getSubId());
            }*/
            bufwriter.write("</table:table-row>");

            OOThesaurusTableWriter wrt = new OOThesaurusTableWriter(bufwriter);
            thesaurusTableExport.export(wrt);

            bufwriter.write("</table:table>");
        }

        /* void writeCroisementTable(BufferedWriter bufwriter,Croisement crx) throws IOException {
        bufwriter.write("<table:table table:name=\"");
        XMLUtils.escapeXML(crx.getSubsetCouple().getSubsetKey1().getSubId(),bufwriter);
        bufwriter.write("_");
        XMLUtils.escapeXML(thesaurusKey.getSubId(),bufwriter);
        bufwriter.write("\">");
        bufwriter.write("<table:table-row>");
        SXUtils.writeStringCell(bufwriter,"idcorpus");
        SXUtils.writeStringCell(bufwriter,"idths");
        if (withIdalpha) {
        SXUtils.writeStringCell(bufwriter,"idalpha");
        }
        SXUtils.writeStringCell(bufwriter,"poids");
        bufwriter.write("</table:table-row>");

        Subset subset =
        for (Iterator it = motcleList.iterator();it.hasNext();) {
        Motcle mtcl = (Motcle) it.next();
        String idalpha = null;
        if (withIdalpha) {
        idalpha = mtcl.getIdalpha();
        }
        CroisementItem croisementItem = crx.getCroisementItem(thesaurusKey,mtcl.getTicket());
        int ciCount = croisementItem.getCount();
        for (int i = 0; i < ciCount;i++) {
        int poids = croisementItem.getPoids(i);
        writeCrossingRow(bufwriter,mtcl.getIdths(), longArray[i],poids,idalpha);
        }
        }
        bufwriter.write("</table:table>");
        }
        
        void writeCrossingRow(BufferedWriter bufwriter, int idths, long l, String idalpha) throws IOException {
        bufwriter.write("<table:table-row>");
        SXUtils.writeIntCell(bufwriter, PrimUtils.getInt1(l));
        SXUtils.writeIntCell(bufwriter, idths);
        if (idalpha != null) {
        SXUtils.writeStringCell(bufwriter,idalpha);
        }
        SXUtils.writeIntCell(bufwriter, PrimUtils.getInt2(l));
        bufwriter.write("</table:table-row>");
        }*/
        public String getEncoding() {
            return "UTF-8";
        }

        public String getFormatName() {
            return "sxc";
        }

        public boolean needDeclaration() {
            return true;
        }

        public boolean needRootElement() {
            return true;
        }


        class OOThesaurusTableWriter implements ThesaurusTableWriter {

            BufferedWriter bufwriter;

            OOThesaurusTableWriter(BufferedWriter bufwriter) {
                this.bufwriter = bufwriter;
            }

            public void startRow(Motcle motcle) {
                try {
                    bufwriter.write("<table:table-row>");
                } catch (IOException ioe) {
                }
            }

            public void addIdthsCell(ThesaurusFieldKey thesaurusFieldKey, Integer itg) {
                try {
                    if (itg != null) {
                        SXUtils.writeIntCell(bufwriter, itg.intValue());
                    } else {
                        SXUtils.writeEmptyCell(bufwriter);
                    }
                } catch (IOException ioe) {
                }
            }

            public void addIdalphaCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
                try {
                    if (s != null) {
                        SXUtils.writeStringCell(bufwriter, s);
                    } else {
                        SXUtils.writeEmptyCell(bufwriter);
                    }
                } catch (IOException ioe) {
                }
            }

            public void addLibCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
                try {
                    if (s != null) {
                        SXUtils.writeStringCell(bufwriter, s);
                    } else {
                        SXUtils.writeEmptyCell(bufwriter);
                    }
                } catch (IOException ioe) {
                }
            }

            public void addBabelienLangCell(int langInteger) {
                try {
                    String lg = LangInteger.toString(langInteger);
                    SXUtils.writeStringCell(bufwriter, lg);
                } catch (IOException ioe) {
                }
            }

            public void endRow() {
                try {
                    /*for (int i = 0; i < croisements.length;i++) {
                    int total = croisements[i].getCrossingCount(thesaurusKey, motcle.getTicket());
                    if (total > 0) {
                    SXUtils.writeIntCell(bufwriter,total);
                    } else {
                    SXUtils.writeEmptyCell(bufwriter);
                    }
                    }*/

                    bufwriter.write("</table:table-row>");
                } catch (IOException ioe) {
                }
            }

        }

    }

    static void writeIntLangCell(Writer writer, String prefixe, int intlang) throws IOException {
        writer.write("<table:table-cell><text:p>");
        if ((prefixe != null)) {
            XMLUtils.writeEscape(prefixe, writer);
        }
        LangInteger.writeString(writer, intlang);
        writer.write("</text:p></table:table-cell>");
    }

}
