/* BasedefichesLib_Tools - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.scrutariexport;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesConstants;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.basedefiches.include.IncludeLibelleManager;
import net.mapeadores.basedefiches.include.IncludeLibelleMap;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.patterns.MotcleValueResolver;
import net.mapeadores.basedefiches.plainexport.PlainConverter;
import net.mapeadores.basedefiches.plainexport.SeparatorOptionsBean;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.scrutariexport.CorpusExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExport;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusExportDef;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusConstants;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.ValueResolver;
import net.mapeadores.util.xml.UtilXMLWriter;
import net.scrutari.dataexport.ScrutariDataExportFactory;
import net.scrutari.dataexport.api.BaseMetadataExport;
import net.scrutari.dataexport.api.CorpusMetadataExport;
import net.scrutari.dataexport.api.FicheExport;
import net.scrutari.dataexport.api.MotcleExport;
import net.scrutari.dataexport.api.ScrutariDataExport;
import net.scrutari.dataexport.api.ThesaurusMetadataExport;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportXMLWriter extends UtilXMLWriter {

    private ScrutariExport rchfExport;
    private int[] workingLangArray;
    private AbregeFicheSortedList abregeFicheSortedList;
    private LocalisationInfo localisationInfo;
    private PlainConverter plainConverter;
    private SeparatorOptionsBean separatorOptionsBean;
    private ThesaurusXMLWriter thesaurusXMLWriter;
    private FicheValueResolver fichePatternValueSet = new FicheValueResolver();
    private IncludeLibelleManager libelleSource;
    private Basedefiches basedefiches;
    private ScrutariDataExport scrutariDataExport;

    public ScrutariExportXMLWriter(Basedefiches basedefiches, Appendable appendable, int nbredetabulations, IncludeLibelleManager libelleSource) {
        super();
        setAppendable(appendable);
        setIndentLength(nbredetabulations);
        this.basedefiches = basedefiches;
        separatorOptionsBean = new SeparatorOptionsBean();
        separatorOptionsBean.setSeparator(',');
        separatorOptionsBean.setLastInclude(false);
        separatorOptionsBean.setSpaceInclude(true);
        this.libelleSource = libelleSource;
        //scrutariDataExport = ScrutariDataExportFactory.newInstance(appendable, nbredetabulations, true);
        scrutariDataExport = ScrutariDataExportFactory.newInstance(appendable, 0, false);
    }

    public void addBase(ScrutariExport rchfExport, int[] workingLangArray) throws IOException {
        this.rchfExport = rchfExport;
        this.workingLangArray = workingLangArray;
        this.localisationInfo = new LocalisationInfo(workingLangArray[0]);
        plainConverter = new PlainConverter(separatorOptionsBean, localisationInfo, rchfExport.getBasedefiches());
        FicheSelectOptions ficheSelectOptions = rchfExport.getFicheSelectOptions();
        FicheSelector ficheSelector = new FicheSelector(basedefiches, workingLangArray[0]);
        ficheSelector.addFicheSelectOptions(ficheSelectOptions);
        abregeFicheSortedList = SelectionUtils.getAbregeFicheSortedList(ficheSelector, AbregeFicheSortedList.IDCORPUS_SORTTYPE);
        this.thesaurusXMLWriter = new ThesaurusXMLWriter();

        BaseMetadataExport basemetaDataExport = scrutariDataExport.startExport();

        addBaseMetadata(basemetaDataExport);
        addCorpusList();
        if (thesaurusXMLWriter.hasThesaurus()) {
            thesaurusXMLWriter.addThesaurusList();
        }

        scrutariDataExport.endExport();
    }

    private void addBaseMetadata(BaseMetadataExport basemetaDataExport) throws IOException {
        BasedefichesMetadata basedefichesMetadata = basedefiches.getBasedefichesMetadata();
        basemetaDataExport.setAuthorityUUID(basedefichesMetadata.getAuthorityUUID());
        String baseName = rchfExport.getBaseName();
        if (baseName == null) {
            baseName = basedefichesMetadata.getBasedefichesName();
        }
        basemetaDataExport.setBaseName(baseName);
        basemetaDataExport.setBaseIcon(rchfExport.getBaseIcon());
        addIntitule(basemetaDataExport, basedefichesMetadata, BasedefichesConstants.SHORT, BaseMetadataExport.INTITULE_SHORT);
        addIntitule(basemetaDataExport, basedefichesMetadata, BasedefichesConstants.LONG, BaseMetadataExport.INTITULE_LONG);
        int[] array = rchfExport.getLangUiArray();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                basemetaDataExport.addLangUI(LangInteger.toString(array[i]));
            }
        }

    }

    private void addIntitule(BaseMetadataExport basemetaDataExport, BasedefichesMetadata basedefichesMetadata, String name, int intituleType) {
        Intitule intitule = basedefichesMetadata.getIntituleByName(name);
        if (intitule != null) {
            int length = workingLangArray.length;
            boolean done = false;
            for (int i = 0; i < length; i++) {
                int langInteger = workingLangArray[i];
                String libelleString = intitule.getLibelleString(langInteger, false);
                if (libelleString != null) {
                    basemetaDataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                    done = true;
                }
            }
            if (!done) {
                basemetaDataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
            }
        }
    }

    private void addCorpusList() throws IOException {
        for (int i = 0; i < abregeFicheSortedList.getCorpusCount(); i++) {
            Corpus corpus = (Corpus) abregeFicheSortedList.getCorpus(i);
            CorpusRchfInfoXMLWriter corpusRchfInfoXMLWriter = new CorpusRchfInfoXMLWriter(corpus);
            corpusRchfInfoXMLWriter.addCorpus();
        }
    }


    private class CorpusRchfInfoXMLWriter {

        Corpus corpus;
        SubsetKey corpusKey;
        SubsetItemList abregeFicheList;
        ComplementKeyBundle[] complementKeyBundleArray;
        CorpusField[] complementCorpusFieldArray;
        int complementLength;
        Croisement[] croisementArray;
        int datationSourceType = 0;
        FieldKey datationType = null;
        AccoladePattern fichePattern;
        FieldKey geolocalisationFieldKey;

        CorpusRchfInfoXMLWriter(Corpus corpus) {
            this.corpus = corpus;
            CorpusField geolocalisationField = corpus.getCorpusMetadata().getGeolocalisationField();
            if (geolocalisationField != null) {
                geolocalisationFieldKey = geolocalisationField.getFieldKey();
            }
            corpusKey = corpus.getSubsetKey();
            this.abregeFicheList = abregeFicheSortedList.getAbregeFicheListByCorpus(corpus);
            CorpusExportDef corpusRchfInfo = rchfExport.getCorpusExportDef(corpus.getSubsetKey());
            if (corpusRchfInfo == null) {
                corpusRchfInfo = new CorpusExportDef(corpus.getSubsetKey());
            }
            List complementList = corpusRchfInfo.getComplementKeyList();
            int size = complementList.size();
            complementKeyBundleArray = new ComplementKeyBundle[size];
            int p = 0;
            for (int i = 0; i < size; i++) {
                Object key = complementList.get(i);
                if (key instanceof FieldKey) {
                    CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey((FieldKey) key);
                    if (corpusField != null) {
                        complementKeyBundleArray[p] = new ComplementKeyBundle(corpusField);
                        p++;
                    }
                } else if (key instanceof FilteredIncludeKey) {
                    FilteredIncludeKey includeBundle = (FilteredIncludeKey) key;
                    complementKeyBundleArray[p] = new ComplementKeyBundle(corpusKey, includeBundle);
                    p++;
                }

            }
            if (p < size) {
                ComplementKeyBundle[] temp = new ComplementKeyBundle[p];
                System.arraycopy(complementKeyBundleArray, 0, temp, 0, p);
                complementKeyBundleArray = temp;
            }
            complementLength = p;
            croisementArray = thesaurusXMLWriter.getCroisementArray(corpus.getSubsetKey());
            initDatationType(corpusRchfInfo.getDateKey());
            fichePattern = rchfExport.getFicheGlobalPattern();
        }

        void initDatationType(Object key) {
            if (key instanceof IncludeKey) {
                if (key.equals(IncludeKey.DATE_CREATION)) {
                    datationSourceType = 1;
                } else if (key.equals(IncludeKey.DATE_MODIFICATION)) {
                    datationSourceType = 2;
                }
            } else {
                CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey((FieldKey) key);
                if (corpusField != null) {
                    if (corpusField.getFicheItemType() == CorpusField.DATATION_FIELD) {
                        datationSourceType = 3;
                        datationType = (FieldKey) key;
                    }
                }
            }
        }

        void addCorpus() {
            CorpusMetadataExport corpusMetadataExport = scrutariDataExport.newCorpus(corpus.getSubsetKey().getSubsetName());
            addCorpusMetadata(corpusMetadataExport, corpus.getCorpusMetadata());
            int ficheCount = abregeFicheList.getSubsetItemCount();
            FichePointeur fichePointeur = new FichePointeur(corpus);
            for (int i = 0; i < ficheCount; i++) {
                fichePointeur.setCurrent((AbregeFiche) abregeFicheList.getSubsetItem(i));
                addFiche(fichePointeur);
            }
        }

        private void addCorpusMetadata(CorpusMetadataExport corpusMetadataExport, CorpusMetadata corpusMetadata) {
            addIntitule(corpusMetadataExport, corpusMetadata, CorpusConstants.CORPUS, CorpusMetadataExport.INTITULE_CORPUS);
            addIntitule(corpusMetadataExport, corpusMetadata, CorpusConstants.FICHE_EXISTING, CorpusMetadataExport.INTITULE_FICHE);
            for (int i = 0; i < complementLength; i++) {
                complementKeyBundleArray[i].addIntitule(corpusMetadataExport);
            }
        }

        private void addIntitule(CorpusMetadataExport corpusMetadataExport, CorpusMetadata corpusMetadata, String name, int intituleType) {
            Intitule intitule = corpusMetadata.getIntituleByName(name);
            if (intitule != null) {
                int length = workingLangArray.length;
                boolean done = false;
                for (int i = 0; i < length; i++) {
                    int langInteger = workingLangArray[i];
                    String libelleString = intitule.getLibelleString(langInteger, false);
                    if (libelleString != null) {
                        corpusMetadataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                        done = true;
                    }
                }
                if (!done) {
                    corpusMetadataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
                }
            }
        }

        private void addFiche(FichePointeur fichePointeur) {
            AbregeFiche abregeFiche = fichePointeur.getAbregeFiche();
            Para para = (Para) fichePointeur.getValue(FieldKey.SOUSTITRE);
            String soustitre = null;
            if (para != null) {
                soustitre = para.toRawString();
            }
            fichePatternValueSet.setCurrent(abregeFiche);
            int idcorpus = abregeFiche.getIdcorpus();
            String idcorpusString = String.valueOf(idcorpus);
            FicheExport ficheExport = scrutariDataExport.newFiche(idcorpusString);
            ficheExport.setTitre(abregeFiche.getTitre());
            ficheExport.setSoustitre(soustitre);
            ficheExport.setLang(LangInteger.toString(abregeFiche.getLangInteger()));
            int date = 0;
            if (datationSourceType == 1) {
                date = abregeFiche.getCreationDate();
            } else if (datationSourceType == 2) {
                date = abregeFiche.getModificationDate();
            } else if (datationSourceType == 3) {
                Object obj = fichePointeur.getValue(datationType);
                if (obj != null) {
                    if (obj instanceof Datation) {
                        date = ((Datation) obj).getDateInteger();
                    } else if (obj instanceof FicheItemList) {
                        FicheItemList fil = (FicheItemList) obj;
                        if (fil.getFicheItemCount() > 0) {
                            FicheItem fi = fil.getFicheItem(0);
                            if (fi instanceof Datation) {
                                date = ((Datation) fi).getDateInteger();
                            }
                        }
                    }
                }
            }
            if (date != 0) {
                ficheExport.setDate(DateInteger.toString(date));
            }
            if (complementLength > 0) {
                for (int i = 0; i < complementLength; i++) {
                    complementKeyBundleArray[i].addComplement(i + 1, fichePointeur, ficheExport);
                }
            }
            if (fichePattern != null) {
                ficheExport.setHref(fichePattern.format(fichePatternValueSet));
            }
            if (geolocalisationFieldKey != null) {
                FicheItem ficheItem = (FicheItem) fichePointeur.getValue(geolocalisationFieldKey);
                if ((ficheItem != null) && (ficheItem instanceof Geopoint)) {
                    Geopoint geopoint = (Geopoint) ficheItem;
                    ficheExport.setGeoloc(geopoint.getLatitude().toString(), geopoint.getLongitude().toString());
                }
            }
            if (croisementArray != null) {
                String corpusName = corpus.getSubsetKey().getSubsetName();
                for (int i = 0; i < croisementArray.length; i++) {
                    CroisementItem croisementItem = croisementArray[i].getCroisementItem(corpusKey, abregeFiche.getTicket());
                    int ciCount = croisementItem.getCount();
                    String thesaurusName = croisementArray[i].getSubsetCouple().getSubsetKey2().getSubsetName();
                    for (int j = 0; j < ciCount; j++) {
                        int poids = croisementItem.getPoids(j);
                        int idths = thesaurusXMLWriter.putTicket(i, croisementItem.getTicket(j));
                        scrutariDataExport.addIndexation(corpusName, idcorpusString, thesaurusName, String.valueOf(idths), poids);
                    }
                }
            }
        }

    }


    private class ThesaurusXMLWriter {

        ThesaurusBundle[] thesaurusBundleArray;
        AccoladePattern currentUrlPattern;
        AccoladePattern currentLibellePattern;

        ThesaurusXMLWriter() {
            Basedefiches basedefiches = rchfExport.getBasedefiches();
            int thesaurusCount = basedefiches.getThesaurusCount();
            List selectedList = new ArrayList();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = basedefiches.getThesaurus(i);
                ThesaurusExportDef info = rchfExport.getThesaurusExportDef(thesaurus.getSubsetKey());
                if ((info != null) && (info.isSelected())) {
                    selectedList.add(new ThesaurusBundle(thesaurus, info));
                }
            }
            int size = selectedList.size();
            thesaurusBundleArray = new ThesaurusBundle[size];
            for (int i = 0; i < size; i++) {
                thesaurusBundleArray[i] = (ThesaurusBundle) selectedList.get(i);
            }
        }

        boolean hasThesaurus() {
            return (thesaurusBundleArray != null);
        }

        int putTicket(int thesaurusIndex, int ticket) {
            return thesaurusBundleArray[thesaurusIndex].putTicket(ticket);
        }

        void addThesaurusList() {
            for (int i = 0; i < thesaurusBundleArray.length; i++) {
                currentUrlPattern = rchfExport.getMotcleGlobalPattern();
                addThesaurus(thesaurusBundleArray[i]);
            }
        }

        void addThesaurus(ThesaurusBundle thesaurusBundle) {
            if (thesaurusBundle.isEmpty()) {
                return;
            }
            Thesaurus thesaurus = thesaurusBundle.getThesaurus();
            ThesaurusMetadataExport thesaurusMetadataExport = scrutariDataExport.newThesaurus(thesaurus.getSubsetKey().getSubsetName());
            addThesaurusMetadata(thesaurusMetadataExport, thesaurus.getThesaurusMetadata());
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if ((motcle != null) && (thesaurusBundle.contains(motcle.getIdths()))) {
                    addMotcle(thesaurusBundle, motcle);
                }
            }
        }

        void addMotcle(ThesaurusBundle thesaurusBundle, Motcle motcle) {
            MotcleValueResolver motclePatternValueSet = new MotcleValueResolver(motcle);
            AccoladePattern libellePattern = thesaurusBundle.getLibellePattern();
            int[] thesaurusLangArray = thesaurusBundle.getThesaurusLangArray();
            MotcleExport motcleExport = scrutariDataExport.newMotcle(String.valueOf(motcle.getIdths()));
            if (thesaurusLangArray != null) {
                for (int i = 0; i < thesaurusLangArray.length; i++) {
                    int langInteger = thesaurusLangArray[i];
                    motclePatternValueSet.setLangInteger(langInteger);
                    String libelleString = libellePattern.format(motclePatternValueSet);
                    if (libelleString.length() > 0) {
                        motcleExport.setLibelle(LangInteger.toString(langInteger), libelleString);
                    }
                }
            } else {
                int langInteger = ThesaurusUtils.getDefaultPreferredLangInteger(motcle);
                motclePatternValueSet.setLangInteger(langInteger);
                String libelleString = libellePattern.format(motclePatternValueSet);
                if (libelleString.length() > 0) {
                    motcleExport.setLibelle(LangInteger.toString(langInteger), libelleString);
                }
            }
            /*if (currentUrlPattern != null) {
            addSimpleElement("href",currentUrlPattern.format(motclePatternValueSet));
            }*/
        }

        private void addThesaurusMetadata(ThesaurusMetadataExport thesaurusMetadataExport, ThesaurusMetadata thesaurusMetadata) {
            addIntitule(thesaurusMetadataExport, thesaurusMetadata, ThesaurusConstants.THESAURUS, ThesaurusMetadataExport.INTITULE_THESAURUS);
        }

        private void addIntitule(ThesaurusMetadataExport thesaurusMetadataExport, ThesaurusMetadata thesaurusMetadata, String name, int intituleType) {
            Intitule intitule = thesaurusMetadata.getIntituleByName(name);
            if (intitule != null) {
                int length = workingLangArray.length;
                boolean done = false;
                for (int i = 0; i < length; i++) {
                    int langInteger = workingLangArray[i];
                    String libelleString = intitule.getLibelleString(langInteger, false);
                    if (libelleString != null) {
                        thesaurusMetadataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                        done = true;
                    }
                }
                if (!done) {
                    thesaurusMetadataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
                }
            }
        }

        Croisement[] getCroisementArray(SubsetKey corpusKey) {
            if (thesaurusBundleArray == null) {
                return null;
            }
            int length = thesaurusBundleArray.length;
            Croisement[] croisementArray = new Croisement[length];
            for (int i = 0; i < length; i++) {
                croisementArray[i] = basedefiches.getCroisement(corpusKey, thesaurusBundleArray[i].getThesaurus().getSubsetKey());
            }
            return croisementArray;
        }


        private class ThesaurusBundle {

            Thesaurus thesaurus;
            ThesaurusExportDef thesaurusRchfInfo;
            HashSet idthsSet = new HashSet();
            AccoladePattern libellePattern;
            int[] thesaurusLangArray;

            ThesaurusBundle(Thesaurus thesaurus, ThesaurusExportDef thesaurusRchfInfo) {
                this.thesaurus = thesaurus;
                this.thesaurusRchfInfo = thesaurusRchfInfo;
                thesaurusLangArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
                libellePattern = thesaurusRchfInfo.getLibellePattern();
                if (libellePattern == null) {
                    try {
                        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                            libellePattern = new AccoladePattern("{idalpha} – {lib}");
                        } else {
                            libellePattern = new AccoladePattern("{lib}");
                        }
                    } catch (java.text.ParseException pe) {
                        throw new ShouldNotOccurException(pe);
                    }
                }
            }

            int putTicket(int ticket) {
                SubsetItem subsetItem = thesaurus.getSubsetItemByTicket(ticket);
                int idths = subsetItem.getIdsubset();
                idthsSet.add(new Integer(idths));
                return idths;
            }

            public Thesaurus getThesaurus() {
                return thesaurus;
            }

            public boolean isEmpty() {
                return idthsSet.isEmpty();
            }

            public boolean contains(int idths) {
                return idthsSet.contains(new Integer(idths));
            }

            public AccoladePattern getLibellePattern() {
                return libellePattern;
            }

            public int[] getThesaurusLangArray() {
                return thesaurusLangArray;
            }

        }

    }


    class FicheValueResolver implements ValueResolver {

        AbregeFiche abregeFiche;

        void setCurrent(AbregeFiche abregeFiche) {
            this.abregeFiche = abregeFiche;
        }

        public String getValue(AccoladeArgument accoladeArgument) {
            String name = accoladeArgument.getName();
            if (name.equals("corpus")) {
                return abregeFiche.getCorpus().getSubsetKey().getSubsetName();
            }
            if (name.equals("idcorpus")) {
                return String.valueOf(abregeFiche.getIdcorpus());
            }
            return null;
        }

    }


    private class ComplementKeyBundle {

        CorpusField corpusField;
        FilteredIncludeKey includeBundle;
        IncludeLibelleMap corpusInclude;
        Croisement croisement;
        SubsetKey corpusKey;
        Corpus otherCorpus;

        ComplementKeyBundle(CorpusField corpusField) {
            this.corpusField = corpusField;
        }

        ComplementKeyBundle(SubsetKey corpusKey, FilteredIncludeKey includeBundle) {
            this.includeBundle = includeBundle;
            this.corpusInclude = libelleSource.getIncludeLibelleMap(corpusKey);
            SubsetKey otherKey = includeBundle.getIncludeKey().getSubsetKey();
            croisement = basedefiches.getCroisement(corpusKey, otherKey);
            this.corpusKey = corpusKey;
            otherCorpus = (Corpus) basedefiches.getSubset(otherKey);
        }

        private void addComplement(int complementNumber, FichePointeur fichePointeur, FicheExport ficheExport) {
            StringBuilder buf = new StringBuilder();
            if (corpusField != null) {
                FieldKey fieldKey = corpusField.getFieldKey();
                Object obj = fichePointeur.getValue(fieldKey);
                if (obj != null) {
                    if (obj instanceof FicheItem) {
                        buf.append(plainConverter.toLibString((FicheItem) obj));
                    } else if (obj instanceof FicheItemList) {
                        buf.append(plainConverter.toLibString((FicheItemList) obj));
                    }
                }
            } else {
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, fichePointeur.getCurrentAbregeFiche().getTicket());
                int count = croisementItem.getCount();
                for (int i = 0; i < count; i++) {
                    int poids = croisementItem.getPoids(i);
                    int ticket = croisementItem.getTicket(i);
                    AbregeFiche other = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
                    buf.append(other.getTitre());
                    if (i > 0) {
                        buf.append(", ");
                    }
                }
            }
            ficheExport.addComplement(complementNumber, buf.toString());
        }

        private void addIntitule(CorpusMetadataExport corpusMetadataExport) {
            if (corpusField != null) {
                addComplementMetadata(corpusMetadataExport, corpusField);
            } else {
                addComplementMetadata(corpusMetadataExport, includeBundle);
            }
        }

        private void addComplementMetadata(CorpusMetadataExport corpusMetadataExport, CorpusField corpusField) {
            int number = corpusMetadataExport.addComplement();
            int length = workingLangArray.length;
            boolean done = false;
            for (int i = 0; i < length; i++) {
                int langInteger = workingLangArray[i];
                String libelleString = corpusField.getLibelleString(langInteger, false);
                if (libelleString != null) {
                    corpusMetadataExport.setComplementIntitule(number, LangInteger.toString(langInteger), libelleString);
                    done = true;
                }
            }
            if (!done) {
                corpusMetadataExport.setComplementIntitule(number, "und", corpusField.getLibelleString(LangInteger.UNDETERMINED, true));
            }
        }

        private void addComplementMetadata(CorpusMetadataExport corpusMetadataExport, FilteredIncludeKey filteredIncludeKey) {
            int number = corpusMetadataExport.addComplement();
            int length = workingLangArray.length;
            for (int i = 0; i < length; i++) {
                int langInteger = workingLangArray[i];
                String libelleString = corpusInclude.getLibelleString(filteredIncludeKey, langInteger);
                corpusMetadataExport.setComplementIntitule(number, LangInteger.toString(langInteger), libelleString);
            }
        }

    }

}
