/* BasedefichesLib_Tools - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.sqlexport;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.sqlexport.SqlExport;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.SubsetTableExport;
import net.mapeadores.basedefiches.tableexport.TableExportConstants;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementTableExport;
import net.mapeadores.basedefiches.tools.tableexport.CroisementTableExportSqlWriter;
import net.mapeadores.basedefiches.tools.tableexport.SqlColDefTableWriter;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.sql.CreateTableWriter;


/**
 *
 * @author Vincent Calame
 */
public class DefaultSqlExport implements SqlExport, TableExportConstants {

    public final static String CHARSET_PARAMNAME = "charset";
    public final static String DROPTABLE_PARAMNAME = "drop_table";
    public final static String DELETETABLE_PARAMNAME = "delete_table";
    public final static String CREATETABLE_PARAMNAME = "create_table";
    public final static short NO = 0;
    public final static short YES_DROPTABLE = 1;
    public final static short IFEXISTS_DROPTABLE = 2;
    public final static int CREATE_TABLE = 1;
    public final static int DELETE_TABLE = 1;
    private Map paramMap = new HashMap();
    private SubsetTableExport[] subsetTableExportArray;
    private CroisementTableExport[] croisementTableExportArray;

    public DefaultSqlExport() {
    }

    public ProcessLog exportDump(OutputStream outputStream) throws IOException {
        String charset = (String) paramMap.get(CHARSET_PARAMNAME);
        if (charset == null) {
            charset = "UTF-8";
        }
        BufferedWriter bufWriter = new BufferedWriter(new OutputStreamWriter(outputStream, charset));
        ExportProcess exportProcess = new ExportProcess(bufWriter);
        exportProcess.run();
        bufWriter.flush();
        return new MutableProcessLog();
    }

    public void setSubsetTableExportArray(SubsetTableExport[] corpusTableExportArray) {
        this.subsetTableExportArray = corpusTableExportArray;
    }

    public void setCroisementTableExportArray(CroisementTableExport[] croisementTableExportArray) {
        this.croisementTableExportArray = croisementTableExportArray;
    }

    public void setParameter(String name, String value) {
        paramMap.put(name, value);
    }

    public void setBasedefiches(Basedefiches basedefiches) {
    }

    private int getDropTableType() {
        String drop = (String) paramMap.get(DROPTABLE_PARAMNAME);
        if (drop == null) {
            return NO;
        }
        if (drop.equals("1")) {
            return YES_DROPTABLE;
        }
        if (drop.equals("2")) {
            return IFEXISTS_DROPTABLE;
        }
        return NO;
    }

    private int getCreateTableType() {
        String drop = (String) paramMap.get(CREATETABLE_PARAMNAME);
        if (drop == null) {
            return NO;
        }
        if (drop.equals("1")) {
            return CREATE_TABLE;
        }
        return NO;
    }

    private int getDeleteTableType() {
        String drop = (String) paramMap.get(DELETETABLE_PARAMNAME);
        if (drop == null) {
            return NO;
        }
        if (drop.equals("1")) {
            return DELETE_TABLE;
        }
        return NO;
    }


    private class ExportProcess {

        private Writer writer;
        private int createTable;
        private int deleteTable;
        private int dropTable;
        private CreateTableWriter createTableWriter;

        ExportProcess(Writer writer) {
            this.writer = writer;
            this.dropTable = getDropTableType();
            this.deleteTable = getDeleteTableType();
            this.createTable = getCreateTableType();
            this.createTableWriter = new CreateTableWriter(writer);
        }

        void run() throws IOException {
            int length = subsetTableExportArray.length;
            for (int i = 0; i < length; i++) {
                blank();
                SubsetTableExport subsetTableExport = subsetTableExportArray[i];
                SubsetKey corpusKey = subsetTableExport.getSubset().getSubsetKey();
                ColDefList colDefList = subsetTableExport.getColDefList();
                String tableName = colDefList.getParameterValue(TABLE_NAME);
                if (tableName == null) {
                    tableName = corpusKey.toString();
                }
                testDrop(tableName);
                if (createTable > 0) {
                    writeCreateTable(tableName, colDefList);
                    endLine();
                    blank();
                }
                testDelete(tableName);
                SqlColDefTableWriter sqlColDefTableWriter = new SqlColDefTableWriter(writer, tableName);
                subsetTableExport.export(sqlColDefTableWriter);
            }
            int croisementLength = croisementTableExportArray.length;
            for (int i = 0; i < croisementLength; i++) {
                blank();
                CroisementTableExport croisementTableExport = croisementTableExportArray[i];
                String tableName = croisementTableExport.getTableName();
                testDrop(tableName);
                testDelete(tableName);
                CroisementTableExportSqlWriter croisementTableExportSqlWriter = new CroisementTableExportSqlWriter(writer, tableName);
                croisementTableExport.export(croisementTableExportSqlWriter);
            }

        }

        private void testDrop(String tableName) throws IOException {
            if (dropTable > 0) {
                writer.write("DROP TABLE ");
                if (dropTable == IFEXISTS_DROPTABLE) {
                    writer.write("IF EXISTS ");
                }
                writer.write(tableName);
                endLine();
            }
        }

        private void testDelete(String tableName) throws IOException {
            if (deleteTable > 0) {
                writer.write("DELETE FROM ");
                writer.write(tableName);
                endLine();
                blank();
            }
        }

        public void endLine() throws IOException {
            writer.write(";\n");
        }

        public void blank() throws IOException {
            writer.write("\n");
        }

        private void writeCreateTable(String tableName, ColDefList colDefList) throws IOException {
            createTableWriter.startTable(tableName);
            int length = colDefList.getColDefCount();
            for (int i = 0; i < length; i++) {
                ColDef colDef = colDefList.getColDef(i);
                int mask = 0;
                String colName = colDef.getColName();
                if (colName.equals("id")) {
                    mask = CreateTableWriter.PRIMARY_KEY;
                }
                BasedefichesFormatDef ficheFormatDef = colDef.getBasedefichesFormatDef();
                short castType = ficheFormatDef.getCastType();
                switch (castType) {
                    case BasedefichesFormatDef.INTEGER_CAST:
                        createTableWriter.addIntegerField(colName, mask);
                        break;
                    case BasedefichesFormatDef.DECIMAL_CAST:
                        createTableWriter.addDoubleField(colName, mask);
                        break;
                    case BasedefichesFormatDef.DATE_CAST:
                        createTableWriter.addDateField(colName, mask);
                        break;
                    default:
                        createTableWriter.addTextField(colName, mask);
                }
            }
            createTableWriter.endTable();
        }

    }

}
