/* BasedefichesLib_Tools - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.tableexport;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Date;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefTableWriter;
import net.mapeadores.basedefiches.tableexport.SubsetTableExport;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.basedefiches.tableexport.TableExportConstants;
import net.mapeadores.opendocument.io.OdContentWriter;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.ZipEntrySource;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.primitives.Decimal;


/**
 *
 * @author Vincent Calame
 */
public class SubsetTableExportOdsWriter implements TableExportConstants {

    private OdZip odZip;
    private SubsetTableExport[] subsetTableExportArray;
    private AbregeFicheSortedList abregeFicheSortedList;
    private short headerType = TableExportCommon.KEY_HEADER;

    public SubsetTableExportOdsWriter() {
        odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        odZip.setContentZipEntrySource(new ContentZipEntrySource());
    }

    public void setSubsetTableExportArray(SubsetTableExport[] subsetTableExportArray) {
        this.subsetTableExportArray = subsetTableExportArray;
    }

    public void setAbregeFicheSortedList(AbregeFicheSortedList abregeFicheSortedList) {
        this.abregeFicheSortedList = abregeFicheSortedList;
    }

    public void setHeaderType(short headerType) {
        this.headerType = headerType;
    }

    public void export(OutputStream outputStream) throws IOException {
        odZip.zip(outputStream);
    }


    private class ContentZipEntrySource implements ZipEntrySource {

        public void writeEntry(OutputStream outputStream) throws IOException {
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
            SubsetTableXMLWriter xmlWriter = new SubsetTableXMLWriter();
            xmlWriter.setAppendable(writer);
            xmlWriter.start();
            int length = subsetTableExportArray.length;
            for (int i = 0; i < length; i++) {
                SubsetTableExport subsetTableExport = subsetTableExportArray[i];
                xmlWriter.export(subsetTableExport);
            }
            xmlWriter.end();
            writer.flush();
        }


        private class SubsetTableXMLWriter extends OdContentWriter implements ColDefTableWriter {

            SubsetTableXMLWriter() {
            }

            private void export(SubsetTableExport subsetTableExport) throws IOException {
                SubsetItemList subsetItemList = null;
                Subset subset = subsetTableExport.getSubset();
                if (subset instanceof Corpus) {
                    if (abregeFicheSortedList != null) {
                        subsetItemList = abregeFicheSortedList.getAbregeFicheListByCorpus((Corpus) subset);
                        if (subsetItemList == null) {
                            return;
                        }
                    }
                }
                ColDefList colDefList = subsetTableExport.getColDefList();
                String tableName = colDefList.getParameterValue(TABLE_NAME);
                if (tableName == null) {
                    tableName = subsetTableExport.getSubset().getSubsetKey().toString();
                }
                openTable(tableName);
                printColumns(colDefList);
                if (headerType == TableExportCommon.LIB_HEADER) {
                    printHeaderLib(colDefList);
                } else if (headerType == TableExportCommon.KEY_HEADER) {
                    printHeaderKey(colDefList);
                }
                try {
                    if (subsetItemList != null) {
                        subsetTableExport.export(this, subsetItemList);
                    } else {
                        subsetTableExport.export(this);
                    }
                } catch (NestedIOException nioe) {
                    throw nioe.getIOException();
                }
                closeTable();
            }

            private void printColumns(ColDefList colDefList) throws IOException {
                int repeat = 0;
                short currentCast = -1;
                String cellStyleName = null;
                int colDefCount = colDefList.getColDefCount();
                for (int i = 0; i < colDefCount; i++) {
                    BasedefichesFormatDef ficheFormatDef = colDefList.getColDef(i).getBasedefichesFormatDef();
                    short cast = ficheFormatDef.getCastType();
                    if (currentCast != cast) {
                        if (currentCast != -1) {
                            printColumn(repeat, cellStyleName);
                        }
                        repeat = 1;
                        currentCast = cast;
                        if (cast == FormatDef.DATE_CAST) {
                            cellStyleName = "ce3";
                        } else {
                            cellStyleName = null;
                        }
                    } else {
                        repeat++;
                    }
                }
                if (currentCast != -1) {
                    printColumn(repeat, cellStyleName);
                }
            }

            private void printColumn(int numberRepeat, String cellStyleName) throws IOException {
                Integer nr = (numberRepeat == 1) ? null : new Integer(numberRepeat);
                if (cellStyleName == null) {
                    cellStyleName = "Default";
                }
                startTableColumnOpenTag("co1", nr, cellStyleName);
                closeEmptyTag();
            }

            private void printHeaderLib(ColDefList colDefList) throws IOException {
                openTableRow();
                int count = colDefList.getColDefCount();
                for (int i = 0; i < count; i++) {
                    ColDef colDef = colDefList.getColDef(i);
                    String lib = colDef.getColLib();
                    if (lib.length() == 0) {
                        lib = colDef.getColName();
                    }
                    addStringTableCell(lib);
                }
                closeTableRow();
            }

            private void printHeaderKey(ColDefList colDefList) throws IOException {
                openTableRow();
                int count = colDefList.getColDefCount();
                for (int i = 0; i < count; i++) {
                    ColDef colDef = colDefList.getColDef(i);
                    addStringTableCell(colDef.getColName());
                }
                closeTableRow();
            }

            public void startRow() {
                try {
                    openTableRow();
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addNullCell(ColDef colDef) {
                try {
                    addEmptyTableCell();
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addIntegerCell(ColDef colDef, Long lg) {
                try {
                    if (lg == null) {
                        addEmptyTableCell();
                    } else {
                        addIntegerTableCell(lg.longValue());
                    }
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addDecimalCell(ColDef colDef, Decimal decimal) {
                try {
                    if (decimal == null) {
                        addEmptyTableCell();
                    } else {
                        addDecimalTableCell(decimal);
                    }
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addDateCell(ColDef colDef, Date date) {
                try {
                    if (date == null) {
                        addEmptyTableCell();
                    } else {
                        this.addDateTableCell(date);
                    }
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addStringCell(ColDef colDef, String s) {
                try {
                    if (s == null) {
                        addEmptyTableCell();
                    } else {
                        addStringTableCell(s);
                    }
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void addLineCell(ColDef colDef, List<String> list) {
                try {
                    if ((list == null) || (list.size() == 0)) {
                        addEmptyTableCell();
                    } else {
                        addLineTableCell(list);
                    }
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            public void endRow() {
                try {
                    closeTableRow();
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }

            private void start() throws IOException {
                appendXMLDeclaration();
                openDocumentContent();
                openAutomaticStyles();

                openStyle("co1", "table-column", null);
                closeStyle();

                startDateStyleOpenTag("N84", "data-style", Boolean.TRUE);
                endOpenTag();
                addISONumber();
                closeDateStyle();

                startStyleOpenTag("ce3", "table-cell", "Default");
                addAttribute("style:data-style-name", "N84");
                closeEmptyTag();

                closeAutomaticStyles();
                openBody();
                openSpreadsheet();
            }

            private void end() throws IOException {
                closeSpreadsheet();
                closeBody();
                closeDocumentContent();
            }

        }

    }

}
