/* BasedefichesLib_Tools - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.thesaurus;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.misc.ThesaurusInverseFactory;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.lexie.LexieMap;
import net.mapeadores.util.text.lexie.Occurence;
import net.mapeadores.util.text.lexie.ParsedLexieSource;


/**
 *
 * @author Vincent Calame
 */
public class RecouvrementEngine {

    private Croisement[] croisements;
    private Corpus[] corpusArray;
    private SubsetKey thesaurusKey;
    private Map<Integer, MotcleStat> statMap = new HashMap<Integer, MotcleStat>();
    private Thesaurus thesaurus;
    private int lang;
    private LexieMap lexieMap;
    private List<RecouvrementUnit> resultList = new ArrayList<RecouvrementUnit>();

    private RecouvrementEngine(Thesaurus thesaurus, int lang) {
        this.thesaurus = thesaurus;
        this.lang = lang;
        this.thesaurusKey = thesaurus.getSubsetKey();
    }

    private void initCroisements() {
        Basedefiches basedefiches = thesaurus.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        croisements = new Croisement[corpusCount];
        corpusArray = new Corpus[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
            croisements[i] = croisement;
            corpusArray[i] = corpus;
        }
    }

    private void initStats(MotcleList motcleList) {
        for (int i = 0; i < motcleList.getMotcleCount(); i++) {
            Motcle motcle = motcleList.getMotcle(i);
            MotcleStat motcleStat = new MotcleStat(motcle);
            statMap.put(motcle.getIdths(), motcleStat);
            initStats(motcle.getChildList());
        }
    }

    public static List<RecouvrementUnit> getRecouvrementItemList(Thesaurus thesaurus, int lang) {
        RecouvrementEngine recouvrementEngine = new RecouvrementEngine(thesaurus, lang);
        recouvrementEngine.initList();
        return recouvrementEngine.resultList;
    }

    private void initList() {
        initCroisements();
        initStats(thesaurus.getFirstLevelList());
        lexieMap = ThesaurusInverseFactory.getLibelleLexieMap(thesaurus, lang);
        for (ParsedLexieSource lexieSource : lexieMap.getParsedLexieSourceMap().values()) {
            String lib = lexieSource.getLexieSource().getSourceText();
            Set<Motcle> set = thesaurus.getMotcleSetByLibelle(lib, TextConstants.CONTAINS, lang);
            if (set.size() > 1) {
                testLexie(lexieSource, set);
            }
        }
    }

    private void testLexie(ParsedLexieSource lexieSource, Set<Motcle> otherSet) {
        Map<String, ParsedLexieSource> map = lexieMap.getParsedLexieSourceMap();
        Motcle motcle = (Motcle) lexieSource.getLexieSource().getSourceObject();
        RecouvrementUnit recouvrementUnit = new RecouvrementUnit(motcle);
        Set<String> lexieSet = new HashSet<String>();
        for (Occurence occurence : lexieSource.getOccurenceList()) {
            lexieSet.add(occurence.getLexieUnit().getCollatedLexie());
        }
        if (lexieSet.size() == 0) {
            return;
        }
        MotcleStat motcleStat = getStat(motcle);
        recouvrementUnit.setStatCount(motcleStat.getCount());
        Set<String> tempSet = new HashSet<String>();
        for (Motcle other : otherSet) {
            tempSet.clear();
            tempSet.addAll(lexieSet);
            if (!other.equals(motcle)) {
                ParsedLexieSource otherLexieSource = (ParsedLexieSource) map.get(String.valueOf(other.getIdths()));
                for (Occurence occurence : otherLexieSource.getOccurenceList()) {
                    String colLex = occurence.getLexieUnit().getCollatedLexie();
                    tempSet.remove(colLex);
                }
                if (tempSet.size() == 0) {
                    MotcleStat otherStat = getStat(other);
                    int count = otherStat.getCount();
                    Set<AbregeFiche> commonSet = otherStat.retains(motcleStat);
                    recouvrementUnit.add(other, count, commonSet);
                }
            }
        }
        if (recouvrementUnit.getSize() > 0) {
            resultList.add(recouvrementUnit);
        }
    }

    private MotcleStat getStat(Motcle motcle) {
        return statMap.get(motcle.getIdths());
    }


    private class MotcleStat {

        HashSet<AbregeFiche> abregeFicheSet = new HashSet<AbregeFiche>();

        MotcleStat(Motcle motcle) {
            init(motcle);
        }

        private void init(Motcle motcle) {
            int length = croisements.length;
            for (int i = 0; i < length; i++) {
                Croisement croisement = croisements[i];
                CroisementItem ci = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
                int size = ci.getCount();
                Corpus corpus = corpusArray[i];
                for (int j = 0; j < size; j++) {
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ci.getTicket(j));
                    if (abregeFiche != null) {
                        abregeFicheSet.add(abregeFiche);
                    }
                }

            }
        }

        public int getCount() {
            return abregeFicheSet.size();
        }

        public Set<AbregeFiche> retains(MotcleStat motcleStat) {
            Set<AbregeFiche> copy = (HashSet<AbregeFiche>) abregeFicheSet.clone();
            copy.retainAll(motcleStat.abregeFicheSet);
            return copy;
        }

    }

}
