/* BasedefichesLib_Xml - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadata;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class AlbumXMLWriter extends UtilXMLWriter {

    public AlbumXMLWriter() {
    }

    public void appendAlbum(Album album) throws IOException {
        openTag("album");
        appendMetadata(album.getAlbumMetadata());
        closeTag("album");
    }

    private void appendMetadata(AlbumMetadata albumMetadata) throws IOException {
        openTag("metadata");
        BasedefichesXMLUtils.appendIntituleList(albumMetadata, this);
        int albumItemCount = albumMetadata.getAlbumDimCount();
        for (int i = 0; i < albumItemCount; i++) {
            appendAlbumDim(albumMetadata.getAlbumDim(i));
        }
        String[] usageNameArray = albumMetadata.getUsageNameArray();
        if (usageNameArray != null) {
            int length = usageNameArray.length;
            for (int i = 0; i < length; i++) {
                String usageName = usageNameArray[i];
                AlbumDim albumDim = albumMetadata.getAlbumDimForUsage(usageName);
                appendDimUsage(usageName, albumDim);
            }
        }
        closeTag("metadata");
    }

    private void appendDimUsage(String usageName, AlbumDim albumDim) throws IOException {
        if (albumDim != null) {
            startOpenTag("dim-usage");
            addAttribute("usage-name", usageName);
            addAttribute("dim-name", albumDim.getName());
            closeEmptyTag();
        }
    }

    private void appendAlbumDim(AlbumDim albumDim) throws IOException {
        short type = albumDim.getType();
        startOpenTag("album-dim");
        addAttribute("name", albumDim.getName());
        addAttribute("type", AlbumUtils.dimTypeToString(type));
        if (AlbumUtils.needWidth(type)) {
            addAttribute("width", albumDim.getWidth());
        }
        if (AlbumUtils.needHeight(type)) {
            addAttribute("height", albumDim.getHeight());
        }
        endOpenTag();
        closeTag("album-dim");
    }

}
