/* BasedefichesLib_Xml - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class ChronoDOMReader extends BasedefichesDOMReader {

    private CorpusEditor corpusEditor;
    private ChronoXmlErrorHandler errorHandler;

    public ChronoDOMReader(ChronoXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.errorHandler = errorHandler;
    }

    public void setCorpusEditor(CorpusEditor corpusEditor) {
        this.corpusEditor = corpusEditor;
    }

    public void fillChrono(Element element_xml) {
        Corpus corpus = corpusEditor.getCorpus();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("ch")) {
                    int[] intArray = getIntArray(el);
                    if (intArray != null) {
                        int idcorpus = intArray[0];
                        AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                        if (abregeFiche == null) {
                            errorHandler.unknownAbregeFicheError(idcorpus);
                            continue;
                        }
                        corpusEditor.setDate(abregeFiche, intArray[1], false);
                        corpusEditor.setDate(abregeFiche, intArray[2], true);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private int[] getIntArray(Element element_xml) {
        String idString = element_xml.getAttribute("id");
        if (idString.length() == 0) {
            errorHandler.emptyAttributeError("ch", "id");
            return null;
        }
        int idcorpus = 0;
        try {
            idcorpus = Integer.parseInt(idString);
        } catch (NumberFormatException nfe) {
            errorHandler.wrongIntegerAttributeError("ch", "id", idString);
            return null;
        }
        String creationDateString = element_xml.getAttribute("c");
        if (creationDateString.length() == 0) {
            errorHandler.emptyAttributeError("ch", "c");
            return null;
        }
        int creationDate = 0;
        try {
            creationDate = DateInteger.parse(creationDateString);
        } catch (DateIntegerException ide) {
            errorHandler.wrongDateAttributeError("ch", "c", creationDateString);
            return null;
        }
        int modificationDate = 0;
        String modificationDateString = element_xml.getAttribute("m");
        if (modificationDateString.length() > 0) {
            try {
                modificationDate = DateInteger.parse(modificationDateString);

            } catch (DateIntegerException ide) {
                errorHandler.wrongDateAttributeError("ch", "m", modificationDateString);
                return null;
            }
        }
        int[] result = new int[3];
        result[0] = idcorpus;
        result[1] = creationDate;
        result[2] = modificationDate;
        return result;
    }

}
