/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CorpusMetadataXMLWriter extends UtilXMLWriter {

    public CorpusMetadataXMLWriter() {
    }

    public void appendCorpusMetadata(CorpusMetadata corpusMetadata) throws IOException {
        openTag("corpus-metadata");
        appendFields(corpusMetadata);
        appendOptions(corpusMetadata);
        appendL10n(corpusMetadata);
        closeTag("corpus-metadata");
    }

    private void appendFields(CorpusMetadata corpusMetadata) throws IOException {
        openTag("fields");
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            addEmptyElement("soustitre");
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            startOpenTag("propriete");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();

        }
        if (CorpusMetadataUtils.hasTexteField(corpusMetadata)) {
            addEmptyElement("texte");
        }
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        for (int i = 0; i < annexeCount; i++) {
            CorpusField corpusField = corpusMetadata.getAnnexeField(i);
            startOpenTag("annexe");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            closeEmptyTag();
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            startOpenTag("information");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();
        }
        closeTag("fields");
    }

    private void appendFicheItemAttribute(CorpusField corpusField) throws IOException {
        addAttribute("fiche-item", CorpusMetadataUtils.ficheItemTypeToString(corpusField.getFicheItemType()));
    }

    private void appendL10n(CorpusMetadata corpusMetadata) throws IOException {
        openTag("l10n");
        List<CorpusField> list = CorpusMetadataUtils.getCorpusFieldList(corpusMetadata);
        for (CorpusField corpusField : list) {
            startOpenTag("field");
            addAttribute("field-key", corpusField.getFieldKey().toString());
            endOpenTag();
            addLibelleList(corpusField);
            closeTag("field");
        }
        BasedefichesXMLUtils.appendIntituleList(corpusMetadata, this);
        closeTag("l10n");
    }

    private void appendOptions(CorpusMetadata corpusMetadata) throws IOException {
        openTag("options");
        CorpusField geolocalisationField = corpusMetadata.getGeolocalisationField();
        if (geolocalisationField != null) {
            String value = null;
            appendOption(FieldOptions.GEOLOCALISATIONFIELD_OPTION, value, geolocalisationField.getFieldKey());
        }
        appendTitreGenerationOption(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE));
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            appendOptions(corpusField);
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            appendOptions(corpusField);
        }
        closeTag("options");
    }

    private void appendOptions(CorpusField corpusField) throws IOException {
        appendInformationDisplayOption(corpusField);
        appendDefaultSphereKeyOption(corpusField);
        appendAlbumKeyOption(corpusField);
        appendSubfieldDisplayOption(corpusField);
        appendCurrencyArrayOption(corpusField);
        appendAddressFieldArrayOption(corpusField);
        String[] stringOptionNameArray = corpusField.getStringOptionNameArray();
        int length = stringOptionNameArray.length;
        for (int i = 0; i < length; i++) {
            appendStringOption(corpusField, stringOptionNameArray[i]);
        }
    }

    private void appendStringOption(CorpusField corpusField, String optionName) throws IOException {
        String optionValue = corpusField.getStringOption(optionName);
        if (optionValue != null) {
            appendOption(optionName, optionValue, corpusField.getFieldKey());
        }
    }

    private void appendInformationDisplayOption(CorpusField corpusField) throws IOException {
        Short informationDisplayShort = (Short) corpusField.getOption(FieldOptions.INFORMATIONDISPLAY_OPTION);
        if (informationDisplayShort != null) {
            short informationDisplay = informationDisplayShort.shortValue();
            if (informationDisplay != FieldOptions.getDefaultInformationDisplay()) {
                appendOption(FieldOptions.INFORMATIONDISPLAY_OPTION, FieldOptions.informationDisplayToString(informationDisplay), corpusField.getFieldKey());
            }
        }
    }

    private void appendDefaultSphereKeyOption(CorpusField corpusField) throws IOException {
        SubsetKey sphereKey = (SubsetKey) corpusField.getOption(FieldOptions.DEFAULTSPHEREKEY_OPTION);
        if (sphereKey != null) {
            appendOption(FieldOptions.DEFAULTSPHEREKEY_OPTION, sphereKey.toString(), corpusField.getFieldKey());
        }
    }

    private void appendAlbumKeyOption(CorpusField corpusField) throws IOException {
        SubsetKey sphereKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
        if (sphereKey != null) {
            appendOption(FieldOptions.ALBUMKEY_OPTION, sphereKey.toString(), corpusField.getFieldKey());
        }
    }

    private void appendSubfieldDisplayOption(CorpusField corpusField) throws IOException {
        Boolean bool = (Boolean) corpusField.getOption(FieldOptions.SUBFIELDDISPLAY_OPTION);
        if ((bool != null) && (bool.equals(Boolean.TRUE))) {
            appendOption(FieldOptions.SUBFIELDDISPLAY_OPTION, "1", corpusField.getFieldKey());
        }
    }

    private void appendTitreGenerationOption(CorpusField corpusField) throws IOException {
        BasedefichesFormatDef ficheFormatDef = (BasedefichesFormatDef) corpusField.getOption(FieldOptions.TITREGENERATIONFORMAT_OPTION);
        if (ficheFormatDef != null) {
            appendOption(FieldOptions.TITREGENERATIONFORMAT_OPTION, ficheFormatDef.toStringArray(), corpusField.getFieldKey());
        }
    }

    private void appendCurrencyArrayOption(CorpusField corpusField) throws IOException {
        int[] currencyArray = (int[]) corpusField.getOption(FieldOptions.CURRENCYARRAY_OPTION);
        if (currencyArray != null) {
            int length = currencyArray.length;
            String[] valueArray = new String[length];
            for (int i = 0; i < length; i++) {
                valueArray[i] = CurrencyInteger.toString(currencyArray[i]);
            }
            appendOption(FieldOptions.CURRENCYARRAY_OPTION, valueArray, corpusField.getFieldKey());
        }
    }

    private void appendAddressFieldArrayOption(CorpusField corpusField) throws IOException {
        CorpusField[] corpusFieldArray = (CorpusField[]) corpusField.getOption(FieldOptions.ADDRESSFIELDARRAY_OPTION);
        if (corpusFieldArray != null) {
            int length = corpusFieldArray.length;
            String[] values = new String[length];
            for (int i = 0; i < length; i++) {
                values[i] = corpusFieldArray[i].getFieldKey().toString();
            }
            appendOption(FieldOptions.ADDRESSFIELDARRAY_OPTION, values, corpusField.getFieldKey());
        }
    }

    private void appendOption(String name, String value, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("value", value);
        addAttribute("field-key", fieldKey.toString());
        closeEmptyTag();
    }

    private void appendOption(String name, String[] values, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("field-key", fieldKey.toString());
        endOpenTag();
        for (int i = 0; i < values.length; i++) {
            String value = values[i];
            if ((value != null) && (value.length() > 0)) {
                addSimpleElement("value", value);
            } else {
                addEmptyElement("value");
            }
        }
        closeTag("option");
    }

}
