/* BasedefichesLib_Xml - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CroisementXMLWriter extends UtilXMLWriter {

    public CroisementXMLWriter() {
    }

    public int appendCroisement(Croisement croisement) throws IOException {
        int croisementCount = 0;
        SubsetCouple couple = croisement.getSubsetCouple();
        boolean isUnique = couple.isUnique();
        SubsetKey subsetKey1 = couple.getSubsetKey1();
        SubsetKey subsetKey2 = couple.getSubsetKey2();
        Subset subset1 = croisement.getBasedefiches().getSubset(subsetKey1);
        Subset subset2 = croisement.getBasedefiches().getSubset(subsetKey2);
        int length = subset1.getTicketLength();
        openTag("croisement");
        for (int i = 0; i < length; i++) {
            SubsetItem subsetItem = subset1.getSubsetItemByTicket(i);
            if (subsetItem != null) {
                CroisementItem croisementItem = croisement.getCroisementItem(subsetKey1, subsetItem.getTicket());
                int count = croisementItem.getCount();
                if (count == 0) {
                    continue;
                }
                int id1 = subsetItem.getIdsubset();
                for (int j = 0; j < count; j++) {
                    int ticket2 = croisementItem.getTicket(j);
                    int poids = croisementItem.getPoids(j);
                    SubsetItem subsetItem2 = subset2.getSubsetItemByTicket(ticket2);
                    if (subsetItem2 != null) {
                        int id2 = subsetItem2.getIdsubset();
                        if ((isUnique) && (id2 <= id1)) {
                            continue;
                        }
                        CroisementItem croisementItem2 = croisement.getCroisementItem(subsetKey2, ticket2);
                        appendCroisementItem(id1, id2, poids, croisementItem2.indexOfTicket(i), j);
                        croisementCount++;
                    }
                }
            }
        }
        closeTag("croisement");
        return croisementCount;
    }

    private void appendCroisementItem(int id1, int id2, int poids, int pos1, int pos2) throws IOException {
        startOpenTag("ci");
        addAttribute("id1", id1);
        addAttribute("id2", id2);
        if (poids > 1) {
            addAttribute("poids", poids);
        }
        addAttribute("pos1", pos1);
        addAttribute("pos2", pos2);
        closeEmptyTag();
    }

}
