/* BasedefichesLib_Xml - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.DocumentSubsetEditor;
import net.mapeadores.basedefiches.association.ExistingBasenameException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class DocumentSubsetDOMReader extends BasedefichesDOMReader {

    private DocumentSubsetEditor documentSubsetEditor;
    private DocumentSubsetXmlErrorHandler errorHandler;

    public DocumentSubsetDOMReader(DocumentSubsetEditor documentSubsetEditor, DocumentSubsetXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.documentSubsetEditor = documentSubsetEditor;
        this.errorHandler = errorHandler;
    }

    public void fillDocumentSubset(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("document")) {
                    updateDocument(el);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void updateDocument(Element element_xml) {
        String idsubsetString = element_xml.getAttribute("idsubset");
        try {
            int idsubset = Integer.parseInt(idsubsetString);
            String basename = element_xml.getAttribute("basename");
            if (basename.length() > 0) {
                Document document = (Document) documentSubsetEditor.getDocumentSubset().getSubsetItemById(idsubset);
                if (document != null) {
                    try {
                        documentSubsetEditor.setBasename(document, basename);
                    } catch (ExistingBasenameException ebe) {
                        errorHandler.basenameDoublonError(basename);
                    } catch (ParseException pe) {
                        errorHandler.wrongBasenameError(basename);
                    }
                }
            }
        } catch (NumberFormatException nfe) {
            errorHandler.wrongIdsubetError(idsubsetString);
        }
    }

}
