/* BasedefichesLib_Xml - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Idsphere;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadataEditor;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SphereDOMReader extends BasedefichesDOMReader {

    private SphereEditor sphereEditor;
    private SphereMetadataEditor sphereMetadataEditor;
    private SphereXmlErrorHandler errorHandler;

    public SphereDOMReader(SphereXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.errorHandler = errorHandler;
    }

    public void setSphereEditor(SphereEditor sphereEditor) {
        this.sphereEditor = sphereEditor;
        this.sphereMetadataEditor = sphereEditor.getSphereMetadataEditor();
    }

    public void fillSphere(Element element_xml) {
        boolean metadone = false;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if ((!metadone) && (tagname.equals("metadata"))) {
                    metadone = true;
                    fillMetadata(element);
                } else if (tagname.equals("redacteur")) {
                    String idsphereString = element.getAttribute("idsphere");
                    try {
                        int idsphere = Idsphere.parse(idsphereString);
                        Redacteur redacteur = sphereEditor.createRedacteur(idsphere);
                        String actifString = element.getAttribute("actif");
                        if (actifString.equals("0")) {
                            sphereEditor.setActif(redacteur, false);
                        }
                        NodeList listered = element.getChildNodes();
                        for (int j = 0; j < listered.getLength(); j++) {
                            if (listered.item(j).getNodeType() == Node.ELEMENT_NODE) {
                                Element elementred = (Element) listered.item(j);
                                String tagnamered = elementred.getTagName();
                                if (tagnamered.equals("personne-bean")) {
                                    PersonneBean personneBean = BeanToXML.toPersonneBean(elementred);
                                    sphereEditor.setPersonneBean(redacteur, personneBean);
                                } else if (tagnamered.equals("courriel-bean")) {
                                    CourrielBean courrielBean = null;
                                    try {
                                        courrielBean = BeanToXML.toCourrielBean(elementred);

                                    } catch (IllegalArgumentException iae) {
                                        errorHandler.wrongAddrSpecError(idsphereString, elementred.getAttribute("addr-spec"));
                                    }
                                    sphereEditor.setCourrielBean(redacteur, courrielBean);
                                } else {
                                    errorHandler.unknownTagWarning(tagname);
                                }
                            }
                        }
                    } catch (ParseException pe) {
                        errorHandler.wrongIdsphereError(idsphereString);

                    } catch (ExistingIdException eie) {
                        errorHandler.redacteurDoublonError(idsphereString);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }

    }

    private void fillMetadata(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("intitule")) {
                    addIntitule(el, sphereMetadataEditor);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

}
