/* BasedefichesLib_Xml - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.filters.SubsetItemFilter;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadata;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class SphereXMLWriter extends UtilXMLWriter {

    public SphereXMLWriter() {
    }

    public void appendSphere(Sphere sphere) throws IOException {
        appendSphere(sphere, null);
    }

    public void appendSphere(Sphere sphere, SubsetItemFilter subsetItemFilter) throws IOException {
        openTag("sphere");
        appendMetadata(sphere.getSphereMetadata());
        for (int i = 0; i < sphere.getTicketLength(); i++) {
            Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
            if (testSubsetItem(redacteur, subsetItemFilter)) {
                appendRedacteur(redacteur);
            }
        }
        closeTag("sphere");
    }

    private boolean testSubsetItem(SubsetItem subsetItem, SubsetItemFilter subsetItemFilter) {
        if (subsetItem == null) {
            return false;
        }
        if (subsetItemFilter == null) {
            return true;
        }
        return subsetItemFilter.containsSubsetItem(subsetItem.getIdsubset());
    }

    private void appendMetadata(SphereMetadata sphereMetadata) throws IOException {
        openTag("metadata");
        BasedefichesXMLUtils.appendIntituleList(sphereMetadata, this);
        closeTag("metadata");
    }

    private void appendRedacteur(Redacteur redacteur) throws IOException {
        boolean actif = redacteur.isActif();
        String actifAttribute = (actif) ? "1" : "0";
        startOpenTag("redacteur");
        addAttribute("idsphere", redacteur.getRedacteurKey().getIdsphereString());
        addAttribute("actif", actifAttribute);
        endOpenTag();
        PersonneBean personneBean = redacteur.getPersonneBean();
        BeanToXML.addXML(this, personneBean);
        CourrielBean courrielBean = redacteur.getCourrielBean();
        if (courrielBean != null) {
            BeanToXML.addXML(this, courrielBean);
        }
        closeTag("redacteur");
    }

}
