/* BasedefichesLib_Xml - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.filters.SubsetItemFilter;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.util.langinteger.LangIntegerUtils;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusXMLWriter extends UtilXMLWriter {

    public ThesaurusXMLWriter() {
    }

    public void appendThesaurus(Thesaurus thesaurus) throws IOException {
        appendThesaurus(thesaurus, null);
    }

    public void appendThesaurus(Thesaurus thesaurus, SubsetItemFilter subsetItemFilter) throws IOException {
        startOpenTag("thesaurus");
        addAttribute("thesaurus-type", ThesaurusUtils.thesaurusTypeToString(thesaurus.getThesaurusMetadata().getThesaurusType()));
        endOpenTag();
        appendMetadata(thesaurus.getThesaurusMetadata());
        MotcleList firstLevel = thesaurus.getFirstLevelList();
        for (int i = 0; i < firstLevel.getMotcleCount(); i++) {
            appendMotcle(firstLevel.getMotcle(i), subsetItemFilter);
        }
        closeTag("thesaurus");
    }

    private void appendMetadata(ThesaurusMetadata thesaurusMetadata) throws IOException {
        startOpenTag("metadata");
        int[] array = thesaurusMetadata.getAuthorizedLangIntegerArray();
        if (array != null) {
            addAttribute("thesaurus-langs", LangIntegerUtils.toString(array, ';'));
        }
        endOpenTag();
        BasedefichesXMLUtils.appendIntituleList(thesaurusMetadata, this);
        closeTag("metadata");
    }

    private void appendMotcle(Motcle motcle, SubsetItemFilter subsetItemFilter) throws IOException {
        if ((subsetItemFilter != null) && (!subsetItemFilter.containsSubsetItem(motcle.getIdths()))) {
            return;
        }
        startOpenTag("motcle");
        addAttribute("idths", motcle.getIdths());
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            addAttribute("idalpha", idalpha);
        }
        endOpenTag();
        addLibelleList(motcle);
        MotcleList children = motcle.getChildList();
        for (int i = 0; i < children.getMotcleCount(); i++) {
            appendMotcle(children.getMotcle(i), subsetItemFilter);
        }
        closeTag("motcle");
    }

}
