/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.Extraction;
import net.mapeadores.basedefiches.extraction.MotcleFilter;
import net.mapeadores.basedefiches.extraction.MotcleFilterWrapper;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.api.GroupClause;
import net.mapeadores.basedefiches.extraction.api.IntituleClause;
import net.mapeadores.basedefiches.extraction.api.PackClause;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.extraction.builders.ExtraitCorpusDefListBuilder;
import net.mapeadores.basedefiches.extraction.builders.FicheFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.GroupClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.IntituleClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.PackClauseBuilder;
import net.mapeadores.basedefiches.extraction.filterinfo.FilterInfoList;
import net.mapeadores.basedefiches.extraction.filterinfo.FilterInfoListBuilder;
import net.mapeadores.basedefiches.selection.AlbumSelectOptions;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.conditions.ConditionGroup;
import net.mapeadores.util.conditions.Conditions;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMReader;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDOMReader implements ExtractionConstants {

    private Basedefiches basedefiches;
    private FicheEligibility ficheEligibility;
    private LocalisationLists localisationLists;

    public ExtractionDOMReader(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        this.basedefiches = basedefiches;
        this.localisationLists = localisationLists;
        this.ficheEligibility = ficheEligibility;
    }

    public Extraction readExtraction(Element element_xml) {
        ExtraitCorpusDef dynamicExtraitCorpusDef = null;
        MotcleFilterWrapper dynamicMotcleFilterWrapper = null;
        Extraction extraction = new Extraction(basedefiches, localisationLists, ficheEligibility);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("static")) {
                    initStaticFilterList(extraction, element);
                } else if ((tagname.equals("extraitcorpus")) && (dynamicExtraitCorpusDef == null)) {
                    dynamicExtraitCorpusDef = initExtraitCorpusDef(element);
                } else if ((tagname.equals("extraitthesaurus")) && (dynamicMotcleFilterWrapper == null)) {
                    dynamicMotcleFilterWrapper = getMotcleFilterWrapper(element);
                }
            }
        }
        extraction.setDynamicExtraitCorpusDef(dynamicExtraitCorpusDef);
        extraction.setDynamicMotcleFilterWrapper(dynamicMotcleFilterWrapper);
        return extraction;
    }

    private void initStaticFilterList(Extraction extraction, Element element_xml) {
        ExtraitCorpusDefListBuilder builder = new ExtraitCorpusDefListBuilder();
        ArrayList motcleFilterWrapperList = new ArrayList();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("extraitcorpus")) {
                    builder.addExtraitCorpusDef(initExtraitCorpusDef(element));
                } else if (tagname.equals("extraitthesaurus")) {
                    motcleFilterWrapperList.add(getMotcleFilterWrapper(element));
                }
            }
        }
        extraction.initStaticCorpusExtractorList(builder.toExtraitCorpusDefList());
        extraction.initStaticThesaurusExtractor(motcleFilterWrapperList);
    }

    public MotcleFilterWrapper getMotcleFilterWrapper(Element element_xml) {
        MotcleFilter motcleFilter = new MotcleFilter();
        String tagName = element_xml.getAttribute("tag-name");
        tagName = testTagName(tagName);
        motcleFilter.setTagName(tagName);
        MotcleFilterWrapper motcleFilterWrapper = new MotcleFilterWrapper(motcleFilter);
        NodeList liste = element_xml.getChildNodes();
        boolean motcleInit = false;
        boolean withSelect = false;
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if ((!motcleInit) && (tagname.equals("idmotcle"))) {
                    motcleInit = true;
                } else if ((!motcleInit) && (tagname.equals("motcle"))) {
                    motcleInit = true;
                    initMotcleFilter(motcleFilter, element);
                } else if (tagname.equals("motcle-select")) {
                    withSelect = true;
                    motcleFilterWrapper.addMotcleSelectOptions(getMotcleSelectOptions(element));
                } else if (tagname.equals("intitule")) {
                    motcleFilter.setWithIntituleThesaurus(true);
                }
            }
        }
        if (!withSelect) {
            MotcleSelectOptions options = getMotcleSelectOptions(element_xml.getAttribute("thesaurus"));
            if (options != null) {
                motcleFilterWrapper.addMotcleSelectOptions(options);
            }
        }
        return motcleFilterWrapper;
    }

    private FicheFilter initFicheFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                boolean isEmpty = isEmpty(element);
                if (tagname.equals("fiche")) {
                    String tagNameAttribute = element.getAttribute("tag-name");
                    if (isEmpty) {
                        return FicheFilterBuilder.newFicheInstance(basedefiches, ficheEligibility, tagNameAttribute);
                    } else {
                        FilterInfoListBuilder ficheInfoListBuilder = new FilterInfoListBuilder();
                        initFieldFilter(ficheInfoListBuilder, element);
                        FicheFilterBuilder builder = new FicheFilterBuilder();
                        initIncludeFilter(builder, element);
                        builder.setFicheEligibility(ficheEligibility);
                        builder.updateFilterInfoList(ficheInfoListBuilder.toFilterInfoList());
                        builder.parseTagName(tagNameAttribute);
                        return builder.toFicheFilter();
                    }
                } else if (tagname.equals("idfiche")) {
                    String tagNameAttribute = element.getAttribute("tag-name");
                    return FicheFilterBuilder.newIdFicheInstance(tagNameAttribute);
                }
            }
        }
        return FicheFilterBuilder.newAbregeFicheInstance(ficheEligibility, null);
    }

    public static FicheSelectOptions getFicheSelectOptions(String s) {
        SubsetKey[] corpusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_CORPUS, s);
        if (corpusKeyArray != null) {
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            return ficheSelectOptions;
        } else {
            return null;
        }
    }

    public static FicheSelectOptions getFicheSelectOptions(Element element_xml) {
        FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        List motcleSelectOptionsList = new ArrayList();
        List redacteurKeyList = new ArrayList();
        List subsetKeyList = new ArrayList();
        if (isEmpty(element_xml)) {
            return ficheSelectOptions;
        }
        StringBuilder corpusNameBuffer = new StringBuilder();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("corpus")) {
                    String corpusName = XMLUtils.getData(element);
                    if (corpusName.length() > 0) {
                        if (corpusNameBuffer.length() > 0) {
                            corpusNameBuffer.append(';');
                        }
                        corpusNameBuffer.append(corpusName);
                    }
                } else if (tagname.equals("range")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        ficheSelectOptions.setRangeList(rangeList);
                    }
                } else if (tagname.equals("entete-conditions")) {
                    ConditionGroup titreConditionGroup = initEntete(element);
                    if ((titreConditionGroup != null) && (titreConditionGroup.getState() != ConditionGroup.U)) {
                        ficheSelectOptions.setTitreConditionGroup(titreConditionGroup);
                    }
                } else if (tagname.equals("motcle-select")) {
                    MotcleSelectOptions motcleSelectOptions = getMotcleSelectOptions(element);
                    motcleSelectOptionsList.add(motcleSelectOptions);
                } else if (tagname.equals("motcle-select-mode")) {
                    short mode = getMotcleSelectMode(element);
                    if (mode != 0) {
                        ficheSelectOptions.setMotcleSelectMode(mode);
                    }
                } else if (tagname.equals("redacteurs")) {
                    initRedacteurList(element, redacteurKeyList, subsetKeyList);
                }
            }
        }
        int motcleSize = motcleSelectOptionsList.size();
        if (motcleSize > 0) {
            MotcleSelectOptions[] motcleSelectOptionsArray = new MotcleSelectOptions[motcleSize];
            for (int i = 0; i < motcleSize; i++) {
                motcleSelectOptionsArray[i] = (MotcleSelectOptions) motcleSelectOptionsList.get(i);
            }
            ficheSelectOptions.setMotcleSelectOptionsArray(motcleSelectOptionsArray);
        }
        if (corpusNameBuffer.length() > 0) {
            SubsetKey[] corpusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_CORPUS, corpusNameBuffer.toString());
            if (corpusKeyArray != null) {
                ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            }
        }
        int size = redacteurKeyList.size();
        if (size > 0) {
            RedacteurKey[] redacteurKeyArray = new RedacteurKey[size];
            for (int i = 0; i < size; i++) {
                redacteurKeyArray[i] = (RedacteurKey) redacteurKeyList.get(i);
            }
            ficheSelectOptions.setRedacteurKeyArray(redacteurKeyArray);
        }
        int subsetSize = subsetKeyList.size();
        if (subsetSize > 0) {
            SubsetKey[] subsetKeyArray = new SubsetKey[subsetSize];
            for (int i = 0; i < subsetSize; i++) {
                subsetKeyArray[i] = (SubsetKey) subsetKeyList.get(i);
            }
            ficheSelectOptions.setSphereKeyArray(subsetKeyArray);
        }
        return ficheSelectOptions;
    }

    private static void initRedacteurList(Element element_xml, List redacteurKeyList, List subsetKeyList) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("redacteur")) {
                    String sphere = element.getAttribute("sphere");
                    String idsphereString = element.getAttribute("idsphere");
                    if (sphere.length() == 0) {
                        sphere = null;
                    }
                    try {
                        redacteurKeyList.add(RedacteurKey.parse(sphere + "_" + idsphereString));
                    } catch (java.text.ParseException pe) {
                    }
                } else if (tagname.equals("sphere")) {
                    String sphereName = XMLUtils.getData(element);
                    if (sphereName.length() > 0) {
                        try {
                            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName);
                            subsetKeyList.add(sphereKey);
                        } catch (java.text.ParseException pe) {
                        }
                    }
                }
            }
        }
    }

    private static short getMotcleSelectMode(Element element_xml) {
        String modeString = element_xml.getAttribute("mode");
        if (modeString.equals("and")) {
            return Conditions.MODE_AND;
        } else if (modeString.equals("or")) {
            return Conditions.MODE_OR;
        } else {
            return 0;
        }
    }

    private static ConditionGroup initEntete(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("condition-group")) {
                    return ConditionsXMLStorage.getConditionGroup(element);
                }
            }
        }
        return null;
    }

    public static MotcleSelectOptions getMotcleSelectOptions(String s) {
        SubsetKey[] thesaurusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_THESAURUS, s);
        if (thesaurusKeyArray != null) {
            MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            return motcleSelectOptions;
        } else {
            return null;
        }
    }

    public static AlbumSelectOptions getAlbumSelectOptions(Element element_xml) {
        AlbumSelectOptions albumSelectOptions = new AlbumSelectOptions();
        if (isEmpty(element_xml)) {
            return albumSelectOptions;
        }
        StringBuilder albumNameBuffer = new StringBuilder();
        List<String> albumDimNameList = new ArrayList<String>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("album")) {
                    String albumName = XMLUtils.getData(element);
                    if (albumName.length() > 0) {
                        if (albumNameBuffer.length() > 0) {
                            albumNameBuffer.append(';');
                        }
                        albumNameBuffer.append(albumName);
                    }
                } else if (tagname.equals("albumdim")) {
                    String albumDimName = XMLUtils.getData(element);
                    if (albumDimName.length() > 0) {
                        albumDimNameList.add(albumDimName);
                    }
                }
            }
        }
        if (albumNameBuffer.length() > 0) {
            SubsetKey[] albumKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_ALBUM, albumNameBuffer.toString());
            if (albumKeyArray != null) {
                albumSelectOptions.setAlbumKeyArray(albumKeyArray);
            }
        }
        int size = albumDimNameList.size();
        if (size > 0) {
            albumSelectOptions.setAlbumDimNameArray(albumDimNameList.toArray(new String[size]));
        }
        return albumSelectOptions;
    }

    public static MotcleSelectOptions getMotcleSelectOptions(Element element_xml) {
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        if (isEmpty(element_xml)) {
            return motcleSelectOptions;
        }
        StringBuilder thesaurusNameBuffer = new StringBuilder();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("thesaurus")) {
                    String nomthesaurus = XMLUtils.getData(element);
                    if (nomthesaurus.length() > 0) {
                        if (thesaurusNameBuffer.length() > 0) {
                            thesaurusNameBuffer.append(';');
                        }
                        thesaurusNameBuffer.append(nomthesaurus);
                    }
                } else if (tagname.equals("range")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        motcleSelectOptions.setIdthsRangeList(rangeList);
                    }
                } else if (tagname.equals("level")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        motcleSelectOptions.setLevelRangeList(rangeList);
                    }
                } else if (tagname.equals("condition-group")) {
                    ConditionGroup conditionGroup = ConditionsXMLStorage.getConditionGroup(element);
                    if (conditionGroup.getState() != ConditionGroup.U) {
                        motcleSelectOptions.initConditionGroup(conditionGroup);
                    }
                }
            }
        }
        if (thesaurusNameBuffer.length() > 0) {
            SubsetKey[] thesaurusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_THESAURUS, thesaurusNameBuffer.toString());
            if (thesaurusKeyArray != null) {
                motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            }
        }
        return motcleSelectOptions;
    }

    private void initIncludeFilter(FicheFilterBuilder builder, Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        FicheSelectorBundle ficheSelectorBundle = new FicheSelectorBundle();
        MotcleSelectorBundle motcleSelectorBundle = new MotcleSelectorBundle();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(j);
                String tagname = element.getTagName();
                if (tagname.equals("liage")) {
                    if (isEmpty(element)) {
                        ficheSelectorBundle.addAllLiage();
                    } else {
                        DOMReader.readChildren(element, ficheSelectorBundle);
                    }
                } else if (tagname.equals("indexation")) {
                    if (isEmpty(element)) {
                        motcleSelectorBundle.setAll();
                    } else {
                        DOMReader.readChildren(element, motcleSelectorBundle);
                    }
                } else if (tagname.equals("association")) {
                    builder.setAssociationInclude(true);
                } else {
                    boolean test = ficheSelectorBundle.readExtraitCorpus(element);
                    if (!test) {
                        test = motcleSelectorBundle.readExtraitThesaurus(element);
                    }
                }
            }
        }
        if (ficheSelectorBundle.isNotEmpty()) {
            builder.setLiageExtraitCorpusDefList(ficheSelectorBundle.toExtraitCorpusDefList());
        }
        if (motcleSelectorBundle.isNotEmpty()) {
            builder.setIndexationMotcleSelector(motcleSelectorBundle.getMotcleSelector());
        }
    }

    public void initMotcleFilter(MotcleFilter motcleFilter, Element element_xml) {
        if ((element_xml == null) || (isEmpty(element_xml))) {
            motcleFilter.setWithLib(true);
            return;
        }
        int empty = 0;
        FicheSelectorBundle ficheSelectorBundle = new FicheSelectorBundle();
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(j);
                String tagname = element.getTagName();
                if (tagname.equals("lib")) {
                    motcleFilter.setWithLib(true);
                } else if (tagname.equals("level")) {
                    motcleFilter.setWithLevel(true);
                } else if (tagname.equals("next")) {
                    MotcleFilter nextFilter = new MotcleFilter();
                    initMotcleFilter(nextFilter, getMotcleElement(element));
                    motcleFilter.setNextFilter(nextFilter);
                } else if (tagname.equals("previous")) {
                    MotcleFilter previousFilter = new MotcleFilter();
                    initMotcleFilter(previousFilter, getMotcleElement(element));
                    motcleFilter.setPreviousFilter(previousFilter);
                } else if ((tagname.equals("children")) || (tagname.equals("motcle"))) {
                    if (isEmpty(element)) {
                        empty = empty | 1;
                    } else {
                        MotcleFilter childrenFilter = new MotcleFilter();
                        Element motcleElement = element;
                        if (tagname.equals("children")) {
                            motcleElement = getMotcleElement(element);
                        }
                        initMotcleFilter(childrenFilter, motcleElement);
                        motcleFilter.setChildrenFilter(childrenFilter);
                    }
                } else if (tagname.equals("parent")) {
                    if (isEmpty(element)) {
                        empty = empty | 2;
                    } else {
                        MotcleFilter parentFilter = new MotcleFilter();
                        initMotcleFilter(parentFilter, getMotcleElement(element));
                        motcleFilter.setParentFilter(parentFilter);
                    }
                } else if (tagname.equals("indexation")) {
                    if (isEmpty(element)) {
                        ficheSelectorBundle.addAllLiage();
                    } else {
                        DOMReader.readChildren(element, ficheSelectorBundle);
                    }
                } else {
                    ficheSelectorBundle.readExtraitCorpus(element);
                }
            }
        }
        if (ficheSelectorBundle.isNotEmpty()) {
            motcleFilter.setFicheEligibility(ficheEligibility);
            motcleFilter.setIndexationExtraitCorpusDefList(ficheSelectorBundle.toExtraitCorpusDefList());
        }
        if ((empty & 1) != 0) {
            motcleFilter.setChildrenFilter(motcleFilter);
            if ((empty & 2) != 0) {
                MotcleFilter parentFilter = new MotcleFilter();
                parentFilter.setWithLib(true);
                motcleFilter.setParentFilter(parentFilter);
            }
        } else if ((empty & 2) != 0) {
            motcleFilter.setParentFilter(motcleFilter);
        }
    }

    private Element getMotcleElement(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(j);
                String tagname = element.getTagName();
                if (tagname.equals("motcle")) {
                    return element;
                }
            }
        }
        return null;
    }

    private void initFieldFilter(FilterInfoListBuilder builder, Element element_xml) {
        if (isEmpty(element_xml)) {
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                testFieldFiltering(builder, element);
            }
        }
    }

    private void testFieldFiltering(FilterInfoListBuilder builder, Element element) {
        String tagname = element.getTagName();
        if (tagname.equals("entete")) {
            builder.addEnteteFilter();
        } else if (tagname.equals("corpsdefiche")) {
            builder.addCorpsdeficheFilter();
        } else if (tagname.equals("titre")) {
            builder.addFieldKey(FieldKey.TITRE);
        } else if (tagname.equals("soustitre")) {
            builder.addFieldKey(FieldKey.SOUSTITRE);
        } else if (tagname.equals("propriete")) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.PROPRIETE_CATEGORY, type);
            }
        } else if (tagname.equals("redacteurs")) {
            builder.addFieldKey(FieldKey.REDACTEURS);
        } else if (tagname.equals("information")) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.INFORMATION_CATEGORY, type);
            }
        } else if (tagname.equals("texte")) {
            builder.addFieldKey(FieldKey.TEXTE);
        } else if (tagname.equals("annexe")) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.ANNEXE_CATEGORY, type);
            }
        } else if (tagname.equals("lot")) {
            String tagName = element.getAttribute("tag-name");
            FilterInfoListBuilder builder2 = new FilterInfoListBuilder();
            initFieldFilter(builder2, element);
            FilterInfoList infoList = builder2.toFilterInfoList();
            if (infoList.getFilterInfoCount() > 0) {
                builder.addLotFilter(tagName, infoList);
            }
        } else if ((tagname.equals("langue")) || (tagname.equals("lang"))) {
            builder.addFieldKey(FieldKey.LANG);
        } else if (tagname.equals("chrono")) {
            builder.addChronoFilter();
        }
    }

    private static boolean isEmpty(Element element) {
        int size = element.getChildNodes().getLength();
        if (size == 0) {
            return true;
        }
        if (size > 1) {
            return false;
        }
        Node node = element.getFirstChild();
        if (node.getNodeType() != Node.TEXT_NODE) {
            return false;
        }
        String data = ((Text) node).getData().trim();
        if (data.length() == 0) {
            return true;
        } else {
            return false;
        }
    }


    private class FicheSelectorBundle implements ElementHandler {

        ExtraitCorpusDefListBuilder builder = new ExtraitCorpusDefListBuilder();
        FicheSelector ficheSelector;

        void addAllLiage() {
            initFicheSelector();
            InternalExtraitCorpusDef extraitCorpusDef = new InternalExtraitCorpusDef();
            extraitCorpusDef.setFicheFilter(FicheFilterBuilder.newAbregeFicheInstance(ficheEligibility, null));
            builder.addExtraitCorpusDef(extraitCorpusDef);
        }

        boolean isNotEmpty() {
            return (builder.size() != 0);
        }

        public ExtraitCorpusDefList toExtraitCorpusDefList() {
            return builder.toExtraitCorpusDefList();
        }

        FicheSelector getFicheSelector() {
            return ficheSelector;
        }

        private void initFicheSelector() {
            if (ficheSelector == null) {
                ficheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
                ficheSelector.setFicheEligibility(ficheEligibility);
            }
        }

        public void readElement(Element element) {
            readExtraitCorpus(element);
        }

        public boolean readExtraitCorpus(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("extraitcorpus")) {
                initFicheSelector();
                ExtraitCorpusDef extraitCorpusDef = initExtraitCorpusDef(element);
                builder.addExtraitCorpusDef(extraitCorpusDef);
                return true;
            }
            return false;
        }

    }


    private class MotcleSelectorBundle implements ElementHandler {

        boolean all = false;
        MotcleSelector motcleSelector;

        void setAll() {
            all = true;
        }

        boolean isNotEmpty() {
            if (motcleSelector != null) {
                return true;
            }
            if (all) {
                return true;
            }
            return false;
        }

        MotcleSelector getMotcleSelector() {
            if (motcleSelector != null) {
                return motcleSelector;
            }
            if (all) {
                initMotcleSelector();
                MotcleFilter motcleFilter = new MotcleFilter();
                motcleFilter.setWithLib(true);
                motcleSelector.addMotcleFilterWrapper(new MotcleFilterWrapper(motcleFilter));
                return motcleSelector;
            }
            return null;
        }

        private void initMotcleSelector() {
            if (motcleSelector == null) {
                motcleSelector = new MotcleSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            }
        }

        public void readElement(Element element) {
            readExtraitThesaurus(element);
        }

        public boolean readExtraitThesaurus(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("extraitthesaurus")) {
                initMotcleSelector();
                motcleSelector.addMotcleFilterWrapper(getMotcleFilterWrapper(element));
                return true;
            }
            return false;
        }

    }

    private static String testTagName(String tagName) {
        if (tagName.length() == 0) {
            return null;
        }
        if (tagName.equals("NULL")) {
            return tagName;
        }
        if (StringUtils.isTechnicalName(tagName, true)) {
            return tagName;
        }
        return null;
    }

    private InternalExtraitCorpusDef initExtraitCorpusDef(Element element_xml) {
        InternalExtraitCorpusDef extraitCorpusDef = new InternalExtraitCorpusDef();
        String extraitName = element_xml.getAttribute("extrait-name");
        if (extraitName.length() > 0) {
            extraitCorpusDef.setExtraitName(extraitName);
        }
        String tagName = element_xml.getAttribute("tag-name");
        tagName = BuilderUtils.testTagName(tagName, EXTRAITCORPUS_DEFAULT_TAGNAME);
        extraitCorpusDef.setTagName(tagName);
        FicheSelectOptions attributeSelectionOptions = null;
        String corpusAttribute = element_xml.getAttribute("corpus");
        if (corpusAttribute.length() > 0) {
            attributeSelectionOptions = getFicheSelectOptions(corpusAttribute);
        }
        GroupClause attributeGroupClause = null;
        String sortAttribute = element_xml.getAttribute("sort");
        if (sortAttribute.length() > 0) {
            attributeGroupClause = GroupClauseBuilder.getGroupClause(sortAttribute, true);
        }
        GroupClause groupClause = null;
        PackClause packClause = null;
        IntituleClause intituleClause = null;
        List selectOptionsList = new ArrayList();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    groupClause = initGroupClause(element);
                } else if (tagname.equals("fiche-select")) {
                    selectOptionsList.add(getFicheSelectOptions(element));
                } else if (tagname.equals("intitule")) {
                    intituleClause = initIntituleClause(element);
                } else if (tagname.equals("pack")) {
                    packClause = initPackClause(element);
                }
            }
        }
        int optionsSize = selectOptionsList.size();
        if ((optionsSize == 0) && (attributeSelectionOptions != null)) {
            selectOptionsList.add(attributeSelectionOptions);
        }
        if (groupClause == null) {
            groupClause = attributeGroupClause;
        }
        FicheFilter ficheFilter = initFicheFilter(element_xml);

        extraitCorpusDef.setGroupClause(groupClause);
        extraitCorpusDef.setPackClause(packClause);
        extraitCorpusDef.setIntituleClause(intituleClause);
        extraitCorpusDef.setFicheFilter(ficheFilter);
        extraitCorpusDef.setFicheSelectOptionsArray(selectOptionsList);
        return extraitCorpusDef;
    }

    private static PackClause initPackClause(Element element_xml) {
        String sizeString = element_xml.getAttribute("size");
        PackClauseBuilder builder = PackClauseBuilder.newInstance(sizeString);
        if (builder == null) {
            return null;
        }
        String moduloString = element_xml.getAttribute("modulo");
        if (moduloString.length() > 0) {
            builder.parseModulo(moduloString);
        }
        String tagName = element_xml.getAttribute("tag-name");
        if (tagName.length() > 0) {
            builder.parseTagName(tagName);
        }
        return builder.toPackClause();
    }

    private static IntituleClause initIntituleClause(Element element_xml) {
        return IntituleClauseBuilder.DEFAULT;
    }

    private static GroupClause initGroupClause(Element element_xml) {
        String typeString = element_xml.getAttribute("type");
        GroupClauseBuilder builder = GroupClauseBuilder.newInstance(typeString);
        if (builder == null) {
            return null;
        }
        String params = element_xml.getAttribute("params");
        if (params.length() == 0) {
            params = element_xml.getAttribute("mode");
        }
        if (params.length() > 0) {
            builder.parseGroupParams(params);
        }
        String sortString = element_xml.getAttribute("sort");
        if (sortString.length() > 0) {
            builder.parseSortOrder(sortString);
        }
        String tagName = element_xml.getAttribute("tag-name");
        if (tagName.length() > 0) {
            builder.parseTagName(tagName);
        }
        GroupClause subGroupClause = null;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    subGroupClause = initGroupClause(element);
                }
            }
        }
        if (subGroupClause != null) {
            builder.setSubGroupClause(subGroupClause);
        }
        GroupClause groupClause = builder.toGroupClause();
        return groupClause;
    }


    private static class InternalExtraitCorpusDef implements ExtraitCorpusDef {

        private String extraitName = null;
        private String tagName = EXTRAITCORPUS_DEFAULT_TAGNAME;
        private GroupClause groupClause;
        private PackClause packClause;
        private IntituleClause intituleClause;
        private FicheFilter ficheFilter;
        private FicheSelectOptions[] ficheSelectOptionsArray = null;

        public String getTagName() {
            return tagName;
        }

        public void setTagName(String tagName) {
            this.tagName = tagName;
        }

        public GroupClause getGroupClause() {
            return groupClause;
        }

        public void setGroupClause(GroupClause groupClause) {
            this.groupClause = groupClause;
        }

        public PackClause getPackClause() {
            return packClause;
        }

        public void setPackClause(PackClause packClause) {
            this.packClause = packClause;
        }

        public String getExtraitName() {
            return extraitName;
        }

        public void setExtraitName(String extraitName) {
            this.extraitName = extraitName;
        }

        public IntituleClause getIntituleClause() {
            return intituleClause;
        }

        public void setIntituleClause(IntituleClause intituleClause) {
            this.intituleClause = intituleClause;
        }

        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        public void setFicheFilter(FicheFilter ficheFilter) {
            this.ficheFilter = ficheFilter;
        }

        public int getFicheSelectOptionsCount() {
            if (ficheSelectOptionsArray == null) {
                return 0;
            }
            return ficheSelectOptionsArray.length;
        }

        public FicheSelectOptions getFicheSelectOptions(int i) {
            return ficheSelectOptionsArray[i];
        }

        void setFicheSelectOptionsArray(List selectOptionsList) {
            int size = selectOptionsList.size();
            if (size == 0) {
                ficheSelectOptionsArray = null;
                return;
            }
            ficheSelectOptionsArray = new FicheSelectOptions[size];
            for (int i = 0; i < size; i++) {
                ficheSelectOptionsArray[i] = (FicheSelectOptions) selectOptionsList.get(i);
            }

        }

    }

}
