/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.Extraction;
import net.mapeadores.basedefiches.extraction.MotcleFilter;
import net.mapeadores.basedefiches.extraction.MotcleFilterWrapper;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.CorpusExtractorList;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.DefaultFicheSelectOptionsList;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionXMLWriter extends AbstractExtractionXMLWriter {

    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    public ExtractionXMLWriter(Basedefiches basedefiches, LocalisationLists localisationLists) {
        super(basedefiches, localisationLists);
    }

    public void addExtraction(Extraction extraction, Date date, Object source) throws IOException {
        startOpenTag("extraction");
        addAttribute("date", simpleDateFormat.format(new Date()));
        endOpenTag();
        if (source != null) {
            addDynamic(extraction, source);
        }
        addStatic(extraction);
        closeTag("extraction");
    }

    private void addDynamic(Extraction extraction, Object source) throws IOException {
        if (source instanceof AbregeFiche) {
            addAbregeFicheDynamic(extraction.getDynamicExtraitCorpusDef(), (AbregeFiche) source);
        } else if (source instanceof Corpus) {
            addCorpusDynamic(extraction, (Corpus) source);
        } else if (source instanceof Motcle) {
            addMotcleDynamic(extraction.getDynamicMotcleFilterWrapper(), (Motcle) source);
        } else if (source instanceof Thesaurus) {
            addThesaurusDynamic(extraction.getDynamicMotcleFilterWrapper(), (Thesaurus) source);
        } else if (source instanceof AbregeFicheSortedList) {
            addSortedListDynamic(extraction.getDynamicExtraitCorpusDef(), (AbregeFicheSortedList) source);
        } else {
            throw new ClassCastException("unable to deal with the class " + source.getClass().getName());
        }
    }

    private void addAbregeFicheDynamic(ExtraitCorpusDef dynamicExtraitCorpusDef, AbregeFiche abregeFiche) throws IOException {
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = new CorpusExtractor(dynamicExtraitCorpusDef, null);
        corpusExtractor.add(abregeFiche, 0);
        ExtraitcorpusXMLWriter xmlWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(getAppendable());
        xmlWriter.setIndentLength(getCurrentIndentValue());
        xmlWriter.addExtraitcorpus(corpusExtractor);
    }

    private void addCorpusDynamic(Extraction extraction, Corpus corpus) throws IOException {
        ExtraitCorpusDef dynamicExtraitCorpusDef = extraction.getDynamicExtraitCorpusDef();
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = new CorpusExtractor(dynamicExtraitCorpusDef, null);
        if (dynamicExtraitCorpusDef.getFicheSelectOptionsCount() > 0) {
            SubsetKey corpusKey = corpus.getSubsetKey();
            SubsetKey[] corpusKeyArray = {corpusKey};
            DefaultFicheSelectOptionsList ficheSelectOptionsList = new DefaultFicheSelectOptionsList();
            for (int i = 0; i < dynamicExtraitCorpusDef.getFicheSelectOptionsCount(); i++) {
                FicheSelectOptions options = dynamicExtraitCorpusDef.getFicheSelectOptions(i);
                SubsetKey[] array = options.getCorpusKeyArray();
                boolean present = false;
                if (array == null) {
                    present = true;
                } else {
                    for (int j = 0; j < array.length; j++) {
                        if (array[j].equals(corpusKey)) {
                            present = true;
                            break;
                        }
                    }
                }
                if (present) {
                    FicheSelectOptions nvSO = new FicheSelectOptions(options);
                    nvSO.setCorpusKeyArray(corpusKeyArray);
                    ficheSelectOptionsList.addFicheSelectOptions(nvSO);
                }
            }
            if (ficheSelectOptionsList.getFicheSelectOptionsCount() == 0) {
                return;
            }
            FicheSelector ficheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            ficheSelector.setFicheEligibility(extraction.getFicheEligibility());
            ficheSelector.addFicheSelectOptionsList(ficheSelectOptionsList);
            for (int i = 0; i < corpus.getTicketLength(); i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche != null) {
                    if (ficheSelector.isSelected(abregeFiche)) {
                        corpusExtractor.add(abregeFiche, 0);
                    }
                }
            }

        } else {
            FicheEligibility ficheEligibility = extraction.getFicheEligibility();
            for (int i = 0; i < corpus.getTicketLength(); i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche != null) {
                    if ((ficheEligibility == null) || (ficheEligibility.accept(abregeFiche))) {
                        corpusExtractor.add(abregeFiche, 0);
                    }
                }
            }
        }
        ExtraitcorpusXMLWriter xmlWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(getAppendable());
        xmlWriter.setIndentLength(getCurrentIndentValue());
        xmlWriter.addExtraitcorpus(corpusExtractor);
    }

    private void addSortedListDynamic(ExtraitCorpusDef dynamicExtraitCorpusDef, AbregeFicheSortedList abregeFicheSortedList) throws IOException {
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = new CorpusExtractor(dynamicExtraitCorpusDef, null);
        int size = abregeFicheSortedList.getAbregeFicheCount();
        for (int i = 0; i < size; i++) {
            AbregeFiche abregeFiche = abregeFicheSortedList.getAbregeFicheAtAbsolutePosition(i);
            corpusExtractor.add(abregeFiche, 0);
        }
        ExtraitcorpusXMLWriter xmlWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(getAppendable());
        xmlWriter.setIndentLength(getCurrentIndentValue());
        xmlWriter.addExtraitcorpus(corpusExtractor);
    }

    private void addMotcleDynamic(MotcleFilterWrapper dynamicThesaurusFilter, Motcle motcle) throws IOException {
        if (dynamicThesaurusFilter == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(basedefiches, null);
        thesaurusExtractor.add(motcle, dynamicThesaurusFilter.getMotcleFilter(), 0);
        ExtraitthesaurusXMLWriter xmlWriter = new ExtraitthesaurusXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(getAppendable());
        xmlWriter.setIndentLength(getCurrentIndentValue());
        xmlWriter.addExtraitthesaurus(thesaurusExtractor);
    }

    private void addThesaurusDynamic(MotcleFilterWrapper dynamicThesaurusFilter, Thesaurus thesaurus) throws IOException {
        if (dynamicThesaurusFilter == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(basedefiches, null);
        if (dynamicThesaurusFilter.getMotcleSelectOptionsCount() > 0) {
            SubsetKey thesaurusKey = thesaurus.getSubsetKey();
            SubsetKey[] thesaurusKeyArray = {thesaurusKey};
            MotcleFilterWrapper wrapper = new MotcleFilterWrapper(dynamicThesaurusFilter.getMotcleFilter());
            for (int i = 0; i < dynamicThesaurusFilter.getMotcleSelectOptionsCount(); i++) {
                MotcleSelectOptions options = dynamicThesaurusFilter.getMotcleSelectOptions(i);
                SubsetKey[] array = options.getThesaurusKeyArray();
                boolean present = false;
                if (array == null) {
                    present = true;
                } else {
                    for (int j = 0; j < array.length; j++) {
                        if (array[j].equals(thesaurusKey)) {
                            present = true;
                            break;
                        }
                    }
                }
                if (present) {
                    MotcleSelectOptions nvSO = new MotcleSelectOptions(options);
                    nvSO.setThesaurusKeyArray(thesaurusKeyArray);
                    wrapper.addMotcleSelectOptions(nvSO);
                }
            }
            if (wrapper.getMotcleSelectOptionsCount() == 0) {
                return;
            }
            MotcleSelector motcleSelector = new MotcleSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            motcleSelector.addMotcleFilterWrapper(wrapper);
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle != null) {
                    MotcleFilter motcleFilter = motcleSelector.getMotcleFilter(motcle);
                    if (motcleFilter != null) {
                        thesaurusExtractor.add(motcle, motcleFilter, 0);
                    }
                }
            }
        } else {
            MotcleFilter motcleFilter = dynamicThesaurusFilter.getMotcleFilter();
            MotcleList levelList = thesaurus.getFirstLevelList();
            for (int i = 0; i < levelList.getMotcleCount(); i++) {
                Motcle motcle = levelList.getMotcle(i);
                thesaurusExtractor.add(motcle, motcleFilter, 0);
            }
        }
        ExtraitthesaurusXMLWriter xmlWriter = new ExtraitthesaurusXMLWriter(basedefiches, localisationLists);
        xmlWriter.setAppendable(getAppendable());
        xmlWriter.setIndentLength(getCurrentIndentValue());
        xmlWriter.addExtraitthesaurus(thesaurusExtractor);
    }

    private void addStatic(Extraction extraction) throws IOException {
        CorpusExtractorList corpusExtractorList = extraction.getStaticCorpusExtractorList();
        boolean containsExtraitcorpus = corpusExtractorList.containsFiche();
        ThesaurusExtractor thesaurusExtractor = extraction.getStaticThesaurusExtractor();
        boolean containsExtraitthesaurus = ((thesaurusExtractor != null) && (thesaurusExtractor.containsMotcle()));
        if ((!containsExtraitcorpus) && (!containsExtraitthesaurus)) {
            return;
        }
        openTag("static");
        if (containsExtraitcorpus) {
            ExtraitcorpusXMLWriter xmlWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
            xmlWriter.setAppendable(getAppendable());
            xmlWriter.setIndentLength(getCurrentIndentValue());
            int count = corpusExtractorList.getCorpusExtractorCount();
            for (int i = 0; i < count; i++) {
                CorpusExtractor corpusExtractor = corpusExtractorList.getCorpusExtractor(i);
                if (corpusExtractor.containsFiche()) {
                    xmlWriter.addExtraitcorpus(corpusExtractor);
                }
            }
        }
        if (containsExtraitthesaurus) {
            ExtraitthesaurusXMLWriter extraitthesaurusXMLWriter = new ExtraitthesaurusXMLWriter(basedefiches, localisationLists);
            extraitthesaurusXMLWriter.setAppendable(getAppendable());
            extraitthesaurusXMLWriter.setIndentLength(getCurrentIndentValue());
            extraitthesaurusXMLWriter.addExtraitthesaurus(thesaurusExtractor);
        }
        closeTag("static");
    }

    /**
     * Convertit une extraction en chaîne.
     * l'argument <code>prettyXml</code> indique s'il faut mettre tabulations et sauts de ligne.
     */
    public static String toString(Extraction extraction, Object extractionSource, boolean prettyXml) {
        StringBuilder buf = new StringBuilder();
        try {
            int nbretabulations = (prettyXml) ? 0 : -999;
            ExtractionXMLWriter xmlWriter = new ExtractionXMLWriter(extraction.getBasedefiches(), extraction.getLocalisationLists());
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(nbretabulations);
            xmlWriter.appendXMLDeclaration();
            xmlWriter.addExtraction(extraction, new Date(), extractionSource);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    /**
     * Convertit une extraction en objet StreamSource permettant une transformation XSLT
     */
    public static StreamSource toStreamSource(Extraction extraction, Object extractionSource) {
        String extractionString = toString(extraction, extractionSource, false);
        StringReader stringReader = new StringReader(extractionString);
        return new StreamSource(stringReader);
    }

}
