/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.extraction.MotcleExtractInfo;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitthesaurusXMLWriter extends AbstractExtractionXMLWriter {

    public ExtraitthesaurusXMLWriter(Basedefiches basedefiches, LocalisationLists localisationLists) {
        super(basedefiches, localisationLists);
    }

    public void addExtraitthesaurus(ThesaurusExtractor thesaurusExtractor) throws IOException {
        MotcleXMLWriter motcleXMLWriter = new MotcleXMLWriter(basedefiches, localisationLists);
        motcleXMLWriter.setAppendable(getAppendable());
        motcleXMLWriter.setIndentLength(getCurrentIndentValue() + 1);
        for (int j = 0; j < thesaurusExtractor.getThesaurusCount(); j++) {
            Thesaurus thesaurus = thesaurusExtractor.getThesaurus(j);
            String tagName = thesaurusExtractor.getTagName(j);
            if (tagName == null) {
                tagName = "extraitthesaurus";
            }
            boolean withTag = (!tagName.equals("NULL"));
            if (withTag) {
                startOpenTag(tagName);
                addAttribute("thesaurus", thesaurus.getSubsetKey().getSubsetName());
                addAttribute("rattachement", thesaurusExtractor.getRattachementType(j));
                endOpenTag();
            }
            MotcleExtractInfo[] array = thesaurusExtractor.getMotcleExtractInfoArray(j);
            for (int i = 0; i < array.length; i++) {
                motcleXMLWriter.addMotcle(array[i]);
            }
            if (thesaurusExtractor.isWithIntituleThesaurus(j)) {
                openTag("intitule");
                int length = localisationLists.getFormatLocaleCount();
                for (int i = 0; i < length; i++) {
                    int workingLang = localisationLists.getWorkingLangInteger(i);
                    Locale locale = localisationLists.getFormatLocale(i);
                    String lib = ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLang);
                    addLibElement(workingLang, lib);
                }
                closeTag("intitule");
            }
            if (withTag) {
                closeTag(tagName);
            }
        }
    }

}
