/* BasedefichesLib_Xml - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLWriterPart;
import java.io.IOException;
import net.mapeadores.basedefiches.corpus.fiche.CdataDiv;
import net.mapeadores.basedefiches.corpus.fiche.Code;
import net.mapeadores.basedefiches.corpus.fiche.Div;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.H;
import net.mapeadores.basedefiches.corpus.fiche.IdentifiedFicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.Insert;
import net.mapeadores.basedefiches.corpus.fiche.Li;
import net.mapeadores.basedefiches.corpus.fiche.Ln;
import net.mapeadores.basedefiches.corpus.fiche.P;
import net.mapeadores.basedefiches.corpus.fiche.ParagraphBlock;
import net.mapeadores.basedefiches.corpus.fiche.S;
import net.mapeadores.basedefiches.corpus.fiche.Table;
import net.mapeadores.basedefiches.corpus.fiche.Td;
import net.mapeadores.basedefiches.corpus.fiche.TextContentList;
import net.mapeadores.basedefiches.corpus.fiche.Tr;
import net.mapeadores.basedefiches.corpus.fiche.Ul;
import net.mapeadores.basedefiches.corpus.fiche.ZoneBlock;


/**
 *
 * @author Vincent Calame
 */
public class FicheBlockXMLPart extends XMLWriterPart {

    private boolean complete = false;
    private Corpus defaultCorpus;
    private Basedefiches basedefiches = null;

    public FicheBlockXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void setCompleteMode(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
        this.complete = (basedefiches != null);
    }

    public void setDefaultCorpus(Corpus defaultCorpus) {
        this.defaultCorpus = defaultCorpus;
    }

    public void addFicheBlock(FicheBlock ficheBlock) throws IOException {
        if (ficheBlock instanceof P) {
            addP((P) ficheBlock);
        } else if (ficheBlock instanceof H) {
            addH((H) ficheBlock);
        } else if (ficheBlock instanceof Ul) {
            addUl((Ul) ficheBlock);
        } else if (ficheBlock instanceof Code) {
            addCode((Code) ficheBlock);
        } else if (ficheBlock instanceof Table) {
            addTable((Table) ficheBlock);
        } else if (ficheBlock instanceof Insert) {
            addInsert((Insert) ficheBlock);
        } else if (ficheBlock instanceof Div) {
            addDiv((Div) ficheBlock);
        } else if (ficheBlock instanceof CdataDiv) {
            addCdataDiv((CdataDiv) ficheBlock);
        } else {
            throw new IllegalArgumentException("unknown FicheBlock Implementation");
        }
    }

    public void addFicheBlockList(FicheBlockList ficheBlockList) throws IOException {
        int size = ficheBlockList.getFicheBlockCount();
        for (int i = 0; i < size; i++) {
            addFicheBlock(ficheBlockList.getFicheBlock(i));
        }
    }

    public void addTextContentList(TextContentList contentList) throws IOException {
        int count = contentList.getTextContentCount();
        for (int i = 0; i < count; i++) {
            Object obj = contentList.getTextContent(i);
            if (obj instanceof String) {
                escape((String) obj);
            } else if (obj instanceof S) {
                addS((S) obj);
            }
        }
    }

    public void addP(P p) throws IOException {
        startOpenTag("p");
        addAttribute("type", p.getTypeString());
        addAttribute("source", p.getSource());
        addIdentifiersAttributes(p);
        endOpenTag(false);
        addTextContentList(p);
        closeTag("p", false, false);
    }

    public void addH(H h) throws IOException {
        startOpenTag("h");
        addAttribute("level", String.valueOf(h.getLevel()));
        addIdentifiersAttributes(h);
        endOpenTag(false);
        addTextContentList(h);
        closeTag("h", false, false);
    }

    public void addUl(Ul ul) throws IOException {
        startOpenTag("ul");
        addIdentifiersAttributes(ul);
        endOpenTag();
        int size = ul.getLiCount();
        for (int i = 0; i < size; i++) {
            addLi(ul.getLi(i));
        }
        closeTag("ul");
    }

    public void addLi(Li li) throws IOException {
        int size = li.getFicheBlockCount();
        if (size > 0) {
            startOpenTag("li");
            addIdentifiersAttributes(li);
            endOpenTag();
            addFicheBlockList(li);
            closeTag("li");
        }
    }

    public void addCode(Code code) throws IOException {
        startOpenTag("code");
        addAttribute("type", Code.typeToString(code.getType()));
        addIdentifiersAttributes(code);
        endOpenTag();
        addZoneBlockElements(code);
        for (int i = 0; i < code.getFicheBlockCount(); i++) {
            addLn(code.getLn(i));
        }
        closeTag("code");
    }

    public void addLn(Ln ln) throws IOException {
        int indentation = ln.getIndentation();
        startOpenTag("ln");
        if (indentation > 0) {
            addAttribute("indent", String.valueOf(indentation));
        }
        addIdentifiersAttributes(ln);
        endOpenTag(false);
        escape(ln.getValue());
        closeTag("ln", false, false);
    }

    public void addDiv(Div div) throws IOException {
        startOpenTag("div");
        addIdentifiersAttributes(div);
        endOpenTag();
        addZoneBlockElements(div);
        openTag("fbl");
        addFicheBlockList(div.getFicheBlockList());
        closeTag("fbl");
        closeTag("div");
    }

    public void addCdataDiv(CdataDiv cdataDiv) throws IOException {
        startOpenTag("cdatadiv");
        addIdentifiersAttributes(cdataDiv);
        endOpenTag();
        addZoneBlockElements(cdataDiv);
        startOpenTag("cdata");
        endOpenTag(false);
        appendCData(cdataDiv.getCdata());
        closeTag("cdata", false, false);
        closeTag("cdatadiv");
    }

    public void addS(S s) throws IOException {
        startOpenTag("s", false);
        addAttribute("type", s.getTypeString());
        int mask = s.getAttributeMask();
        if ((mask & S.REF_BITVALUE) != 0) {
            addAttribute("ref", s.getRef());
        }
        if ((mask & S.SRC_BITVALUE) != 0) {
            addAttribute("src", s.getSrc());
        }
        if ((mask & S.TITLE_BITVALUE) != 0) {
            addAttribute("title", s.getTitle());
        }
        if ((mask & S.LANG_BITVALUE) != 0) {
            addAttribute("lang", s.getLang());
        }
        if ((mask & S.SUBTYPE_BITVALUE) != 0) {
            addAttribute("subtype", s.getSubtype());
        }
        addAttribute("class", s.getClassName());
        addAttribute("htmlid", s.getHtmlId());
        String value = s.getValue();
        if (value.length() > 0) {
            endOpenTag(false);
            escape(value);
            closeTag("s", false, false);
        } else {
            if ((complete) && (s.getType() == S.TYPE_FICHE)) {
                String ref = s.getRef();
                int idx = ref.indexOf('#');
                if (idx > 0) {
                    ref = ref.substring(0, idx);
                }
                AbregeFiche other = CorpusUtils.parse(ref, '/', basedefiches, defaultCorpus);
                if (other == null) {
                    closeEmptyTag();
                } else {
                    String lang = s.getLang();
                    if (lang.length() == 0) {
                        addAttribute("lang", LangInteger.toString(other.getLangInteger()));
                    }
                    endOpenTag(false);
                    escape(other.getTitre());
                    closeTag("s", false, false);
                }
            } else {
                closeEmptyTag();
            }
        }
    }

    public void addTable(Table table) throws IOException {
        table.checkSize();
        startOpenTag("table");
        addIdentifiersAttributes(table);
        addAttribute("cols", String.valueOf(table.getCols()));
        addAttribute("rows", String.valueOf(table.getRows()));
        endOpenTag();
        addZoneBlockElements(table);
        for (int i = 0; i < table.getTrCount(); i++) {
            addTr(table.getTr(i));
        }
        closeTag("table");
    }

    public void addTr(Tr tr) throws IOException {
        startOpenTag("tr");
        addIdentifiersAttributes(tr);
        endOpenTag();
        for (int i = 0; i < tr.getFicheBlockCount(); i++) {
            addTd(tr.getTd(i));
        }
        closeTag("tr");
    }

    public void addTd(Td td) throws IOException {
        int colspan = td.getColSpan();
        int rowspan = td.getRowSpan();
        startOpenTag("td");
        addAttribute("type", td.getTypeString());
        if (colspan > 1) {
            addAttribute("colspan", String.valueOf(colspan));
        }
        if (rowspan > 1) {
            addAttribute("rowspan", String.valueOf(rowspan));
        }
        addIdentifiersAttributes(td);
        endOpenTag(false);
        addTextContentList(td);
        closeTag("td", false, false);
    }

    public void addInsert(Insert insert) throws IOException {
        startOpenTag("insert");
        addAttribute("type", Insert.typeToString(insert.getType()));
        addAttribute("src", insert.getSrc());
        addAttribute("ref", insert.getRef());
        addAttribute("credit", insert.getCredit());
        addAttribute("position", Insert.positionToString(insert.getPosition()));
        int width = insert.getWidth();
        if (width >= 0) {
            addAttribute("width", String.valueOf(width));
        }
        int height = insert.getHeight();
        if (height >= 0) {
            addAttribute("height", String.valueOf(height));
        }
        addIdentifiersAttributes(insert);
        endOpenTag();
        addZoneBlockElements(insert);
        closeTag("insert");
    }

    private void addZoneBlockElements(ZoneBlock zoneBlock) throws IOException {
        addZoneBlockElement("numero", zoneBlock.getNumero());
        addZoneBlockElement("legende", zoneBlock.getLegende());
    }

    private void addZoneBlockElement(String name, ParagraphBlock prg) throws IOException {
        if (prg.getTextContentCount() > 0) {
            startOpenTag(name);
            addIdentifiersAttributes(prg);
            endOpenTag(false);
            addTextContentList(prg);
            closeTag(name, false, false);
        }
    }

    private void addIdentifiersAttributes(FicheBlock ficheBlock) throws IOException {
        addAttribute("class", ficheBlock.getClassName());
        addAttribute("htmlid", ficheBlock.getHtmlId());
    }

    private void addIdentifiersAttributes(IdentifiedFicheBlockList ficheBlockList) throws IOException {
        addAttribute("class", ficheBlockList.getClassName());
        addAttribute("htmlid", ficheBlockList.getHtmlId());
    }

}
