/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.DocumentSubset;
import net.mapeadores.basedefiches.association.FileLength;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.FicheExtractInfo;
import net.mapeadores.basedefiches.extraction.FieldTypeInfo;
import net.mapeadores.basedefiches.extraction.MotcleFilter;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.filterinfo.ChronoFilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.CorpsdeficheFilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.EnteteFilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.FieldKeyFilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.FilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.FilterInfoList;
import net.mapeadores.basedefiches.extraction.filterinfo.LotFilterInfo;
import net.mapeadores.basedefiches.extraction.filterinfo.TypePrefixFilterInfo;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.text.alphabet.Alphabet;
import net.mapeadores.util.xml.DefaultXMLWriter;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLWriterPart;


/**
 *
 * @author Vincent Calame
 */
public class FicheXMLWriter extends AbstractExtractionXMLWriter {

    private InfoFilterXMLPart infoFilterPart;
    private FicheFilter ficheFilter;
    private FichePointeur fichePointeur;

    public FicheXMLWriter(Basedefiches basedefiches, LocalisationLists localisationLists) {
        super(basedefiches, localisationLists);
        infoFilterPart = new InfoFilterXMLPart(this, basedefiches, localisationLists);
    }

    public void addFiche(FicheExtractInfo ficheExtractInfo) throws IOException {
        updatePointeur(ficheExtractInfo);
        ficheFilter = ficheExtractInfo.getFicheFilter();
        AbregeFiche abregeFiche = ficheExtractInfo.getAbregeFiche();

        String tagName = ficheFilter.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("idcorpus", String.valueOf(abregeFiche.getIdcorpus()));
            addXmlLangAttribute(abregeFiche.getLangInteger());
            int poids = ficheExtractInfo.getPoids();
            if (poids > 0) {
                addAttribute("poids", String.valueOf(poids));
            }
            addAttribute("corpus", abregeFiche.getCorpus().getSubsetKey().getSubsetName());
            if (ficheFilter.isEmptyFicheFilter()) {
                closeEmptyTag();
                return;
            }
            endOpenTag();
        }
        addGroupClauseObject(ficheExtractInfo.getGroupClauseObject());

        infoFilterPart.addFilterInfoList(ficheFilter);

        addLiage(abregeFiche);
        addIndexation(abregeFiche);
        addAssociation(abregeFiche);
        addIntituleFiche(abregeFiche);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void updatePointeur(FicheExtractInfo ficheExtractInfo) {
        boolean ficheComplete = (ficheExtractInfo.getFicheFilter().isWithCorpsdefiche());
        AbregeFiche abregeFiche = ficheExtractInfo.getAbregeFiche();
        Corpus corpus = abregeFiche.getCorpus();
        if (fichePointeur != null) {
            if (!fichePointeur.getCorpus().equals(corpus)) {
                fichePointeur = null;
            } else if (fichePointeur.isFicheComplete() != ficheComplete) {
                fichePointeur = null;
            }
        }
        if (fichePointeur == null) {
            fichePointeur = new FichePointeur(corpus, ficheComplete);
        }
        fichePointeur.setCurrent(abregeFiche);
        infoFilterPart.updatePointeur(fichePointeur);
    }

    public void addGroupClauseObject(Object groupClauseObject) throws IOException {
        if (groupClauseObject == null) {
            return;
        }
        if (groupClauseObject instanceof Alphabet) {
            Alphabet alphabet = (Alphabet) groupClauseObject;
            startOpenTag("alphabet");
            if (alphabet.isWithArticle()) {
                addAttribute("article", alphabet.getArticleString());
                addAttribute("espace", (alphabet.isWithEspace()) ? 1 : 0);
            }
            addAttribute("titre", alphabet.getAlphabetString());
            closeEmptyTag();
        }
    }


    private static class InfoFilterXMLPart extends XMLWriterPart {

        FicheItemXMLPart ficheItemXMLPart;
        FicheBlockXMLPart ficheBlockXMLPart;
        FichePointeur fichePointeur;
        Basedefiches basedefiches;
        LocalisationLists localisationLists;

        InfoFilterXMLPart(XMLWriter xmlWriter, Basedefiches basedefiches, LocalisationLists localisationLists) {
            super(xmlWriter);
            ficheItemXMLPart = new FicheItemXMLPart(xmlWriter);
            ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter);
            ficheItemXMLPart.setCompleteMode(basedefiches, localisationLists);
            ficheBlockXMLPart.setCompleteMode(basedefiches);
            this.basedefiches = basedefiches;
            this.localisationLists = localisationLists;
        }

        private void updatePointeur(FichePointeur fichePointeur) {
            this.fichePointeur = fichePointeur;
            ficheBlockXMLPart.setDefaultCorpus(fichePointeur.getCorpus());
        }

        private void addFilterInfoList(FilterInfoList filterInfoList) throws IOException {
            int filterInfoCount = filterInfoList.getFilterInfoCount();
            for (int i = 0; i < filterInfoCount; i++) {
                FilterInfo filterInfo = filterInfoList.getFilterInfo(i);
                if (filterInfo instanceof EnteteFilterInfo) {
                    addEntete();
                } else if (filterInfo instanceof CorpsdeficheFilterInfo) {
                    addCorpsdefiche();
                } else if (filterInfo instanceof FieldKeyFilterInfo) {
                    addFieldKey(((FieldKeyFilterInfo) filterInfo).getFieldKey());
                } else if (filterInfo instanceof TypePrefixFilterInfo) {
                    addPrefixedField((TypePrefixFilterInfo) filterInfo);
                } else if (filterInfo instanceof LotFilterInfo) {
                    addLot((LotFilterInfo) filterInfo);
                } else if (filterInfo instanceof ChronoFilterInfo) {
                    addChrono();
                }
            }
        }

        private void addLot(LotFilterInfo lotFilterInfo) throws IOException {
            StringBuilder buf = new StringBuilder();
            DefaultXMLWriter xw = new DefaultXMLWriter();
            xw.setAppendable(buf);
            xw.setIndentLength(getCurrentIndentValue() + 1);
            InfoFilterXMLPart child = new InfoFilterXMLPart(xw, basedefiches, localisationLists);
            child.updatePointeur(fichePointeur);
            child.addFilterInfoList(lotFilterInfo.getFilterInfoList());
            String result = buf.toString();
            if (result.length() > 0) {
                String tagName = lotFilterInfo.getTagName();
                if (tagName == null) {
                    tagName = "lot";
                }
                boolean withTagName = (!tagName.equals("NULL"));
                if (withTagName) {
                    openTag(tagName);
                }
                append(result);
                if (withTagName) {
                    closeTag(tagName);
                }
            }
        }

        private void addEntete() throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            addTitre();
            addSoustitre();
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                addPropriete(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField));
            }
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                addInformation(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField));
            }
            addRedacteurs();
        }

        private Object getFieldOptionObject(CorpusField corpusField) {
            short type = corpusField.getFicheItemType();
            if (type == CorpusField.IMAGE_FIELD) {
                SubsetKey albumKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
                if (albumKey != null) {
                    Album album = (Album) basedefiches.getSubset(albumKey);
                    return album;
                } else {
                    return corpusField.getStringOption(FieldOptions.BASEURL_OPTION);
                }
            }
            return null;
        }

        private void addCorpsdefiche() throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            addTexte();
            int annexeCount = corpusMetadata.getAnnexeFieldCount();
            for (int i = 0; i < annexeCount; i++) {
                addAnnexe(corpusMetadata.getAnnexeField(i).getFieldKey(), null);
            }
        }

        private void addFieldKey(FieldKey fieldKey) throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                return;
            }
            if (fieldKey.equals(FieldKey.TITRE)) {
                addTitre();
            } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                addSoustitre();
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                addRedacteurs();
            } else if (fieldKey.equals(FieldKey.TEXTE)) {
                addTexte();
            } else if (fieldKey.equals(FieldKey.LANG)) {
                addLang();
            } else if (fieldKey.isProprieteFieldKey()) {
                addPropriete(fieldKey, null, getFieldOptionObject(corpusField));
            } else if (fieldKey.isInformationFieldKey()) {
                addInformation(fieldKey, null, getFieldOptionObject(corpusField));
            } else if (fieldKey.isAnnexeFieldKey()) {
                addAnnexe(fieldKey, null);
            }
        }

        private void addPrefixedField(TypePrefixFilterInfo filterInfo) throws IOException {
            String prefix = filterInfo.getPrefix();
            switch (filterInfo.getCategory()) {
                case FieldKey.PROPRIETE_CATEGORY:
                    addPrefixedPropriete(prefix);
                    break;
                case FieldKey.INFORMATION_CATEGORY:
                    addPrefixeInformation(prefix);
                    break;
                case FieldKey.ANNEXE_CATEGORY:
                    addPrefixedAnnexe(prefix);
                    break;
            }
        }

        private void addPrefixedPropriete(String prefix) throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addPropriete(fieldKey, fieldTypeInfo, getFieldOptionObject(corpusField));
                }
            }
        }

        private void addPrefixeInformation(String prefix) throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addInformation(fieldKey, fieldTypeInfo, getFieldOptionObject(corpusField));
                }
            }
        }

        private void addPrefixedAnnexe(String prefix) throws IOException {
            CorpusMetadata corpusMetadata = fichePointeur.getCorpus().getCorpusMetadata();
            int proprieteCount = corpusMetadata.getAnnexeFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                FieldKey fieldKey = corpusMetadata.getAnnexeField(i).getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addAnnexe(fieldKey, fieldTypeInfo);
                }
            }
        }

        private void addTitre() throws IOException {
            String titre = (String) fichePointeur.getValue(FieldKey.TITRE);
            if (titre.length() == 0) {
                return;
            }
            openTag("titre");
            addSimpleElement("item", titre);
            closeTag("titre");
        }

        private void addSoustitre() throws IOException {
            Para para = (Para) fichePointeur.getValue(FieldKey.SOUSTITRE);
            if (para == null) {
                return;
            }
            openTag("soustitre");
            ficheItemXMLPart.addPara(para);
            closeTag("soustitre");
        }

        private void addRedacteurs() throws IOException {
            FicheItemList ficheItemList = (FicheItemList) fichePointeur.getValue(FieldKey.REDACTEURS);
            if (ficheItemList == null) {
                return;
            }
            openTag("redacteurs");
            ficheItemXMLPart.addFicheItemList(ficheItemList);
            closeTag("redacteurs");
        }

        private void addTexte() throws IOException {
            FicheBlockList texteBlockList = (FicheBlockList) fichePointeur.getValue(FieldKey.TEXTE);
            if (texteBlockList == null) {
                return;
            }
            openTag("texte");
            ficheBlockXMLPart.addFicheBlockList(texteBlockList);
            closeTag("texte");
        }

        private void addPropriete(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo, Object fieldOptionObject) throws IOException {
            FicheItem ficheItem = (FicheItem) fichePointeur.getValue(fieldKey);
            if (ficheItem == null) {
                return;
            }
            startOpenTag("propriete");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItem(ficheItem, fieldOptionObject);
            closeTag("propriete");
        }

        private void addInformation(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo, Object fieldOptionObject) throws IOException {
            FicheItemList ficheItemList = (FicheItemList) fichePointeur.getValue(fieldKey);
            if (ficheItemList == null) {
                return;
            }
            startOpenTag("information");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItemList(ficheItemList, fieldOptionObject);
            closeTag("information");
        }

        private void addAnnexe(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo) throws IOException {
            FicheBlockList blockList = (FicheBlockList) fichePointeur.getValue(fieldKey);
            if (blockList == null) {
                return;
            }
            startOpenTag("annexe");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheBlockXMLPart.addFicheBlockList(blockList);
            closeTag("annexe");
        }

        private void addChrono() throws IOException {
            AbregeFiche abregeFiche = fichePointeur.getAbregeFiche();
            int creationDate = abregeFiche.getCreationDate();
            if (creationDate != 0) {
                openTag("chrono");
                ficheItemXMLPart.addDatation(new Datation(creationDate));
                int modificationDate = abregeFiche.getModificationDate();
                if (modificationDate != creationDate) {
                    ficheItemXMLPart.addDatation(new Datation(modificationDate));
                }
                closeTag("chrono");
            }
        }

        private void addLang() throws IOException {
            openTag("lang");
            ficheItemXMLPart.addLangue(new Langue(fichePointeur.getAbregeFiche().getLangInteger()));
            closeTag("lang");
        }

    }

    public void addLiage(AbregeFiche abregeFiche) throws IOException {
        ExtraitCorpusDefList extraitCorpusDefList = ficheFilter.getLiageExtraitCorpusDefList();
        int count = extraitCorpusDefList.getExtraitCorpusDefCount();
        if (count == 0) {
            return;
        }
        Corpus currentCorpus = abregeFiche.getCorpus();
        SubsetKey subsetKey1 = currentCorpus.getSubsetKey();
        FicheEligibility ficheEligibility = ficheFilter.getFicheEligibility();
        ExtraitcorpusXMLWriter extraitcorpusXMLWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
        extraitcorpusXMLWriter.setAppendable(getAppendable());
        extraitcorpusXMLWriter.setIndentLength(getCurrentIndentValue());
        for (int k = 0; k < count; k++) {
            ExtraitCorpusDef extraitCorpusDef = extraitCorpusDefList.getExtraitCorpusDef(k);
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef, currentCorpus);
            FicheSelector liageFicheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            liageFicheSelector.setFicheEligibility(ficheEligibility);
            liageFicheSelector.addFicheSelectOptionsList(extraitCorpusDef);
            for (int j = 0; j < liageFicheSelector.getCorpusCount(); j++) {
                Corpus other = liageFicheSelector.getCorpus(j);
                SubsetKey subsetKey2 = other.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(subsetKey1, subsetKey2);
                CroisementItem croisementItem = croisement.getCroisementItem(subsetKey1, abregeFiche.getTicket());
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket2 = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    AbregeFiche abrege2 = (AbregeFiche) other.getSubsetItemByTicket(ticket2);
                    if (abrege2 != null) {
                        if (liageFicheSelector.isSelected(abrege2)) {
                            corpusExtractor.add(abrege2, poids);
                        }
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                continue;
            }
            extraitcorpusXMLWriter.addExtraitcorpus(corpusExtractor);

        }
    }

    public void addIndexation(AbregeFiche abregeFiche) throws IOException {
        MotcleSelector indexationMotcleSelector = ficheFilter.getIndexationMotcleSelector();
        if (indexationMotcleSelector == null) {
            return;
        }
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(basedefiches, abregeFiche.getCorpus());
        for (int j = 0; j < indexationMotcleSelector.getThesaurusCount(); j++) {
            Thesaurus thesaurus = indexationMotcleSelector.getThesaurus(j);
            SubsetKey thesaurusKey = thesaurus.getSubsetKey();
            Croisement croisement = basedefiches.getCroisement(corpusKey, thesaurusKey);
            CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, abregeFiche.getTicket());
            int ciCount = croisementItem.getCount();
            for (int i = 0; i < ciCount; i++) {
                int ticket2 = croisementItem.getTicket(i);
                int poids = croisementItem.getPoids(i);
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket2);
                MotcleFilter motcleFilter = indexationMotcleSelector.getMotcleFilter(motcle);
                if (motcleFilter != null) {
                    thesaurusExtractor.add(motcle, motcleFilter, poids);
                }
            }
        }
        if (!thesaurusExtractor.containsMotcle()) {
            return;
        }
        ExtraitthesaurusXMLWriter extraitthesaurusXMLWriter = new ExtraitthesaurusXMLWriter(basedefiches, localisationLists);
        extraitthesaurusXMLWriter.setAppendable(getAppendable());
        extraitthesaurusXMLWriter.setIndentLength(getCurrentIndentValue());
        extraitthesaurusXMLWriter.addExtraitthesaurus(thesaurusExtractor);
    }

    public void addIntituleFiche(AbregeFiche abregeFiche) throws IOException {
        if (!ficheFilter.includeIntituleFiche()) {
            return;
        }
        openTag("intitule");
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int workingLang = localisationLists.getWorkingLangInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            String lib = CorpusMetadataUtils.getFicheNumberString(abregeFiche, workingLang, locale);
            addLibElement(workingLang, lib);
        }
        closeTag("intitule");
    }

    public void addAssociation(AbregeFiche abregeFiche) throws IOException {
        if (!ficheFilter.includeAssociation()) {
            return;
        }
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = basedefiches.getCroisement(SubsetKey.DOCUMENT_SUBSET, corpusKey);
        CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, abregeFiche.getTicket());
        int ciCount = croisementItem.getCount();
        if (ciCount == 0) {
            return;
        }
        DocumentSubset documentSubset = (DocumentSubset) basedefiches.getSubset(SubsetKey.DOCUMENT_SUBSET);
        for (int i = 0; i < ciCount; i++) {
            int ticket2 = croisementItem.getTicket(i);
            Document document = (Document) documentSubset.getSubsetItemByTicket(ticket2);
            if (document != null) {
                addDocument(document);
            }
        }
    }

    private void addDocument(Document document) throws IOException {
        startOpenTag("document");
        addAttribute("basename", document.getBasename());
        endOpenTag();
        for (int i = 0; i < document.getVersionCount(); i++) {
            Version version = document.getVersion(i);
            addVersion(version);
        }
        closeTag("document");
    }

    private void addVersion(Version version) throws IOException {
        FileLength fileLength = version.getFileLength();
        float value = fileLength.getRoundedValue();
        short roundType = fileLength.getRoundType();
        boolean isKibiOctet = (fileLength.getRoundType() == FileLength.KB_ROUND);
        startOpenTag("version");
        addAttribute("extension", version.getVersionName().getExtension());
        endOpenTag();
        startOpenTag("size");
        addAttribute("value", String.valueOf(value));
        String unit = (isKibiOctet) ? "Ki" : "Mi";
        addAttribute("unit", unit);
        endOpenTag();
        addSizeLib(value, isKibiOctet);
        closeTag("size");
        closeTag("version");
    }

    private void addSizeLib(float value, boolean isKibiOctet) throws IOException {
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            NumberFormat format = NumberFormat.getInstance(locale);
            StringBuffer buf = new StringBuffer();
            buf.append(format.format(value));
            buf.append(' ');
            if (isKibiOctet) {
                buf.append(LocalisationUtils.getKibiOctet(localeInteger));
            } else {
                buf.append(LocalisationUtils.getMebiOctet(localeInteger));
            }
            addLibElement(localeInteger, buf.toString());
        }
    }

}
