/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.MotcleExtractInfo;
import net.mapeadores.basedefiches.extraction.MotcleFilter;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class MotcleXMLWriter extends AbstractExtractionXMLWriter {

    public MotcleXMLWriter(Basedefiches basedefiches, LocalisationLists localisationLists) {
        super(basedefiches, localisationLists);
    }

    public void addMotcle(MotcleExtractInfo motcleExtractInfo) throws IOException {
        Motcle motcle = motcleExtractInfo.getMotcle();
        MotcleFilter motcleFilter = motcleExtractInfo.getMotcleFilter();
        startOpenTag("motcle");
        addAttribute("idths", String.valueOf(motcle.getIdths()));
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            addAttribute("idalpha", idalpha);
        }
        int poids = motcleExtractInfo.getPoids();
        if (poids > 0) {
            addAttribute("poids", String.valueOf(poids));
        }
        if (motcleFilter.isNoneFiltering()) {
            closeEmptyTag();
            return;
        }
        endOpenTag();
        if (motcleFilter.withLevel()) {
            addSimpleElement("level", String.valueOf(motcle.getLevel()));
        }
        if (motcleFilter.withLib()) {
            addLibelleList(motcle);
        }
        if (motcleFilter.withChildren()) {
            MotcleList childList = motcle.getChildList();
            for (int i = 0; i < childList.getMotcleCount(); i++) {
                Motcle child = childList.getMotcle(i);
                MotcleExtractInfo childExtractInfo = new MotcleExtractInfo(child, motcleFilter.getChildrenFilter());
                addMotcle(childExtractInfo);
            }
        }
        if (motcleFilter.withParent()) {
            Motcle parentMotcle = motcle.getParent();
            if (parentMotcle != null) {
                addLinkedMotcle("parent", parentMotcle, motcleFilter.getParentFilter());
            }
        }
        if (motcleFilter.withPrevious()) {
            Motcle previousMotcle = ThesaurusUtils.getPrevious(motcle);
            if (previousMotcle != null) {
                addLinkedMotcle("previous", previousMotcle, motcleFilter.getPreviousFilter());
            }
        }
        if (motcleFilter.withNext()) {
            Motcle nextMotcle = ThesaurusUtils.getNext(motcle);
            if (nextMotcle != null) {
                addLinkedMotcle("next", nextMotcle, motcleFilter.getNextFilter());
            }
        }
        addIndexation(motcle, motcleFilter);
        closeTag("motcle");
    }

    private void addLinkedMotcle(String tagname, Motcle linkedMotcle, MotcleFilter linkedMotcleFilter) throws IOException {
        openTag(tagname);
        addMotcle(new MotcleExtractInfo(linkedMotcle, linkedMotcleFilter));
        closeTag(tagname);
    }

    private void addIndexation(Motcle motcle, MotcleFilter motcleFilter) throws IOException {
        if (!motcleFilter.includeIndexation()) {
            return;
        }
        ExtraitCorpusDefList extraitCorpusDefList = motcleFilter.getIndexationExtraitCorpusDefList();
        int count = extraitCorpusDefList.getExtraitCorpusDefCount();
        Thesaurus currentThesaurus = motcle.getThesaurus();
        SubsetKey thesaurusKey = currentThesaurus.getSubsetKey();
        FicheEligibility ficheEligibility = motcleFilter.getFicheEligibility();
        ExtraitcorpusXMLWriter extraitcorpusXMLWriter = new ExtraitcorpusXMLWriter(basedefiches, localisationLists);
        extraitcorpusXMLWriter.setAppendable(getAppendable());
        extraitcorpusXMLWriter.setIndentLength(getCurrentIndentValue() + 1);
        for (int k = 0; k < count; k++) {
            ExtraitCorpusDef extraitCorpusDef = extraitCorpusDefList.getExtraitCorpusDef(k);
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef, currentThesaurus);
            FicheSelector indexationFicheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            indexationFicheSelector.setFicheEligibility(ficheEligibility);
            indexationFicheSelector.addFicheSelectOptionsList(extraitCorpusDef);
            for (int j = 0; j < indexationFicheSelector.getCorpusCount(); j++) {
                Corpus corpus = indexationFicheSelector.getCorpus(j);
                SubsetKey corpusKey = corpus.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(corpusKey, thesaurusKey);
                CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
                    if (abregeFiche != null) {
                        if (indexationFicheSelector.isSelected(abregeFiche)) {
                            corpusExtractor.add(abregeFiche, poids);
                        }
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                continue;
            }
            extraitcorpusXMLWriter.addExtraitcorpus(corpusExtractor);
        }
    }

}
