/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.event.BasedefichesAdapter;
import net.mapeadores.basedefiches.l10n.BasedefichesL10n;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.ui.*;
import java.util.*;
import net.mapeadores.util.text.Libelle;


/**
 * Signale à l'instance de BdfServer à laquelle est rattachée l'instance de BasedefichesListener les événements les plus importants
 * sur la base de fiches.
 *
 * @author Vincent Calame
 */
public class BdfServerBasedefichesListener extends BasedefichesAdapter {

    public final static int SAVE_UIMANAGER = 1;
    public final static int SAVE_POLICYMANAGER = 2;
    public final static int SAVE_PERMISSIONMANAGER = 4;
    public final static int SAVE_PASSWORDMANAGER = 8;
    public final static int CLEAR_DEFAULTTEMPLATES = 16;
    BdfServer bdfServer;
    boolean onMultiChange = false;
    int multiChangeMask = 0;

    public BdfServerBasedefichesListener(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public void buffer() {
        if (!onMultiChange) {
            onMultiChange = true;
            multiChangeMask = 0;
        }
    }

    private void saveUIManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_UIMANAGER;
        } else {
            bdfServer.saveUIManager();
        }
    }

    private void savePolicyManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_POLICYMANAGER;
        } else {
            bdfServer.savePolicyManager();
        }
    }

    private void savePermissionManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_PERMISSIONMANAGER;
        } else {
            bdfServer.savePermissionManager();
        }
    }

    private void savePasswordManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_PASSWORDMANAGER;
        } else {
            bdfServer.savePasswordManager();
        }
    }

    private void clearDefaultTemplates() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | CLEAR_DEFAULTTEMPLATES;
        } else {
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
    }

    public void flush() {
        if ((multiChangeMask & CLEAR_DEFAULTTEMPLATES) != 0) {
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
        if ((multiChangeMask & SAVE_UIMANAGER) != 0) {
            bdfServer.saveUIManager();
        }
        if ((multiChangeMask & SAVE_POLICYMANAGER) != 0) {
            bdfServer.savePolicyManager();
        }
        if ((multiChangeMask & SAVE_PERMISSIONMANAGER) != 0) {
            bdfServer.savePermissionManager();
        }
        if ((multiChangeMask & SAVE_PASSWORDMANAGER) != 0) {
            bdfServer.savePasswordManager();
        }
        onMultiChange = false;
        multiChangeMask = 0;
    }

    @Override
    public void thesaurusCreated(Thesaurus thesaurus) {
        bdfServer.getPolicyManager().createThesaurusPolicy(thesaurus);
        bdfServer.getUIManager().thesaurusCreated(thesaurus);
        savePolicyManager();
        saveUIManager();
    }

    @Override
    public void thesaurusRemoved(SubsetKey thesaurusKey) {
        bdfServer.getUIManager().thesaurusRemoved(thesaurusKey);
        bdfServer.getPolicyManager().removeThesaurusPolicy(thesaurusKey);
        clearDefaultTemplates();
        saveUIManager();
        savePolicyManager();
    }

    @Override
    public void sphereCreated(Sphere sphere) {
        bdfServer.getUIManager().sphereCreated(sphere);
        saveUIManager();
    }

    @Override
    public void sphereRemoved(SubsetKey sphereKey) {
        bdfServer.getUIManager().sphereRemoved(sphereKey);
        saveUIManager();
    }

    @Override
    public void redacteurRemoved(RedacteurKey redacteurKey) {
        boolean done = false;
        /*done = bdfServer.getPermissionManager().removeRole(redacteurKey);
        if (done) savePermissionManager();*/
        done = bdfServer.getPasswordManager().removePassword(redacteurKey);
        if (done) {
            savePasswordManager();
        }
    }

    @Override
    public void corpusCreated(Corpus corpus) {
        initLibelle(bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()));
        bdfServer.getUIManager().corpusCreated(corpus);
        saveUIManager();
    }

    @Override
    public void corpusRemoved(SubsetKey corpusKey, Subset masterSubset) {
        bdfServer.getUIManager().corpusRemoved(corpusKey, masterSubset);
        clearDefaultTemplates();
        saveUIManager();
    }

    @Override
    public void corpusFieldCreated(Corpus corpus, CorpusField corpusField) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if ((fieldKey.equals(FieldKey.SOUSTITRE)) || (fieldKey.equals(FieldKey.TEXTE))) {
            CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
            initLibelle(corpusEditor, corpusField);
        }
        bdfServer.getUIManager().corpusFieldCreated(corpus, corpusField);
        saveUIManager();
    }

    @Override
    public void corpusFieldRemoved(Corpus corpus, FieldKey fieldKey) {
        bdfServer.getUIManager().corpusFieldRemoved(corpus, fieldKey);
        saveUIManager();
    }

    private void initLibelle(CorpusEditor corpusEditor) {
        initLibelle(corpusEditor, FieldKey.IDCORPUS);
        initLibelle(corpusEditor, FieldKey.LANG);
        initLibelle(corpusEditor, FieldKey.TITRE);
        initLibelle(corpusEditor, FieldKey.REDACTEURS);
    }

    private void initLibelle(CorpusEditor corpusEditor, FieldKey fieldKey) {
        initLibelle(corpusEditor, corpusEditor.getCorpus().getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey));
    }

    private void initLibelle(CorpusEditor corpusEditor, CorpusField corpusField) {
        int[] workingLangIntegerArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        List<Libelle> list = BasedefichesL10n.getSpecialFieldKeyLibelle(corpusField.getFieldKey());
        for (Libelle libelle : list) {
            int langInteger = libelle.getLangInteger();
            for (int i = 0; i < workingLangIntegerArray.length; i++) {
                if (langInteger == workingLangIntegerArray[i]) {
                    corpusEditor.getCorpusMetadataEditor().setFieldLibelle(corpusField, libelle.getLibelleString(), langInteger);
                    break;
                }
            }
        }
    }

    private void addSubsetKey(SubsetKeyTree tree, SubsetKey subsetKey) {
        tree.addSubsetKey(subsetKey, null);
    }

}
