/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.actions;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheAction implements BdfAction {

    private Corpus corpus;
    private AbregeFiche abregeFiche;

    protected FicheAction() {
    }

    public SubsetKey getCorpusKey() {
        return corpus.getSubsetKey();
    }

    public Corpus getCorpus() {
        return corpus;
    }

    private void setCorpus(Corpus corpus) {
        this.corpus = corpus;
    }

    public AbregeFiche getAbregeFiche() {
        return abregeFiche;
    }

    private void setAbregeFiche(AbregeFiche abregeFiche) {
        this.abregeFiche = abregeFiche;
    }

    /*
    public int getIdcorpus() {
    return idcorpus;
    }*/
    public boolean isAllFicheAction() {
        return (abregeFiche == null);
    }

    public static ReadFiche getReadFiche(Corpus corpus) {
        return (ReadFiche) init(corpus, new ReadFiche());
    }

    public static WriteFiche getWriteFiche(Corpus corpus) {
        return (WriteFiche) init(corpus, new WriteFiche());
    }

    public static SendFiche getSendFiche(Corpus corpus) {
        return (SendFiche) init(corpus, new SendFiche());
    }

    public static ReadFiche getReadFiche(AbregeFiche abregeFiche) {
        return (ReadFiche) init(abregeFiche, new ReadFiche());
    }

    public static WriteFiche getWriteFiche(AbregeFiche abregeFiche) {
        return (WriteFiche) init(abregeFiche, new WriteFiche());
    }

    public static SendFiche getSendFiche(AbregeFiche abregeFiche) {
        return (SendFiche) init(abregeFiche, new SendFiche());
    }

    private static FicheAction init(Corpus corpus, FicheAction ficheAction) {
        return init(corpus, null, ficheAction);
    }

    private static FicheAction init(AbregeFiche abregeFiche, FicheAction ficheAction) {
        return init(abregeFiche.getCorpus(), abregeFiche, ficheAction);
    }

    private static FicheAction init(Corpus corpus, AbregeFiche abregeFiche, FicheAction ficheAction) {
        ficheAction.setCorpus(corpus);
        ficheAction.setAbregeFiche(abregeFiche);
        return ficheAction;
    }

}
