/* BdfServer
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages;

import java.util.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.balayages.xml.*;
import fr.exemole.bdfserver.balayages.api.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayageBuilder implements BalayageDOMReader.Builder {

    private DefaultBalayage defaultBalayage;
    private ArrayList<FicheSelectOptions> globalFicheSelectOptionsList = new ArrayList<FicheSelectOptions>();
    private Map<String, InternalBalayageMode> modeMap = new LinkedHashMap<String, InternalBalayageMode>();
    private List<InternalBalayageUnitBuilder> unitList = new ArrayList<InternalBalayageUnitBuilder>();

    public BalayageBuilder(String name) {
        defaultBalayage = new DefaultBalayage(name);
    }

    public Balayage toBalayage() {
        int size = globalFicheSelectOptionsList.size();
        if (size > 0) {
            defaultBalayage.setGlobalFicheSelectOptionsArray(globalFicheSelectOptionsList.toArray(new FicheSelectOptions[size]));
        }
        int size2 = modeMap.size();
        if (size2 > 0) {
            BalayageMode[] array = new BalayageMode[size2];
            int p = 0;
            for (BalayageMode balayageMode : modeMap.values()) {
                array[p] = balayageMode;
                p++;
            }
            defaultBalayage.setBalayageModeArray(array);
        }
        int size3 = unitList.size();
        if (size3 > 0) {
            BalayageUnit[] balayageUnitArray = new BalayageUnit[size3];
            for (int i = 0; i < size3; i++) {
                balayageUnitArray[i] = unitList.get(i).toBalayageUnit();
            }
            defaultBalayage.setBalayageUnitArray(balayageUnitArray);
        }
        return defaultBalayage;
    }

    public void addGlobalFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        globalFicheSelectOptionsList.add(ficheSelectOptions);
    }

    public void setRootName(String rootName) {
        if (rootName != null) {
            rootName = rootName.intern();
        }
        defaultBalayage.setRootName(rootName);
    }

    public void setPath(String path) {
        defaultBalayage.setPath(path);
    }

    public void setWithTransformation(boolean withTransformation) {
        defaultBalayage.setWithTransformation(withTransformation);
    }

    public void setPostCommand(String postCommand) {
        defaultBalayage.setPostCommand(postCommand);
    }

    public void setDefaultLangOption(short defaultLangOption) {
        defaultBalayage.setDefaultLangOption(defaultLangOption);
    }

    public void setLocalisationLists(LocalisationLists localisationLists) {
        defaultBalayage.setLocalisationLists(localisationLists);
    }

    public void setSiteMapOption(SiteMapOption siteMapOption) {
        defaultBalayage.setSiteMapOption(siteMapOption);
    }

    public BalayageMode addBalayageMode(String name) {
        InternalBalayageMode ibe = modeMap.get(name.intern());
        if (ibe == null) {
            ibe = new InternalBalayageMode(name);
            modeMap.put(name, ibe);
        }
        return ibe;
    }

    public void setWithCurrentDateLimit(BalayageMode balayageMode, boolean b) {
        ((InternalBalayageMode) balayageMode).setWithCurrentDateLimit(b);
    }

    public BalayageDOMReader.BalayageUnitBuilder addBalayageUnitBuilder(short type) {
        InternalBalayageUnitBuilder unitBuilder = new InternalBalayageUnitBuilder(type);
        unitList.add(unitBuilder);
        return unitBuilder;
    }


    private static class InternalBalayageUnitBuilder implements BalayageDOMReader.BalayageUnitBuilder {

        private DefaultBalayageUnit balayageUnit;
        private LinkedHashSet<String> modeSet = new LinkedHashSet<String>();
        private LinkedHashSet<String> extensionSet = new LinkedHashSet<String>();
        private List<InternalOutputBuilder> outputList = new ArrayList<InternalOutputBuilder>();

        InternalBalayageUnitBuilder(short type) {
            balayageUnit = new DefaultBalayageUnit(type);
        }

        public void setName(String name) {
            balayageUnit.setName(name);
        }

        public void setGlobalSelect(boolean globalSelect) {
            balayageUnit.setGlobalSelect(globalSelect);
        }

        public void setExtractionPath(String extractionPath) {
            balayageUnit.setExtractionPath(extractionPath);
        }

        public void addMode(String mode) {
            modeSet.add(mode.intern());
        }

        public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
            balayageUnit.setFicheSelectOptions(ficheSelectOptions);
        }

        public void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
            balayageUnit.setMotcleSelectOptions(motcleSelectOptions);
        }

        public void setAlbumSelectOptions(AlbumSelectOptions albumSelectOptions) {
            balayageUnit.setAlbumSelectOptions(albumSelectOptions);
        }

        public void addExtension(String s) {
            extensionSet.add(s.intern());
        }

        public BalayageDOMReader.OutputBuilder addOutputBuilder() {
            InternalOutputBuilder iob = new InternalOutputBuilder();
            outputList.add(iob);
            return iob;
        }

        private BalayageUnit toBalayageUnit() {
            int size = modeSet.size();
            if (size > 0) {
                String[] modeArray = new String[size];
                int p = 0;
                for (String mode : modeSet) {
                    modeArray[p] = mode;
                    p++;
                }
                balayageUnit.setModeArray(modeArray);
            }
            int size2 = extensionSet.size();
            if (size2 > 0) {
                String[] extensionArray = new String[size2];
                int p = 0;
                for (String extension : extensionSet) {
                    extensionArray[p] = extension;
                    p++;
                }
                balayageUnit.setExtensionArray(extensionArray);
            }
            int size3 = outputList.size();
            if (size3 > 0) {
                Output[] outputArray = new Output[size3];
                for (int i = 0; i < size3; i++) {
                    outputArray[i] = outputList.get(i).toOutput(balayageUnit);
                }
                balayageUnit.setOutputArray(outputArray);
            }
            return balayageUnit;
        }

    }


    private static class InternalOutputBuilder implements BalayageDOMReader.OutputBuilder {

        private DefaultOutput output = new DefaultOutput();
        private List<InternalOutputParam> outputParamList = new ArrayList<InternalOutputParam>();

        InternalOutputBuilder() {
        }

        public void setName(String name) {
            output.setName(name);
        }

        public void setXsltPath(String xsltPath) {
            output.setXsltPath(xsltPath);
        }

        public void setMode(String mode) {
            output.setMode(mode);
        }

        public void setCleanBefore(boolean cleanBefore) {
            output.setCleanBefore(cleanBefore);
        }

        public void setLangOption(short langOption) {
            output.setLangOption(langOption);
        }

        public void setIncludeOutput(boolean includeOutput) {
            output.setIncludeOutput(includeOutput);
        }

        public void setPattern(AccoladePattern pattern) {
            output.setPattern(pattern);
        }

        public void setOutputPathPattern(AccoladePattern outputPathPattern) {
            output.setOutputPathPattern(outputPathPattern);
        }

        public void setLocalisationLists(LocalisationLists localisationLists) {
            output.setLocalisationLists(localisationLists);
        }

        public void addOutputParam(String name, String value) {
            outputParamList.add(new InternalOutputParam(name, value));
        }

        Output toOutput(BalayageUnit balayageUnit) {
            output.setBalayageUnit(balayageUnit);
            int size = outputParamList.size();
            if (size > 0) {
                OutputParam[] outputParamArray = new OutputParam[size];
                output.setOutputParamArray(outputParamList.toArray(outputParamArray));
            }
            return output;
        }

    }


    private static class InternalBalayageMode implements BalayageMode {

        private String name;
        private boolean withCurrentDateLimit = false;

        InternalBalayageMode(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public boolean isWithCurrentDateLimit() {
            return withCurrentDateLimit;
        }

        void setWithCurrentDateLimit(boolean withCurrentDateLimit) {
            this.withCurrentDateLimit = withCurrentDateLimit;
        }

    }


    private static class InternalOutputParam implements OutputParam {

        String name;
        String value;

        InternalOutputParam(String name, String value) {
            this.name = name;
            this.value = value;
        }

        public String getName() {
            return name;
        }

        public String getValue() {
            return value;
        }

    }

}
