/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages;

import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayageUtils implements BalayageConstants {

    public final static AccoladePattern UNIQUE_PATTERN;
    public final static AccoladePattern CORPUS_PATTERN;
    public final static AccoladePattern FICHE_PATTERN;
    public final static AccoladePattern THESAURUS_PATTERN;
    public final static AccoladePattern MOTCLE_PATTERN;
    public final static AccoladePattern SPHERE_PATTERN;
    public final static AccoladePattern REDACTEUR_PATTERN;
    public final static AccoladePattern DOCUMENT_PATTERN;

    static {
        try {
            UNIQUE_PATTERN = new AccoladePattern("extraction");
            CORPUS_PATTERN = new AccoladePattern("corpus-{corpus}");
            FICHE_PATTERN = new AccoladePattern("fiche-{corpus}-{idcorpus}");
            THESAURUS_PATTERN = new AccoladePattern("thesaurus-{thesaurus}");
            MOTCLE_PATTERN = new AccoladePattern("motcle-{thesaurus}-{idths}");
            SPHERE_PATTERN = new AccoladePattern("sphere-{sphere}");
            REDACTEUR_PATTERN = new AccoladePattern("redacteur-{sphere}-{idsphere}");
            DOCUMENT_PATTERN = new AccoladePattern("{basename}");
        } catch (ParseException pe) {
            throw new IllegalStateException(pe);
        }
    }

    private BalayageUtils() {
    }

    public static short langOptionToShort(String s) {
        if ((s == null) || (s.length() == 0)) {
            return LANG_OPTION_UNKNOWN;
        }
        if (s.equals("none")) {
            return LANG_OPTION_NONE;
        }
        if (s.equals("dir")) {
            return LANG_OPTION_DIR;
        }
        throw new IllegalArgumentException("unknown langOption : " + s);
    }

    /**
     *
     *@throws IllegalArgumentException si le type est incorrect.
     */
    public static final String typeToString(short type) {
        switch (type) {
            case UNIQUE_TYPE:
                return "unique";
            case CORPUS_TYPE:
                return "corpus";
            case FICHE_TYPE:
                return "fiche";
            case THESAURUS_TYPE:
                return "thesaurus";
            case MOTCLE_TYPE:
                return "motcle";
            case SPHERE_TYPE:
                return "sphere";
            case REDACTEUR_TYPE:
                return "redacteur";
            case DOCUMENT_TYPE:
                return "document";
            default:
                throw new IllegalArgumentException("wrong type value");
        }
    }

    /**
     *
     *@throws IllegalArgumentException si le type est incorrect.
     */
    public static final short typeToShort(String type) {
        if (type.equals("unique")) {
            return UNIQUE_TYPE;
        }
        if (type.equals("corpus")) {
            return CORPUS_TYPE;
        }
        if (type.equals("fiche")) {
            return FICHE_TYPE;
        }
        if (type.equals("thesaurus")) {
            return THESAURUS_TYPE;
        }
        if (type.equals("motcle")) {
            return MOTCLE_TYPE;
        }
        if (type.equals("sphere")) {
            return SPHERE_TYPE;
        }
        if (type.equals("redacteur")) {
            return REDACTEUR_TYPE;
        }
        if (type.equals("document")) {
            return DOCUMENT_TYPE;
        }
        throw new IllegalArgumentException("wrong type value");
    }

    /**
     *
     *@throws IllegalArgumentException si le type est incorrect.
     */
    public static final AccoladePattern getDefaultPattern(short type) {
        switch (type) {
            case UNIQUE_TYPE:
                return UNIQUE_PATTERN;
            case CORPUS_TYPE:
                return CORPUS_PATTERN;
            case FICHE_TYPE:
                return FICHE_PATTERN;
            case THESAURUS_TYPE:
                return THESAURUS_PATTERN;
            case MOTCLE_TYPE:
                return MOTCLE_PATTERN;
            case SPHERE_TYPE:
                return SPHERE_PATTERN;
            case REDACTEUR_TYPE:
                return REDACTEUR_PATTERN;
            case DOCUMENT_TYPE:
                return DOCUMENT_PATTERN;
            default:
                throw new SwitchException("wrong type = " + type);
        }
    }

    public final static boolean containsMode(Balayage balayage, String modeName) {
        int count = balayage.getBalayageModeCount();
        if (count == 0) {
            return false;
        }
        for (int i = 0; i < count; i++) {
            BalayageMode balayageMode = balayage.getBalayageMode(i);
            if (balayageMode.getName().equals(modeName)) {
                return true;
            }
        }
        return false;
    }

    public final static BalayageMode getMode(Balayage balayage, String modeName) {
        int count = balayage.getBalayageModeCount();
        if (count == 0) {
            return null;
        }
        for (int i = 0; i < count; i++) {
            BalayageMode balayageMode = balayage.getBalayageMode(i);
            if (balayageMode.getName().equals(modeName)) {
                return balayageMode;
            }
        }
        return null;
    }

    public final static BalayageUnit getBalayageUnit(Balayage balayage, String unitName) {
        int count = balayage.getBalayageUnitCount();
        for (int i = 0; i < count; i++) {
            BalayageUnit balayageUnit = balayage.getBalayageUnit(i);
            String bn = balayageUnit.getName();
            if (bn != null) {
                if (bn.equals(unitName)) {
                    return balayageUnit;
                }
            }
        }
        return null;
    }

    public final static Output getOutput(BalayageUnit balayageUnit, String outputName) {
        int count = balayageUnit.getOutputCount();
        for (int i = 0; i < count; i++) {
            Output output = balayageUnit.getOutput(i);
            String on = output.getName();
            if (on != null) {
                if (on.equals(outputName)) {
                    return output;
                }
            }
        }
        return null;
    }

    public final static AccoladePattern getPattern(Output output, short type) {
        AccoladePattern pattern = output.getPattern();
        if (pattern == null) {
            return getDefaultPattern(type);
        }
        return pattern;
    }

}
