/* BdfServer
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages;

import fr.exemole.bdfserver.balayages.api.SiteMapOption;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultBalayage implements Balayage, BalayageConstants {

    private short defaultLangOption = LANG_OPTION_NONE;
    private String name;
    private String postCommand;
    private LocalisationLists localisationLists;
    private boolean withTransformation;
    private String rootName;
    private String path;
    private FicheSelectOptions[] globalFicheSelectOptionsArray;
    private BalayageMode[] balayageModeArray;
    private BalayageUnit[] balayageUnitArray;
    private SiteMapOption siteMapOption;

    public DefaultBalayage(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public LocalisationLists getLocalisationLists() {
        return localisationLists;
    }

    public String getRootName() {
        return rootName;
    }

    public String getPath() {
        return path;
    }

    public boolean isWithTransformation() {
        return withTransformation;
    }

    public int getBalayageUnitCount() {
        if (balayageUnitArray == null) {
            return 0;
        }
        return balayageUnitArray.length;
    }

    public BalayageUnit getBalayageUnit(int i) {
        return balayageUnitArray[i];
    }

    public int getGlobalFicheSelectOptionsCount() {
        if (globalFicheSelectOptionsArray == null) {
            return 0;
        }
        return globalFicheSelectOptionsArray.length;
    }

    public FicheSelectOptions getGlobalFicheSelectOptions(int i) {
        return globalFicheSelectOptionsArray[i];
    }

    public int getBalayageModeCount() {
        if (balayageModeArray == null) {
            return 0;
        }
        return balayageModeArray.length;
    }

    public BalayageMode getBalayageMode(int i) {
        return balayageModeArray[i];
    }

    public String getPostCommand() {
        return postCommand;
    }

    public short getDefaultLangOption() {
        return defaultLangOption;
    }

    public SiteMapOption getSiteMapOption() {
        return siteMapOption;
    }

    protected void setLocalisationLists(LocalisationLists localisationLists) {
        this.localisationLists = localisationLists;
    }

    protected void setRootName(String rootName) {
        this.rootName = rootName;
    }

    protected void setPath(String path) {
        this.path = path;
    }

    protected void setWithTransformation(boolean withTransformation) {
        this.withTransformation = withTransformation;
    }

    protected void setBalayageModeArray(BalayageMode[] balayageModeArray) {
        this.balayageModeArray = balayageModeArray;
    }

    protected void setGlobalFicheSelectOptionsArray(FicheSelectOptions[] globalFicheSelectOptionsArray) {
        this.globalFicheSelectOptionsArray = globalFicheSelectOptionsArray;
    }

    protected void setPostCommand(String postCommand) {
        this.postCommand = postCommand;
    }

    protected void setDefaultLangOption(short defaultLangOption) {
        this.defaultLangOption = defaultLangOption;
    }

    protected void setBalayageUnitArray(BalayageUnit[] balayageUnitArray) {
        this.balayageUnitArray = balayageUnitArray;
    }

    protected void setSiteMapOption(SiteMapOption siteMapOption) {
        this.siteMapOption = siteMapOption;
    }

}
