/* BdfServer
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages;

import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultBalayageUnit implements BalayageUnit, BalayageConstants {

    private short type;
    private String extractionPath;
    private FicheSelectOptions ficheSelectOptions;
    private MotcleSelectOptions motcleSelectOptions;
    private Output[] outputArray;
    private boolean globalSelect = true;
    private String[] modeArray;
    private String[] extensionArray;
    private String name;
    private AlbumSelectOptions albumSelectOptions;

    public DefaultBalayageUnit(short type) {
        this.type = type;
    }

    public String getName() {
        return name;
    }

    public short getType() {
        return type;
    }

    public boolean isGlobalSelect() {
        return globalSelect;
    }

    public FicheSelectOptions getFicheSelectOptions() {
        return ficheSelectOptions;
    }

    public MotcleSelectOptions getMotcleSelectOptions() {
        return motcleSelectOptions;
    }

    public int getOutputCount() {
        if (outputArray == null) {
            return 0;
        }
        return outputArray.length;
    }

    public Output getOutput(int i) {
        return outputArray[i];
    }

    public boolean hasMode(String s) {
        if (modeArray == null) {
            return false;
        }
        int size = modeArray.length;
        if (size == 0) {
            return false;
        }
        for (int i = 0; i < size; i++) {
            if (modeArray[i].equals(s)) {
                return true;
            }
        }
        return false;
    }

    public int getExtensionCount() {
        if (extensionArray == null) {
            return 0;
        }
        return extensionArray.length;
    }

    public String getExtension(int i) {
        return extensionArray[i];
    }

    public String getExtractionPath() {
        return extractionPath;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setExtractionPath(String extractionPath) {
        this.extractionPath = extractionPath;
    }

    public void setModeArray(String[] modeArray) {
        this.modeArray = modeArray;
    }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.ficheSelectOptions = ficheSelectOptions;
    }

    public void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        this.motcleSelectOptions = motcleSelectOptions;
    }

    public void setExtensionArray(String[] extensionArray) {
        this.extensionArray = extensionArray;
    }

    public void setGlobalSelect(boolean globalSelect) {
        this.globalSelect = globalSelect;
    }

    public void setOutputArray(Output[] outputArray) {
        this.outputArray = outputArray;
    }

    public AlbumSelectOptions getAlbumSelectOptions() {
        return albumSelectOptions;
    }

    public void setAlbumSelectOptions(AlbumSelectOptions albumSelectOptions) {
        this.albumSelectOptions = albumSelectOptions;
    }

}
