/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages;

import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultOutput implements Output, BalayageConstants {

    private String name;
    private short langOption = LANG_OPTION_UNKNOWN;
    private String xsltPath;
    private AccoladePattern pattern;
    private AccoladePattern outputPathPattern;
    private String reversePath = "";
    private OutputParam[] outputParamArray;
    private String mode;
    private boolean cleanBefore = false;
    private boolean includeOutput = false;
    private LocalisationLists localisationLists;
    private BalayageUnit balayageUnit;

    public DefaultOutput() {
    }

    public String getName() {
        return name;
    }

    public String getXsltPath() {
        return xsltPath;
    }

    public AccoladePattern getPattern() {
        return pattern;
    }

    public AccoladePattern getOutputPath() {
        return outputPathPattern;
    }

    public int getOutputParamCount() {
        if (outputParamArray == null) {
            return 0;
        }
        return outputParamArray.length;
    }

    public OutputParam getOutputParam(int i) {
        return outputParamArray[i];
    }

    public String getMode() {
        return mode;
    }

    public boolean isCleanBefore() {
        return cleanBefore;
    }

    public short getLangOption() {
        return langOption;
    }

    public String getReversePath() {
        return reversePath;
    }

    public boolean isIncludeOutput() {
        return includeOutput;
    }

    public LocalisationLists getLocalisationLists() {
        return localisationLists;
    }

    protected void setName(String name) {
        this.name = name;
    }

    protected void setXsltPath(String xsltPath) {
        this.xsltPath = xsltPath;
    }

    protected void setMode(String mode) {
        this.mode = mode;
    }

    protected void setCleanBefore(boolean cleanBefore) {
        this.cleanBefore = cleanBefore;
    }

    protected void setLangOption(short langOption) {
        this.langOption = langOption;
    }

    protected void setIncludeOutput(boolean includeOutput) {
        this.includeOutput = includeOutput;
    }

    protected void setPattern(AccoladePattern pattern) {
        this.pattern = pattern;
    }

    protected void setOutputPathPattern(AccoladePattern outputPathPattern) {
        this.outputPathPattern = outputPathPattern;
        initReversePath();
    }

    protected void setOutputParamArray(OutputParam[] outputParamArray) {
        this.outputParamArray = outputParamArray;
    }

    protected void setLocalisationLists(LocalisationLists localisationLists) {
        this.localisationLists = localisationLists;
    }

    private void initReversePath() {
        if (outputPathPattern != null) {
            String path = outputPathPattern.toString();
            int length = path.length();
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < length; i++) {
                if (path.charAt(i) == '/') {
                    buf.append("../");
                }
            }
            reversePath = buf.toString();
        } else {
            reversePath = "";
        }
    }

    public BalayageUnit getBalayageUnit() {
        return balayageUnit;
    }

    public void setBalayageUnit(BalayageUnit balayageUnit) {
        this.balayageUnit = balayageUnit;
    }

}
