/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.builders;

import fr.exemole.bdfserver.balayages.api.*;


/**
 *
 * @author Vincent Calame
 */
public class SiteMapOptionBuilder {

    private static final String DEFAULT_FILENAME = "sitemap.xml";
    private InternalSiteMapOption siteMapOption = new InternalSiteMapOption();

    public SiteMapOptionBuilder() {
    }

    public SiteMapOption toSiteMapOption() {
        return siteMapOption;
    }

    public void setPath(String path) {
        siteMapOption.setPath(path);
    }

    public void setFileName(String fileName) {
        siteMapOption.setFileName(fileName);
    }

    public void setBaseUrl(String baseUrl) {
        siteMapOption.setBaseUrl(baseUrl);
    }


    private static class InternalSiteMapOption implements SiteMapOption {

        private String path = "";
        private String fileName = DEFAULT_FILENAME;
        private String baseUrl = "";

        public String getPath() {
            return path;
        }

        public String getFileName() {
            return fileName;
        }

        public String getBaseUrl() {
            return baseUrl;
        }

        private void setPath(String path) {
            if ((path == null) || (path.length() == 0)) {
                this.path = "";
            } else {
                this.path = path;
            }
        }

        private void setFileName(String fileName) {
            if ((fileName == null) || (fileName.length() == 0)) {
                this.fileName = DEFAULT_FILENAME;
            } else {
                this.fileName = fileName;
            }
        }

        private void setBaseUrl(String baseUrl) {
            if ((baseUrl == null) || (baseUrl.length() == 0)) {
                this.baseUrl = "";
            } else {
                this.baseUrl = baseUrl;
            }
        }

    }

}
