/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.engine.runners.RunnerFactory;
import fr.exemole.bdfserver.externalscript.ExternalScript;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class BalayageEngine implements BalayageConstants {

    private BalayageParameters balayageParameters;
    private List<BalayageUnitRunner> balayageUnitRunnerList = new ArrayList<BalayageUnitRunner>();
    private SiteMapBuilder siteMapBuilder;

    public BalayageEngine(BalayageParameters balayageParameters) {
        this.balayageParameters = balayageParameters;
        this.siteMapBuilder = (SiteMapBuilder) balayageParameters.getFileGenerationListener();
        Balayage balayage = balayageParameters.getBalayage();
        int balayageUnitCount = balayage.getBalayageUnitCount();
        for (int i = 0; i < balayageUnitCount; i++) {
            BalayageUnit balayageUnit = balayage.getBalayageUnit(i);
            if (doBalayageUnit(balayageUnit)) {
                BalayageUnitRunner bu = RunnerFactory.get(balayageParameters, i, balayageUnit);
                balayageUnitRunnerList.add(bu);
            }
        }
    }

    private boolean doBalayageUnit(BalayageUnit balayageUnit) {
        BalayageMode balayageMode = balayageParameters.getBalayageMode();
        if (balayageMode == null) {
            return true;
        }
        return balayageUnit.hasMode(balayageMode.getName());
    }

    public void run() {
        BalayageLog balayageLog = balayageParameters.getBalayageLog();
        balayageLog.runStarted();
        for (BalayageUnitRunner bu : balayageUnitRunnerList) {
            bu.run();
            bu.dispose();
        }
        balayageUnitRunnerList.clear();
        if (siteMapBuilder != null) {
            try {
                String path = siteMapBuilder.writeSiteMapFile(balayageParameters.getBalayageRootDirectory());
                balayageLog.addSiteMapGeneration(path);
            } catch (IOException ioe) {
                ioe.printStackTrace();
                String msg = ioe.getMessage();
                String message = ioe.getClass().getName();
                if (msg != null) {
                    message = message + " : " + msg;
                }
                balayageLog.addSiteMapException(BalayageLog.IO_EXCEPTION, message);
            }
        }
        execPostCommand();
        balayageLog.runEnded();
    }

    private boolean execPostCommand() {
        BalayageLog balayageLog = balayageParameters.getBalayageLog();
        if (balayageLog.hasError()) {
            return false;
        }
        String command = balayageParameters.getBalayage().getPostCommand();
        if (command == null) {
            return false;
        }
        ExternalScript externalScript = balayageParameters.getBdfServer().getExternalScriptManager().getExternalScript(command);
        if (externalScript == null) {
            balayageLog.addCommandException(BalayageLog.COMMANDCHECK_EXCEPTION, "wrong script name : " + command);
            return false;
        }
        try {
            externalScript.exec();
            balayageLog.addCommandRun(command);
            return true;
        } catch (IOException ioe) {
            balayageLog.addCommandException(BalayageLog.COMMANDIO_EXCEPTION, ioe.getMessage());
            return false;
        } /*catch(InterruptedException ie) {
        balayageLog.addCommandException(BalayageLog.COMMANDINTERRUPTED_EXCEPTION, ie.getMessage());
        return false;
        }*/
    }

}
