/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.*;
import fr.exemole.bdfserver.*;
import java.io.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayageParameters {

    private BalayageMode balayageMode;
    private BalayageLog balayageLog;
    private Balayage balayage;
    private BdfServer bdfServer;
    private File balayageRootDirectory;
    private LocalisationLists localisationsLists;
    private String[] langArray;
    private FicheEligibility ficheEligibility;
    private boolean ficheEligibilityInit = false;
    private FileGenerationListener fileGenerationListener;

    public BalayageParameters(Balayage balayage, BdfServer bdfServer) {
        this.balayage = balayage;
        this.bdfServer = bdfServer;
    }

    public BalayageMode getBalayageMode() {
        return balayageMode;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageLog getBalayageLog() {
        return balayageLog;
    }

    public void setBalayageLog(BalayageLog balayageLog) {
        this.balayageLog = balayageLog;
    }

    public Balayage getBalayage() {
        return balayage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public File getBalayageRootDirectory() {
        return balayageRootDirectory;
    }

    public void setBalayageRootDirectory(File balayageRootDirectory) {
        this.balayageRootDirectory = balayageRootDirectory;
    }

    public LocalisationLists getLocalisationLists() {
        initLocalisationLists();
        return localisationsLists;
    }

    public String[] getLangArray() {
        initLocalisationLists();
        return langArray;
    }

    public FicheEligibility getFicheEligibility() {
        if (ficheEligibilityInit) {
            return ficheEligibility;
        }
        int optionsCount = balayage.getGlobalFicheSelectOptionsCount();
        if (optionsCount > 0) {
            LocalisationLists localisationLists = getLocalisationLists();
            FicheSelector ficheSelector = new FicheSelector(bdfServer.getBasedefiches(), localisationLists.getWorkingLangInteger(0));
            for (int i = 0; i < optionsCount; i++) {
                ficheSelector.addFicheSelectOptions(balayage.getGlobalFicheSelectOptions(i));
            }
            AbregeFicheSortedList abregeFicheSortedList = SelectionUtils.getAbregeFicheSortedList(ficheSelector, AbregeFicheSortedList.NONE_SORTTYPE);
            this.ficheEligibility = abregeFicheSortedList;
        } else {
            this.ficheEligibility = null;
        }
        ficheEligibilityInit = true;
        return ficheEligibility;
    }

    private void initLocalisationLists() {
        if (localisationsLists != null) {
            return;
        }
        localisationsLists = balayage.getLocalisationLists();
        if (localisationsLists == null) {
            localisationsLists = LocalisationLists.newInstance(bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray());
        }
        int count = localisationsLists.getWorkingLangCount();
        langArray = new String[count];
        for (int i = 0; i < count; i++) {
            langArray[i] = LangInteger.toString(localisationsLists.getWorkingLangInteger(i));
        }
    }

    /**
     * @return the fileGenerationListener
     */
    public FileGenerationListener getFileGenerationListener() {
        return fileGenerationListener;
    }

    /**
     * @param fileGenerationListener the fileGenerationListener to set
     */
    public void setFileGenerationListener(FileGenerationListener fileGenerationListener) {
        this.fileGenerationListener = fileGenerationListener;
    }

}
