/* BdfServer
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.patterns.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.text.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class EngineUtils {

    private EngineUtils() {
    }

    public static ValueResolver getValueResolver(Object obj) {
        if (obj == null) {
            return AccoladePattern.DEFAULT_VALUERESOLVER;
        }
        if (obj instanceof Corpus) {
            return new CorpusValueResolver((Corpus) obj);
        }
        if (obj instanceof AbregeFiche) {
            return new FicheValueResolver((AbregeFiche) obj);
        }
        if (obj instanceof Thesaurus) {
            return new ThesaurusValueResolver((Thesaurus) obj);
        }
        if (obj instanceof Motcle) {
            return new MotcleValueResolver((Motcle) obj);
        }
        if (obj instanceof Document) {
            return new DocumentValueResolver((Document) obj);
        }
        if (obj instanceof Album) {
            return new AlbumValueResolver((Album) obj);
        }
        if (obj instanceof AlbumDim) {
            return new AlbumDimValueResolver((AlbumDim) obj);
        }
        return null;
    }

    public static String getPath(Output output, ValueResolver patternValueSet) {
        AccoladePattern outputPathPattern = output.getOutputPath();
        if (outputPathPattern != null) {
            String result = outputPathPattern.format(patternValueSet);
            if (result.length() == 0) {
                return null;
            } else {
                return result;
            }
        }
        return null;
    }

    public static String getFileName(Output output, ValueResolver valueResolver, short type, boolean withTransformation) {
        AccoladePattern pattern = BalayageUtils.getPattern(output, type);
        String fileName = pattern.format(valueResolver);
        if (fileName.lastIndexOf('.') == -1) {
            if (withTransformation) {
                fileName = fileName + ".html";
            } else {
                fileName = fileName + ".xml";
            }
        }
        return fileName;
    }

}
