/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.engine.*;
import fr.exemole.bdfserver.balayages.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.managers.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.xml.extraction.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractBalayageUnitRunner implements BalayageUnitRunner, BalayageConstants {

    protected BalayageParameters balayageParameters;
    protected BalayageLog balayageLog;
    protected BalayageUnit balayageUnit;
    protected int balayageIndex;
    protected Basedefiches basedefiches;
    protected LocalisationLists localisationLists;
    protected FicheEligibility ficheEligibility;

    public AbstractBalayageUnitRunner(BalayageParameters balayageParameters, int balayageIndex, BalayageUnit balayageUnit) {
        this.balayageParameters = balayageParameters;
        this.balayageLog = balayageParameters.getBalayageLog();
        this.balayageIndex = balayageIndex;
        this.balayageUnit = balayageUnit;
        this.basedefiches = balayageParameters.getBdfServer().getBasedefiches();
        this.localisationLists = balayageParameters.getLocalisationLists();
        this.ficheEligibility = balayageParameters.getFicheEligibility();
    }

    public void dispose() {
        balayageUnit = null;
        balayageParameters = null;
        balayageLog = null;
        basedefiches = null;
        localisationLists = null;
        ficheEligibility = null;
    }

    public void run() {
        if (isEmpty()) {
            balayageLog.addBalayageUnitWarning(balayageIndex, BalayageLog.NO_OUTPUT, null, null);
        } else {
            balayageLog.addBalayageUnitRun(balayageIndex);
            run(balayageUnit.getType());
        }
    }

    protected abstract boolean isEmpty();

    protected abstract void run(short balayageUnitType);

}
