/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.engine.*;
import fr.exemole.bdfserver.balayages.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.album.*;


/**
 *
 * @author Vincent Calame
 */
public class AlbumBalayageUnitRunner extends AbstractBalayageUnitRunner {

    private AlbumOutputRunner[] outputRunnerArray;

    public AlbumBalayageUnitRunner(BalayageParameters balayageParameters, int balayageIndex, BalayageUnit balayageUnit) {
        super(balayageParameters, balayageIndex, balayageUnit);
        List<AlbumOutputRunner> outputList = new ArrayList<AlbumOutputRunner>();
        int outputCount = balayageUnit.getOutputCount();
        int p = 0;
        for (int i = 0; i < outputCount; i++) {
            AlbumOutputRunner outputRunner = new AlbumOutputRunner(balayageParameters, p, balayageUnit.getOutput(i), balayageIndex);
            if (outputRunner.isValid()) {
                outputList.add(outputRunner);
                p++;
            }
        }
        outputRunnerArray = outputList.toArray(new AlbumOutputRunner[outputList.size()]);

    }

    protected void run(short balayageUnitType) {
        AlbumSelectOptions albumSelectOptions = balayageUnit.getAlbumSelectOptions();
        AlbumList albumList = SelectionUtils.getAlbumList(basedefiches, albumSelectOptions);
        String[] albumDimNameArray = null;
        if (albumSelectOptions != null) {
            albumDimNameArray = albumSelectOptions.getAlbumDimNameArray();
        }
        int albumCount = albumList.getAlbumCount();
        for (int j = 0; j < albumCount; j++) {
            long startTime = System.currentTimeMillis();
            Album album = albumList.getAlbum(j);
            for (int i = 0; i < outputRunnerArray.length; i++) {
                if (i > 0) {
                    startTime = System.currentTimeMillis();
                }
                AlbumOutputRunner outputRunner = outputRunnerArray[i];
                outputRunner.copy(album, albumDimNameArray, startTime);
            }
        }
    }

    public void dispose() {
        super.dispose();
        int length = outputRunnerArray.length;
        for (int i = 0; i < length; i++) {
            outputRunnerArray[i].dispose();
            outputRunnerArray[i] = null;
        }
        outputRunnerArray = null;
    }

    protected boolean isEmpty() {
        return (outputRunnerArray.length == 0);
    }

}
