/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.*;
import java.io.*;
import net.mapeadores.util.text.ValueResolver;
import net.mapeadores.util.io.IOUtils;
import fr.exemole.bdfserver.balayages.engine.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class AlbumOutputRunner extends AbstractOutputRunner {

    public AlbumOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
    }

    public void copy(Album album, String[] albumDimArray, long startTime) {
        if (albumDimArray == null) {
            copyImages(album, null, startTime);
        } else {
            int length = albumDimArray.length;
            for (int i = 0; i < length; i++) {
                AlbumDim albumDim = album.getAlbumMetadata().getAlbumDimByName(albumDimArray[i]);
                if (albumDim != null) {
                    copyImages(album, albumDim, startTime);
                }
            }
        }
    }

    private void copyImages(Album album, AlbumDim albumDim, long startTime) {
        ValueResolver valueResolver = null;
        if (albumDim != null) {
            valueResolver = EngineUtils.getValueResolver(albumDim);
        } else {
            valueResolver = EngineUtils.getValueResolver(album);
        }

        File directory = rootDirectory;
        String path = EngineUtils.getPath(output, valueResolver);
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        int length = album.getTicketLength();
        for (int i = 0; i < length; i++) {
            AlbumImage albumImage = (AlbumImage) album.getSubsetItemByTicket(i);
            if (albumImage != null) {
                String fileName = AlbumUtils.toFileName(albumImage);
                File destination = new File(directory, fileName);
                try {
                    OutputStream outputStream = new FileOutputStream(destination);
                    InputStream inputStream = albumImage.getInputStream(albumDim);
                    IOUtils.copy(inputStream, outputStream);
                    inputStream.close();
                    outputStream.close();
                    balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, destination, System.currentTimeMillis() - startTime);
                } catch (IOException ioe) {
                    String msg = ioe.getMessage();
                    String message = ioe.getClass().getName();
                    if (msg != null) {
                        message = message + " : " + msg;
                    }
                    balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.IO_EXCEPTION, destination.getName(), message);
                }
            }
        }
    }

}
