/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.*;
import java.io.*;
import net.mapeadores.util.text.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.util.io.IOUtils;
import fr.exemole.bdfserver.balayages.engine.*;


/**
 *
 * @author Vincent Calame
 */
public class DocumentOutputRunner extends AbstractOutputRunner {

    private ExtensionFilter extensionFilter;
    //private String outputMode;

    public DocumentOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
        extensionFilter = ExtensionFilterFactory.fromBalayageUnit(output.getBalayageUnit());
    }

    public void copy(ValueResolver valueResolver, Document document, long startTime) {
        File directory = rootDirectory;
        String path = EngineUtils.getPath(output, valueResolver);
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        for (int i = 0; i < document.getVersionCount(); i++) {
            Version version = document.getVersion(i);
            VersionName versionName = version.getVersionName();
            if (extensionFilter.acceptExtension(versionName.getExtension())) {
                File destination = new File(directory, versionName.toString());
                /*boolean copy = true;*/
                try {
                    /*if (outputMode != null) {
                    boolean done = false;
                    if (outputMode.equals("ln")) {
                    done = version.linkTo(destination, false);
                    } else if (outputMode.equals("ls")) {
                    done = version.linkTo(destination, true);
                    }
                    if (done) {
                    copy = false;
                    }
                    }
                    if (copy) {*/
                    OutputStream outputStream = new FileOutputStream(destination);
                    InputStream inputStream = version.getInputStream();
                    IOUtils.copy(inputStream, outputStream);
                    inputStream.close();
                    outputStream.close();
                    /*}*/
                    balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, destination, System.currentTimeMillis() - startTime);
                } catch (IOException ioe) {
                    String msg = ioe.getMessage();
                    String message = ioe.getClass().getName();
                    if (msg != null) {
                        message = message + " : " + msg;
                    }
                    balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.IO_EXCEPTION, destination.getName(), message);
                }
            }
        }
    }

    public void dispose() {
        super.dispose();
        extensionFilter = null;
        //outputMode = null;
    }

}
