/* BdfServer
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.xml;

import fr.exemole.bdfserver.balayages.*;
import net.mapeadores.basedefiches.xml.extraction.*;
import net.mapeadores.util.langinteger.*;
import org.w3c.dom.*;
import java.text.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.balayages.api.*;
import fr.exemole.bdfserver.balayages.builders.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayageDOMReader implements BalayageConstants {

    private Builder builder;

    public BalayageDOMReader(Builder builder) {
        this.builder = builder;
    }

    public void readBalayage(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        boolean metadone = false;
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if ((!metadone) && (tagname.equals("metadata"))) {
                    metadone = true;
                    readMetadata(el);
                } else if (tagname.equals("global-fiche-select")) {
                    FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(el);
                    if (ficheSelectOptions.containsOption()) {
                        builder.addGlobalFicheSelectOptions(ficheSelectOptions);
                    }
                } else if (tagname.equals("corpus-balayage")) {
                    addCorpusBalayageUnit(el);
                } else if (tagname.equals("fiche-balayage")) {
                    addFicheBalayageUnit(el);
                } else if (tagname.equals("thesaurus-balayage")) {
                    addThesaurusBalayageUnit(el);
                } else if (tagname.equals("motcle-balayage")) {
                    addMotcleBalayageUnit(el);
                } else if (tagname.equals("unique-balayage")) {
                    addUniqueBalayageUnit(el);
                } else if (tagname.equals("document-balayage")) {
                    addDocumentBalayageUnit(el);
                } else if (tagname.equals("album-balayage")) {
                    addAlbumBalayageUnit(el);
                } else if (tagname.equals("sitemap")) {
                    setSiteMapOption(el);
                }
            }
        }
    }

    private void setSiteMapOption(Element element_xml) {
        SiteMapOptionBuilder siteMapOptionBuilder = new SiteMapOptionBuilder();
        siteMapOptionBuilder.setBaseUrl(element_xml.getAttribute("base-url"));
        siteMapOptionBuilder.setPath(element_xml.getAttribute("path"));
        siteMapOptionBuilder.setFileName(element_xml.getAttribute("file-name"));
        builder.setSiteMapOption(siteMapOptionBuilder.toSiteMapOption());
    }

    private void readMetadata(Element element_xml) {
        String root = element_xml.getAttribute("root");
        if (root.length() > 0) {
            builder.setRootName(root);
        }
        String path = element_xml.getAttribute("path");
        if (path.length() > 0) {
            builder.setPath(path);
        }
        String param = element_xml.getAttribute("transform");
        if (param.equals("false")) {
            builder.setWithTransformation(false);
        } else {
            builder.setWithTransformation(true);
        }
        String postCommand = element_xml.getAttribute("post-command").trim();
        if (postCommand.length() > 0) {
            builder.setPostCommand(postCommand);
        }
        String defaultLangOption = element_xml.getAttribute("default-lang-option");
        try {
            short langOption = BalayageUtils.langOptionToShort(defaultLangOption);
            if (langOption == LANG_OPTION_UNKNOWN) {
                langOption = this.LANG_OPTION_NONE;
            }
            builder.setDefaultLangOption(langOption);
        } catch (IllegalArgumentException iae) {
        }
        String langs = element_xml.getAttribute("langs");
        if (langs.length() > 0) {
            int[] workingLangArray = LangIntegerUtils.toCleanLangIntegerArray(langs);
            if (workingLangArray.length > 0) {
                builder.setLocalisationLists(LocalisationLists.newInstance(workingLangArray));
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("mode")) {
                    String name = el.getAttribute("name");
                    BalayageMode mode = null;
                    try {
                        StringUtils.checkTechnicalName(name, true);
                        mode = builder.addBalayageMode(name);
                        String limit = el.getAttribute("limit");
                        if (limit.length() > 0) {
                            if (limit.equals("date")) {
                                builder.setWithCurrentDateLimit(mode, true);
                            }
                        }

                    } catch (java.text.ParseException pe) {
                    }
                }
            }
        }
    }

    private void addCorpusBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(CORPUS_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(element_xml);
        if (ficheSelectOptions.containsOption()) {
            balayageUnitBuilder.setFicheSelectOptions(ficheSelectOptions);
        }
    }

    private void addFicheBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(FICHE_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(element_xml);
        if (ficheSelectOptions.containsOption()) {
            balayageUnitBuilder.setFicheSelectOptions(ficheSelectOptions);
        }
    }

    private void addThesaurusBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(THESAURUS_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        MotcleSelectOptions motcleSelectOptions = ExtractionDOMReader.getMotcleSelectOptions(element_xml);
        if (motcleSelectOptions.containsOption()) {
            balayageUnitBuilder.setMotcleSelectOptions(motcleSelectOptions);
        }
    }

    private void addMotcleBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(MOTCLE_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        MotcleSelectOptions motcleSelectOptions = ExtractionDOMReader.getMotcleSelectOptions(element_xml);
        if (motcleSelectOptions.containsOption()) {
            balayageUnitBuilder.setMotcleSelectOptions(motcleSelectOptions);
        }
    }

    private void addUniqueBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(UNIQUE_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
    }

    private void addDocumentBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(DOCUMENT_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(element_xml);
        if (ficheSelectOptions.containsOption()) {
            balayageUnitBuilder.setFicheSelectOptions(ficheSelectOptions);
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("extension")) {
                    String extension = XMLUtils.getData(el);
                    if (extension.length() > 0) {
                        balayageUnitBuilder.addExtension(extension);
                    }
                }
            }
        }
    }

    private void addAlbumBalayageUnit(Element element_xml) {
        BalayageUnitBuilder balayageUnitBuilder = builder.addBalayageUnitBuilder(ALBUM_TYPE);
        commonBalayageUnit(balayageUnitBuilder, element_xml);
        AlbumSelectOptions albumSelectOptions = ExtractionDOMReader.getAlbumSelectOptions(element_xml);
        if (albumSelectOptions.containsOption()) {
            balayageUnitBuilder.setAlbumSelectOptions(albumSelectOptions);
        }
    }

    private void commonBalayageUnit(BalayageUnitBuilder balayageUnitBuilder, Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() > 0) {
            if (StringUtils.isTechnicalName(name, true)) {
                balayageUnitBuilder.setName(name);
            } else {
                //@todo
            }
        }
        String globalselect = element_xml.getAttribute("global-select");
        if (globalselect.equals("false")) {
            balayageUnitBuilder.setGlobalSelect(false);
        }
        String extractionPath = element_xml.getAttribute("extraction");
        if (extractionPath.length() > 0) {
            balayageUnitBuilder.setExtractionPath(extractionPath);
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("output")) {
                    OutputBuilder outputBuilder = balayageUnitBuilder.addOutputBuilder();
                    parseOutput(el, outputBuilder);
                }
            }
        }
        String mode = element_xml.getAttribute("mode");
        String[] tokens = StringUtils.getTechnicalTokens(mode, true);
        for (int i = 0; i < tokens.length; i++) {
            balayageUnitBuilder.addMode(tokens[i]);
        }

    }

    private void parseOutput(Element element_xml, OutputBuilder outputBuilder) {
        String outputName = element_xml.getAttribute("name");
        if (outputName.length() > 0) {
            if (StringUtils.isTechnicalName(outputName, true)) {
                outputBuilder.setName(outputName);
            } else {
                //@todo
            }
        }
        String xsltPath = element_xml.getAttribute("xslt");
        if (xsltPath.length() > 0) {
            outputBuilder.setXsltPath(xsltPath);
        }
        String mode = element_xml.getAttribute("mode");
        if (mode.length() > 0) {
            outputBuilder.setMode(mode);
        }
        String clean = element_xml.getAttribute("clean");
        boolean cleanBefore = (clean.equals("1"));
        outputBuilder.setCleanBefore(cleanBefore);
        String patternString = element_xml.getAttribute("pattern");
        if (patternString.length() > 0) {
            try {
                AccoladePattern pattern = new AccoladePattern(patternString);
                outputBuilder.setPattern(pattern);
            } catch (ParseException pe) {
                //balayageLog.addOutputWarning(balayageUnitIndex, outputIndex, "pattern", BalayageLog.WRONG_PATTERN, pattern);
                pe.printStackTrace();
            }
        }
        String langs = element_xml.getAttribute("langs");
        if (langs.length() > 0) {
            int[] workingLangArray = LangIntegerUtils.toCleanLangIntegerArray(langs);
            if (workingLangArray.length > 0) {
                outputBuilder.setLocalisationLists(LocalisationLists.newInstance(workingLangArray));
            }
        }
        String path = normalizePath(element_xml.getAttribute("path"));
        if (path != null) {
            if (path.equals("_inc_/")) {
                outputBuilder.setIncludeOutput(true);
            } else {
                outputBuilder.setIncludeOutput(false);
                try {
                    AccoladePattern outputPathPattern = new AccoladePattern(path);
                    outputBuilder.setOutputPathPattern(outputPathPattern);
                } catch (ParseException pe) {
                    //balayageLog.addOutputWarning(balayageUnitIndex, outputIndex, "path", BalayageLog.WRONG_PATH, path);
                    pe.printStackTrace();
                }
            }

        }
        String langOption = element_xml.getAttribute("lang-option");
        if (langOption.length() > 0) {
            try {
                outputBuilder.setLangOption(BalayageUtils.langOptionToShort(langOption));
            } catch (IllegalArgumentException iae) {
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("param")) {
                    String name = el.getAttribute("name");
                    if (name.length() > 0) {
                        String value = el.getAttribute("value");
                        outputBuilder.addOutputParam(name, value);
                    }
                }
            }
        }
    }

    private String normalizePath(String path) {
        int length = path.length();
        if (length == 0) {
            return null;
        }
        if (path.charAt(0) == '/') {
            return normalizePath(path.substring(1));
        }
        char lastChar = path.charAt(length - 1);
        if (lastChar != '/') {
            path = path + '/';
            length = length + 1;
        }
        return path;
    }


    public interface Builder {

        public void addGlobalFicheSelectOptions(FicheSelectOptions ficheSelectOptions);

        public void setRootName(String rootName);

        public void setPath(String path);

        public void setWithTransformation(boolean withTransformation);

        public void setPostCommand(String postCommand);

        public void setDefaultLangOption(short defaultLangOption);

        public void setLocalisationLists(LocalisationLists localisationLists);

        public BalayageMode addBalayageMode(String name);

        public void setWithCurrentDateLimit(BalayageMode balayageMode, boolean b);

        public BalayageUnitBuilder addBalayageUnitBuilder(short type);

        public void setSiteMapOption(SiteMapOption siteMapOption);

    }


    public interface BalayageUnitBuilder {

        public void setName(String name);

        public void setGlobalSelect(boolean globalSelect);

        public void addMode(String mode);

        public void setExtractionPath(String extractionPath);

        public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions);

        public void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions);

        public void setAlbumSelectOptions(AlbumSelectOptions albumSelectOptions);

        public void addExtension(String s);

        public OutputBuilder addOutputBuilder();

    }


    public interface OutputBuilder {

        public void setName(String name);

        public void setXsltPath(String xsltPath);

        public void setMode(String mode);

        public void setCleanBefore(boolean cleanBefore);

        public void setLangOption(short langOption);

        public void setIncludeOutput(boolean includeOutput);

        public void setPattern(AccoladePattern pattern);

        public void setOutputPathPattern(AccoladePattern outputPathPattern);

        public void addOutputParam(String name, String value);

        public void setLocalisationLists(LocalisationLists localisationLists);

    }

}
