/* BdfServer
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.beans;

import java.io.Serializable;


/**
 *
 *@author  Vincent Calame
 */
public final class HtmlInputBean implements Serializable {

    public final static int SIZE_SMALL = -1;
    public final static int SIZE_MEDIUM = -2;
    public final static int SIZE_LARGE = - 3;
    public final static short INPUT_TEXT = 10;
    public final static short INPUT_SELECT = 11;
    private int size = SIZE_LARGE;
    private int rows = 1;
    private short inputType = INPUT_TEXT;

    public HtmlInputBean() {
    }

    public HtmlInputBean(HtmlInputBean oldInstance) {
        this.size = oldInstance.size;
        this.rows = oldInstance.rows;
        this.inputType = oldInstance.inputType;
    }

    /*
     *
     *@throws IllegalArgumentException si un des arguments a une valeur incorrecte
     */
    public HtmlInputBean(int size, int rows, short inputType) {
        setSize(size);
        setRows(rows);
        setInputType(inputType);
    }

    /**
     * Retourne la taille que doit prendre le champ de saisie.
     * Cette valeur est, soit un entier strictement positif, soit une des valeurs (n�gatives) SIZE_SMALL, SIZE_MEDIUM ou SIZE_LARGE.
     */
    public int getSize() {
        return size;
    }

    /*
     *
     *@throws IllegalArgumentException si size <= 0 (sauf les valeurs SIZE_SMALL, SIZE_MEDIUM, SIZE_LARGE)
     */
    public void setSize(int size) {
        if ((size == 0) || (size < -3)) {
            throw new IllegalArgumentException("size <= 0");
        }
        this.size = size;
    }

    /**
     * Retourne le nombre de ligne minimal. La valeur est un entier strictement positif.
     */
    public int getRows() {
        return rows;
    }

    /*
     *
     *@throws IllegalArgumentException si rows <= 0
     */
    public void setRows(int rows) {
        if (rows <= 0) {
            throw new IllegalArgumentException("rows <= 0");
        }
        this.rows = rows;
    }

    public short getInputType() {
        return inputType;
    }

    /*
     *
     *@throws IllegalArgumentException si inputType n'est pas égal à INPUT_TEXT ou INPUT_SELECT
     */
    public void setInputType(short inputType) {
        switch (inputType) {
            case INPUT_TEXT:
            case INPUT_SELECT:
                this.inputType = inputType;
                break;
            default:
                throw new IllegalArgumentException("inputType has Wrong value");

        }
    }

    public int hashCode() {
        return rows * 1000 + size * 10 + inputType;
    }

    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof HtmlInputBean)) {
            return false;
        }
        HtmlInputBean otherBean = (HtmlInputBean) obj;
        return ((this.size == otherBean.size) && (this.rows == otherBean.rows) && (this.inputType == otherBean.inputType));
    }

    public final static String inputTypeToString(short inputType) {
        switch (inputType) {
            case INPUT_TEXT:
                return "text";
            case INPUT_SELECT:
                return "select";
            default:
                throw new IllegalArgumentException("wrong value");
        }
    }

    public final static short inputTypeToShort(String s) {
        if (s.equals("text")) {
            return INPUT_TEXT;
        }
        if (s.equals("select")) {
            return INPUT_SELECT;
        }
        throw new IllegalArgumentException("wrong value");
    }

    public final static String sizeToString(int size) {
        switch (size) {
            case SIZE_SMALL:
                return "small";
            case SIZE_MEDIUM:
                return "medium";
            case SIZE_LARGE:
                return "large";
            default:
                throw new IllegalArgumentException("wrong value");
        }
    }

    public final static int sizeToInt(String s) {
        if (s.equals("small")) {
            return SIZE_SMALL;
        }
        if (s.equals("medium")) {
            return SIZE_MEDIUM;
        }
        if (s.equals("large")) {
            return SIZE_LARGE;
        }
        throw new IllegalArgumentException("wrong value");
    }

}
