/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.commands;

import java.util.*;
import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class CommandFactory {

    private CommandFactory() {
    }

    public static CommandDefList build(InputStream inputStream) throws IOException, SAXException {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(inputStream);
            CommandBuilder commandBuilder = new CommandBuilder();
            CommandListDOMReader reader = new CommandListDOMReader(commandBuilder, null);
            reader.readCommandList(doc.getDocumentElement());
            return commandBuilder.toCommandDefList();
        } catch (ParserConfigurationException pce) {
            throw new NestedLibraryException(pce);
        }
    }


    private static class CommandBuilder implements CommandListDOMReader.Builder {

        private LinkedHashMap<String, InternalCommandDef> map = new LinkedHashMap<String, InternalCommandDef>();

        public CommandListDOMReader.CommandDefBuilder createCommand(String name) {
            InternalCommandDef internalCommandDef = map.get(name);
            if (internalCommandDef == null) {
                internalCommandDef = new InternalCommandDef(name);
                map.put(name, internalCommandDef);
            }
            return internalCommandDef;
        }

        private CommandDefList toCommandDefList() {
            int size = map.size();
            if (size == 0) {
                return new DefaultCommandDefList(null);
            }
            CommandDef[] array = new CommandDef[size];
            int p = 0;
            for (InternalCommandDef internalCommandDef : map.values()) {
                array[p] = internalCommandDef.toCommandDef();
                p++;
            }
            return new DefaultCommandDefList(array);
        }

    }


    private static class InternalCommandDef implements CommandListDOMReader.CommandDefBuilder {

        private String name;
        private String key;
        private List<DefaultParamDef> paramList = new ArrayList<DefaultParamDef>();

        InternalCommandDef(String name) {
            this.name = name;
        }

        public void setKey(String key) {
            this.key = key;
        }

        public void addParameter(String name) {
            paramList.add(new DefaultParamDef(name));
        }

        private CommandDef toCommandDef() {
            int size = paramList.size();
            if (size == 0) {
                return new DefaultCommandDef(name, key, null);
            }
            ParamDef[] array = new ParamDef[size];
            int p = 0;
            for (DefaultParamDef paramDef : paramList) {
                array[p] = paramDef;
                p++;
            }
            return new DefaultCommandDef(name, key, array);
        }

    }

}
