/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.externalscript.impl;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.externalscript.ExternalScript;
import fr.exemole.bdfserver.externalscript.ExternalScriptManager;
import java.io.File;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class DefaultExternalScriptManager implements ExternalScriptManager {

    private BdfServer bdfServer;
    private String externalScriptPath;

    public DefaultExternalScriptManager(BdfServer bdfServer, String externalScriptPath) {
        this.bdfServer = bdfServer;
        this.externalScriptPath = externalScriptPath;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public String getExternalScriptPath() {
        return externalScriptPath;
    }

    public ExternalScript getExternalScript(String name) {
        if (externalScriptPath == null) {
            return null;
        }
        if ((name == null) || (name.length() == 0)) {
            return null;
        }
        File dir = new File(externalScriptPath);
        if ((!dir.exists()) || (!dir.isDirectory())) {
            return null;
        }
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            File file = files[i];
            if (!file.isDirectory()) {
                String fileName = file.getName();
                if (fileName.equals(name)) {
                    return new ImplExternalScript(name, file);
                }
            }
        }
        return null;
    }


    private static class ImplExternalScript implements ExternalScript {

        private String name;
        private File file;

        ImplExternalScript(String name, File file) {
            this.name = name;
            this.file = file;
        }

        public String getName() {
            return name;
        }

        public Process exec() throws IOException {
            Runtime runtime = Runtime.getRuntime();
            return runtime.exec(file.getPath());
        }

    }

}

