/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.imagesource;

import net.mapeadores.util.json.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.logging.*;
import java.io.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.album.*;


/**
 *
 * @author Vincent Calame
 */
public class ActionResultToJson implements JsonProducer {

    private ActionResult actionResult;
    private ResourceLocalisation resourceLocalisation;

    public ActionResultToJson(ActionResult actionResult, ResourceLocalisation resourceLocalisation) {
        this.actionResult = actionResult;
        this.resourceLocalisation = resourceLocalisation;
    }

    public void writeJson(Appendable appendable) throws IOException {
        short action = actionResult.getActionType();
        String tempFileName = actionResult.getTempFileName();
        AlbumImage albumImage = actionResult.getAlbumImage();
        CommandMessage commandMessage = actionResult.getCommandMessage();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("action");
            jsonWriter.value(ImageSourceUtils.actionToString(action));
            if (tempFileName != null) {
                jsonWriter.key("tempFileName");
                jsonWriter.value(tempFileName);
                jsonWriter.key("tempFileSrc");
                jsonWriter.value(actionResult.getTempFileSrc());
            }
            if (albumImage != null) {
                jsonWriter.key("albumImageName");
                jsonWriter.value(AlbumUtils.toFileName(albumImage));
                jsonWriter.key("albumImageSrc");
                jsonWriter.value(actionResult.getAlbumImageSrc());
            }
            if (commandMessage != null) {
                jsonWriter.key("key");
                jsonWriter.value(commandMessage.getKeyResourceItem().getResourceKey());
                if (resourceLocalisation != null) {
                    jsonWriter.key("text");
                    jsonWriter.value(resourceLocalisation.toString(commandMessage.getKeyResourceItem()));
                }
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
