/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.imagesource;

import net.mapeadores.util.logging.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.album.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultActionResult implements ActionResult, ImageSourceConstants {

    private short actionType;
    private short state;
    private CommandMessage commandMessage;
    private CorpusField corpusField;
    private String tempFileName;
    private String tempFileSrc;
    private AlbumImage albumImage;
    private String albumImageSrc;

    public DefaultActionResult() {
    }

    public short getActionType() {
        return actionType;
    }

    public short getState() {
        return state;
    }

    public CommandMessage getCommandMessage() {
        return commandMessage;
    }

    public static DefaultActionResult successInstance(short action, CorpusField corpusField) {
        DefaultActionResult actionResult = new DefaultActionResult();
        actionResult.actionType = action;
        actionResult.corpusField = corpusField;
        actionResult.state = SUCCESS_STATE;
        return actionResult;
    }

    public static DefaultActionResult errorInstance(short action, CorpusField corpusField) {
        DefaultActionResult actionResult = new DefaultActionResult();
        actionResult.actionType = action;
        actionResult.corpusField = corpusField;
        actionResult.state = ERROR_STATE;
        return actionResult;
    }

    public static DefaultActionResult errorInstance(short action, CorpusField corpusField, String errorKey) {
        return errorInstance(action, corpusField, errorKey, null);
    }

    public static DefaultActionResult errorInstance(short action, CorpusField corpusField, String errorKey, String errorValue) {
        DefaultActionResult actionResult = new DefaultActionResult();
        actionResult.actionType = action;
        actionResult.corpusField = corpusField;
        actionResult.commandMessage = CommandMessage.error(errorKey, errorValue);
        actionResult.state = ERROR_STATE;
        return actionResult;
    }

    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    public CorpusField getCorpusField() {
        return corpusField;
    }

    public void setTempFile(String tempFileName, String tempFileSrc) {
        this.tempFileName = tempFileName;
        this.tempFileSrc = tempFileSrc;
    }

    public void setAlbumImage(AlbumImage albumImage, String albumImageSrc) {
        this.albumImage = albumImage;
        this.albumImageSrc = albumImageSrc;
    }

    public String getTempFileName() {
        return tempFileName;
    }

    public String getTempFileSrc() {
        return tempFileSrc;
    }

    public AlbumImage getAlbumImage() {
        return albumImage;
    }

    public String getAlbumImageSrc() {
        return albumImageSrc;
    }

}
