/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.implementations;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.xsltfactory.XsltResource;
import java.io.File;
import java.io.IOException;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.DocStream;


/**
 *
 * @author Vincent Calame
 */
public class TransformationURIResolver implements URIResolver {

    private BdfServer bdfServer;

    public TransformationURIResolver(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public Source resolve(String href, String base) throws TransformerException {
        String resourceName = href;
        if (resourceName.startsWith("../_inc_/")) {
            resourceName = "bfd:_inc_/" + resourceName.substring("../_inc_/".length());
        }
        if (resourceName.startsWith("bdf:")) {
            resourceName = resourceName.substring(4);
        } else {
            int idx = href.lastIndexOf('/');
            if (idx != -1) {
                return null;
            }
        }
        if (resourceName.startsWith("_inc_/")) {
            File incDir = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.BALAYAGEINCLUDE_OUTPUTDIRECTORY);
            File f = new File(incDir, resourceName.substring("_inc_/".length()));
            if (f.exists() && (!f.isDirectory())) {
                return new StreamSource(f);
            } else {
                return null;
            }
        }
        XsltResource innerResource = XsltResource.parse(resourceName);
        if (innerResource == null) {
            return null;
        }
        DocStream docStream = bdfServer.getInnerResourceDocStream(innerResource, true);
        if (docStream == null) {
            return null;
        }
        try {
            return new StreamSource(docStream.getInputStream());
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

}
