/* BdfServer
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.libscol;

import java.io.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.xml.libs.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class LibsColXMLWriter extends DefaultXMLWriter {

    LibsXMLWriter libsXMLWriter;
    private int langInteger;
    private BdfServer bdfServer;
    private Basedefiches basedefiches;

    public LibsColXMLWriter(BdfServer bdfServer, int langInteger, Appendable appendable, int indentLength) {
        super();
        setAppendable(appendable);
        setIndentLength(indentLength);
        this.langInteger = langInteger;
        libsXMLWriter = new LibsXMLWriter(langInteger, appendable, indentLength + 1);
        this.bdfServer = bdfServer;
        this.basedefiches = bdfServer.getBasedefiches();
    }

    public void appendLibsCol(LibsColKey libsColKey) throws IOException {
        openTag("libscol");
        if (libsColKey.isFromSubset()) {
            SubsetKey subsetKey = libsColKey.getSubsetKey();
            Subset subset = bdfServer.getBasedefiches().getSubset(subsetKey);
            if (subset instanceof Corpus) {
                appendCorpus((Corpus) subset);
            }
        } else if (libsColKey.equals(LibsColKey.ALLCORPUS)) {
            appendAllCorpus();
        } else {
            throw new SwitchException("Unknown libsColKey = " + libsColKey.toString());
        }
        closeTag("libscol");
    }

    private void appendAllCorpus() throws IOException {
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            appendCorpus(basedefiches.getCorpus(i));
        }
    }

    private void appendCorpus(Corpus corpus) throws IOException {
        libsXMLWriter.appendCorpusLibs(corpus.getCorpusMetadata());
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        appendCorpusUI(corpusUI);
    }

    private void appendCorpusUI(CorpusUI corpusUI) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "corpusui");
        addAttribute("corpus", corpusUI.getCorpusSubsetKey().getSubsetName());
        endOpenTag();
        IncludeUILibelleMap libelleSourceCorpusInclude = bdfServer.getIncludeLibelleManager().getIncludeUILibelleMap(corpusUI.getCorpusSubsetKey());
        List includeList = corpusUI.getOrderedComponentUIList(CorpusUI.INCLUDEUI_LIST);
        for (Iterator it = includeList.iterator(); it.hasNext();) {
            IncludeUI includeUI = (IncludeUI) it.next();
            String lib = libelleSourceCorpusInclude.getLibelleString(includeUI, langInteger);
            appendIncludeUILib(includeUI.getFiteredIncludeKey().toString(), lib);
        }
        appendIncludeKeyLib(libelleSourceCorpusInclude, IncludeKey.DATE_CREATION);
        appendIncludeKeyLib(libelleSourceCorpusInclude, IncludeKey.DATE_MODIFICATION);
        List commentList = corpusUI.getOrderedComponentUIList(CorpusUI.COMMENT_LIST);
        for (Iterator it = commentList.iterator(); it.hasNext();) {
            CommentUI commentUI = (CommentUI) it.next();
            appendCommentUILib(commentUI.getName(), commentUI.getLibelleString(langInteger, false));
        }
        closeTag("libs");
    }

    private void appendIncludeKeyLib(IncludeLibelleMap includeLibelleMap, IncludeKey includeKey) throws IOException {
        String lib = includeLibelleMap.getLibelleString(new FilteredIncludeKey(includeKey), langInteger);
        appendIncludeUILib(includeKey.toString(), lib);
    }

    private void appendIncludeUILib(String includeKey, String lib) throws IOException {
        startOpenTag("lib");
        addAttribute("include-key", includeKey.toString());
        endOpenTag(false);
        escape(lib);
        closeTag("lib", false, false);
    }

    private void appendCommentUILib(String commentName, String lib) throws IOException {
        startOpenTag("lib");
        addAttribute("comment-name", commentName);
        endOpenTag(false);
        escape(lib);
        closeTag("lib", false, false);
    }

    public static String getLibsColXML(int langInteger, BdfServer bdfServer, LibsColKey libsColKey, boolean withTabs) {
        StringBuilder buf = new StringBuilder(1024);
        try {
            LibsColXMLWriter libsXMLWriter = new LibsColXMLWriter(bdfServer, langInteger, buf, (withTabs) ? 0 : -999);
            libsXMLWriter.appendXMLDeclaration();
            libsXMLWriter.appendLibsCol(libsColKey);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
