/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.storage.BalayagesStorage;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.extraction.Extraction;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.xml.extraction.ExtractionDOMReader;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.NestedSAXException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.LocalisationLists;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class BalayageManager implements Manager, BalayageConstants {

    private BdfServer bdfServer;
    private BalayagesStorage balayageStorage;

    public BalayageManager(BdfServer bdfServer, BalayagesStorage balayageStorage) {
        this.bdfServer = bdfServer;
        this.balayageStorage = balayageStorage;
    }

    public void update() {
        balayageStorage.update();
    }

    public int getBalayageCount() {
        return balayageStorage.getBalayageCount();
    }

    public Balayage getBalayage(int i) {
        return balayageStorage.getBalayage(i);
    }

    public Balayage getBalayage(String name) {
        return balayageStorage.getBalayage(name);
    }

    /**
     * Retourne <em>null</em> si la ressource n'existe pas et une exception s'il y a un problème
     * pour y accéder.
     *
     */
    public Extraction getExtraction(Balayage balayage, String extractionPath, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        try {
            InputStream inputStream = balayageStorage.getExtractionInputStream(balayage.getName(), extractionPath);
            if (inputStream == null) {
                return null;
            }
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docbuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = docbuilder.parse(inputStream);
            ExtractionDOMReader extractionDomReader = new ExtractionDOMReader(bdfServer.getBasedefiches(), localisationLists, ficheEligibility);
            Extraction extraction = extractionDomReader.readExtraction(document.getDocumentElement());
            return extraction;
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        } catch (org.xml.sax.SAXException se) {
            throw new NestedSAXException(se);
        } catch (ParserConfigurationException pce) {
            throw new NestedLibraryException(pce);
        }
    }

    public Extraction getDefaultExtraction(short balayageUnitType, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        switch (balayageUnitType) {
            case CORPUS_TYPE:
                return Extraction.newDefaultCorpusInstance(basedefiches, localisationLists, ficheEligibility);
            case FICHE_TYPE:
                return Extraction.newDefaultFicheInstance(basedefiches, localisationLists, ficheEligibility);
            case MOTCLE_TYPE:
                return Extraction.newDefaultMotcleInstance(basedefiches, localisationLists, ficheEligibility);
            case THESAURUS_TYPE:
                return Extraction.newDefaultThesaurusInstance(basedefiches, localisationLists, ficheEligibility);
            case UNIQUE_TYPE:
                return Extraction.newDefaultUniqueInstance(basedefiches, localisationLists, ficheEligibility);
            default:
                throw new SwitchException("wrong balayageUnitType = " + balayageUnitType);
        }
    }

    public Source getTransformerSource(Balayage balayage, String xsltPath) {
        return balayageStorage.getTransformerSource(balayage.getName(), xsltPath);
    }

}
