/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import net.mapeadores.basedefiches.include.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.util.text.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class BdfServerIncludeLibelleManager implements IncludeLibelleManager {

    private DefaultIncludeLibelleMap defaultInclude;
    private BdfServer bdfServer;
    private Map ilmMap = new HashMap();

    public BdfServerIncludeLibelleManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        defaultInclude = new DefaultIncludeLibelleMap(bdfServer.getBasedefiches());
    }

    public IncludeLibelleMap getIncludeLibelleMap(SubsetKey corpusKey) {
        return get(corpusKey);
    }

    public IncludeUILibelleMap getIncludeUILibelleMap(SubsetKey corpusKey) {
        return get(corpusKey);
    }

    private BdfServerIncludeLibelleMap get(SubsetKey corpusKey) {
        BdfServerIncludeLibelleMap ilm = (BdfServerIncludeLibelleMap) ilmMap.get(corpusKey);
        if (ilm == null) {
            ilm = new BdfServerIncludeLibelleMap(corpusKey);
            ilmMap.put(corpusKey, ilm);
        }
        return ilm;
    }


    private class BdfServerIncludeLibelleMap implements IncludeUILibelleMap {

        SubsetKey corpusKey;

        BdfServerIncludeLibelleMap(SubsetKey corpusKey) {
            this.corpusKey = corpusKey;
        }

        public String getLibelleString(FilteredIncludeKey filteredIncludeKey, int langInteger) {
            Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
            CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
            if (corpusUI == null) {
                return defaultInclude.getLibelleString(filteredIncludeKey, langInteger);
            }
            IncludeUI includeUI = (IncludeUI) corpusUI.getIncludeUI(filteredIncludeKey);
            if (includeUI == null) {
                return defaultInclude.getLibelleString(filteredIncludeKey, langInteger);
            } else {
                return getLibelleString(includeUI, langInteger);
            }
        }

        public String getLibelleString(IncludeUI includeUI, int langInteger) {
            LibelleHolder libelleHolder = includeUI.getCustomLibelle();
            if (libelleHolder == null) {
                return defaultInclude.getLibelleString(includeUI.getFiteredIncludeKey(), langInteger);
            } else {
                String lib = libelleHolder.getLibelleString(langInteger, false);
                if (lib == null) {
                    return defaultInclude.getLibelleString(includeUI.getFiteredIncludeKey(), langInteger);
                }
                return lib;
            }
        }

    }

}
