/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import java.security.MessageDigest;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class PasswordManager implements BeanableManager {

    private String confirmPassword = "test";
    private Map<RedacteurKey, String> passwordMap = new HashMap<RedacteurKey, String>();
    private BdfServer bdfServer;

    public PasswordManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public Redacteur getRedacteur(RedacteurKey redacteurKey) {
        return SphereUtils.getRedacteur(bdfServer.getBasedefiches(), redacteurKey);
    }

    public String getConfirmPassword() {
        return confirmPassword;
    }

    /**
     *
     * @throws IllegalArgumentException si confirmPassword est nul.
     */
    public void setConfirmPassword(String confirmPassword) {
        if (confirmPassword == null) {
            throw new IllegalArgumentException("confirmPassword is null");
        }
        this.confirmPassword = confirmPassword;
    }

    public String getPassword(RedacteurKey redacteurKey) {
        return passwordMap.get(redacteurKey);
    }

    public void setPassword(Redacteur redacteur, String password) {
        RedacteurKey redacteurKey = redacteur.getRedacteurKey();
        if (password == null) {
            passwordMap.remove(redacteurKey);
        } else {
            passwordMap.put(redacteurKey, password);
        }
    }

    /**
     * Supprime tout référence à un mot de passe pour la clé RedacteurKey.
     * Retourne <em>true</em> s'il y a effectivement eu une suppression.
     */
    public boolean removePassword(RedacteurKey redacteurKey) {
        String old = passwordMap.remove(redacteurKey);
        return (old != null);
    }

    public Set<Map.Entry<RedacteurKey, String>> getEntrySet() {
        return passwordMap.entrySet();
    }

    public String getMD5Password(RedacteurKey redacteurKey) {
        String password = getPassword(redacteurKey);
        if (password == null) {
            return null;
        }
        try {
            MessageDigest msgDigest = MessageDigest.getInstance("MD5");
            msgDigest.update(password.getBytes());
            byte[] digest = msgDigest.digest();
            String md5 = "";
            for (int i = 0; i < digest.length; ++i) {
                int value = digest[i];
                if (value < 0) {
                    value += 256;
                }
                String hex = Integer.toHexString(value);
                if (hex.length() == 1) {
                    hex = "0" + hex;
                }
                md5 = md5 + hex;
            }
            return md5;
        } catch (Exception e) {
            throw new ShouldNotOccurException(e);
        }

    }

}
