/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class PolicyManager implements BeanableManager {

    private BdfServer bdfServer;
    private Map<SubsetKey, ThesaurusPolicy> policyMap = new HashMap<SubsetKey, ThesaurusPolicy>();
    private boolean passwordModifAllowed = false;
    private boolean dataModifAllowed = false;

    public PolicyManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            createThesaurusPolicy(thesaurus);
        }
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public ThesaurusPolicy getThesaurusPolicy(SubsetKey thesaurusKey) {
        return policyMap.get(thesaurusKey);
    }

    public ThesaurusPolicy createThesaurusPolicy(Thesaurus thesaurus) {
        ThesaurusPolicy thesaurusPolicy = new ThesaurusPolicy(thesaurus.getSubsetKey(), thesaurus.getThesaurusMetadata().getThesaurusType());
        policyMap.put(thesaurus.getSubsetKey(), thesaurusPolicy);
        return thesaurusPolicy;
    }

    public void removeThesaurusPolicy(SubsetKey thesaurusKey) {
        policyMap.remove(thesaurusKey);
        for (ThesaurusPolicy thesaurusPolicy : policyMap.values()) {
            short currentPolicy = thesaurusPolicy.getAjoutPolicy();
            if (currentPolicy == ThesaurusPolicy.AJOUT_TRANSFERT) {
                SubsetKey transferThesaurusKey = thesaurusPolicy.getTransferThesaurusKey();
                if (transferThesaurusKey.equals(thesaurusKey)) {
                    thesaurusPolicy.setAjoutPolicy(ThesaurusPolicy.AJOUT_AUCUN, null);
                }
            } else if (currentPolicy == ThesaurusPolicy.AJOUT_VERIFICATION) {
                List<SubsetKey> list = thesaurusPolicy.getVerifThesaurusKeyList();
                int idx = list.indexOf(thesaurusKey);
                if (idx != -1) {
                    if (list.size() == 0) {
                        thesaurusPolicy.setAjoutPolicy(ThesaurusPolicy.AJOUT_AUTORISE, null);
                    } else {
                        ArrayList<SubsetKey> newList = new ArrayList<SubsetKey>();
                        for (SubsetKey key2 : list) {
                            if (!thesaurusKey.equals(key2)) {
                                newList.add(key2);
                            }
                        }
                        thesaurusPolicy.setAjoutPolicy(ThesaurusPolicy.AJOUT_VERIFICATION, newList);
                    }
                }
            }
        }
    }

    public boolean isPasswordModifAllowed() {
        return passwordModifAllowed;
    }

    public boolean setPasswordModifAllowed(boolean passwordModifAllowed) {
        if (this.passwordModifAllowed == passwordModifAllowed) {
            return false;
        }
        this.passwordModifAllowed = passwordModifAllowed;
        return true;
    }

    public boolean isDataModifAllowed() {
        return dataModifAllowed;
    }

    public boolean setDataModifAllowed(boolean dataModifAllowed) {
        if (this.dataModifAllowed == dataModifAllowed) {
            return false;
        }
        this.dataModifAllowed = dataModifAllowed;
        return true;
    }

}
