/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.configuration;

import java.io.File;
import java.net.URL;


/**
 *
 *
 * @author Vincent Calame
 */
public class ConfigurationPath {

    private String name;
    private URL publicURL;
    private File systemDirectory;

    public ConfigurationPath() {
    }

    public ConfigurationPath(String name, URL publicURL, File systemDirectory) {
        setName(name);
        setPublicURL(publicURL);
        setSystemFile(systemDirectory);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public File getSystemDirectory() {
        return systemDirectory;
    }

    public String getSystemDirectoryString() {
        if (systemDirectory == null) {
            return "";
        }
        return systemDirectory.getAbsolutePath();
    }

    public String getPublicURLString() {
        if (publicURL == null) {
            return "";
        }
        return publicURL.toString();
    }

    public void setSystemFile(File systemFile) {
        this.systemDirectory = systemFile;
    }

    public URL getPublicURL() {
        return publicURL;
    }

    public void setPublicURL(URL publicURL) {
        this.publicURL = publicURL;
    }

    public boolean hasPublicURL() {
        return (publicURL != null);
    }

    public boolean hasSystemDirectory() {
        return (systemDirectory != null);
    }

    public ConfigurationPath copy() {
        ConfigurationPath newObj = new ConfigurationPath();
        newObj.setName(getName());
        newObj.setPublicURL(getPublicURL());
        newObj.setSystemFile(getSystemDirectory());
        return newObj;
    }

    public boolean isSame(ConfigurationPath configurationPath) {
        if (configurationPath == null) {
            return false;
        }
        if (!equalsToNul(configurationPath.getPublicURLString(), getPublicURLString())) {
            return false;
        }
        if (!equalsToNul(configurationPath.getSystemDirectoryString(), getSystemDirectoryString())) {
            return false;
        }
        return true;
    }

    private boolean equalsToNul(String newString, String oldString) {
        if (oldString == null) {
            return (newString == null);
        }
        if (newString == null) {
            return false;
        }
        return (oldString.equals(newString));
    }

}
