/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.policies;

import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;


/**
 *
 * @author Vincent Calame
 */
public class PolicyUtils {

    private PolicyUtils() {
    }

    /**
     *
     *@throws IllegalStateException si un des thésaurusKey ne correspond pas à un thésaurus de la base ou si c'est un thésaurus avec idalpha.
     */
    public static Thesaurus[] getVerifThesaurusArray(Basedefiches basedefiches, ThesaurusPolicy thesaurusPolicy) {
        List<SubsetKey> keyList = thesaurusPolicy.getVerifThesaurusKeyList();
        int size = keyList.size();
        Thesaurus[] array = new Thesaurus[size];
        for (int i = 0; i < array.length; i++) {
            SubsetKey thesaurusKey = keyList.get(i);
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
            if (thesaurus == null) {
                throw new IllegalStateException("thesaurusPolicy.getVerifThesaurusKeyList() contains a wrong key");
            }
            if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                throw new IllegalStateException("thesaurus with idalpha cannot own to verif list");
            }
            array[i] = thesaurus;
        }
        return array;
    }

    /**
     *
     *@throws IllegalArgumentException si le thésaurus n'est pas un thésaurus de transfert.
     *@throws IllegalStateException si le thésaurus de transfer n'est pas babélien.
     */
    public static Thesaurus getTransferThesaurus(Basedefiches basedefiches, ThesaurusPolicy thesaurusPolicy) {
        if (thesaurusPolicy.getAjoutPolicy() != ThesaurusPolicy.AJOUT_TRANSFERT) {
            throw new IllegalArgumentException();
        }
        Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusPolicy.getTransferThesaurusKey());
        if (!ThesaurusUtils.isBabelienType(thesaurus)) {
            throw new IllegalStateException("thesaurus is not babelien");
        }
        return thesaurus;
    }

}
