/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.tableexport;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUILibelleMap;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.include.IncludeSummaryBuilder;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.localisation.LocalisationInfo;




/**
 *
 * @author Vincent Calame
 */
public class TableExportUtils {

    private TableExportUtils() {
    }

    public static ColDefList getColDefListFromCorpusUI(BdfServer bdfServer, Corpus corpus, TableExportParametersBean tableExportParametersBean, LocalisationInfo localisationInfo) {
        ColDefBuilder builder = new ColDefBuilder(bdfServer, corpus, tableExportParametersBean, localisationInfo);
        builder.init();
        return ColDefUtils.fromArray(builder.toColDefArray());
    }


    private static class ColDefBuilder {

        BdfServer bdfServer;
        Corpus corpus;
        CorpusMetadata corpusMetadata;
        TableExportParametersBean tableExportParametersBean;
        CorpusUI corpusUI;
        List colDefList = new ArrayList();
        LocalisationInfo localisationInfo;
        IncludeUILibelleMap bdfServerCorpusInclude;
        boolean noIndexation;
        IncludeSummaryBuilder includeSummaryBuilder;

        ColDefBuilder(BdfServer bdfServer, Corpus corpus, TableExportParametersBean tableExportParametersBean, LocalisationInfo localisationInfo) {
            this.bdfServer = bdfServer;
            this.corpus = corpus;
            this.localisationInfo = localisationInfo;
            this.corpusMetadata = corpus.getCorpusMetadata();
            this.tableExportParametersBean = tableExportParametersBean;
            corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
            bdfServerCorpusInclude = bdfServer.getIncludeLibelleManager().getIncludeUILibelleMap(corpus.getSubsetKey());
            short indexationIncludeType = tableExportParametersBean.getIndexationIncludeType();
            noIndexation = (indexationIncludeType == TableExportCommon.NO_INDEXATION);
            if (indexationIncludeType == TableExportCommon.ALL_INDEXATION) {
                includeSummaryBuilder = new IncludeSummaryBuilder();
            }
        }

        ColDef[] toColDefArray() {
            int size = colDefList.size();
            ColDef[] result = new ColDef[size];
            for (int i = 0; i < size; i++) {
                result[i] = (ColDef) colDefList.get(i);
            }
            return result;
        }

        void init() {
            colDefList.add(getFieldKeyColDef(FieldKey.IDCORPUS));
            if (tableExportParametersBean.isWithChrono()) {
                colDefList.add(getChronoIncludeKey(IncludeKey.DATE_CREATION));
                colDefList.add(getChronoIncludeKey(IncludeKey.DATE_MODIFICATION));
            }
            List uiList = corpusUI.getOrderedComponentUIList(CorpusUI.FIELD_AND_INCLUDE_LIST);
            for (Iterator it = uiList.iterator(); it.hasNext();) {
                Object obj = it.next();
                if (obj instanceof FieldUI) {
                    FieldUI fieldUI = (FieldUI) obj;
                    CorpusField corpusField = fieldUI.getCorpusField();
                    if ((!tableExportParametersBean.isWithTextes()) && (corpusField.getFieldKey().ownsToCorpsdefiche())) {
                        continue;
                    }
                    colDefList.add(getFieldKeyColDef(corpusField));
                } else if (obj instanceof IncludeUI) {
                    IncludeUI includeUI = (IncludeUI) obj;
                    FilteredIncludeKey filteredIncludeKey = includeUI.getFiteredIncludeKey();
                    if (testInclude(filteredIncludeKey)) {
                        colDefList.add(getFilteredIncludeKeyColDef(filteredIncludeKey));
                    }
                }
            }
            if (includeSummaryBuilder != null) {
                IncludeSummary includeSummary = includeSummaryBuilder.getIncludeSummary();
                Basedefiches basedefiches = bdfServer.getBasedefiches();
                int thesaurusCount = basedefiches.getThesaurusCount();
                for (int i = 0; i < thesaurusCount; i++) {
                    Thesaurus thesaurus = basedefiches.getThesaurus(i);
                    IncludeKey includeKey = IncludeKey.createIndexationInstance(thesaurus.getSubsetKey());
                    IncludeSummary.IncludeKeyInfo info = includeSummary.getIncludeKeyInfo(includeKey);
                    if ((info == null) || (!info.hasAnyPoidsInclude())) {
                        colDefList.add(getFilteredIncludeKeyColDef(includeKey));
                    }
                }
            }
        }

        boolean testInclude(FilteredIncludeKey filteredIncludeKey) {
            IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
            if (includeKey.isSpecialIncludeKey()) {
                if (includeKey.equals(IncludeKey.LIAGE)) {
                    return tableExportParametersBean.isWithLiage();
                } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                    return tableExportParametersBean.isWithAssociation();
                } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                    return false;
                } else {
                    throw new SwitchException("unkonwn special key : " + includeKey.toString());
                }
            } else if (includeKey.isIndexationIncludeKey()) {
                if (noIndexation) {
                    return false;
                }
                if (includeSummaryBuilder != null) {
                    includeSummaryBuilder.append(filteredIncludeKey);
                }
            } else if (includeKey.isLiageIncludeKey()) {
                if (!tableExportParametersBean.isWithLiage()) {
                    return false;
                }
            }
            return true;
        }

        ColDef getFieldKeyColDef(FieldKey fieldKey) {
            return getFieldKeyColDef(corpusMetadata.getCorpusFieldByFieldKey(fieldKey));
        }

        ColDef getFieldKeyColDef(CorpusField corpusField) {
            FieldKey fieldKey = corpusField.getFieldKey();
            BasedefichesFormatDef ficheFormatDef = new BasedefichesFormatDef(fieldKey);
            if (fieldKey.equals(FieldKey.IDCORPUS)) {
                ficheFormatDef.setCastType(FormatDef.INTEGER_CAST);
            }
            if (fieldKey.ownsToCorpsdefiche()) {
                ficheFormatDef.setSeparator("\n");
            }
            int ficheItemType = corpusField.getFicheItemType();
            switch (ficheItemType) {
                case CorpusField.MONTANT_FIELD:
                case CorpusField.NOMBRE_FIELD:
                    ficheFormatDef.addFormatPattern("{lib}");
                    break;
            }
            String lib = "";
            if (localisationInfo != null) {
                lib = corpusField.getLibelleString(localisationInfo.getWorkingLangInteger(), true);
            }
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(fieldKey.toString(), lib, ficheFormatDef);
            return colDef;
        }

        ColDef getFilteredIncludeKeyColDef(IncludeKey includeKey) {
            return getFilteredIncludeKeyColDef(new FilteredIncludeKey(includeKey));
        }

        ColDef getFilteredIncludeKeyColDef(FilteredIncludeKey filteredIncludeKey) {
            BasedefichesFormatDef ficheFormatDef = new BasedefichesFormatDef(filteredIncludeKey);
            String lib = "";
            if (localisationInfo != null) {
                lib = bdfServerCorpusInclude.getLibelleString(filteredIncludeKey, localisationInfo.getWorkingLangInteger());
            }
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(filteredIncludeKey.toString(), lib, ficheFormatDef);
            return colDef;
        }

        ColDef getChronoIncludeKey(IncludeKey includeKey) {
            ColDef colDef = getFilteredIncludeKeyColDef(includeKey);
            BasedefichesFormatDef ficheFormatDef = colDef.getBasedefichesFormatDef();
            ficheFormatDef.setCastType(FormatDef.DATE_CAST);
            ficheFormatDef.addFormatPattern("{iso}");
            return colDef;
        }

    }

}
