/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class TransformationInfo {

    private TransformationKey transformationKey;
    private XslTransformerInfo[] xslTransformerInfoArray;
    private XslTransformerInfo[] unvalidXslTransformerInfoArray;
    private ListByExtension[] extensionListArray;
    private ListByExtension[] unvalidExtensionListArray;
    private String[] onlyDefaultTransformerAvailableArray;

    private TransformationInfo(TransformationKey transformationKey) {
        this.transformationKey = transformationKey;
    }

    public boolean hasXslTransformer() {
        return (xslTransformerInfoArray.length != 0);
    }

    public boolean hasStreamTransformer() {
        return (extensionListArray.length != 0);
    }

    public int getXslTransformerCount() {
        return xslTransformerInfoArray.length;
    }

    /*public boolean isDefaultTransformerAvailable(String extension) {
    for(int i = 0; i < onlyDefaultTransformerAvailableArray.length; i++) {
    if (onlyDefaultTransformerAvailableArray[i].equals(extension)) {
    return true;
    }
    }
    return false;
    }*/
    public String[] getAllExtensionArray() {
        int count1 = extensionListArray.length;
        int count2 = onlyDefaultTransformerAvailableArray.length;
        String[] result = new String[count1 + count2];
        for (int j = 0; j < count1; j++) {
            result[j] = extensionListArray[j].getExtension();
        }
        for (int j = 0; j < count2; j++) {
            result[j + count1] = onlyDefaultTransformerAvailableArray[j];
        }
        return result;
    }

    public String[] getOnlyDefaultTransformerAvailableArray() {
        return onlyDefaultTransformerAvailableArray;
    }

    public int getExtensionListCount() {
        return extensionListArray.length;
    }

    public int indexOfExtension(String extension) {
        for (int i = 0; i < extensionListArray.length; i++) {
            if (extensionListArray[i].getExtension().equals(extension)) {
                return i;
            }
        }
        return -1;
    }

    public XslTransformerKey getXslTransformerKey(int i) {
        return xslTransformerInfoArray[i].getXslTransformerKey();
    }

    public XslTransformerInfo getXslTransformerInfo(int i) {
        return xslTransformerInfoArray[i];
    }

    public String getExtension(int i) {
        return extensionListArray[i].getExtension();
    }

    public ListByExtension getExtensionList(int i) {
        return extensionListArray[i];
    }

    public ListByExtension[] getUnvalidExtensionListArray() {
        return unvalidExtensionListArray;
    }

    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    public boolean containsXslTransformer(String transformerName) {
        for (int i = 0; i < xslTransformerInfoArray.length; i++) {
            if (xslTransformerInfoArray[i].getXslTransformerKey().getTransformerName().equals(transformerName)) {
                return true;
            }
        }
        return false;
    }

    public XslTransformerInfo[] getUnvalidXslTransformerInfoArray() {
        return unvalidXslTransformerInfoArray;
    }


    public static class ListByExtension {

        private TransformationKey transformationKey;
        private String extension;
        private StreamTransformerInfo[] streamTransformerInfoArray;
        private StreamTransformerInfo[] unvalidStreamTransformerInfoArray;
        private boolean defaultTransformerAvailable = false;

        ListByExtension(TransformationKey transformationKey, String extension) {
            this.extension = extension;
            this.transformationKey = transformationKey;
        }

        public String getExtension() {
            return extension;
        }

        public int getStreamTransformerCount() {
            return streamTransformerInfoArray.length;
        }

        public boolean isDefaultTransformerAvailable() {
            return defaultTransformerAvailable;
        }

        public StreamTransformerKey getStreamTransformerKey(int i) {
            return streamTransformerInfoArray[i].getStreamTransformerKey();
        }

        public StreamTransformerInfo getStreamTransformerInfo(int i) {
            return streamTransformerInfoArray[i];
        }

        public StreamTransformerInfo[] getUnvalidStreamTransformerInfoArray() {
            return unvalidStreamTransformerInfoArray;
        }

        public boolean hasValidTransformer() {
            return (streamTransformerInfoArray.length > 0);
        }

        public TransformationKey getTransformationKey() {
            return transformationKey;
        }

    }

    public static Builder getBuilder(TransformationKey transformationKey) {
        return new Builder(transformationKey);
    }

    public static TransformationInfo getEmptyInstance(TransformationKey transformationKey, String[] defaultTransformerAvailableArray) {
        TransformationInfo transformationInfo = new TransformationInfo(transformationKey);
        transformationInfo.xslTransformerInfoArray = new XslTransformerInfo[0];
        transformationInfo.unvalidXslTransformerInfoArray = new XslTransformerInfo[0];
        transformationInfo.extensionListArray = new ListByExtension[0];
        transformationInfo.unvalidExtensionListArray = new ListByExtension[0];
        if (defaultTransformerAvailableArray == null) {
            defaultTransformerAvailableArray = new String[0];
        }
        transformationInfo.onlyDefaultTransformerAvailableArray = defaultTransformerAvailableArray;
        return transformationInfo;
    }


    public static class Builder {

        private TransformationKey transformationKey;
        private List validXslTransformerList = new ArrayList();
        private List unvalidXslTransformerList = new ArrayList();
        private TreeMap extensionMap = new TreeMap();
        private Set defaultTransformerAvailableSet = new LinkedHashSet();

        Builder(TransformationKey transformationKey) {
            this.transformationKey = transformationKey;
        }

        public TransformationKey getTransformationKey() {
            return transformationKey;
        }

        public void addDefaultTransformerAvailable(String extension) {
            defaultTransformerAvailableSet.add(extension);
        }

        public TransformationInfo toTransformationInfo() {
            TransformationInfo transformationInfo = new TransformationInfo(transformationKey);
            transformationInfo.xslTransformerInfoArray = getArray(validXslTransformerList);
            transformationInfo.unvalidXslTransformerInfoArray = getArray(unvalidXslTransformerList);
            List validExtension = new ArrayList();
            List unvalidExtension = new ArrayList();
            for (Iterator it = extensionMap.values().iterator(); it.hasNext();) {
                ListByExtensionBuilder byExtensionBuilder = (ListByExtensionBuilder) it.next();
                String extension = byExtensionBuilder.extension;
                boolean def = defaultTransformerAvailableSet.contains(extension);
                if (def) {
                    byExtensionBuilder.setdefaultTransformerAvailable(def);
                    defaultTransformerAvailableSet.remove(extension);
                }
                ListByExtension byExtension = byExtensionBuilder.toListByExtension();
                if (byExtension.hasValidTransformer()) {
                    validExtension.add(byExtension);
                } else {
                    unvalidExtension.add(byExtension);
                }
            }
            transformationInfo.onlyDefaultTransformerAvailableArray = getArray(defaultTransformerAvailableSet);
            transformationInfo.extensionListArray = getListByExtensionArray(validExtension);
            transformationInfo.unvalidExtensionListArray = getListByExtensionArray(unvalidExtension);
            return transformationInfo;
        }

        private String[] getArray(Set set) {
            int size = set.size();
            String[] result = new String[size];
            int p = 0;
            for (Iterator it = set.iterator(); it.hasNext();) {
                String extension = (String) it.next();
                result[p] = extension;
                p++;
            }
            return result;
        }

        public void addXslTransformerInfo(XslTransformerInfo xslTransformerInfo) {
            if (xslTransformerInfo.isValid()) {
                validXslTransformerList.add(xslTransformerInfo);
            } else {
                unvalidXslTransformerList.add(xslTransformerInfo);
            }
        }

        public void addStreamTransformerInfo(StreamTransformerInfo streamTransformerInfo) {
            String extension = streamTransformerInfo.getStreamTransformerKey().getExtension();
            ListByExtensionBuilder byExtensionBuilder = (ListByExtensionBuilder) extensionMap.get(extension);
            if (byExtensionBuilder == null) {
                byExtensionBuilder = new ListByExtensionBuilder(extension);
                extensionMap.put(extension, byExtensionBuilder);
            }
            byExtensionBuilder.add(streamTransformerInfo);
        }

        private XslTransformerInfo[] getArray(List list) {
            int size = list.size();
            XslTransformerInfo[] xslTransformerInfoArray = new XslTransformerInfo[size];
            for (int i = 0; i < size; i++) {
                xslTransformerInfoArray[i] = (XslTransformerInfo) list.get(i);
            }
            return xslTransformerInfoArray;
        }

        private ListByExtension[] getListByExtensionArray(List list) {
            int size = list.size();
            ListByExtension[] byExtensionArray = new ListByExtension[size];
            for (int i = 0; i < size; i++) {
                byExtensionArray[i] = (ListByExtension) list.get(i);
            }
            return byExtensionArray;
        }


        private class ListByExtensionBuilder {

            List validStreamTransformerList = new ArrayList();
            List unvalidStreamTransformerList = new ArrayList();
            String extension;
            boolean defaultTransformerAvailable = false;

            ListByExtensionBuilder(String extension) {
                this.extension = extension;
            }

            void setdefaultTransformerAvailable(boolean defaultTransformerAvailable) {
                this.defaultTransformerAvailable = defaultTransformerAvailable;
            }

            void add(StreamTransformerInfo streamTransformerInfo) {
                if (streamTransformerInfo.isValid()) {
                    validStreamTransformerList.add(streamTransformerInfo);
                } else {
                    unvalidStreamTransformerList.add(streamTransformerInfo);
                }
            }

            private ListByExtension toListByExtension() {
                ListByExtension byExtension = new ListByExtension(transformationKey, extension);
                byExtension.streamTransformerInfoArray = getStreamArray(validStreamTransformerList);
                byExtension.unvalidStreamTransformerInfoArray = getStreamArray(unvalidStreamTransformerList);
                byExtension.defaultTransformerAvailable = defaultTransformerAvailable;
                return byExtension;
            }

            private StreamTransformerInfo[] getStreamArray(List list) {
                int size = list.size();
                StreamTransformerInfo[] streamTransformerInfoArray = new StreamTransformerInfo[size];
                for (int i = 0; i < size; i++) {
                    streamTransformerInfoArray[i] = (StreamTransformerInfo) list.get(i);
                }
                return streamTransformerInfoArray;
            }

        }

    }

}
