/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import java.io.Serializable;
import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 * Clé indiquant le type de transformation à appliquer. Cette clé est utilisée
 * pour obtenir les modèles de transformation.
 *
 * @author Vincent Calame
 */
public final class TransformationKey implements Serializable {

    public final static TransformationKey COMPILATION_TRANSFORMATIONKEY = new TransformationKey("compilation");
    public final static TransformationKey STATTHESAURUS_TRANSFORMATIONKEY = new TransformationKey("statthesaurus");
    public final static TransformationKey MOTCLE_TRANSFORMATIONKEY = new TransformationKey("motcle");
    public final static TransformationKey INVERSETHESAURUS_TRANSFORMATIONKEY = new TransformationKey("inversethesaurus");
    private String key;
    private boolean isCorpusTransformationKey;

    private TransformationKey(String key) {
        this.key = key;
        this.isCorpusTransformationKey = false;
    }

    private TransformationKey(String key, boolean isCorpusTransformationKey) {
        this.key = key;
        this.isCorpusTransformationKey = isCorpusTransformationKey;
    }

    /**
     *
     *@throws IllegalArgumentException
     */
    public TransformationKey(SubsetKey corpusKey) {
        if (!corpusKey.isCorpusSubset()) {
            throw new IllegalArgumentException("corpusKey.isCorpusSubset() == false");
        }
        key = corpusKey.toString();
        this.isCorpusTransformationKey = true;
    }

    public boolean isCorpusTransformationKey() {
        return isCorpusTransformationKey;
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof TransformationKey)) {
            return false;
        }
        return (this.key.equals(((TransformationKey) obj).key));
    }

    @Override
    public String toString() {
        return key;
    }

    /**
     *@throws IllegalStateException 
     */
    public SubsetKey toCorpusKey() {
        if (!isCorpusTransformationKey) {
            throw new IllegalStateException("transformationKey is not a corpus transformationKey");
        }
        try {
            return SubsetKey.parse(key);
        } catch (java.text.ParseException pe) {
            throw new ShouldNotOccurException(pe);
        }
    }

    public static TransformationKey parse(String s) throws ParseException {
        if (s.equals(COMPILATION_TRANSFORMATIONKEY.key)) {
            return COMPILATION_TRANSFORMATIONKEY;
        }
        if (s.equals(STATTHESAURUS_TRANSFORMATIONKEY.key)) {
            return STATTHESAURUS_TRANSFORMATIONKEY;
        }
        if (s.equals(MOTCLE_TRANSFORMATIONKEY.key)) {
            return MOTCLE_TRANSFORMATIONKEY;
        }
        if (s.equals(INVERSETHESAURUS_TRANSFORMATIONKEY.key)) {
            return INVERSETHESAURUS_TRANSFORMATIONKEY;
        }
        if (!s.startsWith("corpus_")) {
            throw new ParseException("!s.startsWith(\"corpus_\")", 0);
        }
        return new TransformationKey(s, true);
    }

}
