/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.sqlexport.SqlExportManager;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.sqlexport.beans.SqlExportBean;
import net.mapeadores.util.beans.ParamBean;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMReader;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportManagerDOMReader extends ManagerDOMReader {

    private SqlExportManager currentSqlExportManager;

    public SqlExportManagerDOMReader() {
    }

    public void readManager(Object manager, Element element_xml) {
        readSqlExportManager((SqlExportManager) manager, element_xml);
    }

    public void readSqlExportManager(SqlExportManager sqlExportManager, Element element_xml) {
        this.currentSqlExportManager = sqlExportManager;
        DOMReader.readChildren(element_xml, new RootHandler());
        this.currentSqlExportManager = null;

    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("sqlexport")) {
                String name = element.getAttribute("name");
                if (!StringUtils.isTechnicalName(name, true)) {
                    return;
                }
                SqlExportBean sqlExport = new SqlExportBean();
                sqlExport.setName(name);
                SqlExportHandler sqlExportHandler = new SqlExportHandler(sqlExport);
                DOMReader.readChildren(element, sqlExportHandler);
                sqlExportHandler.flush();
                currentSqlExportManager.putSqlExportBean(sqlExport);
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class SqlExportHandler implements ElementHandler {

        SqlExportBean sqlExport;
        List paramList = new ArrayList();

        SqlExportHandler(SqlExportBean sqlExport) {
            this.sqlExport = sqlExport;
        }

        public void flush() {
            int size = paramList.size();
            if (size > 0) {
                ParamBean[] array = new ParamBean[size];
                for (int i = 0; i < size; i++) {
                    array[i] = (ParamBean) paramList.get(i);
                }
                sqlExport.setParamBeanArray(array);
            }
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();

            if (tagName.equals("param")) {
                String name = element.getAttribute("name").trim();
                if (name.length() == 0) {
                    return;
                }
                String value = element.getAttribute("value").trim();
                ParamBean paramBean = new ParamBean(name, value);
                paramList.add(paramBean);
            } else {
                String contenu = DOMReader.readSimpleElement(element);
                if (contenu.length() == 0) {
                    return;
                }
                if (tagName.equals("path")) {
                    sqlExport.setPath(contenu);
                } else if (tagName.equals("url")) {
                    sqlExport.setUrl(contenu);
                } else if (tagName.equals("tableexport-name")) {
                    sqlExport.setTableExportName(contenu);
                } else if (tagName.equals("sqlexport-classname")) {
                    sqlExport.setSqlExportClassName(contenu);
                } else if (tagName.equals("post-command")) {
                    sqlExport.setPostCommand(contenu);
                } else {
                    unknownTagWarning(tagName);
                }
            }
        }

    }

}
