/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request;

import net.mapeadores.util.logging.CommandMessage;


/**
 * Cette exception concerne les erreurs qui ne devraient pas arriver (des erreurs systèmes) et non les erreurs dues à l'utilisateur.
 *
 * @author  Vincent Calame
 */
public class ParameterException extends Exception {

    private short type;
    private String texte;
    public static final short ADMIN_ONLY = 0;
    public static final short CORPUS_UNDEFINED = 1;
    public static final short CORPUS_NOTEXIST = 2;
    public static final short IDCORPUS_NOTEXIST = 3;
    public static final short IDCORPUS_NOTINTEGER = 4;
    public static final short CORPUS_NODOCFICHE = 5;
    public static final short SUBSTANTIF_UNDEFINED = 6;
    public static final short SUBSTANTIF_NOTEXIST = 7;
    public static final short CMD_UNDEFINED = 8;
    public static final short CMD_NOTEXIST = 9;
    public static final short IDCORPUS_UNDEFINED = 10;
    public static final short THESAURUS_UNDEFINED = 11;
    public static final short THESAURUS_NOTEXIST = 12;
    public static final short IDTHS_UNDEFINED = 13;
    public static final short IDTHS_NOTEXIST = 14;
    public static final short IDTHS_NOTINTEGER = 15;
    public static final short NOMEXPORT_NOTEXIST = 16;
    public static final short SPHERE_NOTEXIST = 17;
    public static final short SPHERE_UNDEFINED = 18;
    public static final short IDSPHERE_UNDEFINED = 20;
    public static final short IDSPHERE_NOTEXIST = 21;
    public static final short CMD_UNVALAIBLE_GET = 22;
    public static final short CMD_UNVALAIBLE_POST = 23;
    public static final short PARAMETER_UNDEFINED = 24;
    public static final short NOMTHSFUSION_UNDEFINED = 25;
    public static final short NOMTHSFUSION_NOTEXIST = 26;
    public static final short NOMTHSFUSION_NOTEDITABLE = 27;
    public static final short NOMTHSFUSION_IDALPHANOTEDITABLE = 28;
    public static final short IDALPHA_NOTEXIST = 29;
    public static final short IDTHSFUSION_UNDEFINED = 30;
    public static final short IDTHSFUSION_NOTEXIST = 31;
    public static final short IDTHSFUSION_NOTINTEGER = 32;
    public static final short IDALPHA_NOTAVAILABLE = 33;
    public static final short ALLID_MTCL_UNDEFINED = 34;
    public static final short IDMTCL_NOTEXIST = 35;
    public static final short IDMTCL_NOTINTEGER = 36;
    public static final short PARAMETER_NOTINTEGER = 38;
    public static final short CMD_UNVALAIBLE_IDALPHA = 39;
    public static final short PARAMETER_EQUALSTHESAURUS = 40;
    public static final short VERSION_NOTEXIST = 41;
    public static final short SUBCOMMAND_NOTEXIST = 42;
    public static final short CMD_CLASS_NOTFOUND = 43;
    public static final short MOTCLE_NOTREMOVEABLE = 44;
    /**
     * La valeur doit être sous la forme paramètre = valeur
     */
    public static final short PARAMETER_WRONGVALUE = 45;
    public static final short PARAMETER_NOTLANG = 46;
    public static final short PAGE_WRONGVALUE = 47;
    public static final short PAGE_UNVALAIBLE_IDALPHA = 48;
    public static final short IDSUBSET_UNDEFINED = 51;
    public static final short IDSUBSET_NOTEXIST = 52;
    public static final short IDSUBSET_NOTINTEGER = 53;
    public static final short TABLEEXPORT_UNDEFINED = 61;
    public static final short TABLEEXPORT_NOTEXIST = 62;
    public static final short SCRUTARIEXPORT_UNDEFINED = 63;
    public static final short SCRUTARIEXPORT_NOTEXIST = 64;
    public static final short SQLEXPORT_UNDEFINED = 65;
    public static final short SQLEXPORT_NOTEXIST = 66;
    public static final short ROLE_UNDEFINED = 67;
    public static final short ROLE_NOTEXIST = 68;
    public static final short NO_CREATEFICHE = 71;
    public static final short NO_READFICHE = 72;
    public static final short NO_WRITEFICHE = 73;
    public static final short NO_SENDFICHE = 74;
    public static final short REDACTEUR_NOTREMOVEABLE = 81;
    public static final short REDACTEUR_ISADMIN = 82;
    public static final short REDACTEUR_NOTACTIF = 83;
    public static final short BALAYAGE_UNDEFINED = 84;
    public static final short BALAYAGE_NOTEXIST = 85;
    public static final short GET_METHOD_FORBIDDEN = 100;
    public static final short RESPONSEFACTORY_CLASS_NOTFOUND = 101;
    private boolean potentialHumanError = false;
    private CommandMessage cmdMessage;
    private String value;

    public ParameterException(short type, String paramValue) {
        this.type = type;
        this.value = paramValue;
        switch (type) {
            case ADMIN_ONLY:
                texte = "this request is reserved to administrators";
                break;
            case CORPUS_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.CORPUS_PARAMNAME + "\" is not defined";
                break;
            case CORPUS_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.CORPUS_PARAMNAME + "\" does not match a valid corpus";
                break;
            case IDCORPUS_NOTEXIST:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDCORPUS_PARAMNAME + "\" does not match a valid fiche";
                break;
            case IDCORPUS_NOTINTEGER:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDCORPUS_PARAMNAME + "\" must be an positive integer";
                break;
            case CORPUS_NODOCFICHE:
                texte = RequestUtils.CORPUS_PARAMNAME + " (" + paramValue + ") does not allow attached document";
                break;
            case IDCORPUS_UNDEFINED:
                texte = "\"idcorpus\" is not defined";
                break;
            case SUBSTANTIF_UNDEFINED:
                texte = "\"substantif\" is not defined";
                break;
            case SUBSTANTIF_NOTEXIST:
                texte = "substantif=" + paramValue + " does not exist in this corpus";
                break;
            case CMD_UNDEFINED:
                texte = "\"cmd\" is not defined";
                break;
            case CMD_NOTEXIST:
                texte = "cmd=" + paramValue + " is not a valid command";
                break;
            case THESAURUS_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.THESAURUS_PARAMNAME + "\" is not defined";
                break;
            case THESAURUS_NOTEXIST:
                texte = "the value (" + paramValue + ") of the parameter \"" + RequestUtils.THESAURUS_PARAMNAME + "\" does not match a valid thesaurus";
                break;
            case IDTHS_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.IDTHS_PARAMNAME + "\" is not defined";
                break;
            case IDTHS_NOTEXIST:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDTHS_PARAMNAME + "\" does not match a valid motcle";
                break;
            case IDTHS_NOTINTEGER:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDTHS_PARAMNAME + "\" must be an integer";
                break;
            case NOMEXPORT_NOTEXIST:
                texte = "nomexport=" + paramValue + " is not a valid ExportSBF";
                break;
            case SPHERE_NOTEXIST:
                texte = "the value (" + paramValue + ") of the parameter \"" + RequestUtils.SPHERE_PARAMNAME + "\" does not match a valid sphere";
                break;
            case SPHERE_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.SPHERE_PARAMNAME + "\" is not defined";
                break;
            case IDSPHERE_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.IDSPHERE_PARAMNAME + "\" is not defined";
                break;
            case IDSPHERE_NOTEXIST:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDSPHERE_PARAMNAME + "\" does not match a valid redacteur";
                break;
            case CMD_UNVALAIBLE_GET:
                texte = "cmd=" + paramValue + " is unvalaible in the GET method";
                break;
            case CMD_UNVALAIBLE_POST:
                texte = "cmd=" + paramValue + " is unvalaible in the POST method";
                break;
            case PARAMETER_UNDEFINED:
                texte = "this parameter=" + paramValue + " is not defined";
                break;
            case NOMTHSFUSION_UNDEFINED:
                texte = "\"nomthsfusion\" is not defined";
                break;
            case NOMTHSFUSION_NOTEXIST:
                texte = "nomthsfusion=" + paramValue + " is not a valid thesaurus";
                break;
            case NOMTHSFUSION_NOTEDITABLE:
                texte = "nomthsfusion=" + paramValue + " is not a editable thesaurus";
                break;
            case NOMTHSFUSION_IDALPHANOTEDITABLE:
                texte = "nomthsfusion=" + paramValue + " is not a editable thesaurus because with idalpha";
                break;
            case IDALPHA_NOTEXIST:
                texte = "idalpha=" + paramValue + " does not exist in this thesaurus";
                break;
            case IDTHSFUSION_UNDEFINED:
                texte = "\"idthsfusion\" is not defined";
                break;
            case IDTHSFUSION_NOTEXIST:
                texte = "idthsfusion=" + paramValue + " does not exist in this thesaurus";
                break;
            case IDTHSFUSION_NOTINTEGER:
                texte = "idthsfusion=" + paramValue + " must be an integer";
                break;
            case IDALPHA_NOTAVAILABLE:
                texte = "idalpha is not avalaible is the thesaurus=" + paramValue;
                break;
            case ALLID_MTCL_UNDEFINED:
                texte = "\"idths\", \"idalpha\" and \"idmtcl\" are not defined";
                break;
            case IDMTCL_NOTEXIST:
                texte = "idmtcl=" + paramValue + " does not exist in this thesaurus";
                break;
            case IDMTCL_NOTINTEGER:
                texte = "idmtcl=" + paramValue + " must be an integer";
                break;
            case PARAMETER_NOTINTEGER:
                texte = "the value of the parameter \"" + paramValue + "\" must be an integer";
                break;
            case CMD_UNVALAIBLE_IDALPHA:
                texte = "this command cmd=" + paramValue + " is unvalaible for thesaurus with idalpha";
                break;
            case PAGE_UNVALAIBLE_IDALPHA:
                texte = "this page=" + paramValue + " is unvalaible for thesaurus with idalpha";
                break;
            case PARAMETER_EQUALSTHESAURUS:
                texte = "the value of the parameter \"" + paramValue + "\" cannot equal the value of the parameter \"" + RequestUtils.THESAURUS_PARAMNAME + "\"";
                break;
            case VERSION_NOTEXIST:
                texte = "the document has no extension with the value " + paramValue;
                break;
            case SUBCOMMAND_NOTEXIST:
                texte = "the subcommand " + paramValue + " does not exist";
                break;
            case CMD_CLASS_NOTFOUND:
                texte = "class command =" + paramValue + " is not found";
                break;
            case RESPONSEFACTORY_CLASS_NOTFOUND:
                texte = "responseFactory class =" + paramValue + " is not found";
                break;
            case MOTCLE_NOTREMOVEABLE:
                texte = "motcle is not removeable";
                break;
            case REDACTEUR_NOTREMOVEABLE:
                texte = "redacteur is not removeable";
                break;
            case REDACTEUR_ISADMIN:
                texte = "redacteur is an administrator";
                break;
            case REDACTEUR_NOTACTIF:
                texte = "redacteur is not actif";
                break;
            case PARAMETER_WRONGVALUE:
                texte = "this couple (" + paramValue + ") of parameter and value is wrong";
                break;
            case PAGE_WRONGVALUE:
                texte = "the value (" + paramValue + ") of the parameter \"page\" is wrong";
                break;
            case PARAMETER_NOTLANG:
                texte = "the value of the parameter \"" + paramValue + "\" must be a valid lang";
                break;
            case IDSUBSET_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.IDSUBSET_PARAMNAME + "\" is not defined";
                break;
            case IDSUBSET_NOTEXIST:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDSUBSET_PARAMNAME + "\" does not match a valid document";
                break;
            case IDSUBSET_NOTINTEGER:
                texte = "the value (" + paramValue + ")  of the parameter \"" + RequestUtils.IDSUBSET_PARAMNAME + "\" must be an integer";
                break;
            case TABLEEXPORT_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.TABLEEXPORT_PARAMNAME + "\" is not defined";
                break;
            case TABLEEXPORT_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.TABLEEXPORT_PARAMNAME + "\" does not match a valid tableExport";
                break;
            case SCRUTARIEXPORT_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.SCRUTARIEXPORT_PARAMNAME + "\" is not defined";
                break;
            case SCRUTARIEXPORT_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.SCRUTARIEXPORT_PARAMNAME + "\" does not match a valid scrutariExport";
                break;
            case SQLEXPORT_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.SQLEXPORT_PARAMNAME + "\" is not defined";
                break;
            case SQLEXPORT_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.SQLEXPORT_PARAMNAME + "\" does not match a valid sqlExport";
                break;
            case ROLE_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.ROLE_PARAMNAME + "\" is not defined";
                break;
            case ROLE_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.ROLE_PARAMNAME + "\" does not match a valid role";
                break;
            case NO_CREATEFICHE:
                texte = "Not allowed to create : " + paramValue;
                break;
            case NO_READFICHE:
                texte = "Not allowed to read : " + paramValue;
                break;
            case NO_WRITEFICHE:
                texte = "Not allowed to modify : " + paramValue;
                break;
            case NO_SENDFICHE:
                texte = "Not allowed to send : " + paramValue;
                break;
            case GET_METHOD_FORBIDDEN:
                texte = "Method GET cannot be used for Command Request";
                break;
            case BALAYAGE_UNDEFINED:
                texte = "the parameter \"" + RequestUtils.BALAYAGE_PARAMNAME + "\" is not defined";
                break;
            case BALAYAGE_NOTEXIST:
                texte = "the value (" + paramValue + ") of the paramater \"" + RequestUtils.BALAYAGE_PARAMNAME + "\" does not match a valid balayage";
                break;
        }
    }

    public short getType() {
        return type;
    }

    public String getValue() {
        return value;
    }

    public String getText() {
        return texte;
    }

    public ParameterException(String texte) {
        super(texte);
        this.texte = texte;
    }

    public boolean isPotentialHumanError() {
        return potentialHumanError;
    }

    public void setPotentialHumanError(boolean potentialHumanError) {
        this.potentialHumanError = potentialHumanError;
    }

    public CommandMessage getCmdMessage() {
        return cmdMessage;
    }

    public void setCmdMessage(CommandMessage cmdMessage) {
        this.cmdMessage = cmdMessage;
    }

}
