/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request;

import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.sqlexport.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.scrutariexport.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.sqlexport.beans.*;
import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.roles.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.util.html.*;
import java.util.*;
import net.mapeadores.basedefiches.sphere.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.balayages.*;
import net.mapeadores.util.localisation.*;
import java.text.NumberFormat;


/**
 *
 * @author  Vincent Calame
 */
public class RequestUtils {

    public final static String ERR_PARAMNAME = "err";
    public final static String SPHERE_PARAMNAME = "sphere";
    public final static String IDSPHERE_PARAMNAME = "idsphere";
    public final static String CORPUS_PARAMNAME = "corpus";
    public final static String IDCORPUS_PARAMNAME = "idcorpus";
    public final static String THESAURUS_PARAMNAME = "thesaurus";
    public final static String IDTHS_PARAMNAME = "idths";
    public final static String IDALPHA_PARAMNAME = "idalpha";
    public final static String IDMTCL_PARAMNAME = "idmtcl";
    public final static String IDSUBSET_PARAMNAME = "idsubset";
    public final static String CONFIRMPASSWORD_PARAMNAME = "confirmpassword";
    public final static String TABLEEXPORT_PARAMNAME = "tableexport";
    public final static String SCRUTARIEXPORT_PARAMNAME = "scrutariexport";
    public final static String SQLEXPORT_PARAMNAME = "sqlexport";
    public final static String ROLE_PARAMNAME = "role";
    public final static String BALAYAGE_PARAMNAME = "balayage";

    private RequestUtils() {
    }

    public static Map<String, String> getHiddenParameterMap(Object obj) {
        Map hiddenParameterMap = new HashMap();
        if (obj instanceof Corpus) {
            hiddenParameterMap.put(CORPUS_PARAMNAME, ((Corpus) obj).getSubsetKey().getSubsetName());
        } else if (obj instanceof Role) {
            hiddenParameterMap.put(ROLE_PARAMNAME, ((Role) obj).getName());
        } else if (obj instanceof Redacteur) {
            Redacteur redacteur = (Redacteur) obj;
            RedacteurKey redacteurKey = redacteur.getRedacteurKey();
            hiddenParameterMap.put(SPHERE_PARAMNAME, redacteurKey.getSphereKey().getSubsetName());
            hiddenParameterMap.put(IDSPHERE_PARAMNAME, redacteurKey.getIdsphereString());
        } else if (obj instanceof Sphere) {
            hiddenParameterMap.put(SPHERE_PARAMNAME, ((Sphere) obj).getSubsetKey().getSubsetName());
        }
        return hiddenParameterMap;
    }

    public static boolean isPotentialHumanError(RequestMap requestMap) {
        String err = requestMap.getParameter(ERR_PARAMNAME);
        if ((err != null) && (err.equals("h"))) {
            return true;
        } else {
            return false;
        }
    }

    public static AbregeFiche getAbregeFiche(RequestMap requete, Basedefiches basedefiches) throws ParameterException {
        Corpus corpus = getCorpus(requete, basedefiches);
        return getAbregeFiche(requete, corpus);
    }

    public static AbregeFiche getAbregeFiche(RequestMap requestMap, Corpus corpus) throws ParameterException {
        return getAbregeFiche(requestMap.getParameter(IDCORPUS_PARAMNAME), corpus);
    }

    private static AbregeFiche getAbregeFiche(String param_idcorpus, Corpus corpus) throws ParameterException {
        if (param_idcorpus == null) {
            throw new ParameterException(ParameterException.IDCORPUS_UNDEFINED, null);
        }
        int idcorpus = 0;
        try {
            idcorpus = Integer.parseInt(param_idcorpus);
        } catch (NumberFormatException nfe) {
            throw new ParameterException(ParameterException.IDCORPUS_NOTINTEGER, param_idcorpus);
        }
        if (idcorpus < 1) {
            throw new ParameterException(ParameterException.IDCORPUS_NOTINTEGER, param_idcorpus);
        }
        AbregeFiche abrege = corpus.getAbregeFicheByIdcorpus(idcorpus);
        if (abrege == null) {
            throw new ParameterException(ParameterException.IDCORPUS_NOTEXIST, param_idcorpus);
        }
        return abrege;
    }

    public static Document getDocument(RequestMap requete, Basedefiches basedefiches) throws ParameterException {
        String param_idsubset = requete.getParameter(IDSUBSET_PARAMNAME);
        return getDocument(param_idsubset, basedefiches);
    }

    private static Document getDocument(String param_idsubset, Basedefiches basedefiches) throws ParameterException {
        if (param_idsubset == null) {
            throw new ParameterException(ParameterException.IDSUBSET_UNDEFINED, null);
        }
        int idsubset = 0;
        try {
            idsubset = Integer.parseInt(param_idsubset);
        } catch (NumberFormatException nfe) {
            throw new ParameterException(ParameterException.IDSUBSET_NOTINTEGER, param_idsubset);
        }
        DocumentSubset documentSubset = (DocumentSubset) basedefiches.getSubset(SubsetKey.DOCUMENT_SUBSET);
        Document document = (Document) documentSubset.getSubsetItemById(idsubset);
        if (document == null) {
            throw new ParameterException(ParameterException.IDSUBSET_NOTEXIST, param_idsubset);
        }
        return document;
    }

    public static Corpus getCorpus(RequestMap requete, Basedefiches basedefiches) throws ParameterException {
        String param_corpus = requete.getParameter(CORPUS_PARAMNAME);
        return getCorpus(param_corpus, basedefiches);
    }

    private static Corpus getCorpus(String param_corpus, Basedefiches basedefiches) throws ParameterException {
        if (param_corpus == null) {
            throw new ParameterException(ParameterException.CORPUS_UNDEFINED, null);
        }
        Corpus corpus = BasedefichesUtils.getCorpus(basedefiches, param_corpus);
        if (corpus == null) {
            throw new ParameterException(ParameterException.CORPUS_NOTEXIST, param_corpus);
        }
        return corpus;
    }

    public static Thesaurus getThesaurus(RequestMap requete, Basedefiches basedefiches) throws ParameterException {
        return getThesaurus(requete.getParameter(THESAURUS_PARAMNAME), basedefiches);
    }

    private static Thesaurus getThesaurus(String thesaurusParameter, Basedefiches basedefiches) throws ParameterException {
        if (thesaurusParameter == null) {
            throw new ParameterException(ParameterException.THESAURUS_UNDEFINED, null);
        }
        Thesaurus thesaurus = BasedefichesUtils.getThesaurus(basedefiches, thesaurusParameter);
        if (thesaurus == null) {
            throw new ParameterException(ParameterException.THESAURUS_NOTEXIST, thesaurusParameter);
        }
        return thesaurus;
    }

    public static Sphere getSphere(RequestMap requete, Basedefiches basedefiches) throws ParameterException {
        return getSphere(requete.getParameter(SPHERE_PARAMNAME), basedefiches);
    }

    private static Sphere getSphere(String param_sphere, Basedefiches basedefiches) throws ParameterException {
        if (param_sphere == null) {
            throw new ParameterException(ParameterException.SPHERE_UNDEFINED, null);
        }
        Sphere sphere = BasedefichesUtils.getSphere(basedefiches, param_sphere);
        if (sphere == null) {
            throw new ParameterException(ParameterException.SPHERE_NOTEXIST, param_sphere);
        }
        return sphere;
    }

    public static Redacteur getRedacteur(RequestMap requete, Sphere sphere) throws ParameterException {
        return getRedacteur(requete.getParameter(IDSPHERE_PARAMNAME), sphere);
    }

    private static Redacteur getRedacteur(String param_idsphere, Sphere sphere) throws ParameterException {
        if (param_idsphere == null) {
            throw new ParameterException(ParameterException.IDSPHERE_UNDEFINED, null);
        }
        Redacteur redacteur = SphereUtils.getRedacteur(sphere, param_idsphere);
        if (redacteur == null) {
            throw new ParameterException(ParameterException.IDSPHERE_NOTEXIST, param_idsphere);
        }
        return redacteur;
    }

    /**
     * Regarde si idths, idalpha ou idmtcl existent
     */
    public static boolean hasMotcleParameter(RequestMap requete) {
        if (requete.getParameter(IDTHS_PARAMNAME) != null) {
            return true;
        }
        if (requete.getParameter(IDALPHA_PARAMNAME) != null) {
            return true;
        }
        if (requete.getParameter(IDMTCL_PARAMNAME) != null) {
            return true;
        }
        return false;
    }

    public static Motcle getMotcle(RequestMap requete, Thesaurus thesaurus) throws ParameterException {
        String param_idths = requete.getParameter(IDTHS_PARAMNAME);
        if (param_idths != null) {
            try {
                int idths = Integer.parseInt(param_idths);
                Motcle motcle = thesaurus.getMotcleByIdths(idths);
                if (motcle == null) {
                    throw new ParameterException(ParameterException.IDTHS_NOTEXIST, param_idths);
                }
                return motcle;
            } catch (NumberFormatException nfe) {
                throw new ParameterException(ParameterException.IDTHS_NOTINTEGER, param_idths);
            }

        }
        String param_idalpha = requete.getParameter(IDALPHA_PARAMNAME);
        if (param_idalpha != null) {
            if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                param_idalpha = XMLUtils.cleanString(param_idalpha);
                Motcle mtcl = thesaurus.getMotcleByIdalpha(param_idalpha);
                if (mtcl == null) {
                    throw new ParameterException(ParameterException.IDALPHA_NOTEXIST, param_idalpha);
                }
                return mtcl;
            } else {
                throw new ParameterException(ParameterException.IDALPHA_NOTAVAILABLE, thesaurus.getSubsetKey().getSubsetName());
            }
        }
        String param_idmtcl = requete.getParameter(IDMTCL_PARAMNAME);
        if (param_idmtcl != null) {
            param_idmtcl = XMLUtils.cleanString(param_idmtcl);
            if (ThesaurusUtils.isIdalphaType(thesaurus)) {
                Motcle mtcl = thesaurus.getMotcleByIdalpha(param_idmtcl);
                if (mtcl == null) {
                    throw new ParameterException(ParameterException.IDMTCL_NOTEXIST, param_idmtcl);
                }
                return mtcl;
            } else {
                try {
                    int idmtcl = Integer.parseInt(param_idmtcl);
                    Motcle mtcl = thesaurus.getMotcleByIdths(idmtcl);
                    if (mtcl == null) {
                        throw new ParameterException(ParameterException.IDMTCL_NOTEXIST, param_idmtcl);
                    }
                    return mtcl;
                } catch (NumberFormatException nfe) {
                    throw new ParameterException(ParameterException.IDMTCL_NOTINTEGER, param_idmtcl);
                }
            }
        }
        throw new ParameterException(ParameterException.ALLID_MTCL_UNDEFINED, null);
    }

    /**
     * Retourne <em>null</em> si la valeur est vide, lance pas d'exception mais retourne la valeur nulle.
     * Retourne les exceptions PARAMETER_UNDEFINED, PARAMETER_NOTINTEGER et PARAMETER_NOTEXIST
     */
    public static Motcle getMtclByIdmtcl(RequestMap requete, Thesaurus thesaurus, String param_name) throws ParameterException {
        String param_value = requete.getParameter(param_name);
        if (param_value == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, param_name);
        }
        param_value = XMLUtils.cleanString(param_value);
        if (param_value.length() == 0) {
            return null;
        }
        Motcle mtcl = null;
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            mtcl = thesaurus.getMotcleByIdalpha(param_value);
        }
        if (mtcl == null) {
            try {
                int idmtcl = Integer.parseInt(param_value);
                mtcl = thesaurus.getMotcleByIdths(idmtcl);
            } catch (NumberFormatException nfe) {
                throw new ParameterException(ParameterException.PARAMETER_NOTINTEGER, param_name + "=" + param_value);
            }
        }
        if (mtcl == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, param_name + "=" + param_value);
        }
        return mtcl;
    }

    public static ScrutariExport getScrutariExport(RequestMap requete, ScrutariExportManager scrutariExportManager) throws ParameterException {
        String param_tableexport = requete.getParameter(SCRUTARIEXPORT_PARAMNAME);
        return getScrutariExport(param_tableexport, scrutariExportManager);
    }

    private static ScrutariExport getScrutariExport(String param_scrutariexport, ScrutariExportManager scrutariExportManager) throws ParameterException {
        if (param_scrutariexport == null) {
            throw new ParameterException(ParameterException.SCRUTARIEXPORT_UNDEFINED, null);
        }
        ScrutariExport scrutariExport = scrutariExportManager.getScrutariExport(param_scrutariexport);
        if (scrutariExport == null) {
            throw new ParameterException(ParameterException.SCRUTARIEXPORT_NOTEXIST, param_scrutariexport);
        }
        return scrutariExport;
    }

    public static SqlExportBean getSqlExport(RequestMap requete, SqlExportManager sqlExportManager) throws ParameterException {
        String param_sqlexport = requete.getParameter(SQLEXPORT_PARAMNAME);
        return getSqlExport(param_sqlexport, sqlExportManager);
    }

    private static SqlExportBean getSqlExport(String param_sqlexport, SqlExportManager sqlExportManager) throws ParameterException {
        if (param_sqlexport == null) {
            throw new ParameterException(ParameterException.SQLEXPORT_UNDEFINED, null);
        }
        SqlExportBean sqlExportBean = sqlExportManager.getSqlExportBeanByName(param_sqlexport);
        if (sqlExportBean == null) {
            throw new ParameterException(ParameterException.SQLEXPORT_NOTEXIST, param_sqlexport);
        }
        return sqlExportBean;
    }

    public static Balayage getBalayage(RequestMap requete, BalayageManager balayageManager) throws ParameterException {
        String param_balayage = requete.getParameter(BALAYAGE_PARAMNAME);
        return getBalayage(param_balayage, balayageManager);
    }

    private static Balayage getBalayage(String param_sqlexport, BalayageManager balayageManager) throws ParameterException {
        if (param_sqlexport == null) {
            throw new ParameterException(ParameterException.BALAYAGE_UNDEFINED, null);
        }
        Balayage balayage = balayageManager.getBalayage(param_sqlexport);
        if (balayage == null) {
            throw new ParameterException(ParameterException.BALAYAGE_NOTEXIST, param_sqlexport);
        }
        return balayage;
    }

    public static Role getRole(RequestMap requete, PermissionManager permissionManager) throws ParameterException {
        String param_role = requete.getParameter(ROLE_PARAMNAME);
        return getRole(param_role, permissionManager);
    }

    private static Role getRole(String param_role, PermissionManager permissionManager) throws ParameterException {
        if (param_role == null) {
            throw new ParameterException(ParameterException.ROLE_UNDEFINED, null);
        }
        Role role = permissionManager.getRole(param_role);
        if (role == null) {
            throw new ParameterException(ParameterException.ROLE_UNDEFINED, param_role);
        }
        return role;
    }

    public static void printDocument(HtmlPrinter htmlPrinter, Document document, Locale formatLocale, boolean aveclien) {
        int count = document.getVersionCount();
        String chaine = "<a href=\"docs/";
        htmlPrinter.append("[&nbsp;");
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                htmlPrinter.append(" / ");
            }
            Version version = document.getVersion(i);
            VersionName versionName = version.getVersionName();
            if (aveclien) {
                htmlPrinter.append(chaine);
                htmlPrinter.append(versionName.toString());
                htmlPrinter.append("\">");
            }
            htmlPrinter.append(".");
            htmlPrinter.append(versionName.getExtension());
            if (aveclien) {
                FileLength fileLength = version.getFileLength();
                NumberFormat format = NumberFormat.getInstance(formatLocale);
                htmlPrinter.endA();
                htmlPrinter.startSmall();
                htmlPrinter.append(" (");
                float value = fileLength.getRoundedValue();
                htmlPrinter.escape(format.format(value));
                htmlPrinter.append("&nbsp;");
                if (fileLength.getRoundType() == FileLength.KB_ROUND) {
                    htmlPrinter.escape(LocalisationUtils.getKibiOctet(formatLocale));
                } else {
                    htmlPrinter.escape(LocalisationUtils.getMebiOctet(formatLocale));
                }
                htmlPrinter.append(")");
                htmlPrinter.endSmall();
                htmlPrinter.space();
            }
        }
        htmlPrinter.append("&nbsp;]");
    }

    static public String toBasename(AbregeFiche abregeFiche) {
        return abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "-" + String.valueOf(abregeFiche.getIdcorpus());
    }

    static public String toParameterString(Thesaurus thesaurus) {
        return THESAURUS_PARAMNAME + "=" + thesaurus.getSubsetKey().getSubsetName();
    }

    static public String toParameterString(Motcle motcle) {
        return THESAURUS_PARAMNAME + "=" + motcle.getThesaurus().getSubsetKey().getSubsetName() + "&amp;idths=" + String.valueOf(motcle.getIdths());
    }

    static public String toParameterString(Sphere sphere) {
        return SPHERE_PARAMNAME + "=" + sphere.getSubsetKey().getSubsetName();
    }

    static public String toParameterString(Corpus corpus) {
        return CORPUS_PARAMNAME + "=" + corpus.getSubsetKey().getSubsetName();
    }

    static public String toParameterString(AbregeFiche abregeFiche) {
        return CORPUS_PARAMNAME + "=" + abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "&amp;" + IDCORPUS_PARAMNAME + "=" + String.valueOf(abregeFiche.getIdcorpus());
    }

    static public String toParameterString(SubsetKey corpusKey, int idcorpus) {
        return CORPUS_PARAMNAME + "=" + corpusKey.getSubsetName() + "&amp;" + IDCORPUS_PARAMNAME + "=" + String.valueOf(idcorpus);
    }

    static public String toParameterString(Redacteur redacteur) {
        return SPHERE_PARAMNAME + "=" + redacteur.getSphere().getSubsetKey().getSubsetName() + "&amp;" + IDSPHERE_PARAMNAME + "=" + redacteur.getRedacteurKey().getIdsphereString();
    }

    static public String toParameterString(Document document) {
        return IDSUBSET_PARAMNAME + "=" + document.getIdsubset();
    }

    static public String toParameterString(ScrutariExport scrutariExport) {
        return SCRUTARIEXPORT_PARAMNAME + "=" + scrutariExport.getName();
    }

    static public String toHiddenParameter(Thesaurus thesaurus) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(THESAURUS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(thesaurus.getSubsetKey().getSubsetName());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(Motcle motcle) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(THESAURUS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(motcle.getThesaurus().getSubsetKey().getSubsetName());
        buf.append("\">");
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(IDTHS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(motcle.getIdths());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(Redacteur redacteur) {
        return toHiddenParameter(redacteur.getRedacteurKey());
    }

    static public String toHiddenParameter(RedacteurKey redacteurKey) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(SPHERE_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(redacteurKey.getSphereKey().getSubsetName());
        buf.append("\">");
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(IDSPHERE_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(redacteurKey.getIdsphereString());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(AbregeFiche abregeFiche) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(CORPUS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(abregeFiche.getCorpus().getSubsetKey().getSubsetName());
        buf.append("\">");
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(IDCORPUS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(abregeFiche.getIdcorpus());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(Sphere sphere) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(SPHERE_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(sphere.getSubsetKey().getSubsetName());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(Corpus corpus) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(CORPUS_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(corpus.getSubsetKey().getSubsetName());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(Document document) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(IDSUBSET_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(document.getIdsubset());
        buf.append("\">");
        return buf.toString();
    }

    static public String toHiddenParameter(ScrutariExport scrutariExport) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(SCRUTARIEXPORT_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(scrutariExport.getName());
        buf.append("\">");
        return buf.toString();
    }

    public static String toHiddenParameter(SqlExportBean sqlExportBean) {
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"hidden\" name=\"");
        buf.append(SQLEXPORT_PARAMNAME);
        buf.append("\" value=\"");
        buf.append(sqlExportBean.getName());
        buf.append("\">");
        return buf.toString();
    }

    public static boolean checkConfirmPassword(RequestMap requestMap, BdfServer bdfServer) throws ParameterException {
        String param_password = requestMap.getParameter(CONFIRMPASSWORD_PARAMNAME);
        if (param_password == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, CONFIRMPASSWORD_PARAMNAME);
        }
        return param_password.equals(bdfServer.getPasswordManager().getConfirmPassword());
    }

    public static Libelle[] getLibelleArray(RequestMap requestMap, String prefix, int[] langArray) {
        TextParser textParser = new TextParser();
        int length = langArray.length;
        MutableLibelle[] result = new MutableLibelle[length];
        for (int i = 0; i < length; i++) {
            int langInteger = langArray[i];
            String param_name = prefix + LangInteger.toString(langInteger);
            String param_value = requestMap.getParameter(param_name);
            if (param_value == null) {
                continue;
            }
            TypoOptions typoOptions = TypoOptions.getTypoOptions(langInteger);
            textParser.setQuoteOptions(typoOptions.getQuoteOptions());
            String libelleString = textParser.parseString(param_value);
            MutableLibelle libelle = new MutableLibelle();
            libelle.setLangInteger(langInteger);
            libelle.setLibelleString(libelleString);
            result[i] = libelle;
        }
        return result;
    }

}
