/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request;

import net.mapeadores.basedefiches.corpus.fiche.S;


/**
 *
 * @author Vincent Calame
 */
public class SpanRules {

    public final static SpanRules EMPHASIS_RULES = new SpanRules(2, 1);

    static {
        EMPHASIS_RULES.add(0, S.VALUE_BITVALUE, true);
        EMPHASIS_RULES.add(1, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules ABBR_RULES = new SpanRules(3, 2);

    static {
        ABBR_RULES.add(0, S.VALUE_BITVALUE, true);
        ABBR_RULES.add(1, S.TITLE_BITVALUE, true);
        ABBR_RULES.add(2, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules TECH_RULES = new SpanRules(2, 1);

    static {
        TECH_RULES.add(0, S.VALUE_BITVALUE, false);
        TECH_RULES.add(1, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules IMG_RULES = new SpanRules(5, 2);

    static {
        IMG_RULES.add(0, S.SRC_BITVALUE, false);
        IMG_RULES.add(1, S.VALUE_BITVALUE, true);
        IMG_RULES.add(2, S.REF_BITVALUE, false);
        IMG_RULES.add(3, S.LANG_BITVALUE, false);
        IMG_RULES.add(4, S.IDENTIFIERS_BITVALUE, false);

    }

    public final static SpanRules LINK_RULES = new SpanRules(4, 2);

    static {
        LINK_RULES.add(0, S.REF_BITVALUE, false);
        LINK_RULES.add(1, S.VALUE_BITVALUE, true);
        LINK_RULES.add(2, S.LANG_BITVALUE, false);
        LINK_RULES.add(3, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules INTERNALREF_RULES = new SpanRules(3, 2);

    static {
        INTERNALREF_RULES.add(0, S.REF_BITVALUE, false);
        INTERNALREF_RULES.add(1, S.VALUE_BITVALUE, true);
        INTERNALREF_RULES.add(2, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules OPTLINK_RULES = new SpanRules(4, 1);

    static {
        OPTLINK_RULES.add(0, S.VALUE_BITVALUE, true);
        OPTLINK_RULES.add(1, S.REF_BITVALUE, false);
        OPTLINK_RULES.add(2, S.LANG_BITVALUE, false);
        OPTLINK_RULES.add(3, S.IDENTIFIERS_BITVALUE, false);
    }

    public final static SpanRules SPAN_RULES = new SpanRules(2, 2);

    static {
        SPAN_RULES.add(0, S.IDENTIFIERS_BITVALUE, false);
        SPAN_RULES.add(1, S.VALUE_BITVALUE, true);
    }

    public final static SpanRules BR_RULES = new SpanRules(1, 1);

    static {
        BR_RULES.add(0, S.IDENTIFIERS_BITVALUE, false);
    }

    private int[] attributesBitValue;
    private boolean[] typo;
    private int importantAttribuesCount;

    private SpanRules(int count, int importantAttribuesCount) {
        attributesBitValue = new int[count];
        typo = new boolean[count];
        this.importantAttribuesCount = importantAttribuesCount;
    }

    private void add(int index, int bit, boolean is) {
        attributesBitValue[index] = bit;
        typo[index] = is;
    }

    public SpanRules(int[] attributesBitValue, boolean[] typo, int importantAttribuesCount) {
        this.attributesBitValue = attributesBitValue;
        this.typo = typo;
        this.importantAttribuesCount = importantAttribuesCount;
    }

    public int getAttributesCount() {
        return attributesBitValue.length;
    }

    public int getImportantAttributesCount() {
        return importantAttribuesCount;
    }

    public boolean isTypoAttribute(int index) {
        return typo[index];
    }

    public int getAttributeBitValue(int index) {
        return attributesBitValue[index];
    }

    static public SpanRules getSpanRules(short type) {
        switch (type) {
            case S.TYPE_EM:
            case S.TYPE_STRG:
            case S.TYPE_EMSTRG:
            case S.TYPE_SUP:
            case S.TYPE_SUB:
            case S.TYPE_DEL:
            case S.TYPE_INS:
            case S.TYPE_QUOTE:
                return EMPHASIS_RULES;
            case S.TYPE_ABBR:
            case S.TYPE_ACRONYM:
                return ABBR_RULES;
            case S.TYPE_SAMP:
            case S.TYPE_KBD:
            case S.TYPE_VAR:
            case S.TYPE_CODE:
                return TECH_RULES;
            case S.TYPE_IMG:
                return IMG_RULES;
            case S.TYPE_CITE:
            case S.TYPE_DFN:
                return OPTLINK_RULES;
            case S.TYPE_LINK:
            case S.TYPE_FICHE:
            case S.TYPE_MOTCLE:
                return LINK_RULES;
            case S.TYPE_ANCHOR:
            case S.TYPE_IREF:
                return INTERNALREF_RULES;
            case S.TYPE_SPAN:
                return SPAN_RULES;
            case S.TYPE_BR:
                return BR_RULES;
            default:
                return TECH_RULES;
        }

    }

}
