/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request;

import net.mapeadores.basedefiches.corpus.fiche.S;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class TextParser {

    private QuoteOptions quoteOptions = QuoteOptions.FRENCH_STYLE;

    public TextParser() {
    }

    public void setQuoteOptions(QuoteOptions quoteOptions) {
        this.quoteOptions = quoteOptions;
    }

    public String parseString(String s) {
        if (s == null) {
            return "";
        }
        int length = s.length();
        if (length == 0) {
            return "";
        }
        StringBuilder buf = new StringBuilder(length + 10);
        try {
            for (int i = 0; i < length; i++) {
                i = parseTypoChar(s, i, buf, true);
            }
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static String escapeString(String s) {
        StringBuilder buf = new StringBuilder(s.length() + 20);
        try {
            escapeString(buf, s);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void escapeString(Appendable buf, String s) throws IOException {
        int length = s.length();
        for (int i = 0; i < length; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '\u00A0':
                    buf.append('~');
                    break;
                case '~':
                case '\\':
                case '\u0060':
                case '\u00B4':
                    buf.append('\\');
                    buf.append(carac);
                    break;
                case '^':
                    if (i < (length - 1)) {
                        if (hasSuperscriptForm(s.charAt(i + 1))) {
                            buf.append('\\');
                        }
                    }
                    buf.append('^');
                    break;
                default:
                    buf.append(carac);

            }
        }
    }

    public int parseInSpanChar(String sourceString, int charIndex, Appendable buf, boolean isTypo) throws IOException {
        if (isTypo) {
            return parseTypoChar(sourceString, charIndex, buf, false);
        }
        char carac = sourceString.charAt(charIndex);
        if (carac == '\\') {
            if (charIndex < sourceString.length()) {
                boolean escape = false;
                char suivant = sourceString.charAt(charIndex + 1);
                if (isInSpanEscapeChar(suivant, false)) {
                    buf.append(suivant);
                } else {
                    buf.append('\\');
                    buf.append(suivant);
                }
                return charIndex + 1;
            } else {
                buf.append('\\');
                return charIndex;
            }
        } else {
            buf.append(carac);
            return charIndex;
        }
    }

    public int parseOutsideChar(String sourceString, int charIndex, Appendable buf) throws IOException {
        return parseTypoChar(sourceString, charIndex, buf, true);
    }

    private int parseTypoChar(String sourceString, int charIndex, Appendable buf, boolean outside) throws IOException {
        int length = sourceString.length();
        char carac = sourceString.charAt(charIndex);
        switch (carac) {
            case ' ':
                if (charIndex < (length - 1)) {
                    char caracsuivant = sourceString.charAt(charIndex + 1);
                    switch (caracsuivant) {
                        case '?':
                        case '!':
                        case ':':
                        case ';':
                        case '>':
                        case '=':
                        case '+':
                        case '/':
                        case '\u00BB': // »
                        case '\u2026': //…
                            buf.append('\u00A0');
                            break;
                        default:
                            buf.append(' ');
                    }
                } else {
                    buf.append(carac);
                }
                break;
            case '~':
                buf.append('\u00A0'); // &nbsp;
                break;

            case '\u0060': // `
                buf.append('\u2018'); // ‘
                break;

            case '\u00B4': // ´
                buf.append('\u2019'); // ’
                break;

            case '\'':
                charIndex = parseSimpleQuote(sourceString, charIndex, buf);
                break;

            case '\"':
                parseDoubleQuote(sourceString, charIndex, buf);
                break;

            case '.':
                if (charIndex < length - 2) {
                    if ((sourceString.charAt(charIndex + 1) == '.') && (sourceString.charAt(charIndex + 2) == '.')) {
                        buf.append('\u2026');
                        charIndex = charIndex + 2;
                    } else {
                        buf.append('.');
                    }
                } else {
                    buf.append('.');
                }
                break;

            case '\u00AB': //«
            case '<':
                buf.append(carac);
                if (charIndex < length - 1) {
                    char caracsuiv = sourceString.charAt(charIndex + 1);
                    if (caracsuiv == ' ') {
                        buf.append('\u00A0');
                        charIndex++;
                    }
                }
                break;

            case '-':
                if (charIndex < length - 1) {
                    if (sourceString.charAt(charIndex + 1) == '-') {
                        if ((charIndex == length - 2) || (sourceString.charAt(charIndex + 2) != '-')) {
                            buf.append('\u2013');
                            charIndex++;
                        } else {
                            buf.append('\u2014');
                            charIndex = charIndex + 2;
                        }
                    } else {
                        buf.append('-');
                    }
                } else {
                    buf.append('-');
                }
                break;

            case '^':
                charIndex = superscript(sourceString, charIndex, buf);
                break;
            case '\\':
                if (charIndex < sourceString.length() - 1) {
                    char char2 = sourceString.charAt(charIndex + 1);
                    if (char2 == 'u') {
                        charIndex = testUnicode(sourceString, charIndex + 1, buf);
                    } else {
                        boolean isEscapeChar = (outside) ? isOutsideEscapeChar(char2) : isInSpanEscapeChar(char2, true);
                        if (isEscapeChar) {
                            buf.append(char2);
                            charIndex++;
                        } else {
                            buf.append('\\');
                        }
                    }
                } else {
                    buf.append('\\');
                }
                break;

            default:
                buf.append(carac);
        }
        return charIndex;
    }

    private int parseSimpleQuote(String sourceString, int charIndex, Appendable buf) throws IOException {
        if (charIndex == 0) {
            charIndex = appendOpeningSimpleQuote(sourceString, charIndex, buf);
        } else if (charIndex == (sourceString.length() - 1)) {
            buf.append('\u2019');
        } else {
            char next = sourceString.charAt(charIndex + 1);
            char previous = sourceString.charAt(charIndex - 1);
            if (Character.isLetterOrDigit(previous)) {
                if (next == '\'') {
                    buf.append('\u201D');
                    charIndex++;
                } else {
                    buf.append('\u2019');
                }
            } else if (Character.isLetterOrDigit(next)) {
                buf.append('\u2018');
            } else if (next == '\'') {
                if ((charIndex + 1) == (sourceString.length() - 1)) {
                    buf.append('\u201D');
                } else {
                    char nextOfNext = sourceString.charAt(charIndex + 2);
                    if (Character.isLetterOrDigit(nextOfNext)) {
                        buf.append('\u201C');
                    } else if (Character.isWhitespace(nextOfNext)) {
                        if (Character.isWhitespace(previous)) {
                            buf.append('\'');
                            buf.append('\'');
                        } else {
                            buf.append('\u201D');
                        }
                    } else if (Character.isWhitespace(previous)) {
                        buf.append('\u201C');
                    } else {
                        buf.append('\'');
                        buf.append('\'');
                    }
                }
                charIndex++;
            } else if (Character.isWhitespace(next)) {
                if (Character.isWhitespace(previous)) {
                    buf.append('\'');
                } else {
                    buf.append('\u2019');
                }
            } else if (Character.isWhitespace(previous)) {
                buf.append('\u2018');
            } else {
                buf.append('\'');
            }
        }
        return charIndex;
    }

    private int appendOpeningSimpleQuote(String sourceString, int charIndex, Appendable buf) throws IOException {
        if (charIndex < (sourceString.length() - 1)) {
            if (sourceString.charAt(charIndex + 1) == '\'') {
                buf.append('\u201C');
                charIndex++;
            } else {
                buf.append('\u2018');
            }
        } else {
            buf.append('\u2018');
        }
        return charIndex;
    }

    private void parseDoubleQuote(String sourceString, int charIndex, Appendable buf) throws IOException {
        if (charIndex == 0) {
            appendOpeningQuote(buf);
        } else if (charIndex == (sourceString.length() - 1)) {
            appendClosingQuote(buf);
        } else {
            char next = sourceString.charAt(charIndex + 1);
            char previous = sourceString.charAt(charIndex - 1);
            if (Character.isLetterOrDigit(next)) {
                if (Character.isLetterOrDigit(previous)) {
                    buf.append('\"');
                } else {
                    appendOpeningQuote(buf);
                }
            } else if (Character.isLetterOrDigit(previous)) {
                appendClosingQuote(buf);
            } else if (Character.isWhitespace(next)) {
                if (Character.isWhitespace(previous)) {
                    buf.append('\"');
                } else {
                    appendClosingQuote(buf);
                }
            } else if (Character.isWhitespace(previous)) {
                appendOpeningQuote(buf);
            } else {
                buf.append('\"');
            }
        }
    }

    private void appendOpeningQuote(Appendable buf) throws IOException {
        buf.append(quoteOptions.getOpeningQuote());
        if (quoteOptions.isWithNonBreakingSpaces()) {
            buf.append('\u00A0');
        }
    }

    private void appendClosingQuote(Appendable buf) throws IOException {
        if (quoteOptions.isWithNonBreakingSpaces()) {
            buf.append('\u00A0');
        }
        buf.append(quoteOptions.getClosingQuote());
    }

    private static int testUnicode(String sourceString, int charIndex, Appendable buf) throws IOException {
        int min = Math.min(charIndex + 5, sourceString.length());
        int p = 0;
        int result = 0;
        StringBuilder numBuffer = new StringBuilder();
        for (int i = charIndex + 1; i < min; i++) {
            char carac = sourceString.charAt(i);
            int digit = Character.digit(carac, 16);
            if (digit >= 0) {
                result = result * 16;
                result = result + digit;
                p++;
            }
        }
        if (p == 0) {
            buf.append("\\u");
        } else {
            buf.append((char) result);
        }
        return charIndex + p;
    }

    private static int superscript(String sourceString, int charIndex, Appendable buf) throws IOException {
        int p = 0;
        for (int i = charIndex + 1; i < sourceString.length(); i++) {
            char carac = sourceString.charAt(i);
            char superchar = getsSuperscriptForm(carac);
            if (superchar == '0') {
                break;
            } else {
                buf.append(superchar);
                p++;
            }
        }
        if (p == 0) {
            buf.append('^');
        }
        return charIndex + p;
    }

    private static boolean isOutsideEscapeChar(char carac) {
        if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        }
        if (carac > '~') {
            return false;
        }
        switch (carac) {
            case '~':
            case '{':
            case '[':
            case ']':
            case '(':
            case ')':
            case '}':
            case '\\':
            case '#':
            case ':':
            case '>':
            case '!':
            case '<':
            case '=':
            case '+':
            case '-':
            case '^':
            case '_':
            case '*':
            case '\u0060':
            case '\u00B4':
                return true;
            default:
                return false;
        }
    }

    private static boolean isInSpanEscapeChar(char carac, boolean isTypo) {
        switch (carac) {
            case '~':
            case '^':
            case '\u0060':
            case '\u00B4':
                return isTypo;
            case '{':
            case '[':
            case ']':
            case '}':
            case '\\':
                return true;
            default:
                return false;
        }
    }

    private static void appendInSpan(Appendable buf, String s, boolean isTypo) throws IOException {
        for (int i = 0; i < s.length(); i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '\u00A0':
                    if (isTypo) {
                        carac = '~';
                    }
                    break;
                case '~':
                case '^':
                case '\u0060':
                case '\u00B4':
                    if (isTypo) {
                        buf.append('\\');
                    }
                    break;
                case '{':
                case '}':
                case '[':
                case ']':
                case '\\':
                    buf.append('\\');
                    break;
            }
            buf.append(carac);
        }
    }

    public static void escapeText(Appendable buf, String s, boolean beginParagraph) throws IOException {
        int length = s.length();
        for (int i = 0; i < length; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '\u00A0':
                    buf.append('~');
                    break;
                case '~':
                case '{':
                case '}':
                case '\\':
                case '\u0060':
                case '\u00B4':
                    buf.append('\\');
                    buf.append(carac);
                    break;
                case '-':
                    if ((beginParagraph) && (i == 0)) {
                        buf.append('\\');
                    } else if ((i < length - 1) && (s.charAt(i + 1) == '-')) {
                        buf.append('\\');
                    }
                    buf.append('-');
                    break;

                case '_':
                case '#':
                case '>':
                case '<':
                case '+':
                case '!':
                case ':':
                case '=':
                case '*':
                    if ((beginParagraph) && (i == 0)) {
                        buf.append('\\');
                    }
                    buf.append(carac);
                    break;

                case '[':
                case '(':
                    if ((i == 0) && (carac == '[')) {
                        buf.append('\\');
                    } else if (i < length - 2) {
                        char carsuiv = s.charAt(i + 1);
                        if ((carsuiv == '#') || (carsuiv == '>') || (carsuiv == '+')) {
                            buf.append('\\');
                        } else if (s.charAt(i + 2) == carac) {
                            buf.append('\\');
                        }
                    }
                    buf.append(carac);
                    break;

                case 'h':
                    if (i < length - 7) {
                        if (s.substring(i + 1, i + 7).equals("ttp://")) {
                            buf.append("\\h");
                        } else {
                            buf.append('h');
                        }
                    } else {
                        buf.append('h');
                    }
                    break;

                case 'w':
                    if (i < length - 4) {
                        if (s.substring(i + 1, i + 4).equals("ww.")) {
                            buf.append("\\w");
                        } else {
                            buf.append('w');
                        }
                    } else {
                        buf.append('w');
                    }
                    break;

                case '^':
                    if (i < (length - 1)) {
                        if (hasSuperscriptForm(s.charAt(i + 1))) {
                            buf.append('\\');
                        }
                    }
                    buf.append('^');
                    break;

                default:
                    buf.append(carac);

            }
        }
    }

    public static void escapeSpan(Appendable buf, S span, boolean specialchars) throws IOException {
        if (specialchars) {
            switch (span.getType()) {
                case S.TYPE_EM:
                case S.TYPE_STRG:
                case S.TYPE_EMSTRG:
                    escapeEmphasisSpan(buf, span);
                    break;
                case S.TYPE_IREF:
                case S.TYPE_ANCHOR:
                    escapeInternalRefSpan(buf, span);
                    break;
                case S.TYPE_BR:
                    escapeBrSpan(buf, span);
                    break;
                default:
                    escapeBracketsSpan(buf, span);
            }
        } else {
            escapeBracketsSpan(buf, span);
        }
    }

    private static void escapeBrSpan(Appendable buf, S span) throws IOException {
        if (span.hasIdentifiers()) {
            escapeBracketsSpan(buf, span);
        } else {
            buf.append("[%]");
        }
    }

    private static void escapeEmphasisSpan(Appendable buf, S span) throws IOException {
        if (span.hasIdentifiers()) {
            escapeBracketsSpan(buf, span);
            return;
        }
        int num = 1;
        switch (span.getType()) {
            case S.TYPE_EM:
                num = 1;
                break;
            case S.TYPE_STRG:
                num = 2;
                break;
            case S.TYPE_EMSTRG:
                num = 3;
                break;
        }
        for (int i = 0; i < num; i++) {
            buf.append('{');
        }
        appendInSpan(buf, span.getValue(), true);
        for (int i = 0; i < num; i++) {
            buf.append('}');
        }
    }

    private static void escapeInternalRefSpan(Appendable buf, S span) throws IOException {
        if (span.hasIdentifiers()) {
            escapeBracketsSpan(buf, span);
            return;
        }
        String text = span.getValue();
        String ref = span.getRef();
        if ((text.length() > 0) && (!text.equals(ref))) {
            escapeBracketsSpan(buf, span);
            return;
        }
        char sep = '#';
        switch (span.getType()) {
            case S.TYPE_IREF:
                sep = '>';
                break;
            case S.TYPE_ANCHOR:
                sep = '#';
                break;
        }
        buf.append('[');
        buf.append(sep);
        appendInSpan(buf, ref, false);
        if (text.length() == 0) {
            buf.append('!');
        }
        buf.append(']');
    }

    private static void escapeBracketsSpan(Appendable buf, S span) throws IOException {
        int mask = span.getAttributeMask();
        span.checkHttpValue(false);
        buf.append('[');
        buf.append(S.getTypeToInitial(span.getType()));
        char carac = S.getSubtypeToInitial(span.getSubtype());
        if (carac != '0') {
            buf.append(carac);
        }
        SpanRules spanRules = SpanRules.getSpanRules(span.getType());
        int importantAttributesCount = spanRules.getImportantAttributesCount();
        int attributesCount = spanRules.getAttributesCount();
        for (int i = 0; i < importantAttributesCount; i++) {
            String attribute = span.getAttribute(spanRules.getAttributeBitValue(i));
            appendBrackets(buf, attribute, spanRules.isTypoAttribute(i));
        }
        if (importantAttributesCount < attributesCount) {
            int p = 0;
            for (int i = importantAttributesCount; i < attributesCount; i++) {
                String attribute = span.getAttribute(spanRules.getAttributeBitValue(i));
                if (attribute.length() == 0) {
                    p++;
                } else {
                    if (p > 0) {
                        for (int j = 0; j < p; j++) {
                            buf.append("[ ]");
                        }
                        p = 0;
                    }
                    appendBrackets(buf, attribute, spanRules.isTypoAttribute(i));
                }
            }
        }
        buf.append(']');
        span.checkHttpValue(true);
    }

    private static void appendBrackets(Appendable buf, String s, boolean isTypo) throws IOException {
        buf.append('[');
        if (s.length() == 0) {
            buf.append(' ');
        } else {
            appendInSpan(buf, s, isTypo);
        }
        buf.append(']');
    }

    public static boolean hasSuperscriptForm(char c) {
        switch (c) {
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
            case 'i':
            case '+':
            case '-':
            case '=':
            case '(':
            case ')':
            case 'n':
                return true;
            default:
                return false;
        }
    }

    public static char getsSuperscriptForm(char c) {
        switch (c) {
            case '0':
                return '\u2070';
            case '1':
                return '\u00B9';
            case '2':
                return '\u00B2';
            case '3':
                return '\u00B3';
            case '4':
                return '\u2074';
            case '5':
                return '\u2075';
            case '6':
                return '\u2076';
            case '7':
                return '\u2077';
            case '8':
                return '\u2078';
            case '9':
                return '\u2079';
            case 'i':
                return '\u2071';
            case '+':
                return '\u207A';
            case '-':
                return '\u207B';
            case '=':
                return '\u207C';
            case '(':
                return '\u207D';
            case ')':
                return '\u207E';
            case 'n':
                return '\u207F';
            default:
                return '0';
        }
    }

}
