/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formfactory;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.request.GenericParameters;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.MasterSubsetItemPointeur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.text.SeparatorOptions;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormParameters {

    private static final SeparatorOptions DEFAULT_INLINE_SEPARATOROPTIONS = new SeparatorOptions(';', true, true);
    private static final SeparatorOptions DEFAULT_BLOCK_SEPARATOROPTIONS = new SeparatorOptions('\n', true, false);
    private BdfContext bdfContext;
    private BdfServer bdfServer;
    private FicheItemToString.Parameters ficheItemToStringParameters;
    private FichePointeur fichePointeur;
    private MasterSubsetItemPointeur masterSubsetItemPointeur;
    private RedacteurKey defaultRedacteurKey;
    private boolean newFiche = false;
    private boolean withSpecialParseChars = false;
    private SeparatorOptions inlineSeparatorOptions = DEFAULT_INLINE_SEPARATOROPTIONS;
    private SeparatorOptions blockSeparatorOptions = DEFAULT_BLOCK_SEPARATOROPTIONS;
    private IncludeSummary includeSummary;
    private GenericParameters genericParameters;
    private BdfUser bdfUser;
    private LocalisationInfo localisationInfo;

    private FicheFormParameters(BdfContext bdfContext, BdfUser bdfUser, FichePointeur fichePointeur) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.bdfUser = bdfUser;
        this.localisationInfo = bdfUser.getLocalisationInfo();
        this.fichePointeur = fichePointeur;
        initDefaultFicheItemToStringParameters();
        initDefaultIncludeSummary();
    }

    private FicheFormParameters(BdfContext bdfContext, LocalisationInfo localisationInfo, FichePointeur fichePointeur) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.localisationInfo = localisationInfo;
        this.fichePointeur = fichePointeur;
        initDefaultFicheItemToStringParameters();
        initDefaultIncludeSummary();
    }

    private FicheFormParameters(BdfServer bdfServer, BdfUser bdfUser, FichePointeur fichePointeur) {
        this.bdfServer = bdfServer;
        this.bdfUser = bdfUser;
        this.localisationInfo = bdfUser.getLocalisationInfo();
        this.fichePointeur = fichePointeur;
        initDefaultFicheItemToStringParameters();
        initDefaultIncludeSummary();
    }

    private void initDefaultFicheItemToStringParameters() {
        if (bdfUser != null) {
            this.ficheItemToStringParameters = BdfUserUtils.getFicheItemToStringParameters(bdfUser);
        } else {
            this.ficheItemToStringParameters = BdfUserUtils.getFicheItemToStringParameters(localisationInfo);
        }
    }

    private void initDefaultIncludeSummary() {
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(fichePointeur.getCorpus());
        this.includeSummary = CorpusUIUtils.getFilteredIncludeSummary(corpusUI);
    }

    public BdfContext getBdfContext() {
        return bdfContext;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public BdfUser getBdfUser() {
        return bdfUser;
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public int getWorkingLangInteger() {
        return localisationInfo.getWorkingLangInteger();
    }

    public FichePointeur getFichePointeur() {
        return fichePointeur;
    }

    public FicheItemToString.Parameters getFicheItemToStringParameters() {
        return ficheItemToStringParameters;
    }

    public void setFicheItemToStringParameters(FicheItemToString.Parameters ficheItemToStringParameters) {
        if (ficheItemToStringParameters == null) {
            initDefaultFicheItemToStringParameters();
        } else {
            this.ficheItemToStringParameters = ficheItemToStringParameters;
        }
    }

    public boolean isNewFiche() {
        return newFiche;
    }

    public void setNewFiche(boolean newFiche) {
        this.newFiche = newFiche;
    }

    public boolean isWithSpecialParseChars() {
        return withSpecialParseChars;
    }

    public void setWithSpecialParseChars(boolean withSpecialParseChars) {
        this.withSpecialParseChars = withSpecialParseChars;
    }

    /**
     *  Peut être nul.
     */
    public RedacteurKey getDefaultRedacteurKey() {
        return defaultRedacteurKey;
    }

    /**
     * Peut être nul.
     */
    public SubsetKey getDefaultSphereKey() {
        if (defaultRedacteurKey == null) {
            return null;
        }
        return defaultRedacteurKey.getSphereKey();
    }

    public void setDefaultRedacteurKey(RedacteurKey defaultRedacteurKey) {
        this.defaultRedacteurKey = defaultRedacteurKey;
    }

    public SeparatorOptions getInlineSeparatorOptions() {
        return inlineSeparatorOptions;
    }

    public void setInlineSeparatorOptions(SeparatorOptions inlineSeparatorOptions) {
        if (inlineSeparatorOptions == null) {
            this.inlineSeparatorOptions = DEFAULT_INLINE_SEPARATOROPTIONS;
        } else {
            this.inlineSeparatorOptions = inlineSeparatorOptions;
        }
    }

    public SeparatorOptions getBlockSeparatorOptions() {
        return blockSeparatorOptions;
    }

    public void setBlockSeparatorOptions(SeparatorOptions blockSeparatorOptions) {
        if (blockSeparatorOptions == null) {
            this.blockSeparatorOptions = DEFAULT_BLOCK_SEPARATOROPTIONS;
        } else {
            this.blockSeparatorOptions = blockSeparatorOptions;
        }
    }

    /**
     * Peut être nul.
     */
    public MasterSubsetItemPointeur getMasterSubsetItemPointeur() {
        return masterSubsetItemPointeur;
    }

    public void setMasterSubsetItemPointeur(MasterSubsetItemPointeur masterSubsetItemPointeur) {
        this.masterSubsetItemPointeur = masterSubsetItemPointeur;
    }

    public IncludeSummary getIncludeSummary() {
        return includeSummary;
    }

    public void setIncludeSummary(IncludeSummary includeSummary) {
        if (includeSummary == null) {
            initDefaultIncludeSummary();
        } else {
            this.includeSummary = includeSummary;
        }
    }

    /**
     * Peut être nul.
     */
    public GenericParameters getGenericParameters() {
        return genericParameters;
    }

    public void setGenericParameters(GenericParameters genericParameters) {
        this.genericParameters = genericParameters;
    }

    public static FicheFormParameters newInstance(BdfContext bdfContext, BdfUser bdfUser, FichePointeur fichePointeur) {
        FicheFormParameters ficheFormParameters = new FicheFormParameters(bdfContext, bdfUser, fichePointeur);
        ficheFormParameters.setDefaultRedacteurKey(bdfUser.getRedacteurKey());
        return ficheFormParameters;
    }

    public static FicheFormParameters newInstance(BdfContext bdfContext, LocalisationInfo localisationInfo, FichePointeur fichePointeur) {
        FicheFormParameters ficheFormParameters = new FicheFormParameters(bdfContext, localisationInfo, fichePointeur);
        return ficheFormParameters;
    }

    public static FicheFormParameters newInstance(BdfServer bdfServer, BdfUser bdfUser, FichePointeur fichePointeur) {
        FicheFormParameters ficheFormParameters = new FicheFormParameters(bdfServer, bdfUser, fichePointeur);
        ficheFormParameters.setDefaultRedacteurKey(bdfUser.getRedacteurKey());
        return ficheFormParameters;
    }

}
