/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formfactory;

import fr.exemole.bdfserver.request.tostring.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.pointeurs.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.text.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.plainexport.SeparatorOptionsBean;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.primitives.*;
import java.io.*;
import net.mapeadores.util.html.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormUtils {

    private FicheFormUtils() {
    }

    public static String getStringValue(CorpusField corpusField, FicheFormParameters ficheFormParameters) {
        FicheItemToString.Parameters parameters = getParameters(ficheFormParameters, corpusField);
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        FieldKey fieldKey = corpusField.getFieldKey();
        Object obj = fichePointeur.getValue(fieldKey);
        if (obj == null) {
            return "";
        }
        if (obj instanceof Integer) {
            return obj.toString();
        }
        if (obj instanceof String) {
            return TextParser.escapeString((String) obj);
        }
        if (obj instanceof FicheItem) {
            return FicheItemToString.toString((FicheItem) obj, parameters);
        }
        if (obj instanceof FicheItemList) {
            boolean isBlock = CorpusMetadataUtils.isBlockDisplayInformationField(corpusField);
            SeparatorOptions separatorOptions = (isBlock) ? ficheFormParameters.getBlockSeparatorOptions() : ficheFormParameters.getInlineSeparatorOptions();
            return FicheItemToString.toString((FicheItemList) obj, separatorOptions, parameters);
        }
        if (obj instanceof FicheBlockList) {
            return FicheBlockToString.toString((FicheBlockList) obj, ficheFormParameters.isWithSpecialParseChars());
        }
        throw new SwitchException("Unexpected instance of " + obj.getClass().getName());
    }

    public static FicheItemToString.Parameters getParameters(FicheFormParameters ficheFormParameters, CorpusField corpusField) {
        SubsetKey defaultSphereKey = (SubsetKey) corpusField.getOption(FieldOptions.DEFAULTSPHEREKEY_OPTION);
        if (defaultSphereKey == null) {
            return ficheFormParameters.getFicheItemToStringParameters();
        }
        FicheItemToString.Parameters newParam = new FicheItemToString.Parameters(ficheFormParameters.getFicheItemToStringParameters());
        newParam.setDefautSphereKey(defaultSphereKey);
        return newParam;
    }

    public static String getStringValue(Thesaurus thesaurus, MotcleCroisementList motcleList, boolean ignorePoids, FicheFormParameters ficheFormParameters) {
        if (motcleList == null) {
            return "";
        }
        boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        boolean isBabelien = ThesaurusUtils.isBabelienType(thesaurus);
        int langInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, ficheFormParameters.getWorkingLangInteger());
        SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
        int size = motcleList.getSubsetItemCount();
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                appendSeparator(buf, separatorOptions);
            }
            Motcle motcle = motcleList.getMotcle(i);
            int poids = motcleList.getPoids(i);
            if (withIdalpha) {
                buf.append(motcle.getIdalpha());
            } else {
                String lib = motcle.getLibelleString(langInteger, false);
                if ((lib == null) || (lib.length() == 0)) {
                    if (!isBabelien) {
                        lib = String.valueOf(motcle.getIdths());
                    } else {
                        lib = motcle.getLibelleString(0, true);
                    }
                }
                buf.append(lib);
            }
            if (!ignorePoids) {
                appendPoids(buf, motcleList.getPoids(i));
            }
        }
        if (separatorOptions.isLastInclude()) {
            appendSeparator(buf, separatorOptions);
        }
        return buf.toString();
    }

    private static void appendPoids(StringBuffer stringBuffer, int poids) {
        if (poids > 1) {
            if (poids == 2) {
                stringBuffer.append('+');
            } else if (poids == 3) {
                stringBuffer.append("++");
            } else if (poids == 4) {
                stringBuffer.append("+++");
            } else {
                stringBuffer.append(" <");
                stringBuffer.append(poids);
                stringBuffer.append('>');
            }
        }
    }

    private static void appendSeparator(StringBuffer stringBuffer, SeparatorOptions separatorOptions) {
        stringBuffer.append(separatorOptions.getSeparator());
        if (separatorOptions.isSpaceInclude()) {
            stringBuffer.append(' ');
        }
    }

}
