/* BdfServer
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp;

import fr.exemole.bdfserver.request.form.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultLiageElement extends AbstractIncludeElement implements LiageElement {

    private LocalisationInfo localisationInfo;
    private AbregeFiche[] abregeFicheArray;
    private int[] poidsArray;
    private int rows = 1;
    private boolean globalLiage = false;
    private String piocheSelectString;

    public DefaultLiageElement(FilteredIncludeKey filteredIncludeKey) {
        super(filteredIncludeKey);
        if (filteredIncludeKey.getIncludeKey().equals(IncludeKey.LIAGE)) {
            globalLiage = true;
            rows = 4;
        }
    }

    public boolean isGlobalLiage() {
        return globalLiage;
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        this.localisationInfo = localisationInfo;
    }

    public int getAbregeFicheCount() {
        if (abregeFicheArray == null) {
            return 0;
        }
        return abregeFicheArray.length;
    }

    public int getPoids(int i) {
        if (poidsArray == null) {
            return -1;
        }
        return poidsArray[i];
    }

    public AbregeFiche getAbregeFiche(int i) {
        return abregeFicheArray[i];
    }

    public String getValue(int i) {
        AbregeFiche abregeFiche = abregeFicheArray[i];
        String value = null;
        if (globalLiage) {
            value = abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "/" + String.valueOf(abregeFiche.getIdcorpus());
        } else {
            value = String.valueOf(abregeFiche.getIdcorpus());
        }
        int poids = getPoids(i);
        if (poids > 1) {
            value = value + "<" + poids + ">";
        }
        return value;
    }

    public void setAbregeFicheList(AbregeFiche[] abregeFicheArray, int[] poidsArray) {
        this.abregeFicheArray = abregeFicheArray;
        this.poidsArray = poidsArray;
    }

    public int getRows() {
        return rows;
    }

    public void setRows(int rows) {
        this.rows = rows;
    }

    public String getPiocheSelectString() {
        return piocheSelectString;
    }

    public void setPiocheSelectString(String piocheSelectString) {
        this.piocheSelectString = piocheSelectString;
    }

}
