/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.formfactory.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.formimp.*;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormImpUtils {

    private FicheFormImpUtils() {
    }

    public static void setIntitule(FicheFormParameters ficheFormParameters, AbstractIncludeElement includeElement, IncludeUI includeUI) {
        IncludeUILibelleMap libelleMap = getIncludeUILibelleMap(ficheFormParameters);
        includeElement.setIntitule(libelleMap.getLibelleString(includeUI, ficheFormParameters.getWorkingLangInteger()));
    }

    public static void setIntitule(FicheFormParameters ficheFormParameters, AbstractCorpusFieldElement fieldElement, CorpusField corpusField) {
        fieldElement.setIntitule(corpusField.getLibelleString(ficheFormParameters.getWorkingLangInteger(), true));
    }

    public static IncludeUILibelleMap getIncludeUILibelleMap(FicheFormParameters ficheFormParameters) {
        return ficheFormParameters.getBdfServer().getIncludeLibelleManager().getIncludeUILibelleMap(ficheFormParameters.getFichePointeur().getCorpus().getSubsetKey());
    }

    public static int checkRows(int rows, int valueLength) {
        if (rows > 1) {
            if (valueLength > (rows * 140)) {
                rows = rows + 2;
            }
            return rows;
        }
        if (valueLength > 80) {
            return 3;
        }
        return 1;
    }

    public static boolean isMandatory(IncludeUI includeUI) {
        ValueOption valueOption = includeUI.getValueOption();
        if (valueOption == null) {
            return false;
        }
        return valueOption.isMandatory();
    }

    public static boolean isMandatory(FieldUI fieldUI) {
        ValueOption valueOption = fieldUI.getValueOption();
        if (valueOption == null) {
            return false;
        }
        return valueOption.isMandatory();
    }

    public static String getDefVal(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        ValueOption valueOption = includeUI.getValueOption();
        if (valueOption == null) {
            return "";
        }
        return replaceGenericParameters(ficheFormParameters, valueOption.getDefaultValue());
    }

    public static String getDefVal(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        CorpusField corpusField = fieldUI.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        if (fieldKey == FieldKey.REDACTEURS) {
            RedacteurKey redacteurKey = ficheFormParameters.getDefaultRedacteurKey();
            if (redacteurKey == null) {
                return "";
            }
            String value = redacteurKey.getIdsphereString();
            SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
            if (separatorOptions.isLastInclude()) {
                value = value + separatorOptions.getSeparator();
                if (separatorOptions.isSpaceInclude()) {
                    value = value + " ";
                }
            }
            return value;
        } else {
            ValueOption valueOption = fieldUI.getValueOption();
            if (valueOption != null) {
                String defaultValue = valueOption.getDefaultValue();
                defaultValue = replaceGenericParameters(ficheFormParameters, defaultValue);
                if (defaultValue.length() > 0) {
                    if ((fieldKey.isInformationFieldKey()) && (!CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
                        SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
                        if (separatorOptions.isLastInclude()) {
                            defaultValue = defaultValue + separatorOptions.getSeparator();
                            if (separatorOptions.isSpaceInclude()) {
                                defaultValue = defaultValue + " ";
                            }
                        }
                    }
                    return defaultValue;
                }
            }
            return "";
        }
    }

    /*private static String testGenericParameters(FicheFormParameters ficheFormParameters, String value) {
    int length = value.length();
    if (length < 9) {
    return value;
    }
    GenericParameters genericParameters = ficheFormParameters.getGenericParameters();
    if (genericParameters == null) {
    return value;
    }
    if (value.charAt((length - 1)) != '$') {
    return value;
    }
    if (!value.startsWith("$PARAM_")) {
    return value;
    }
    String name = value.substring("$PARAM_".length(), length - 1);
    String val = genericParameters.getGenericParameter(name);
    if (val != null) {
    return val;
    }
    return value;
    }*/
    public static String replaceGenericParameters(FicheFormParameters ficheFormParameters, String s) {
        int length = s.length();
        if (length < 9) {
            return s;
        }
        GenericParameters genericParameters = ficheFormParameters.getGenericParameters();
        if (genericParameters == null) {
            return s;
        }
        int idx1 = s.indexOf("$PARAM_");
        if (idx1 == -1) {
            return s;
        }
        StringBuilder buf = new StringBuilder();
        buf.append(s.substring(0, idx1));
        while (true) {
            int idx2 = s.indexOf('$', idx1 + 1);
            if (idx2 == -1) {
                buf.append(s.substring(idx1));
                break;
            }
            String name = s.substring(idx1 + 7, idx2);
            String val = genericParameters.getGenericParameter(name);
            if (val != null) {
                buf.append(val);
            } else {
                buf.append("??PARAM_" + name + "??");
            }
            idx1 = s.indexOf("$PARAM_", idx2 + 1);
            if (idx1 == -1) {
                buf.append(s.substring(idx2 + 1));
                break;
            } else {
                buf.append(s.substring(idx2 + 1, idx1));
            }
        }
        return buf.toString();
    }

}
