/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.formimp.*;


/**
 *
 * @author Vincent Calame
 */
public class FieldElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;
    private FieldKey fieldKey;

    public FieldElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
        this.fieldKey = corpusField.getFieldKey();
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        FieldElementBuilder builder = new FieldElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        if (fieldKey.ownsToCorpsdefiche()) {
            return toTextFieldElement();
        }
        InputOption inputOption = fieldUI.getInputOption();
        switch (inputOption.getInputType()) {
            case InputOption.INPUT_HIDDEN:
                return toHiddenFieldElement();
            default:
                return toItemFieldElement();
        }
    }

    private DefaultHiddenFieldElement toHiddenFieldElement() {
        DefaultHiddenFieldElement formElement = new DefaultHiddenFieldElement(corpusField);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setValue(value);
        return formElement;
    }

    private DefaultTextFieldElement toTextFieldElement() {
        DefaultTextFieldElement formElement = new DefaultTextFieldElement(corpusField);
        formElement.setMandatory(isMandatory);
        formElement.setRows(fieldUI.getInputOption().getRows());
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setFormattedText(value);
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

    private DefaultItemFieldElement toItemFieldElement() {
        DefaultItemFieldElement formElement = new DefaultItemFieldElement(corpusField);
        formElement.setMandatory(isMandatory);
        InputOption inputOption = fieldUI.getInputOption();
        boolean redimAllowed = getDefaultRedimAllowed();
        formElement.setRedimAllowed(redimAllowed);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setValue(value);
        int rows = inputOption.getRows();
        if (redimAllowed) {
            rows = FicheFormImpUtils.checkRows(rows, value.length());
        }
        if ((rows == 1) && (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
            rows = 3;
        }
        formElement.setRows(rows);
        formElement.setWidthType(inputOption.getWidthType());
        if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
            SubsetKey subsetKey = (SubsetKey) corpusField.getOption(FieldOptions.DEFAULTSPHEREKEY_OPTION);
            if (subsetKey == null) {
                subsetKey = ficheFormParameters.getDefaultSphereKey();
            }
            formElement.setSphereKey(subsetKey);
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

    private boolean getDefaultRedimAllowed() {
        if (!fieldKey.isProprieteFieldKey()) {
            return true;
        }
        switch (corpusField.getFicheItemType()) {
            case CorpusField.ITEM_FIELD:
            case CorpusField.PARA_FIELD:
                return true;
            default:
                return false;
        }
    }

}
