/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.form.*;
import fr.exemole.bdfserver.request.formfactory.*;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.util.text.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.basedefiches.thesaurus.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.request.formimp.ficheform.*;
import fr.exemole.bdfserver.request.formimp.*;


/**
 *
 * @author Vincent Calame
 */
public class GeopointSubfieldsElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private GeopointSubfieldsElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        GeopointSubfieldsElementBuilder builder = new GeopointSubfieldsElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FicheItemToString.Parameters parameters = ficheFormParameters.getFicheItemToStringParameters();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultGeopointSubfieldsElement formElement = new DefaultGeopointSubfieldsElement(corpusField);
        formElement.setMandatory(isMandatory);
        CorpusField[] corpusFieldArray = (CorpusField[]) corpusField.getOption(FieldOptions.ADDRESSFIELDARRAY_OPTION);
        if ((corpusFieldArray != null) && (corpusFieldArray.length > 0)) {
            int length = corpusFieldArray.length;
            String array[] = new String[length];
            for (int i = 0; i < length; i++) {
                array[i] = corpusFieldArray[i].getFieldKey().toString();
            }
            formElement.setAdresseFieldNameArray(array);
        }
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        if (ficheItem != null) {
            if (ficheItem instanceof Geopoint) {
                Geopoint geopoint = (Geopoint) ficheItem;
                formElement.setLatitude(FicheItemToString.toString(geopoint.getLatitude(), parameters));
                formElement.setLongitude(FicheItemToString.toString(geopoint.getLongitude(), parameters));
            } else {
                formElement.setLatitude(FicheItemToString.toString(ficheItem, parameters));
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
