/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.*;
import fr.exemole.bdfserver.request.tostring.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.request.formimp.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class ImageFieldElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private ImageFieldElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        ImageFieldElementBuilder builder = new ImageFieldElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultImageFieldElement formElement = new DefaultImageFieldElement(corpusField);
        formElement.setMandatory(isMandatory);
        boolean subfields = false;
        Object obj = corpusField.getOption(FieldOptions.SUBFIELDDISPLAY_OPTION);
        if ((obj != null) && (obj.equals(Boolean.TRUE))) {
            subfields = true;
        }
        formElement.setWithSubfields(subfields);
        String baseUrl = null;
        SubsetKey albumKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
        if (albumKey != null) {
            Album album = (Album) ficheFormParameters.getBdfServer().getBasedefiches().getSubset(albumKey);
            formElement.setAlbum(album);
            AlbumDim vignetteDim = album.getAlbumMetadata().getAlbumDimForUsage(AlbumConstants.VIGNETTE_USAGE);
            baseUrl = "images/" + albumKey.getSubsetName() + "/";
            if (vignetteDim != null) {
                baseUrl = baseUrl + vignetteDim.getName() + "/";
            }
        } else {
            baseUrl = corpusField.getStringOption(FieldOptions.BASEURL_OPTION);
        }
        formElement.setBaseUrl(baseUrl);
        formElement.setImageSource(corpusField.getStringOption(FieldOptions.IMAGESOURCE_OPTION));
        formElement.setSrcAlwaysEditable(BdfUserUtils.isSrcAlwaysEditable(ficheFormParameters.getBdfUser()));
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        if (ficheItem != null) {
            if (ficheItem instanceof Image) {
                Image image = (Image) ficheItem;
                formElement.setSrc(image.getSrc());
                formElement.setAlt(image.getAlt());
                formElement.setTitle(image.getTitle());
            } else {
                formElement.setSrc(FicheItemToString.toString(ficheItem, null));
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
