/* BdfServer
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.*;
import net.mapeadores.basedefiches.corpus.*;

import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import fr.exemole.bdfserver.request.formimp.*;
import fr.exemole.bdfserver.request.form.*;


/**
 *
 * @author Vincent Calame
 */
public class LiageElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private IncludeUI includeUI;
    private FilteredIncludeKey filteredIncludeKey;
    private IncludeKey includeKey;
    private boolean hasPoidsFilter;
    private ArrayList<AbregeFiche> abrList = new ArrayList<AbregeFiche>();
    private ArrayList<Integer> poidsList;

    private LiageElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.includeUI = includeUI;
        this.filteredIncludeKey = includeUI.getFiteredIncludeKey();
        this.includeKey = filteredIncludeKey.getIncludeKey();
        this.hasPoidsFilter = filteredIncludeKey.hasPoidsFilter();
    }

    public static IncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        LiageElementBuilder builder = new LiageElementBuilder(ficheFormParameters, includeUI);
        AbstractIncludeElement includeElement = builder.toIncludeElement();
        if (includeElement != null) {
            FicheFormImpUtils.setIntitule(ficheFormParameters, includeElement, includeUI);
            includeElement.setMandatory(FicheFormImpUtils.isMandatory(includeUI));
        }
        return includeElement;
    }

    private AbstractIncludeElement toIncludeElement() {
        DefaultLiageElement element = new DefaultLiageElement(filteredIncludeKey);
        element.setLocalisationInfo(ficheFormParameters.getLocalisationInfo());
        if (includeKey.equals(IncludeKey.LIAGE)) {
            if (!ficheFormParameters.isNewFiche()) {
                initAbregeFicheList();
            }
            LimitOption limitOption = includeUI.getLimitOption();
            boolean noNew = limitOption.isNoNewAllowed();
            if (noNew) {
                if (ficheFormParameters.isNewFiche()) {
                    return null;
                } else {
                    if (abrList.size() == 0) {
                        return null;
                    }
                }
            }
        } else {
            Corpus corpus = (Corpus) ficheFormParameters.getBdfServer().getBasedefiches().getSubset(includeKey.getSubsetKey());
            if (corpus == null) {
                return null;
            }
            if (!ficheFormParameters.isNewFiche()) {
                initAbregeFicheList(corpus);
            }
        }
        int size = abrList.size();
        if (size > 0) {
            AbregeFiche[] abregeFicheArray = new AbregeFiche[size];
            int[] poidsArray = new int[size];
            for (int i = 0; i < size; i++) {
                abregeFicheArray[i] = abrList.get(i);
                if (poidsList != null) {
                    poidsArray[i] = poidsList.get(i).intValue();
                } else {
                    poidsArray[i] = -1;
                }
            }
            element.setAbregeFicheList(abregeFicheArray, poidsArray);
        }
        PiocheSelectOption piocheSelectOption = includeUI.getPiocheSelectOption();
        if (piocheSelectOption != null) {
            String piocheSelectString = piocheSelectOption.getPiocheSelectString();
            if (piocheSelectString != null) {
                piocheSelectString = FicheFormImpUtils.replaceGenericParameters(ficheFormParameters, piocheSelectString);
                element.setPiocheSelectString(piocheSelectString);
            }
        }


        return element;
    }

    private void initAbregeFicheList() {
        poidsList = new ArrayList<Integer>();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        FicheCroisementList[] array = FichePointeurUtils.getNoList(fichePointeur, ficheFormParameters.getIncludeSummary());
        int length = array.length;
        for (int j = 0; j < length; j++) {
            FicheCroisementList abregeFicheList = array[j];
            int size = abregeFicheList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                AbregeFiche abrg2 = abregeFicheList.getAbregeFiche(i);
                int poids = abregeFicheList.getPoids(i);
                abrList.add(abrg2);
                poidsList.add(new Integer(poids));
            }
        }
    }

    private void initAbregeFicheList(Corpus corpus) {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        CorpusCroisementAccess acces = fichePointeur.getCorpusCroisementAccess(corpus);
        if (hasPoidsFilter) {
            FicheCroisementList abregeFicheList = acces.getFicheCroisement(filteredIncludeKey.getPoidsFilter());
            int size = abregeFicheList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                AbregeFiche other = abregeFicheList.getAbregeFiche(i);
                abrList.add(other);
            }
        } else {
            IncludeSummary.IncludeKeyInfo info = ficheFormParameters.getIncludeSummary().getIncludeKeyInfo(includeKey);
            RangeList rangeList = null;
            if (info != null) {
                rangeList = info.getPoidsFilterRangeList();
            }
            poidsList = new ArrayList<Integer>();
            FicheCroisementList abregeFicheList = acces.getFicheCroisement(rangeList);
            int size = abregeFicheList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                AbregeFiche other = abregeFicheList.getAbregeFiche(i);
                abrList.add(other);
                poidsList.add(new Integer(abregeFicheList.getPoids(i)));
            }
        }
    }

}
