/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.*;
import fr.exemole.bdfserver.request.tostring.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.request.formimp.*;


/**
 *
 * @author Vincent Calame
 */
public class PersonneSubfieldBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private PersonneSubfieldBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        PersonneSubfieldBuilder builder = new PersonneSubfieldBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultPersonneSubfieldsElement formElement = new DefaultPersonneSubfieldsElement(corpusField);
        LangConfig langConfig = ficheFormParameters.getBdfServer().getConfigurationManager().getLangConfig();
        formElement.setMandatory(isMandatory);
        formElement.setWithOriginal(langConfig.isWithOriginalName());
        boolean withoutNomAvant = langConfig.isWithoutNomAvant();
        if (withoutNomAvant) {
            withoutNomAvant = !(BdfUserUtils.forceNomAvant(ficheFormParameters.getBdfUser()));
        }
        formElement.setWithoutNomAvant(withoutNomAvant);
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        if (ficheItem != null) {
            if (ficheItem instanceof Personne) {
                Personne personne = (Personne) ficheItem;
                if (personne.isRedacteurSphere()) {
                    SubsetKey defaultSphereKey = null;
                    formElement.setNom(FicheItemToString.toString(personne, defaultSphereKey));
                } else {
                    formElement.setFromPersonneBean(personne.getPersonneBean());
                }
            } else {
                formElement.setNom(FicheItemToString.toString(ficheItem, null));
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
