/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.pointeurs.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.include.*;
import fr.exemole.bdfserver.request.formimp.*;
import fr.exemole.bdfserver.ui.options.*;


/**
 *
 * @author Vincent Calame
 */
public class SpecialIncludeElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private FilteredIncludeKey filteredIncludeKey;
    private IncludeKey includeKey;
    private boolean isMandatory;
    private IncludeUI includeUI;

    private SpecialIncludeElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.filteredIncludeKey = includeUI.getFiteredIncludeKey();
        this.includeKey = filteredIncludeKey.getIncludeKey();
        this.isMandatory = FicheFormImpUtils.isMandatory(includeUI);
        this.includeUI = includeUI;
    }

    public static AbstractIncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        SpecialIncludeElementBuilder builder = new SpecialIncludeElementBuilder(ficheFormParameters, includeUI);
        return builder.toIncludeElement();
    }

    private AbstractIncludeElement toIncludeElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
            Subset masterSubset = fichePointeur.getMasterSubset();
            if (masterSubset != null) {
                SubsetItem masterSubsetItem = fichePointeur.getMasterSubsetItem();
                if (masterSubsetItem == null) {
                    MasterSubsetItemPointeur pointeur = ficheFormParameters.getMasterSubsetItemPointeur();
                    if (pointeur != null) {
                        masterSubsetItem = pointeur.getCurrentSubsetItem();
                    }
                    if (masterSubsetItem == null) {
                        return null;
                    }
                }
                DefautRattachementToMasterElement formElement = new DefautRattachementToMasterElement(filteredIncludeKey);
                formElement.setMasterSubset(masterSubset);
                formElement.setMasterSubsetItem(masterSubsetItem);
                initIntitule(formElement);
                return formElement;
            } else {
                FicheCroisementList[] array = FichePointeurUtils.getRattachementList(fichePointeur);
                DefaultRattachementToSlaveElement formElement = new DefaultRattachementToSlaveElement(filteredIncludeKey);
                formElement.setSlaveAbregeFicheListArray(array);
                initIntitule(formElement);
                return formElement;
            }
        } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
            LimitOption limitOption = includeUI.getLimitOption();
            if (limitOption.isNoNewAllowed()) {
                if (ficheFormParameters.isNewFiche()) {
                    return null;
                }
                DefaultNotEditableAssociationElement formElement = new DefaultNotEditableAssociationElement(filteredIncludeKey);
                Document[] documentArray = fichePointeur.getDocumentArray();
                formElement.setDocumentArray(documentArray);
                initIntitule(formElement);
                return formElement;
            } else {
                DefaultEditableAssociationElement formElement = new DefaultEditableAssociationElement(filteredIncludeKey);
                Document[] documentArray = fichePointeur.getDocumentArray();
                formElement.setDocumentArray(documentArray);
                initIntitule(formElement);
                return formElement;
            }
        } else {
            return null;
        }
    }

    private void initIntitule(AbstractIncludeElement includeElement) {
        FicheFormImpUtils.setIntitule(ficheFormParameters, includeElement, includeUI);
    }

}
