/* BdfServer
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.parsers;

import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.include.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class LiageParser {

    private Map<SubsetKey, DefaultFicheCroisementList> listMap = new HashMap<SubsetKey, DefaultFicheCroisementList>();
    AbregeFiche abregeFiche;
    SubsetKey currentCorpusKey;
    Basedefiches basedefiches;
    private Subset masterSubset;
    private Subset[] slaveSubsetArray;

    public LiageParser(AbregeFiche abregeFiche) {
        this.abregeFiche = abregeFiche;
        currentCorpusKey = abregeFiche.getCorpus().getSubsetKey();
        basedefiches = abregeFiche.getCorpus().getBasedefiches();
        this.masterSubset = abregeFiche.getCorpus().getMasterSubset();
        this.slaveSubsetArray = abregeFiche.getCorpus().getSlaveSubsetArray();
    }

    public void parse(String paramName, String[] tokens) {
        if (paramName.equals(IncludeKey.LIAGE.toString())) {
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                try {
                    FicheToken ficheToken = new FicheToken(token);
                    addFiche(ficheToken);
                } catch (ParseException pe) {
                }
            }
        } else {
            FilteredIncludeKey filteredIncludeKey = null;
            try {
                filteredIncludeKey = FilteredIncludeKey.parse(paramName);
            } catch (ParseException pe) {
                throw new IllegalArgumentException("not a corpus Parameter : " + paramName);
            }
            SubsetKey corpusKey = filteredIncludeKey.getIncludeKey().getSubsetKey();
            if (!corpusKey.isCorpusSubset()) {
                throw new IllegalArgumentException("not a corpus Parameter : " + paramName);
            }
            int poidsFilter = filteredIncludeKey.getPoidsFilter();
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                try {
                    FicheToken ficheToken = new FicheToken(token, corpusKey, poidsFilter);
                    addFiche(ficheToken);
                } catch (ParseException pe) {
                }
            }
        }
    }

    private void addFiche(FicheToken ficheToken) {
        AbregeFiche abregeFiche = getAbregeFiche(ficheToken);
        if (abregeFiche == null) {
            return;
        }
        DefaultFicheCroisementList abregeFicheList = getAbregeFicheList(ficheToken.getCorpusKey());
        abregeFicheList.add(abregeFiche, ficheToken.getPoids());
    }

    private DefaultFicheCroisementList getAbregeFicheList(SubsetKey crpsKey) {
        DefaultFicheCroisementList abregeFicheList = listMap.get(crpsKey);
        if (abregeFicheList == null) {
            abregeFicheList = new DefaultFicheCroisementList(crpsKey, 4);
            listMap.put(crpsKey, abregeFicheList);
        }
        return abregeFicheList;
    }

    private AbregeFiche getAbregeFiche(FicheToken ficheToken) {
        Corpus corpus = (Corpus) basedefiches.getSubset(ficheToken.getCorpusKey());
        if (corpus == null) {
            return null;
        }
        if (isNotEditableCroisement(corpus)) {
            return null;
        }
        return corpus.getAbregeFicheByIdcorpus(ficheToken.getIdcorpus());
    }

    public void save(CroisementEditor croisementEditor) {
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            if (!isNotEditableCroisement(corpus)) {
                FicheCroisementList abregeFicheList = getAbregeFicheList(corpus.getSubsetKey());
                croisementEditor.updateCroisement(abregeFiche, abregeFicheList, CroisementEditor.CLEAR_EXISTING);
            }
        }
    }

    private boolean isNotEditableCroisement(Subset subset) {
        if (masterSubset != null) {
            return (masterSubset.equals(subset));
        }
        if (slaveSubsetArray != null) {
            int length = slaveSubsetArray.length;
            for (int i = 0; i < length; i++) {
                if (slaveSubsetArray[i].equals(subset)) {
                    return true;
                }
            }
            return false;
        }
        return false;
    }


    private class FicheToken {

        private int poids = 1;
        private int idcorpus;
        private SubsetKey corpusKey;

        FicheToken(String s2) throws ParseException {
            PoidsToken poidsToken = PoidsToken.parse(s2);
            poids = poidsToken.getPoids();
            String text = poidsToken.getText();
            int idx = text.indexOf('/');
            if (idx == 0) {
                if (text.length() == 1) {
                    throw new ParseException(s2, 0);
                }
                idx = -1;
                text = text.substring(1);
            }
            if (idx == -1) {
                corpusKey = currentCorpusKey;
                try {
                    idcorpus = Integer.parseInt(text);
                } catch (NumberFormatException nfe) {
                    throw new ParseException(s2, 0);
                }
            } else {
                if (idx == text.length() - 1) {
                    throw new ParseException(s2, 0);
                }
                String corpusSubid = text.substring(0, idx);
                corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusSubid);
                try {
                    idcorpus = Integer.parseInt(text.substring(idx + 1));

                } catch (NumberFormatException nfe) {
                    throw new ParseException(s2, idx);
                }
            }
            if (idcorpus < 0) {
                throw new ParseException(s2, 0);
            }
        }

        FicheToken(String s, SubsetKey corpusKey, int poidsFilter) throws ParseException {
            PoidsToken poidsToken = PoidsToken.parse(s);
            this.corpusKey = corpusKey;
            if (poidsFilter < 1) {
                poids = poidsToken.getPoids();
            } else {
                poids = poidsFilter;
            }
            try {
                idcorpus = Integer.parseInt(poidsToken.getText());
            } catch (NumberFormatException nfe) {
                throw new ParseException(s, 0);
            }
            if (idcorpus < 0) {
                throw new ParseException(s, 0);
            }
        }

        public int getPoids() {
            return poids;
        }

        public int getIdcorpus() {
            return idcorpus;
        }

        public SubsetKey getCorpusKey() {
            return corpusKey;
        }

    }

}
