/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;


/**
 *
 * @author Vincent Calame
 */
public class LineInfo {

    private String line;
    private String className = "";
    private String id = "";

    public LineInfo(String line) {
        this.line = line;
    }

    public boolean isAbsolutlyEmpty() {
        return (line.length() == 0) && (className.length() == 0) && (id.length() == 0);
    }

    public boolean hasIdentifiers() {
        return (className.length() > 0) || (id.length() > 0);
    }

    public String getClassName() {
        return className;
    }

    public String getHtmlId() {
        return id;
    }

    public String getLine() {
        return line;
    }

    public void setLine(String line) {
        this.line = line;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public void setHtmlId(String id) {
        this.id = id;
    }

    public static LineInfo parse(String line) {
        if (line.length() == 0) {
            return new LineInfo("");
        }
        char debut = line.charAt(0);
        if (debut != '[') {
            return new LineInfo(line);
        }
        StringBuffer classNameBuffer = new StringBuffer();
        StringBuffer htmlIdBuffer = new StringBuffer();
        boolean onHtmlId = false;
        for (int i = 1; i < line.length(); i++) {
            char carac = line.charAt(i);
            if (carac == ']') {
                LineInfo lineInfo = new LineInfo(line.substring(i + 1).trim());
                lineInfo.setClassName(classNameBuffer.toString());
                lineInfo.setHtmlId(htmlIdBuffer.toString());
                return lineInfo;
            } else if (carac == '!') {
                if (onHtmlId) {
                    break;
                } else {
                    onHtmlId = true;
                    continue;
                }
            }
            if (testChar(carac)) {
                if (onHtmlId) {
                    htmlIdBuffer.append(carac);
                } else {
                    classNameBuffer.append(carac);
                }
            } else {
                break;
            }
        }
        return new LineInfo(line);
    }

    private static boolean testChar(char carac) {
        if (carac > 'z') {
            return false;
        }
        if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        }
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        }
        if ((carac >= '0') && (carac <= '9')) {
            return true;
        }
        switch (carac) {
            case '-':
            case '_':
            case ' ':
                return true;
            default:
                return false;
        }
    }

}
