/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.parsers.spanparsers.SpanParser;
import fr.exemole.bdfserver.request.TextParser;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class ParagraphParser {

    private SpanParser bracketsParser;
    private SpanParser parenthesisParser;
    private SpanParser emphasisParser;
    private SpanParser httpParser = SpanParser.getHttpParser();
    private SpanParser wwwParser = SpanParser.getWwwParser();
    private StringBuilder currentBuffer;
    private TextContentBuilder currentBuilder;
    private TextParser textParser;

    public ParagraphParser(TextParser textParser) {
        bracketsParser = SpanParser.getBracketsParser(textParser);
        emphasisParser = SpanParser.getEmphasisParser(textParser);
        parenthesisParser = SpanParser.getParenthesisParser(textParser);
        this.textParser = textParser;
    }

    private void flushBuffer() {
        if (currentBuffer.length() > 0) {
            currentBuilder.addText(currentBuffer.toString());
            currentBuffer = new StringBuilder();
        }
    }

    private int parse(SpanParser parser, String token, int i) {
        boolean parseSucceed = parser.parse(token, i);
        if (parseSucceed) {
            flushBuffer();
            currentBuilder.addS(parser.getS());
            return parser.getEndIndex();
        } else {
            int startCharCount = parser.getStartCharCount();
            char carac = parser.getStartChar();
            for (int j = 0; j < startCharCount; j++) {
                currentBuffer.append(carac);
            }
            if (startCharCount > 0) {
                i = i + startCharCount - 1;
            }
            return i;
        }
    }

    public void parseParagraphBlock(ParagraphBlock block, LineInfo lineInfo) {
        block.setClassName(lineInfo.getClassName());
        block.setHtmlId(lineInfo.getHtmlId());
        parseParagraph(block, lineInfo.getLine());
    }

    public void parseParagraph(TextContentBuilder paragraphBuilder, String token) {
        int taille = token.length();
        if (taille == 0) {
            return;
        }
        currentBuilder = paragraphBuilder;
        currentBuffer = new StringBuilder(token.length());
        int longueurS = 0;
        for (int i = 0; i < taille; i++) {
            char carac = token.charAt(i);
            if (carac == '[') {
                i = parse(bracketsParser, token, i);
            } else if (carac == '(') {
                i = parse(parenthesisParser, token, i);
            } else if (carac == '{') {
                i = parse(emphasisParser, token, i);
            } else if (carac == 'h') {
                i = parse(httpParser, token, i);
            } else if (carac == 'w') {
                i = parse(wwwParser, token, i);
            } else {
                try {
                    i = textParser.parseOutsideChar(token, i, currentBuffer);
                } catch (IOException ioe) {
                }
            }
        }
        flushBuffer();
    }

}
