/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class PoidsMotcleToken {

    private boolean idthsBundle = false;
    private int poids = 1;
    private int idths;
    private String text;

    private PoidsMotcleToken(int idths, int poids) {
        this.idths = idths;
        this.poids = poids;
        idthsBundle = true;
    }

    private PoidsMotcleToken(String text, int poids) {
        this.text = text;
        this.poids = poids;
        idthsBundle = false;

    }

    public static PoidsMotcleToken parse(String s, boolean withIdalpha) {
        PoidsToken poidsToken = PoidsToken.parse(s);
        String text = poidsToken.getText();
        int poids = poidsToken.getPoids();
        if (!withIdalpha) {
            try {
                int idths = Integer.parseInt(text);
                return new PoidsMotcleToken(idths, poids);
            } catch (NumberFormatException nfe) {
            }

        }
        if (text.length() == 0) {
            return null;
        } else {
            return new PoidsMotcleToken(text, poids);
        }
    }

    public boolean isIdthsBundle() {
        return idthsBundle;
    }

    public int getPoids() {
        return poids;
    }

    public int getIdths() {
        return idths;
    }

    public void setPoids(int poids) {
        this.poids = poids;
    }

    /**
     * Soit null, soit de longueur non nulle (et débarassé des espaces blancs).
     */
    public String getText() {
        return text;
    }

    public static Motcle getMotcle(PoidsMotcleToken motcleToken, Thesaurus thesaurus, int langInteger) {
        if (motcleToken.isIdthsBundle()) {
            return thesaurus.getMotcleByIdths(motcleToken.getIdths());
        } else if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            return thesaurus.getMotcleByIdalpha(motcleToken.getText());
        } else {
            return thesaurus.getMotcleByLibelle(motcleToken.getText(), langInteger);
        }
    }

}
