/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;


/**
 *
 * @author Vincent Calame
 */
public class PoidsToken {

    private String text;
    private int poids = 0;

    private PoidsToken(String text, int poids) {
        this.text = text;
        this.poids = poids;
    }

    public int getPoids() {
        return poids;
    }

    public String getText() {
        return text;
    }

    /**
     * On suppose s nettoyé avant (trim() et s non vide et non nul).
     */
    static public PoidsToken parse(String s) {
        char lastChar = s.charAt(s.length() - 1);
        if (lastChar == '+') {
            return parsePlus(s);
        } else if (lastChar == '>') {
            return parsePropre(s);
        } else if ((lastChar >= '0') && (lastChar <= '9')) {
            return parseNombre(s);
        } else {
            return new PoidsToken(s, 1);
        }
    }

    static private PoidsToken parseNombre(String s) {
        int dizaine = 1;
        int poids = 0;
        int idx = 0;
        for (int i = (s.length() - 1); i >= 0; i--) {
            char c = s.charAt(i);
            if ((c >= '0') && (c <= '9')) {
                if (i == 1) {
                    return new PoidsToken(s, 1);
                }
                int p = c - 48;
                poids = poids + p * dizaine;
                dizaine = dizaine * 10;
            } else {
                idx = i;
                break;
            }
        }
        for (int i = idx; i >= 1; i--) {
            char c = s.charAt(i);
            if (c == ' ') {
                continue;
            } else if (c == '+') {
                return new PoidsToken(s.substring(0, i).trim(), poids + 1);
            } else if (c == '<') {
                return new PoidsToken(s.substring(0, i).trim(), poids);
            } else {
                break;
            }
        }
        return new PoidsToken(s, 1);
    }

    static private PoidsToken parsePlus(String s) {
        int p = 1;
        int lastIdx = s.length() - 1;
        for (int i = (lastIdx - 1); i >= 0; i--) {
            if (s.charAt(i) == '+') {
                p++;
                lastIdx--;
            } else {
                break;
            }
        }
        return new PoidsToken(s.substring(0, lastIdx).trim(), p + 1);
    }

    static private PoidsToken parsePropre(String s) {
        int idx = s.lastIndexOf('<');
        if (idx == -1) {
            return new PoidsToken(s, 1);
        }
        try {
            int poids = Integer.parseInt(s.substring(idx + 1, s.length() - 1).trim());
            if (poids < 1) {
                poids = 1;
            }
            return new PoidsToken(s.substring(0, idx).trim(), poids);
        } catch (NumberFormatException nfe) {
            return new PoidsToken(s, 1);
        }
    }

}
