/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import java.util.*;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
class TableParser extends ZoneBlockParser {

    private ImportParser importParser = new ImportParser();
    private SyntaxParser syntaxParser;

    TableParser(TextParser textParser) {
        super(textParser);
        syntaxParser = new SyntaxParser(textParser);
    }

    public boolean isStartLine(String line) {
        return line.startsWith("===");
    }

    public void parse(String[] lineArray, int startIndex) {
        setLineArray(lineArray);
        LineInfo lineInfo = getLineInfo(startIndex);
        Table table = new Table();
        addIdentifiersInfo(table, lineInfo);
        char carac = initSeparator(lineInfo.getLine());
        InternalParser internalParser = null;
        if (carac == '0') {
            internalParser = syntaxParser;
        } else {
            importParser.setSeparator(carac);
            internalParser = importParser;
        }
        internalParser.setTable(table);
        internalParser.setHeaderMask(getHeaderMask(lineInfo.getLine()));
        int beginIndex = parseZoneBlockElements(table, startIndex + 1);
        int stop = getLineArrayLength() - 1;
        for (int i = beginIndex; i <= stop; i++) {
            String line = getLine(i);
            if (line.trim().startsWith("===")) {
                stop = i;
                break;
            } else {
                internalParser.parseLine(line);
            }
        }
        internalParser.flush();
        setParseResult(stop, table);
    }

    private int getHeaderMask(String s) {
        int headerMask = 0;
        if (s.indexOf('h') != -1) {
            headerMask = headerMask | Table.HORIZONTALHEADER_BITVALUE;
        }
        if (s.indexOf('v') != -1) {
            headerMask = headerMask | Table.VERTICALHEADER_BITVALUE;
        }
        return headerMask;
    }

    private char initSeparator(String s) {
        for (int i = 0; i < s.length(); i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case 't':
                    return '\t';
                case ',':
                    return ',';
                case ';':
                    return ';';
                case ':':
                    return ':';
                case '|':
                    return '|';
                case '*':
                    return '*';
            }
        }
        return '0';
    }


    private abstract class InternalParser {

        Table table;
        int headerMask;
        int rowIndex;

        public abstract void parseLine(String line);

        public abstract void flush();

        public abstract void reinit();

        public void setTable(Table table) {
            this.table = table;
            this.rowIndex = 0;
            reinit();
        }

        public void setHeaderMask(int headerMask) {
            this.headerMask = headerMask;
        }

        protected Td createTd(int colIndex) {
            if ((rowIndex == 0) && ((headerMask & Table.HORIZONTALHEADER_BITVALUE) != 0)) {
                return Td.getHeaderTdInstance();
            }
            if ((colIndex == 0) && ((headerMask & Table.VERTICALHEADER_BITVALUE) != 0)) {
                return Td.getHeaderTdInstance();
            }
            return new Td();
        }

        protected void addRow(Tr tr) {
            table.addTr(tr);
            rowIndex++;
        }

    }


    private class ImportParser extends InternalParser {

        char separator = '\t';

        public void parseLine(String line) {
            if (line.length() == 0) {
                return;
            }
            Tr tr = new Tr();
            StringBuffer buf = new StringBuffer();
            int q = 0;
            for (int i = 0; i < line.length(); i++) {
                char c = line.charAt(i);
                if (c == separator) {
                    Td td = createTd(q);
                    td.addText(StringUtils.cleanString(buf.toString()));
                    tr.addTd(td);
                    buf = new StringBuffer();
                    q++;
                } else {
                    buf.append(c);
                }
            }
            String result = StringUtils.cleanString(buf.toString());
            if (result.length() == 0) {
                if (tr.getFicheBlockCount() == 0) {
                    return;
                }
            } else {
                Td td = createTd(q);
                td.addText(result);
                tr.addTd(td);
            }
            addRow(tr);
        }

        public void reinit() {
        }

        public void setSeparator(char separator) {
            this.separator = separator;
        }

        public void flush() {
        }

    }


    private class SyntaxParser extends InternalParser {

        ParagraphParser parser;
        Tr currentRow;
        int tdIndex = 0;
        LineInfo potentialTrRow;

        SyntaxParser(TextParser textParser) {
            parser = new ParagraphParser(textParser);
        }

        public void parseLine(String line) {
            line = StringUtils.cleanString(line);
            LineInfo lineInfo = LineInfo.parse(line);
            line = lineInfo.getLine();
            if (line.length() == 0) {
                flush();
                potentialTrRow = lineInfo;
            } else {
                if (line.equals("-")) {
                    lineInfo.setLine("");
                }
                if (currentRow == null) {
                    currentRow = new Tr();
                    if (potentialTrRow != null) {
                        currentRow.setClassName(potentialTrRow.getClassName());
                        currentRow.setHtmlId(potentialTrRow.getHtmlId());
                    }
                }
                if (line.equals("=")) {
                    int size = currentRow.getFicheBlockCount();
                    if (size == 0) {
                        Td td = createTd(tdIndex);
                        tdIndex++;
                        currentRow.addTd(td);
                    } else {
                        Td previous = currentRow.getTd(size - 1);
                        previous.setColSpan(previous.getColSpan() + 1);
                    }
                } else {
                    int rowspan = parseRowSpan(lineInfo);
                    Td td = createTd(tdIndex);
                    td.setRowSpan(rowspan);
                    tdIndex++;
                    parseParagraph(td, lineInfo);
                    currentRow.addTd(td);
                }
            }
        }

        private int parseRowSpan(LineInfo lineInfo) {
            String line = lineInfo.getLine();
            int length = line.length();
            if (length == 0) {
                return 1;
            }
            if (line.charAt(length - 1) != ']') {
                return 1;
            }
            int plus = 0;
            int stop = -1;
            for (int i = length - 2; i > 0; i--) {
                char carac = line.charAt(i);
                if ((carac >= '0') && (carac <= '9')) {
                    plus = plus + (((int) carac) - 48) * ((int) Math.pow(10, length - 2 - i));
                } else if (carac == '+') {
                    if (line.charAt(i - 1) == '[') {
                        stop = i - 1;
                        if (i > 1) {
                            if (line.charAt(i - 2) == '\\') {
                                return 1;
                            }
                        }
                        break;
                    } else {
                        return 1;
                    }
                } else {
                    return 1;
                }
            }
            if (stop == -1) {
                return 1;
            }
            lineInfo.setLine(line.substring(0, stop).trim());
            return 1 + plus;
        }

        public void flush() {
            if (currentRow != null) {
                addRow(currentRow);
                currentRow = null;
                potentialTrRow = null;
                tdIndex = 0;
            }
        }

        public void reinit() {
            currentRow = null;
            tdIndex = 0;
        }

    }

}
