/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.tostring;

import net.mapeadores.basedefiches.plainexport.SeparatorOptionsBean;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.InputListPrinter;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;


/**
 *
 * @author Vincent Calame
 */
public class IndexationWriter {

    HtmlPrinter htmlPrinter;
    SeparatorOptionsBean separatorOptions = new SeparatorOptionsBean(';', true, true);
    int workingLangInteger;
    Thesaurus currentThesaurus;
    MotcleCroisementList currentMotcleList;
    private Set<Motcle> currentMotcleSet;
    private String idalphaSeparator = " – ";
    private boolean ignorePoids = false;
    private boolean isBabelien;
    private boolean withIdalpha;
    private int langInteger;

    public IndexationWriter(HtmlPrinter htmlPrinter, int workingLangInteger) {
        this.workingLangInteger = workingLangInteger;
        this.htmlPrinter = htmlPrinter;
    }

    public void setCurrentThesaurus(Thesaurus currentThesaurus) {
        this.currentThesaurus = currentThesaurus;
        isBabelien = ThesaurusUtils.isBabelienType(currentThesaurus);
        withIdalpha = ThesaurusUtils.isIdalphaType(currentThesaurus);
        langInteger = ThesaurusUtils.checkLangIntegerDisponibility(currentThesaurus, workingLangInteger);
    }

    public void setCurrentMotcleList(MotcleCroisementList motcleList) {
        if ((motcleList != null) && (motcleList.getSubsetItemCount() == 0)) {
            currentMotcleList = null;
        } else {
            this.currentMotcleList = motcleList;
        }
        this.currentMotcleSet = null;
    }

    public int getMotcleCount() {
        if (currentMotcleList == null) {
            return 0;
        }
        return currentMotcleList.getSubsetItemCount();
    }

    public int getThesaurusSize() {
        return currentThesaurus.size();
    }

    public void setIgnorePoids(boolean ignorePoids) {
        this.ignorePoids = ignorePoids;
    }

    public String getMotcleLisToString() {
        if (currentMotcleList == null) {
            return "";
        }
        int size = currentMotcleList.getSubsetItemCount();
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                appendSeparator(buf);
            }
            appendMotcle(buf, currentMotcleList.getMotcle(i));
            if (!ignorePoids) {
                appendPoids(buf, currentMotcleList.getPoids(i));
            }
        }
        if (separatorOptions.isLastInclude()) {
            appendSeparator(buf);
        }
        return buf.toString();
    }

    private void appendSeparator(StringBuffer stringBuffer) {
        stringBuffer.append(separatorOptions.getSeparator());
        if (separatorOptions.isSpaceInclude()) {
            stringBuffer.append(' ');
        }
    }

    private void appendMotcle(StringBuffer buf, Motcle motcle) {
        if (withIdalpha) {
            buf.append(motcle.getIdalpha());
        } else {
            String lib = motcle.getLibelleString(langInteger, false);
            if ((lib == null) || (lib.length() == 0)) {
                if (!isBabelien) {
                    lib = String.valueOf(motcle.getIdths());
                } else {
                    lib = motcle.getLibelleString(0, true);
                }
            }
            buf.append(lib);
        }
    }

    private void appendPoids(StringBuffer stringBuffer, int poids) {
        if (poids > 1) {
            if (poids == 2) {
                stringBuffer.append('+');
            } else if (poids == 3) {
                stringBuffer.append("++");
            } else if (poids == 4) {
                stringBuffer.append("+++");
            } else {
                stringBuffer.append(" <");
                stringBuffer.append(poids);
                stringBuffer.append('>');
            }
        }
    }

    public void printMotcleList(InputListPrinter inputListPrinter) {
        Set<Motcle> selectedMotcleSet = getMotcleSet();
        short valueType = (selectedMotcleSet.isEmpty()) ? InputListPrinter.VALUE_NO : InputListPrinter.VALUE_YES;
        inputListPrinter.start(valueType);
        Motcle motcle = ThesaurusUtils.getFirst(currentThesaurus);
        while (motcle != null) {
            String valcourant = null;
            if (withIdalpha) {
                valcourant = motcle.getIdalpha();
            } else {
                valcourant = String.valueOf(motcle.getIdths());
            }
            boolean selected = selectedMotcleSet.contains(motcle);
            String lib = motcle.getLibelleString(workingLangInteger, false);
            if ((lib == null) || (lib.length() == 0)) {
                if (isBabelien) {
                    lib = motcle.getLibelleString(0, true);
                } else if (withIdalpha) {
                    lib = motcle.getIdalpha();
                } else {
                    lib = String.valueOf(motcle.getIdths());
                }
            } else if (withIdalpha) {
                lib = motcle.getIdalpha() + idalphaSeparator + lib;
            }
            inputListPrinter.add(valcourant, lib, motcle.getLevel(), selected);
            motcle = ThesaurusUtils.getNext(motcle);
        }
        inputListPrinter.end();
    }

    private Set getMotcleSet() {
        if (currentMotcleSet == null) {
            if (currentMotcleList == null) {
                currentMotcleSet = Collections.emptySet();
            } else {
                int size = currentMotcleList.getSubsetItemCount();
                currentMotcleSet = new HashSet<Motcle>();
                for (int i = 0; i < size; i++) {
                    currentMotcleSet.add(currentMotcleList.getMotcle(i));
                }
            }
        }
        return currentMotcleSet;
    }

}
