/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.roles;

import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.ui.SubsetGrouping;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.RedacteurKey;


/**
 *
 * @author Vincent Calame
 */
public class RoleAndSubsetUtils {

    private RoleAndSubsetUtils() {
    }

    public static SubsetGrouping checkCorpusTree(BdfUser bdfUser, SubsetGrouping subsetGrouping) {
        if ((bdfUser == null) || BdfUserUtils.isAdmin(bdfUser)) {
            return subsetGrouping;
        }
        CheckedSubsetGrouping checked = new CheckedSubsetGrouping(bdfUser, subsetGrouping);
        return checked;

    }


    private static class CheckedSubsetGrouping implements SubsetGrouping {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();
        private String[] roleNameArray;
        private PermissionManager permissionManager;
        private Basedefiches basedefiches;
        private BdfUser bdfUser;
        private RedacteurKey redacteurKey;

        CheckedSubsetGrouping(BdfUser bdfUser, SubsetGrouping subsetGrouping) {
            this.bdfUser = bdfUser;
            this.redacteurKey = bdfUser.getRedacteurKey();
            permissionManager = bdfUser.getBdfServer().getPermissionManager();
            basedefiches = bdfUser.getBdfServer().getBasedefiches();
            roleNameArray = bdfUser.getRoleNameArray();
            for (SubsetKeyGroup subsetKeyGroup : subsetGrouping.getSubsetKeyGroupList()) {
                SubsetKeyGroup newGroup = check(subsetKeyGroup);
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public List<SubsetKeyGroup> getSubsetKeyGroupList() {
            return groupList;
        }

        public SubsetKeyGroup check(SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                ReadFiche readFiche = FicheAction.getReadFiche(corpus);
                if (!permissionManager.cannotDo(redacteurKey, roleNameArray, readFiche)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }

}
