/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.roles.xml;

import fr.exemole.bdfserver.roles.CorpusPermission;
import fr.exemole.bdfserver.roles.FicheActionPermission;
import fr.exemole.bdfserver.roles.Role;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.TagUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class RoleDOMReader {

    private Role currentRole;
    private ErrorHandler errorHandler;

    public RoleDOMReader(ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    public void readRole(Role role, Element element_xml) {
        this.currentRole = role;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib-list")) {
                    readLibelleList(element);
                } else if (tagName.equals("corpus-permission")) {
                    readCorpusPermission(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        this.currentRole = null;
    }

    private void readLibelleList(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle lib = TagUtils.getLibelle(element);
                        currentRole.setLibelle(lib.getLibelleString(), lib.getLangInteger());
                    } catch (LangIntegerException ile) {
                        errorHandler.wrongLangAttributeError("lib", element.getAttribute("xml:lang"));
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readCorpusPermission(Element element_xml) {
        String corpusSubId = element_xml.getAttribute("corpus");
        SubsetKey corpusKey = null;
        try {
            corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusSubId);
        } catch (java.text.ParseException pe) {
            return;
        }
        CorpusPermission corpusPermission = new CorpusPermission();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("create-fiche")) {
                    corpusPermission.setCreateFichePermission(true);
                } else if (tagName.equals("read-fiche")) {
                    readFicheActionPermission(corpusPermission.getReadFichePermission(), element);
                } else if (tagName.equals("write-fiche")) {
                    readFicheActionPermission(corpusPermission.getWriteFichePermission(), element);
                } else if (tagName.equals("send-fiche")) {
                    readFicheActionPermission(corpusPermission.getSendFichePermission(), element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        currentRole.setCorpusPermission(corpusKey, corpusPermission);
    }

    private void readFicheActionPermission(FicheActionPermission ficheActionPermission, Element element_xml) {
        String typeString = element_xml.getAttribute("type");
        if (typeString.length() == 0) {
            errorHandler.emptyTypeAttributeError(element_xml.getTagName());
            return;
        }
        try {
            short type = FicheActionPermission.typeToShort(typeString);
            ficheActionPermission.setType(type);
        } catch (IllegalArgumentException iae) {
            errorHandler.wrongTypeSyntaxError(element_xml.getTagName(), typeString);
        }
    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void wrongLangAttributeError(String tagName, String lang);

        public void emptyTypeAttributeError(String tagName);

        public void wrongTypeSyntaxError(String tagName, String type);

    }

}
