/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.sqlexport.impl;

import net.mapeadores.basedefiches.sqlexport.beans.*;
import fr.exemole.bdfserver.sqlexport.*;
import fr.exemole.bdfserver.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultSqlExportManager implements SqlExportManager {

    private BdfServer bdfServer;
    private List<SqlExportBean> beanList = new ArrayList<SqlExportBean>();
    private TreeMap<String, SqlExportBean> sqlExportMap = new TreeMap<String, SqlExportBean>();

    public DefaultSqlExportManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public int getSqlExportCount() {
        return beanList.size();
    }

    public String getSqlExportName(int i) {
        return beanList.get(i).getName();
    }

    public SqlExportBean getSqlExportBean(int i) {
        return new SqlExportBean(beanList.get(i));
    }

    public void removeSqlExport(String name) {
        int idx = indexOf(name);
        if (idx != -1) {
            beanList.remove(idx);
        }
    }

    public SqlExportBean getSqlExportBeanByName(String name) {
        int idx = indexOf(name);
        if (idx == -1) {
            return null;
        }
        return new SqlExportBean(beanList.get(idx));
    }

    public void putSqlExportBean(SqlExportBean sqlExportBean) {
        SqlExportBean newBean = new SqlExportBean(sqlExportBean);
        int size = beanList.size();
        if (size == 0) {
            beanList.add(newBean);
            return;
        }
        String name = sqlExportBean.getName();
        for (int i = 0; i < size; i++) {
            String currentName = beanList.get(i).getName();
            int comp = currentName.compareTo(name);
            if (comp == 0) {
                beanList.set(i, newBean);
                return;
            } else if (comp > 0) {
                beanList.add(i, newBean);
                return;
            }
        }
        beanList.add(newBean);
    }

    private int indexOf(String name) {
        for (int i = 0; i < beanList.size(); i++) {
            if (beanList.get(i).getName().equals(name)) {
                return i;
            }
        }
        return -1;
    }

}
