/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;


/**
 *
 * @author Vincent Calame
 */
public class SubsetGroupingUtils {

    private SubsetGroupingUtils() {
    }

    public static SubsetGrouping checkCorpusTree(BdfUser bdfUser, SubsetGrouping subsetGrouping) {
        if ((bdfUser == null) || BdfUserUtils.isAdmin(bdfUser)) {
            return subsetGrouping;
        }
        CheckedSubsetGrouping checked = new CheckedSubsetGrouping(bdfUser, subsetGrouping);
        return checked;

    }

    public static SubsetGrouping checkNoSlaveSubset(Basedefiches basedefiches, SubsetGrouping subsetGrouping) {
        NoSlaveSubsetGrouping noSlaveSubsetGrouping = new NoSlaveSubsetGrouping();
        noSlaveSubsetGrouping.init(basedefiches, subsetGrouping);
        return noSlaveSubsetGrouping;
    }

    public static SubsetGrouping chekcNoRattachementSubset(Corpus corpus, SubsetGrouping subsetGrouping) {
        Set<SubsetKey> rattachementSet = CorpusUtils.getRattachementSet(corpus);
        if (rattachementSet.size() == 0) {
            return subsetGrouping;
        }
        ExclusionSubsetGrouping noRattachementSubsetGrouping = new ExclusionSubsetGrouping();
        noRattachementSubsetGrouping.init(rattachementSet, subsetGrouping);
        return noRattachementSubsetGrouping;
    }


    private static class ExclusionSubsetGrouping implements SubsetGrouping {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();

        private ExclusionSubsetGrouping() {
        }

        public List<SubsetKeyGroup> getSubsetKeyGroupList() {
            return groupList;
        }

        private void init(Set<SubsetKey> exclusionSet, SubsetGrouping subsetGrouping) {
            for (SubsetKeyGroup subsetKeyGroup : subsetGrouping.getSubsetKeyGroupList()) {
                SubsetKeyGroup newGroup = check(exclusionSet, subsetKeyGroup);
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public SubsetKeyGroup check(Set<SubsetKey> exclusionSet, SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                if (!exclusionSet.contains(subsetKey)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }


    private static class NoSlaveSubsetGrouping implements SubsetGrouping {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();

        private NoSlaveSubsetGrouping() {
        }

        public List<SubsetKeyGroup> getSubsetKeyGroupList() {
            return groupList;
        }

        private void init(Basedefiches basedefiches, SubsetGrouping subsetGrouping) {
            for (SubsetKeyGroup subsetKeyGroup : subsetGrouping.getSubsetKeyGroupList()) {
                SubsetKeyGroup newGroup = check(basedefiches, subsetKeyGroup);
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public SubsetKeyGroup check(Basedefiches basedefiches, SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                Subset subset = basedefiches.getSubset(subsetKey);
                if ((subset != null) && (subset.getMasterSubset() == null)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }


    private static class CheckedSubsetGrouping implements SubsetGrouping {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();
        private String[] roleNameArray;
        private PermissionManager permissionManager;
        private Basedefiches basedefiches;
        private BdfUser bdfUser;
        private RedacteurKey redacteurKey;

        CheckedSubsetGrouping(BdfUser bdfUser, SubsetGrouping subsetGrouping) {
            this.bdfUser = bdfUser;
            this.redacteurKey = bdfUser.getRedacteurKey();
            permissionManager = bdfUser.getBdfServer().getPermissionManager();
            basedefiches = bdfUser.getBdfServer().getBasedefiches();
            roleNameArray = bdfUser.getRoleNameArray();
            for (SubsetKeyGroup subsetKeyGroup : subsetGrouping.getSubsetKeyGroupList()) {
                SubsetKeyGroup newGroup = check(subsetKeyGroup);
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public List<SubsetKeyGroup> getSubsetKeyGroupList() {
            return groupList;
        }

        public SubsetKeyGroup check(SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                ReadFiche readFiche = FicheAction.getReadFiche(corpus);
                if (!permissionManager.cannotDo(redacteurKey, roleNameArray, readFiche)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }

}
