/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyTreeUtils {

    private SubsetKeyTreeUtils() {
    }

    public static String[] toTokens(SubsetKeyTree subsetKeyTree) {
        List<String> list = new ArrayList<String>();
        for (SubsetKeyGroup group : subsetKeyTree.getSubsetKeyGroupList()) {
            boolean noneGroup = group.isNoTemplateGroup();
            if (!noneGroup) {
                list.add("grp_" + group.getTemplateName());
            }
            for (SubsetKey subsetKey : group.getSubsetKeyList()) {
                list.add(subsetKey.toString());
            }
            if (!noneGroup) {
                list.add("---");
            }
        }
        int size = list.size();
        String[] tokens = new String[size];
        for (int i = 0; i < size; i++) {
            tokens[i] = (String) list.get(i);
        }
        return tokens;
    }

    public static void parseTokens(SubsetKeyTree subsetKeyTree, String[] tokens) {
        PositionParser positionParser = new PositionParser(subsetKeyTree, tokens);
        positionParser.parse();
    }


    private static class PositionParser {

        String[] tokens;
        String currentGroupName = null;
        SubsetKeyTree subsetKeyTree;

        PositionParser(SubsetKeyTree subsetKeyTree, String[] tokens) {
            this.tokens = tokens;
            this.subsetKeyTree = subsetKeyTree;
        }

        void parse() {
            for (int i = 0; i < tokens.length; i++) {
                updatePosition(tokens[i]);
            }
        }

        private void updatePosition(String token) {
            if (token == null) {
                return;
            }
            token = token.trim();
            if (token.length() == 0) {
                return;
            }
            try {
                SubsetKey subsetKey = SubsetKey.parse(token);
                updatePosition(subsetKey);
            } catch (ParseException pe) {
                checkGroup(token);
            }
        }

        private void checkGroup(String token) {
            if (token.startsWith("---")) {
                currentGroupName = null;
            } else if (token.startsWith("grp_")) {
                String name = token.substring(4);
                if (subsetKeyTree.getSubsetKeyGroupTemplates().containsGroupTemplate(name)) {
                    currentGroupName = name;
                }
            }
        }

        private void updatePosition(SubsetKey subsetKey) {
            if (subsetKeyTree.isLegalSubsetKey(subsetKey)) {
                subsetKeyTree.addSubsetKey(subsetKey, currentGroupName);
            }
        }

    }

}
