/* BdfServer
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.ui.corpus;

import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;
import java.text.ParseException;
import java.util.*;
import net.mapeadores.basedefiches.include.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUIUtils {

    static final InputOption DEFAULT_PROPRIETE = new DefaultInputOption(InputOption.WIDTH_MEDIUM, 1, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_ANNEXE = new DefaultInputOption(InputOption.WIDTH_LARGE, 8, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_INFORMATION = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_TITRE = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_REDACTEURS = new DefaultInputOption(InputOption.WIDTH_SMALL, 1, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_TEXTE = new DefaultInputOption(InputOption.WIDTH_LARGE, 15, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_INDEXATION = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    static final InputOption DEFAULT_LANG = new DefaultInputOption(InputOption.WIDTH_SMALL, 1, InputOption.INPUT_SELECT);

    private CorpusUIUtils() {
    }

    public static MutableInputOption getDefaultInputOption(FieldKey fieldKey) {
        if (fieldKey.isProprieteFieldKey()) {
            return new MutableInputOption(DEFAULT_PROPRIETE);
        }
        if (fieldKey.isAnnexeFieldKey()) {
            return new MutableInputOption(DEFAULT_ANNEXE);
        }
        if (fieldKey.isInformationFieldKey()) {
            return new MutableInputOption(DEFAULT_INFORMATION);
        }
        if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return new MutableInputOption(DEFAULT_TITRE);
        }
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return new MutableInputOption(DEFAULT_REDACTEURS);
        }
        if (fieldKey.equals(FieldKey.TEXTE)) {
            return new MutableInputOption(DEFAULT_TEXTE);
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return new MutableInputOption(DEFAULT_LANG);
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            throw new IllegalArgumentException("No InputOption for FieldKey.IDCORPUS");
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

    /* Peut-être nul */
    public static MutableValueOption getDefaultValueOption(FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return null;
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return null;
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            return null;
        }
        return new MutableValueOption();
    }

    public static boolean equalsToDefault(FieldKey fieldKey, InputOption inputOption) {
        if (fieldKey.isProprieteFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_PROPRIETE);
        }
        if (fieldKey.isAnnexeFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_ANNEXE);
        }
        if (fieldKey.isInformationFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_INFORMATION);
        }
        if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return inputOptionEquality(inputOption, DEFAULT_TITRE);
        }
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return inputOptionEquality(inputOption, DEFAULT_REDACTEURS);
        }
        if (fieldKey.equals(FieldKey.TEXTE)) {
            return inputOptionEquality(inputOption, DEFAULT_TEXTE);
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return inputOptionEquality(inputOption, DEFAULT_LANG);
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            throw new IllegalArgumentException("No htmlInputBean for FieldKey.IDCORPUS");
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

    public static MutableInputOption getDefaultIndexationInputOption() {
        return new MutableInputOption(DEFAULT_INDEXATION);
    }

    public static MutableLimitOption getDefaultSpecialLiageLimitOption() {
        MutableLimitOption mutableLimitOption = new MutableLimitOption();
        return mutableLimitOption;
    }

    public static MutableLimitOption getDefaultSpecialAssociationLimitOption() {
        MutableLimitOption mutableLimitOption = new MutableLimitOption();
        mutableLimitOption.setNoNewAllowed(true);
        return mutableLimitOption;
    }

    public static boolean inputOptionEquality(InputOption inputOption1, InputOption inputOption2) {
        if (inputOption1.getInputType() != inputOption2.getInputType()) {
            return false;
        }
        if (inputOption1.getRows() != inputOption2.getRows()) {
            return false;
        }
        if (inputOption1.getWidthType() != inputOption2.getWidthType()) {
            return false;
        }
        return true;
    }

    public static boolean equalstoDefaultIndexation(InputOption inputOption) {
        return inputOptionEquality(inputOption, DEFAULT_INDEXATION);
    }

    public static CorpusUI getCorpusUI(UIManager bdfUI, CorpusMetadata corpusMetadata) {
        return bdfUI.getCorpusUI(corpusMetadata.getCorpus());
    }

    public static boolean isCommentUIKey(String key) {
        return (key.startsWith("comment_"));
    }

    public static boolean isIndexationUIKey(String key) {
        return (key.startsWith("thesaurus_"));
    }

    public static boolean isLiageUIKey(String key) {
        return (key.startsWith("corpus_"));
    }

    public static FieldKey isFieldKey(String key) {
        try {
            FieldKey fieldKey = FieldKey.parse(key);
            return fieldKey;
        } catch (ParseException pe) {
            return null;
        }
    }

    public static IncludeSummary getFilteredIncludeSummary(CorpusUI corpusUI) {
        IncludeSummaryBuilder builder = new IncludeSummaryBuilder();
        analyse(builder, corpusUI, CorpusUI.INDEXATION_INCLUDEUI_LIST);
        analyse(builder, corpusUI, CorpusUI.LIAGE_INCLUDEUI_LIST);
        return builder.getIncludeSummary();
    }

    private static void analyse(IncludeSummaryBuilder builder, CorpusUI corpusUI, int listIndex) {
        List includeUIList = corpusUI.getOrderedComponentUIList(listIndex);
        for (Iterator it = includeUIList.iterator(); it.hasNext();) {
            IncludeUI includeUI = (IncludeUI) it.next();
            builder.append(includeUI.getFiteredIncludeKey());
        }
    }

}
