/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommentUI extends AbstractLibelleHolder implements CommentUI {

    private String name;
    private int location = ALL_MASK;

    public DefaultCommentUI(String name) {
        this.name = name;
    }

    public DefaultCommentUI(CommentUI commentUI) {
        this.name = commentUI.getName();
        this.location = commentUI.getLocation();
        int libelleCount = getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = getLibelleByIndex(i);
            internalSetLibelleString(libelle.getLangInteger(), libelle.getLibelleString());
        }
    }

    public int getLocation() {
        return location;
    }

    public String getName() {
        return name;
    }

    public String getKeyString() {
        return "comment_" + name;
    }

    protected boolean internalSetLocation(int location) {
        if ((location < 1) || (location > ALL_MASK)) {
            location = ALL_MASK;
        }
        if (this.location == location) {
            return false;
        }
        this.location = location;
        return true;
    }

    protected boolean internalSetLibelleString(int langInteger, String nvlibelle) {
        return setLibelle(nvlibelle, langInteger);
    }

    public static String maskToString(int position) {
        MaskBuffer maskBuffer = new MaskBuffer(position);
        maskBuffer.append(FORM_BITVALUE, "form");
        maskBuffer.append(AFFICHAGE_BITVALUE, "affichage");
        return maskBuffer.toString();
    }

    public static int maskStringToInt(String s) {
        int location = 0;
        String[] array = StringUtils.getTokens(s, '+', StringUtils.EMPTY_EXCLUDE);
        for (int i = 0; i < array.length; i++) {
            String token = array[i];
            if (token.equals("form")) {
                location = location | FORM_BITVALUE;
            } else if (token.equals("affichage")) {
                location = location | AFFICHAGE_BITVALUE;
            }
        }
        if (location == 0) {
            location = ALL_MASK;
        }
        return location;
    }


    private static class MaskBuffer {

        StringBuffer buf = new StringBuffer();
        int location;
        boolean premier = true;

        MaskBuffer(int location) {
            this.location = location;
        }

        void append(int mask, String maskString) {
            if ((location & mask) != 0) {
                if (premier) {
                    premier = false;
                } else {
                    buf.append("+");
                }
                buf.append(maskString);
            }
        }

        public String toString() {
            return buf.toString();
        }

    }

}
