/* BdfServer
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.ui.corpus;

import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.options.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFieldUI implements FieldUI {

    private CorpusField corpusField;
    private InternalInputOption internalInputOption;
    private InternalValueOption internalValueOption;

    public DefaultFieldUI(CorpusField corpusField) {
        this.internalInputOption = new InternalInputOption(CorpusUIUtils.getDefaultInputOption(corpusField.getFieldKey()));
        this.corpusField = corpusField;
        ValueOption defValueOption = CorpusUIUtils.getDefaultValueOption(corpusField.getFieldKey());
        if (defValueOption == null) {
            this.internalValueOption = null;
        } else {
            this.internalValueOption = new InternalValueOption(defValueOption);
        }
    }

    public DefaultFieldUI(FieldUI fieldUI) {
        this.corpusField = fieldUI.getCorpusField();
        this.internalInputOption = new InternalInputOption(fieldUI.getInputOption());
        ValueOption valueOption = fieldUI.getValueOption();
        if (valueOption != null) {
            this.internalValueOption = new InternalValueOption();
        } else {
            this.internalValueOption = null;
        }
    }

    public CorpusField getCorpusField() {
        return corpusField;
    }

    public InputOption getInputOption() {
        return internalInputOption;
    }

    public ValueOption getValueOption() {
        return internalValueOption;
    }

    public String getKeyString() {
        return corpusField.getFieldKey().toString();
    }

    protected void internalUpdateInputOption(InputOption inputOption) {
        internalInputOption.update(inputOption);
    }

    protected void internalUpdateValueOption(ValueOption valueOption) {
        if (internalValueOption != null) {
            internalValueOption.update(valueOption);
        }
    }


    private class InternalInputOption extends DefaultInputOption {

        InternalInputOption(InputOption inputOption) {
            super(inputOption);
        }

        private void update(InputOption inputOption) {
            innerUpdate(inputOption);
        }

    }


    private class InternalValueOption extends DefaultValueOption {

        InternalValueOption() {
            innerSetMandatory(false);
            innerSetDefaultValue("");
        }

        InternalValueOption(ValueOption valueOption) {
            super(valueOption);
        }

        private void update(ValueOption valueOption) {
            innerUpdate(valueOption);
        }

    }

}
