/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.DefaultLimitOption;
import fr.exemole.bdfserver.ui.options.DefaultPiocheSelectOption;
import fr.exemole.bdfserver.ui.options.DefaultValueOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.PiocheSelectOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.text.AbstractLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class DefaultIncludeUI implements IncludeUI {

    private FilteredIncludeKey filteredIncludeKey;
    private boolean defaultLibelle = true;
    private boolean withIdalpha = true;
    private CustomLibelleHolder customLibelleHolder;
    private InternalInputOption internalInputOption;
    private InternalLimitOption internalLimitOption;
    private InternalValueOption internalValueOption;
    private InternalPiocheSelectOption internalPiocheSelectOption;

    public DefaultIncludeUI(IncludeKey includeKey) {
        this.filteredIncludeKey = new FilteredIncludeKey(includeKey);
        init();
    }

    public DefaultIncludeUI(FilteredIncludeKey filteredIncludeKey) {
        this.filteredIncludeKey = filteredIncludeKey;
        init();
    }

    private void init() {
        IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            internalInputOption = new InternalInputOption(CorpusUIUtils.getDefaultIndexationInputOption());
        }
        if (includeKey.isLiageIncludeKey()) {
            internalPiocheSelectOption = new InternalPiocheSelectOption();
        }
        if (includeKey.equals(IncludeKey.LIAGE)) {
            internalLimitOption = new InternalLimitOption(CorpusUIUtils.getDefaultSpecialLiageLimitOption());
            internalPiocheSelectOption = new InternalPiocheSelectOption();
        }
        if (includeKey.equals(IncludeKey.ASSOCIATION)) {
            internalLimitOption = new InternalLimitOption(CorpusUIUtils.getDefaultSpecialAssociationLimitOption());
        }
        this.internalValueOption = new InternalValueOption();
    }

    public DefaultIncludeUI(IncludeUI includeUI) {
        this.filteredIncludeKey = includeUI.getFiteredIncludeKey();
        this.internalValueOption = new InternalValueOption(includeUI.getValueOption());
        LimitOption limitOption = includeUI.getLimitOption();
        if (limitOption != null) {
            this.internalLimitOption = new InternalLimitOption(limitOption);
        } else {
            this.internalLimitOption = null;
        }
        InputOption inputOption = includeUI.getInputOption();
        if (inputOption != null) {
            this.internalInputOption = new InternalInputOption(inputOption);
        } else {
            this.internalInputOption = null;
        }
        PiocheSelectOption piocheSelectOption = includeUI.getPiocheSelectOption();
        if (piocheSelectOption != null) {
            this.internalPiocheSelectOption = new InternalPiocheSelectOption(piocheSelectOption);
        } else {
            this.internalPiocheSelectOption = null;
        }
        boolean otherUsage = includeUI.useDefaultLibelle();
        this.defaultLibelle = otherUsage;
        if (!otherUsage) {
            customLibelleHolder = new CustomLibelleHolder();
            LibelleHolder holder = includeUI.getCustomLibelle();
            int libelleCount = holder.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = holder.getLibelleByIndex(i);
                customLibelleHolder.internalSetLibelle(libelle.getLibelleString(), libelle.getLangInteger());
            }
        }
    }

    public InputOption getInputOption() {
        return internalInputOption;
    }

    public LimitOption getLimitOption() {
        return internalLimitOption;
    }

    public ValueOption getValueOption() {
        return internalValueOption;
    }

    public PiocheSelectOption getPiocheSelectOption() {
        return internalPiocheSelectOption;
    }

    public String getKeyString() {
        return filteredIncludeKey.toString();
    }

    public LibelleHolder getCustomLibelle() {
        return customLibelleHolder;
    }

    public FilteredIncludeKey getFiteredIncludeKey() {
        return filteredIncludeKey;
    }

    public boolean useDefaultLibelle() {
        return defaultLibelle;
    }

    public boolean prefixWithIdalpha() {
        return withIdalpha;
    }

    protected void internalUpdateInputOption(InputOption inputOption) {
        if (internalInputOption != null) {
            internalInputOption.update(inputOption);
        }
    }

    protected void internalUpdateLimitOption(LimitOption limitOption) {
        if (internalLimitOption != null) {
            internalLimitOption.update(limitOption);
        }
    }

    protected void internalUpdatePiocheSelectOption(PiocheSelectOption piocheSelectOption) {
        if (internalPiocheSelectOption != null) {
            internalPiocheSelectOption.update(piocheSelectOption);
        }
    }

    protected void internalUpdateValueOption(ValueOption valueOption) {
        internalValueOption.update(valueOption);
    }

    protected boolean internalSetDefaultLibelleUsage(boolean def) {
        if (def == defaultLibelle) {
            return false;
        }
        this.defaultLibelle = def;
        if (def) {
            customLibelleHolder = null;
        } else {
            customLibelleHolder = new CustomLibelleHolder();
        }
        return true;
    }

    protected boolean internalSetWithIdalpha(boolean withIdalpha) {
        if (this.withIdalpha == withIdalpha) {
            return false;
        }
        this.withIdalpha = withIdalpha;
        return true;
    }

    protected boolean internalSetLibelleString(int langInteger, String nvlibelle) {
        if (customLibelleHolder == null) {
            throw new IllegalArgumentException("defaultLibelleUsage is set");
        }
        return customLibelleHolder.internalSetLibelle(nvlibelle, langInteger);
    }


    private class CustomLibelleHolder extends AbstractLibelleHolder {

        private boolean internalSetLibelle(String nvlibelle, int langInteger) {
            return setLibelle(nvlibelle, langInteger);
        }

    }


    private class InternalInputOption extends DefaultInputOption {

        InternalInputOption(InputOption inputOption) {
            super(inputOption);
        }

        private void update(InputOption inputOption) {
            innerUpdate(inputOption);
        }

    }


    private class InternalLimitOption extends DefaultLimitOption {

        InternalLimitOption(LimitOption limitOption) {
            super(limitOption);
        }

        private void update(LimitOption limitOption) {
            innerUpdate(limitOption);
        }

    }


    private class InternalValueOption extends DefaultValueOption {

        InternalValueOption() {
            innerSetMandatory(false);
            innerSetDefaultValue("");
        }

        InternalValueOption(ValueOption valueOption) {
            super(valueOption);
        }

        private void update(ValueOption valueOption) {
            innerUpdate(valueOption);
        }

    }


    private class InternalPiocheSelectOption extends DefaultPiocheSelectOption {

        public InternalPiocheSelectOption() {
        }

        public InternalPiocheSelectOption(PiocheSelectOption piocheSelectOption) {
            super(piocheSelectOption);
        }

        private void update(PiocheSelectOption piocheSelectOption) {
            innerUpdate(piocheSelectOption);
        }

    }

}
