/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.DefaultCommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUiXMLWriter extends UtilXMLWriter {

    public CorpusUiXMLWriter() {
    }

    public void appendCorpusUI(CorpusUI corpusUI) throws IOException {
        openTag("corpus-ui");
        openTag("metadata");
        appendCommentDef(corpusUI);
        closeTag("metadata");
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);
        int size = list.size();
        for (int i = 0; i < size; i++) {
            ComponentUI componentUI = (ComponentUI) list.get(i);
            if (componentUI instanceof CommentUI) {
                CommentUI commentUI = (CommentUI) componentUI;
                startOpenTag("comment");
                addAttribute("name", commentUI.getName());
                closeEmptyTag();
            } else if (componentUI instanceof FieldUI) {
                appendFieldUI((FieldUI) componentUI);
            } else if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                IncludeKey includeKey = includeUI.getFiteredIncludeKey().getIncludeKey();
                if (includeKey.isSpecialIncludeKey()) {
                    appendSpecialIncludeUI(includeUI);
                } else if ((includeKey.isLiageIncludeKey()) || (includeKey.isIndexationIncludeKey())) {
                    appendSubsetIncludeUI(includeUI);
                }
            }
        }
        closeTag("corpus-ui");
    }

    private void appendCommentDef(CorpusUI corpusUI) throws IOException {
        List list = corpusUI.getOrderedComponentUIList(CorpusUI.COMMENT_LIST);
        int size = list.size();
        for (int i = 0; i < size; i++) {
            CommentUI commentUI = (CommentUI) list.get(i);
            startOpenTag("comment-def");
            addAttribute("name", commentUI.getName());
            addAttribute("location", DefaultCommentUI.maskToString(commentUI.getLocation()));
            endOpenTag();
            addLibelleList(commentUI);
            closeTag("comment-def");
        }
    }

    private void appendFieldUI(FieldUI fieldUI) throws IOException {
        CorpusField corpusField = fieldUI.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        startOpenTag("field-ui");
        addAttribute("field-key", fieldKey.toString());
        endOpenTag();
        appendInputOption(fieldUI.getInputOption());
        appendValueOption(fieldUI.getValueOption());
        closeTag("field-ui");
    }

    private void appendSpecialIncludeUI(IncludeUI includeUI) throws IOException {
        startOpenTag("special-ui");
        addAttribute("include-key", includeUI.getFiteredIncludeKey().toString());
        endOpenTag();
        appendCustomLib(includeUI);
        appendLimitOption(includeUI.getLimitOption());
        appendValueOption(includeUI.getValueOption());
        closeTag("special-ui");
    }

    private void appendSubsetIncludeUI(IncludeUI includeUI) throws IOException {
        FilteredIncludeKey filteredIncludeKey = includeUI.getFiteredIncludeKey();
        openTag("subset-ui");
        addSimpleElement("subset", filteredIncludeKey.getIncludeKey().toString());
        int poidsFilter = filteredIncludeKey.getPoidsFilter();
        if (poidsFilter != -1) {
            addSimpleElement("poids", String.valueOf(poidsFilter));
        }
        appendCustomLib(includeUI);
        appendInputOption(includeUI.getInputOption());
        appendValueOption(includeUI.getValueOption());
        closeTag("subset-ui");
    }

    private void appendInputOption(InputOption inputOption) throws IOException {
        if (inputOption == null) {
            return;
        }
        startOpenTag("input");
        addAttribute("type", DefaultInputOption.inputTypeToString(inputOption.getInputType()));
        addAttribute("width", DefaultInputOption.widthTypeToString(inputOption.getWidthType()));
        addAttribute("rows", inputOption.getRows());
        closeEmptyTag();
    }

    private void appendLimitOption(LimitOption limitOption) throws IOException {
        if (limitOption == null) {
            return;
        }
        startOpenTag("limit");
        addAttribute("nonew", (limitOption.isNoNewAllowed()) ? "1" : "0");
        closeEmptyTag();
    }

    private void appendCustomLib(IncludeUI includeUI) throws IOException {
        LibelleHolder libelleHolder = includeUI.getCustomLibelle();
        if (libelleHolder == null) {
            return;
        }
        openTag("custom");
        addLibelleList(libelleHolder);
        closeTag("custom");
    }

    private void appendValueOption(ValueOption valueOption) throws IOException {
        if (valueOption == null) {
            return;
        }
        boolean mandatory = valueOption.isMandatory();
        String defaultValue = valueOption.getDefaultValue();
        int length = defaultValue.length();
        if ((mandatory) || (length > 0)) {
            startOpenTag("value");
            if (mandatory) {
                addAttribute("mandatory", "1");
            }
            if (length > 0) {
                addAttribute("default", defaultValue);
            }
            closeEmptyTag();
        }
    }

}
