/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import java.io.IOException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class TreesXMLWriter extends UtilXMLWriter {

    public TreesXMLWriter() {
    }

    public void appendTrees(UIManager uiManager) throws IOException {
        openTag("trees");
        openTag("metadata");
        appendGroupDef(uiManager.getSubsetKeyGroupTemplates());
        closeTag("metadata");
        appendTree(uiManager.getCorpusSubsetKeyTree());
        appendTree(uiManager.getThesaurusSubsetKeyTree());
        appendTree(uiManager.getSphereSubsetKeyTree());
        closeTag("trees");
    }

    private void appendGroupDef(SubsetKeyGroupTemplates subsetKeyGroupTemplates) throws IOException {
        int size = subsetKeyGroupTemplates.size();
        for (int i = 0; i < size; i++) {
            String groupName = subsetKeyGroupTemplates.getGroupName(i);
            startOpenTag("group-def");
            addAttribute("name", groupName);
            endOpenTag();
            LibelleHolder libelleHolder = subsetKeyGroupTemplates.getLibelleHolder(groupName);
            if (libelleHolder != null) {
                addLibelleList(libelleHolder);
            }
            closeTag("group-def");
        }
    }

    private void appendTree(SubsetKeyTree subsetKeyTree) throws IOException {
        startOpenTag("tree");
        addAttribute("type", SubsetKeyTree.toStringType(subsetKeyTree.getType()));
        endOpenTag();
        for (SubsetKeyGroup group : subsetKeyTree.getSubsetKeyGroupList()) {
            boolean noneGroup = group.isNoTemplateGroup();
            if (!noneGroup) {
                startOpenTag("group");
                addAttribute("name", group.getTemplateName());
                endOpenTag();
            }
            for (SubsetKey subsetKey : group.getSubsetKeyList()) {
                addSimpleElement("subset", subsetKey.toString());
            }
            if (!noneGroup) {
                closeTag("group");
            }
        }
        closeTag("tree");
    }

}
