/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users;

import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import fr.exemole.bdfserver.AvailableFormatLocales;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.request.QuoteOptions;
import fr.exemole.bdfserver.request.TypoOptions;
import net.mapeadores.util.localisation.LocalisationInfo;
import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class BdfUser implements BdfUserConstants {

private Map<TransformationKey, String> templateNameMap = new HashMap<TransformationKey, String>();
    private Map<String, StreamTransformerKey> streamTransformerKeyMap = new HashMap<String, StreamTransformerKey>();
    private Map<String, Object> parameterMap = new HashMap<String, Object>();
    private Map<TransformationKey, String> unmodifiableTemplateNameMap = Collections.unmodifiableMap(templateNameMap);
    private Map<String, StreamTransformerKey> unmodifiableStreamTransformerKeyMap = Collections.unmodifiableMap(streamTransformerKeyMap);
    private Object sessionObject = null;
    private RedacteurKey redacteurKey;
    private BdfServer bdfServer;
    private FicheSelectOptions ficheSelectOptions;
    private FicheSelectOptions defaultFicheSelectOptions;
    private short abregeFicheSortType;
    private LocalisationInfo localisationInfo;
    private TableExportParametersBean tableExportParametersBean;
    private TypoOptions typoOptions;
    private String[] roleNameArray;
    private boolean withJQuery = false;
    private short screenWidth;
    private AbregeFicheSortedList abregeFicheSortedList = null;
    private String baseDeFichesURL;
    private boolean customAbregeFicheList = false;

    public BdfUser(BdfServer bdfServer, RedacteurKey redacteurKey) {
        this.redacteurKey = redacteurKey;
        this.bdfServer = bdfServer;
        ficheSelectOptions = new FicheSelectOptions();
        RedacteurKey[] array = {redacteurKey};
        ficheSelectOptions.setRedacteurKeyArray(array);
        defaultFicheSelectOptions = ficheSelectOptions;
        abregeFicheSortType = AbregeFicheSortedList.NONE_SORTTYPE;
        this.localisationInfo = bdfServer.getConfigurationManager().getLangConfig().getDefaultLocalisationInfo();
        initTypoOptions();
        bdfServer.getBdfStorage().loadBdfUser(this);
    }

    private void initTypoOptions() {
        typoOptions = new TypoOptions();
        typoOptions.setQuoteOptions(QuoteOptions.getQuoteOptions(localisationInfo.getFormatLocale()));
    }

    public RedacteurKey getRedacteurKey() {
        return redacteurKey;
    }

    /**
     * équivalent de getLocalisationInfo().getWorkingLangInteger()
     */
    public int getWorkingLangInteger() {
        return localisationInfo.getWorkingLangInteger();
    }

    public Locale getFormatLocale() {
        return localisationInfo.getFormatLocale();
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public String[] getRoleNameArray() {
        return roleNameArray;
    }

    public void setRoleNameArray(String[] roleNameArray) {
        this.roleNameArray = roleNameArray;
    }

    /**
     * LocalisationInfo peut être nul, ce sont alors les valeurs par défaut qui sont appliquées.
     * Ne pas oublier la procédure de UsedFormatLocaleListener
     */
    public void setLocalisationInfo(LocalisationInfo locInfo) {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        if (locInfo == null) {
            this.localisationInfo = langConfig.getDefaultLocalisationInfo();
            return;
        }
        int newWorkingLangInteger = locInfo.getWorkingLangInteger();
        if (!langConfig.containsWorkingLangInteger(newWorkingLangInteger)) {
            newWorkingLangInteger = langConfig.getDefaultWorkingLangInteger();
        }

        Locale newFormatLocale = AvailableFormatLocales.getNearestAvailableLocale(locInfo.getFormatLocaleInteger());
        if (newFormatLocale == null) {
            newFormatLocale = AvailableFormatLocales.getNearestAvailableLocale(newWorkingLangInteger);
            if (newFormatLocale == null) {
                newFormatLocale = langConfig.getDefaultFormatLocale();
            }
        }
        this.localisationInfo = new LocalisationInfo(newWorkingLangInteger, newFormatLocale);
        initTypoOptions();
    }

    /**
     * Retourne la session à laquelle est attachée l'instance de BdfUser.
     */
    public Object getSessionObject() {
        return sessionObject;
    }

    public XslTransformerKey getXslTransformerKey(TransformationKey transformationKey) {
        String transformerName = templateNameMap.get(transformationKey);
        return new XslTransformerKey(transformationKey, transformerName);
    }

    public StreamTransformerKey getStreamTransformerKey(TransformationKey transformationKey, String extension) {
        StreamTransformerKey streamTransformerKey = streamTransformerKeyMap.get(StreamTransformerKey.toStringKey(transformationKey, extension));
        return streamTransformerKey;
    }

    public void setXSLTemplateName(TransformationKey transformationKey, String templateName) {
        if (templateName == null) {
            templateNameMap.remove(transformationKey);
        } else {
            templateNameMap.put(transformationKey, templateName);
        }
    }

    public void putStreamTransformerKey(StreamTransformerKey streamTransformerKey) {
        streamTransformerKeyMap.put(StreamTransformerKey.toStringKey(streamTransformerKey), streamTransformerKey);
    }

    public Map<TransformationKey, String> getTemplateNameMap() {
        return unmodifiableTemplateNameMap;
    }

    public Map<String, StreamTransformerKey> getStreamTransformerKeyMap() {
        return unmodifiableStreamTransformerKeyMap;
    }

    public FicheSelectOptions getFicheSelectOptions() {
        return ficheSelectOptions;
    }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.ficheSelectOptions = ficheSelectOptions;
        this.customAbregeFicheList = false;
    }

    public void setDefaultFicheSelectOptions(FicheSelectOptions defaultFicheSelectOptions) {
        this.defaultFicheSelectOptions = defaultFicheSelectOptions;
    }

    public FicheSelectOptions getDefaultFicheSelectOptions() {
        return defaultFicheSelectOptions;
    }

    public short getAbregeFicheSortType() {
        return abregeFicheSortType;
    }

    public void setAbregeFicheSortType(short abregeFicheSortType) {
        if (!AbregeFicheSortedList.isValideShortType(abregeFicheSortType)) {
            throw new IllegalArgumentException("abregeFicheSortType is not valid");
        }
        this.abregeFicheSortType = abregeFicheSortType;
    }

    public AbregeFicheSortedList getAbregeFicheSortedList() {
        return abregeFicheSortedList;
    }

    public void setAbregeFicheSortedList(AbregeFicheSortedList abregeFicheSortedList) {
        this.abregeFicheSortedList = abregeFicheSortedList;
        this.customAbregeFicheList = true;
    }

    public boolean updateAbregeFicheSortedList() {
        if (customAbregeFicheList) {
            return false;
        }
        FicheSelector ficheSelector = new FicheSelector(bdfServer.getBasedefiches(), getWorkingLangInteger());
        ficheSelector.addFicheSelectOptions(getFicheSelectOptions());
        ficheSelector.setFicheEligibility(bdfServer.getPermissionManager().getFicheEligibility(this));
        this.abregeFicheSortedList = SelectionUtils.getAbregeFicheSortedList(ficheSelector, getAbregeFicheSortType());
        return true;
    }

    /**
     * Retourne la valeur d'un paramètre stocké le temps de la session.
     * L'instance de Bdfuser se contente d'être un lieu de stockage, il ne fait rien avec les valeurs en question.
     * C'est aux applications utilisant ces paramètres d'utiliser une convention de nommage évitant les carambolages.
     */
    public Object getParameterValue(String parameterName) {
        return parameterMap.get(parameterName);
    }

    public void setParameterValue(String parameterName, Object parameterValue) {
        parameterMap.put(parameterName, parameterValue);
    }

    public TableExportParametersBean getTableExportParametersBean() {
        return tableExportParametersBean;
    }

    public void setTableExportParametersBean(TableExportParametersBean tableExportParametersBean) {
        if (tableExportParametersBean != null) {
            this.tableExportParametersBean = new TableExportParametersBean(tableExportParametersBean);
        } else {
            this.tableExportParametersBean = null;
        }
    }

    public TypoOptions getTypoOptions() {
        return typoOptions;
    }

    public void setSessionObject(Object sessionObject) {
        this.sessionObject = sessionObject;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public boolean isWithJQuery() {
        return withJQuery;
    }

    public void setWithJQuery(boolean withJQuery) {
        this.withJQuery = withJQuery;
    }

    public short getScreenWidth() {
        return screenWidth;
    }

    public void setScreenWidth(short screenWidth) {
        this.screenWidth = screenWidth;
    }

    public String getBaseDeFichesURL() {
        return baseDeFichesURL;
    }

    public void setBaseDeFichesURL(String baseDeFichesURL) {
        this.baseDeFichesURL = baseDeFichesURL;
    }

}
