/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users.xml;

import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.LinkedHashSet;
import java.util.Locale;
import java.util.Set;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.basedefiches.xml.extraction.ExtractionDOMReader;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class UserDOMReader {

    public UserDOMReader() {
    }

    public void fillUser(BdfUser bdfUser, Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("selection")) {
                    FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(element);
                    bdfUser.setDefaultFicheSelectOptions(ficheSelectOptions);
                    bdfUser.setFicheSelectOptions(ficheSelectOptions);
                    String sort = element.getAttribute("sort");
                    try {
                        bdfUser.setAbregeFicheSortType(AbregeFicheSortedList.typeToShort(sort));
                    } catch (IllegalArgumentException iae) {
                    }
                } else if (tagName.equals("transformation")) {
                    readTransformation(bdfUser, element);
                } else if (tagName.equals("loc-info")) {
                    readLocalisationInfo(bdfUser, element);
                } else if (tagName.equals("table-export-parameters")) {
                    readTableExportParameters(bdfUser, element);
                } else if (tagName.equals("roles")) {
                    readRoles(bdfUser, element);
                }
            }
        }
    }

    private void readLocalisationInfo(BdfUser bdfUser, Element element_xml) {
        String langIntegerString = element_xml.getAttribute("working-lang");
        int workingLangInteger = LangInteger.UNDETERMINED;
        try {
            workingLangInteger = LangInteger.parse(langIntegerString);
        } catch (LangIntegerException lie) {
            return;
        }
        Locale locale = null;
        String localeString = element_xml.getAttribute("format-locale");
        if (localeString.length() > 0) {
            try {
                int localeLangInteger = LangInteger.parse(localeString);
                locale = LangInteger.getLocale(localeLangInteger);
            } catch (LangIntegerException lie) {
            }
        }
        LocalisationInfo localisationInfo = null;
        if (locale != null) {
            localisationInfo = new LocalisationInfo(workingLangInteger, locale);
        } else {
            localisationInfo = new LocalisationInfo(workingLangInteger);
        }
        bdfUser.setLocalisationInfo(localisationInfo);
    }

    private void readTransformation(BdfUser bdfUser, Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                String templateName = element.getAttribute("name");
                if (templateName.length() == 0) {
                    templateName = null;
                }
                TransformationKey transformationKey = null;
                try {
                    transformationKey = TransformationKey.parse(element.getAttribute("key"));
                } catch (java.text.ParseException pe) {
                    continue;
                }
                if (tagName.equals("xsl")) {
                    bdfUser.setXSLTemplateName(transformationKey, templateName);
                } else if (tagName.equals("stream")) {
                    String extension = element.getAttribute("extension");
                    if (extension.length() == 0) {
                        continue;
                    }
                    StreamTransformerKey streamTransformerKey = new StreamTransformerKey(transformationKey, extension, templateName);
                    bdfUser.putStreamTransformerKey(streamTransformerKey);
                }
            }
        }
    }

    private void readTableExportParameters(BdfUser bdfUser, Element element_xml) {
        TableExportParametersBean tableExportParameters = new TableExportParametersBean();
        tableExportParameters.setWithAssociation(getBoolean(element_xml, "with-association"));
        tableExportParameters.setWithChrono(getBoolean(element_xml, "with-chrono"));
        tableExportParameters.setWithLiage(getBoolean(element_xml, "with-liage"));
        tableExportParameters.setWithTextes(getBoolean(element_xml, "with-textes"));
        String name = element_xml.getAttribute("name");
        if (name.length() > 0) {
            tableExportParameters.setTableExportName(name);
        }
        try {
            String headerType = element_xml.getAttribute("header-type");
            tableExportParameters.setHeaderType(TableExportCommon.headerTypeToShort(headerType));
        } catch (IllegalArgumentException iae) {
        }
        try {
            String indexationInclueType = element_xml.getAttribute("indexation-include-type");
            tableExportParameters.setIndexationIncludeType(TableExportCommon.indexationIncludeTypeToShort(indexationInclueType));
        } catch (IllegalArgumentException iae) {
        }
        bdfUser.setTableExportParametersBean(tableExportParameters);
    }

    private boolean getBoolean(Element element_xml, String attr) {
        String val = element_xml.getAttribute(attr);
        if (val.equals("1")) {
            return true;
        }
        return false;
    }

    private void readRoles(BdfUser bdfUser, Element element_xml) {
        Set<String> result = new LinkedHashSet<String>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("role")) {
                    String name = element.getAttribute("name");
                    if (name.length() > 0) {
                        result.add(name);
                    }
                }
            }
        }
        int size = result.size();
        String[] roleNameArray = new String[size];
        int p = 0;
        for (String name : result) {
            roleNameArray[p] = name;
            p++;
        }
        bdfUser.setRoleNameArray(roleNameArray);
    }

}
