/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users.xml;

import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import java.util.Map;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.basedefiches.xml.extraction.ExtractionUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class UserXMLWriter extends UtilXMLWriter {

    public UserXMLWriter() {
    }

    public void appendUser(BdfUser bdfUser) throws IOException {
        startOpenTag("user");
        endOpenTag();
        appendLocalisationInfo(bdfUser);
        appendRoles(bdfUser);
        appendTransformation(bdfUser);
        appendSelection(bdfUser);
        appendTableExportParameters(bdfUser);
        closeTag("user");
    }

    private void appendRoles(BdfUser bdfUser) throws IOException {
        String[] roleArray = bdfUser.getRoleNameArray();
        if (roleArray == null) {
            return;
        }
        openTag("roles");
        for (int i = 0; i < roleArray.length; i++) {
            startOpenTag("role");
            addAttribute("name", roleArray[i]);
            closeEmptyTag();
        }
        closeTag("roles");
    }

    private void appendTransformation(BdfUser bdfUser) throws IOException {
        openTag("transformation");
        for (Map.Entry<TransformationKey, String> entry : bdfUser.getTemplateNameMap().entrySet()) {
            startOpenTag("xsl");
            addAttribute("key", entry.getKey().toString());
            addAttribute("name", entry.getValue());
            closeEmptyTag();
        }
        for (StreamTransformerKey key : bdfUser.getStreamTransformerKeyMap().values()) {
            startOpenTag("stream");
            addAttribute("key", key.getTransformationKey().toString());
            addAttribute("extension", key.getExtension());
            addAttribute("name", key.getTransformerName());
            closeEmptyTag();
        }
        closeTag("transformation");

    }

    private void appendLocalisationInfo(BdfUser bdfUser) throws IOException {
        LocalisationInfo localisationInfo = bdfUser.getLocalisationInfo();
        if (localisationInfo == null) {
            return;
        }
        startOpenTag("loc-info");
        addAttribute("working-lang", LangInteger.toString(localisationInfo.getWorkingLangInteger()));
        if (!localisationInfo.isSame()) {
            addAttribute("format-locale", LangInteger.toString(localisationInfo.getFormatLocaleInteger()));
        }
        closeEmptyTag();
    }

    private void appendSelection(BdfUser bdfUser) throws IOException {
        FicheSelectOptions ficheSelectOptions = bdfUser.getFicheSelectOptions();
        if (ficheSelectOptions == null) {
            return;
        }
        startOpenTag("selection");
        addAttribute("sort", AbregeFicheSortedList.typeToString(bdfUser.getAbregeFicheSortType()));
        endOpenTag();
        ExtractionUtils.writeFicheSelectOptions(this, ficheSelectOptions);
        closeTag("selection");
    }

    private void appendTableExportParameters(BdfUser bdfUser) throws IOException {
        TableExportParametersBean tableExportParametersBean = bdfUser.getTableExportParametersBean();
        if (tableExportParametersBean == null) {
            return;
        }
        startOpenTag("table-export-parameters");
        addAttribute("name", tableExportParametersBean.getTableExportName());
        appendBoolean("with-textes", tableExportParametersBean.isWithTextes());
        appendBoolean("with-association", tableExportParametersBean.isWithAssociation());
        appendBoolean("with-liage", tableExportParametersBean.isWithLiage());
        appendBoolean("with-chrono", tableExportParametersBean.isWithChrono());
        addAttribute("header-type", TableExportCommon.headerTypeToString(tableExportParametersBean.getHeaderType()));
        addAttribute("indexation-include-type", TableExportCommon.indexationIncludeTypeToString(tableExportParametersBean.getIndexationIncludeType()));
        closeEmptyTag();
    }

    private void appendBoolean(String name, boolean booleanValue) throws IOException {
        String value = (booleanValue) ? "1" : "0";
        addAttribute(name, value);
    }

}
