/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.util;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.actions.CreateFiche;
import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;


/**
 *
 * @author Vincent Calame
 */
public class BdfServerUtils {

    private static RattachementPermission EMPTY_RP = new EmptyRattachementPermission();

    private BdfServerUtils() {
    }

    public static RattachementPermission getRattachementPermission(BdfServer bdfServer, BdfUser bdfUser, AbregeFiche abregeFiche) {
        Corpus corpus = abregeFiche.getCorpus();
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        Subset[] slaveCorpusArray = corpus.getSlaveSubsetArray();
        if (slaveCorpusArray == null) {
            return EMPTY_RP;
        }
        int idcorpus = abregeFiche.getIdcorpus();
        int length = slaveCorpusArray.length;
        InternalRattachementPermission irp = new InternalRattachementPermission();
        for (int i = 0; i < length; i++) {
            Corpus slaveCorpus = (Corpus) slaveCorpusArray[i];
            AbregeFiche otherAbregeFiche = slaveCorpus.getAbregeFicheByIdcorpus(idcorpus);
            if (otherAbregeFiche == null) {
                if (permissionManager.canDo(bdfUser, CreateFiche.newInstance(slaveCorpus, idcorpus))) {
                    irp.add(slaveCorpus);
                }
            } else {
                if (permissionManager.canDo(bdfUser, FicheAction.getWriteFiche(otherAbregeFiche))) {
                    irp.add(otherAbregeFiche, RattachementPermission.WRITE_ACTION);
                } else if (permissionManager.canDo(bdfUser, FicheAction.getReadFiche(otherAbregeFiche))) {
                    irp.add(otherAbregeFiche, RattachementPermission.READ_ACTION);
                }
            }
        }
        return irp;
    }


    private static class InternalRattachementPermission implements RattachementPermission {

        private List<Bundle> bundleList = new ArrayList<Bundle>();
        private boolean onlyReadAllowed = true;

        public int getSlaveCorpusCount() {
            return bundleList.size();
        }

        public Corpus getCorpus(int i) {
            return getBundle(i).getCorpus();
        }

        public short getActionType(int i) {
            return getBundle(i).getType();
        }

        public AbregeFiche getAbregeFiche(int i) {
            return getBundle(i).getAbregeFiche();
        }

        public boolean onlyReadAllowed() {
            return onlyReadAllowed;
        }

        Bundle getBundle(int i) {
            return bundleList.get(i);
        }

        void add(Corpus corpus) {
            onlyReadAllowed = false;
            bundleList.add(new Bundle(corpus, CREATE_ACTION));
        }

        void add(AbregeFiche abregeFiche, short type) {
            if (type != READ_ACTION) {
                onlyReadAllowed = false;
            }
            bundleList.add(new Bundle(abregeFiche, type));
        }


        class Bundle {

            private Corpus corpus;
            private AbregeFiche abregeFiche;
            private short type;

            Bundle(Corpus corpus, short type) {
                this.corpus = corpus;
                this.type = type;
                this.abregeFiche = null;
            }

            Bundle(AbregeFiche abregeFiche, short type) {
                this.corpus = abregeFiche.getCorpus();
                this.type = type;
                this.abregeFiche = abregeFiche;
            }

            public Corpus getCorpus() {
                return corpus;
            }

            public AbregeFiche getAbregeFiche() {
                return abregeFiche;
            }

            public short getType() {
                return type;
            }

        }

    }


    private static class EmptyRattachementPermission implements RattachementPermission {

        public int getSlaveCorpusCount() {
            return 0;
        }

        public Corpus getCorpus(int i) {
            throw new IndexOutOfBoundsException();
        }

        public short getActionType(int i) {
            throw new IndexOutOfBoundsException();
        }

        public AbregeFiche getAbregeFiche(int i) {
            throw new IndexOutOfBoundsException();
        }

        public boolean onlyReadAllowed() {
            return true;
        }

    }

}
