/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.xsltfactory.odcss.BdfServerCssSource;
import java.io.*;
import net.mapeadores.opendocument.css.elements.ElementMaps;
import net.mapeadores.opendocument.css.output.StylesOutput;
import net.mapeadores.opendocument.css.parse.CssParser;
import net.mapeadores.util.io.*;
import net.mapeadores.opendocument.transform.*;
import net.mapeadores.opendocument.css.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultOdStreamTransformerBuilder {

    ODStreamTransformer.Builder odStreamTransformerBuilder;
    private BdfServer bdfServer;

    public DefaultOdStreamTransformerBuilder(BdfServer bdfServer, String extension) {
        odStreamTransformerBuilder = new ODStreamTransformer.Builder(extension);
        this.bdfServer = bdfServer;
    }

    private void setOdCss(String odCssRessourcUrl) {
        BdfServerCssSource cssSource = BdfServerCssSource.getBdfServerCssSource(odCssRessourcUrl, bdfServer);
        if (cssSource == null) {
            throw new InternalResourceException("missing resource = " + odCssRessourcUrl);
        }
        try {
            ElementMaps elementMaps = CssParser.parse(cssSource);
            ByteArrayOutputStream stylesOutputStream = new ByteArrayOutputStream();
            ByteArrayOutputStream contentAutomaticStylesOutputStream = new ByteArrayOutputStream();
            StylesOutput.write(stylesOutputStream, elementMaps);
            StylesOutput.writeAutomaticStyles(contentAutomaticStylesOutputStream, elementMaps);
            odStreamTransformerBuilder.setStyles(stylesOutputStream.toByteArray());
            odStreamTransformerBuilder.setContentAutomaticStyles(contentAutomaticStylesOutputStream.toByteArray());
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }

    }

    public StreamTransformer toStreamTransformer(String odCssRessourceUrl, String contentStyleSheet) {
        setOdCss(odCssRessourceUrl);
        odStreamTransformerBuilder.setURIResolver(bdfServer.getURIResolver());
        odStreamTransformerBuilder.setContentURI("_default_");
        odStreamTransformerBuilder.setStreamTransformerErrorHandler(new InternalStreamTransformerErrorHandler());
        try {
            odStreamTransformerBuilder.setContentXsl(contentStyleSheet.getBytes("UTF-8"));
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        return odStreamTransformerBuilder.toODStreamTransformer();
    }


    private static class InternalStreamTransformerErrorHandler implements StreamTransformerErrorHandler {

        public void missingResourceError(String resourceURI) {
            throw new InternalResourceException("StreamTransformerErrorHandler - missingResourceError : " + resourceURI);
        }

        public void wrongResourceError(String resourceURI, String message) {
            throw new InternalResourceException("StreamTransformerErrorHandler - wrongResourceError : " + resourceURI + " / " + message);
        }

    }

}
