/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.managers.transformation.*;
import fr.exemole.bdfserver.*;
import java.io.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.text.insertpattern.api.*;
import fr.exemole.bdfserver.xsltfactory.*;
import java.io.*;
import net.mapeadores.util.io.*;
import net.mapeadores.opendocument.transform.*;
import net.mapeadores.opendocument.css.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.xsltfactory.*;
import javax.xml.transform.*;
import fr.exemole.bdfserver.xsltfactory.xslpattern.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultXSLTFactory implements XSLTFactoryConstants {

    private DefaultXSLTFactory() {
    }

    public static String getDefaultStyleSheet(BdfServer bdfServer, TransformationKey transformationKey, short format) {
        String pattern = XSLTPatternBuilder.getXSLTPattern(bdfServer, transformationKey, false, format);
        if (pattern == null) {
            return null;
        }
        return PatternUtils.getStyleSheet(bdfServer, pattern, new XSLTErrorHandler(), null);
    }


    static private class XSLTErrorHandler implements ErrorHandler {

        public void characterError(char carac, String key, int row, int col) {
            throw new InternalResourceException("XSLTErrorHandler characterError = " + carac + " / " + key + " / " + row + "," + col);
        }

        public void warning(String key, int row, int col) {
            throw new InternalResourceException("XSLTErrorHandler warning = " + key + " / " + row + "," + col);
        }

    }

    public static StreamTransformer getDefaultStreamTransformer(BdfServer bdfServer, TransformationKey transformationKey, short format) {
        switch (format) {
            case ODT_FORMAT:
                DefaultOdStreamTransformerBuilder odBuilder = new DefaultOdStreamTransformerBuilder(bdfServer, "odt");
                return odBuilder.toStreamTransformer(getOdCssResourceUrl(transformationKey), getDefaultStyleSheet(bdfServer, transformationKey, format));
            case XSL_FORMAT:
                return null;
            default:
                throw new SwitchException("format = " + format);
        }
    }

    private static String getOdCssResourceUrl(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey() || (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY))) {
            return "bdf:fiche_odt.css";
        }
        throw new SwitchException("transformationKey = " + transformationKey.toString());
    }

}
