/* BdfServer
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory;

import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.managers.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.configuration.*;
import fr.exemole.bdfserver.managers.transformation.*;
import java.net.*;


/**
 *
 * @author Vincent Calame
 */
public class ImportXSLTWriter extends XSLTWriter implements XSLTFactoryConstants, TransformationConstants {

    public static final short FICHE_IMPORT = 1;
    public static final short STATTHESAURUS_IMPORT = 2;
    private short importType = 1;
    private BdfServer bdfServer;

    public ImportXSLTWriter(short importType, BdfServer bdfServer) {
        super();
        this.bdfServer = bdfServer;
        this.importType = importType;
    }

    public void addImportXSLT(short format) throws IOException {
        openXslStyleSheet();
        switch (format) {
            case XSL_FORMAT:
                switch (importType) {
                    case FICHE_IMPORT:
                        addXslImport("bdf:_common_ficheitem.xsl");
                        addXslImport("bdf:_common_ficheblock.xsl");
                        addXslImport("bdf:_common_field.xsl");
                        addXslImport("bdf:_common_include.xsl");
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle.xsl");
                        break;
                    case STATTHESAURUS_IMPORT:
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle.xsl");
                        break;
                    default:
                        throw new SwitchException("importtype = " + importType);
                }
                break;
            case ODT_FORMAT:
                switch (importType) {
                    case FICHE_IMPORT:
                        addXslImport("bdf:_common_ficheitem_odt.xsl");
                        addXslImport("bdf:_common_ficheblock_odt.xsl");
                        addXslImport("bdf:_common_field.xsl");
                        addXslImport("bdf:_common_include_odt.xsl");
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle_odt.xsl");
                        break;
                    default:
                        throw new SwitchException("importtype = " + importType);
                }
                break;
            default:
                throw new SwitchException("format = " + format);
        }

        addXslParam(WORKINGLANG_PARAMETER, "fr", true);
        addXslParam(FORMATLOCALE_PARAMETER, "fr", true);
        addXslParam(BDF_BASEDEFICHESPATH_PARAMETER, "", true);

        addFicheURLTemplate();
        addMotcleURLTemplate();
        addAlbumImageURLTemplate();
        addVersionURLTemplate();
        addInsertURLTemplate();
        closeXslStyleSheet();
    }

    private void addFicheURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_FICHEURL");
        addXslParam("PARAM_CORPUS", null, false);
        addXslParam("PARAM_IDCORPUS", null, false);
        addXslParam("PARAM_FRAGMENT", "", true);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        addXslText("fiches/");
        addXslValueOf("$PARAM_CORPUS");
        addXslText("-");
        addXslValueOf("$PARAM_IDCORPUS");
        addXslText(".html");
        openXslIf("string-length($PARAM_FRAGMENT) > 0");
        appendIndent();
        addXslText("#");
        addXslValueOf("$PARAM_FRAGMENT");
        closeXslIf();
        closeXslTemplate();
    }

    private void addMotcleURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_MOTCLEURL");
        addXslParam("PARAM_THESAURUS", null, false);
        addXslParam("PARAM_IDTHS", null, false);
        addXslParam("PARAM_FRAGMENT", "", true);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        append("Thesaurus?thesaurus=");
        addXslValueOf("$PARAM_THESAURUS");
        escape("&idths=");
        addXslValueOf("$PARAM_IDTHS");
        closeXslTemplate();
    }

    private void addAlbumImageURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_ALBUMIMAGEURL");
        addXslParam("PARAM_ALBUM", null, false);
        addXslParam("PARAM_DIM", "", true);
        addXslParam("PARAM_IMAGENAME", null, false);
        appendIndent();
        //addXslText(contextUrl);
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        addXslText("images/");
        addXslValueOf("$PARAM_ALBUM");
        addXslText("/");
        openXslIf("(string-length($PARAM_DIM) > 0) and ($PARAM_DIM != '_')");
        addXslValueOf("$PARAM_DIM");
        addXslText("/");
        closeXslIf();
        addXslValueOf("$PARAM_IMAGENAME");
        closeXslTemplate();
    }

    private void addVersionURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_VERSIONURL");
        addXslParam("PARAM_BASENAME", null, false);
        addXslParam("PARAM_EXTENSION", null, false);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        append("docs/");
        addXslValueOf("$PARAM_BASENAME");
        append(".");
        addXslValueOf("$PARAM_EXTENSION");
        closeXslTemplate();
    }

    private void addInsertURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_INSERTURL");
        addXslParam("PARAM_SRC", null, false);
        appendIndent();
        addXslText(getUrl(ConfigurationManager.INSERT_URL));
        addXslValueOf("$PARAM_SRC");
        closeXslTemplate();
    }

    private String getUrl(String name) {
        ConfigurationPath configurationPath = bdfServer.getConfigurationManager().getConfigurationPath(name);
        if (configurationPath != null) {
            URL url = configurationPath.getPublicURL();
            if (url != null) {
                return url.toString();
            }
        }
        return "/";
    }

    /*public static String getImportXSLTName(short importType) {
    StringBuffer buf = new StringBuffer();
    buf.append("_import_");
    buf.append(importTypeToString(importType));
    buf.append(".xsl");
    return buf.toString();
    }*/
    /**
     * retourne nul si importType incorrect.
     */
    public static String getImportXSLT(String name, BdfServer bdfServer, boolean withTabs) {
        short importType = 0;
        short format = XSL_FORMAT;
        int index = name.indexOf('_');
        if (index != -1) {
            String formatString = name.substring(index + 1);
            if (formatString.equals("odt")) {
                format = ODT_FORMAT;
            } else if (!formatString.equals("xsl")) {
                return null;
            }
            name = name.substring(0, index);
        }
        if (name.equals("fiche")) {
            importType = FICHE_IMPORT;
        } else if (name.equals("statthesaurus")) {
            importType = STATTHESAURUS_IMPORT;
        } else {
            return null;
        }
        return getImportXSLT(importType, bdfServer, format, withTabs);
    }

    /**
     *@throw IllegalArgumentException si le type est incorrect
     */
    public static String getImportXSLT(short importType, BdfServer bdfServer, short format, boolean withTabs) {
        StringBuilder buf = new StringBuilder();
        try {
            ImportXSLTWriter importXSLTWriter = new ImportXSLTWriter(importType, bdfServer);
            importXSLTWriter.setAppendable(buf);
            importXSLTWriter.setIndentLength((withTabs) ? 0 : -999);
            importXSLTWriter.appendXMLDeclaration();
            importXSLTWriter.addImportXSLT(format);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    /*public static String importTypeToString(short importType) {
    switch(importType) {
    case FICHE_IMPORT :
    return "fiche";
    case STATTHESAURUS_IMPORT :
    return "statthesaurus";
    default :
    throw new IllegalArgumentException("wrong importType : " + importType);
    }
    }*/
}
