/* BdfServer
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory;

import net.mapeadores.util.xml.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.l10n.*;


/**
 *
 * @author Vincent Calame
 */
public class LocaleStringsXMLWriter extends DefaultXMLWriter {

    LocaleBundle localeBundle;

    public LocaleStringsXMLWriter(LocaleBundle localeBundle) {
        this.localeBundle = localeBundle;
    }

    public void addLocaleStrings() throws IOException {
        openTag("localestrings");
        for (Iterator it = LocaleBundle.KEYLIST.iterator(); it.hasNext();) {
            addStringElement((String) it.next());
        }
        closeTag("localestrings");
    }

    private void addStringElement(String key) throws IOException {
        addStringElement(key, localeBundle.getLocalizedString(key));
    }

    private void addStringElement(String key, String value) throws IOException {
        startOpenTag("string");
        addAttribute("key", key);
        endOpenTag();
        escape(value);
        closeTag("string");
    }

    public static String getLocaleStringsXML(LocaleBundle localeBundle, boolean withTabs) {
        StringBuilder buf = new StringBuilder();
        try {
            LocaleStringsXMLWriter localeStringsWriter = new LocaleStringsXMLWriter(localeBundle);
            localeStringsWriter.setAppendable(buf);
            localeStringsWriter.setIndentLength((withTabs) ? 0 : -999);
            localeStringsWriter.appendXMLDeclaration();
            localeStringsWriter.addLocaleStrings();
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
