/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.odcss;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceUtils;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import net.mapeadores.opendocument.css.parse.CssSource;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.io.DocStream;


/**
 *
 * @author Vincent Calame
 */
public class BdfServerCssSource implements CssSource {

    private BdfServer bdfServer;
    private String resourcePath;

    private BdfServerCssSource(String resourcePath, BdfServer bdfServer) {
        this.resourcePath = resourcePath;
        this.bdfServer = bdfServer;
    }

    public Reader getReader() throws IOException {
        try {
            DocStream docStream = BdfResourceUtils.getBdfResourceDocStream(bdfServer.getBdfResourceStorage(), BdfResourceConstants.CSS_RESOURCE, resourcePath);
            if (docStream == null) {
                throw new ShouldNotOccurException("test of " + resourcePath + " done before");
            }
            Reader reader = new InputStreamReader(docStream.getInputStream(), "UTF-8");
            return reader;
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    public CssSource getImportCssSource(String url) {
        if ((url.startsWith("bdf:")) && (url.endsWith(".css"))) {
            return getBdfServerCssSource(url, bdfServer);
        }
        return null;
    }

    public static boolean isBdfServerCssSourceUrl(String url) {
        return ((url.startsWith("bdf:")) && (url.endsWith(".css")));
    }

    public static BdfServerCssSource getBdfServerCssSource(String url, BdfServer bdfServer) {
        if (!isBdfServerCssSourceUrl(url)) {
            return null;
        }
        String resourcePath = url.substring(4);
        if (!BdfResourceUtils.containsBdfResource(bdfServer.getBdfResourceStorage(), BdfResourceConstants.CSS_RESOURCE, resourcePath, true)) {
            return null;
        }
        return new BdfServerCssSource(resourcePath, bdfServer);
    }

}
