/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.ClauseTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.CommentTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.CorpsdeficheTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.FicheFieldAndIncludeTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.IdcorpusTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.ImportTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.InsertTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.LibTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.ModeTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.TitreTranslator;
import fr.exemole.bdfserver.xsltfactory.xslpattern.bih.VariableTranslator;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.text.insertpattern.api.InsertionBuffer;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.xml.PatternXSLTResolver;


/**
 *
 * @author Vincent Calame
 */
public class BdfCustomResolver implements PatternXSLTResolver.CustomResolver {

    private BdfServer bdfServer;
    private Map<Integer, InsertionUnitTranslator> insertionUnitTranslatorMap = new HashMap<Integer, InsertionUnitTranslator>();
    private PatternXSLTResolver patternXSLTResolver;
    private InsertionUnitTranslator insertTranslator;
    private InsertionUnitTranslator importTranslator;
    private InsertionUnitTranslator libTranslator;
    private InsertionUnitTranslator titreTranslator;
    private InsertionUnitTranslator idcorpusTranslator;
    private InsertionUnitTranslator commentTranslator;
    private InsertionUnitTranslator modeTranslator;
    private InsertionUnitTranslator corpsdeficheTranslator;
    private InsertionUnitTranslator variableTranslator;
    private FicheFieldAndIncludeTranslator currentFicheFieldAndIncludeTranslator;
    private InsertionUnitTranslator currentClauseTranslator;
    private File baseDirectory;
    private boolean insertLibsVar = false;

    public BdfCustomResolver(BdfServer bdfServer, File baseDirectory) {
        this.bdfServer = bdfServer;
        this.baseDirectory = baseDirectory;
    }

    public void setPatternXSLTResolver(PatternXSLTResolver pxw) {
        this.patternXSLTResolver = pxw;
        this.importTranslator = new ImportTranslator(pxw);
        this.variableTranslator = new VariableTranslator(pxw);
        this.insertTranslator = new InsertTranslator(bdfServer, pxw, baseDirectory);
        this.libTranslator = new LibTranslator(pxw);
        this.titreTranslator = new TitreTranslator(pxw);
        this.commentTranslator = new CommentTranslator(pxw);
        this.modeTranslator = new ModeTranslator(pxw);
        this.currentClauseTranslator = new ClauseTranslator(pxw, null);
        this.idcorpusTranslator = new IdcorpusTranslator(pxw);
        this.corpsdeficheTranslator = new CorpsdeficheTranslator(pxw);
        patternXSLTResolver.setCustomResolver(this);
    }

    public boolean isTemplateInstruction(InsertionUnit insertionUnit) {
        InsertionUnitTranslator insertionUnitTranslator = insertionUnitTranslatorMap.get(new Integer(insertionUnit.getIndex()));
        if (insertionUnitTranslator == null) {
            return false;
        }
        return insertionUnitTranslator.isTemplateInsertion(insertionUnit);
    }

    public void testInsertionUnit(InsertionUnit insertionUnit) {
        InsertionUnitTranslator insertionUnitTranslator = null;
        switch (insertionUnit.getType()) {
            case InsertionBuffer.DO_INSERTION:
                insertionUnitTranslator = testInsertion(insertionUnit.getIndex(), insertionUnit.getInstructionArray());
                break;
            case InsertionBuffer.START_IF:
            case InsertionBuffer.START_ELSEIF:
            case InsertionBuffer.START_ELSE:
            case InsertionBuffer.END_IF:
                insertionUnitTranslator = currentClauseTranslator;
                break;
        }
        if (insertionUnitTranslator != null) {
            insertionUnitTranslatorMap.put(insertionUnit.getIndex(), insertionUnitTranslator);
        }
    }

    public void resolveInsertionUnit(InsertionUnit insertionUnit) throws IOException {
        InsertionUnitTranslator insertionUnitTranslator = insertionUnitTranslatorMap.get(new Integer(insertionUnit.getIndex()));
        if (insertionUnitTranslator == null) {
            return;
        }
        insertionUnitTranslator.translate(insertionUnit);
    }

    private InsertionUnitTranslator testInsertion(int index, Instruction[] instructionArray) {
        String firstKey = instructionArray[0].getArgument(0).getKey();
        if (firstKey.equals("insert")) {
            return insertTranslator;
        } else if (firstKey.equals("import")) {
            return importTranslator;
        } else if (firstKey.equals("lib")) {
            return libTranslator;
        } else if (firstKey.equals("fiche")) {
            FicheFieldAndIncludeTranslator ficheFieldAndIncludeTranslator = new FicheFieldAndIncludeTranslator(patternXSLTResolver, index);
            ficheFieldAndIncludeTranslator.setInsertLibsVar(insertLibsVar);
            currentFicheFieldAndIncludeTranslator = ficheFieldAndIncludeTranslator;
            currentClauseTranslator = new ClauseTranslator(patternXSLTResolver, ficheFieldAndIncludeTranslator);
            return ficheFieldAndIncludeTranslator;
        } else if (firstKey.equals("idcorpus")) {
            return idcorpusTranslator;
        } else if (firstKey.equals("titre")) {
            return titreTranslator;
        } else if (firstKey.equals("texte")) {
            return corpsdeficheTranslator;
        } else if (firstKey.equals("mode")) {
            return modeTranslator;
        } else if (firstKey.startsWith("var_")) {
            if (firstKey.equals("var_libs")) {
                insertLibsVar = true;
            }
            return variableTranslator;
        } else if (firstKey.startsWith("comment_")) {
            return commentTranslator;
        } else if (firstKey.startsWith("annexe_")) {
            return corpsdeficheTranslator;
        } else if (firstKey.startsWith("fichesection")) {
            if (currentFicheFieldAndIncludeTranslator != null) {
                currentFicheFieldAndIncludeTranslator.checkFicheSection(index);
                return currentFicheFieldAndIncludeTranslator;
            } else {
                return null;
            }
        } else {
            if (currentFicheFieldAndIncludeTranslator != null) {
                currentFicheFieldAndIncludeTranslator.addKey(index, instructionArray);
                return currentFicheFieldAndIncludeTranslator;
            } else {
                return null;
            }
        }
    }

}
