/* BdfServer
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.include.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.ui.options.*;


/**
 *
 * @author Vincent Calame
 */
public class ComponentInfo {

    public final static short MODE_INLINE = 1;
    public final static short MODE_BLOCK = 2;
    public final static short TEXTE = 3;
    public final static short MODE_LISTITEM = 4;
    private String variableString;
    private short mode;
    private boolean forceWithIf = false;

    public ComponentInfo() {
    }

    public static ComponentInfo newInstance(CorpusField corpusField) {
        FieldKey fieldKey = corpusField.getFieldKey();
        ComponentInfo componentInfo = new ComponentInfo();
        if (fieldKey.ownsToCorpsdefiche()) {
            componentInfo.setMode(TEXTE);
        } else if (fieldKey.isInformationFieldKey()) {
            if (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField)) {
                switch (corpusField.getFicheItemType()) {
                    case CorpusField.ITEM_FIELD:
                    case CorpusField.PARA_FIELD:
                        componentInfo.setMode(MODE_BLOCK);
                        break;
                    default:
                        componentInfo.setMode(MODE_LISTITEM);
                }
            } else {
                componentInfo.setMode(MODE_INLINE);
            }
        } else {
            componentInfo.setMode(MODE_INLINE);
        }
        componentInfo.setVariableString(fieldKey.toString());
        return componentInfo;
    }

    public static ComponentInfo newInstance(IncludeUI includeUI) {
        FilteredIncludeKey fik = includeUI.getFiteredIncludeKey();
        ComponentInfo componentInfo = new ComponentInfo();
        componentInfo.setVariableString(fik.toString());
        IncludeKey includeKey = fik.getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            componentInfo.setMode(MODE_INLINE);
        } else {
            componentInfo.setMode(MODE_LISTITEM);
        }
        LimitOption limitOption = includeUI.getLimitOption();
        if (limitOption != null) {
            componentInfo.setForceWithIf(limitOption.isNoNewAllowed());
        }
        return componentInfo;
    }

    public boolean isInLine() {
        return (mode == MODE_INLINE);
    }

    public short getMode() {
        return mode;
    }

    public void setMode(short mode) {
        this.mode = mode;
    }

    public String getVariableString() {
        return variableString;
    }

    public void setVariableString(String variableString) {
        this.variableString = variableString;
    }

    public boolean isForceWithIf() {
        return forceWithIf;
    }

    public void setForceWithIf(boolean forceWithIf) {
        this.forceWithIf = forceWithIf;
    }

}
