/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.ui.corpus.*;
import java.io.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.xsltfactory.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.xsltfactory.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class FichePattern extends PatternBuffer {

    private boolean uniqueLang;
    boolean previousDiv;
    boolean withIf = false;
    private boolean multiple = false;
    private boolean currentForceWithIf = false;
    CorpusMetadata corpusMetadata;
    BdfServer bdfServer;

    public FichePattern() {
    }

    public void appendPatternStart(String name) {
        appendResource(name);
    }

    public void setWithIf(boolean withIf) {
        this.withIf = withIf;
    }

    public void setBdfServer(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        this.uniqueLang = langTest();
    }

    public void setPreviousDiv(boolean previousDiv) {
        this.previousDiv = previousDiv;
    }

    public boolean withSoustitre() {
        return CorpusMetadataUtils.hasSoustitreField(corpusMetadata);
    }

    public abstract boolean insertComment();

    public abstract void appendEntete();

    public boolean isPreviousDiv() {
        return previousDiv;
    }

    public void testNewLine() {
        if (!previousDiv) {
            newLine();
        }
        previousDiv = true;
    }

    public void appendCorpusPattern(Corpus corpus) {
        corpusMetadata = corpus.getCorpusMetadata();
        CorpusUI corpusUI = CorpusUIUtils.getCorpusUI(bdfServer.getUIManager(), corpus.getCorpusMetadata());
        newLine();
        newLine();
        appendSimpleInstruction("fiche", corpus.getSubsetKey().getSubsetName());
        newLine();
        newLine();

        appendEntete();
        newLine();
        setPreviousDiv(true);

        int p = 0;
        List liste = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);
        for (Iterator it = liste.iterator(); it.hasNext();) {
            Object obj = it.next();
            if (obj instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) obj;
                FieldKey fieldKey = fieldUI.getCorpusField().getFieldKey();
                if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
                    continue;
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    if (!uniqueLang) {
                        ComponentInfo componentInfo = ComponentInfo.newInstance(fieldUI.getCorpusField());
                        appendComponentInfo(componentInfo);
                    } else {
                        continue;
                    }
                } else {
                    ComponentInfo componentInfo = ComponentInfo.newInstance(fieldUI.getCorpusField());
                    appendComponentInfo(componentInfo);
                }
            } else if (obj instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) obj;
                ComponentInfo componentInfo = ComponentInfo.newInstance(includeUI);
                appendComponentInfo(componentInfo);
            } else if (obj instanceof CommentUI) {
                if (insertComment()) {
                    CommentUI commentUI = (CommentUI) obj;
                    if ((commentUI.getLocation() & CommentUI.AFFICHAGE_BITVALUE) != 0) {
                        testNewLine();
                        appendInstruction("comment_" + commentUI.getName());
                        newLine();
                        newLine();
                    }
                }
            }
            p++;
            if (p == 50) {
                testNewLine();
                appendInstruction("fichesection");
                newLine();
                newLine();
                p = 0;
            }
        }
        newLine();
        appendChrono();
    }

    public abstract void appendChrono();

    public abstract void appendInLine(ComponentInfo componentInfo);

    public abstract void appendBlock(ComponentInfo componentInfo);

    public abstract void appendListItem(ComponentInfo componentInfo);

    public abstract void appendTexte(ComponentInfo componentInfo);

    public void startIf(String clause) {
        if ((!withIf) && (!currentForceWithIf)) {
            return;
        }
        super.startIf(clause);
    }

    public void endIf() {
        if ((!withIf) && (!currentForceWithIf)) {
            return;
        }
        super.endIf();
    }

    private void appendComponentInfo(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        boolean old = currentForceWithIf;
        currentForceWithIf = componentInfo.isForceWithIf();
        short mode = componentInfo.getMode();
        switch (mode) {
            case ComponentInfo.MODE_INLINE:
                startIf(variableString);
                appendInLine(componentInfo);
                endIf();
                setPreviousDiv(false);
                break;
            case ComponentInfo.MODE_BLOCK:
                testNewLine();
                startIf(variableString);
                appendBlock(componentInfo);
                endIf();
                break;
            case ComponentInfo.MODE_LISTITEM:
                testNewLine();
                startIf(variableString);
                appendListItem(componentInfo);
                endIf();
                break;
            case ComponentInfo.TEXTE:
                testNewLine();
                startIf(variableString);
                appendTexte(componentInfo);
                endIf();
                break;
        }
        currentForceWithIf = old;
    }

    private boolean langTest() {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        int[] supplementaryArray = langConfig.getSupplementaryLangIntegerArray();
        if ((!langConfig.isAllLanguages()) && (workingArray.length + supplementaryArray.length == 1)) {
            return true;
        } else {
            return false;
        }
    }

    public boolean isMultiple() {
        return multiple;
    }

    public void setMultiple(boolean multiple) {
        this.multiple = multiple;
    }

}
