/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.ui.corpus.*;
import java.io.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.xsltfactory.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.xsltfactory.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class PatternBuffer implements XSLTFactoryConstants {

    private StringBuffer buf = new StringBuffer();

    public PatternBuffer() {
    }

    public String toString() {
        return buf.toString();
    }

    public boolean testPremier(boolean premier) {
        if (!premier) {
            addSeparator();
        }
        return false;
    }

    public void addSeparator() {
        buf.append(", ");
    }

    public void appendResource(String ressourceName) {
        try {
            InputStreamReader reader = new InputStreamReader(getClass().getResourceAsStream(ressourceName), "UTF-8");
            int n;
            char[] buffer = new char[1004];
            while ((n = reader.read(buffer)) != -1) {
                for (int i = 0; i < n; i++) {
                    append(buffer[i]);
                }
            }
            reader.close();
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    public void appendInstruction(String name) {
        buf.append('{');
        buf.append(name);
        buf.append('}');
    }

    public void appendLibInstruction(String value) {
        buf.append("{lib=");
        buf.append(value);
        buf.append('}');
    }

    public void appendSimpleInstruction(String name, String value) {
        buf.append('{');
        buf.append(name);
        buf.append('=');
        buf.append(value);
        buf.append('}');
    }

    public void appendModeInstruction(String name, String mode, String className, String sep) {
        buf.append('{');
        buf.append(name);
        buf.append("; ");
        buf.append("mode=");
        buf.append(mode);
        if (className != null) {
            buf.append(",");
            buf.append(" class=");
            buf.append(className);
        }
        if (sep != null) {
            buf.append(", ");
            buf.append(" sep=");
            buf.append("\"'");
            buf.append(sep);
            buf.append("'\"");
        }
        buf.append('}');
    }

    public void append(char carac) {
        buf.append(carac);
    }

    public void append(String s) {
        buf.append(s);
    }

    public void newLine() {
        buf.append('\n');
    }

    public void endIf() {
        append("{FI}");
        newLine();
    }

    public void startIf(String clause) {
        append("{IF ");
        append(clause);
        append("}");
        newLine();
    }

    public void openHtmlSpan(String className) {
        openTag("span", className, false);
    }

    public void openOdtSpan(String styleName) {
        openOdtTag("text:span", styleName, false);
    }

    public void closeHtmlSpan() {
        closeTag("span", false);
    }

    public void closeOdtSpan() {
        closeTag("text:span", false);
    }

    public void openHtmlP(String className) {
        openTag("p", className, false);
    }

    public void openOdtP(String styleName) {
        openOdtTag("text:p", styleName, false);
    }

    public void closeOdtP() {
        closeTag("text:p", true);
    }

    public void closeHtmlP() {
        closeTag("p", true);
    }

    public void openHtmlUl(String className) {
        openTag("ul", className, true);
    }

    public void openOdtUl(String styleName) {
        openOdtTag("text:list", styleName, true);
    }

    public void closeOdtUl() {
        closeTag("text:list", true);
    }

    public void closeHtmlUl() {
        closeTag("ul", true);
    }

    public void openOdtH(int level) {
        String levelString = String.valueOf(level);
        append("<");
        append("text:h");
        append(" text:style-name=\"");
        append("Heading_20_");
        append(levelString);
        append("\"");
        append(" text:outline-level=\"");
        append(levelString);
        append("\"");
        append(">");
    }

    public void closeOdtH() {
        closeTag("text:h", true);
    }

    public void openTag(String name, String className, boolean newLine) {
        append("<");
        append(name);
        if (className != null) {
            append(" class=\"");
            append(className);
            append("\"");
        }
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void openOdtTag(String name, String styleName, boolean newLine) {
        append("<");
        append(name);
        if (styleName != null) {
            append(" text:style-name=\"");
            append(styleName);
            append("\"");
        }
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void closeTag(String name, boolean newLine) {
        append("</");
        append(name);
        append(">");
        if (newLine) {
            newLine();
        }
    }

    public void openDiv(String className) {
        openTag("div", className, true);
    }

    public void closeDiv() {
        closeTag("div", true);
    }

    public void appendApplyTemplates(String select, String mode) {
        append("{_XAT= .");
        if (mode != null) {
            append(", mode= ");
            append(mode);
        }
        append("}");
    }

    public void appendValueOf(String select) {
        append("{_XVO= ");
        append(select);
        append("}");
    }

    public void appendTemplate(String select) {
        newLine();
        newLine();
        append("{_XT= ");
        append(select);
        append("}");
        newLine();
        newLine();
    }

}
