/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.xsltfactory.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.text.insertpattern.api.*;
import java.io.*;
import fr.exemole.bdfserver.*;


/**
 *
 * @author Vincent Calame
 */
public class PatternUtils implements XSLTFactoryConstants {

    private PatternUtils() {
    }

    public static PatternErrorHandler processPattern(File txtFile, String destinationName, BdfServer bdfServer) throws IOException {
        PatternErrorHandler patternErrorHandler = new PatternErrorHandler();
        InputStream inputStream = new FileInputStream(txtFile);
        String pattern = IOUtils.toString(inputStream, "UTF-8");
        inputStream.close();
        String xsl = getStyleSheet(bdfServer, pattern, patternErrorHandler, txtFile.getParentFile());
        if (!patternErrorHandler.hasError()) {
            File xslFile = new File(txtFile.getParentFile(), destinationName);
            StringReader reader = new StringReader(xsl);
            OutputStream outputStream = new FileOutputStream(xslFile);
            IOUtils.copy(reader, outputStream, "UTF-8");
            outputStream.close();
        }
        return patternErrorHandler;
    }

    public static String getStyleSheet(BdfServer bdfServer, String pattern, ErrorHandler errorHandler, File parentFile) {
        StringBuilder buf = new StringBuilder();
        PatternXSLTResolver xsltResolver = new PatternXSLTResolver();
        xsltResolver.setAppendable(buf, 0);
        BdfCustomResolver bdfCustomResolver = new BdfCustomResolver(bdfServer, parentFile);
        bdfCustomResolver.setPatternXSLTResolver(xsltResolver);
        StringReader sr = new StringReader(pattern);
        try {
            xsltResolver.parse(sr, errorHandler);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
