/* BdfServer
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.managers.transformation.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.include.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.ui.corpus.*;
import java.io.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.xsltfactory.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.xsltfactory.*;


/**
 *
 * @author Vincent Calame
 */
public class XslFichePattern extends FichePattern {

    public XslFichePattern() {
    }

    public boolean insertComment() {
        return true;
    }

    public void appendEntete() {
        openDiv("entete");
        boolean mult = isMultiple();
        String tagName = (mult) ? "h2" : "h1";
        openTag(tagName, null, false);
        appendModeInstruction("titre", "_intitule", null, null);
        closeTag(tagName, true);
        if (withSoustitre()) {
            startIf("soustitre");
            openP("soustitre");
            appendInstruction("soustitre");
            closeP();
            endIf();
        }
        closeDiv();
    }

    public void appendChrono() {
        openP("chrono");
        appendLibInstruction("date_creation");
        append(" : ");
        appendInstruction("date_creation");
        append(" — ");
        appendLibInstruction("date_modification");
        append(" : ");
        appendInstruction("date_modification");
        closeP();
    }

    public void appendInLine(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        openP("champ");
        openSpan("intitule");
        appendLibInstruction(variableString);
        append(" : ");
        closeSpan();
        appendInstruction(variableString);
        closeP();
    }

    public void appendListItem(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        appendIntituleP(variableString, "intitule");
        openUl("block");
        appendModeInstruction(variableString, "_listitem", null, null);
        closeUl();
    }

    public void appendBlock(ComponentInfo componentInfo) {
        appendBlock(componentInfo.getVariableString(), "intitule", "block", "_block");
    }

    public void appendTexte(ComponentInfo componentInfo) {
        appendBlock(componentInfo.getVariableString(), "intituletexte", "texte", null);
    }

    private void appendBlock(String variableString, String className, String divClassName, String mode) {
        appendIntituleP(variableString, className);
        openDiv(divClassName);
        if (mode == null) {
            appendInstruction(variableString);
        } else {
            appendModeInstruction(variableString, mode, null, null);
        }
        newLine();
        closeDiv();
    }

    private void appendIntituleP(String variableString, String className) {
        openP(className);
        appendLibInstruction(variableString);
        append(" :");
        closeP();
    }

    public void openP(String className) {
        openHtmlP(className);
    }

    public void closeP() {
        closeHtmlP();
    }

    public void openSpan(String className) {
        openHtmlSpan(className);
    }

    public void closeSpan() {
        closeHtmlSpan();
    }

    public void openUl(String className) {
        openHtmlUl(className);
    }

    public void closeUl() {
        closeHtmlUl();
    }

}
