/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern.bih;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.Range;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.Instruction;


/**
 *
 * @author Vincent Calame
 */
public class BihUtils {

    private BihUtils() {
    }

    public static String checkMode(String mode) {
        if (mode == null) {
            return null;
        }
        if (mode.equals("_inline")) {
            return "MODE_INLINE";
        }
        if (mode.equals("_block")) {
            return "MODE_BLOCK";
        }
        if (mode.equals("_listitem")) {
            return "MODE_LISTITEM";
        }
        if (mode.equals("_intitule")) {
            return "MODE_INTITULE";
        }
        if (mode.equals("_motclelib")) {
            return "MODE_MOTCLELIB";
        }
        if (mode.equals("_prgcontent")) {
            return "MODE_PRGCONTENT";
        }
        if (mode.equals("_code")) {
            return "MODE_CODE";
        }
        return mode;
    }

    public static String checkValue(String value) {
        if (value == null) {
            return null;
        }
        if (value.length() == 0) {
            return null;
        }
        return value;
    }

    public static void appendPoidsSelect(RangeList rangeList, StringBuilder buf) {
        int count = rangeList.size();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(" and ");
            }
            Range range = rangeList.getRange(i);
            appendRange(range, buf);
        }
    }

    public static void appendRange(Range range, StringBuilder buf) {
        if (range.length() == 1) {
            buf.append("@poids !=");
            buf.append(range.min());
        } else {
            buf.append("(@poids < ");
            buf.append(range.min());
            buf.append(" or ");
            buf.append("@poids > ");
            buf.append(range.max());
            buf.append(")");
        }
    }

    public static void appendWithPoidsFilterIncludeKey(StringBuilder buf, FilteredIncludeKey filteredIncludeKey) {
        IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
        int poidsFilter = filteredIncludeKey.getPoidsFilter();
        if (includeKey.isIndexationIncludeKey()) {
            appendIndexationPath(buf, includeKey, true);

        } else {
            appendLiagePath(buf, includeKey, true);
        }
        buf.append("[@poids=");
        buf.append(poidsFilter);
        buf.append("]");
    }

    public static void appendChronoIncludeKey(StringBuilder buf) {
        buf.append("chrono/datation");
    }

    public static void appendChronoIncludeKey(StringBuilder buf, IncludeKey includeKey) {
        if (includeKey.equals(IncludeKey.DATE_CREATION)) {
            buf.append("chrono/datation[1]");
        } else if (includeKey.equals(IncludeKey.DATE_MODIFICATION)) {
            buf.append("chrono/datation[position() = last()]");
        } else {
            throw new SwitchException("unknown includeKey = " + includeKey.toString());
        }
    }

    public static void appendAssocation(StringBuilder buf) {
        buf.append("document");
    }

    public static void appendIndexationPath(StringBuilder path, IncludeKey includeKey, boolean withChildren) {
        path.append("extraitthesaurus[@thesaurus='");
        path.append(includeKey.getSubsetKey().getSubsetName());
        path.append("']");
        if (withChildren) {
            path.append("/motcle");
        }
    }

    public static void appendLiagePath(StringBuilder path, IncludeKey includeKey, boolean withChildren) {
        path.append("extraitcorpus[@corpus='");
        path.append(includeKey.getSubsetKey().getSubsetName());
        path.append("']");
        if (withChildren) {
            path.append("/fiche");
        }

    }

    public static void appendRattachementPath(StringBuilder path, boolean withChildren) {
        path.append("extraitcorpus[@rattachement]");
        if (withChildren) {
            path.append("/fiche");
        }
        path.append(" | extraitthesaurus[@rattachement]");
        if (withChildren) {
            path.append("/motcle");
        }
    }

    public static String getMode(Instruction[] instructionArray) {
        int length = instructionArray.length;
        String mode = null;
        if (length > 1) {
            Instruction secondInstruction = instructionArray[1];
            int lg2 = secondInstruction.getLength();
            for (int i = 0; i < lg2; i++) {
                Argument argument = secondInstruction.getArgument(i);
                String key = argument.getKey();
                String value = BihUtils.checkValue(argument.getValue());
                if (key.equals("mode")) {
                    mode = value;
                }
            }
        }
        return mode;
    }

    public static void appendPath(StringBuilder buf, FieldKey fieldKey, boolean typetag, boolean withChildren) {
        String typeString = fieldKey.getTypeString();
        if (fieldKey.isProprieteFieldKey()) {
            buf.append("propriete");
            if (typetag) {
                buf.append("_");
                buf.append(typeString);
            } else {
                buf.append("[@type='");
                buf.append(typeString);
                buf.append("']");
            }
        } else if (fieldKey.isAnnexeFieldKey()) {
            buf.append("annexe");
            if (typetag) {
                buf.append("_");
                buf.append(typeString);
            } else {
                buf.append("[@type='");
                buf.append(typeString);
                buf.append("']");
            }
        } else if (fieldKey.isInformationFieldKey()) {
            buf.append("information");
            if (typetag) {
                buf.append("_");
                buf.append(typeString);
            } else {
                buf.append("[@type='");
                buf.append(typeString);
                buf.append("']");
            }
        } else if (fieldKey.equals(FieldKey.IDCORPUS)) {
            buf.append("@idcorpus");
        } else if (fieldKey.equals(FieldKey.LANG)) {
            buf.append("lang");
        } else if (fieldKey.equals(FieldKey.TITRE)) {
            buf.append("titre");
        } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            buf.append("soustitre");
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            buf.append("redacteurs");
        } else if (fieldKey.equals(FieldKey.TEXTE)) {
            buf.append("texte");
        } else {
            throw new SwitchException("fieldKey = " + fieldKey.toString());
        }
        if (withChildren) {
            buf.append("/*");
        }
    }

}
