/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern.bih;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceUtils;
import fr.exemole.bdfserver.xsltfactory.xslpattern.InsertionUnitTranslator;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.xml.PatternXSLTResolver;


/**
 *
 * @author Vincent Calame
 */
public class InsertTranslator implements InsertionUnitTranslator {

    private final static String CSS_BDFRESOURCE_PREFIX = "%CSS/";
    private BdfServer bdfServer;
    private File baseDirectory;
    private PatternXSLTResolver patternXSLTResolver;

    public InsertTranslator(BdfServer bdfServer, PatternXSLTResolver pxw, File baseDirectory) {
        this.bdfServer = bdfServer;
        this.patternXSLTResolver = pxw;
        this.baseDirectory = baseDirectory;
    }

    public void translate(InsertionUnit insertionUnit) throws IOException {
        Instruction[] array = insertionUnit.getInstructionArray();
        Instruction instruction = array[0];
        String arg = instruction.getArgument(0).getValue();
        if (arg != null) {
            appendResource(arg);
        }
        int size = instruction.getLength();
        for (int i = 1; i < size; i++) {
            String key = instruction.getArgument(i).getKey();
            appendResource(key);
        }
    }

    public boolean isTemplateInsertion(InsertionUnit insertionUnit) {
        return false;
    }

    private void appendResource(String ressourceName) throws IOException {
        try {
            InputStreamReader reader = null;
            if (ressourceName.startsWith(CSS_BDFRESOURCE_PREFIX)) {
                String rn2 = ressourceName.substring(CSS_BDFRESOURCE_PREFIX.length());
                DocStream docStream = BdfResourceUtils.getBdfResourceDocStream(bdfServer.getBdfResourceStorage(), BdfResourceConstants.CSS_RESOURCE, rn2);
                if (docStream != null) {
                    reader = new InputStreamReader(docStream.getInputStream(), "UTF-8");
                } else {
                    patternXSLTResolver.getCurrentXSLTWriter().append("#UNKNWON DEFAULT RESOURCE = " + ressourceName);
                    return;
                }
            } else {
                File file = new File(baseDirectory, ressourceName);
                ressourceName = file.getAbsolutePath();
                if (!file.exists()) {
                    patternXSLTResolver.getCurrentXSLTWriter().append("#MISSING RESOURCE = " + ressourceName);
                    return;
                }
                reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
            }
            int n;
            char[] buffer = new char[1004];
            while ((n = reader.read(buffer)) != -1) {
                for (int i = 0; i < n; i++) {
                    try {
                        patternXSLTResolver.getCurrentXSLTWriter().append(buffer[i]);
                    } catch (IOException ioe) {
                        throw new NestedIOException(ioe);
                    }
                }
            }
            reader.close();
        } catch (IOException ioe) {
            patternXSLTResolver.getCurrentXSLTWriter().append("#RESOURCE ERROR = " + ressourceName + " : " + ioe.getMessage() + "#");
        }
    }

}
