/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern.bih;

import fr.exemole.bdfserver.xsltfactory.xslpattern.InsertionUnitTranslator;
import java.io.IOException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.basedefiches.include.IncludeUtils;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.xml.PatternXSLTResolver;
import net.mapeadores.util.xml.XSLTWriter;


/**
 *
 * @author Vincent Calame
 */
public class LibTranslator implements InsertionUnitTranslator {

    private PatternXSLTResolver xsltResolver;

    public LibTranslator(PatternXSLTResolver xsltResolver) {
        this.xsltResolver = xsltResolver;
    }

    public void translate(InsertionUnit insertionUnit) throws IOException {
        Instruction[] instructionArray = insertionUnit.getInstructionArray();
        XSLTWriter xw = xsltResolver.getCurrentXSLTWriter();
        String keyString = instructionArray[0].getArgument(0).getValue();
        if ((keyString == null) || (keyString.length() == 0)) {
            xw.append("#ERROR : ");
            xw.append("empty lib");
            xw.append(" #");
            return;
        }
        Object key = IncludeUtils.parseKey(keyString);
        if (key == null) {
            xw.append("#ERROR : ");
            xw.append("lib = ");
            xw.append(keyString);
            xw.append(" #");
            return;
        }
        if (key instanceof FieldKey) {
            xw.addXslValueOf("$libs_metadata/lib[@field-key='" + key.toString() + "']");
        } else if (key instanceof FilteredIncludeKey) {
            IncludeKey includeKey = ((FilteredIncludeKey) key).getIncludeKey();
            xw.addXslValueOf("$libs_corpusui/lib[@include-key='" + key.toString() + "']");
        }
    }

    public boolean isTemplateInsertion(InsertionUnit insertionUnit) {
        return false;
    }

}
