/* BdfServer_Commands - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands;

import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractBdfCommand implements BdfCommand {

    public static final String YES_PARAMVALUE = "1";
    public static final String NO_PARAMVALUE = "0";
    protected BdfServer bdfServer;
    protected BdfUser bdfUser;
    protected BdfBundle bdfBundle;
    protected BdfRequestMap requestMap;

    public AbstractBdfCommand() {
    }

    public void setBdfBundle(BdfBundle bdfBundle) {
        this.bdfBundle = bdfBundle;
        this.bdfServer = bdfBundle.getBdfServer();
        this.bdfUser = bdfBundle.getBdfUser();
    }

    public void setRequestMap(BdfRequestMap requestMap) {
        this.requestMap = requestMap;
    }

    public CommandTest testCommand() {
        try {
            CommandTest commandTest = checkParameters();
            return commandTest;
        } catch (ParameterException pe) {
            return convertToErrorCommandText(pe);
        }

    }

    public static CommandTest convertToErrorCommandText(ParameterException pe) {
        CommandMessage errorMessage = pe.getCmdMessage();
        if (errorMessage != null) {
            return ErrorCommandTest.newInstance(errorMessage);
        } else {
            return ErrorCommandTest.newSimpleErrorInstance("global_err_parameter", pe.getText());
        }
    }

    /**
     * @todo : rendre abstrait, ne fait rien
     */
    protected CommandTest checkParameters() throws ParameterException {
        return new SuccessCommandTest();
    }

    public String getMandatoryParameter(String name) throws ParameterException {
        String param = requestMap.getParameter(name);
        if (param == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, name);
        }
        return param;
    }

    public boolean getBooleanParameter(String name) throws ParameterException {
        String param = requestMap.getParameter(name);
        if (param == null) {
            return false;
        }
        if (param.equals(YES_PARAMVALUE)) {
            return true;
        }
        if (param.equals(NO_PARAMVALUE)) {
            return false;
        }
        throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, name + " = " + param);
    }

    public int getLangParameter(String name, boolean mandatory) throws ParameterException {
        String langString = null;
        if (mandatory) {
            langString = getMandatoryParameter(name);
        } else {
            langString = requestMap.getParameter(name);
            if (langString == null) {
                return LangInteger.UNDETERMINED;
            }
        }
        try {
            return LangInteger.parse(langString);
        } catch (LangIntegerException lie) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, name + " = " + langString);
        }
    }

    public short getCommandType() {
        return NORMAL_COMMAND;
    }

}
