/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import java.text.ParseException;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class AlbumDimCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "AlbumDimCreation";
    public final static String COMMANDKEY = "ALB-03";
    public final static String NEWALBUMDIM_PARAMNAME = "newalbumdim";
    public final static String ALBUMDIMTYPE_PARAMNAME = "albumdimtype";
    private Album album;
    private String newAlbumDimName;
    private short albumDimType;

    public AlbumDimCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        AlbumEditor albumEditor = basedefichesEditor.getAlbumEditor(album.getSubsetKey());
        try {
            AlbumDim albumDim = albumEditor.getAlbumMetadataEditor().createAlbumDim(newAlbumDimName, albumDimType);
            setDone("album_done_albumdimcreation", newAlbumDimName);
            putResultObject(ALBUM_OBJ, album);
            putResultObject(ALBUMDIM_OBJ, albumDim);
        } catch (ParseException pe) {
            throw new ShouldNotOccurException("test done before", pe);
        } catch (ExistingNameException ee) {
            throw new ShouldNotOccurException("test done before", ee);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        album = BdfCommand2Utils.getMandatoryAlbum(basedefiches, requestMap);
        newAlbumDimName = getMandatoryParameter(NEWALBUMDIM_PARAMNAME);
        newAlbumDimName = newAlbumDimName.trim();
        if (newAlbumDimName.length() == 0) {
            setError("album_err_missingalbumdimname", null);
            return false;
        }
        if (album.getAlbumMetadata().getAlbumDimByName(newAlbumDimName) != null) {
            setError("album_err_existingalbumdimname", newAlbumDimName);
            return false;
        }
        if (!AlbumUtils.isValidAlbumDimName(newAlbumDimName)) {
            setError("album_err_wrongalbumdimname", newAlbumDimName);
            return false;
        }
        String albumDimTypeString = getMandatoryParameter(ALBUMDIMTYPE_PARAMNAME);
        try {
            albumDimType = AlbumUtils.dimTypeToShort(albumDimTypeString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, ALBUMDIMTYPE_PARAMNAME, albumDimTypeString);
        }
        return true;
    }

}
