/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class DimUsageModifCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "DimUsageModif";
    public final static String COMMANDKEY = "ALB-06";
    public final static String VIGNETTE_PARAMNAME = "vignette";
    public final static String DETAIL_PARAMNAME = "detail";
    private Album album;
    private Map<String, AlbumDim> usageMap = new HashMap<String, AlbumDim>();

    public DimUsageModifCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        AlbumMetadataEditor albumMetadataEditor = basedefichesEditor.getAlbumEditor(album.getSubsetKey()).getAlbumMetadataEditor();
        boolean done = false;
        for (Map.Entry<String, AlbumDim> entry : usageMap.entrySet()) {
            String usageName = entry.getKey();
            AlbumDim albumDim = entry.getValue();
            boolean done2 = albumMetadataEditor.setAlbumDimForUsage(usageName, albumDim);
            if (done2) {
                done = true;
            }
        }
        if (done) {
            setDone("album_done_dimusagemodif", null);
        }
        putResultObject(ALBUM_OBJ, album);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        album = BdfCommand2Utils.getMandatoryAlbum(basedefiches, requestMap);
        String vignetteName = BdfCommand2Utils.getMandatoryParameter(requestMap, VIGNETTE_PARAMNAME).trim();
        String detailName = BdfCommand2Utils.getMandatoryParameter(requestMap, DETAIL_PARAMNAME).trim();
        AlbumDim vignetteDim = null;
        AlbumDim detailDim = null;
        if (vignetteName.length() > 0) {
            vignetteDim = album.getAlbumMetadata().getAlbumDimByName(vignetteName);
        }
        if (detailName.length() > 0) {
            detailDim = album.getAlbumMetadata().getAlbumDimByName(detailName);
        }
        usageMap.put(AlbumConstants.VIGNETTE_USAGE, vignetteDim);
        usageMap.put(AlbumConstants.DETAIL_USAGE, detailDim);
        return true;
    }

}
