/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import java.io.IOException;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class BdfResourceUpdateCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "BdfResourceUpdate";
    public final static String ICON16_COMMANDKEY = "CNF-07";
    public final static String ICON32_COMMANDKEY = "CNF-08";
    public final static String LOGO_COMMANDKEY = "CNF-09";
    public final static String RSCPATH_PARAMNAME = "rscpath";
    public final static String RSCFILE_PARAMNAME = "rscfile";
    private String resourcePath;

    public BdfResourceUpdateCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        FileValue fileValue = requestMap.getFileValue(RSCFILE_PARAMNAME);
        if (fileValue == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.MANDATORY, RSCFILE_PARAMNAME, null);
        }
        boolean done = false;
        if (fileValue.length() > 0) {
            try {
                bdfServer.getBdfResourceStorage().saveBdfResource(BdfResourceConstants.OTHER_RESOURCE, resourcePath, fileValue.getInputStream());
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
            done = true;
        }
        fileValue.free();
        if (done) {
            setDone("configuration_done_rsc", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        resourcePath = BdfCommand2Utils.getMandatoryParameter(requestMap, RSCPATH_PARAMNAME);
        return true;
    }

}
