/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.TypoOptions;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import java.util.Set;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class GroupChangeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "GroupChange";
    public final static String COMMANDKEY = "CNF-11";
    public final static String GROUPNAME_PARAMNAME = "groupname";
    public final static String GROUP_PARAMPREFIX = "group";
    private String groupName;

    public GroupChangeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        SubsetKeyGroupTemplates subsetKeyGroupTemplates = bdfServer.getUIManager().getSubsetKeyGroupTemplates();
        updateGroup(subsetKeyGroupTemplates, groupName, requestMap);
        setDone("configuration_done_groupchange", groupName);
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        SubsetKeyGroupTemplates subsetKeyGroupTemplates = bdfServer.getUIManager().getSubsetKeyGroupTemplates();
        groupName = BdfCommand2Utils.getMandatoryParameter(requestMap, GROUPNAME_PARAMNAME);
        if (!subsetKeyGroupTemplates.containsGroupTemplate(groupName)) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, GROUPNAME_PARAMNAME, groupName);
        }
        return true;
    }

    public static void updateGroup(SubsetKeyGroupTemplates templates, String groupName, RequestMap requestMap) {
        Set<String> set = requestMap.getParameterNameSet();
        TextParser textParser = new TextParser();
        for (String paramName : set) {
            if (!paramName.startsWith("group/")) {
                continue;
            }
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(6));
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
                textParser.setQuoteOptions(typoOptions.getQuoteOptions());
                String nvlibelle = textParser.parseString(requestMap.getParameter(paramName));
                templates.setLibelleString(groupName, currentLangInteger, nvlibelle);
            } catch (LangIntegerException ile) {
            }
        }
    }

}
