/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class IdentificationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Identification";
    public final static String COMMANDKEY = "CNF-01";
    public final static String AUTHORITYUUID_PARAMNAME = "authorityuuid";
    public final static String BASENAME_PARAMNAME = "basename";
    private String baseName;
    private String authorityUUID;

    public IdentificationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        BasedefichesMetadataEditor basedefichesMetadataEditor = bdfServer.getBasedefichesEditor().getBasedefichesMetadataEditor();
        boolean done1 = basedefichesMetadataEditor.setAuthorityUUID(authorityUUID);
        boolean done2 = basedefichesMetadataEditor.setBasedefichesName(baseName);
        if ((done1) || (done2)) {
            setDone("configuration_done_core", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        baseName = getMandatoryParameter(BASENAME_PARAMNAME);
        authorityUUID = getMandatoryParameter(AUTHORITYUUID_PARAMNAME);
        if (!StringUtils.isTechnicalName(baseName, true)) {
            addMultiError("configuration_err_basename", baseName);
        }
        if (!StringUtils.checkUUID(authorityUUID)) {
            addMultiError("configuration_err_authorityuuid", authorityUUID);
        }
        if (hasMultiError()) {
            setMultiErrorTitle("configuration_err_core", null);
            return false;
        }
        return true;
    }

}
