/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.LinkPathGroup;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;

/**
 *
 * @author Vincent Calame
 */
public class LinkPathGroupCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "LinkPathGroup";
    public final static String COMMANDKEY = "CNF-17";
    public final static String GROUP_PARAMPREFIX = "group|";
    private List<LinkPathGroup> linkPathGroupList = new ArrayList<LinkPathGroup>();

    public LinkPathGroupCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        for(LinkPathGroup linkPathGroup : linkPathGroupList) {
            String groupName = linkPathGroup.getName();
            if (linkPathGroup.getLinkPathCount() == 0) {
                configurationManager.removeLinkPathGroup(groupName);
            } else {
                configurationManager.setLinkPathGroup(linkPathGroup);
            }
        }
        bdfServer.saveConfigurationManager();
        setDone("configuration_done_linkpathgroup", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        Set<String> paramNameSet = requestMap.getParameterNameSet();
        for(String paramName : paramNameSet) {
            if (paramName.startsWith(GROUP_PARAMPREFIX)) {
                String groupName = paramName.substring(GROUP_PARAMPREFIX.length());
                LinkPathGroup linkPathGroup = new LinkPathGroup(groupName);
                linkPathGroupList.add(linkPathGroup);
                String value = requestMap.getParameter(paramName);
                String[] tokens = StringUtils.getLineTokens(value, StringUtils.EMPTY_EXCLUDE);
                int length = tokens.length;
                for(int i = 0; i < length; i++) {
                    linkPathGroup.addLinkPath(tokens[i]);
                }
            }
        }
        return true;
    }



}
