/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPath;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPathException;
import fr.exemole.bdfserver.managers.configuration.ConfigurationUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URL;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class PathsCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Paths";
    public final static String COMMANDKEY = "CNF-06";
    public final static String WWWROOT_PARAMNAME = "wwwroot";
    public final static String DIRECTORY_PARAMPREFIX = "directory_";
    public final static String EXTERNALURL_PARAMPREFIX = "externalurl_";
    private String wwwPath;
    private ConfigurationPath publicPath;
    private ConfigurationPath privatePath;

    public PathsCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        boolean done = false;
        try {
            boolean done1 = configurationManager.setSystemDirectory(ConfigurationManager.WWW_ROOT, wwwPath);
            if (done1) {
                done = true;
            }
            boolean done2 = configurationManager.setConfigurationPath(publicPath);
            if (done2) {
                done = true;
            }
            boolean done3 = configurationManager.setConfigurationPath(privatePath);
            if (done3) {
                done = true;
            }
        } catch (ConfigurationPathException cpe) {
            throw new ShouldNotOccurException("test done before");
        }
        if (done) {
            bdfServer.saveConfigurationManager();
            setDone("configuration_done_paths", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        wwwPath = BdfCommand2Utils.getMandatoryParameter(requestMap, WWWROOT_PARAMNAME);
        try {
            ConfigurationUtils.toDirectory(wwwPath);
        } catch (FileNotFoundException fffne) {
            addMultiError("configuration_err_path_directory", wwwPath);
        }
        publicPath = initConfigurationPath(ConfigurationManager.PUBLIC_OUTPUTDIRECTORY);
        privatePath = initConfigurationPath(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY);
        if (hasMultiError()) {
            setMultiErrorTitle("configuration_err_paths", null);
            return false;
        }
        return true;
    }

    private ConfigurationPath initConfigurationPath(String name) throws BdfCommand2Exception {
        String directoryValue = getMandatoryParameter(DIRECTORY_PARAMPREFIX + name);
        String externalUrl = getMandatoryParameter(EXTERNALURL_PARAMPREFIX + name);
        boolean error = false;
        URL publicURL = null;
        File systemDirectory = null;
        try {
            publicURL = ConfigurationUtils.toURL(externalUrl);
        } catch (MalformedURLException mue) {
            addMultiError("configuration_err_path_externalurl", externalUrl);
            error = true;
        }
        try {
            systemDirectory = ConfigurationUtils.toDirectory(directoryValue);
        } catch (FileNotFoundException fnne) {
            addMultiError("configuration_err_path_directory", directoryValue);
            error = true;
        }
        if (error) {
            return null;
        }
        ConfigurationPath configurationPath = new ConfigurationPath(name, publicURL, systemDirectory);
        return configurationPath;
    }

}
