/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.ui.SubsetKeyTreeUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class PositionCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Position";
    public final static String CORPUS_COMMANDKEY = "CNF-13";
    public final static String THESAURUS_COMMANDKEY = "CNF-14";
    public final static String SPHERE_COMMANDKEY = "CNF-15";
    public final static String GROUPTYPE_PARAMNAME = "grouptype";
    public final static String SUBSETPOSITION_PARAMNAME = "subsetposition";
    private String subsetPosition;
    private short type;

    public PositionCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        SubsetKeyTree subsetKeyTree = getSubsetKeyTree(type);
        subsetKeyTree.clear();
        String[] tokens = StringUtils.getLineTokens(subsetPosition, StringUtils.EMPTY_EXCLUDE);
        SubsetKeyTreeUtils.parseTokens(subsetKeyTree, tokens);
        subsetKeyTree.checkBasedefiches(bdfServer.getBasedefiches());
        setDone("configuration_done_ordre", null);
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String typeString = BdfCommand2Utils.getMandatoryParameter(requestMap, GROUPTYPE_PARAMNAME);
        try {
            type = SubsetKeyTree.toShortType(typeString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, GROUPTYPE_PARAMNAME, typeString);
        }
        subsetPosition = BdfCommand2Utils.getMandatoryParameter(requestMap, SUBSETPOSITION_PARAMNAME);
        return true;
    }

    private SubsetKeyTree getSubsetKeyTree(short type) {
        UIManager uiManager = bdfServer.getUIManager();
        switch (type) {
            case SubsetKeyTree.CORPUS_GROUP:
                return uiManager.getCorpusSubsetKeyTree();
            case SubsetKeyTree.THESAURUS_GROUP:
                return uiManager.getThesaurusSubsetKeyTree();
            case SubsetKeyTree.SPHERE_GROUP:
                return uiManager.getSphereSubsetKeyTree();
            default:
                throw new SwitchException("type = " + type);
        }
    }

}
