/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPathException;
import fr.exemole.bdfserver.managers.configuration.ConfigurationUtils;
import java.net.MalformedURLException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class UrlCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Url";
    public final static String COMMANDKEY = "CNF-02";
    public final static String INSERTURL_PARAMNAME = "inserturl";
    private String insertUrlString;

    public UrlCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        try {
            boolean done = configurationManager.setPublicURL(ConfigurationManager.INSERT_URL, insertUrlString);
            if (done) {
                setDone("configuration_done_core", null);
                bdfServer.saveConfigurationManager();
                bdfServer.updateTransformation();
            }
        } catch (ConfigurationPathException cpe) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        insertUrlString = getMandatoryParameter(INSERTURL_PARAMNAME);
        try {
            ConfigurationUtils.toURL(insertUrlString);
            return true;
        } catch (MalformedURLException mue) {
            setError("configuration_err_inserturl", insertUrlString);
            return false;
        }
    }

}
