/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.PolicyManager;
import net.mapeadores.util.request.RequestMap;

/**
 *
 * @author Vincent Calame
 */
public class UserAllowChangeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "UserAllowChange";
    public final static String COMMANDKEY = "CNF-05";
    public final static String DATA_PARAMNAME = "data";
    public final static String PASSWORD_PARAMNAME = "password";
    private boolean allowData = false;
    private boolean allowPassword = false;

    public UserAllowChangeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        PolicyManager policyManager = bdfServer.getPolicyManager();
        boolean done = false;
        if (policyManager.setDataModifAllowed(allowData)) {
            done = true;
        }
        if (policyManager.setPasswordModifAllowed(allowPassword)) {
            done = true;
        }
        if (done) {
            bdfServer.savePolicyManager();
            setDone("configuration_done_userallowchange", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        allowData = BdfCommand2Utils.getBooleanParameter(requestMap,DATA_PARAMNAME);
        allowPassword = BdfCommand2Utils.getBooleanParameter(requestMap,PASSWORD_PARAMNAME);
        return true;
    }
}
