/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.saisie;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.request.parsers.SaisieEngine;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.NoMasterIdsubsetException;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class FicheCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Create";
    public final static String COMMANDKEY = "SAI-01";
    private Corpus corpus;

    public FicheCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
        AbregeFiche abregeFiche = null;
        try {
            abregeFiche = corpusEditor.createFiche(-1);
        } catch (ExistingIdException eii) {
            throw new ImplementationException(eii);
        } catch (NoMasterIdsubsetException nmie) {
            throw new ShouldNotOccurException(nmie);
        }
        corpusEditor.setDate(abregeFiche, DateInteger.current(), false);
        SaisieEngine saisieEngine = new SaisieEngine(bdfServer, bdfUser, requestMap, SaisieEngine.ALL_MASK);
        saisieEngine.updateFiche(abregeFiche);
        setDone("saisie_done_create", BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
        putResultObject(CORPUS_OBJ, corpus);
        putResultObject(ABREGEFICHE_OBJ, abregeFiche);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        if (corpus.getMasterSubset() != null) {
            throw new BdfCommand2Exception("corpus.getMasterSubset() != null");
        }
        BdfCommand2Utils.testCreateFichePermission(bdfServer, bdfUser, corpus, -1);
        return true;
    }

}
