/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurChangeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurChange";
    public final static String COMMANDKEY = "USR-01";
    public final static String NOM_PARAMNAME = "nom";
    public final static String ORIGINAL_PARAMNAME = "original";
    public final static String PRENOM_PARAMNAME = "prenom";
    public final static String ADDRSPEC_PARAMNAME = "addrspec";
    public final static String NOMAVANT_PARAMNAME = "nomavant";
    private PersonneBean personneBean;
    private CourrielBean courrielBean;
    private Redacteur redacteur;

    public RedacteurChangeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        if (!bdfServer.getPolicyManager().isDataModifAllowed()) {
            return ADMIN_ONLY_BDFUSER;
        } else {
            return MANDATORY_BDFUSER;
        }
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        SphereEditor sphereEditor = bdfServer.getBasedefichesEditor().getSphereEditor(redacteur.getSphere().getSubsetKey());
        sphereEditor.setPersonneBean(redacteur, personneBean);
        sphereEditor.setCourrielBean(redacteur, courrielBean);
        setDone("sphere_done_change", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        if (!BdfUserUtils.isAdmin(bdfUser)) {
            if (!bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
                return false;
            }
        }
        boolean done = initCourrielBean();
        if (!done) {
            return false;
        }
        initPersonneBean();
        return true;
    }

    private boolean initCourrielBean() {
        String addrspec = StringUtils.cleanString(requestMap.getParameter(ADDRSPEC_PARAMNAME));
        if ((addrspec == null) || (addrspec.length() == 0)) {
            courrielBean = null;
            return true;
        }
        try {
            courrielBean = CourrielBean.parse(addrspec);
            return true;
        } catch (java.text.ParseException pe) {
            setError("sphere_err_courriel", addrspec);
            return false;
        }
    }

    private void initPersonneBean() throws BdfCommand2Exception {
        String nom = BdfCommand2Utils.getMandatoryParameter(requestMap, NOM_PARAMNAME);
        nom = StringUtils.cleanString(nom);
        String prenom = StringUtils.cleanString(requestMap.getParameter(PRENOM_PARAMNAME));
        if (prenom == null) {
            prenom = "";
        }
        String original = StringUtils.cleanString(requestMap.getParameter(ORIGINAL_PARAMNAME));
        if (original == null) {
            original = "";
        }
        boolean nomAvant = false;
        nomAvant = BdfCommand2Utils.getBooleanParameter(requestMap, NOMAVANT_PARAMNAME);
        personneBean = new PersonneBean(nom, prenom);
        personneBean.setOriginal(original);
        personneBean.setNomAvant(nomAvant);
    }

}
