/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurPasswordCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurPassword";
    public final static String COMMANDKEY = "USR-03";
    public final static String PASSWORD1_PARAMNAME = "password1";
    public final static String PASSWORD2_PARAMNAME = "password2";
    private Redacteur redacteur;
    private String password;

    public RedacteurPasswordCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        if (!bdfServer.getPolicyManager().isPasswordModifAllowed()) {
            return ADMIN_ONLY_BDFUSER;
        } else {
            return MANDATORY_BDFUSER;
        }
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        PasswordManager passwordManager = bdfServer.getPasswordManager();
        passwordManager.setPassword(redacteur, password);
        bdfServer.savePasswordManager();
        setDone("sphere_done_password", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        if (!BdfUserUtils.isAdmin(bdfUser)) {
            if (!bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
                return false;
            }
        }
        String mdp1 = BdfCommand2Utils.getMandatoryParameter(requestMap, PASSWORD1_PARAMNAME);
        String mdp2 = BdfCommand2Utils.getMandatoryParameter(requestMap, PASSWORD2_PARAMNAME);
        if (!mdp1.equals(mdp2)) {
            setError("sphere_err_pwd_different", null);
            return false;
        }
        password = mdp1;
        if (password.length() == 0) {
            setError("sphere_err_pwd_wrong", null);
            return false;
        }
        if (password.length() < 4) {
            setError("sphere_err_pwd_tooshort", null);
            return false;
        }
        return true;
    }

}
