/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurRemove";
    public final static String COMMANDKEY = "SPH-06";
    private Redacteur redacteur;

    public RedacteurRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        RedacteurKey redacteurKey = redacteur.getRedacteurKey();
        Sphere sphere = redacteur.getSphere();
        bdfServer.getBasedefichesEditor().getSphereEditor(sphere.getSubsetKey()).removeRedacteur(redacteur);
        putResultObject(SPHERE_OBJ, sphere);
        setDone("sphere_done_redacteurremove", redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        if (!bdfServer.getPermissionManager().isRemoveable(redacteur)) {
            setError("sphere_err_redacteurnotremoveable", redacteur.getRedacteurKey());
        }
        return true;
    }

}
