/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.parsers.RedacteurItemParser;
import java.util.List;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.RedacteurStats;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.request.RequestMap;

/**
 *
 * @author Vincent Calame
 */
public class RedacteurReplaceCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurReplace";
    public final static String COMMANDKEY = "SPH-07";
    public final static String REPLACETYPE_PARAMNAME = "replacetype";
    public final static String OTHER_REPLACETYPE_PARAMVALUE = "other";
    public final static String LITTERAL_REPLACETYPE_PARAMVALUE = "litteral";
    public final static String OTHER_PARAMNAME = "other";
    public final static String LITTERAL_PARAMNAME = "litteral";
    private Redacteur redacteur;
    private boolean otherRedac = false;
    private Redacteur otherRedacteur = null;
    private FicheItem ficheItem = null;

    public RedacteurReplaceCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

     protected void doCommand() throws BdfCommand2Exception {
        FicheItem oldFicheItem = new Personne(redacteur.getRedacteurKey().toString());
        FicheItem newFicheItem = null;
        if (otherRedacteur != null) {
            newFicheItem = new Personne(otherRedacteur.getRedacteurKey().toString());
        } else {
            newFicheItem = ficheItem;
        }
        RedacteurStats redacteurStats = new RedacteurStats(redacteur);
        int count = redacteurStats.getFicheCount();
        List<RedacteurStats.ByCorpusStats> byCorpusStatsList = redacteurStats.getByCorpusStatsList();
        for (RedacteurStats.ByCorpusStats byCorpusStats : byCorpusStatsList) {
            Corpus corpus = byCorpusStats.getCorpus();
            CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
            List<RedacteurStats.ByFicheStats> list = byCorpusStats.getByFicheStatsList();
            CorpusUtils.FieldSelection fieldSelection = CorpusUtils.getPersonneFieldSelection(corpus);
            for (RedacteurStats.ByFicheStats byFicheStats : list) {
                int idcorpus = byFicheStats.getIdcorpus();
                CorpusUtils.replaceInField(corpusEditor, idcorpus, fieldSelection, oldFicheItem, newFicheItem);
            }
        }
        setDone("sphere_done_redacteurreplace", new Integer(count));
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        String type = getMandatoryParameter(REPLACETYPE_PARAMNAME);
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        try {
            otherRedac = replaceTypeToBoolean(type);
        } catch (IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, REPLACETYPE_PARAMNAME, type);
        }
        if (otherRedac) {
            String otherString = getMandatoryParameter(OTHER_PARAMNAME);
            otherString = otherString.trim();
            if (otherString.length() == 0) {
                setError("global_err_emptyfield", null);
                return false;
            }
            RedacteurKey otherRedacteurKey = RedacteurKey.parseFromUserString(otherString, redacteur.getSphere().getSubsetKey());
            if (otherRedacteurKey == null) {
                setError("sphere_err_redacteurkey", otherString);
                return false;
            }
            Sphere otherSphere = (Sphere) bdfServer.getBasedefiches().getSubset(otherRedacteurKey.getSphereKey());
            if (otherSphere != null) {
                otherRedacteur = otherSphere.getRedacteurByIdsphere(otherRedacteurKey.getIdsphereString());
            }
            if (otherRedacteur == null) {
                setError("sphere_err_noredacteur", otherString);
                return false;
            }
        } else {
            String litteralString = getMandatoryParameter(LITTERAL_PARAMNAME);
            litteralString = litteralString.trim();
            if (litteralString.length() == 0) {
                setError("global_err_emptyfield", null);
                return false;
            }
            TextParser textParser = new TextParser();
            textParser.setQuoteOptions(bdfUser.getTypoOptions().getQuoteOptions());
            Personne personne = RedacteurItemParser.parsePersonne(litteralString, textParser);
            if (personne != null) {
                ficheItem = personne;
            } else {
                ficheItem = new Item(textParser.parseString(litteralString));
            }
        }
        return true;
    }

    public static boolean replaceTypeToBoolean(String replaceTypeValue) {
        if (replaceTypeValue.equals(OTHER_REPLACETYPE_PARAMVALUE)) {
            return true;
        } else if (replaceTypeValue.equals(LITTERAL_REPLACETYPE_PARAMVALUE)) {
            return false;
        } else {
            throw new IllegalArgumentException("replaceTypeValue = " + replaceTypeValue);
        }
    }

}
