/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurRoleCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurRole";
    public final static String COMMANDKEY = "ROL-04";
    public final static String ADMIN_PARAMNAME = "admin";
    public final static String ROLE_PARAMNAME = "role";
    private Boolean isAdmin;
    private String[] roleNameArray;
    private Redacteur redacteur;

    public RedacteurRoleCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        BdfUser otherBdfUser = new BdfUser(bdfServer, redacteur.getRedacteurKey());
        otherBdfUser.setRoleNameArray(roleNameArray);
        BdfUserUtils.saveBdfUser(otherBdfUser);
        if (isAdmin != null) {
            bdfServer.getPermissionManager().setAdmin(redacteur, isAdmin.booleanValue());
            bdfServer.savePermissionManager();
        }
        setDone("sphere_done_role", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        boolean admin = BdfCommand2Utils.getBooleanParameter(requestMap, ADMIN_PARAMNAME);
        RedacteurKey redacteurKey = redacteur.getRedacteurKey();
        boolean current = BdfUserUtils.isAdmin(bdfServer, redacteurKey);
        if (current == admin) {
            isAdmin = null;
        } else {
            if (current) {
                if (bdfServer.getPermissionManager().getAdminCount() == 1) {
                    setError("sphere_err_lastadmin", redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    return false;
                }
            }
            isAdmin = new Boolean(admin);
        }
        roleNameArray = requestMap.getParameterValues(ROLE_PARAMNAME);
        if (roleNameArray == null) {
            roleNameArray = new String[0];
        }
        return true;
    }

}
