/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.request.RequestUtils;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereConstants;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadataEditor;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class SphereIntituleCommand extends AbstractBdfCommand2 {

    public final static String SPHEREINTITULE_PARAMPREFIX = "intitule_sphere";
    public final static String COMMANDNAME = "SphereIntitule";
    public final static String COMMANDKEY = "SPH-02";
    private Sphere sphere;
    private Libelle[] sphereIntituleArray;

    public SphereIntituleCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        SphereMetadataEditor sphereMetadataEditor = bdfServer.getBasedefichesEditor().getSphereEditor(sphere.getSubsetKey()).getSphereMetadataEditor();
        boolean done = false;
        if (update(sphereMetadataEditor, SphereConstants.SPHERE, sphereIntituleArray)) {
            done = true;
        }
        if (done) {
            setDone("sphere_done_intitules", null);
        }
        putResultObject(SPHERE_OBJ,sphere);
    }

    private boolean update(SphereMetadataEditor sphereMetadataEditor, String intituleName, Libelle[] array) {
        boolean done = false;
        Intitule intitule = BasedefichesUtils.getOrCreateIntitule(sphereMetadataEditor, intituleName);
        for (int i = 0; i < array.length; i++) {
            Libelle libelle = array[i];
            if (libelle != null) {
                boolean stepdone = sphereMetadataEditor.setIntituleLibelle(intitule, libelle.getLibelleString(), libelle.getLangInteger());
                if (stepdone) {
                    done = true;
                }
            }
        }
        return done;
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        sphere = BdfCommand2Utils.getMandatorySphere(basedefiches, requestMap);
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        sphereIntituleArray = RequestUtils.getLibelleArray(requestMap, SPHEREINTITULE_PARAMPREFIX + "/", workingArray);
        return true;
    }

}
