/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.NoRemoveableSubsetException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class SphereRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "SphereRemove";
    public final static String COMMANDKEY = "SPH-03";
    private Sphere sphere;

    public SphereRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        try {
            SubsetKey sphereKey = sphere.getSubsetKey();
            bdfServer.getBasedefichesEditor().removeSphere(sphere);
            setDone("sphere_done_sphereremove", sphereKey.getSubsetName());
            putResultObject(SUBSETKEY_OBJ,sphereKey);
        } catch (NoRemoveableSubsetException nrse) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        sphere = BdfCommand2Utils.getMandatorySphere(basedefiches, requestMap);
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        if (!bdfServer.getBasedefiches().isRemoveable(sphere)) {
            setError("sphere_err_spherenotremoveable", null);
            return false;
        }
        return true;
    }

}
