/*  BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers;

import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.roles.Role;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class CommandBoxUtils {

    private CommandBoxUtils() {
    }

    public static void insertRole(CommandBox commandBox, Role role) {
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        if (hiddenParameterMap == null) {
            hiddenParameterMap = new HashMap<String, String>();
            commandBox.setHiddenAttributesMap(hiddenParameterMap);
        }
        hiddenParameterMap.put("role", role.getName());
    }

    public static void insertCorpus(CommandBox commandBox, Corpus corpus) {
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        if (hiddenParameterMap == null) {
            hiddenParameterMap = new HashMap<String, String>();
            commandBox.setHiddenAttributesMap(hiddenParameterMap);
        }
        BdfHtmlUtils.addSubsetParameter(hiddenParameterMap, corpus);
    }

    public static void insertSphere(CommandBox commandBox, Sphere sphere) {
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        if (hiddenParameterMap == null) {
            hiddenParameterMap = new HashMap<String, String>();
            commandBox.setHiddenAttributesMap(hiddenParameterMap);
        }
        BdfHtmlUtils.addSubsetParameter(hiddenParameterMap, sphere);
    }

    public static void insertRedacteur(CommandBox commandBox, Redacteur redacteur) {
        insertRedacteur(commandBox, redacteur.getRedacteurKey());
    }

    public static void insertRedacteur(CommandBox commandBox, RedacteurKey redacteurKey) {
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        if (hiddenParameterMap == null) {
            hiddenParameterMap = new HashMap<String, String>();
            commandBox.setHiddenAttributesMap(hiddenParameterMap);
        }
        BdfHtmlUtils.addSubsetParameter(hiddenParameterMap, redacteurKey.getSphereKey());
        hiddenParameterMap.put(BdfHtmlUtils.getDefaultIdsubsetParamName(SubsetKey.CATEGORY_SPHERE), redacteurKey.getIdsphereString());
    }

    public static void insertAlbum(CommandBox commandBox, Album album) {
        Map<String, String> hiddenParameterMap = commandBox.getHiddenAttributesMap();
        if (hiddenParameterMap == null) {
            hiddenParameterMap = new HashMap<String, String>();
            commandBox.setHiddenAttributesMap(hiddenParameterMap);
        }
        BdfHtmlUtils.addSubsetParameter(hiddenParameterMap, album);
    }

    public static void insertAlbumDim(CommandBox commandBox, AlbumDim albumDim) {
        insertAlbum(commandBox, albumDim.getAlbumMetadata().getAlbum());
        commandBox.getHiddenAttributesMap().put("albumdim", albumDim.getName());
    }

}
