/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.*;
import fr.exemole.bdfserver.commands2.envoi.SendCommand;
import fr.exemole.bdfserver.commands2.saisie.SelectionIndexationCommand;
import fr.exemole.bdfserver.email.EnvoiParameters;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.util.request.RequestMap;
import fr.exemole.bdfserver.htmlproducers.album.*;
import fr.exemole.bdfserver.html.CommandHtmlProducer;
import fr.exemole.bdfserver.html.FicheResultHtmlProducer;
import fr.exemole.bdfserver.htmlproducers.saisie.*;
import fr.exemole.bdfserver.htmlproducers.sphere.*;
import fr.exemole.bdfserver.htmlproducers.selection.*;
import fr.exemole.bdfserver.htmlproducers.configuration.*;
import fr.exemole.bdfserver.htmlproducers.affichage.*;
import fr.exemole.bdfserver.htmlproducers.envoi.*;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.request.GenericParameters;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.response.BdfConstants;
import fr.exemole.bdfserver.ui.AlterUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.MasterSubsetItemPointeur;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.localisation.BundleLocalisation;
import net.mapeadores.util.logging.CommandMessage;


/**
 *
 * @author Vincent Calame
 */
public class HtmlProducerFactory implements BdfConstants, AlbumHtmlConstants, SaisieHtmlConstants, SphereHtmlConstants, ConfigurationHtmlConstants, SelectionHtmlConstants, AffichageHtmlConstants, EnvoiHtmlConstants {

    public static final String ALTER_PARAMNAME = "alter";

    public static HtmlProducer getHtmlProducer(String page, BdfContext bdfContext, BdfCommandPageParameters bdfCommandPageParameters) throws BdfCommand2Exception {
        String actionName = bdfCommandPageParameters.getActionName();
        RequestMap requestMap = bdfCommandPageParameters.getRequestMap();
        BdfUser bdfUser = bdfCommandPageParameters.getBdfUser();
        BdfCommand2Result bdfCommandResult = bdfCommandPageParameters.getBdfCommandResult();
        CommandHtmlProducer htmlProducer = null;
        if (actionName.equals("Album")) {
            htmlProducer = getAlbumHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Saisie")) {
            htmlProducer = getSaisieHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Sphere")) {
            htmlProducer = getSphereHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Configuration")) {
            htmlProducer = getConfigurationHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Selection")) {
            htmlProducer = getSelectionHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Affichage")) {
            htmlProducer = getAffichageHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        } else if (actionName.equals("Envoi")) {
            htmlProducer = getEnvoiHtmlProducer(page, bdfContext, requestMap, bdfUser, bdfCommandResult);
        }
        if (htmlProducer == null) {
            return null;
        }
        if (bdfCommandResult != null) {
            CommandMessage commandMessage = bdfCommandResult.getCommandMessage();
            if (commandMessage != null) {
                htmlProducer.setCommandMessage(commandMessage);
            }
        }
        if (htmlProducer instanceof BdfHtmlProducer) {
            BdfHtmlProducer bdfHtmlProducer = (BdfHtmlProducer) htmlProducer;
            ((BdfHtmlProducer) bdfHtmlProducer).setCommandKeyLocalisation(BundleLocalisation.getBundleLocalisation("fr.exemole.bdfserver.servlets.resources.l10n.bdfcommand", bdfHtmlProducer.getLocalisationInfo().getFormatLocale()));
        }
        return htmlProducer;
    }

    private static CommandHtmlProducer getAlbumHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        if (page.equals(ALBUMINDEX_PAGE)) {
            return new AlbumIndexHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(ALBUMCREATIONFORM_PAGE)) {
            return new AlbumCreationFormHtmlProducer(bdfContext, bdfUser);
        }
        Album album = null;
        if (bdfCommandResult != null) {
            album = (Album) bdfCommandResult.getResultObject(ALBUM_OBJ);
        }
        if (album == null) {
            album = BdfCommand2Utils.getMandatoryAlbum(bdfContext.getBdfServer().getBasedefiches(), requestMap);
        }
        if (page.equals(ALBUMMETADATAFORM_PAGE)) {
            return new AlbumMetadataFormHtmlProducer(bdfContext, bdfUser, album);
        }
        return null;
    }

    private static CommandHtmlProducer getEnvoiHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        SendCommand.testSmtpManager(bdfContext);
        EnvoiParameters envoiParameters = null;
        if (bdfCommandResult != null) {
            envoiParameters = (EnvoiParameters) bdfCommandResult.getResultObject(ENVOIPARAMETERS_OBJ);
        }
        if (envoiParameters == null) {
            envoiParameters = SendCommand.buildFromRequest(bdfContext.getBdfServer(), bdfUser, requestMap, true);
        }
        if (page.equals(SENDFORM_PAGE)) {
            return new SendFormHtmlProducer(bdfContext, bdfUser, envoiParameters);
        }
        if (page.equals(SENDCONFIRMATION_PAGE)) {
            return new SendConfirmationHtmlProducer(bdfContext, bdfUser, envoiParameters);
        }
        return null;
    }

    private static CommandHtmlProducer getSphereHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        if (page.equals(SPHERECREATIONFORM_PAGE)) {
            return new SphereCreationFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(REDACTEURUSERFORM_PAGE)) {

            return new RedacteurUserFormHtmlProducer(bdfContext, bdfUser);
        }
        Sphere sphere = null;
        if (bdfCommandResult != null) {
            sphere = (Sphere) bdfCommandResult.getResultObject(SPHERE_OBJ);
        }
        if (sphere == null) {
            sphere = BdfCommand2Utils.getMandatorySphere(bdfContext.getBdfServer().getBasedefiches(), requestMap);
        }
        if (page.equals(SPHERE_PAGE)) {
            return new SphereHtmlProducer(bdfContext, bdfUser, sphere);
        }
        if (page.equals(SPHEREMETADATAFORM_PAGE)) {
            return new SphereMetadataFormHtmlProducer(bdfContext, bdfUser, sphere);
        }
        if (page.equals(REDACTEURCREATIONFORM_PAGE)) {
            return new RedacteurCreationFormHtmlProducer(bdfContext, bdfUser, sphere);
        }
        Redacteur redacteur = null;
        if (bdfCommandResult != null) {
            redacteur = (Redacteur) bdfCommandResult.getResultObject(REDACTEUR_OBJ);
        }
        if (redacteur == null) {
            redacteur = BdfCommand2Utils.getMandatoryRedacteur(bdfContext.getBdfServer().getBasedefiches(), requestMap);
        }
        if (page.equals(REDACTEURADMINFORM_PAGE)) {
            return new RedacteurAdminFormHtmlProducer(bdfContext, bdfUser, redacteur);
        }
        if (page.equals(REDACTEURSTATE_PAGE)) {
            return new RedacteurStateHtmlProducer(bdfContext, bdfUser, redacteur);
        }
        return null;
    }

    private static CommandHtmlProducer getSaisieHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        BdfServer bdfServer = bdfContext.getBdfServer();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        if (page.equals(ZOOMEDIT_PAGE)) {
            String appelant = BdfCommand2Utils.getMandatoryParameter(requestMap, "appelant");
            return new ZoomEditHtmlProducer(bdfContext, bdfUser, appelant);
        }
        if (page.equals(SELECTIONINDEXATION_PIOCHE_PAGE)) {
            return new SelectionIndexationPiocheHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(SELECTIONINDEXATION_FORM_PAGE)) {
            Motcle motcle = null;
            if (bdfCommandResult != null) {
                motcle = (Motcle) bdfCommandResult.getResultObject(MOTCLE_OBJ);
            }
            if (motcle == null) {
                motcle = BdfCommand2Utils.getMandatoryMotcle(basedefiches, requestMap);
            }
            return new SelectionIndexationFormHtmlProducer(bdfContext, bdfUser, motcle);
        }
        if (page.equals(SELECTIONINDEXATION_RESULT_PAGE)) {
            if (bdfCommandResult == null) {
                throw new ResultPageBdfCommand2Exception(SELECTIONINDEXATION_RESULT_PAGE);
            }
            int[] result = (int[]) bdfCommandResult.getResultObject(SelectionIndexationCommand.INTARRAY_OBJ);
            Motcle motcle = (Motcle) bdfCommandResult.getResultObject(MOTCLE_OBJ);
            return new SelectionIndexationResultHtmlProducer(bdfContext, bdfUser, motcle, result);
        }
        Corpus corpus = null;
        AbregeFiche abregeFiche = null;
        if (bdfCommandResult != null) {
            corpus = (Corpus) bdfCommandResult.getResultObject(CORPUS_OBJ);
            abregeFiche = (AbregeFiche) bdfCommandResult.getResultObject(ABREGEFICHE_OBJ);
        }
        if (corpus == null) {
            if (abregeFiche != null) {
                corpus = abregeFiche.getCorpus();
            } else {
                corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
            }
        }
        Subset masterSubset = corpus.getMasterSubset();
        FicheFormHtmlProducer ficheFormHtmlProducer = null;
        ComponentUIList componentUIList = getComponentUIList(bdfContext, requestMap, corpus);
        if (page.equals(FICHECREATION_PAGE)) {
            BdfCommand2Utils.testCreateFichePermission(bdfServer, bdfUser, corpus, -1);
            if (masterSubset != null) {
                return new SlaveFicheCreationHtmlProducer(bdfContext, bdfUser, corpus);
            } else {
                FicheFormParameters ficheFormParameters = initFicheFormParameters(bdfContext, requestMap, bdfUser, corpus);
                ficheFormParameters.setNewFiche(true);
                ficheFormHtmlProducer = new FicheFormHtmlProducer(ficheFormParameters, componentUIList, corpus);
            }
        } else if (page.equals(FICHEMODIF_PAGE)) {
            int idcorpus = -1;
            SubsetItem masterSubsetItem = null;
            FicheFormParameters ficheFormParameters = initFicheFormParameters(bdfContext, requestMap, bdfUser, corpus);
            if (abregeFiche == null) {
                idcorpus = BdfCommand2Utils.getMandatoryIdsubset(basedefiches, requestMap, SubsetKey.CATEGORY_CORPUS);
                abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche == null) {
                    if (masterSubset != null) {
                        masterSubsetItem = masterSubset.getSubsetItemById(idcorpus);
                        if (masterSubsetItem == null) {
                            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSETITEM, "idcorpus", String.valueOf(idcorpus));
                        }
                        MasterSubsetItemPointeur pointeur = new MasterSubsetItemPointeur();
                        pointeur.setCurrentSubsetItem(masterSubsetItem);
                        ficheFormParameters.setMasterSubsetItemPointeur(pointeur);
                    }
                }
            }
            if (abregeFiche != null) {
                BdfCommand2Utils.testWriteFichePermission(bdfServer, bdfUser, abregeFiche);
                ficheFormParameters.getFichePointeur().setCurrent(abregeFiche);
                ficheFormParameters.setNewFiche(false);
                return new FicheFormHtmlProducer(ficheFormParameters, componentUIList, abregeFiche);
            } else {
                BdfCommand2Utils.testCreateFichePermission(bdfServer, bdfUser, corpus, idcorpus);
                ficheFormParameters.setNewFiche(true);
                if (masterSubsetItem != null) {
                    ficheFormHtmlProducer = new FicheFormHtmlProducer(ficheFormParameters, componentUIList, masterSubsetItem);
                } else {
                    ficheFormHtmlProducer = new FicheFormHtmlProducer(ficheFormParameters, componentUIList, idcorpus);
                }
            }
        }
        if (ficheFormHtmlProducer != null) {
            String pageResult = requestMap.getParameter("page-result");
            if (pageResult != null) {
                ficheFormHtmlProducer.setFicheResultPage(pageResult);
            }
            return ficheFormHtmlProducer;
        }
        if (page.equals(FORMEXPORT_PAGE)) {
            FicheFormParameters ficheFormParameters = initFicheFormParameters(bdfContext, requestMap, bdfUser, corpus);
            return new FormExportHtmlProducer(ficheFormParameters, componentUIList);
        } else if (page.equals(FICHERESULT_PAGE)) {
            if (abregeFiche == null) {
                abregeFiche = BdfCommand2Utils.getMandatoryAbregeFiche(basedefiches, requestMap, corpus);
            }
            FicheResultHtmlProducer ficheResultHtmlProducer = getImplementation(bdfContext);
            ficheResultHtmlProducer.init(bdfContext, bdfUser, abregeFiche);
            return ficheResultHtmlProducer;
        } else if (page.equals(FICHEINDEXATION_PAGE)) {
            if (abregeFiche == null) {
                abregeFiche = BdfCommand2Utils.getMandatoryAbregeFiche(basedefiches, requestMap, corpus);
            }
            Thesaurus uniqueThesaurus = null;
            if (requestMap.getParameter("thesaurus") != null) {
                uniqueThesaurus = BdfCommand2Utils.getMandatoryThesaurus(basedefiches, requestMap);
            }
            FicheIndexationHtmlProducer ficheIndexationHtmlProducer = new FicheIndexationHtmlProducer();
            FicheFormParameters ficheFormParameters = initFicheFormParameters(bdfContext, requestMap, bdfUser, corpus);
            ficheIndexationHtmlProducer.init(ficheFormParameters, abregeFiche, uniqueThesaurus);
            return ficheIndexationHtmlProducer;
        }
        return null;
    }

    private static CommandHtmlProducer getConfigurationHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        if (page.equals(COREFORM_PAGE)) {
            return new CoreFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(INTITULESFORM_PAGE)) {
            return new IntitulesFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(PATHSFORM_PAGE)) {
            return new PathsFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(LOGOSFORM_PAGE)) {
            return new LogosFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(SUBSETKEYTREEFORM_PAGE)) {
            return new SubsetKeyTreeFormHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(EXTENSIONSFORM_PAGE)) {
            return new ExtensionsFormHtmlProducer(bdfContext, bdfUser);
        }
        return null;
    }

    private static CommandHtmlProducer getAffichageHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        if (page.equals(MENU_PAGE)) {
            return new MenuHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(ACCUEIL_PAGE)) {
            return new AccueilHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(LIAGE_PAGE)) {
            return new LiageHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(RECENSEMENTS_PAGE)) {
            return new RecensementsHtmlProducer(bdfContext, bdfUser);
        }
        if (page.equals(FICHES_PAGE)) {
            bdfUser.updateAbregeFicheSortedList();
            return new FichesHtmlProducer(bdfContext, bdfUser);
        }
        return null;
    }

    private static CommandHtmlProducer getSelectionHtmlProducer(String page, BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, BdfCommand2Result bdfCommandResult) throws BdfCommand2Exception {
        if (page.equals(SELECTIONFORM_PAGE)) {
            String selecionName = requestMap.getParameter("selectionname");
            return new SelectionFormHtmlProducer(bdfContext, bdfUser, selecionName);
        }
        return null;
    }

    private static FicheResultHtmlProducer getImplementation(BdfContext bdfContext) {
        return new DefaultFicheResultHtmlProducer();
    }

    private static FicheFormParameters initFicheFormParameters(BdfContext bdfContext, RequestMap requestMap, BdfUser bdfUser, Corpus corpus) {
        FicheFormParameters ficheFormParameters = FicheFormParameters.newInstance(bdfContext, bdfUser, new FichePointeur(corpus, true));
        ficheFormParameters.setGenericParameters(GenericParameters.buildFromRequestMap(requestMap));
        String bracketsParam = requestMap.getParameter("brackets");
        if ((bracketsParam != null) && (bracketsParam.equals("1"))) {
            ficheFormParameters.setWithSpecialParseChars(false);
        } else {
            ficheFormParameters.setWithSpecialParseChars(true);
        }
        return ficheFormParameters;
    }

    private static ComponentUIList getComponentUIList(BdfContext bdfContext, RequestMap requestMap, Corpus corpus) {
        UIManager uiManager = bdfContext.getBdfServer().getUIManager();
        CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
        AlterUI alterUI = null;
        String alterName = requestMap.getParameter(ALTER_PARAMNAME);
        if (alterName != null) {
            alterUI = uiManager.getAlterUI(corpusUI, alterName);
        }
        return uiManager.getComponentUIList(corpusUI, alterUI);
    }

}
