/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.affichage;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.email.EnvoiParameters;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.htmlproducers.envoi.EnvoiHtmlConstants;
import fr.exemole.bdfserver.htmlproducers.saisie.SaisieHtmlConstants;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.transformation.TransformationInfo;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.response.EnvoiResponseParameters;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import fr.exemole.bdfserver.util.BdfServerUtils;
import fr.exemole.bdfserver.util.RattachementPermission;
import java.text.DateFormat;
import java.text.MessageFormat;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class FichesHtmlProducer extends BdfHtmlProducer implements AffichageHtmlConstants {

    private final static int LIMIT = 250;
    private final static String ENVOI_URL = "Envoi?page=" + EnvoiHtmlConstants.SENDFORM_PAGE + "&type=";
    private final static String ENVOIFICHE_URL = ENVOI_URL + EnvoiParameters.typeToString(EnvoiParameters.ENVOI_FICHEUNIQUE) + "&";
    private TransformationManager transformationManager;
    private PermissionManager permissionManager;
    private DateFormat dateFormat;
    private MessageFormat idcorpusFormat;
    private boolean avecEnvoi;
    private RedacteurKey redacteurKey;

    public FichesHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        transformationManager = bdfServer.getTransformationManager();
        permissionManager = bdfServer.getPermissionManager();
        dateFormat = DateFormat.getDateInstance(DateFormat.MEDIUM, formatLocale);
        idcorpusFormat = new MessageFormat(getLocalization("listefiches_fichenumero"), formatLocale);
        Redacteur redacteur = BdfUserUtils.getRedacteur(bdfUser);
        avecEnvoi = (redacteur.getCourrielBean() != null);
        redacteurKey = redacteur.getRedacteurKey();
        setCssThemeFile("affichage.css");
        addJavascriptFiles(GlobalCatalog.BDF_JAVASCRIPT);
    }

    public void printHtml() {
        start();
        AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
        int ficheCount = abregeFicheSortedList.getAbregeFicheCount();
        if (ficheCount == 0) {
            printEmptyMessage();
            return;
        }
        int corpusCount = abregeFicheSortedList.getCorpusCount();
        boolean geolocalisation = false;
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = abregeFicheSortedList.getCorpus(i);
            if (corpus.getCorpusMetadata().getGeolocalisationField() != null) {
                geolocalisation = true;
                break;
            }
        }
        TransformationInfo compilationTransformationInfo = transformationManager.getTransformationInfo(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
        String[] compilationExtensionArray = compilationTransformationInfo.getAllExtensionArray();
        startP(GlobalCatalog.GLB_P_TOOLBARLINKS);
        String compilationHref = "fiches/compilation.";
        if (ficheCount > LIMIT) {
            localize("listefiches_compilation");
        } else {
            appendA(compilationHref + "html", "listefiches_compilation", "Saisie");
        }
        if (compilationExtensionArray.length > 0) {
            append(" (");
            for (int i = 0; i < compilationExtensionArray.length; i++) {
                String extension = compilationExtensionArray[i];
                if (i > 0) {
                    BdfHtmlUtils.printTiret(this);
                }
                String href = compilationHref + extension;
                if (ficheCount > LIMIT) {
                    append(extension);
                } else {
                    startA(href, "Saisie");
                    append(extension);
                    endA();
                }
            }
            append(")");
        }
        if (avecEnvoi) {
            append(" (");
            String envoiHref = ENVOI_URL + EnvoiParameters.typeToString(EnvoiParameters.ENVOI_COMPILATION);
            if (ficheCount > LIMIT) {
                localize("listefiches_envoicompilation");
            } else {
                appendA(envoiHref, "listefiches_envoicompilation", "Saisie");
            }
            append(")");
        }
        if (isAdmin()) {
            BdfHtmlUtils.printTiret(this);
            String masspiocheHref = "Saisie?page=" + SaisieHtmlConstants.SELECTIONINDEXATION_PIOCHE_PAGE;
            appendA(masspiocheHref, "saisie_link_masspioche", "Saisie");
        }
        if (avecEnvoi) {
            BdfHtmlUtils.printTiret(this);
            String envoiHref = ENVOI_URL + EnvoiResponseParameters.typeToString(EnvoiResponseParameters.ENVOI_SELECTION);
            appendA(envoiHref, "listefiches_envoiselection", "Saisie");
        }
        if (geolocalisation) {
            BdfHtmlUtils.printTiret(this);
            String geolocalisationHref = "fiches/geolocalisation.html";
            appendA(geolocalisationHref, "listefiches_geolocalisation", "Saisie");
        }
        endP();
        String[] roleNameArray = bdfUser.getRoleNameArray();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = abregeFicheSortedList.getCorpus(i);
            boolean withRattachement = (corpus.getSlaveSubsetArray() != null);
            boolean globalWriteRight = permissionManager.canDo(redacteurKey, roleNameArray, FicheAction.getWriteFiche(corpus));
            SubsetItemList abregeFicheList = abregeFicheSortedList.getCorpusAbregeFicheList(i);
            int corpusFicheCount = abregeFicheList.getSubsetItemCount();
            boolean avectransferdoc = bdfServer.getUIManager().getCorpusUI(corpus).isAssociationActive();
            TransformationInfo transformationInfo = transformationManager.getTransformationInfo(new TransformationKey(corpus.getSubsetKey()));
            startH2();
            escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
            append(" [");
            append(String.valueOf(corpusFicheCount));
            append("]");
            endH2();
            startUl();
            String[] extensionArray = transformationInfo.getAllExtensionArray();
            for (int j = 0; j < corpusFicheCount; j++) {
                startLi();
                AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                boolean writePermission = globalWriteRight;
                if (!writePermission) {
                    writePermission = permissionManager.canDo(redacteurKey, roleNameArray, FicheAction.getWriteFiche(abrege));
                }
                printAbrege(abrege, writePermission, avectransferdoc, extensionArray, withRattachement);
                endLi();
            }
            endUl();
        }
        end();
    }

    private void printEmptyMessage() {
        String key = (CorpusUtils.getFicheCount(bdfServer.getBasedefiches()) == 0) ? "listefiches_zerofiche" : "listefiches_zeroselection";
        appendP(null, key);
        end();
    }

    private void printAbrege(AbregeFiche abregeFiche, boolean droitmodif, boolean avectransfertdoc, String[] extensionArray, boolean hasRattachement) {
        String titre = abregeFiche.getTitre();
        String liendirect = "fiches/" + RequestUtils.toBasename(abregeFiche);
        if (titre.length() == 0) {
            titre = "__";
        }
        startP();
        if (droitmodif) {
            escape(titre);
        } else {
            String href = liendirect + ".html";
            startA(href, "Saisie");
            escape(titre);
            endA();
        }
        endP();
        startP(GlobalCatalog.GLB_P_SMALLLINKS);
        Object[] objs = new Object[1];
        objs[0] = new Integer(abregeFiche.getIdcorpus());
        escape(idcorpusFormat.format(objs));
        BdfHtmlUtils.printTiret(this);
        int datesaisie = abregeFiche.getCreationDate();
        int datemodif = abregeFiche.getModificationDate();
        if (datesaisie > 0) {
            append(dateFormat.format(DateInteger.toDate(datesaisie)));
        }
        if ((datemodif != datesaisie) && (datemodif > 0)) {
            append(" | ");
            append(dateFormat.format(DateInteger.toDate(datemodif)));
        }
        int length = extensionArray.length;
        if (length > 0) {
            BdfHtmlUtils.printTiret(this);
            append("(");
            for (int i = 0; i < length; i++) {
                String extension = extensionArray[i];
                if (i > 0) {
                    BdfHtmlUtils.printTiret(this);
                }
                String href = liendirect + "." + extension;
                startA(href, "Saisie");
                append(extension);
                endA();
            }
            append(")");
        }
        if ((!droitmodif) && (avecEnvoi)) {
            BdfHtmlUtils.printTiret(this);
            String href = ENVOIFICHE_URL + RequestUtils.toParameterString(abregeFiche);
            appendA(href, "listefiches_envoyer", "Saisie");
        }
        endP();
        if (droitmodif) {
            startP(GlobalCatalog.GLB_P_SMALLLINKS);
            printFicheLinks(abregeFiche, avectransfertdoc);
            endP();
            if (hasRattachement) {
                printRattachementPermission(abregeFiche);
            }
        }
    }

    private void printRattachementPermission(AbregeFiche abregeFiche) {
        RattachementPermission rattachementPermission = BdfServerUtils.getRattachementPermission(bdfServer, bdfUser, abregeFiche);
        int slaveCorpusCount = rattachementPermission.getSlaveCorpusCount();
        if (slaveCorpusCount == 0) {
            return;
        }
        String actionkey = abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "_" + String.valueOf(abregeFiche.getIdcorpus());
        BdfHtmlUtils.printDeployTitle(this, actionkey, "global_link_rattachement");
        HTMLAttributes ulAttributes = new HTMLAttributes();
        ulAttributes.setId("DEPLOYBLOCK_" + actionkey);
        ulAttributes.addClass(GlobalCatalog.UL_GLB_SMALLLINKLIST);
        if (isWithJQuery()) {
            ulAttributes.addClass(GlobalCatalog.INVISIBLE);
        }
        startUl(ulAttributes);

        for (int i = 0; i < slaveCorpusCount; i++) {
            startLi();
            short type = rattachementPermission.getActionType(i);
            switch (type) {
                case RattachementPermission.READ_ACTION:
                case RattachementPermission.WRITE_ACTION:
                    AbregeFiche other = rattachementPermission.getAbregeFiche(i);
                    String affichageHref = "fiches/" + RequestUtils.toBasename(other) + ".html";
                    String ficheString = CorpusMetadataUtils.getFicheIntituleString(other, localisationInfo);
                    if (type == RattachementPermission.READ_ACTION) {
                        startA(affichageHref, "Saisie");
                        escape(ficheString);
                        endA();
                    } else {
                        escape(ficheString);
                        space();
                        append("(");
                        String conditionFiche = "?" + RequestUtils.toParameterString(other);
                        String ficheHref = "Saisie" + conditionFiche + "&page=" + SaisieHtmlConstants.FICHEMODIF_PAGE;
                        appendA(ficheHref, "listefiches_modifier", "Saisie");
                        BdfHtmlUtils.printTiret(this);
                        appendA(affichageHref, "listefiches_afficher", "Saisie");
                        append(")");
                    }
                    break;
                case RattachementPermission.CREATE_ACTION:
                    Corpus otherCorpus = rattachementPermission.getCorpus(i);
                    printLibColon("saisie_lib_rattachement_create");
                    space();
                    String createHref = "Saisie?" + RequestUtils.toParameterString(otherCorpus) + "&amp;" + RequestUtils.IDCORPUS_PARAMNAME + "=" + String.valueOf(abregeFiche.getIdcorpus()) + "&page=" + SaisieHtmlConstants.FICHEMODIF_PAGE;
                    startA(createHref, "Saisie");
                    escape(CorpusMetadataUtils.getCorpusIntituleString(otherCorpus, workingLangInteger));
                    endA();
                    break;
                default:
                    throw new SwitchException("wrong type : " + type);
            }
            endLi();
        }
        endUl();
    }

    private void printFicheLinks(AbregeFiche abregeFiche, boolean withDoc) {
        String conditioncorpus = RequestUtils.toParameterString(abregeFiche);
        appendA("Saisie?" + conditioncorpus + "&page=" + SaisieHtmlConstants.FICHEMODIF_PAGE, "listefiches_modifier", "Saisie");
        BdfHtmlUtils.printTiret(this);
        String affichageHref = "fiches/" + RequestUtils.toBasename(abregeFiche) + ".html";
        appendA(affichageHref, "listefiches_afficher", "Saisie");
        if (isAdmin()) {
            BdfHtmlUtils.printTiret(this);
            String indexationHref = "Saisie?" + conditioncorpus + "&amp;page=" + SaisieHtmlConstants.FICHEINDEXATION_PAGE;
            appendA(indexationHref, "listefiches_indexer", "Saisie");
        }
        if (withDoc) {
            BdfHtmlUtils.printTiret(this);
            String associationHref = "Association?" + conditioncorpus;
            appendA(associationHref, "listefiches_docfiche", "Saisie");
        }
        if (avecEnvoi) {
            BdfHtmlUtils.printTiret(this);
            String envoiHref = ENVOIFICHE_URL + conditioncorpus;
            appendA(envoiHref, "listefiches_envoyer", "Saisie");
        }
    }

}
