/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.affichage;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.PrintWriter;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;


/**
 *
 * @author Vincent Calame
 */
public class LiageHtmlProducer extends BdfHtmlProducer implements AffichageHtmlConstants {

    private Basedefiches basedefiches;

    public LiageHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        this.basedefiches = bdfServer.getBasedefiches();
        setCssThemeFile("affichage.css");
    }

    public void printHtml() {
        start();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            printCorpus(pw, corpus);
        }
        end();
    }

    private void printCorpus(PrintWriter pw, Corpus corpusPrincipal) {
        SubsetKey corpusKey = corpusPrincipal.getSubsetKey();
        int corpusCount = basedefiches.getCorpusCount();
        startH2();
        append(corpusPrincipal.getSubsetKey().getSubsetName());
        endH2();
        startUl();
        for (int k = 0; k < corpusCount; k++) {
            Corpus otherCorpus = basedefiches.getCorpus(k);
            SubsetKey otherCorpusKey = otherCorpus.getSubsetKey();
            Croisement croisement = basedefiches.getCroisement(corpusKey, otherCorpusKey);
            boolean first = true;
            int length = otherCorpus.getTicketLength();
            for (int i = 0; i < length; i++) {
                SubsetItem otherSubsetItem = otherCorpus.getSubsetItemByTicket(i);
                if (otherSubsetItem != null) {
                    CroisementItem croisementItem = croisement.getCroisementItem(otherCorpusKey, otherSubsetItem.getTicket());
                    int longLength = croisementItem.getCount();
                    if (longLength == 0) {
                        continue;
                    }
                    if (first) {
                        startLi();
                        startP();
                        append(otherCorpusKey.getSubsetName());
                        endP();
                        startUl();
                        first = false;
                    }
                    startLi();
                    append(String.valueOf(otherSubsetItem.getIdsubset()));
                    append(" : ");
                    for (int j = 0; j < longLength; j++) {
                        if (j > 0) {
                            append(", ");
                        }
                        SubsetItem subsetItem = corpusPrincipal.getSubsetItemByTicket(croisementItem.getTicket(j));
                        append(String.valueOf(subsetItem.getIdsubset()));
                    }
                    endLi();
                }
            }
            if (!first) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

}
