/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.affichage;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.menus.MenuLink;
import fr.exemole.bdfserver.htmlproducers.saisie.SaisieHtmlConstants;
import fr.exemole.bdfserver.htmlproducers.sphere.SphereHtmlConstants;
import fr.exemole.bdfserver.ui.SubsetGrouping;
import fr.exemole.bdfserver.ui.SubsetGroupingUtils;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.List;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class MenuHtmlProducer extends BdfHtmlProducer implements AffichageHtmlConstants {

    private Redacteur redacteur;
    private boolean isAdmin;

    public MenuHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        redacteur = BdfUserUtils.getRedacteur(bdfUser);
        isAdmin = isAdmin();
        setCssThemeFile("affichage_menu.css");
        addJavascriptFiles(GlobalCatalog.MENU_JAVASCRIPT);
        this.getCommandKeyLocalisation();
    }

    public void printHtml() {
        String sessionString = getSessionString();
        start();
        if (isWithJQuery()) {
            HTMLAttributes scriptAttributes = new HTMLAttributes();
            scriptAttributes.setType("text/javascript");
            startScript(scriptAttributes);
            append("\n var err_goto = \"");
            append(StringUtils.escapeDoubleQuote(getLocalization("menu_err_goto")));
            append("\";\n");
            endScript();
        }
        HTMLAttributes imgHTMLAttributes = new HTMLAttributes();
        imgHTMLAttributes.setSrc("rsc/icon32.png");
        imgHTMLAttributes.setTitle(sessionString);
        imgHTMLAttributes.setAlt(sessionString);
        imgHTMLAttributes.setWidth("32");
        imgHTMLAttributes.setHeight("32");
        imgHTMLAttributes.setId("BLOC_GAUCHE");
        appendImg(imgHTMLAttributes);

        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId("BLOC_DROIT");

        startDiv(divAttributes);
        printBlocDroitLink("Session?page=" + "new", "_top", "menu_link_newsession");
        printBlocDroitLink("Session", "_blank", "menu_link_duplicate");
        endDiv();
        HTMLAttributes tableHtmlAttributes = new HTMLAttributes();
        tableHtmlAttributes.setId("TABLE_MENU");
        startTable(tableHtmlAttributes);
        startRow();

        printLinkCell("Affichage?page=fiches", "Liste", "menu_link_fiches");
        printLinkCell("Selection?page=selection-form", "Saisie", "menu_link_selection");

        if (isWithJQuery()) {
            printActionSelect();
        } else {
            printLinkCell("Exportation", "Saisie", "menu_link_exportoo");
        }

        endRow();
        startRow();
        printLinkCell("Affichage?page=recensements", "Liste", "menu_link_recensements");
        if (isAdmin) {
            printLinkCell("Administration", "Saisie", "menu_link_administration");
        } else {
            appendEmptyCell();
        }
        if (isWithJQuery()) {
            startCell();
            HTMLAttributes redoAttributes = new HTMLAttributes();
            redoAttributes.setId("JQ_REDOSPAN");
            startSpan(redoAttributes);
            endSpan();
            endCell();
            /*HTMLAttributes redoAttributes = new HTMLAttributes();
            redoAttributes.setId("JQ_REDOACTION");
            printLinkCell("#", null, "menu_link_redo", redoAttributes);*/
        } else {
            printLinkCell("Sphere?page=" + SphereHtmlConstants.REDACTEURUSERFORM_PAGE, "Saisie", "menu_link_preferences");
        }
        endRow();
        endTable();

        SubsetKeyTree keyTree = bdfServer.getUIManager().getCorpusSubsetKeyTree();
        SubsetGrouping subsetGrouping = SubsetGroupingUtils.checkCorpusTree(bdfUser, keyTree);
        if (subsetGrouping.getSubsetKeyGroupList().size() > 0) {
            divAttributes.setId("SSBLOC_GO1");
            startDiv(divAttributes);
            HTMLAttributes formAttributes = new HTMLAttributes();
            formAttributes.setMethod("get");
            formAttributes.setAction("Saisie");
            formAttributes.setTarget("Saisie");
            startForm(formAttributes);
            HTMLAttributes selectAtt = new HTMLAttributes();
            selectAtt.setId("corpusSelect");
            selectAtt.setName("corpus");
            BdfHtmlUtils.printSubsetSelect(this, subsetGrouping, selectAtt, null);
            space();
            if (isWithJQuery()) {
                HTMLAttributes htmlAttributes = getAAttributes("fiche_new");
                htmlAttributes.setClasses("Icon");
                startA(htmlAttributes);
                appendAnonymImg("global_link_fiche_new");
                space();
                localize("global_link_fiche_new");
                endA();
            } else {
                appendHiddenInput("page", SaisieHtmlConstants.FICHECREATION_PAGE);
                addSubmit("global_link_fiche_new", false);
            }
            endForm();
            endDiv();
            divAttributes.setId("SSBLOC_GO2");
            startDiv(divAttributes);
            printGoTo();
            endDiv();
        }

        end();
    }

    private void printActionSelect() {
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setId("JQ_ACTIONSELECT");
        selectAttributes.setStyle("background-image: url(theme/icons/16x16/menu_link_actions.png);");
        startCell();
        startSelect(selectAttributes);
        printOption("", "menu_link_actions", "OPTION_NOACTION");
        List<MenuLink> menuLinkList = AffichageHtmlUtils.getMenuLinkList(this);
        for (MenuLink menuLink : menuLinkList) {
            printOption(menuLink);
        }
        if (isAdmin) {
            printSeparatorOptionGroup("menu_group_update");
            printOption("Administration?page=transformationindex&reinit=1", "menu_link_update_transform", null);
            printOption("Administration?page=tableexportindex&reinit=1", "menu_link_update_tableexport", null);
            printOption("Administration?page=balayageindex&reinit=1", "menu_link_update_balayage", null);
            endOptGroup();
        }
        endSelect();
        endCell();
    }

    private void printOption(String value, String key, String id) {
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setId(id);
        if (value.length() > 0) {
            value = "Saisie:" + value;
            optionAttributes.setStyle("background-image: url(theme/icons/16x16/" + key + ".png);");
        }
        optionAttributes.setValue(value);
        startOption(optionAttributes);
        localize(key);
        endOption();
    }

    private void printOption(MenuLink menuEntry) {
        String target = menuEntry.getTarget();
        if (target == null) {
            target = "Saisie";
        }
        String url = menuEntry.getUrl();
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setValue(target + ":" + url);
        String iconPath = menuEntry.getIconPath();
        if (iconPath != null) {
            optionAttributes.setStyle("background-image: url(" + iconPath + ");");
        }
        startOption(optionAttributes);
        escape(menuEntry.getIntitule());
        endOption();
    }

    private void printSeparatorOptionGroup(String key) {
        HTMLAttributes optAttributes = new HTMLAttributes();
        optAttributes.setLabel(this.getLocalization(key));
        startOptGroup(optAttributes);

    }

    private void printLinkCell(String href, String target, String key, HTMLAttributes divAttributes) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setHref(href);
        htmlAttributes.setTarget(target);
        htmlAttributes.setClasses("Icon");
        startCell();
        if (divAttributes != null) {
            startDiv(divAttributes);
        }
        startA(htmlAttributes);
        appendAnonymImg(key);
        space();
        localize(key);
        endA();
        if (divAttributes != null) {
            endDiv();
        }
        endCell();
    }

    private void printLinkCell(String href, String target, String key) {
        printLinkCell(href, target, key, null);
    }

    private void appendAnonymImg(String key) {
        HTMLAttributes imgHTMLAttributes = new HTMLAttributes();
        imgHTMLAttributes.setSrc("theme/icons/16x16/" + key + ".png");
        imgHTMLAttributes.setAlt("");
        imgHTMLAttributes.setWidth("16");
        imgHTMLAttributes.setHeight("16");
        appendImg(imgHTMLAttributes);
    }

    private void printBlocDroitLink(String href, String target, String key) {
        startA(href, target);
        appendImg("theme/icons/16x16/" + key + ".png", key, null, 16, 16, "Icon");
        endA();
    }

    private void printGoTo() {
        if (isWithJQuery()) {
            BdfHtmlUtils.printLibColon(this, "menu_lib_goto");
            HTMLAttributes txtAttributes = new HTMLAttributes();
            txtAttributes.setName("goto");
            txtAttributes.setSize("6");
            txtAttributes.setId("goto_fiche");
            appendTextInput(txtAttributes);
            printButton("fiche_display");
            printButton("fiche_edit");
            if (isAdmin) {
                printButton("fiche_index");
            }
        }
    }

    private void printButton(String key) {
        space();
        startA(getAAttributes(key));
        key = "global_link_" + key;
        appendImg("theme/icons/16x16/" + key + ".png", key, null, 16, 16, "Icon");
        endA();
    }

    private HTMLAttributes getAAttributes(String key) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setHref("#");
        htmlAttributes.setTarget("Saisie");
        htmlAttributes.setOnClick("return BDF_Menu.goToFiche(this,'" + key + "');");
        return htmlAttributes;
    }

    private String getSessionString() {
        StringBuffer buf = new StringBuffer();
        buf.append(getLocalization("menu_lib_openby"));
        buf.append(" ");
        buf.append(redacteur.getPersonneBean().getNomComplet(PersonneBean.NORMAL_STYLE));
        buf.append(" – ");
        buf.append(redacteur.getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE));
        return buf.toString();
    }

}
