/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.album;

import fr.exemole.bdfserver.commands2.album.AlbumCreationCommand;
import fr.exemole.bdfserver.commands2.album.AlbumDimModifCommand;
import fr.exemole.bdfserver.commands2.album.AlbumDimCreationCommand;
import fr.exemole.bdfserver.commands2.album.DimUsageModifCommand;
import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.UtilHtmlPrinter;


/**
 *
 * @author Vincent Calame
 */
public class AlbumCommandBoxUtils implements AlbumConstants {

    public static void printAlbumCreationBox(UtilHtmlPrinter printer, CommandBox commandBox) {
        commandBox.setCommandString(AlbumCreationCommand.COMMANDNAME);
        commandBox.setCommandKey(AlbumCreationCommand.COMMANDKEY);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("album_lib_albumcreation");
        printer.appendMediumInput(AlbumCreationCommand.NEWALBUM_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("album_submit_albumcreation");
    }

    public static void printAlbumDimModifBox(UtilHtmlPrinter printer, CommandBox commandBox, AlbumDim albumDim) {
        commandBox.setCommandString(AlbumDimModifCommand.COMMANDNAME);
        commandBox.setCommandKey(AlbumDimModifCommand.COMMANDKEY);
        CommandBoxUtils.insertAlbumDim(commandBox, albumDim);
        printer.startCommandBox(commandBox);
        short type = albumDim.getType();
        if (AlbumUtils.needWidth(type)) {
            printer.startP();
            printer.printLibColon("album_lib_albumdim_width");
            printer.appendSmallInput(AlbumDimModifCommand.ALBUMDIM_WIDTH_PARAMNAME, String.valueOf(albumDim.getWidth()));
            printer.endP();
        }
        if (AlbumUtils.needHeight(type)) {
            printer.startP();
            printer.printLibColon("album_lib_albumdim_height");
            printer.appendSmallInput(AlbumDimModifCommand.ALBUMDIM_HEIGHT_PARAMNAME, String.valueOf(albumDim.getHeight()));
            printer.endP();
        }
        printer.endCommandBox("album_submit_albumdimmodif");
    }

    public static void printDimUsageModifBox(UtilHtmlPrinter printer, CommandBox commandBox, Album album) {
        String vignetteName = "";
        AlbumDim vignetteDim = album.getAlbumMetadata().getAlbumDimForUsage(VIGNETTE_USAGE);
        if (vignetteDim != null) {
            vignetteName = vignetteDim.getName();
        }
        String detailName = "";
        AlbumDim detailDim = album.getAlbumMetadata().getAlbumDimForUsage(DETAIL_USAGE);
        if (detailDim != null) {
            detailName = detailDim.getName();
        }
        commandBox.setCommandString(DimUsageModifCommand.COMMANDNAME);
        commandBox.setCommandKey(DimUsageModifCommand.COMMANDKEY);
        CommandBoxUtils.insertAlbum(commandBox, album);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("album_lib_dimusage_vignette");
        printer.appendMediumInput(DimUsageModifCommand.VIGNETTE_PARAMNAME, vignetteName);
        printer.endP();
        printer.startP();
        printer.printLibColon("album_lib_dimusage_detail");
        printer.appendMediumInput(DimUsageModifCommand.DETAIL_PARAMNAME, detailName);
        printer.endP();
        printer.endCommandBox("album_submit_dimusagemodif");
    }

    public static void printAlbumDimRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, AlbumDim albumDim) {
        //printer.startCommandBox(AlbumDimRemoveCommand.COMMANDNAME,AlbumDimRemoveCommand.COMMANDKEY);
        /*printer.startP();
        printer.printLibColon("sphere_lib_albumcreation");
        printer.appendMediumInput(AlbumCreationCommand.NEWALBUM_PARAMNAME, "");
        printer.endP();*/
        //printer.endCommandBox("album_submit_albumcreation");
    }

    public static void printAlbumDimCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Album album) {
        commandBox.setCommandString(AlbumDimCreationCommand.COMMANDNAME);
        commandBox.setCommandKey(AlbumDimCreationCommand.COMMANDKEY);
        CommandBoxUtils.insertAlbum(commandBox, album);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("sphere_lib_newalbumdim");
        printer.appendMediumInput(AlbumDimCreationCommand.NEWALBUMDIM_PARAMNAME, "");
        printer.endP();
        printer.startP();
        printer.printLibColon("sphere_lib_albumdimtype");
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setName(AlbumDimCreationCommand.ALBUMDIMTYPE_PARAMNAME);
        printer.startSelect(selectAttributes);
        printOption(printer, FIXED_WIDTH);
        printOption(printer, FIXED_HEIGHT);
        printOption(printer, MAX_WIDTH);
        printOption(printer, MAX_HEIGHT);
        printOption(printer, MAX_DIM);
        printer.endSelect();
        printer.endP();
        printer.endCommandBox("album_submit_albumdimcreation");
    }

    private static void printOption(UtilHtmlPrinter printer, short albumDimType) {
        String value = AlbumUtils.dimTypeToString(albumDimType);
        printer.appendOption(value, false, "album_albumdimtype_" + value);
    }

}
