/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.configuration.LinkPathGroupCommand;
import fr.exemole.bdfserver.commands2.configuration.PageSubstitutionCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.LinkPathGroup;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.List;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class ExtensionsFormHtmlProducer extends BdfHtmlProducer implements ConfigurationHtmlConstants {

    private ConfigurationManager configurationManager;

    public ExtensionsFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        BdfHtmlUtils.addTestJavascript(this);
        configurationManager = bdfServer.getConfigurationManager();
    }

    public void printHtml() {
        start();
        ConfigurationHtmlUtils.printTitre(this, "configuration_title_extensions", EXTENSIONSFORM_PAGE);
        printMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Configuration");
        commandBox.setCommandFamily("CNF");
        commandBox.setPage(EXTENSIONSFORM_PAGE);
        printPageSubstitution(commandBox);
        printLinkPathGroupList(commandBox);
        end();
    }

    private void printPageSubstitution(CommandBox commandBox) {
        List<PageSubstitution> pageSubstitutionList = bdfServer.getConfigurationManager().getPageSubstitutionList();
        commandBox.setCommandString(PageSubstitutionCommand.COMMANDNAME);
        commandBox.setCommandKey(PageSubstitutionCommand.COMMANDKEY);
        startCommandBox(commandBox);
        HTMLAttributes pagesAttributes = new HTMLAttributes();
        pagesAttributes.setName(PageSubstitutionCommand.PAGES_PARAMNAME);
        pagesAttributes.setCols(75);
        int rows = 10;
        if (pageSubstitutionList.size() > 8) {
            rows = 25;
        }
        pagesAttributes.setRows(rows);
        startTextArea(pagesAttributes);
        for (PageSubstitution pageSubstitution : pageSubstitutionList) {
            escape(pageSubstitution.getOriginalName());
            append(" = ");
            escape(pageSubstitution.getSubstitutionName());
            append("\n");
        }
        endTextArea();
        endCommandBox("configuration_submit_pagesubstitution");
    }

    private void printLinkPathGroupList(CommandBox commandBox) {
        commandBox.setCommandString(LinkPathGroupCommand.COMMANDNAME);
        commandBox.setCommandKey(LinkPathGroupCommand.COMMANDKEY);
        startCommandBox(commandBox);
        printMainLinkPathGroup(ConfigurationManager.ACTIONMENU_MAINLINKPATHGROUP, "configuration_title_main_actionmenu");
        List<LinkPathGroup> list = configurationManager.getLinkPathGroupList();
        for (LinkPathGroup linkPathGroup : list) {
            String groupName = linkPathGroup.getName();
            if (!groupName.startsWith("main.")) {
                startH3();
                startCode();
                append("[");
                escape(groupName);
                append("]");
                endCode();
                endH3();
                int count = linkPathGroup.getLinkPathCount();
                HTMLAttributes groupAttributes = new HTMLAttributes();
                groupAttributes.setName(LinkPathGroupCommand.GROUP_PARAMPREFIX + groupName);
                groupAttributes.setCols(75);
                int rows = 10;
                if (count > 8) {
                    rows = 25;
                }
                groupAttributes.setRows(rows);
                startTextArea(groupAttributes);
                for (int i = 0; i < count; i++) {
                    escape(linkPathGroup.getLinkPath(i));
                    append("\n");
                }
                endTextArea();
            }
        }
        endCommandBox("configuration_submit_linkpagegroup");
    }

    private void printMainLinkPathGroup(String groupName, String libKey) {
        appendH3(libKey);
        LinkPathGroup linkPathGroup = configurationManager.getLinkPathGroup(groupName);
        int count = 0;
        if (linkPathGroup != null) {
            count = linkPathGroup.getLinkPathCount();
        }
        HTMLAttributes groupAttributes = new HTMLAttributes();
        groupAttributes.setName(LinkPathGroupCommand.GROUP_PARAMPREFIX + groupName);
        groupAttributes.setCols(75);
        int rows = 10;
        if (count > 8) {
            rows = 25;
        }
        groupAttributes.setRows(rows);
        startTextArea(groupAttributes);
        if (count > 0) {
            for (int i = 0; i < count; i++) {
                escape(linkPathGroup.getLinkPath(i));
                append("\n");
            }
        }
        endTextArea();
    }

}
