/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.configuration.GroupChangeCommand;
import fr.exemole.bdfserver.commands2.configuration.GroupCreationCommand;
import fr.exemole.bdfserver.commands2.configuration.GroupRemoveCommand;
import fr.exemole.bdfserver.commands2.configuration.PositionCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyTreeFormHtmlProducer extends BdfHtmlProducer implements ConfigurationHtmlConstants {

    private int[] workingArray;
    private SubsetKeyGroupTemplates subsetKeyGroupTemplates;

    public SubsetKeyTreeFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        BdfHtmlUtils.addTestJavascript(this);
        workingArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        subsetKeyGroupTemplates = bdfServer.getUIManager().getSubsetKeyGroupTemplates();
    }

    public void printHtml() {
        UIManager uiManager = bdfServer.getUIManager();
        start();
        ConfigurationHtmlUtils.printTitre(this, "configuration_title_listes", SUBSETKEYTREEFORM_PAGE);
        printMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Configuration");
        commandBox.setCommandFamily("CNF");
        commandBox.setPage(SUBSETKEYTREEFORM_PAGE);
        Map<String, String> hiddenMap = new HashMap<String, String>();
        commandBox.setHiddenAttributesMap(hiddenMap);
        printForm(commandBox, SubsetKeyTree.CORPUS_GROUP, PositionCommand.CORPUS_COMMANDKEY, uiManager.getCorpusSubsetKeyTree());
        printForm(commandBox, SubsetKeyTree.THESAURUS_GROUP, PositionCommand.THESAURUS_COMMANDKEY, uiManager.getThesaurusSubsetKeyTree());
        printForm(commandBox, SubsetKeyTree.SPHERE_GROUP, PositionCommand.SPHERE_COMMANDKEY, uiManager.getSphereSubsetKeyTree());
        printGroupChangeForms(commandBox);
        printGroupCreationForm(commandBox);
        end();
    }

    private void printForm(CommandBox commandBox, short type, String commandKey, SubsetKeyTree subsetKeyTree) {
        String typeString = SubsetKeyTree.toStringType(type);
        Map<String, String> hiddenMap = commandBox.getHiddenAttributesMap();
        hiddenMap.put(PositionCommand.GROUPTYPE_PARAMNAME, typeString);
        commandBox.setCommandString(PositionCommand.COMMANDNAME);
        commandBox.setCommandKey(commandKey);
        startCommandBox(commandBox);
        startTextArea(PositionCommand.SUBSETPOSITION_PARAMNAME, 40, subsetKeyTree.getSubsetKeyCount() + 3);
        List<SubsetKeyGroup> subsetKeyGroupList = subsetKeyTree.getSubsetKeyGroupList();
        for (SubsetKeyGroup group : subsetKeyGroupList) {
            boolean noneGroup = group.isNoTemplateGroup();
            if (!noneGroup) {
                append("\n");
                append("grp_");
                append(group.getTemplateName());
                append("\n");
            }
            List<SubsetKey> subsetKeyList = group.getSubsetKeyList();
            for (SubsetKey subsetKey : subsetKeyList) {
                append(subsetKey.toString());
                append("\n");
            }
            if (!noneGroup) {
                append("---");
                append("\n\n");
            }
        }
        endTextArea();
        endCommandBox("configuration_submit_ordre");
        hiddenMap.put(PositionCommand.GROUPTYPE_PARAMNAME, null);
    }

    private void printGroupChangeForms(CommandBox commandBox) {
        int size = subsetKeyGroupTemplates.size();
        if (size == 0) {
            return;
        }
        Map<String, String> hiddenMap = commandBox.getHiddenAttributesMap();
        appendH2("configuration_title_groupchange");
        for (int j = 0; j < size; j++) {
            String groupName = subsetKeyGroupTemplates.getGroupName(j);
            startH3();
            startCode();
            append("[grp_");
            append(groupName);
            append("]");
            endCode();
            BdfHtmlUtils.printTiret(this);
            escape(subsetKeyGroupTemplates.getLibelleString(groupName, workingLangInteger));
            endH3();
            commandBox.setCommandString(GroupChangeCommand.COMMANDNAME);
            commandBox.setCommandKey(GroupChangeCommand.COMMANDKEY);
            hiddenMap.put(GroupChangeCommand.GROUPNAME_PARAMNAME, groupName);
            startCommandBox(commandBox);
            startTable();
            for (int i = 0; i < workingArray.length; i++) {
                int langInteger = workingArray[i];
                BdfHtmlUtils.printLangFieldRow(this, GroupChangeCommand.GROUP_PARAMPREFIX, langInteger, subsetKeyGroupTemplates.getLibelleString(groupName, langInteger));
            }
            endTable();
            endCommandBox("configuration_submit_groupchange");
            hiddenMap.put(GroupChangeCommand.GROUPNAME_PARAMNAME, null);
            commandBox.setCommandString(GroupRemoveCommand.COMMANDNAME);
            commandBox.setCommandKey(GroupRemoveCommand.COMMANDKEY);
            hiddenMap.put(GroupRemoveCommand.GROUPNAME_PARAMNAME, groupName);
            startCommandBox(commandBox);
            BdfHtmlUtils.printConfirmPassword(this);
            endCommandBox("configuration_submit_groupremove");
            hiddenMap.put(GroupRemoveCommand.GROUPNAME_PARAMNAME, null);
        }
    }

    private void printGroupCreationForm(CommandBox commandBox) {
        commandBox.setCommandString(GroupCreationCommand.COMMANDNAME);
        commandBox.setCommandKey(GroupCreationCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            BdfHtmlUtils.printLangFieldRow(this, GroupChangeCommand.GROUP_PARAMPREFIX, langInteger, "");
        }
        endTable();
        endCommandBox("configuration_submit_groupcreation");
    }

}
