/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.envoi;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.email.EnvoiParameters;
import fr.exemole.bdfserver.email.ValidAddress;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.response.EnvoiResponseParameters;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.List;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.sphere.RedacteurKey;


/**
 *
 * @author Vincent Calame
 */
public class SendConfirmationHtmlProducer extends BdfHtmlProducer implements EnvoiHtmlConstants {

    private EnvoiParameters envoiParameters;

    public SendConfirmationHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, EnvoiParameters envoiParameters) {
        init(bdfContext, bdfUser);
        setCssThemeFile("envoi.css");
        this.envoiParameters = envoiParameters;
    }

    public void printHtml() {
        start();
        printMessage();
        appendH1("envoi_title_rappel");
        short envoiType = envoiParameters.getEnvoiType();
        AbregeFiche[] abregeFicheArray = envoiParameters.getAbregeFicheArray();
        if (envoiType == EnvoiParameters.ENVOI_COMPILATION) {
            appendH2("envoi_title_rpcompil");
            AbregeFicheSortedList triAbregesFiches = bdfUser.getAbregeFicheSortedList();
            for (int i = 0; i < triAbregesFiches.getCorpusCount(); i++) {
                Corpus corpus = triAbregesFiches.getCorpus(i);
                SubsetItemList abregeFicheList = triAbregesFiches.getCorpusAbregeFicheList(i);
                startH3();
                escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
                endH3();
                startUl();
                int ficheCount = abregeFicheList.getSubsetItemCount();
                for (int j = 0; j < ficheCount; j++) {
                    AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                    startLi();
                    startP();
                    escape(BdfUserUtils.getAbregeFicheString(bdfUser, abrege));
                    endP();
                    endLi();
                }
                endUl();
            }
        } else if (envoiType == EnvoiResponseParameters.ENVOI_FICHEUNIQUE) {
            appendH2("envoi_title_rpfiches");
            startUl();
            startLi();
            startP();
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFicheArray[0]));
            endP();
            endLi();
            endUl();
        } else {
            if (abregeFicheArray == null) {
                startP();
                localize("envoi_info_nofiche");
                endP();
            } else {
                appendH2("envoi_title_rpfiches");
                int length = abregeFicheArray.length;
                startUl();
                for (int i = 0; i < length; i++) {
                    AbregeFiche abregeFiche = abregeFicheArray[i];
                    startLi();
                    startP();
                    escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
                    endP();
                    endLi();
                }
                endUl();
            }
        }
        appendH2("envoi_title_rpto");
        List<ValidAddress> validAddressList = envoiParameters.getValidAddressList();
        startUl();
        for (ValidAddress validAddress : validAddressList) {
            startLi();
            startP();
            RedacteurKey redacteurKey = validAddress.getRedacteurKey();
            CourrielBean courrielBean = validAddress.getCourrielBean();
            escape(courrielBean.toString());
            if (redacteurKey != null) {
                append(" (");
                escape(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                append(")");
            }
            endP();
            endLi();
        }
        endUl();
        end();
    }

}
