/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.FicheResultHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.util.BdfServerUtils;
import fr.exemole.bdfserver.util.RattachementPermission;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFicheResultHtmlProducer extends BdfHtmlProducer implements FicheResultHtmlProducer, SaisieHtmlConstants {

    private AbregeFiche abregeFiche;

    public DefaultFicheResultHtmlProducer() {
    }

    public void init(BdfContext bdfContext, BdfUser bdfUser, AbregeFiche abregeFiche) {
        init(bdfContext, bdfUser);
        this.abregeFiche = abregeFiche;
        setTitle("BDF - " + FICHERESULT_PAGE);
        setCssThemeFile("saisie.css");
    }

    public void printHtml() {
        start();
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setStyle("height: 90%; width: 90%;position: absolute;");
        String fichePath = "fiches/" + RequestUtils.toBasename(abregeFiche) + ".html";
        start();
        startDiv(divAtt);
        printMessage();
        startP();
        appendA("Affichage?page=fiches", "saisie_link_listemaj", "Liste");
        endP();
        String backHref = "Saisie?" + RequestUtils.toParameterString(abregeFiche) + "&page=" + FICHEMODIF_PAGE;
        startP();
        appendA(backHref, "saisie_link_back");
        endP();
        if (bdfServer.getUIManager().getCorpusUI(abregeFiche.getCorpus()).isAssociationActive()) {
            String associationHref = "Association?" + RequestUtils.toParameterString(abregeFiche);
            startP();
            appendA(associationHref, "saisie_link_association");
            endP();
        }
        printRattachement();
        HTMLAttributes iframeAttributes = new HTMLAttributes();
        iframeAttributes.setWidth("95%");
        iframeAttributes.setHeight("80%");
        iframeAttributes.setSrc(fichePath);
        iframeAttributes.setScrolling("yes");
        startIframe(iframeAttributes);
        startP();
        appendA(fichePath, "saisie_link_affichage");
        endP();
        endIframe();
        startP();
        appendA(fichePath, "saisie_link_affichage");
        endP();
        endDiv();
        end();
    }

    private void printRattachement() {
        RattachementPermission rattachementPermission = BdfServerUtils.getRattachementPermission(bdfServer, bdfUser, abregeFiche);
        if (rattachementPermission.onlyReadAllowed()) {
            return;
        }
        int length = rattachementPermission.getSlaveCorpusCount();
        if (length == 0) {
            return;
        }
        String createSuffix = "&amp;" + RequestUtils.IDCORPUS_PARAMNAME + "=" + String.valueOf(abregeFiche.getIdcorpus());
        startH2();
        printLibColon("saisie_title_rattachement");
        endH2();
        startUl();
        for (int i = 0; i < length; i++) {
            short type = rattachementPermission.getActionType(i);
            switch (type) {
                case RattachementPermission.READ_ACTION:
                    continue;
                case RattachementPermission.WRITE_ACTION:
                    AbregeFiche other = rattachementPermission.getAbregeFiche(i);
                    startLi();
                    startP();
                    printLibColon("saisie_lib_rattachement_modif");
                    space();
                    String href = "Saisie?" + RequestUtils.toParameterString(other);
                    startA(href);
                    escape(CorpusMetadataUtils.getFicheIntituleString(other, localisationInfo));
                    endA();
                    endP();
                    endLi();
                    break;
                case RattachementPermission.CREATE_ACTION:
                    Corpus otherCorpus = rattachementPermission.getCorpus(i);
                    startLi();
                    startP();
                    printLibColon("saisie_lib_rattachement_create");
                    space();
                    String createHref = "Saisie?" + RequestUtils.toParameterString(otherCorpus) + createSuffix;
                    startA(createHref);
                    escape(CorpusMetadataUtils.getCorpusIntituleString(otherCorpus, workingLangInteger));
                    endA();
                    endP();
                    endLi();
                    break;
                default:
                    throw new SwitchException("wrong type : " + type);
            }
        }
        endUl();
    }

}
