/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.commands2.saisie.FicheCreationCommand;
import fr.exemole.bdfserver.commands2.saisie.FicheModifCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.forms.CommonFormHtml;
import fr.exemole.bdfserver.html.forms.CorpusFieldFormHtml;
import fr.exemole.bdfserver.html.forms.IncludeFormHtml;
import fr.exemole.bdfserver.request.form.CorpusFieldElement;
import fr.exemole.bdfserver.request.form.EditableAssociationElement;
import fr.exemole.bdfserver.request.form.FicheForm;
import fr.exemole.bdfserver.request.form.FormElement;
import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormFactory;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormHtmlProducer extends BdfHtmlProducer implements SaisieHtmlConstants, FicheFormCatalog, GlobalCatalog {

    private Corpus corpus;
    private String title;
    private SubsetItem masterSubsetItem = null;
    private FicheForm ficheForm;
    private List<Object> elementList;
    private boolean isMultiPart = false;
    private String ficheResultPage = FICHERESULT_PAGE;
    private String commandName;
    int idcorpus = -1;

    private FicheFormHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList) {
        init(ficheFormParameters.getBdfContext(), ficheFormParameters.getBdfUser());
        addJavascriptFiles(FICHEFORM_JAVASCRIPT_LIB);
        this.corpus = ficheFormParameters.getFichePointeur().getCorpus();
        ficheForm = FicheFormFactory.newInstance(ficheFormParameters);
        initElementList(componentUIList);
        setTitle("BDF - " + FICHEMODIF_PAGE);
        setCssThemeFile("saisie.css");
    }

    public FicheFormHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList, Corpus corpus) {
        this(ficheFormParameters, componentUIList);
        this.commandName = FicheCreationCommand.COMMANDNAME;
        title = CorpusMetadataUtils.getIntituleString(corpus.getCorpusMetadata(), CorpusConstants.FICHE_NEW, workingLangInteger, true);
    }

    public FicheFormHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList, SubsetItem masterSubsetItem) {
        this(ficheFormParameters, componentUIList);
        this.commandName = FicheModifCommand.COMMANDNAME;
        this.idcorpus = masterSubsetItem.getIdsubset();
        this.masterSubsetItem = masterSubsetItem;
        title = CorpusMetadataUtils.getIntituleString(corpus.getCorpusMetadata(), CorpusConstants.FICHE_NEW, workingLangInteger, true);
    }

    public FicheFormHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList, int idcorpus) {
        this(ficheFormParameters, componentUIList);
        this.commandName = FicheModifCommand.COMMANDNAME;
        this.idcorpus = idcorpus;
        title = CorpusMetadataUtils.getIntituleString(corpus.getCorpusMetadata(), CorpusConstants.FICHE_NEW, workingLangInteger, true);
    }

    public FicheFormHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList, AbregeFiche abregeFiche) {
        this(ficheFormParameters, componentUIList);
        this.commandName = FicheModifCommand.COMMANDNAME;
        this.idcorpus = abregeFiche.getIdcorpus();
        title = BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche);
    }

    public void setFicheResultPage(String ficheResultPage) {
        this.ficheResultPage = ficheResultPage;
    }

    private void initElementList(ComponentUIList componentUIList) {
        elementList = new ArrayList();
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                CorpusFieldElement formElement = ficheForm.getFormElement((FieldUI) componentUI);
                if (formElement != null) {
                    elementList.add(formElement);
                }
            } else if (componentUI instanceof IncludeUI) {
                IncludeElement includeElement = ficheForm.getFormElement((IncludeUI) componentUI);
                if (includeElement != null) {
                    if (includeElement instanceof EditableAssociationElement) {
                        isMultiPart = true;
                    }
                    elementList.add(includeElement);
                }
            } else if (componentUI instanceof CommentUI) {
                elementList.add((CommentUI) componentUI);
            } else {
                throw new IllegalArgumentException("unknown ComponentUI implementation : " + componentUI.getClass().getName());
            }
        }
    }

    public void printHtml() {
        start();
        if (isWithJQuery()) {
            CommonFormHtml.printFFCoreLocValues(this, getLocalisationInfo());
            CorpusFieldFormHtml.printCommonLocValues(this);
            IncludeFormHtml.printFFCoreLocValues(this);
        }
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction("Saisie");
        formAttributes.setMethod("POST");
        if (isMultiPart) {
            formAttributes.setEnctype(HTMLAttributes.MULTIPART_ENCTYPE);
        }
        Map<String, String> hiddenParametersMap = new HashMap<String, String>();
        BdfHtmlUtils.addCommandParameter(hiddenParametersMap, commandName);
        BdfHtmlUtils.addPageParameter(hiddenParametersMap, ficheResultPage);
        BdfHtmlUtils.addSubsetParameter(hiddenParametersMap, corpus);
        if (idcorpus != -1) {
            BdfHtmlUtils.addSubsetItemParameter(hiddenParametersMap, SubsetKey.CATEGORY_CORPUS, idcorpus);
        }
        BdfHtmlUtils.startForm(this, formAttributes, hiddenParametersMap);
        BdfHtmlUtils.printFixedFormTitle(this, title);
        HTMLAttributes formBodyAttributes = new HTMLAttributes();
        formBodyAttributes.addClass(DIV_GLB_FORMBODY);
        startDiv(formBodyAttributes);
        breakLine();
        if (masterSubsetItem != null) {
            startP();
            if (masterSubsetItem instanceof AbregeFiche) {
                printLibColon("saisie_lib_slave_corpus");
                space();
                String titre = CorpusMetadataUtils.getFicheIntituleString((AbregeFiche) masterSubsetItem, localisationInfo);
                escape(titre);
            } else if (masterSubsetItem instanceof Motcle) {
                printLibColon("saisie_lib_slave_thesaurus");
                space();
                BdfHtmlUtils.printMotcleLibelle(this, (Motcle) masterSubsetItem);
            }
            endP();
        }
        int count = elementList.size();
        for (int i = 0; i < count; i++) {
            Object obj = elementList.get(i);
            if (obj instanceof FormElement) {
                CommonFormHtml.printFormElement(this, (FormElement) obj, null);
            } else if (obj instanceof CommentUI) {
                printCommentUI((CommentUI) obj);
            }
        }
        endDiv();
        BdfHtmlUtils.endForm(this, "global_submit_ok");
        end();
    }

    private void printCommentUI(CommentUI commentUI) {
        if ((commentUI.getLocation() & CommentUI.FORM_BITVALUE) != 0) {
            String lib = commentUI.getLibelleString(workingLangInteger, false);
            if (lib != null) {
                CorpusFieldFormHtml.printComment(this, lib);
            }
        }
    }

}
