/* @@@@@APPLICATION@@@@@ - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.saisie.SelectionIndexationCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class SelectionIndexationFormHtmlProducer extends BdfHtmlProducer implements SaisieHtmlConstants {

    private static HTMLAttributes delCheckDiv = new HTMLAttributes();
    private static HTMLAttributes addCheckDiv = new HTMLAttributes();
    private static HTMLAttributes addFicheCell = new HTMLAttributes();
    private static HTMLAttributes delFicheCell = new HTMLAttributes();
    private static HTMLAttributes corpusTitleCell = new HTMLAttributes();
    private Motcle motcle;

    static {
        delCheckDiv.setClasses("DelCheck");
        addCheckDiv.setClasses("AddCheck");
        addFicheCell.setClasses("AddFiche");
        delFicheCell.setClasses("DelFiche");
        corpusTitleCell.setClasses("CorpusTitle");
        corpusTitleCell.setColSpan(3);
    }

    private HTMLAttributes checkbox = new HTMLAttributes();
    private HTMLAttributes text = new HTMLAttributes();

    {
        checkbox.setType("checkbox");
        text.setType("text");
        text.setOnKeyUp("check(this);");
        text.setSize("1");
        text.setValue("");
    }

    public SelectionIndexationFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Motcle motcle) {
        init(bdfContext, bdfUser);
        setCssThemeFile("saisie_massform.css");
        this.motcle = motcle;
    }

    public void printHtml() {
        Thesaurus thesaurus = motcle.getThesaurus();
        start();
        startP("Title");
        localize("saisie_title_massform");
        endP();
        startP();
        localize("global_type_motcle");
        space();
        startA("Thesaurus?" + RequestUtils.toParameterString(motcle));
        BdfHtmlUtils.printMotcleLibelle(this, motcle, workingLangInteger);
        endA();
        endP();
        startP();
        localize("global_type_thesaurus");
        space();
        escape(ThesaurusUtils.getThesaurusIntitule(motcle.getThesaurus(), workingLangInteger));
        endP();
        startTable("100%");
        HTMLAttributes colAttributes = new HTMLAttributes();
        colAttributes.setWidth("*");
        colAttributes.setSpan("2");
        appendCol(colAttributes);
        HTMLAttributes cellAttributes = new HTMLAttributes();
        startRow();
        cellAttributes.setClasses("AddTitle");
        startCell(cellAttributes);
        localize("saisie_title_massform_ajouter");
        endCell();
        cellAttributes.setClasses("DelTitle");
        startCell(cellAttributes);
        localize("saisie_title_massform_supprimer");
        endCell();
        endRow();
        startRow();
        startCell();
        printButton("selection('ajout',true);", "saisie_button_selajout");
        printButton("selection('ajout',false);", "saisie_button_deselajout");
        endCell();
        HTMLAttributes cell2Attributes = new HTMLAttributes();
        cell2Attributes.setStyle("text-align: right;");
        startCell(cell2Attributes);
        printButton("selection('supp',true);", "saisie_button_selsupp");
        printButton("selection('supp',false);", "saisie_button_deselsupp");
        endCell();
        endRow();
        endTable();
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction("Saisie");
        formAttributes.setMethod("POST");
        formAttributes.setName("saisie");
        Map<String, String> hiddenAttributesMap = new HashMap<String, String>();
        BdfHtmlUtils.addCommandParameter(hiddenAttributesMap, SelectionIndexationCommand.COMMANDNAME);
        BdfHtmlUtils.addPageParameter(hiddenAttributesMap, SELECTIONINDEXATION_RESULT_PAGE);
        BdfHtmlUtils.addSubsetParameter(hiddenAttributesMap, thesaurus);
        BdfHtmlUtils.addSubsetItemParameter(hiddenAttributesMap, motcle);
        BdfHtmlUtils.startForm(this, formAttributes, hiddenAttributesMap);
        startTable("100%");
        HTMLAttributes col2Attributes = new HTMLAttributes();
        col2Attributes.setWidth("90");
        appendCol(col2Attributes);
        col2Attributes.setWidth("0*");
        appendCol(col2Attributes);
        col2Attributes.setWidth("45");
        appendCol(col2Attributes);
        AbregeFicheSortedList sortedList = bdfUser.getAbregeFicheSortedList();
        HashSet set = new HashSet();
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        StringBuffer arrayBuffer = new StringBuffer();
        for (int i = 0; i < sortedList.getCorpusCount(); i++) {
            Corpus corpus = sortedList.getCorpus(i);
            Subset masterSubset = corpus.getMasterSubset();
            if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                continue;
            }
            SubsetItemList abregeFicheList = sortedList.getCorpusAbregeFicheList(i);
            int ficheCount = abregeFicheList.getSubsetItemCount();
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurusKey);
            if (arrayBuffer.length() > 0) {
                arrayBuffer.append(",");
            }
            arrayBuffer.append("\"");
            arrayBuffer.append(corpus.getSubsetKey().getSubsetName());
            arrayBuffer.append("\"");
            set.clear();
            CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
            int ciCount = croisementItem.getCount();
            for (int j = 0; j < ciCount; j++) {
                int tcktcrps = croisementItem.getTicket(j);
                set.add(new Integer(tcktcrps));
            }
            startRow();
            startCell();
            append("&nbsp;");
            endCell();
            startCell();
            append("&nbsp;");
            endCell();
            startCell();
            append("&nbsp;");
            endCell();
            endRow();
            startRow();
            startCell(corpusTitleCell);
            escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
            append(" [");
            append(String.valueOf(ficheCount));
            append("]");
            endCell();
            endRow();
            for (int j = 0; j < ficheCount; j++) {
                AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                boolean indexe = set.contains(new Integer(abrege.getTicket()));
                printLigneAbrege(abrege, indexe);
            }
        }
        endTable();
        BdfHtmlUtils.endForm(this, "global_submit_ok");
        startScript(null);
        append("var corpusArray= [");
        append(arrayBuffer.toString());
        append("];");
        endScript();
        end();
    }

    private void printLigneAbrege(AbregeFiche abrege, boolean indexe) {
        String nomCorpus = abrege.getCorpus().getSubsetKey().getSubsetName();
        String idcorpus_string = String.valueOf(abrege.getIdcorpus());
        checkbox.setValue(idcorpus_string);
        startRow();
        startCell();
        if (!indexe) {
            String id = nomCorpus + "_" + idcorpus_string;
            startDiv(addCheckDiv);
            checkbox.setName("ajout_" + nomCorpus);
            checkbox.setId(id);
            appendInput(checkbox);
            append(" ");
            text.setName("poids_" + id);
            appendInput(text);
            append("&nbsp;");
            endDiv();
        }
        endCell();
        if (indexe) {
            startCell(delFicheCell);
        } else {
            startCell(addFicheCell);
        }
        append(idcorpus_string);
        BdfHtmlUtils.printTiret(this);
        escape(abrege.getTitre());
        endCell();
        startCell();
        if (indexe) {
            startDiv(delCheckDiv);
            checkbox.setName("supp_" + nomCorpus);
            checkbox.setId(null);
            appendInput(checkbox);
            endDiv();
        }
        endCell();
        endRow();
    }

    private void printButton(String onclick, String key) {
        HTMLAttributes buttonAttributes = new HTMLAttributes();
        buttonAttributes.setType("button");
        buttonAttributes.setOnClick(onclick);
        buttonAttributes.setValue(getLocalization(key));
        appendInput(buttonAttributes);
    }

}
