/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.actions.CreateFiche;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.localisation.ResourceItem;


/**
 *
 * @author Vincent Calame
 */
public class SlaveFicheCreationHtmlProducer extends BdfHtmlProducer implements SaisieHtmlConstants {

    static final String CORPUS_TYPE = "corpus";
    static final String THESAURUS_TYPE = "thesaurus";
    private Corpus corpus;
    private Subset masterSubset;
    private TreeMap<Object, SubsetItem> subsetItemMap = new TreeMap();
    private String subsetIntitule;
    private String corpusIntitule;
    private String type;
    private int thesaurusLangInteger;

    public SlaveFicheCreationHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Corpus corpus) {
        init(bdfContext, bdfUser);
        this.corpus = corpus;
        masterSubset = corpus.getMasterSubset();
        if (masterSubset instanceof Corpus) {
            type = CORPUS_TYPE;
            subsetIntitule = CorpusMetadataUtils.getCorpusIntituleString((Corpus) masterSubset, workingLangInteger);
        } else if (masterSubset instanceof Thesaurus) {
            type = THESAURUS_TYPE;
            subsetIntitule = ThesaurusUtils.getThesaurusIntitule((Thesaurus) masterSubset, workingLangInteger);
            thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility((Thesaurus) masterSubset, workingLangInteger);
        }
        initSubsetItemMap();
        corpusIntitule = CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger);
        setCssThemeFile("saisie.css");
    }

    public void printHtml() {
        start();
        if (subsetItemMap.size() == 0) {
            ResourceItem resourceItem = getResourceItem("saisie_err_slaveall_");
            startP();
            startSpan(ERRORMESSAGE_SPANCLASS);
            localize(resourceItem);
            endSpan();
            endP();
        } else {
            startP();
            localize(getResourceItem("saisie_info_slave_"));
            endP();
            startH1();
            escape(CorpusMetadataUtils.getIntituleString(corpus.getCorpusMetadata(), CorpusConstants.FICHE_NEW, workingLangInteger, true));
            endH1();
            HTMLAttributes formAttributes = new HTMLAttributes();
            formAttributes.setMethod("POST");
            formAttributes.setAction("Saisie");
            Map<String, String> hiddenParametersMap = new HashMap<String, String>();
            BdfHtmlUtils.addSubsetParameter(hiddenParametersMap, corpus);
            BdfHtmlUtils.addPageParameter(hiddenParametersMap, FICHEMODIF_PAGE);
            BdfHtmlUtils.startForm(this, formAttributes, hiddenParametersMap);
            startP();
            printLibColon(getResourceItem("saisie_lib_slave_").getResourceKey());
            space();
            HTMLAttributes selectAttributes = new HTMLAttributes();
            selectAttributes.setName(RequestUtils.IDCORPUS_PARAMNAME);
            startSelect(selectAttributes);
            for (SubsetItem subsetItem : subsetItemMap.values()) {
                if (subsetItem instanceof AbregeFiche) {
                    printOption((AbregeFiche) subsetItem);
                } else if (subsetItem instanceof Motcle) {
                    printOption((Motcle) subsetItem);
                }
            }
            endSelect();
            endP();
            BdfHtmlUtils.endForm(this, "global_submit_ok");
        }
        end();
    }

    private void printOption(AbregeFiche abregeFiche) {
        String titre = CorpusMetadataUtils.getFicheIntituleString(abregeFiche, localisationInfo);
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setValue(String.valueOf(abregeFiche.getIdcorpus()));
        startOption(optionAttributes);
        escape(titre);
        endOption();
    }

    private void printOption(Motcle motcle) {
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setValue(String.valueOf(motcle.getIdths()));
        startOption(optionAttributes);
        BdfHtmlUtils.printMotcleLibelle(this, motcle, thesaurusLangInteger);
        endOption();
    }

    private void initSubsetItemMap() {
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        int ticketLength = masterSubset.getTicketLength();
        for (int i = 0; i < ticketLength; i++) {
            SubsetItem subsetItem = masterSubset.getSubsetItemByTicket(i);
            if (subsetItem != null) {
                CreateFiche createFiche = CreateFiche.newInstance(corpus, subsetItem.getIdsubset());
                if (permissionManager.canDo(bdfUser, createFiche)) {
                    subsetItemMap.put(getKey(subsetItem), subsetItem);
                }
            }
        }
    }

    private ResourceItem getResourceItem(String key) {
        key = key + type;
        return ResourceItem.newInstance(key, corpusIntitule, subsetIntitule);
    }

    private Object getKey(SubsetItem subsetItem) {
        if (subsetItem instanceof Motcle) {
            String idalpha = ((Motcle) subsetItem).getIdalpha();
            if (idalpha != null) {
                return idalpha;
            }
        }
        return new Integer(subsetItem.getIdsubset());
    }

}
