/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.AvailableFormatLocales;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.sphere.*;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationInfo;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.roles.Role;
import fr.exemole.bdfserver.roles.RoleMap;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereConstants;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadata;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class SphereCommandBoxUtils {

    private SphereCommandBoxUtils() {
    }

    public static void printSphereCreationBox(UtilHtmlPrinter printer, CommandBox commandBox) {
        commandBox.setCommandString(SphereCreationCommand.COMMANDNAME);
        commandBox.setCommandKey(SphereCreationCommand.COMMANDKEY);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("sphere_lib_spherecreation");
        printer.appendMediumInput(SphereCreationCommand.NEWSPHERE_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("sphere_submit_spherecreation");
    }

    public static void printSphereIntituleBox(UtilHtmlPrinter printer, CommandBox commandBox, Sphere sphere, BdfServer bdfServer) {
        SphereMetadata sphereMetadata = sphere.getSphereMetadata();
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        commandBox.setCommandString(SphereIntituleCommand.COMMANDNAME);
        commandBox.setCommandKey(SphereIntituleCommand.COMMANDKEY);
        CommandBoxUtils.insertSphere(commandBox, sphere);
        printer.startCommandBox(commandBox);
        printSphereIntituleTable(printer, sphereMetadata, SphereIntituleCommand.SPHEREINTITULE_PARAMPREFIX, SphereConstants.SPHERE, workingArray);
        printer.endCommandBox("sphere_submit_intitules");
    }

    public static void printSphereRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Sphere sphere) {
        commandBox.setCommandString(SphereRemoveCommand.COMMANDNAME);
        commandBox.setCommandKey(SphereRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertSphere(commandBox, sphere);
        printer.startCommandBox(commandBox);
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("sphere_submit_sphereremove");
    }

    public static void printRedacteurCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Sphere sphere) {
        commandBox.setCommandString(RedacteurCreationCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurCreationCommand.COMMANDKEY);
        CommandBoxUtils.insertSphere(commandBox, sphere);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("sphere_lib_newidsphere");
        printer.appendSmallInput(RedacteurCreationCommand.NEWIDSPHERE_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("sphere_submit_newidsphere");
    }

    public static void printRedacteurPasswordBox(UtilHtmlPrinter printer, CommandBox commandBox, Redacteur redacteur) {
        commandBox.setCommandString(RedacteurPasswordCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurPasswordCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacteur);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("sphere_lib_password1");
        printer.appendPasswordInput(RedacteurPasswordCommand.PASSWORD1_PARAMNAME, 20);
        printer.endP();
        printer.startP();
        printer.printLibColon("sphere_lib_password2");
        printer.appendPasswordInput(RedacteurPasswordCommand.PASSWORD2_PARAMNAME, 20);
        printer.endP();
        printer.endCommandBox("sphere_submit_password");
    }

    public static void printRedacteurChangeBox(UtilHtmlPrinter printer, CommandBox commandBox, Redacteur redacteur, BdfServer bdfServer) {
        commandBox.setCommandString(RedacteurChangeCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurChangeCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacteur);
        printer.startCommandBox(commandBox);
        CourrielBean courrielBean = redacteur.getCourrielBean();
        PersonneBean personneBean = redacteur.getPersonneBean();
        printer.startTable();
        boolean withOriginal = false;
        if ((personneBean.getOriginal().length() > 0) || (bdfServer.getConfigurationManager().getLangConfig().isWithOriginalName())) {
            withOriginal = true;
        }
        if (withOriginal) {
            printField(printer, personneBean, "sphere_lib_original", RedacteurChangeCommand.ORIGINAL_PARAMNAME, personneBean.getOriginal());
            printField(printer, personneBean, "sphere_lib_nom_latin", RedacteurChangeCommand.NOM_PARAMNAME, personneBean.getNom());
            printField(printer, personneBean, "sphere_lib_prenom_latin", RedacteurChangeCommand.PRENOM_PARAMNAME, personneBean.getPrenom());
        } else {
            printField(printer, personneBean, "sphere_lib_nom", RedacteurChangeCommand.NOM_PARAMNAME, personneBean.getNom());
            printField(printer, personneBean, "sphere_lib_prenom", RedacteurChangeCommand.PRENOM_PARAMNAME, personneBean.getPrenom());
        }
        String courrielValue = (courrielBean != null) ? courrielBean.getAddrSpec() : "";
        printField(printer, personneBean, "sphere_lib_courriel", RedacteurChangeCommand.ADDRSPEC_PARAMNAME, courrielValue);
        printer.endTable();
        printer.endCommandBox("sphere_submit_redacteurchange");
    }

    public static void printLangBox(UtilHtmlPrinter printer, CommandBox commandBox, BdfUser redacBdfUser, BdfServer bdfServer, Locale formatLocale) {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        int[] langArray = configurationManager.getLangConfig().getWorkingLangIntegerArray();
        int currentLang = redacBdfUser.getWorkingLangInteger();
        int localeInteger = redacBdfUser.getLocalisationInfo().getFormatLocaleInteger();
        commandBox.setCommandString(RedacteurLangCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurLangCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacBdfUser.getRedacteurKey());
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.printLibColon("user_lib_workinglang");
        printer.startSelect(RedacteurLangCommand.WORKINGLANG_PARAMNAME);
        Map locMap = LangIntegerLocalisation.getLanguageMap(formatLocale);
        for (int i = 0; i < langArray.length; i++) {
            String langString = LangInteger.toString(langArray[i]);
            printer.startOption(langString, (langArray[i] == currentLang));
            printer.append(langString);
            BdfHtmlUtils.printTiret(printer);
            printer.escape((String) locMap.get(langString));
            printer.endOption();
        }
        printer.endSelect();
        printer.endP();
        printer.startP();
        printer.printLibColon("user_lib_formatlocale");
        printer.startSelect(RedacteurLangCommand.FORMATLOCALE_PARAMNAME);
        int[] available = AvailableFormatLocales.getAvailableArray();
        for (int i = 0; i < available.length; i++) {
            int availableInteger = available[i];
            printer.startOption(LangInteger.toString(availableInteger), (availableInteger == localeInteger));
            printer.append(LangInteger.toString(availableInteger));
            BdfHtmlUtils.printTiret(printer);
            printer.escape(LangIntegerLocalisation.getVOLibelle(availableInteger));
            printer.endOption();
        }
        printer.endSelect();
        printer.endP();
        printer.endCommandBox("user_submit_lang");
    }

    public static void printTemplatesBox(BdfHtmlProducer printer, CommandBox commandBox, BdfUser redacBdfUser) {
        commandBox.setCommandString(RedacteurTemplatesCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurTemplatesCommand.COMMANDKEY);
        TransformationPrinter lists = new TransformationPrinter(printer, redacBdfUser);
        printer.startCommandBox(commandBox);
        CommandBoxUtils.insertRedacteur(commandBox, redacBdfUser.getRedacteurKey());
        if (lists.isEmpty()) {
            printer.appendP(null, "user_lib_noaltertemplates");
            printer.endCommandBox(null);
        } else {
            if (lists.withSpecial()) {
                lists.printSpecialList();
            }
            if (lists.withCorpus()) {
                printer.appendH2("user_title_corpustemplate");
                printer.startTable();
                lists.printCorpusList();
                printer.endTable();
            }
            printer.endCommandBox("user_submit_templates");
        }
    }

    public static void printRedacteurRoleBox(BdfHtmlProducer printer, CommandBox commandBox, BdfUser redacBdfUser) {
        BdfServer bdfServer = printer.getBdfServer();
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        boolean isAdmin = BdfUserUtils.isAdmin(bdfServer, redacBdfUser.getRedacteurKey());
        commandBox.setCommandString(RedacteurRoleCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurRoleCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacBdfUser.getRedacteurKey());
        printer.startCommandBox(commandBox);
        HTMLAttributes adminAttributes = new HTMLAttributes();
        adminAttributes.setName(RedacteurRoleCommand.ADMIN_PARAMNAME);
        adminAttributes.setValue("1");
        adminAttributes.setChecked(isAdmin);
        printer.startP();
        printer.addCheck(adminAttributes, "sphere_lib_admin", null);
        printer.endP();
        printer.appendH3("sphere_title_rolelist");
        printer.startUl();
        RolePrinter rolePrinter = new RolePrinter(printer, redacBdfUser);
        rolePrinter.printRoleLi(permissionManager.getDefaultRole());
        Role[] roleArray = permissionManager.getRoleArray(false);
        for (int i = 0; i < roleArray.length; i++) {
            rolePrinter.printRoleLi(roleArray[i]);
        }
        printer.endUl();
        printer.endCommandBox("sphere_submit_role");
    }

    public static void printRedacteurActifBox(UtilHtmlPrinter printer, CommandBox commandBox, Redacteur redacteur) {
        commandBox.setCommandString(RedacteurActifCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurActifCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacteur);
        printer.startCommandBox(commandBox);
        HTMLAttributes actifAttributes = new HTMLAttributes();
        actifAttributes.setName(RedacteurActifCommand.ACTIF_PARAMNAME);
        boolean actif = redacteur.isActif();
        actifAttributes.setValue("1");
        actifAttributes.setChecked(actif);
        printer.startP();
        printer.addRadio(actifAttributes, "sphere_lib_redacteuractif_yes", null);
        printer.endP();
        actifAttributes.setValue("0");
        actifAttributes.setChecked(!actif);
        printer.startP();
        printer.addRadio(actifAttributes, "sphere_lib_redacteuractif_no", null);
        printer.endP();
        printer.endCommandBox("sphere_submit_redacteuractif");
    }

    public static void printRedacteurRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Redacteur redacteur) {
        commandBox.setCommandString(RedacteurRemoveCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacteur);
        printer.startCommandBox(commandBox);
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("sphere_submit_redacteurremove");
    }

    public static void printRedacteurReplaceBox(UtilHtmlPrinter printer, CommandBox commandBox, Redacteur redacteur) {
        commandBox.setCommandString(RedacteurReplaceCommand.COMMANDNAME);
        commandBox.setCommandKey(RedacteurReplaceCommand.COMMANDKEY);
        CommandBoxUtils.insertRedacteur(commandBox, redacteur);
        printer.startCommandBox(commandBox);
        HTMLAttributes radioAttributes = new HTMLAttributes();
        radioAttributes.setName(RedacteurReplaceCommand.REPLACETYPE_PARAMNAME);
        radioAttributes.setValue(RedacteurReplaceCommand.OTHER_REPLACETYPE_PARAMVALUE);
        radioAttributes.setChecked(true);
        printer.startP();
        printer.addRadio(radioAttributes, "sphere_lib_otherredac", null);
        BdfHtmlUtils.printColon(printer);
        printer.appendMediumInput(RedacteurReplaceCommand.OTHER_PARAMNAME, "");
        printer.endP();
        radioAttributes.setValue(RedacteurReplaceCommand.LITTERAL_REPLACETYPE_PARAMVALUE);
        radioAttributes.setChecked(false);
        printer.startP();
        printer.addRadio(radioAttributes, "sphere_lib_litteral", null);
        BdfHtmlUtils.printColon(printer);
        printer.appendMediumInput(RedacteurReplaceCommand.LITTERAL_PARAMNAME, "");
        printer.endP();
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("sphere_submit_redacteurreplace");
    }

    private static void printSphereIntituleTable(HtmlPrinter printer, SphereMetadata sphereMetadata, String paramPrefix, String name, int[] workingArray) {
        printer.appendP(null, "sphere_sphere" + paramPrefix);
        printer.startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String intitule = getLibelleString(sphereMetadata.getIntituleByName(name), langInteger);
            BdfHtmlUtils.printLangFieldRow(printer, paramPrefix, langInteger, intitule);
        }
        printer.endTable();
    }

    private static String getLibelleString(Intitule sphereIntitule, int langInteger) {
        if (sphereIntitule == null) {
            return "";
        }
        String s = sphereIntitule.getLibelleString(langInteger, false);
        if (s == null) {
            return "";
        }
        return s;
    }

    private static void printField(UtilHtmlPrinter printer, PersonneBean personneBean, String libKey, String paramName, String value) {
        printer.startRow();
        printer.startCell();
        printer.localize(libKey);
        printer.endCell();
        printer.startCell();
        printer.appendMediumInput(paramName, value);
        if (paramName.equals(RedacteurChangeCommand.NOM_PARAMNAME)) {
            printer.space();
            printer.startSmall();
            HTMLAttributes checkAttributes = new HTMLAttributes();
            checkAttributes.setName(RedacteurChangeCommand.NOMAVANT_PARAMNAME);
            checkAttributes.setValue("1");
            checkAttributes.setChecked(personneBean.isNomAvant());
            printer.addCheck(checkAttributes, "sphere_lib_nomavant", null);
            printer.endSmall();
        }
        printer.endCell();
        printer.endRow();
    }


    private static class RolePrinter {

        private BdfHtmlProducer bdfHtmlProducer;
        private Set<String> roleNameSet = new HashSet<String>();

        RolePrinter(BdfHtmlProducer bdfHtmlProducer, BdfUser redacBdfUser) {
            this.bdfHtmlProducer = bdfHtmlProducer;
            String[] roleNameArray = redacBdfUser.getRoleNameArray();
            if (roleNameArray == null) {
                roleNameSet.add(RoleMap.DEFAULT_ROLE);
            } else {
                for (int i = 0; i < roleNameArray.length; i++) {
                    roleNameSet.add(roleNameArray[i]);
                }
            }
        }

        private void printRoleLi(Role role) {
            HtmlPrinter printer = bdfHtmlProducer;
            HTMLAttributes checkAttributes = new HTMLAttributes();
            checkAttributes.setName(RedacteurRoleCommand.ROLE_PARAMNAME);
            checkAttributes.setValue(role.getName());
            if (roleNameSet.contains(role.getName())) {
                checkAttributes.setChecked(true);
            }
            printer.startLi();
            printer.startP();
            printer.startLabel(null);
            checkAttributes.setType(HTMLAttributes.CHECKBOX_TYPE);
            printer.appendInput(checkAttributes);
            BdfHtmlUtils.printRoleIntitule(bdfHtmlProducer, role);
            printer.endLabel();
            printer.endP();
            printer.endLi();
        }

    }


    private static class TransformationPrinter {

        private BdfHtmlProducer bdfHtmlProducer;
        private BdfUser redacBdfUser;
        private List<TransformationBundle> specialList = new ArrayList<TransformationBundle>();
        private List<TransformationBundle> corpusList = new ArrayList<TransformationBundle>();
        private TransformationManager transformationManager;

        TransformationPrinter(BdfHtmlProducer bdfHtmlProducer, BdfUser redacBdfUser) {
            this.bdfHtmlProducer = bdfHtmlProducer;
            this.redacBdfUser = redacBdfUser;
            BdfServer bdfServer = bdfHtmlProducer.getBdfServer();
            this.transformationManager = bdfServer.getTransformationManager();
            addInList(specialList, TransformationKey.COMPILATION_TRANSFORMATIONKEY, null);
            addInList(specialList, TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY, null);
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                TransformationKey transformationKey = new TransformationKey(corpus.getSubsetKey());
                addInList(corpusList, transformationKey, corpus);
            }
        }

        private void addInList(List<TransformationBundle> list, TransformationKey transformationKey, Corpus corpus) {
            TransformationInfo transformationInfo = transformationManager.getTransformationInfo(transformationKey);
            if ((transformationInfo.hasXslTransformer()) || (transformationInfo.hasStreamTransformer())) {
                list.add(new TransformationBundle(transformationInfo, corpus));
            }
        }

        public boolean isEmpty() {
            return (specialList.isEmpty()) && (corpusList.isEmpty());
        }

        public boolean withSpecial() {
            return !specialList.isEmpty();
        }

        public boolean withCorpus() {
            return !corpusList.isEmpty();
        }

        private void printSpecialList() {
            for (TransformationBundle transformationBundle : specialList) {
                printTransformationBundle(transformationBundle);
            }
        }

        private void printCorpusList() {
            for (TransformationBundle transformationBundle : corpusList) {
                printTransformationBundle(transformationBundle);
            }
        }

        private void printTransformationBundle(TransformationBundle transformationBundle) {
            HtmlPrinter printer = bdfHtmlProducer;
            TransformationInfo transformationInfo = transformationBundle.getTransformationInfo();
            Corpus corpus = transformationBundle.getCorpus();
            printer.startRow();
            printer.startCell();
            if (corpus != null) {
                printer.escape(CorpusMetadataUtils.getCorpusIntituleString(transformationBundle.corpus, bdfHtmlProducer.getWorkingLangInteger()));
            } else {
                printer.localize("user_templates_" + transformationInfo.getTransformationKey().toString());
            }
            printer.endCell();
            printer.startCell();
            BdfHtmlUtils.printColon(printer);
            printer.endCell();
            printer.startCell();
            if (transformationInfo.hasXslTransformer()) {
                printXslTransformerSelect(transformationInfo);
            }
            printer.endCell();
            printer.endRow();
            if (transformationInfo.hasStreamTransformer()) {
                int size = transformationInfo.getExtensionListCount();
                for (int i = 0; i < size; i++) {
                    TransformationInfo.ListByExtension listByExtension = transformationInfo.getExtensionList(i);
                    String extension = listByExtension.getExtension();
                    printer.startRow();
                    printer.appendEmptyCell();
                    printer.appendEmptyCell();
                    printer.startCell();
                    printer.append(".");
                    printer.escape(extension);
                    BdfHtmlUtils.printColon(printer);
                    printStreamTransformerSelect(listByExtension);
                    printer.endP();

                }
            }
        }

        private void printXslTransformerSelect(TransformationInfo transformationInfo) {
            HtmlPrinter printer = bdfHtmlProducer;
            HTMLAttributes optionAttributes = new HTMLAttributes();
            int size = transformationInfo.getXslTransformerCount();
            XslTransformerKey currentXslTransformerKey = redacBdfUser.getXslTransformerKey(transformationInfo.getTransformationKey());
            String currentTransformerName = currentXslTransformerKey.getTransformerName();
            boolean defaultSelected = (currentTransformerName == null);
            HTMLAttributes htmlAttributes = new HTMLAttributes();
            htmlAttributes.setName(transformationInfo.getTransformationKey().toString());
            printer.startSelect(htmlAttributes);
            optionAttributes.setValue("#null#");
            optionAttributes.setSelected(defaultSelected);
            printer.startOption(optionAttributes);
            printer.localize("user_templates_default");
            printer.endOption();
            for (int i = 0; i < size; i++) {
                XslTransformerKey xslTransformerKey = transformationInfo.getXslTransformerKey(i);
                String name = xslTransformerKey.getTransformerName();
                boolean selected = false;
                if (!defaultSelected) {
                    selected = (name.equals(currentTransformerName));
                }
                optionAttributes.setValue(name);
                optionAttributes.setSelected(selected);
                printer.startOption(optionAttributes);
                printer.append(name);
                printer.append(".xsl");
                printer.endOption();
            }
            printer.endSelect();
        }

        private void printStreamTransformerSelect(TransformationInfo.ListByExtension listByExtension) {
            HtmlPrinter printer = bdfHtmlProducer;
            TransformationKey transformationKey = listByExtension.getTransformationKey();
            String extension = listByExtension.getExtension();

            int count = listByExtension.getStreamTransformerCount();
            StreamTransformerKey currentTransformerKey = redacBdfUser.getStreamTransformerKey(listByExtension.getTransformationKey(), extension);
            if (currentTransformerKey == null) {
                if (listByExtension.isDefaultTransformerAvailable()) {
                    currentTransformerKey = StreamTransformerKey.getDefaultStreamTransformerKey(transformationKey, extension);
                } else {
                    currentTransformerKey = listByExtension.getStreamTransformerKey(0);
                }
            }
            HTMLAttributes htmlAttributes = new HTMLAttributes();
            htmlAttributes.setName(StreamTransformerKey.toStringKey(transformationKey, extension));

            printer.startSelect(htmlAttributes);
            if (listByExtension.isDefaultTransformerAvailable()) {
                printOption(currentTransformerKey.toDefaultTransformerKey(), currentTransformerKey);
            }
            for (int j = 0; j < count; j++) {
                printOption(listByExtension.getStreamTransformerKey(j), currentTransformerKey);
            }
            printer.endSelect();
        }

        private void printOption(StreamTransformerKey transformerKey, StreamTransformerKey currentTransformerKey) {
            HtmlPrinter printer = bdfHtmlProducer;
            String transformerName = transformerKey.getTransformerName();
            HTMLAttributes optionAttributes = new HTMLAttributes();
            optionAttributes.setSelected(transformerKey.equals(currentTransformerKey));
            optionAttributes.setValue(transformerKey.toString());
            printer.startOption(optionAttributes);
            if (transformerName == null) {
                printer.localize("user_templates_default");
            } else {
                printer.append(transformerName);
            }
            printer.endOption();
        }

    }


    private static class TransformationBundle {

        Corpus corpus;
        TransformationInfo transformationInfo;

        TransformationBundle(TransformationInfo transformationInfo) {
            this.transformationInfo = transformationInfo;
        }

        TransformationBundle(TransformationInfo transformationInfo, Corpus corpus) {
            this.corpus = corpus;
            this.transformationInfo = transformationInfo;
        }

        public Corpus getCorpus() {
            return corpus;
        }

        public TransformationInfo getTransformationInfo() {
            return transformationInfo;
        }

    }

}


