/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class SphereHtmlProducer extends BdfHtmlProducer implements SphereHtmlConstants {

    private Sphere sphere;
    private SortedMap<String, Redacteur> actifMap = new TreeMap<String, Redacteur>();
    private SortedMap<String, Redacteur> inactifMap = new TreeMap<String, Redacteur>();

    public SphereHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Sphere sphere) {
        init(bdfContext, bdfUser);
        addJavascriptFiles(GlobalCatalog.BDF_JAVASCRIPT);
        setCssThemeFile("sphere.css");
        setTitle(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
        this.sphere = sphere;
        initMaps();
    }

    public void printHtml() {
        start();
        if (isWithJQuery()) {
            HTMLAttributes divSubsetSelectAttributes = new HTMLAttributes();
            divSubsetSelectAttributes.addClass(GlobalCatalog.DIV_GLB_SUBSETSELECT);
            startDiv(divSubsetSelectAttributes);
            BdfHtmlUtils.printChangeSubsetSelect(this, sphere, SPHERE_PAGE);
            endDiv();
        }
        startH1();
        BdfHtmlUtils.print16pxStaticIcon(this, "global_subset_sphere", "");
        space();
        escape(getTitle());
        endH1();
        if (isAdmin()) {
            startP(GlobalCatalog.GLB_P_SMALLLINKS);
            printSphereLink(REDACTEURCREATIONFORM_PAGE, "sphere_link_newidsphere", "Saisie");
            BdfHtmlUtils.printTiret(this);
            printSphereLink(SPHEREMETADATAFORM_PAGE, "sphere_title_spheremanagement", "Saisie");
            BdfHtmlUtils.printTiret(this);
            printSphereLink(SPHERE_PAGE, "global_link_maj", null);
            BdfHtmlUtils.printTiret(this);
            BdfHtmlUtils.printOds(this, "redacteurs/" + sphere.getSubsetKey().getSubsetName());
            endP();
        }
        startTable();
        printMap(actifMap);
        endTable();

        if (!inactifMap.isEmpty()) {
            appendH2("sphere_title_inactif");
            HTMLAttributes htmlAttributes = new HTMLAttributes();
            htmlAttributes.setClasses(GlobalCatalog.TABLE_SPH_INACTIF);
            startTable(htmlAttributes);
            printMap(inactifMap);
            endTable();
        }
        end();
    }

    private void printSphereLink(String page, String locKey, String target) {
        HTMLAttributes aAttributes = new HTMLAttributes();
        String href = "Sphere?" + RequestUtils.toParameterString(sphere) + "&amp;page=" + page;
        aAttributes.setHref(href);
        if (target != null) {
            aAttributes.setTarget(target);
        }
        startA(aAttributes);
        localize(locKey);
        endA();
    }

    private void printMap(Map<String, Redacteur> map) {
        for (Map.Entry<String, Redacteur> entry : map.entrySet()) {
            String strg_idsphere = entry.getKey();
            Redacteur redacteur = entry.getValue();
            startRow();
            startCell();
            if (isAdmin()) {
                String href = "Sphere?" + RequestUtils.toParameterString(redacteur) + "&amp;page=" + REDACTEURADMINFORM_PAGE;
                startA(href, "Saisie");
                append(strg_idsphere);
                endA();
            } else {
                append(strg_idsphere);
            }
            endCell();
            startCell();
            escape(redacteur.getPersonneBean().getNomComplet(PersonneBean.NORMAL_STYLE));
            endCell();
            endRow();
        }
    }

    private void initMaps() {
        for (int i = 0; i < sphere.getTicketLength(); i++) {
            Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
            if (redacteur != null) {
                Map<String, Redacteur> map = (redacteur.isActif()) ? actifMap : inactifMap;
                map.put(redacteur.getRedacteurKey().getIdsphereString(), redacteur);
            }
        }
    }

}
