/* BdfServer_Commands
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.response;

import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class EnvoiResponseParameters extends BdfResponseParameters {

    public final static short ENVOIFORM_PAGE = 1;
    public final static short CONFIRMATION_PAGE = 2;
    public final static short ECHEC_PAGE = 3;
    public final static short ENVOI_FICHEUNIQUE = 1;
    public final static short ENVOI_SELECTION = 2;
    public final static short ENVOI_COMPILATION = 3;
    public final static String NOADDRESS_ERRORKEY = "envoi_err_noto";
    public final static String NOSUBJECT_ERRORKEY = "envoi_err_nosubject";
    public final static String WRONGADRESSES_ERRORKEY = "envoi_msg_errto";
    public final static String WRONGADRESSES_PARAMNAME = "wrongadresses";
    public final static String MAILEXCEPTION_PARAMNAME = "mailexception";
    public final static String UNIQUEABREGEFICHE_PARAMNAME = "uniqueabregefiche";
    public final static String ABREGEFICHESET_PARAMNAME = "abregeficheset";
    short envoiType;
    boolean bcc = true;
    String subject = null;
    String message = null;
    List toList;
    String baseDeFichesUrl;

    public EnvoiResponseParameters(BdfServer bdfServer, BdfUser bdfUser) {
        super(bdfServer, bdfUser);
        this.baseDeFichesUrl = bdfUser.getBaseDeFichesURL();
    }

    public short getEnvoiType() {
        return envoiType;
    }

    public void setEnvoiType(short envoiType) {
        this.envoiType = envoiType;
    }

    public static String typeToString(short envoiType) {
        switch (envoiType) {
            case EnvoiResponseParameters.ENVOI_FICHEUNIQUE:
                return "ficheunique";
            case EnvoiResponseParameters.ENVOI_COMPILATION:
                return "compilation";
            case EnvoiResponseParameters.ENVOI_SELECTION:
                return "selection";
            default:
                return null;
        }
    }

    /**
     *@throws IllegalArgumentException si le type n'a pas la bonne valeur
     */
    public static short typeToShort(String envoiType) {
        if (envoiType.equals("ficheunique")) {
            return ENVOI_FICHEUNIQUE;
        }
        if (envoiType.equals("compilation")) {
            return ENVOI_COMPILATION;
        }
        if (envoiType.equals("selection")) {
            return ENVOI_SELECTION;
        }
        throw new IllegalArgumentException();
    }

    /**
     * Indique qu'une copie doit être transmise à l'envoyeur.
     */
    public boolean isBcc() {
        return bcc;
    }

    public void setBcc(boolean bcc) {
        this.bcc = bcc;
    }

    /**
     * Retourne le sujet du message.
     */
    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        this.subject = subject;
        if ((subject != null) && (subject.length() == 0)) {
            this.subject = null;
        }
    }

    public boolean hasSubject() {
        return (subject != null);
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
        if ((message != null) && (message.length() == 0)) {
            this.message = null;
        }
    }

    public boolean hasMessage() {
        return (message != null);
    }

    public List getToList() {
        return toList;
    }

    public void setToList(List toList) {
        if ((toList != null) && (toList.size() == 0)) {
            this.toList = null;
        } else {
            this.toList = toList;
        }
    }

    public boolean hasToList() {
        return (toList != null);
    }

    public String getBaseDeFichesUrl() {
        return baseDeFichesUrl;
    }

}
