/* BdfServer_Commands
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.response;

import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class XMLReportResponsePrinter implements ResponseHandler {

    public final static String NONE_RESULT = "none";
    public final static String DONE_RESULT = "done";
    public final static String EXCEPTION_RESULT = "exception";
    public final static String USERERROR_RESULT = "user_error";
    public final static String PARAMETERERROR_RESULT = "parameter_error";
    public final static String LOGINERROR_RESULT = "login_error";
    private String result;
    private String doneMessage;
    private Exception exception;
    private List errorResourceItemList;
    private String[] litteralErrorString;
    private ResourceLocalisation resourceLocalisation;

    private XMLReportResponsePrinter(String result) {
        this.result = result;
    }

    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    public static XMLReportResponsePrinter newReport(BdfResponseParameters responseParameters, ResourceLocalisation resourceLocalisation) {
        XMLReportResponsePrinter reportXmlData = null;
        reportXmlData.resourceLocalisation = resourceLocalisation;
        CommandMessage message = responseParameters.getCommandMessage();
        if (message == null) {
            reportXmlData = new XMLReportResponsePrinter(NONE_RESULT);
        } else {
            switch (message.getType()) {
                case CommandMessage.DONE_MESSAGE:
                    reportXmlData = new XMLReportResponsePrinter(DONE_RESULT);
                    reportXmlData.doneMessage = resourceLocalisation.toString(message.getKeyResourceItem());
                    break;
                case CommandMessage.ERROR_MESSAGE:
                    reportXmlData = new XMLReportResponsePrinter(USERERROR_RESULT);
                    reportXmlData.errorResourceItemList = Collections.singletonList(message.getKeyResourceItem());
                    break;
                case CommandMessage.MULTIERROR_MESSAGE:
                    reportXmlData = new XMLReportResponsePrinter(USERERROR_RESULT);
                    reportXmlData.errorResourceItemList = message.getMultiErrorList();
                    break;
                default:
                    throw new SwitchException("message.getType() = " + message.getType());

            }
        }
        return reportXmlData;
    }

    public static XMLReportResponsePrinter newExceptionReport(Exception exception) {
        XMLReportResponsePrinter reportXmlData = new XMLReportResponsePrinter(EXCEPTION_RESULT);
        reportXmlData.exception = exception;
        reportXmlData.doneMessage = exception.getMessage();
        return reportXmlData;
    }

    public static XMLReportResponsePrinter newParameterExceptionReport(ParameterException parameterException) {
        XMLReportResponsePrinter reportXmlData = new XMLReportResponsePrinter(PARAMETERERROR_RESULT);
        reportXmlData.initLitteralError(String.valueOf(parameterException.getType()), parameterException.getMessage());
        return reportXmlData;
    }

    public static XMLReportResponsePrinter newLoginErrorReport(String loginErrorKey, String message) {
        XMLReportResponsePrinter reportXmlData = new XMLReportResponsePrinter(PARAMETERERROR_RESULT);
        reportXmlData.initLitteralError("login_err_" + loginErrorKey, message);
        return reportXmlData;
    }

    private void initLitteralError(String key, String message) {
        String[] array = {key, message};
        litteralErrorString = array;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        ReportXmlWriter xmlWriter = new ReportXmlWriter(pw, 0);
        xmlWriter.write();
        pw.close();
    }


    class ReportXmlWriter extends DefaultXMLWriter {

        PrintWriter pw;

        ReportXmlWriter(PrintWriter pw, int indentLength) {
            super();
            setAppendable(pw);
            setIndentLength(indentLength);
            this.pw = pw;
        }

        void write() throws IOException {
            appendXMLDeclaration();
            startOpenTag("report");
            addAttribute("result", result);
            endOpenTag();
            if (doneMessage != null) {
                addSimpleElement("done-message", doneMessage);
            }
            if (errorResourceItemList != null) {
                for (Iterator it = errorResourceItemList.iterator(); it.hasNext();) {
                    ResourceItem resourceItem = (ResourceItem) it.next();
                    startOpenTag("error-message");
                    addAttribute("error-key", resourceItem.getResourceKey());
                    endOpenTag(false);
                    escape(resourceLocalisation.toString(resourceItem));
                    closeTag("error-message", false, false);
                }
            }
            if (litteralErrorString != null) {
                startOpenTag("error-message");
                addAttribute("error-key", litteralErrorString[0]);
                endOpenTag(false);
                escape(litteralErrorString[1]);
                closeTag("error-message", false, false);
            }
            if (exception != null) {
                addSimpleElement("class", exception.getClass().getName());
                openTag("trace");
                exception.printStackTrace(pw);
                closeTag("trace");
            }
            closeTag("report");
        }

    }

}
