/* BdfServer_DirectoryStorage - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.managers.xml.ManagerXMLWriter;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.xml.CorpusUiXMLWriter;
import fr.exemole.bdfserver.ui.xml.TreesXMLWriter;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.xml.UserXMLWriter;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.association.DocumentSubset;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.xml.AlbumXMLWriter;
import net.mapeadores.basedefiches.xml.BasedefichesMetadataXMLWriter;
import net.mapeadores.basedefiches.xml.ChronoXMLWriter;
import net.mapeadores.basedefiches.xml.CorpusMetadataXMLWriter;
import net.mapeadores.basedefiches.xml.CroisementXMLWriter;
import net.mapeadores.basedefiches.xml.DocumentSubsetXMLWriter;
import net.mapeadores.basedefiches.xml.SphereXMLWriter;
import net.mapeadores.basedefiches.xml.ThesaurusXMLWriter;


/**
 *
 *
 * @author Vincent Calame
 */
public class BdfStorageUtils implements BdfDirectoryConstants {

    public static StorageFile getUserStorageFile(BdfDirectories bdfDirectories, RedacteurKey redacteurKey) {
        String path = USERS + File.separator + redacteurKey.toString(RedacteurKey.UNDERSCORE_STYLE) + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getChronoStorageFile(BdfDirectories bdfDirectories, SubsetKey subsetKey) {
        String path = getChronoPath() + File.separator + subsetKey.toString() + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getSubsetCoupleStorageFile(BdfDirectories bdfDirectories, SubsetCouple subsetCouple) {
        SubsetKey subsetKey1 = subsetCouple.getSubsetKey1();
        SubsetKey subsetKey2 = subsetCouple.getSubsetKey2();
        String name1 = subsetKey1.getSubsetName();
        String name2 = subsetKey2.getSubsetName();
        String category1 = SubsetKey.categoryToString(subsetKey1.getCategory());
        String category2 = SubsetKey.categoryToString(subsetKey2.getCategory());
        String path = BASEDEFICHES + File.separator + category1 + "_" + category2 + File.separator + name1 + "_" + name2 + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getBasedefichesMetadataFile(BdfDirectories bdfDirectories) {
        String path = BASEDEFICHES + File.separator + "metadata.xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getSubsetStorageFile(BdfDirectories bdfDirectories, Subset subset) {
        Subset masterSubset = subset.getMasterSubset();
        String prefix = "";
        if (masterSubset != null) {
            SubsetKey masterSubsetKey = masterSubset.getSubsetKey();
            StringBuffer prefixBuffer = new StringBuffer();
            switch (masterSubsetKey.getCategory()) {
                case SubsetKey.CATEGORY_CORPUS:
                    prefixBuffer.append("_mc_");
                    break;
                case SubsetKey.CATEGORY_THESAURUS:
                    prefixBuffer.append("_mt_");
                    break;
            }
            prefixBuffer.append(masterSubsetKey.getSubsetName());
            prefixBuffer.append("_");
            prefix = prefixBuffer.toString();
        }
        SubsetKey subsetKey = subset.getSubsetKey();
        String path = getRoughPath(subsetKey, prefix) + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile findStorageFile(BdfDirectories bdfDirectories, SubsetKey subsetKey) {
        String path = getRoughPath(subsetKey, "") + ".xml";
        StorageFile storageFile = new StorageFile(bdfDirectories, path);
        if (subsetKey.isCorpusSubset()) {
            if (!storageFile.exists()) {
                path = findRoughPath(bdfDirectories, subsetKey);
                if (path != null) {
                    storageFile = new StorageFile(bdfDirectories, path);
                }
            }
        }
        return storageFile;
    }

    public static StorageFile getCorpusDirectory(BdfDirectories bdfDirectories, SubsetKey corpusKey) {
        String path = getRoughPath(corpusKey, "");
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getConfStorageFile(BdfDirectories bdfDirectories, String filebasename) {
        String path = CONF + File.separator + filebasename + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getUiStorageFile(BdfDirectories bdfDirectories, String filebasename) {
        String path = CONF + File.separator + UI + File.separator + filebasename + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    public static StorageFile getAlterUiStorageFile(BdfDirectories bdfDirectories, SubsetKey subsetKey, String name) {
        String path = CONF + File.separator + UI + File.separator + subsetKey.toString() + File.separator + name + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

    private static String getRoughPath(SubsetKey subsetKey, String prefix) {
        String category = SubsetKey.categoryToString(subsetKey.getCategory());
        if (subsetKey.equals(SubsetKey.DOCUMENT_SUBSET)) {
            return BASEDEFICHES + File.separator + category + File.separator + category;
        }
        String name = subsetKey.getSubsetName();
        return BASEDEFICHES + File.separator + category + File.separator + prefix + name;
    }

    private static String findRoughPath(BdfDirectories bdfDirectories, SubsetKey subsetKey) {
        String category = SubsetKey.categoryToString(subsetKey.getCategory());
        File directory = new File(bdfDirectories.getBaseDirectory(), BASEDEFICHES + File.separator + category);
        if (!directory.exists()) {
            return null;
        }
        String end = "_" + subsetKey.getSubsetName() + ".xml";
        File[] list = directory.listFiles();
        for (int i = 0; i < list.length; i++) {
            File f = list[i];
            if (!f.isDirectory()) {
                String name = f.getName();
                if (name.endsWith(end)) {
                    return BASEDEFICHES + File.separator + category + File.separator + name;
                }
            }
        }
        return null;
    }

    public static SubsetInfoList getSubsetInfoList(BdfDirectories bdfDirectories, short subsetCategory) {
        String rootPath = BASEDEFICHES + File.separator + SubsetKey.categoryToString(subsetCategory);
        File subsetDirectory = new File(bdfDirectories.getBaseDirectory(), rootPath);
        InternalSubsetInfoList result = new InternalSubsetInfoList();
        if (!subsetDirectory.exists()) {
            subsetDirectory.mkdirs();
            return result;
        } else if (!subsetDirectory.isDirectory()) {
            throw new BdfStorageException(subsetDirectory.getPath() + " is not a directory");
        }
        File[] fileList = subsetDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String name = f.getName();
            if (name.endsWith(".xml")) {
                String basename = name.substring(0, name.length() - 4);
                try {
                    InternalSubsetInfo subsetInfo = new InternalSubsetInfo();
                    subsetInfo.parse(subsetCategory, basename);
                    StorageFile storageFile = new StorageFile(bdfDirectories, rootPath + File.separator + name);
                    subsetInfo.setStorageFile(storageFile);
                    result.add(subsetInfo);
                } catch (IllegalArgumentException iae) {
                    iae.printStackTrace();
                    continue;
                } catch (ParseException pe) {
                    pe.printStackTrace();
                    continue;
                }
            }
        }
        return result;
    }

    public static String getChronoPath() {
        return BASEDEFICHES + File.separator + "chrono";
    }

    public static Map<SubsetKey, StorageFile> getChronoStorageFileMap(BdfDirectories bdfDirectories) {
        String chronoPath = getChronoPath();
        File chronoDirectory = new File(bdfDirectories.getBaseDirectory(), chronoPath);
        if (!chronoDirectory.exists()) {
            chronoDirectory.mkdirs();
            return Collections.emptyMap();
        } else if (!chronoDirectory.isDirectory()) {
            throw new BdfStorageException(chronoDirectory.getPath() + " is not a directory");
        }
        Map<SubsetKey, StorageFile> result = new HashMap<SubsetKey, StorageFile>();
        File[] fileList = chronoDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (name.endsWith(".xml")) {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(name.substring(0, name.length() - 4));
                    StorageFile storageFile = new StorageFile(bdfDirectories, chronoPath + File.separator + name);
                    result.put(subsetKey, storageFile);
                } catch (java.text.ParseException pe) {
                }
            }
        }
        return result;
    }

    public static Map<SubsetCouple, StorageFile> getStorageFileList(BdfDirectories bdfDirectories, short subsetCategory1, short subsetCategory2) {
        if (subsetCategory1 > subsetCategory2) {
            short temp = subsetCategory1;
            subsetCategory1 = subsetCategory2;
            subsetCategory2 = temp;
        }
        String rootPath = BASEDEFICHES + File.separator + SubsetKey.categoryToString(subsetCategory1) + "_" + SubsetKey.categoryToString(subsetCategory2);
        File coupleDirectory = new File(bdfDirectories.getBaseDirectory(), rootPath);
        if (!coupleDirectory.exists()) {
            coupleDirectory.mkdirs();
            return Collections.emptyMap();
        } else if (!coupleDirectory.isDirectory()) {
            throw new BdfStorageException(coupleDirectory.getPath() + " is not a directory");
        }
        Map<SubsetCouple, StorageFile> result = new HashMap<SubsetCouple, StorageFile>();
        File[] fileList = coupleDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String name = f.getName();
            if (name.endsWith(".xml")) {
                String basename = name.substring(0, name.length() - 4);
                int idx = basename.indexOf('_');
                if (idx == -1) {
                    continue;
                }
                try {
                    SubsetKey subsetKey1 = SubsetKey.parse(subsetCategory1, basename.substring(0, idx));
                    SubsetKey subsetKey2 = SubsetKey.parse(subsetCategory2, basename.substring(idx + 1));
                    StorageFile storageFile = new StorageFile(bdfDirectories, rootPath + File.separator + name);
                    result.put(new SubsetCouple(subsetKey1, subsetKey2), storageFile);
                } catch (IllegalArgumentException iae) {
                    iae.printStackTrace();
                    continue;
                } catch (ParseException pe) {
                    pe.printStackTrace();
                    continue;
                }

            }
        }
        return result;
    }

    private static BufferedWriter getXMLWriter(BdfDirectories bdfDirectories, Subset subset) throws IOException {
        SubsetKey subsetKey = subset.getSubsetKey();
        bdfDirectories.testCategoryDirectory(subsetKey.getCategory());
        StorageFile storageFile = getSubsetStorageFile(bdfDirectories, subset);
        return storageFile.getXMLWriter();
    }

    public static void saveThesaurus(BdfDirectories bdfDirectories, Thesaurus thesaurus) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, thesaurus);
            ThesaurusXMLWriter thesaurusXMLWriter = new ThesaurusXMLWriter();
            thesaurusXMLWriter.setAppendable(buf);
            thesaurusXMLWriter.setIndentLength(0);
            thesaurusXMLWriter.appendThesaurus(thesaurus);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveSphere(BdfDirectories bdfDirectories, Sphere sphere) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, sphere);
            SphereXMLWriter xmlWriter = new SphereXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendSphere(sphere);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveAlbum(BdfDirectories bdfDirectories, Album album) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, album);
            AlbumXMLWriter xmlWriter = new AlbumXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendAlbum(album);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveBasedefichesMetadata(BdfDirectories bdfDirectories, BasedefichesMetadata basedefichesMetadata) {
        StorageFile storageFile = BdfStorageUtils.getBasedefichesMetadataFile(bdfDirectories);
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            BasedefichesMetadataXMLWriter xmlWriter = new BasedefichesMetadataXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendBasedefichesMetadata(basedefichesMetadata);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveCorpusMetadata(BdfDirectories bdfDirectories, Corpus corpus) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, corpus);
            CorpusMetadataXMLWriter xmlWriter = new CorpusMetadataXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendCorpusMetadata(corpus.getCorpusMetadata());
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveDocumentSubset(BdfDirectories bdfDirectories, DocumentSubset documentSubset) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, documentSubset);
            DocumentSubsetXMLWriter documentSubsetXMLWriter = new DocumentSubsetXMLWriter();
            documentSubsetXMLWriter.setAppendable(buf);
            documentSubsetXMLWriter.setIndentLength(0);
            documentSubsetXMLWriter.appendDocumentSubset(documentSubset);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void deleteSubset(BdfDirectories bdfDirectories, SubsetKey subsetKey) {
        StorageFile storageFile = findStorageFile(bdfDirectories, subsetKey);
        storageFile.delete();
    }

    public static void deleteSubsetCouple(BdfDirectories bdfDirectories, SubsetCouple subsetCouple) {
        StorageFile storageFile = getSubsetCoupleStorageFile(bdfDirectories, subsetCouple);
        storageFile.delete();
    }

    public static void deleteChrono(BdfDirectories bdfDirectories, SubsetKey subsetKey) {
        StorageFile storageFile = getChronoStorageFile(bdfDirectories, subsetKey);
        storageFile.delete();
    }

    public static void saveCroisement(BdfDirectories bdfDirectories, Croisement croisement) {
        try {
            BufferedWriter buf = getXMLWriter(bdfDirectories, croisement.getSubsetCouple());
            CroisementXMLWriter xmlWriter = new CroisementXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendCroisement(croisement);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    private static BufferedWriter getXMLWriter(BdfDirectories bdfDirectories, SubsetCouple subsetCouple) throws IOException {
        bdfDirectories.testCroisementDirectory(subsetCouple);
        StorageFile storageFile = getSubsetCoupleStorageFile(bdfDirectories, subsetCouple);
        return storageFile.getXMLWriter();
    }

    public static void saveChrono(BdfDirectories bdfDirectories, Corpus corpus) {
        bdfDirectories.testChronoDirectory();
        StorageFile storageFile = getChronoStorageFile(bdfDirectories, corpus.getSubsetKey());
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            ChronoXMLWriter chronoXMLWriter = new ChronoXMLWriter();
            chronoXMLWriter.setAppendable(buf);
            chronoXMLWriter.setIndentLength(0);
            chronoXMLWriter.appendChrono(corpus);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveUser(BdfDirectories bdfDirectories, BdfUser bdfUser) {
        StorageFile storageFile = BdfStorageUtils.getUserStorageFile(bdfDirectories, bdfUser.getRedacteurKey());
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            UserXMLWriter xmlWriter = new UserXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendUser(bdfUser);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveManager(BdfDirectories bdfDirectories, Object manager, String managerName, ManagerXMLWriter managerXMLWriter) {
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, managerName);
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            managerXMLWriter.setAppendable(buf);
            managerXMLWriter.setIndentLength(0);
            managerXMLWriter.appendManager(manager);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void saveUIManager(BdfDirectories bdfDirectories, UIManager uiManager) {
        StorageFile treeStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, "trees");
        try {
            BufferedWriter buf = treeStorageFile.getXMLWriter();
            TreesXMLWriter treesXMLWriter = new TreesXMLWriter();
            treesXMLWriter.setAppendable(buf);
            treesXMLWriter.setIndentLength(0);
            treesXMLWriter.appendTrees(uiManager);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
        Basedefiches basedefiches = uiManager.getBdfServer().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            saveCorpusUI(bdfDirectories, uiManager.getCorpusUI(corpus));
        }
    }

    private static void saveCorpusUI(BdfDirectories bdfDirectories, CorpusUI corpusUI) {
        StorageFile corpusUiStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, corpusUI.getCorpusSubsetKey().toString());
        try {
            BufferedWriter buf = corpusUiStorageFile.getXMLWriter();
            CorpusUiXMLWriter corpusUiXMLWriter = new CorpusUiXMLWriter();
            corpusUiXMLWriter.setAppendable(buf);
            corpusUiXMLWriter.setIndentLength(0);
            corpusUiXMLWriter.appendCorpusUI(corpusUI);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static void getAlterUI(BdfDirectories bdfDirectories, SubsetKey subsetKey, String uiName) {
    }


    private static class InternalSubsetInfoList implements SubsetInfoList {

        private List<InternalSubsetInfo> subsetList = new ArrayList<InternalSubsetInfo>();
        private List<InternalSubsetInfo> slaveSubsetList = new ArrayList<InternalSubsetInfo>();

        public int getNormalSubsetCount() {
            return subsetList.size();
        }

        public SubsetInfo getNormalSubsetInfo(int i) {
            return subsetList.get(i);
        }

        public int getSlaveSubsetCount() {
            return slaveSubsetList.size();
        }

        public SubsetInfo getSlaveSubsetInfo(int i) {
            return slaveSubsetList.get(i);
        }

        void add(InternalSubsetInfo subsetInfo) {
            if (subsetInfo.isSlaveSubset()) {
                slaveSubsetList.add(subsetInfo);
            } else {
                subsetList.add(subsetInfo);
            }
        }

        void sort() {
            Collections.sort(subsetList);
            Collections.sort(slaveSubsetList);
        }

    }


    private static class InternalSubsetInfo implements SubsetInfo, Comparable<InternalSubsetInfo> {

        private SubsetKey subsetKey;
        private StorageFile storageFile;
        private SubsetKey masterSubsetKey;

        public SubsetKey getSubsetKey() {
            return subsetKey;
        }

        public int compareTo(InternalSubsetInfo other) {
            int result = this.subsetKey.compareTo(other.subsetKey);
            if (result == 0) {
                result = this.masterSubsetKey.compareTo(other.masterSubsetKey);
            }
            return result;
        }

        public StorageFile getStorageFile() {
            return storageFile;
        }

        void setStorageFile(StorageFile storageFile) {
            this.storageFile = storageFile;
        }

        public SubsetKey getMasterSubsetKey() {
            return masterSubsetKey;
        }

        public boolean isSlaveSubset() {
            return (masterSubsetKey != null);
        }

        void parse(short subsetCategory, String baseName) throws ParseException {
            if (baseName.startsWith("_mc_")) {
                parserSlaveSubsetId(subsetCategory, baseName.substring(4), SubsetKey.CATEGORY_CORPUS);
            } else if (baseName.startsWith("_mt_")) {
                parserSlaveSubsetId(subsetCategory, baseName.substring(4), SubsetKey.CATEGORY_THESAURUS);
            } else {
                subsetKey = SubsetKey.parse(subsetCategory, baseName);
                masterSubsetKey = null;
            }
        }

        private void parserSlaveSubsetId(short subsetCategory, String baseName, short masterSubsetCategory) throws ParseException {
            int idx = baseName.indexOf("_");
            if (idx == -1) {
                throw new ParseException("second _ is missing", 4);
            }
            masterSubsetKey = SubsetKey.parse(masterSubsetCategory, baseName.substring(0, idx));
            subsetKey = SubsetKey.parse(subsetCategory, baseName.substring(idx + 1));
        }

    }

}
