/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumImage;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.impl.AlbumDataSource;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.images.ImageRescale;
import net.mapeadores.util.images.RescaleDim;
import net.mapeadores.util.io.IOUtils;
import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.IIOException;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryAlbumDataSource implements AlbumDataSource, AlbumConstants {

    private BdfDirectories bdfDirectories;

    public DirectoryAlbumDataSource(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public InputStream getInputStream(AlbumImage albumImage, AlbumDim albumDim) {
        File albumDirectory = bdfDirectories.getAlbumDirectory(albumImage.getAlbum().getSubsetKey());
        if ((!albumDirectory.exists()) || (!albumDirectory.isDirectory())) {
            return null;
        }
        String fileName = AlbumUtils.toFileName(albumImage);
        File f = new File(albumDirectory, fileName);
        if (!f.exists()) {
            return null;
        }
        if (albumDim == null) {
            try {
                return new FileInputStream(f);
            } catch (IOException ioe) {
                throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
            }
        } else {
            File dimDir = new File(albumDirectory, albumDim.getName());
            if (!dimDir.exists()) {
                dimDir.mkdir();
            } else if (!dimDir.isDirectory()) {
                dimDir.delete();
                dimDir.mkdir();
            }
            File dimFile = new File(dimDir, fileName);
            if (!dimFile.exists()) {
                try {
                    ImageRescale.rescale(f, dimFile, getRescaleDim(albumDim), AlbumUtils.getFormatString(albumImage.getType()));
                } catch (IOException ioe) {
                    if (ioe instanceof IIOException) {
                        try {
                            InputStream inputStream = this.getClass().getResourceAsStream("ecran-noir.png");
                            if (inputStream == null) {
                                throw new InternalResourceException("ecran-noir.png not found");
                            }
                            ImageRescale.rescale(inputStream, dimFile, getRescaleDim(albumDim), AlbumUtils.getFormatString(albumImage.getType()));
                            inputStream.close();
                        } catch (IOException ioe2) {
                            throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe2);
                        }
                    } else {
                        throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe);
                    }
                }
            }
            try {
                return new FileInputStream(dimFile);
            } catch (IOException ioe) {
                throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe);
            }
        }
    }

    public void update(AlbumImage albumImage, InputStream inputStream) {
        File albumDirectory = bdfDirectories.getAlbumDirectory(albumImage.getAlbum().getSubsetKey());
        if (!albumDirectory.isDirectory()) {
            albumDirectory.delete();
        }
        albumDirectory.mkdir();
        String fileName = AlbumUtils.toFileName(albumImage);
        File f = new File(albumDirectory, AlbumUtils.toFileName(albumImage));
        try {
            FileOutputStream outputStream = new FileOutputStream(f);
            IOUtils.copy(inputStream, outputStream);
            outputStream.close();
        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
        }
        cleanAlbumDimDirectories(albumDirectory, fileName);
    }

    private RescaleDim getRescaleDim(AlbumDim albumDim) {
        switch (albumDim.getType()) {
            case FIXED_WIDTH:
                return RescaleDim.newFixedWidthInstance(albumDim.getWidth());
            case FIXED_HEIGHT:
                return RescaleDim.newFixedHeightInstance(albumDim.getHeight());
            case MAX_WIDTH:
                return RescaleDim.newMaxWidthInstance(albumDim.getWidth());
            case MAX_HEIGHT:
                return RescaleDim.newMaxHeightInstance(albumDim.getHeight());
            case MAX_DIM:
                return RescaleDim.newMaxDimInstance(albumDim.getWidth(), albumDim.getHeight());
            default:
                throw new SwitchException("unknownType = " + albumDim.getType());
        }
    }

    private void cleanAlbumDimDirectories(File albumDirectory, String fileName) {
        File[] files = albumDirectory.listFiles();
        int length = files.length;
        for (int i = 0; i < length; i++) {
            File d = files[i];
            if (d.isDirectory()) {
                File f = new File(d, fileName);
                if (f.exists()) {
                    f.delete();
                }
            }
        }
    }

}
