/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogBasedefichesXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogChronoXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogCorpusMetadataXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogDocumentSubsetXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogSphereXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogThesaurusXmlErrorHandler;
import fr.exemole.bdfserver.storage.directory.jdbm.BasedefichesJdbm;
import fr.exemole.bdfserver.storage.directory.jdbm.EnteteFiche;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import jdbm.RecordManager;
import jdbm.RecordManagerFactory;
import jdbm.RecordManagerOptions;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumImage;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.association.DocumentSubset;
import net.mapeadores.basedefiches.association.DocumentSubsetEditor;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.NoMasterIdsubsetException;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.NotEditableCroisementException;
import net.mapeadores.basedefiches.impl.BdfimplBasedefiches;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.xml.AlbumDOMReader;
import net.mapeadores.basedefiches.xml.BasedefichesMetadataDOMReader;
import net.mapeadores.basedefiches.xml.ChronoDOMReader;
import net.mapeadores.basedefiches.xml.CorpusMetadataDOMReader;
import net.mapeadores.basedefiches.xml.CroisementDOMReader;
import net.mapeadores.basedefiches.xml.DocumentSubsetDOMReader;
import net.mapeadores.basedefiches.xml.SphereDOMReader;
import net.mapeadores.basedefiches.xml.ThesaurusDOMReader;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBasedefiches extends BdfimplBasedefiches {

    private BdfDirectories bdfDirectories;
    private BasedefichesJdbm basedefichesJdbm;

    public DirectoryBasedefiches(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;

    }

    /**
     * iexceptionHandler peuvent être nul.
     */
    public void init(MutableProcessLog initResultLog, ExceptionHandler exceptionHandler) {
        BasedefichesInitializer bi = new BasedefichesInitializer(initResultLog);
        bi.init();
        setDocumentDataSource(new DirectoryDocumentDataSource(bdfDirectories));
        setAlbumDataSource(new DirectoryAlbumDataSource(bdfDirectories));
        setSubsetDataListener(new DirectorySubsetDataListener(bdfDirectories, this, exceptionHandler));
        setFicheDataSource(new JdbmFicheDataSource(bdfDirectories, basedefichesJdbm));
    }


    private class BasedefichesInitializer {

        private BasedefichesEditor basedefichesEditor;
        private MutableProcessLog initResultLog;
        private LogBasedefichesXmlErrorHandler logBasedefichesXmlErrorHandler;
        private boolean initFromJdbm = false;
        private String basedefichesJdbmPath;
        private Properties recManagerProps = new Properties();
        private RecordManager initRecordManager;

        BasedefichesInitializer(MutableProcessLog initResultLog) {
            this.initResultLog = initResultLog;
            basedefichesEditor = getInternalBasedefichesEditor();
            this.logBasedefichesXmlErrorHandler = new LogBasedefichesXmlErrorHandler(initResultLog);
        }

        private void initBasedefichesJdbm() {
            File cacheDirectory = bdfDirectories.getCacheDirectory();
            File jdbmFile = new File(cacheDirectory, "basedefiches.db");
            if (jdbmFile.exists()) {
                initFromJdbm = true;
            }
            basedefichesJdbmPath = (new File(cacheDirectory, "basedefiches")).getPath();
            recManagerProps.setProperty(RecordManagerOptions.PROVIDER_FACTORY, "fr.exemole.bdfserver.storage.directory.jdbm.Provider");
            recManagerProps.setProperty(RecordManagerOptions.DISABLE_TRANSACTIONS, "true");
            initRecordManager = null;
            try {
                initRecordManager = RecordManagerFactory.createRecordManager(basedefichesJdbmPath, recManagerProps);
            } catch (IOException ioe) {
                if (initFromJdbm) {
                    jdbmFile.delete();
                    initFromJdbm = false;
                    try {
                        initRecordManager = RecordManagerFactory.createRecordManager(basedefichesJdbmPath, recManagerProps);
                    } catch (IOException ioe2) {
                        throw new BdfStorageException(ioe2);
                    }
                }
                throw new BdfStorageException(ioe);
            }
            if (initFromJdbm) {
                basedefichesJdbm = BasedefichesJdbm.checkJdbmVersion(initRecordManager);
                if (basedefichesJdbm == null) {
                    jdbmFile.delete();
                    initFromJdbm = false;
                    try {
                        initRecordManager = RecordManagerFactory.createRecordManager(basedefichesJdbmPath, recManagerProps);
                        basedefichesJdbm = BasedefichesJdbm.initEmptyInstance(initRecordManager);
                    } catch (IOException ioe2) {
                        throw new BdfStorageException(ioe2);
                    }
                }
            } else {
                basedefichesJdbm = BasedefichesJdbm.initEmptyInstance(initRecordManager);
            }
        }

        private void setLogSource(StorageFile storageFile) {
            initResultLog.setCurrentURI(storageFile.toProcessURI());
        }

        void init() {
            long l1 = System.currentTimeMillis();
            initBasedefichesJdbm();
            initBasedefichesMetadata();
            initSpheres();
            initThesaurus();
            initAlbums();
            initDocumentSubset();
            initCorpus();
            initFiches();
            initCouple(SubsetKey.CATEGORY_CORPUS, SubsetKey.CATEGORY_THESAURUS);
            initCouple(SubsetKey.CATEGORY_CORPUS, SubsetKey.CATEGORY_CORPUS);
            initCouple(SubsetKey.CATEGORY_CORPUS, SubsetKey.CATEGORY_DOCUMENT);
            initChrono();
            basedefichesJdbm.endInit();
            try {
                initRecordManager.close();
                recManagerProps.setProperty(RecordManagerOptions.DISABLE_TRANSACTIONS, "false");
                RecordManager recordManager = RecordManagerFactory.createRecordManager(basedefichesJdbmPath, recManagerProps);
                basedefichesJdbm.setRecordManager(recordManager);
            } catch (IOException ioe2) {
                throw new BdfStorageException(ioe2);
            }
            long l2 = System.currentTimeMillis();
            long s = (l2 - l1) / 1000;
            //System.out.println(s);
        }

        void initBasedefichesMetadata() {
            StorageFile storageFile = BdfStorageUtils.getBasedefichesMetadataFile(bdfDirectories);
            Document doc = storageFile.readDocument();
            if (doc != null) {
                setLogSource(storageFile);
                BasedefichesMetadataDOMReader domReader = new BasedefichesMetadataDOMReader(logBasedefichesXmlErrorHandler);
                domReader.setBasedefichesMetadataEditor(basedefichesEditor.getBasedefichesMetadataEditor());
                domReader.fillBasedefichesMetadata(doc.getDocumentElement());
            }
        }

        void initSpheres() {
            SphereDOMReader sphereDOMReader = new SphereDOMReader(new LogSphereXmlErrorHandler(initResultLog));
            SubsetInfoList infoList = BdfStorageUtils.getSubsetInfoList(bdfDirectories, SubsetKey.CATEGORY_SPHERE);
            int normalCount = infoList.getNormalSubsetCount();
            for (int i = 0; i < normalCount; i++) {
                SubsetInfo subsetInfo = infoList.getNormalSubsetInfo(i);
                SubsetKey sphereKey = subsetInfo.getSubsetKey();
                StorageFile storageFile = subsetInfo.getStorageFile();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                try {
                    SphereEditor sphereEditor = basedefichesEditor.createSphere(sphereKey);
                    sphereDOMReader.setSphereEditor(sphereEditor);
                    sphereDOMReader.fillSphere(document.getDocumentElement());
                } catch (ExistingSubsetException ese) {
                    throw new ImplementationException(ese);
                }

            }
        }

        void initThesaurus() {
            ThesaurusDOMReader thesaurusDOMReader = new ThesaurusDOMReader(new LogThesaurusXmlErrorHandler(initResultLog));
            SubsetInfoList infoList = BdfStorageUtils.getSubsetInfoList(bdfDirectories, SubsetKey.CATEGORY_THESAURUS);
            int normalCount = infoList.getNormalSubsetCount();
            for (int i = 0; i < normalCount; i++) {
                SubsetInfo subsetInfo = infoList.getNormalSubsetInfo(i);
                SubsetKey thesaurusKey = subsetInfo.getSubsetKey();
                StorageFile storageFile = subsetInfo.getStorageFile();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                try {
                    Element root = document.getDocumentElement();
                    short thesaurusType = ThesaurusDOMReader.getThesaurusType(root);
                    ThesaurusEditor thesaurusEditor = basedefichesEditor.createThesaurus(thesaurusKey, thesaurusType);
                    thesaurusDOMReader.setThesaurusEditor(thesaurusEditor);
                    thesaurusDOMReader.fillThesaurus(root);
                } catch (ExistingSubsetException ese) {
                    throw new ImplementationException(ese);
                }

            }
        }

        void initAlbums() {
            AlbumDOMReader albumDOMReader = new AlbumDOMReader(logBasedefichesXmlErrorHandler);
            SubsetInfoList infoList = BdfStorageUtils.getSubsetInfoList(bdfDirectories, SubsetKey.CATEGORY_ALBUM);
            int normalCount = infoList.getNormalSubsetCount();
            for (int i = 0; i < normalCount; i++) {
                SubsetInfo subsetInfo = infoList.getNormalSubsetInfo(i);
                SubsetKey albumKey = subsetInfo.getSubsetKey();
                StorageFile storageFile = subsetInfo.getStorageFile();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                try {
                    Element root = document.getDocumentElement();
                    AlbumEditor albumEditor = basedefichesEditor.createAlbum(albumKey);
                    albumDOMReader.setAlbumEditor(albumEditor);
                    albumDOMReader.fillAlbum(root);
                    File albumDirectory = bdfDirectories.getAlbumDirectory(albumKey);
                    if ((albumDirectory.exists()) && (!albumDirectory.isDirectory())) {
                        albumDirectory.delete();
                    }
                    if (!albumDirectory.exists()) {
                        albumDirectory.mkdir();
                        return;
                    }
                    File[] list = albumDirectory.listFiles();
                    for (int j = 0; j < list.length; j++) {
                        File f = list[j];
                        if (f.isDirectory()) {
                            continue;
                        }
                        String name = f.getName();
                        int idx = name.lastIndexOf('.');
                        if (idx == -1) {
                            continue;
                        }
                        short type = 0;
                        String imageName = "";
                        try {
                            type = AlbumUtils.imageTypeToShort(name.substring(idx + 1));
                        } catch (IllegalArgumentException iae) {
                            continue;
                        }
                        imageName = name.substring(0, idx);
                        if (!AlbumUtils.isValidImageName(imageName)) {
                            continue;
                        }
                        try {
                            AlbumImage albumImage = albumEditor.createAlbumImage(type, imageName);
                            String fileName = AlbumUtils.toFileName(albumImage);
                            if (!fileName.equals(name)) {
                                File current = new File(albumDirectory, fileName);
                                if (current.exists()) {
                                    current.delete();
                                }
                                f.renameTo(current);
                            }
                        } catch (ExistingNameException ene) {
                            continue;
                        }
                    }
                } catch (ExistingSubsetException ese) {
                    throw new ImplementationException(ese);
                }

            }
        }

        void initDocumentSubset() {
            DocumentSubsetEditor documentSubsetEditor = basedefichesEditor.getDocumentSubsetEditor();
            DocumentSubset documentSubset = documentSubsetEditor.getDocumentSubset();
            List<DocumentFile> list = DocumentFile.getDocumentFileList(bdfDirectories);
            for (int i = 0; i < list.size(); i++) {
                DocumentFile documentFile = list.get(i);
                DocumentFile.DocumentInfo documentInfo = documentFile.getDocumentInfo();
                if (documentInfo != null) {
                    int idsubset = documentInfo.getId();
                    String[] extensionArray = documentInfo.getExtensionArray();
                    long[] lengthArray = documentInfo.getLengthArray();
                    try {
                        documentSubsetEditor.createDocument(idsubset, extensionArray, lengthArray);
                    } catch (ExistingIdException eie) {
                        throw new ImplementationException(eie);
                    }
                }
            }
            StorageFile storageFile = BdfStorageUtils.getSubsetStorageFile(bdfDirectories, documentSubset);
            if (storageFile.exists()) {
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                Element root = document.getDocumentElement();
                DocumentSubsetDOMReader documentSubsetDOMReader = new DocumentSubsetDOMReader(documentSubsetEditor, new LogDocumentSubsetXmlErrorHandler(initResultLog));
                documentSubsetDOMReader.fillDocumentSubset(root);
            }
        }

        void initCorpus() {
            SubsetInfoList infoList = BdfStorageUtils.getSubsetInfoList(bdfDirectories, SubsetKey.CATEGORY_CORPUS);
            int normalCount = infoList.getNormalSubsetCount();
            for (int i = 0; i < normalCount; i++) {
                SubsetInfo subsetInfo = infoList.getNormalSubsetInfo(i);
                SubsetKey corpusKey = subsetInfo.getSubsetKey();
                StorageFile storageFile = subsetInfo.getStorageFile();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                try {
                    CorpusEditor corpusEditor = basedefichesEditor.createCorpus(corpusKey, null);
                    CorpusMetadataDOMReader domReader = new CorpusMetadataDOMReader(corpusEditor.getCorpusMetadataEditor(), new LogCorpusMetadataXmlErrorHandler(initResultLog));
                    domReader.fillCorpusMetadata(document.getDocumentElement());
                } catch (ExistingSubsetException ese) {
                    throw new ImplementationException(ese);
                }
            }
            int slaveCount = infoList.getSlaveSubsetCount();
            for (int i = 0; i < slaveCount; i++) {
                SubsetInfo subsetInfo = infoList.getSlaveSubsetInfo(i);
                SubsetKey corpusKey = subsetInfo.getSubsetKey();
                StorageFile storageFile = subsetInfo.getStorageFile();
                setLogSource(storageFile);
                Basedefiches basedefiches = basedefichesEditor.getBasedefiches();
                if (basedefiches.getSubset(corpusKey) != null) {
                    ProcessLogItem logItem = initResultLog.newLogItemInstance();
                    logItem.setCategory("ERR");
                    logItem.setKey("doublonSlaveSubsetKey");
                    logItem.setObject(corpusKey);
                    continue;
                }
                Subset masterSubset = basedefiches.getSubset(subsetInfo.getMasterSubsetKey());
                if (masterSubset == null) {
                    ProcessLogItem logItem = initResultLog.newLogItemInstance();
                    logItem.setCategory("ERR");
                    logItem.setKey("unknwonMasterSubsetKey");
                    logItem.setObject(subsetInfo.getMasterSubsetKey());
                    continue;
                }
                Document document = storageFile.readDocument();
                try {
                    CorpusEditor corpusEditor = basedefichesEditor.createCorpus(corpusKey, masterSubset);
                    CorpusMetadataDOMReader domReader = new CorpusMetadataDOMReader(corpusEditor.getCorpusMetadataEditor(), new LogCorpusMetadataXmlErrorHandler(initResultLog));
                    domReader.fillCorpusMetadata(document.getDocumentElement());
                } catch (ExistingSubsetException ese) {
                    throw new ImplementationException(ese);
                }
            }
        }

        void initFiches() {
            List<Corpus> slaveCorpus = new ArrayList<Corpus>();
            int corpusCount = getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = getCorpus(i);
                if (corpus.getMasterSubset() != null) {
                    slaveCorpus.add(corpus);
                } else {
                    if (initFromJdbm) {
                        initFichesFromJdbm(corpus);
                    } else {
                        initFichesFromXml(corpus);
                    }
                }
            }
            for (Corpus corpus : slaveCorpus) {
                if (initFromJdbm) {
                    initFichesFromJdbm(corpus);
                } else {
                    initFichesFromXml(corpus);
                }
            }

        }

        void initFichesFromJdbm(Corpus corpus) {
            CorpusEditor corpusEditor = basedefichesEditor.getCorpusEditor(corpus.getSubsetKey());
            EnteteFiche[] array = basedefichesJdbm.getEnteteFicheArray(corpus);
            int length = array.length;
            for (int i = 0; i < length; i++) {
                EnteteFiche enteteFiche = array[i];
                try {
                    AbregeFiche abregeFiche = corpusEditor.createFiche(enteteFiche.getIdcorpus());
                    updateFiche(abregeFiche, enteteFiche.getFiche());
                } catch (ExistingIdException eie) {
                } catch (NoMasterIdsubsetException noMasterIdcorpusException) {
                }
            }
        }

        void initFichesFromXml(Corpus corpus) {
            CorpusEditor corpusEditor = basedefichesEditor.getCorpusEditor(corpus.getSubsetKey());
            String corpusPath = "basedefiches" + File.separator + "corpus" + File.separator + corpus.getSubsetKey().getSubsetName();
            File corpusDirectory = new File(bdfDirectories.getBaseDirectory(), corpusPath);
            if (!corpusDirectory.exists()) {
                corpusDirectory.mkdir();
                return;
            }
            File[] list = corpusDirectory.listFiles();
            for (int i = 0; i < list.length; i++) {
                File f = list[i];
                if ((f.isDirectory()) && (f.getName().startsWith("f."))) {
                    String fPath = corpusPath + File.separator + f.getName();
                    File[] list2 = f.listFiles();
                    for (int j = 0; j < list2.length; j++) {
                        File ficheFile = list2[j];
                        String name = ficheFile.getName();
                        if ((!ficheFile.isDirectory()) && (name.startsWith("fiche-")) && (name.endsWith(".xml"))) {
                            FicheStorageFile fsf = new FicheStorageFile(bdfDirectories, fPath + File.separator + name);
                            setLogSource(fsf);
                            try {
                                int idcorpus = Integer.parseInt(name.substring(6, 12));
                                AbregeFiche abregeFiche = corpusEditor.createFiche(idcorpus);
                                Fiche fiche = fsf.getFiche(abregeFiche.getCorpus(), true);
                                basedefichesJdbm.updateFiche(abregeFiche, fiche, false);
                                updateFiche(abregeFiche, fiche);
                            } catch (NumberFormatException nfe) {
                                ProcessLogItem logItem = initResultLog.newLogItemInstance();
                                logItem.setCategory("ERR");
                                logItem.setKey("numberFormatException");
                                continue;
                            } catch (ExistingIdException eie) {
                                ProcessLogItem logItem = initResultLog.newLogItemInstance();
                                logItem.setCategory("ERR");
                                logItem.setKey("existingIdcorpusException");
                                continue;
                            } catch (NoMasterIdsubsetException noMasterIdcorpusException) {
                                ProcessLogItem logItem = initResultLog.newLogItemInstance();
                                logItem.setCategory("ERR");
                                logItem.setKey("noMasterIdsubsetException");
                                continue;
                            }
                        }
                    }
                }
            }
        }

        void initCouple(short category1, short category2) {
            Map<SubsetCouple, StorageFile> map = BdfStorageUtils.getStorageFileList(bdfDirectories, category1, category2);
            CroisementEditor croisementEditor = basedefichesEditor.getCroisementEditor();
            CroisementDOMReader croisementDOMReader = new CroisementDOMReader(croisementEditor, logBasedefichesXmlErrorHandler);
            for (Map.Entry<SubsetCouple, StorageFile> mapEntry : map.entrySet()) {
                SubsetCouple couple = mapEntry.getKey();
                StorageFile storageFile = mapEntry.getValue();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                try {
                    croisementDOMReader.initCroisement(couple, document.getDocumentElement());
                } catch (NotEditableCroisementException nece) {
                    System.out.println(nece.getMessage());
                }
            }
        }

        void initChrono() {
            ChronoDOMReader chronoDOMReader = new ChronoDOMReader(new LogChronoXmlErrorHandler(initResultLog));
            Map<SubsetKey, StorageFile> map = BdfStorageUtils.getChronoStorageFileMap(bdfDirectories);
            for (Map.Entry<SubsetKey, StorageFile> mapEntry : map.entrySet()) {
                SubsetKey subsetKey = mapEntry.getKey();
                StorageFile storageFile = mapEntry.getValue();
                setLogSource(storageFile);
                Document document = storageFile.readDocument();
                if (subsetKey.isCorpusSubset()) {
                    CorpusEditor corpusEditor = basedefichesEditor.getCorpusEditor(subsetKey);
                    if (corpusEditor != null) {
                        chronoDOMReader.setCorpusEditor(corpusEditor);
                        chronoDOMReader.fillChrono(document.getDocumentElement());
                    }
                }
            }
        }

    }

    BasedefichesEditor getInternalBasedefichesEditor() {
        return super.getBasedefichesEditor();
    }

}
