/* BdfServer_DirectoryStorage - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.resource;

import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceList;
import fr.exemole.bdfserver.storage.BdfResourceStorage;
import net.mapeadores.util.io.IOUtils;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.MimeTypeResolver;
import net.mapeadores.util.io.docstream.FileDocStream;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBdfResourceStorage implements BdfResourceStorage {

    private Map<Short, Class> referenceClassMap = new HashMap<Short, Class>();
    private MimeTypeResolver mimeTypeResolver;
    private File confRscDirectory;
    private File varRscDirectory;

    public DirectoryBdfResourceStorage(MimeTypeResolver mimeTypeResolver, File confRscDirectory, File varRscDirectory) {
        this.mimeTypeResolver = mimeTypeResolver;
        this.confRscDirectory = confRscDirectory;
        this.varRscDirectory = varRscDirectory;
    }

    public MimeTypeResolver getMimeTypeResolver() {
        return mimeTypeResolver;
    }

    public boolean containsBdfResource(short resourceType, String path) {
        File file = getFile(resourceType, path, false);
        if (file == null) {
            return false;
        } else {
            return true;
        }
    }

    public DocStream getBdfResourceDocStream(short resourceType, String path) {
        File file = getFile(resourceType, path, false);
        if (file == null) {
            return null;
        } else {
            FileDocStream fileDocStream = new FileDocStream(file);
            String mimeType = mimeTypeResolver.getMimeTypeFromFileName(file.getName());
            fileDocStream.setMimeType(mimeType);
            if (mimeType.startsWith("text/")) {
                fileDocStream.setCharset("UTF-8");
            }
            return fileDocStream;
        }
    }

    public void saveBdfResource(short resourceType, String path, InputStream inputStream) throws IOException {
        File file = getFile(resourceType, path, true);
        if (file == null) {
            inputStream.close();
            return;
        }
        file.getParentFile().mkdirs();
        OutputStream output = new FileOutputStream(file);
        int result = IOUtils.copy(inputStream, output);
        inputStream.close();
        output.close();
    }

    public Class getDefaultBdfResourceReferenceClass(short resourceType) {
        return referenceClassMap.get(resourceType);
    }

    public void setDefaultBdfResourceReferenceClass(short resourceType, Class referenceClass) {
        referenceClassMap.put(resourceType, referenceClass);
    }

    public BdfResourceList getDatafResourceList() {
        DirectoryBdfResourceList directoryBdfResourceList = new DirectoryBdfResourceList();
        File rootDirectory = varRscDirectory;
        File[] fileArray = rootDirectory.listFiles();
        for(int i = 0; i < fileArray.length; i++) {
            File f = fileArray[i];
            String name = f.getName();
            if (f.isDirectory()) {
                if (name.equals("_jquery")) {
                    directoryBdfResourceList.appendDir(BdfResourceConstants.JQUERY_RESOURCE, f, "");
                }
                else if (name.equals("_js")) {
                    directoryBdfResourceList.appendDir(BdfResourceConstants.JS_RESOURCE, f, "");
                }
                else if (name.equals("_theme")) {
                    directoryBdfResourceList.appendDir(BdfResourceConstants.THEME_RESOURCE, f, "");
                }
                else if (name.equals("_css")) {
                    directoryBdfResourceList.appendDir(BdfResourceConstants.CSS_RESOURCE, f, "");
                }
                else {
                    directoryBdfResourceList.appendDir(BdfResourceConstants.OTHER_RESOURCE, f, name + "/");
                }
            } else {
                directoryBdfResourceList.add(BdfResourceConstants.OTHER_RESOURCE, name);
            }
        }
        return directoryBdfResourceList;
    }



    private File getFile(short resourceType, String resourcePath, boolean modification) {
        String prefix = getPrefix(resourceType);
        resourcePath = prefix + resourcePath;
        File resourceFile = new File(varRscDirectory, resourcePath);
        if (modification) {
            if ((resourceFile.exists()) && (resourceFile.isDirectory())) {
                return null;
            }
            return resourceFile;
        }
        if ((resourceFile.exists()) && (!resourceFile.isDirectory())) {
            return resourceFile;
        }
        resourceFile = new File(confRscDirectory, resourcePath);
        if ((resourceFile.exists()) && (!resourceFile.isDirectory())) {
            return resourceFile;
        }
        return null;
    }

    private static String getPrefix(short resourceType) {
        switch (resourceType) {
            case BdfResourceConstants.JQUERY_RESOURCE:
                return "_jquery/";
            case BdfResourceConstants.JS_RESOURCE:
                return "_js/";
            case BdfResourceConstants.THEME_RESOURCE:
                return "_theme/";
            case BdfResourceConstants.CSS_RESOURCE:
                return "_css/";
            case BdfResourceConstants.OTHER_RESOURCE:
            default:
                return "";
        }
    }

    private static class DirectoryBdfResourceList implements BdfResourceList {

        private List<Short> resourceTypeList = new ArrayList<Short>();
        private List<String> resourcePathList = new ArrayList<String>();

        private DirectoryBdfResourceList() {

        }

        private void add(short resourceType, String resourcePath) {
            resourceTypeList.add(resourceType);
            resourcePathList.add(resourcePath);
        }

        public int getResourceCount() {
            return resourceTypeList.size();
        }

        public short getResourceType(int i) {
            return resourceTypeList.get(i);
        }

        public String getResourcePath(int i) {
            return resourcePathList.get(i);
        }

        private void appendDir(short resourceType, File dir, String path) {
            File[] fileArray = dir.listFiles();
            for(int i = 0; i < fileArray.length; i++) {
                File f = fileArray[i];
                String name = f.getName();
                if (name.startsWith(".")) continue;
                if (f.isDirectory()) {
                    appendDir(resourceType, f, path + name + "/");
                } else {
                    add(resourceType, path + name);
                }
            }
        }

    }


}
