/* BdfServer_DirectoryStorage - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.transformationstorage;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.XslTransformerInfo;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternErrorHandler;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternUtils;
import java.io.File;
import java.io.IOException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class FileXslTransformerStorageInfo implements TransformationStorage.XslTransformerStorageInfo {

    private XslTransformerKey xslTransformerKey;
    private File txtFile;
    private File xslFile;
    private XslTransformerInfo xslTransformerInfo;

    public FileXslTransformerStorageInfo(XslTransformerKey xslTransformerKey) {
        this.xslTransformerKey = xslTransformerKey;
        xslTransformerInfo = new XslTransformerInfo(xslTransformerKey);
    }

    public XslTransformerInfo getXslTransformerInfo() {
        return xslTransformerInfo;
    }

    public Source getSource() {
        if (xslFile == null) {
            return null;
        } else {
            return new StreamSource(xslFile);
        }
    }

    public void addTxtFile(File f) {
        txtFile = f;
    }

    public void addXslFile(File f) {
        xslFile = f;
    }

    public void process(BdfServer bdfServer) throws IOException {
        StreamSource streamSource = null;
        if (txtFile != null) {
            xslTransformerInfo.setFromPattern(true);
            String destinationName = xslTransformerKey.getTransformerName() + ".xsl";
            PatternErrorHandler patternErrorHandler = PatternUtils.processPattern(txtFile, destinationName, bdfServer);
            xslTransformerInfo.setPatternWarning(patternErrorHandler.getPatternWarning());
            xslTransformerInfo.setPatternURI(txtFile.getPath());
            if (patternErrorHandler.hasError()) {
                xslTransformerInfo.setError(XslTransformerInfo.PATTERN_ERROR, patternErrorHandler.getPatternError());
                return;
            } else {
                xslFile = new File(txtFile.getParentFile(), destinationName);
            }
        } else {
            xslTransformerInfo.setFromPattern(false);
        }
        xslTransformerInfo.setXslURI(xslFile.getPath());
        Object test = XMLUtils.testXml(xslFile);
        if (!(test.equals(Boolean.TRUE))) {
            xslTransformerInfo.setError(XslTransformerInfo.XMLOFXSL_ERROR, test.toString());
            xslFile = null;
        }
    }

}
