/* BdfServer_DirectoryStorage - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.transformationstorage.stream;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerInfo;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.storage.TransformationStorage;
import java.io.File;
import java.io.IOException;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.sxio.SXIO;


/**
 *
 * @author Vincent Calame
 */
public abstract class DirectoryStreamTransformerStorageInfo implements TransformationStorage.StreamTransformerStorageInfo {

    private StreamTransformerKey streamTransformerKey;
    private StreamTransformer streamTransformer;
    private StreamTransformerInfo streamTransformerInfo;

    DirectoryStreamTransformerStorageInfo() {
    }

    protected void setStreamTransformerKey(StreamTransformerKey streamTransformerKey) {
        this.streamTransformerKey = streamTransformerKey;
        streamTransformerInfo = new StreamTransformerInfo(streamTransformerKey);
    }

    public StreamTransformerInfo getStreamTransformerInfo() {
        return streamTransformerInfo;
    }

    public StreamTransformer getStreamTransformer() {
        return streamTransformer;
    }

    protected void setStreamTransformer(StreamTransformer streamTransformer) {
        this.streamTransformer = streamTransformer;
    }

    protected void setError(short errorType, String errorMessage) {
        streamTransformer = null;
        streamTransformerInfo.setError(errorType, errorMessage);
    }

    protected void setTransformerError(String errorMessage) {
        setError(StreamTransformerInfo.STREAM_TRANSFORMER_ERROR, errorMessage);
    }

    public static DirectoryStreamTransformerStorageInfo newInstance(StreamTransformerKey streamTransformerKey, File directory, BdfServer bdfServer) throws IOException {
        String extension = streamTransformerKey.getExtension();
        DirectoryStreamTransformerStorageInfo result = null;
        if (OdUtils.isOdExtension(extension)) {
            result = new OdStreamTransformerStorageInfo();
        } else {
            String mimeType = SXIO.getMimeType(extension);
            if (mimeType.length() > 0) {
                result = new SxStreamTransformerStorageInfo();
            } else {
                result = new UnknownDirectoryStreamTransformerStorageInfo();
            }
        }
        result.setStreamTransformerKey(streamTransformerKey);
        result.process(extension, directory, bdfServer);
        return result;
    }

    protected abstract void process(String extension, File directory, BdfServer bdfServer) throws IOException;


    private static class UnknownDirectoryStreamTransformerStorageInfo extends DirectoryStreamTransformerStorageInfo {

        protected void process(String extension, File directory, BdfServer bdfServer) throws IOException {
            setError(StreamTransformerInfo.UNKNOWN_EXTENSION_ERROR, extension);
        }

    }

}
