/* BdfServer_DirectoryStorage - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.transformationstorage.stream;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.xsltfactory.odcss.BdfServerCssSource;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternErrorHandler;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import net.mapeadores.opendocument.css.elements.ElementMaps;
import net.mapeadores.opendocument.css.output.StylesOutput;
import net.mapeadores.opendocument.css.parse.CssParser;
import net.mapeadores.opendocument.css.parse.CssSource;
import net.mapeadores.opendocument.css.parse.FileCssSource;
import net.mapeadores.opendocument.transform.ODStreamTransformer;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class OdStreamTransformerStorageInfo extends DirectoryStreamTransformerStorageInfo {

    public OdStreamTransformerStorageInfo() {
    }

    protected void process(String extension, File directory, BdfServer bdfServer) throws IOException {
        MessageStreamTransformerErrorHandler eh = new MessageStreamTransformerErrorHandler();
        File cssStyles = new File(directory, "styles.css");
        if (cssStyles.exists()) {
            processCss(cssStyles, bdfServer);
        }
        File txtFile = new File(directory, "content.txt");
        if (txtFile.exists()) {
            processPattern(txtFile, bdfServer);
        }

        File contentXslt = new File(directory, "content.xsl");

        ODStreamTransformer.Builder odStreamTransformerBuilder = new ODStreamTransformer.Builder(extension);

        byte[] stylesByteArray = testFile(directory, "styles.xml", eh, false);
        if (stylesByteArray != null) {
            odStreamTransformerBuilder.setStyles(stylesByteArray);
        }
        byte[] contentAutomaticStylesArray = testFile(directory, "_content_automaticstyles.xml", eh, false);
        if (contentAutomaticStylesArray != null) {
            odStreamTransformerBuilder.setContentAutomaticStyles(contentAutomaticStylesArray);
        }
        byte[] contentXsltByteArray = testFile(directory, "content.xsl", eh, true);
        if (contentXsltByteArray != null) {
            odStreamTransformerBuilder.setURIResolver(bdfServer.getURIResolver());
            odStreamTransformerBuilder.setContentURI((new File(directory, "content.xsl")).toURI().toString());
            odStreamTransformerBuilder.setStreamTransformerErrorHandler(eh);
            odStreamTransformerBuilder.setContentXsl(contentXsltByteArray);
        }
        if (eh.hasError()) {
            this.setTransformerError(eh.getErrorMessage());
        } else {
            setStreamTransformer(odStreamTransformerBuilder.toODStreamTransformer());
        }
    }

    private byte[] testFile(File directory, String fileName, MessageStreamTransformerErrorHandler eh, boolean isMissingError) {
        File result = new File(directory, fileName);
        if (!result.exists()) {
            if (isMissingError) {
                eh.missingResourceError(result.toURI().toString());
            }
            return null;
        }
        if (result.isDirectory()) {
            eh.wrongResourceError(result.toURI().toString(), "directory URI");
            return null;
        }
        Object xmlTest = XMLUtils.testXml(result);
        if (!xmlTest.equals(Boolean.TRUE)) {
            eh.wrongResourceError(result.toURI().toString(), "XML check error :: " + xmlTest.toString());
            return null;
        }
        try {
            InputStream inputStream = new FileInputStream(result);
            byte[] array = IOUtils.toByteArray(inputStream);
            inputStream.close();
            return array;
        } catch (IOException ioe) {
            eh.wrongResourceError(result.toURI().toString(), ioe.getMessage());
            return null;
        }
    }

    private void processCss(File cssFile, BdfServer bdfServer) throws IOException {
        File destination = new File(cssFile.getParentFile(), "styles.xml");
        File automaticStyles = new File(cssFile.getParentFile(), "_content_automaticstyles.xml");
        CssSource cssSource = new InternalCssSource(cssFile, bdfServer);
        ElementMaps elementMaps = CssParser.parse(cssSource);
        StylesOutput.write(destination, elementMaps);
        StylesOutput.writeAutomaticStyles(automaticStyles, elementMaps);
    }

    private void processPattern(File txtFile, BdfServer bdfServer) throws IOException {
        String destinationName = "content.xsl";
        PatternErrorHandler patternErrorHandler = PatternUtils.processPattern(txtFile, destinationName, bdfServer);
    }


    private class InternalCssSource extends FileCssSource {

        private BdfServer bdfServer;

        InternalCssSource(File f, BdfServer bdfServer) {
            super(f);
            this.bdfServer = bdfServer;
        }

        public CssSource getImportCssSource(String url) {
            if (BdfServerCssSource.isBdfServerCssSourceUrl(url)) {
                return BdfServerCssSource.getBdfServerCssSource(url, bdfServer);
            }
            File newFile = getFile(url);
            if (!newFile.exists()) {
                return null;
            }
            return new InternalCssSource(newFile, bdfServer);
        }

    }

}
