/* BdfServer_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class EnvoiParameters {

    public final static short ENVOI_FICHEUNIQUE = 1;
    public final static short ENVOI_SELECTION = 2;
    public final static short ENVOI_COMPILATION = 3;
    private short envoiType;
    private boolean withRedacteurBcc;
    private String subject;
    private String message;
    private AbregeFiche[] abregeFicheArray;
    private List<ValidAddress> validAddressList;
    private List<WrongAddress> wrongAddressList;
    private CourrielBean fromCourrielBean;
    private FicheAttachmentParameters ficheAttachmentParameters;

    public EnvoiParameters(short envoiType) {
        this.envoiType = envoiType;
    }

    public short getEnvoiType() {
        return envoiType;
    }

    public static String typeToString(short envoiType) {
        switch (envoiType) {
            case ENVOI_FICHEUNIQUE:
                return "ficheunique";
            case ENVOI_COMPILATION:
                return "compilation";
            case ENVOI_SELECTION:
                return "selection";
            default:
                return null;
        }
    }

    /**
     *@throws IllegalArgumentException si le type n'a pas la bonne valeur
     */
    public static short typeToShort(String envoiType) {
        if (envoiType.equals("ficheunique")) {
            return ENVOI_FICHEUNIQUE;
        }
        if (envoiType.equals("compilation")) {
            return ENVOI_COMPILATION;
        }
        if (envoiType.equals("selection")) {
            return ENVOI_SELECTION;
        }
        throw new IllegalArgumentException();
    }

    public boolean isWithRedacteurBcc() {
        return withRedacteurBcc;
    }

    public void setWithRedacteurBcc(boolean withRedacteurBcc) {
        this.withRedacteurBcc = withRedacteurBcc;
    }

    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        if (subject != null) {
            subject = StringUtils.cleanString(subject);
            if (subject.length() == 0) {
                subject = null;
            }
        }
        this.subject = subject;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        if (message != null) {
            message = message.trim();
            if (message.length() == 0) {
                message = null;
            }
        }
        this.message = message;
    }

    public AbregeFiche[] getAbregeFicheArray() {
        return abregeFicheArray;
    }

    public void setAbregeFicheArray(AbregeFiche[] abregeFicheArray) {
        this.abregeFicheArray = abregeFicheArray;
    }

    public void setAbregeFicheArray(AbregeFiche abregeFiche) {
        AbregeFiche[] abregeFicheArray = new AbregeFiche[1];
        abregeFicheArray[0] = abregeFiche;
        this.abregeFicheArray = abregeFicheArray;
    }

    public boolean hasNoTo() {
        return ((wrongAddressList == null) && (validAddressList == null));
    }

    public List<WrongAddress> getWrongAddressList() {
        return wrongAddressList;
    }

    public List<ValidAddress> getValidAddressList() {
        return validAddressList;
    }

    public void setValidAddressList(List<ValidAddress> validAddressList) {
        this.validAddressList = validAddressList;
    }

    public void parseToArray(Basedefiches basedefiches, List<String> toList, SubsetKey defaultSphereKey) {
        wrongAddressList = new ArrayList<WrongAddress>();
        validAddressList = new ArrayList<ValidAddress>();
        for (String to : toList) {
            RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(to, defaultSphereKey);
            if (redacteurKey != null) {
                Redacteur redacteur = SphereUtils.getRedacteur(basedefiches, redacteurKey);
                WrongAddress wrongAdress = null;
                if (redacteur == null) {
                    wrongAdress = WrongAddress.redacteurUnknownError(redacteurKey);
                } else if (!redacteur.isActif()) {
                    wrongAdress = WrongAddress.redacteurInactiveError(redacteurKey);
                }
                if (wrongAdress == null) {
                    CourrielBean courrielBean = redacteur.getCourrielBean();
                    if (courrielBean == null) {
                        wrongAdress = WrongAddress.redacteurNoAdressError(redacteurKey);
                    } else {
                        validAddressList.add(new ValidAddress(courrielBean, redacteurKey));
                    }
                }
                if (wrongAdress != null) {
                    wrongAddressList.add(wrongAdress);
                }
            } else {
                try {
                    CourrielBean courrielBean = CourrielBean.parse(to);
                    validAddressList.add(new ValidAddress(courrielBean, to));
                } catch (ParseException pe) {
                    wrongAddressList.add(WrongAddress.malformedAddressError(to));
                }
            }
        }
        if (wrongAddressList.size() == 0) {
            wrongAddressList = null;
        }
        if (validAddressList.size() == 0) {
            validAddressList = null;
        }
    }

    public CourrielBean getFromCourrielBean() {
        return fromCourrielBean;
    }

    public void setFromCourrielBean(CourrielBean fromCourrielBean) {
        this.fromCourrielBean = fromCourrielBean;
    }

    public FicheAttachmentParameters getFicheAttachmentParameters() {
        return ficheAttachmentParameters;
    }

    public void setFicheAttachmentParameters(FicheAttachmentParameters ficheAttachmentParameters) {
        this.ficheAttachmentParameters = ficheAttachmentParameters;
    }

}
