/* BdfServer_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import net.mapeadores.util.smtp.SmtpParameters;
import net.mapeadores.util.smtp.SmtpUtils;


/**
 *
 * @author Vincent Calame
 */
public class SmtpManager {

    public final static short INIT_OK = 1;
    public final static short INIT_NOFILE = -1;
    public final static short INIT_IOEXCEPTION = -2;
    public final static short INIT_PROPERTIESERRORS = - 3;
    private File propertiesFile;
    private List<String> errorList;
    private SmtpParameters smtpParameters;
    private short initState;

    public SmtpManager(File propertiesFile) {
        this.propertiesFile = propertiesFile;
        init();
    }

    public boolean isInit() {
        return (initState == INIT_OK);
    }

    public short getInitState() {
        return initState;
    }

    public List<String> getErrorList() {
        return errorList;
    }

    public SmtpParameters getSmtpParameters() {
        return smtpParameters;
    }

    private void init() {
        if (!propertiesFile.exists()) {
            initState = INIT_NOFILE;
            smtpParameters = null;
            errorList = null;
            return;
        }
        Properties properties = new Properties();
        try {
            FileInputStream inputStream = new FileInputStream(propertiesFile);
            properties.load(inputStream);
            inputStream.close();
        } catch (IOException ioe) {
            initState = INIT_IOEXCEPTION;
            smtpParameters = null;
            errorList = Collections.singletonList(ioe.getMessage());
        }
        List<String> newErrorList = new ArrayList<String>();
        smtpParameters = SmtpUtils.buildFromProperties(properties, newErrorList);
        if (newErrorList.size() > 0) {
            smtpParameters = null;
            initState = INIT_PROPERTIESERRORS;
            errorList = newErrorList;
        } else {
            initState = INIT_OK;
            errorList = null;
        }
    }

}
