/* BdfServer_Html - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserConstants;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.localisation.BundleLocalisation;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public abstract class BdfHtmlProducer extends UtilHtmlPrinter implements CommandHtmlProducer {

    protected BdfServer bdfServer;
    protected BdfUser bdfUser;
    protected int workingLangInteger;
    protected Locale formatLocale;
    protected LocalisationInfo localisationInfo;
    protected Basedefiches basedefiches;
    protected BdfContext bdfContext;
    private boolean admin = false;
    private boolean withJQuery = false;
    private String reversePath = "";
    private String cssThemeFile = "_global.css";
    private boolean jqueryIncluded = false;
    private CommandMessage commandMessage;

    public BdfHtmlProducer() {
    }

    protected void init(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser.getLocalisationInfo(), bdfUser.isWithJQuery());
        this.bdfUser = bdfUser;
        this.admin = BdfUserUtils.isAdmin(bdfUser);
        this.bdfContext = bdfContext;

    }

    protected void init(BdfContext bdfContext, LocalisationInfo localisationInfo, boolean withJQuery) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.basedefiches = bdfServer.getBasedefiches();
        this.localisationInfo = localisationInfo;
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
        this.formatLocale = localisationInfo.getFormatLocale();
        this.withJQuery = withJQuery;
        addResourceLocalisation(BundleLocalisation.getBundleLocalisation("fr.exemole.bdfserver.servlets.resources.l10n.servlets", formatLocale));
        setTitle("BDF");
    }

    public BdfContext getBdfContext() {
        return bdfContext;
    }

    public void setReversePath(String reversePath) {
        if (reversePath == null) {
            this.reversePath = "";
        } else {
            this.reversePath = reversePath;
        }
    }

    public String getReversePath() {
        return reversePath;
    }

    public void setCssThemeFile(String cssThemeFile) {
        this.cssThemeFile = cssThemeFile;
    }

    public void addJavascriptFiles(String fileNames) {
        if (isWithJQuery()) {
            if (!jqueryIncluded) {
                jqueryIncluded = true;
                addJsUrl(reversePath + "jquery/jquery.js");
            }
            if (fileNames != null) {
                if (fileNames.indexOf(',') != -1) {
                    String[] tokens = StringUtils.getCleanedTokens(fileNames, ',');
                    int length = tokens.length;
                    for (int i = 0; i < length; i++) {
                        addJsUrl(reversePath + "js/" + tokens[i]);
                    }
                } else {
                    addJsUrl(reversePath + "js/" + fileNames);
                }
            }
        }
    }

    public void addJqueryExtensions(String extensionNames) {
        if (isWithJQuery()) {
            if (!jqueryIncluded) {
                jqueryIncluded = true;
                addJsUrl(reversePath + "jquery/jquery.js");
            }
            String[] tokens = StringUtils.getCleanedTokens(extensionNames, ',');
            int length = tokens.length;
            for (int i = 0; i < length; i++) {
                addJsUrl(reversePath + "jquery/jquery." + tokens[i] + ".js");
            }
        }
    }

    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    public boolean isWithJQuery() {
        return withJQuery;
    }

    public boolean isAdmin() {
        return admin;
    }

    public int getWorkingLangInteger() {
        return workingLangInteger;
    }

    public void writeHtml(Appendable appendable) {
        initPrinter(appendable);
        printHtml();
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public abstract void printHtml();

    public void start() {
        if (cssThemeFile != null) {
            String themePath = reversePath + "theme/css/";
            addCssUrl(themePath + cssThemeFile);
            short screenWidth = BdfUserUtils.getScreenWidth(bdfUser);
            switch (screenWidth) {
                case BdfUserConstants.SCREENWIDTH_800:
                    addCssUrl(themePath + "_screen_800.css");
                    break;
                case BdfUserConstants.SCREENWIDTH_1024:
                    addCssUrl(themePath + "_screen_1024.css");
                    break;
                case BdfUserConstants.SCREENWIDTH_1280:
                    addCssUrl(themePath + "_screen_1280.css");
                    break;

            }
        }
        super.start(localisationInfo.getFormatLocaleInteger());
    }

    public void printMessage() {
        if (commandMessage != null) {
            printMessage(commandMessage);
        }
    }

}
