/* BdfServer_Html - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.request.form.EditableAssociationElement;
import fr.exemole.bdfserver.request.form.FormElement;
import fr.exemole.bdfserver.request.form.GeopointSubfieldsElement;
import fr.exemole.bdfserver.request.form.HiddenFieldElement;
import fr.exemole.bdfserver.request.form.HiddenIncludeElement;
import fr.exemole.bdfserver.request.form.ImageFieldElement;
import fr.exemole.bdfserver.request.form.ItemFieldElement;
import fr.exemole.bdfserver.request.form.LangFieldElement;
import fr.exemole.bdfserver.request.form.LiageElement;
import fr.exemole.bdfserver.request.form.ListIndexationElement;
import fr.exemole.bdfserver.request.form.MontantSubfieldsElement;
import fr.exemole.bdfserver.request.form.NotEditableAssociationElement;
import fr.exemole.bdfserver.request.form.NotEditableIndexationElement;
import fr.exemole.bdfserver.request.form.PersonneSubfieldsElement;
import fr.exemole.bdfserver.request.form.RattachementToMasterElement;
import fr.exemole.bdfserver.request.form.RattachementToSlaveElement;
import fr.exemole.bdfserver.request.form.TextFieldElement;
import fr.exemole.bdfserver.request.form.TextIndexationElement;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class CommonFormHtml {

    private CommonFormHtml() {
    }

    public static void printFFCoreLocValues(HtmlPrinter htmlPrinter, LocalisationInfo localisationInfo) {
        htmlPrinter.startScript(null);
        CommonFormHtml.printFFCoreParamValue(htmlPrinter, "langui", LangInteger.toString(localisationInfo.getFormatLocaleInteger()));
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_info_mandatory");
        htmlPrinter.append("\n");
        htmlPrinter.endScript();
    }

    public static void printFFCoreLocValue(HtmlPrinter htmlPrinter, String locKey) {
        htmlPrinter.append("\n FF_Core.setLoc(\"");
        htmlPrinter.append(locKey);
        htmlPrinter.append("\",\"");
        htmlPrinter.localize(locKey);
        htmlPrinter.append("\");");
    }

    public static void printFFCoreLocValue(HtmlPrinter htmlPrinter, String jsLocKey, String htmlPrinterLocKey, Object value) {
        htmlPrinter.append("\n FF_Core.setLoc(\"");
        htmlPrinter.append(jsLocKey);
        htmlPrinter.append("\",\"");
        htmlPrinter.localize(htmlPrinterLocKey, value);
        htmlPrinter.append("\");");
    }

    public static void printFFCoreLimiteValue(HtmlPrinter htmlPrinter, String name, String limite) {
        htmlPrinter.append("\n FF_Core.setLimite(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",");
        htmlPrinter.append(limite);
        htmlPrinter.append(");");
    }

    public static void printFFCoreParamValue(HtmlPrinter htmlPrinter, String name, String params) {
        htmlPrinter.append("\n FF_Core.setParam(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(params);
        htmlPrinter.append("\");");
    }

    public static void printFFCorePiocheSelectString(HtmlPrinter htmlPrinter, String name, String piocheSelectString) {
        htmlPrinter.append("\n FF_Core.setPiocheSelect(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.escapeScriptString(piocheSelectString);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreLocValues(HtmlPrinter htmlPrinter) {
        htmlPrinter.startScript(null);
        CommonFormHtml.printSLCCoreLocValue(htmlPrinter, "global_link_pioche_redacteur");
        CommonFormHtml.printSLCCoreLocValue(htmlPrinter, "global_link_pioche_motcle_all");
        CommonFormHtml.printSLCCoreLocValue(htmlPrinter, "global_link_pioche_motcle_selection");
        CommonFormHtml.printSLCCoreLocValue(htmlPrinter, "global_err_motcle_selection");
        htmlPrinter.append("\n");
        htmlPrinter.endScript();
    }

    public static void printSLCCoreLocValue(HtmlPrinter htmlPrinter, String locKey) {
        htmlPrinter.append("\n SLC_Core.setLoc(\"");
        htmlPrinter.append(locKey);
        htmlPrinter.append("\",\"");
        htmlPrinter.localize(locKey);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreLocValue(HtmlPrinter htmlPrinter, String jsLocKey, String htmlPrinterLocKey, Object value) {
        htmlPrinter.append("\n SLC_Core.setLoc(\"");
        htmlPrinter.append(jsLocKey);
        htmlPrinter.append("\",\"");
        htmlPrinter.localize(htmlPrinterLocKey, value);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreLimiteValue(HtmlPrinter htmlPrinter, String name, String limite) {
        htmlPrinter.append("\n SLC_Core.setLimite(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",");
        htmlPrinter.append(limite);
        htmlPrinter.append(");");
    }

    public static void printSLCCoreCategoryValue(HtmlPrinter htmlPrinter, String name, String limite) {
        htmlPrinter.append("\n SLC_Core.setCategory(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(limite);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreParamValue(HtmlPrinter htmlPrinter, String name, String params) {
        htmlPrinter.append("\n SLC_Core.setParam(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(params);
        htmlPrinter.append("\");");
    }

    public static void printSLCPiocheAdd(HtmlPrinter htmlPrinter, String appelantId, String toolId) {
        htmlPrinter.append("\n SLC_Pioche.add(\"");
        htmlPrinter.append(appelantId);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(toolId);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreSubsetsValue(HtmlPrinter htmlPrinter, String name, String subsets) {
        htmlPrinter.append("\n SLC_Core.setSubsets(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(subsets);
        htmlPrinter.append("\");");
    }

    public static void printSLCCoreTitleKeyValue(HtmlPrinter htmlPrinter, String name, String titleKey) {
        htmlPrinter.append("\n SLC_Core.setTitleKey(\"");
        htmlPrinter.append(name);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(titleKey);
        htmlPrinter.append("\");");
    }

    public static void printFormElement(BdfHtmlProducer bdfHtmlProducer, FormElement formElement, String namePrefix) {
        if (formElement instanceof TextFieldElement) {
            CorpusFieldFormHtml.printTextField(bdfHtmlProducer, (TextFieldElement) formElement, namePrefix);
        } else if (formElement instanceof LangFieldElement) {
            CorpusFieldFormHtml.printLangField(bdfHtmlProducer, (LangFieldElement) formElement, namePrefix);
        } else if (formElement instanceof PersonneSubfieldsElement) {
            CorpusFieldFormHtml.printPersonneSubfields(bdfHtmlProducer, (PersonneSubfieldsElement) formElement, namePrefix);
        } else if (formElement instanceof GeopointSubfieldsElement) {
            CorpusFieldFormHtml.printGeopointSubfields(bdfHtmlProducer, (GeopointSubfieldsElement) formElement, namePrefix);
        } else if (formElement instanceof MontantSubfieldsElement) {
            CorpusFieldFormHtml.printCurrencyArrayField(bdfHtmlProducer, (MontantSubfieldsElement) formElement, namePrefix);
        } else if (formElement instanceof ItemFieldElement) {
            CorpusFieldFormHtml.printItemField(bdfHtmlProducer, (ItemFieldElement) formElement, namePrefix);
        } else if (formElement instanceof HiddenFieldElement) {
            CorpusFieldFormHtml.printHiddenField(bdfHtmlProducer, (HiddenFieldElement) formElement, namePrefix);
        } else if (formElement instanceof ImageFieldElement) {
            CorpusFieldFormHtml.printImageField(bdfHtmlProducer, (ImageFieldElement) formElement, namePrefix);
        } else if (formElement instanceof RattachementToMasterElement) {
            IncludeFormHtml.printRattachementToMaster(bdfHtmlProducer, (RattachementToMasterElement) formElement);
        } else if (formElement instanceof RattachementToSlaveElement) {
            IncludeFormHtml.printRattachementToSlave(bdfHtmlProducer, (RattachementToSlaveElement) formElement);
        } else if (formElement instanceof NotEditableAssociationElement) {
            IncludeFormHtml.printNotEditableAssociation(bdfHtmlProducer, (NotEditableAssociationElement) formElement);
        } else if (formElement instanceof EditableAssociationElement) {
            IncludeFormHtml.printEditableAssociation(bdfHtmlProducer, (EditableAssociationElement) formElement, namePrefix);
        } else if (formElement instanceof TextIndexationElement) {
            IncludeFormHtml.printTextIndexation(bdfHtmlProducer, (TextIndexationElement) formElement, namePrefix);
        } else if (formElement instanceof ListIndexationElement) {
            IncludeFormHtml.printListIndexation(bdfHtmlProducer, (ListIndexationElement) formElement, namePrefix);
        } else if (formElement instanceof HiddenIncludeElement) {
            IncludeFormHtml.printHiddenInclude(bdfHtmlProducer, (HiddenIncludeElement) formElement, namePrefix);
        } else if (formElement instanceof NotEditableIndexationElement) {
            IncludeFormHtml.printNotEditableIndexation(bdfHtmlProducer, (NotEditableIndexationElement) formElement, namePrefix);
        } else if (formElement instanceof LiageElement) {
            IncludeFormHtml.printLiage(bdfHtmlProducer, (LiageElement) formElement, namePrefix);
        }
    }

}
