/* BdfServer_Html - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class FormRowPrinter {

    private BdfHtmlProducer bhp;
    private int currentTextInputSize;
    private String currentTextInputSizeString;
    private String currentRowLibelle = "";

    public FormRowPrinter(BdfHtmlProducer bdfHtmlProducer) {
        this.bhp = bdfHtmlProducer;
    }

    public void setCurrentCorpusField(CorpusField corpusField) {
        this.currentRowLibelle = corpusField.getLibelleString(bhp.getWorkingLangInteger(), true);
    }

    public void setCurrentTextInputSize(int size) {
        this.currentTextInputSize = size;
        currentTextInputSizeString = String.valueOf(currentTextInputSize);
    }

    public void startSubfieldTable() {
        bhp.startRow();
        bhp.startCell(3, 0);
        bhp.escape(currentRowLibelle);
        bhp.printColon();
        bhp.endCell();
        bhp.endRow();
        bhp.startRow();
        bhp.startCell(3, 0);
        bhp.startTable();
    }

    public void endSubfieldTable() {
        bhp.endTable();
        bhp.endCell();
        bhp.endRow();
    }

    public void startSubFieldRow(String libKey) {
        bhp.startRow();
        bhp.startCell();
        bhp.doublespace();
        bhp.startSmall();
        bhp.localize(libKey);
        bhp.endSmall();
        bhp.endCell();
        bhp.startCell();
    }

    public void endSubfieldRow() {
        bhp.endCell();
        bhp.endRow();
    }

    public void printTextInput(SubfieldKey subfieldKey, String value) {
        String name = subfieldKey.toString();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(name);
        htmlAttributes.setId(name);
        htmlAttributes.setValue(value);
        htmlAttributes.setSize(currentTextInputSizeString);
        bhp.appendTextInput(htmlAttributes);
    }

    public void printNomPersonneSubField(SubfieldKey subfieldKey, String value, boolean nomAvant) {
        printTextInput(subfieldKey, value);
        SubfieldKey nomAvantKey = new SubfieldKey(subfieldKey.getFieldKey(), SubfieldKey.NOMAVANT_SUBTYPE);
        String name = nomAvantKey.toString();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(name);
        htmlAttributes.setId(name);
        htmlAttributes.setValue("1");
        htmlAttributes.setChecked(nomAvant);
        bhp.endCell();
        bhp.startCell();
        bhp.startSmall();
        bhp.startSmall();
        bhp.addCheck(htmlAttributes, "sphere_lib_nomavant", null);
        bhp.endSmall();
        bhp.endSmall();
    }

    public void printCommentRow(CommentUI commentUI) {
        String lib = commentUI.getLibelleString(bhp.getWorkingLangInteger(), false);
        if (lib == null) {
            lib = "";
        }
        bhp.startRow();
        bhp.startCell(3, 0);
        bhp.append(lib);
        bhp.endCell();
        bhp.endRow();
    }

}
