/* BdfServer_Html - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.menus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.extension.BdfExtension;
import fr.exemole.bdfserver.extension.BdfExtensionManager;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class MenuLinkUtils {

    public static MenuLink getThemeMenuLink(String url, String menuKey, ResourceLocalisation resourceLocalisation) {
        MutableMenuLink menuLink = new MutableMenuLink(url);
        menuLink.setIconPath("theme/icons/16x16/" + menuKey + ".png");
        menuLink.setIntitule(resourceLocalisation.toString(menuKey, null));
        return menuLink;
    }

    public static MenuLink getExtensionMenuLink(BdfContext bdfContext, String linkPath, LocalisationInfo localisationInfo) {
        int idx = linkPath.indexOf('/');
        if (idx == -1) {
            return new ErrorMenuLink("slashMissing: " + linkPath);
        }
        String extensionName = linkPath.substring(0, idx);
        String linkName = linkPath.substring(idx + 1);
        BdfExtensionManager bdfExtensionManager = (BdfExtensionManager) bdfContext.getContextObject(BdfContextConstants.BDFEXTENSIONMANAGER_NAME);
        BdfExtension bdfExtension = bdfExtensionManager.getBdfExtension(bdfContext, extensionName);
        if (bdfExtension == null) {
            return new ErrorMenuLink("unknwonExtension: " + extensionName);
        }
        Object obj = bdfExtension.newInstance("interface:MenuLinkProvider", null);
        if (obj == null) {
            return new ErrorMenuLink("noMenuLinkProviderInterface: " + extensionName);
        }
        if (!(obj instanceof MenuLinkProvider)) {
            return new ErrorMenuLink("notMenuLinkProviderImplementation: " + obj.getClass().getName());
        }
        MenuLinkProvider menuLinkProvider = (MenuLinkProvider) obj;
        MenuLink menuLink = menuLinkProvider.getMenuLink(linkName, null);
        if (menuLink == null) {
            return new ErrorMenuLink("unknownLinkName:" + linkName);
        }
        return menuLink;
    }


    private static class ErrorMenuLink implements MenuLink {

        private String errorMessage;

        private ErrorMenuLink(String errorMessage) {
            this.errorMessage = errorMessage;
        }

        public String getIconPath() {
            return null;
        }

        public String getTarget() {
            return null;
        }

        public String getUrl() {
            return "#";
        }

        public String getIntitule() {
            return errorMessage;
        }

    }

}
