/* BdfServer_Interaction - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.session;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.LinkedList;
import java.net.*;
import java.text.ParseException;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;


/**
 *
 * @author Vincent Calame
 */
public class SessionManager implements SessionConstants {

    private BdfServer bdfServer;
    private Map<String, BdfUser> sessionsMap = new LinkedHashMap();
    static int maxInactiveInterval = 18000;

    public SessionManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfUser getBdfUser(BdfRequestMap requestBundle) throws LoginException {
        HttpSession session = requestBundle.getRequest().getSession(false);
        BdfUser bdfUser = (session != null) ? (BdfUser) session.getAttribute("bdfUser") : null;
        if (bdfUser == null) {
            bdfUser = createBdfUser(requestBundle);
        }
        return bdfUser;
    }

    private BdfUser createBdfUser(BdfRequestMap requestBundle) throws LoginException {
        BdfRequestMap.SessionParameters sessionParameters = requestBundle.getSessionParameters();
        String param_sphere = sessionParameters.getBdfSphere();
        String param_idsphere = sessionParameters.getBdfIdsphere();
        String param_password = sessionParameters.getBdfPassword();
        HttpServletRequest request = requestBundle.getRequest();
        if ((param_idsphere == null) || (param_password == null)) {
            throw new LoginException(LoginException.UNDEFINED_ERROR);
        }
        param_idsphere = param_idsphere.trim();
        RedacteurKey redacteurKey = null;
        try {
            if (param_sphere != null) {
                param_sphere = param_sphere.trim();
                if (param_sphere.length() > 0) {
                    redacteurKey = RedacteurKey.parse(param_sphere + "_" + param_idsphere);
                } else {
                    redacteurKey = RedacteurKey.parse(param_idsphere);
                }

            } else {
                redacteurKey = RedacteurKey.parse(param_idsphere);
            }
        } catch (ParseException pe) {
            throw new LoginException(LoginException.IDSPHERE_ERROR);
        }
        Sphere sphere = (Sphere) bdfServer.getBasedefiches().getSubset(redacteurKey.getSphereKey());
        if (sphere == null) {
            throw new LoginException(LoginException.SPHERE_ERROR);
        }
        Redacteur redacteur = sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString());
        if (redacteur == null) {
            throw new LoginException(LoginException.IDSPHERE_ERROR);
        }
        if (!redacteur.isActif()) {
            throw new LoginException(LoginException.INACTIF_ERROR);
        }
        if (!isLocalHost(request)) {
            String password = bdfServer.getPasswordManager().getPassword(redacteurKey);
            if ((param_password == null) || (!param_password.equals(password))) {
                throw new LoginException(LoginException.PASSWORD_ERROR);
            }
        }
        HttpSession httpsession = request.getSession();
        BdfUser bdfUser = new BdfUser(bdfServer, redacteurKey);
        bdfUser.setBaseDeFichesURL(getBaseDeFichesURL(request));
        bdfUser.setSessionObject(httpsession);
        bdfUser.setScreenWidth(sessionParameters.getScreenWidth());
        bdfUser.setWithJQuery(sessionParameters.isWithJQuery());
        bdfUser.updateAbregeFicheSortedList();
        httpsession.setAttribute("bdfUser", bdfUser);
        httpsession.setAttribute("bdfBinding", new BdfBinding());
        httpsession.setMaxInactiveInterval(maxInactiveInterval);
        sessionsMap.put(httpsession.getId(), bdfUser);
        return bdfUser;
    }

    private String getBaseDeFichesURL(HttpServletRequest request) {
        StringBuilder url = new StringBuilder();
        String scheme = request.getScheme();
        int port = request.getServerPort();
        if (port < 0) {
            port = 80; // Work around java.net.URL bug
        }
        url.append(scheme);
        url.append("://");
        url.append(request.getServerName());
        if ((scheme.equals("http") && (port != 80))
                || (scheme.equals("https") && (port != 443))) {
            url.append(':');
            url.append(port);
        }
        url.append(request.getContextPath());
        url.append("/");
        return url.toString();
    }

    synchronized public void clear() {
        List<BdfUser> liste = new LinkedList(sessionsMap.values());
        for (BdfUser bdfUser : liste) {
            invalidateBdfUser(bdfUser);
        }
        sessionsMap.clear();
    }

    synchronized public void invalidateBdfUser(BdfUser bdfUser) {
        HttpSession httpSession = (HttpSession) bdfUser.getSessionObject();
        sessionsMap.remove(httpSession.getId());
        httpSession.removeAttribute("bdfUser");
        httpSession.removeAttribute("bdfBinding");
        httpSession.invalidate();
    }

    synchronized public BdfUser[] getCurrentBdfUserArray() {
        BdfUser[] array = new BdfUser[sessionsMap.size()];
        int p = 0;
        for (BdfUser bdfUser : sessionsMap.values()) {
            array[p] = bdfUser;
            p++;
        }
        return array;
    }


    class BdfBinding implements HttpSessionBindingListener {

        public void valueBound(HttpSessionBindingEvent httpSessionEvent) {
        }

        public void valueUnbound(HttpSessionBindingEvent httpSessionEvent) {
            HttpSession session = httpSessionEvent.getSession();
            sessionsMap.remove(session.getId());
        }

    }

    static boolean isLocalHost(HttpServletRequest request) {
        try {
            InetAddress inet = InetAddress.getByName(request.getRemoteAddr());
            return inet.equals(InetAddress.getLocalHost());
        } catch (UnknownHostException uhe) {
            return false;
        }
    }

}
