/* @@@@@APPLICATION@@@@@ - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.output;

import net.mapeadores.opendocument.css.elements.ElementMaps;
import net.mapeadores.opendocument.css.elements.StyleElement;
import net.mapeadores.opendocument.css.elements.ListStyleElement;
import java.util.Map;
import java.util.Collection;
import java.io.File;
import java.io.IOException;
import java.io.BufferedWriter;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;


/**
 *
 * @author Vincent Calame
 */
public class StylesOutput {

    private StylesXMLWriter stylesWriter;

    private StylesOutput(Appendable appendable) throws IOException {
        stylesWriter = new StylesXMLWriter(appendable, 0);
    }

    private void start() throws IOException {
        stylesWriter.appendXMLDeclaration();
        stylesWriter.openStyleDocument();
        stylesWriter.openStyles();
    }

    private void close() throws IOException {
        stylesWriter.closeStyles();
        stylesWriter.closeStyleDocument();
    }

    private void startAutomaticStyles() throws IOException {
        stylesWriter.appendXMLDeclaration();
        stylesWriter.startOpenTag("office:automatic-styles");
        stylesWriter.appendNameSpaceAttributes();
        stylesWriter.endOpenTag();
    }

    private void closeAutomaticStyles() throws IOException {
        stylesWriter.closeTag("office:automatic-styles");
    }

    private void writeAutomaticStyles(ElementMaps elementMaps) throws IOException {
        writeStyles(elementMaps, StyleElement.TABLE_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLECOLUMN_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLECELL_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLEROW_FAMILY);
    }

    private void writeStyles(ElementMaps elementMaps) throws IOException {
        writeStyles(elementMaps.getHMap().values());
        writeStyles(elementMaps, StyleElement.PARAGRAPH_FAMILY);
        writeStyles(elementMaps, StyleElement.TEXT_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLE_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLECOLUMN_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLECELL_FAMILY);
        writeStyles(elementMaps, StyleElement.TABLEROW_FAMILY);
        Map<String, ListStyleElement> map = elementMaps.getListStyleMap();
        for (ListStyleElement listStyleElement : map.values()) {
            stylesWriter.addListStyleElement(listStyleElement);
        }
    }

    private void writeStyles(ElementMaps elementMaps, String family) throws IOException {
        Map<String, StyleElement> map = elementMaps.getFamilyMap(family);
        if ((map == null) || (map.size() == 0)) {
            return;
        }
        writeStyles(map.values());
    }

    private void writeStyles(Collection<StyleElement> collection) throws IOException {
        for (StyleElement styleElement : collection) {
            stylesWriter.addStyleElement(styleElement);
        }
    }

    public static void write(OutputStream outputStream, ElementMaps elementMaps) throws IOException {
        BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
        StylesOutput stylesOutput = new StylesOutput(bufwriter);
        stylesOutput.start();
        stylesOutput.writeStyles(elementMaps);
        stylesOutput.close();
        bufwriter.flush();
    }

    public static void writeAutomaticStyles(OutputStream outputStream, ElementMaps elementMaps) throws IOException {
        BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
        StylesOutput stylesOutput = new StylesOutput(bufwriter);
        stylesOutput.startAutomaticStyles();
        stylesOutput.writeAutomaticStyles(elementMaps);
        stylesOutput.closeAutomaticStyles();
        bufwriter.flush();
    }

    public static void write(File file, ElementMaps elementMaps) throws IOException {
        OutputStream os = new FileOutputStream(file);
        write(os, elementMaps);
        os.close();
    }

    /**
     * Écrit les styles de tables (uniquement celles-là) dans un fichier à part.
     */
    public static void writeAutomaticStyles(File file, ElementMaps elementMaps) throws IOException {
        OutputStream os = new FileOutputStream(file);
        writeAutomaticStyles(os, elementMaps);
        os.close();
    }

}
