/* @@@@@APPLICATION@@@@@ - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.parse;

import net.mapeadores.opendocument.css.OdCssLibException;
import net.mapeadores.opendocument.css.elements.ElementMaps;
import java.util.List;
import java.io.File;
import java.io.IOException;
import java.io.Reader;
import org.w3c.css.sac.InputSource;
import org.w3c.css.sac.Parser;
import org.w3c.css.sac.helpers.ParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class CssParser {

    public CssParser() {
    }

    public static ElementMaps parse(CssSource cssSource) throws IOException {
        Reader reader = cssSource.getReader();
        InputSource inputSource = new InputSource(reader);
        OdCssDocumentHandler documentHandler = new OdCssDocumentHandler();
        OdCssErrorHandler errorHandler = new OdCssErrorHandler();
        ParserFactory parserFactory = new ParserFactory();
        try {
            Parser parser = parserFactory.makeParser();
            parser.setDocumentHandler(documentHandler);
            parser.setErrorHandler(errorHandler);
            parser.parseStyleSheet(inputSource);
        } catch (ClassNotFoundException nf) {
            throw new OdCssLibException(nf);
        } catch (IllegalAccessException nf) {
            throw new OdCssLibException(nf);
        } catch (InstantiationException nf) {
            throw new OdCssLibException(nf);
        }
        ElementMaps elementMaps = documentHandler.getElementMaps();
        List<String> list = documentHandler.getImportList();
        int size = list.size();
        if (size > 0) {
            for (int i = (size - 1); i >= 0; i--) {
                String url = list.get(i);
                CssSource importSource = cssSource.getImportCssSource(url);
                if (importSource != null) {
                    ElementMaps importElementMaps = parse(importSource);
                    elementMaps.append(importElementMaps);
                }
            }
        }
        return elementMaps;
    }

    public static ElementMaps parse(File file) throws IOException {
        FileCssSource cssSource = new FileCssSource(file);
        return parse(cssSource);
    }

}
