/* ScrutariDataExport - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.dataexport.xml;

import net.scrutari.dataexport.api.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class XmlCorpusMetadataExport extends XmlBuilder implements CorpusMetadataExport {

    private String corpusType;
    private String corpusIcon;
    private String hrefParent;
    private Map<String, String> corpusMap = new LinkedHashMap<String, String>();
    private Map<String, String> ficheMap = new LinkedHashMap<String, String>();
    private List<Map<String, String>> complementMapList = new ArrayList<Map<String, String>>();

    public void setCorpusType(String corpusType) {
        this.corpusType = corpusType;
    }

    public void setCorpusIcon(String corpusIcon) {
        this.corpusIcon = corpusIcon;
    }

    public void setHrefParent(String hrefParent) {
        this.hrefParent = hrefParent;
    }

    public void setIntitule(int intituleType, String lang, String intituleValue) {
        Map<String, String> map = getMap(intituleType);
        map.put(lang, intituleValue);
    }

    public int addComplement() {
        complementMapList.add(new LinkedHashMap<String, String>());
        return complementMapList.size();
    }

    public void setComplementIntitule(int complementNumber, String lang, String intituleValue) {
        if ((complementNumber < 0) || (complementNumber > complementMapList.size())) {
            return;
        }
        Map<String, String> map = complementMapList.get(complementNumber - 1);
        map.put(lang, intituleValue);
    }

    public void writeXML(XmlWriter xmlWriter) {
        xmlWriter.openTag("corpus-metadata");
        xmlWriter.addSimpleElement("corpus-type", corpusType);
        addMap(INTITULE_CORPUS, xmlWriter);
        addMap(INTITULE_FICHE, xmlWriter);
        xmlWriter.addSimpleElement("href-parent", hrefParent);
        xmlWriter.addSimpleElement("corpus-icon", corpusIcon);
        for (Map<String, String> map : complementMapList) {
            addComplementMap(map, xmlWriter);
        }
        xmlWriter.closeTag("corpus-metadata");
    }

    private void addComplementMap(Map<String, String> map, XmlWriter xmlWriter) {
        xmlWriter.openTag("complement-metadata");
        for (Map.Entry<String, String> entry : map.entrySet()) {
            xmlWriter.addLibElement(entry.getKey(), entry.getValue());
        }
        xmlWriter.closeTag("complement-metadata");
    }

    private void addMap(int intituleType, XmlWriter xmlWriter) {
        Map<String, String> map = getMap(intituleType);
        if (map.isEmpty()) {
            return;
        }
        String suffix = getSuffix(intituleType);
        xmlWriter.openTag("intitule-" + suffix);
        for (Map.Entry<String, String> entry : map.entrySet()) {
            xmlWriter.addLibElement(entry.getKey(), entry.getValue());
        }
        xmlWriter.closeTag("intitule-" + suffix);
    }

    private Map<String, String> getMap(int type) {
        switch (type) {
            case INTITULE_CORPUS:
                return corpusMap;
            case INTITULE_FICHE:
                return ficheMap;
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

    private static String getSuffix(int type) {
        switch (type) {
            case INTITULE_CORPUS:
                return "corpus";
            case INTITULE_FICHE:
                return "fiche";
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

}
