/* ScrutariDataExport - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.dataexport.xml;

import net.scrutari.dataexport.api.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class XmlFicheExport extends XmlBuilder implements FicheExport {

    private String ficheId;
    private String titre;
    private String soustitre;
    private String date;
    private String lang;
    private String href;
    private String ficheType;
    private String ficheIcon;
    private String latitude;
    private String longitude;
    private SortedMap<Integer, String> complementMap = new TreeMap<Integer, String>();

    public XmlFicheExport() {
    }

    public void reinit(String ficheId) {
        this.ficheId = ficheId;
        this.titre = null;
        this.soustitre = null;
        this.date = null;
        this.lang = null;
        this.href = null;
        this.ficheType = null;
        this.ficheIcon = null;
        this.latitude = null;
        this.longitude = null;
        this.complementMap.clear();
    }

    public void setTitre(String titre) {
        this.titre = titre;
    }

    public void setSoustitre(String soustitre) {
        this.soustitre = soustitre;
    }

    public void setDate(String date) {
        this.date = date;
    }

    public void setLang(String lang) {
        this.lang = lang;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public void setFicheType(String ficheType) {
        this.ficheType = ficheType;
    }

    public void setFicheIcon(String ficheIcon) {
        this.ficheIcon = ficheIcon;
    }

    public void setGeoloc(String latitude, String longitude) {
        this.latitude = latitude;
        this.longitude = longitude;
    }

    public void addComplement(int complementNumber, String complementValue) {
        if (complementNumber < 1) {
            return;
        }
        this.complementMap.put(new Integer(complementNumber), complementValue);
    }

    public void writeXML(XmlWriter xmlWriter) {
        xmlWriter.openTagWithAttribute("fiche", "fiche-id", ficheId);
        xmlWriter.addSimpleElement("titre", titre);
        xmlWriter.addSimpleElement("soustitre", soustitre);
        xmlWriter.addSimpleElement("date", date);
        xmlWriter.addSimpleElement("lang", lang);
        xmlWriter.addSimpleElement("href", href);
        xmlWriter.addSimpleElement("fiche-type", ficheType);
        xmlWriter.addSimpleElement("fiche-icon", ficheIcon);
        if ((latitude != null) && (longitude != null)) {
            xmlWriter.openTag("geoloc");
            xmlWriter.addSimpleElement("lat", latitude);
            xmlWriter.addSimpleElement("lon", longitude);
            xmlWriter.closeTag("geoloc");
        }
        if (complementMap.size() > 0) {
            int max = complementMap.lastKey().intValue();
            for (int i = 1; i <= max; i++) {
                String s = complementMap.get(new Integer(i));
                if ((s == null) || (s.length() == 0)) {
                    xmlWriter.addEmptyElement("complement", null, null);
                } else {
                    xmlWriter.addSimpleElement("complement", s);
                }
            }
        }
        xmlWriter.closeTag("fiche");
    }

}
