/* ScrutariDataExport - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.dataexport.xml;

import net.scrutari.dataexport.api.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class XmlThesaurusMetadataExport extends XmlBuilder implements ThesaurusMetadataExport {

    private Map<String, String> thesaurusMap = new LinkedHashMap<String, String>();

    public void setIntitule(int intituleType, String lang, String intituleValue) {
        Map<String, String> map = getMap(intituleType);
        map.put(lang, intituleValue);
    }

    public void writeXML(XmlWriter xmlWriter) {
        xmlWriter.openTag("thesaurus-metadata");
        addMap(INTITULE_THESAURUS, xmlWriter);
        xmlWriter.closeTag("thesaurus-metadata");
    }

    private void addMap(int intituleType, XmlWriter xmlWriter) {
        Map<String, String> map = getMap(intituleType);
        if (map.isEmpty()) {
            return;
        }
        String suffix = getSuffix(intituleType);
        xmlWriter.openTag("intitule-" + suffix);
        for (Map.Entry<String, String> entry : map.entrySet()) {
            xmlWriter.addLibElement(entry.getKey(), entry.getValue());
        }
        xmlWriter.closeTag("intitule-" + suffix);
    }

    private Map<String, String> getMap(int type) {
        switch (type) {
            case INTITULE_THESAURUS:
                return thesaurusMap;
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

    private static String getSuffix(int type) {
        switch (type) {
            case INTITULE_THESAURUS:
                return "thesaurus";
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

}
