/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.util.List;
import java.util.ArrayList;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class ConditionGroup implements ConditionsConstants {

    private short mode;
    private short state = U;
    private List<ContentConditionBean> excludingList = new ArrayList<ContentConditionBean>();
    private List<ContentConditionBean> includingList = new ArrayList<ContentConditionBean>();

    public ConditionGroup(short mode) {
        this.mode = mode;
    }

    public ConditionGroup(ConditionGroupBean bean) {
        this.mode = bean.getMode();
        ContentConditionBean[] contentConditionBeanArray = bean.getContentConditionBeanArray();
        if (contentConditionBeanArray != null) {
            for (int i = 0; i < contentConditionBeanArray.length; i++) {
                if (contentConditionBeanArray[i] != null) {
                    addContentCondition(contentConditionBeanArray[i]);
                }
            }
        }
    }

    public short getMode() {
        return mode;
    }

    public short getState() {
        return state;
    }

    public void addContentCondition(ContentConditionBean contentConditionBean) {
        short conditionType = contentConditionBean.getConditionType();
        short addingState = Conditions.getState(conditionType);
        state = Conditions.merge(mode, state, addingState);
        if (!Conditions.isPartState(state)) {
            excludingList.clear();
            includingList.clear();
        } else {
            if (Conditions.isExcludingPartConditionType(conditionType)) {
                excludingList.add(new ContentConditionBean(contentConditionBean)); //@todo cover
            } else if (Conditions.isIncludingPartConditionType(conditionType)) {
                includingList.add(new ContentConditionBean(contentConditionBean)); //@todo cover
            }
        }
    }

    public ContentConditionBean[] getExcludingContentConditionBeanArray() {
        return getContentConditionBeanArray(excludingList);
    }

    public ContentConditionBean[] getIncludingContentConditionBeanArray() {
        return getContentConditionBeanArray(includingList);
    }

    private ContentConditionBean[] getContentConditionBeanArray(List<ContentConditionBean> list) {
        int size = list.size();
        ContentConditionBean[] result = new ContentConditionBean[size];
        for (int i = 0; i < size; i++) {
            result[i] = new ContentConditionBean(list.get(i));
        }
        return result;
    }

    /**
     * Retourne lui-même si son statut est P, si PA une instance de ConditionGroup
     * une instance 
     */
    public ConditionGroup toPartConditionGroup() {
        if (state == P) {
            return this;
        }
        if (state == PA) {
            ConditionGroup cg = new ConditionGroup(mode);
            cg.state = P;
            cg.excludingList.addAll(excludingList);
            cg.includingList.addAll(includingList);
            return cg;
        }
        return null;
    }

    public ConditionGroupBean toConditionGroupBean() {
        return new ConditionGroupBean(mode, getContentConditionArray());
    }

    public ContentConditionBean[] getContentConditionArray() {
        switch (state) {
            case A:
                return getA();
            case T:
                return getT();
            case P:
                return getP();
            case PA:
                return getPA();
            case X:
                return getX();
            case U:
            case N:
                return new ContentConditionBean[0];
            default:
                throw new SwitchException("state =" + state);
        }
    }

    private ContentConditionBean[] getA() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        return array;
    }

    private ContentConditionBean[] getT() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }

    private ContentConditionBean[] getP() {
        int includingSize = includingList.size();
        int excludingSize = excludingList.size();
        ContentConditionBean[] array = new ContentConditionBean[includingSize + excludingSize];
        for (int i = 0; i < includingSize; i++) {
            array[i] = new ContentConditionBean(includingList.get(i));
        }
        for (int i = 0; i < excludingSize; i++) {
            array[i + includingSize] = new ContentConditionBean(excludingList.get(i));
        }
        return array;
    }

    private ContentConditionBean[] getPA() {
        int excludingSize = excludingList.size();
        if (excludingSize > 0) {
            return getP();
        }
        int includingSize = includingList.size();
        ContentConditionBean[] array = new ContentConditionBean[1 + includingSize];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        for (int i = 0; i < includingSize; i++) {
            array[i + 1] = new ContentConditionBean(includingList.get(i));
        }
        return array;
    }

    private ContentConditionBean[] getX() {
        ContentConditionBean[] array = new ContentConditionBean[2];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        array[1] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }

}
