/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import java.io.IOException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;


/**
 *
 * @author Vincent Calame
 */
public class ConditionsXMLStorage {

    private ConditionsXMLStorage() {
    }

    public static void appendConditionGroup(XMLWriter xmlWriter, ConditionGroup conditionGroup) throws IOException {
        xmlWriter.startOpenTag("condition-group");
        xmlWriter.addAttribute("mode", Conditions.modeToString(conditionGroup.getMode()));
        xmlWriter.endOpenTag();
        ContentConditionBean[] array = conditionGroup.getContentConditionArray();
        for (int i = 0; i < array.length; i++) {
            ContentConditionBean contentCondition = array[i];
            xmlWriter.startOpenTag("condition");
            xmlWriter.addAttribute("type", Conditions.typeToString(contentCondition.getConditionType()));
            xmlWriter.endOpenTag(false);
            xmlWriter.escape(contentCondition.getText());
            xmlWriter.closeTag("condition", false, false);
        }
        xmlWriter.closeTag("condition-group");
    }

    public static ConditionGroup getConditionGroup(Element element_xml) {
        String mode_strg = element_xml.getAttribute("mode");
        short mode = (mode_strg.equals("and")) ? ConditionGroup.MODE_AND : ConditionGroup.MODE_OR;
        String q = element_xml.getAttribute("q");
        if (q.length() == 0) {
            q = element_xml.getAttribute("raw");
        }
        if (q.length() > 0) {
            return Conditions.parseConditionGroup(q, mode);
        }
        ConditionGroup conditionGroup = new ConditionGroup(mode);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("condition")) {
                    ContentConditionBean ccb = getContentConditionBean(el);
                    if (ccb != null) {
                        conditionGroup.addContentCondition(ccb);
                    }
                }
            }
        }
        return conditionGroup;
    }

    private static ContentConditionBean getContentConditionBean(Element element_xml) {
        String text = XMLUtils.getData(element_xml);
        String type = element_xml.getAttribute("type");
        short conditionType = 0;
        if (type.length() == 0) {
            conditionType = Conditions.CONTIENT;
        } else {
            try {
                conditionType = Conditions.typeToShort(type);
            } catch (IllegalArgumentException iae) {
                return null;
            }
        }
        return new ContentConditionBean(conditionType, text);

    }

}
