/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class ContentConditionBean implements ContentCondition, Serializable {

    private short conditionType = Conditions.EST_NON_VIDE;
    private String text = "";

    public ContentConditionBean() {
    }

    /**
     *
     *@throws IllegalArgumentException si text est nul
     */
    public ContentConditionBean(short conditionType, String text) {
        setConditionType(conditionType);
        setText(text);
    }

    public ContentConditionBean(ContentConditionBean contentConditionBean) {
        this.conditionType = contentConditionBean.conditionType;
        this.text = contentConditionBean.text;
    }

    public short getConditionType() {
        return conditionType;
    }

    public void setConditionType(short conditionType) {
        this.conditionType = conditionType;
    }

    public String getText() {
        return text;
    }

    /**
     *
     *@throws IllegalArgumentException si text est nul
     */
    public void setText(String text) {
        if (text == null) {
            throw new IllegalArgumentException("text is null");
        }
        this.text = text;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof ContentConditionBean)) {
            return false;
        }
        ContentConditionBean other = (ContentConditionBean) obj;
        return ((other.conditionType == this.conditionType) && (other.text.equals(this.text)));
    }

    @Override
    public int hashCode() {
        return conditionType + text.hashCode();
    }

}
