/* UtilLib - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conf;

import net.mapeadores.util.xml.DOMReader;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ConfDOMReader {

    private ConfDOMErrorHandler errorHandler;
    private Conf conf;

    public ConfDOMReader(Conf conf, ConfDOMErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.conf = conf;
    }

    public void readConf(Element element_xml) {
        DOMReader.readChildren(element_xml, new ConfHandler());
    }


    private class ConfHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("param")) {
                String name = element.getAttribute("name");
                if (name.length() == 0) {
                    errorHandler.missingParamNameAttributeError();
                    return;
                }
                String value = element.getAttribute("value");
                short result = conf.set(name, value);
                if (result == Conf.UNKNOWNKEY_RESULT) {
                    errorHandler.unknownKeyError(name);
                } else if (result == Conf.BADVALUE_RESULT) {
                    errorHandler.badValueError(name, value);
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

    }

}
