/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.format;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.primitives.PrimUtils;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public abstract class FormatDef {

    public final static int MAX_POSITION = 999999999;
    public final static short NO_CAST = 0;
    public final static short INTEGER_CAST = 1;
    public final static short DECIMAL_CAST = 2;
    public final static short DATE_CAST = 3;
    private short castType = -9;
    private int limit = -9;
    private int position = -9;
    private String separator = null;
    private boolean fillByLast = false;
    private String defaultValue = null;
    private List<String> formatPatternList = new ArrayList<String>();
    private Map<Number, String> separatorMap = new HashMap<Number, String>();
    private Calcul calcul;
    private boolean convertEmptyToNull = false;

    public FormatDef() {
    }

    public short getCastType() {
        if (castType == -9) {
            return NO_CAST;
        }
        return castType;
    }

    public void setCastType(short castType) {
        this.castType = castType;
    }

    public String getFormatPattern(int i) {
        int size = formatPatternList.size();
        if (size == 0) {
            return null;
        }
        if (i >= size) {
            i = (size - 1);
        }
        return getInternalFormatPattern(i);
    }

    private String getInternalFormatPattern(int i) {
        String fP = formatPatternList.get(i);
        if (fP != null) {
            return fP;
        }
        if (i == 0) {
            return null;
        }
        return getInternalFormatPattern(i - 1);
    }

    public void addFormatPattern(String fP) {
        if (fP != null) {
            fP = fP.trim();
            if (fP.length() == 0) {
                fP = null;
            }
        }
        if ((fP == null) && (formatPatternList.isEmpty())) {
            return;
        }
        formatPatternList.add(fP);
    }

    public String formatPatternToString() {
        int size = formatPatternList.size();
        if (size == 0) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < size; i++) {
            String fp = formatPatternList.get(i);
            if (i > 0) {
                buf.append("||");
            }
            if (fp == null) {
                buf.append(' ');
            } else {
                buf.append(fp);
            }
        }
        return buf.toString();
    }

    public static short castTypeToShort(String s) {
        if (s.equals("no")) {
            return NO_CAST;
        }
        if ((s.equals("int")) || (s.equals("integer"))) {
            return INTEGER_CAST;
        }
        if ((s.equals("double")) || (s.equals("decimal")) || (s.equals("float"))) {
            return DECIMAL_CAST;
        }
        if (s.equals("date")) {
            return DATE_CAST;
        }
        throw new IllegalArgumentException("Unknown cast type : " + s);
    }

    public static String castTypeToString(short castType) {
        if (castType == NO_CAST) {
            return "no";
        }
        if (castType == INTEGER_CAST) {
            return "integer";
        }
        if (castType == DECIMAL_CAST) {
            return "decimal";
        }
        if (castType == DATE_CAST) {
            return "date";
        }
        throw new IllegalArgumentException("Unknown cast type : " + castType);
    }

    public int getLimit() {
        if (limit == -9) {
            return -1;
        }
        return limit;
    }

    public void setLimit(int limit) {
        if (limit < 0) {
            this.limit = -1;
        } else {
            this.limit = limit;
        }
    }

    public int getPosition() {
        if (position == -9) {
            return -1;
        }
        return position;
    }

    public void setPosition(int pos) {
        if (pos < 0) {
            this.position = -1;
        } else {
            this.position = pos;
            if (pos == MAX_POSITION) {
                fillByLast = true;
            }
        }
    }

    public boolean fillByLast() {
        return fillByLast;
    }

    public void setFillByLast(boolean fillByLast) {
        this.fillByLast = fillByLast;
    }

    public String getInternalSeparator(int sourceIndex) {
        String s = separatorMap.get(new Integer(sourceIndex));
        if (s == null) {
            return getSeparator();
        }
        return s;
    }

    public String getSourceSeparator(int sourceIndex1, int sourceIndex2) {
        String s = getFromLong(sourceIndex1, sourceIndex2);
        if (s == null) {
            return getSeparator();
        }
        return s;
    }

    private String getFromLong(int sourceIndex1, int sourceIndex2) {
        Long l = new Long(PrimUtils.toLong(sourceIndex1, sourceIndex2));
        String s = separatorMap.get(l);
        if (s != null) {
            return s;
        }
        if (sourceIndex2 > (sourceIndex1 + 1)) {
            return getFromLong(sourceIndex1, sourceIndex2 - 1);
        }
        return null;
    }

    public String getSeparator() {
        if (separator == null) {
            return ";";
        }
        return separator;
    }

    public void setSeparator(String separator) {
        this.separator = separator;
    }

    public void setInternalSeparator(int sourceIndex, String separator) {
        separatorMap.put(new Integer(sourceIndex), separator);
    }

    public void setSourceSeparator(int sourceIndex1, int sourceIndex2, String separator) {
        long l = PrimUtils.toLong(sourceIndex1, sourceIndex2);
        separatorMap.put(new Long(l), separator);
    }

    public boolean isLineSeparator() {
        if (separator == null) {
            return false;
        }
        return separator.equals("\n");
    }

    public String paramToString() {
        ParamStringBuilder stringBuilder = new ParamStringBuilder();
        String result = stringBuilder.toString();
        if (result.length() == 0) {
            return null;
        }
        return result;
    }

    public abstract void appendCustomParams(Appendable buf, boolean stillParams) throws IOException;


    private class ParamStringBuilder {

        StringBuilder buf;
        boolean suite = false;

        ParamStringBuilder() {
            buf = new StringBuilder();
            initCast();
            initPosition();
            initLast();
            initLimit();
            initSeparator();
            initDefaultValue();
            initconvertEmptyToNull();
            for (Map.Entry<Number, String> entry : separatorMap.entrySet()) {
                Number obj = entry.getKey();
                String value = entry.getValue();
                if (obj instanceof Integer) {
                    appendInternalSeparator((Integer) obj, value);
                } else if (obj instanceof Long) {
                    appendSourceSeparator((Long) obj, value);
                }
            }
            try {
                appendCustomParams(buf, suite);
            } catch (IOException ioe) {
            }
        }

        public String toString() {
            return buf.toString();
        }

        private void initDefaultValue() {
            if (defaultValue != null) {
                testSuite();
                buf.append("defval=");
                appendString(defaultValue);
            }
        }

        private void initCast() {
            if (castType != -9) {
                testSuite();
                buf.append("cast=");
                buf.append(castTypeToString(castType));
            }
        }

        private void initLimit() {
            if (limit != -9) {
                testSuite();
                buf.append("limit=");
                buf.append(limit);
            }
        }

        private void initPosition() {
            if (position != -9) {
                testSuite();
                buf.append("pos=");
                if (position == MAX_POSITION) {
                    buf.append("last");
                } else {
                    buf.append(position + 1);
                }
            }
        }

        private void initLast() {
            if (fillByLast) {
                if (position != MAX_POSITION) {
                    testSuite();
                    buf.append("last");
                }
            }
        }

        private void initconvertEmptyToNull() {
            if (convertEmptyToNull) {
                testSuite();
                buf.append("null");
            }
        }

        private void initSeparator() {
            if (separator != null) {
                testSuite();
                buf.append("sep=");
                if (separator.equals("\n")) {
                    appendString("NL");
                } else {
                    appendString(separator);
                }
            }
        }

        private void appendInternalSeparator(Integer itg, String value) {
            testSuite();
            int pos = itg.intValue() + 1;
            buf.append("sep");
            buf.append(pos);
            buf.append('=');
            appendString(value);
        }

        private void appendSourceSeparator(Long l, String value) {
            testSuite();
            int pos1 = PrimUtils.getInt1(l.longValue()) + 1;
            int pos2 = PrimUtils.getInt2(l.longValue()) + 1;
            buf.append("sep");
            buf.append(pos1);
            buf.append('_');
            buf.append(pos2);
            buf.append('=');
            appendString(value);
        }

        private void appendString(String s) {
            if (s.length() == 0) {
                buf.append("\"\"");
            } else if (StringUtils.isValidAsciiString(s)) {
                buf.append(s);
            } else {
                buf.append('\"');
                StringUtils.escapeDoubleQuote(s, buf);
                buf.append('\"');
            }
        }

        private void testSuite() {
            if (suite) {
                buf.append(',');
            } else {
                suite = true;
            }
        }

    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultString(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public Calcul getCalcul() {
        return calcul;
    }

    public void setCalcul(Calcul calcul) {
        this.calcul = calcul;
    }

    public boolean isConvertEmptyToNull() {
        return convertEmptyToNull;
    }

    public void setConvertEmptyToNull(boolean convertEmptyToNull) {
        this.convertEmptyToNull = convertEmptyToNull;
    }

}
