/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import net.mapeadores.util.langinteger.LangInteger;
import java.io.PrintWriter;


/**
 * Classe obsolète, ne plus utiliser.
 *
 * @author Vincent Calame
 */
public class HTMLUtils {

    public final static short ENDFORM_DOUBLE = 2;
    public final static short ENDFORM_P = 1;
    public final static short ENDFORM_INLINE = 0;

    private HTMLUtils() {
    }

    /**
     * Écrit la chaîne <var>s</var> en remplaçant les caractères &amp;, &quot;, &lt; et &gt; par leur forme HTML et en remplaçant les caractères blancs par des espaces.
     */
    static public void printHTML(PrintWriter pw, String s) {
        if (s == null) {
            pw.print("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            if (carac < 32) {
                pw.print(' ');
            } else {
                switch (carac) {
                    case '&':
                        pw.print("&amp;");
                        break;
                    case '"':
                        pw.print("&quot;");
                        break;
                    case '<':
                        pw.print("&lt;");
                        break;
                    case '>':
                        pw.print("&gt;");
                        break;
                    default:
                        pw.print(carac);
                }
            }
        }
    }

    /**
     * Écrit la chaîne <var>s</var> en remplaçant les caractères spéciaux et en conservant les tabulations et les sauts de ligne.
     */
    static public void printPreformattedHTML(PrintWriter pw, String s) {
        if (s == null) {
            pw.print("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '&':
                    pw.print("&amp;");
                    break;
                case '"':
                    pw.print("&quot;");
                    break;
                case '<':
                    pw.print("&lt;");
                    break;
                case '>':
                    pw.print("&gt;");
                    break;
                default:
                    pw.print(carac);
            }
        }
    }

    static public String escapeHTML(String s) {
        if (s == null) {
            return s;
        }
        StringBuffer buf = new StringBuffer();
        appendHTML(buf, s);
        return buf.toString();
    }

    static public void appendHTML(StringBuffer buf, String s) {
        if (s == null) {
            buf.append("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            if (carac < 32) {
                buf.append(' ');
            } else {
                switch (carac) {
                    case '&':
                        buf.append("&amp;");
                        break;
                    case '"':
                        buf.append("&quot;");
                        break;
                    case '<':
                        buf.append("&lt;");
                        break;
                    case '>':
                        buf.append("&gt;");
                        break;
                    default:
                        buf.append(carac);
                }
            }
        }
    }

    static public void appendPreformattedHTML(StringBuffer buf, String s) {
        if (s == null) {
            buf.append("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '&':
                    buf.append("&amp;");
                    break;
                case '"':
                    buf.append("&quot;");
                    break;
                case '<':
                    buf.append("&lt;");
                    break;
                case '>':
                    buf.append("&gt;");
                    break;
                default:
                    buf.append(carac);
            }
        }
    }

    static public void printLabelledRow(PrintWriter pw, String label, String message, boolean escapeMessage) {
        pw.print("<tr>");
        pw.print("<td valign=\"top\">");
        printHTML(pw, label);
        pw.print("</td>");
        pw.print("<td valign=\"top\">");
        pw.print("&nbsp;:&nbsp;");
        pw.print("</td>");
        pw.print("<td valign=\"top\">");
        if (message == null) {
            pw.print("<em>NULL</em>");
        } else if (escapeMessage) {
            printHTML(pw, message);
        } else {
            pw.print(message);
        }
        pw.println("</td>");
        pw.println("</tr>");
    }

    /**
     * Le champ résultant a comme nom « fieldName/LangInteger.toString(langInteger) ».
     * Le séparateur utilisé est la barre oblique '/' qui normalement n'est présent ni dans fieldName ni dans LangInteger.toString(langInteger)
     * Ce qui permet de retrouver facilement les valeurs de chaque.
     */
    static public void printLangFieldRow(PrintWriter pw, int langInteger, String fieldName, String text, int cols, int rows) {
        pw.print("<tr>");
        pw.print("<td");
        if (rows > 1) {
            pw.print(" valign=\"top\"");
        }
        pw.print(">");
        LangInteger.appendString(pw, langInteger);
        pw.print("</td>");
        pw.print("<td");
        if (rows > 1) {
            pw.print(" valign=\"top\"");
        }
        pw.print(">");
        pw.print("&nbsp;:&nbsp;");
        pw.print("</td>");
        pw.print("<td>");
        if (rows > 1) {
            pw.print("<textarea name=\"");
            pw.print(fieldName);
            pw.print('/');
            LangInteger.appendString(pw, langInteger);
            pw.print("\" cols=\"");
            pw.print(String.valueOf(cols));
            pw.print("\" rows=\"");
            pw.print(String.valueOf(rows));
            pw.print("\">");
            printPreformattedHTML(pw, text);
            pw.print("</textarea>");
        } else {
            pw.print("<input name=\"");
            pw.print(fieldName);
            pw.print('/');
            LangInteger.appendString(pw, langInteger);
            pw.print("\" value=\"");
            printHTML(pw, text);
            pw.print("\" size=\"");
            pw.print(String.valueOf(cols));
            pw.print("\">");
        }
        pw.print("</td>");
        pw.print("<tr>");
    }

    static public void printLangValueRow(PrintWriter pw, int langInteger, String value, boolean pre) {
        pw.print("<tr>");
        pw.print("<td");
        if (pre) {
            pw.print(" valign=\"top\"");
        }
        pw.print(">");
        LangInteger.appendString(pw, langInteger);
        pw.print("</td>");
        pw.print("<td");
        if (pre) {
            pw.print(" valign=\"top\"");
        }
        pw.print(">");
        pw.print("&nbsp;:&nbsp;");
        pw.print("</td>");
        pw.print("<td>");

        if (pre) {
            pw.print("<pre>");
            printPreformattedHTML(pw, value);
            pw.print("</pre>");
        } else {
            pw.print("<kbd>");
            printHTML(pw, value);
            pw.print("</kbd>");
        }

        pw.print("</td>");
        pw.print("<tr>");
    }

    static public void printHiddenInput(PrintWriter pw, String name, String value) {
        pw.print("<input type=\"hidden\" name=\"");
        pw.print(name);
        pw.print("\" value=\"");
        printHTML(pw, value);
        pw.print("\">");
    }

    static public void printLabelledRow(PrintWriter pw, String label, int value) {
        printLabelledRow(pw, label, String.valueOf(value), false);
    }

    static public void printTag(PrintWriter pw, String text, String tagName) {
        pw.print("<");
        pw.print(tagName);
        pw.print(">");
        printHTML(pw, text);
        pw.print("</");
        pw.print(tagName);
        pw.print(">");
    }

    static public void printBodyStart(PrintWriter pw, boolean dtd, String title, String cssUrl) {
        printBodyStart(pw, dtd, title, cssUrl, null);
    }

    static public void printBodyStart(PrintWriter pw, boolean dtd, String title, String cssUrl, String javascriptURL) {
        if (dtd) {
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
        }
        pw.print("<html><head><title>");
        printHTML(pw, title);
        pw.print("</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        if (javascriptURL != null) {
            pw.print("<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
        }
        if (cssUrl != null) {
            pw.print("<link href=\"");
            pw.print(cssUrl);
            pw.print("\" rel=\"stylesheet\" type=\"text/css\">");
        }
        if (javascriptURL != null) {
            pw.print("<script src=\"");
            pw.print(javascriptURL);
            pw.print("\" defer></script>");
        }
        pw.println("</head><body>");
    }

    static public void printBodyEnd(PrintWriter pw) {
        pw.print("</body></html>");
    }

    static public void printFormStart(PrintWriter pw, String action, String method, String cmd) {
        printFormStart(pw, action, method, cmd, null, null, null);
    }

    static public void printMultiPartFormStart(PrintWriter pw, String action, String method, String cmd) {
        printFormStart(pw, action, method, cmd, null, null, "multipart/form-data");
    }

    static public void printFormStart(PrintWriter pw, String action, String method, String cmd, String target, String name) {
        printFormStart(pw, action, method, cmd, target, name, null);
    }

    static public void printFormStart(PrintWriter pw, String action, String method, String cmd, String target, String name, String encType) {
        pw.print("<form action=\"");
        pw.print(action);
        pw.print("\" method=\"");
        pw.print(method);
        if (encType != null) {
            pw.print("\" enctype=\"");
            pw.print(encType);
        }
        if (target != null) {
            pw.print("\" target=\"");
            pw.print(target);
        }
        if (name != null) {
            pw.print("\" name=\"");
            pw.print(name);
        }
        pw.print("\">");
        if (cmd != null) {
            pw.print("<input type=\"hidden\" name=\"cmd\" value=\"");
            pw.print(cmd);
            pw.print("\">");
        }
    }

    static public void printFormEnd(PrintWriter pw, String submitValue) {
        printFormEnd(pw, submitValue, ENDFORM_P);
    }

    static public void printFormEnd(PrintWriter pw, String submitValue, short type) {
        if (type == ENDFORM_DOUBLE) {
            pw.print("<p>&nbsp;</p>");
            pw.print("<table width=\"100%\"><col span=\"2\">");
            pw.print("<tr><td>");
            pw.print("<input type=\"submit\" value=\"");
            printHTML(pw, submitValue);
            pw.println("\">");
            pw.print("</td><td style=\"text-align: right;\">");
            pw.print("<input type=\"submit\" value=\"");
            printHTML(pw, submitValue);
            pw.println("\">");
            pw.print("</td></tr>");
            pw.print("</table>");
        } else if (type == ENDFORM_P) {
            pw.print("<p><input type=\"submit\" value=\"");
            printHTML(pw, submitValue);
            pw.println("\"></p>");
        } else {
            pw.print("<input type=\"submit\" value=\"");
            printHTML(pw, submitValue);
            pw.println("\">");
        }
        pw.println("</form>");
    }

    static public void printFormEnd(PrintWriter pw) {
        pw.println("</form>");
    }

    static public void printUniqueButtonForm(PrintWriter pw, String action, String method, String cmd, String submitValue) {
        printFormStart(pw, action, method, cmd, null, null);
        printFormEnd(pw, submitValue, ENDFORM_INLINE);
    }

}
