/* UtilLib - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.io.docstream;

import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import net.mapeadores.util.io.DocStream;


/**
 *
 * @author Vincent Calame
 */
public class ClassResourceDocStream implements DocStream {

    /*
     * La date de dernière modification des ressources internes
     * est fixée à la date d'initialisation de cette classe
     */
    private static long lastModified;
    /*
     * Suppression des millisecondes.
     */
    static {
        long l = System.currentTimeMillis();
        l = l / 1000;
        lastModified = l * 1000;
    }
    private Class referenceClass;
    private String resourcePath;
    private String mimeType = "application/octet-stream";
    private String charset = null;
    private int length = -1;

    private ClassResourceDocStream(Class referenceClass, String resourcePath) {
        this.referenceClass = referenceClass;
        this.resourcePath = resourcePath;
    }

    public InputStream getInputStream() throws IOException {
        return referenceClass.getResourceAsStream(resourcePath);
    }

    public int getLength() {
        return length;
    }

    public void setLength(int length) {
        this.length = length;
    }

    public String getMimeType() {
        return mimeType;
    }

    public void setMimeType(String mimeType) {
        if (mimeType == null) {
            throw new IllegalArgumentException("mimeType cannot be null");
        }
        this.mimeType = mimeType;
    }

    public String getCharset() {
        return charset;
    }

    public void setCharset(String charset) {
        this.charset = charset;
    }

    public long getLastModified() {
        return lastModified;
    }

    public static ClassResourceDocStream newInstance(Class referenceClass, String resourcePath) {
        URL url = referenceClass.getResource(resourcePath);
        if (url == null) {
            return null;
        }
        return new ClassResourceDocStream(referenceClass, resourcePath);
    }
}
