/* UtilLib - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.langinteger;

import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.TreeMap;
import java.util.HashMap;
import java.util.Set;
import java.util.Collections;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;


/**
 *
 * @author Vincent Calame
 */
public class CountryUtils {

    private static IntHashMap geolocMap;
    private static TreeMap<String, Integer> iso3166CodeMap;
    private static HashMap<String, Integer> iso3166ConversionMap;
    private static Set<String> iso3166CodeSet;

    static {
        iso3166CodeMap = new TreeMap<String, Integer>();
        try {
            String ligne = null;
            InputStream is = CountryUtils.class.getResourceAsStream("iso3166.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            while ((ligne = bufReader.readLine()) != null) {
                ligne = ligne.trim();
                if (ligne.length() > 0) {
                    int code = LangInteger.parse("zxx-" + ligne, true);
                    iso3166CodeMap.put(ligne, new Integer(code));
                }
            }
            InputStream is2 = CountryUtils.class.getResourceAsStream("iso3166-complement.csv");
            BufferedReader bufReader2 = new BufferedReader(new InputStreamReader(is2, "UTF-8"));
            while ((ligne = bufReader2.readLine()) != null) {
                int idx = ligne.indexOf(',');
                if (idx != -1) {
                    ligne = ligne.substring(0, idx);
                }
                ligne = ligne.trim();
                if (ligne.length() > 0) {
                    int code = LangInteger.parse("zxx-" + ligne, true);
                    iso3166CodeMap.put(ligne, new Integer(code));
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        iso3166CodeSet = Collections.unmodifiableSet(iso3166CodeMap.keySet());
    }

    static {
        String ligne = null;
        iso3166ConversionMap = new HashMap<String, Integer>();
        try {
            InputStream is = CountryUtils.class.getResourceAsStream("iso3166-conversion.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            while ((ligne = bufReader.readLine()) != null) {
                int idx = ligne.indexOf(',');
                if (idx > 0) {
                    String code1 = ligne.substring(0, idx).trim();
                    String code2 = ligne.substring(idx + 1).trim();
                    if (code1.length() > 0) {
                        int code = LangInteger.parse("zxx-" + code2, true);
                        iso3166ConversionMap.put(code1, new Integer(code));
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }

    static {
        geolocMap = new IntHashMap();
        try {
            InputStream is = CountryUtils.class.getResourceAsStream("geoloc.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx1 = ligne.indexOf(',');
                    int idx2 = ligne.indexOf(',', idx1 + 1);
                    if ((idx1 != -1) && (idx2 != -1)) {
                        String countryCode = ligne.substring(0, idx1).trim();
                        int countryInteger = parseCountryCode(countryCode);
                        DegreDecimal latitude = new DegreDecimal(StringUtils.parseDecimal(ligne.substring(idx1 + 1, idx2).trim()));
                        DegreDecimal longitude = new DegreDecimal(StringUtils.parseDecimal(ligne.substring(idx2 + 1).trim()));
                        geolocMap.put(countryInteger, new GeoPoint(latitude, longitude));
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        } catch (NumberFormatException nfe) {
            throw new InternalResourceException(nfe);
        }
    }

    private CountryUtils() {
    }

    public static GeoPoint getGeolocalisation(int countryCode) {
        return (GeoPoint) geolocMap.get(countryCode);
    }

    /**
     * Classé dans l'ordre alphabétique.
     */
    public static Set<String> getIsoCodeSet() {
        return iso3166CodeSet;
    }


    public static class GeoPoint {

        private DegreDecimal latitude;
        private DegreDecimal longitude;

        private GeoPoint(DegreDecimal latitude, DegreDecimal longitude) {
            this.latitude = latitude;
            this.longitude = longitude;
        }

        public DegreDecimal getLatitude() {
            return latitude;
        }

        public DegreDecimal getLongitude() {
            return longitude;
        }

    }

    public static int parseCountryCode(String countryCode) throws LangIntegerException {
        countryCode = countryCode.toUpperCase();
        Integer itg = (Integer) iso3166CodeMap.get(countryCode);
        if (itg == null) {
            itg = (Integer) iso3166ConversionMap.get(countryCode);
        }
        if (itg != null) {
            return itg.intValue();
        } else {
            throw new LangIntegerException(countryCode);
        }
    }

}
