/* UtilLib - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.langinteger;

import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.HashMap;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;


/**
 *
 * @author Vincent Calame
 */
public class LangIntegerLocalisation {

    private static final Map<Locale, LocaleItem> localeMap = new HashMap<Locale, LocaleItem>();
    private static IntHashMap langvoMap;
    /*
     *Initialisation des langues pour lequel il existe un libellé pour tous les codes,
     * pour les autres langues on utilisera les libellés fournis par la machine virtuelle pour les codes à deux lettres et l'anglais pour les codes à trois lettres
     */

    static {
        init("en");
        init("fr");
    }

    static {
        langvoMap = new IntHashMap();
        try {
            InputStream is = LangIntegerLocalisation.class.getResourceAsStream("langvo.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        int langInteger = LangInteger.parse(ligne.substring(0, idx));
                        langvoMap.put(langInteger, ligne.substring(idx + 1));
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }

    private LangIntegerLocalisation() {
    }

    public static String getVOLibelle(int langInteger) {
        String lib = (String) langvoMap.get(langInteger);
        if (lib == null) {
            lib = getLangLibelle(langInteger, Locale.getDefault());
        }
        return lib;
    }

    public static String getLangLibelle(int langInteger, Locale locale) {
        return getLangLibelle(langInteger, locale, true);
    }

    public static String getLangLibelle(int langInteger, Locale locale, boolean onlyMain) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLangLibelle(langInteger, onlyMain);
    }

    public static String getCountryLibelle(int langInteger, Locale locale) {
        return getCountryLibelle(langInteger, locale, true);
    }

    public static String getCountryLibelle(int langInteger, Locale locale, boolean onlyMain) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getCountryLibelle(langInteger, onlyMain);
    }

    public static SortedMap<String, String> getCountryMap(Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getCountrySortedMap();
    }

    public static SortedMap<String, String> getLanguageMap(Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLanguageSortedMap();
    }

    public static Integer getLangInteger(String lib, Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLangInteger(lib);
    }

    public static String getCardinalDirection(short direction, boolean onlyInitiale, Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getCardinalDirection(direction, onlyInitiale);
    }

    public static LocaleItem getLocaleItem(Locale locale) {
        LocaleItem localeItem = localeMap.get(locale);
        if (localeItem != null) {
            return localeItem;
        }
        synchronized (localeMap) {
            localeItem = localeMap.get(locale);
            if (localeItem != null) {
                return localeItem;
            }
            String language = locale.getLanguage();
            if (language.length() == 0) {
                Locale defaultLocale = new Locale("en");
                localeItem = localeMap.get(defaultLocale);
                if (localeItem == null) {
                    localeItem = new LocaleItem(defaultLocale);
                    localeMap.put(defaultLocale, localeItem);
                }
                return localeItem;
            } else {
                Locale otherLocale = new Locale(language);
                localeItem = localeMap.get(otherLocale);
                if (localeItem == null) {
                    localeItem = new LocaleItem(locale);
                    localeMap.put(locale, localeItem);
                }
                return localeItem;
            }
        }
    }

    private static LocaleItem init(String langString) {
        LocaleItem localeItem = new LocaleItem(langString);
        localeMap.put(localeItem.getLocale(), localeItem);
        return localeItem;
    }

}
