/* UtilLib - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.langinteger;

import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.text.StringUtils;
import java.util.Locale;


/**
 *
 * @author Vincent Calame
 */
public class LangIntegerUtils {

    private LangIntegerUtils() {
    }

    public static String toString(int[] langIntegerArray, char separator) {
        int length = langIntegerArray.length;
        StringBuilder buf = new StringBuilder(length * 6);
        for (int i = 0; i < length; i++) {
            if (i > 0) {
                buf.append(separator);
            }
            LangInteger.appendString(buf, langIntegerArray[i]);
        }
        return buf.toString();
    }

    public static int[] toCleanLangIntegerArray(String s) {
        String[] tokens = StringUtils.getTechnicalTokens(s);
        int p = 0;
        int maxLength = tokens.length;
        int[] result = new int[maxLength];
        for (int i = 0; i < maxLength; i++) {
            try {
                int intlang = LangInteger.parse(tokens[i], false);
                if (!ArrayUtils.containsBetweenIndexes(result, intlang, 0, p)) {
                    result[p] = intlang;
                    p++;
                }
            } catch (LangIntegerException mce) {
            }
        }
        return ArrayUtils.checkIntArray(result, p);
    }

    /**
     * supprime les valeurs incorrectes, retourne null si null.
     */
    static public int[] toLangIntegerArray(String[] stringArray) {
        if (stringArray == null) {
            return null;
        }
        int length = stringArray.length;
        int[] langIntegerArray = new int[length];
        int p = 0;
        for (int i = 0; i < length; i++) {
            try {
                langIntegerArray[p] = LangInteger.parse(stringArray[i]);
                p++;
            } catch (LangIntegerException lie) {
            }
        }
        if (p == 0) {
            return new int[0];
        }
        if (p < length) {
            int[] temp = new int[p];
            System.arraycopy(langIntegerArray, 0, temp, 0, p);
            langIntegerArray = temp;
        }
        return langIntegerArray;
    }

    /**
     * Retourne null si langIntegerArray est nul
     */
    static public String[] toStringArray(int[] langIntegerArray) {
        if (langIntegerArray == null) {
            return null;
        }
        int length = langIntegerArray.length;
        String[] stringArray = new String[length];
        for (int i = 0; i < length; i++) {
            stringArray[i] = LangInteger.toString(langIntegerArray[i]);
        }
        return stringArray;
    }

    /**
     * Retourne l'information de langue d'un nom de fichier, en partant du principe qu'elle est exprimée de la façon suivante :
     * {nom de base du fichier}_{langue}.{extension}. Par exemple, localisation_fr.xml
     *
     *@throws LangIntegerException s'il n'arrive pas à retrouve la langue.
     */
    static public int getLangIntegerInfo(String fileName) throws LangIntegerException {
        int souligneIndex = fileName.lastIndexOf('_');
        if (souligneIndex == -1) {
            throw new LangIntegerException();
        }
        int pointIndex = fileName.lastIndexOf('.');
        if (pointIndex == -1) {
            throw new LangIntegerException();
        }
        if (pointIndex < souligneIndex) {
            throw new LangIntegerException();
        }
        return LangInteger.parse(fileName, souligneIndex + 1, pointIndex - souligneIndex - 1, false);
    }

    static public int getNearestToLocale(int[] langIntegerArray) {
        return getNearestToLocale(langIntegerArray, Locale.getDefault());
    }

    static public int getNearestToLocale(int[] langIntegerArray, Locale locale) {
        int localeInteger = LangInteger.fromLocale(locale);
        int localeLangPart = LangInteger.getLangPart(localeInteger);
        int option1 = 0;
        int option2 = 0;
        for (int i = 0; i < langIntegerArray.length; i++) {
            int li = langIntegerArray[i];
            if (li == localeInteger) {
                return localeInteger;
            } else if (li == localeLangPart) {
                option1 = li;
            } else if (LangInteger.getLangPart(li) == localeLangPart) {
                option2 = li;
            }
        }
        if (option1 != 0) {
            return option1;
        }
        if (option2 != 0) {
            return option2;
        }
        return langIntegerArray[0];
    }

}
