/* UtilLib - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import net.mapeadores.util.text.StringUtils;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Locale;


/**
 *
 * @author Vincent Calame
 */
public class BundleLocalisation implements ResourceLocalisation {

    private static ConcurrentMap<CacheKey, BundleLocalisation> localisationMap = new ConcurrentHashMap<CacheKey, BundleLocalisation>();
    private ResourceBundle resourceBundle;

    public BundleLocalisation(ResourceBundle resourceBundle) {
        this.resourceBundle = resourceBundle;
    }

    public String toString(ResourceItem resourceItem) {
        return StringUtils.getMessage(resourceBundle, resourceItem.getResourceKey(), resourceItem.getValue());
    }

    public String toString(String resourceKey, Object value) {
        return StringUtils.getMessage(resourceBundle, resourceKey, value);
    }

    public boolean containsKey(String resourceKey) {
        try {
            resourceBundle.getString(resourceKey);
            return true;
        } catch (MissingResourceException mre) {
            return false;
        }
    }

    public ResourceBundle getResourceBundle() {
        return resourceBundle;
    }

    public static BundleLocalisation getBundleLocalisation(String baseName, Locale locale) {
        CacheKey cacheKey = new CacheKey(baseName, locale);
        BundleLocalisation resourceLocalisation = localisationMap.get(cacheKey);
        if (resourceLocalisation == null) {
            ResourceBundle servletsBundle = ResourceBundle.getBundle(baseName, locale);
            resourceLocalisation = new BundleLocalisation(servletsBundle);
            localisationMap.put(cacheKey, resourceLocalisation);
        }
        return resourceLocalisation;
    }


    private static final class CacheKey {

        private String name;
        private Locale locale;
        private int hashCodeCache;

        CacheKey(String baseName, Locale locale) {
            this.name = baseName;
            this.locale = locale;
            calculateHashCode();
        }

        String getName() {
            return name;
        }

        Locale getLocale() {
            return locale;
        }

        public boolean equals(Object other) {
            if (this == other) {
                return true;
            }
            try {
                final CacheKey otherEntry = (CacheKey) other;
                if (hashCodeCache != otherEntry.hashCodeCache) {
                    return false;
                }
                if (!name.equals(otherEntry.name)) {
                    return false;
                }
                if (!locale.equals(otherEntry.locale)) {
                    return false;
                }
                return true;
            } catch (NullPointerException e) {
            } catch (ClassCastException e) {
            }
            return false;
        }

        public int hashCode() {
            return hashCodeCache;
        }

        private void calculateHashCode() {
            hashCodeCache = name.hashCode() << 3;
            hashCodeCache ^= locale.hashCode();
        }

        public String toString() {
            String l = locale.toString();
            if (l.length() == 0) {
                if (locale.getVariant().length() != 0) {
                    l = "__" + locale.getVariant();
                } else {
                    l = "\"\"";
                }
            }
            return "CacheKey[" + name + ", lc=" + l + "]";
        }

    }

}
