/* UtilLib - Copyright (c) 2005 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.html.HTMLUtils;
import java.util.List;
import java.util.ArrayList;
import java.io.PrintWriter;
import java.util.logging.Logger;
import java.util.logging.LogRecord;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.Handler;


/**
 *
 * @author Vincent Calame
 */
public class BufferLogHandler extends Handler {

    List<LogRecord> logRecordList = new ArrayList<LogRecord>();
    Logger logger;

    public BufferLogHandler() {
        setFormatter(new InternalFormatter());
    }

    public void setLogger(Logger logger) {
        if (this.logger != null) {
            this.logger.removeHandler(this);
        }
        this.logger = logger;
        logger.addHandler(this);
    }

    public void close() throws SecurityException {
        logRecordList.clear();
        logger.removeHandler(this);
    }

    public void flush() {
    }

    public void publish(LogRecord logRecord) {
        Level current = this.getLevel();
        if (current != null) {
            Level level = logRecord.getLevel();
            if (level.intValue() >= current.intValue()) {
                logRecordList.add(logRecord);
            }
        } else {
            logRecordList.add(logRecord);
        }
    }

    public int bufferSize() {
        return logRecordList.size();
    }

    public Logger getLogger() {
        return logger;
    }

    public void print(PrintWriter printWriter, boolean escapeHTML) {
        Formatter formatter = getFormatter();
        if (formatter == null) {
            formatter = new InternalFormatter();
        }
        for (int i = 0; i < logRecordList.size(); i++) {
            LogRecord logRecord = (LogRecord) logRecordList.get(i);
            if (escapeHTML) {
                HTMLUtils.printPreformattedHTML(printWriter, formatter.format(logRecord));
            } else {
                printWriter.println(formatter.format(logRecord));
            }
        }
        logRecordList.clear();
    }


    class InternalFormatter extends Formatter {

        public String format(LogRecord logRecord) {
            StringBuilder buf = new StringBuilder();
            buf.append('[');
            buf.append(logRecord.getLevel().getLocalizedName());
            buf.append("] ");
            buf.append(logRecord.getMessage());
            Object[] objs = logRecord.getParameters();
            if (objs != null) {
                for (int i = 0; i < objs.length; i++) {
                    buf.append(" - ");
                    buf.append(objs[i].toString());
                }
            }
            buf.append('\n');
            return buf.toString();
        }

    }

}
