/* UtilLib - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.ResourceItem;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessage {

    public static final short ERROR_MESSAGE = 1;
    public static final short DONE_MESSAGE = 2;
    public static final short MULTIERROR_MESSAGE = 3;
    public static final short RESULTLOG_MESSAGE = 4;
    private short type;
    private ResourceItem keyResourceItem;
    private ProcessLog processLog;
    private List<ResourceItem> multiErrorList;

    private CommandMessage(short type, String key, Object value) {
        if (key == null) {
            throw new IllegalArgumentException("key is null");
        }
        this.type = type;
        keyResourceItem = new ResourceItem(key, value);
    }

    private CommandMessage(ResourceItem resourceItem, List<ResourceItem> multiErrorList) {
        this.type = MULTIERROR_MESSAGE;
        this.keyResourceItem = resourceItem;
        this.multiErrorList = multiErrorList;
    }

    private CommandMessage(short type, ResourceItem resourceItem) {
        if (resourceItem == null) {
            throw new IllegalArgumentException("keyResourceItem is null");
        }
        this.type = type;
        this.keyResourceItem = resourceItem;
    }

    private CommandMessage(ResourceItem titleKeyResourceItem, ProcessLog processLog) {
        this.type = RESULTLOG_MESSAGE;
        this.processLog = processLog;
        this.keyResourceItem = titleKeyResourceItem;
    }

    public short getType() {
        return type;
    }

    public boolean isErrorMessage() {
        return ((type == ERROR_MESSAGE) || (type == MULTIERROR_MESSAGE));
    }

    public ResourceItem getKeyResourceItem() {
        return keyResourceItem;
    }

    public List<ResourceItem> getMultiErrorList() {
        return multiErrorList;
    }

    public ProcessLog getProcessLog() {
        return processLog;
    }

    public static CommandMessage error(String key, Object value) {
        return new CommandMessage(ERROR_MESSAGE, key, value);
    }

    public static CommandMessage error(ResourceItem keyResourceItem) {
        return new CommandMessage(ERROR_MESSAGE, keyResourceItem);
    }

    /**
     * la liste doit contenir des instances de RessourceItem
     */
    public static CommandMessage multiError(ResourceItem titleKeyResourceItem, List<ResourceItem> ressourceItemList) {
        if (titleKeyResourceItem == null) {
            throw new IllegalArgumentException("titleKeyResourceItem is null");
        }
        if (ressourceItemList == null) {
            throw new IllegalArgumentException("errorMap is null");
        }
        return new CommandMessage(titleKeyResourceItem, ressourceItemList);
    }

    public static CommandMessage done(String key, Object value) {
        return new CommandMessage(DONE_MESSAGE, key, value);
    }

    public static CommandMessage done(ResourceItem keyResourceItem) {
        return new CommandMessage(DONE_MESSAGE, keyResourceItem);
    }

    public static CommandMessage log(ResourceItem titleKeyResourceItem, ProcessLog processLog) {
        if (titleKeyResourceItem == null) {
            throw new IllegalArgumentException("titleKeyResourceItem is null");
        }
        if (processLog == null) {
            throw new IllegalArgumentException("processLog is null");
        }
        return new CommandMessage(titleKeyResourceItem, processLog);
    }

    public static String typeToString(short type) {
        switch (type) {
            case ERROR_MESSAGE:
                return "error";
            case DONE_MESSAGE:
                return "message";
            case MULTIERROR_MESSAGE:
                return "multierror";
            case RESULTLOG_MESSAGE:
                return "resultlog";
            default:
                throw new IllegalArgumentException("unknown type = " + type);
        }
    }

}
