/* UtilLib - Copyright (c) 2005 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.List;
import java.util.Set;
import java.util.Collections;


/**
 *
 * @author Vincent Calame
 */
public class LoggingUtils {

    public static ProcessLog EMPTY_PROCESSLOG = new EmptyProcessLog();

    private LoggingUtils() {
    }

    public static void logWithThrowable(Logger logger, String key, Throwable throwable, Object parameter) {
        Object[] objs = null;
        if (parameter != null) {
            objs = new Object[2];
            objs[0] = parameter;
            objs[1] = throwable;
        } else {
            objs = new Object[1];
            objs[0] = throwable;
        }
        logger.log(Level.SEVERE, key, objs);
    }

    public static void logWithThrowable(Logger logger, String key, Throwable throwable, Object parameter1, Object parameter2) {
        Object[] objs = new Object[3];
        objs[0] = parameter1;
        objs[1] = parameter2;
        objs[2] = throwable;
        logger.log(Level.SEVERE, key, objs);
    }

    public static void logWithThrowable(Logger logger, Level level, String key, Throwable throwable, Object[] parameters) {
        Object[] objs = new Object[parameters.length + 1];
        System.arraycopy(parameters, 0, objs, 0, parameters.length);
        objs[parameters.length] = throwable;
        logger.log(level, key, objs);
    }


    private static class EmptyProcessLog implements ProcessLog {

        public boolean hasLog() {
            return false;
        }

        public boolean hasLog(String processSourceURI) {
            return false;
        }

        public List<ProcessLogItem> getProcessLogItemList(String processSourceURI) {
            return Collections.emptyList();
        }

        public Set<String> getProcessSourceURISet() {
            return Collections.emptySet();
        }

    }

}
