/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.smtp;

import java.util.List;
import java.util.Properties;


/**
 *
 * @author Vincent Calame
 */
public class SmtpUtils {

    private SmtpUtils() {
    }

    public static SmtpParameters buildFromProperties(Properties properties, List<String> errorList) {
        DefaultSmtpParameters smtpParameters = new DefaultSmtpParameters();
        String authentificationString = properties.getProperty("authentification");
        short authentificationType = 0;
        if (authentificationString == null) {
            errorList.add("Missing key: authentification");
        } else {
            try {
                authentificationType = authentificationTypeToShort(authentificationString);
                smtpParameters.internalSetAuthentificationType(authentificationType);
            } catch (IllegalArgumentException iae) {
                errorList.add("Wrong value: authentification / " + authentificationString);
            }
        }
        if (authentificationType == SmtpParameters.AUTHENTIFICATION_SSL) {
            smtpParameters.internalSetPort(465);
        }
        String host = properties.getProperty("host");
        if (host == null) {
            errorList.add("Missing key: host");
        } else {
            smtpParameters.internalSetHost(host);
        }
        String portString = properties.getProperty("port");
        if (portString != null) {
            portString = portString.trim();
            if (portString.length() > 0) {
                try {
                    int port = Integer.parseInt(portString);
                    smtpParameters.internalSetPort(port);
                } catch (NumberFormatException nfe) {
                    errorList.add("Not integer value: port / " + portString);
                }
            }
        }
        String username = properties.getProperty("username");
        if (username != null) {
            smtpParameters.internalSetUsername(username);
        }
        String password = properties.getProperty("password");
        if (password != null) {
            smtpParameters.internalSetPassword(password);
        }
        String popHost = properties.getProperty("pophost");
        if (popHost != null) {
            smtpParameters.internalSetPopHost(popHost);
        }
        String popUsername = properties.getProperty("popusername");
        if (popUsername != null) {
            smtpParameters.internalSetPopUsername(popUsername);
        }
        String popPassword = properties.getProperty("poppassword");
        if (popPassword != null) {
            smtpParameters.internalSetPopPassword(popPassword);
        }
        if ((authentificationType == SmtpParameters.AUTHENTIFICATION_BASIC) || (authentificationType == SmtpParameters.AUTHENTIFICATION_SSL)) {
            if (smtpParameters.getUsername() == null) {
                errorList.add("Missing key: username");
            }
            if (smtpParameters.getPassword() == null) {
                errorList.add("Missing key: password");
            }

        }
        if (authentificationType == SmtpParameters.AUTHENTIFICATION_POP) {
            if (smtpParameters.getPopUsername() == null) {
                errorList.add("Missing keys: username or popusername");
            }
            if (smtpParameters.getPopPassword() == null) {
                errorList.add("Missing key: password or poppassword");
            }

        }
        return smtpParameters;
    }

    public static short authentificationTypeToShort(String authentificationString) {
        if (authentificationString.equals("none")) {
            return SmtpParameters.AUTHENTIFICATION_NONE;
        }
        if (authentificationString.equals("basic")) {
            return SmtpParameters.AUTHENTIFICATION_BASIC;
        }
        if (authentificationString.equals("pop")) {
            return SmtpParameters.AUTHENTIFICATION_POP;
        }
        if (authentificationString.equals("ssl")) {
            return SmtpParameters.AUTHENTIFICATION_SSL;
        }
        throw new IllegalArgumentException("Uknown authentificationString: " + authentificationString);
    }

}
