/* UtilLib - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import java.io.Serializable;


/**
 * Implémentation de l'interface Libelle qui suit les spécifications Java Beans.
 *
 * @author Vincent Calame
 */
public final class LibelleBean implements Serializable {

    private String lang = "und";
    private String libelleString;

    public LibelleBean() {
    }

    public LibelleBean(Libelle libelle) {
        setLangInteger(libelle.getLangInteger());
        setLibelleString(libelle.getLibelleString());
    }

    public LibelleBean(int langInteger, String libelleString) {
        this.lang = LangInteger.toString(langInteger);
        this.libelleString = libelleString;
    }

    public String getLang() {
        return lang;
    }

    public void setLang(String lang) {
        if (lang == null) {
            throw new IllegalArgumentException("lang cannot be null");
        }
        this.lang = lang;
    }

    public void setLangInteger(int langInteger) {
        this.lang = LangInteger.toString(langInteger);
    }

    public String getLibelleString() {
        return libelleString;
    }

    public void setLibelleString(String libelleString) {
        this.libelleString = libelleString;
    }

    public Libelle toLibelle() {
        return new InternalLibelle(this);
    }


    private static class InternalLibelle implements Libelle {

        private int langInteger;
        private String libelleString;

        private InternalLibelle(LibelleBean libelleBean) {
            this.libelleString = libelleBean.libelleString;
            try {
                langInteger = LangInteger.parse(libelleBean.lang);
            } catch (LangIntegerException lie) {
                langInteger = LangInteger.UNDETERMINED;
            }
        }

        public String getLibelleString() {
            return libelleString;
        }

        public int getLangInteger() {
            return langInteger;
        }

    }

}
