/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.AbstractList;
import java.util.RandomAccess;


/**
 * Classe permettant le stockage de listes de libellés pour des clés quelconques.
 * Cette classe est utile pour conserver des données de localisation (souvent initialisées une fois pour toute).
 *Cette classe ne propose d'ailleurs que des fonctions d'ajout.
 *Cela permet d'avoir des bibliothèques de libellés.
 *
 * @author Vincent Calame
 */
public class LibelleListMap {

    private Map<Object, LibList> libelleListMap = new HashMap<Object, LibList>();

    public List<Libelle> getLibelleList(Object key) {
        return libelleListMap.get(key);
    }

    public Libelle getLibelle(Object key, int langInteger) {
        LibList libList = libelleListMap.get(key);
        if (libList == null) {
            return null;
        }
        return libList.getLibelle(langInteger);
    }

    public void setLibelle(Object key, int langInteger, String libelle) {
        if (libelle == null) {
            throw new IllegalArgumentException("libelle argument is null");
        }
        LibList libList = (LibList) libelleListMap.get(key);
        if (libList == null) {
            libList = new LibList(langInteger, libelle);
            libelleListMap.put(key, libList);
        } else {
            libList.add(langInteger, libelle);
        }
    }


    private class UnmodifiableLib implements Libelle {

        private int langInteger;
        private String libelle;

        UnmodifiableLib(int langInteger, String libelle) {
            this.langInteger = langInteger;
            this.libelle = libelle;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public String getLibelleString() {
            return libelle;
        }

    }


    private class LibList extends AbstractList<Libelle> implements RandomAccess {

        private UnmodifiableLib[] tab;

        LibList(int langInteger, String libelle) {
            tab = new UnmodifiableLib[1];
            tab[0] = new UnmodifiableLib(langInteger, libelle);
        }

        void add(int langInteger, String lib) {
            int length = tab.length;
            for (int i = 0; i < length; i++) {
                if (tab[i].langInteger == langInteger) {
                    tab[i].libelle = lib;
                    return;
                }
            }
            UnmodifiableLib[] nvtab = new UnmodifiableLib[length + 1];
            System.arraycopy(tab, 0, nvtab, 0, length);
            nvtab[length] = new UnmodifiableLib(langInteger, lib);
            tab = nvtab;
        }

        public Libelle get(int index) {
            return tab[index];
        }

        public int size() {
            return tab.length;
        }

        public Libelle getLibelle(int langInteger) {
            int length = tab.length;
            for (int i = 0; i < length; i++) {
                if (tab[i].langInteger == langInteger) {
                    return tab[i];
                }
            }
            return null;
        }

    }

}
