/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.insertpattern.impl;

import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.ErrorHandler;
import net.mapeadores.util.text.insertpattern.api.InsertionHandler;
import net.mapeadores.util.text.StringUtils;
import java.io.Writer;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class CheckHandler implements ErrorHandler, InsertionHandler {

    final static String INC_INSTRUCTION = "\n   ";
    final static String INC_ARGUMENT = "\n      ";
    final static String INC_KEY = "            ";
    final static String INC_VALUE = "            ";
    private Writer writer;
    private int ifCount = 0;
    private String ifInc = "";
    final static String INC_IF = "   ";

    public CheckHandler(OutputStream outputStream) throws IOException {
        writer = new OutputStreamWriter(outputStream, "UTF-8");
    }

    public void characterError(char carac, String key, int row, int col) {
        try {
            writer.write("\n%%%%% ParseError : ");
            writer.write(key);
            writer.write(" ");
            writer.write(StringUtils.charToString(carac));
            writer.write(" at ");
            writer.write(String.valueOf(row));
            writer.write(":");
            writer.write(String.valueOf(col));
            writer.write(" %%%%%\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public void warning(String key, int row, int col) {
        try {
            writer.write("\n%%%%% Warning : ");
            writer.write(key);
            writer.write(" at ");
            writer.write(String.valueOf(row));
            writer.write(":");
            writer.write(String.valueOf(col));
            writer.write(" %%%%%\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public void appendString(String litteralString) {
        try {
            //writer.write("\n------ LiteralString:\n");
            writer.write(litteralString);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public void close() throws IOException {
        writer.close();
    }

    public void flush() throws IOException {
        writer.flush();
    }

    public void doInsertion(Instruction[] instructionArray) {
        try {
            int length = instructionArray.length;
            writer.write("\n------ Insertion :\n");
            for (int i = 0; i < length; i++) {
                Instruction arg = instructionArray[i];
                writer.write(INC_INSTRUCTION);
                writer.write("---- Instruction ");
                writer.write(String.valueOf(i + 1));
                writer.write(" ----\n");
                int ln = arg.getLength();
                for (int j = 0; j < ln; j++) {
                    writer.write(INC_ARGUMENT);
                    writer.write("--  argument ");
                    writer.write(String.valueOf(j + 1));
                    writer.write("  --\n");
                    Argument argEn = arg.getArgument(j);
                    writer.write(INC_KEY);
                    writer.write("key = ");
                    writer.write(argEn.getKey());
                    writer.write("\n");
                    String value = argEn.getValue();
                    if (value != null) {
                        writer.write(INC_VALUE);
                        writer.write("value = ");
                        writer.write(value);
                        writer.write("\n");
                    }
                }
            }
            writer.write("\n------\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public void startIf(String clause, boolean unique) {
        increaseInc();
        try {
            writer.write("\n\n");
            writer.write(ifInc);
            writer.write("***** IF");
            if (unique) {
                writer.write("F");
            }
            writer.write("  ");
            writer.write(clause);
            writer.write("  :\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

    }

    private void increaseInc() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < ifCount; i++) {
            buf.append(INC_IF);
        }
        ifInc = buf.toString();
        ifCount++;
    }

    private void decreaseInc() {
        ifCount--;
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < ifCount; i++) {
            buf.append(INC_IF);
        }
        ifInc = buf.toString();
    }

    public void startElseIf(String clause) {
        try {
            writer.write("\n\n");
            writer.write(ifInc);
            writer.write("***** ELSEIF  ");
            writer.write(clause);
            writer.write("  :\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

    }

    public void startElse() {
        try {
            writer.write("\n\n");
            writer.write(ifInc);
            writer.write("***** ELSE  :\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public void endIf(boolean unique, boolean onElse) {
        decreaseInc();
        try {
            writer.write("\n\n");
            writer.write(ifInc);
            writer.write("***** FI  *****\n");
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

}
