/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.lexie;

import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class LexieParser {

    LexieParserListener listener;
    int wordStart = -1;
    int bufLength = 0;
    char[] bufArray;

    private LexieParser(LexieParserListener listener) {
        this.listener = listener;
    }

    public static void parse(String chaine, LexieParserListener listener) {
        if (chaine == null) {
            return;
        }
        LexieParser lexieParser = new LexieParser(listener);
        lexieParser.parse(chaine);
    }

    private void parse(String chaine) {
        int taille = chaine.length();
        bufArray = new char[taille];
        for (int i = 0; i < taille; i++) {
            char carac = chaine.charAt(i);
            if (Character.isLetterOrDigit(carac)) {
                if (StringUtils.isIdeogram(carac)) {
                    flushWord();
                    wordStart = i;
                    bufLength = 1;
                    bufArray[0] = carac;
                    flushWord();
                } else {
                    bufArray[bufLength] = carac;
                    bufLength++;
                    if (wordStart == -1) {
                        wordStart = i;
                    }
                }
            } else {
                flushWord();
                listener.checkSpecialChar(carac);
            }
        }
        flushWord();
    }

    private void flushWord() {
        if (wordStart > -1) {
            String word = new String(bufArray, 0, bufLength);
            listener.flushLexie(word, wordStart);
            bufLength = 0;
            wordStart = -1;
        }
    }

}
