/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class MysqlParameters implements TableParser.Parameters {

    public final static String ALLOWED_FIELDS_TERMINATEDBY = ",;\t|";
    public final static String ALLOWED_LINES_TERMINATEDBY = "\n§\u00B6$";
    private char linesTerminatedBy = '\n';
    private char escapedBy = '\\';
    private char fieldsTerminatedBy = '\t';

    public MysqlParameters() {
    }

    public char getEscapedBy() {
        return escapedBy;
    }

    public char getFieldsTerminatedBy() {
        return fieldsTerminatedBy;
    }

    public char getLinesTerminatedBy() {
        return linesTerminatedBy;
    }

    public void setFieldsTerminatedBy(char fieldsTerminatedBy) {
        if (ALLOWED_FIELDS_TERMINATEDBY.indexOf(fieldsTerminatedBy) == -1) {
            throw new IllegalArgumentException("Wrong fieldsTerminatedBy = " + fieldsTerminatedBy);
        }
        this.fieldsTerminatedBy = fieldsTerminatedBy;
    }

    public void setLinesTerminatedBy(char linesTerminatedBy) {
        if (ALLOWED_LINES_TERMINATEDBY.indexOf(linesTerminatedBy) == -1) {
            throw new IllegalArgumentException("Wrong linesTerminatedBy = " + linesTerminatedBy);
        }
        this.linesTerminatedBy = linesTerminatedBy;
    }

    public static char getDefaultFieldsTerminatedBy() {
        return '\t';
    }

    public static char getDefaultLinesTerminatedBy() {
        return '\n';
    }

    public static char[] getAvailableFieldsTerminatedByArray() {
        return ALLOWED_FIELDS_TERMINATEDBY.toCharArray();
    }

    public static char[] getAvailableLinesTerminatedByArray() {
        return ALLOWED_LINES_TERMINATEDBY.toCharArray();
    }

    public static String[] getAvalaibleFieldsTerminatedByArrayToString() {
        return StringUtils.toStringArray(ALLOWED_FIELDS_TERMINATEDBY);
    }

    public static String[] getAvalaibleLinesTerminatedByArrayToString() {
        return StringUtils.toStringArray(ALLOWED_LINES_TERMINATEDBY);
    }

}
