/* UtilLib - Copyright (c) 2005-20006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LangFilter;
import java.awt.Color;
import java.io.IOException;
import org.w3c.dom.Element;


/**
 *
 * @author  Vincent Calame
 */
public class TagUtils {

    private TagUtils() {
    }

    /**
     * @throw LangIntegerException si la valeur de l'attribut xml:lang est faux ou égal à LangInteger.NO_LINGUISTIC_CONTENT ("zxx")
     */
    static public Libelle getLibelle(Element element) throws LangIntegerException {
        int langInteger = LangInteger.UNDETERMINED;
        String langIntegerString = element.getAttribute("xml:lang");
        if (langIntegerString.length() > 0) {
            langInteger = LangInteger.parse(langIntegerString);
            if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
                throw new LangIntegerException();
            }
        }
        String libelle = XMLUtils.getData(element);
        String vo = element.getAttribute("vo");
        boolean isvo = true;
        if ((vo.equals("0")) || (vo.equals("false"))) {
            isvo = false;
        }
        return new XMLLib(langInteger, libelle, isvo);
    }

    static public Color getColor(Element element) {
        try {
            return Color.decode(XMLUtils.getData(element));
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    /**
     *langFilter peut être null.
     */
    static public void writeLibelleList(Appendable writer, LibelleHolder libelleHolder, int nbredetabulations, LangFilter langFilter) throws IOException {
        int libelleCount = libelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle lib = libelleHolder.getLibelleByIndex(i);
            if ((langFilter == null) || (langFilter.accept(lib.getLangInteger()))) {
                XMLUtils.writeTabs(writer, nbredetabulations);
                writeLibelle(writer, lib.getLibelleString(), lib.getLangInteger());
            }
        }
    }

    static public void writeLibelle(Appendable writer, String libelleString, int langInteger) throws IOException {
        if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
            return;
        }
        writer.append("<lib");
        writeXmlLangAttribute(writer, langInteger);
        writer.append(">");
        XMLUtils.writeEscape(writer, libelleString);
        writer.append("</lib>");
    }

    static public void writeXmlLangAttribute(Appendable writer, int langInteger) throws IOException {
        if (langInteger != LangInteger.UNDETERMINED) {
            writer.append(" xml:lang=\"");
            LangInteger.writeString(writer, langInteger);
            writer.append("\"");
        }
    }


    static class XMLLib implements Libelle {

        int langInteger;
        String libelle;
        boolean libellevo;

        XMLLib(int langInteger, String libelle, boolean libellevo) {
            this.langInteger = langInteger;
            this.libelle = libelle;
            this.libellevo = libellevo;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public String getLibelleString() {
            return libelle;
        }

    }

}
