/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import net.mapeadores.util.text.StringUtils;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringReader;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.awt.Color;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;


/**
 * Contients des méthodes statiques
 */
public class XMLUtils {

    public static String toString(Element element, boolean elementInclude) {
        StringBuilder buf = new StringBuilder();
        DefaultXMLWriter xmlWriter = new DefaultXMLWriter();
        xmlWriter.setAppendable(buf);
        xmlWriter.setIndentLength(-999);
        try {
            addElement(xmlWriter, element, elementInclude);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void addElement(XMLWriter xmlWriter, Element element, boolean elementInclude) throws IOException {
        String tagName = element.getTagName();
        if (elementInclude) {
            xmlWriter.startOpenTag(tagName);
            if (element.hasAttributes()) {
                NamedNodeMap map = element.getAttributes();
                int count = map.getLength();
                for (int i = 0; i < count; i++) {
                    Attr attribute = (Attr) map.item(i);
                    xmlWriter.addAttribute(attribute.getName(), attribute.getValue());
                }
            }
        }
        NodeList children = element.getChildNodes();
        if (elementInclude) {
            if (children.getLength() == 0) {
                xmlWriter.closeEmptyTag();
                return;
            } else {
                xmlWriter.endOpenTag();
            }
        }
        int count = children.getLength();
        for (int i = 0; i < count; i++) {
            Node node = children.item(i);
            if (node instanceof Text) {
                xmlWriter.append(((Text) node).getData());
            } else if (node instanceof Element) {
                addElement(xmlWriter, (Element) node, true);
            }
        }
        if (elementInclude) {
            xmlWriter.closeTag(tagName);
        }
    }

    public static void writeDomTree(Document doc, OutputStream outputStream) throws TransformerException {
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        DOMSource source = new DOMSource(doc);
        StreamResult result = new StreamResult(outputStream);
        transformer.transform(source, result);
    }

    public static void printTabs(Appendable appendable, int tabCount) {
        try {
            writeTabs(appendable, tabCount);
        } catch (IOException ioe) {
        }
    }

    public static void writeTabs(Appendable appendable, int tabCount) throws IOException {
        if (tabCount > - 1) {
            appendable.append('\n');
            for (int i = 0; i < tabCount; i++) {
                appendable.append('\t');
            }
        }
    }

    @Deprecated
    public static void printEscape(String s, Appendable appendable) {
        try {
            writeEscape(appendable, s);
        } catch (IOException ioe) {
        }
    }

    public static void printEscape(Appendable appendable, String s) {
        try {
            writeEscape(appendable, s);
        } catch (IOException ioe) {
        }
    }

    @Deprecated
    public static void writeEscape(String s, Appendable appendable) throws IOException {
        writeEscape(appendable, s);
    }

    public static void writeEscape(Appendable appendable, String s) throws IOException {
        if ((s == null) || (s.length() == 0)) {
            return;
        }
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length; i++) {
            if (caracs[i] < 32) {
                appendable.append(' ');
            } else {
                switch (caracs[i]) {
                    case '&':
                        appendable.append("&amp;");
                        break;
                    case '"':
                        appendable.append("&quot;");
                        break;
                    case '\'':
                        appendable.append("&apos;");
                        break;
                    case '<':
                        appendable.append("&lt;");
                        break;
                    case '>':
                        appendable.append("&gt;");
                        break;
                    case '\u00A0':
                        appendable.append("&#x00A0;");
                        break;
                    default:
                        appendable.append(caracs[i]);
                }
            }
        }
    }

    public static void printTechEscape(Appendable appendable, String s) throws IOException {
        try {
            writeTechEscape(appendable, s);
        } catch (IOException ioe) {
        }

    }

    public static void writeTechEscape(Appendable appendable, String s) throws IOException {
        if ((s == null) || (s.length() == 0)) {
            return;
        }
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length; i++) {
            if (caracs[i] < 32) {
                appendable.append(' ');
            } else {
                switch (caracs[i]) {
                    case '&':
                        appendable.append("&amp;");
                        break;
                    case '"':
                        appendable.append("&quot;");
                        break;
                    case '<':
                        appendable.append("&lt;");
                        break;
                    case '>':
                        appendable.append("&gt;");
                        break;
                    case '\u00A0':
                        appendable.append("&#x00A0;");
                        break;
                    default:
                        appendable.append(caracs[i]);
                }
            }
        }
    }

    public static void printEscape(Appendable appendable, char carac) {
        try {
            writeEscape(appendable, carac);
        } catch (IOException ioe) {
        }
    }

    public static void writeEscape(Appendable appendable, char carac) throws IOException {
        if (carac < 32) {
            appendable.append(' ');
        } else {
            switch (carac) {
                case '&':
                    appendable.append("&amp;");
                    break;
                case '"':
                    appendable.append("&quot;");
                    break;
                case '\'':
                    appendable.append("&apos;");
                    break;
                case '<':
                    appendable.append("&lt;");
                    break;
                case '>':
                    appendable.append("&gt;");
                    break;
                case '\u00A0':
                    appendable.append("&#x00A0;");
                    break;
                default:
                    appendable.append(carac);
            }
        }
    }

    public static String cleanString(String s) {
        return StringUtils.cleanString(s, true);
    }

    public static String cleanString(String s, boolean trim) {
        return StringUtils.cleanString(s, trim);
    }

    public static String getData(Element el) {
        return getData(el, true);
    }

    public static String getData(Element el, boolean trim) {
        String s = getRawData(el);
        return cleanString(s, trim);
    }

    public static String getRawData(Element el) {
        StringBuffer buf = new StringBuffer();
        NodeList list = el.getChildNodes();
        for (int i = 0; i < list.getLength(); i++) {
            Node nd = list.item(i);
            short nodeType = nd.getNodeType();
            if ((nodeType == Node.TEXT_NODE) || (nodeType == Node.CDATA_SECTION_NODE)) {
                String data = ((Text) nd).getData();
                if (data != null) {
                    buf.append(data);
                }
            }
        }
        return buf.toString();
    }

    public static Element getFirstElement(Element el) {
        NodeList list = el.getChildNodes();
        for (int i = 0; i < list.getLength(); i++) {
            Node nd = list.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                return (Element) nd;
            }
        }
        return null;
    }

    /**
     * Retourne l'expression de la couleur en argument sous la forme de la chaîne utlisée en HTML #00FF00.
     **/
    public static String toHexFormat(Color color) {
        StringBuffer buf = new StringBuffer();
        buf.append('#');
        appendHexString(color.getRed(), buf);
        appendHexString(color.getGreen(), buf);
        appendHexString(color.getBlue(), buf);
        return buf.toString();
    }

    /*
     * Méthode privée utilisée par toHexFormat pour traduire un entier en un hexadécimal de deux lettres.
     */
    private static void appendHexString(int codecouleur, StringBuffer buf) {
        String hex = Integer.toHexString(codecouleur);
        if (hex.length() == 1) {
            buf.append('0');
        }
        buf.append(hex);
    }

    public static void printAttribute(Appendable appendable, String attributename, String value) {
        try {
            writeAttribute(appendable, attributename, value);
        } catch (IOException ioe) {
        }
    }

    public static void writeAttribute(Appendable appendable, String attributename, String value) throws IOException {
        if ((value != null) && (value.length() > 0)) {
            appendable.append(' ');
            appendable.append(attributename);
            appendable.append("=\"");
            writeEscape(appendable, value);
            appendable.append('\"');
        }
    }

    public static void printTag(Appendable appendable, String tagname, String value, int tabCount) {
        try {
            writeTag(appendable, tagname, value, tabCount);
        } catch (IOException ioe) {
        }
    }

    public static void writeTag(Appendable appendable, String tagname, String value, int tabCount) throws IOException {
        if ((value != null) && (value.length() > 0)) {
            writeTabs(appendable, tabCount);
            appendable.append("<");
            appendable.append(tagname);
            appendable.append(">");
            writeEscape(appendable, value);
            appendable.append("</");
            appendable.append(tagname);
            appendable.append(">");
        }
    }

    public static void printXMLDeclaration(Appendable appendable) {
        try {
            writeXMLDeclaration(appendable);
        } catch (IOException ioe) {
        }
    }

    public static void writeXMLDeclaration(Appendable appendable) throws IOException {
        appendable.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }

    public static void printXMLStylesheet(Appendable appendable, String type, String href) {
        try {
            writeXMLStylesheet(appendable, type, href);
        } catch (IOException ioe) {
        }
    }

    public static void writeXMLStylesheet(Appendable appendable, String type, String href) throws IOException {
        appendable.append("<?xml-stylesheet type=\"");
        appendable.append(type);
        appendable.append("\" href=\"");
        appendable.append(href);
        appendable.append("\"?>");
    }

    public static Document getDocument(String xml) throws XMLException {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            InputSource inputSource = new InputSource(new StringReader(xml));
            Document doc = db.parse(inputSource);
            return doc;
        } catch (ParserConfigurationException pce) {
            throw new XMLException(pce);
        } catch (SAXParseException spe) {
            throw new XMLException(spe);
        } catch (SAXException se) {
            throw new XMLException(se);
        } catch (IOException ioe) {
            throw new XMLException(ioe);
        }
    }

    public static Object testXml(File xslFile) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docbuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = docbuilder.parse(xslFile);
            return Boolean.TRUE;
        } catch (Exception e) {
            String message = e.getMessage();
            if (e instanceof SAXParseException) {
                SAXParseException sax = (SAXParseException) e;
                message = message + " at " + sax.getLineNumber() + "," + sax.getColumnNumber();
            }
            return message;
        }
    }

}
