/* UtilLib_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.email.tools;

import net.mapeadores.util.smtp.SmtpParameters;
import java.util.Properties;
import net.mapeadores.util.email.Email;
import net.mapeadores.util.email.MultiPartEmail;
import net.mapeadores.util.email.SimpleEmail;


/**
 *
 * @author Vincent Calame
 */
public class EmailUtils {

    private EmailUtils() {
    }

    public static SimpleEmail newSimpleEmail(SmtpParameters smtpParameters) {
        SimpleEmail email = new SimpleEmail();
        init(email, smtpParameters);
        return email;
    }

    public static MultiPartEmail newMultipartEmail(SmtpParameters smtpParameters) {
        MultiPartEmail email = new MultiPartEmail();
        init(email, smtpParameters);
        return email;
    }

    private static void init(Email email, SmtpParameters smtpParameters) {
        email.setHostName(smtpParameters.getHost());
        email.setSmtpPort(smtpParameters.getPort());
        short authentificationType = smtpParameters.getAuthentificationType();
        switch (authentificationType) {
            case SmtpParameters.AUTHENTIFICATION_BASIC:
            case SmtpParameters.AUTHENTIFICATION_SSL:
                email.setAuthentication(smtpParameters.getUsername(), smtpParameters.getPassword());
                break;

            case SmtpParameters.AUTHENTIFICATION_POP:
                email.setPopBeforeSmtp(true, smtpParameters.getPopHost(), smtpParameters.getPopUsername(), smtpParameters.getPopPassword());
                break;
        }
        email.setCharset("UTF-8");
        email.setCharsetSubject("UTF-8");
        if (authentificationType == SmtpParameters.AUTHENTIFICATION_SSL) {
            Properties supplementarySessionProperties = new Properties();
            supplementarySessionProperties.put("mail.smtp.starttls.enable", "true");
            supplementarySessionProperties.put("mail.smtp.socketFactory.port", String.valueOf(smtpParameters.getPort()));
            //supplementarySessionProperties.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            supplementarySessionProperties.put("mail.smtp.socketFactory.class", "net.mapeadores.util.email.tools.LaxSSLSocketFactory");
            supplementarySessionProperties.put("mail.smtp.socketFactory.fallback", "false");
            email.setSupplementarySessionProperties(supplementarySessionProperties);
        }
    }

}
