/* UtilLib_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.email.tools;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import javax.net.SocketFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;


/**
 * Original de Eugen Kuleshov et Dmitry I. Platonoff
 * http://www.javaworld.com/javatips/jw-javatip115.html
 * ,
 * @author Vincent Calame
 */
public class LaxSSLSocketFactory extends SSLSocketFactory {

    private SSLSocketFactory internalSSLSocketFactory;

    public LaxSSLSocketFactory() {
        try {
            SSLContext sslcontext = SSLContext.getInstance("TLS");
            sslcontext.init(null, new TrustManager[]{new LaxTrustManager()}, new java.security.SecureRandom());
            internalSSLSocketFactory = (SSLSocketFactory) sslcontext.getSocketFactory();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static SocketFactory getDefault() {
        return new LaxSSLSocketFactory();
    }

    public Socket createSocket(Socket socket, String host, int port, boolean autoClose) throws IOException {
        return internalSSLSocketFactory.createSocket(socket, host, port, autoClose);
    }

    public Socket createSocket(InetAddress address, int port, InetAddress localAddress, int localPort) throws IOException {
        return internalSSLSocketFactory.createSocket(address, port, localAddress, localPort);
    }

    public Socket createSocket(InetAddress address, int port) throws IOException {
        return internalSSLSocketFactory.createSocket(address, port);
    }

    public Socket createSocket(String host, int port, InetAddress localAddress, int localPort) throws IOException {
        return internalSSLSocketFactory.createSocket(host, port, localAddress, localPort);
    }

    public Socket createSocket(String host, int port) throws IOException {
        return internalSSLSocketFactory.createSocket(host, port);
    }

    public String[] getDefaultCipherSuites() {
        return internalSSLSocketFactory.getSupportedCipherSuites();
    }

    public String[] getSupportedCipherSuites() {
        return internalSSLSocketFactory.getSupportedCipherSuites();
    }

}

