/* UtilLib_Servlet - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets;

import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.json.JsonProducer;
import java.util.List;
import java.util.Map;
import java.util.HashMap;


/**
 *
 * @author Vincent Calame
 */
public class ResponseParameters implements CommandTest, ResponseDataType {

    public static final short HTMLPAGE_CATEGORY = 1;
    public static final short XMLDATA_CATEGORY = 2;
    public static final short CONTENTSTREAM_CATEGORY = 3;
    private Map<String, Object> paramMap;
    private short responseCode = 0;
    private short responseCodeCategory = 0;
    private CommandMessage message;
    private int profondeur = 0;
    private String reversePath = "";
    private JsonProducer jsonSource = null;
    private short dataType = HTML_DATATYPE;

    public ResponseParameters() {
    }

    /**
     * Précise le code de la page à utiliser.
     * Les valeurs de l'argument <code>code</code> sont à la discrétion de la classe étendant ResponseParameters,
     *la valeur 0 étant cependant interdite.
     *
     *@throws IllegalArgumentException si code = 0
     */
    public void setHtmlPageCode(short code) {
        if (code == 0) {
            throw new IllegalArgumentException("type == 0");
        }
        this.responseCode = code;
        this.responseCodeCategory = HTMLPAGE_CATEGORY;
    }

    /**
     * Précise le code de donnée xml à retourner.
     * la valeur 0 étant cependant interdite.
     *Les valeurs de l'argument <code>code</code> sont à la discrétion de la classe étendant ResponseParameters
     *
     *@throws IllegalArgumentException si code = 0
     */
    public void setXmlDataCode(short code) {
        if (code == 0) {
            throw new IllegalArgumentException("type == 0");
        }
        this.responseCode = code;
        this.responseCodeCategory = XMLDATA_CATEGORY;
    }

    /**
     * Précise le code de contenu à fournir.
     * Les valeurs de l'argument <code>code</code> sont à la discrétion de la classe étendant ResponseParameters,
     *la valeur 0 étant cependant interdite.
     *
     *@throws IllegalArgumentException si code = 0
     */
    public void setContentStreamCode(short code) {
        if (code == 0) {
            throw new IllegalArgumentException("type == 0");
        }
        this.responseCode = code;
        this.responseCodeCategory = CONTENTSTREAM_CATEGORY;
    }

    public short getResponseCodeCategory() {
        return responseCodeCategory;
    }

    public short getResponseCategory() {
        return responseCodeCategory;
    }

    /**
     *@throws UnsupportedOperationException si la catégorie n'est pas HTMLPAGE_CATEGORY
     */
    public short getResponseCode() {
        return responseCode;
    }

    public void putParameter(String key, Object value) {
        if (paramMap == null) {
            paramMap = new HashMap<String, Object>();
        }
        paramMap.put(key, value);
    }

    public Object getParameter(String key) {
        if (paramMap == null) {
            return null;
        }
        return paramMap.get(key);
    }

    /**
     *
     *@throws IllegalArgumentException si le messageType est incorrect.
     */
    public void setCommandMessage(short messageType, String messageKey, Object messageValue) {
        if (messageType == CommandMessage.DONE_MESSAGE) {
            setCommandMessage(CommandMessage.done(messageKey, messageValue));
            return;
        }
        if (messageType == CommandMessage.ERROR_MESSAGE) {
            setCommandMessage(CommandMessage.error(messageKey, messageValue));
            return;
        }
        throw new IllegalArgumentException("wrong messageType");
    }

    public void setCommandMessage(CommandMessage message) {
        this.message = message;
    }

    public void setDoneMessage(String messageKey, Object messageValue) {
        this.message = CommandMessage.done(messageKey, messageValue);
    }

    public void setErrorMessage(String messageKey, Object messageValue) {
        this.message = CommandMessage.error(messageKey, messageValue);
    }

    public void setMultiErrorMessage(String messageKey, List<ResourceItem> resourceItemList) {
        this.message = CommandMessage.multiError(new ResourceItem(messageKey), resourceItemList);
    }

    public CommandMessage getCommandMessage() {
        return message;
    }

    public boolean mayBeDone() {
        if ((message != null) && (message.isErrorMessage())) {
            return false;
        } else {
            return true;
        }
    }

    public boolean hasErrorMessage() {
        return ((message != null) && (message.isErrorMessage()));
    }

    public int getProfondeur() {
        return profondeur;
    }

    public void setProfondeur(int profondeur) {
        this.profondeur = profondeur;
        if (profondeur < 1) {
            reversePath = "";
        } else {
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < profondeur; i++) {
                buf.append("../");
            }
            reversePath = buf.toString();
        }
    }

    public String getReversePath() {
        return reversePath;
    }

    public short getDataType() {
        return dataType;
    }

    public void setDataType(short dataType) {
        this.dataType = dataType;
    }

    /**
     * @return the jsonSource
     */
    public JsonProducer getJsonSource() {
        return jsonSource;
    }

    /**
     * @param jsonSource the jsonSource to set
     */
    public void setJsonSource(JsonProducer jsonSource) {
        this.jsonSource = jsonSource;
    }

}
