/* UtilLib_Servlet  - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets.handlers;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogUtils;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageResponseHandler implements ResponseHandler {

    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;
    private CommandMessage commandMessage;
    private ResourceLocalisation resourceLocalisation;
    private boolean isHtml = false;

    public CommandMessageResponseHandler(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation) {
        this.commandMessage = commandMessage;
        this.resourceLocalisation = resourceLocalisation;
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        boolean error = commandMessage.isErrorMessage();
        short type = commandMessage.getType();
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        HtmlPrinter htmlPrinter = new HtmlPrinter();
        htmlPrinter.initPrinter(pw);
        htmlPrinter.addResourceLocalisation(resourceLocalisation);
        htmlPrinter.startHtml(0);
        htmlPrinter.startHead();
        htmlPrinter.startTitle();
        htmlPrinter.escape(commandMessage.getKeyResourceItem().getResourceKey());
        htmlPrinter.endTitle();
        htmlPrinter.endHead();
        htmlPrinter.startBody();
        htmlPrinter.startP();
        HTMLAttributes spanAttributes = new HTMLAttributes();
        if (error) {
            spanAttributes.setStyle("color: red");
        }
        else {
            spanAttributes.setStyle("color: green");
        }
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.localize(commandMessage.getKeyResourceItem());
        htmlPrinter.endSpan();
        htmlPrinter.endP();
        if (type == CommandMessage.MULTIERROR_MESSAGE) {
            htmlPrinter.startUl();
            List<ResourceItem> list = commandMessage.getMultiErrorList();
            for (ResourceItem resourceItem : list) {
                htmlPrinter.startLi();
                htmlPrinter.localize(resourceItem);
                htmlPrinter.endLi();
            }
            htmlPrinter.endUl();
        } else if (type == CommandMessage.RESULTLOG_MESSAGE) {
            ProcessLog processLog = commandMessage.getProcessLog();
            if (processLog.hasLog()) {
                htmlPrinter.startPre();
                htmlPrinter.escape(ProcessLogUtils.toString(processLog), true);
                htmlPrinter.endPre();
            }
        }
        htmlPrinter.endBody();
        pw.close();
    }

}
