/* UtilLib_Servlet - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets.handlers;

import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.io.OutputStream;
import java.io.IOException;
import java.io.Writer;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseDataTypeUtils;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class JsonResponseHandler implements ResponseHandler, ResponseDataType {

    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;
    private JsonProducer jsonProducer;
    private short dataType;
    private String callbackFunctionName;

    /**
     * JsonSource peut être nul. Dans ce cas, l'objet Json renvoyé est {"noOutput":true}
     *
     * @param jsonSource
     */
    public JsonResponseHandler(JsonProducer jsonProducer) {
        this.jsonProducer = jsonProducer;
        this.dataType = JSON_DATATYPE;
    }

    public JsonResponseHandler(JsonProducer jsonProducer, boolean embedded) {
        this.jsonProducer = jsonProducer;
        if (embedded) {
            this.dataType = JSON_IN_HTML_DATATYPE;
        } else {
            this.dataType = JSON_DATATYPE;
        }
    }

    public JsonResponseHandler(JsonProducer jsonProducer, String callbackFunctionName) {
        this.jsonProducer = jsonProducer;
        this.callbackFunctionName = callbackFunctionName;
        if (callbackFunctionName != null) {
            this.dataType = JSON_P_DATATYPE;
        } else {
            this.dataType = JSON_DATATYPE;
        }
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(ResponseDataTypeUtils.dataTypeToContentType(dataType));
        OutputStream outputStream = response.getOutputStream();
        BufferedWriter bufWriter = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
        boolean embedded = (dataType == JSON_IN_HTML_DATATYPE);
        boolean callback = (callbackFunctionName != null);
        if (embedded) {
            bufWriter.write("<textarea>");
        }
        if (callback) {
            bufWriter.write(callbackFunctionName);
            bufWriter.write("(");
        }
        if (jsonProducer != null) {
            jsonProducer.writeJson(bufWriter);
        } else {
            writeNull(bufWriter);
        }
        if (callback) {
            bufWriter.write(");");
        }
        if (embedded) {
            bufWriter.write("</textarea>");
        }
        bufWriter.close();
    }

    private void writeNull(Writer writer) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(writer);
        try {
            jsonWriter.object();
            jsonWriter.key("noOutput");
            jsonWriter.value(true);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
